
########################################################
#
# Copyright (c) 2003-2012 by University of Queensland
# Earth Systems Science Computational Center (ESSCC)
# http://www.uq.edu.au/esscc
#
# Primary Business: Queensland, Australia
# Licensed under the Open Software License version 3.0
# http://www.opensource.org/licenses/osl-3.0.php
#
########################################################

__copyright__="""Copyright (c) 2003-2012 by University of Queensland
Earth Systems Science Computational Center (ESSCC)
http://www.uq.edu.au/esscc
Primary Business: Queensland, Australia"""
__license__="""Licensed under the Open Software License version 3.0
http://www.opensource.org/licenses/osl-3.0.php"""
__url__="https://launchpad.net/escript-finley"

"""
basic tests for functions in util.py effecting the spatial distribution 

it is assumed that the domain is the usint square/cube 

not all these test will run for all domains. check the doc string for the assumptions of a particular test

:var __author__: name of author
:var __copyright__: copyrights
:var __license__: licence agreement
:var __url__: url entry point on documentation
:var __version__: version
:var __date__: date of the version
"""
__author__="Lutz Gross, l.gross@uq.edu.au"

import unittest
from esys.escript import *
from numpy import array


from test_util_spatial_functions2 import Test_Util_SpatialFunctions_noGradOnBoundary


class Test_Util_SpatialFunctions(Test_Util_SpatialFunctions_noGradOnBoundary):

   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.741017463552)*x[0]**o+(0.665549604887)*x[0]+(-0.294688627656)*x[1]**o+(-0.188608956137)*x[1]
        ref[0]=o*(0.741017463552)*x_ref[0]**(o-1)+(0.665549604887)
        ref[1]=o*(-0.294688627656)*x_ref[1]**(o-1)+(-0.188608956137)
      else:
        arg=(-0.182264095574)*x[0]**o+(0.388884343071)*x[0]+(0.0250471058578)*x[1]**o+(0.242724421847)*x[1]+(-0.863568419189)*x[2]**o+(-0.797427046061)*x[2]
        ref[0]=o*(-0.182264095574)*x_ref[0]**(o-1)+(0.388884343071)
        ref[1]=o*(0.0250471058578)*x_ref[1]**(o-1)+(0.242724421847)
        ref[2]=o*(-0.863568419189)*x_ref[2]**(o-1)+(-0.797427046061)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.566748845779)*x[0]**o+(0.284156548246)*x[0]+(0.213805183732)*x[1]**o+(0.887833417271)*x[1]
        arg[1]=(-0.935376067943)*x[0]**o+(0.31417056704)*x[0]+(0.157729703937)*x[1]**o+(0.781264180355)*x[1]
        ref[0,0]=o*(-0.566748845779)*x_ref[0]**(o-1)+(0.284156548246)
        ref[0,1]=o*(0.213805183732)*x_ref[1]**(o-1)+(0.887833417271)
        ref[1,0]=o*(-0.935376067943)*x_ref[0]**(o-1)+(0.31417056704)
        ref[1,1]=o*(0.157729703937)*x_ref[1]**(o-1)+(0.781264180355)
      else:
        arg[0]=(0.787106597246)*x[0]**o+(0.0477259256332)*x[0]+(0.0657839729355)*x[1]**o+(-0.367242386854)*x[1]+(0.638047637996)*x[2]**o+(0.292293368262)*x[2]
        arg[1]=(0.325241621537)*x[0]**o+(-0.995465871567)*x[0]+(-0.589833259992)*x[1]**o+(0.252602994164)*x[1]+(0.271544628727)*x[2]**o+(0.401222388174)*x[2]
        ref[0,0]=o*(0.787106597246)*x_ref[0]**(o-1)+(0.0477259256332)
        ref[0,1]=o*(0.0657839729355)*x_ref[1]**(o-1)+(-0.367242386854)
        ref[0,2]=o*(0.638047637996)*x_ref[2]**(o-1)+(0.292293368262)
        ref[1,0]=o*(0.325241621537)*x_ref[0]**(o-1)+(-0.995465871567)
        ref[1,1]=o*(-0.589833259992)*x_ref[1]**(o-1)+(0.252602994164)
        ref[1,2]=o*(0.271544628727)*x_ref[2]**(o-1)+(0.401222388174)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.535138548297)*x[0]**o+(0.320793182084)*x[0]+(-0.363556615611)*x[1]**o+(0.80028981928)*x[1]
        arg[0,1]=(0.50712290341)*x[0]**o+(0.0696582374725)*x[0]+(-0.565430065964)*x[1]**o+(0.840341269012)*x[1]
        arg[0,2]=(0.222608517758)*x[0]**o+(0.0741096149609)*x[0]+(-0.926916099396)*x[1]**o+(0.840471681255)*x[1]
        arg[0,3]=(-0.0729061065831)*x[0]**o+(-0.361006912293)*x[0]+(-0.796381694915)*x[1]**o+(-0.87399027927)*x[1]
        arg[0,4]=(0.946982642563)*x[0]**o+(0.776871793528)*x[0]+(0.906858449271)*x[1]**o+(0.23076439514)*x[1]
        arg[1,0]=(-0.121641842365)*x[0]**o+(0.755274749862)*x[0]+(-0.905815080361)*x[1]**o+(-0.27711865979)*x[1]
        arg[1,1]=(0.870675475328)*x[0]**o+(0.000245820302085)*x[0]+(0.990534371185)*x[1]**o+(-0.0132722523111)*x[1]
        arg[1,2]=(0.665375098591)*x[0]**o+(0.602779210152)*x[0]+(0.112732227406)*x[1]**o+(0.973600268944)*x[1]
        arg[1,3]=(-0.760686885095)*x[0]**o+(-0.993893837808)*x[0]+(-0.279242757928)*x[1]**o+(0.0970216781461)*x[1]
        arg[1,4]=(-0.326265155118)*x[0]**o+(-0.302971035826)*x[0]+(-0.991053864037)*x[1]**o+(-0.787776341406)*x[1]
        arg[2,0]=(-0.798558123695)*x[0]**o+(-0.648773673567)*x[0]+(0.911336567339)*x[1]**o+(0.525888531875)*x[1]
        arg[2,1]=(0.334481967771)*x[0]**o+(0.559633318828)*x[0]+(-0.738361234791)*x[1]**o+(0.863808627657)*x[1]
        arg[2,2]=(-0.660270802705)*x[0]**o+(-0.500800405285)*x[0]+(0.250991213972)*x[1]**o+(-0.855371865339)*x[1]
        arg[2,3]=(0.573495152924)*x[0]**o+(0.841810214664)*x[0]+(0.816402403997)*x[1]**o+(0.871435278968)*x[1]
        arg[2,4]=(0.00700234490903)*x[0]**o+(0.609696655384)*x[0]+(-0.908261420109)*x[1]**o+(-0.0469518512659)*x[1]
        arg[3,0]=(-0.78381944233)*x[0]**o+(0.547849085605)*x[0]+(-0.481333073199)*x[1]**o+(-0.100496681813)*x[1]
        arg[3,1]=(0.53995869966)*x[0]**o+(-0.503303606531)*x[0]+(0.783394329024)*x[1]**o+(0.989828473603)*x[1]
        arg[3,2]=(0.723590003109)*x[0]**o+(-0.75511396868)*x[0]+(0.249651386017)*x[1]**o+(0.545636109436)*x[1]
        arg[3,3]=(-0.126863462818)*x[0]**o+(0.553295312002)*x[0]+(-0.286155398085)*x[1]**o+(0.425889110789)*x[1]
        arg[3,4]=(-0.970069386779)*x[0]**o+(0.457400568855)*x[0]+(-0.245839304835)*x[1]**o+(-0.463000940469)*x[1]
        ref[0,0,0]=o*(-0.535138548297)*x_ref[0]**(o-1)+(0.320793182084)
        ref[0,0,1]=o*(-0.363556615611)*x_ref[1]**(o-1)+(0.80028981928)
        ref[0,1,0]=o*(0.50712290341)*x_ref[0]**(o-1)+(0.0696582374725)
        ref[0,1,1]=o*(-0.565430065964)*x_ref[1]**(o-1)+(0.840341269012)
        ref[0,2,0]=o*(0.222608517758)*x_ref[0]**(o-1)+(0.0741096149609)
        ref[0,2,1]=o*(-0.926916099396)*x_ref[1]**(o-1)+(0.840471681255)
        ref[0,3,0]=o*(-0.0729061065831)*x_ref[0]**(o-1)+(-0.361006912293)
        ref[0,3,1]=o*(-0.796381694915)*x_ref[1]**(o-1)+(-0.87399027927)
        ref[0,4,0]=o*(0.946982642563)*x_ref[0]**(o-1)+(0.776871793528)
        ref[0,4,1]=o*(0.906858449271)*x_ref[1]**(o-1)+(0.23076439514)
        ref[1,0,0]=o*(-0.121641842365)*x_ref[0]**(o-1)+(0.755274749862)
        ref[1,0,1]=o*(-0.905815080361)*x_ref[1]**(o-1)+(-0.27711865979)
        ref[1,1,0]=o*(0.870675475328)*x_ref[0]**(o-1)+(0.000245820302085)
        ref[1,1,1]=o*(0.990534371185)*x_ref[1]**(o-1)+(-0.0132722523111)
        ref[1,2,0]=o*(0.665375098591)*x_ref[0]**(o-1)+(0.602779210152)
        ref[1,2,1]=o*(0.112732227406)*x_ref[1]**(o-1)+(0.973600268944)
        ref[1,3,0]=o*(-0.760686885095)*x_ref[0]**(o-1)+(-0.993893837808)
        ref[1,3,1]=o*(-0.279242757928)*x_ref[1]**(o-1)+(0.0970216781461)
        ref[1,4,0]=o*(-0.326265155118)*x_ref[0]**(o-1)+(-0.302971035826)
        ref[1,4,1]=o*(-0.991053864037)*x_ref[1]**(o-1)+(-0.787776341406)
        ref[2,0,0]=o*(-0.798558123695)*x_ref[0]**(o-1)+(-0.648773673567)
        ref[2,0,1]=o*(0.911336567339)*x_ref[1]**(o-1)+(0.525888531875)
        ref[2,1,0]=o*(0.334481967771)*x_ref[0]**(o-1)+(0.559633318828)
        ref[2,1,1]=o*(-0.738361234791)*x_ref[1]**(o-1)+(0.863808627657)
        ref[2,2,0]=o*(-0.660270802705)*x_ref[0]**(o-1)+(-0.500800405285)
        ref[2,2,1]=o*(0.250991213972)*x_ref[1]**(o-1)+(-0.855371865339)
        ref[2,3,0]=o*(0.573495152924)*x_ref[0]**(o-1)+(0.841810214664)
        ref[2,3,1]=o*(0.816402403997)*x_ref[1]**(o-1)+(0.871435278968)
        ref[2,4,0]=o*(0.00700234490903)*x_ref[0]**(o-1)+(0.609696655384)
        ref[2,4,1]=o*(-0.908261420109)*x_ref[1]**(o-1)+(-0.0469518512659)
        ref[3,0,0]=o*(-0.78381944233)*x_ref[0]**(o-1)+(0.547849085605)
        ref[3,0,1]=o*(-0.481333073199)*x_ref[1]**(o-1)+(-0.100496681813)
        ref[3,1,0]=o*(0.53995869966)*x_ref[0]**(o-1)+(-0.503303606531)
        ref[3,1,1]=o*(0.783394329024)*x_ref[1]**(o-1)+(0.989828473603)
        ref[3,2,0]=o*(0.723590003109)*x_ref[0]**(o-1)+(-0.75511396868)
        ref[3,2,1]=o*(0.249651386017)*x_ref[1]**(o-1)+(0.545636109436)
        ref[3,3,0]=o*(-0.126863462818)*x_ref[0]**(o-1)+(0.553295312002)
        ref[3,3,1]=o*(-0.286155398085)*x_ref[1]**(o-1)+(0.425889110789)
        ref[3,4,0]=o*(-0.970069386779)*x_ref[0]**(o-1)+(0.457400568855)
        ref[3,4,1]=o*(-0.245839304835)*x_ref[1]**(o-1)+(-0.463000940469)
      else:
        arg[0,0]=(-0.0252675874258)*x[0]**o+(-0.998043629662)*x[0]+(-0.956603187221)*x[1]**o+(-0.642113208329)*x[1]+(-0.59687133215)*x[2]**o+(-0.222380006695)*x[2]
        arg[0,1]=(0.540057023826)*x[0]**o+(0.660021105706)*x[0]+(-0.0847035816563)*x[1]**o+(0.734935147879)*x[1]+(0.895080236848)*x[2]**o+(0.759076168225)*x[2]
        arg[0,2]=(0.208276460123)*x[0]**o+(-0.842726599314)*x[0]+(0.514146640689)*x[1]**o+(-0.931115443386)*x[1]+(-0.427990918054)*x[2]**o+(-0.0944702225036)*x[2]
        arg[0,3]=(-0.669873393665)*x[0]**o+(-0.164211687812)*x[0]+(0.257292402022)*x[1]**o+(0.788249196864)*x[1]+(-0.0160232742588)*x[2]**o+(0.593261720532)*x[2]
        arg[0,4]=(0.574685018662)*x[0]**o+(-0.404093341935)*x[0]+(-0.613704015926)*x[1]**o+(-0.388769512229)*x[1]+(0.65014059769)*x[2]**o+(0.796497720581)*x[2]
        arg[1,0]=(0.579425469692)*x[0]**o+(0.706786266065)*x[0]+(-0.489805412824)*x[1]**o+(0.412673120993)*x[1]+(-0.261487328773)*x[2]**o+(0.323016314178)*x[2]
        arg[1,1]=(-0.406115852457)*x[0]**o+(-0.320011474154)*x[0]+(0.83353566302)*x[1]**o+(-0.770757586792)*x[1]+(-0.497021878205)*x[2]**o+(-0.108546214912)*x[2]
        arg[1,2]=(-0.799159182698)*x[0]**o+(0.221424788319)*x[0]+(0.882092644719)*x[1]**o+(0.25804039903)*x[1]+(-0.788642442802)*x[2]**o+(-0.434293659417)*x[2]
        arg[1,3]=(-0.453596867354)*x[0]**o+(-0.855684542333)*x[0]+(0.421257402196)*x[1]**o+(0.162268279507)*x[1]+(-0.618396236314)*x[2]**o+(-0.650684413266)*x[2]
        arg[1,4]=(-0.464749011522)*x[0]**o+(-0.050661862641)*x[0]+(-0.7554290975)*x[1]**o+(-0.213826578781)*x[1]+(-0.536460109672)*x[2]**o+(-0.441490452069)*x[2]
        arg[2,0]=(0.335438338384)*x[0]**o+(-0.598515232086)*x[0]+(0.396958696838)*x[1]**o+(0.712094780848)*x[1]+(0.0112487672163)*x[2]**o+(-0.106361601342)*x[2]
        arg[2,1]=(0.157333767552)*x[0]**o+(0.763247557186)*x[0]+(0.454047596703)*x[1]**o+(0.874633696245)*x[1]+(0.80992740886)*x[2]**o+(-0.465565575276)*x[2]
        arg[2,2]=(-0.668932235683)*x[0]**o+(0.99055962791)*x[0]+(-0.473440344595)*x[1]**o+(-0.694789559356)*x[1]+(-0.337501506478)*x[2]**o+(-0.552461610916)*x[2]
        arg[2,3]=(-0.174040039098)*x[0]**o+(0.878604594327)*x[0]+(-0.488684761288)*x[1]**o+(-0.225626690649)*x[1]+(0.307550460093)*x[2]**o+(0.279172145442)*x[2]
        arg[2,4]=(0.771496476361)*x[0]**o+(-0.528231004311)*x[0]+(0.318620411204)*x[1]**o+(-0.831636376644)*x[1]+(-0.23477599759)*x[2]**o+(0.452986258707)*x[2]
        arg[3,0]=(0.689878836595)*x[0]**o+(-0.723887531161)*x[0]+(0.906077469738)*x[1]**o+(-0.729986756151)*x[1]+(0.860506345609)*x[2]**o+(-0.437203511895)*x[2]
        arg[3,1]=(0.867909616506)*x[0]**o+(-0.0480808079676)*x[0]+(-0.439925732789)*x[1]**o+(-0.753945614279)*x[1]+(-0.0015149141551)*x[2]**o+(-0.465559525628)*x[2]
        arg[3,2]=(0.915231983332)*x[0]**o+(0.77428136852)*x[0]+(0.902598621987)*x[1]**o+(-0.49204288782)*x[1]+(0.77362022057)*x[2]**o+(0.872135547313)*x[2]
        arg[3,3]=(0.274436414079)*x[0]**o+(-0.291721638943)*x[0]+(0.202501119852)*x[1]**o+(0.911330931145)*x[1]+(-0.61382801878)*x[2]**o+(0.386704897682)*x[2]
        arg[3,4]=(-0.0258695378217)*x[0]**o+(0.263046110195)*x[0]+(-0.162461126288)*x[1]**o+(-0.123570278155)*x[1]+(0.598012400912)*x[2]**o+(0.769951340801)*x[2]
        ref[0,0,0]=o*(-0.0252675874258)*x_ref[0]**(o-1)+(-0.998043629662)
        ref[0,0,1]=o*(-0.956603187221)*x_ref[1]**(o-1)+(-0.642113208329)
        ref[0,0,2]=o*(-0.59687133215)*x_ref[2]**(o-1)+(-0.222380006695)
        ref[0,1,0]=o*(0.540057023826)*x_ref[0]**(o-1)+(0.660021105706)
        ref[0,1,1]=o*(-0.0847035816563)*x_ref[1]**(o-1)+(0.734935147879)
        ref[0,1,2]=o*(0.895080236848)*x_ref[2]**(o-1)+(0.759076168225)
        ref[0,2,0]=o*(0.208276460123)*x_ref[0]**(o-1)+(-0.842726599314)
        ref[0,2,1]=o*(0.514146640689)*x_ref[1]**(o-1)+(-0.931115443386)
        ref[0,2,2]=o*(-0.427990918054)*x_ref[2]**(o-1)+(-0.0944702225036)
        ref[0,3,0]=o*(-0.669873393665)*x_ref[0]**(o-1)+(-0.164211687812)
        ref[0,3,1]=o*(0.257292402022)*x_ref[1]**(o-1)+(0.788249196864)
        ref[0,3,2]=o*(-0.0160232742588)*x_ref[2]**(o-1)+(0.593261720532)
        ref[0,4,0]=o*(0.574685018662)*x_ref[0]**(o-1)+(-0.404093341935)
        ref[0,4,1]=o*(-0.613704015926)*x_ref[1]**(o-1)+(-0.388769512229)
        ref[0,4,2]=o*(0.65014059769)*x_ref[2]**(o-1)+(0.796497720581)
        ref[1,0,0]=o*(0.579425469692)*x_ref[0]**(o-1)+(0.706786266065)
        ref[1,0,1]=o*(-0.489805412824)*x_ref[1]**(o-1)+(0.412673120993)
        ref[1,0,2]=o*(-0.261487328773)*x_ref[2]**(o-1)+(0.323016314178)
        ref[1,1,0]=o*(-0.406115852457)*x_ref[0]**(o-1)+(-0.320011474154)
        ref[1,1,1]=o*(0.83353566302)*x_ref[1]**(o-1)+(-0.770757586792)
        ref[1,1,2]=o*(-0.497021878205)*x_ref[2]**(o-1)+(-0.108546214912)
        ref[1,2,0]=o*(-0.799159182698)*x_ref[0]**(o-1)+(0.221424788319)
        ref[1,2,1]=o*(0.882092644719)*x_ref[1]**(o-1)+(0.25804039903)
        ref[1,2,2]=o*(-0.788642442802)*x_ref[2]**(o-1)+(-0.434293659417)
        ref[1,3,0]=o*(-0.453596867354)*x_ref[0]**(o-1)+(-0.855684542333)
        ref[1,3,1]=o*(0.421257402196)*x_ref[1]**(o-1)+(0.162268279507)
        ref[1,3,2]=o*(-0.618396236314)*x_ref[2]**(o-1)+(-0.650684413266)
        ref[1,4,0]=o*(-0.464749011522)*x_ref[0]**(o-1)+(-0.050661862641)
        ref[1,4,1]=o*(-0.7554290975)*x_ref[1]**(o-1)+(-0.213826578781)
        ref[1,4,2]=o*(-0.536460109672)*x_ref[2]**(o-1)+(-0.441490452069)
        ref[2,0,0]=o*(0.335438338384)*x_ref[0]**(o-1)+(-0.598515232086)
        ref[2,0,1]=o*(0.396958696838)*x_ref[1]**(o-1)+(0.712094780848)
        ref[2,0,2]=o*(0.0112487672163)*x_ref[2]**(o-1)+(-0.106361601342)
        ref[2,1,0]=o*(0.157333767552)*x_ref[0]**(o-1)+(0.763247557186)
        ref[2,1,1]=o*(0.454047596703)*x_ref[1]**(o-1)+(0.874633696245)
        ref[2,1,2]=o*(0.80992740886)*x_ref[2]**(o-1)+(-0.465565575276)
        ref[2,2,0]=o*(-0.668932235683)*x_ref[0]**(o-1)+(0.99055962791)
        ref[2,2,1]=o*(-0.473440344595)*x_ref[1]**(o-1)+(-0.694789559356)
        ref[2,2,2]=o*(-0.337501506478)*x_ref[2]**(o-1)+(-0.552461610916)
        ref[2,3,0]=o*(-0.174040039098)*x_ref[0]**(o-1)+(0.878604594327)
        ref[2,3,1]=o*(-0.488684761288)*x_ref[1]**(o-1)+(-0.225626690649)
        ref[2,3,2]=o*(0.307550460093)*x_ref[2]**(o-1)+(0.279172145442)
        ref[2,4,0]=o*(0.771496476361)*x_ref[0]**(o-1)+(-0.528231004311)
        ref[2,4,1]=o*(0.318620411204)*x_ref[1]**(o-1)+(-0.831636376644)
        ref[2,4,2]=o*(-0.23477599759)*x_ref[2]**(o-1)+(0.452986258707)
        ref[3,0,0]=o*(0.689878836595)*x_ref[0]**(o-1)+(-0.723887531161)
        ref[3,0,1]=o*(0.906077469738)*x_ref[1]**(o-1)+(-0.729986756151)
        ref[3,0,2]=o*(0.860506345609)*x_ref[2]**(o-1)+(-0.437203511895)
        ref[3,1,0]=o*(0.867909616506)*x_ref[0]**(o-1)+(-0.0480808079676)
        ref[3,1,1]=o*(-0.439925732789)*x_ref[1]**(o-1)+(-0.753945614279)
        ref[3,1,2]=o*(-0.0015149141551)*x_ref[2]**(o-1)+(-0.465559525628)
        ref[3,2,0]=o*(0.915231983332)*x_ref[0]**(o-1)+(0.77428136852)
        ref[3,2,1]=o*(0.902598621987)*x_ref[1]**(o-1)+(-0.49204288782)
        ref[3,2,2]=o*(0.77362022057)*x_ref[2]**(o-1)+(0.872135547313)
        ref[3,3,0]=o*(0.274436414079)*x_ref[0]**(o-1)+(-0.291721638943)
        ref[3,3,1]=o*(0.202501119852)*x_ref[1]**(o-1)+(0.911330931145)
        ref[3,3,2]=o*(-0.61382801878)*x_ref[2]**(o-1)+(0.386704897682)
        ref[3,4,0]=o*(-0.0258695378217)*x_ref[0]**(o-1)+(0.263046110195)
        ref[3,4,1]=o*(-0.162461126288)*x_ref[1]**(o-1)+(-0.123570278155)
        ref[3,4,2]=o*(0.598012400912)*x_ref[2]**(o-1)+(0.769951340801)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.301542954432)*x[0]**o+(-0.126029003998)*x[0]+(-0.408561086014)*x[1]**o+(0.090642210886)*x[1]
        arg[0,0,1]=(0.456052591865)*x[0]**o+(0.624896766511)*x[0]+(0.87137087467)*x[1]**o+(-0.296114354421)*x[1]
        arg[0,1,0]=(0.370833001032)*x[0]**o+(-0.700862384031)*x[0]+(-0.276076372016)*x[1]**o+(-0.820884025764)*x[1]
        arg[0,1,1]=(0.550805148631)*x[0]**o+(-0.628825647329)*x[0]+(0.906362219793)*x[1]**o+(0.093148354989)*x[1]
        arg[1,0,0]=(-0.80347190678)*x[0]**o+(-0.112285193049)*x[0]+(-0.703090064994)*x[1]**o+(0.738893806856)*x[1]
        arg[1,0,1]=(0.716113817265)*x[0]**o+(-0.403563175278)*x[0]+(-0.545474326725)*x[1]**o+(-0.085377932579)*x[1]
        arg[1,1,0]=(0.0151945295253)*x[0]**o+(0.419681120006)*x[0]+(0.605889120207)*x[1]**o+(0.94921670969)*x[1]
        arg[1,1,1]=(0.330888675781)*x[0]**o+(0.536336632067)*x[0]+(0.130243661103)*x[1]**o+(0.249581698903)*x[1]
        arg[2,0,0]=(-0.94168189216)*x[0]**o+(0.42899125004)*x[0]+(0.104206704223)*x[1]**o+(0.992451993183)*x[1]
        arg[2,0,1]=(-0.557898869487)*x[0]**o+(-0.0387050073802)*x[0]+(-0.880396260326)*x[1]**o+(-0.649734782543)*x[1]
        arg[2,1,0]=(0.0362873703011)*x[0]**o+(0.641116555832)*x[0]+(-0.878565186806)*x[1]**o+(-0.305825058029)*x[1]
        arg[2,1,1]=(0.926144430868)*x[0]**o+(0.207902756992)*x[0]+(-0.823834000424)*x[1]**o+(0.541980766739)*x[1]
        arg[3,0,0]=(-0.497934527226)*x[0]**o+(0.904514339359)*x[0]+(0.0627382682436)*x[1]**o+(-0.886845257384)*x[1]
        arg[3,0,1]=(0.458163004678)*x[0]**o+(-0.890966354569)*x[0]+(0.779511556433)*x[1]**o+(0.911407858547)*x[1]
        arg[3,1,0]=(0.643197533583)*x[0]**o+(-0.445952483015)*x[0]+(-0.491648538422)*x[1]**o+(-0.339803793944)*x[1]
        arg[3,1,1]=(0.564910446806)*x[0]**o+(-0.251342703228)*x[0]+(-0.102526510378)*x[1]**o+(0.989034603428)*x[1]
        arg[4,0,0]=(0.0728812877818)*x[0]**o+(-0.160839673847)*x[0]+(0.078039622926)*x[1]**o+(-0.778740146006)*x[1]
        arg[4,0,1]=(-0.848970759855)*x[0]**o+(0.83795370612)*x[0]+(-0.652802628253)*x[1]**o+(0.570900978851)*x[1]
        arg[4,1,0]=(0.492154273484)*x[0]**o+(-0.138488354185)*x[0]+(0.269142955069)*x[1]**o+(-0.849995493706)*x[1]
        arg[4,1,1]=(0.945032529684)*x[0]**o+(0.611284327327)*x[0]+(0.549856083886)*x[1]**o+(0.347585489606)*x[1]
        arg[5,0,0]=(0.481456547858)*x[0]**o+(0.238848312219)*x[0]+(-0.5849679413)*x[1]**o+(-0.749140400072)*x[1]
        arg[5,0,1]=(0.27081490311)*x[0]**o+(-0.101642948148)*x[0]+(0.31847076679)*x[1]**o+(0.192182993758)*x[1]
        arg[5,1,0]=(-0.873348151897)*x[0]**o+(-0.613499952221)*x[0]+(0.54087826152)*x[1]**o+(-0.269096416524)*x[1]
        arg[5,1,1]=(-0.803277233859)*x[0]**o+(-0.714237882827)*x[0]+(-0.205544031812)*x[1]**o+(0.188144864673)*x[1]
        ref[0,0,0,0]=o*(-0.301542954432)*x_ref[0]**(o-1)+(-0.126029003998)
        ref[0,0,0,1]=o*(-0.408561086014)*x_ref[1]**(o-1)+(0.090642210886)
        ref[0,0,1,0]=o*(0.456052591865)*x_ref[0]**(o-1)+(0.624896766511)
        ref[0,0,1,1]=o*(0.87137087467)*x_ref[1]**(o-1)+(-0.296114354421)
        ref[0,1,0,0]=o*(0.370833001032)*x_ref[0]**(o-1)+(-0.700862384031)
        ref[0,1,0,1]=o*(-0.276076372016)*x_ref[1]**(o-1)+(-0.820884025764)
        ref[0,1,1,0]=o*(0.550805148631)*x_ref[0]**(o-1)+(-0.628825647329)
        ref[0,1,1,1]=o*(0.906362219793)*x_ref[1]**(o-1)+(0.093148354989)
        ref[1,0,0,0]=o*(-0.80347190678)*x_ref[0]**(o-1)+(-0.112285193049)
        ref[1,0,0,1]=o*(-0.703090064994)*x_ref[1]**(o-1)+(0.738893806856)
        ref[1,0,1,0]=o*(0.716113817265)*x_ref[0]**(o-1)+(-0.403563175278)
        ref[1,0,1,1]=o*(-0.545474326725)*x_ref[1]**(o-1)+(-0.085377932579)
        ref[1,1,0,0]=o*(0.0151945295253)*x_ref[0]**(o-1)+(0.419681120006)
        ref[1,1,0,1]=o*(0.605889120207)*x_ref[1]**(o-1)+(0.94921670969)
        ref[1,1,1,0]=o*(0.330888675781)*x_ref[0]**(o-1)+(0.536336632067)
        ref[1,1,1,1]=o*(0.130243661103)*x_ref[1]**(o-1)+(0.249581698903)
        ref[2,0,0,0]=o*(-0.94168189216)*x_ref[0]**(o-1)+(0.42899125004)
        ref[2,0,0,1]=o*(0.104206704223)*x_ref[1]**(o-1)+(0.992451993183)
        ref[2,0,1,0]=o*(-0.557898869487)*x_ref[0]**(o-1)+(-0.0387050073802)
        ref[2,0,1,1]=o*(-0.880396260326)*x_ref[1]**(o-1)+(-0.649734782543)
        ref[2,1,0,0]=o*(0.0362873703011)*x_ref[0]**(o-1)+(0.641116555832)
        ref[2,1,0,1]=o*(-0.878565186806)*x_ref[1]**(o-1)+(-0.305825058029)
        ref[2,1,1,0]=o*(0.926144430868)*x_ref[0]**(o-1)+(0.207902756992)
        ref[2,1,1,1]=o*(-0.823834000424)*x_ref[1]**(o-1)+(0.541980766739)
        ref[3,0,0,0]=o*(-0.497934527226)*x_ref[0]**(o-1)+(0.904514339359)
        ref[3,0,0,1]=o*(0.0627382682436)*x_ref[1]**(o-1)+(-0.886845257384)
        ref[3,0,1,0]=o*(0.458163004678)*x_ref[0]**(o-1)+(-0.890966354569)
        ref[3,0,1,1]=o*(0.779511556433)*x_ref[1]**(o-1)+(0.911407858547)
        ref[3,1,0,0]=o*(0.643197533583)*x_ref[0]**(o-1)+(-0.445952483015)
        ref[3,1,0,1]=o*(-0.491648538422)*x_ref[1]**(o-1)+(-0.339803793944)
        ref[3,1,1,0]=o*(0.564910446806)*x_ref[0]**(o-1)+(-0.251342703228)
        ref[3,1,1,1]=o*(-0.102526510378)*x_ref[1]**(o-1)+(0.989034603428)
        ref[4,0,0,0]=o*(0.0728812877818)*x_ref[0]**(o-1)+(-0.160839673847)
        ref[4,0,0,1]=o*(0.078039622926)*x_ref[1]**(o-1)+(-0.778740146006)
        ref[4,0,1,0]=o*(-0.848970759855)*x_ref[0]**(o-1)+(0.83795370612)
        ref[4,0,1,1]=o*(-0.652802628253)*x_ref[1]**(o-1)+(0.570900978851)
        ref[4,1,0,0]=o*(0.492154273484)*x_ref[0]**(o-1)+(-0.138488354185)
        ref[4,1,0,1]=o*(0.269142955069)*x_ref[1]**(o-1)+(-0.849995493706)
        ref[4,1,1,0]=o*(0.945032529684)*x_ref[0]**(o-1)+(0.611284327327)
        ref[4,1,1,1]=o*(0.549856083886)*x_ref[1]**(o-1)+(0.347585489606)
        ref[5,0,0,0]=o*(0.481456547858)*x_ref[0]**(o-1)+(0.238848312219)
        ref[5,0,0,1]=o*(-0.5849679413)*x_ref[1]**(o-1)+(-0.749140400072)
        ref[5,0,1,0]=o*(0.27081490311)*x_ref[0]**(o-1)+(-0.101642948148)
        ref[5,0,1,1]=o*(0.31847076679)*x_ref[1]**(o-1)+(0.192182993758)
        ref[5,1,0,0]=o*(-0.873348151897)*x_ref[0]**(o-1)+(-0.613499952221)
        ref[5,1,0,1]=o*(0.54087826152)*x_ref[1]**(o-1)+(-0.269096416524)
        ref[5,1,1,0]=o*(-0.803277233859)*x_ref[0]**(o-1)+(-0.714237882827)
        ref[5,1,1,1]=o*(-0.205544031812)*x_ref[1]**(o-1)+(0.188144864673)
      else:
        arg[0,0,0]=(-0.639251981322)*x[0]**o+(-0.115972267818)*x[0]+(0.491134645863)*x[1]**o+(0.345343639029)*x[1]+(-0.663597762251)*x[2]**o+(-0.328150609455)*x[2]
        arg[0,0,1]=(-0.76280151252)*x[0]**o+(0.933789965647)*x[0]+(-0.117386438957)*x[1]**o+(0.738801880577)*x[1]+(-0.823759196233)*x[2]**o+(-0.591225057134)*x[2]
        arg[0,1,0]=(-0.515479158328)*x[0]**o+(-0.498840147412)*x[0]+(-0.07870152457)*x[1]**o+(0.240823897179)*x[1]+(-0.529483054724)*x[2]**o+(0.957444878465)*x[2]
        arg[0,1,1]=(0.659163197783)*x[0]**o+(-0.987017484017)*x[0]+(0.316073196672)*x[1]**o+(-0.858184573568)*x[1]+(-0.324909144593)*x[2]**o+(-0.89624674398)*x[2]
        arg[1,0,0]=(0.582390600104)*x[0]**o+(-0.808724939916)*x[0]+(-0.204186916478)*x[1]**o+(0.63212764696)*x[1]+(-0.272947627588)*x[2]**o+(-0.133369751815)*x[2]
        arg[1,0,1]=(-0.955892141909)*x[0]**o+(0.915832405494)*x[0]+(-0.823428836636)*x[1]**o+(-0.527176552593)*x[1]+(-0.439872810771)*x[2]**o+(0.282431362763)*x[2]
        arg[1,1,0]=(0.269192050139)*x[0]**o+(0.304289733459)*x[0]+(-0.252137123064)*x[1]**o+(0.159346881758)*x[1]+(-0.875713938743)*x[2]**o+(-0.804681982204)*x[2]
        arg[1,1,1]=(0.100159906585)*x[0]**o+(0.652670710888)*x[0]+(-0.0471177296503)*x[1]**o+(0.15344212377)*x[1]+(0.162356023341)*x[2]**o+(0.242855769737)*x[2]
        arg[2,0,0]=(-0.0214334988311)*x[0]**o+(0.444654672491)*x[0]+(0.235478960156)*x[1]**o+(0.62191559883)*x[1]+(0.124531773653)*x[2]**o+(-0.603748942811)*x[2]
        arg[2,0,1]=(-0.00696114124898)*x[0]**o+(-0.350635993795)*x[0]+(0.766513244466)*x[1]**o+(-0.99996024111)*x[1]+(0.976036637033)*x[2]**o+(0.00703160282837)*x[2]
        arg[2,1,0]=(-0.210499708873)*x[0]**o+(-0.763641982769)*x[0]+(0.435435481771)*x[1]**o+(0.0886400511888)*x[1]+(0.540913993886)*x[2]**o+(-0.864904989707)*x[2]
        arg[2,1,1]=(0.609017357846)*x[0]**o+(-0.681305727596)*x[0]+(-0.291763309802)*x[1]**o+(0.12465245807)*x[1]+(-0.265362499082)*x[2]**o+(-0.978509447338)*x[2]
        arg[3,0,0]=(0.622861389608)*x[0]**o+(-0.341108338962)*x[0]+(0.908588438318)*x[1]**o+(-0.583012719535)*x[1]+(0.33513654797)*x[2]**o+(-0.835513934636)*x[2]
        arg[3,0,1]=(0.164841305439)*x[0]**o+(-0.622242302911)*x[0]+(-0.0469305037163)*x[1]**o+(-0.859517953063)*x[1]+(-0.112485630662)*x[2]**o+(0.31475831421)*x[2]
        arg[3,1,0]=(-0.177913851996)*x[0]**o+(0.74607750787)*x[0]+(-0.188838634745)*x[1]**o+(-0.803617744529)*x[1]+(0.18839438068)*x[2]**o+(-0.857696671728)*x[2]
        arg[3,1,1]=(0.485172697497)*x[0]**o+(-0.554750723313)*x[0]+(-0.59838039994)*x[1]**o+(-0.83177053032)*x[1]+(0.655490277067)*x[2]**o+(0.0645924856059)*x[2]
        arg[4,0,0]=(0.900678166492)*x[0]**o+(-0.485573346705)*x[0]+(0.0108149783491)*x[1]**o+(0.982616024342)*x[1]+(-0.374371784319)*x[2]**o+(-0.825014166204)*x[2]
        arg[4,0,1]=(0.682753472393)*x[0]**o+(0.810927347906)*x[0]+(-0.128780174224)*x[1]**o+(0.514607304836)*x[1]+(0.606285011416)*x[2]**o+(-0.691191486471)*x[2]
        arg[4,1,0]=(0.0103661338625)*x[0]**o+(0.184111752707)*x[0]+(-0.270751890504)*x[1]**o+(0.207351712222)*x[1]+(-0.366280871535)*x[2]**o+(-0.638450092731)*x[2]
        arg[4,1,1]=(-0.111302974207)*x[0]**o+(-0.520252159917)*x[0]+(-0.911197545741)*x[1]**o+(-0.779732160528)*x[1]+(-0.443379349795)*x[2]**o+(-0.783540988384)*x[2]
        arg[5,0,0]=(0.319701766792)*x[0]**o+(-0.953929305858)*x[0]+(-0.783942145061)*x[1]**o+(0.687992583635)*x[1]+(0.873868267323)*x[2]**o+(-0.204836707955)*x[2]
        arg[5,0,1]=(-0.240340717622)*x[0]**o+(-0.799598459684)*x[0]+(-0.827096078861)*x[1]**o+(0.229071234102)*x[1]+(0.309197705487)*x[2]**o+(-0.286088934882)*x[2]
        arg[5,1,0]=(-0.0926450319507)*x[0]**o+(0.638985747884)*x[0]+(0.604913216138)*x[1]**o+(-0.364251815312)*x[1]+(-0.47971664069)*x[2]**o+(-0.20655528518)*x[2]
        arg[5,1,1]=(0.418341452333)*x[0]**o+(0.430002851684)*x[0]+(0.624025385599)*x[1]**o+(0.338601201629)*x[1]+(0.0900364406242)*x[2]**o+(0.193533337073)*x[2]
        ref[0,0,0,0]=o*(-0.639251981322)*x_ref[0]**(o-1)+(-0.115972267818)
        ref[0,0,0,1]=o*(0.491134645863)*x_ref[1]**(o-1)+(0.345343639029)
        ref[0,0,0,2]=o*(-0.663597762251)*x_ref[2]**(o-1)+(-0.328150609455)
        ref[0,0,1,0]=o*(-0.76280151252)*x_ref[0]**(o-1)+(0.933789965647)
        ref[0,0,1,1]=o*(-0.117386438957)*x_ref[1]**(o-1)+(0.738801880577)
        ref[0,0,1,2]=o*(-0.823759196233)*x_ref[2]**(o-1)+(-0.591225057134)
        ref[0,1,0,0]=o*(-0.515479158328)*x_ref[0]**(o-1)+(-0.498840147412)
        ref[0,1,0,1]=o*(-0.07870152457)*x_ref[1]**(o-1)+(0.240823897179)
        ref[0,1,0,2]=o*(-0.529483054724)*x_ref[2]**(o-1)+(0.957444878465)
        ref[0,1,1,0]=o*(0.659163197783)*x_ref[0]**(o-1)+(-0.987017484017)
        ref[0,1,1,1]=o*(0.316073196672)*x_ref[1]**(o-1)+(-0.858184573568)
        ref[0,1,1,2]=o*(-0.324909144593)*x_ref[2]**(o-1)+(-0.89624674398)
        ref[1,0,0,0]=o*(0.582390600104)*x_ref[0]**(o-1)+(-0.808724939916)
        ref[1,0,0,1]=o*(-0.204186916478)*x_ref[1]**(o-1)+(0.63212764696)
        ref[1,0,0,2]=o*(-0.272947627588)*x_ref[2]**(o-1)+(-0.133369751815)
        ref[1,0,1,0]=o*(-0.955892141909)*x_ref[0]**(o-1)+(0.915832405494)
        ref[1,0,1,1]=o*(-0.823428836636)*x_ref[1]**(o-1)+(-0.527176552593)
        ref[1,0,1,2]=o*(-0.439872810771)*x_ref[2]**(o-1)+(0.282431362763)
        ref[1,1,0,0]=o*(0.269192050139)*x_ref[0]**(o-1)+(0.304289733459)
        ref[1,1,0,1]=o*(-0.252137123064)*x_ref[1]**(o-1)+(0.159346881758)
        ref[1,1,0,2]=o*(-0.875713938743)*x_ref[2]**(o-1)+(-0.804681982204)
        ref[1,1,1,0]=o*(0.100159906585)*x_ref[0]**(o-1)+(0.652670710888)
        ref[1,1,1,1]=o*(-0.0471177296503)*x_ref[1]**(o-1)+(0.15344212377)
        ref[1,1,1,2]=o*(0.162356023341)*x_ref[2]**(o-1)+(0.242855769737)
        ref[2,0,0,0]=o*(-0.0214334988311)*x_ref[0]**(o-1)+(0.444654672491)
        ref[2,0,0,1]=o*(0.235478960156)*x_ref[1]**(o-1)+(0.62191559883)
        ref[2,0,0,2]=o*(0.124531773653)*x_ref[2]**(o-1)+(-0.603748942811)
        ref[2,0,1,0]=o*(-0.00696114124898)*x_ref[0]**(o-1)+(-0.350635993795)
        ref[2,0,1,1]=o*(0.766513244466)*x_ref[1]**(o-1)+(-0.99996024111)
        ref[2,0,1,2]=o*(0.976036637033)*x_ref[2]**(o-1)+(0.00703160282837)
        ref[2,1,0,0]=o*(-0.210499708873)*x_ref[0]**(o-1)+(-0.763641982769)
        ref[2,1,0,1]=o*(0.435435481771)*x_ref[1]**(o-1)+(0.0886400511888)
        ref[2,1,0,2]=o*(0.540913993886)*x_ref[2]**(o-1)+(-0.864904989707)
        ref[2,1,1,0]=o*(0.609017357846)*x_ref[0]**(o-1)+(-0.681305727596)
        ref[2,1,1,1]=o*(-0.291763309802)*x_ref[1]**(o-1)+(0.12465245807)
        ref[2,1,1,2]=o*(-0.265362499082)*x_ref[2]**(o-1)+(-0.978509447338)
        ref[3,0,0,0]=o*(0.622861389608)*x_ref[0]**(o-1)+(-0.341108338962)
        ref[3,0,0,1]=o*(0.908588438318)*x_ref[1]**(o-1)+(-0.583012719535)
        ref[3,0,0,2]=o*(0.33513654797)*x_ref[2]**(o-1)+(-0.835513934636)
        ref[3,0,1,0]=o*(0.164841305439)*x_ref[0]**(o-1)+(-0.622242302911)
        ref[3,0,1,1]=o*(-0.0469305037163)*x_ref[1]**(o-1)+(-0.859517953063)
        ref[3,0,1,2]=o*(-0.112485630662)*x_ref[2]**(o-1)+(0.31475831421)
        ref[3,1,0,0]=o*(-0.177913851996)*x_ref[0]**(o-1)+(0.74607750787)
        ref[3,1,0,1]=o*(-0.188838634745)*x_ref[1]**(o-1)+(-0.803617744529)
        ref[3,1,0,2]=o*(0.18839438068)*x_ref[2]**(o-1)+(-0.857696671728)
        ref[3,1,1,0]=o*(0.485172697497)*x_ref[0]**(o-1)+(-0.554750723313)
        ref[3,1,1,1]=o*(-0.59838039994)*x_ref[1]**(o-1)+(-0.83177053032)
        ref[3,1,1,2]=o*(0.655490277067)*x_ref[2]**(o-1)+(0.0645924856059)
        ref[4,0,0,0]=o*(0.900678166492)*x_ref[0]**(o-1)+(-0.485573346705)
        ref[4,0,0,1]=o*(0.0108149783491)*x_ref[1]**(o-1)+(0.982616024342)
        ref[4,0,0,2]=o*(-0.374371784319)*x_ref[2]**(o-1)+(-0.825014166204)
        ref[4,0,1,0]=o*(0.682753472393)*x_ref[0]**(o-1)+(0.810927347906)
        ref[4,0,1,1]=o*(-0.128780174224)*x_ref[1]**(o-1)+(0.514607304836)
        ref[4,0,1,2]=o*(0.606285011416)*x_ref[2]**(o-1)+(-0.691191486471)
        ref[4,1,0,0]=o*(0.0103661338625)*x_ref[0]**(o-1)+(0.184111752707)
        ref[4,1,0,1]=o*(-0.270751890504)*x_ref[1]**(o-1)+(0.207351712222)
        ref[4,1,0,2]=o*(-0.366280871535)*x_ref[2]**(o-1)+(-0.638450092731)
        ref[4,1,1,0]=o*(-0.111302974207)*x_ref[0]**(o-1)+(-0.520252159917)
        ref[4,1,1,1]=o*(-0.911197545741)*x_ref[1]**(o-1)+(-0.779732160528)
        ref[4,1,1,2]=o*(-0.443379349795)*x_ref[2]**(o-1)+(-0.783540988384)
        ref[5,0,0,0]=o*(0.319701766792)*x_ref[0]**(o-1)+(-0.953929305858)
        ref[5,0,0,1]=o*(-0.783942145061)*x_ref[1]**(o-1)+(0.687992583635)
        ref[5,0,0,2]=o*(0.873868267323)*x_ref[2]**(o-1)+(-0.204836707955)
        ref[5,0,1,0]=o*(-0.240340717622)*x_ref[0]**(o-1)+(-0.799598459684)
        ref[5,0,1,1]=o*(-0.827096078861)*x_ref[1]**(o-1)+(0.229071234102)
        ref[5,0,1,2]=o*(0.309197705487)*x_ref[2]**(o-1)+(-0.286088934882)
        ref[5,1,0,0]=o*(-0.0926450319507)*x_ref[0]**(o-1)+(0.638985747884)
        ref[5,1,0,1]=o*(0.604913216138)*x_ref[1]**(o-1)+(-0.364251815312)
        ref[5,1,0,2]=o*(-0.47971664069)*x_ref[2]**(o-1)+(-0.20655528518)
        ref[5,1,1,0]=o*(0.418341452333)*x_ref[0]**(o-1)+(0.430002851684)
        ref[5,1,1,1]=o*(0.624025385599)*x_ref[1]**(o-1)+(0.338601201629)
        ref[5,1,1,2]=o*(0.0900364406242)*x_ref[2]**(o-1)+(0.193533337073)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_Solution_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.828412007781)*x[0]**o+(0.392494008682)*x[0]+(0.424780018344)*x[1]**o+(-0.75790685259)*x[1]
        ref[0]=o*(0.828412007781)*x_ref[0]**(o-1)+(0.392494008682)
        ref[1]=o*(0.424780018344)*x_ref[1]**(o-1)+(-0.75790685259)
      else:
        arg=(0.0243949626425)*x[0]**o+(0.99040714323)*x[0]+(0.455478130443)*x[1]**o+(-0.917504050554)*x[1]+(-0.642882027126)*x[2]**o+(-0.0751962677977)*x[2]
        ref[0]=o*(0.0243949626425)*x_ref[0]**(o-1)+(0.99040714323)
        ref[1]=o*(0.455478130443)*x_ref[1]**(o-1)+(-0.917504050554)
        ref[2]=o*(-0.642882027126)*x_ref[2]**(o-1)+(-0.0751962677977)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_Solution_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.107613645086)*x[0]**o+(-0.942156383662)*x[0]+(0.294425370324)*x[1]**o+(-0.177019031098)*x[1]
        arg[1]=(-0.120283674295)*x[0]**o+(-0.0644985036825)*x[0]+(-0.818412208612)*x[1]**o+(-0.375335380433)*x[1]
        ref[0,0]=o*(-0.107613645086)*x_ref[0]**(o-1)+(-0.942156383662)
        ref[0,1]=o*(0.294425370324)*x_ref[1]**(o-1)+(-0.177019031098)
        ref[1,0]=o*(-0.120283674295)*x_ref[0]**(o-1)+(-0.0644985036825)
        ref[1,1]=o*(-0.818412208612)*x_ref[1]**(o-1)+(-0.375335380433)
      else:
        arg[0]=(-0.888739890088)*x[0]**o+(0.874877363684)*x[0]+(-0.104101311962)*x[1]**o+(-0.124590333446)*x[1]+(-0.311645832684)*x[2]**o+(-0.249470325769)*x[2]
        arg[1]=(0.680288601947)*x[0]**o+(0.0595361804285)*x[0]+(-0.477099840976)*x[1]**o+(-0.695340106154)*x[1]+(-0.371044873416)*x[2]**o+(-0.442877128189)*x[2]
        ref[0,0]=o*(-0.888739890088)*x_ref[0]**(o-1)+(0.874877363684)
        ref[0,1]=o*(-0.104101311962)*x_ref[1]**(o-1)+(-0.124590333446)
        ref[0,2]=o*(-0.311645832684)*x_ref[2]**(o-1)+(-0.249470325769)
        ref[1,0]=o*(0.680288601947)*x_ref[0]**(o-1)+(0.0595361804285)
        ref[1,1]=o*(-0.477099840976)*x_ref[1]**(o-1)+(-0.695340106154)
        ref[1,2]=o*(-0.371044873416)*x_ref[2]**(o-1)+(-0.442877128189)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_Solution_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.0237486351119)*x[0]**o+(0.751375149587)*x[0]+(0.0776560601509)*x[1]**o+(-0.363340006998)*x[1]
        arg[0,1]=(0.129612138219)*x[0]**o+(0.377878821946)*x[0]+(0.166470268672)*x[1]**o+(-0.0442064974104)*x[1]
        arg[0,2]=(0.692350462477)*x[0]**o+(-0.439155965586)*x[0]+(0.655583353562)*x[1]**o+(0.3753639938)*x[1]
        arg[0,3]=(-0.909698820615)*x[0]**o+(0.382887632024)*x[0]+(-0.108821308794)*x[1]**o+(0.330111344773)*x[1]
        arg[0,4]=(-0.694552793316)*x[0]**o+(-0.38920298235)*x[0]+(0.174040833682)*x[1]**o+(-0.41756882346)*x[1]
        arg[1,0]=(-0.194901264915)*x[0]**o+(-0.325546767846)*x[0]+(0.507614203136)*x[1]**o+(-0.964255763976)*x[1]
        arg[1,1]=(-0.634480793567)*x[0]**o+(0.787029027123)*x[0]+(-0.067769912764)*x[1]**o+(0.439969554037)*x[1]
        arg[1,2]=(-0.59461925815)*x[0]**o+(-0.799215148911)*x[0]+(-0.286848861016)*x[1]**o+(-0.0274325066994)*x[1]
        arg[1,3]=(0.564664310607)*x[0]**o+(-0.00188717695265)*x[0]+(0.984042638249)*x[1]**o+(-0.524339915306)*x[1]
        arg[1,4]=(-0.673190201485)*x[0]**o+(0.149574403846)*x[0]+(0.667179612419)*x[1]**o+(-0.279377079395)*x[1]
        arg[2,0]=(0.270490028978)*x[0]**o+(0.498901404478)*x[0]+(0.485707956493)*x[1]**o+(-0.340861889384)*x[1]
        arg[2,1]=(-0.785952730285)*x[0]**o+(0.514446635701)*x[0]+(0.925557655147)*x[1]**o+(0.616971745561)*x[1]
        arg[2,2]=(0.788196502068)*x[0]**o+(0.651461860731)*x[0]+(0.160564742956)*x[1]**o+(-0.681831134057)*x[1]
        arg[2,3]=(0.892258488111)*x[0]**o+(0.752252063583)*x[0]+(0.853514779349)*x[1]**o+(-0.181884189513)*x[1]
        arg[2,4]=(-0.824886296933)*x[0]**o+(0.211931432354)*x[0]+(0.924362197939)*x[1]**o+(0.036482432278)*x[1]
        arg[3,0]=(-0.0616004277583)*x[0]**o+(-0.77338193907)*x[0]+(-0.966899521855)*x[1]**o+(-0.395764830291)*x[1]
        arg[3,1]=(-0.607976110595)*x[0]**o+(-0.448875768616)*x[0]+(-0.414034580078)*x[1]**o+(0.185483819416)*x[1]
        arg[3,2]=(-0.918071244524)*x[0]**o+(0.924225885038)*x[0]+(0.418000509852)*x[1]**o+(0.431330768344)*x[1]
        arg[3,3]=(0.897504318738)*x[0]**o+(0.66265738089)*x[0]+(0.720830604485)*x[1]**o+(-0.713433930025)*x[1]
        arg[3,4]=(0.526823069245)*x[0]**o+(-0.255965317744)*x[0]+(-0.326371546564)*x[1]**o+(-0.436881501009)*x[1]
        ref[0,0,0]=o*(0.0237486351119)*x_ref[0]**(o-1)+(0.751375149587)
        ref[0,0,1]=o*(0.0776560601509)*x_ref[1]**(o-1)+(-0.363340006998)
        ref[0,1,0]=o*(0.129612138219)*x_ref[0]**(o-1)+(0.377878821946)
        ref[0,1,1]=o*(0.166470268672)*x_ref[1]**(o-1)+(-0.0442064974104)
        ref[0,2,0]=o*(0.692350462477)*x_ref[0]**(o-1)+(-0.439155965586)
        ref[0,2,1]=o*(0.655583353562)*x_ref[1]**(o-1)+(0.3753639938)
        ref[0,3,0]=o*(-0.909698820615)*x_ref[0]**(o-1)+(0.382887632024)
        ref[0,3,1]=o*(-0.108821308794)*x_ref[1]**(o-1)+(0.330111344773)
        ref[0,4,0]=o*(-0.694552793316)*x_ref[0]**(o-1)+(-0.38920298235)
        ref[0,4,1]=o*(0.174040833682)*x_ref[1]**(o-1)+(-0.41756882346)
        ref[1,0,0]=o*(-0.194901264915)*x_ref[0]**(o-1)+(-0.325546767846)
        ref[1,0,1]=o*(0.507614203136)*x_ref[1]**(o-1)+(-0.964255763976)
        ref[1,1,0]=o*(-0.634480793567)*x_ref[0]**(o-1)+(0.787029027123)
        ref[1,1,1]=o*(-0.067769912764)*x_ref[1]**(o-1)+(0.439969554037)
        ref[1,2,0]=o*(-0.59461925815)*x_ref[0]**(o-1)+(-0.799215148911)
        ref[1,2,1]=o*(-0.286848861016)*x_ref[1]**(o-1)+(-0.0274325066994)
        ref[1,3,0]=o*(0.564664310607)*x_ref[0]**(o-1)+(-0.00188717695265)
        ref[1,3,1]=o*(0.984042638249)*x_ref[1]**(o-1)+(-0.524339915306)
        ref[1,4,0]=o*(-0.673190201485)*x_ref[0]**(o-1)+(0.149574403846)
        ref[1,4,1]=o*(0.667179612419)*x_ref[1]**(o-1)+(-0.279377079395)
        ref[2,0,0]=o*(0.270490028978)*x_ref[0]**(o-1)+(0.498901404478)
        ref[2,0,1]=o*(0.485707956493)*x_ref[1]**(o-1)+(-0.340861889384)
        ref[2,1,0]=o*(-0.785952730285)*x_ref[0]**(o-1)+(0.514446635701)
        ref[2,1,1]=o*(0.925557655147)*x_ref[1]**(o-1)+(0.616971745561)
        ref[2,2,0]=o*(0.788196502068)*x_ref[0]**(o-1)+(0.651461860731)
        ref[2,2,1]=o*(0.160564742956)*x_ref[1]**(o-1)+(-0.681831134057)
        ref[2,3,0]=o*(0.892258488111)*x_ref[0]**(o-1)+(0.752252063583)
        ref[2,3,1]=o*(0.853514779349)*x_ref[1]**(o-1)+(-0.181884189513)
        ref[2,4,0]=o*(-0.824886296933)*x_ref[0]**(o-1)+(0.211931432354)
        ref[2,4,1]=o*(0.924362197939)*x_ref[1]**(o-1)+(0.036482432278)
        ref[3,0,0]=o*(-0.0616004277583)*x_ref[0]**(o-1)+(-0.77338193907)
        ref[3,0,1]=o*(-0.966899521855)*x_ref[1]**(o-1)+(-0.395764830291)
        ref[3,1,0]=o*(-0.607976110595)*x_ref[0]**(o-1)+(-0.448875768616)
        ref[3,1,1]=o*(-0.414034580078)*x_ref[1]**(o-1)+(0.185483819416)
        ref[3,2,0]=o*(-0.918071244524)*x_ref[0]**(o-1)+(0.924225885038)
        ref[3,2,1]=o*(0.418000509852)*x_ref[1]**(o-1)+(0.431330768344)
        ref[3,3,0]=o*(0.897504318738)*x_ref[0]**(o-1)+(0.66265738089)
        ref[3,3,1]=o*(0.720830604485)*x_ref[1]**(o-1)+(-0.713433930025)
        ref[3,4,0]=o*(0.526823069245)*x_ref[0]**(o-1)+(-0.255965317744)
        ref[3,4,1]=o*(-0.326371546564)*x_ref[1]**(o-1)+(-0.436881501009)
      else:
        arg[0,0]=(-0.66471814098)*x[0]**o+(0.417241001417)*x[0]+(-0.228677853733)*x[1]**o+(-0.706946716128)*x[1]+(0.977168565988)*x[2]**o+(-0.0293128613918)*x[2]
        arg[0,1]=(-0.161577085067)*x[0]**o+(0.86957667281)*x[0]+(-0.246783815579)*x[1]**o+(0.495318651008)*x[1]+(-0.500962203485)*x[2]**o+(0.903521194042)*x[2]
        arg[0,2]=(0.0764824142976)*x[0]**o+(-0.0831710166954)*x[0]+(0.538063052387)*x[1]**o+(0.472451075914)*x[1]+(-0.354702552367)*x[2]**o+(-0.820126246007)*x[2]
        arg[0,3]=(-0.286219487301)*x[0]**o+(0.561959250696)*x[0]+(-0.65668352495)*x[1]**o+(-0.276560290133)*x[1]+(-0.500866980002)*x[2]**o+(0.71188245613)*x[2]
        arg[0,4]=(-0.13704169085)*x[0]**o+(0.107954139186)*x[0]+(0.498233399424)*x[1]**o+(-0.201539549426)*x[1]+(-0.889578977028)*x[2]**o+(0.33670078332)*x[2]
        arg[1,0]=(0.411075168668)*x[0]**o+(-0.982783292982)*x[0]+(0.919652087888)*x[1]**o+(0.657917523808)*x[1]+(-0.220747064163)*x[2]**o+(-0.65863499098)*x[2]
        arg[1,1]=(0.978122893048)*x[0]**o+(-0.638310308996)*x[0]+(-0.182955841813)*x[1]**o+(0.386322350256)*x[1]+(0.0238129376001)*x[2]**o+(0.415141411589)*x[2]
        arg[1,2]=(-0.410102590894)*x[0]**o+(-0.0527422034609)*x[0]+(0.218532590216)*x[1]**o+(0.338645963161)*x[1]+(-0.559262295901)*x[2]**o+(0.00154425206109)*x[2]
        arg[1,3]=(-0.509603507286)*x[0]**o+(0.220014659177)*x[0]+(-0.850244819031)*x[1]**o+(0.851103142912)*x[1]+(0.485740207202)*x[2]**o+(0.625073715619)*x[2]
        arg[1,4]=(-0.990056784415)*x[0]**o+(0.822920800823)*x[0]+(-0.310061384265)*x[1]**o+(-0.337394401914)*x[1]+(0.139776472055)*x[2]**o+(0.615477120599)*x[2]
        arg[2,0]=(0.434733128443)*x[0]**o+(-0.23992608444)*x[0]+(0.532847685888)*x[1]**o+(0.641963266518)*x[1]+(-0.421857805771)*x[2]**o+(0.385408885641)*x[2]
        arg[2,1]=(0.502146118236)*x[0]**o+(-0.476835388064)*x[0]+(-0.676891574566)*x[1]**o+(0.713401740075)*x[1]+(0.220183315534)*x[2]**o+(-0.863327755288)*x[2]
        arg[2,2]=(-0.694614904655)*x[0]**o+(-0.881664834998)*x[0]+(0.318605270866)*x[1]**o+(0.55884249433)*x[1]+(0.286031997395)*x[2]**o+(-0.947477517785)*x[2]
        arg[2,3]=(-0.046201338948)*x[0]**o+(-0.316456331808)*x[0]+(0.968975602807)*x[1]**o+(0.549024517551)*x[1]+(-0.473260734234)*x[2]**o+(0.889462696171)*x[2]
        arg[2,4]=(0.316754513953)*x[0]**o+(0.43553288512)*x[0]+(0.962932728727)*x[1]**o+(0.887900662435)*x[1]+(-0.235820939423)*x[2]**o+(0.947057541177)*x[2]
        arg[3,0]=(-0.835914763446)*x[0]**o+(-0.488811438785)*x[0]+(-0.573283060197)*x[1]**o+(0.963078263225)*x[1]+(0.601345307969)*x[2]**o+(-0.185658285596)*x[2]
        arg[3,1]=(0.597606596118)*x[0]**o+(0.0839878651552)*x[0]+(0.0836232765815)*x[1]**o+(0.594866612681)*x[1]+(-0.361174348867)*x[2]**o+(0.701364726038)*x[2]
        arg[3,2]=(-0.172328462662)*x[0]**o+(-0.856154059105)*x[0]+(0.533657669403)*x[1]**o+(-0.305657234912)*x[1]+(0.828787414246)*x[2]**o+(0.993286809537)*x[2]
        arg[3,3]=(0.181845363409)*x[0]**o+(0.184210025662)*x[0]+(-0.538269933964)*x[1]**o+(0.731384195858)*x[1]+(0.900272670238)*x[2]**o+(-0.152731605913)*x[2]
        arg[3,4]=(0.812640675362)*x[0]**o+(-0.330495179989)*x[0]+(0.455533774723)*x[1]**o+(0.32100819695)*x[1]+(-0.332730198093)*x[2]**o+(-0.692589097458)*x[2]
        ref[0,0,0]=o*(-0.66471814098)*x_ref[0]**(o-1)+(0.417241001417)
        ref[0,0,1]=o*(-0.228677853733)*x_ref[1]**(o-1)+(-0.706946716128)
        ref[0,0,2]=o*(0.977168565988)*x_ref[2]**(o-1)+(-0.0293128613918)
        ref[0,1,0]=o*(-0.161577085067)*x_ref[0]**(o-1)+(0.86957667281)
        ref[0,1,1]=o*(-0.246783815579)*x_ref[1]**(o-1)+(0.495318651008)
        ref[0,1,2]=o*(-0.500962203485)*x_ref[2]**(o-1)+(0.903521194042)
        ref[0,2,0]=o*(0.0764824142976)*x_ref[0]**(o-1)+(-0.0831710166954)
        ref[0,2,1]=o*(0.538063052387)*x_ref[1]**(o-1)+(0.472451075914)
        ref[0,2,2]=o*(-0.354702552367)*x_ref[2]**(o-1)+(-0.820126246007)
        ref[0,3,0]=o*(-0.286219487301)*x_ref[0]**(o-1)+(0.561959250696)
        ref[0,3,1]=o*(-0.65668352495)*x_ref[1]**(o-1)+(-0.276560290133)
        ref[0,3,2]=o*(-0.500866980002)*x_ref[2]**(o-1)+(0.71188245613)
        ref[0,4,0]=o*(-0.13704169085)*x_ref[0]**(o-1)+(0.107954139186)
        ref[0,4,1]=o*(0.498233399424)*x_ref[1]**(o-1)+(-0.201539549426)
        ref[0,4,2]=o*(-0.889578977028)*x_ref[2]**(o-1)+(0.33670078332)
        ref[1,0,0]=o*(0.411075168668)*x_ref[0]**(o-1)+(-0.982783292982)
        ref[1,0,1]=o*(0.919652087888)*x_ref[1]**(o-1)+(0.657917523808)
        ref[1,0,2]=o*(-0.220747064163)*x_ref[2]**(o-1)+(-0.65863499098)
        ref[1,1,0]=o*(0.978122893048)*x_ref[0]**(o-1)+(-0.638310308996)
        ref[1,1,1]=o*(-0.182955841813)*x_ref[1]**(o-1)+(0.386322350256)
        ref[1,1,2]=o*(0.0238129376001)*x_ref[2]**(o-1)+(0.415141411589)
        ref[1,2,0]=o*(-0.410102590894)*x_ref[0]**(o-1)+(-0.0527422034609)
        ref[1,2,1]=o*(0.218532590216)*x_ref[1]**(o-1)+(0.338645963161)
        ref[1,2,2]=o*(-0.559262295901)*x_ref[2]**(o-1)+(0.00154425206109)
        ref[1,3,0]=o*(-0.509603507286)*x_ref[0]**(o-1)+(0.220014659177)
        ref[1,3,1]=o*(-0.850244819031)*x_ref[1]**(o-1)+(0.851103142912)
        ref[1,3,2]=o*(0.485740207202)*x_ref[2]**(o-1)+(0.625073715619)
        ref[1,4,0]=o*(-0.990056784415)*x_ref[0]**(o-1)+(0.822920800823)
        ref[1,4,1]=o*(-0.310061384265)*x_ref[1]**(o-1)+(-0.337394401914)
        ref[1,4,2]=o*(0.139776472055)*x_ref[2]**(o-1)+(0.615477120599)
        ref[2,0,0]=o*(0.434733128443)*x_ref[0]**(o-1)+(-0.23992608444)
        ref[2,0,1]=o*(0.532847685888)*x_ref[1]**(o-1)+(0.641963266518)
        ref[2,0,2]=o*(-0.421857805771)*x_ref[2]**(o-1)+(0.385408885641)
        ref[2,1,0]=o*(0.502146118236)*x_ref[0]**(o-1)+(-0.476835388064)
        ref[2,1,1]=o*(-0.676891574566)*x_ref[1]**(o-1)+(0.713401740075)
        ref[2,1,2]=o*(0.220183315534)*x_ref[2]**(o-1)+(-0.863327755288)
        ref[2,2,0]=o*(-0.694614904655)*x_ref[0]**(o-1)+(-0.881664834998)
        ref[2,2,1]=o*(0.318605270866)*x_ref[1]**(o-1)+(0.55884249433)
        ref[2,2,2]=o*(0.286031997395)*x_ref[2]**(o-1)+(-0.947477517785)
        ref[2,3,0]=o*(-0.046201338948)*x_ref[0]**(o-1)+(-0.316456331808)
        ref[2,3,1]=o*(0.968975602807)*x_ref[1]**(o-1)+(0.549024517551)
        ref[2,3,2]=o*(-0.473260734234)*x_ref[2]**(o-1)+(0.889462696171)
        ref[2,4,0]=o*(0.316754513953)*x_ref[0]**(o-1)+(0.43553288512)
        ref[2,4,1]=o*(0.962932728727)*x_ref[1]**(o-1)+(0.887900662435)
        ref[2,4,2]=o*(-0.235820939423)*x_ref[2]**(o-1)+(0.947057541177)
        ref[3,0,0]=o*(-0.835914763446)*x_ref[0]**(o-1)+(-0.488811438785)
        ref[3,0,1]=o*(-0.573283060197)*x_ref[1]**(o-1)+(0.963078263225)
        ref[3,0,2]=o*(0.601345307969)*x_ref[2]**(o-1)+(-0.185658285596)
        ref[3,1,0]=o*(0.597606596118)*x_ref[0]**(o-1)+(0.0839878651552)
        ref[3,1,1]=o*(0.0836232765815)*x_ref[1]**(o-1)+(0.594866612681)
        ref[3,1,2]=o*(-0.361174348867)*x_ref[2]**(o-1)+(0.701364726038)
        ref[3,2,0]=o*(-0.172328462662)*x_ref[0]**(o-1)+(-0.856154059105)
        ref[3,2,1]=o*(0.533657669403)*x_ref[1]**(o-1)+(-0.305657234912)
        ref[3,2,2]=o*(0.828787414246)*x_ref[2]**(o-1)+(0.993286809537)
        ref[3,3,0]=o*(0.181845363409)*x_ref[0]**(o-1)+(0.184210025662)
        ref[3,3,1]=o*(-0.538269933964)*x_ref[1]**(o-1)+(0.731384195858)
        ref[3,3,2]=o*(0.900272670238)*x_ref[2]**(o-1)+(-0.152731605913)
        ref[3,4,0]=o*(0.812640675362)*x_ref[0]**(o-1)+(-0.330495179989)
        ref[3,4,1]=o*(0.455533774723)*x_ref[1]**(o-1)+(0.32100819695)
        ref[3,4,2]=o*(-0.332730198093)*x_ref[2]**(o-1)+(-0.692589097458)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_Solution_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.253396883727)*x[0]**o+(0.491235248599)*x[0]+(-0.468734602938)*x[1]**o+(0.730479725967)*x[1]
        arg[0,0,1]=(0.344036208598)*x[0]**o+(-0.394302857752)*x[0]+(-0.608765577652)*x[1]**o+(-0.564065946953)*x[1]
        arg[0,1,0]=(-0.84520693538)*x[0]**o+(0.934285247148)*x[0]+(0.0841237910789)*x[1]**o+(0.455488803423)*x[1]
        arg[0,1,1]=(0.484027534363)*x[0]**o+(-0.374633264234)*x[0]+(-0.0665482018417)*x[1]**o+(0.528984445833)*x[1]
        arg[1,0,0]=(-0.470188044426)*x[0]**o+(-0.328178128746)*x[0]+(0.67571857307)*x[1]**o+(0.116985630247)*x[1]
        arg[1,0,1]=(0.986586449605)*x[0]**o+(-0.25691197068)*x[0]+(0.755235660838)*x[1]**o+(0.211004338524)*x[1]
        arg[1,1,0]=(-0.569322712396)*x[0]**o+(0.900009729013)*x[0]+(0.748881276361)*x[1]**o+(0.850464087662)*x[1]
        arg[1,1,1]=(0.625042758036)*x[0]**o+(-0.72757111333)*x[0]+(-0.986053078492)*x[1]**o+(-0.633517003425)*x[1]
        arg[2,0,0]=(-0.716192724085)*x[0]**o+(-0.23327162426)*x[0]+(0.206440629344)*x[1]**o+(0.816324161671)*x[1]
        arg[2,0,1]=(-0.803679553386)*x[0]**o+(0.445433027195)*x[0]+(-0.111465589792)*x[1]**o+(-0.109772986088)*x[1]
        arg[2,1,0]=(0.374386571048)*x[0]**o+(-0.317187375606)*x[0]+(0.133248401591)*x[1]**o+(-0.971347533424)*x[1]
        arg[2,1,1]=(-0.48476474399)*x[0]**o+(0.979379654323)*x[0]+(0.371437992948)*x[1]**o+(-0.110228097225)*x[1]
        arg[3,0,0]=(0.37507701433)*x[0]**o+(0.451375135714)*x[0]+(0.228204578568)*x[1]**o+(0.0935454426192)*x[1]
        arg[3,0,1]=(-0.322375030256)*x[0]**o+(-0.0412650889525)*x[0]+(0.23495348361)*x[1]**o+(-0.704960519241)*x[1]
        arg[3,1,0]=(-0.478413043796)*x[0]**o+(0.373637701709)*x[0]+(0.346845166424)*x[1]**o+(0.00492075723836)*x[1]
        arg[3,1,1]=(0.0753795682919)*x[0]**o+(0.51446748543)*x[0]+(-0.337079735899)*x[1]**o+(0.976335320865)*x[1]
        arg[4,0,0]=(0.173293972121)*x[0]**o+(0.78972296986)*x[0]+(-0.518624390388)*x[1]**o+(-0.587962330417)*x[1]
        arg[4,0,1]=(0.747393467297)*x[0]**o+(0.940132823774)*x[0]+(-0.292996377712)*x[1]**o+(0.980334604245)*x[1]
        arg[4,1,0]=(-0.836497499117)*x[0]**o+(0.625161912911)*x[0]+(0.978929091652)*x[1]**o+(-0.676430396031)*x[1]
        arg[4,1,1]=(-0.195907406253)*x[0]**o+(-0.388427741642)*x[0]+(0.233500679929)*x[1]**o+(0.535047073421)*x[1]
        arg[5,0,0]=(0.983591245443)*x[0]**o+(-0.356589240979)*x[0]+(0.328974933764)*x[1]**o+(-0.875152262477)*x[1]
        arg[5,0,1]=(0.901430256705)*x[0]**o+(0.74247442514)*x[0]+(0.189211012786)*x[1]**o+(0.930520401277)*x[1]
        arg[5,1,0]=(-0.319591878243)*x[0]**o+(0.174836571432)*x[0]+(0.953875085784)*x[1]**o+(-0.51684941838)*x[1]
        arg[5,1,1]=(-0.916869714992)*x[0]**o+(0.121572609124)*x[0]+(0.791850838139)*x[1]**o+(-0.588663035087)*x[1]
        ref[0,0,0,0]=o*(0.253396883727)*x_ref[0]**(o-1)+(0.491235248599)
        ref[0,0,0,1]=o*(-0.468734602938)*x_ref[1]**(o-1)+(0.730479725967)
        ref[0,0,1,0]=o*(0.344036208598)*x_ref[0]**(o-1)+(-0.394302857752)
        ref[0,0,1,1]=o*(-0.608765577652)*x_ref[1]**(o-1)+(-0.564065946953)
        ref[0,1,0,0]=o*(-0.84520693538)*x_ref[0]**(o-1)+(0.934285247148)
        ref[0,1,0,1]=o*(0.0841237910789)*x_ref[1]**(o-1)+(0.455488803423)
        ref[0,1,1,0]=o*(0.484027534363)*x_ref[0]**(o-1)+(-0.374633264234)
        ref[0,1,1,1]=o*(-0.0665482018417)*x_ref[1]**(o-1)+(0.528984445833)
        ref[1,0,0,0]=o*(-0.470188044426)*x_ref[0]**(o-1)+(-0.328178128746)
        ref[1,0,0,1]=o*(0.67571857307)*x_ref[1]**(o-1)+(0.116985630247)
        ref[1,0,1,0]=o*(0.986586449605)*x_ref[0]**(o-1)+(-0.25691197068)
        ref[1,0,1,1]=o*(0.755235660838)*x_ref[1]**(o-1)+(0.211004338524)
        ref[1,1,0,0]=o*(-0.569322712396)*x_ref[0]**(o-1)+(0.900009729013)
        ref[1,1,0,1]=o*(0.748881276361)*x_ref[1]**(o-1)+(0.850464087662)
        ref[1,1,1,0]=o*(0.625042758036)*x_ref[0]**(o-1)+(-0.72757111333)
        ref[1,1,1,1]=o*(-0.986053078492)*x_ref[1]**(o-1)+(-0.633517003425)
        ref[2,0,0,0]=o*(-0.716192724085)*x_ref[0]**(o-1)+(-0.23327162426)
        ref[2,0,0,1]=o*(0.206440629344)*x_ref[1]**(o-1)+(0.816324161671)
        ref[2,0,1,0]=o*(-0.803679553386)*x_ref[0]**(o-1)+(0.445433027195)
        ref[2,0,1,1]=o*(-0.111465589792)*x_ref[1]**(o-1)+(-0.109772986088)
        ref[2,1,0,0]=o*(0.374386571048)*x_ref[0]**(o-1)+(-0.317187375606)
        ref[2,1,0,1]=o*(0.133248401591)*x_ref[1]**(o-1)+(-0.971347533424)
        ref[2,1,1,0]=o*(-0.48476474399)*x_ref[0]**(o-1)+(0.979379654323)
        ref[2,1,1,1]=o*(0.371437992948)*x_ref[1]**(o-1)+(-0.110228097225)
        ref[3,0,0,0]=o*(0.37507701433)*x_ref[0]**(o-1)+(0.451375135714)
        ref[3,0,0,1]=o*(0.228204578568)*x_ref[1]**(o-1)+(0.0935454426192)
        ref[3,0,1,0]=o*(-0.322375030256)*x_ref[0]**(o-1)+(-0.0412650889525)
        ref[3,0,1,1]=o*(0.23495348361)*x_ref[1]**(o-1)+(-0.704960519241)
        ref[3,1,0,0]=o*(-0.478413043796)*x_ref[0]**(o-1)+(0.373637701709)
        ref[3,1,0,1]=o*(0.346845166424)*x_ref[1]**(o-1)+(0.00492075723836)
        ref[3,1,1,0]=o*(0.0753795682919)*x_ref[0]**(o-1)+(0.51446748543)
        ref[3,1,1,1]=o*(-0.337079735899)*x_ref[1]**(o-1)+(0.976335320865)
        ref[4,0,0,0]=o*(0.173293972121)*x_ref[0]**(o-1)+(0.78972296986)
        ref[4,0,0,1]=o*(-0.518624390388)*x_ref[1]**(o-1)+(-0.587962330417)
        ref[4,0,1,0]=o*(0.747393467297)*x_ref[0]**(o-1)+(0.940132823774)
        ref[4,0,1,1]=o*(-0.292996377712)*x_ref[1]**(o-1)+(0.980334604245)
        ref[4,1,0,0]=o*(-0.836497499117)*x_ref[0]**(o-1)+(0.625161912911)
        ref[4,1,0,1]=o*(0.978929091652)*x_ref[1]**(o-1)+(-0.676430396031)
        ref[4,1,1,0]=o*(-0.195907406253)*x_ref[0]**(o-1)+(-0.388427741642)
        ref[4,1,1,1]=o*(0.233500679929)*x_ref[1]**(o-1)+(0.535047073421)
        ref[5,0,0,0]=o*(0.983591245443)*x_ref[0]**(o-1)+(-0.356589240979)
        ref[5,0,0,1]=o*(0.328974933764)*x_ref[1]**(o-1)+(-0.875152262477)
        ref[5,0,1,0]=o*(0.901430256705)*x_ref[0]**(o-1)+(0.74247442514)
        ref[5,0,1,1]=o*(0.189211012786)*x_ref[1]**(o-1)+(0.930520401277)
        ref[5,1,0,0]=o*(-0.319591878243)*x_ref[0]**(o-1)+(0.174836571432)
        ref[5,1,0,1]=o*(0.953875085784)*x_ref[1]**(o-1)+(-0.51684941838)
        ref[5,1,1,0]=o*(-0.916869714992)*x_ref[0]**(o-1)+(0.121572609124)
        ref[5,1,1,1]=o*(0.791850838139)*x_ref[1]**(o-1)+(-0.588663035087)
      else:
        arg[0,0,0]=(-0.513971266383)*x[0]**o+(-0.117127749421)*x[0]+(-0.551292004692)*x[1]**o+(-0.268920693702)*x[1]+(0.0872538366694)*x[2]**o+(0.855491592762)*x[2]
        arg[0,0,1]=(0.156425207213)*x[0]**o+(-0.299808724719)*x[0]+(0.827383156334)*x[1]**o+(-0.594506082498)*x[1]+(-0.289589281068)*x[2]**o+(0.559159349352)*x[2]
        arg[0,1,0]=(-0.925822763448)*x[0]**o+(0.518052108584)*x[0]+(-0.774761536373)*x[1]**o+(-0.0329613155479)*x[1]+(0.957805160206)*x[2]**o+(0.0606445544683)*x[2]
        arg[0,1,1]=(-0.337663244379)*x[0]**o+(-0.634947360886)*x[0]+(0.232117517887)*x[1]**o+(-0.534317817241)*x[1]+(0.554880796818)*x[2]**o+(-0.3101266685)*x[2]
        arg[1,0,0]=(-0.888548906608)*x[0]**o+(0.517747675888)*x[0]+(-0.614381113253)*x[1]**o+(-0.757891891198)*x[1]+(0.618261909378)*x[2]**o+(-0.0853347346644)*x[2]
        arg[1,0,1]=(-0.686920109418)*x[0]**o+(0.000673193523604)*x[0]+(0.251800447271)*x[1]**o+(-0.105619412646)*x[1]+(0.00990847148242)*x[2]**o+(0.861798595402)*x[2]
        arg[1,1,0]=(0.170484209485)*x[0]**o+(-0.620257643847)*x[0]+(0.057743430303)*x[1]**o+(0.0640298192279)*x[1]+(0.0757100493935)*x[2]**o+(-0.414863410391)*x[2]
        arg[1,1,1]=(-0.714807290039)*x[0]**o+(-0.562420622765)*x[0]+(0.788769732877)*x[1]**o+(-0.472413621829)*x[1]+(-0.466159435785)*x[2]**o+(-0.476510555737)*x[2]
        arg[2,0,0]=(0.875340499144)*x[0]**o+(-0.598283854175)*x[0]+(0.206593884126)*x[1]**o+(-0.987568956833)*x[1]+(0.666527095968)*x[2]**o+(-0.352753155564)*x[2]
        arg[2,0,1]=(0.677005187408)*x[0]**o+(0.760176905882)*x[0]+(-0.0863648949101)*x[1]**o+(-0.607384058071)*x[1]+(-0.117434466077)*x[2]**o+(-0.670292588633)*x[2]
        arg[2,1,0]=(0.888629811684)*x[0]**o+(0.35826389662)*x[0]+(0.00390435387028)*x[1]**o+(-0.0937518531166)*x[1]+(-0.365569888759)*x[2]**o+(0.380035526283)*x[2]
        arg[2,1,1]=(0.138275666033)*x[0]**o+(-0.437710809963)*x[0]+(0.773451227027)*x[1]**o+(0.834016138485)*x[1]+(0.847339165612)*x[2]**o+(-0.417687340903)*x[2]
        arg[3,0,0]=(-0.298714836243)*x[0]**o+(-0.268961434956)*x[0]+(0.232096942083)*x[1]**o+(-0.455570990604)*x[1]+(-0.294542554349)*x[2]**o+(-0.37112817156)*x[2]
        arg[3,0,1]=(0.0903365333358)*x[0]**o+(0.338265717129)*x[0]+(-0.755237281599)*x[1]**o+(-0.608535543682)*x[1]+(0.812136572153)*x[2]**o+(-0.146674314466)*x[2]
        arg[3,1,0]=(0.159422014459)*x[0]**o+(0.899761817668)*x[0]+(0.467507279621)*x[1]**o+(0.47028740641)*x[1]+(-0.905453092566)*x[2]**o+(0.325443171334)*x[2]
        arg[3,1,1]=(0.706113787474)*x[0]**o+(0.411358047385)*x[0]+(0.710178001185)*x[1]**o+(-0.248170072927)*x[1]+(0.2228478152)*x[2]**o+(-0.6845238748)*x[2]
        arg[4,0,0]=(-0.810987300482)*x[0]**o+(-0.20061683347)*x[0]+(-0.362480203096)*x[1]**o+(0.23493442788)*x[1]+(-0.0262576869803)*x[2]**o+(0.40114062378)*x[2]
        arg[4,0,1]=(-0.0377982138529)*x[0]**o+(-0.844585963856)*x[0]+(-0.94360590097)*x[1]**o+(-0.775666510211)*x[1]+(-0.484314539768)*x[2]**o+(0.472661289406)*x[2]
        arg[4,1,0]=(0.579937359489)*x[0]**o+(0.384020744083)*x[0]+(-0.303096661906)*x[1]**o+(-0.0640353004314)*x[1]+(0.693425296881)*x[2]**o+(0.410980796125)*x[2]
        arg[4,1,1]=(0.152139667717)*x[0]**o+(-0.49610371397)*x[0]+(-0.884008900347)*x[1]**o+(-0.364570486388)*x[1]+(0.126918854811)*x[2]**o+(0.409453971802)*x[2]
        arg[5,0,0]=(0.409834559571)*x[0]**o+(-0.347724104753)*x[0]+(-0.263327417846)*x[1]**o+(0.18147743546)*x[1]+(-0.399848371177)*x[2]**o+(0.928317631431)*x[2]
        arg[5,0,1]=(-0.686702422025)*x[0]**o+(-0.918530462662)*x[0]+(0.478936304167)*x[1]**o+(-0.641691106861)*x[1]+(-0.582780732375)*x[2]**o+(0.30255037752)*x[2]
        arg[5,1,0]=(0.50568688101)*x[0]**o+(0.378682510558)*x[0]+(0.59801824354)*x[1]**o+(-0.316028163905)*x[1]+(-0.554966057648)*x[2]**o+(0.637313154931)*x[2]
        arg[5,1,1]=(0.300318472719)*x[0]**o+(-0.794010312167)*x[0]+(-0.314402790698)*x[1]**o+(0.991870648498)*x[1]+(0.949603731589)*x[2]**o+(-0.302054056398)*x[2]
        ref[0,0,0,0]=o*(-0.513971266383)*x_ref[0]**(o-1)+(-0.117127749421)
        ref[0,0,0,1]=o*(-0.551292004692)*x_ref[1]**(o-1)+(-0.268920693702)
        ref[0,0,0,2]=o*(0.0872538366694)*x_ref[2]**(o-1)+(0.855491592762)
        ref[0,0,1,0]=o*(0.156425207213)*x_ref[0]**(o-1)+(-0.299808724719)
        ref[0,0,1,1]=o*(0.827383156334)*x_ref[1]**(o-1)+(-0.594506082498)
        ref[0,0,1,2]=o*(-0.289589281068)*x_ref[2]**(o-1)+(0.559159349352)
        ref[0,1,0,0]=o*(-0.925822763448)*x_ref[0]**(o-1)+(0.518052108584)
        ref[0,1,0,1]=o*(-0.774761536373)*x_ref[1]**(o-1)+(-0.0329613155479)
        ref[0,1,0,2]=o*(0.957805160206)*x_ref[2]**(o-1)+(0.0606445544683)
        ref[0,1,1,0]=o*(-0.337663244379)*x_ref[0]**(o-1)+(-0.634947360886)
        ref[0,1,1,1]=o*(0.232117517887)*x_ref[1]**(o-1)+(-0.534317817241)
        ref[0,1,1,2]=o*(0.554880796818)*x_ref[2]**(o-1)+(-0.3101266685)
        ref[1,0,0,0]=o*(-0.888548906608)*x_ref[0]**(o-1)+(0.517747675888)
        ref[1,0,0,1]=o*(-0.614381113253)*x_ref[1]**(o-1)+(-0.757891891198)
        ref[1,0,0,2]=o*(0.618261909378)*x_ref[2]**(o-1)+(-0.0853347346644)
        ref[1,0,1,0]=o*(-0.686920109418)*x_ref[0]**(o-1)+(0.000673193523604)
        ref[1,0,1,1]=o*(0.251800447271)*x_ref[1]**(o-1)+(-0.105619412646)
        ref[1,0,1,2]=o*(0.00990847148242)*x_ref[2]**(o-1)+(0.861798595402)
        ref[1,1,0,0]=o*(0.170484209485)*x_ref[0]**(o-1)+(-0.620257643847)
        ref[1,1,0,1]=o*(0.057743430303)*x_ref[1]**(o-1)+(0.0640298192279)
        ref[1,1,0,2]=o*(0.0757100493935)*x_ref[2]**(o-1)+(-0.414863410391)
        ref[1,1,1,0]=o*(-0.714807290039)*x_ref[0]**(o-1)+(-0.562420622765)
        ref[1,1,1,1]=o*(0.788769732877)*x_ref[1]**(o-1)+(-0.472413621829)
        ref[1,1,1,2]=o*(-0.466159435785)*x_ref[2]**(o-1)+(-0.476510555737)
        ref[2,0,0,0]=o*(0.875340499144)*x_ref[0]**(o-1)+(-0.598283854175)
        ref[2,0,0,1]=o*(0.206593884126)*x_ref[1]**(o-1)+(-0.987568956833)
        ref[2,0,0,2]=o*(0.666527095968)*x_ref[2]**(o-1)+(-0.352753155564)
        ref[2,0,1,0]=o*(0.677005187408)*x_ref[0]**(o-1)+(0.760176905882)
        ref[2,0,1,1]=o*(-0.0863648949101)*x_ref[1]**(o-1)+(-0.607384058071)
        ref[2,0,1,2]=o*(-0.117434466077)*x_ref[2]**(o-1)+(-0.670292588633)
        ref[2,1,0,0]=o*(0.888629811684)*x_ref[0]**(o-1)+(0.35826389662)
        ref[2,1,0,1]=o*(0.00390435387028)*x_ref[1]**(o-1)+(-0.0937518531166)
        ref[2,1,0,2]=o*(-0.365569888759)*x_ref[2]**(o-1)+(0.380035526283)
        ref[2,1,1,0]=o*(0.138275666033)*x_ref[0]**(o-1)+(-0.437710809963)
        ref[2,1,1,1]=o*(0.773451227027)*x_ref[1]**(o-1)+(0.834016138485)
        ref[2,1,1,2]=o*(0.847339165612)*x_ref[2]**(o-1)+(-0.417687340903)
        ref[3,0,0,0]=o*(-0.298714836243)*x_ref[0]**(o-1)+(-0.268961434956)
        ref[3,0,0,1]=o*(0.232096942083)*x_ref[1]**(o-1)+(-0.455570990604)
        ref[3,0,0,2]=o*(-0.294542554349)*x_ref[2]**(o-1)+(-0.37112817156)
        ref[3,0,1,0]=o*(0.0903365333358)*x_ref[0]**(o-1)+(0.338265717129)
        ref[3,0,1,1]=o*(-0.755237281599)*x_ref[1]**(o-1)+(-0.608535543682)
        ref[3,0,1,2]=o*(0.812136572153)*x_ref[2]**(o-1)+(-0.146674314466)
        ref[3,1,0,0]=o*(0.159422014459)*x_ref[0]**(o-1)+(0.899761817668)
        ref[3,1,0,1]=o*(0.467507279621)*x_ref[1]**(o-1)+(0.47028740641)
        ref[3,1,0,2]=o*(-0.905453092566)*x_ref[2]**(o-1)+(0.325443171334)
        ref[3,1,1,0]=o*(0.706113787474)*x_ref[0]**(o-1)+(0.411358047385)
        ref[3,1,1,1]=o*(0.710178001185)*x_ref[1]**(o-1)+(-0.248170072927)
        ref[3,1,1,2]=o*(0.2228478152)*x_ref[2]**(o-1)+(-0.6845238748)
        ref[4,0,0,0]=o*(-0.810987300482)*x_ref[0]**(o-1)+(-0.20061683347)
        ref[4,0,0,1]=o*(-0.362480203096)*x_ref[1]**(o-1)+(0.23493442788)
        ref[4,0,0,2]=o*(-0.0262576869803)*x_ref[2]**(o-1)+(0.40114062378)
        ref[4,0,1,0]=o*(-0.0377982138529)*x_ref[0]**(o-1)+(-0.844585963856)
        ref[4,0,1,1]=o*(-0.94360590097)*x_ref[1]**(o-1)+(-0.775666510211)
        ref[4,0,1,2]=o*(-0.484314539768)*x_ref[2]**(o-1)+(0.472661289406)
        ref[4,1,0,0]=o*(0.579937359489)*x_ref[0]**(o-1)+(0.384020744083)
        ref[4,1,0,1]=o*(-0.303096661906)*x_ref[1]**(o-1)+(-0.0640353004314)
        ref[4,1,0,2]=o*(0.693425296881)*x_ref[2]**(o-1)+(0.410980796125)
        ref[4,1,1,0]=o*(0.152139667717)*x_ref[0]**(o-1)+(-0.49610371397)
        ref[4,1,1,1]=o*(-0.884008900347)*x_ref[1]**(o-1)+(-0.364570486388)
        ref[4,1,1,2]=o*(0.126918854811)*x_ref[2]**(o-1)+(0.409453971802)
        ref[5,0,0,0]=o*(0.409834559571)*x_ref[0]**(o-1)+(-0.347724104753)
        ref[5,0,0,1]=o*(-0.263327417846)*x_ref[1]**(o-1)+(0.18147743546)
        ref[5,0,0,2]=o*(-0.399848371177)*x_ref[2]**(o-1)+(0.928317631431)
        ref[5,0,1,0]=o*(-0.686702422025)*x_ref[0]**(o-1)+(-0.918530462662)
        ref[5,0,1,1]=o*(0.478936304167)*x_ref[1]**(o-1)+(-0.641691106861)
        ref[5,0,1,2]=o*(-0.582780732375)*x_ref[2]**(o-1)+(0.30255037752)
        ref[5,1,0,0]=o*(0.50568688101)*x_ref[0]**(o-1)+(0.378682510558)
        ref[5,1,0,1]=o*(0.59801824354)*x_ref[1]**(o-1)+(-0.316028163905)
        ref[5,1,0,2]=o*(-0.554966057648)*x_ref[2]**(o-1)+(0.637313154931)
        ref[5,1,1,0]=o*(0.300318472719)*x_ref[0]**(o-1)+(-0.794010312167)
        ref[5,1,1,1]=o*(-0.314402790698)*x_ref[1]**(o-1)+(0.991870648498)
        ref[5,1,1,2]=o*(0.949603731589)*x_ref[2]**(o-1)+(-0.302054056398)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.306947418999)*x[0]+(-0.706616694784)*x[1]
        ref[0]=(0.306947418999)
        ref[1]=(-0.706616694784)
      else:
        arg=(0.490435692707)*x[0]+(0.929248719763)*x[1]+(0.269868131113)*x[2]
        ref[0]=(0.490435692707)
        ref[1]=(0.929248719763)
        ref[2]=(0.269868131113)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.882633673888)*x[0]+(-0.790592442848)*x[1]
        arg[1]=(1.55495669126)*x[0]+(0.496011926182)*x[1]
        ref[0,0]=(-0.882633673888)
        ref[0,1]=(-0.790592442848)
        ref[1,0]=(1.55495669126)
        ref[1,1]=(0.496011926182)
      else:
        arg[0]=(0.683098278161)*x[0]+(-0.151590511238)*x[1]+(0.54461031868)*x[2]
        arg[1]=(0.59998040214)*x[0]+(-0.995466442084)*x[1]+(-1.09468062911)*x[2]
        ref[0,0]=(0.683098278161)
        ref[0,1]=(-0.151590511238)
        ref[0,2]=(0.54461031868)
        ref[1,0]=(0.59998040214)
        ref[1,1]=(-0.995466442084)
        ref[1,2]=(-1.09468062911)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.159129528541)*x[0]+(0.107835972384)*x[1]
        arg[0,1]=(-1.52388770934)*x[0]+(0.262089610596)*x[1]
        arg[0,2]=(0.505454501555)*x[0]+(-0.247061440744)*x[1]
        arg[0,3]=(-0.556075420202)*x[0]+(0.702144110506)*x[1]
        arg[0,4]=(0.810518335626)*x[0]+(-0.76077949525)*x[1]
        arg[1,0]=(-0.491052732808)*x[0]+(-0.272913467279)*x[1]
        arg[1,1]=(0.0141908236388)*x[0]+(1.10868804645)*x[1]
        arg[1,2]=(0.719028425382)*x[0]+(-0.544780305018)*x[1]
        arg[1,3]=(-1.15474920716)*x[0]+(0.888461325396)*x[1]
        arg[1,4]=(-0.842901893766)*x[0]+(-0.465916012282)*x[1]
        arg[2,0]=(1.24606386746)*x[0]+(0.817235825831)*x[1]
        arg[2,1]=(0.400861923494)*x[0]+(1.24545704982)*x[1]
        arg[2,2]=(0.656546253691)*x[0]+(-0.0269228973404)*x[1]
        arg[2,3]=(-0.590512343874)*x[0]+(-0.0654952556025)*x[1]
        arg[2,4]=(0.806342618281)*x[0]+(-0.183166204195)*x[1]
        arg[3,0]=(0.637045776935)*x[0]+(-0.512820906995)*x[1]
        arg[3,1]=(-0.596216711833)*x[0]+(-0.221853749777)*x[1]
        arg[3,2]=(0.113235739927)*x[0]+(-0.210641618992)*x[1]
        arg[3,3]=(-1.11423980423)*x[0]+(-1.46748285012)*x[1]
        arg[3,4]=(0.573260999643)*x[0]+(0.55016627172)*x[1]
        ref[0,0,0]=(-0.159129528541)
        ref[0,0,1]=(0.107835972384)
        ref[0,1,0]=(-1.52388770934)
        ref[0,1,1]=(0.262089610596)
        ref[0,2,0]=(0.505454501555)
        ref[0,2,1]=(-0.247061440744)
        ref[0,3,0]=(-0.556075420202)
        ref[0,3,1]=(0.702144110506)
        ref[0,4,0]=(0.810518335626)
        ref[0,4,1]=(-0.76077949525)
        ref[1,0,0]=(-0.491052732808)
        ref[1,0,1]=(-0.272913467279)
        ref[1,1,0]=(0.0141908236388)
        ref[1,1,1]=(1.10868804645)
        ref[1,2,0]=(0.719028425382)
        ref[1,2,1]=(-0.544780305018)
        ref[1,3,0]=(-1.15474920716)
        ref[1,3,1]=(0.888461325396)
        ref[1,4,0]=(-0.842901893766)
        ref[1,4,1]=(-0.465916012282)
        ref[2,0,0]=(1.24606386746)
        ref[2,0,1]=(0.817235825831)
        ref[2,1,0]=(0.400861923494)
        ref[2,1,1]=(1.24545704982)
        ref[2,2,0]=(0.656546253691)
        ref[2,2,1]=(-0.0269228973404)
        ref[2,3,0]=(-0.590512343874)
        ref[2,3,1]=(-0.0654952556025)
        ref[2,4,0]=(0.806342618281)
        ref[2,4,1]=(-0.183166204195)
        ref[3,0,0]=(0.637045776935)
        ref[3,0,1]=(-0.512820906995)
        ref[3,1,0]=(-0.596216711833)
        ref[3,1,1]=(-0.221853749777)
        ref[3,2,0]=(0.113235739927)
        ref[3,2,1]=(-0.210641618992)
        ref[3,3,0]=(-1.11423980423)
        ref[3,3,1]=(-1.46748285012)
        ref[3,4,0]=(0.573260999643)
        ref[3,4,1]=(0.55016627172)
      else:
        arg[0,0]=(-0.025505130975)*x[0]+(-0.128352022583)*x[1]+(-0.0645809876948)*x[2]
        arg[0,1]=(-0.974556985555)*x[0]+(0.503751568036)*x[1]+(-0.524784840232)*x[2]
        arg[0,2]=(-1.23653778531)*x[0]+(-0.73883453445)*x[1]+(-1.59259101746)*x[2]
        arg[0,3]=(-0.543656268618)*x[0]+(0.110486603472)*x[1]+(-0.636246757745)*x[2]
        arg[0,4]=(-0.566088225133)*x[0]+(-0.272577812471)*x[1]+(-0.282075405014)*x[2]
        arg[1,0]=(-0.28252305722)*x[0]+(0.497103425073)*x[1]+(0.997942367464)*x[2]
        arg[1,1]=(-0.444059543356)*x[0]+(-0.149004039444)*x[1]+(-0.871853047384)*x[2]
        arg[1,2]=(-0.610830496789)*x[0]+(0.196494475826)*x[1]+(0.0123852343883)*x[2]
        arg[1,3]=(-0.327903465165)*x[0]+(0.523560519595)*x[1]+(-0.409961704581)*x[2]
        arg[1,4]=(0.633897214746)*x[0]+(-0.9032002551)*x[1]+(0.624042996391)*x[2]
        arg[2,0]=(0.469489049636)*x[0]+(1.46158418286)*x[1]+(-0.538453674545)*x[2]
        arg[2,1]=(0.0488906165423)*x[0]+(0.483867458959)*x[1]+(0.0419643463366)*x[2]
        arg[2,2]=(-1.89716754038)*x[0]+(-0.577554008519)*x[1]+(-0.0328835832126)*x[2]
        arg[2,3]=(0.507329501469)*x[0]+(0.354939444351)*x[1]+(0.0136766029313)*x[2]
        arg[2,4]=(-0.313182267444)*x[0]+(-1.08684309178)*x[1]+(-1.39365968958)*x[2]
        arg[3,0]=(0.272322829309)*x[0]+(-0.233645276231)*x[1]+(0.491500838791)*x[2]
        arg[3,1]=(1.27603407856)*x[0]+(-0.709429989875)*x[1]+(-0.998371438244)*x[2]
        arg[3,2]=(0.637972716248)*x[0]+(-1.37896009762)*x[1]+(0.496830616786)*x[2]
        arg[3,3]=(-1.67735443481)*x[0]+(-0.741884420811)*x[1]+(1.23125805904)*x[2]
        arg[3,4]=(-0.593290763881)*x[0]+(-0.881909104701)*x[1]+(-1.0425499934)*x[2]
        ref[0,0,0]=(-0.025505130975)
        ref[0,0,1]=(-0.128352022583)
        ref[0,0,2]=(-0.0645809876948)
        ref[0,1,0]=(-0.974556985555)
        ref[0,1,1]=(0.503751568036)
        ref[0,1,2]=(-0.524784840232)
        ref[0,2,0]=(-1.23653778531)
        ref[0,2,1]=(-0.73883453445)
        ref[0,2,2]=(-1.59259101746)
        ref[0,3,0]=(-0.543656268618)
        ref[0,3,1]=(0.110486603472)
        ref[0,3,2]=(-0.636246757745)
        ref[0,4,0]=(-0.566088225133)
        ref[0,4,1]=(-0.272577812471)
        ref[0,4,2]=(-0.282075405014)
        ref[1,0,0]=(-0.28252305722)
        ref[1,0,1]=(0.497103425073)
        ref[1,0,2]=(0.997942367464)
        ref[1,1,0]=(-0.444059543356)
        ref[1,1,1]=(-0.149004039444)
        ref[1,1,2]=(-0.871853047384)
        ref[1,2,0]=(-0.610830496789)
        ref[1,2,1]=(0.196494475826)
        ref[1,2,2]=(0.0123852343883)
        ref[1,3,0]=(-0.327903465165)
        ref[1,3,1]=(0.523560519595)
        ref[1,3,2]=(-0.409961704581)
        ref[1,4,0]=(0.633897214746)
        ref[1,4,1]=(-0.9032002551)
        ref[1,4,2]=(0.624042996391)
        ref[2,0,0]=(0.469489049636)
        ref[2,0,1]=(1.46158418286)
        ref[2,0,2]=(-0.538453674545)
        ref[2,1,0]=(0.0488906165423)
        ref[2,1,1]=(0.483867458959)
        ref[2,1,2]=(0.0419643463366)
        ref[2,2,0]=(-1.89716754038)
        ref[2,2,1]=(-0.577554008519)
        ref[2,2,2]=(-0.0328835832126)
        ref[2,3,0]=(0.507329501469)
        ref[2,3,1]=(0.354939444351)
        ref[2,3,2]=(0.0136766029313)
        ref[2,4,0]=(-0.313182267444)
        ref[2,4,1]=(-1.08684309178)
        ref[2,4,2]=(-1.39365968958)
        ref[3,0,0]=(0.272322829309)
        ref[3,0,1]=(-0.233645276231)
        ref[3,0,2]=(0.491500838791)
        ref[3,1,0]=(1.27603407856)
        ref[3,1,1]=(-0.709429989875)
        ref[3,1,2]=(-0.998371438244)
        ref[3,2,0]=(0.637972716248)
        ref[3,2,1]=(-1.37896009762)
        ref[3,2,2]=(0.496830616786)
        ref[3,3,0]=(-1.67735443481)
        ref[3,3,1]=(-0.741884420811)
        ref[3,3,2]=(1.23125805904)
        ref[3,4,0]=(-0.593290763881)
        ref[3,4,1]=(-0.881909104701)
        ref[3,4,2]=(-1.0425499934)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.289868084377)*x[0]+(-1.636093365)*x[1]
        arg[0,0,1]=(0.841764493453)*x[0]+(-0.397262590373)*x[1]
        arg[0,1,0]=(0.318610649633)*x[0]+(1.59837795884)*x[1]
        arg[0,1,1]=(1.41408600658)*x[0]+(0.212012530707)*x[1]
        arg[1,0,0]=(-1.13663878903)*x[0]+(-0.535752609596)*x[1]
        arg[1,0,1]=(-0.795140440626)*x[0]+(-0.733456054749)*x[1]
        arg[1,1,0]=(-0.174874940569)*x[0]+(0.304037654801)*x[1]
        arg[1,1,1]=(0.835374268658)*x[0]+(0.645365790264)*x[1]
        arg[2,0,0]=(0.684064169293)*x[0]+(-0.580502209076)*x[1]
        arg[2,0,1]=(0.342542841673)*x[0]+(-0.620912562797)*x[1]
        arg[2,1,0]=(-0.457226288553)*x[0]+(-0.163180645461)*x[1]
        arg[2,1,1]=(-0.494067356015)*x[0]+(-0.604667785982)*x[1]
        arg[3,0,0]=(-0.224173319974)*x[0]+(-0.288566962407)*x[1]
        arg[3,0,1]=(-0.83578982053)*x[0]+(0.290663773612)*x[1]
        arg[3,1,0]=(0.700494058094)*x[0]+(0.244036699451)*x[1]
        arg[3,1,1]=(0.960756120912)*x[0]+(-0.129549309953)*x[1]
        arg[4,0,0]=(0.942292060296)*x[0]+(-0.148922577041)*x[1]
        arg[4,0,1]=(-0.363876552144)*x[0]+(0.903847614766)*x[1]
        arg[4,1,0]=(-1.91719369833)*x[0]+(0.961392595186)*x[1]
        arg[4,1,1]=(-0.141662097711)*x[0]+(-0.695061685931)*x[1]
        arg[5,0,0]=(0.416693171023)*x[0]+(1.04455773845)*x[1]
        arg[5,0,1]=(0.325327980312)*x[0]+(-1.01452792883)*x[1]
        arg[5,1,0]=(-1.43970903355)*x[0]+(-1.04382682561)*x[1]
        arg[5,1,1]=(-0.969700314248)*x[0]+(-0.286024748854)*x[1]
        ref[0,0,0,0]=(0.289868084377)
        ref[0,0,0,1]=(-1.636093365)
        ref[0,0,1,0]=(0.841764493453)
        ref[0,0,1,1]=(-0.397262590373)
        ref[0,1,0,0]=(0.318610649633)
        ref[0,1,0,1]=(1.59837795884)
        ref[0,1,1,0]=(1.41408600658)
        ref[0,1,1,1]=(0.212012530707)
        ref[1,0,0,0]=(-1.13663878903)
        ref[1,0,0,1]=(-0.535752609596)
        ref[1,0,1,0]=(-0.795140440626)
        ref[1,0,1,1]=(-0.733456054749)
        ref[1,1,0,0]=(-0.174874940569)
        ref[1,1,0,1]=(0.304037654801)
        ref[1,1,1,0]=(0.835374268658)
        ref[1,1,1,1]=(0.645365790264)
        ref[2,0,0,0]=(0.684064169293)
        ref[2,0,0,1]=(-0.580502209076)
        ref[2,0,1,0]=(0.342542841673)
        ref[2,0,1,1]=(-0.620912562797)
        ref[2,1,0,0]=(-0.457226288553)
        ref[2,1,0,1]=(-0.163180645461)
        ref[2,1,1,0]=(-0.494067356015)
        ref[2,1,1,1]=(-0.604667785982)
        ref[3,0,0,0]=(-0.224173319974)
        ref[3,0,0,1]=(-0.288566962407)
        ref[3,0,1,0]=(-0.83578982053)
        ref[3,0,1,1]=(0.290663773612)
        ref[3,1,0,0]=(0.700494058094)
        ref[3,1,0,1]=(0.244036699451)
        ref[3,1,1,0]=(0.960756120912)
        ref[3,1,1,1]=(-0.129549309953)
        ref[4,0,0,0]=(0.942292060296)
        ref[4,0,0,1]=(-0.148922577041)
        ref[4,0,1,0]=(-0.363876552144)
        ref[4,0,1,1]=(0.903847614766)
        ref[4,1,0,0]=(-1.91719369833)
        ref[4,1,0,1]=(0.961392595186)
        ref[4,1,1,0]=(-0.141662097711)
        ref[4,1,1,1]=(-0.695061685931)
        ref[5,0,0,0]=(0.416693171023)
        ref[5,0,0,1]=(1.04455773845)
        ref[5,0,1,0]=(0.325327980312)
        ref[5,0,1,1]=(-1.01452792883)
        ref[5,1,0,0]=(-1.43970903355)
        ref[5,1,0,1]=(-1.04382682561)
        ref[5,1,1,0]=(-0.969700314248)
        ref[5,1,1,1]=(-0.286024748854)
      else:
        arg[0,0,0]=(-1.52443210214)*x[0]+(-1.48020118993)*x[1]+(1.2552138343)*x[2]
        arg[0,0,1]=(-0.102900544448)*x[0]+(-1.14209442568)*x[1]+(-0.975626338621)*x[2]
        arg[0,1,0]=(0.909668629294)*x[0]+(1.29127341353)*x[1]+(-1.65221956404)*x[2]
        arg[0,1,1]=(-0.00309732749698)*x[0]+(0.0561682749786)*x[1]+(0.330039911431)*x[2]
        arg[1,0,0]=(0.966608417982)*x[0]+(-0.181702539284)*x[1]+(0.377393548129)*x[2]
        arg[1,0,1]=(0.0273333293331)*x[0]+(0.745172228806)*x[1]+(0.756302500232)*x[2]
        arg[1,1,0]=(-0.550382204645)*x[0]+(0.531191986475)*x[1]+(-1.02269665633)*x[2]
        arg[1,1,1]=(-0.928522491259)*x[0]+(-1.25241213648)*x[1]+(0.635412990933)*x[2]
        arg[2,0,0]=(-0.355234026557)*x[0]+(-1.69834232334)*x[1]+(1.17827811215)*x[2]
        arg[2,0,1]=(-0.905805855166)*x[0]+(1.4023187683)*x[1]+(-0.570168366306)*x[2]
        arg[2,1,0]=(0.550150523384)*x[0]+(-1.59933962526)*x[1]+(-0.388203251081)*x[2]
        arg[2,1,1]=(0.138470563927)*x[0]+(0.245233545344)*x[1]+(-0.293840556681)*x[2]
        arg[3,0,0]=(1.19700673733)*x[0]+(-1.04510591446)*x[1]+(1.03589008728)*x[2]
        arg[3,0,1]=(0.713948123924)*x[0]+(-1.42841558036)*x[1]+(1.07248428525)*x[2]
        arg[3,1,0]=(0.11223781122)*x[0]+(-0.00982612208261)*x[1]+(-0.263153847134)*x[2]
        arg[3,1,1]=(1.33011049869)*x[0]+(0.691016997799)*x[1]+(-0.0547947379786)*x[2]
        arg[4,0,0]=(1.0577224608)*x[0]+(-0.263034881475)*x[1]+(-0.854540775969)*x[2]
        arg[4,0,1]=(0.686363734714)*x[0]+(0.198120868875)*x[1]+(1.46326635417)*x[2]
        arg[4,1,0]=(-0.745244187008)*x[0]+(1.31766298118)*x[1]+(1.51544532829)*x[2]
        arg[4,1,1]=(0.878390723411)*x[0]+(-0.427741582253)*x[1]+(0.0324341706939)*x[2]
        arg[5,0,0]=(-0.445960121294)*x[0]+(-0.215668362178)*x[1]+(0.147632419682)*x[2]
        arg[5,0,1]=(-0.0781270285912)*x[0]+(0.782990026347)*x[1]+(-1.28046463036)*x[2]
        arg[5,1,0]=(-0.385003656622)*x[0]+(-1.02087438188)*x[1]+(-0.594357838151)*x[2]
        arg[5,1,1]=(-0.676669731714)*x[0]+(-0.28685220691)*x[1]+(0.335774697674)*x[2]
        ref[0,0,0,0]=(-1.52443210214)
        ref[0,0,0,1]=(-1.48020118993)
        ref[0,0,0,2]=(1.2552138343)
        ref[0,0,1,0]=(-0.102900544448)
        ref[0,0,1,1]=(-1.14209442568)
        ref[0,0,1,2]=(-0.975626338621)
        ref[0,1,0,0]=(0.909668629294)
        ref[0,1,0,1]=(1.29127341353)
        ref[0,1,0,2]=(-1.65221956404)
        ref[0,1,1,0]=(-0.00309732749698)
        ref[0,1,1,1]=(0.0561682749786)
        ref[0,1,1,2]=(0.330039911431)
        ref[1,0,0,0]=(0.966608417982)
        ref[1,0,0,1]=(-0.181702539284)
        ref[1,0,0,2]=(0.377393548129)
        ref[1,0,1,0]=(0.0273333293331)
        ref[1,0,1,1]=(0.745172228806)
        ref[1,0,1,2]=(0.756302500232)
        ref[1,1,0,0]=(-0.550382204645)
        ref[1,1,0,1]=(0.531191986475)
        ref[1,1,0,2]=(-1.02269665633)
        ref[1,1,1,0]=(-0.928522491259)
        ref[1,1,1,1]=(-1.25241213648)
        ref[1,1,1,2]=(0.635412990933)
        ref[2,0,0,0]=(-0.355234026557)
        ref[2,0,0,1]=(-1.69834232334)
        ref[2,0,0,2]=(1.17827811215)
        ref[2,0,1,0]=(-0.905805855166)
        ref[2,0,1,1]=(1.4023187683)
        ref[2,0,1,2]=(-0.570168366306)
        ref[2,1,0,0]=(0.550150523384)
        ref[2,1,0,1]=(-1.59933962526)
        ref[2,1,0,2]=(-0.388203251081)
        ref[2,1,1,0]=(0.138470563927)
        ref[2,1,1,1]=(0.245233545344)
        ref[2,1,1,2]=(-0.293840556681)
        ref[3,0,0,0]=(1.19700673733)
        ref[3,0,0,1]=(-1.04510591446)
        ref[3,0,0,2]=(1.03589008728)
        ref[3,0,1,0]=(0.713948123924)
        ref[3,0,1,1]=(-1.42841558036)
        ref[3,0,1,2]=(1.07248428525)
        ref[3,1,0,0]=(0.11223781122)
        ref[3,1,0,1]=(-0.00982612208261)
        ref[3,1,0,2]=(-0.263153847134)
        ref[3,1,1,0]=(1.33011049869)
        ref[3,1,1,1]=(0.691016997799)
        ref[3,1,1,2]=(-0.0547947379786)
        ref[4,0,0,0]=(1.0577224608)
        ref[4,0,0,1]=(-0.263034881475)
        ref[4,0,0,2]=(-0.854540775969)
        ref[4,0,1,0]=(0.686363734714)
        ref[4,0,1,1]=(0.198120868875)
        ref[4,0,1,2]=(1.46326635417)
        ref[4,1,0,0]=(-0.745244187008)
        ref[4,1,0,1]=(1.31766298118)
        ref[4,1,0,2]=(1.51544532829)
        ref[4,1,1,0]=(0.878390723411)
        ref[4,1,1,1]=(-0.427741582253)
        ref[4,1,1,2]=(0.0324341706939)
        ref[5,0,0,0]=(-0.445960121294)
        ref[5,0,0,1]=(-0.215668362178)
        ref[5,0,0,2]=(0.147632419682)
        ref[5,0,1,0]=(-0.0781270285912)
        ref[5,0,1,1]=(0.782990026347)
        ref[5,0,1,2]=(-1.28046463036)
        ref[5,1,0,0]=(-0.385003656622)
        ref[5,1,0,1]=(-1.02087438188)
        ref[5,1,0,2]=(-0.594357838151)
        ref[5,1,1,0]=(-0.676669731714)
        ref[5,1,1,1]=(-0.28685220691)
        ref[5,1,1,2]=(0.335774697674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.306947418999)*x[0]+(-0.706616694784)*x[1]
        ref[0]=(0.306947418999)
        ref[1]=(-0.706616694784)
      else:
        arg=(0.490435692707)*x[0]+(0.929248719763)*x[1]+(0.269868131113)*x[2]
        ref[0]=(0.490435692707)
        ref[1]=(0.929248719763)
        ref[2]=(0.269868131113)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.882633673888)*x[0]+(-0.790592442848)*x[1]
        arg[1]=(1.55495669126)*x[0]+(0.496011926182)*x[1]
        ref[0,0]=(-0.882633673888)
        ref[0,1]=(-0.790592442848)
        ref[1,0]=(1.55495669126)
        ref[1,1]=(0.496011926182)
      else:
        arg[0]=(0.683098278161)*x[0]+(-0.151590511238)*x[1]+(0.54461031868)*x[2]
        arg[1]=(0.59998040214)*x[0]+(-0.995466442084)*x[1]+(-1.09468062911)*x[2]
        ref[0,0]=(0.683098278161)
        ref[0,1]=(-0.151590511238)
        ref[0,2]=(0.54461031868)
        ref[1,0]=(0.59998040214)
        ref[1,1]=(-0.995466442084)
        ref[1,2]=(-1.09468062911)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.159129528541)*x[0]+(0.107835972384)*x[1]
        arg[0,1]=(-1.52388770934)*x[0]+(0.262089610596)*x[1]
        arg[0,2]=(0.505454501555)*x[0]+(-0.247061440744)*x[1]
        arg[0,3]=(-0.556075420202)*x[0]+(0.702144110506)*x[1]
        arg[0,4]=(0.810518335626)*x[0]+(-0.76077949525)*x[1]
        arg[1,0]=(-0.491052732808)*x[0]+(-0.272913467279)*x[1]
        arg[1,1]=(0.0141908236388)*x[0]+(1.10868804645)*x[1]
        arg[1,2]=(0.719028425382)*x[0]+(-0.544780305018)*x[1]
        arg[1,3]=(-1.15474920716)*x[0]+(0.888461325396)*x[1]
        arg[1,4]=(-0.842901893766)*x[0]+(-0.465916012282)*x[1]
        arg[2,0]=(1.24606386746)*x[0]+(0.817235825831)*x[1]
        arg[2,1]=(0.400861923494)*x[0]+(1.24545704982)*x[1]
        arg[2,2]=(0.656546253691)*x[0]+(-0.0269228973404)*x[1]
        arg[2,3]=(-0.590512343874)*x[0]+(-0.0654952556025)*x[1]
        arg[2,4]=(0.806342618281)*x[0]+(-0.183166204195)*x[1]
        arg[3,0]=(0.637045776935)*x[0]+(-0.512820906995)*x[1]
        arg[3,1]=(-0.596216711833)*x[0]+(-0.221853749777)*x[1]
        arg[3,2]=(0.113235739927)*x[0]+(-0.210641618992)*x[1]
        arg[3,3]=(-1.11423980423)*x[0]+(-1.46748285012)*x[1]
        arg[3,4]=(0.573260999643)*x[0]+(0.55016627172)*x[1]
        ref[0,0,0]=(-0.159129528541)
        ref[0,0,1]=(0.107835972384)
        ref[0,1,0]=(-1.52388770934)
        ref[0,1,1]=(0.262089610596)
        ref[0,2,0]=(0.505454501555)
        ref[0,2,1]=(-0.247061440744)
        ref[0,3,0]=(-0.556075420202)
        ref[0,3,1]=(0.702144110506)
        ref[0,4,0]=(0.810518335626)
        ref[0,4,1]=(-0.76077949525)
        ref[1,0,0]=(-0.491052732808)
        ref[1,0,1]=(-0.272913467279)
        ref[1,1,0]=(0.0141908236388)
        ref[1,1,1]=(1.10868804645)
        ref[1,2,0]=(0.719028425382)
        ref[1,2,1]=(-0.544780305018)
        ref[1,3,0]=(-1.15474920716)
        ref[1,3,1]=(0.888461325396)
        ref[1,4,0]=(-0.842901893766)
        ref[1,4,1]=(-0.465916012282)
        ref[2,0,0]=(1.24606386746)
        ref[2,0,1]=(0.817235825831)
        ref[2,1,0]=(0.400861923494)
        ref[2,1,1]=(1.24545704982)
        ref[2,2,0]=(0.656546253691)
        ref[2,2,1]=(-0.0269228973404)
        ref[2,3,0]=(-0.590512343874)
        ref[2,3,1]=(-0.0654952556025)
        ref[2,4,0]=(0.806342618281)
        ref[2,4,1]=(-0.183166204195)
        ref[3,0,0]=(0.637045776935)
        ref[3,0,1]=(-0.512820906995)
        ref[3,1,0]=(-0.596216711833)
        ref[3,1,1]=(-0.221853749777)
        ref[3,2,0]=(0.113235739927)
        ref[3,2,1]=(-0.210641618992)
        ref[3,3,0]=(-1.11423980423)
        ref[3,3,1]=(-1.46748285012)
        ref[3,4,0]=(0.573260999643)
        ref[3,4,1]=(0.55016627172)
      else:
        arg[0,0]=(-0.025505130975)*x[0]+(-0.128352022583)*x[1]+(-0.0645809876948)*x[2]
        arg[0,1]=(-0.974556985555)*x[0]+(0.503751568036)*x[1]+(-0.524784840232)*x[2]
        arg[0,2]=(-1.23653778531)*x[0]+(-0.73883453445)*x[1]+(-1.59259101746)*x[2]
        arg[0,3]=(-0.543656268618)*x[0]+(0.110486603472)*x[1]+(-0.636246757745)*x[2]
        arg[0,4]=(-0.566088225133)*x[0]+(-0.272577812471)*x[1]+(-0.282075405014)*x[2]
        arg[1,0]=(-0.28252305722)*x[0]+(0.497103425073)*x[1]+(0.997942367464)*x[2]
        arg[1,1]=(-0.444059543356)*x[0]+(-0.149004039444)*x[1]+(-0.871853047384)*x[2]
        arg[1,2]=(-0.610830496789)*x[0]+(0.196494475826)*x[1]+(0.0123852343883)*x[2]
        arg[1,3]=(-0.327903465165)*x[0]+(0.523560519595)*x[1]+(-0.409961704581)*x[2]
        arg[1,4]=(0.633897214746)*x[0]+(-0.9032002551)*x[1]+(0.624042996391)*x[2]
        arg[2,0]=(0.469489049636)*x[0]+(1.46158418286)*x[1]+(-0.538453674545)*x[2]
        arg[2,1]=(0.0488906165423)*x[0]+(0.483867458959)*x[1]+(0.0419643463366)*x[2]
        arg[2,2]=(-1.89716754038)*x[0]+(-0.577554008519)*x[1]+(-0.0328835832126)*x[2]
        arg[2,3]=(0.507329501469)*x[0]+(0.354939444351)*x[1]+(0.0136766029313)*x[2]
        arg[2,4]=(-0.313182267444)*x[0]+(-1.08684309178)*x[1]+(-1.39365968958)*x[2]
        arg[3,0]=(0.272322829309)*x[0]+(-0.233645276231)*x[1]+(0.491500838791)*x[2]
        arg[3,1]=(1.27603407856)*x[0]+(-0.709429989875)*x[1]+(-0.998371438244)*x[2]
        arg[3,2]=(0.637972716248)*x[0]+(-1.37896009762)*x[1]+(0.496830616786)*x[2]
        arg[3,3]=(-1.67735443481)*x[0]+(-0.741884420811)*x[1]+(1.23125805904)*x[2]
        arg[3,4]=(-0.593290763881)*x[0]+(-0.881909104701)*x[1]+(-1.0425499934)*x[2]
        ref[0,0,0]=(-0.025505130975)
        ref[0,0,1]=(-0.128352022583)
        ref[0,0,2]=(-0.0645809876948)
        ref[0,1,0]=(-0.974556985555)
        ref[0,1,1]=(0.503751568036)
        ref[0,1,2]=(-0.524784840232)
        ref[0,2,0]=(-1.23653778531)
        ref[0,2,1]=(-0.73883453445)
        ref[0,2,2]=(-1.59259101746)
        ref[0,3,0]=(-0.543656268618)
        ref[0,3,1]=(0.110486603472)
        ref[0,3,2]=(-0.636246757745)
        ref[0,4,0]=(-0.566088225133)
        ref[0,4,1]=(-0.272577812471)
        ref[0,4,2]=(-0.282075405014)
        ref[1,0,0]=(-0.28252305722)
        ref[1,0,1]=(0.497103425073)
        ref[1,0,2]=(0.997942367464)
        ref[1,1,0]=(-0.444059543356)
        ref[1,1,1]=(-0.149004039444)
        ref[1,1,2]=(-0.871853047384)
        ref[1,2,0]=(-0.610830496789)
        ref[1,2,1]=(0.196494475826)
        ref[1,2,2]=(0.0123852343883)
        ref[1,3,0]=(-0.327903465165)
        ref[1,3,1]=(0.523560519595)
        ref[1,3,2]=(-0.409961704581)
        ref[1,4,0]=(0.633897214746)
        ref[1,4,1]=(-0.9032002551)
        ref[1,4,2]=(0.624042996391)
        ref[2,0,0]=(0.469489049636)
        ref[2,0,1]=(1.46158418286)
        ref[2,0,2]=(-0.538453674545)
        ref[2,1,0]=(0.0488906165423)
        ref[2,1,1]=(0.483867458959)
        ref[2,1,2]=(0.0419643463366)
        ref[2,2,0]=(-1.89716754038)
        ref[2,2,1]=(-0.577554008519)
        ref[2,2,2]=(-0.0328835832126)
        ref[2,3,0]=(0.507329501469)
        ref[2,3,1]=(0.354939444351)
        ref[2,3,2]=(0.0136766029313)
        ref[2,4,0]=(-0.313182267444)
        ref[2,4,1]=(-1.08684309178)
        ref[2,4,2]=(-1.39365968958)
        ref[3,0,0]=(0.272322829309)
        ref[3,0,1]=(-0.233645276231)
        ref[3,0,2]=(0.491500838791)
        ref[3,1,0]=(1.27603407856)
        ref[3,1,1]=(-0.709429989875)
        ref[3,1,2]=(-0.998371438244)
        ref[3,2,0]=(0.637972716248)
        ref[3,2,1]=(-1.37896009762)
        ref[3,2,2]=(0.496830616786)
        ref[3,3,0]=(-1.67735443481)
        ref[3,3,1]=(-0.741884420811)
        ref[3,3,2]=(1.23125805904)
        ref[3,4,0]=(-0.593290763881)
        ref[3,4,1]=(-0.881909104701)
        ref[3,4,2]=(-1.0425499934)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.289868084377)*x[0]+(-1.636093365)*x[1]
        arg[0,0,1]=(0.841764493453)*x[0]+(-0.397262590373)*x[1]
        arg[0,1,0]=(0.318610649633)*x[0]+(1.59837795884)*x[1]
        arg[0,1,1]=(1.41408600658)*x[0]+(0.212012530707)*x[1]
        arg[1,0,0]=(-1.13663878903)*x[0]+(-0.535752609596)*x[1]
        arg[1,0,1]=(-0.795140440626)*x[0]+(-0.733456054749)*x[1]
        arg[1,1,0]=(-0.174874940569)*x[0]+(0.304037654801)*x[1]
        arg[1,1,1]=(0.835374268658)*x[0]+(0.645365790264)*x[1]
        arg[2,0,0]=(0.684064169293)*x[0]+(-0.580502209076)*x[1]
        arg[2,0,1]=(0.342542841673)*x[0]+(-0.620912562797)*x[1]
        arg[2,1,0]=(-0.457226288553)*x[0]+(-0.163180645461)*x[1]
        arg[2,1,1]=(-0.494067356015)*x[0]+(-0.604667785982)*x[1]
        arg[3,0,0]=(-0.224173319974)*x[0]+(-0.288566962407)*x[1]
        arg[3,0,1]=(-0.83578982053)*x[0]+(0.290663773612)*x[1]
        arg[3,1,0]=(0.700494058094)*x[0]+(0.244036699451)*x[1]
        arg[3,1,1]=(0.960756120912)*x[0]+(-0.129549309953)*x[1]
        arg[4,0,0]=(0.942292060296)*x[0]+(-0.148922577041)*x[1]
        arg[4,0,1]=(-0.363876552144)*x[0]+(0.903847614766)*x[1]
        arg[4,1,0]=(-1.91719369833)*x[0]+(0.961392595186)*x[1]
        arg[4,1,1]=(-0.141662097711)*x[0]+(-0.695061685931)*x[1]
        arg[5,0,0]=(0.416693171023)*x[0]+(1.04455773845)*x[1]
        arg[5,0,1]=(0.325327980312)*x[0]+(-1.01452792883)*x[1]
        arg[5,1,0]=(-1.43970903355)*x[0]+(-1.04382682561)*x[1]
        arg[5,1,1]=(-0.969700314248)*x[0]+(-0.286024748854)*x[1]
        ref[0,0,0,0]=(0.289868084377)
        ref[0,0,0,1]=(-1.636093365)
        ref[0,0,1,0]=(0.841764493453)
        ref[0,0,1,1]=(-0.397262590373)
        ref[0,1,0,0]=(0.318610649633)
        ref[0,1,0,1]=(1.59837795884)
        ref[0,1,1,0]=(1.41408600658)
        ref[0,1,1,1]=(0.212012530707)
        ref[1,0,0,0]=(-1.13663878903)
        ref[1,0,0,1]=(-0.535752609596)
        ref[1,0,1,0]=(-0.795140440626)
        ref[1,0,1,1]=(-0.733456054749)
        ref[1,1,0,0]=(-0.174874940569)
        ref[1,1,0,1]=(0.304037654801)
        ref[1,1,1,0]=(0.835374268658)
        ref[1,1,1,1]=(0.645365790264)
        ref[2,0,0,0]=(0.684064169293)
        ref[2,0,0,1]=(-0.580502209076)
        ref[2,0,1,0]=(0.342542841673)
        ref[2,0,1,1]=(-0.620912562797)
        ref[2,1,0,0]=(-0.457226288553)
        ref[2,1,0,1]=(-0.163180645461)
        ref[2,1,1,0]=(-0.494067356015)
        ref[2,1,1,1]=(-0.604667785982)
        ref[3,0,0,0]=(-0.224173319974)
        ref[3,0,0,1]=(-0.288566962407)
        ref[3,0,1,0]=(-0.83578982053)
        ref[3,0,1,1]=(0.290663773612)
        ref[3,1,0,0]=(0.700494058094)
        ref[3,1,0,1]=(0.244036699451)
        ref[3,1,1,0]=(0.960756120912)
        ref[3,1,1,1]=(-0.129549309953)
        ref[4,0,0,0]=(0.942292060296)
        ref[4,0,0,1]=(-0.148922577041)
        ref[4,0,1,0]=(-0.363876552144)
        ref[4,0,1,1]=(0.903847614766)
        ref[4,1,0,0]=(-1.91719369833)
        ref[4,1,0,1]=(0.961392595186)
        ref[4,1,1,0]=(-0.141662097711)
        ref[4,1,1,1]=(-0.695061685931)
        ref[5,0,0,0]=(0.416693171023)
        ref[5,0,0,1]=(1.04455773845)
        ref[5,0,1,0]=(0.325327980312)
        ref[5,0,1,1]=(-1.01452792883)
        ref[5,1,0,0]=(-1.43970903355)
        ref[5,1,0,1]=(-1.04382682561)
        ref[5,1,1,0]=(-0.969700314248)
        ref[5,1,1,1]=(-0.286024748854)
      else:
        arg[0,0,0]=(-1.52443210214)*x[0]+(-1.48020118993)*x[1]+(1.2552138343)*x[2]
        arg[0,0,1]=(-0.102900544448)*x[0]+(-1.14209442568)*x[1]+(-0.975626338621)*x[2]
        arg[0,1,0]=(0.909668629294)*x[0]+(1.29127341353)*x[1]+(-1.65221956404)*x[2]
        arg[0,1,1]=(-0.00309732749698)*x[0]+(0.0561682749786)*x[1]+(0.330039911431)*x[2]
        arg[1,0,0]=(0.966608417982)*x[0]+(-0.181702539284)*x[1]+(0.377393548129)*x[2]
        arg[1,0,1]=(0.0273333293331)*x[0]+(0.745172228806)*x[1]+(0.756302500232)*x[2]
        arg[1,1,0]=(-0.550382204645)*x[0]+(0.531191986475)*x[1]+(-1.02269665633)*x[2]
        arg[1,1,1]=(-0.928522491259)*x[0]+(-1.25241213648)*x[1]+(0.635412990933)*x[2]
        arg[2,0,0]=(-0.355234026557)*x[0]+(-1.69834232334)*x[1]+(1.17827811215)*x[2]
        arg[2,0,1]=(-0.905805855166)*x[0]+(1.4023187683)*x[1]+(-0.570168366306)*x[2]
        arg[2,1,0]=(0.550150523384)*x[0]+(-1.59933962526)*x[1]+(-0.388203251081)*x[2]
        arg[2,1,1]=(0.138470563927)*x[0]+(0.245233545344)*x[1]+(-0.293840556681)*x[2]
        arg[3,0,0]=(1.19700673733)*x[0]+(-1.04510591446)*x[1]+(1.03589008728)*x[2]
        arg[3,0,1]=(0.713948123924)*x[0]+(-1.42841558036)*x[1]+(1.07248428525)*x[2]
        arg[3,1,0]=(0.11223781122)*x[0]+(-0.00982612208261)*x[1]+(-0.263153847134)*x[2]
        arg[3,1,1]=(1.33011049869)*x[0]+(0.691016997799)*x[1]+(-0.0547947379786)*x[2]
        arg[4,0,0]=(1.0577224608)*x[0]+(-0.263034881475)*x[1]+(-0.854540775969)*x[2]
        arg[4,0,1]=(0.686363734714)*x[0]+(0.198120868875)*x[1]+(1.46326635417)*x[2]
        arg[4,1,0]=(-0.745244187008)*x[0]+(1.31766298118)*x[1]+(1.51544532829)*x[2]
        arg[4,1,1]=(0.878390723411)*x[0]+(-0.427741582253)*x[1]+(0.0324341706939)*x[2]
        arg[5,0,0]=(-0.445960121294)*x[0]+(-0.215668362178)*x[1]+(0.147632419682)*x[2]
        arg[5,0,1]=(-0.0781270285912)*x[0]+(0.782990026347)*x[1]+(-1.28046463036)*x[2]
        arg[5,1,0]=(-0.385003656622)*x[0]+(-1.02087438188)*x[1]+(-0.594357838151)*x[2]
        arg[5,1,1]=(-0.676669731714)*x[0]+(-0.28685220691)*x[1]+(0.335774697674)*x[2]
        ref[0,0,0,0]=(-1.52443210214)
        ref[0,0,0,1]=(-1.48020118993)
        ref[0,0,0,2]=(1.2552138343)
        ref[0,0,1,0]=(-0.102900544448)
        ref[0,0,1,1]=(-1.14209442568)
        ref[0,0,1,2]=(-0.975626338621)
        ref[0,1,0,0]=(0.909668629294)
        ref[0,1,0,1]=(1.29127341353)
        ref[0,1,0,2]=(-1.65221956404)
        ref[0,1,1,0]=(-0.00309732749698)
        ref[0,1,1,1]=(0.0561682749786)
        ref[0,1,1,2]=(0.330039911431)
        ref[1,0,0,0]=(0.966608417982)
        ref[1,0,0,1]=(-0.181702539284)
        ref[1,0,0,2]=(0.377393548129)
        ref[1,0,1,0]=(0.0273333293331)
        ref[1,0,1,1]=(0.745172228806)
        ref[1,0,1,2]=(0.756302500232)
        ref[1,1,0,0]=(-0.550382204645)
        ref[1,1,0,1]=(0.531191986475)
        ref[1,1,0,2]=(-1.02269665633)
        ref[1,1,1,0]=(-0.928522491259)
        ref[1,1,1,1]=(-1.25241213648)
        ref[1,1,1,2]=(0.635412990933)
        ref[2,0,0,0]=(-0.355234026557)
        ref[2,0,0,1]=(-1.69834232334)
        ref[2,0,0,2]=(1.17827811215)
        ref[2,0,1,0]=(-0.905805855166)
        ref[2,0,1,1]=(1.4023187683)
        ref[2,0,1,2]=(-0.570168366306)
        ref[2,1,0,0]=(0.550150523384)
        ref[2,1,0,1]=(-1.59933962526)
        ref[2,1,0,2]=(-0.388203251081)
        ref[2,1,1,0]=(0.138470563927)
        ref[2,1,1,1]=(0.245233545344)
        ref[2,1,1,2]=(-0.293840556681)
        ref[3,0,0,0]=(1.19700673733)
        ref[3,0,0,1]=(-1.04510591446)
        ref[3,0,0,2]=(1.03589008728)
        ref[3,0,1,0]=(0.713948123924)
        ref[3,0,1,1]=(-1.42841558036)
        ref[3,0,1,2]=(1.07248428525)
        ref[3,1,0,0]=(0.11223781122)
        ref[3,1,0,1]=(-0.00982612208261)
        ref[3,1,0,2]=(-0.263153847134)
        ref[3,1,1,0]=(1.33011049869)
        ref[3,1,1,1]=(0.691016997799)
        ref[3,1,1,2]=(-0.0547947379786)
        ref[4,0,0,0]=(1.0577224608)
        ref[4,0,0,1]=(-0.263034881475)
        ref[4,0,0,2]=(-0.854540775969)
        ref[4,0,1,0]=(0.686363734714)
        ref[4,0,1,1]=(0.198120868875)
        ref[4,0,1,2]=(1.46326635417)
        ref[4,1,0,0]=(-0.745244187008)
        ref[4,1,0,1]=(1.31766298118)
        ref[4,1,0,2]=(1.51544532829)
        ref[4,1,1,0]=(0.878390723411)
        ref[4,1,1,1]=(-0.427741582253)
        ref[4,1,1,2]=(0.0324341706939)
        ref[5,0,0,0]=(-0.445960121294)
        ref[5,0,0,1]=(-0.215668362178)
        ref[5,0,0,2]=(0.147632419682)
        ref[5,0,1,0]=(-0.0781270285912)
        ref[5,0,1,1]=(0.782990026347)
        ref[5,0,1,2]=(-1.28046463036)
        ref[5,1,0,0]=(-0.385003656622)
        ref[5,1,0,1]=(-1.02087438188)
        ref[5,1,0,2]=(-0.594357838151)
        ref[5,1,1,0]=(-0.676669731714)
        ref[5,1,1,1]=(-0.28685220691)
        ref[5,1,1,2]=(0.335774697674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.729541999217)*x[0]**o+(-0.0840038177686)*x[0]+(-0.843340387423)*x[1]**o+(0.835228033597)*x[1]
        ref[0]=o*(0.729541999217)*x_ref[0]**(o-1)+(-0.0840038177686)
        ref[1]=o*(-0.843340387423)*x_ref[1]**(o-1)+(0.835228033597)
      else:
        s=(-0.842281311478)*x[0]**o+(-0.814605655879)*x[0]+(0.626451534537)*x[1]**o+(-0.933916049385)*x[1]+(0.665647219119)*x[2]**o+(-0.320674717559)*x[2]
        ref[0]=o*(-0.842281311478)*x_ref[0]**(o-1)+(-0.814605655879)
        ref[1]=o*(0.626451534537)*x_ref[1]**(o-1)+(-0.933916049385)
        ref[2]=o*(0.665647219119)*x_ref[2]**(o-1)+(-0.320674717559)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.439352379619)*x[0]**o+(0.147328198635)*x[0]+(0.247834811224)*x[1]**o+(-0.464888499837)*x[1]
        s[1]=(0.468286938618)*x[0]**o+(-0.37473605363)*x[0]+(-0.957221570057)*x[1]**o+(0.192131525732)*x[1]
        ref[0,0]=o*(0.439352379619)*x_ref[0]**(o-1)+(0.147328198635)
        ref[0,1]=o*(0.247834811224)*x_ref[1]**(o-1)+(-0.464888499837)
        ref[1,0]=o*(0.468286938618)*x_ref[0]**(o-1)+(-0.37473605363)
        ref[1,1]=o*(-0.957221570057)*x_ref[1]**(o-1)+(0.192131525732)
      else:
        s[0]=(-0.608854640717)*x[0]**o+(-0.185932852919)*x[0]+(0.845083724014)*x[1]**o+(0.674307303063)*x[1]+(-0.981577175928)*x[2]**o+(0.59694635612)*x[2]
        s[1]=(0.229588079023)*x[0]**o+(0.0251610523352)*x[0]+(0.120319529556)*x[1]**o+(-0.6189763968)*x[1]+(-0.630398590893)*x[2]**o+(-0.675207219617)*x[2]
        ref[0,0]=o*(-0.608854640717)*x_ref[0]**(o-1)+(-0.185932852919)
        ref[0,1]=o*(0.845083724014)*x_ref[1]**(o-1)+(0.674307303063)
        ref[0,2]=o*(-0.981577175928)*x_ref[2]**(o-1)+(0.59694635612)
        ref[1,0]=o*(0.229588079023)*x_ref[0]**(o-1)+(0.0251610523352)
        ref[1,1]=o*(0.120319529556)*x_ref[1]**(o-1)+(-0.6189763968)
        ref[1,2]=o*(-0.630398590893)*x_ref[2]**(o-1)+(-0.675207219617)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.047995114526)*x[0]**o+(-0.0882500144924)*x[0]+(-0.418291362721)*x[1]**o+(0.76104926053)*x[1]
        s[0,1]=(-0.862451726051)*x[0]**o+(-0.752156684628)*x[0]+(0.0539876744263)*x[1]**o+(0.0136018280853)*x[1]
        s[0,2]=(0.356154299906)*x[0]**o+(-0.119273405197)*x[0]+(-0.55202199349)*x[1]**o+(0.976767610299)*x[1]
        s[0,3]=(0.26210854969)*x[0]**o+(0.492516191788)*x[0]+(-0.187803726963)*x[1]**o+(0.331043415981)*x[1]
        s[0,4]=(0.995092330192)*x[0]**o+(0.798072438804)*x[0]+(-0.851959660211)*x[1]**o+(0.376241209687)*x[1]
        s[1,0]=(-0.819203501556)*x[0]**o+(0.158244838297)*x[0]+(-0.0854093261543)*x[1]**o+(-0.734166609629)*x[1]
        s[1,1]=(0.953095931566)*x[0]**o+(0.245967282177)*x[0]+(0.024143077003)*x[1]**o+(0.591611742804)*x[1]
        s[1,2]=(0.515929639772)*x[0]**o+(0.0656789332058)*x[0]+(-0.486542111002)*x[1]**o+(-0.35357048557)*x[1]
        s[1,3]=(0.687461558204)*x[0]**o+(-0.885516846762)*x[0]+(-0.837556168903)*x[1]**o+(-0.263453643201)*x[1]
        s[1,4]=(0.202647080146)*x[0]**o+(0.0998694048742)*x[0]+(0.963315271661)*x[1]**o+(0.0738433857728)*x[1]
        s[2,0]=(-0.431227566486)*x[0]**o+(-0.811861069902)*x[0]+(0.207189408046)*x[1]**o+(-0.843636004903)*x[1]
        s[2,1]=(-0.947866812956)*x[0]**o+(-0.779542963233)*x[0]+(0.788757232024)*x[1]**o+(-0.798061295304)*x[1]
        s[2,2]=(-0.316894005317)*x[0]**o+(-0.0766086555356)*x[0]+(-0.321413016846)*x[1]**o+(-0.586742246307)*x[1]
        s[2,3]=(-0.366402163737)*x[0]**o+(0.906207257774)*x[0]+(0.42214475079)*x[1]**o+(-0.927560770588)*x[1]
        s[2,4]=(-0.355651171461)*x[0]**o+(0.636115063757)*x[0]+(0.425219309267)*x[1]**o+(0.461939146758)*x[1]
        s[3,0]=(-0.0691712814483)*x[0]**o+(0.221346711903)*x[0]+(0.223896511845)*x[1]**o+(0.407991412109)*x[1]
        s[3,1]=(-0.437261446252)*x[0]**o+(0.0175743759667)*x[0]+(-0.937854235512)*x[1]**o+(0.814464696622)*x[1]
        s[3,2]=(-0.246296303859)*x[0]**o+(0.290577742331)*x[0]+(0.648443639897)*x[1]**o+(-0.233763313065)*x[1]
        s[3,3]=(-0.574728457793)*x[0]**o+(-0.544796515995)*x[0]+(-0.411409144577)*x[1]**o+(-0.491277131741)*x[1]
        s[3,4]=(0.894563379439)*x[0]**o+(-0.495844199423)*x[0]+(-0.580716302019)*x[1]**o+(-0.0192249355836)*x[1]
        ref[0,0,0]=o*(0.047995114526)*x_ref[0]**(o-1)+(-0.0882500144924)
        ref[0,0,1]=o*(-0.418291362721)*x_ref[1]**(o-1)+(0.76104926053)
        ref[0,1,0]=o*(-0.862451726051)*x_ref[0]**(o-1)+(-0.752156684628)
        ref[0,1,1]=o*(0.0539876744263)*x_ref[1]**(o-1)+(0.0136018280853)
        ref[0,2,0]=o*(0.356154299906)*x_ref[0]**(o-1)+(-0.119273405197)
        ref[0,2,1]=o*(-0.55202199349)*x_ref[1]**(o-1)+(0.976767610299)
        ref[0,3,0]=o*(0.26210854969)*x_ref[0]**(o-1)+(0.492516191788)
        ref[0,3,1]=o*(-0.187803726963)*x_ref[1]**(o-1)+(0.331043415981)
        ref[0,4,0]=o*(0.995092330192)*x_ref[0]**(o-1)+(0.798072438804)
        ref[0,4,1]=o*(-0.851959660211)*x_ref[1]**(o-1)+(0.376241209687)
        ref[1,0,0]=o*(-0.819203501556)*x_ref[0]**(o-1)+(0.158244838297)
        ref[1,0,1]=o*(-0.0854093261543)*x_ref[1]**(o-1)+(-0.734166609629)
        ref[1,1,0]=o*(0.953095931566)*x_ref[0]**(o-1)+(0.245967282177)
        ref[1,1,1]=o*(0.024143077003)*x_ref[1]**(o-1)+(0.591611742804)
        ref[1,2,0]=o*(0.515929639772)*x_ref[0]**(o-1)+(0.0656789332058)
        ref[1,2,1]=o*(-0.486542111002)*x_ref[1]**(o-1)+(-0.35357048557)
        ref[1,3,0]=o*(0.687461558204)*x_ref[0]**(o-1)+(-0.885516846762)
        ref[1,3,1]=o*(-0.837556168903)*x_ref[1]**(o-1)+(-0.263453643201)
        ref[1,4,0]=o*(0.202647080146)*x_ref[0]**(o-1)+(0.0998694048742)
        ref[1,4,1]=o*(0.963315271661)*x_ref[1]**(o-1)+(0.0738433857728)
        ref[2,0,0]=o*(-0.431227566486)*x_ref[0]**(o-1)+(-0.811861069902)
        ref[2,0,1]=o*(0.207189408046)*x_ref[1]**(o-1)+(-0.843636004903)
        ref[2,1,0]=o*(-0.947866812956)*x_ref[0]**(o-1)+(-0.779542963233)
        ref[2,1,1]=o*(0.788757232024)*x_ref[1]**(o-1)+(-0.798061295304)
        ref[2,2,0]=o*(-0.316894005317)*x_ref[0]**(o-1)+(-0.0766086555356)
        ref[2,2,1]=o*(-0.321413016846)*x_ref[1]**(o-1)+(-0.586742246307)
        ref[2,3,0]=o*(-0.366402163737)*x_ref[0]**(o-1)+(0.906207257774)
        ref[2,3,1]=o*(0.42214475079)*x_ref[1]**(o-1)+(-0.927560770588)
        ref[2,4,0]=o*(-0.355651171461)*x_ref[0]**(o-1)+(0.636115063757)
        ref[2,4,1]=o*(0.425219309267)*x_ref[1]**(o-1)+(0.461939146758)
        ref[3,0,0]=o*(-0.0691712814483)*x_ref[0]**(o-1)+(0.221346711903)
        ref[3,0,1]=o*(0.223896511845)*x_ref[1]**(o-1)+(0.407991412109)
        ref[3,1,0]=o*(-0.437261446252)*x_ref[0]**(o-1)+(0.0175743759667)
        ref[3,1,1]=o*(-0.937854235512)*x_ref[1]**(o-1)+(0.814464696622)
        ref[3,2,0]=o*(-0.246296303859)*x_ref[0]**(o-1)+(0.290577742331)
        ref[3,2,1]=o*(0.648443639897)*x_ref[1]**(o-1)+(-0.233763313065)
        ref[3,3,0]=o*(-0.574728457793)*x_ref[0]**(o-1)+(-0.544796515995)
        ref[3,3,1]=o*(-0.411409144577)*x_ref[1]**(o-1)+(-0.491277131741)
        ref[3,4,0]=o*(0.894563379439)*x_ref[0]**(o-1)+(-0.495844199423)
        ref[3,4,1]=o*(-0.580716302019)*x_ref[1]**(o-1)+(-0.0192249355836)
      else:
        s[0,0]=(-0.440629362137)*x[0]**o+(0.432027111899)*x[0]+(0.485612378024)*x[1]**o+(0.100195330045)*x[1]+(-0.107489415991)*x[2]**o+(-0.871419715951)*x[2]
        s[0,1]=(-0.998817226455)*x[0]**o+(-0.27667660331)*x[0]+(-0.867725500303)*x[1]**o+(0.936977636906)*x[1]+(0.525970908002)*x[2]**o+(-0.279717686524)*x[2]
        s[0,2]=(-0.422100254422)*x[0]**o+(0.554292080667)*x[0]+(-0.0240387239116)*x[1]**o+(0.899626314131)*x[1]+(-0.982259001765)*x[2]**o+(-0.171863457765)*x[2]
        s[0,3]=(-0.683481609192)*x[0]**o+(0.604674984874)*x[0]+(0.707470056964)*x[1]**o+(0.775025878411)*x[1]+(0.370390201812)*x[2]**o+(-0.59247534281)*x[2]
        s[0,4]=(-0.435112832975)*x[0]**o+(-0.448385079313)*x[0]+(-0.467168230746)*x[1]**o+(0.824665416362)*x[1]+(-0.375723812297)*x[2]**o+(0.219591728252)*x[2]
        s[1,0]=(-0.946577267946)*x[0]**o+(-0.235619974774)*x[0]+(-0.440845837468)*x[1]**o+(0.523173708473)*x[1]+(0.138768169193)*x[2]**o+(0.0800449024418)*x[2]
        s[1,1]=(0.856208167581)*x[0]**o+(-0.949001152315)*x[0]+(0.603732417952)*x[1]**o+(-0.144434407904)*x[1]+(0.771914944319)*x[2]**o+(0.295663032072)*x[2]
        s[1,2]=(-0.0748077749478)*x[0]**o+(-0.800653291967)*x[0]+(-0.723295002911)*x[1]**o+(-0.459397931378)*x[1]+(-0.680567962158)*x[2]**o+(-0.397991544426)*x[2]
        s[1,3]=(-0.160866462235)*x[0]**o+(-0.269568432554)*x[0]+(0.760053321601)*x[1]**o+(0.636712875038)*x[1]+(0.718827472409)*x[2]**o+(-0.285961313882)*x[2]
        s[1,4]=(-0.192019161878)*x[0]**o+(0.376965303206)*x[0]+(-0.196459950888)*x[1]**o+(-0.816226430578)*x[1]+(0.627440208737)*x[2]**o+(0.846061231487)*x[2]
        s[2,0]=(-0.726210859697)*x[0]**o+(-0.931805040577)*x[0]+(-0.330082314232)*x[1]**o+(-0.594241678324)*x[1]+(-0.660189952658)*x[2]**o+(-0.788716966781)*x[2]
        s[2,1]=(-0.434348461759)*x[0]**o+(0.386897792713)*x[0]+(0.0471151132814)*x[1]**o+(-0.399299502588)*x[1]+(-0.412695129309)*x[2]**o+(-0.286460071055)*x[2]
        s[2,2]=(-0.702766222291)*x[0]**o+(-0.296328795371)*x[0]+(-0.569776274584)*x[1]**o+(-0.369361432195)*x[1]+(-0.308516249924)*x[2]**o+(0.025646183412)*x[2]
        s[2,3]=(-0.286989129847)*x[0]**o+(-0.152361831045)*x[0]+(-0.909624633862)*x[1]**o+(-0.920451186077)*x[1]+(-0.375390879331)*x[2]**o+(-0.905199847241)*x[2]
        s[2,4]=(0.619800447118)*x[0]**o+(-0.27317497769)*x[0]+(-0.0136479991947)*x[1]**o+(0.141634912432)*x[1]+(-0.61405995454)*x[2]**o+(-0.505857399048)*x[2]
        s[3,0]=(-0.782882873567)*x[0]**o+(-0.7736979883)*x[0]+(-0.322501480607)*x[1]**o+(0.839395743431)*x[1]+(-0.434294403981)*x[2]**o+(0.542823987177)*x[2]
        s[3,1]=(-0.802044187937)*x[0]**o+(-0.442612586193)*x[0]+(0.436047557305)*x[1]**o+(0.497708094931)*x[1]+(0.642169738987)*x[2]**o+(0.944050667638)*x[2]
        s[3,2]=(0.557014549515)*x[0]**o+(0.491428899905)*x[0]+(0.983181819191)*x[1]**o+(0.132058074766)*x[1]+(-0.260405939651)*x[2]**o+(0.101577990826)*x[2]
        s[3,3]=(0.595058474104)*x[0]**o+(-0.822604538462)*x[0]+(-0.723088531137)*x[1]**o+(-0.313808088359)*x[1]+(0.559816139446)*x[2]**o+(-0.513647272314)*x[2]
        s[3,4]=(-0.87608199975)*x[0]**o+(0.0585891509732)*x[0]+(-0.194566629493)*x[1]**o+(0.278612435937)*x[1]+(0.170683503576)*x[2]**o+(0.936525425446)*x[2]
        ref[0,0,0]=o*(-0.440629362137)*x_ref[0]**(o-1)+(0.432027111899)
        ref[0,0,1]=o*(0.485612378024)*x_ref[1]**(o-1)+(0.100195330045)
        ref[0,0,2]=o*(-0.107489415991)*x_ref[2]**(o-1)+(-0.871419715951)
        ref[0,1,0]=o*(-0.998817226455)*x_ref[0]**(o-1)+(-0.27667660331)
        ref[0,1,1]=o*(-0.867725500303)*x_ref[1]**(o-1)+(0.936977636906)
        ref[0,1,2]=o*(0.525970908002)*x_ref[2]**(o-1)+(-0.279717686524)
        ref[0,2,0]=o*(-0.422100254422)*x_ref[0]**(o-1)+(0.554292080667)
        ref[0,2,1]=o*(-0.0240387239116)*x_ref[1]**(o-1)+(0.899626314131)
        ref[0,2,2]=o*(-0.982259001765)*x_ref[2]**(o-1)+(-0.171863457765)
        ref[0,3,0]=o*(-0.683481609192)*x_ref[0]**(o-1)+(0.604674984874)
        ref[0,3,1]=o*(0.707470056964)*x_ref[1]**(o-1)+(0.775025878411)
        ref[0,3,2]=o*(0.370390201812)*x_ref[2]**(o-1)+(-0.59247534281)
        ref[0,4,0]=o*(-0.435112832975)*x_ref[0]**(o-1)+(-0.448385079313)
        ref[0,4,1]=o*(-0.467168230746)*x_ref[1]**(o-1)+(0.824665416362)
        ref[0,4,2]=o*(-0.375723812297)*x_ref[2]**(o-1)+(0.219591728252)
        ref[1,0,0]=o*(-0.946577267946)*x_ref[0]**(o-1)+(-0.235619974774)
        ref[1,0,1]=o*(-0.440845837468)*x_ref[1]**(o-1)+(0.523173708473)
        ref[1,0,2]=o*(0.138768169193)*x_ref[2]**(o-1)+(0.0800449024418)
        ref[1,1,0]=o*(0.856208167581)*x_ref[0]**(o-1)+(-0.949001152315)
        ref[1,1,1]=o*(0.603732417952)*x_ref[1]**(o-1)+(-0.144434407904)
        ref[1,1,2]=o*(0.771914944319)*x_ref[2]**(o-1)+(0.295663032072)
        ref[1,2,0]=o*(-0.0748077749478)*x_ref[0]**(o-1)+(-0.800653291967)
        ref[1,2,1]=o*(-0.723295002911)*x_ref[1]**(o-1)+(-0.459397931378)
        ref[1,2,2]=o*(-0.680567962158)*x_ref[2]**(o-1)+(-0.397991544426)
        ref[1,3,0]=o*(-0.160866462235)*x_ref[0]**(o-1)+(-0.269568432554)
        ref[1,3,1]=o*(0.760053321601)*x_ref[1]**(o-1)+(0.636712875038)
        ref[1,3,2]=o*(0.718827472409)*x_ref[2]**(o-1)+(-0.285961313882)
        ref[1,4,0]=o*(-0.192019161878)*x_ref[0]**(o-1)+(0.376965303206)
        ref[1,4,1]=o*(-0.196459950888)*x_ref[1]**(o-1)+(-0.816226430578)
        ref[1,4,2]=o*(0.627440208737)*x_ref[2]**(o-1)+(0.846061231487)
        ref[2,0,0]=o*(-0.726210859697)*x_ref[0]**(o-1)+(-0.931805040577)
        ref[2,0,1]=o*(-0.330082314232)*x_ref[1]**(o-1)+(-0.594241678324)
        ref[2,0,2]=o*(-0.660189952658)*x_ref[2]**(o-1)+(-0.788716966781)
        ref[2,1,0]=o*(-0.434348461759)*x_ref[0]**(o-1)+(0.386897792713)
        ref[2,1,1]=o*(0.0471151132814)*x_ref[1]**(o-1)+(-0.399299502588)
        ref[2,1,2]=o*(-0.412695129309)*x_ref[2]**(o-1)+(-0.286460071055)
        ref[2,2,0]=o*(-0.702766222291)*x_ref[0]**(o-1)+(-0.296328795371)
        ref[2,2,1]=o*(-0.569776274584)*x_ref[1]**(o-1)+(-0.369361432195)
        ref[2,2,2]=o*(-0.308516249924)*x_ref[2]**(o-1)+(0.025646183412)
        ref[2,3,0]=o*(-0.286989129847)*x_ref[0]**(o-1)+(-0.152361831045)
        ref[2,3,1]=o*(-0.909624633862)*x_ref[1]**(o-1)+(-0.920451186077)
        ref[2,3,2]=o*(-0.375390879331)*x_ref[2]**(o-1)+(-0.905199847241)
        ref[2,4,0]=o*(0.619800447118)*x_ref[0]**(o-1)+(-0.27317497769)
        ref[2,4,1]=o*(-0.0136479991947)*x_ref[1]**(o-1)+(0.141634912432)
        ref[2,4,2]=o*(-0.61405995454)*x_ref[2]**(o-1)+(-0.505857399048)
        ref[3,0,0]=o*(-0.782882873567)*x_ref[0]**(o-1)+(-0.7736979883)
        ref[3,0,1]=o*(-0.322501480607)*x_ref[1]**(o-1)+(0.839395743431)
        ref[3,0,2]=o*(-0.434294403981)*x_ref[2]**(o-1)+(0.542823987177)
        ref[3,1,0]=o*(-0.802044187937)*x_ref[0]**(o-1)+(-0.442612586193)
        ref[3,1,1]=o*(0.436047557305)*x_ref[1]**(o-1)+(0.497708094931)
        ref[3,1,2]=o*(0.642169738987)*x_ref[2]**(o-1)+(0.944050667638)
        ref[3,2,0]=o*(0.557014549515)*x_ref[0]**(o-1)+(0.491428899905)
        ref[3,2,1]=o*(0.983181819191)*x_ref[1]**(o-1)+(0.132058074766)
        ref[3,2,2]=o*(-0.260405939651)*x_ref[2]**(o-1)+(0.101577990826)
        ref[3,3,0]=o*(0.595058474104)*x_ref[0]**(o-1)+(-0.822604538462)
        ref[3,3,1]=o*(-0.723088531137)*x_ref[1]**(o-1)+(-0.313808088359)
        ref[3,3,2]=o*(0.559816139446)*x_ref[2]**(o-1)+(-0.513647272314)
        ref[3,4,0]=o*(-0.87608199975)*x_ref[0]**(o-1)+(0.0585891509732)
        ref[3,4,1]=o*(-0.194566629493)*x_ref[1]**(o-1)+(0.278612435937)
        ref[3,4,2]=o*(0.170683503576)*x_ref[2]**(o-1)+(0.936525425446)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(0.212897121743)*x[0]**o+(0.270129409854)*x[0]+(0.187351601416)*x[1]**o+(0.681503494742)*x[1]
        s[0,0,1]=(0.0979281898609)*x[0]**o+(0.469261780546)*x[0]+(0.285691941306)*x[1]**o+(0.605427118235)*x[1]
        s[0,1,0]=(0.798192631806)*x[0]**o+(0.710948168289)*x[0]+(0.37314849797)*x[1]**o+(0.101332891852)*x[1]
        s[0,1,1]=(0.670619373787)*x[0]**o+(0.87986446998)*x[0]+(0.273598886571)*x[1]**o+(0.642775771189)*x[1]
        s[1,0,0]=(-0.579525230101)*x[0]**o+(-0.336522843918)*x[0]+(0.794313044824)*x[1]**o+(0.138149528558)*x[1]
        s[1,0,1]=(-0.790589960141)*x[0]**o+(-0.519541468607)*x[0]+(0.047053031477)*x[1]**o+(-0.376948328317)*x[1]
        s[1,1,0]=(-0.63652515985)*x[0]**o+(-0.796241202497)*x[0]+(0.961598412032)*x[1]**o+(0.392696716004)*x[1]
        s[1,1,1]=(-0.406921118202)*x[0]**o+(-0.117551313946)*x[0]+(-0.666279462502)*x[1]**o+(-0.0899789554615)*x[1]
        s[2,0,0]=(-0.863791811378)*x[0]**o+(0.594940266664)*x[0]+(0.872448531721)*x[1]**o+(0.874689552083)*x[1]
        s[2,0,1]=(0.243777114221)*x[0]**o+(-0.776695107306)*x[0]+(0.794794829454)*x[1]**o+(-0.318628386313)*x[1]
        s[2,1,0]=(0.0962259662597)*x[0]**o+(0.541353121994)*x[0]+(-0.672755845501)*x[1]**o+(-0.257046721225)*x[1]
        s[2,1,1]=(-0.943502231769)*x[0]**o+(-0.202236730464)*x[0]+(-0.411084369491)*x[1]**o+(0.954831689569)*x[1]
        s[3,0,0]=(0.768277437177)*x[0]**o+(0.641045872586)*x[0]+(0.456435914145)*x[1]**o+(-0.538634781112)*x[1]
        s[3,0,1]=(0.0107901515277)*x[0]**o+(0.204809134773)*x[0]+(0.632781493458)*x[1]**o+(0.813936796774)*x[1]
        s[3,1,0]=(0.187196185344)*x[0]**o+(-0.974329803149)*x[0]+(-0.131567139248)*x[1]**o+(-0.205725336514)*x[1]
        s[3,1,1]=(0.747378585446)*x[0]**o+(0.126887766052)*x[0]+(-0.660302189752)*x[1]**o+(0.227362395796)*x[1]
        s[4,0,0]=(0.822029556554)*x[0]**o+(0.742977721062)*x[0]+(0.972349490061)*x[1]**o+(0.307748083079)*x[1]
        s[4,0,1]=(-0.0347009705896)*x[0]**o+(0.506556900045)*x[0]+(-0.700430813955)*x[1]**o+(0.606413581865)*x[1]
        s[4,1,0]=(-0.69868976773)*x[0]**o+(0.447214222886)*x[0]+(0.290522797177)*x[1]**o+(0.139281488147)*x[1]
        s[4,1,1]=(0.850722242552)*x[0]**o+(0.578613819769)*x[0]+(-0.983680277805)*x[1]**o+(-0.0737991679361)*x[1]
        s[5,0,0]=(0.164023891553)*x[0]**o+(-0.398812116857)*x[0]+(0.754040638026)*x[1]**o+(0.890632418705)*x[1]
        s[5,0,1]=(0.176036561413)*x[0]**o+(0.551905509271)*x[0]+(0.584521803718)*x[1]**o+(0.359816759406)*x[1]
        s[5,1,0]=(0.728503623092)*x[0]**o+(0.448628087199)*x[0]+(0.7693890417)*x[1]**o+(-0.148028432439)*x[1]
        s[5,1,1]=(-0.974113865308)*x[0]**o+(-0.271026924938)*x[0]+(0.0918858163816)*x[1]**o+(0.441895253556)*x[1]
        ref[0,0,0,0]=o*(0.212897121743)*x_ref[0]**(o-1)+(0.270129409854)
        ref[0,0,0,1]=o*(0.187351601416)*x_ref[1]**(o-1)+(0.681503494742)
        ref[0,0,1,0]=o*(0.0979281898609)*x_ref[0]**(o-1)+(0.469261780546)
        ref[0,0,1,1]=o*(0.285691941306)*x_ref[1]**(o-1)+(0.605427118235)
        ref[0,1,0,0]=o*(0.798192631806)*x_ref[0]**(o-1)+(0.710948168289)
        ref[0,1,0,1]=o*(0.37314849797)*x_ref[1]**(o-1)+(0.101332891852)
        ref[0,1,1,0]=o*(0.670619373787)*x_ref[0]**(o-1)+(0.87986446998)
        ref[0,1,1,1]=o*(0.273598886571)*x_ref[1]**(o-1)+(0.642775771189)
        ref[1,0,0,0]=o*(-0.579525230101)*x_ref[0]**(o-1)+(-0.336522843918)
        ref[1,0,0,1]=o*(0.794313044824)*x_ref[1]**(o-1)+(0.138149528558)
        ref[1,0,1,0]=o*(-0.790589960141)*x_ref[0]**(o-1)+(-0.519541468607)
        ref[1,0,1,1]=o*(0.047053031477)*x_ref[1]**(o-1)+(-0.376948328317)
        ref[1,1,0,0]=o*(-0.63652515985)*x_ref[0]**(o-1)+(-0.796241202497)
        ref[1,1,0,1]=o*(0.961598412032)*x_ref[1]**(o-1)+(0.392696716004)
        ref[1,1,1,0]=o*(-0.406921118202)*x_ref[0]**(o-1)+(-0.117551313946)
        ref[1,1,1,1]=o*(-0.666279462502)*x_ref[1]**(o-1)+(-0.0899789554615)
        ref[2,0,0,0]=o*(-0.863791811378)*x_ref[0]**(o-1)+(0.594940266664)
        ref[2,0,0,1]=o*(0.872448531721)*x_ref[1]**(o-1)+(0.874689552083)
        ref[2,0,1,0]=o*(0.243777114221)*x_ref[0]**(o-1)+(-0.776695107306)
        ref[2,0,1,1]=o*(0.794794829454)*x_ref[1]**(o-1)+(-0.318628386313)
        ref[2,1,0,0]=o*(0.0962259662597)*x_ref[0]**(o-1)+(0.541353121994)
        ref[2,1,0,1]=o*(-0.672755845501)*x_ref[1]**(o-1)+(-0.257046721225)
        ref[2,1,1,0]=o*(-0.943502231769)*x_ref[0]**(o-1)+(-0.202236730464)
        ref[2,1,1,1]=o*(-0.411084369491)*x_ref[1]**(o-1)+(0.954831689569)
        ref[3,0,0,0]=o*(0.768277437177)*x_ref[0]**(o-1)+(0.641045872586)
        ref[3,0,0,1]=o*(0.456435914145)*x_ref[1]**(o-1)+(-0.538634781112)
        ref[3,0,1,0]=o*(0.0107901515277)*x_ref[0]**(o-1)+(0.204809134773)
        ref[3,0,1,1]=o*(0.632781493458)*x_ref[1]**(o-1)+(0.813936796774)
        ref[3,1,0,0]=o*(0.187196185344)*x_ref[0]**(o-1)+(-0.974329803149)
        ref[3,1,0,1]=o*(-0.131567139248)*x_ref[1]**(o-1)+(-0.205725336514)
        ref[3,1,1,0]=o*(0.747378585446)*x_ref[0]**(o-1)+(0.126887766052)
        ref[3,1,1,1]=o*(-0.660302189752)*x_ref[1]**(o-1)+(0.227362395796)
        ref[4,0,0,0]=o*(0.822029556554)*x_ref[0]**(o-1)+(0.742977721062)
        ref[4,0,0,1]=o*(0.972349490061)*x_ref[1]**(o-1)+(0.307748083079)
        ref[4,0,1,0]=o*(-0.0347009705896)*x_ref[0]**(o-1)+(0.506556900045)
        ref[4,0,1,1]=o*(-0.700430813955)*x_ref[1]**(o-1)+(0.606413581865)
        ref[4,1,0,0]=o*(-0.69868976773)*x_ref[0]**(o-1)+(0.447214222886)
        ref[4,1,0,1]=o*(0.290522797177)*x_ref[1]**(o-1)+(0.139281488147)
        ref[4,1,1,0]=o*(0.850722242552)*x_ref[0]**(o-1)+(0.578613819769)
        ref[4,1,1,1]=o*(-0.983680277805)*x_ref[1]**(o-1)+(-0.0737991679361)
        ref[5,0,0,0]=o*(0.164023891553)*x_ref[0]**(o-1)+(-0.398812116857)
        ref[5,0,0,1]=o*(0.754040638026)*x_ref[1]**(o-1)+(0.890632418705)
        ref[5,0,1,0]=o*(0.176036561413)*x_ref[0]**(o-1)+(0.551905509271)
        ref[5,0,1,1]=o*(0.584521803718)*x_ref[1]**(o-1)+(0.359816759406)
        ref[5,1,0,0]=o*(0.728503623092)*x_ref[0]**(o-1)+(0.448628087199)
        ref[5,1,0,1]=o*(0.7693890417)*x_ref[1]**(o-1)+(-0.148028432439)
        ref[5,1,1,0]=o*(-0.974113865308)*x_ref[0]**(o-1)+(-0.271026924938)
        ref[5,1,1,1]=o*(0.0918858163816)*x_ref[1]**(o-1)+(0.441895253556)
      else:
        s[0,0,0]=(0.902798077985)*x[0]**o+(-0.513564836003)*x[0]+(-0.56740021175)*x[1]**o+(-0.987494297109)*x[1]+(0.701885375414)*x[2]**o+(0.457818844785)*x[2]
        s[0,0,1]=(0.961682822532)*x[0]**o+(-0.155805753312)*x[0]+(0.664632247096)*x[1]**o+(-0.416626235035)*x[1]+(-0.710160165264)*x[2]**o+(0.604159028896)*x[2]
        s[0,1,0]=(-0.331535871964)*x[0]**o+(-0.417629864265)*x[0]+(0.206294208748)*x[1]**o+(-0.259792550744)*x[1]+(-0.125585762102)*x[2]**o+(-0.893437959215)*x[2]
        s[0,1,1]=(0.614124081239)*x[0]**o+(-0.974461795616)*x[0]+(-0.978571310051)*x[1]**o+(-0.696753351954)*x[1]+(-0.125579053404)*x[2]**o+(0.744152004821)*x[2]
        s[1,0,0]=(0.470195589624)*x[0]**o+(-0.543298404217)*x[0]+(-0.050519561506)*x[1]**o+(-0.140360629835)*x[1]+(0.198214376652)*x[2]**o+(0.424647243144)*x[2]
        s[1,0,1]=(0.726479572534)*x[0]**o+(0.553604934856)*x[0]+(0.0319217924147)*x[1]**o+(0.281723632246)*x[1]+(-0.525936361143)*x[2]**o+(0.0826305292145)*x[2]
        s[1,1,0]=(0.731149931727)*x[0]**o+(-0.792038038336)*x[0]+(0.376591980524)*x[1]**o+(-0.533851270214)*x[1]+(0.228328372658)*x[2]**o+(-0.685994122324)*x[2]
        s[1,1,1]=(0.0316092245143)*x[0]**o+(0.272078804647)*x[0]+(0.232701533258)*x[1]**o+(0.484166193421)*x[1]+(0.810979214906)*x[2]**o+(-0.678008492162)*x[2]
        s[2,0,0]=(-0.25934895114)*x[0]**o+(0.855703849413)*x[0]+(0.435458381315)*x[1]**o+(0.287983708847)*x[1]+(0.637280806162)*x[2]**o+(-0.871379201278)*x[2]
        s[2,0,1]=(-0.0628315253456)*x[0]**o+(-0.456689468581)*x[0]+(-0.990061219179)*x[1]**o+(0.103941195754)*x[1]+(0.683984409836)*x[2]**o+(-0.786055311041)*x[2]
        s[2,1,0]=(0.574774054041)*x[0]**o+(0.861386252917)*x[0]+(-0.817046727565)*x[1]**o+(-0.132779923567)*x[1]+(0.230879541156)*x[2]**o+(-0.774875301646)*x[2]
        s[2,1,1]=(-0.549231428363)*x[0]**o+(0.640633896033)*x[0]+(-0.0935356199455)*x[1]**o+(-0.139090953413)*x[1]+(0.00853390900484)*x[2]**o+(-0.974008998234)*x[2]
        s[3,0,0]=(-0.318959630087)*x[0]**o+(0.572260168906)*x[0]+(-0.715323370515)*x[1]**o+(-0.29409167385)*x[1]+(-0.417828539621)*x[2]**o+(-0.527658924511)*x[2]
        s[3,0,1]=(0.662794455331)*x[0]**o+(0.655652283888)*x[0]+(0.314776154296)*x[1]**o+(0.568698535574)*x[1]+(-0.715089514092)*x[2]**o+(-0.976898245896)*x[2]
        s[3,1,0]=(0.775469693327)*x[0]**o+(0.894695158486)*x[0]+(-0.0845263659865)*x[1]**o+(0.613321238688)*x[1]+(-0.261167781047)*x[2]**o+(-0.858407379125)*x[2]
        s[3,1,1]=(0.879739525497)*x[0]**o+(-0.196410556198)*x[0]+(0.245451670145)*x[1]**o+(0.881860540375)*x[1]+(-0.406601916222)*x[2]**o+(-0.698825194269)*x[2]
        s[4,0,0]=(-0.295318870536)*x[0]**o+(-0.386975265135)*x[0]+(-0.4694457277)*x[1]**o+(0.381202263202)*x[1]+(-0.246058013597)*x[2]**o+(-0.117116349193)*x[2]
        s[4,0,1]=(-0.52750487839)*x[0]**o+(0.239477947116)*x[0]+(0.415068508509)*x[1]**o+(0.499147016759)*x[1]+(-0.757947686224)*x[2]**o+(0.0356840223308)*x[2]
        s[4,1,0]=(-0.503519047063)*x[0]**o+(-0.983884111712)*x[0]+(0.289499262273)*x[1]**o+(0.477655179172)*x[1]+(0.267544491427)*x[2]**o+(0.577972060216)*x[2]
        s[4,1,1]=(-0.765677873271)*x[0]**o+(-0.574403742889)*x[0]+(0.0365362593269)*x[1]**o+(0.718127942002)*x[1]+(-0.882868921566)*x[2]**o+(-0.866965724039)*x[2]
        s[5,0,0]=(0.477311133484)*x[0]**o+(0.970292462756)*x[0]+(0.400262807291)*x[1]**o+(0.839779920106)*x[1]+(0.91075067745)*x[2]**o+(-0.171648787827)*x[2]
        s[5,0,1]=(-0.14817999498)*x[0]**o+(-0.284074361404)*x[0]+(-0.809114317075)*x[1]**o+(-0.421481538417)*x[1]+(0.617793146872)*x[2]**o+(0.808487061398)*x[2]
        s[5,1,0]=(-0.750064834677)*x[0]**o+(0.595717210851)*x[0]+(0.499697404346)*x[1]**o+(-0.890435534805)*x[1]+(-0.705182209377)*x[2]**o+(-0.831416292443)*x[2]
        s[5,1,1]=(-0.93237665803)*x[0]**o+(-0.608748654097)*x[0]+(-0.163402758239)*x[1]**o+(0.762304956717)*x[1]+(-0.236439076616)*x[2]**o+(-0.827173574464)*x[2]
        ref[0,0,0,0]=o*(0.902798077985)*x_ref[0]**(o-1)+(-0.513564836003)
        ref[0,0,0,1]=o*(-0.56740021175)*x_ref[1]**(o-1)+(-0.987494297109)
        ref[0,0,0,2]=o*(0.701885375414)*x_ref[2]**(o-1)+(0.457818844785)
        ref[0,0,1,0]=o*(0.961682822532)*x_ref[0]**(o-1)+(-0.155805753312)
        ref[0,0,1,1]=o*(0.664632247096)*x_ref[1]**(o-1)+(-0.416626235035)
        ref[0,0,1,2]=o*(-0.710160165264)*x_ref[2]**(o-1)+(0.604159028896)
        ref[0,1,0,0]=o*(-0.331535871964)*x_ref[0]**(o-1)+(-0.417629864265)
        ref[0,1,0,1]=o*(0.206294208748)*x_ref[1]**(o-1)+(-0.259792550744)
        ref[0,1,0,2]=o*(-0.125585762102)*x_ref[2]**(o-1)+(-0.893437959215)
        ref[0,1,1,0]=o*(0.614124081239)*x_ref[0]**(o-1)+(-0.974461795616)
        ref[0,1,1,1]=o*(-0.978571310051)*x_ref[1]**(o-1)+(-0.696753351954)
        ref[0,1,1,2]=o*(-0.125579053404)*x_ref[2]**(o-1)+(0.744152004821)
        ref[1,0,0,0]=o*(0.470195589624)*x_ref[0]**(o-1)+(-0.543298404217)
        ref[1,0,0,1]=o*(-0.050519561506)*x_ref[1]**(o-1)+(-0.140360629835)
        ref[1,0,0,2]=o*(0.198214376652)*x_ref[2]**(o-1)+(0.424647243144)
        ref[1,0,1,0]=o*(0.726479572534)*x_ref[0]**(o-1)+(0.553604934856)
        ref[1,0,1,1]=o*(0.0319217924147)*x_ref[1]**(o-1)+(0.281723632246)
        ref[1,0,1,2]=o*(-0.525936361143)*x_ref[2]**(o-1)+(0.0826305292145)
        ref[1,1,0,0]=o*(0.731149931727)*x_ref[0]**(o-1)+(-0.792038038336)
        ref[1,1,0,1]=o*(0.376591980524)*x_ref[1]**(o-1)+(-0.533851270214)
        ref[1,1,0,2]=o*(0.228328372658)*x_ref[2]**(o-1)+(-0.685994122324)
        ref[1,1,1,0]=o*(0.0316092245143)*x_ref[0]**(o-1)+(0.272078804647)
        ref[1,1,1,1]=o*(0.232701533258)*x_ref[1]**(o-1)+(0.484166193421)
        ref[1,1,1,2]=o*(0.810979214906)*x_ref[2]**(o-1)+(-0.678008492162)
        ref[2,0,0,0]=o*(-0.25934895114)*x_ref[0]**(o-1)+(0.855703849413)
        ref[2,0,0,1]=o*(0.435458381315)*x_ref[1]**(o-1)+(0.287983708847)
        ref[2,0,0,2]=o*(0.637280806162)*x_ref[2]**(o-1)+(-0.871379201278)
        ref[2,0,1,0]=o*(-0.0628315253456)*x_ref[0]**(o-1)+(-0.456689468581)
        ref[2,0,1,1]=o*(-0.990061219179)*x_ref[1]**(o-1)+(0.103941195754)
        ref[2,0,1,2]=o*(0.683984409836)*x_ref[2]**(o-1)+(-0.786055311041)
        ref[2,1,0,0]=o*(0.574774054041)*x_ref[0]**(o-1)+(0.861386252917)
        ref[2,1,0,1]=o*(-0.817046727565)*x_ref[1]**(o-1)+(-0.132779923567)
        ref[2,1,0,2]=o*(0.230879541156)*x_ref[2]**(o-1)+(-0.774875301646)
        ref[2,1,1,0]=o*(-0.549231428363)*x_ref[0]**(o-1)+(0.640633896033)
        ref[2,1,1,1]=o*(-0.0935356199455)*x_ref[1]**(o-1)+(-0.139090953413)
        ref[2,1,1,2]=o*(0.00853390900484)*x_ref[2]**(o-1)+(-0.974008998234)
        ref[3,0,0,0]=o*(-0.318959630087)*x_ref[0]**(o-1)+(0.572260168906)
        ref[3,0,0,1]=o*(-0.715323370515)*x_ref[1]**(o-1)+(-0.29409167385)
        ref[3,0,0,2]=o*(-0.417828539621)*x_ref[2]**(o-1)+(-0.527658924511)
        ref[3,0,1,0]=o*(0.662794455331)*x_ref[0]**(o-1)+(0.655652283888)
        ref[3,0,1,1]=o*(0.314776154296)*x_ref[1]**(o-1)+(0.568698535574)
        ref[3,0,1,2]=o*(-0.715089514092)*x_ref[2]**(o-1)+(-0.976898245896)
        ref[3,1,0,0]=o*(0.775469693327)*x_ref[0]**(o-1)+(0.894695158486)
        ref[3,1,0,1]=o*(-0.0845263659865)*x_ref[1]**(o-1)+(0.613321238688)
        ref[3,1,0,2]=o*(-0.261167781047)*x_ref[2]**(o-1)+(-0.858407379125)
        ref[3,1,1,0]=o*(0.879739525497)*x_ref[0]**(o-1)+(-0.196410556198)
        ref[3,1,1,1]=o*(0.245451670145)*x_ref[1]**(o-1)+(0.881860540375)
        ref[3,1,1,2]=o*(-0.406601916222)*x_ref[2]**(o-1)+(-0.698825194269)
        ref[4,0,0,0]=o*(-0.295318870536)*x_ref[0]**(o-1)+(-0.386975265135)
        ref[4,0,0,1]=o*(-0.4694457277)*x_ref[1]**(o-1)+(0.381202263202)
        ref[4,0,0,2]=o*(-0.246058013597)*x_ref[2]**(o-1)+(-0.117116349193)
        ref[4,0,1,0]=o*(-0.52750487839)*x_ref[0]**(o-1)+(0.239477947116)
        ref[4,0,1,1]=o*(0.415068508509)*x_ref[1]**(o-1)+(0.499147016759)
        ref[4,0,1,2]=o*(-0.757947686224)*x_ref[2]**(o-1)+(0.0356840223308)
        ref[4,1,0,0]=o*(-0.503519047063)*x_ref[0]**(o-1)+(-0.983884111712)
        ref[4,1,0,1]=o*(0.289499262273)*x_ref[1]**(o-1)+(0.477655179172)
        ref[4,1,0,2]=o*(0.267544491427)*x_ref[2]**(o-1)+(0.577972060216)
        ref[4,1,1,0]=o*(-0.765677873271)*x_ref[0]**(o-1)+(-0.574403742889)
        ref[4,1,1,1]=o*(0.0365362593269)*x_ref[1]**(o-1)+(0.718127942002)
        ref[4,1,1,2]=o*(-0.882868921566)*x_ref[2]**(o-1)+(-0.866965724039)
        ref[5,0,0,0]=o*(0.477311133484)*x_ref[0]**(o-1)+(0.970292462756)
        ref[5,0,0,1]=o*(0.400262807291)*x_ref[1]**(o-1)+(0.839779920106)
        ref[5,0,0,2]=o*(0.91075067745)*x_ref[2]**(o-1)+(-0.171648787827)
        ref[5,0,1,0]=o*(-0.14817999498)*x_ref[0]**(o-1)+(-0.284074361404)
        ref[5,0,1,1]=o*(-0.809114317075)*x_ref[1]**(o-1)+(-0.421481538417)
        ref[5,0,1,2]=o*(0.617793146872)*x_ref[2]**(o-1)+(0.808487061398)
        ref[5,1,0,0]=o*(-0.750064834677)*x_ref[0]**(o-1)+(0.595717210851)
        ref[5,1,0,1]=o*(0.499697404346)*x_ref[1]**(o-1)+(-0.890435534805)
        ref[5,1,0,2]=o*(-0.705182209377)*x_ref[2]**(o-1)+(-0.831416292443)
        ref[5,1,1,0]=o*(-0.93237665803)*x_ref[0]**(o-1)+(-0.608748654097)
        ref[5,1,1,1]=o*(-0.163402758239)*x_ref[1]**(o-1)+(0.762304956717)
        ref[5,1,1,2]=o*(-0.236439076616)*x_ref[2]**(o-1)+(-0.827173574464)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_Solution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.293125312175)*x[0]**o+(0.376150381261)*x[0]+(0.252430693686)*x[1]**o+(0.150428455322)*x[1]
        ref[0]=o*(0.293125312175)*x_ref[0]**(o-1)+(0.376150381261)
        ref[1]=o*(0.252430693686)*x_ref[1]**(o-1)+(0.150428455322)
      else:
        s=(0.978180509747)*x[0]**o+(0.768503254374)*x[0]+(0.113267806213)*x[1]**o+(-0.997552294934)*x[1]+(-0.643998032917)*x[2]**o+(-0.17168440847)*x[2]
        ref[0]=o*(0.978180509747)*x_ref[0]**(o-1)+(0.768503254374)
        ref[1]=o*(0.113267806213)*x_ref[1]**(o-1)+(-0.997552294934)
        ref[2]=o*(-0.643998032917)*x_ref[2]**(o-1)+(-0.17168440847)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_Solution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.138895230208)*x[0]**o+(-0.448856094505)*x[0]+(-0.82305383921)*x[1]**o+(-0.161535603589)*x[1]
        s[1]=(-0.302535177393)*x[0]**o+(0.811653356478)*x[0]+(0.05548155455)*x[1]**o+(-0.343903467756)*x[1]
        ref[0,0]=o*(-0.138895230208)*x_ref[0]**(o-1)+(-0.448856094505)
        ref[0,1]=o*(-0.82305383921)*x_ref[1]**(o-1)+(-0.161535603589)
        ref[1,0]=o*(-0.302535177393)*x_ref[0]**(o-1)+(0.811653356478)
        ref[1,1]=o*(0.05548155455)*x_ref[1]**(o-1)+(-0.343903467756)
      else:
        s[0]=(0.583525061796)*x[0]**o+(-0.891145747091)*x[0]+(-0.469656381595)*x[1]**o+(-0.0103398578248)*x[1]+(0.0970809480199)*x[2]**o+(0.652357321835)*x[2]
        s[1]=(-0.488885199473)*x[0]**o+(0.481806213481)*x[0]+(-0.7328445706)*x[1]**o+(0.28590334803)*x[1]+(0.044964568194)*x[2]**o+(0.474397408351)*x[2]
        ref[0,0]=o*(0.583525061796)*x_ref[0]**(o-1)+(-0.891145747091)
        ref[0,1]=o*(-0.469656381595)*x_ref[1]**(o-1)+(-0.0103398578248)
        ref[0,2]=o*(0.0970809480199)*x_ref[2]**(o-1)+(0.652357321835)
        ref[1,0]=o*(-0.488885199473)*x_ref[0]**(o-1)+(0.481806213481)
        ref[1,1]=o*(-0.7328445706)*x_ref[1]**(o-1)+(0.28590334803)
        ref[1,2]=o*(0.044964568194)*x_ref[2]**(o-1)+(0.474397408351)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_Solution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.872000208895)*x[0]**o+(-0.304556916188)*x[0]+(0.534581283951)*x[1]**o+(-0.626579480196)*x[1]
        s[0,1]=(-0.240150296068)*x[0]**o+(-0.31754292824)*x[0]+(-0.0574070276326)*x[1]**o+(-0.616687559393)*x[1]
        s[0,2]=(0.570345724046)*x[0]**o+(0.110952065807)*x[0]+(0.997553898152)*x[1]**o+(-0.503380742681)*x[1]
        s[0,3]=(0.872704383137)*x[0]**o+(0.686594842527)*x[0]+(0.798635469075)*x[1]**o+(0.43649341733)*x[1]
        s[0,4]=(-0.236636103417)*x[0]**o+(0.0762117276629)*x[0]+(-0.774027674603)*x[1]**o+(0.823651416506)*x[1]
        s[1,0]=(0.922041581692)*x[0]**o+(-0.284368621571)*x[0]+(-0.71200852665)*x[1]**o+(0.661348856406)*x[1]
        s[1,1]=(-0.402436007071)*x[0]**o+(-0.436850834963)*x[0]+(0.62748269836)*x[1]**o+(0.504691317707)*x[1]
        s[1,2]=(-0.793104721592)*x[0]**o+(-0.00597773372911)*x[0]+(0.0410611356405)*x[1]**o+(0.169129450548)*x[1]
        s[1,3]=(0.821521972958)*x[0]**o+(-0.784664003887)*x[0]+(-0.880383627011)*x[1]**o+(0.127311373905)*x[1]
        s[1,4]=(0.775700618787)*x[0]**o+(-0.837475890825)*x[0]+(0.426581954939)*x[1]**o+(0.404905797587)*x[1]
        s[2,0]=(-0.100956366427)*x[0]**o+(0.0933662209553)*x[0]+(-0.680046458225)*x[1]**o+(-0.145587503687)*x[1]
        s[2,1]=(0.166159597084)*x[0]**o+(-0.900241170518)*x[0]+(-0.168407179398)*x[1]**o+(-0.82352508068)*x[1]
        s[2,2]=(-0.476621892618)*x[0]**o+(-0.422712025977)*x[0]+(0.298151628461)*x[1]**o+(0.862589525974)*x[1]
        s[2,3]=(-0.0148783218305)*x[0]**o+(0.203140203381)*x[0]+(0.19407097983)*x[1]**o+(-0.772615096404)*x[1]
        s[2,4]=(0.656255800409)*x[0]**o+(-0.946330057363)*x[0]+(-0.257965999122)*x[1]**o+(0.71359373028)*x[1]
        s[3,0]=(-0.214762058983)*x[0]**o+(0.51144571871)*x[0]+(-0.147282047629)*x[1]**o+(0.709339363468)*x[1]
        s[3,1]=(-0.9112564909)*x[0]**o+(-0.900437328847)*x[0]+(-0.975324195939)*x[1]**o+(0.202889107529)*x[1]
        s[3,2]=(0.753787529389)*x[0]**o+(0.0630834387277)*x[0]+(-0.759168264562)*x[1]**o+(0.697873996051)*x[1]
        s[3,3]=(-0.0335165829084)*x[0]**o+(0.274748931927)*x[0]+(0.0686458601464)*x[1]**o+(-0.117608157399)*x[1]
        s[3,4]=(0.853158371839)*x[0]**o+(-0.166315314677)*x[0]+(0.789241330781)*x[1]**o+(-0.108867830466)*x[1]
        ref[0,0,0]=o*(0.872000208895)*x_ref[0]**(o-1)+(-0.304556916188)
        ref[0,0,1]=o*(0.534581283951)*x_ref[1]**(o-1)+(-0.626579480196)
        ref[0,1,0]=o*(-0.240150296068)*x_ref[0]**(o-1)+(-0.31754292824)
        ref[0,1,1]=o*(-0.0574070276326)*x_ref[1]**(o-1)+(-0.616687559393)
        ref[0,2,0]=o*(0.570345724046)*x_ref[0]**(o-1)+(0.110952065807)
        ref[0,2,1]=o*(0.997553898152)*x_ref[1]**(o-1)+(-0.503380742681)
        ref[0,3,0]=o*(0.872704383137)*x_ref[0]**(o-1)+(0.686594842527)
        ref[0,3,1]=o*(0.798635469075)*x_ref[1]**(o-1)+(0.43649341733)
        ref[0,4,0]=o*(-0.236636103417)*x_ref[0]**(o-1)+(0.0762117276629)
        ref[0,4,1]=o*(-0.774027674603)*x_ref[1]**(o-1)+(0.823651416506)
        ref[1,0,0]=o*(0.922041581692)*x_ref[0]**(o-1)+(-0.284368621571)
        ref[1,0,1]=o*(-0.71200852665)*x_ref[1]**(o-1)+(0.661348856406)
        ref[1,1,0]=o*(-0.402436007071)*x_ref[0]**(o-1)+(-0.436850834963)
        ref[1,1,1]=o*(0.62748269836)*x_ref[1]**(o-1)+(0.504691317707)
        ref[1,2,0]=o*(-0.793104721592)*x_ref[0]**(o-1)+(-0.00597773372911)
        ref[1,2,1]=o*(0.0410611356405)*x_ref[1]**(o-1)+(0.169129450548)
        ref[1,3,0]=o*(0.821521972958)*x_ref[0]**(o-1)+(-0.784664003887)
        ref[1,3,1]=o*(-0.880383627011)*x_ref[1]**(o-1)+(0.127311373905)
        ref[1,4,0]=o*(0.775700618787)*x_ref[0]**(o-1)+(-0.837475890825)
        ref[1,4,1]=o*(0.426581954939)*x_ref[1]**(o-1)+(0.404905797587)
        ref[2,0,0]=o*(-0.100956366427)*x_ref[0]**(o-1)+(0.0933662209553)
        ref[2,0,1]=o*(-0.680046458225)*x_ref[1]**(o-1)+(-0.145587503687)
        ref[2,1,0]=o*(0.166159597084)*x_ref[0]**(o-1)+(-0.900241170518)
        ref[2,1,1]=o*(-0.168407179398)*x_ref[1]**(o-1)+(-0.82352508068)
        ref[2,2,0]=o*(-0.476621892618)*x_ref[0]**(o-1)+(-0.422712025977)
        ref[2,2,1]=o*(0.298151628461)*x_ref[1]**(o-1)+(0.862589525974)
        ref[2,3,0]=o*(-0.0148783218305)*x_ref[0]**(o-1)+(0.203140203381)
        ref[2,3,1]=o*(0.19407097983)*x_ref[1]**(o-1)+(-0.772615096404)
        ref[2,4,0]=o*(0.656255800409)*x_ref[0]**(o-1)+(-0.946330057363)
        ref[2,4,1]=o*(-0.257965999122)*x_ref[1]**(o-1)+(0.71359373028)
        ref[3,0,0]=o*(-0.214762058983)*x_ref[0]**(o-1)+(0.51144571871)
        ref[3,0,1]=o*(-0.147282047629)*x_ref[1]**(o-1)+(0.709339363468)
        ref[3,1,0]=o*(-0.9112564909)*x_ref[0]**(o-1)+(-0.900437328847)
        ref[3,1,1]=o*(-0.975324195939)*x_ref[1]**(o-1)+(0.202889107529)
        ref[3,2,0]=o*(0.753787529389)*x_ref[0]**(o-1)+(0.0630834387277)
        ref[3,2,1]=o*(-0.759168264562)*x_ref[1]**(o-1)+(0.697873996051)
        ref[3,3,0]=o*(-0.0335165829084)*x_ref[0]**(o-1)+(0.274748931927)
        ref[3,3,1]=o*(0.0686458601464)*x_ref[1]**(o-1)+(-0.117608157399)
        ref[3,4,0]=o*(0.853158371839)*x_ref[0]**(o-1)+(-0.166315314677)
        ref[3,4,1]=o*(0.789241330781)*x_ref[1]**(o-1)+(-0.108867830466)
      else:
        s[0,0]=(0.110594722699)*x[0]**o+(-0.980877918241)*x[0]+(-0.515559287698)*x[1]**o+(-0.434426926239)*x[1]+(-0.0366559310163)*x[2]**o+(0.387597823386)*x[2]
        s[0,1]=(-0.661423535056)*x[0]**o+(0.361082618968)*x[0]+(-0.614579461592)*x[1]**o+(0.212089227265)*x[1]+(-0.409309067892)*x[2]**o+(-0.237106482925)*x[2]
        s[0,2]=(0.704845609334)*x[0]**o+(-0.682094657708)*x[0]+(0.0483054658314)*x[1]**o+(0.992561055244)*x[1]+(-0.676784605979)*x[2]**o+(-0.992707421428)*x[2]
        s[0,3]=(0.901583311581)*x[0]**o+(-0.751695552218)*x[0]+(-0.869549394358)*x[1]**o+(-0.889541184356)*x[1]+(-0.969435364186)*x[2]**o+(0.72152104042)*x[2]
        s[0,4]=(-0.105522466219)*x[0]**o+(0.230764909205)*x[0]+(-0.714498630414)*x[1]**o+(-0.715271005406)*x[1]+(0.0748854473277)*x[2]**o+(0.0678300049229)*x[2]
        s[1,0]=(-0.275856624456)*x[0]**o+(-0.876662969004)*x[0]+(0.681719113093)*x[1]**o+(-0.24943399785)*x[1]+(0.0837599776409)*x[2]**o+(-0.0572825122918)*x[2]
        s[1,1]=(0.359136507147)*x[0]**o+(0.663934928768)*x[0]+(0.669380381609)*x[1]**o+(0.716889420418)*x[1]+(0.0903945351303)*x[2]**o+(-0.975420985116)*x[2]
        s[1,2]=(0.284213351844)*x[0]**o+(0.634377813314)*x[0]+(-0.0935851210685)*x[1]**o+(-0.195280560003)*x[1]+(0.379637016222)*x[2]**o+(0.57302910168)*x[2]
        s[1,3]=(-0.370451421268)*x[0]**o+(-0.619498493655)*x[0]+(0.386199258196)*x[1]**o+(0.150857936022)*x[1]+(0.123336255712)*x[2]**o+(0.0144448266576)*x[2]
        s[1,4]=(-0.517413398717)*x[0]**o+(-0.952688078253)*x[0]+(-0.146422080083)*x[1]**o+(0.0871413294299)*x[1]+(0.027398040628)*x[2]**o+(0.391317414872)*x[2]
        s[2,0]=(-0.126511587518)*x[0]**o+(0.696693700404)*x[0]+(0.408515262838)*x[1]**o+(-0.251009002512)*x[1]+(-0.641227451468)*x[2]**o+(0.679579864261)*x[2]
        s[2,1]=(0.527972863795)*x[0]**o+(-0.796977004243)*x[0]+(0.596836552415)*x[1]**o+(-0.810757416401)*x[1]+(-0.282708831988)*x[2]**o+(0.127311583541)*x[2]
        s[2,2]=(-0.777216458022)*x[0]**o+(-0.676149863396)*x[0]+(0.696096143251)*x[1]**o+(-0.725078530289)*x[1]+(-0.244393979264)*x[2]**o+(-0.0585093928601)*x[2]
        s[2,3]=(0.312124516125)*x[0]**o+(0.733713522534)*x[0]+(0.706643983792)*x[1]**o+(-0.361913672386)*x[1]+(-0.0131736734016)*x[2]**o+(-0.530500138249)*x[2]
        s[2,4]=(-0.399862024703)*x[0]**o+(-0.98706730829)*x[0]+(0.90279618711)*x[1]**o+(-0.120842398107)*x[1]+(-0.621221270703)*x[2]**o+(-0.44680972674)*x[2]
        s[3,0]=(0.060963938467)*x[0]**o+(0.409545603066)*x[0]+(-0.399416656677)*x[1]**o+(-0.154729705297)*x[1]+(-0.821106571182)*x[2]**o+(0.917110134569)*x[2]
        s[3,1]=(0.395083986916)*x[0]**o+(0.957423333841)*x[0]+(-0.753576884181)*x[1]**o+(0.402825662464)*x[1]+(0.444980287919)*x[2]**o+(-0.793575717385)*x[2]
        s[3,2]=(0.519344966169)*x[0]**o+(-0.919203821647)*x[0]+(0.472250030015)*x[1]**o+(0.912498896282)*x[1]+(0.30696323239)*x[2]**o+(-0.742821847869)*x[2]
        s[3,3]=(-0.762199299359)*x[0]**o+(0.245025069538)*x[0]+(-0.576731219257)*x[1]**o+(-0.818632810068)*x[1]+(0.295773945963)*x[2]**o+(-0.0803897317676)*x[2]
        s[3,4]=(-0.339788378435)*x[0]**o+(-0.498108849772)*x[0]+(-0.779105108948)*x[1]**o+(-0.31350292532)*x[1]+(0.576582365303)*x[2]**o+(0.733406974144)*x[2]
        ref[0,0,0]=o*(0.110594722699)*x_ref[0]**(o-1)+(-0.980877918241)
        ref[0,0,1]=o*(-0.515559287698)*x_ref[1]**(o-1)+(-0.434426926239)
        ref[0,0,2]=o*(-0.0366559310163)*x_ref[2]**(o-1)+(0.387597823386)
        ref[0,1,0]=o*(-0.661423535056)*x_ref[0]**(o-1)+(0.361082618968)
        ref[0,1,1]=o*(-0.614579461592)*x_ref[1]**(o-1)+(0.212089227265)
        ref[0,1,2]=o*(-0.409309067892)*x_ref[2]**(o-1)+(-0.237106482925)
        ref[0,2,0]=o*(0.704845609334)*x_ref[0]**(o-1)+(-0.682094657708)
        ref[0,2,1]=o*(0.0483054658314)*x_ref[1]**(o-1)+(0.992561055244)
        ref[0,2,2]=o*(-0.676784605979)*x_ref[2]**(o-1)+(-0.992707421428)
        ref[0,3,0]=o*(0.901583311581)*x_ref[0]**(o-1)+(-0.751695552218)
        ref[0,3,1]=o*(-0.869549394358)*x_ref[1]**(o-1)+(-0.889541184356)
        ref[0,3,2]=o*(-0.969435364186)*x_ref[2]**(o-1)+(0.72152104042)
        ref[0,4,0]=o*(-0.105522466219)*x_ref[0]**(o-1)+(0.230764909205)
        ref[0,4,1]=o*(-0.714498630414)*x_ref[1]**(o-1)+(-0.715271005406)
        ref[0,4,2]=o*(0.0748854473277)*x_ref[2]**(o-1)+(0.0678300049229)
        ref[1,0,0]=o*(-0.275856624456)*x_ref[0]**(o-1)+(-0.876662969004)
        ref[1,0,1]=o*(0.681719113093)*x_ref[1]**(o-1)+(-0.24943399785)
        ref[1,0,2]=o*(0.0837599776409)*x_ref[2]**(o-1)+(-0.0572825122918)
        ref[1,1,0]=o*(0.359136507147)*x_ref[0]**(o-1)+(0.663934928768)
        ref[1,1,1]=o*(0.669380381609)*x_ref[1]**(o-1)+(0.716889420418)
        ref[1,1,2]=o*(0.0903945351303)*x_ref[2]**(o-1)+(-0.975420985116)
        ref[1,2,0]=o*(0.284213351844)*x_ref[0]**(o-1)+(0.634377813314)
        ref[1,2,1]=o*(-0.0935851210685)*x_ref[1]**(o-1)+(-0.195280560003)
        ref[1,2,2]=o*(0.379637016222)*x_ref[2]**(o-1)+(0.57302910168)
        ref[1,3,0]=o*(-0.370451421268)*x_ref[0]**(o-1)+(-0.619498493655)
        ref[1,3,1]=o*(0.386199258196)*x_ref[1]**(o-1)+(0.150857936022)
        ref[1,3,2]=o*(0.123336255712)*x_ref[2]**(o-1)+(0.0144448266576)
        ref[1,4,0]=o*(-0.517413398717)*x_ref[0]**(o-1)+(-0.952688078253)
        ref[1,4,1]=o*(-0.146422080083)*x_ref[1]**(o-1)+(0.0871413294299)
        ref[1,4,2]=o*(0.027398040628)*x_ref[2]**(o-1)+(0.391317414872)
        ref[2,0,0]=o*(-0.126511587518)*x_ref[0]**(o-1)+(0.696693700404)
        ref[2,0,1]=o*(0.408515262838)*x_ref[1]**(o-1)+(-0.251009002512)
        ref[2,0,2]=o*(-0.641227451468)*x_ref[2]**(o-1)+(0.679579864261)
        ref[2,1,0]=o*(0.527972863795)*x_ref[0]**(o-1)+(-0.796977004243)
        ref[2,1,1]=o*(0.596836552415)*x_ref[1]**(o-1)+(-0.810757416401)
        ref[2,1,2]=o*(-0.282708831988)*x_ref[2]**(o-1)+(0.127311583541)
        ref[2,2,0]=o*(-0.777216458022)*x_ref[0]**(o-1)+(-0.676149863396)
        ref[2,2,1]=o*(0.696096143251)*x_ref[1]**(o-1)+(-0.725078530289)
        ref[2,2,2]=o*(-0.244393979264)*x_ref[2]**(o-1)+(-0.0585093928601)
        ref[2,3,0]=o*(0.312124516125)*x_ref[0]**(o-1)+(0.733713522534)
        ref[2,3,1]=o*(0.706643983792)*x_ref[1]**(o-1)+(-0.361913672386)
        ref[2,3,2]=o*(-0.0131736734016)*x_ref[2]**(o-1)+(-0.530500138249)
        ref[2,4,0]=o*(-0.399862024703)*x_ref[0]**(o-1)+(-0.98706730829)
        ref[2,4,1]=o*(0.90279618711)*x_ref[1]**(o-1)+(-0.120842398107)
        ref[2,4,2]=o*(-0.621221270703)*x_ref[2]**(o-1)+(-0.44680972674)
        ref[3,0,0]=o*(0.060963938467)*x_ref[0]**(o-1)+(0.409545603066)
        ref[3,0,1]=o*(-0.399416656677)*x_ref[1]**(o-1)+(-0.154729705297)
        ref[3,0,2]=o*(-0.821106571182)*x_ref[2]**(o-1)+(0.917110134569)
        ref[3,1,0]=o*(0.395083986916)*x_ref[0]**(o-1)+(0.957423333841)
        ref[3,1,1]=o*(-0.753576884181)*x_ref[1]**(o-1)+(0.402825662464)
        ref[3,1,2]=o*(0.444980287919)*x_ref[2]**(o-1)+(-0.793575717385)
        ref[3,2,0]=o*(0.519344966169)*x_ref[0]**(o-1)+(-0.919203821647)
        ref[3,2,1]=o*(0.472250030015)*x_ref[1]**(o-1)+(0.912498896282)
        ref[3,2,2]=o*(0.30696323239)*x_ref[2]**(o-1)+(-0.742821847869)
        ref[3,3,0]=o*(-0.762199299359)*x_ref[0]**(o-1)+(0.245025069538)
        ref[3,3,1]=o*(-0.576731219257)*x_ref[1]**(o-1)+(-0.818632810068)
        ref[3,3,2]=o*(0.295773945963)*x_ref[2]**(o-1)+(-0.0803897317676)
        ref[3,4,0]=o*(-0.339788378435)*x_ref[0]**(o-1)+(-0.498108849772)
        ref[3,4,1]=o*(-0.779105108948)*x_ref[1]**(o-1)+(-0.31350292532)
        ref[3,4,2]=o*(0.576582365303)*x_ref[2]**(o-1)+(0.733406974144)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_Solution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.259416702848)*x[0]**o+(-0.245749999215)*x[0]+(-0.832153732959)*x[1]**o+(-0.739170215675)*x[1]
        s[0,0,1]=(0.348837228856)*x[0]**o+(-0.666178442607)*x[0]+(-0.849566479561)*x[1]**o+(-0.602086600784)*x[1]
        s[0,1,0]=(-0.155423946179)*x[0]**o+(0.236260314596)*x[0]+(-0.161071268117)*x[1]**o+(-0.86073460521)*x[1]
        s[0,1,1]=(0.00159453654506)*x[0]**o+(0.854588501463)*x[0]+(-0.932772016744)*x[1]**o+(-0.241490278915)*x[1]
        s[1,0,0]=(0.954888979422)*x[0]**o+(-0.0656825754869)*x[0]+(-0.685117910338)*x[1]**o+(0.329351311251)*x[1]
        s[1,0,1]=(0.653313879148)*x[0]**o+(-0.100353950439)*x[0]+(-0.427779024915)*x[1]**o+(0.541043842928)*x[1]
        s[1,1,0]=(-0.529344191036)*x[0]**o+(0.927248256927)*x[0]+(0.637824503976)*x[1]**o+(-0.510873834148)*x[1]
        s[1,1,1]=(0.392461722459)*x[0]**o+(0.867028086856)*x[0]+(-0.0972014158405)*x[1]**o+(0.930607205251)*x[1]
        s[2,0,0]=(-0.896743306281)*x[0]**o+(-0.44962668492)*x[0]+(0.510004368477)*x[1]**o+(-0.218091766014)*x[1]
        s[2,0,1]=(0.236477487758)*x[0]**o+(0.868982620792)*x[0]+(0.596196651626)*x[1]**o+(0.362649185465)*x[1]
        s[2,1,0]=(0.959619355116)*x[0]**o+(-0.502578509326)*x[0]+(-0.903069740877)*x[1]**o+(0.389137844229)*x[1]
        s[2,1,1]=(-0.683715298632)*x[0]**o+(0.577696755527)*x[0]+(-0.306714659414)*x[1]**o+(-0.72219199497)*x[1]
        s[3,0,0]=(-0.266756580775)*x[0]**o+(-0.360900565587)*x[0]+(-0.0715666567511)*x[1]**o+(-0.576677423431)*x[1]
        s[3,0,1]=(0.234720540152)*x[0]**o+(0.174728976827)*x[0]+(-0.347025720807)*x[1]**o+(-0.42072768341)*x[1]
        s[3,1,0]=(-0.470982762526)*x[0]**o+(-0.177781625128)*x[0]+(-0.0706858877364)*x[1]**o+(-0.722096785258)*x[1]
        s[3,1,1]=(0.598814426535)*x[0]**o+(0.767055301394)*x[0]+(0.229632024426)*x[1]**o+(0.944250372458)*x[1]
        s[4,0,0]=(0.451029698166)*x[0]**o+(-0.0341544883586)*x[0]+(0.785553686295)*x[1]**o+(0.455034664029)*x[1]
        s[4,0,1]=(-0.477486417869)*x[0]**o+(0.509646735364)*x[0]+(0.857098988263)*x[1]**o+(-0.330863877708)*x[1]
        s[4,1,0]=(-0.582320009575)*x[0]**o+(0.469724725179)*x[0]+(-0.482992293774)*x[1]**o+(-0.306261026308)*x[1]
        s[4,1,1]=(0.635452311265)*x[0]**o+(-0.795606745931)*x[0]+(-0.659604106287)*x[1]**o+(-0.0576280668882)*x[1]
        s[5,0,0]=(-0.621743078369)*x[0]**o+(0.889286951305)*x[0]+(-0.487280887249)*x[1]**o+(-0.377618071831)*x[1]
        s[5,0,1]=(0.558391219866)*x[0]**o+(-0.265622963303)*x[0]+(-0.0801965105564)*x[1]**o+(0.800983693747)*x[1]
        s[5,1,0]=(-0.291431222644)*x[0]**o+(-0.259327626357)*x[0]+(0.597600388624)*x[1]**o+(-0.890810042322)*x[1]
        s[5,1,1]=(-0.391398128176)*x[0]**o+(0.994837784671)*x[0]+(-0.467189874705)*x[1]**o+(0.626474390794)*x[1]
        ref[0,0,0,0]=o*(-0.259416702848)*x_ref[0]**(o-1)+(-0.245749999215)
        ref[0,0,0,1]=o*(-0.832153732959)*x_ref[1]**(o-1)+(-0.739170215675)
        ref[0,0,1,0]=o*(0.348837228856)*x_ref[0]**(o-1)+(-0.666178442607)
        ref[0,0,1,1]=o*(-0.849566479561)*x_ref[1]**(o-1)+(-0.602086600784)
        ref[0,1,0,0]=o*(-0.155423946179)*x_ref[0]**(o-1)+(0.236260314596)
        ref[0,1,0,1]=o*(-0.161071268117)*x_ref[1]**(o-1)+(-0.86073460521)
        ref[0,1,1,0]=o*(0.00159453654506)*x_ref[0]**(o-1)+(0.854588501463)
        ref[0,1,1,1]=o*(-0.932772016744)*x_ref[1]**(o-1)+(-0.241490278915)
        ref[1,0,0,0]=o*(0.954888979422)*x_ref[0]**(o-1)+(-0.0656825754869)
        ref[1,0,0,1]=o*(-0.685117910338)*x_ref[1]**(o-1)+(0.329351311251)
        ref[1,0,1,0]=o*(0.653313879148)*x_ref[0]**(o-1)+(-0.100353950439)
        ref[1,0,1,1]=o*(-0.427779024915)*x_ref[1]**(o-1)+(0.541043842928)
        ref[1,1,0,0]=o*(-0.529344191036)*x_ref[0]**(o-1)+(0.927248256927)
        ref[1,1,0,1]=o*(0.637824503976)*x_ref[1]**(o-1)+(-0.510873834148)
        ref[1,1,1,0]=o*(0.392461722459)*x_ref[0]**(o-1)+(0.867028086856)
        ref[1,1,1,1]=o*(-0.0972014158405)*x_ref[1]**(o-1)+(0.930607205251)
        ref[2,0,0,0]=o*(-0.896743306281)*x_ref[0]**(o-1)+(-0.44962668492)
        ref[2,0,0,1]=o*(0.510004368477)*x_ref[1]**(o-1)+(-0.218091766014)
        ref[2,0,1,0]=o*(0.236477487758)*x_ref[0]**(o-1)+(0.868982620792)
        ref[2,0,1,1]=o*(0.596196651626)*x_ref[1]**(o-1)+(0.362649185465)
        ref[2,1,0,0]=o*(0.959619355116)*x_ref[0]**(o-1)+(-0.502578509326)
        ref[2,1,0,1]=o*(-0.903069740877)*x_ref[1]**(o-1)+(0.389137844229)
        ref[2,1,1,0]=o*(-0.683715298632)*x_ref[0]**(o-1)+(0.577696755527)
        ref[2,1,1,1]=o*(-0.306714659414)*x_ref[1]**(o-1)+(-0.72219199497)
        ref[3,0,0,0]=o*(-0.266756580775)*x_ref[0]**(o-1)+(-0.360900565587)
        ref[3,0,0,1]=o*(-0.0715666567511)*x_ref[1]**(o-1)+(-0.576677423431)
        ref[3,0,1,0]=o*(0.234720540152)*x_ref[0]**(o-1)+(0.174728976827)
        ref[3,0,1,1]=o*(-0.347025720807)*x_ref[1]**(o-1)+(-0.42072768341)
        ref[3,1,0,0]=o*(-0.470982762526)*x_ref[0]**(o-1)+(-0.177781625128)
        ref[3,1,0,1]=o*(-0.0706858877364)*x_ref[1]**(o-1)+(-0.722096785258)
        ref[3,1,1,0]=o*(0.598814426535)*x_ref[0]**(o-1)+(0.767055301394)
        ref[3,1,1,1]=o*(0.229632024426)*x_ref[1]**(o-1)+(0.944250372458)
        ref[4,0,0,0]=o*(0.451029698166)*x_ref[0]**(o-1)+(-0.0341544883586)
        ref[4,0,0,1]=o*(0.785553686295)*x_ref[1]**(o-1)+(0.455034664029)
        ref[4,0,1,0]=o*(-0.477486417869)*x_ref[0]**(o-1)+(0.509646735364)
        ref[4,0,1,1]=o*(0.857098988263)*x_ref[1]**(o-1)+(-0.330863877708)
        ref[4,1,0,0]=o*(-0.582320009575)*x_ref[0]**(o-1)+(0.469724725179)
        ref[4,1,0,1]=o*(-0.482992293774)*x_ref[1]**(o-1)+(-0.306261026308)
        ref[4,1,1,0]=o*(0.635452311265)*x_ref[0]**(o-1)+(-0.795606745931)
        ref[4,1,1,1]=o*(-0.659604106287)*x_ref[1]**(o-1)+(-0.0576280668882)
        ref[5,0,0,0]=o*(-0.621743078369)*x_ref[0]**(o-1)+(0.889286951305)
        ref[5,0,0,1]=o*(-0.487280887249)*x_ref[1]**(o-1)+(-0.377618071831)
        ref[5,0,1,0]=o*(0.558391219866)*x_ref[0]**(o-1)+(-0.265622963303)
        ref[5,0,1,1]=o*(-0.0801965105564)*x_ref[1]**(o-1)+(0.800983693747)
        ref[5,1,0,0]=o*(-0.291431222644)*x_ref[0]**(o-1)+(-0.259327626357)
        ref[5,1,0,1]=o*(0.597600388624)*x_ref[1]**(o-1)+(-0.890810042322)
        ref[5,1,1,0]=o*(-0.391398128176)*x_ref[0]**(o-1)+(0.994837784671)
        ref[5,1,1,1]=o*(-0.467189874705)*x_ref[1]**(o-1)+(0.626474390794)
      else:
        s[0,0,0]=(-0.283421054619)*x[0]**o+(-0.389803426219)*x[0]+(-0.641735725139)*x[1]**o+(-0.184182767783)*x[1]+(0.559369811956)*x[2]**o+(0.4130246739)*x[2]
        s[0,0,1]=(-0.685739480994)*x[0]**o+(-0.412746444255)*x[0]+(-0.977924434029)*x[1]**o+(0.866683883992)*x[1]+(-0.104721288552)*x[2]**o+(-0.854137553522)*x[2]
        s[0,1,0]=(0.326677121892)*x[0]**o+(0.0346511145836)*x[0]+(-0.583479722363)*x[1]**o+(0.552232008446)*x[1]+(-0.66426015414)*x[2]**o+(-0.0282350207962)*x[2]
        s[0,1,1]=(-0.743104988438)*x[0]**o+(-0.250100474341)*x[0]+(-0.0100118927398)*x[1]**o+(0.421549551364)*x[1]+(0.774348452339)*x[2]**o+(0.0114324989752)*x[2]
        s[1,0,0]=(-0.969248079823)*x[0]**o+(0.554289530783)*x[0]+(-0.437222589525)*x[1]**o+(-0.454550411233)*x[1]+(-0.555157485811)*x[2]**o+(0.360201196386)*x[2]
        s[1,0,1]=(0.428483060047)*x[0]**o+(-0.305500955239)*x[0]+(-0.918692317879)*x[1]**o+(-0.0749149504003)*x[1]+(-0.146230161925)*x[2]**o+(-0.328660463631)*x[2]
        s[1,1,0]=(-0.534394777023)*x[0]**o+(-0.0160564112453)*x[0]+(-0.446201680182)*x[1]**o+(-0.457243622333)*x[1]+(0.316031457633)*x[2]**o+(-0.0867540461561)*x[2]
        s[1,1,1]=(-0.180636289042)*x[0]**o+(0.409910302887)*x[0]+(-0.261132577936)*x[1]**o+(0.976758759795)*x[1]+(-0.298504887785)*x[2]**o+(0.0720200683864)*x[2]
        s[2,0,0]=(-0.965162583973)*x[0]**o+(-0.885802239228)*x[0]+(-0.166151347797)*x[1]**o+(0.968122595961)*x[1]+(0.580364754552)*x[2]**o+(0.213039816836)*x[2]
        s[2,0,1]=(0.7871160079)*x[0]**o+(-0.235526954906)*x[0]+(0.496553426537)*x[1]**o+(-0.0239289077236)*x[1]+(-0.633193916071)*x[2]**o+(-0.777984393332)*x[2]
        s[2,1,0]=(-0.79465182095)*x[0]**o+(-0.0880694356168)*x[0]+(-0.686736039005)*x[1]**o+(-0.221174710075)*x[1]+(0.0208981523074)*x[2]**o+(-0.961469223363)*x[2]
        s[2,1,1]=(-0.455839585127)*x[0]**o+(0.941828964284)*x[0]+(0.998331788392)*x[1]**o+(0.0750455958592)*x[1]+(0.608504253916)*x[2]**o+(-0.374386910681)*x[2]
        s[3,0,0]=(-0.909949427027)*x[0]**o+(-0.396086474642)*x[0]+(0.899299670063)*x[1]**o+(0.141272919633)*x[1]+(0.53792064575)*x[2]**o+(0.595764269338)*x[2]
        s[3,0,1]=(-0.49521850368)*x[0]**o+(0.207120861953)*x[0]+(0.569903370263)*x[1]**o+(0.588491719962)*x[1]+(0.452965587121)*x[2]**o+(-0.628229661128)*x[2]
        s[3,1,0]=(-0.875095641635)*x[0]**o+(0.11799580621)*x[0]+(0.109152019439)*x[1]**o+(-0.941762743205)*x[1]+(-0.124413788962)*x[2]**o+(-0.733044880598)*x[2]
        s[3,1,1]=(0.691739414808)*x[0]**o+(-0.0580142383494)*x[0]+(-0.625563645068)*x[1]**o+(-0.459983356734)*x[1]+(0.743764207387)*x[2]**o+(0.200950835048)*x[2]
        s[4,0,0]=(-0.686339571296)*x[0]**o+(0.310431821345)*x[0]+(0.0697719589995)*x[1]**o+(0.413649564597)*x[1]+(0.311083457053)*x[2]**o+(-0.764323862328)*x[2]
        s[4,0,1]=(0.82709146654)*x[0]**o+(-0.958833756078)*x[0]+(-0.293388295126)*x[1]**o+(0.789172794162)*x[1]+(-0.47291926184)*x[2]**o+(0.447003729661)*x[2]
        s[4,1,0]=(-0.968741249623)*x[0]**o+(-0.977865313489)*x[0]+(0.777352281599)*x[1]**o+(-0.0177223674634)*x[1]+(-0.437430561766)*x[2]**o+(0.403327933356)*x[2]
        s[4,1,1]=(0.873657912127)*x[0]**o+(-0.432188278068)*x[0]+(-0.294192343439)*x[1]**o+(-0.199868940744)*x[1]+(0.379614508084)*x[2]**o+(0.610834295156)*x[2]
        s[5,0,0]=(-0.452716736875)*x[0]**o+(0.298248314499)*x[0]+(0.850779531393)*x[1]**o+(-0.024084377803)*x[1]+(-0.106948635789)*x[2]**o+(0.926343375096)*x[2]
        s[5,0,1]=(-0.22629592934)*x[0]**o+(-0.481026079587)*x[0]+(0.713198238543)*x[1]**o+(-0.290540996335)*x[1]+(-0.775028431788)*x[2]**o+(0.839104261372)*x[2]
        s[5,1,0]=(0.938919479885)*x[0]**o+(-0.833254411673)*x[0]+(-0.0920737395198)*x[1]**o+(-0.863263173274)*x[1]+(-0.326022820082)*x[2]**o+(-0.162067090943)*x[2]
        s[5,1,1]=(0.800986633012)*x[0]**o+(-0.102366666332)*x[0]+(-0.35704808806)*x[1]**o+(-0.710763760013)*x[1]+(0.249411990506)*x[2]**o+(0.569232273165)*x[2]
        ref[0,0,0,0]=o*(-0.283421054619)*x_ref[0]**(o-1)+(-0.389803426219)
        ref[0,0,0,1]=o*(-0.641735725139)*x_ref[1]**(o-1)+(-0.184182767783)
        ref[0,0,0,2]=o*(0.559369811956)*x_ref[2]**(o-1)+(0.4130246739)
        ref[0,0,1,0]=o*(-0.685739480994)*x_ref[0]**(o-1)+(-0.412746444255)
        ref[0,0,1,1]=o*(-0.977924434029)*x_ref[1]**(o-1)+(0.866683883992)
        ref[0,0,1,2]=o*(-0.104721288552)*x_ref[2]**(o-1)+(-0.854137553522)
        ref[0,1,0,0]=o*(0.326677121892)*x_ref[0]**(o-1)+(0.0346511145836)
        ref[0,1,0,1]=o*(-0.583479722363)*x_ref[1]**(o-1)+(0.552232008446)
        ref[0,1,0,2]=o*(-0.66426015414)*x_ref[2]**(o-1)+(-0.0282350207962)
        ref[0,1,1,0]=o*(-0.743104988438)*x_ref[0]**(o-1)+(-0.250100474341)
        ref[0,1,1,1]=o*(-0.0100118927398)*x_ref[1]**(o-1)+(0.421549551364)
        ref[0,1,1,2]=o*(0.774348452339)*x_ref[2]**(o-1)+(0.0114324989752)
        ref[1,0,0,0]=o*(-0.969248079823)*x_ref[0]**(o-1)+(0.554289530783)
        ref[1,0,0,1]=o*(-0.437222589525)*x_ref[1]**(o-1)+(-0.454550411233)
        ref[1,0,0,2]=o*(-0.555157485811)*x_ref[2]**(o-1)+(0.360201196386)
        ref[1,0,1,0]=o*(0.428483060047)*x_ref[0]**(o-1)+(-0.305500955239)
        ref[1,0,1,1]=o*(-0.918692317879)*x_ref[1]**(o-1)+(-0.0749149504003)
        ref[1,0,1,2]=o*(-0.146230161925)*x_ref[2]**(o-1)+(-0.328660463631)
        ref[1,1,0,0]=o*(-0.534394777023)*x_ref[0]**(o-1)+(-0.0160564112453)
        ref[1,1,0,1]=o*(-0.446201680182)*x_ref[1]**(o-1)+(-0.457243622333)
        ref[1,1,0,2]=o*(0.316031457633)*x_ref[2]**(o-1)+(-0.0867540461561)
        ref[1,1,1,0]=o*(-0.180636289042)*x_ref[0]**(o-1)+(0.409910302887)
        ref[1,1,1,1]=o*(-0.261132577936)*x_ref[1]**(o-1)+(0.976758759795)
        ref[1,1,1,2]=o*(-0.298504887785)*x_ref[2]**(o-1)+(0.0720200683864)
        ref[2,0,0,0]=o*(-0.965162583973)*x_ref[0]**(o-1)+(-0.885802239228)
        ref[2,0,0,1]=o*(-0.166151347797)*x_ref[1]**(o-1)+(0.968122595961)
        ref[2,0,0,2]=o*(0.580364754552)*x_ref[2]**(o-1)+(0.213039816836)
        ref[2,0,1,0]=o*(0.7871160079)*x_ref[0]**(o-1)+(-0.235526954906)
        ref[2,0,1,1]=o*(0.496553426537)*x_ref[1]**(o-1)+(-0.0239289077236)
        ref[2,0,1,2]=o*(-0.633193916071)*x_ref[2]**(o-1)+(-0.777984393332)
        ref[2,1,0,0]=o*(-0.79465182095)*x_ref[0]**(o-1)+(-0.0880694356168)
        ref[2,1,0,1]=o*(-0.686736039005)*x_ref[1]**(o-1)+(-0.221174710075)
        ref[2,1,0,2]=o*(0.0208981523074)*x_ref[2]**(o-1)+(-0.961469223363)
        ref[2,1,1,0]=o*(-0.455839585127)*x_ref[0]**(o-1)+(0.941828964284)
        ref[2,1,1,1]=o*(0.998331788392)*x_ref[1]**(o-1)+(0.0750455958592)
        ref[2,1,1,2]=o*(0.608504253916)*x_ref[2]**(o-1)+(-0.374386910681)
        ref[3,0,0,0]=o*(-0.909949427027)*x_ref[0]**(o-1)+(-0.396086474642)
        ref[3,0,0,1]=o*(0.899299670063)*x_ref[1]**(o-1)+(0.141272919633)
        ref[3,0,0,2]=o*(0.53792064575)*x_ref[2]**(o-1)+(0.595764269338)
        ref[3,0,1,0]=o*(-0.49521850368)*x_ref[0]**(o-1)+(0.207120861953)
        ref[3,0,1,1]=o*(0.569903370263)*x_ref[1]**(o-1)+(0.588491719962)
        ref[3,0,1,2]=o*(0.452965587121)*x_ref[2]**(o-1)+(-0.628229661128)
        ref[3,1,0,0]=o*(-0.875095641635)*x_ref[0]**(o-1)+(0.11799580621)
        ref[3,1,0,1]=o*(0.109152019439)*x_ref[1]**(o-1)+(-0.941762743205)
        ref[3,1,0,2]=o*(-0.124413788962)*x_ref[2]**(o-1)+(-0.733044880598)
        ref[3,1,1,0]=o*(0.691739414808)*x_ref[0]**(o-1)+(-0.0580142383494)
        ref[3,1,1,1]=o*(-0.625563645068)*x_ref[1]**(o-1)+(-0.459983356734)
        ref[3,1,1,2]=o*(0.743764207387)*x_ref[2]**(o-1)+(0.200950835048)
        ref[4,0,0,0]=o*(-0.686339571296)*x_ref[0]**(o-1)+(0.310431821345)
        ref[4,0,0,1]=o*(0.0697719589995)*x_ref[1]**(o-1)+(0.413649564597)
        ref[4,0,0,2]=o*(0.311083457053)*x_ref[2]**(o-1)+(-0.764323862328)
        ref[4,0,1,0]=o*(0.82709146654)*x_ref[0]**(o-1)+(-0.958833756078)
        ref[4,0,1,1]=o*(-0.293388295126)*x_ref[1]**(o-1)+(0.789172794162)
        ref[4,0,1,2]=o*(-0.47291926184)*x_ref[2]**(o-1)+(0.447003729661)
        ref[4,1,0,0]=o*(-0.968741249623)*x_ref[0]**(o-1)+(-0.977865313489)
        ref[4,1,0,1]=o*(0.777352281599)*x_ref[1]**(o-1)+(-0.0177223674634)
        ref[4,1,0,2]=o*(-0.437430561766)*x_ref[2]**(o-1)+(0.403327933356)
        ref[4,1,1,0]=o*(0.873657912127)*x_ref[0]**(o-1)+(-0.432188278068)
        ref[4,1,1,1]=o*(-0.294192343439)*x_ref[1]**(o-1)+(-0.199868940744)
        ref[4,1,1,2]=o*(0.379614508084)*x_ref[2]**(o-1)+(0.610834295156)
        ref[5,0,0,0]=o*(-0.452716736875)*x_ref[0]**(o-1)+(0.298248314499)
        ref[5,0,0,1]=o*(0.850779531393)*x_ref[1]**(o-1)+(-0.024084377803)
        ref[5,0,0,2]=o*(-0.106948635789)*x_ref[2]**(o-1)+(0.926343375096)
        ref[5,0,1,0]=o*(-0.22629592934)*x_ref[0]**(o-1)+(-0.481026079587)
        ref[5,0,1,1]=o*(0.713198238543)*x_ref[1]**(o-1)+(-0.290540996335)
        ref[5,0,1,2]=o*(-0.775028431788)*x_ref[2]**(o-1)+(0.839104261372)
        ref[5,1,0,0]=o*(0.938919479885)*x_ref[0]**(o-1)+(-0.833254411673)
        ref[5,1,0,1]=o*(-0.0920737395198)*x_ref[1]**(o-1)+(-0.863263173274)
        ref[5,1,0,2]=o*(-0.326022820082)*x_ref[2]**(o-1)+(-0.162067090943)
        ref[5,1,1,0]=o*(0.800986633012)*x_ref[0]**(o-1)+(-0.102366666332)
        ref[5,1,1,1]=o*(-0.35704808806)*x_ref[1]**(o-1)+(-0.710763760013)
        ref[5,1,1,2]=o*(0.249411990506)*x_ref[2]**(o-1)+(0.569232273165)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.049981434374)*x[0]+(1.21186502577)*x[1]
        ref[0]=(-0.049981434374)
        ref[1]=(1.21186502577)
      else:
        s=(-0.0752432283484)*x[0]+(0.11438092577)*x[1]+(-0.434888190035)*x[2]
        ref[0]=(-0.0752432283484)
        ref[1]=(0.11438092577)
        ref[2]=(-0.434888190035)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.456351387646)*x[0]+(-0.116715426711)*x[1]
        s[1]=(-0.854853163245)*x[0]+(-0.00664827097294)*x[1]
        ref[0,0]=(-0.456351387646)
        ref[0,1]=(-0.116715426711)
        ref[1,0]=(-0.854853163245)
        ref[1,1]=(-0.00664827097294)
      else:
        s[0]=(0.623100837308)*x[0]+(0.266907813705)*x[1]+(1.38575183924)*x[2]
        s[1]=(-1.48702785688)*x[0]+(-1.42426212001)*x[1]+(0.317785468787)*x[2]
        ref[0,0]=(0.623100837308)
        ref[0,1]=(0.266907813705)
        ref[0,2]=(1.38575183924)
        ref[1,0]=(-1.48702785688)
        ref[1,1]=(-1.42426212001)
        ref[1,2]=(0.317785468787)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-1.0789465623)*x[0]+(-0.347959175817)*x[1]
        s[0,1]=(-0.551011467115)*x[0]+(1.35264465045)*x[1]
        s[0,2]=(-0.570905806915)*x[0]+(0.0584374344724)*x[1]
        s[0,3]=(0.596777443685)*x[0]+(0.414694389615)*x[1]
        s[0,4]=(0.611144733018)*x[0]+(-0.758856666701)*x[1]
        s[1,0]=(-0.0733659967272)*x[0]+(1.27900296295)*x[1]
        s[1,1]=(-0.305565660896)*x[0]+(0.9434988738)*x[1]
        s[1,2]=(-0.992776824928)*x[0]+(-0.120817150952)*x[1]
        s[1,3]=(0.0902201920474)*x[0]+(1.04097295319)*x[1]
        s[1,4]=(-1.06448209062)*x[0]+(1.13868467732)*x[1]
        s[2,0]=(0.855203060995)*x[0]+(0.707160314719)*x[1]
        s[2,1]=(0.792799717854)*x[0]+(-0.36484734497)*x[1]
        s[2,2]=(-1.14542452528)*x[0]+(0.98847021504)*x[1]
        s[2,3]=(0.145237440613)*x[0]+(0.214714794234)*x[1]
        s[2,4]=(0.179556206154)*x[0]+(-1.21107721861)*x[1]
        s[3,0]=(0.700250733001)*x[0]+(1.25331090966)*x[1]
        s[3,1]=(0.258700850981)*x[0]+(-0.637474578328)*x[1]
        s[3,2]=(0.382709435533)*x[0]+(1.29575763411)*x[1]
        s[3,3]=(-0.254008873764)*x[0]+(-1.54340970006)*x[1]
        s[3,4]=(-0.658082774987)*x[0]+(0.647639696672)*x[1]
        ref[0,0,0]=(-1.0789465623)
        ref[0,0,1]=(-0.347959175817)
        ref[0,1,0]=(-0.551011467115)
        ref[0,1,1]=(1.35264465045)
        ref[0,2,0]=(-0.570905806915)
        ref[0,2,1]=(0.0584374344724)
        ref[0,3,0]=(0.596777443685)
        ref[0,3,1]=(0.414694389615)
        ref[0,4,0]=(0.611144733018)
        ref[0,4,1]=(-0.758856666701)
        ref[1,0,0]=(-0.0733659967272)
        ref[1,0,1]=(1.27900296295)
        ref[1,1,0]=(-0.305565660896)
        ref[1,1,1]=(0.9434988738)
        ref[1,2,0]=(-0.992776824928)
        ref[1,2,1]=(-0.120817150952)
        ref[1,3,0]=(0.0902201920474)
        ref[1,3,1]=(1.04097295319)
        ref[1,4,0]=(-1.06448209062)
        ref[1,4,1]=(1.13868467732)
        ref[2,0,0]=(0.855203060995)
        ref[2,0,1]=(0.707160314719)
        ref[2,1,0]=(0.792799717854)
        ref[2,1,1]=(-0.36484734497)
        ref[2,2,0]=(-1.14542452528)
        ref[2,2,1]=(0.98847021504)
        ref[2,3,0]=(0.145237440613)
        ref[2,3,1]=(0.214714794234)
        ref[2,4,0]=(0.179556206154)
        ref[2,4,1]=(-1.21107721861)
        ref[3,0,0]=(0.700250733001)
        ref[3,0,1]=(1.25331090966)
        ref[3,1,0]=(0.258700850981)
        ref[3,1,1]=(-0.637474578328)
        ref[3,2,0]=(0.382709435533)
        ref[3,2,1]=(1.29575763411)
        ref[3,3,0]=(-0.254008873764)
        ref[3,3,1]=(-1.54340970006)
        ref[3,4,0]=(-0.658082774987)
        ref[3,4,1]=(0.647639696672)
      else:
        s[0,0]=(1.12878816262)*x[0]+(-1.74114589021)*x[1]+(0.248765786072)*x[2]
        s[0,1]=(1.72583176211)*x[0]+(-1.23114549996)*x[1]+(-0.203157732085)*x[2]
        s[0,2]=(-0.422314546839)*x[0]+(0.224260123741)*x[1]+(0.171252253031)*x[2]
        s[0,3]=(-0.870712689882)*x[0]+(-0.555793590179)*x[1]+(0.290927600809)*x[2]
        s[0,4]=(-0.920833142524)*x[0]+(-0.0497652513943)*x[1]+(-0.39718342747)*x[2]
        s[1,0]=(-0.727450597273)*x[0]+(1.09323195342)*x[1]+(0.298478430357)*x[2]
        s[1,1]=(0.12608256775)*x[0]+(-0.459788156475)*x[1]+(1.13249234657)*x[2]
        s[1,2]=(0.898608507221)*x[0]+(-0.46334527476)*x[1]+(-0.445291041098)*x[2]
        s[1,3]=(-0.327616894595)*x[0]+(-0.288577792837)*x[1]+(1.26565355504)*x[2]
        s[1,4]=(0.0630186390647)*x[0]+(-0.383734001362)*x[1]+(-0.116975162517)*x[2]
        s[2,0]=(-0.309394005751)*x[0]+(-0.12474002694)*x[1]+(0.699555331484)*x[2]
        s[2,1]=(0.997752898742)*x[0]+(-0.183982572346)*x[1]+(1.22982190192)*x[2]
        s[2,2]=(1.03496586697)*x[0]+(-0.097256212804)*x[1]+(1.42052511301)*x[2]
        s[2,3]=(0.614378729663)*x[0]+(0.154397435133)*x[1]+(-0.863004330742)*x[2]
        s[2,4]=(1.20904264903)*x[0]+(-0.452388361293)*x[1]+(-1.13286356474)*x[2]
        s[3,0]=(-0.812538381038)*x[0]+(0.00308372111735)*x[1]+(-0.117937095444)*x[2]
        s[3,1]=(0.45479320042)*x[0]+(-0.45927716926)*x[1]+(0.4787228586)*x[2]
        s[3,2]=(-0.487685328452)*x[0]+(0.871241301367)*x[1]+(0.78769505788)*x[2]
        s[3,3]=(0.860788048428)*x[0]+(1.39676274038)*x[1]+(0.407326483464)*x[2]
        s[3,4]=(0.00409247820271)*x[0]+(0.44466332801)*x[1]+(-0.66721280275)*x[2]
        ref[0,0,0]=(1.12878816262)
        ref[0,0,1]=(-1.74114589021)
        ref[0,0,2]=(0.248765786072)
        ref[0,1,0]=(1.72583176211)
        ref[0,1,1]=(-1.23114549996)
        ref[0,1,2]=(-0.203157732085)
        ref[0,2,0]=(-0.422314546839)
        ref[0,2,1]=(0.224260123741)
        ref[0,2,2]=(0.171252253031)
        ref[0,3,0]=(-0.870712689882)
        ref[0,3,1]=(-0.555793590179)
        ref[0,3,2]=(0.290927600809)
        ref[0,4,0]=(-0.920833142524)
        ref[0,4,1]=(-0.0497652513943)
        ref[0,4,2]=(-0.39718342747)
        ref[1,0,0]=(-0.727450597273)
        ref[1,0,1]=(1.09323195342)
        ref[1,0,2]=(0.298478430357)
        ref[1,1,0]=(0.12608256775)
        ref[1,1,1]=(-0.459788156475)
        ref[1,1,2]=(1.13249234657)
        ref[1,2,0]=(0.898608507221)
        ref[1,2,1]=(-0.46334527476)
        ref[1,2,2]=(-0.445291041098)
        ref[1,3,0]=(-0.327616894595)
        ref[1,3,1]=(-0.288577792837)
        ref[1,3,2]=(1.26565355504)
        ref[1,4,0]=(0.0630186390647)
        ref[1,4,1]=(-0.383734001362)
        ref[1,4,2]=(-0.116975162517)
        ref[2,0,0]=(-0.309394005751)
        ref[2,0,1]=(-0.12474002694)
        ref[2,0,2]=(0.699555331484)
        ref[2,1,0]=(0.997752898742)
        ref[2,1,1]=(-0.183982572346)
        ref[2,1,2]=(1.22982190192)
        ref[2,2,0]=(1.03496586697)
        ref[2,2,1]=(-0.097256212804)
        ref[2,2,2]=(1.42052511301)
        ref[2,3,0]=(0.614378729663)
        ref[2,3,1]=(0.154397435133)
        ref[2,3,2]=(-0.863004330742)
        ref[2,4,0]=(1.20904264903)
        ref[2,4,1]=(-0.452388361293)
        ref[2,4,2]=(-1.13286356474)
        ref[3,0,0]=(-0.812538381038)
        ref[3,0,1]=(0.00308372111735)
        ref[3,0,2]=(-0.117937095444)
        ref[3,1,0]=(0.45479320042)
        ref[3,1,1]=(-0.45927716926)
        ref[3,1,2]=(0.4787228586)
        ref[3,2,0]=(-0.487685328452)
        ref[3,2,1]=(0.871241301367)
        ref[3,2,2]=(0.78769505788)
        ref[3,3,0]=(0.860788048428)
        ref[3,3,1]=(1.39676274038)
        ref[3,3,2]=(0.407326483464)
        ref[3,4,0]=(0.00409247820271)
        ref[3,4,1]=(0.44466332801)
        ref[3,4,2]=(-0.66721280275)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.905736881137)*x[0]+(0.694861875477)*x[1]
        s[0,0,1]=(-1.45890910436)*x[0]+(-1.52565465047)*x[1]
        s[0,1,0]=(-0.312122938001)*x[0]+(-0.868878770447)*x[1]
        s[0,1,1]=(1.54725692119)*x[0]+(0.536122055612)*x[1]
        s[1,0,0]=(0.196190980599)*x[0]+(0.328328643332)*x[1]
        s[1,0,1]=(-1.08863159071)*x[0]+(-1.33920996273)*x[1]
        s[1,1,0]=(1.73274170273)*x[0]+(-0.958793175994)*x[1]
        s[1,1,1]=(0.182836505159)*x[0]+(-0.121541234518)*x[1]
        s[2,0,0]=(0.144461277784)*x[0]+(1.83781976673)*x[1]
        s[2,0,1]=(-0.194587892421)*x[0]+(-1.27090147949)*x[1]
        s[2,1,0]=(0.379488182396)*x[0]+(0.030493966182)*x[1]
        s[2,1,1]=(1.09003510354)*x[0]+(-0.454578719663)*x[1]
        s[3,0,0]=(1.58446081096)*x[0]+(-0.52113508443)*x[1]
        s[3,0,1]=(-1.33593796269)*x[0]+(-0.165961948897)*x[1]
        s[3,1,0]=(0.377402371367)*x[0]+(-0.0184973476961)*x[1]
        s[3,1,1]=(0.764313855665)*x[0]+(0.294443818887)*x[1]
        s[4,0,0]=(-0.180639063236)*x[0]+(1.75864081648)*x[1]
        s[4,0,1]=(-1.62662138708)*x[0]+(1.81611541058)*x[1]
        s[4,1,0]=(1.37678767999)*x[0]+(-0.271115982236)*x[1]
        s[4,1,1]=(1.32861834819)*x[0]+(0.768596947238)*x[1]
        s[5,0,0]=(1.39342920577)*x[0]+(0.155666506926)*x[1]
        s[5,0,1]=(0.0397474261373)*x[0]+(0.921852245407)*x[1]
        s[5,1,0]=(-0.950337694161)*x[0]+(-0.011042822071)*x[1]
        s[5,1,1]=(0.0369412335225)*x[0]+(-0.473468894798)*x[1]
        ref[0,0,0,0]=(-0.905736881137)
        ref[0,0,0,1]=(0.694861875477)
        ref[0,0,1,0]=(-1.45890910436)
        ref[0,0,1,1]=(-1.52565465047)
        ref[0,1,0,0]=(-0.312122938001)
        ref[0,1,0,1]=(-0.868878770447)
        ref[0,1,1,0]=(1.54725692119)
        ref[0,1,1,1]=(0.536122055612)
        ref[1,0,0,0]=(0.196190980599)
        ref[1,0,0,1]=(0.328328643332)
        ref[1,0,1,0]=(-1.08863159071)
        ref[1,0,1,1]=(-1.33920996273)
        ref[1,1,0,0]=(1.73274170273)
        ref[1,1,0,1]=(-0.958793175994)
        ref[1,1,1,0]=(0.182836505159)
        ref[1,1,1,1]=(-0.121541234518)
        ref[2,0,0,0]=(0.144461277784)
        ref[2,0,0,1]=(1.83781976673)
        ref[2,0,1,0]=(-0.194587892421)
        ref[2,0,1,1]=(-1.27090147949)
        ref[2,1,0,0]=(0.379488182396)
        ref[2,1,0,1]=(0.030493966182)
        ref[2,1,1,0]=(1.09003510354)
        ref[2,1,1,1]=(-0.454578719663)
        ref[3,0,0,0]=(1.58446081096)
        ref[3,0,0,1]=(-0.52113508443)
        ref[3,0,1,0]=(-1.33593796269)
        ref[3,0,1,1]=(-0.165961948897)
        ref[3,1,0,0]=(0.377402371367)
        ref[3,1,0,1]=(-0.0184973476961)
        ref[3,1,1,0]=(0.764313855665)
        ref[3,1,1,1]=(0.294443818887)
        ref[4,0,0,0]=(-0.180639063236)
        ref[4,0,0,1]=(1.75864081648)
        ref[4,0,1,0]=(-1.62662138708)
        ref[4,0,1,1]=(1.81611541058)
        ref[4,1,0,0]=(1.37678767999)
        ref[4,1,0,1]=(-0.271115982236)
        ref[4,1,1,0]=(1.32861834819)
        ref[4,1,1,1]=(0.768596947238)
        ref[5,0,0,0]=(1.39342920577)
        ref[5,0,0,1]=(0.155666506926)
        ref[5,0,1,0]=(0.0397474261373)
        ref[5,0,1,1]=(0.921852245407)
        ref[5,1,0,0]=(-0.950337694161)
        ref[5,1,0,1]=(-0.011042822071)
        ref[5,1,1,0]=(0.0369412335225)
        ref[5,1,1,1]=(-0.473468894798)
      else:
        s[0,0,0]=(0.0887916493027)*x[0]+(-0.648868378605)*x[1]+(-0.0728819090295)*x[2]
        s[0,0,1]=(-1.41771256969)*x[0]+(-1.5907790724)*x[1]+(-0.208960186234)*x[2]
        s[0,1,0]=(-0.846039306108)*x[0]+(0.451942850498)*x[1]+(-0.305613220999)*x[2]
        s[0,1,1]=(-0.990248401344)*x[0]+(0.934658804462)*x[1]+(0.703927399604)*x[2]
        s[1,0,0]=(0.0172966310708)*x[0]+(-0.097716216369)*x[1]+(0.564328624475)*x[2]
        s[1,0,1]=(-0.285571273779)*x[0]+(-0.478508810269)*x[1]+(0.713694990763)*x[2]
        s[1,1,0]=(-1.72203517221)*x[0]+(0.90209571161)*x[1]+(0.0404190111547)*x[2]
        s[1,1,1]=(0.662367771483)*x[0]+(0.683618857708)*x[1]+(0.220237165119)*x[2]
        s[2,0,0]=(-1.23109104652)*x[0]+(-0.344954703402)*x[1]+(-0.845700366118)*x[2]
        s[2,0,1]=(-1.78196237278)*x[0]+(0.352288083535)*x[1]+(1.02993741657)*x[2]
        s[2,1,0]=(-0.101867429012)*x[0]+(-0.810130865002)*x[1]+(0.682007840621)*x[2]
        s[2,1,1]=(0.424977434864)*x[0]+(-0.193692196905)*x[1]+(1.40079084873)*x[2]
        s[3,0,0]=(0.471393445911)*x[0]+(-0.56961804428)*x[1]+(1.1096657312)*x[2]
        s[3,0,1]=(1.09919079441)*x[0]+(1.82571072835)*x[1]+(0.604299398913)*x[2]
        s[3,1,0]=(0.155938760923)*x[0]+(0.841141932215)*x[1]+(0.558240114855)*x[2]
        s[3,1,1]=(-0.0697992380617)*x[0]+(1.28687270053)*x[1]+(-1.65036825798)*x[2]
        s[4,0,0]=(-0.497902210966)*x[0]+(0.495675372467)*x[1]+(1.4729295959)*x[2]
        s[4,0,1]=(-0.278468085457)*x[0]+(0.515340965551)*x[1]+(-0.233864738508)*x[2]
        s[4,1,0]=(0.574455672589)*x[0]+(-1.41224801693)*x[1]+(-0.0851508668768)*x[2]
        s[4,1,1]=(-0.392015012986)*x[0]+(0.211411943089)*x[1]+(0.175695876916)*x[2]
        s[5,0,0]=(-1.3317854233)*x[0]+(1.30129857885)*x[1]+(-0.96126616548)*x[2]
        s[5,0,1]=(-1.00763984246)*x[0]+(0.252485810112)*x[1]+(0.0349533247409)*x[2]
        s[5,1,0]=(1.23304668416)*x[0]+(0.978792435109)*x[1]+(0.0784889665787)*x[2]
        s[5,1,1]=(-0.164813552675)*x[0]+(0.363900322694)*x[1]+(-0.388851362667)*x[2]
        ref[0,0,0,0]=(0.0887916493027)
        ref[0,0,0,1]=(-0.648868378605)
        ref[0,0,0,2]=(-0.0728819090295)
        ref[0,0,1,0]=(-1.41771256969)
        ref[0,0,1,1]=(-1.5907790724)
        ref[0,0,1,2]=(-0.208960186234)
        ref[0,1,0,0]=(-0.846039306108)
        ref[0,1,0,1]=(0.451942850498)
        ref[0,1,0,2]=(-0.305613220999)
        ref[0,1,1,0]=(-0.990248401344)
        ref[0,1,1,1]=(0.934658804462)
        ref[0,1,1,2]=(0.703927399604)
        ref[1,0,0,0]=(0.0172966310708)
        ref[1,0,0,1]=(-0.097716216369)
        ref[1,0,0,2]=(0.564328624475)
        ref[1,0,1,0]=(-0.285571273779)
        ref[1,0,1,1]=(-0.478508810269)
        ref[1,0,1,2]=(0.713694990763)
        ref[1,1,0,0]=(-1.72203517221)
        ref[1,1,0,1]=(0.90209571161)
        ref[1,1,0,2]=(0.0404190111547)
        ref[1,1,1,0]=(0.662367771483)
        ref[1,1,1,1]=(0.683618857708)
        ref[1,1,1,2]=(0.220237165119)
        ref[2,0,0,0]=(-1.23109104652)
        ref[2,0,0,1]=(-0.344954703402)
        ref[2,0,0,2]=(-0.845700366118)
        ref[2,0,1,0]=(-1.78196237278)
        ref[2,0,1,1]=(0.352288083535)
        ref[2,0,1,2]=(1.02993741657)
        ref[2,1,0,0]=(-0.101867429012)
        ref[2,1,0,1]=(-0.810130865002)
        ref[2,1,0,2]=(0.682007840621)
        ref[2,1,1,0]=(0.424977434864)
        ref[2,1,1,1]=(-0.193692196905)
        ref[2,1,1,2]=(1.40079084873)
        ref[3,0,0,0]=(0.471393445911)
        ref[3,0,0,1]=(-0.56961804428)
        ref[3,0,0,2]=(1.1096657312)
        ref[3,0,1,0]=(1.09919079441)
        ref[3,0,1,1]=(1.82571072835)
        ref[3,0,1,2]=(0.604299398913)
        ref[3,1,0,0]=(0.155938760923)
        ref[3,1,0,1]=(0.841141932215)
        ref[3,1,0,2]=(0.558240114855)
        ref[3,1,1,0]=(-0.0697992380617)
        ref[3,1,1,1]=(1.28687270053)
        ref[3,1,1,2]=(-1.65036825798)
        ref[4,0,0,0]=(-0.497902210966)
        ref[4,0,0,1]=(0.495675372467)
        ref[4,0,0,2]=(1.4729295959)
        ref[4,0,1,0]=(-0.278468085457)
        ref[4,0,1,1]=(0.515340965551)
        ref[4,0,1,2]=(-0.233864738508)
        ref[4,1,0,0]=(0.574455672589)
        ref[4,1,0,1]=(-1.41224801693)
        ref[4,1,0,2]=(-0.0851508668768)
        ref[4,1,1,0]=(-0.392015012986)
        ref[4,1,1,1]=(0.211411943089)
        ref[4,1,1,2]=(0.175695876916)
        ref[5,0,0,0]=(-1.3317854233)
        ref[5,0,0,1]=(1.30129857885)
        ref[5,0,0,2]=(-0.96126616548)
        ref[5,0,1,0]=(-1.00763984246)
        ref[5,0,1,1]=(0.252485810112)
        ref[5,0,1,2]=(0.0349533247409)
        ref[5,1,0,0]=(1.23304668416)
        ref[5,1,0,1]=(0.978792435109)
        ref[5,1,0,2]=(0.0784889665787)
        ref[5,1,1,0]=(-0.164813552675)
        ref[5,1,1,1]=(0.363900322694)
        ref[5,1,1,2]=(-0.388851362667)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.049981434374)*x[0]+(1.21186502577)*x[1]
        ref[0]=(-0.049981434374)
        ref[1]=(1.21186502577)
      else:
        s=(-0.0752432283484)*x[0]+(0.11438092577)*x[1]+(-0.434888190035)*x[2]
        ref[0]=(-0.0752432283484)
        ref[1]=(0.11438092577)
        ref[2]=(-0.434888190035)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.456351387646)*x[0]+(-0.116715426711)*x[1]
        s[1]=(-0.854853163245)*x[0]+(-0.00664827097294)*x[1]
        ref[0,0]=(-0.456351387646)
        ref[0,1]=(-0.116715426711)
        ref[1,0]=(-0.854853163245)
        ref[1,1]=(-0.00664827097294)
      else:
        s[0]=(0.623100837308)*x[0]+(0.266907813705)*x[1]+(1.38575183924)*x[2]
        s[1]=(-1.48702785688)*x[0]+(-1.42426212001)*x[1]+(0.317785468787)*x[2]
        ref[0,0]=(0.623100837308)
        ref[0,1]=(0.266907813705)
        ref[0,2]=(1.38575183924)
        ref[1,0]=(-1.48702785688)
        ref[1,1]=(-1.42426212001)
        ref[1,2]=(0.317785468787)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-1.0789465623)*x[0]+(-0.347959175817)*x[1]
        s[0,1]=(-0.551011467115)*x[0]+(1.35264465045)*x[1]
        s[0,2]=(-0.570905806915)*x[0]+(0.0584374344724)*x[1]
        s[0,3]=(0.596777443685)*x[0]+(0.414694389615)*x[1]
        s[0,4]=(0.611144733018)*x[0]+(-0.758856666701)*x[1]
        s[1,0]=(-0.0733659967272)*x[0]+(1.27900296295)*x[1]
        s[1,1]=(-0.305565660896)*x[0]+(0.9434988738)*x[1]
        s[1,2]=(-0.992776824928)*x[0]+(-0.120817150952)*x[1]
        s[1,3]=(0.0902201920474)*x[0]+(1.04097295319)*x[1]
        s[1,4]=(-1.06448209062)*x[0]+(1.13868467732)*x[1]
        s[2,0]=(0.855203060995)*x[0]+(0.707160314719)*x[1]
        s[2,1]=(0.792799717854)*x[0]+(-0.36484734497)*x[1]
        s[2,2]=(-1.14542452528)*x[0]+(0.98847021504)*x[1]
        s[2,3]=(0.145237440613)*x[0]+(0.214714794234)*x[1]
        s[2,4]=(0.179556206154)*x[0]+(-1.21107721861)*x[1]
        s[3,0]=(0.700250733001)*x[0]+(1.25331090966)*x[1]
        s[3,1]=(0.258700850981)*x[0]+(-0.637474578328)*x[1]
        s[3,2]=(0.382709435533)*x[0]+(1.29575763411)*x[1]
        s[3,3]=(-0.254008873764)*x[0]+(-1.54340970006)*x[1]
        s[3,4]=(-0.658082774987)*x[0]+(0.647639696672)*x[1]
        ref[0,0,0]=(-1.0789465623)
        ref[0,0,1]=(-0.347959175817)
        ref[0,1,0]=(-0.551011467115)
        ref[0,1,1]=(1.35264465045)
        ref[0,2,0]=(-0.570905806915)
        ref[0,2,1]=(0.0584374344724)
        ref[0,3,0]=(0.596777443685)
        ref[0,3,1]=(0.414694389615)
        ref[0,4,0]=(0.611144733018)
        ref[0,4,1]=(-0.758856666701)
        ref[1,0,0]=(-0.0733659967272)
        ref[1,0,1]=(1.27900296295)
        ref[1,1,0]=(-0.305565660896)
        ref[1,1,1]=(0.9434988738)
        ref[1,2,0]=(-0.992776824928)
        ref[1,2,1]=(-0.120817150952)
        ref[1,3,0]=(0.0902201920474)
        ref[1,3,1]=(1.04097295319)
        ref[1,4,0]=(-1.06448209062)
        ref[1,4,1]=(1.13868467732)
        ref[2,0,0]=(0.855203060995)
        ref[2,0,1]=(0.707160314719)
        ref[2,1,0]=(0.792799717854)
        ref[2,1,1]=(-0.36484734497)
        ref[2,2,0]=(-1.14542452528)
        ref[2,2,1]=(0.98847021504)
        ref[2,3,0]=(0.145237440613)
        ref[2,3,1]=(0.214714794234)
        ref[2,4,0]=(0.179556206154)
        ref[2,4,1]=(-1.21107721861)
        ref[3,0,0]=(0.700250733001)
        ref[3,0,1]=(1.25331090966)
        ref[3,1,0]=(0.258700850981)
        ref[3,1,1]=(-0.637474578328)
        ref[3,2,0]=(0.382709435533)
        ref[3,2,1]=(1.29575763411)
        ref[3,3,0]=(-0.254008873764)
        ref[3,3,1]=(-1.54340970006)
        ref[3,4,0]=(-0.658082774987)
        ref[3,4,1]=(0.647639696672)
      else:
        s[0,0]=(1.12878816262)*x[0]+(-1.74114589021)*x[1]+(0.248765786072)*x[2]
        s[0,1]=(1.72583176211)*x[0]+(-1.23114549996)*x[1]+(-0.203157732085)*x[2]
        s[0,2]=(-0.422314546839)*x[0]+(0.224260123741)*x[1]+(0.171252253031)*x[2]
        s[0,3]=(-0.870712689882)*x[0]+(-0.555793590179)*x[1]+(0.290927600809)*x[2]
        s[0,4]=(-0.920833142524)*x[0]+(-0.0497652513943)*x[1]+(-0.39718342747)*x[2]
        s[1,0]=(-0.727450597273)*x[0]+(1.09323195342)*x[1]+(0.298478430357)*x[2]
        s[1,1]=(0.12608256775)*x[0]+(-0.459788156475)*x[1]+(1.13249234657)*x[2]
        s[1,2]=(0.898608507221)*x[0]+(-0.46334527476)*x[1]+(-0.445291041098)*x[2]
        s[1,3]=(-0.327616894595)*x[0]+(-0.288577792837)*x[1]+(1.26565355504)*x[2]
        s[1,4]=(0.0630186390647)*x[0]+(-0.383734001362)*x[1]+(-0.116975162517)*x[2]
        s[2,0]=(-0.309394005751)*x[0]+(-0.12474002694)*x[1]+(0.699555331484)*x[2]
        s[2,1]=(0.997752898742)*x[0]+(-0.183982572346)*x[1]+(1.22982190192)*x[2]
        s[2,2]=(1.03496586697)*x[0]+(-0.097256212804)*x[1]+(1.42052511301)*x[2]
        s[2,3]=(0.614378729663)*x[0]+(0.154397435133)*x[1]+(-0.863004330742)*x[2]
        s[2,4]=(1.20904264903)*x[0]+(-0.452388361293)*x[1]+(-1.13286356474)*x[2]
        s[3,0]=(-0.812538381038)*x[0]+(0.00308372111735)*x[1]+(-0.117937095444)*x[2]
        s[3,1]=(0.45479320042)*x[0]+(-0.45927716926)*x[1]+(0.4787228586)*x[2]
        s[3,2]=(-0.487685328452)*x[0]+(0.871241301367)*x[1]+(0.78769505788)*x[2]
        s[3,3]=(0.860788048428)*x[0]+(1.39676274038)*x[1]+(0.407326483464)*x[2]
        s[3,4]=(0.00409247820271)*x[0]+(0.44466332801)*x[1]+(-0.66721280275)*x[2]
        ref[0,0,0]=(1.12878816262)
        ref[0,0,1]=(-1.74114589021)
        ref[0,0,2]=(0.248765786072)
        ref[0,1,0]=(1.72583176211)
        ref[0,1,1]=(-1.23114549996)
        ref[0,1,2]=(-0.203157732085)
        ref[0,2,0]=(-0.422314546839)
        ref[0,2,1]=(0.224260123741)
        ref[0,2,2]=(0.171252253031)
        ref[0,3,0]=(-0.870712689882)
        ref[0,3,1]=(-0.555793590179)
        ref[0,3,2]=(0.290927600809)
        ref[0,4,0]=(-0.920833142524)
        ref[0,4,1]=(-0.0497652513943)
        ref[0,4,2]=(-0.39718342747)
        ref[1,0,0]=(-0.727450597273)
        ref[1,0,1]=(1.09323195342)
        ref[1,0,2]=(0.298478430357)
        ref[1,1,0]=(0.12608256775)
        ref[1,1,1]=(-0.459788156475)
        ref[1,1,2]=(1.13249234657)
        ref[1,2,0]=(0.898608507221)
        ref[1,2,1]=(-0.46334527476)
        ref[1,2,2]=(-0.445291041098)
        ref[1,3,0]=(-0.327616894595)
        ref[1,3,1]=(-0.288577792837)
        ref[1,3,2]=(1.26565355504)
        ref[1,4,0]=(0.0630186390647)
        ref[1,4,1]=(-0.383734001362)
        ref[1,4,2]=(-0.116975162517)
        ref[2,0,0]=(-0.309394005751)
        ref[2,0,1]=(-0.12474002694)
        ref[2,0,2]=(0.699555331484)
        ref[2,1,0]=(0.997752898742)
        ref[2,1,1]=(-0.183982572346)
        ref[2,1,2]=(1.22982190192)
        ref[2,2,0]=(1.03496586697)
        ref[2,2,1]=(-0.097256212804)
        ref[2,2,2]=(1.42052511301)
        ref[2,3,0]=(0.614378729663)
        ref[2,3,1]=(0.154397435133)
        ref[2,3,2]=(-0.863004330742)
        ref[2,4,0]=(1.20904264903)
        ref[2,4,1]=(-0.452388361293)
        ref[2,4,2]=(-1.13286356474)
        ref[3,0,0]=(-0.812538381038)
        ref[3,0,1]=(0.00308372111735)
        ref[3,0,2]=(-0.117937095444)
        ref[3,1,0]=(0.45479320042)
        ref[3,1,1]=(-0.45927716926)
        ref[3,1,2]=(0.4787228586)
        ref[3,2,0]=(-0.487685328452)
        ref[3,2,1]=(0.871241301367)
        ref[3,2,2]=(0.78769505788)
        ref[3,3,0]=(0.860788048428)
        ref[3,3,1]=(1.39676274038)
        ref[3,3,2]=(0.407326483464)
        ref[3,4,0]=(0.00409247820271)
        ref[3,4,1]=(0.44466332801)
        ref[3,4,2]=(-0.66721280275)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.905736881137)*x[0]+(0.694861875477)*x[1]
        s[0,0,1]=(-1.45890910436)*x[0]+(-1.52565465047)*x[1]
        s[0,1,0]=(-0.312122938001)*x[0]+(-0.868878770447)*x[1]
        s[0,1,1]=(1.54725692119)*x[0]+(0.536122055612)*x[1]
        s[1,0,0]=(0.196190980599)*x[0]+(0.328328643332)*x[1]
        s[1,0,1]=(-1.08863159071)*x[0]+(-1.33920996273)*x[1]
        s[1,1,0]=(1.73274170273)*x[0]+(-0.958793175994)*x[1]
        s[1,1,1]=(0.182836505159)*x[0]+(-0.121541234518)*x[1]
        s[2,0,0]=(0.144461277784)*x[0]+(1.83781976673)*x[1]
        s[2,0,1]=(-0.194587892421)*x[0]+(-1.27090147949)*x[1]
        s[2,1,0]=(0.379488182396)*x[0]+(0.030493966182)*x[1]
        s[2,1,1]=(1.09003510354)*x[0]+(-0.454578719663)*x[1]
        s[3,0,0]=(1.58446081096)*x[0]+(-0.52113508443)*x[1]
        s[3,0,1]=(-1.33593796269)*x[0]+(-0.165961948897)*x[1]
        s[3,1,0]=(0.377402371367)*x[0]+(-0.0184973476961)*x[1]
        s[3,1,1]=(0.764313855665)*x[0]+(0.294443818887)*x[1]
        s[4,0,0]=(-0.180639063236)*x[0]+(1.75864081648)*x[1]
        s[4,0,1]=(-1.62662138708)*x[0]+(1.81611541058)*x[1]
        s[4,1,0]=(1.37678767999)*x[0]+(-0.271115982236)*x[1]
        s[4,1,1]=(1.32861834819)*x[0]+(0.768596947238)*x[1]
        s[5,0,0]=(1.39342920577)*x[0]+(0.155666506926)*x[1]
        s[5,0,1]=(0.0397474261373)*x[0]+(0.921852245407)*x[1]
        s[5,1,0]=(-0.950337694161)*x[0]+(-0.011042822071)*x[1]
        s[5,1,1]=(0.0369412335225)*x[0]+(-0.473468894798)*x[1]
        ref[0,0,0,0]=(-0.905736881137)
        ref[0,0,0,1]=(0.694861875477)
        ref[0,0,1,0]=(-1.45890910436)
        ref[0,0,1,1]=(-1.52565465047)
        ref[0,1,0,0]=(-0.312122938001)
        ref[0,1,0,1]=(-0.868878770447)
        ref[0,1,1,0]=(1.54725692119)
        ref[0,1,1,1]=(0.536122055612)
        ref[1,0,0,0]=(0.196190980599)
        ref[1,0,0,1]=(0.328328643332)
        ref[1,0,1,0]=(-1.08863159071)
        ref[1,0,1,1]=(-1.33920996273)
        ref[1,1,0,0]=(1.73274170273)
        ref[1,1,0,1]=(-0.958793175994)
        ref[1,1,1,0]=(0.182836505159)
        ref[1,1,1,1]=(-0.121541234518)
        ref[2,0,0,0]=(0.144461277784)
        ref[2,0,0,1]=(1.83781976673)
        ref[2,0,1,0]=(-0.194587892421)
        ref[2,0,1,1]=(-1.27090147949)
        ref[2,1,0,0]=(0.379488182396)
        ref[2,1,0,1]=(0.030493966182)
        ref[2,1,1,0]=(1.09003510354)
        ref[2,1,1,1]=(-0.454578719663)
        ref[3,0,0,0]=(1.58446081096)
        ref[3,0,0,1]=(-0.52113508443)
        ref[3,0,1,0]=(-1.33593796269)
        ref[3,0,1,1]=(-0.165961948897)
        ref[3,1,0,0]=(0.377402371367)
        ref[3,1,0,1]=(-0.0184973476961)
        ref[3,1,1,0]=(0.764313855665)
        ref[3,1,1,1]=(0.294443818887)
        ref[4,0,0,0]=(-0.180639063236)
        ref[4,0,0,1]=(1.75864081648)
        ref[4,0,1,0]=(-1.62662138708)
        ref[4,0,1,1]=(1.81611541058)
        ref[4,1,0,0]=(1.37678767999)
        ref[4,1,0,1]=(-0.271115982236)
        ref[4,1,1,0]=(1.32861834819)
        ref[4,1,1,1]=(0.768596947238)
        ref[5,0,0,0]=(1.39342920577)
        ref[5,0,0,1]=(0.155666506926)
        ref[5,0,1,0]=(0.0397474261373)
        ref[5,0,1,1]=(0.921852245407)
        ref[5,1,0,0]=(-0.950337694161)
        ref[5,1,0,1]=(-0.011042822071)
        ref[5,1,1,0]=(0.0369412335225)
        ref[5,1,1,1]=(-0.473468894798)
      else:
        s[0,0,0]=(0.0887916493027)*x[0]+(-0.648868378605)*x[1]+(-0.0728819090295)*x[2]
        s[0,0,1]=(-1.41771256969)*x[0]+(-1.5907790724)*x[1]+(-0.208960186234)*x[2]
        s[0,1,0]=(-0.846039306108)*x[0]+(0.451942850498)*x[1]+(-0.305613220999)*x[2]
        s[0,1,1]=(-0.990248401344)*x[0]+(0.934658804462)*x[1]+(0.703927399604)*x[2]
        s[1,0,0]=(0.0172966310708)*x[0]+(-0.097716216369)*x[1]+(0.564328624475)*x[2]
        s[1,0,1]=(-0.285571273779)*x[0]+(-0.478508810269)*x[1]+(0.713694990763)*x[2]
        s[1,1,0]=(-1.72203517221)*x[0]+(0.90209571161)*x[1]+(0.0404190111547)*x[2]
        s[1,1,1]=(0.662367771483)*x[0]+(0.683618857708)*x[1]+(0.220237165119)*x[2]
        s[2,0,0]=(-1.23109104652)*x[0]+(-0.344954703402)*x[1]+(-0.845700366118)*x[2]
        s[2,0,1]=(-1.78196237278)*x[0]+(0.352288083535)*x[1]+(1.02993741657)*x[2]
        s[2,1,0]=(-0.101867429012)*x[0]+(-0.810130865002)*x[1]+(0.682007840621)*x[2]
        s[2,1,1]=(0.424977434864)*x[0]+(-0.193692196905)*x[1]+(1.40079084873)*x[2]
        s[3,0,0]=(0.471393445911)*x[0]+(-0.56961804428)*x[1]+(1.1096657312)*x[2]
        s[3,0,1]=(1.09919079441)*x[0]+(1.82571072835)*x[1]+(0.604299398913)*x[2]
        s[3,1,0]=(0.155938760923)*x[0]+(0.841141932215)*x[1]+(0.558240114855)*x[2]
        s[3,1,1]=(-0.0697992380617)*x[0]+(1.28687270053)*x[1]+(-1.65036825798)*x[2]
        s[4,0,0]=(-0.497902210966)*x[0]+(0.495675372467)*x[1]+(1.4729295959)*x[2]
        s[4,0,1]=(-0.278468085457)*x[0]+(0.515340965551)*x[1]+(-0.233864738508)*x[2]
        s[4,1,0]=(0.574455672589)*x[0]+(-1.41224801693)*x[1]+(-0.0851508668768)*x[2]
        s[4,1,1]=(-0.392015012986)*x[0]+(0.211411943089)*x[1]+(0.175695876916)*x[2]
        s[5,0,0]=(-1.3317854233)*x[0]+(1.30129857885)*x[1]+(-0.96126616548)*x[2]
        s[5,0,1]=(-1.00763984246)*x[0]+(0.252485810112)*x[1]+(0.0349533247409)*x[2]
        s[5,1,0]=(1.23304668416)*x[0]+(0.978792435109)*x[1]+(0.0784889665787)*x[2]
        s[5,1,1]=(-0.164813552675)*x[0]+(0.363900322694)*x[1]+(-0.388851362667)*x[2]
        ref[0,0,0,0]=(0.0887916493027)
        ref[0,0,0,1]=(-0.648868378605)
        ref[0,0,0,2]=(-0.0728819090295)
        ref[0,0,1,0]=(-1.41771256969)
        ref[0,0,1,1]=(-1.5907790724)
        ref[0,0,1,2]=(-0.208960186234)
        ref[0,1,0,0]=(-0.846039306108)
        ref[0,1,0,1]=(0.451942850498)
        ref[0,1,0,2]=(-0.305613220999)
        ref[0,1,1,0]=(-0.990248401344)
        ref[0,1,1,1]=(0.934658804462)
        ref[0,1,1,2]=(0.703927399604)
        ref[1,0,0,0]=(0.0172966310708)
        ref[1,0,0,1]=(-0.097716216369)
        ref[1,0,0,2]=(0.564328624475)
        ref[1,0,1,0]=(-0.285571273779)
        ref[1,0,1,1]=(-0.478508810269)
        ref[1,0,1,2]=(0.713694990763)
        ref[1,1,0,0]=(-1.72203517221)
        ref[1,1,0,1]=(0.90209571161)
        ref[1,1,0,2]=(0.0404190111547)
        ref[1,1,1,0]=(0.662367771483)
        ref[1,1,1,1]=(0.683618857708)
        ref[1,1,1,2]=(0.220237165119)
        ref[2,0,0,0]=(-1.23109104652)
        ref[2,0,0,1]=(-0.344954703402)
        ref[2,0,0,2]=(-0.845700366118)
        ref[2,0,1,0]=(-1.78196237278)
        ref[2,0,1,1]=(0.352288083535)
        ref[2,0,1,2]=(1.02993741657)
        ref[2,1,0,0]=(-0.101867429012)
        ref[2,1,0,1]=(-0.810130865002)
        ref[2,1,0,2]=(0.682007840621)
        ref[2,1,1,0]=(0.424977434864)
        ref[2,1,1,1]=(-0.193692196905)
        ref[2,1,1,2]=(1.40079084873)
        ref[3,0,0,0]=(0.471393445911)
        ref[3,0,0,1]=(-0.56961804428)
        ref[3,0,0,2]=(1.1096657312)
        ref[3,0,1,0]=(1.09919079441)
        ref[3,0,1,1]=(1.82571072835)
        ref[3,0,1,2]=(0.604299398913)
        ref[3,1,0,0]=(0.155938760923)
        ref[3,1,0,1]=(0.841141932215)
        ref[3,1,0,2]=(0.558240114855)
        ref[3,1,1,0]=(-0.0697992380617)
        ref[3,1,1,1]=(1.28687270053)
        ref[3,1,1,2]=(-1.65036825798)
        ref[4,0,0,0]=(-0.497902210966)
        ref[4,0,0,1]=(0.495675372467)
        ref[4,0,0,2]=(1.4729295959)
        ref[4,0,1,0]=(-0.278468085457)
        ref[4,0,1,1]=(0.515340965551)
        ref[4,0,1,2]=(-0.233864738508)
        ref[4,1,0,0]=(0.574455672589)
        ref[4,1,0,1]=(-1.41224801693)
        ref[4,1,0,2]=(-0.0851508668768)
        ref[4,1,1,0]=(-0.392015012986)
        ref[4,1,1,1]=(0.211411943089)
        ref[4,1,1,2]=(0.175695876916)
        ref[5,0,0,0]=(-1.3317854233)
        ref[5,0,0,1]=(1.30129857885)
        ref[5,0,0,2]=(-0.96126616548)
        ref[5,0,1,0]=(-1.00763984246)
        ref[5,0,1,1]=(0.252485810112)
        ref[5,0,1,2]=(0.0349533247409)
        ref[5,1,0,0]=(1.23304668416)
        ref[5,1,0,1]=(0.978792435109)
        ref[5,1,0,2]=(0.0784889665787)
        ref[5,1,1,0]=(-0.164813552675)
        ref[5,1,1,1]=(0.363900322694)
        ref[5,1,1,2]=(-0.388851362667)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.741017463552)*x[0]**o+(0.665549604887)*x[0]+(-0.294688627656)*x[1]**o+(-0.188608956137)*x[1]
        ref[0]=o*(0.741017463552)*x_ref[0]**(o-1)+(0.665549604887)
        ref[1]=o*(-0.294688627656)*x_ref[1]**(o-1)+(-0.188608956137)
      else:
        arg=(-0.182264095574)*x[0]**o+(0.388884343071)*x[0]+(0.0250471058578)*x[1]**o+(0.242724421847)*x[1]+(-0.863568419189)*x[2]**o+(-0.797427046061)*x[2]
        ref[0]=o*(-0.182264095574)*x_ref[0]**(o-1)+(0.388884343071)
        ref[1]=o*(0.0250471058578)*x_ref[1]**(o-1)+(0.242724421847)
        ref[2]=o*(-0.863568419189)*x_ref[2]**(o-1)+(-0.797427046061)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.566748845779)*x[0]**o+(0.284156548246)*x[0]+(0.213805183732)*x[1]**o+(0.887833417271)*x[1]
        arg[1]=(-0.935376067943)*x[0]**o+(0.31417056704)*x[0]+(0.157729703937)*x[1]**o+(0.781264180355)*x[1]
        ref[0,0]=o*(-0.566748845779)*x_ref[0]**(o-1)+(0.284156548246)
        ref[0,1]=o*(0.213805183732)*x_ref[1]**(o-1)+(0.887833417271)
        ref[1,0]=o*(-0.935376067943)*x_ref[0]**(o-1)+(0.31417056704)
        ref[1,1]=o*(0.157729703937)*x_ref[1]**(o-1)+(0.781264180355)
      else:
        arg[0]=(0.787106597246)*x[0]**o+(0.0477259256332)*x[0]+(0.0657839729355)*x[1]**o+(-0.367242386854)*x[1]+(0.638047637996)*x[2]**o+(0.292293368262)*x[2]
        arg[1]=(0.325241621537)*x[0]**o+(-0.995465871567)*x[0]+(-0.589833259992)*x[1]**o+(0.252602994164)*x[1]+(0.271544628727)*x[2]**o+(0.401222388174)*x[2]
        ref[0,0]=o*(0.787106597246)*x_ref[0]**(o-1)+(0.0477259256332)
        ref[0,1]=o*(0.0657839729355)*x_ref[1]**(o-1)+(-0.367242386854)
        ref[0,2]=o*(0.638047637996)*x_ref[2]**(o-1)+(0.292293368262)
        ref[1,0]=o*(0.325241621537)*x_ref[0]**(o-1)+(-0.995465871567)
        ref[1,1]=o*(-0.589833259992)*x_ref[1]**(o-1)+(0.252602994164)
        ref[1,2]=o*(0.271544628727)*x_ref[2]**(o-1)+(0.401222388174)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.535138548297)*x[0]**o+(0.320793182084)*x[0]+(-0.363556615611)*x[1]**o+(0.80028981928)*x[1]
        arg[0,1]=(0.50712290341)*x[0]**o+(0.0696582374725)*x[0]+(-0.565430065964)*x[1]**o+(0.840341269012)*x[1]
        arg[0,2]=(0.222608517758)*x[0]**o+(0.0741096149609)*x[0]+(-0.926916099396)*x[1]**o+(0.840471681255)*x[1]
        arg[0,3]=(-0.0729061065831)*x[0]**o+(-0.361006912293)*x[0]+(-0.796381694915)*x[1]**o+(-0.87399027927)*x[1]
        arg[0,4]=(0.946982642563)*x[0]**o+(0.776871793528)*x[0]+(0.906858449271)*x[1]**o+(0.23076439514)*x[1]
        arg[1,0]=(-0.121641842365)*x[0]**o+(0.755274749862)*x[0]+(-0.905815080361)*x[1]**o+(-0.27711865979)*x[1]
        arg[1,1]=(0.870675475328)*x[0]**o+(0.000245820302085)*x[0]+(0.990534371185)*x[1]**o+(-0.0132722523111)*x[1]
        arg[1,2]=(0.665375098591)*x[0]**o+(0.602779210152)*x[0]+(0.112732227406)*x[1]**o+(0.973600268944)*x[1]
        arg[1,3]=(-0.760686885095)*x[0]**o+(-0.993893837808)*x[0]+(-0.279242757928)*x[1]**o+(0.0970216781461)*x[1]
        arg[1,4]=(-0.326265155118)*x[0]**o+(-0.302971035826)*x[0]+(-0.991053864037)*x[1]**o+(-0.787776341406)*x[1]
        arg[2,0]=(-0.798558123695)*x[0]**o+(-0.648773673567)*x[0]+(0.911336567339)*x[1]**o+(0.525888531875)*x[1]
        arg[2,1]=(0.334481967771)*x[0]**o+(0.559633318828)*x[0]+(-0.738361234791)*x[1]**o+(0.863808627657)*x[1]
        arg[2,2]=(-0.660270802705)*x[0]**o+(-0.500800405285)*x[0]+(0.250991213972)*x[1]**o+(-0.855371865339)*x[1]
        arg[2,3]=(0.573495152924)*x[0]**o+(0.841810214664)*x[0]+(0.816402403997)*x[1]**o+(0.871435278968)*x[1]
        arg[2,4]=(0.00700234490903)*x[0]**o+(0.609696655384)*x[0]+(-0.908261420109)*x[1]**o+(-0.0469518512659)*x[1]
        arg[3,0]=(-0.78381944233)*x[0]**o+(0.547849085605)*x[0]+(-0.481333073199)*x[1]**o+(-0.100496681813)*x[1]
        arg[3,1]=(0.53995869966)*x[0]**o+(-0.503303606531)*x[0]+(0.783394329024)*x[1]**o+(0.989828473603)*x[1]
        arg[3,2]=(0.723590003109)*x[0]**o+(-0.75511396868)*x[0]+(0.249651386017)*x[1]**o+(0.545636109436)*x[1]
        arg[3,3]=(-0.126863462818)*x[0]**o+(0.553295312002)*x[0]+(-0.286155398085)*x[1]**o+(0.425889110789)*x[1]
        arg[3,4]=(-0.970069386779)*x[0]**o+(0.457400568855)*x[0]+(-0.245839304835)*x[1]**o+(-0.463000940469)*x[1]
        ref[0,0,0]=o*(-0.535138548297)*x_ref[0]**(o-1)+(0.320793182084)
        ref[0,0,1]=o*(-0.363556615611)*x_ref[1]**(o-1)+(0.80028981928)
        ref[0,1,0]=o*(0.50712290341)*x_ref[0]**(o-1)+(0.0696582374725)
        ref[0,1,1]=o*(-0.565430065964)*x_ref[1]**(o-1)+(0.840341269012)
        ref[0,2,0]=o*(0.222608517758)*x_ref[0]**(o-1)+(0.0741096149609)
        ref[0,2,1]=o*(-0.926916099396)*x_ref[1]**(o-1)+(0.840471681255)
        ref[0,3,0]=o*(-0.0729061065831)*x_ref[0]**(o-1)+(-0.361006912293)
        ref[0,3,1]=o*(-0.796381694915)*x_ref[1]**(o-1)+(-0.87399027927)
        ref[0,4,0]=o*(0.946982642563)*x_ref[0]**(o-1)+(0.776871793528)
        ref[0,4,1]=o*(0.906858449271)*x_ref[1]**(o-1)+(0.23076439514)
        ref[1,0,0]=o*(-0.121641842365)*x_ref[0]**(o-1)+(0.755274749862)
        ref[1,0,1]=o*(-0.905815080361)*x_ref[1]**(o-1)+(-0.27711865979)
        ref[1,1,0]=o*(0.870675475328)*x_ref[0]**(o-1)+(0.000245820302085)
        ref[1,1,1]=o*(0.990534371185)*x_ref[1]**(o-1)+(-0.0132722523111)
        ref[1,2,0]=o*(0.665375098591)*x_ref[0]**(o-1)+(0.602779210152)
        ref[1,2,1]=o*(0.112732227406)*x_ref[1]**(o-1)+(0.973600268944)
        ref[1,3,0]=o*(-0.760686885095)*x_ref[0]**(o-1)+(-0.993893837808)
        ref[1,3,1]=o*(-0.279242757928)*x_ref[1]**(o-1)+(0.0970216781461)
        ref[1,4,0]=o*(-0.326265155118)*x_ref[0]**(o-1)+(-0.302971035826)
        ref[1,4,1]=o*(-0.991053864037)*x_ref[1]**(o-1)+(-0.787776341406)
        ref[2,0,0]=o*(-0.798558123695)*x_ref[0]**(o-1)+(-0.648773673567)
        ref[2,0,1]=o*(0.911336567339)*x_ref[1]**(o-1)+(0.525888531875)
        ref[2,1,0]=o*(0.334481967771)*x_ref[0]**(o-1)+(0.559633318828)
        ref[2,1,1]=o*(-0.738361234791)*x_ref[1]**(o-1)+(0.863808627657)
        ref[2,2,0]=o*(-0.660270802705)*x_ref[0]**(o-1)+(-0.500800405285)
        ref[2,2,1]=o*(0.250991213972)*x_ref[1]**(o-1)+(-0.855371865339)
        ref[2,3,0]=o*(0.573495152924)*x_ref[0]**(o-1)+(0.841810214664)
        ref[2,3,1]=o*(0.816402403997)*x_ref[1]**(o-1)+(0.871435278968)
        ref[2,4,0]=o*(0.00700234490903)*x_ref[0]**(o-1)+(0.609696655384)
        ref[2,4,1]=o*(-0.908261420109)*x_ref[1]**(o-1)+(-0.0469518512659)
        ref[3,0,0]=o*(-0.78381944233)*x_ref[0]**(o-1)+(0.547849085605)
        ref[3,0,1]=o*(-0.481333073199)*x_ref[1]**(o-1)+(-0.100496681813)
        ref[3,1,0]=o*(0.53995869966)*x_ref[0]**(o-1)+(-0.503303606531)
        ref[3,1,1]=o*(0.783394329024)*x_ref[1]**(o-1)+(0.989828473603)
        ref[3,2,0]=o*(0.723590003109)*x_ref[0]**(o-1)+(-0.75511396868)
        ref[3,2,1]=o*(0.249651386017)*x_ref[1]**(o-1)+(0.545636109436)
        ref[3,3,0]=o*(-0.126863462818)*x_ref[0]**(o-1)+(0.553295312002)
        ref[3,3,1]=o*(-0.286155398085)*x_ref[1]**(o-1)+(0.425889110789)
        ref[3,4,0]=o*(-0.970069386779)*x_ref[0]**(o-1)+(0.457400568855)
        ref[3,4,1]=o*(-0.245839304835)*x_ref[1]**(o-1)+(-0.463000940469)
      else:
        arg[0,0]=(-0.0252675874258)*x[0]**o+(-0.998043629662)*x[0]+(-0.956603187221)*x[1]**o+(-0.642113208329)*x[1]+(-0.59687133215)*x[2]**o+(-0.222380006695)*x[2]
        arg[0,1]=(0.540057023826)*x[0]**o+(0.660021105706)*x[0]+(-0.0847035816563)*x[1]**o+(0.734935147879)*x[1]+(0.895080236848)*x[2]**o+(0.759076168225)*x[2]
        arg[0,2]=(0.208276460123)*x[0]**o+(-0.842726599314)*x[0]+(0.514146640689)*x[1]**o+(-0.931115443386)*x[1]+(-0.427990918054)*x[2]**o+(-0.0944702225036)*x[2]
        arg[0,3]=(-0.669873393665)*x[0]**o+(-0.164211687812)*x[0]+(0.257292402022)*x[1]**o+(0.788249196864)*x[1]+(-0.0160232742588)*x[2]**o+(0.593261720532)*x[2]
        arg[0,4]=(0.574685018662)*x[0]**o+(-0.404093341935)*x[0]+(-0.613704015926)*x[1]**o+(-0.388769512229)*x[1]+(0.65014059769)*x[2]**o+(0.796497720581)*x[2]
        arg[1,0]=(0.579425469692)*x[0]**o+(0.706786266065)*x[0]+(-0.489805412824)*x[1]**o+(0.412673120993)*x[1]+(-0.261487328773)*x[2]**o+(0.323016314178)*x[2]
        arg[1,1]=(-0.406115852457)*x[0]**o+(-0.320011474154)*x[0]+(0.83353566302)*x[1]**o+(-0.770757586792)*x[1]+(-0.497021878205)*x[2]**o+(-0.108546214912)*x[2]
        arg[1,2]=(-0.799159182698)*x[0]**o+(0.221424788319)*x[0]+(0.882092644719)*x[1]**o+(0.25804039903)*x[1]+(-0.788642442802)*x[2]**o+(-0.434293659417)*x[2]
        arg[1,3]=(-0.453596867354)*x[0]**o+(-0.855684542333)*x[0]+(0.421257402196)*x[1]**o+(0.162268279507)*x[1]+(-0.618396236314)*x[2]**o+(-0.650684413266)*x[2]
        arg[1,4]=(-0.464749011522)*x[0]**o+(-0.050661862641)*x[0]+(-0.7554290975)*x[1]**o+(-0.213826578781)*x[1]+(-0.536460109672)*x[2]**o+(-0.441490452069)*x[2]
        arg[2,0]=(0.335438338384)*x[0]**o+(-0.598515232086)*x[0]+(0.396958696838)*x[1]**o+(0.712094780848)*x[1]+(0.0112487672163)*x[2]**o+(-0.106361601342)*x[2]
        arg[2,1]=(0.157333767552)*x[0]**o+(0.763247557186)*x[0]+(0.454047596703)*x[1]**o+(0.874633696245)*x[1]+(0.80992740886)*x[2]**o+(-0.465565575276)*x[2]
        arg[2,2]=(-0.668932235683)*x[0]**o+(0.99055962791)*x[0]+(-0.473440344595)*x[1]**o+(-0.694789559356)*x[1]+(-0.337501506478)*x[2]**o+(-0.552461610916)*x[2]
        arg[2,3]=(-0.174040039098)*x[0]**o+(0.878604594327)*x[0]+(-0.488684761288)*x[1]**o+(-0.225626690649)*x[1]+(0.307550460093)*x[2]**o+(0.279172145442)*x[2]
        arg[2,4]=(0.771496476361)*x[0]**o+(-0.528231004311)*x[0]+(0.318620411204)*x[1]**o+(-0.831636376644)*x[1]+(-0.23477599759)*x[2]**o+(0.452986258707)*x[2]
        arg[3,0]=(0.689878836595)*x[0]**o+(-0.723887531161)*x[0]+(0.906077469738)*x[1]**o+(-0.729986756151)*x[1]+(0.860506345609)*x[2]**o+(-0.437203511895)*x[2]
        arg[3,1]=(0.867909616506)*x[0]**o+(-0.0480808079676)*x[0]+(-0.439925732789)*x[1]**o+(-0.753945614279)*x[1]+(-0.0015149141551)*x[2]**o+(-0.465559525628)*x[2]
        arg[3,2]=(0.915231983332)*x[0]**o+(0.77428136852)*x[0]+(0.902598621987)*x[1]**o+(-0.49204288782)*x[1]+(0.77362022057)*x[2]**o+(0.872135547313)*x[2]
        arg[3,3]=(0.274436414079)*x[0]**o+(-0.291721638943)*x[0]+(0.202501119852)*x[1]**o+(0.911330931145)*x[1]+(-0.61382801878)*x[2]**o+(0.386704897682)*x[2]
        arg[3,4]=(-0.0258695378217)*x[0]**o+(0.263046110195)*x[0]+(-0.162461126288)*x[1]**o+(-0.123570278155)*x[1]+(0.598012400912)*x[2]**o+(0.769951340801)*x[2]
        ref[0,0,0]=o*(-0.0252675874258)*x_ref[0]**(o-1)+(-0.998043629662)
        ref[0,0,1]=o*(-0.956603187221)*x_ref[1]**(o-1)+(-0.642113208329)
        ref[0,0,2]=o*(-0.59687133215)*x_ref[2]**(o-1)+(-0.222380006695)
        ref[0,1,0]=o*(0.540057023826)*x_ref[0]**(o-1)+(0.660021105706)
        ref[0,1,1]=o*(-0.0847035816563)*x_ref[1]**(o-1)+(0.734935147879)
        ref[0,1,2]=o*(0.895080236848)*x_ref[2]**(o-1)+(0.759076168225)
        ref[0,2,0]=o*(0.208276460123)*x_ref[0]**(o-1)+(-0.842726599314)
        ref[0,2,1]=o*(0.514146640689)*x_ref[1]**(o-1)+(-0.931115443386)
        ref[0,2,2]=o*(-0.427990918054)*x_ref[2]**(o-1)+(-0.0944702225036)
        ref[0,3,0]=o*(-0.669873393665)*x_ref[0]**(o-1)+(-0.164211687812)
        ref[0,3,1]=o*(0.257292402022)*x_ref[1]**(o-1)+(0.788249196864)
        ref[0,3,2]=o*(-0.0160232742588)*x_ref[2]**(o-1)+(0.593261720532)
        ref[0,4,0]=o*(0.574685018662)*x_ref[0]**(o-1)+(-0.404093341935)
        ref[0,4,1]=o*(-0.613704015926)*x_ref[1]**(o-1)+(-0.388769512229)
        ref[0,4,2]=o*(0.65014059769)*x_ref[2]**(o-1)+(0.796497720581)
        ref[1,0,0]=o*(0.579425469692)*x_ref[0]**(o-1)+(0.706786266065)
        ref[1,0,1]=o*(-0.489805412824)*x_ref[1]**(o-1)+(0.412673120993)
        ref[1,0,2]=o*(-0.261487328773)*x_ref[2]**(o-1)+(0.323016314178)
        ref[1,1,0]=o*(-0.406115852457)*x_ref[0]**(o-1)+(-0.320011474154)
        ref[1,1,1]=o*(0.83353566302)*x_ref[1]**(o-1)+(-0.770757586792)
        ref[1,1,2]=o*(-0.497021878205)*x_ref[2]**(o-1)+(-0.108546214912)
        ref[1,2,0]=o*(-0.799159182698)*x_ref[0]**(o-1)+(0.221424788319)
        ref[1,2,1]=o*(0.882092644719)*x_ref[1]**(o-1)+(0.25804039903)
        ref[1,2,2]=o*(-0.788642442802)*x_ref[2]**(o-1)+(-0.434293659417)
        ref[1,3,0]=o*(-0.453596867354)*x_ref[0]**(o-1)+(-0.855684542333)
        ref[1,3,1]=o*(0.421257402196)*x_ref[1]**(o-1)+(0.162268279507)
        ref[1,3,2]=o*(-0.618396236314)*x_ref[2]**(o-1)+(-0.650684413266)
        ref[1,4,0]=o*(-0.464749011522)*x_ref[0]**(o-1)+(-0.050661862641)
        ref[1,4,1]=o*(-0.7554290975)*x_ref[1]**(o-1)+(-0.213826578781)
        ref[1,4,2]=o*(-0.536460109672)*x_ref[2]**(o-1)+(-0.441490452069)
        ref[2,0,0]=o*(0.335438338384)*x_ref[0]**(o-1)+(-0.598515232086)
        ref[2,0,1]=o*(0.396958696838)*x_ref[1]**(o-1)+(0.712094780848)
        ref[2,0,2]=o*(0.0112487672163)*x_ref[2]**(o-1)+(-0.106361601342)
        ref[2,1,0]=o*(0.157333767552)*x_ref[0]**(o-1)+(0.763247557186)
        ref[2,1,1]=o*(0.454047596703)*x_ref[1]**(o-1)+(0.874633696245)
        ref[2,1,2]=o*(0.80992740886)*x_ref[2]**(o-1)+(-0.465565575276)
        ref[2,2,0]=o*(-0.668932235683)*x_ref[0]**(o-1)+(0.99055962791)
        ref[2,2,1]=o*(-0.473440344595)*x_ref[1]**(o-1)+(-0.694789559356)
        ref[2,2,2]=o*(-0.337501506478)*x_ref[2]**(o-1)+(-0.552461610916)
        ref[2,3,0]=o*(-0.174040039098)*x_ref[0]**(o-1)+(0.878604594327)
        ref[2,3,1]=o*(-0.488684761288)*x_ref[1]**(o-1)+(-0.225626690649)
        ref[2,3,2]=o*(0.307550460093)*x_ref[2]**(o-1)+(0.279172145442)
        ref[2,4,0]=o*(0.771496476361)*x_ref[0]**(o-1)+(-0.528231004311)
        ref[2,4,1]=o*(0.318620411204)*x_ref[1]**(o-1)+(-0.831636376644)
        ref[2,4,2]=o*(-0.23477599759)*x_ref[2]**(o-1)+(0.452986258707)
        ref[3,0,0]=o*(0.689878836595)*x_ref[0]**(o-1)+(-0.723887531161)
        ref[3,0,1]=o*(0.906077469738)*x_ref[1]**(o-1)+(-0.729986756151)
        ref[3,0,2]=o*(0.860506345609)*x_ref[2]**(o-1)+(-0.437203511895)
        ref[3,1,0]=o*(0.867909616506)*x_ref[0]**(o-1)+(-0.0480808079676)
        ref[3,1,1]=o*(-0.439925732789)*x_ref[1]**(o-1)+(-0.753945614279)
        ref[3,1,2]=o*(-0.0015149141551)*x_ref[2]**(o-1)+(-0.465559525628)
        ref[3,2,0]=o*(0.915231983332)*x_ref[0]**(o-1)+(0.77428136852)
        ref[3,2,1]=o*(0.902598621987)*x_ref[1]**(o-1)+(-0.49204288782)
        ref[3,2,2]=o*(0.77362022057)*x_ref[2]**(o-1)+(0.872135547313)
        ref[3,3,0]=o*(0.274436414079)*x_ref[0]**(o-1)+(-0.291721638943)
        ref[3,3,1]=o*(0.202501119852)*x_ref[1]**(o-1)+(0.911330931145)
        ref[3,3,2]=o*(-0.61382801878)*x_ref[2]**(o-1)+(0.386704897682)
        ref[3,4,0]=o*(-0.0258695378217)*x_ref[0]**(o-1)+(0.263046110195)
        ref[3,4,1]=o*(-0.162461126288)*x_ref[1]**(o-1)+(-0.123570278155)
        ref[3,4,2]=o*(0.598012400912)*x_ref[2]**(o-1)+(0.769951340801)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.301542954432)*x[0]**o+(-0.126029003998)*x[0]+(-0.408561086014)*x[1]**o+(0.090642210886)*x[1]
        arg[0,0,1]=(0.456052591865)*x[0]**o+(0.624896766511)*x[0]+(0.87137087467)*x[1]**o+(-0.296114354421)*x[1]
        arg[0,1,0]=(0.370833001032)*x[0]**o+(-0.700862384031)*x[0]+(-0.276076372016)*x[1]**o+(-0.820884025764)*x[1]
        arg[0,1,1]=(0.550805148631)*x[0]**o+(-0.628825647329)*x[0]+(0.906362219793)*x[1]**o+(0.093148354989)*x[1]
        arg[1,0,0]=(-0.80347190678)*x[0]**o+(-0.112285193049)*x[0]+(-0.703090064994)*x[1]**o+(0.738893806856)*x[1]
        arg[1,0,1]=(0.716113817265)*x[0]**o+(-0.403563175278)*x[0]+(-0.545474326725)*x[1]**o+(-0.085377932579)*x[1]
        arg[1,1,0]=(0.0151945295253)*x[0]**o+(0.419681120006)*x[0]+(0.605889120207)*x[1]**o+(0.94921670969)*x[1]
        arg[1,1,1]=(0.330888675781)*x[0]**o+(0.536336632067)*x[0]+(0.130243661103)*x[1]**o+(0.249581698903)*x[1]
        arg[2,0,0]=(-0.94168189216)*x[0]**o+(0.42899125004)*x[0]+(0.104206704223)*x[1]**o+(0.992451993183)*x[1]
        arg[2,0,1]=(-0.557898869487)*x[0]**o+(-0.0387050073802)*x[0]+(-0.880396260326)*x[1]**o+(-0.649734782543)*x[1]
        arg[2,1,0]=(0.0362873703011)*x[0]**o+(0.641116555832)*x[0]+(-0.878565186806)*x[1]**o+(-0.305825058029)*x[1]
        arg[2,1,1]=(0.926144430868)*x[0]**o+(0.207902756992)*x[0]+(-0.823834000424)*x[1]**o+(0.541980766739)*x[1]
        arg[3,0,0]=(-0.497934527226)*x[0]**o+(0.904514339359)*x[0]+(0.0627382682436)*x[1]**o+(-0.886845257384)*x[1]
        arg[3,0,1]=(0.458163004678)*x[0]**o+(-0.890966354569)*x[0]+(0.779511556433)*x[1]**o+(0.911407858547)*x[1]
        arg[3,1,0]=(0.643197533583)*x[0]**o+(-0.445952483015)*x[0]+(-0.491648538422)*x[1]**o+(-0.339803793944)*x[1]
        arg[3,1,1]=(0.564910446806)*x[0]**o+(-0.251342703228)*x[0]+(-0.102526510378)*x[1]**o+(0.989034603428)*x[1]
        arg[4,0,0]=(0.0728812877818)*x[0]**o+(-0.160839673847)*x[0]+(0.078039622926)*x[1]**o+(-0.778740146006)*x[1]
        arg[4,0,1]=(-0.848970759855)*x[0]**o+(0.83795370612)*x[0]+(-0.652802628253)*x[1]**o+(0.570900978851)*x[1]
        arg[4,1,0]=(0.492154273484)*x[0]**o+(-0.138488354185)*x[0]+(0.269142955069)*x[1]**o+(-0.849995493706)*x[1]
        arg[4,1,1]=(0.945032529684)*x[0]**o+(0.611284327327)*x[0]+(0.549856083886)*x[1]**o+(0.347585489606)*x[1]
        arg[5,0,0]=(0.481456547858)*x[0]**o+(0.238848312219)*x[0]+(-0.5849679413)*x[1]**o+(-0.749140400072)*x[1]
        arg[5,0,1]=(0.27081490311)*x[0]**o+(-0.101642948148)*x[0]+(0.31847076679)*x[1]**o+(0.192182993758)*x[1]
        arg[5,1,0]=(-0.873348151897)*x[0]**o+(-0.613499952221)*x[0]+(0.54087826152)*x[1]**o+(-0.269096416524)*x[1]
        arg[5,1,1]=(-0.803277233859)*x[0]**o+(-0.714237882827)*x[0]+(-0.205544031812)*x[1]**o+(0.188144864673)*x[1]
        ref[0,0,0,0]=o*(-0.301542954432)*x_ref[0]**(o-1)+(-0.126029003998)
        ref[0,0,0,1]=o*(-0.408561086014)*x_ref[1]**(o-1)+(0.090642210886)
        ref[0,0,1,0]=o*(0.456052591865)*x_ref[0]**(o-1)+(0.624896766511)
        ref[0,0,1,1]=o*(0.87137087467)*x_ref[1]**(o-1)+(-0.296114354421)
        ref[0,1,0,0]=o*(0.370833001032)*x_ref[0]**(o-1)+(-0.700862384031)
        ref[0,1,0,1]=o*(-0.276076372016)*x_ref[1]**(o-1)+(-0.820884025764)
        ref[0,1,1,0]=o*(0.550805148631)*x_ref[0]**(o-1)+(-0.628825647329)
        ref[0,1,1,1]=o*(0.906362219793)*x_ref[1]**(o-1)+(0.093148354989)
        ref[1,0,0,0]=o*(-0.80347190678)*x_ref[0]**(o-1)+(-0.112285193049)
        ref[1,0,0,1]=o*(-0.703090064994)*x_ref[1]**(o-1)+(0.738893806856)
        ref[1,0,1,0]=o*(0.716113817265)*x_ref[0]**(o-1)+(-0.403563175278)
        ref[1,0,1,1]=o*(-0.545474326725)*x_ref[1]**(o-1)+(-0.085377932579)
        ref[1,1,0,0]=o*(0.0151945295253)*x_ref[0]**(o-1)+(0.419681120006)
        ref[1,1,0,1]=o*(0.605889120207)*x_ref[1]**(o-1)+(0.94921670969)
        ref[1,1,1,0]=o*(0.330888675781)*x_ref[0]**(o-1)+(0.536336632067)
        ref[1,1,1,1]=o*(0.130243661103)*x_ref[1]**(o-1)+(0.249581698903)
        ref[2,0,0,0]=o*(-0.94168189216)*x_ref[0]**(o-1)+(0.42899125004)
        ref[2,0,0,1]=o*(0.104206704223)*x_ref[1]**(o-1)+(0.992451993183)
        ref[2,0,1,0]=o*(-0.557898869487)*x_ref[0]**(o-1)+(-0.0387050073802)
        ref[2,0,1,1]=o*(-0.880396260326)*x_ref[1]**(o-1)+(-0.649734782543)
        ref[2,1,0,0]=o*(0.0362873703011)*x_ref[0]**(o-1)+(0.641116555832)
        ref[2,1,0,1]=o*(-0.878565186806)*x_ref[1]**(o-1)+(-0.305825058029)
        ref[2,1,1,0]=o*(0.926144430868)*x_ref[0]**(o-1)+(0.207902756992)
        ref[2,1,1,1]=o*(-0.823834000424)*x_ref[1]**(o-1)+(0.541980766739)
        ref[3,0,0,0]=o*(-0.497934527226)*x_ref[0]**(o-1)+(0.904514339359)
        ref[3,0,0,1]=o*(0.0627382682436)*x_ref[1]**(o-1)+(-0.886845257384)
        ref[3,0,1,0]=o*(0.458163004678)*x_ref[0]**(o-1)+(-0.890966354569)
        ref[3,0,1,1]=o*(0.779511556433)*x_ref[1]**(o-1)+(0.911407858547)
        ref[3,1,0,0]=o*(0.643197533583)*x_ref[0]**(o-1)+(-0.445952483015)
        ref[3,1,0,1]=o*(-0.491648538422)*x_ref[1]**(o-1)+(-0.339803793944)
        ref[3,1,1,0]=o*(0.564910446806)*x_ref[0]**(o-1)+(-0.251342703228)
        ref[3,1,1,1]=o*(-0.102526510378)*x_ref[1]**(o-1)+(0.989034603428)
        ref[4,0,0,0]=o*(0.0728812877818)*x_ref[0]**(o-1)+(-0.160839673847)
        ref[4,0,0,1]=o*(0.078039622926)*x_ref[1]**(o-1)+(-0.778740146006)
        ref[4,0,1,0]=o*(-0.848970759855)*x_ref[0]**(o-1)+(0.83795370612)
        ref[4,0,1,1]=o*(-0.652802628253)*x_ref[1]**(o-1)+(0.570900978851)
        ref[4,1,0,0]=o*(0.492154273484)*x_ref[0]**(o-1)+(-0.138488354185)
        ref[4,1,0,1]=o*(0.269142955069)*x_ref[1]**(o-1)+(-0.849995493706)
        ref[4,1,1,0]=o*(0.945032529684)*x_ref[0]**(o-1)+(0.611284327327)
        ref[4,1,1,1]=o*(0.549856083886)*x_ref[1]**(o-1)+(0.347585489606)
        ref[5,0,0,0]=o*(0.481456547858)*x_ref[0]**(o-1)+(0.238848312219)
        ref[5,0,0,1]=o*(-0.5849679413)*x_ref[1]**(o-1)+(-0.749140400072)
        ref[5,0,1,0]=o*(0.27081490311)*x_ref[0]**(o-1)+(-0.101642948148)
        ref[5,0,1,1]=o*(0.31847076679)*x_ref[1]**(o-1)+(0.192182993758)
        ref[5,1,0,0]=o*(-0.873348151897)*x_ref[0]**(o-1)+(-0.613499952221)
        ref[5,1,0,1]=o*(0.54087826152)*x_ref[1]**(o-1)+(-0.269096416524)
        ref[5,1,1,0]=o*(-0.803277233859)*x_ref[0]**(o-1)+(-0.714237882827)
        ref[5,1,1,1]=o*(-0.205544031812)*x_ref[1]**(o-1)+(0.188144864673)
      else:
        arg[0,0,0]=(-0.639251981322)*x[0]**o+(-0.115972267818)*x[0]+(0.491134645863)*x[1]**o+(0.345343639029)*x[1]+(-0.663597762251)*x[2]**o+(-0.328150609455)*x[2]
        arg[0,0,1]=(-0.76280151252)*x[0]**o+(0.933789965647)*x[0]+(-0.117386438957)*x[1]**o+(0.738801880577)*x[1]+(-0.823759196233)*x[2]**o+(-0.591225057134)*x[2]
        arg[0,1,0]=(-0.515479158328)*x[0]**o+(-0.498840147412)*x[0]+(-0.07870152457)*x[1]**o+(0.240823897179)*x[1]+(-0.529483054724)*x[2]**o+(0.957444878465)*x[2]
        arg[0,1,1]=(0.659163197783)*x[0]**o+(-0.987017484017)*x[0]+(0.316073196672)*x[1]**o+(-0.858184573568)*x[1]+(-0.324909144593)*x[2]**o+(-0.89624674398)*x[2]
        arg[1,0,0]=(0.582390600104)*x[0]**o+(-0.808724939916)*x[0]+(-0.204186916478)*x[1]**o+(0.63212764696)*x[1]+(-0.272947627588)*x[2]**o+(-0.133369751815)*x[2]
        arg[1,0,1]=(-0.955892141909)*x[0]**o+(0.915832405494)*x[0]+(-0.823428836636)*x[1]**o+(-0.527176552593)*x[1]+(-0.439872810771)*x[2]**o+(0.282431362763)*x[2]
        arg[1,1,0]=(0.269192050139)*x[0]**o+(0.304289733459)*x[0]+(-0.252137123064)*x[1]**o+(0.159346881758)*x[1]+(-0.875713938743)*x[2]**o+(-0.804681982204)*x[2]
        arg[1,1,1]=(0.100159906585)*x[0]**o+(0.652670710888)*x[0]+(-0.0471177296503)*x[1]**o+(0.15344212377)*x[1]+(0.162356023341)*x[2]**o+(0.242855769737)*x[2]
        arg[2,0,0]=(-0.0214334988311)*x[0]**o+(0.444654672491)*x[0]+(0.235478960156)*x[1]**o+(0.62191559883)*x[1]+(0.124531773653)*x[2]**o+(-0.603748942811)*x[2]
        arg[2,0,1]=(-0.00696114124898)*x[0]**o+(-0.350635993795)*x[0]+(0.766513244466)*x[1]**o+(-0.99996024111)*x[1]+(0.976036637033)*x[2]**o+(0.00703160282837)*x[2]
        arg[2,1,0]=(-0.210499708873)*x[0]**o+(-0.763641982769)*x[0]+(0.435435481771)*x[1]**o+(0.0886400511888)*x[1]+(0.540913993886)*x[2]**o+(-0.864904989707)*x[2]
        arg[2,1,1]=(0.609017357846)*x[0]**o+(-0.681305727596)*x[0]+(-0.291763309802)*x[1]**o+(0.12465245807)*x[1]+(-0.265362499082)*x[2]**o+(-0.978509447338)*x[2]
        arg[3,0,0]=(0.622861389608)*x[0]**o+(-0.341108338962)*x[0]+(0.908588438318)*x[1]**o+(-0.583012719535)*x[1]+(0.33513654797)*x[2]**o+(-0.835513934636)*x[2]
        arg[3,0,1]=(0.164841305439)*x[0]**o+(-0.622242302911)*x[0]+(-0.0469305037163)*x[1]**o+(-0.859517953063)*x[1]+(-0.112485630662)*x[2]**o+(0.31475831421)*x[2]
        arg[3,1,0]=(-0.177913851996)*x[0]**o+(0.74607750787)*x[0]+(-0.188838634745)*x[1]**o+(-0.803617744529)*x[1]+(0.18839438068)*x[2]**o+(-0.857696671728)*x[2]
        arg[3,1,1]=(0.485172697497)*x[0]**o+(-0.554750723313)*x[0]+(-0.59838039994)*x[1]**o+(-0.83177053032)*x[1]+(0.655490277067)*x[2]**o+(0.0645924856059)*x[2]
        arg[4,0,0]=(0.900678166492)*x[0]**o+(-0.485573346705)*x[0]+(0.0108149783491)*x[1]**o+(0.982616024342)*x[1]+(-0.374371784319)*x[2]**o+(-0.825014166204)*x[2]
        arg[4,0,1]=(0.682753472393)*x[0]**o+(0.810927347906)*x[0]+(-0.128780174224)*x[1]**o+(0.514607304836)*x[1]+(0.606285011416)*x[2]**o+(-0.691191486471)*x[2]
        arg[4,1,0]=(0.0103661338625)*x[0]**o+(0.184111752707)*x[0]+(-0.270751890504)*x[1]**o+(0.207351712222)*x[1]+(-0.366280871535)*x[2]**o+(-0.638450092731)*x[2]
        arg[4,1,1]=(-0.111302974207)*x[0]**o+(-0.520252159917)*x[0]+(-0.911197545741)*x[1]**o+(-0.779732160528)*x[1]+(-0.443379349795)*x[2]**o+(-0.783540988384)*x[2]
        arg[5,0,0]=(0.319701766792)*x[0]**o+(-0.953929305858)*x[0]+(-0.783942145061)*x[1]**o+(0.687992583635)*x[1]+(0.873868267323)*x[2]**o+(-0.204836707955)*x[2]
        arg[5,0,1]=(-0.240340717622)*x[0]**o+(-0.799598459684)*x[0]+(-0.827096078861)*x[1]**o+(0.229071234102)*x[1]+(0.309197705487)*x[2]**o+(-0.286088934882)*x[2]
        arg[5,1,0]=(-0.0926450319507)*x[0]**o+(0.638985747884)*x[0]+(0.604913216138)*x[1]**o+(-0.364251815312)*x[1]+(-0.47971664069)*x[2]**o+(-0.20655528518)*x[2]
        arg[5,1,1]=(0.418341452333)*x[0]**o+(0.430002851684)*x[0]+(0.624025385599)*x[1]**o+(0.338601201629)*x[1]+(0.0900364406242)*x[2]**o+(0.193533337073)*x[2]
        ref[0,0,0,0]=o*(-0.639251981322)*x_ref[0]**(o-1)+(-0.115972267818)
        ref[0,0,0,1]=o*(0.491134645863)*x_ref[1]**(o-1)+(0.345343639029)
        ref[0,0,0,2]=o*(-0.663597762251)*x_ref[2]**(o-1)+(-0.328150609455)
        ref[0,0,1,0]=o*(-0.76280151252)*x_ref[0]**(o-1)+(0.933789965647)
        ref[0,0,1,1]=o*(-0.117386438957)*x_ref[1]**(o-1)+(0.738801880577)
        ref[0,0,1,2]=o*(-0.823759196233)*x_ref[2]**(o-1)+(-0.591225057134)
        ref[0,1,0,0]=o*(-0.515479158328)*x_ref[0]**(o-1)+(-0.498840147412)
        ref[0,1,0,1]=o*(-0.07870152457)*x_ref[1]**(o-1)+(0.240823897179)
        ref[0,1,0,2]=o*(-0.529483054724)*x_ref[2]**(o-1)+(0.957444878465)
        ref[0,1,1,0]=o*(0.659163197783)*x_ref[0]**(o-1)+(-0.987017484017)
        ref[0,1,1,1]=o*(0.316073196672)*x_ref[1]**(o-1)+(-0.858184573568)
        ref[0,1,1,2]=o*(-0.324909144593)*x_ref[2]**(o-1)+(-0.89624674398)
        ref[1,0,0,0]=o*(0.582390600104)*x_ref[0]**(o-1)+(-0.808724939916)
        ref[1,0,0,1]=o*(-0.204186916478)*x_ref[1]**(o-1)+(0.63212764696)
        ref[1,0,0,2]=o*(-0.272947627588)*x_ref[2]**(o-1)+(-0.133369751815)
        ref[1,0,1,0]=o*(-0.955892141909)*x_ref[0]**(o-1)+(0.915832405494)
        ref[1,0,1,1]=o*(-0.823428836636)*x_ref[1]**(o-1)+(-0.527176552593)
        ref[1,0,1,2]=o*(-0.439872810771)*x_ref[2]**(o-1)+(0.282431362763)
        ref[1,1,0,0]=o*(0.269192050139)*x_ref[0]**(o-1)+(0.304289733459)
        ref[1,1,0,1]=o*(-0.252137123064)*x_ref[1]**(o-1)+(0.159346881758)
        ref[1,1,0,2]=o*(-0.875713938743)*x_ref[2]**(o-1)+(-0.804681982204)
        ref[1,1,1,0]=o*(0.100159906585)*x_ref[0]**(o-1)+(0.652670710888)
        ref[1,1,1,1]=o*(-0.0471177296503)*x_ref[1]**(o-1)+(0.15344212377)
        ref[1,1,1,2]=o*(0.162356023341)*x_ref[2]**(o-1)+(0.242855769737)
        ref[2,0,0,0]=o*(-0.0214334988311)*x_ref[0]**(o-1)+(0.444654672491)
        ref[2,0,0,1]=o*(0.235478960156)*x_ref[1]**(o-1)+(0.62191559883)
        ref[2,0,0,2]=o*(0.124531773653)*x_ref[2]**(o-1)+(-0.603748942811)
        ref[2,0,1,0]=o*(-0.00696114124898)*x_ref[0]**(o-1)+(-0.350635993795)
        ref[2,0,1,1]=o*(0.766513244466)*x_ref[1]**(o-1)+(-0.99996024111)
        ref[2,0,1,2]=o*(0.976036637033)*x_ref[2]**(o-1)+(0.00703160282837)
        ref[2,1,0,0]=o*(-0.210499708873)*x_ref[0]**(o-1)+(-0.763641982769)
        ref[2,1,0,1]=o*(0.435435481771)*x_ref[1]**(o-1)+(0.0886400511888)
        ref[2,1,0,2]=o*(0.540913993886)*x_ref[2]**(o-1)+(-0.864904989707)
        ref[2,1,1,0]=o*(0.609017357846)*x_ref[0]**(o-1)+(-0.681305727596)
        ref[2,1,1,1]=o*(-0.291763309802)*x_ref[1]**(o-1)+(0.12465245807)
        ref[2,1,1,2]=o*(-0.265362499082)*x_ref[2]**(o-1)+(-0.978509447338)
        ref[3,0,0,0]=o*(0.622861389608)*x_ref[0]**(o-1)+(-0.341108338962)
        ref[3,0,0,1]=o*(0.908588438318)*x_ref[1]**(o-1)+(-0.583012719535)
        ref[3,0,0,2]=o*(0.33513654797)*x_ref[2]**(o-1)+(-0.835513934636)
        ref[3,0,1,0]=o*(0.164841305439)*x_ref[0]**(o-1)+(-0.622242302911)
        ref[3,0,1,1]=o*(-0.0469305037163)*x_ref[1]**(o-1)+(-0.859517953063)
        ref[3,0,1,2]=o*(-0.112485630662)*x_ref[2]**(o-1)+(0.31475831421)
        ref[3,1,0,0]=o*(-0.177913851996)*x_ref[0]**(o-1)+(0.74607750787)
        ref[3,1,0,1]=o*(-0.188838634745)*x_ref[1]**(o-1)+(-0.803617744529)
        ref[3,1,0,2]=o*(0.18839438068)*x_ref[2]**(o-1)+(-0.857696671728)
        ref[3,1,1,0]=o*(0.485172697497)*x_ref[0]**(o-1)+(-0.554750723313)
        ref[3,1,1,1]=o*(-0.59838039994)*x_ref[1]**(o-1)+(-0.83177053032)
        ref[3,1,1,2]=o*(0.655490277067)*x_ref[2]**(o-1)+(0.0645924856059)
        ref[4,0,0,0]=o*(0.900678166492)*x_ref[0]**(o-1)+(-0.485573346705)
        ref[4,0,0,1]=o*(0.0108149783491)*x_ref[1]**(o-1)+(0.982616024342)
        ref[4,0,0,2]=o*(-0.374371784319)*x_ref[2]**(o-1)+(-0.825014166204)
        ref[4,0,1,0]=o*(0.682753472393)*x_ref[0]**(o-1)+(0.810927347906)
        ref[4,0,1,1]=o*(-0.128780174224)*x_ref[1]**(o-1)+(0.514607304836)
        ref[4,0,1,2]=o*(0.606285011416)*x_ref[2]**(o-1)+(-0.691191486471)
        ref[4,1,0,0]=o*(0.0103661338625)*x_ref[0]**(o-1)+(0.184111752707)
        ref[4,1,0,1]=o*(-0.270751890504)*x_ref[1]**(o-1)+(0.207351712222)
        ref[4,1,0,2]=o*(-0.366280871535)*x_ref[2]**(o-1)+(-0.638450092731)
        ref[4,1,1,0]=o*(-0.111302974207)*x_ref[0]**(o-1)+(-0.520252159917)
        ref[4,1,1,1]=o*(-0.911197545741)*x_ref[1]**(o-1)+(-0.779732160528)
        ref[4,1,1,2]=o*(-0.443379349795)*x_ref[2]**(o-1)+(-0.783540988384)
        ref[5,0,0,0]=o*(0.319701766792)*x_ref[0]**(o-1)+(-0.953929305858)
        ref[5,0,0,1]=o*(-0.783942145061)*x_ref[1]**(o-1)+(0.687992583635)
        ref[5,0,0,2]=o*(0.873868267323)*x_ref[2]**(o-1)+(-0.204836707955)
        ref[5,0,1,0]=o*(-0.240340717622)*x_ref[0]**(o-1)+(-0.799598459684)
        ref[5,0,1,1]=o*(-0.827096078861)*x_ref[1]**(o-1)+(0.229071234102)
        ref[5,0,1,2]=o*(0.309197705487)*x_ref[2]**(o-1)+(-0.286088934882)
        ref[5,1,0,0]=o*(-0.0926450319507)*x_ref[0]**(o-1)+(0.638985747884)
        ref[5,1,0,1]=o*(0.604913216138)*x_ref[1]**(o-1)+(-0.364251815312)
        ref[5,1,0,2]=o*(-0.47971664069)*x_ref[2]**(o-1)+(-0.20655528518)
        ref[5,1,1,0]=o*(0.418341452333)*x_ref[0]**(o-1)+(0.430002851684)
        ref[5,1,1,1]=o*(0.624025385599)*x_ref[1]**(o-1)+(0.338601201629)
        ref[5,1,1,2]=o*(0.0900364406242)*x_ref[2]**(o-1)+(0.193533337073)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_Solution_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.828412007781)*x[0]**o+(0.392494008682)*x[0]+(0.424780018344)*x[1]**o+(-0.75790685259)*x[1]
        ref[0]=o*(0.828412007781)*x_ref[0]**(o-1)+(0.392494008682)
        ref[1]=o*(0.424780018344)*x_ref[1]**(o-1)+(-0.75790685259)
      else:
        arg=(0.0243949626425)*x[0]**o+(0.99040714323)*x[0]+(0.455478130443)*x[1]**o+(-0.917504050554)*x[1]+(-0.642882027126)*x[2]**o+(-0.0751962677977)*x[2]
        ref[0]=o*(0.0243949626425)*x_ref[0]**(o-1)+(0.99040714323)
        ref[1]=o*(0.455478130443)*x_ref[1]**(o-1)+(-0.917504050554)
        ref[2]=o*(-0.642882027126)*x_ref[2]**(o-1)+(-0.0751962677977)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_Solution_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.107613645086)*x[0]**o+(-0.942156383662)*x[0]+(0.294425370324)*x[1]**o+(-0.177019031098)*x[1]
        arg[1]=(-0.120283674295)*x[0]**o+(-0.0644985036825)*x[0]+(-0.818412208612)*x[1]**o+(-0.375335380433)*x[1]
        ref[0,0]=o*(-0.107613645086)*x_ref[0]**(o-1)+(-0.942156383662)
        ref[0,1]=o*(0.294425370324)*x_ref[1]**(o-1)+(-0.177019031098)
        ref[1,0]=o*(-0.120283674295)*x_ref[0]**(o-1)+(-0.0644985036825)
        ref[1,1]=o*(-0.818412208612)*x_ref[1]**(o-1)+(-0.375335380433)
      else:
        arg[0]=(-0.888739890088)*x[0]**o+(0.874877363684)*x[0]+(-0.104101311962)*x[1]**o+(-0.124590333446)*x[1]+(-0.311645832684)*x[2]**o+(-0.249470325769)*x[2]
        arg[1]=(0.680288601947)*x[0]**o+(0.0595361804285)*x[0]+(-0.477099840976)*x[1]**o+(-0.695340106154)*x[1]+(-0.371044873416)*x[2]**o+(-0.442877128189)*x[2]
        ref[0,0]=o*(-0.888739890088)*x_ref[0]**(o-1)+(0.874877363684)
        ref[0,1]=o*(-0.104101311962)*x_ref[1]**(o-1)+(-0.124590333446)
        ref[0,2]=o*(-0.311645832684)*x_ref[2]**(o-1)+(-0.249470325769)
        ref[1,0]=o*(0.680288601947)*x_ref[0]**(o-1)+(0.0595361804285)
        ref[1,1]=o*(-0.477099840976)*x_ref[1]**(o-1)+(-0.695340106154)
        ref[1,2]=o*(-0.371044873416)*x_ref[2]**(o-1)+(-0.442877128189)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_Solution_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.0237486351119)*x[0]**o+(0.751375149587)*x[0]+(0.0776560601509)*x[1]**o+(-0.363340006998)*x[1]
        arg[0,1]=(0.129612138219)*x[0]**o+(0.377878821946)*x[0]+(0.166470268672)*x[1]**o+(-0.0442064974104)*x[1]
        arg[0,2]=(0.692350462477)*x[0]**o+(-0.439155965586)*x[0]+(0.655583353562)*x[1]**o+(0.3753639938)*x[1]
        arg[0,3]=(-0.909698820615)*x[0]**o+(0.382887632024)*x[0]+(-0.108821308794)*x[1]**o+(0.330111344773)*x[1]
        arg[0,4]=(-0.694552793316)*x[0]**o+(-0.38920298235)*x[0]+(0.174040833682)*x[1]**o+(-0.41756882346)*x[1]
        arg[1,0]=(-0.194901264915)*x[0]**o+(-0.325546767846)*x[0]+(0.507614203136)*x[1]**o+(-0.964255763976)*x[1]
        arg[1,1]=(-0.634480793567)*x[0]**o+(0.787029027123)*x[0]+(-0.067769912764)*x[1]**o+(0.439969554037)*x[1]
        arg[1,2]=(-0.59461925815)*x[0]**o+(-0.799215148911)*x[0]+(-0.286848861016)*x[1]**o+(-0.0274325066994)*x[1]
        arg[1,3]=(0.564664310607)*x[0]**o+(-0.00188717695265)*x[0]+(0.984042638249)*x[1]**o+(-0.524339915306)*x[1]
        arg[1,4]=(-0.673190201485)*x[0]**o+(0.149574403846)*x[0]+(0.667179612419)*x[1]**o+(-0.279377079395)*x[1]
        arg[2,0]=(0.270490028978)*x[0]**o+(0.498901404478)*x[0]+(0.485707956493)*x[1]**o+(-0.340861889384)*x[1]
        arg[2,1]=(-0.785952730285)*x[0]**o+(0.514446635701)*x[0]+(0.925557655147)*x[1]**o+(0.616971745561)*x[1]
        arg[2,2]=(0.788196502068)*x[0]**o+(0.651461860731)*x[0]+(0.160564742956)*x[1]**o+(-0.681831134057)*x[1]
        arg[2,3]=(0.892258488111)*x[0]**o+(0.752252063583)*x[0]+(0.853514779349)*x[1]**o+(-0.181884189513)*x[1]
        arg[2,4]=(-0.824886296933)*x[0]**o+(0.211931432354)*x[0]+(0.924362197939)*x[1]**o+(0.036482432278)*x[1]
        arg[3,0]=(-0.0616004277583)*x[0]**o+(-0.77338193907)*x[0]+(-0.966899521855)*x[1]**o+(-0.395764830291)*x[1]
        arg[3,1]=(-0.607976110595)*x[0]**o+(-0.448875768616)*x[0]+(-0.414034580078)*x[1]**o+(0.185483819416)*x[1]
        arg[3,2]=(-0.918071244524)*x[0]**o+(0.924225885038)*x[0]+(0.418000509852)*x[1]**o+(0.431330768344)*x[1]
        arg[3,3]=(0.897504318738)*x[0]**o+(0.66265738089)*x[0]+(0.720830604485)*x[1]**o+(-0.713433930025)*x[1]
        arg[3,4]=(0.526823069245)*x[0]**o+(-0.255965317744)*x[0]+(-0.326371546564)*x[1]**o+(-0.436881501009)*x[1]
        ref[0,0,0]=o*(0.0237486351119)*x_ref[0]**(o-1)+(0.751375149587)
        ref[0,0,1]=o*(0.0776560601509)*x_ref[1]**(o-1)+(-0.363340006998)
        ref[0,1,0]=o*(0.129612138219)*x_ref[0]**(o-1)+(0.377878821946)
        ref[0,1,1]=o*(0.166470268672)*x_ref[1]**(o-1)+(-0.0442064974104)
        ref[0,2,0]=o*(0.692350462477)*x_ref[0]**(o-1)+(-0.439155965586)
        ref[0,2,1]=o*(0.655583353562)*x_ref[1]**(o-1)+(0.3753639938)
        ref[0,3,0]=o*(-0.909698820615)*x_ref[0]**(o-1)+(0.382887632024)
        ref[0,3,1]=o*(-0.108821308794)*x_ref[1]**(o-1)+(0.330111344773)
        ref[0,4,0]=o*(-0.694552793316)*x_ref[0]**(o-1)+(-0.38920298235)
        ref[0,4,1]=o*(0.174040833682)*x_ref[1]**(o-1)+(-0.41756882346)
        ref[1,0,0]=o*(-0.194901264915)*x_ref[0]**(o-1)+(-0.325546767846)
        ref[1,0,1]=o*(0.507614203136)*x_ref[1]**(o-1)+(-0.964255763976)
        ref[1,1,0]=o*(-0.634480793567)*x_ref[0]**(o-1)+(0.787029027123)
        ref[1,1,1]=o*(-0.067769912764)*x_ref[1]**(o-1)+(0.439969554037)
        ref[1,2,0]=o*(-0.59461925815)*x_ref[0]**(o-1)+(-0.799215148911)
        ref[1,2,1]=o*(-0.286848861016)*x_ref[1]**(o-1)+(-0.0274325066994)
        ref[1,3,0]=o*(0.564664310607)*x_ref[0]**(o-1)+(-0.00188717695265)
        ref[1,3,1]=o*(0.984042638249)*x_ref[1]**(o-1)+(-0.524339915306)
        ref[1,4,0]=o*(-0.673190201485)*x_ref[0]**(o-1)+(0.149574403846)
        ref[1,4,1]=o*(0.667179612419)*x_ref[1]**(o-1)+(-0.279377079395)
        ref[2,0,0]=o*(0.270490028978)*x_ref[0]**(o-1)+(0.498901404478)
        ref[2,0,1]=o*(0.485707956493)*x_ref[1]**(o-1)+(-0.340861889384)
        ref[2,1,0]=o*(-0.785952730285)*x_ref[0]**(o-1)+(0.514446635701)
        ref[2,1,1]=o*(0.925557655147)*x_ref[1]**(o-1)+(0.616971745561)
        ref[2,2,0]=o*(0.788196502068)*x_ref[0]**(o-1)+(0.651461860731)
        ref[2,2,1]=o*(0.160564742956)*x_ref[1]**(o-1)+(-0.681831134057)
        ref[2,3,0]=o*(0.892258488111)*x_ref[0]**(o-1)+(0.752252063583)
        ref[2,3,1]=o*(0.853514779349)*x_ref[1]**(o-1)+(-0.181884189513)
        ref[2,4,0]=o*(-0.824886296933)*x_ref[0]**(o-1)+(0.211931432354)
        ref[2,4,1]=o*(0.924362197939)*x_ref[1]**(o-1)+(0.036482432278)
        ref[3,0,0]=o*(-0.0616004277583)*x_ref[0]**(o-1)+(-0.77338193907)
        ref[3,0,1]=o*(-0.966899521855)*x_ref[1]**(o-1)+(-0.395764830291)
        ref[3,1,0]=o*(-0.607976110595)*x_ref[0]**(o-1)+(-0.448875768616)
        ref[3,1,1]=o*(-0.414034580078)*x_ref[1]**(o-1)+(0.185483819416)
        ref[3,2,0]=o*(-0.918071244524)*x_ref[0]**(o-1)+(0.924225885038)
        ref[3,2,1]=o*(0.418000509852)*x_ref[1]**(o-1)+(0.431330768344)
        ref[3,3,0]=o*(0.897504318738)*x_ref[0]**(o-1)+(0.66265738089)
        ref[3,3,1]=o*(0.720830604485)*x_ref[1]**(o-1)+(-0.713433930025)
        ref[3,4,0]=o*(0.526823069245)*x_ref[0]**(o-1)+(-0.255965317744)
        ref[3,4,1]=o*(-0.326371546564)*x_ref[1]**(o-1)+(-0.436881501009)
      else:
        arg[0,0]=(-0.66471814098)*x[0]**o+(0.417241001417)*x[0]+(-0.228677853733)*x[1]**o+(-0.706946716128)*x[1]+(0.977168565988)*x[2]**o+(-0.0293128613918)*x[2]
        arg[0,1]=(-0.161577085067)*x[0]**o+(0.86957667281)*x[0]+(-0.246783815579)*x[1]**o+(0.495318651008)*x[1]+(-0.500962203485)*x[2]**o+(0.903521194042)*x[2]
        arg[0,2]=(0.0764824142976)*x[0]**o+(-0.0831710166954)*x[0]+(0.538063052387)*x[1]**o+(0.472451075914)*x[1]+(-0.354702552367)*x[2]**o+(-0.820126246007)*x[2]
        arg[0,3]=(-0.286219487301)*x[0]**o+(0.561959250696)*x[0]+(-0.65668352495)*x[1]**o+(-0.276560290133)*x[1]+(-0.500866980002)*x[2]**o+(0.71188245613)*x[2]
        arg[0,4]=(-0.13704169085)*x[0]**o+(0.107954139186)*x[0]+(0.498233399424)*x[1]**o+(-0.201539549426)*x[1]+(-0.889578977028)*x[2]**o+(0.33670078332)*x[2]
        arg[1,0]=(0.411075168668)*x[0]**o+(-0.982783292982)*x[0]+(0.919652087888)*x[1]**o+(0.657917523808)*x[1]+(-0.220747064163)*x[2]**o+(-0.65863499098)*x[2]
        arg[1,1]=(0.978122893048)*x[0]**o+(-0.638310308996)*x[0]+(-0.182955841813)*x[1]**o+(0.386322350256)*x[1]+(0.0238129376001)*x[2]**o+(0.415141411589)*x[2]
        arg[1,2]=(-0.410102590894)*x[0]**o+(-0.0527422034609)*x[0]+(0.218532590216)*x[1]**o+(0.338645963161)*x[1]+(-0.559262295901)*x[2]**o+(0.00154425206109)*x[2]
        arg[1,3]=(-0.509603507286)*x[0]**o+(0.220014659177)*x[0]+(-0.850244819031)*x[1]**o+(0.851103142912)*x[1]+(0.485740207202)*x[2]**o+(0.625073715619)*x[2]
        arg[1,4]=(-0.990056784415)*x[0]**o+(0.822920800823)*x[0]+(-0.310061384265)*x[1]**o+(-0.337394401914)*x[1]+(0.139776472055)*x[2]**o+(0.615477120599)*x[2]
        arg[2,0]=(0.434733128443)*x[0]**o+(-0.23992608444)*x[0]+(0.532847685888)*x[1]**o+(0.641963266518)*x[1]+(-0.421857805771)*x[2]**o+(0.385408885641)*x[2]
        arg[2,1]=(0.502146118236)*x[0]**o+(-0.476835388064)*x[0]+(-0.676891574566)*x[1]**o+(0.713401740075)*x[1]+(0.220183315534)*x[2]**o+(-0.863327755288)*x[2]
        arg[2,2]=(-0.694614904655)*x[0]**o+(-0.881664834998)*x[0]+(0.318605270866)*x[1]**o+(0.55884249433)*x[1]+(0.286031997395)*x[2]**o+(-0.947477517785)*x[2]
        arg[2,3]=(-0.046201338948)*x[0]**o+(-0.316456331808)*x[0]+(0.968975602807)*x[1]**o+(0.549024517551)*x[1]+(-0.473260734234)*x[2]**o+(0.889462696171)*x[2]
        arg[2,4]=(0.316754513953)*x[0]**o+(0.43553288512)*x[0]+(0.962932728727)*x[1]**o+(0.887900662435)*x[1]+(-0.235820939423)*x[2]**o+(0.947057541177)*x[2]
        arg[3,0]=(-0.835914763446)*x[0]**o+(-0.488811438785)*x[0]+(-0.573283060197)*x[1]**o+(0.963078263225)*x[1]+(0.601345307969)*x[2]**o+(-0.185658285596)*x[2]
        arg[3,1]=(0.597606596118)*x[0]**o+(0.0839878651552)*x[0]+(0.0836232765815)*x[1]**o+(0.594866612681)*x[1]+(-0.361174348867)*x[2]**o+(0.701364726038)*x[2]
        arg[3,2]=(-0.172328462662)*x[0]**o+(-0.856154059105)*x[0]+(0.533657669403)*x[1]**o+(-0.305657234912)*x[1]+(0.828787414246)*x[2]**o+(0.993286809537)*x[2]
        arg[3,3]=(0.181845363409)*x[0]**o+(0.184210025662)*x[0]+(-0.538269933964)*x[1]**o+(0.731384195858)*x[1]+(0.900272670238)*x[2]**o+(-0.152731605913)*x[2]
        arg[3,4]=(0.812640675362)*x[0]**o+(-0.330495179989)*x[0]+(0.455533774723)*x[1]**o+(0.32100819695)*x[1]+(-0.332730198093)*x[2]**o+(-0.692589097458)*x[2]
        ref[0,0,0]=o*(-0.66471814098)*x_ref[0]**(o-1)+(0.417241001417)
        ref[0,0,1]=o*(-0.228677853733)*x_ref[1]**(o-1)+(-0.706946716128)
        ref[0,0,2]=o*(0.977168565988)*x_ref[2]**(o-1)+(-0.0293128613918)
        ref[0,1,0]=o*(-0.161577085067)*x_ref[0]**(o-1)+(0.86957667281)
        ref[0,1,1]=o*(-0.246783815579)*x_ref[1]**(o-1)+(0.495318651008)
        ref[0,1,2]=o*(-0.500962203485)*x_ref[2]**(o-1)+(0.903521194042)
        ref[0,2,0]=o*(0.0764824142976)*x_ref[0]**(o-1)+(-0.0831710166954)
        ref[0,2,1]=o*(0.538063052387)*x_ref[1]**(o-1)+(0.472451075914)
        ref[0,2,2]=o*(-0.354702552367)*x_ref[2]**(o-1)+(-0.820126246007)
        ref[0,3,0]=o*(-0.286219487301)*x_ref[0]**(o-1)+(0.561959250696)
        ref[0,3,1]=o*(-0.65668352495)*x_ref[1]**(o-1)+(-0.276560290133)
        ref[0,3,2]=o*(-0.500866980002)*x_ref[2]**(o-1)+(0.71188245613)
        ref[0,4,0]=o*(-0.13704169085)*x_ref[0]**(o-1)+(0.107954139186)
        ref[0,4,1]=o*(0.498233399424)*x_ref[1]**(o-1)+(-0.201539549426)
        ref[0,4,2]=o*(-0.889578977028)*x_ref[2]**(o-1)+(0.33670078332)
        ref[1,0,0]=o*(0.411075168668)*x_ref[0]**(o-1)+(-0.982783292982)
        ref[1,0,1]=o*(0.919652087888)*x_ref[1]**(o-1)+(0.657917523808)
        ref[1,0,2]=o*(-0.220747064163)*x_ref[2]**(o-1)+(-0.65863499098)
        ref[1,1,0]=o*(0.978122893048)*x_ref[0]**(o-1)+(-0.638310308996)
        ref[1,1,1]=o*(-0.182955841813)*x_ref[1]**(o-1)+(0.386322350256)
        ref[1,1,2]=o*(0.0238129376001)*x_ref[2]**(o-1)+(0.415141411589)
        ref[1,2,0]=o*(-0.410102590894)*x_ref[0]**(o-1)+(-0.0527422034609)
        ref[1,2,1]=o*(0.218532590216)*x_ref[1]**(o-1)+(0.338645963161)
        ref[1,2,2]=o*(-0.559262295901)*x_ref[2]**(o-1)+(0.00154425206109)
        ref[1,3,0]=o*(-0.509603507286)*x_ref[0]**(o-1)+(0.220014659177)
        ref[1,3,1]=o*(-0.850244819031)*x_ref[1]**(o-1)+(0.851103142912)
        ref[1,3,2]=o*(0.485740207202)*x_ref[2]**(o-1)+(0.625073715619)
        ref[1,4,0]=o*(-0.990056784415)*x_ref[0]**(o-1)+(0.822920800823)
        ref[1,4,1]=o*(-0.310061384265)*x_ref[1]**(o-1)+(-0.337394401914)
        ref[1,4,2]=o*(0.139776472055)*x_ref[2]**(o-1)+(0.615477120599)
        ref[2,0,0]=o*(0.434733128443)*x_ref[0]**(o-1)+(-0.23992608444)
        ref[2,0,1]=o*(0.532847685888)*x_ref[1]**(o-1)+(0.641963266518)
        ref[2,0,2]=o*(-0.421857805771)*x_ref[2]**(o-1)+(0.385408885641)
        ref[2,1,0]=o*(0.502146118236)*x_ref[0]**(o-1)+(-0.476835388064)
        ref[2,1,1]=o*(-0.676891574566)*x_ref[1]**(o-1)+(0.713401740075)
        ref[2,1,2]=o*(0.220183315534)*x_ref[2]**(o-1)+(-0.863327755288)
        ref[2,2,0]=o*(-0.694614904655)*x_ref[0]**(o-1)+(-0.881664834998)
        ref[2,2,1]=o*(0.318605270866)*x_ref[1]**(o-1)+(0.55884249433)
        ref[2,2,2]=o*(0.286031997395)*x_ref[2]**(o-1)+(-0.947477517785)
        ref[2,3,0]=o*(-0.046201338948)*x_ref[0]**(o-1)+(-0.316456331808)
        ref[2,3,1]=o*(0.968975602807)*x_ref[1]**(o-1)+(0.549024517551)
        ref[2,3,2]=o*(-0.473260734234)*x_ref[2]**(o-1)+(0.889462696171)
        ref[2,4,0]=o*(0.316754513953)*x_ref[0]**(o-1)+(0.43553288512)
        ref[2,4,1]=o*(0.962932728727)*x_ref[1]**(o-1)+(0.887900662435)
        ref[2,4,2]=o*(-0.235820939423)*x_ref[2]**(o-1)+(0.947057541177)
        ref[3,0,0]=o*(-0.835914763446)*x_ref[0]**(o-1)+(-0.488811438785)
        ref[3,0,1]=o*(-0.573283060197)*x_ref[1]**(o-1)+(0.963078263225)
        ref[3,0,2]=o*(0.601345307969)*x_ref[2]**(o-1)+(-0.185658285596)
        ref[3,1,0]=o*(0.597606596118)*x_ref[0]**(o-1)+(0.0839878651552)
        ref[3,1,1]=o*(0.0836232765815)*x_ref[1]**(o-1)+(0.594866612681)
        ref[3,1,2]=o*(-0.361174348867)*x_ref[2]**(o-1)+(0.701364726038)
        ref[3,2,0]=o*(-0.172328462662)*x_ref[0]**(o-1)+(-0.856154059105)
        ref[3,2,1]=o*(0.533657669403)*x_ref[1]**(o-1)+(-0.305657234912)
        ref[3,2,2]=o*(0.828787414246)*x_ref[2]**(o-1)+(0.993286809537)
        ref[3,3,0]=o*(0.181845363409)*x_ref[0]**(o-1)+(0.184210025662)
        ref[3,3,1]=o*(-0.538269933964)*x_ref[1]**(o-1)+(0.731384195858)
        ref[3,3,2]=o*(0.900272670238)*x_ref[2]**(o-1)+(-0.152731605913)
        ref[3,4,0]=o*(0.812640675362)*x_ref[0]**(o-1)+(-0.330495179989)
        ref[3,4,1]=o*(0.455533774723)*x_ref[1]**(o-1)+(0.32100819695)
        ref[3,4,2]=o*(-0.332730198093)*x_ref[2]**(o-1)+(-0.692589097458)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_Solution_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.253396883727)*x[0]**o+(0.491235248599)*x[0]+(-0.468734602938)*x[1]**o+(0.730479725967)*x[1]
        arg[0,0,1]=(0.344036208598)*x[0]**o+(-0.394302857752)*x[0]+(-0.608765577652)*x[1]**o+(-0.564065946953)*x[1]
        arg[0,1,0]=(-0.84520693538)*x[0]**o+(0.934285247148)*x[0]+(0.0841237910789)*x[1]**o+(0.455488803423)*x[1]
        arg[0,1,1]=(0.484027534363)*x[0]**o+(-0.374633264234)*x[0]+(-0.0665482018417)*x[1]**o+(0.528984445833)*x[1]
        arg[1,0,0]=(-0.470188044426)*x[0]**o+(-0.328178128746)*x[0]+(0.67571857307)*x[1]**o+(0.116985630247)*x[1]
        arg[1,0,1]=(0.986586449605)*x[0]**o+(-0.25691197068)*x[0]+(0.755235660838)*x[1]**o+(0.211004338524)*x[1]
        arg[1,1,0]=(-0.569322712396)*x[0]**o+(0.900009729013)*x[0]+(0.748881276361)*x[1]**o+(0.850464087662)*x[1]
        arg[1,1,1]=(0.625042758036)*x[0]**o+(-0.72757111333)*x[0]+(-0.986053078492)*x[1]**o+(-0.633517003425)*x[1]
        arg[2,0,0]=(-0.716192724085)*x[0]**o+(-0.23327162426)*x[0]+(0.206440629344)*x[1]**o+(0.816324161671)*x[1]
        arg[2,0,1]=(-0.803679553386)*x[0]**o+(0.445433027195)*x[0]+(-0.111465589792)*x[1]**o+(-0.109772986088)*x[1]
        arg[2,1,0]=(0.374386571048)*x[0]**o+(-0.317187375606)*x[0]+(0.133248401591)*x[1]**o+(-0.971347533424)*x[1]
        arg[2,1,1]=(-0.48476474399)*x[0]**o+(0.979379654323)*x[0]+(0.371437992948)*x[1]**o+(-0.110228097225)*x[1]
        arg[3,0,0]=(0.37507701433)*x[0]**o+(0.451375135714)*x[0]+(0.228204578568)*x[1]**o+(0.0935454426192)*x[1]
        arg[3,0,1]=(-0.322375030256)*x[0]**o+(-0.0412650889525)*x[0]+(0.23495348361)*x[1]**o+(-0.704960519241)*x[1]
        arg[3,1,0]=(-0.478413043796)*x[0]**o+(0.373637701709)*x[0]+(0.346845166424)*x[1]**o+(0.00492075723836)*x[1]
        arg[3,1,1]=(0.0753795682919)*x[0]**o+(0.51446748543)*x[0]+(-0.337079735899)*x[1]**o+(0.976335320865)*x[1]
        arg[4,0,0]=(0.173293972121)*x[0]**o+(0.78972296986)*x[0]+(-0.518624390388)*x[1]**o+(-0.587962330417)*x[1]
        arg[4,0,1]=(0.747393467297)*x[0]**o+(0.940132823774)*x[0]+(-0.292996377712)*x[1]**o+(0.980334604245)*x[1]
        arg[4,1,0]=(-0.836497499117)*x[0]**o+(0.625161912911)*x[0]+(0.978929091652)*x[1]**o+(-0.676430396031)*x[1]
        arg[4,1,1]=(-0.195907406253)*x[0]**o+(-0.388427741642)*x[0]+(0.233500679929)*x[1]**o+(0.535047073421)*x[1]
        arg[5,0,0]=(0.983591245443)*x[0]**o+(-0.356589240979)*x[0]+(0.328974933764)*x[1]**o+(-0.875152262477)*x[1]
        arg[5,0,1]=(0.901430256705)*x[0]**o+(0.74247442514)*x[0]+(0.189211012786)*x[1]**o+(0.930520401277)*x[1]
        arg[5,1,0]=(-0.319591878243)*x[0]**o+(0.174836571432)*x[0]+(0.953875085784)*x[1]**o+(-0.51684941838)*x[1]
        arg[5,1,1]=(-0.916869714992)*x[0]**o+(0.121572609124)*x[0]+(0.791850838139)*x[1]**o+(-0.588663035087)*x[1]
        ref[0,0,0,0]=o*(0.253396883727)*x_ref[0]**(o-1)+(0.491235248599)
        ref[0,0,0,1]=o*(-0.468734602938)*x_ref[1]**(o-1)+(0.730479725967)
        ref[0,0,1,0]=o*(0.344036208598)*x_ref[0]**(o-1)+(-0.394302857752)
        ref[0,0,1,1]=o*(-0.608765577652)*x_ref[1]**(o-1)+(-0.564065946953)
        ref[0,1,0,0]=o*(-0.84520693538)*x_ref[0]**(o-1)+(0.934285247148)
        ref[0,1,0,1]=o*(0.0841237910789)*x_ref[1]**(o-1)+(0.455488803423)
        ref[0,1,1,0]=o*(0.484027534363)*x_ref[0]**(o-1)+(-0.374633264234)
        ref[0,1,1,1]=o*(-0.0665482018417)*x_ref[1]**(o-1)+(0.528984445833)
        ref[1,0,0,0]=o*(-0.470188044426)*x_ref[0]**(o-1)+(-0.328178128746)
        ref[1,0,0,1]=o*(0.67571857307)*x_ref[1]**(o-1)+(0.116985630247)
        ref[1,0,1,0]=o*(0.986586449605)*x_ref[0]**(o-1)+(-0.25691197068)
        ref[1,0,1,1]=o*(0.755235660838)*x_ref[1]**(o-1)+(0.211004338524)
        ref[1,1,0,0]=o*(-0.569322712396)*x_ref[0]**(o-1)+(0.900009729013)
        ref[1,1,0,1]=o*(0.748881276361)*x_ref[1]**(o-1)+(0.850464087662)
        ref[1,1,1,0]=o*(0.625042758036)*x_ref[0]**(o-1)+(-0.72757111333)
        ref[1,1,1,1]=o*(-0.986053078492)*x_ref[1]**(o-1)+(-0.633517003425)
        ref[2,0,0,0]=o*(-0.716192724085)*x_ref[0]**(o-1)+(-0.23327162426)
        ref[2,0,0,1]=o*(0.206440629344)*x_ref[1]**(o-1)+(0.816324161671)
        ref[2,0,1,0]=o*(-0.803679553386)*x_ref[0]**(o-1)+(0.445433027195)
        ref[2,0,1,1]=o*(-0.111465589792)*x_ref[1]**(o-1)+(-0.109772986088)
        ref[2,1,0,0]=o*(0.374386571048)*x_ref[0]**(o-1)+(-0.317187375606)
        ref[2,1,0,1]=o*(0.133248401591)*x_ref[1]**(o-1)+(-0.971347533424)
        ref[2,1,1,0]=o*(-0.48476474399)*x_ref[0]**(o-1)+(0.979379654323)
        ref[2,1,1,1]=o*(0.371437992948)*x_ref[1]**(o-1)+(-0.110228097225)
        ref[3,0,0,0]=o*(0.37507701433)*x_ref[0]**(o-1)+(0.451375135714)
        ref[3,0,0,1]=o*(0.228204578568)*x_ref[1]**(o-1)+(0.0935454426192)
        ref[3,0,1,0]=o*(-0.322375030256)*x_ref[0]**(o-1)+(-0.0412650889525)
        ref[3,0,1,1]=o*(0.23495348361)*x_ref[1]**(o-1)+(-0.704960519241)
        ref[3,1,0,0]=o*(-0.478413043796)*x_ref[0]**(o-1)+(0.373637701709)
        ref[3,1,0,1]=o*(0.346845166424)*x_ref[1]**(o-1)+(0.00492075723836)
        ref[3,1,1,0]=o*(0.0753795682919)*x_ref[0]**(o-1)+(0.51446748543)
        ref[3,1,1,1]=o*(-0.337079735899)*x_ref[1]**(o-1)+(0.976335320865)
        ref[4,0,0,0]=o*(0.173293972121)*x_ref[0]**(o-1)+(0.78972296986)
        ref[4,0,0,1]=o*(-0.518624390388)*x_ref[1]**(o-1)+(-0.587962330417)
        ref[4,0,1,0]=o*(0.747393467297)*x_ref[0]**(o-1)+(0.940132823774)
        ref[4,0,1,1]=o*(-0.292996377712)*x_ref[1]**(o-1)+(0.980334604245)
        ref[4,1,0,0]=o*(-0.836497499117)*x_ref[0]**(o-1)+(0.625161912911)
        ref[4,1,0,1]=o*(0.978929091652)*x_ref[1]**(o-1)+(-0.676430396031)
        ref[4,1,1,0]=o*(-0.195907406253)*x_ref[0]**(o-1)+(-0.388427741642)
        ref[4,1,1,1]=o*(0.233500679929)*x_ref[1]**(o-1)+(0.535047073421)
        ref[5,0,0,0]=o*(0.983591245443)*x_ref[0]**(o-1)+(-0.356589240979)
        ref[5,0,0,1]=o*(0.328974933764)*x_ref[1]**(o-1)+(-0.875152262477)
        ref[5,0,1,0]=o*(0.901430256705)*x_ref[0]**(o-1)+(0.74247442514)
        ref[5,0,1,1]=o*(0.189211012786)*x_ref[1]**(o-1)+(0.930520401277)
        ref[5,1,0,0]=o*(-0.319591878243)*x_ref[0]**(o-1)+(0.174836571432)
        ref[5,1,0,1]=o*(0.953875085784)*x_ref[1]**(o-1)+(-0.51684941838)
        ref[5,1,1,0]=o*(-0.916869714992)*x_ref[0]**(o-1)+(0.121572609124)
        ref[5,1,1,1]=o*(0.791850838139)*x_ref[1]**(o-1)+(-0.588663035087)
      else:
        arg[0,0,0]=(-0.513971266383)*x[0]**o+(-0.117127749421)*x[0]+(-0.551292004692)*x[1]**o+(-0.268920693702)*x[1]+(0.0872538366694)*x[2]**o+(0.855491592762)*x[2]
        arg[0,0,1]=(0.156425207213)*x[0]**o+(-0.299808724719)*x[0]+(0.827383156334)*x[1]**o+(-0.594506082498)*x[1]+(-0.289589281068)*x[2]**o+(0.559159349352)*x[2]
        arg[0,1,0]=(-0.925822763448)*x[0]**o+(0.518052108584)*x[0]+(-0.774761536373)*x[1]**o+(-0.0329613155479)*x[1]+(0.957805160206)*x[2]**o+(0.0606445544683)*x[2]
        arg[0,1,1]=(-0.337663244379)*x[0]**o+(-0.634947360886)*x[0]+(0.232117517887)*x[1]**o+(-0.534317817241)*x[1]+(0.554880796818)*x[2]**o+(-0.3101266685)*x[2]
        arg[1,0,0]=(-0.888548906608)*x[0]**o+(0.517747675888)*x[0]+(-0.614381113253)*x[1]**o+(-0.757891891198)*x[1]+(0.618261909378)*x[2]**o+(-0.0853347346644)*x[2]
        arg[1,0,1]=(-0.686920109418)*x[0]**o+(0.000673193523604)*x[0]+(0.251800447271)*x[1]**o+(-0.105619412646)*x[1]+(0.00990847148242)*x[2]**o+(0.861798595402)*x[2]
        arg[1,1,0]=(0.170484209485)*x[0]**o+(-0.620257643847)*x[0]+(0.057743430303)*x[1]**o+(0.0640298192279)*x[1]+(0.0757100493935)*x[2]**o+(-0.414863410391)*x[2]
        arg[1,1,1]=(-0.714807290039)*x[0]**o+(-0.562420622765)*x[0]+(0.788769732877)*x[1]**o+(-0.472413621829)*x[1]+(-0.466159435785)*x[2]**o+(-0.476510555737)*x[2]
        arg[2,0,0]=(0.875340499144)*x[0]**o+(-0.598283854175)*x[0]+(0.206593884126)*x[1]**o+(-0.987568956833)*x[1]+(0.666527095968)*x[2]**o+(-0.352753155564)*x[2]
        arg[2,0,1]=(0.677005187408)*x[0]**o+(0.760176905882)*x[0]+(-0.0863648949101)*x[1]**o+(-0.607384058071)*x[1]+(-0.117434466077)*x[2]**o+(-0.670292588633)*x[2]
        arg[2,1,0]=(0.888629811684)*x[0]**o+(0.35826389662)*x[0]+(0.00390435387028)*x[1]**o+(-0.0937518531166)*x[1]+(-0.365569888759)*x[2]**o+(0.380035526283)*x[2]
        arg[2,1,1]=(0.138275666033)*x[0]**o+(-0.437710809963)*x[0]+(0.773451227027)*x[1]**o+(0.834016138485)*x[1]+(0.847339165612)*x[2]**o+(-0.417687340903)*x[2]
        arg[3,0,0]=(-0.298714836243)*x[0]**o+(-0.268961434956)*x[0]+(0.232096942083)*x[1]**o+(-0.455570990604)*x[1]+(-0.294542554349)*x[2]**o+(-0.37112817156)*x[2]
        arg[3,0,1]=(0.0903365333358)*x[0]**o+(0.338265717129)*x[0]+(-0.755237281599)*x[1]**o+(-0.608535543682)*x[1]+(0.812136572153)*x[2]**o+(-0.146674314466)*x[2]
        arg[3,1,0]=(0.159422014459)*x[0]**o+(0.899761817668)*x[0]+(0.467507279621)*x[1]**o+(0.47028740641)*x[1]+(-0.905453092566)*x[2]**o+(0.325443171334)*x[2]
        arg[3,1,1]=(0.706113787474)*x[0]**o+(0.411358047385)*x[0]+(0.710178001185)*x[1]**o+(-0.248170072927)*x[1]+(0.2228478152)*x[2]**o+(-0.6845238748)*x[2]
        arg[4,0,0]=(-0.810987300482)*x[0]**o+(-0.20061683347)*x[0]+(-0.362480203096)*x[1]**o+(0.23493442788)*x[1]+(-0.0262576869803)*x[2]**o+(0.40114062378)*x[2]
        arg[4,0,1]=(-0.0377982138529)*x[0]**o+(-0.844585963856)*x[0]+(-0.94360590097)*x[1]**o+(-0.775666510211)*x[1]+(-0.484314539768)*x[2]**o+(0.472661289406)*x[2]
        arg[4,1,0]=(0.579937359489)*x[0]**o+(0.384020744083)*x[0]+(-0.303096661906)*x[1]**o+(-0.0640353004314)*x[1]+(0.693425296881)*x[2]**o+(0.410980796125)*x[2]
        arg[4,1,1]=(0.152139667717)*x[0]**o+(-0.49610371397)*x[0]+(-0.884008900347)*x[1]**o+(-0.364570486388)*x[1]+(0.126918854811)*x[2]**o+(0.409453971802)*x[2]
        arg[5,0,0]=(0.409834559571)*x[0]**o+(-0.347724104753)*x[0]+(-0.263327417846)*x[1]**o+(0.18147743546)*x[1]+(-0.399848371177)*x[2]**o+(0.928317631431)*x[2]
        arg[5,0,1]=(-0.686702422025)*x[0]**o+(-0.918530462662)*x[0]+(0.478936304167)*x[1]**o+(-0.641691106861)*x[1]+(-0.582780732375)*x[2]**o+(0.30255037752)*x[2]
        arg[5,1,0]=(0.50568688101)*x[0]**o+(0.378682510558)*x[0]+(0.59801824354)*x[1]**o+(-0.316028163905)*x[1]+(-0.554966057648)*x[2]**o+(0.637313154931)*x[2]
        arg[5,1,1]=(0.300318472719)*x[0]**o+(-0.794010312167)*x[0]+(-0.314402790698)*x[1]**o+(0.991870648498)*x[1]+(0.949603731589)*x[2]**o+(-0.302054056398)*x[2]
        ref[0,0,0,0]=o*(-0.513971266383)*x_ref[0]**(o-1)+(-0.117127749421)
        ref[0,0,0,1]=o*(-0.551292004692)*x_ref[1]**(o-1)+(-0.268920693702)
        ref[0,0,0,2]=o*(0.0872538366694)*x_ref[2]**(o-1)+(0.855491592762)
        ref[0,0,1,0]=o*(0.156425207213)*x_ref[0]**(o-1)+(-0.299808724719)
        ref[0,0,1,1]=o*(0.827383156334)*x_ref[1]**(o-1)+(-0.594506082498)
        ref[0,0,1,2]=o*(-0.289589281068)*x_ref[2]**(o-1)+(0.559159349352)
        ref[0,1,0,0]=o*(-0.925822763448)*x_ref[0]**(o-1)+(0.518052108584)
        ref[0,1,0,1]=o*(-0.774761536373)*x_ref[1]**(o-1)+(-0.0329613155479)
        ref[0,1,0,2]=o*(0.957805160206)*x_ref[2]**(o-1)+(0.0606445544683)
        ref[0,1,1,0]=o*(-0.337663244379)*x_ref[0]**(o-1)+(-0.634947360886)
        ref[0,1,1,1]=o*(0.232117517887)*x_ref[1]**(o-1)+(-0.534317817241)
        ref[0,1,1,2]=o*(0.554880796818)*x_ref[2]**(o-1)+(-0.3101266685)
        ref[1,0,0,0]=o*(-0.888548906608)*x_ref[0]**(o-1)+(0.517747675888)
        ref[1,0,0,1]=o*(-0.614381113253)*x_ref[1]**(o-1)+(-0.757891891198)
        ref[1,0,0,2]=o*(0.618261909378)*x_ref[2]**(o-1)+(-0.0853347346644)
        ref[1,0,1,0]=o*(-0.686920109418)*x_ref[0]**(o-1)+(0.000673193523604)
        ref[1,0,1,1]=o*(0.251800447271)*x_ref[1]**(o-1)+(-0.105619412646)
        ref[1,0,1,2]=o*(0.00990847148242)*x_ref[2]**(o-1)+(0.861798595402)
        ref[1,1,0,0]=o*(0.170484209485)*x_ref[0]**(o-1)+(-0.620257643847)
        ref[1,1,0,1]=o*(0.057743430303)*x_ref[1]**(o-1)+(0.0640298192279)
        ref[1,1,0,2]=o*(0.0757100493935)*x_ref[2]**(o-1)+(-0.414863410391)
        ref[1,1,1,0]=o*(-0.714807290039)*x_ref[0]**(o-1)+(-0.562420622765)
        ref[1,1,1,1]=o*(0.788769732877)*x_ref[1]**(o-1)+(-0.472413621829)
        ref[1,1,1,2]=o*(-0.466159435785)*x_ref[2]**(o-1)+(-0.476510555737)
        ref[2,0,0,0]=o*(0.875340499144)*x_ref[0]**(o-1)+(-0.598283854175)
        ref[2,0,0,1]=o*(0.206593884126)*x_ref[1]**(o-1)+(-0.987568956833)
        ref[2,0,0,2]=o*(0.666527095968)*x_ref[2]**(o-1)+(-0.352753155564)
        ref[2,0,1,0]=o*(0.677005187408)*x_ref[0]**(o-1)+(0.760176905882)
        ref[2,0,1,1]=o*(-0.0863648949101)*x_ref[1]**(o-1)+(-0.607384058071)
        ref[2,0,1,2]=o*(-0.117434466077)*x_ref[2]**(o-1)+(-0.670292588633)
        ref[2,1,0,0]=o*(0.888629811684)*x_ref[0]**(o-1)+(0.35826389662)
        ref[2,1,0,1]=o*(0.00390435387028)*x_ref[1]**(o-1)+(-0.0937518531166)
        ref[2,1,0,2]=o*(-0.365569888759)*x_ref[2]**(o-1)+(0.380035526283)
        ref[2,1,1,0]=o*(0.138275666033)*x_ref[0]**(o-1)+(-0.437710809963)
        ref[2,1,1,1]=o*(0.773451227027)*x_ref[1]**(o-1)+(0.834016138485)
        ref[2,1,1,2]=o*(0.847339165612)*x_ref[2]**(o-1)+(-0.417687340903)
        ref[3,0,0,0]=o*(-0.298714836243)*x_ref[0]**(o-1)+(-0.268961434956)
        ref[3,0,0,1]=o*(0.232096942083)*x_ref[1]**(o-1)+(-0.455570990604)
        ref[3,0,0,2]=o*(-0.294542554349)*x_ref[2]**(o-1)+(-0.37112817156)
        ref[3,0,1,0]=o*(0.0903365333358)*x_ref[0]**(o-1)+(0.338265717129)
        ref[3,0,1,1]=o*(-0.755237281599)*x_ref[1]**(o-1)+(-0.608535543682)
        ref[3,0,1,2]=o*(0.812136572153)*x_ref[2]**(o-1)+(-0.146674314466)
        ref[3,1,0,0]=o*(0.159422014459)*x_ref[0]**(o-1)+(0.899761817668)
        ref[3,1,0,1]=o*(0.467507279621)*x_ref[1]**(o-1)+(0.47028740641)
        ref[3,1,0,2]=o*(-0.905453092566)*x_ref[2]**(o-1)+(0.325443171334)
        ref[3,1,1,0]=o*(0.706113787474)*x_ref[0]**(o-1)+(0.411358047385)
        ref[3,1,1,1]=o*(0.710178001185)*x_ref[1]**(o-1)+(-0.248170072927)
        ref[3,1,1,2]=o*(0.2228478152)*x_ref[2]**(o-1)+(-0.6845238748)
        ref[4,0,0,0]=o*(-0.810987300482)*x_ref[0]**(o-1)+(-0.20061683347)
        ref[4,0,0,1]=o*(-0.362480203096)*x_ref[1]**(o-1)+(0.23493442788)
        ref[4,0,0,2]=o*(-0.0262576869803)*x_ref[2]**(o-1)+(0.40114062378)
        ref[4,0,1,0]=o*(-0.0377982138529)*x_ref[0]**(o-1)+(-0.844585963856)
        ref[4,0,1,1]=o*(-0.94360590097)*x_ref[1]**(o-1)+(-0.775666510211)
        ref[4,0,1,2]=o*(-0.484314539768)*x_ref[2]**(o-1)+(0.472661289406)
        ref[4,1,0,0]=o*(0.579937359489)*x_ref[0]**(o-1)+(0.384020744083)
        ref[4,1,0,1]=o*(-0.303096661906)*x_ref[1]**(o-1)+(-0.0640353004314)
        ref[4,1,0,2]=o*(0.693425296881)*x_ref[2]**(o-1)+(0.410980796125)
        ref[4,1,1,0]=o*(0.152139667717)*x_ref[0]**(o-1)+(-0.49610371397)
        ref[4,1,1,1]=o*(-0.884008900347)*x_ref[1]**(o-1)+(-0.364570486388)
        ref[4,1,1,2]=o*(0.126918854811)*x_ref[2]**(o-1)+(0.409453971802)
        ref[5,0,0,0]=o*(0.409834559571)*x_ref[0]**(o-1)+(-0.347724104753)
        ref[5,0,0,1]=o*(-0.263327417846)*x_ref[1]**(o-1)+(0.18147743546)
        ref[5,0,0,2]=o*(-0.399848371177)*x_ref[2]**(o-1)+(0.928317631431)
        ref[5,0,1,0]=o*(-0.686702422025)*x_ref[0]**(o-1)+(-0.918530462662)
        ref[5,0,1,1]=o*(0.478936304167)*x_ref[1]**(o-1)+(-0.641691106861)
        ref[5,0,1,2]=o*(-0.582780732375)*x_ref[2]**(o-1)+(0.30255037752)
        ref[5,1,0,0]=o*(0.50568688101)*x_ref[0]**(o-1)+(0.378682510558)
        ref[5,1,0,1]=o*(0.59801824354)*x_ref[1]**(o-1)+(-0.316028163905)
        ref[5,1,0,2]=o*(-0.554966057648)*x_ref[2]**(o-1)+(0.637313154931)
        ref[5,1,1,0]=o*(0.300318472719)*x_ref[0]**(o-1)+(-0.794010312167)
        ref[5,1,1,1]=o*(-0.314402790698)*x_ref[1]**(o-1)+(0.991870648498)
        ref[5,1,1,2]=o*(0.949603731589)*x_ref[2]**(o-1)+(-0.302054056398)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.306947418999)*x[0]+(-0.706616694784)*x[1]
        ref[0]=(0.306947418999)
        ref[1]=(-0.706616694784)
      else:
        arg=(0.490435692707)*x[0]+(0.929248719763)*x[1]+(0.269868131113)*x[2]
        ref[0]=(0.490435692707)
        ref[1]=(0.929248719763)
        ref[2]=(0.269868131113)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.882633673888)*x[0]+(-0.790592442848)*x[1]
        arg[1]=(1.55495669126)*x[0]+(0.496011926182)*x[1]
        ref[0,0]=(-0.882633673888)
        ref[0,1]=(-0.790592442848)
        ref[1,0]=(1.55495669126)
        ref[1,1]=(0.496011926182)
      else:
        arg[0]=(0.683098278161)*x[0]+(-0.151590511238)*x[1]+(0.54461031868)*x[2]
        arg[1]=(0.59998040214)*x[0]+(-0.995466442084)*x[1]+(-1.09468062911)*x[2]
        ref[0,0]=(0.683098278161)
        ref[0,1]=(-0.151590511238)
        ref[0,2]=(0.54461031868)
        ref[1,0]=(0.59998040214)
        ref[1,1]=(-0.995466442084)
        ref[1,2]=(-1.09468062911)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.159129528541)*x[0]+(0.107835972384)*x[1]
        arg[0,1]=(-1.52388770934)*x[0]+(0.262089610596)*x[1]
        arg[0,2]=(0.505454501555)*x[0]+(-0.247061440744)*x[1]
        arg[0,3]=(-0.556075420202)*x[0]+(0.702144110506)*x[1]
        arg[0,4]=(0.810518335626)*x[0]+(-0.76077949525)*x[1]
        arg[1,0]=(-0.491052732808)*x[0]+(-0.272913467279)*x[1]
        arg[1,1]=(0.0141908236388)*x[0]+(1.10868804645)*x[1]
        arg[1,2]=(0.719028425382)*x[0]+(-0.544780305018)*x[1]
        arg[1,3]=(-1.15474920716)*x[0]+(0.888461325396)*x[1]
        arg[1,4]=(-0.842901893766)*x[0]+(-0.465916012282)*x[1]
        arg[2,0]=(1.24606386746)*x[0]+(0.817235825831)*x[1]
        arg[2,1]=(0.400861923494)*x[0]+(1.24545704982)*x[1]
        arg[2,2]=(0.656546253691)*x[0]+(-0.0269228973404)*x[1]
        arg[2,3]=(-0.590512343874)*x[0]+(-0.0654952556025)*x[1]
        arg[2,4]=(0.806342618281)*x[0]+(-0.183166204195)*x[1]
        arg[3,0]=(0.637045776935)*x[0]+(-0.512820906995)*x[1]
        arg[3,1]=(-0.596216711833)*x[0]+(-0.221853749777)*x[1]
        arg[3,2]=(0.113235739927)*x[0]+(-0.210641618992)*x[1]
        arg[3,3]=(-1.11423980423)*x[0]+(-1.46748285012)*x[1]
        arg[3,4]=(0.573260999643)*x[0]+(0.55016627172)*x[1]
        ref[0,0,0]=(-0.159129528541)
        ref[0,0,1]=(0.107835972384)
        ref[0,1,0]=(-1.52388770934)
        ref[0,1,1]=(0.262089610596)
        ref[0,2,0]=(0.505454501555)
        ref[0,2,1]=(-0.247061440744)
        ref[0,3,0]=(-0.556075420202)
        ref[0,3,1]=(0.702144110506)
        ref[0,4,0]=(0.810518335626)
        ref[0,4,1]=(-0.76077949525)
        ref[1,0,0]=(-0.491052732808)
        ref[1,0,1]=(-0.272913467279)
        ref[1,1,0]=(0.0141908236388)
        ref[1,1,1]=(1.10868804645)
        ref[1,2,0]=(0.719028425382)
        ref[1,2,1]=(-0.544780305018)
        ref[1,3,0]=(-1.15474920716)
        ref[1,3,1]=(0.888461325396)
        ref[1,4,0]=(-0.842901893766)
        ref[1,4,1]=(-0.465916012282)
        ref[2,0,0]=(1.24606386746)
        ref[2,0,1]=(0.817235825831)
        ref[2,1,0]=(0.400861923494)
        ref[2,1,1]=(1.24545704982)
        ref[2,2,0]=(0.656546253691)
        ref[2,2,1]=(-0.0269228973404)
        ref[2,3,0]=(-0.590512343874)
        ref[2,3,1]=(-0.0654952556025)
        ref[2,4,0]=(0.806342618281)
        ref[2,4,1]=(-0.183166204195)
        ref[3,0,0]=(0.637045776935)
        ref[3,0,1]=(-0.512820906995)
        ref[3,1,0]=(-0.596216711833)
        ref[3,1,1]=(-0.221853749777)
        ref[3,2,0]=(0.113235739927)
        ref[3,2,1]=(-0.210641618992)
        ref[3,3,0]=(-1.11423980423)
        ref[3,3,1]=(-1.46748285012)
        ref[3,4,0]=(0.573260999643)
        ref[3,4,1]=(0.55016627172)
      else:
        arg[0,0]=(-0.025505130975)*x[0]+(-0.128352022583)*x[1]+(-0.0645809876948)*x[2]
        arg[0,1]=(-0.974556985555)*x[0]+(0.503751568036)*x[1]+(-0.524784840232)*x[2]
        arg[0,2]=(-1.23653778531)*x[0]+(-0.73883453445)*x[1]+(-1.59259101746)*x[2]
        arg[0,3]=(-0.543656268618)*x[0]+(0.110486603472)*x[1]+(-0.636246757745)*x[2]
        arg[0,4]=(-0.566088225133)*x[0]+(-0.272577812471)*x[1]+(-0.282075405014)*x[2]
        arg[1,0]=(-0.28252305722)*x[0]+(0.497103425073)*x[1]+(0.997942367464)*x[2]
        arg[1,1]=(-0.444059543356)*x[0]+(-0.149004039444)*x[1]+(-0.871853047384)*x[2]
        arg[1,2]=(-0.610830496789)*x[0]+(0.196494475826)*x[1]+(0.0123852343883)*x[2]
        arg[1,3]=(-0.327903465165)*x[0]+(0.523560519595)*x[1]+(-0.409961704581)*x[2]
        arg[1,4]=(0.633897214746)*x[0]+(-0.9032002551)*x[1]+(0.624042996391)*x[2]
        arg[2,0]=(0.469489049636)*x[0]+(1.46158418286)*x[1]+(-0.538453674545)*x[2]
        arg[2,1]=(0.0488906165423)*x[0]+(0.483867458959)*x[1]+(0.0419643463366)*x[2]
        arg[2,2]=(-1.89716754038)*x[0]+(-0.577554008519)*x[1]+(-0.0328835832126)*x[2]
        arg[2,3]=(0.507329501469)*x[0]+(0.354939444351)*x[1]+(0.0136766029313)*x[2]
        arg[2,4]=(-0.313182267444)*x[0]+(-1.08684309178)*x[1]+(-1.39365968958)*x[2]
        arg[3,0]=(0.272322829309)*x[0]+(-0.233645276231)*x[1]+(0.491500838791)*x[2]
        arg[3,1]=(1.27603407856)*x[0]+(-0.709429989875)*x[1]+(-0.998371438244)*x[2]
        arg[3,2]=(0.637972716248)*x[0]+(-1.37896009762)*x[1]+(0.496830616786)*x[2]
        arg[3,3]=(-1.67735443481)*x[0]+(-0.741884420811)*x[1]+(1.23125805904)*x[2]
        arg[3,4]=(-0.593290763881)*x[0]+(-0.881909104701)*x[1]+(-1.0425499934)*x[2]
        ref[0,0,0]=(-0.025505130975)
        ref[0,0,1]=(-0.128352022583)
        ref[0,0,2]=(-0.0645809876948)
        ref[0,1,0]=(-0.974556985555)
        ref[0,1,1]=(0.503751568036)
        ref[0,1,2]=(-0.524784840232)
        ref[0,2,0]=(-1.23653778531)
        ref[0,2,1]=(-0.73883453445)
        ref[0,2,2]=(-1.59259101746)
        ref[0,3,0]=(-0.543656268618)
        ref[0,3,1]=(0.110486603472)
        ref[0,3,2]=(-0.636246757745)
        ref[0,4,0]=(-0.566088225133)
        ref[0,4,1]=(-0.272577812471)
        ref[0,4,2]=(-0.282075405014)
        ref[1,0,0]=(-0.28252305722)
        ref[1,0,1]=(0.497103425073)
        ref[1,0,2]=(0.997942367464)
        ref[1,1,0]=(-0.444059543356)
        ref[1,1,1]=(-0.149004039444)
        ref[1,1,2]=(-0.871853047384)
        ref[1,2,0]=(-0.610830496789)
        ref[1,2,1]=(0.196494475826)
        ref[1,2,2]=(0.0123852343883)
        ref[1,3,0]=(-0.327903465165)
        ref[1,3,1]=(0.523560519595)
        ref[1,3,2]=(-0.409961704581)
        ref[1,4,0]=(0.633897214746)
        ref[1,4,1]=(-0.9032002551)
        ref[1,4,2]=(0.624042996391)
        ref[2,0,0]=(0.469489049636)
        ref[2,0,1]=(1.46158418286)
        ref[2,0,2]=(-0.538453674545)
        ref[2,1,0]=(0.0488906165423)
        ref[2,1,1]=(0.483867458959)
        ref[2,1,2]=(0.0419643463366)
        ref[2,2,0]=(-1.89716754038)
        ref[2,2,1]=(-0.577554008519)
        ref[2,2,2]=(-0.0328835832126)
        ref[2,3,0]=(0.507329501469)
        ref[2,3,1]=(0.354939444351)
        ref[2,3,2]=(0.0136766029313)
        ref[2,4,0]=(-0.313182267444)
        ref[2,4,1]=(-1.08684309178)
        ref[2,4,2]=(-1.39365968958)
        ref[3,0,0]=(0.272322829309)
        ref[3,0,1]=(-0.233645276231)
        ref[3,0,2]=(0.491500838791)
        ref[3,1,0]=(1.27603407856)
        ref[3,1,1]=(-0.709429989875)
        ref[3,1,2]=(-0.998371438244)
        ref[3,2,0]=(0.637972716248)
        ref[3,2,1]=(-1.37896009762)
        ref[3,2,2]=(0.496830616786)
        ref[3,3,0]=(-1.67735443481)
        ref[3,3,1]=(-0.741884420811)
        ref[3,3,2]=(1.23125805904)
        ref[3,4,0]=(-0.593290763881)
        ref[3,4,1]=(-0.881909104701)
        ref[3,4,2]=(-1.0425499934)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.289868084377)*x[0]+(-1.636093365)*x[1]
        arg[0,0,1]=(0.841764493453)*x[0]+(-0.397262590373)*x[1]
        arg[0,1,0]=(0.318610649633)*x[0]+(1.59837795884)*x[1]
        arg[0,1,1]=(1.41408600658)*x[0]+(0.212012530707)*x[1]
        arg[1,0,0]=(-1.13663878903)*x[0]+(-0.535752609596)*x[1]
        arg[1,0,1]=(-0.795140440626)*x[0]+(-0.733456054749)*x[1]
        arg[1,1,0]=(-0.174874940569)*x[0]+(0.304037654801)*x[1]
        arg[1,1,1]=(0.835374268658)*x[0]+(0.645365790264)*x[1]
        arg[2,0,0]=(0.684064169293)*x[0]+(-0.580502209076)*x[1]
        arg[2,0,1]=(0.342542841673)*x[0]+(-0.620912562797)*x[1]
        arg[2,1,0]=(-0.457226288553)*x[0]+(-0.163180645461)*x[1]
        arg[2,1,1]=(-0.494067356015)*x[0]+(-0.604667785982)*x[1]
        arg[3,0,0]=(-0.224173319974)*x[0]+(-0.288566962407)*x[1]
        arg[3,0,1]=(-0.83578982053)*x[0]+(0.290663773612)*x[1]
        arg[3,1,0]=(0.700494058094)*x[0]+(0.244036699451)*x[1]
        arg[3,1,1]=(0.960756120912)*x[0]+(-0.129549309953)*x[1]
        arg[4,0,0]=(0.942292060296)*x[0]+(-0.148922577041)*x[1]
        arg[4,0,1]=(-0.363876552144)*x[0]+(0.903847614766)*x[1]
        arg[4,1,0]=(-1.91719369833)*x[0]+(0.961392595186)*x[1]
        arg[4,1,1]=(-0.141662097711)*x[0]+(-0.695061685931)*x[1]
        arg[5,0,0]=(0.416693171023)*x[0]+(1.04455773845)*x[1]
        arg[5,0,1]=(0.325327980312)*x[0]+(-1.01452792883)*x[1]
        arg[5,1,0]=(-1.43970903355)*x[0]+(-1.04382682561)*x[1]
        arg[5,1,1]=(-0.969700314248)*x[0]+(-0.286024748854)*x[1]
        ref[0,0,0,0]=(0.289868084377)
        ref[0,0,0,1]=(-1.636093365)
        ref[0,0,1,0]=(0.841764493453)
        ref[0,0,1,1]=(-0.397262590373)
        ref[0,1,0,0]=(0.318610649633)
        ref[0,1,0,1]=(1.59837795884)
        ref[0,1,1,0]=(1.41408600658)
        ref[0,1,1,1]=(0.212012530707)
        ref[1,0,0,0]=(-1.13663878903)
        ref[1,0,0,1]=(-0.535752609596)
        ref[1,0,1,0]=(-0.795140440626)
        ref[1,0,1,1]=(-0.733456054749)
        ref[1,1,0,0]=(-0.174874940569)
        ref[1,1,0,1]=(0.304037654801)
        ref[1,1,1,0]=(0.835374268658)
        ref[1,1,1,1]=(0.645365790264)
        ref[2,0,0,0]=(0.684064169293)
        ref[2,0,0,1]=(-0.580502209076)
        ref[2,0,1,0]=(0.342542841673)
        ref[2,0,1,1]=(-0.620912562797)
        ref[2,1,0,0]=(-0.457226288553)
        ref[2,1,0,1]=(-0.163180645461)
        ref[2,1,1,0]=(-0.494067356015)
        ref[2,1,1,1]=(-0.604667785982)
        ref[3,0,0,0]=(-0.224173319974)
        ref[3,0,0,1]=(-0.288566962407)
        ref[3,0,1,0]=(-0.83578982053)
        ref[3,0,1,1]=(0.290663773612)
        ref[3,1,0,0]=(0.700494058094)
        ref[3,1,0,1]=(0.244036699451)
        ref[3,1,1,0]=(0.960756120912)
        ref[3,1,1,1]=(-0.129549309953)
        ref[4,0,0,0]=(0.942292060296)
        ref[4,0,0,1]=(-0.148922577041)
        ref[4,0,1,0]=(-0.363876552144)
        ref[4,0,1,1]=(0.903847614766)
        ref[4,1,0,0]=(-1.91719369833)
        ref[4,1,0,1]=(0.961392595186)
        ref[4,1,1,0]=(-0.141662097711)
        ref[4,1,1,1]=(-0.695061685931)
        ref[5,0,0,0]=(0.416693171023)
        ref[5,0,0,1]=(1.04455773845)
        ref[5,0,1,0]=(0.325327980312)
        ref[5,0,1,1]=(-1.01452792883)
        ref[5,1,0,0]=(-1.43970903355)
        ref[5,1,0,1]=(-1.04382682561)
        ref[5,1,1,0]=(-0.969700314248)
        ref[5,1,1,1]=(-0.286024748854)
      else:
        arg[0,0,0]=(-1.52443210214)*x[0]+(-1.48020118993)*x[1]+(1.2552138343)*x[2]
        arg[0,0,1]=(-0.102900544448)*x[0]+(-1.14209442568)*x[1]+(-0.975626338621)*x[2]
        arg[0,1,0]=(0.909668629294)*x[0]+(1.29127341353)*x[1]+(-1.65221956404)*x[2]
        arg[0,1,1]=(-0.00309732749698)*x[0]+(0.0561682749786)*x[1]+(0.330039911431)*x[2]
        arg[1,0,0]=(0.966608417982)*x[0]+(-0.181702539284)*x[1]+(0.377393548129)*x[2]
        arg[1,0,1]=(0.0273333293331)*x[0]+(0.745172228806)*x[1]+(0.756302500232)*x[2]
        arg[1,1,0]=(-0.550382204645)*x[0]+(0.531191986475)*x[1]+(-1.02269665633)*x[2]
        arg[1,1,1]=(-0.928522491259)*x[0]+(-1.25241213648)*x[1]+(0.635412990933)*x[2]
        arg[2,0,0]=(-0.355234026557)*x[0]+(-1.69834232334)*x[1]+(1.17827811215)*x[2]
        arg[2,0,1]=(-0.905805855166)*x[0]+(1.4023187683)*x[1]+(-0.570168366306)*x[2]
        arg[2,1,0]=(0.550150523384)*x[0]+(-1.59933962526)*x[1]+(-0.388203251081)*x[2]
        arg[2,1,1]=(0.138470563927)*x[0]+(0.245233545344)*x[1]+(-0.293840556681)*x[2]
        arg[3,0,0]=(1.19700673733)*x[0]+(-1.04510591446)*x[1]+(1.03589008728)*x[2]
        arg[3,0,1]=(0.713948123924)*x[0]+(-1.42841558036)*x[1]+(1.07248428525)*x[2]
        arg[3,1,0]=(0.11223781122)*x[0]+(-0.00982612208261)*x[1]+(-0.263153847134)*x[2]
        arg[3,1,1]=(1.33011049869)*x[0]+(0.691016997799)*x[1]+(-0.0547947379786)*x[2]
        arg[4,0,0]=(1.0577224608)*x[0]+(-0.263034881475)*x[1]+(-0.854540775969)*x[2]
        arg[4,0,1]=(0.686363734714)*x[0]+(0.198120868875)*x[1]+(1.46326635417)*x[2]
        arg[4,1,0]=(-0.745244187008)*x[0]+(1.31766298118)*x[1]+(1.51544532829)*x[2]
        arg[4,1,1]=(0.878390723411)*x[0]+(-0.427741582253)*x[1]+(0.0324341706939)*x[2]
        arg[5,0,0]=(-0.445960121294)*x[0]+(-0.215668362178)*x[1]+(0.147632419682)*x[2]
        arg[5,0,1]=(-0.0781270285912)*x[0]+(0.782990026347)*x[1]+(-1.28046463036)*x[2]
        arg[5,1,0]=(-0.385003656622)*x[0]+(-1.02087438188)*x[1]+(-0.594357838151)*x[2]
        arg[5,1,1]=(-0.676669731714)*x[0]+(-0.28685220691)*x[1]+(0.335774697674)*x[2]
        ref[0,0,0,0]=(-1.52443210214)
        ref[0,0,0,1]=(-1.48020118993)
        ref[0,0,0,2]=(1.2552138343)
        ref[0,0,1,0]=(-0.102900544448)
        ref[0,0,1,1]=(-1.14209442568)
        ref[0,0,1,2]=(-0.975626338621)
        ref[0,1,0,0]=(0.909668629294)
        ref[0,1,0,1]=(1.29127341353)
        ref[0,1,0,2]=(-1.65221956404)
        ref[0,1,1,0]=(-0.00309732749698)
        ref[0,1,1,1]=(0.0561682749786)
        ref[0,1,1,2]=(0.330039911431)
        ref[1,0,0,0]=(0.966608417982)
        ref[1,0,0,1]=(-0.181702539284)
        ref[1,0,0,2]=(0.377393548129)
        ref[1,0,1,0]=(0.0273333293331)
        ref[1,0,1,1]=(0.745172228806)
        ref[1,0,1,2]=(0.756302500232)
        ref[1,1,0,0]=(-0.550382204645)
        ref[1,1,0,1]=(0.531191986475)
        ref[1,1,0,2]=(-1.02269665633)
        ref[1,1,1,0]=(-0.928522491259)
        ref[1,1,1,1]=(-1.25241213648)
        ref[1,1,1,2]=(0.635412990933)
        ref[2,0,0,0]=(-0.355234026557)
        ref[2,0,0,1]=(-1.69834232334)
        ref[2,0,0,2]=(1.17827811215)
        ref[2,0,1,0]=(-0.905805855166)
        ref[2,0,1,1]=(1.4023187683)
        ref[2,0,1,2]=(-0.570168366306)
        ref[2,1,0,0]=(0.550150523384)
        ref[2,1,0,1]=(-1.59933962526)
        ref[2,1,0,2]=(-0.388203251081)
        ref[2,1,1,0]=(0.138470563927)
        ref[2,1,1,1]=(0.245233545344)
        ref[2,1,1,2]=(-0.293840556681)
        ref[3,0,0,0]=(1.19700673733)
        ref[3,0,0,1]=(-1.04510591446)
        ref[3,0,0,2]=(1.03589008728)
        ref[3,0,1,0]=(0.713948123924)
        ref[3,0,1,1]=(-1.42841558036)
        ref[3,0,1,2]=(1.07248428525)
        ref[3,1,0,0]=(0.11223781122)
        ref[3,1,0,1]=(-0.00982612208261)
        ref[3,1,0,2]=(-0.263153847134)
        ref[3,1,1,0]=(1.33011049869)
        ref[3,1,1,1]=(0.691016997799)
        ref[3,1,1,2]=(-0.0547947379786)
        ref[4,0,0,0]=(1.0577224608)
        ref[4,0,0,1]=(-0.263034881475)
        ref[4,0,0,2]=(-0.854540775969)
        ref[4,0,1,0]=(0.686363734714)
        ref[4,0,1,1]=(0.198120868875)
        ref[4,0,1,2]=(1.46326635417)
        ref[4,1,0,0]=(-0.745244187008)
        ref[4,1,0,1]=(1.31766298118)
        ref[4,1,0,2]=(1.51544532829)
        ref[4,1,1,0]=(0.878390723411)
        ref[4,1,1,1]=(-0.427741582253)
        ref[4,1,1,2]=(0.0324341706939)
        ref[5,0,0,0]=(-0.445960121294)
        ref[5,0,0,1]=(-0.215668362178)
        ref[5,0,0,2]=(0.147632419682)
        ref[5,0,1,0]=(-0.0781270285912)
        ref[5,0,1,1]=(0.782990026347)
        ref[5,0,1,2]=(-1.28046463036)
        ref[5,1,0,0]=(-0.385003656622)
        ref[5,1,0,1]=(-1.02087438188)
        ref[5,1,0,2]=(-0.594357838151)
        ref[5,1,1,0]=(-0.676669731714)
        ref[5,1,1,1]=(-0.28685220691)
        ref[5,1,1,2]=(0.335774697674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.306947418999)*x[0]+(-0.706616694784)*x[1]
        ref[0]=(0.306947418999)
        ref[1]=(-0.706616694784)
      else:
        arg=(0.490435692707)*x[0]+(0.929248719763)*x[1]+(0.269868131113)*x[2]
        ref[0]=(0.490435692707)
        ref[1]=(0.929248719763)
        ref[2]=(0.269868131113)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.882633673888)*x[0]+(-0.790592442848)*x[1]
        arg[1]=(1.55495669126)*x[0]+(0.496011926182)*x[1]
        ref[0,0]=(-0.882633673888)
        ref[0,1]=(-0.790592442848)
        ref[1,0]=(1.55495669126)
        ref[1,1]=(0.496011926182)
      else:
        arg[0]=(0.683098278161)*x[0]+(-0.151590511238)*x[1]+(0.54461031868)*x[2]
        arg[1]=(0.59998040214)*x[0]+(-0.995466442084)*x[1]+(-1.09468062911)*x[2]
        ref[0,0]=(0.683098278161)
        ref[0,1]=(-0.151590511238)
        ref[0,2]=(0.54461031868)
        ref[1,0]=(0.59998040214)
        ref[1,1]=(-0.995466442084)
        ref[1,2]=(-1.09468062911)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.159129528541)*x[0]+(0.107835972384)*x[1]
        arg[0,1]=(-1.52388770934)*x[0]+(0.262089610596)*x[1]
        arg[0,2]=(0.505454501555)*x[0]+(-0.247061440744)*x[1]
        arg[0,3]=(-0.556075420202)*x[0]+(0.702144110506)*x[1]
        arg[0,4]=(0.810518335626)*x[0]+(-0.76077949525)*x[1]
        arg[1,0]=(-0.491052732808)*x[0]+(-0.272913467279)*x[1]
        arg[1,1]=(0.0141908236388)*x[0]+(1.10868804645)*x[1]
        arg[1,2]=(0.719028425382)*x[0]+(-0.544780305018)*x[1]
        arg[1,3]=(-1.15474920716)*x[0]+(0.888461325396)*x[1]
        arg[1,4]=(-0.842901893766)*x[0]+(-0.465916012282)*x[1]
        arg[2,0]=(1.24606386746)*x[0]+(0.817235825831)*x[1]
        arg[2,1]=(0.400861923494)*x[0]+(1.24545704982)*x[1]
        arg[2,2]=(0.656546253691)*x[0]+(-0.0269228973404)*x[1]
        arg[2,3]=(-0.590512343874)*x[0]+(-0.0654952556025)*x[1]
        arg[2,4]=(0.806342618281)*x[0]+(-0.183166204195)*x[1]
        arg[3,0]=(0.637045776935)*x[0]+(-0.512820906995)*x[1]
        arg[3,1]=(-0.596216711833)*x[0]+(-0.221853749777)*x[1]
        arg[3,2]=(0.113235739927)*x[0]+(-0.210641618992)*x[1]
        arg[3,3]=(-1.11423980423)*x[0]+(-1.46748285012)*x[1]
        arg[3,4]=(0.573260999643)*x[0]+(0.55016627172)*x[1]
        ref[0,0,0]=(-0.159129528541)
        ref[0,0,1]=(0.107835972384)
        ref[0,1,0]=(-1.52388770934)
        ref[0,1,1]=(0.262089610596)
        ref[0,2,0]=(0.505454501555)
        ref[0,2,1]=(-0.247061440744)
        ref[0,3,0]=(-0.556075420202)
        ref[0,3,1]=(0.702144110506)
        ref[0,4,0]=(0.810518335626)
        ref[0,4,1]=(-0.76077949525)
        ref[1,0,0]=(-0.491052732808)
        ref[1,0,1]=(-0.272913467279)
        ref[1,1,0]=(0.0141908236388)
        ref[1,1,1]=(1.10868804645)
        ref[1,2,0]=(0.719028425382)
        ref[1,2,1]=(-0.544780305018)
        ref[1,3,0]=(-1.15474920716)
        ref[1,3,1]=(0.888461325396)
        ref[1,4,0]=(-0.842901893766)
        ref[1,4,1]=(-0.465916012282)
        ref[2,0,0]=(1.24606386746)
        ref[2,0,1]=(0.817235825831)
        ref[2,1,0]=(0.400861923494)
        ref[2,1,1]=(1.24545704982)
        ref[2,2,0]=(0.656546253691)
        ref[2,2,1]=(-0.0269228973404)
        ref[2,3,0]=(-0.590512343874)
        ref[2,3,1]=(-0.0654952556025)
        ref[2,4,0]=(0.806342618281)
        ref[2,4,1]=(-0.183166204195)
        ref[3,0,0]=(0.637045776935)
        ref[3,0,1]=(-0.512820906995)
        ref[3,1,0]=(-0.596216711833)
        ref[3,1,1]=(-0.221853749777)
        ref[3,2,0]=(0.113235739927)
        ref[3,2,1]=(-0.210641618992)
        ref[3,3,0]=(-1.11423980423)
        ref[3,3,1]=(-1.46748285012)
        ref[3,4,0]=(0.573260999643)
        ref[3,4,1]=(0.55016627172)
      else:
        arg[0,0]=(-0.025505130975)*x[0]+(-0.128352022583)*x[1]+(-0.0645809876948)*x[2]
        arg[0,1]=(-0.974556985555)*x[0]+(0.503751568036)*x[1]+(-0.524784840232)*x[2]
        arg[0,2]=(-1.23653778531)*x[0]+(-0.73883453445)*x[1]+(-1.59259101746)*x[2]
        arg[0,3]=(-0.543656268618)*x[0]+(0.110486603472)*x[1]+(-0.636246757745)*x[2]
        arg[0,4]=(-0.566088225133)*x[0]+(-0.272577812471)*x[1]+(-0.282075405014)*x[2]
        arg[1,0]=(-0.28252305722)*x[0]+(0.497103425073)*x[1]+(0.997942367464)*x[2]
        arg[1,1]=(-0.444059543356)*x[0]+(-0.149004039444)*x[1]+(-0.871853047384)*x[2]
        arg[1,2]=(-0.610830496789)*x[0]+(0.196494475826)*x[1]+(0.0123852343883)*x[2]
        arg[1,3]=(-0.327903465165)*x[0]+(0.523560519595)*x[1]+(-0.409961704581)*x[2]
        arg[1,4]=(0.633897214746)*x[0]+(-0.9032002551)*x[1]+(0.624042996391)*x[2]
        arg[2,0]=(0.469489049636)*x[0]+(1.46158418286)*x[1]+(-0.538453674545)*x[2]
        arg[2,1]=(0.0488906165423)*x[0]+(0.483867458959)*x[1]+(0.0419643463366)*x[2]
        arg[2,2]=(-1.89716754038)*x[0]+(-0.577554008519)*x[1]+(-0.0328835832126)*x[2]
        arg[2,3]=(0.507329501469)*x[0]+(0.354939444351)*x[1]+(0.0136766029313)*x[2]
        arg[2,4]=(-0.313182267444)*x[0]+(-1.08684309178)*x[1]+(-1.39365968958)*x[2]
        arg[3,0]=(0.272322829309)*x[0]+(-0.233645276231)*x[1]+(0.491500838791)*x[2]
        arg[3,1]=(1.27603407856)*x[0]+(-0.709429989875)*x[1]+(-0.998371438244)*x[2]
        arg[3,2]=(0.637972716248)*x[0]+(-1.37896009762)*x[1]+(0.496830616786)*x[2]
        arg[3,3]=(-1.67735443481)*x[0]+(-0.741884420811)*x[1]+(1.23125805904)*x[2]
        arg[3,4]=(-0.593290763881)*x[0]+(-0.881909104701)*x[1]+(-1.0425499934)*x[2]
        ref[0,0,0]=(-0.025505130975)
        ref[0,0,1]=(-0.128352022583)
        ref[0,0,2]=(-0.0645809876948)
        ref[0,1,0]=(-0.974556985555)
        ref[0,1,1]=(0.503751568036)
        ref[0,1,2]=(-0.524784840232)
        ref[0,2,0]=(-1.23653778531)
        ref[0,2,1]=(-0.73883453445)
        ref[0,2,2]=(-1.59259101746)
        ref[0,3,0]=(-0.543656268618)
        ref[0,3,1]=(0.110486603472)
        ref[0,3,2]=(-0.636246757745)
        ref[0,4,0]=(-0.566088225133)
        ref[0,4,1]=(-0.272577812471)
        ref[0,4,2]=(-0.282075405014)
        ref[1,0,0]=(-0.28252305722)
        ref[1,0,1]=(0.497103425073)
        ref[1,0,2]=(0.997942367464)
        ref[1,1,0]=(-0.444059543356)
        ref[1,1,1]=(-0.149004039444)
        ref[1,1,2]=(-0.871853047384)
        ref[1,2,0]=(-0.610830496789)
        ref[1,2,1]=(0.196494475826)
        ref[1,2,2]=(0.0123852343883)
        ref[1,3,0]=(-0.327903465165)
        ref[1,3,1]=(0.523560519595)
        ref[1,3,2]=(-0.409961704581)
        ref[1,4,0]=(0.633897214746)
        ref[1,4,1]=(-0.9032002551)
        ref[1,4,2]=(0.624042996391)
        ref[2,0,0]=(0.469489049636)
        ref[2,0,1]=(1.46158418286)
        ref[2,0,2]=(-0.538453674545)
        ref[2,1,0]=(0.0488906165423)
        ref[2,1,1]=(0.483867458959)
        ref[2,1,2]=(0.0419643463366)
        ref[2,2,0]=(-1.89716754038)
        ref[2,2,1]=(-0.577554008519)
        ref[2,2,2]=(-0.0328835832126)
        ref[2,3,0]=(0.507329501469)
        ref[2,3,1]=(0.354939444351)
        ref[2,3,2]=(0.0136766029313)
        ref[2,4,0]=(-0.313182267444)
        ref[2,4,1]=(-1.08684309178)
        ref[2,4,2]=(-1.39365968958)
        ref[3,0,0]=(0.272322829309)
        ref[3,0,1]=(-0.233645276231)
        ref[3,0,2]=(0.491500838791)
        ref[3,1,0]=(1.27603407856)
        ref[3,1,1]=(-0.709429989875)
        ref[3,1,2]=(-0.998371438244)
        ref[3,2,0]=(0.637972716248)
        ref[3,2,1]=(-1.37896009762)
        ref[3,2,2]=(0.496830616786)
        ref[3,3,0]=(-1.67735443481)
        ref[3,3,1]=(-0.741884420811)
        ref[3,3,2]=(1.23125805904)
        ref[3,4,0]=(-0.593290763881)
        ref[3,4,1]=(-0.881909104701)
        ref[3,4,2]=(-1.0425499934)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.289868084377)*x[0]+(-1.636093365)*x[1]
        arg[0,0,1]=(0.841764493453)*x[0]+(-0.397262590373)*x[1]
        arg[0,1,0]=(0.318610649633)*x[0]+(1.59837795884)*x[1]
        arg[0,1,1]=(1.41408600658)*x[0]+(0.212012530707)*x[1]
        arg[1,0,0]=(-1.13663878903)*x[0]+(-0.535752609596)*x[1]
        arg[1,0,1]=(-0.795140440626)*x[0]+(-0.733456054749)*x[1]
        arg[1,1,0]=(-0.174874940569)*x[0]+(0.304037654801)*x[1]
        arg[1,1,1]=(0.835374268658)*x[0]+(0.645365790264)*x[1]
        arg[2,0,0]=(0.684064169293)*x[0]+(-0.580502209076)*x[1]
        arg[2,0,1]=(0.342542841673)*x[0]+(-0.620912562797)*x[1]
        arg[2,1,0]=(-0.457226288553)*x[0]+(-0.163180645461)*x[1]
        arg[2,1,1]=(-0.494067356015)*x[0]+(-0.604667785982)*x[1]
        arg[3,0,0]=(-0.224173319974)*x[0]+(-0.288566962407)*x[1]
        arg[3,0,1]=(-0.83578982053)*x[0]+(0.290663773612)*x[1]
        arg[3,1,0]=(0.700494058094)*x[0]+(0.244036699451)*x[1]
        arg[3,1,1]=(0.960756120912)*x[0]+(-0.129549309953)*x[1]
        arg[4,0,0]=(0.942292060296)*x[0]+(-0.148922577041)*x[1]
        arg[4,0,1]=(-0.363876552144)*x[0]+(0.903847614766)*x[1]
        arg[4,1,0]=(-1.91719369833)*x[0]+(0.961392595186)*x[1]
        arg[4,1,1]=(-0.141662097711)*x[0]+(-0.695061685931)*x[1]
        arg[5,0,0]=(0.416693171023)*x[0]+(1.04455773845)*x[1]
        arg[5,0,1]=(0.325327980312)*x[0]+(-1.01452792883)*x[1]
        arg[5,1,0]=(-1.43970903355)*x[0]+(-1.04382682561)*x[1]
        arg[5,1,1]=(-0.969700314248)*x[0]+(-0.286024748854)*x[1]
        ref[0,0,0,0]=(0.289868084377)
        ref[0,0,0,1]=(-1.636093365)
        ref[0,0,1,0]=(0.841764493453)
        ref[0,0,1,1]=(-0.397262590373)
        ref[0,1,0,0]=(0.318610649633)
        ref[0,1,0,1]=(1.59837795884)
        ref[0,1,1,0]=(1.41408600658)
        ref[0,1,1,1]=(0.212012530707)
        ref[1,0,0,0]=(-1.13663878903)
        ref[1,0,0,1]=(-0.535752609596)
        ref[1,0,1,0]=(-0.795140440626)
        ref[1,0,1,1]=(-0.733456054749)
        ref[1,1,0,0]=(-0.174874940569)
        ref[1,1,0,1]=(0.304037654801)
        ref[1,1,1,0]=(0.835374268658)
        ref[1,1,1,1]=(0.645365790264)
        ref[2,0,0,0]=(0.684064169293)
        ref[2,0,0,1]=(-0.580502209076)
        ref[2,0,1,0]=(0.342542841673)
        ref[2,0,1,1]=(-0.620912562797)
        ref[2,1,0,0]=(-0.457226288553)
        ref[2,1,0,1]=(-0.163180645461)
        ref[2,1,1,0]=(-0.494067356015)
        ref[2,1,1,1]=(-0.604667785982)
        ref[3,0,0,0]=(-0.224173319974)
        ref[3,0,0,1]=(-0.288566962407)
        ref[3,0,1,0]=(-0.83578982053)
        ref[3,0,1,1]=(0.290663773612)
        ref[3,1,0,0]=(0.700494058094)
        ref[3,1,0,1]=(0.244036699451)
        ref[3,1,1,0]=(0.960756120912)
        ref[3,1,1,1]=(-0.129549309953)
        ref[4,0,0,0]=(0.942292060296)
        ref[4,0,0,1]=(-0.148922577041)
        ref[4,0,1,0]=(-0.363876552144)
        ref[4,0,1,1]=(0.903847614766)
        ref[4,1,0,0]=(-1.91719369833)
        ref[4,1,0,1]=(0.961392595186)
        ref[4,1,1,0]=(-0.141662097711)
        ref[4,1,1,1]=(-0.695061685931)
        ref[5,0,0,0]=(0.416693171023)
        ref[5,0,0,1]=(1.04455773845)
        ref[5,0,1,0]=(0.325327980312)
        ref[5,0,1,1]=(-1.01452792883)
        ref[5,1,0,0]=(-1.43970903355)
        ref[5,1,0,1]=(-1.04382682561)
        ref[5,1,1,0]=(-0.969700314248)
        ref[5,1,1,1]=(-0.286024748854)
      else:
        arg[0,0,0]=(-1.52443210214)*x[0]+(-1.48020118993)*x[1]+(1.2552138343)*x[2]
        arg[0,0,1]=(-0.102900544448)*x[0]+(-1.14209442568)*x[1]+(-0.975626338621)*x[2]
        arg[0,1,0]=(0.909668629294)*x[0]+(1.29127341353)*x[1]+(-1.65221956404)*x[2]
        arg[0,1,1]=(-0.00309732749698)*x[0]+(0.0561682749786)*x[1]+(0.330039911431)*x[2]
        arg[1,0,0]=(0.966608417982)*x[0]+(-0.181702539284)*x[1]+(0.377393548129)*x[2]
        arg[1,0,1]=(0.0273333293331)*x[0]+(0.745172228806)*x[1]+(0.756302500232)*x[2]
        arg[1,1,0]=(-0.550382204645)*x[0]+(0.531191986475)*x[1]+(-1.02269665633)*x[2]
        arg[1,1,1]=(-0.928522491259)*x[0]+(-1.25241213648)*x[1]+(0.635412990933)*x[2]
        arg[2,0,0]=(-0.355234026557)*x[0]+(-1.69834232334)*x[1]+(1.17827811215)*x[2]
        arg[2,0,1]=(-0.905805855166)*x[0]+(1.4023187683)*x[1]+(-0.570168366306)*x[2]
        arg[2,1,0]=(0.550150523384)*x[0]+(-1.59933962526)*x[1]+(-0.388203251081)*x[2]
        arg[2,1,1]=(0.138470563927)*x[0]+(0.245233545344)*x[1]+(-0.293840556681)*x[2]
        arg[3,0,0]=(1.19700673733)*x[0]+(-1.04510591446)*x[1]+(1.03589008728)*x[2]
        arg[3,0,1]=(0.713948123924)*x[0]+(-1.42841558036)*x[1]+(1.07248428525)*x[2]
        arg[3,1,0]=(0.11223781122)*x[0]+(-0.00982612208261)*x[1]+(-0.263153847134)*x[2]
        arg[3,1,1]=(1.33011049869)*x[0]+(0.691016997799)*x[1]+(-0.0547947379786)*x[2]
        arg[4,0,0]=(1.0577224608)*x[0]+(-0.263034881475)*x[1]+(-0.854540775969)*x[2]
        arg[4,0,1]=(0.686363734714)*x[0]+(0.198120868875)*x[1]+(1.46326635417)*x[2]
        arg[4,1,0]=(-0.745244187008)*x[0]+(1.31766298118)*x[1]+(1.51544532829)*x[2]
        arg[4,1,1]=(0.878390723411)*x[0]+(-0.427741582253)*x[1]+(0.0324341706939)*x[2]
        arg[5,0,0]=(-0.445960121294)*x[0]+(-0.215668362178)*x[1]+(0.147632419682)*x[2]
        arg[5,0,1]=(-0.0781270285912)*x[0]+(0.782990026347)*x[1]+(-1.28046463036)*x[2]
        arg[5,1,0]=(-0.385003656622)*x[0]+(-1.02087438188)*x[1]+(-0.594357838151)*x[2]
        arg[5,1,1]=(-0.676669731714)*x[0]+(-0.28685220691)*x[1]+(0.335774697674)*x[2]
        ref[0,0,0,0]=(-1.52443210214)
        ref[0,0,0,1]=(-1.48020118993)
        ref[0,0,0,2]=(1.2552138343)
        ref[0,0,1,0]=(-0.102900544448)
        ref[0,0,1,1]=(-1.14209442568)
        ref[0,0,1,2]=(-0.975626338621)
        ref[0,1,0,0]=(0.909668629294)
        ref[0,1,0,1]=(1.29127341353)
        ref[0,1,0,2]=(-1.65221956404)
        ref[0,1,1,0]=(-0.00309732749698)
        ref[0,1,1,1]=(0.0561682749786)
        ref[0,1,1,2]=(0.330039911431)
        ref[1,0,0,0]=(0.966608417982)
        ref[1,0,0,1]=(-0.181702539284)
        ref[1,0,0,2]=(0.377393548129)
        ref[1,0,1,0]=(0.0273333293331)
        ref[1,0,1,1]=(0.745172228806)
        ref[1,0,1,2]=(0.756302500232)
        ref[1,1,0,0]=(-0.550382204645)
        ref[1,1,0,1]=(0.531191986475)
        ref[1,1,0,2]=(-1.02269665633)
        ref[1,1,1,0]=(-0.928522491259)
        ref[1,1,1,1]=(-1.25241213648)
        ref[1,1,1,2]=(0.635412990933)
        ref[2,0,0,0]=(-0.355234026557)
        ref[2,0,0,1]=(-1.69834232334)
        ref[2,0,0,2]=(1.17827811215)
        ref[2,0,1,0]=(-0.905805855166)
        ref[2,0,1,1]=(1.4023187683)
        ref[2,0,1,2]=(-0.570168366306)
        ref[2,1,0,0]=(0.550150523384)
        ref[2,1,0,1]=(-1.59933962526)
        ref[2,1,0,2]=(-0.388203251081)
        ref[2,1,1,0]=(0.138470563927)
        ref[2,1,1,1]=(0.245233545344)
        ref[2,1,1,2]=(-0.293840556681)
        ref[3,0,0,0]=(1.19700673733)
        ref[3,0,0,1]=(-1.04510591446)
        ref[3,0,0,2]=(1.03589008728)
        ref[3,0,1,0]=(0.713948123924)
        ref[3,0,1,1]=(-1.42841558036)
        ref[3,0,1,2]=(1.07248428525)
        ref[3,1,0,0]=(0.11223781122)
        ref[3,1,0,1]=(-0.00982612208261)
        ref[3,1,0,2]=(-0.263153847134)
        ref[3,1,1,0]=(1.33011049869)
        ref[3,1,1,1]=(0.691016997799)
        ref[3,1,1,2]=(-0.0547947379786)
        ref[4,0,0,0]=(1.0577224608)
        ref[4,0,0,1]=(-0.263034881475)
        ref[4,0,0,2]=(-0.854540775969)
        ref[4,0,1,0]=(0.686363734714)
        ref[4,0,1,1]=(0.198120868875)
        ref[4,0,1,2]=(1.46326635417)
        ref[4,1,0,0]=(-0.745244187008)
        ref[4,1,0,1]=(1.31766298118)
        ref[4,1,0,2]=(1.51544532829)
        ref[4,1,1,0]=(0.878390723411)
        ref[4,1,1,1]=(-0.427741582253)
        ref[4,1,1,2]=(0.0324341706939)
        ref[5,0,0,0]=(-0.445960121294)
        ref[5,0,0,1]=(-0.215668362178)
        ref[5,0,0,2]=(0.147632419682)
        ref[5,0,1,0]=(-0.0781270285912)
        ref[5,0,1,1]=(0.782990026347)
        ref[5,0,1,2]=(-1.28046463036)
        ref[5,1,0,0]=(-0.385003656622)
        ref[5,1,0,1]=(-1.02087438188)
        ref[5,1,0,2]=(-0.594357838151)
        ref[5,1,1,0]=(-0.676669731714)
        ref[5,1,1,1]=(-0.28685220691)
        ref[5,1,1,2]=(0.335774697674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.729541999217)*x[0]**o+(-0.0840038177686)*x[0]+(-0.843340387423)*x[1]**o+(0.835228033597)*x[1]
        ref[0]=o*(0.729541999217)*x_ref[0]**(o-1)+(-0.0840038177686)
        ref[1]=o*(-0.843340387423)*x_ref[1]**(o-1)+(0.835228033597)
      else:
        s=(-0.842281311478)*x[0]**o+(-0.814605655879)*x[0]+(0.626451534537)*x[1]**o+(-0.933916049385)*x[1]+(0.665647219119)*x[2]**o+(-0.320674717559)*x[2]
        ref[0]=o*(-0.842281311478)*x_ref[0]**(o-1)+(-0.814605655879)
        ref[1]=o*(0.626451534537)*x_ref[1]**(o-1)+(-0.933916049385)
        ref[2]=o*(0.665647219119)*x_ref[2]**(o-1)+(-0.320674717559)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.439352379619)*x[0]**o+(0.147328198635)*x[0]+(0.247834811224)*x[1]**o+(-0.464888499837)*x[1]
        s[1]=(0.468286938618)*x[0]**o+(-0.37473605363)*x[0]+(-0.957221570057)*x[1]**o+(0.192131525732)*x[1]
        ref[0,0]=o*(0.439352379619)*x_ref[0]**(o-1)+(0.147328198635)
        ref[0,1]=o*(0.247834811224)*x_ref[1]**(o-1)+(-0.464888499837)
        ref[1,0]=o*(0.468286938618)*x_ref[0]**(o-1)+(-0.37473605363)
        ref[1,1]=o*(-0.957221570057)*x_ref[1]**(o-1)+(0.192131525732)
      else:
        s[0]=(-0.608854640717)*x[0]**o+(-0.185932852919)*x[0]+(0.845083724014)*x[1]**o+(0.674307303063)*x[1]+(-0.981577175928)*x[2]**o+(0.59694635612)*x[2]
        s[1]=(0.229588079023)*x[0]**o+(0.0251610523352)*x[0]+(0.120319529556)*x[1]**o+(-0.6189763968)*x[1]+(-0.630398590893)*x[2]**o+(-0.675207219617)*x[2]
        ref[0,0]=o*(-0.608854640717)*x_ref[0]**(o-1)+(-0.185932852919)
        ref[0,1]=o*(0.845083724014)*x_ref[1]**(o-1)+(0.674307303063)
        ref[0,2]=o*(-0.981577175928)*x_ref[2]**(o-1)+(0.59694635612)
        ref[1,0]=o*(0.229588079023)*x_ref[0]**(o-1)+(0.0251610523352)
        ref[1,1]=o*(0.120319529556)*x_ref[1]**(o-1)+(-0.6189763968)
        ref[1,2]=o*(-0.630398590893)*x_ref[2]**(o-1)+(-0.675207219617)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.047995114526)*x[0]**o+(-0.0882500144924)*x[0]+(-0.418291362721)*x[1]**o+(0.76104926053)*x[1]
        s[0,1]=(-0.862451726051)*x[0]**o+(-0.752156684628)*x[0]+(0.0539876744263)*x[1]**o+(0.0136018280853)*x[1]
        s[0,2]=(0.356154299906)*x[0]**o+(-0.119273405197)*x[0]+(-0.55202199349)*x[1]**o+(0.976767610299)*x[1]
        s[0,3]=(0.26210854969)*x[0]**o+(0.492516191788)*x[0]+(-0.187803726963)*x[1]**o+(0.331043415981)*x[1]
        s[0,4]=(0.995092330192)*x[0]**o+(0.798072438804)*x[0]+(-0.851959660211)*x[1]**o+(0.376241209687)*x[1]
        s[1,0]=(-0.819203501556)*x[0]**o+(0.158244838297)*x[0]+(-0.0854093261543)*x[1]**o+(-0.734166609629)*x[1]
        s[1,1]=(0.953095931566)*x[0]**o+(0.245967282177)*x[0]+(0.024143077003)*x[1]**o+(0.591611742804)*x[1]
        s[1,2]=(0.515929639772)*x[0]**o+(0.0656789332058)*x[0]+(-0.486542111002)*x[1]**o+(-0.35357048557)*x[1]
        s[1,3]=(0.687461558204)*x[0]**o+(-0.885516846762)*x[0]+(-0.837556168903)*x[1]**o+(-0.263453643201)*x[1]
        s[1,4]=(0.202647080146)*x[0]**o+(0.0998694048742)*x[0]+(0.963315271661)*x[1]**o+(0.0738433857728)*x[1]
        s[2,0]=(-0.431227566486)*x[0]**o+(-0.811861069902)*x[0]+(0.207189408046)*x[1]**o+(-0.843636004903)*x[1]
        s[2,1]=(-0.947866812956)*x[0]**o+(-0.779542963233)*x[0]+(0.788757232024)*x[1]**o+(-0.798061295304)*x[1]
        s[2,2]=(-0.316894005317)*x[0]**o+(-0.0766086555356)*x[0]+(-0.321413016846)*x[1]**o+(-0.586742246307)*x[1]
        s[2,3]=(-0.366402163737)*x[0]**o+(0.906207257774)*x[0]+(0.42214475079)*x[1]**o+(-0.927560770588)*x[1]
        s[2,4]=(-0.355651171461)*x[0]**o+(0.636115063757)*x[0]+(0.425219309267)*x[1]**o+(0.461939146758)*x[1]
        s[3,0]=(-0.0691712814483)*x[0]**o+(0.221346711903)*x[0]+(0.223896511845)*x[1]**o+(0.407991412109)*x[1]
        s[3,1]=(-0.437261446252)*x[0]**o+(0.0175743759667)*x[0]+(-0.937854235512)*x[1]**o+(0.814464696622)*x[1]
        s[3,2]=(-0.246296303859)*x[0]**o+(0.290577742331)*x[0]+(0.648443639897)*x[1]**o+(-0.233763313065)*x[1]
        s[3,3]=(-0.574728457793)*x[0]**o+(-0.544796515995)*x[0]+(-0.411409144577)*x[1]**o+(-0.491277131741)*x[1]
        s[3,4]=(0.894563379439)*x[0]**o+(-0.495844199423)*x[0]+(-0.580716302019)*x[1]**o+(-0.0192249355836)*x[1]
        ref[0,0,0]=o*(0.047995114526)*x_ref[0]**(o-1)+(-0.0882500144924)
        ref[0,0,1]=o*(-0.418291362721)*x_ref[1]**(o-1)+(0.76104926053)
        ref[0,1,0]=o*(-0.862451726051)*x_ref[0]**(o-1)+(-0.752156684628)
        ref[0,1,1]=o*(0.0539876744263)*x_ref[1]**(o-1)+(0.0136018280853)
        ref[0,2,0]=o*(0.356154299906)*x_ref[0]**(o-1)+(-0.119273405197)
        ref[0,2,1]=o*(-0.55202199349)*x_ref[1]**(o-1)+(0.976767610299)
        ref[0,3,0]=o*(0.26210854969)*x_ref[0]**(o-1)+(0.492516191788)
        ref[0,3,1]=o*(-0.187803726963)*x_ref[1]**(o-1)+(0.331043415981)
        ref[0,4,0]=o*(0.995092330192)*x_ref[0]**(o-1)+(0.798072438804)
        ref[0,4,1]=o*(-0.851959660211)*x_ref[1]**(o-1)+(0.376241209687)
        ref[1,0,0]=o*(-0.819203501556)*x_ref[0]**(o-1)+(0.158244838297)
        ref[1,0,1]=o*(-0.0854093261543)*x_ref[1]**(o-1)+(-0.734166609629)
        ref[1,1,0]=o*(0.953095931566)*x_ref[0]**(o-1)+(0.245967282177)
        ref[1,1,1]=o*(0.024143077003)*x_ref[1]**(o-1)+(0.591611742804)
        ref[1,2,0]=o*(0.515929639772)*x_ref[0]**(o-1)+(0.0656789332058)
        ref[1,2,1]=o*(-0.486542111002)*x_ref[1]**(o-1)+(-0.35357048557)
        ref[1,3,0]=o*(0.687461558204)*x_ref[0]**(o-1)+(-0.885516846762)
        ref[1,3,1]=o*(-0.837556168903)*x_ref[1]**(o-1)+(-0.263453643201)
        ref[1,4,0]=o*(0.202647080146)*x_ref[0]**(o-1)+(0.0998694048742)
        ref[1,4,1]=o*(0.963315271661)*x_ref[1]**(o-1)+(0.0738433857728)
        ref[2,0,0]=o*(-0.431227566486)*x_ref[0]**(o-1)+(-0.811861069902)
        ref[2,0,1]=o*(0.207189408046)*x_ref[1]**(o-1)+(-0.843636004903)
        ref[2,1,0]=o*(-0.947866812956)*x_ref[0]**(o-1)+(-0.779542963233)
        ref[2,1,1]=o*(0.788757232024)*x_ref[1]**(o-1)+(-0.798061295304)
        ref[2,2,0]=o*(-0.316894005317)*x_ref[0]**(o-1)+(-0.0766086555356)
        ref[2,2,1]=o*(-0.321413016846)*x_ref[1]**(o-1)+(-0.586742246307)
        ref[2,3,0]=o*(-0.366402163737)*x_ref[0]**(o-1)+(0.906207257774)
        ref[2,3,1]=o*(0.42214475079)*x_ref[1]**(o-1)+(-0.927560770588)
        ref[2,4,0]=o*(-0.355651171461)*x_ref[0]**(o-1)+(0.636115063757)
        ref[2,4,1]=o*(0.425219309267)*x_ref[1]**(o-1)+(0.461939146758)
        ref[3,0,0]=o*(-0.0691712814483)*x_ref[0]**(o-1)+(0.221346711903)
        ref[3,0,1]=o*(0.223896511845)*x_ref[1]**(o-1)+(0.407991412109)
        ref[3,1,0]=o*(-0.437261446252)*x_ref[0]**(o-1)+(0.0175743759667)
        ref[3,1,1]=o*(-0.937854235512)*x_ref[1]**(o-1)+(0.814464696622)
        ref[3,2,0]=o*(-0.246296303859)*x_ref[0]**(o-1)+(0.290577742331)
        ref[3,2,1]=o*(0.648443639897)*x_ref[1]**(o-1)+(-0.233763313065)
        ref[3,3,0]=o*(-0.574728457793)*x_ref[0]**(o-1)+(-0.544796515995)
        ref[3,3,1]=o*(-0.411409144577)*x_ref[1]**(o-1)+(-0.491277131741)
        ref[3,4,0]=o*(0.894563379439)*x_ref[0]**(o-1)+(-0.495844199423)
        ref[3,4,1]=o*(-0.580716302019)*x_ref[1]**(o-1)+(-0.0192249355836)
      else:
        s[0,0]=(-0.440629362137)*x[0]**o+(0.432027111899)*x[0]+(0.485612378024)*x[1]**o+(0.100195330045)*x[1]+(-0.107489415991)*x[2]**o+(-0.871419715951)*x[2]
        s[0,1]=(-0.998817226455)*x[0]**o+(-0.27667660331)*x[0]+(-0.867725500303)*x[1]**o+(0.936977636906)*x[1]+(0.525970908002)*x[2]**o+(-0.279717686524)*x[2]
        s[0,2]=(-0.422100254422)*x[0]**o+(0.554292080667)*x[0]+(-0.0240387239116)*x[1]**o+(0.899626314131)*x[1]+(-0.982259001765)*x[2]**o+(-0.171863457765)*x[2]
        s[0,3]=(-0.683481609192)*x[0]**o+(0.604674984874)*x[0]+(0.707470056964)*x[1]**o+(0.775025878411)*x[1]+(0.370390201812)*x[2]**o+(-0.59247534281)*x[2]
        s[0,4]=(-0.435112832975)*x[0]**o+(-0.448385079313)*x[0]+(-0.467168230746)*x[1]**o+(0.824665416362)*x[1]+(-0.375723812297)*x[2]**o+(0.219591728252)*x[2]
        s[1,0]=(-0.946577267946)*x[0]**o+(-0.235619974774)*x[0]+(-0.440845837468)*x[1]**o+(0.523173708473)*x[1]+(0.138768169193)*x[2]**o+(0.0800449024418)*x[2]
        s[1,1]=(0.856208167581)*x[0]**o+(-0.949001152315)*x[0]+(0.603732417952)*x[1]**o+(-0.144434407904)*x[1]+(0.771914944319)*x[2]**o+(0.295663032072)*x[2]
        s[1,2]=(-0.0748077749478)*x[0]**o+(-0.800653291967)*x[0]+(-0.723295002911)*x[1]**o+(-0.459397931378)*x[1]+(-0.680567962158)*x[2]**o+(-0.397991544426)*x[2]
        s[1,3]=(-0.160866462235)*x[0]**o+(-0.269568432554)*x[0]+(0.760053321601)*x[1]**o+(0.636712875038)*x[1]+(0.718827472409)*x[2]**o+(-0.285961313882)*x[2]
        s[1,4]=(-0.192019161878)*x[0]**o+(0.376965303206)*x[0]+(-0.196459950888)*x[1]**o+(-0.816226430578)*x[1]+(0.627440208737)*x[2]**o+(0.846061231487)*x[2]
        s[2,0]=(-0.726210859697)*x[0]**o+(-0.931805040577)*x[0]+(-0.330082314232)*x[1]**o+(-0.594241678324)*x[1]+(-0.660189952658)*x[2]**o+(-0.788716966781)*x[2]
        s[2,1]=(-0.434348461759)*x[0]**o+(0.386897792713)*x[0]+(0.0471151132814)*x[1]**o+(-0.399299502588)*x[1]+(-0.412695129309)*x[2]**o+(-0.286460071055)*x[2]
        s[2,2]=(-0.702766222291)*x[0]**o+(-0.296328795371)*x[0]+(-0.569776274584)*x[1]**o+(-0.369361432195)*x[1]+(-0.308516249924)*x[2]**o+(0.025646183412)*x[2]
        s[2,3]=(-0.286989129847)*x[0]**o+(-0.152361831045)*x[0]+(-0.909624633862)*x[1]**o+(-0.920451186077)*x[1]+(-0.375390879331)*x[2]**o+(-0.905199847241)*x[2]
        s[2,4]=(0.619800447118)*x[0]**o+(-0.27317497769)*x[0]+(-0.0136479991947)*x[1]**o+(0.141634912432)*x[1]+(-0.61405995454)*x[2]**o+(-0.505857399048)*x[2]
        s[3,0]=(-0.782882873567)*x[0]**o+(-0.7736979883)*x[0]+(-0.322501480607)*x[1]**o+(0.839395743431)*x[1]+(-0.434294403981)*x[2]**o+(0.542823987177)*x[2]
        s[3,1]=(-0.802044187937)*x[0]**o+(-0.442612586193)*x[0]+(0.436047557305)*x[1]**o+(0.497708094931)*x[1]+(0.642169738987)*x[2]**o+(0.944050667638)*x[2]
        s[3,2]=(0.557014549515)*x[0]**o+(0.491428899905)*x[0]+(0.983181819191)*x[1]**o+(0.132058074766)*x[1]+(-0.260405939651)*x[2]**o+(0.101577990826)*x[2]
        s[3,3]=(0.595058474104)*x[0]**o+(-0.822604538462)*x[0]+(-0.723088531137)*x[1]**o+(-0.313808088359)*x[1]+(0.559816139446)*x[2]**o+(-0.513647272314)*x[2]
        s[3,4]=(-0.87608199975)*x[0]**o+(0.0585891509732)*x[0]+(-0.194566629493)*x[1]**o+(0.278612435937)*x[1]+(0.170683503576)*x[2]**o+(0.936525425446)*x[2]
        ref[0,0,0]=o*(-0.440629362137)*x_ref[0]**(o-1)+(0.432027111899)
        ref[0,0,1]=o*(0.485612378024)*x_ref[1]**(o-1)+(0.100195330045)
        ref[0,0,2]=o*(-0.107489415991)*x_ref[2]**(o-1)+(-0.871419715951)
        ref[0,1,0]=o*(-0.998817226455)*x_ref[0]**(o-1)+(-0.27667660331)
        ref[0,1,1]=o*(-0.867725500303)*x_ref[1]**(o-1)+(0.936977636906)
        ref[0,1,2]=o*(0.525970908002)*x_ref[2]**(o-1)+(-0.279717686524)
        ref[0,2,0]=o*(-0.422100254422)*x_ref[0]**(o-1)+(0.554292080667)
        ref[0,2,1]=o*(-0.0240387239116)*x_ref[1]**(o-1)+(0.899626314131)
        ref[0,2,2]=o*(-0.982259001765)*x_ref[2]**(o-1)+(-0.171863457765)
        ref[0,3,0]=o*(-0.683481609192)*x_ref[0]**(o-1)+(0.604674984874)
        ref[0,3,1]=o*(0.707470056964)*x_ref[1]**(o-1)+(0.775025878411)
        ref[0,3,2]=o*(0.370390201812)*x_ref[2]**(o-1)+(-0.59247534281)
        ref[0,4,0]=o*(-0.435112832975)*x_ref[0]**(o-1)+(-0.448385079313)
        ref[0,4,1]=o*(-0.467168230746)*x_ref[1]**(o-1)+(0.824665416362)
        ref[0,4,2]=o*(-0.375723812297)*x_ref[2]**(o-1)+(0.219591728252)
        ref[1,0,0]=o*(-0.946577267946)*x_ref[0]**(o-1)+(-0.235619974774)
        ref[1,0,1]=o*(-0.440845837468)*x_ref[1]**(o-1)+(0.523173708473)
        ref[1,0,2]=o*(0.138768169193)*x_ref[2]**(o-1)+(0.0800449024418)
        ref[1,1,0]=o*(0.856208167581)*x_ref[0]**(o-1)+(-0.949001152315)
        ref[1,1,1]=o*(0.603732417952)*x_ref[1]**(o-1)+(-0.144434407904)
        ref[1,1,2]=o*(0.771914944319)*x_ref[2]**(o-1)+(0.295663032072)
        ref[1,2,0]=o*(-0.0748077749478)*x_ref[0]**(o-1)+(-0.800653291967)
        ref[1,2,1]=o*(-0.723295002911)*x_ref[1]**(o-1)+(-0.459397931378)
        ref[1,2,2]=o*(-0.680567962158)*x_ref[2]**(o-1)+(-0.397991544426)
        ref[1,3,0]=o*(-0.160866462235)*x_ref[0]**(o-1)+(-0.269568432554)
        ref[1,3,1]=o*(0.760053321601)*x_ref[1]**(o-1)+(0.636712875038)
        ref[1,3,2]=o*(0.718827472409)*x_ref[2]**(o-1)+(-0.285961313882)
        ref[1,4,0]=o*(-0.192019161878)*x_ref[0]**(o-1)+(0.376965303206)
        ref[1,4,1]=o*(-0.196459950888)*x_ref[1]**(o-1)+(-0.816226430578)
        ref[1,4,2]=o*(0.627440208737)*x_ref[2]**(o-1)+(0.846061231487)
        ref[2,0,0]=o*(-0.726210859697)*x_ref[0]**(o-1)+(-0.931805040577)
        ref[2,0,1]=o*(-0.330082314232)*x_ref[1]**(o-1)+(-0.594241678324)
        ref[2,0,2]=o*(-0.660189952658)*x_ref[2]**(o-1)+(-0.788716966781)
        ref[2,1,0]=o*(-0.434348461759)*x_ref[0]**(o-1)+(0.386897792713)
        ref[2,1,1]=o*(0.0471151132814)*x_ref[1]**(o-1)+(-0.399299502588)
        ref[2,1,2]=o*(-0.412695129309)*x_ref[2]**(o-1)+(-0.286460071055)
        ref[2,2,0]=o*(-0.702766222291)*x_ref[0]**(o-1)+(-0.296328795371)
        ref[2,2,1]=o*(-0.569776274584)*x_ref[1]**(o-1)+(-0.369361432195)
        ref[2,2,2]=o*(-0.308516249924)*x_ref[2]**(o-1)+(0.025646183412)
        ref[2,3,0]=o*(-0.286989129847)*x_ref[0]**(o-1)+(-0.152361831045)
        ref[2,3,1]=o*(-0.909624633862)*x_ref[1]**(o-1)+(-0.920451186077)
        ref[2,3,2]=o*(-0.375390879331)*x_ref[2]**(o-1)+(-0.905199847241)
        ref[2,4,0]=o*(0.619800447118)*x_ref[0]**(o-1)+(-0.27317497769)
        ref[2,4,1]=o*(-0.0136479991947)*x_ref[1]**(o-1)+(0.141634912432)
        ref[2,4,2]=o*(-0.61405995454)*x_ref[2]**(o-1)+(-0.505857399048)
        ref[3,0,0]=o*(-0.782882873567)*x_ref[0]**(o-1)+(-0.7736979883)
        ref[3,0,1]=o*(-0.322501480607)*x_ref[1]**(o-1)+(0.839395743431)
        ref[3,0,2]=o*(-0.434294403981)*x_ref[2]**(o-1)+(0.542823987177)
        ref[3,1,0]=o*(-0.802044187937)*x_ref[0]**(o-1)+(-0.442612586193)
        ref[3,1,1]=o*(0.436047557305)*x_ref[1]**(o-1)+(0.497708094931)
        ref[3,1,2]=o*(0.642169738987)*x_ref[2]**(o-1)+(0.944050667638)
        ref[3,2,0]=o*(0.557014549515)*x_ref[0]**(o-1)+(0.491428899905)
        ref[3,2,1]=o*(0.983181819191)*x_ref[1]**(o-1)+(0.132058074766)
        ref[3,2,2]=o*(-0.260405939651)*x_ref[2]**(o-1)+(0.101577990826)
        ref[3,3,0]=o*(0.595058474104)*x_ref[0]**(o-1)+(-0.822604538462)
        ref[3,3,1]=o*(-0.723088531137)*x_ref[1]**(o-1)+(-0.313808088359)
        ref[3,3,2]=o*(0.559816139446)*x_ref[2]**(o-1)+(-0.513647272314)
        ref[3,4,0]=o*(-0.87608199975)*x_ref[0]**(o-1)+(0.0585891509732)
        ref[3,4,1]=o*(-0.194566629493)*x_ref[1]**(o-1)+(0.278612435937)
        ref[3,4,2]=o*(0.170683503576)*x_ref[2]**(o-1)+(0.936525425446)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(0.212897121743)*x[0]**o+(0.270129409854)*x[0]+(0.187351601416)*x[1]**o+(0.681503494742)*x[1]
        s[0,0,1]=(0.0979281898609)*x[0]**o+(0.469261780546)*x[0]+(0.285691941306)*x[1]**o+(0.605427118235)*x[1]
        s[0,1,0]=(0.798192631806)*x[0]**o+(0.710948168289)*x[0]+(0.37314849797)*x[1]**o+(0.101332891852)*x[1]
        s[0,1,1]=(0.670619373787)*x[0]**o+(0.87986446998)*x[0]+(0.273598886571)*x[1]**o+(0.642775771189)*x[1]
        s[1,0,0]=(-0.579525230101)*x[0]**o+(-0.336522843918)*x[0]+(0.794313044824)*x[1]**o+(0.138149528558)*x[1]
        s[1,0,1]=(-0.790589960141)*x[0]**o+(-0.519541468607)*x[0]+(0.047053031477)*x[1]**o+(-0.376948328317)*x[1]
        s[1,1,0]=(-0.63652515985)*x[0]**o+(-0.796241202497)*x[0]+(0.961598412032)*x[1]**o+(0.392696716004)*x[1]
        s[1,1,1]=(-0.406921118202)*x[0]**o+(-0.117551313946)*x[0]+(-0.666279462502)*x[1]**o+(-0.0899789554615)*x[1]
        s[2,0,0]=(-0.863791811378)*x[0]**o+(0.594940266664)*x[0]+(0.872448531721)*x[1]**o+(0.874689552083)*x[1]
        s[2,0,1]=(0.243777114221)*x[0]**o+(-0.776695107306)*x[0]+(0.794794829454)*x[1]**o+(-0.318628386313)*x[1]
        s[2,1,0]=(0.0962259662597)*x[0]**o+(0.541353121994)*x[0]+(-0.672755845501)*x[1]**o+(-0.257046721225)*x[1]
        s[2,1,1]=(-0.943502231769)*x[0]**o+(-0.202236730464)*x[0]+(-0.411084369491)*x[1]**o+(0.954831689569)*x[1]
        s[3,0,0]=(0.768277437177)*x[0]**o+(0.641045872586)*x[0]+(0.456435914145)*x[1]**o+(-0.538634781112)*x[1]
        s[3,0,1]=(0.0107901515277)*x[0]**o+(0.204809134773)*x[0]+(0.632781493458)*x[1]**o+(0.813936796774)*x[1]
        s[3,1,0]=(0.187196185344)*x[0]**o+(-0.974329803149)*x[0]+(-0.131567139248)*x[1]**o+(-0.205725336514)*x[1]
        s[3,1,1]=(0.747378585446)*x[0]**o+(0.126887766052)*x[0]+(-0.660302189752)*x[1]**o+(0.227362395796)*x[1]
        s[4,0,0]=(0.822029556554)*x[0]**o+(0.742977721062)*x[0]+(0.972349490061)*x[1]**o+(0.307748083079)*x[1]
        s[4,0,1]=(-0.0347009705896)*x[0]**o+(0.506556900045)*x[0]+(-0.700430813955)*x[1]**o+(0.606413581865)*x[1]
        s[4,1,0]=(-0.69868976773)*x[0]**o+(0.447214222886)*x[0]+(0.290522797177)*x[1]**o+(0.139281488147)*x[1]
        s[4,1,1]=(0.850722242552)*x[0]**o+(0.578613819769)*x[0]+(-0.983680277805)*x[1]**o+(-0.0737991679361)*x[1]
        s[5,0,0]=(0.164023891553)*x[0]**o+(-0.398812116857)*x[0]+(0.754040638026)*x[1]**o+(0.890632418705)*x[1]
        s[5,0,1]=(0.176036561413)*x[0]**o+(0.551905509271)*x[0]+(0.584521803718)*x[1]**o+(0.359816759406)*x[1]
        s[5,1,0]=(0.728503623092)*x[0]**o+(0.448628087199)*x[0]+(0.7693890417)*x[1]**o+(-0.148028432439)*x[1]
        s[5,1,1]=(-0.974113865308)*x[0]**o+(-0.271026924938)*x[0]+(0.0918858163816)*x[1]**o+(0.441895253556)*x[1]
        ref[0,0,0,0]=o*(0.212897121743)*x_ref[0]**(o-1)+(0.270129409854)
        ref[0,0,0,1]=o*(0.187351601416)*x_ref[1]**(o-1)+(0.681503494742)
        ref[0,0,1,0]=o*(0.0979281898609)*x_ref[0]**(o-1)+(0.469261780546)
        ref[0,0,1,1]=o*(0.285691941306)*x_ref[1]**(o-1)+(0.605427118235)
        ref[0,1,0,0]=o*(0.798192631806)*x_ref[0]**(o-1)+(0.710948168289)
        ref[0,1,0,1]=o*(0.37314849797)*x_ref[1]**(o-1)+(0.101332891852)
        ref[0,1,1,0]=o*(0.670619373787)*x_ref[0]**(o-1)+(0.87986446998)
        ref[0,1,1,1]=o*(0.273598886571)*x_ref[1]**(o-1)+(0.642775771189)
        ref[1,0,0,0]=o*(-0.579525230101)*x_ref[0]**(o-1)+(-0.336522843918)
        ref[1,0,0,1]=o*(0.794313044824)*x_ref[1]**(o-1)+(0.138149528558)
        ref[1,0,1,0]=o*(-0.790589960141)*x_ref[0]**(o-1)+(-0.519541468607)
        ref[1,0,1,1]=o*(0.047053031477)*x_ref[1]**(o-1)+(-0.376948328317)
        ref[1,1,0,0]=o*(-0.63652515985)*x_ref[0]**(o-1)+(-0.796241202497)
        ref[1,1,0,1]=o*(0.961598412032)*x_ref[1]**(o-1)+(0.392696716004)
        ref[1,1,1,0]=o*(-0.406921118202)*x_ref[0]**(o-1)+(-0.117551313946)
        ref[1,1,1,1]=o*(-0.666279462502)*x_ref[1]**(o-1)+(-0.0899789554615)
        ref[2,0,0,0]=o*(-0.863791811378)*x_ref[0]**(o-1)+(0.594940266664)
        ref[2,0,0,1]=o*(0.872448531721)*x_ref[1]**(o-1)+(0.874689552083)
        ref[2,0,1,0]=o*(0.243777114221)*x_ref[0]**(o-1)+(-0.776695107306)
        ref[2,0,1,1]=o*(0.794794829454)*x_ref[1]**(o-1)+(-0.318628386313)
        ref[2,1,0,0]=o*(0.0962259662597)*x_ref[0]**(o-1)+(0.541353121994)
        ref[2,1,0,1]=o*(-0.672755845501)*x_ref[1]**(o-1)+(-0.257046721225)
        ref[2,1,1,0]=o*(-0.943502231769)*x_ref[0]**(o-1)+(-0.202236730464)
        ref[2,1,1,1]=o*(-0.411084369491)*x_ref[1]**(o-1)+(0.954831689569)
        ref[3,0,0,0]=o*(0.768277437177)*x_ref[0]**(o-1)+(0.641045872586)
        ref[3,0,0,1]=o*(0.456435914145)*x_ref[1]**(o-1)+(-0.538634781112)
        ref[3,0,1,0]=o*(0.0107901515277)*x_ref[0]**(o-1)+(0.204809134773)
        ref[3,0,1,1]=o*(0.632781493458)*x_ref[1]**(o-1)+(0.813936796774)
        ref[3,1,0,0]=o*(0.187196185344)*x_ref[0]**(o-1)+(-0.974329803149)
        ref[3,1,0,1]=o*(-0.131567139248)*x_ref[1]**(o-1)+(-0.205725336514)
        ref[3,1,1,0]=o*(0.747378585446)*x_ref[0]**(o-1)+(0.126887766052)
        ref[3,1,1,1]=o*(-0.660302189752)*x_ref[1]**(o-1)+(0.227362395796)
        ref[4,0,0,0]=o*(0.822029556554)*x_ref[0]**(o-1)+(0.742977721062)
        ref[4,0,0,1]=o*(0.972349490061)*x_ref[1]**(o-1)+(0.307748083079)
        ref[4,0,1,0]=o*(-0.0347009705896)*x_ref[0]**(o-1)+(0.506556900045)
        ref[4,0,1,1]=o*(-0.700430813955)*x_ref[1]**(o-1)+(0.606413581865)
        ref[4,1,0,0]=o*(-0.69868976773)*x_ref[0]**(o-1)+(0.447214222886)
        ref[4,1,0,1]=o*(0.290522797177)*x_ref[1]**(o-1)+(0.139281488147)
        ref[4,1,1,0]=o*(0.850722242552)*x_ref[0]**(o-1)+(0.578613819769)
        ref[4,1,1,1]=o*(-0.983680277805)*x_ref[1]**(o-1)+(-0.0737991679361)
        ref[5,0,0,0]=o*(0.164023891553)*x_ref[0]**(o-1)+(-0.398812116857)
        ref[5,0,0,1]=o*(0.754040638026)*x_ref[1]**(o-1)+(0.890632418705)
        ref[5,0,1,0]=o*(0.176036561413)*x_ref[0]**(o-1)+(0.551905509271)
        ref[5,0,1,1]=o*(0.584521803718)*x_ref[1]**(o-1)+(0.359816759406)
        ref[5,1,0,0]=o*(0.728503623092)*x_ref[0]**(o-1)+(0.448628087199)
        ref[5,1,0,1]=o*(0.7693890417)*x_ref[1]**(o-1)+(-0.148028432439)
        ref[5,1,1,0]=o*(-0.974113865308)*x_ref[0]**(o-1)+(-0.271026924938)
        ref[5,1,1,1]=o*(0.0918858163816)*x_ref[1]**(o-1)+(0.441895253556)
      else:
        s[0,0,0]=(0.902798077985)*x[0]**o+(-0.513564836003)*x[0]+(-0.56740021175)*x[1]**o+(-0.987494297109)*x[1]+(0.701885375414)*x[2]**o+(0.457818844785)*x[2]
        s[0,0,1]=(0.961682822532)*x[0]**o+(-0.155805753312)*x[0]+(0.664632247096)*x[1]**o+(-0.416626235035)*x[1]+(-0.710160165264)*x[2]**o+(0.604159028896)*x[2]
        s[0,1,0]=(-0.331535871964)*x[0]**o+(-0.417629864265)*x[0]+(0.206294208748)*x[1]**o+(-0.259792550744)*x[1]+(-0.125585762102)*x[2]**o+(-0.893437959215)*x[2]
        s[0,1,1]=(0.614124081239)*x[0]**o+(-0.974461795616)*x[0]+(-0.978571310051)*x[1]**o+(-0.696753351954)*x[1]+(-0.125579053404)*x[2]**o+(0.744152004821)*x[2]
        s[1,0,0]=(0.470195589624)*x[0]**o+(-0.543298404217)*x[0]+(-0.050519561506)*x[1]**o+(-0.140360629835)*x[1]+(0.198214376652)*x[2]**o+(0.424647243144)*x[2]
        s[1,0,1]=(0.726479572534)*x[0]**o+(0.553604934856)*x[0]+(0.0319217924147)*x[1]**o+(0.281723632246)*x[1]+(-0.525936361143)*x[2]**o+(0.0826305292145)*x[2]
        s[1,1,0]=(0.731149931727)*x[0]**o+(-0.792038038336)*x[0]+(0.376591980524)*x[1]**o+(-0.533851270214)*x[1]+(0.228328372658)*x[2]**o+(-0.685994122324)*x[2]
        s[1,1,1]=(0.0316092245143)*x[0]**o+(0.272078804647)*x[0]+(0.232701533258)*x[1]**o+(0.484166193421)*x[1]+(0.810979214906)*x[2]**o+(-0.678008492162)*x[2]
        s[2,0,0]=(-0.25934895114)*x[0]**o+(0.855703849413)*x[0]+(0.435458381315)*x[1]**o+(0.287983708847)*x[1]+(0.637280806162)*x[2]**o+(-0.871379201278)*x[2]
        s[2,0,1]=(-0.0628315253456)*x[0]**o+(-0.456689468581)*x[0]+(-0.990061219179)*x[1]**o+(0.103941195754)*x[1]+(0.683984409836)*x[2]**o+(-0.786055311041)*x[2]
        s[2,1,0]=(0.574774054041)*x[0]**o+(0.861386252917)*x[0]+(-0.817046727565)*x[1]**o+(-0.132779923567)*x[1]+(0.230879541156)*x[2]**o+(-0.774875301646)*x[2]
        s[2,1,1]=(-0.549231428363)*x[0]**o+(0.640633896033)*x[0]+(-0.0935356199455)*x[1]**o+(-0.139090953413)*x[1]+(0.00853390900484)*x[2]**o+(-0.974008998234)*x[2]
        s[3,0,0]=(-0.318959630087)*x[0]**o+(0.572260168906)*x[0]+(-0.715323370515)*x[1]**o+(-0.29409167385)*x[1]+(-0.417828539621)*x[2]**o+(-0.527658924511)*x[2]
        s[3,0,1]=(0.662794455331)*x[0]**o+(0.655652283888)*x[0]+(0.314776154296)*x[1]**o+(0.568698535574)*x[1]+(-0.715089514092)*x[2]**o+(-0.976898245896)*x[2]
        s[3,1,0]=(0.775469693327)*x[0]**o+(0.894695158486)*x[0]+(-0.0845263659865)*x[1]**o+(0.613321238688)*x[1]+(-0.261167781047)*x[2]**o+(-0.858407379125)*x[2]
        s[3,1,1]=(0.879739525497)*x[0]**o+(-0.196410556198)*x[0]+(0.245451670145)*x[1]**o+(0.881860540375)*x[1]+(-0.406601916222)*x[2]**o+(-0.698825194269)*x[2]
        s[4,0,0]=(-0.295318870536)*x[0]**o+(-0.386975265135)*x[0]+(-0.4694457277)*x[1]**o+(0.381202263202)*x[1]+(-0.246058013597)*x[2]**o+(-0.117116349193)*x[2]
        s[4,0,1]=(-0.52750487839)*x[0]**o+(0.239477947116)*x[0]+(0.415068508509)*x[1]**o+(0.499147016759)*x[1]+(-0.757947686224)*x[2]**o+(0.0356840223308)*x[2]
        s[4,1,0]=(-0.503519047063)*x[0]**o+(-0.983884111712)*x[0]+(0.289499262273)*x[1]**o+(0.477655179172)*x[1]+(0.267544491427)*x[2]**o+(0.577972060216)*x[2]
        s[4,1,1]=(-0.765677873271)*x[0]**o+(-0.574403742889)*x[0]+(0.0365362593269)*x[1]**o+(0.718127942002)*x[1]+(-0.882868921566)*x[2]**o+(-0.866965724039)*x[2]
        s[5,0,0]=(0.477311133484)*x[0]**o+(0.970292462756)*x[0]+(0.400262807291)*x[1]**o+(0.839779920106)*x[1]+(0.91075067745)*x[2]**o+(-0.171648787827)*x[2]
        s[5,0,1]=(-0.14817999498)*x[0]**o+(-0.284074361404)*x[0]+(-0.809114317075)*x[1]**o+(-0.421481538417)*x[1]+(0.617793146872)*x[2]**o+(0.808487061398)*x[2]
        s[5,1,0]=(-0.750064834677)*x[0]**o+(0.595717210851)*x[0]+(0.499697404346)*x[1]**o+(-0.890435534805)*x[1]+(-0.705182209377)*x[2]**o+(-0.831416292443)*x[2]
        s[5,1,1]=(-0.93237665803)*x[0]**o+(-0.608748654097)*x[0]+(-0.163402758239)*x[1]**o+(0.762304956717)*x[1]+(-0.236439076616)*x[2]**o+(-0.827173574464)*x[2]
        ref[0,0,0,0]=o*(0.902798077985)*x_ref[0]**(o-1)+(-0.513564836003)
        ref[0,0,0,1]=o*(-0.56740021175)*x_ref[1]**(o-1)+(-0.987494297109)
        ref[0,0,0,2]=o*(0.701885375414)*x_ref[2]**(o-1)+(0.457818844785)
        ref[0,0,1,0]=o*(0.961682822532)*x_ref[0]**(o-1)+(-0.155805753312)
        ref[0,0,1,1]=o*(0.664632247096)*x_ref[1]**(o-1)+(-0.416626235035)
        ref[0,0,1,2]=o*(-0.710160165264)*x_ref[2]**(o-1)+(0.604159028896)
        ref[0,1,0,0]=o*(-0.331535871964)*x_ref[0]**(o-1)+(-0.417629864265)
        ref[0,1,0,1]=o*(0.206294208748)*x_ref[1]**(o-1)+(-0.259792550744)
        ref[0,1,0,2]=o*(-0.125585762102)*x_ref[2]**(o-1)+(-0.893437959215)
        ref[0,1,1,0]=o*(0.614124081239)*x_ref[0]**(o-1)+(-0.974461795616)
        ref[0,1,1,1]=o*(-0.978571310051)*x_ref[1]**(o-1)+(-0.696753351954)
        ref[0,1,1,2]=o*(-0.125579053404)*x_ref[2]**(o-1)+(0.744152004821)
        ref[1,0,0,0]=o*(0.470195589624)*x_ref[0]**(o-1)+(-0.543298404217)
        ref[1,0,0,1]=o*(-0.050519561506)*x_ref[1]**(o-1)+(-0.140360629835)
        ref[1,0,0,2]=o*(0.198214376652)*x_ref[2]**(o-1)+(0.424647243144)
        ref[1,0,1,0]=o*(0.726479572534)*x_ref[0]**(o-1)+(0.553604934856)
        ref[1,0,1,1]=o*(0.0319217924147)*x_ref[1]**(o-1)+(0.281723632246)
        ref[1,0,1,2]=o*(-0.525936361143)*x_ref[2]**(o-1)+(0.0826305292145)
        ref[1,1,0,0]=o*(0.731149931727)*x_ref[0]**(o-1)+(-0.792038038336)
        ref[1,1,0,1]=o*(0.376591980524)*x_ref[1]**(o-1)+(-0.533851270214)
        ref[1,1,0,2]=o*(0.228328372658)*x_ref[2]**(o-1)+(-0.685994122324)
        ref[1,1,1,0]=o*(0.0316092245143)*x_ref[0]**(o-1)+(0.272078804647)
        ref[1,1,1,1]=o*(0.232701533258)*x_ref[1]**(o-1)+(0.484166193421)
        ref[1,1,1,2]=o*(0.810979214906)*x_ref[2]**(o-1)+(-0.678008492162)
        ref[2,0,0,0]=o*(-0.25934895114)*x_ref[0]**(o-1)+(0.855703849413)
        ref[2,0,0,1]=o*(0.435458381315)*x_ref[1]**(o-1)+(0.287983708847)
        ref[2,0,0,2]=o*(0.637280806162)*x_ref[2]**(o-1)+(-0.871379201278)
        ref[2,0,1,0]=o*(-0.0628315253456)*x_ref[0]**(o-1)+(-0.456689468581)
        ref[2,0,1,1]=o*(-0.990061219179)*x_ref[1]**(o-1)+(0.103941195754)
        ref[2,0,1,2]=o*(0.683984409836)*x_ref[2]**(o-1)+(-0.786055311041)
        ref[2,1,0,0]=o*(0.574774054041)*x_ref[0]**(o-1)+(0.861386252917)
        ref[2,1,0,1]=o*(-0.817046727565)*x_ref[1]**(o-1)+(-0.132779923567)
        ref[2,1,0,2]=o*(0.230879541156)*x_ref[2]**(o-1)+(-0.774875301646)
        ref[2,1,1,0]=o*(-0.549231428363)*x_ref[0]**(o-1)+(0.640633896033)
        ref[2,1,1,1]=o*(-0.0935356199455)*x_ref[1]**(o-1)+(-0.139090953413)
        ref[2,1,1,2]=o*(0.00853390900484)*x_ref[2]**(o-1)+(-0.974008998234)
        ref[3,0,0,0]=o*(-0.318959630087)*x_ref[0]**(o-1)+(0.572260168906)
        ref[3,0,0,1]=o*(-0.715323370515)*x_ref[1]**(o-1)+(-0.29409167385)
        ref[3,0,0,2]=o*(-0.417828539621)*x_ref[2]**(o-1)+(-0.527658924511)
        ref[3,0,1,0]=o*(0.662794455331)*x_ref[0]**(o-1)+(0.655652283888)
        ref[3,0,1,1]=o*(0.314776154296)*x_ref[1]**(o-1)+(0.568698535574)
        ref[3,0,1,2]=o*(-0.715089514092)*x_ref[2]**(o-1)+(-0.976898245896)
        ref[3,1,0,0]=o*(0.775469693327)*x_ref[0]**(o-1)+(0.894695158486)
        ref[3,1,0,1]=o*(-0.0845263659865)*x_ref[1]**(o-1)+(0.613321238688)
        ref[3,1,0,2]=o*(-0.261167781047)*x_ref[2]**(o-1)+(-0.858407379125)
        ref[3,1,1,0]=o*(0.879739525497)*x_ref[0]**(o-1)+(-0.196410556198)
        ref[3,1,1,1]=o*(0.245451670145)*x_ref[1]**(o-1)+(0.881860540375)
        ref[3,1,1,2]=o*(-0.406601916222)*x_ref[2]**(o-1)+(-0.698825194269)
        ref[4,0,0,0]=o*(-0.295318870536)*x_ref[0]**(o-1)+(-0.386975265135)
        ref[4,0,0,1]=o*(-0.4694457277)*x_ref[1]**(o-1)+(0.381202263202)
        ref[4,0,0,2]=o*(-0.246058013597)*x_ref[2]**(o-1)+(-0.117116349193)
        ref[4,0,1,0]=o*(-0.52750487839)*x_ref[0]**(o-1)+(0.239477947116)
        ref[4,0,1,1]=o*(0.415068508509)*x_ref[1]**(o-1)+(0.499147016759)
        ref[4,0,1,2]=o*(-0.757947686224)*x_ref[2]**(o-1)+(0.0356840223308)
        ref[4,1,0,0]=o*(-0.503519047063)*x_ref[0]**(o-1)+(-0.983884111712)
        ref[4,1,0,1]=o*(0.289499262273)*x_ref[1]**(o-1)+(0.477655179172)
        ref[4,1,0,2]=o*(0.267544491427)*x_ref[2]**(o-1)+(0.577972060216)
        ref[4,1,1,0]=o*(-0.765677873271)*x_ref[0]**(o-1)+(-0.574403742889)
        ref[4,1,1,1]=o*(0.0365362593269)*x_ref[1]**(o-1)+(0.718127942002)
        ref[4,1,1,2]=o*(-0.882868921566)*x_ref[2]**(o-1)+(-0.866965724039)
        ref[5,0,0,0]=o*(0.477311133484)*x_ref[0]**(o-1)+(0.970292462756)
        ref[5,0,0,1]=o*(0.400262807291)*x_ref[1]**(o-1)+(0.839779920106)
        ref[5,0,0,2]=o*(0.91075067745)*x_ref[2]**(o-1)+(-0.171648787827)
        ref[5,0,1,0]=o*(-0.14817999498)*x_ref[0]**(o-1)+(-0.284074361404)
        ref[5,0,1,1]=o*(-0.809114317075)*x_ref[1]**(o-1)+(-0.421481538417)
        ref[5,0,1,2]=o*(0.617793146872)*x_ref[2]**(o-1)+(0.808487061398)
        ref[5,1,0,0]=o*(-0.750064834677)*x_ref[0]**(o-1)+(0.595717210851)
        ref[5,1,0,1]=o*(0.499697404346)*x_ref[1]**(o-1)+(-0.890435534805)
        ref[5,1,0,2]=o*(-0.705182209377)*x_ref[2]**(o-1)+(-0.831416292443)
        ref[5,1,1,0]=o*(-0.93237665803)*x_ref[0]**(o-1)+(-0.608748654097)
        ref[5,1,1,1]=o*(-0.163402758239)*x_ref[1]**(o-1)+(0.762304956717)
        ref[5,1,1,2]=o*(-0.236439076616)*x_ref[2]**(o-1)+(-0.827173574464)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_Solution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.293125312175)*x[0]**o+(0.376150381261)*x[0]+(0.252430693686)*x[1]**o+(0.150428455322)*x[1]
        ref[0]=o*(0.293125312175)*x_ref[0]**(o-1)+(0.376150381261)
        ref[1]=o*(0.252430693686)*x_ref[1]**(o-1)+(0.150428455322)
      else:
        s=(0.978180509747)*x[0]**o+(0.768503254374)*x[0]+(0.113267806213)*x[1]**o+(-0.997552294934)*x[1]+(-0.643998032917)*x[2]**o+(-0.17168440847)*x[2]
        ref[0]=o*(0.978180509747)*x_ref[0]**(o-1)+(0.768503254374)
        ref[1]=o*(0.113267806213)*x_ref[1]**(o-1)+(-0.997552294934)
        ref[2]=o*(-0.643998032917)*x_ref[2]**(o-1)+(-0.17168440847)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_Solution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.138895230208)*x[0]**o+(-0.448856094505)*x[0]+(-0.82305383921)*x[1]**o+(-0.161535603589)*x[1]
        s[1]=(-0.302535177393)*x[0]**o+(0.811653356478)*x[0]+(0.05548155455)*x[1]**o+(-0.343903467756)*x[1]
        ref[0,0]=o*(-0.138895230208)*x_ref[0]**(o-1)+(-0.448856094505)
        ref[0,1]=o*(-0.82305383921)*x_ref[1]**(o-1)+(-0.161535603589)
        ref[1,0]=o*(-0.302535177393)*x_ref[0]**(o-1)+(0.811653356478)
        ref[1,1]=o*(0.05548155455)*x_ref[1]**(o-1)+(-0.343903467756)
      else:
        s[0]=(0.583525061796)*x[0]**o+(-0.891145747091)*x[0]+(-0.469656381595)*x[1]**o+(-0.0103398578248)*x[1]+(0.0970809480199)*x[2]**o+(0.652357321835)*x[2]
        s[1]=(-0.488885199473)*x[0]**o+(0.481806213481)*x[0]+(-0.7328445706)*x[1]**o+(0.28590334803)*x[1]+(0.044964568194)*x[2]**o+(0.474397408351)*x[2]
        ref[0,0]=o*(0.583525061796)*x_ref[0]**(o-1)+(-0.891145747091)
        ref[0,1]=o*(-0.469656381595)*x_ref[1]**(o-1)+(-0.0103398578248)
        ref[0,2]=o*(0.0970809480199)*x_ref[2]**(o-1)+(0.652357321835)
        ref[1,0]=o*(-0.488885199473)*x_ref[0]**(o-1)+(0.481806213481)
        ref[1,1]=o*(-0.7328445706)*x_ref[1]**(o-1)+(0.28590334803)
        ref[1,2]=o*(0.044964568194)*x_ref[2]**(o-1)+(0.474397408351)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_Solution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.872000208895)*x[0]**o+(-0.304556916188)*x[0]+(0.534581283951)*x[1]**o+(-0.626579480196)*x[1]
        s[0,1]=(-0.240150296068)*x[0]**o+(-0.31754292824)*x[0]+(-0.0574070276326)*x[1]**o+(-0.616687559393)*x[1]
        s[0,2]=(0.570345724046)*x[0]**o+(0.110952065807)*x[0]+(0.997553898152)*x[1]**o+(-0.503380742681)*x[1]
        s[0,3]=(0.872704383137)*x[0]**o+(0.686594842527)*x[0]+(0.798635469075)*x[1]**o+(0.43649341733)*x[1]
        s[0,4]=(-0.236636103417)*x[0]**o+(0.0762117276629)*x[0]+(-0.774027674603)*x[1]**o+(0.823651416506)*x[1]
        s[1,0]=(0.922041581692)*x[0]**o+(-0.284368621571)*x[0]+(-0.71200852665)*x[1]**o+(0.661348856406)*x[1]
        s[1,1]=(-0.402436007071)*x[0]**o+(-0.436850834963)*x[0]+(0.62748269836)*x[1]**o+(0.504691317707)*x[1]
        s[1,2]=(-0.793104721592)*x[0]**o+(-0.00597773372911)*x[0]+(0.0410611356405)*x[1]**o+(0.169129450548)*x[1]
        s[1,3]=(0.821521972958)*x[0]**o+(-0.784664003887)*x[0]+(-0.880383627011)*x[1]**o+(0.127311373905)*x[1]
        s[1,4]=(0.775700618787)*x[0]**o+(-0.837475890825)*x[0]+(0.426581954939)*x[1]**o+(0.404905797587)*x[1]
        s[2,0]=(-0.100956366427)*x[0]**o+(0.0933662209553)*x[0]+(-0.680046458225)*x[1]**o+(-0.145587503687)*x[1]
        s[2,1]=(0.166159597084)*x[0]**o+(-0.900241170518)*x[0]+(-0.168407179398)*x[1]**o+(-0.82352508068)*x[1]
        s[2,2]=(-0.476621892618)*x[0]**o+(-0.422712025977)*x[0]+(0.298151628461)*x[1]**o+(0.862589525974)*x[1]
        s[2,3]=(-0.0148783218305)*x[0]**o+(0.203140203381)*x[0]+(0.19407097983)*x[1]**o+(-0.772615096404)*x[1]
        s[2,4]=(0.656255800409)*x[0]**o+(-0.946330057363)*x[0]+(-0.257965999122)*x[1]**o+(0.71359373028)*x[1]
        s[3,0]=(-0.214762058983)*x[0]**o+(0.51144571871)*x[0]+(-0.147282047629)*x[1]**o+(0.709339363468)*x[1]
        s[3,1]=(-0.9112564909)*x[0]**o+(-0.900437328847)*x[0]+(-0.975324195939)*x[1]**o+(0.202889107529)*x[1]
        s[3,2]=(0.753787529389)*x[0]**o+(0.0630834387277)*x[0]+(-0.759168264562)*x[1]**o+(0.697873996051)*x[1]
        s[3,3]=(-0.0335165829084)*x[0]**o+(0.274748931927)*x[0]+(0.0686458601464)*x[1]**o+(-0.117608157399)*x[1]
        s[3,4]=(0.853158371839)*x[0]**o+(-0.166315314677)*x[0]+(0.789241330781)*x[1]**o+(-0.108867830466)*x[1]
        ref[0,0,0]=o*(0.872000208895)*x_ref[0]**(o-1)+(-0.304556916188)
        ref[0,0,1]=o*(0.534581283951)*x_ref[1]**(o-1)+(-0.626579480196)
        ref[0,1,0]=o*(-0.240150296068)*x_ref[0]**(o-1)+(-0.31754292824)
        ref[0,1,1]=o*(-0.0574070276326)*x_ref[1]**(o-1)+(-0.616687559393)
        ref[0,2,0]=o*(0.570345724046)*x_ref[0]**(o-1)+(0.110952065807)
        ref[0,2,1]=o*(0.997553898152)*x_ref[1]**(o-1)+(-0.503380742681)
        ref[0,3,0]=o*(0.872704383137)*x_ref[0]**(o-1)+(0.686594842527)
        ref[0,3,1]=o*(0.798635469075)*x_ref[1]**(o-1)+(0.43649341733)
        ref[0,4,0]=o*(-0.236636103417)*x_ref[0]**(o-1)+(0.0762117276629)
        ref[0,4,1]=o*(-0.774027674603)*x_ref[1]**(o-1)+(0.823651416506)
        ref[1,0,0]=o*(0.922041581692)*x_ref[0]**(o-1)+(-0.284368621571)
        ref[1,0,1]=o*(-0.71200852665)*x_ref[1]**(o-1)+(0.661348856406)
        ref[1,1,0]=o*(-0.402436007071)*x_ref[0]**(o-1)+(-0.436850834963)
        ref[1,1,1]=o*(0.62748269836)*x_ref[1]**(o-1)+(0.504691317707)
        ref[1,2,0]=o*(-0.793104721592)*x_ref[0]**(o-1)+(-0.00597773372911)
        ref[1,2,1]=o*(0.0410611356405)*x_ref[1]**(o-1)+(0.169129450548)
        ref[1,3,0]=o*(0.821521972958)*x_ref[0]**(o-1)+(-0.784664003887)
        ref[1,3,1]=o*(-0.880383627011)*x_ref[1]**(o-1)+(0.127311373905)
        ref[1,4,0]=o*(0.775700618787)*x_ref[0]**(o-1)+(-0.837475890825)
        ref[1,4,1]=o*(0.426581954939)*x_ref[1]**(o-1)+(0.404905797587)
        ref[2,0,0]=o*(-0.100956366427)*x_ref[0]**(o-1)+(0.0933662209553)
        ref[2,0,1]=o*(-0.680046458225)*x_ref[1]**(o-1)+(-0.145587503687)
        ref[2,1,0]=o*(0.166159597084)*x_ref[0]**(o-1)+(-0.900241170518)
        ref[2,1,1]=o*(-0.168407179398)*x_ref[1]**(o-1)+(-0.82352508068)
        ref[2,2,0]=o*(-0.476621892618)*x_ref[0]**(o-1)+(-0.422712025977)
        ref[2,2,1]=o*(0.298151628461)*x_ref[1]**(o-1)+(0.862589525974)
        ref[2,3,0]=o*(-0.0148783218305)*x_ref[0]**(o-1)+(0.203140203381)
        ref[2,3,1]=o*(0.19407097983)*x_ref[1]**(o-1)+(-0.772615096404)
        ref[2,4,0]=o*(0.656255800409)*x_ref[0]**(o-1)+(-0.946330057363)
        ref[2,4,1]=o*(-0.257965999122)*x_ref[1]**(o-1)+(0.71359373028)
        ref[3,0,0]=o*(-0.214762058983)*x_ref[0]**(o-1)+(0.51144571871)
        ref[3,0,1]=o*(-0.147282047629)*x_ref[1]**(o-1)+(0.709339363468)
        ref[3,1,0]=o*(-0.9112564909)*x_ref[0]**(o-1)+(-0.900437328847)
        ref[3,1,1]=o*(-0.975324195939)*x_ref[1]**(o-1)+(0.202889107529)
        ref[3,2,0]=o*(0.753787529389)*x_ref[0]**(o-1)+(0.0630834387277)
        ref[3,2,1]=o*(-0.759168264562)*x_ref[1]**(o-1)+(0.697873996051)
        ref[3,3,0]=o*(-0.0335165829084)*x_ref[0]**(o-1)+(0.274748931927)
        ref[3,3,1]=o*(0.0686458601464)*x_ref[1]**(o-1)+(-0.117608157399)
        ref[3,4,0]=o*(0.853158371839)*x_ref[0]**(o-1)+(-0.166315314677)
        ref[3,4,1]=o*(0.789241330781)*x_ref[1]**(o-1)+(-0.108867830466)
      else:
        s[0,0]=(0.110594722699)*x[0]**o+(-0.980877918241)*x[0]+(-0.515559287698)*x[1]**o+(-0.434426926239)*x[1]+(-0.0366559310163)*x[2]**o+(0.387597823386)*x[2]
        s[0,1]=(-0.661423535056)*x[0]**o+(0.361082618968)*x[0]+(-0.614579461592)*x[1]**o+(0.212089227265)*x[1]+(-0.409309067892)*x[2]**o+(-0.237106482925)*x[2]
        s[0,2]=(0.704845609334)*x[0]**o+(-0.682094657708)*x[0]+(0.0483054658314)*x[1]**o+(0.992561055244)*x[1]+(-0.676784605979)*x[2]**o+(-0.992707421428)*x[2]
        s[0,3]=(0.901583311581)*x[0]**o+(-0.751695552218)*x[0]+(-0.869549394358)*x[1]**o+(-0.889541184356)*x[1]+(-0.969435364186)*x[2]**o+(0.72152104042)*x[2]
        s[0,4]=(-0.105522466219)*x[0]**o+(0.230764909205)*x[0]+(-0.714498630414)*x[1]**o+(-0.715271005406)*x[1]+(0.0748854473277)*x[2]**o+(0.0678300049229)*x[2]
        s[1,0]=(-0.275856624456)*x[0]**o+(-0.876662969004)*x[0]+(0.681719113093)*x[1]**o+(-0.24943399785)*x[1]+(0.0837599776409)*x[2]**o+(-0.0572825122918)*x[2]
        s[1,1]=(0.359136507147)*x[0]**o+(0.663934928768)*x[0]+(0.669380381609)*x[1]**o+(0.716889420418)*x[1]+(0.0903945351303)*x[2]**o+(-0.975420985116)*x[2]
        s[1,2]=(0.284213351844)*x[0]**o+(0.634377813314)*x[0]+(-0.0935851210685)*x[1]**o+(-0.195280560003)*x[1]+(0.379637016222)*x[2]**o+(0.57302910168)*x[2]
        s[1,3]=(-0.370451421268)*x[0]**o+(-0.619498493655)*x[0]+(0.386199258196)*x[1]**o+(0.150857936022)*x[1]+(0.123336255712)*x[2]**o+(0.0144448266576)*x[2]
        s[1,4]=(-0.517413398717)*x[0]**o+(-0.952688078253)*x[0]+(-0.146422080083)*x[1]**o+(0.0871413294299)*x[1]+(0.027398040628)*x[2]**o+(0.391317414872)*x[2]
        s[2,0]=(-0.126511587518)*x[0]**o+(0.696693700404)*x[0]+(0.408515262838)*x[1]**o+(-0.251009002512)*x[1]+(-0.641227451468)*x[2]**o+(0.679579864261)*x[2]
        s[2,1]=(0.527972863795)*x[0]**o+(-0.796977004243)*x[0]+(0.596836552415)*x[1]**o+(-0.810757416401)*x[1]+(-0.282708831988)*x[2]**o+(0.127311583541)*x[2]
        s[2,2]=(-0.777216458022)*x[0]**o+(-0.676149863396)*x[0]+(0.696096143251)*x[1]**o+(-0.725078530289)*x[1]+(-0.244393979264)*x[2]**o+(-0.0585093928601)*x[2]
        s[2,3]=(0.312124516125)*x[0]**o+(0.733713522534)*x[0]+(0.706643983792)*x[1]**o+(-0.361913672386)*x[1]+(-0.0131736734016)*x[2]**o+(-0.530500138249)*x[2]
        s[2,4]=(-0.399862024703)*x[0]**o+(-0.98706730829)*x[0]+(0.90279618711)*x[1]**o+(-0.120842398107)*x[1]+(-0.621221270703)*x[2]**o+(-0.44680972674)*x[2]
        s[3,0]=(0.060963938467)*x[0]**o+(0.409545603066)*x[0]+(-0.399416656677)*x[1]**o+(-0.154729705297)*x[1]+(-0.821106571182)*x[2]**o+(0.917110134569)*x[2]
        s[3,1]=(0.395083986916)*x[0]**o+(0.957423333841)*x[0]+(-0.753576884181)*x[1]**o+(0.402825662464)*x[1]+(0.444980287919)*x[2]**o+(-0.793575717385)*x[2]
        s[3,2]=(0.519344966169)*x[0]**o+(-0.919203821647)*x[0]+(0.472250030015)*x[1]**o+(0.912498896282)*x[1]+(0.30696323239)*x[2]**o+(-0.742821847869)*x[2]
        s[3,3]=(-0.762199299359)*x[0]**o+(0.245025069538)*x[0]+(-0.576731219257)*x[1]**o+(-0.818632810068)*x[1]+(0.295773945963)*x[2]**o+(-0.0803897317676)*x[2]
        s[3,4]=(-0.339788378435)*x[0]**o+(-0.498108849772)*x[0]+(-0.779105108948)*x[1]**o+(-0.31350292532)*x[1]+(0.576582365303)*x[2]**o+(0.733406974144)*x[2]
        ref[0,0,0]=o*(0.110594722699)*x_ref[0]**(o-1)+(-0.980877918241)
        ref[0,0,1]=o*(-0.515559287698)*x_ref[1]**(o-1)+(-0.434426926239)
        ref[0,0,2]=o*(-0.0366559310163)*x_ref[2]**(o-1)+(0.387597823386)
        ref[0,1,0]=o*(-0.661423535056)*x_ref[0]**(o-1)+(0.361082618968)
        ref[0,1,1]=o*(-0.614579461592)*x_ref[1]**(o-1)+(0.212089227265)
        ref[0,1,2]=o*(-0.409309067892)*x_ref[2]**(o-1)+(-0.237106482925)
        ref[0,2,0]=o*(0.704845609334)*x_ref[0]**(o-1)+(-0.682094657708)
        ref[0,2,1]=o*(0.0483054658314)*x_ref[1]**(o-1)+(0.992561055244)
        ref[0,2,2]=o*(-0.676784605979)*x_ref[2]**(o-1)+(-0.992707421428)
        ref[0,3,0]=o*(0.901583311581)*x_ref[0]**(o-1)+(-0.751695552218)
        ref[0,3,1]=o*(-0.869549394358)*x_ref[1]**(o-1)+(-0.889541184356)
        ref[0,3,2]=o*(-0.969435364186)*x_ref[2]**(o-1)+(0.72152104042)
        ref[0,4,0]=o*(-0.105522466219)*x_ref[0]**(o-1)+(0.230764909205)
        ref[0,4,1]=o*(-0.714498630414)*x_ref[1]**(o-1)+(-0.715271005406)
        ref[0,4,2]=o*(0.0748854473277)*x_ref[2]**(o-1)+(0.0678300049229)
        ref[1,0,0]=o*(-0.275856624456)*x_ref[0]**(o-1)+(-0.876662969004)
        ref[1,0,1]=o*(0.681719113093)*x_ref[1]**(o-1)+(-0.24943399785)
        ref[1,0,2]=o*(0.0837599776409)*x_ref[2]**(o-1)+(-0.0572825122918)
        ref[1,1,0]=o*(0.359136507147)*x_ref[0]**(o-1)+(0.663934928768)
        ref[1,1,1]=o*(0.669380381609)*x_ref[1]**(o-1)+(0.716889420418)
        ref[1,1,2]=o*(0.0903945351303)*x_ref[2]**(o-1)+(-0.975420985116)
        ref[1,2,0]=o*(0.284213351844)*x_ref[0]**(o-1)+(0.634377813314)
        ref[1,2,1]=o*(-0.0935851210685)*x_ref[1]**(o-1)+(-0.195280560003)
        ref[1,2,2]=o*(0.379637016222)*x_ref[2]**(o-1)+(0.57302910168)
        ref[1,3,0]=o*(-0.370451421268)*x_ref[0]**(o-1)+(-0.619498493655)
        ref[1,3,1]=o*(0.386199258196)*x_ref[1]**(o-1)+(0.150857936022)
        ref[1,3,2]=o*(0.123336255712)*x_ref[2]**(o-1)+(0.0144448266576)
        ref[1,4,0]=o*(-0.517413398717)*x_ref[0]**(o-1)+(-0.952688078253)
        ref[1,4,1]=o*(-0.146422080083)*x_ref[1]**(o-1)+(0.0871413294299)
        ref[1,4,2]=o*(0.027398040628)*x_ref[2]**(o-1)+(0.391317414872)
        ref[2,0,0]=o*(-0.126511587518)*x_ref[0]**(o-1)+(0.696693700404)
        ref[2,0,1]=o*(0.408515262838)*x_ref[1]**(o-1)+(-0.251009002512)
        ref[2,0,2]=o*(-0.641227451468)*x_ref[2]**(o-1)+(0.679579864261)
        ref[2,1,0]=o*(0.527972863795)*x_ref[0]**(o-1)+(-0.796977004243)
        ref[2,1,1]=o*(0.596836552415)*x_ref[1]**(o-1)+(-0.810757416401)
        ref[2,1,2]=o*(-0.282708831988)*x_ref[2]**(o-1)+(0.127311583541)
        ref[2,2,0]=o*(-0.777216458022)*x_ref[0]**(o-1)+(-0.676149863396)
        ref[2,2,1]=o*(0.696096143251)*x_ref[1]**(o-1)+(-0.725078530289)
        ref[2,2,2]=o*(-0.244393979264)*x_ref[2]**(o-1)+(-0.0585093928601)
        ref[2,3,0]=o*(0.312124516125)*x_ref[0]**(o-1)+(0.733713522534)
        ref[2,3,1]=o*(0.706643983792)*x_ref[1]**(o-1)+(-0.361913672386)
        ref[2,3,2]=o*(-0.0131736734016)*x_ref[2]**(o-1)+(-0.530500138249)
        ref[2,4,0]=o*(-0.399862024703)*x_ref[0]**(o-1)+(-0.98706730829)
        ref[2,4,1]=o*(0.90279618711)*x_ref[1]**(o-1)+(-0.120842398107)
        ref[2,4,2]=o*(-0.621221270703)*x_ref[2]**(o-1)+(-0.44680972674)
        ref[3,0,0]=o*(0.060963938467)*x_ref[0]**(o-1)+(0.409545603066)
        ref[3,0,1]=o*(-0.399416656677)*x_ref[1]**(o-1)+(-0.154729705297)
        ref[3,0,2]=o*(-0.821106571182)*x_ref[2]**(o-1)+(0.917110134569)
        ref[3,1,0]=o*(0.395083986916)*x_ref[0]**(o-1)+(0.957423333841)
        ref[3,1,1]=o*(-0.753576884181)*x_ref[1]**(o-1)+(0.402825662464)
        ref[3,1,2]=o*(0.444980287919)*x_ref[2]**(o-1)+(-0.793575717385)
        ref[3,2,0]=o*(0.519344966169)*x_ref[0]**(o-1)+(-0.919203821647)
        ref[3,2,1]=o*(0.472250030015)*x_ref[1]**(o-1)+(0.912498896282)
        ref[3,2,2]=o*(0.30696323239)*x_ref[2]**(o-1)+(-0.742821847869)
        ref[3,3,0]=o*(-0.762199299359)*x_ref[0]**(o-1)+(0.245025069538)
        ref[3,3,1]=o*(-0.576731219257)*x_ref[1]**(o-1)+(-0.818632810068)
        ref[3,3,2]=o*(0.295773945963)*x_ref[2]**(o-1)+(-0.0803897317676)
        ref[3,4,0]=o*(-0.339788378435)*x_ref[0]**(o-1)+(-0.498108849772)
        ref[3,4,1]=o*(-0.779105108948)*x_ref[1]**(o-1)+(-0.31350292532)
        ref[3,4,2]=o*(0.576582365303)*x_ref[2]**(o-1)+(0.733406974144)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_Solution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.259416702848)*x[0]**o+(-0.245749999215)*x[0]+(-0.832153732959)*x[1]**o+(-0.739170215675)*x[1]
        s[0,0,1]=(0.348837228856)*x[0]**o+(-0.666178442607)*x[0]+(-0.849566479561)*x[1]**o+(-0.602086600784)*x[1]
        s[0,1,0]=(-0.155423946179)*x[0]**o+(0.236260314596)*x[0]+(-0.161071268117)*x[1]**o+(-0.86073460521)*x[1]
        s[0,1,1]=(0.00159453654506)*x[0]**o+(0.854588501463)*x[0]+(-0.932772016744)*x[1]**o+(-0.241490278915)*x[1]
        s[1,0,0]=(0.954888979422)*x[0]**o+(-0.0656825754869)*x[0]+(-0.685117910338)*x[1]**o+(0.329351311251)*x[1]
        s[1,0,1]=(0.653313879148)*x[0]**o+(-0.100353950439)*x[0]+(-0.427779024915)*x[1]**o+(0.541043842928)*x[1]
        s[1,1,0]=(-0.529344191036)*x[0]**o+(0.927248256927)*x[0]+(0.637824503976)*x[1]**o+(-0.510873834148)*x[1]
        s[1,1,1]=(0.392461722459)*x[0]**o+(0.867028086856)*x[0]+(-0.0972014158405)*x[1]**o+(0.930607205251)*x[1]
        s[2,0,0]=(-0.896743306281)*x[0]**o+(-0.44962668492)*x[0]+(0.510004368477)*x[1]**o+(-0.218091766014)*x[1]
        s[2,0,1]=(0.236477487758)*x[0]**o+(0.868982620792)*x[0]+(0.596196651626)*x[1]**o+(0.362649185465)*x[1]
        s[2,1,0]=(0.959619355116)*x[0]**o+(-0.502578509326)*x[0]+(-0.903069740877)*x[1]**o+(0.389137844229)*x[1]
        s[2,1,1]=(-0.683715298632)*x[0]**o+(0.577696755527)*x[0]+(-0.306714659414)*x[1]**o+(-0.72219199497)*x[1]
        s[3,0,0]=(-0.266756580775)*x[0]**o+(-0.360900565587)*x[0]+(-0.0715666567511)*x[1]**o+(-0.576677423431)*x[1]
        s[3,0,1]=(0.234720540152)*x[0]**o+(0.174728976827)*x[0]+(-0.347025720807)*x[1]**o+(-0.42072768341)*x[1]
        s[3,1,0]=(-0.470982762526)*x[0]**o+(-0.177781625128)*x[0]+(-0.0706858877364)*x[1]**o+(-0.722096785258)*x[1]
        s[3,1,1]=(0.598814426535)*x[0]**o+(0.767055301394)*x[0]+(0.229632024426)*x[1]**o+(0.944250372458)*x[1]
        s[4,0,0]=(0.451029698166)*x[0]**o+(-0.0341544883586)*x[0]+(0.785553686295)*x[1]**o+(0.455034664029)*x[1]
        s[4,0,1]=(-0.477486417869)*x[0]**o+(0.509646735364)*x[0]+(0.857098988263)*x[1]**o+(-0.330863877708)*x[1]
        s[4,1,0]=(-0.582320009575)*x[0]**o+(0.469724725179)*x[0]+(-0.482992293774)*x[1]**o+(-0.306261026308)*x[1]
        s[4,1,1]=(0.635452311265)*x[0]**o+(-0.795606745931)*x[0]+(-0.659604106287)*x[1]**o+(-0.0576280668882)*x[1]
        s[5,0,0]=(-0.621743078369)*x[0]**o+(0.889286951305)*x[0]+(-0.487280887249)*x[1]**o+(-0.377618071831)*x[1]
        s[5,0,1]=(0.558391219866)*x[0]**o+(-0.265622963303)*x[0]+(-0.0801965105564)*x[1]**o+(0.800983693747)*x[1]
        s[5,1,0]=(-0.291431222644)*x[0]**o+(-0.259327626357)*x[0]+(0.597600388624)*x[1]**o+(-0.890810042322)*x[1]
        s[5,1,1]=(-0.391398128176)*x[0]**o+(0.994837784671)*x[0]+(-0.467189874705)*x[1]**o+(0.626474390794)*x[1]
        ref[0,0,0,0]=o*(-0.259416702848)*x_ref[0]**(o-1)+(-0.245749999215)
        ref[0,0,0,1]=o*(-0.832153732959)*x_ref[1]**(o-1)+(-0.739170215675)
        ref[0,0,1,0]=o*(0.348837228856)*x_ref[0]**(o-1)+(-0.666178442607)
        ref[0,0,1,1]=o*(-0.849566479561)*x_ref[1]**(o-1)+(-0.602086600784)
        ref[0,1,0,0]=o*(-0.155423946179)*x_ref[0]**(o-1)+(0.236260314596)
        ref[0,1,0,1]=o*(-0.161071268117)*x_ref[1]**(o-1)+(-0.86073460521)
        ref[0,1,1,0]=o*(0.00159453654506)*x_ref[0]**(o-1)+(0.854588501463)
        ref[0,1,1,1]=o*(-0.932772016744)*x_ref[1]**(o-1)+(-0.241490278915)
        ref[1,0,0,0]=o*(0.954888979422)*x_ref[0]**(o-1)+(-0.0656825754869)
        ref[1,0,0,1]=o*(-0.685117910338)*x_ref[1]**(o-1)+(0.329351311251)
        ref[1,0,1,0]=o*(0.653313879148)*x_ref[0]**(o-1)+(-0.100353950439)
        ref[1,0,1,1]=o*(-0.427779024915)*x_ref[1]**(o-1)+(0.541043842928)
        ref[1,1,0,0]=o*(-0.529344191036)*x_ref[0]**(o-1)+(0.927248256927)
        ref[1,1,0,1]=o*(0.637824503976)*x_ref[1]**(o-1)+(-0.510873834148)
        ref[1,1,1,0]=o*(0.392461722459)*x_ref[0]**(o-1)+(0.867028086856)
        ref[1,1,1,1]=o*(-0.0972014158405)*x_ref[1]**(o-1)+(0.930607205251)
        ref[2,0,0,0]=o*(-0.896743306281)*x_ref[0]**(o-1)+(-0.44962668492)
        ref[2,0,0,1]=o*(0.510004368477)*x_ref[1]**(o-1)+(-0.218091766014)
        ref[2,0,1,0]=o*(0.236477487758)*x_ref[0]**(o-1)+(0.868982620792)
        ref[2,0,1,1]=o*(0.596196651626)*x_ref[1]**(o-1)+(0.362649185465)
        ref[2,1,0,0]=o*(0.959619355116)*x_ref[0]**(o-1)+(-0.502578509326)
        ref[2,1,0,1]=o*(-0.903069740877)*x_ref[1]**(o-1)+(0.389137844229)
        ref[2,1,1,0]=o*(-0.683715298632)*x_ref[0]**(o-1)+(0.577696755527)
        ref[2,1,1,1]=o*(-0.306714659414)*x_ref[1]**(o-1)+(-0.72219199497)
        ref[3,0,0,0]=o*(-0.266756580775)*x_ref[0]**(o-1)+(-0.360900565587)
        ref[3,0,0,1]=o*(-0.0715666567511)*x_ref[1]**(o-1)+(-0.576677423431)
        ref[3,0,1,0]=o*(0.234720540152)*x_ref[0]**(o-1)+(0.174728976827)
        ref[3,0,1,1]=o*(-0.347025720807)*x_ref[1]**(o-1)+(-0.42072768341)
        ref[3,1,0,0]=o*(-0.470982762526)*x_ref[0]**(o-1)+(-0.177781625128)
        ref[3,1,0,1]=o*(-0.0706858877364)*x_ref[1]**(o-1)+(-0.722096785258)
        ref[3,1,1,0]=o*(0.598814426535)*x_ref[0]**(o-1)+(0.767055301394)
        ref[3,1,1,1]=o*(0.229632024426)*x_ref[1]**(o-1)+(0.944250372458)
        ref[4,0,0,0]=o*(0.451029698166)*x_ref[0]**(o-1)+(-0.0341544883586)
        ref[4,0,0,1]=o*(0.785553686295)*x_ref[1]**(o-1)+(0.455034664029)
        ref[4,0,1,0]=o*(-0.477486417869)*x_ref[0]**(o-1)+(0.509646735364)
        ref[4,0,1,1]=o*(0.857098988263)*x_ref[1]**(o-1)+(-0.330863877708)
        ref[4,1,0,0]=o*(-0.582320009575)*x_ref[0]**(o-1)+(0.469724725179)
        ref[4,1,0,1]=o*(-0.482992293774)*x_ref[1]**(o-1)+(-0.306261026308)
        ref[4,1,1,0]=o*(0.635452311265)*x_ref[0]**(o-1)+(-0.795606745931)
        ref[4,1,1,1]=o*(-0.659604106287)*x_ref[1]**(o-1)+(-0.0576280668882)
        ref[5,0,0,0]=o*(-0.621743078369)*x_ref[0]**(o-1)+(0.889286951305)
        ref[5,0,0,1]=o*(-0.487280887249)*x_ref[1]**(o-1)+(-0.377618071831)
        ref[5,0,1,0]=o*(0.558391219866)*x_ref[0]**(o-1)+(-0.265622963303)
        ref[5,0,1,1]=o*(-0.0801965105564)*x_ref[1]**(o-1)+(0.800983693747)
        ref[5,1,0,0]=o*(-0.291431222644)*x_ref[0]**(o-1)+(-0.259327626357)
        ref[5,1,0,1]=o*(0.597600388624)*x_ref[1]**(o-1)+(-0.890810042322)
        ref[5,1,1,0]=o*(-0.391398128176)*x_ref[0]**(o-1)+(0.994837784671)
        ref[5,1,1,1]=o*(-0.467189874705)*x_ref[1]**(o-1)+(0.626474390794)
      else:
        s[0,0,0]=(-0.283421054619)*x[0]**o+(-0.389803426219)*x[0]+(-0.641735725139)*x[1]**o+(-0.184182767783)*x[1]+(0.559369811956)*x[2]**o+(0.4130246739)*x[2]
        s[0,0,1]=(-0.685739480994)*x[0]**o+(-0.412746444255)*x[0]+(-0.977924434029)*x[1]**o+(0.866683883992)*x[1]+(-0.104721288552)*x[2]**o+(-0.854137553522)*x[2]
        s[0,1,0]=(0.326677121892)*x[0]**o+(0.0346511145836)*x[0]+(-0.583479722363)*x[1]**o+(0.552232008446)*x[1]+(-0.66426015414)*x[2]**o+(-0.0282350207962)*x[2]
        s[0,1,1]=(-0.743104988438)*x[0]**o+(-0.250100474341)*x[0]+(-0.0100118927398)*x[1]**o+(0.421549551364)*x[1]+(0.774348452339)*x[2]**o+(0.0114324989752)*x[2]
        s[1,0,0]=(-0.969248079823)*x[0]**o+(0.554289530783)*x[0]+(-0.437222589525)*x[1]**o+(-0.454550411233)*x[1]+(-0.555157485811)*x[2]**o+(0.360201196386)*x[2]
        s[1,0,1]=(0.428483060047)*x[0]**o+(-0.305500955239)*x[0]+(-0.918692317879)*x[1]**o+(-0.0749149504003)*x[1]+(-0.146230161925)*x[2]**o+(-0.328660463631)*x[2]
        s[1,1,0]=(-0.534394777023)*x[0]**o+(-0.0160564112453)*x[0]+(-0.446201680182)*x[1]**o+(-0.457243622333)*x[1]+(0.316031457633)*x[2]**o+(-0.0867540461561)*x[2]
        s[1,1,1]=(-0.180636289042)*x[0]**o+(0.409910302887)*x[0]+(-0.261132577936)*x[1]**o+(0.976758759795)*x[1]+(-0.298504887785)*x[2]**o+(0.0720200683864)*x[2]
        s[2,0,0]=(-0.965162583973)*x[0]**o+(-0.885802239228)*x[0]+(-0.166151347797)*x[1]**o+(0.968122595961)*x[1]+(0.580364754552)*x[2]**o+(0.213039816836)*x[2]
        s[2,0,1]=(0.7871160079)*x[0]**o+(-0.235526954906)*x[0]+(0.496553426537)*x[1]**o+(-0.0239289077236)*x[1]+(-0.633193916071)*x[2]**o+(-0.777984393332)*x[2]
        s[2,1,0]=(-0.79465182095)*x[0]**o+(-0.0880694356168)*x[0]+(-0.686736039005)*x[1]**o+(-0.221174710075)*x[1]+(0.0208981523074)*x[2]**o+(-0.961469223363)*x[2]
        s[2,1,1]=(-0.455839585127)*x[0]**o+(0.941828964284)*x[0]+(0.998331788392)*x[1]**o+(0.0750455958592)*x[1]+(0.608504253916)*x[2]**o+(-0.374386910681)*x[2]
        s[3,0,0]=(-0.909949427027)*x[0]**o+(-0.396086474642)*x[0]+(0.899299670063)*x[1]**o+(0.141272919633)*x[1]+(0.53792064575)*x[2]**o+(0.595764269338)*x[2]
        s[3,0,1]=(-0.49521850368)*x[0]**o+(0.207120861953)*x[0]+(0.569903370263)*x[1]**o+(0.588491719962)*x[1]+(0.452965587121)*x[2]**o+(-0.628229661128)*x[2]
        s[3,1,0]=(-0.875095641635)*x[0]**o+(0.11799580621)*x[0]+(0.109152019439)*x[1]**o+(-0.941762743205)*x[1]+(-0.124413788962)*x[2]**o+(-0.733044880598)*x[2]
        s[3,1,1]=(0.691739414808)*x[0]**o+(-0.0580142383494)*x[0]+(-0.625563645068)*x[1]**o+(-0.459983356734)*x[1]+(0.743764207387)*x[2]**o+(0.200950835048)*x[2]
        s[4,0,0]=(-0.686339571296)*x[0]**o+(0.310431821345)*x[0]+(0.0697719589995)*x[1]**o+(0.413649564597)*x[1]+(0.311083457053)*x[2]**o+(-0.764323862328)*x[2]
        s[4,0,1]=(0.82709146654)*x[0]**o+(-0.958833756078)*x[0]+(-0.293388295126)*x[1]**o+(0.789172794162)*x[1]+(-0.47291926184)*x[2]**o+(0.447003729661)*x[2]
        s[4,1,0]=(-0.968741249623)*x[0]**o+(-0.977865313489)*x[0]+(0.777352281599)*x[1]**o+(-0.0177223674634)*x[1]+(-0.437430561766)*x[2]**o+(0.403327933356)*x[2]
        s[4,1,1]=(0.873657912127)*x[0]**o+(-0.432188278068)*x[0]+(-0.294192343439)*x[1]**o+(-0.199868940744)*x[1]+(0.379614508084)*x[2]**o+(0.610834295156)*x[2]
        s[5,0,0]=(-0.452716736875)*x[0]**o+(0.298248314499)*x[0]+(0.850779531393)*x[1]**o+(-0.024084377803)*x[1]+(-0.106948635789)*x[2]**o+(0.926343375096)*x[2]
        s[5,0,1]=(-0.22629592934)*x[0]**o+(-0.481026079587)*x[0]+(0.713198238543)*x[1]**o+(-0.290540996335)*x[1]+(-0.775028431788)*x[2]**o+(0.839104261372)*x[2]
        s[5,1,0]=(0.938919479885)*x[0]**o+(-0.833254411673)*x[0]+(-0.0920737395198)*x[1]**o+(-0.863263173274)*x[1]+(-0.326022820082)*x[2]**o+(-0.162067090943)*x[2]
        s[5,1,1]=(0.800986633012)*x[0]**o+(-0.102366666332)*x[0]+(-0.35704808806)*x[1]**o+(-0.710763760013)*x[1]+(0.249411990506)*x[2]**o+(0.569232273165)*x[2]
        ref[0,0,0,0]=o*(-0.283421054619)*x_ref[0]**(o-1)+(-0.389803426219)
        ref[0,0,0,1]=o*(-0.641735725139)*x_ref[1]**(o-1)+(-0.184182767783)
        ref[0,0,0,2]=o*(0.559369811956)*x_ref[2]**(o-1)+(0.4130246739)
        ref[0,0,1,0]=o*(-0.685739480994)*x_ref[0]**(o-1)+(-0.412746444255)
        ref[0,0,1,1]=o*(-0.977924434029)*x_ref[1]**(o-1)+(0.866683883992)
        ref[0,0,1,2]=o*(-0.104721288552)*x_ref[2]**(o-1)+(-0.854137553522)
        ref[0,1,0,0]=o*(0.326677121892)*x_ref[0]**(o-1)+(0.0346511145836)
        ref[0,1,0,1]=o*(-0.583479722363)*x_ref[1]**(o-1)+(0.552232008446)
        ref[0,1,0,2]=o*(-0.66426015414)*x_ref[2]**(o-1)+(-0.0282350207962)
        ref[0,1,1,0]=o*(-0.743104988438)*x_ref[0]**(o-1)+(-0.250100474341)
        ref[0,1,1,1]=o*(-0.0100118927398)*x_ref[1]**(o-1)+(0.421549551364)
        ref[0,1,1,2]=o*(0.774348452339)*x_ref[2]**(o-1)+(0.0114324989752)
        ref[1,0,0,0]=o*(-0.969248079823)*x_ref[0]**(o-1)+(0.554289530783)
        ref[1,0,0,1]=o*(-0.437222589525)*x_ref[1]**(o-1)+(-0.454550411233)
        ref[1,0,0,2]=o*(-0.555157485811)*x_ref[2]**(o-1)+(0.360201196386)
        ref[1,0,1,0]=o*(0.428483060047)*x_ref[0]**(o-1)+(-0.305500955239)
        ref[1,0,1,1]=o*(-0.918692317879)*x_ref[1]**(o-1)+(-0.0749149504003)
        ref[1,0,1,2]=o*(-0.146230161925)*x_ref[2]**(o-1)+(-0.328660463631)
        ref[1,1,0,0]=o*(-0.534394777023)*x_ref[0]**(o-1)+(-0.0160564112453)
        ref[1,1,0,1]=o*(-0.446201680182)*x_ref[1]**(o-1)+(-0.457243622333)
        ref[1,1,0,2]=o*(0.316031457633)*x_ref[2]**(o-1)+(-0.0867540461561)
        ref[1,1,1,0]=o*(-0.180636289042)*x_ref[0]**(o-1)+(0.409910302887)
        ref[1,1,1,1]=o*(-0.261132577936)*x_ref[1]**(o-1)+(0.976758759795)
        ref[1,1,1,2]=o*(-0.298504887785)*x_ref[2]**(o-1)+(0.0720200683864)
        ref[2,0,0,0]=o*(-0.965162583973)*x_ref[0]**(o-1)+(-0.885802239228)
        ref[2,0,0,1]=o*(-0.166151347797)*x_ref[1]**(o-1)+(0.968122595961)
        ref[2,0,0,2]=o*(0.580364754552)*x_ref[2]**(o-1)+(0.213039816836)
        ref[2,0,1,0]=o*(0.7871160079)*x_ref[0]**(o-1)+(-0.235526954906)
        ref[2,0,1,1]=o*(0.496553426537)*x_ref[1]**(o-1)+(-0.0239289077236)
        ref[2,0,1,2]=o*(-0.633193916071)*x_ref[2]**(o-1)+(-0.777984393332)
        ref[2,1,0,0]=o*(-0.79465182095)*x_ref[0]**(o-1)+(-0.0880694356168)
        ref[2,1,0,1]=o*(-0.686736039005)*x_ref[1]**(o-1)+(-0.221174710075)
        ref[2,1,0,2]=o*(0.0208981523074)*x_ref[2]**(o-1)+(-0.961469223363)
        ref[2,1,1,0]=o*(-0.455839585127)*x_ref[0]**(o-1)+(0.941828964284)
        ref[2,1,1,1]=o*(0.998331788392)*x_ref[1]**(o-1)+(0.0750455958592)
        ref[2,1,1,2]=o*(0.608504253916)*x_ref[2]**(o-1)+(-0.374386910681)
        ref[3,0,0,0]=o*(-0.909949427027)*x_ref[0]**(o-1)+(-0.396086474642)
        ref[3,0,0,1]=o*(0.899299670063)*x_ref[1]**(o-1)+(0.141272919633)
        ref[3,0,0,2]=o*(0.53792064575)*x_ref[2]**(o-1)+(0.595764269338)
        ref[3,0,1,0]=o*(-0.49521850368)*x_ref[0]**(o-1)+(0.207120861953)
        ref[3,0,1,1]=o*(0.569903370263)*x_ref[1]**(o-1)+(0.588491719962)
        ref[3,0,1,2]=o*(0.452965587121)*x_ref[2]**(o-1)+(-0.628229661128)
        ref[3,1,0,0]=o*(-0.875095641635)*x_ref[0]**(o-1)+(0.11799580621)
        ref[3,1,0,1]=o*(0.109152019439)*x_ref[1]**(o-1)+(-0.941762743205)
        ref[3,1,0,2]=o*(-0.124413788962)*x_ref[2]**(o-1)+(-0.733044880598)
        ref[3,1,1,0]=o*(0.691739414808)*x_ref[0]**(o-1)+(-0.0580142383494)
        ref[3,1,1,1]=o*(-0.625563645068)*x_ref[1]**(o-1)+(-0.459983356734)
        ref[3,1,1,2]=o*(0.743764207387)*x_ref[2]**(o-1)+(0.200950835048)
        ref[4,0,0,0]=o*(-0.686339571296)*x_ref[0]**(o-1)+(0.310431821345)
        ref[4,0,0,1]=o*(0.0697719589995)*x_ref[1]**(o-1)+(0.413649564597)
        ref[4,0,0,2]=o*(0.311083457053)*x_ref[2]**(o-1)+(-0.764323862328)
        ref[4,0,1,0]=o*(0.82709146654)*x_ref[0]**(o-1)+(-0.958833756078)
        ref[4,0,1,1]=o*(-0.293388295126)*x_ref[1]**(o-1)+(0.789172794162)
        ref[4,0,1,2]=o*(-0.47291926184)*x_ref[2]**(o-1)+(0.447003729661)
        ref[4,1,0,0]=o*(-0.968741249623)*x_ref[0]**(o-1)+(-0.977865313489)
        ref[4,1,0,1]=o*(0.777352281599)*x_ref[1]**(o-1)+(-0.0177223674634)
        ref[4,1,0,2]=o*(-0.437430561766)*x_ref[2]**(o-1)+(0.403327933356)
        ref[4,1,1,0]=o*(0.873657912127)*x_ref[0]**(o-1)+(-0.432188278068)
        ref[4,1,1,1]=o*(-0.294192343439)*x_ref[1]**(o-1)+(-0.199868940744)
        ref[4,1,1,2]=o*(0.379614508084)*x_ref[2]**(o-1)+(0.610834295156)
        ref[5,0,0,0]=o*(-0.452716736875)*x_ref[0]**(o-1)+(0.298248314499)
        ref[5,0,0,1]=o*(0.850779531393)*x_ref[1]**(o-1)+(-0.024084377803)
        ref[5,0,0,2]=o*(-0.106948635789)*x_ref[2]**(o-1)+(0.926343375096)
        ref[5,0,1,0]=o*(-0.22629592934)*x_ref[0]**(o-1)+(-0.481026079587)
        ref[5,0,1,1]=o*(0.713198238543)*x_ref[1]**(o-1)+(-0.290540996335)
        ref[5,0,1,2]=o*(-0.775028431788)*x_ref[2]**(o-1)+(0.839104261372)
        ref[5,1,0,0]=o*(0.938919479885)*x_ref[0]**(o-1)+(-0.833254411673)
        ref[5,1,0,1]=o*(-0.0920737395198)*x_ref[1]**(o-1)+(-0.863263173274)
        ref[5,1,0,2]=o*(-0.326022820082)*x_ref[2]**(o-1)+(-0.162067090943)
        ref[5,1,1,0]=o*(0.800986633012)*x_ref[0]**(o-1)+(-0.102366666332)
        ref[5,1,1,1]=o*(-0.35704808806)*x_ref[1]**(o-1)+(-0.710763760013)
        ref[5,1,1,2]=o*(0.249411990506)*x_ref[2]**(o-1)+(0.569232273165)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.049981434374)*x[0]+(1.21186502577)*x[1]
        ref[0]=(-0.049981434374)
        ref[1]=(1.21186502577)
      else:
        s=(-0.0752432283484)*x[0]+(0.11438092577)*x[1]+(-0.434888190035)*x[2]
        ref[0]=(-0.0752432283484)
        ref[1]=(0.11438092577)
        ref[2]=(-0.434888190035)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.456351387646)*x[0]+(-0.116715426711)*x[1]
        s[1]=(-0.854853163245)*x[0]+(-0.00664827097294)*x[1]
        ref[0,0]=(-0.456351387646)
        ref[0,1]=(-0.116715426711)
        ref[1,0]=(-0.854853163245)
        ref[1,1]=(-0.00664827097294)
      else:
        s[0]=(0.623100837308)*x[0]+(0.266907813705)*x[1]+(1.38575183924)*x[2]
        s[1]=(-1.48702785688)*x[0]+(-1.42426212001)*x[1]+(0.317785468787)*x[2]
        ref[0,0]=(0.623100837308)
        ref[0,1]=(0.266907813705)
        ref[0,2]=(1.38575183924)
        ref[1,0]=(-1.48702785688)
        ref[1,1]=(-1.42426212001)
        ref[1,2]=(0.317785468787)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-1.0789465623)*x[0]+(-0.347959175817)*x[1]
        s[0,1]=(-0.551011467115)*x[0]+(1.35264465045)*x[1]
        s[0,2]=(-0.570905806915)*x[0]+(0.0584374344724)*x[1]
        s[0,3]=(0.596777443685)*x[0]+(0.414694389615)*x[1]
        s[0,4]=(0.611144733018)*x[0]+(-0.758856666701)*x[1]
        s[1,0]=(-0.0733659967272)*x[0]+(1.27900296295)*x[1]
        s[1,1]=(-0.305565660896)*x[0]+(0.9434988738)*x[1]
        s[1,2]=(-0.992776824928)*x[0]+(-0.120817150952)*x[1]
        s[1,3]=(0.0902201920474)*x[0]+(1.04097295319)*x[1]
        s[1,4]=(-1.06448209062)*x[0]+(1.13868467732)*x[1]
        s[2,0]=(0.855203060995)*x[0]+(0.707160314719)*x[1]
        s[2,1]=(0.792799717854)*x[0]+(-0.36484734497)*x[1]
        s[2,2]=(-1.14542452528)*x[0]+(0.98847021504)*x[1]
        s[2,3]=(0.145237440613)*x[0]+(0.214714794234)*x[1]
        s[2,4]=(0.179556206154)*x[0]+(-1.21107721861)*x[1]
        s[3,0]=(0.700250733001)*x[0]+(1.25331090966)*x[1]
        s[3,1]=(0.258700850981)*x[0]+(-0.637474578328)*x[1]
        s[3,2]=(0.382709435533)*x[0]+(1.29575763411)*x[1]
        s[3,3]=(-0.254008873764)*x[0]+(-1.54340970006)*x[1]
        s[3,4]=(-0.658082774987)*x[0]+(0.647639696672)*x[1]
        ref[0,0,0]=(-1.0789465623)
        ref[0,0,1]=(-0.347959175817)
        ref[0,1,0]=(-0.551011467115)
        ref[0,1,1]=(1.35264465045)
        ref[0,2,0]=(-0.570905806915)
        ref[0,2,1]=(0.0584374344724)
        ref[0,3,0]=(0.596777443685)
        ref[0,3,1]=(0.414694389615)
        ref[0,4,0]=(0.611144733018)
        ref[0,4,1]=(-0.758856666701)
        ref[1,0,0]=(-0.0733659967272)
        ref[1,0,1]=(1.27900296295)
        ref[1,1,0]=(-0.305565660896)
        ref[1,1,1]=(0.9434988738)
        ref[1,2,0]=(-0.992776824928)
        ref[1,2,1]=(-0.120817150952)
        ref[1,3,0]=(0.0902201920474)
        ref[1,3,1]=(1.04097295319)
        ref[1,4,0]=(-1.06448209062)
        ref[1,4,1]=(1.13868467732)
        ref[2,0,0]=(0.855203060995)
        ref[2,0,1]=(0.707160314719)
        ref[2,1,0]=(0.792799717854)
        ref[2,1,1]=(-0.36484734497)
        ref[2,2,0]=(-1.14542452528)
        ref[2,2,1]=(0.98847021504)
        ref[2,3,0]=(0.145237440613)
        ref[2,3,1]=(0.214714794234)
        ref[2,4,0]=(0.179556206154)
        ref[2,4,1]=(-1.21107721861)
        ref[3,0,0]=(0.700250733001)
        ref[3,0,1]=(1.25331090966)
        ref[3,1,0]=(0.258700850981)
        ref[3,1,1]=(-0.637474578328)
        ref[3,2,0]=(0.382709435533)
        ref[3,2,1]=(1.29575763411)
        ref[3,3,0]=(-0.254008873764)
        ref[3,3,1]=(-1.54340970006)
        ref[3,4,0]=(-0.658082774987)
        ref[3,4,1]=(0.647639696672)
      else:
        s[0,0]=(1.12878816262)*x[0]+(-1.74114589021)*x[1]+(0.248765786072)*x[2]
        s[0,1]=(1.72583176211)*x[0]+(-1.23114549996)*x[1]+(-0.203157732085)*x[2]
        s[0,2]=(-0.422314546839)*x[0]+(0.224260123741)*x[1]+(0.171252253031)*x[2]
        s[0,3]=(-0.870712689882)*x[0]+(-0.555793590179)*x[1]+(0.290927600809)*x[2]
        s[0,4]=(-0.920833142524)*x[0]+(-0.0497652513943)*x[1]+(-0.39718342747)*x[2]
        s[1,0]=(-0.727450597273)*x[0]+(1.09323195342)*x[1]+(0.298478430357)*x[2]
        s[1,1]=(0.12608256775)*x[0]+(-0.459788156475)*x[1]+(1.13249234657)*x[2]
        s[1,2]=(0.898608507221)*x[0]+(-0.46334527476)*x[1]+(-0.445291041098)*x[2]
        s[1,3]=(-0.327616894595)*x[0]+(-0.288577792837)*x[1]+(1.26565355504)*x[2]
        s[1,4]=(0.0630186390647)*x[0]+(-0.383734001362)*x[1]+(-0.116975162517)*x[2]
        s[2,0]=(-0.309394005751)*x[0]+(-0.12474002694)*x[1]+(0.699555331484)*x[2]
        s[2,1]=(0.997752898742)*x[0]+(-0.183982572346)*x[1]+(1.22982190192)*x[2]
        s[2,2]=(1.03496586697)*x[0]+(-0.097256212804)*x[1]+(1.42052511301)*x[2]
        s[2,3]=(0.614378729663)*x[0]+(0.154397435133)*x[1]+(-0.863004330742)*x[2]
        s[2,4]=(1.20904264903)*x[0]+(-0.452388361293)*x[1]+(-1.13286356474)*x[2]
        s[3,0]=(-0.812538381038)*x[0]+(0.00308372111735)*x[1]+(-0.117937095444)*x[2]
        s[3,1]=(0.45479320042)*x[0]+(-0.45927716926)*x[1]+(0.4787228586)*x[2]
        s[3,2]=(-0.487685328452)*x[0]+(0.871241301367)*x[1]+(0.78769505788)*x[2]
        s[3,3]=(0.860788048428)*x[0]+(1.39676274038)*x[1]+(0.407326483464)*x[2]
        s[3,4]=(0.00409247820271)*x[0]+(0.44466332801)*x[1]+(-0.66721280275)*x[2]
        ref[0,0,0]=(1.12878816262)
        ref[0,0,1]=(-1.74114589021)
        ref[0,0,2]=(0.248765786072)
        ref[0,1,0]=(1.72583176211)
        ref[0,1,1]=(-1.23114549996)
        ref[0,1,2]=(-0.203157732085)
        ref[0,2,0]=(-0.422314546839)
        ref[0,2,1]=(0.224260123741)
        ref[0,2,2]=(0.171252253031)
        ref[0,3,0]=(-0.870712689882)
        ref[0,3,1]=(-0.555793590179)
        ref[0,3,2]=(0.290927600809)
        ref[0,4,0]=(-0.920833142524)
        ref[0,4,1]=(-0.0497652513943)
        ref[0,4,2]=(-0.39718342747)
        ref[1,0,0]=(-0.727450597273)
        ref[1,0,1]=(1.09323195342)
        ref[1,0,2]=(0.298478430357)
        ref[1,1,0]=(0.12608256775)
        ref[1,1,1]=(-0.459788156475)
        ref[1,1,2]=(1.13249234657)
        ref[1,2,0]=(0.898608507221)
        ref[1,2,1]=(-0.46334527476)
        ref[1,2,2]=(-0.445291041098)
        ref[1,3,0]=(-0.327616894595)
        ref[1,3,1]=(-0.288577792837)
        ref[1,3,2]=(1.26565355504)
        ref[1,4,0]=(0.0630186390647)
        ref[1,4,1]=(-0.383734001362)
        ref[1,4,2]=(-0.116975162517)
        ref[2,0,0]=(-0.309394005751)
        ref[2,0,1]=(-0.12474002694)
        ref[2,0,2]=(0.699555331484)
        ref[2,1,0]=(0.997752898742)
        ref[2,1,1]=(-0.183982572346)
        ref[2,1,2]=(1.22982190192)
        ref[2,2,0]=(1.03496586697)
        ref[2,2,1]=(-0.097256212804)
        ref[2,2,2]=(1.42052511301)
        ref[2,3,0]=(0.614378729663)
        ref[2,3,1]=(0.154397435133)
        ref[2,3,2]=(-0.863004330742)
        ref[2,4,0]=(1.20904264903)
        ref[2,4,1]=(-0.452388361293)
        ref[2,4,2]=(-1.13286356474)
        ref[3,0,0]=(-0.812538381038)
        ref[3,0,1]=(0.00308372111735)
        ref[3,0,2]=(-0.117937095444)
        ref[3,1,0]=(0.45479320042)
        ref[3,1,1]=(-0.45927716926)
        ref[3,1,2]=(0.4787228586)
        ref[3,2,0]=(-0.487685328452)
        ref[3,2,1]=(0.871241301367)
        ref[3,2,2]=(0.78769505788)
        ref[3,3,0]=(0.860788048428)
        ref[3,3,1]=(1.39676274038)
        ref[3,3,2]=(0.407326483464)
        ref[3,4,0]=(0.00409247820271)
        ref[3,4,1]=(0.44466332801)
        ref[3,4,2]=(-0.66721280275)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.905736881137)*x[0]+(0.694861875477)*x[1]
        s[0,0,1]=(-1.45890910436)*x[0]+(-1.52565465047)*x[1]
        s[0,1,0]=(-0.312122938001)*x[0]+(-0.868878770447)*x[1]
        s[0,1,1]=(1.54725692119)*x[0]+(0.536122055612)*x[1]
        s[1,0,0]=(0.196190980599)*x[0]+(0.328328643332)*x[1]
        s[1,0,1]=(-1.08863159071)*x[0]+(-1.33920996273)*x[1]
        s[1,1,0]=(1.73274170273)*x[0]+(-0.958793175994)*x[1]
        s[1,1,1]=(0.182836505159)*x[0]+(-0.121541234518)*x[1]
        s[2,0,0]=(0.144461277784)*x[0]+(1.83781976673)*x[1]
        s[2,0,1]=(-0.194587892421)*x[0]+(-1.27090147949)*x[1]
        s[2,1,0]=(0.379488182396)*x[0]+(0.030493966182)*x[1]
        s[2,1,1]=(1.09003510354)*x[0]+(-0.454578719663)*x[1]
        s[3,0,0]=(1.58446081096)*x[0]+(-0.52113508443)*x[1]
        s[3,0,1]=(-1.33593796269)*x[0]+(-0.165961948897)*x[1]
        s[3,1,0]=(0.377402371367)*x[0]+(-0.0184973476961)*x[1]
        s[3,1,1]=(0.764313855665)*x[0]+(0.294443818887)*x[1]
        s[4,0,0]=(-0.180639063236)*x[0]+(1.75864081648)*x[1]
        s[4,0,1]=(-1.62662138708)*x[0]+(1.81611541058)*x[1]
        s[4,1,0]=(1.37678767999)*x[0]+(-0.271115982236)*x[1]
        s[4,1,1]=(1.32861834819)*x[0]+(0.768596947238)*x[1]
        s[5,0,0]=(1.39342920577)*x[0]+(0.155666506926)*x[1]
        s[5,0,1]=(0.0397474261373)*x[0]+(0.921852245407)*x[1]
        s[5,1,0]=(-0.950337694161)*x[0]+(-0.011042822071)*x[1]
        s[5,1,1]=(0.0369412335225)*x[0]+(-0.473468894798)*x[1]
        ref[0,0,0,0]=(-0.905736881137)
        ref[0,0,0,1]=(0.694861875477)
        ref[0,0,1,0]=(-1.45890910436)
        ref[0,0,1,1]=(-1.52565465047)
        ref[0,1,0,0]=(-0.312122938001)
        ref[0,1,0,1]=(-0.868878770447)
        ref[0,1,1,0]=(1.54725692119)
        ref[0,1,1,1]=(0.536122055612)
        ref[1,0,0,0]=(0.196190980599)
        ref[1,0,0,1]=(0.328328643332)
        ref[1,0,1,0]=(-1.08863159071)
        ref[1,0,1,1]=(-1.33920996273)
        ref[1,1,0,0]=(1.73274170273)
        ref[1,1,0,1]=(-0.958793175994)
        ref[1,1,1,0]=(0.182836505159)
        ref[1,1,1,1]=(-0.121541234518)
        ref[2,0,0,0]=(0.144461277784)
        ref[2,0,0,1]=(1.83781976673)
        ref[2,0,1,0]=(-0.194587892421)
        ref[2,0,1,1]=(-1.27090147949)
        ref[2,1,0,0]=(0.379488182396)
        ref[2,1,0,1]=(0.030493966182)
        ref[2,1,1,0]=(1.09003510354)
        ref[2,1,1,1]=(-0.454578719663)
        ref[3,0,0,0]=(1.58446081096)
        ref[3,0,0,1]=(-0.52113508443)
        ref[3,0,1,0]=(-1.33593796269)
        ref[3,0,1,1]=(-0.165961948897)
        ref[3,1,0,0]=(0.377402371367)
        ref[3,1,0,1]=(-0.0184973476961)
        ref[3,1,1,0]=(0.764313855665)
        ref[3,1,1,1]=(0.294443818887)
        ref[4,0,0,0]=(-0.180639063236)
        ref[4,0,0,1]=(1.75864081648)
        ref[4,0,1,0]=(-1.62662138708)
        ref[4,0,1,1]=(1.81611541058)
        ref[4,1,0,0]=(1.37678767999)
        ref[4,1,0,1]=(-0.271115982236)
        ref[4,1,1,0]=(1.32861834819)
        ref[4,1,1,1]=(0.768596947238)
        ref[5,0,0,0]=(1.39342920577)
        ref[5,0,0,1]=(0.155666506926)
        ref[5,0,1,0]=(0.0397474261373)
        ref[5,0,1,1]=(0.921852245407)
        ref[5,1,0,0]=(-0.950337694161)
        ref[5,1,0,1]=(-0.011042822071)
        ref[5,1,1,0]=(0.0369412335225)
        ref[5,1,1,1]=(-0.473468894798)
      else:
        s[0,0,0]=(0.0887916493027)*x[0]+(-0.648868378605)*x[1]+(-0.0728819090295)*x[2]
        s[0,0,1]=(-1.41771256969)*x[0]+(-1.5907790724)*x[1]+(-0.208960186234)*x[2]
        s[0,1,0]=(-0.846039306108)*x[0]+(0.451942850498)*x[1]+(-0.305613220999)*x[2]
        s[0,1,1]=(-0.990248401344)*x[0]+(0.934658804462)*x[1]+(0.703927399604)*x[2]
        s[1,0,0]=(0.0172966310708)*x[0]+(-0.097716216369)*x[1]+(0.564328624475)*x[2]
        s[1,0,1]=(-0.285571273779)*x[0]+(-0.478508810269)*x[1]+(0.713694990763)*x[2]
        s[1,1,0]=(-1.72203517221)*x[0]+(0.90209571161)*x[1]+(0.0404190111547)*x[2]
        s[1,1,1]=(0.662367771483)*x[0]+(0.683618857708)*x[1]+(0.220237165119)*x[2]
        s[2,0,0]=(-1.23109104652)*x[0]+(-0.344954703402)*x[1]+(-0.845700366118)*x[2]
        s[2,0,1]=(-1.78196237278)*x[0]+(0.352288083535)*x[1]+(1.02993741657)*x[2]
        s[2,1,0]=(-0.101867429012)*x[0]+(-0.810130865002)*x[1]+(0.682007840621)*x[2]
        s[2,1,1]=(0.424977434864)*x[0]+(-0.193692196905)*x[1]+(1.40079084873)*x[2]
        s[3,0,0]=(0.471393445911)*x[0]+(-0.56961804428)*x[1]+(1.1096657312)*x[2]
        s[3,0,1]=(1.09919079441)*x[0]+(1.82571072835)*x[1]+(0.604299398913)*x[2]
        s[3,1,0]=(0.155938760923)*x[0]+(0.841141932215)*x[1]+(0.558240114855)*x[2]
        s[3,1,1]=(-0.0697992380617)*x[0]+(1.28687270053)*x[1]+(-1.65036825798)*x[2]
        s[4,0,0]=(-0.497902210966)*x[0]+(0.495675372467)*x[1]+(1.4729295959)*x[2]
        s[4,0,1]=(-0.278468085457)*x[0]+(0.515340965551)*x[1]+(-0.233864738508)*x[2]
        s[4,1,0]=(0.574455672589)*x[0]+(-1.41224801693)*x[1]+(-0.0851508668768)*x[2]
        s[4,1,1]=(-0.392015012986)*x[0]+(0.211411943089)*x[1]+(0.175695876916)*x[2]
        s[5,0,0]=(-1.3317854233)*x[0]+(1.30129857885)*x[1]+(-0.96126616548)*x[2]
        s[5,0,1]=(-1.00763984246)*x[0]+(0.252485810112)*x[1]+(0.0349533247409)*x[2]
        s[5,1,0]=(1.23304668416)*x[0]+(0.978792435109)*x[1]+(0.0784889665787)*x[2]
        s[5,1,1]=(-0.164813552675)*x[0]+(0.363900322694)*x[1]+(-0.388851362667)*x[2]
        ref[0,0,0,0]=(0.0887916493027)
        ref[0,0,0,1]=(-0.648868378605)
        ref[0,0,0,2]=(-0.0728819090295)
        ref[0,0,1,0]=(-1.41771256969)
        ref[0,0,1,1]=(-1.5907790724)
        ref[0,0,1,2]=(-0.208960186234)
        ref[0,1,0,0]=(-0.846039306108)
        ref[0,1,0,1]=(0.451942850498)
        ref[0,1,0,2]=(-0.305613220999)
        ref[0,1,1,0]=(-0.990248401344)
        ref[0,1,1,1]=(0.934658804462)
        ref[0,1,1,2]=(0.703927399604)
        ref[1,0,0,0]=(0.0172966310708)
        ref[1,0,0,1]=(-0.097716216369)
        ref[1,0,0,2]=(0.564328624475)
        ref[1,0,1,0]=(-0.285571273779)
        ref[1,0,1,1]=(-0.478508810269)
        ref[1,0,1,2]=(0.713694990763)
        ref[1,1,0,0]=(-1.72203517221)
        ref[1,1,0,1]=(0.90209571161)
        ref[1,1,0,2]=(0.0404190111547)
        ref[1,1,1,0]=(0.662367771483)
        ref[1,1,1,1]=(0.683618857708)
        ref[1,1,1,2]=(0.220237165119)
        ref[2,0,0,0]=(-1.23109104652)
        ref[2,0,0,1]=(-0.344954703402)
        ref[2,0,0,2]=(-0.845700366118)
        ref[2,0,1,0]=(-1.78196237278)
        ref[2,0,1,1]=(0.352288083535)
        ref[2,0,1,2]=(1.02993741657)
        ref[2,1,0,0]=(-0.101867429012)
        ref[2,1,0,1]=(-0.810130865002)
        ref[2,1,0,2]=(0.682007840621)
        ref[2,1,1,0]=(0.424977434864)
        ref[2,1,1,1]=(-0.193692196905)
        ref[2,1,1,2]=(1.40079084873)
        ref[3,0,0,0]=(0.471393445911)
        ref[3,0,0,1]=(-0.56961804428)
        ref[3,0,0,2]=(1.1096657312)
        ref[3,0,1,0]=(1.09919079441)
        ref[3,0,1,1]=(1.82571072835)
        ref[3,0,1,2]=(0.604299398913)
        ref[3,1,0,0]=(0.155938760923)
        ref[3,1,0,1]=(0.841141932215)
        ref[3,1,0,2]=(0.558240114855)
        ref[3,1,1,0]=(-0.0697992380617)
        ref[3,1,1,1]=(1.28687270053)
        ref[3,1,1,2]=(-1.65036825798)
        ref[4,0,0,0]=(-0.497902210966)
        ref[4,0,0,1]=(0.495675372467)
        ref[4,0,0,2]=(1.4729295959)
        ref[4,0,1,0]=(-0.278468085457)
        ref[4,0,1,1]=(0.515340965551)
        ref[4,0,1,2]=(-0.233864738508)
        ref[4,1,0,0]=(0.574455672589)
        ref[4,1,0,1]=(-1.41224801693)
        ref[4,1,0,2]=(-0.0851508668768)
        ref[4,1,1,0]=(-0.392015012986)
        ref[4,1,1,1]=(0.211411943089)
        ref[4,1,1,2]=(0.175695876916)
        ref[5,0,0,0]=(-1.3317854233)
        ref[5,0,0,1]=(1.30129857885)
        ref[5,0,0,2]=(-0.96126616548)
        ref[5,0,1,0]=(-1.00763984246)
        ref[5,0,1,1]=(0.252485810112)
        ref[5,0,1,2]=(0.0349533247409)
        ref[5,1,0,0]=(1.23304668416)
        ref[5,1,0,1]=(0.978792435109)
        ref[5,1,0,2]=(0.0784889665787)
        ref[5,1,1,0]=(-0.164813552675)
        ref[5,1,1,1]=(0.363900322694)
        ref[5,1,1,2]=(-0.388851362667)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.049981434374)*x[0]+(1.21186502577)*x[1]
        ref[0]=(-0.049981434374)
        ref[1]=(1.21186502577)
      else:
        s=(-0.0752432283484)*x[0]+(0.11438092577)*x[1]+(-0.434888190035)*x[2]
        ref[0]=(-0.0752432283484)
        ref[1]=(0.11438092577)
        ref[2]=(-0.434888190035)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.456351387646)*x[0]+(-0.116715426711)*x[1]
        s[1]=(-0.854853163245)*x[0]+(-0.00664827097294)*x[1]
        ref[0,0]=(-0.456351387646)
        ref[0,1]=(-0.116715426711)
        ref[1,0]=(-0.854853163245)
        ref[1,1]=(-0.00664827097294)
      else:
        s[0]=(0.623100837308)*x[0]+(0.266907813705)*x[1]+(1.38575183924)*x[2]
        s[1]=(-1.48702785688)*x[0]+(-1.42426212001)*x[1]+(0.317785468787)*x[2]
        ref[0,0]=(0.623100837308)
        ref[0,1]=(0.266907813705)
        ref[0,2]=(1.38575183924)
        ref[1,0]=(-1.48702785688)
        ref[1,1]=(-1.42426212001)
        ref[1,2]=(0.317785468787)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-1.0789465623)*x[0]+(-0.347959175817)*x[1]
        s[0,1]=(-0.551011467115)*x[0]+(1.35264465045)*x[1]
        s[0,2]=(-0.570905806915)*x[0]+(0.0584374344724)*x[1]
        s[0,3]=(0.596777443685)*x[0]+(0.414694389615)*x[1]
        s[0,4]=(0.611144733018)*x[0]+(-0.758856666701)*x[1]
        s[1,0]=(-0.0733659967272)*x[0]+(1.27900296295)*x[1]
        s[1,1]=(-0.305565660896)*x[0]+(0.9434988738)*x[1]
        s[1,2]=(-0.992776824928)*x[0]+(-0.120817150952)*x[1]
        s[1,3]=(0.0902201920474)*x[0]+(1.04097295319)*x[1]
        s[1,4]=(-1.06448209062)*x[0]+(1.13868467732)*x[1]
        s[2,0]=(0.855203060995)*x[0]+(0.707160314719)*x[1]
        s[2,1]=(0.792799717854)*x[0]+(-0.36484734497)*x[1]
        s[2,2]=(-1.14542452528)*x[0]+(0.98847021504)*x[1]
        s[2,3]=(0.145237440613)*x[0]+(0.214714794234)*x[1]
        s[2,4]=(0.179556206154)*x[0]+(-1.21107721861)*x[1]
        s[3,0]=(0.700250733001)*x[0]+(1.25331090966)*x[1]
        s[3,1]=(0.258700850981)*x[0]+(-0.637474578328)*x[1]
        s[3,2]=(0.382709435533)*x[0]+(1.29575763411)*x[1]
        s[3,3]=(-0.254008873764)*x[0]+(-1.54340970006)*x[1]
        s[3,4]=(-0.658082774987)*x[0]+(0.647639696672)*x[1]
        ref[0,0,0]=(-1.0789465623)
        ref[0,0,1]=(-0.347959175817)
        ref[0,1,0]=(-0.551011467115)
        ref[0,1,1]=(1.35264465045)
        ref[0,2,0]=(-0.570905806915)
        ref[0,2,1]=(0.0584374344724)
        ref[0,3,0]=(0.596777443685)
        ref[0,3,1]=(0.414694389615)
        ref[0,4,0]=(0.611144733018)
        ref[0,4,1]=(-0.758856666701)
        ref[1,0,0]=(-0.0733659967272)
        ref[1,0,1]=(1.27900296295)
        ref[1,1,0]=(-0.305565660896)
        ref[1,1,1]=(0.9434988738)
        ref[1,2,0]=(-0.992776824928)
        ref[1,2,1]=(-0.120817150952)
        ref[1,3,0]=(0.0902201920474)
        ref[1,3,1]=(1.04097295319)
        ref[1,4,0]=(-1.06448209062)
        ref[1,4,1]=(1.13868467732)
        ref[2,0,0]=(0.855203060995)
        ref[2,0,1]=(0.707160314719)
        ref[2,1,0]=(0.792799717854)
        ref[2,1,1]=(-0.36484734497)
        ref[2,2,0]=(-1.14542452528)
        ref[2,2,1]=(0.98847021504)
        ref[2,3,0]=(0.145237440613)
        ref[2,3,1]=(0.214714794234)
        ref[2,4,0]=(0.179556206154)
        ref[2,4,1]=(-1.21107721861)
        ref[3,0,0]=(0.700250733001)
        ref[3,0,1]=(1.25331090966)
        ref[3,1,0]=(0.258700850981)
        ref[3,1,1]=(-0.637474578328)
        ref[3,2,0]=(0.382709435533)
        ref[3,2,1]=(1.29575763411)
        ref[3,3,0]=(-0.254008873764)
        ref[3,3,1]=(-1.54340970006)
        ref[3,4,0]=(-0.658082774987)
        ref[3,4,1]=(0.647639696672)
      else:
        s[0,0]=(1.12878816262)*x[0]+(-1.74114589021)*x[1]+(0.248765786072)*x[2]
        s[0,1]=(1.72583176211)*x[0]+(-1.23114549996)*x[1]+(-0.203157732085)*x[2]
        s[0,2]=(-0.422314546839)*x[0]+(0.224260123741)*x[1]+(0.171252253031)*x[2]
        s[0,3]=(-0.870712689882)*x[0]+(-0.555793590179)*x[1]+(0.290927600809)*x[2]
        s[0,4]=(-0.920833142524)*x[0]+(-0.0497652513943)*x[1]+(-0.39718342747)*x[2]
        s[1,0]=(-0.727450597273)*x[0]+(1.09323195342)*x[1]+(0.298478430357)*x[2]
        s[1,1]=(0.12608256775)*x[0]+(-0.459788156475)*x[1]+(1.13249234657)*x[2]
        s[1,2]=(0.898608507221)*x[0]+(-0.46334527476)*x[1]+(-0.445291041098)*x[2]
        s[1,3]=(-0.327616894595)*x[0]+(-0.288577792837)*x[1]+(1.26565355504)*x[2]
        s[1,4]=(0.0630186390647)*x[0]+(-0.383734001362)*x[1]+(-0.116975162517)*x[2]
        s[2,0]=(-0.309394005751)*x[0]+(-0.12474002694)*x[1]+(0.699555331484)*x[2]
        s[2,1]=(0.997752898742)*x[0]+(-0.183982572346)*x[1]+(1.22982190192)*x[2]
        s[2,2]=(1.03496586697)*x[0]+(-0.097256212804)*x[1]+(1.42052511301)*x[2]
        s[2,3]=(0.614378729663)*x[0]+(0.154397435133)*x[1]+(-0.863004330742)*x[2]
        s[2,4]=(1.20904264903)*x[0]+(-0.452388361293)*x[1]+(-1.13286356474)*x[2]
        s[3,0]=(-0.812538381038)*x[0]+(0.00308372111735)*x[1]+(-0.117937095444)*x[2]
        s[3,1]=(0.45479320042)*x[0]+(-0.45927716926)*x[1]+(0.4787228586)*x[2]
        s[3,2]=(-0.487685328452)*x[0]+(0.871241301367)*x[1]+(0.78769505788)*x[2]
        s[3,3]=(0.860788048428)*x[0]+(1.39676274038)*x[1]+(0.407326483464)*x[2]
        s[3,4]=(0.00409247820271)*x[0]+(0.44466332801)*x[1]+(-0.66721280275)*x[2]
        ref[0,0,0]=(1.12878816262)
        ref[0,0,1]=(-1.74114589021)
        ref[0,0,2]=(0.248765786072)
        ref[0,1,0]=(1.72583176211)
        ref[0,1,1]=(-1.23114549996)
        ref[0,1,2]=(-0.203157732085)
        ref[0,2,0]=(-0.422314546839)
        ref[0,2,1]=(0.224260123741)
        ref[0,2,2]=(0.171252253031)
        ref[0,3,0]=(-0.870712689882)
        ref[0,3,1]=(-0.555793590179)
        ref[0,3,2]=(0.290927600809)
        ref[0,4,0]=(-0.920833142524)
        ref[0,4,1]=(-0.0497652513943)
        ref[0,4,2]=(-0.39718342747)
        ref[1,0,0]=(-0.727450597273)
        ref[1,0,1]=(1.09323195342)
        ref[1,0,2]=(0.298478430357)
        ref[1,1,0]=(0.12608256775)
        ref[1,1,1]=(-0.459788156475)
        ref[1,1,2]=(1.13249234657)
        ref[1,2,0]=(0.898608507221)
        ref[1,2,1]=(-0.46334527476)
        ref[1,2,2]=(-0.445291041098)
        ref[1,3,0]=(-0.327616894595)
        ref[1,3,1]=(-0.288577792837)
        ref[1,3,2]=(1.26565355504)
        ref[1,4,0]=(0.0630186390647)
        ref[1,4,1]=(-0.383734001362)
        ref[1,4,2]=(-0.116975162517)
        ref[2,0,0]=(-0.309394005751)
        ref[2,0,1]=(-0.12474002694)
        ref[2,0,2]=(0.699555331484)
        ref[2,1,0]=(0.997752898742)
        ref[2,1,1]=(-0.183982572346)
        ref[2,1,2]=(1.22982190192)
        ref[2,2,0]=(1.03496586697)
        ref[2,2,1]=(-0.097256212804)
        ref[2,2,2]=(1.42052511301)
        ref[2,3,0]=(0.614378729663)
        ref[2,3,1]=(0.154397435133)
        ref[2,3,2]=(-0.863004330742)
        ref[2,4,0]=(1.20904264903)
        ref[2,4,1]=(-0.452388361293)
        ref[2,4,2]=(-1.13286356474)
        ref[3,0,0]=(-0.812538381038)
        ref[3,0,1]=(0.00308372111735)
        ref[3,0,2]=(-0.117937095444)
        ref[3,1,0]=(0.45479320042)
        ref[3,1,1]=(-0.45927716926)
        ref[3,1,2]=(0.4787228586)
        ref[3,2,0]=(-0.487685328452)
        ref[3,2,1]=(0.871241301367)
        ref[3,2,2]=(0.78769505788)
        ref[3,3,0]=(0.860788048428)
        ref[3,3,1]=(1.39676274038)
        ref[3,3,2]=(0.407326483464)
        ref[3,4,0]=(0.00409247820271)
        ref[3,4,1]=(0.44466332801)
        ref[3,4,2]=(-0.66721280275)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnBoundary_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.905736881137)*x[0]+(0.694861875477)*x[1]
        s[0,0,1]=(-1.45890910436)*x[0]+(-1.52565465047)*x[1]
        s[0,1,0]=(-0.312122938001)*x[0]+(-0.868878770447)*x[1]
        s[0,1,1]=(1.54725692119)*x[0]+(0.536122055612)*x[1]
        s[1,0,0]=(0.196190980599)*x[0]+(0.328328643332)*x[1]
        s[1,0,1]=(-1.08863159071)*x[0]+(-1.33920996273)*x[1]
        s[1,1,0]=(1.73274170273)*x[0]+(-0.958793175994)*x[1]
        s[1,1,1]=(0.182836505159)*x[0]+(-0.121541234518)*x[1]
        s[2,0,0]=(0.144461277784)*x[0]+(1.83781976673)*x[1]
        s[2,0,1]=(-0.194587892421)*x[0]+(-1.27090147949)*x[1]
        s[2,1,0]=(0.379488182396)*x[0]+(0.030493966182)*x[1]
        s[2,1,1]=(1.09003510354)*x[0]+(-0.454578719663)*x[1]
        s[3,0,0]=(1.58446081096)*x[0]+(-0.52113508443)*x[1]
        s[3,0,1]=(-1.33593796269)*x[0]+(-0.165961948897)*x[1]
        s[3,1,0]=(0.377402371367)*x[0]+(-0.0184973476961)*x[1]
        s[3,1,1]=(0.764313855665)*x[0]+(0.294443818887)*x[1]
        s[4,0,0]=(-0.180639063236)*x[0]+(1.75864081648)*x[1]
        s[4,0,1]=(-1.62662138708)*x[0]+(1.81611541058)*x[1]
        s[4,1,0]=(1.37678767999)*x[0]+(-0.271115982236)*x[1]
        s[4,1,1]=(1.32861834819)*x[0]+(0.768596947238)*x[1]
        s[5,0,0]=(1.39342920577)*x[0]+(0.155666506926)*x[1]
        s[5,0,1]=(0.0397474261373)*x[0]+(0.921852245407)*x[1]
        s[5,1,0]=(-0.950337694161)*x[0]+(-0.011042822071)*x[1]
        s[5,1,1]=(0.0369412335225)*x[0]+(-0.473468894798)*x[1]
        ref[0,0,0,0]=(-0.905736881137)
        ref[0,0,0,1]=(0.694861875477)
        ref[0,0,1,0]=(-1.45890910436)
        ref[0,0,1,1]=(-1.52565465047)
        ref[0,1,0,0]=(-0.312122938001)
        ref[0,1,0,1]=(-0.868878770447)
        ref[0,1,1,0]=(1.54725692119)
        ref[0,1,1,1]=(0.536122055612)
        ref[1,0,0,0]=(0.196190980599)
        ref[1,0,0,1]=(0.328328643332)
        ref[1,0,1,0]=(-1.08863159071)
        ref[1,0,1,1]=(-1.33920996273)
        ref[1,1,0,0]=(1.73274170273)
        ref[1,1,0,1]=(-0.958793175994)
        ref[1,1,1,0]=(0.182836505159)
        ref[1,1,1,1]=(-0.121541234518)
        ref[2,0,0,0]=(0.144461277784)
        ref[2,0,0,1]=(1.83781976673)
        ref[2,0,1,0]=(-0.194587892421)
        ref[2,0,1,1]=(-1.27090147949)
        ref[2,1,0,0]=(0.379488182396)
        ref[2,1,0,1]=(0.030493966182)
        ref[2,1,1,0]=(1.09003510354)
        ref[2,1,1,1]=(-0.454578719663)
        ref[3,0,0,0]=(1.58446081096)
        ref[3,0,0,1]=(-0.52113508443)
        ref[3,0,1,0]=(-1.33593796269)
        ref[3,0,1,1]=(-0.165961948897)
        ref[3,1,0,0]=(0.377402371367)
        ref[3,1,0,1]=(-0.0184973476961)
        ref[3,1,1,0]=(0.764313855665)
        ref[3,1,1,1]=(0.294443818887)
        ref[4,0,0,0]=(-0.180639063236)
        ref[4,0,0,1]=(1.75864081648)
        ref[4,0,1,0]=(-1.62662138708)
        ref[4,0,1,1]=(1.81611541058)
        ref[4,1,0,0]=(1.37678767999)
        ref[4,1,0,1]=(-0.271115982236)
        ref[4,1,1,0]=(1.32861834819)
        ref[4,1,1,1]=(0.768596947238)
        ref[5,0,0,0]=(1.39342920577)
        ref[5,0,0,1]=(0.155666506926)
        ref[5,0,1,0]=(0.0397474261373)
        ref[5,0,1,1]=(0.921852245407)
        ref[5,1,0,0]=(-0.950337694161)
        ref[5,1,0,1]=(-0.011042822071)
        ref[5,1,1,0]=(0.0369412335225)
        ref[5,1,1,1]=(-0.473468894798)
      else:
        s[0,0,0]=(0.0887916493027)*x[0]+(-0.648868378605)*x[1]+(-0.0728819090295)*x[2]
        s[0,0,1]=(-1.41771256969)*x[0]+(-1.5907790724)*x[1]+(-0.208960186234)*x[2]
        s[0,1,0]=(-0.846039306108)*x[0]+(0.451942850498)*x[1]+(-0.305613220999)*x[2]
        s[0,1,1]=(-0.990248401344)*x[0]+(0.934658804462)*x[1]+(0.703927399604)*x[2]
        s[1,0,0]=(0.0172966310708)*x[0]+(-0.097716216369)*x[1]+(0.564328624475)*x[2]
        s[1,0,1]=(-0.285571273779)*x[0]+(-0.478508810269)*x[1]+(0.713694990763)*x[2]
        s[1,1,0]=(-1.72203517221)*x[0]+(0.90209571161)*x[1]+(0.0404190111547)*x[2]
        s[1,1,1]=(0.662367771483)*x[0]+(0.683618857708)*x[1]+(0.220237165119)*x[2]
        s[2,0,0]=(-1.23109104652)*x[0]+(-0.344954703402)*x[1]+(-0.845700366118)*x[2]
        s[2,0,1]=(-1.78196237278)*x[0]+(0.352288083535)*x[1]+(1.02993741657)*x[2]
        s[2,1,0]=(-0.101867429012)*x[0]+(-0.810130865002)*x[1]+(0.682007840621)*x[2]
        s[2,1,1]=(0.424977434864)*x[0]+(-0.193692196905)*x[1]+(1.40079084873)*x[2]
        s[3,0,0]=(0.471393445911)*x[0]+(-0.56961804428)*x[1]+(1.1096657312)*x[2]
        s[3,0,1]=(1.09919079441)*x[0]+(1.82571072835)*x[1]+(0.604299398913)*x[2]
        s[3,1,0]=(0.155938760923)*x[0]+(0.841141932215)*x[1]+(0.558240114855)*x[2]
        s[3,1,1]=(-0.0697992380617)*x[0]+(1.28687270053)*x[1]+(-1.65036825798)*x[2]
        s[4,0,0]=(-0.497902210966)*x[0]+(0.495675372467)*x[1]+(1.4729295959)*x[2]
        s[4,0,1]=(-0.278468085457)*x[0]+(0.515340965551)*x[1]+(-0.233864738508)*x[2]
        s[4,1,0]=(0.574455672589)*x[0]+(-1.41224801693)*x[1]+(-0.0851508668768)*x[2]
        s[4,1,1]=(-0.392015012986)*x[0]+(0.211411943089)*x[1]+(0.175695876916)*x[2]
        s[5,0,0]=(-1.3317854233)*x[0]+(1.30129857885)*x[1]+(-0.96126616548)*x[2]
        s[5,0,1]=(-1.00763984246)*x[0]+(0.252485810112)*x[1]+(0.0349533247409)*x[2]
        s[5,1,0]=(1.23304668416)*x[0]+(0.978792435109)*x[1]+(0.0784889665787)*x[2]
        s[5,1,1]=(-0.164813552675)*x[0]+(0.363900322694)*x[1]+(-0.388851362667)*x[2]
        ref[0,0,0,0]=(0.0887916493027)
        ref[0,0,0,1]=(-0.648868378605)
        ref[0,0,0,2]=(-0.0728819090295)
        ref[0,0,1,0]=(-1.41771256969)
        ref[0,0,1,1]=(-1.5907790724)
        ref[0,0,1,2]=(-0.208960186234)
        ref[0,1,0,0]=(-0.846039306108)
        ref[0,1,0,1]=(0.451942850498)
        ref[0,1,0,2]=(-0.305613220999)
        ref[0,1,1,0]=(-0.990248401344)
        ref[0,1,1,1]=(0.934658804462)
        ref[0,1,1,2]=(0.703927399604)
        ref[1,0,0,0]=(0.0172966310708)
        ref[1,0,0,1]=(-0.097716216369)
        ref[1,0,0,2]=(0.564328624475)
        ref[1,0,1,0]=(-0.285571273779)
        ref[1,0,1,1]=(-0.478508810269)
        ref[1,0,1,2]=(0.713694990763)
        ref[1,1,0,0]=(-1.72203517221)
        ref[1,1,0,1]=(0.90209571161)
        ref[1,1,0,2]=(0.0404190111547)
        ref[1,1,1,0]=(0.662367771483)
        ref[1,1,1,1]=(0.683618857708)
        ref[1,1,1,2]=(0.220237165119)
        ref[2,0,0,0]=(-1.23109104652)
        ref[2,0,0,1]=(-0.344954703402)
        ref[2,0,0,2]=(-0.845700366118)
        ref[2,0,1,0]=(-1.78196237278)
        ref[2,0,1,1]=(0.352288083535)
        ref[2,0,1,2]=(1.02993741657)
        ref[2,1,0,0]=(-0.101867429012)
        ref[2,1,0,1]=(-0.810130865002)
        ref[2,1,0,2]=(0.682007840621)
        ref[2,1,1,0]=(0.424977434864)
        ref[2,1,1,1]=(-0.193692196905)
        ref[2,1,1,2]=(1.40079084873)
        ref[3,0,0,0]=(0.471393445911)
        ref[3,0,0,1]=(-0.56961804428)
        ref[3,0,0,2]=(1.1096657312)
        ref[3,0,1,0]=(1.09919079441)
        ref[3,0,1,1]=(1.82571072835)
        ref[3,0,1,2]=(0.604299398913)
        ref[3,1,0,0]=(0.155938760923)
        ref[3,1,0,1]=(0.841141932215)
        ref[3,1,0,2]=(0.558240114855)
        ref[3,1,1,0]=(-0.0697992380617)
        ref[3,1,1,1]=(1.28687270053)
        ref[3,1,1,2]=(-1.65036825798)
        ref[4,0,0,0]=(-0.497902210966)
        ref[4,0,0,1]=(0.495675372467)
        ref[4,0,0,2]=(1.4729295959)
        ref[4,0,1,0]=(-0.278468085457)
        ref[4,0,1,1]=(0.515340965551)
        ref[4,0,1,2]=(-0.233864738508)
        ref[4,1,0,0]=(0.574455672589)
        ref[4,1,0,1]=(-1.41224801693)
        ref[4,1,0,2]=(-0.0851508668768)
        ref[4,1,1,0]=(-0.392015012986)
        ref[4,1,1,1]=(0.211411943089)
        ref[4,1,1,2]=(0.175695876916)
        ref[5,0,0,0]=(-1.3317854233)
        ref[5,0,0,1]=(1.30129857885)
        ref[5,0,0,2]=(-0.96126616548)
        ref[5,0,1,0]=(-1.00763984246)
        ref[5,0,1,1]=(0.252485810112)
        ref[5,0,1,2]=(0.0349533247409)
        ref[5,1,0,0]=(1.23304668416)
        ref[5,1,0,1]=(0.978792435109)
        ref[5,1,0,2]=(0.0784889665787)
        ref[5,1,1,0]=(-0.164813552675)
        ref[5,1,1,1]=(0.363900322694)
        ref[5,1,1,2]=(-0.388851362667)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(-0.447987377779)*x[0]**o+(-0.3185298109)*x[0]+(0.748689324197)*x[1]**o+(-0.979369398128)*x[1]
        ref[0]=o*(-0.447987377779)*x_ref[0]**(o-1)+(-0.3185298109)
        ref[1]=o*(0.748689324197)*x_ref[1]**(o-1)+(-0.979369398128)
      else:
        arg=(-0.991175677257)*x[0]**o+(0.486310659385)*x[0]+(0.366862476014)*x[1]**o+(0.718311286349)*x[1]+(-0.593137377946)*x[2]**o+(-0.98259729892)*x[2]
        ref[0]=o*(-0.991175677257)*x_ref[0]**(o-1)+(0.486310659385)
        ref[1]=o*(0.366862476014)*x_ref[1]**(o-1)+(0.718311286349)
        ref[2]=o*(-0.593137377946)*x_ref[2]**(o-1)+(-0.98259729892)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(0.532427358862)*x[0]**o+(-0.699829406888)*x[0]+(-0.378181075858)*x[1]**o+(-0.163897932934)*x[1]
        arg[1]=(0.76393225396)*x[0]**o+(0.87331212216)*x[0]+(0.0261520274749)*x[1]**o+(-0.475078274702)*x[1]
        ref[0,0]=o*(0.532427358862)*x_ref[0]**(o-1)+(-0.699829406888)
        ref[0,1]=o*(-0.378181075858)*x_ref[1]**(o-1)+(-0.163897932934)
        ref[1,0]=o*(0.76393225396)*x_ref[0]**(o-1)+(0.87331212216)
        ref[1,1]=o*(0.0261520274749)*x_ref[1]**(o-1)+(-0.475078274702)
      else:
        arg[0]=(-0.559640148983)*x[0]**o+(0.868360358061)*x[0]+(0.148807680806)*x[1]**o+(-0.488426629736)*x[1]+(0.99381766343)*x[2]**o+(-0.0241703848022)*x[2]
        arg[1]=(-0.00270447073555)*x[0]**o+(0.599732416474)*x[0]+(-0.455449579349)*x[1]**o+(-0.791088816655)*x[1]+(0.850362671968)*x[2]**o+(-0.180726138889)*x[2]
        ref[0,0]=o*(-0.559640148983)*x_ref[0]**(o-1)+(0.868360358061)
        ref[0,1]=o*(0.148807680806)*x_ref[1]**(o-1)+(-0.488426629736)
        ref[0,2]=o*(0.99381766343)*x_ref[2]**(o-1)+(-0.0241703848022)
        ref[1,0]=o*(-0.00270447073555)*x_ref[0]**(o-1)+(0.599732416474)
        ref[1,1]=o*(-0.455449579349)*x_ref[1]**(o-1)+(-0.791088816655)
        ref[1,2]=o*(0.850362671968)*x_ref[2]**(o-1)+(-0.180726138889)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.157399152858)*x[0]**o+(0.00598352783814)*x[0]+(-0.904853822462)*x[1]**o+(0.767645769461)*x[1]
        arg[0,1]=(0.764556091331)*x[0]**o+(-0.167733604181)*x[0]+(0.723254547109)*x[1]**o+(-0.863633737324)*x[1]
        arg[0,2]=(0.480913138549)*x[0]**o+(0.505712705322)*x[0]+(-0.295398027463)*x[1]**o+(-0.0497846093953)*x[1]
        arg[0,3]=(-0.957208239652)*x[0]**o+(0.608346818502)*x[0]+(-0.194623401251)*x[1]**o+(0.670671588806)*x[1]
        arg[0,4]=(-0.179165159786)*x[0]**o+(-0.146090255471)*x[0]+(-0.413009197676)*x[1]**o+(-0.168508218849)*x[1]
        arg[1,0]=(0.659914628278)*x[0]**o+(0.123177996909)*x[0]+(0.713971230628)*x[1]**o+(-0.687569461154)*x[1]
        arg[1,1]=(0.857554477911)*x[0]**o+(0.119604916634)*x[0]+(-0.123185856515)*x[1]**o+(0.968800869965)*x[1]
        arg[1,2]=(0.566596883167)*x[0]**o+(-0.667227639224)*x[0]+(0.29838343622)*x[1]**o+(0.0172896834006)*x[1]
        arg[1,3]=(0.340418732306)*x[0]**o+(-0.115031635126)*x[0]+(0.212495048781)*x[1]**o+(-0.690939141829)*x[1]
        arg[1,4]=(-0.513225187356)*x[0]**o+(0.975159534619)*x[0]+(0.159694719764)*x[1]**o+(-0.441973185517)*x[1]
        arg[2,0]=(0.00598247134499)*x[0]**o+(-0.680866157243)*x[0]+(0.198740044836)*x[1]**o+(-0.776778848701)*x[1]
        arg[2,1]=(0.385572341196)*x[0]**o+(-0.911229388405)*x[0]+(-0.869563350649)*x[1]**o+(-0.453073292811)*x[1]
        arg[2,2]=(-0.678405510051)*x[0]**o+(0.570159849365)*x[0]+(-0.998909274112)*x[1]**o+(-0.56867397999)*x[1]
        arg[2,3]=(0.592650344218)*x[0]**o+(-0.990760992864)*x[0]+(0.54832364584)*x[1]**o+(-0.62848423403)*x[1]
        arg[2,4]=(-0.267222595774)*x[0]**o+(-0.398631721649)*x[0]+(-0.139351142788)*x[1]**o+(-0.973211734322)*x[1]
        arg[3,0]=(-0.748270220449)*x[0]**o+(-0.432648285296)*x[0]+(-0.572322019432)*x[1]**o+(-0.361258377472)*x[1]
        arg[3,1]=(-0.872970937458)*x[0]**o+(-0.412714429527)*x[0]+(0.28965384698)*x[1]**o+(0.0899128505144)*x[1]
        arg[3,2]=(-0.448552675711)*x[0]**o+(-0.0531593149976)*x[0]+(-0.445359129375)*x[1]**o+(-0.357551800127)*x[1]
        arg[3,3]=(-0.120676057603)*x[0]**o+(-0.736959379114)*x[0]+(-0.437089829553)*x[1]**o+(-0.272117257464)*x[1]
        arg[3,4]=(0.342279309395)*x[0]**o+(0.182206506048)*x[0]+(0.843948330758)*x[1]**o+(0.165354303251)*x[1]
        ref[0,0,0]=o*(0.157399152858)*x_ref[0]**(o-1)+(0.00598352783814)
        ref[0,0,1]=o*(-0.904853822462)*x_ref[1]**(o-1)+(0.767645769461)
        ref[0,1,0]=o*(0.764556091331)*x_ref[0]**(o-1)+(-0.167733604181)
        ref[0,1,1]=o*(0.723254547109)*x_ref[1]**(o-1)+(-0.863633737324)
        ref[0,2,0]=o*(0.480913138549)*x_ref[0]**(o-1)+(0.505712705322)
        ref[0,2,1]=o*(-0.295398027463)*x_ref[1]**(o-1)+(-0.0497846093953)
        ref[0,3,0]=o*(-0.957208239652)*x_ref[0]**(o-1)+(0.608346818502)
        ref[0,3,1]=o*(-0.194623401251)*x_ref[1]**(o-1)+(0.670671588806)
        ref[0,4,0]=o*(-0.179165159786)*x_ref[0]**(o-1)+(-0.146090255471)
        ref[0,4,1]=o*(-0.413009197676)*x_ref[1]**(o-1)+(-0.168508218849)
        ref[1,0,0]=o*(0.659914628278)*x_ref[0]**(o-1)+(0.123177996909)
        ref[1,0,1]=o*(0.713971230628)*x_ref[1]**(o-1)+(-0.687569461154)
        ref[1,1,0]=o*(0.857554477911)*x_ref[0]**(o-1)+(0.119604916634)
        ref[1,1,1]=o*(-0.123185856515)*x_ref[1]**(o-1)+(0.968800869965)
        ref[1,2,0]=o*(0.566596883167)*x_ref[0]**(o-1)+(-0.667227639224)
        ref[1,2,1]=o*(0.29838343622)*x_ref[1]**(o-1)+(0.0172896834006)
        ref[1,3,0]=o*(0.340418732306)*x_ref[0]**(o-1)+(-0.115031635126)
        ref[1,3,1]=o*(0.212495048781)*x_ref[1]**(o-1)+(-0.690939141829)
        ref[1,4,0]=o*(-0.513225187356)*x_ref[0]**(o-1)+(0.975159534619)
        ref[1,4,1]=o*(0.159694719764)*x_ref[1]**(o-1)+(-0.441973185517)
        ref[2,0,0]=o*(0.00598247134499)*x_ref[0]**(o-1)+(-0.680866157243)
        ref[2,0,1]=o*(0.198740044836)*x_ref[1]**(o-1)+(-0.776778848701)
        ref[2,1,0]=o*(0.385572341196)*x_ref[0]**(o-1)+(-0.911229388405)
        ref[2,1,1]=o*(-0.869563350649)*x_ref[1]**(o-1)+(-0.453073292811)
        ref[2,2,0]=o*(-0.678405510051)*x_ref[0]**(o-1)+(0.570159849365)
        ref[2,2,1]=o*(-0.998909274112)*x_ref[1]**(o-1)+(-0.56867397999)
        ref[2,3,0]=o*(0.592650344218)*x_ref[0]**(o-1)+(-0.990760992864)
        ref[2,3,1]=o*(0.54832364584)*x_ref[1]**(o-1)+(-0.62848423403)
        ref[2,4,0]=o*(-0.267222595774)*x_ref[0]**(o-1)+(-0.398631721649)
        ref[2,4,1]=o*(-0.139351142788)*x_ref[1]**(o-1)+(-0.973211734322)
        ref[3,0,0]=o*(-0.748270220449)*x_ref[0]**(o-1)+(-0.432648285296)
        ref[3,0,1]=o*(-0.572322019432)*x_ref[1]**(o-1)+(-0.361258377472)
        ref[3,1,0]=o*(-0.872970937458)*x_ref[0]**(o-1)+(-0.412714429527)
        ref[3,1,1]=o*(0.28965384698)*x_ref[1]**(o-1)+(0.0899128505144)
        ref[3,2,0]=o*(-0.448552675711)*x_ref[0]**(o-1)+(-0.0531593149976)
        ref[3,2,1]=o*(-0.445359129375)*x_ref[1]**(o-1)+(-0.357551800127)
        ref[3,3,0]=o*(-0.120676057603)*x_ref[0]**(o-1)+(-0.736959379114)
        ref[3,3,1]=o*(-0.437089829553)*x_ref[1]**(o-1)+(-0.272117257464)
        ref[3,4,0]=o*(0.342279309395)*x_ref[0]**(o-1)+(0.182206506048)
        ref[3,4,1]=o*(0.843948330758)*x_ref[1]**(o-1)+(0.165354303251)
      else:
        arg[0,0]=(-0.912611504385)*x[0]**o+(0.730660242466)*x[0]+(-0.807306513948)*x[1]**o+(-0.611619910819)*x[1]+(-0.567616988907)*x[2]**o+(0.554411283236)*x[2]
        arg[0,1]=(-0.900956431107)*x[0]**o+(-0.731441474835)*x[0]+(0.366905510731)*x[1]**o+(0.444763254457)*x[1]+(0.223337203463)*x[2]**o+(0.0823258304577)*x[2]
        arg[0,2]=(-0.300262400629)*x[0]**o+(0.781835992318)*x[0]+(-0.961025746467)*x[1]**o+(-0.0255420888548)*x[1]+(-0.195336914392)*x[2]**o+(0.631006440014)*x[2]
        arg[0,3]=(-0.0362225627667)*x[0]**o+(-0.258309391935)*x[0]+(0.915114590957)*x[1]**o+(-0.0261151730116)*x[1]+(-0.0359136191693)*x[2]**o+(0.34837541797)*x[2]
        arg[0,4]=(0.718483283111)*x[0]**o+(-0.981945434022)*x[0]+(0.715616160215)*x[1]**o+(-0.213673997408)*x[1]+(0.703864386687)*x[2]**o+(-0.258970624794)*x[2]
        arg[1,0]=(0.562339421788)*x[0]**o+(-0.469800316995)*x[0]+(-0.58860202695)*x[1]**o+(0.592493291759)*x[1]+(-0.0747446509827)*x[2]**o+(0.459846703357)*x[2]
        arg[1,1]=(-0.739792007188)*x[0]**o+(0.69804646711)*x[0]+(0.685574577582)*x[1]**o+(0.34906822136)*x[1]+(-0.346626103556)*x[2]**o+(-0.186290616908)*x[2]
        arg[1,2]=(0.605479936712)*x[0]**o+(0.23811484281)*x[0]+(-0.18270059933)*x[1]**o+(-0.883573478283)*x[1]+(0.558417469124)*x[2]**o+(0.0921284647122)*x[2]
        arg[1,3]=(0.829453952054)*x[0]**o+(0.0943225421949)*x[0]+(-0.377472410614)*x[1]**o+(0.422596289908)*x[1]+(0.992700203574)*x[2]**o+(0.954704595957)*x[2]
        arg[1,4]=(0.713437745608)*x[0]**o+(-0.90755469394)*x[0]+(-0.116969245983)*x[1]**o+(-0.615609779318)*x[1]+(-0.631014272933)*x[2]**o+(0.196556707186)*x[2]
        arg[2,0]=(-0.643142569205)*x[0]**o+(-0.522571508238)*x[0]+(-0.221094875426)*x[1]**o+(0.00263621704394)*x[1]+(-0.346088170378)*x[2]**o+(-0.323346603492)*x[2]
        arg[2,1]=(0.121192120404)*x[0]**o+(-0.910581465748)*x[0]+(0.661403514322)*x[1]**o+(0.314356472557)*x[1]+(0.740349932503)*x[2]**o+(0.397923840477)*x[2]
        arg[2,2]=(0.712887835251)*x[0]**o+(0.802827824977)*x[0]+(0.225275507294)*x[1]**o+(0.00655056099106)*x[1]+(-0.0211189465513)*x[2]**o+(-0.672450969446)*x[2]
        arg[2,3]=(0.220976204507)*x[0]**o+(0.384529933174)*x[0]+(-0.824967564367)*x[1]**o+(0.382079183434)*x[1]+(0.975192346085)*x[2]**o+(-0.845325222223)*x[2]
        arg[2,4]=(-0.316551069119)*x[0]**o+(0.194934627874)*x[0]+(0.192096230967)*x[1]**o+(-0.423775967466)*x[1]+(-0.669140845203)*x[2]**o+(0.367813824992)*x[2]
        arg[3,0]=(-0.361410077778)*x[0]**o+(0.377521588398)*x[0]+(-0.795093364568)*x[1]**o+(0.475257686757)*x[1]+(-0.996619080341)*x[2]**o+(-0.909105543506)*x[2]
        arg[3,1]=(-0.324100979259)*x[0]**o+(-0.950807456193)*x[0]+(-0.141687301441)*x[1]**o+(0.153568029515)*x[1]+(-0.698514966417)*x[2]**o+(0.312395968777)*x[2]
        arg[3,2]=(-0.997783460035)*x[0]**o+(0.328724967881)*x[0]+(-0.561572087716)*x[1]**o+(-0.0148890799876)*x[1]+(0.236142454998)*x[2]**o+(-0.36513107554)*x[2]
        arg[3,3]=(0.750818510719)*x[0]**o+(-0.484598204959)*x[0]+(-0.599436804638)*x[1]**o+(-0.236232976078)*x[1]+(-0.0789148183785)*x[2]**o+(-0.74437164566)*x[2]
        arg[3,4]=(-0.730664688591)*x[0]**o+(-0.94753100979)*x[0]+(-0.671463334034)*x[1]**o+(-0.139992464354)*x[1]+(0.286757519111)*x[2]**o+(0.512453551537)*x[2]
        ref[0,0,0]=o*(-0.912611504385)*x_ref[0]**(o-1)+(0.730660242466)
        ref[0,0,1]=o*(-0.807306513948)*x_ref[1]**(o-1)+(-0.611619910819)
        ref[0,0,2]=o*(-0.567616988907)*x_ref[2]**(o-1)+(0.554411283236)
        ref[0,1,0]=o*(-0.900956431107)*x_ref[0]**(o-1)+(-0.731441474835)
        ref[0,1,1]=o*(0.366905510731)*x_ref[1]**(o-1)+(0.444763254457)
        ref[0,1,2]=o*(0.223337203463)*x_ref[2]**(o-1)+(0.0823258304577)
        ref[0,2,0]=o*(-0.300262400629)*x_ref[0]**(o-1)+(0.781835992318)
        ref[0,2,1]=o*(-0.961025746467)*x_ref[1]**(o-1)+(-0.0255420888548)
        ref[0,2,2]=o*(-0.195336914392)*x_ref[2]**(o-1)+(0.631006440014)
        ref[0,3,0]=o*(-0.0362225627667)*x_ref[0]**(o-1)+(-0.258309391935)
        ref[0,3,1]=o*(0.915114590957)*x_ref[1]**(o-1)+(-0.0261151730116)
        ref[0,3,2]=o*(-0.0359136191693)*x_ref[2]**(o-1)+(0.34837541797)
        ref[0,4,0]=o*(0.718483283111)*x_ref[0]**(o-1)+(-0.981945434022)
        ref[0,4,1]=o*(0.715616160215)*x_ref[1]**(o-1)+(-0.213673997408)
        ref[0,4,2]=o*(0.703864386687)*x_ref[2]**(o-1)+(-0.258970624794)
        ref[1,0,0]=o*(0.562339421788)*x_ref[0]**(o-1)+(-0.469800316995)
        ref[1,0,1]=o*(-0.58860202695)*x_ref[1]**(o-1)+(0.592493291759)
        ref[1,0,2]=o*(-0.0747446509827)*x_ref[2]**(o-1)+(0.459846703357)
        ref[1,1,0]=o*(-0.739792007188)*x_ref[0]**(o-1)+(0.69804646711)
        ref[1,1,1]=o*(0.685574577582)*x_ref[1]**(o-1)+(0.34906822136)
        ref[1,1,2]=o*(-0.346626103556)*x_ref[2]**(o-1)+(-0.186290616908)
        ref[1,2,0]=o*(0.605479936712)*x_ref[0]**(o-1)+(0.23811484281)
        ref[1,2,1]=o*(-0.18270059933)*x_ref[1]**(o-1)+(-0.883573478283)
        ref[1,2,2]=o*(0.558417469124)*x_ref[2]**(o-1)+(0.0921284647122)
        ref[1,3,0]=o*(0.829453952054)*x_ref[0]**(o-1)+(0.0943225421949)
        ref[1,3,1]=o*(-0.377472410614)*x_ref[1]**(o-1)+(0.422596289908)
        ref[1,3,2]=o*(0.992700203574)*x_ref[2]**(o-1)+(0.954704595957)
        ref[1,4,0]=o*(0.713437745608)*x_ref[0]**(o-1)+(-0.90755469394)
        ref[1,4,1]=o*(-0.116969245983)*x_ref[1]**(o-1)+(-0.615609779318)
        ref[1,4,2]=o*(-0.631014272933)*x_ref[2]**(o-1)+(0.196556707186)
        ref[2,0,0]=o*(-0.643142569205)*x_ref[0]**(o-1)+(-0.522571508238)
        ref[2,0,1]=o*(-0.221094875426)*x_ref[1]**(o-1)+(0.00263621704394)
        ref[2,0,2]=o*(-0.346088170378)*x_ref[2]**(o-1)+(-0.323346603492)
        ref[2,1,0]=o*(0.121192120404)*x_ref[0]**(o-1)+(-0.910581465748)
        ref[2,1,1]=o*(0.661403514322)*x_ref[1]**(o-1)+(0.314356472557)
        ref[2,1,2]=o*(0.740349932503)*x_ref[2]**(o-1)+(0.397923840477)
        ref[2,2,0]=o*(0.712887835251)*x_ref[0]**(o-1)+(0.802827824977)
        ref[2,2,1]=o*(0.225275507294)*x_ref[1]**(o-1)+(0.00655056099106)
        ref[2,2,2]=o*(-0.0211189465513)*x_ref[2]**(o-1)+(-0.672450969446)
        ref[2,3,0]=o*(0.220976204507)*x_ref[0]**(o-1)+(0.384529933174)
        ref[2,3,1]=o*(-0.824967564367)*x_ref[1]**(o-1)+(0.382079183434)
        ref[2,3,2]=o*(0.975192346085)*x_ref[2]**(o-1)+(-0.845325222223)
        ref[2,4,0]=o*(-0.316551069119)*x_ref[0]**(o-1)+(0.194934627874)
        ref[2,4,1]=o*(0.192096230967)*x_ref[1]**(o-1)+(-0.423775967466)
        ref[2,4,2]=o*(-0.669140845203)*x_ref[2]**(o-1)+(0.367813824992)
        ref[3,0,0]=o*(-0.361410077778)*x_ref[0]**(o-1)+(0.377521588398)
        ref[3,0,1]=o*(-0.795093364568)*x_ref[1]**(o-1)+(0.475257686757)
        ref[3,0,2]=o*(-0.996619080341)*x_ref[2]**(o-1)+(-0.909105543506)
        ref[3,1,0]=o*(-0.324100979259)*x_ref[0]**(o-1)+(-0.950807456193)
        ref[3,1,1]=o*(-0.141687301441)*x_ref[1]**(o-1)+(0.153568029515)
        ref[3,1,2]=o*(-0.698514966417)*x_ref[2]**(o-1)+(0.312395968777)
        ref[3,2,0]=o*(-0.997783460035)*x_ref[0]**(o-1)+(0.328724967881)
        ref[3,2,1]=o*(-0.561572087716)*x_ref[1]**(o-1)+(-0.0148890799876)
        ref[3,2,2]=o*(0.236142454998)*x_ref[2]**(o-1)+(-0.36513107554)
        ref[3,3,0]=o*(0.750818510719)*x_ref[0]**(o-1)+(-0.484598204959)
        ref[3,3,1]=o*(-0.599436804638)*x_ref[1]**(o-1)+(-0.236232976078)
        ref[3,3,2]=o*(-0.0789148183785)*x_ref[2]**(o-1)+(-0.74437164566)
        ref[3,4,0]=o*(-0.730664688591)*x_ref[0]**(o-1)+(-0.94753100979)
        ref[3,4,1]=o*(-0.671463334034)*x_ref[1]**(o-1)+(-0.139992464354)
        ref[3,4,2]=o*(0.286757519111)*x_ref[2]**(o-1)+(0.512453551537)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.467597203998)*x[0]**o+(-0.878331243959)*x[0]+(-0.971530784104)*x[1]**o+(0.685854853102)*x[1]
        arg[0,0,1]=(0.291345615954)*x[0]**o+(-0.745469654319)*x[0]+(0.976621664002)*x[1]**o+(-0.881452217847)*x[1]
        arg[0,1,0]=(0.537843295475)*x[0]**o+(0.489619951145)*x[0]+(0.77330251637)*x[1]**o+(0.998031266089)*x[1]
        arg[0,1,1]=(0.355984068681)*x[0]**o+(-0.814720548776)*x[0]+(0.495564264626)*x[1]**o+(0.525926194911)*x[1]
        arg[1,0,0]=(0.0459062782507)*x[0]**o+(0.745363380399)*x[0]+(0.254962338793)*x[1]**o+(-0.818268107226)*x[1]
        arg[1,0,1]=(0.806972777032)*x[0]**o+(0.65759411882)*x[0]+(-0.714015350868)*x[1]**o+(0.785611067194)*x[1]
        arg[1,1,0]=(0.576206375389)*x[0]**o+(-0.502513595266)*x[0]+(0.258287392463)*x[1]**o+(0.0897418205816)*x[1]
        arg[1,1,1]=(-0.854645709831)*x[0]**o+(0.48910569315)*x[0]+(0.684503579045)*x[1]**o+(0.156890859871)*x[1]
        arg[2,0,0]=(-0.535650648427)*x[0]**o+(-0.189186476574)*x[0]+(-0.62891575537)*x[1]**o+(-0.37603690255)*x[1]
        arg[2,0,1]=(-0.153001902814)*x[0]**o+(0.028214773063)*x[0]+(-0.605102549169)*x[1]**o+(-0.566991398262)*x[1]
        arg[2,1,0]=(0.607976055886)*x[0]**o+(-0.247395663109)*x[0]+(0.424340230656)*x[1]**o+(0.210489427749)*x[1]
        arg[2,1,1]=(-0.513543937157)*x[0]**o+(0.345931529219)*x[0]+(0.893744279358)*x[1]**o+(-0.519602594857)*x[1]
        arg[3,0,0]=(-0.618812192169)*x[0]**o+(-0.897898654184)*x[0]+(-0.717700928924)*x[1]**o+(-0.21348885325)*x[1]
        arg[3,0,1]=(-0.582829504396)*x[0]**o+(-0.0726162868757)*x[0]+(0.366457797377)*x[1]**o+(0.479006714526)*x[1]
        arg[3,1,0]=(0.929143108169)*x[0]**o+(-0.302254589362)*x[0]+(-0.518817476772)*x[1]**o+(0.854860903894)*x[1]
        arg[3,1,1]=(-0.815422381361)*x[0]**o+(-0.183407261786)*x[0]+(0.841334455272)*x[1]**o+(-0.560841747647)*x[1]
        arg[4,0,0]=(0.746229089137)*x[0]**o+(-0.00173525360161)*x[0]+(0.729473923491)*x[1]**o+(-0.0152936558265)*x[1]
        arg[4,0,1]=(-0.0474071267025)*x[0]**o+(-0.380505263812)*x[0]+(0.166181839511)*x[1]**o+(-0.421069915995)*x[1]
        arg[4,1,0]=(-0.826801927705)*x[0]**o+(0.704799677183)*x[0]+(-0.0346961982898)*x[1]**o+(-0.401235214163)*x[1]
        arg[4,1,1]=(-0.973646144788)*x[0]**o+(0.469976655077)*x[0]+(-0.458846284633)*x[1]**o+(0.818397008807)*x[1]
        arg[5,0,0]=(0.155460716472)*x[0]**o+(-0.509095443426)*x[0]+(-0.0942934981524)*x[1]**o+(0.754087972651)*x[1]
        arg[5,0,1]=(-0.584172835053)*x[0]**o+(-0.904579020559)*x[0]+(0.156301832106)*x[1]**o+(0.169995805721)*x[1]
        arg[5,1,0]=(-0.424977437614)*x[0]**o+(0.244566550647)*x[0]+(-0.474400097866)*x[1]**o+(-0.315912223349)*x[1]
        arg[5,1,1]=(0.764961378837)*x[0]**o+(-0.83633430067)*x[0]+(-0.446378328981)*x[1]**o+(-0.278460992776)*x[1]
        ref[0,0,0,0]=o*(0.467597203998)*x_ref[0]**(o-1)+(-0.878331243959)
        ref[0,0,0,1]=o*(-0.971530784104)*x_ref[1]**(o-1)+(0.685854853102)
        ref[0,0,1,0]=o*(0.291345615954)*x_ref[0]**(o-1)+(-0.745469654319)
        ref[0,0,1,1]=o*(0.976621664002)*x_ref[1]**(o-1)+(-0.881452217847)
        ref[0,1,0,0]=o*(0.537843295475)*x_ref[0]**(o-1)+(0.489619951145)
        ref[0,1,0,1]=o*(0.77330251637)*x_ref[1]**(o-1)+(0.998031266089)
        ref[0,1,1,0]=o*(0.355984068681)*x_ref[0]**(o-1)+(-0.814720548776)
        ref[0,1,1,1]=o*(0.495564264626)*x_ref[1]**(o-1)+(0.525926194911)
        ref[1,0,0,0]=o*(0.0459062782507)*x_ref[0]**(o-1)+(0.745363380399)
        ref[1,0,0,1]=o*(0.254962338793)*x_ref[1]**(o-1)+(-0.818268107226)
        ref[1,0,1,0]=o*(0.806972777032)*x_ref[0]**(o-1)+(0.65759411882)
        ref[1,0,1,1]=o*(-0.714015350868)*x_ref[1]**(o-1)+(0.785611067194)
        ref[1,1,0,0]=o*(0.576206375389)*x_ref[0]**(o-1)+(-0.502513595266)
        ref[1,1,0,1]=o*(0.258287392463)*x_ref[1]**(o-1)+(0.0897418205816)
        ref[1,1,1,0]=o*(-0.854645709831)*x_ref[0]**(o-1)+(0.48910569315)
        ref[1,1,1,1]=o*(0.684503579045)*x_ref[1]**(o-1)+(0.156890859871)
        ref[2,0,0,0]=o*(-0.535650648427)*x_ref[0]**(o-1)+(-0.189186476574)
        ref[2,0,0,1]=o*(-0.62891575537)*x_ref[1]**(o-1)+(-0.37603690255)
        ref[2,0,1,0]=o*(-0.153001902814)*x_ref[0]**(o-1)+(0.028214773063)
        ref[2,0,1,1]=o*(-0.605102549169)*x_ref[1]**(o-1)+(-0.566991398262)
        ref[2,1,0,0]=o*(0.607976055886)*x_ref[0]**(o-1)+(-0.247395663109)
        ref[2,1,0,1]=o*(0.424340230656)*x_ref[1]**(o-1)+(0.210489427749)
        ref[2,1,1,0]=o*(-0.513543937157)*x_ref[0]**(o-1)+(0.345931529219)
        ref[2,1,1,1]=o*(0.893744279358)*x_ref[1]**(o-1)+(-0.519602594857)
        ref[3,0,0,0]=o*(-0.618812192169)*x_ref[0]**(o-1)+(-0.897898654184)
        ref[3,0,0,1]=o*(-0.717700928924)*x_ref[1]**(o-1)+(-0.21348885325)
        ref[3,0,1,0]=o*(-0.582829504396)*x_ref[0]**(o-1)+(-0.0726162868757)
        ref[3,0,1,1]=o*(0.366457797377)*x_ref[1]**(o-1)+(0.479006714526)
        ref[3,1,0,0]=o*(0.929143108169)*x_ref[0]**(o-1)+(-0.302254589362)
        ref[3,1,0,1]=o*(-0.518817476772)*x_ref[1]**(o-1)+(0.854860903894)
        ref[3,1,1,0]=o*(-0.815422381361)*x_ref[0]**(o-1)+(-0.183407261786)
        ref[3,1,1,1]=o*(0.841334455272)*x_ref[1]**(o-1)+(-0.560841747647)
        ref[4,0,0,0]=o*(0.746229089137)*x_ref[0]**(o-1)+(-0.00173525360161)
        ref[4,0,0,1]=o*(0.729473923491)*x_ref[1]**(o-1)+(-0.0152936558265)
        ref[4,0,1,0]=o*(-0.0474071267025)*x_ref[0]**(o-1)+(-0.380505263812)
        ref[4,0,1,1]=o*(0.166181839511)*x_ref[1]**(o-1)+(-0.421069915995)
        ref[4,1,0,0]=o*(-0.826801927705)*x_ref[0]**(o-1)+(0.704799677183)
        ref[4,1,0,1]=o*(-0.0346961982898)*x_ref[1]**(o-1)+(-0.401235214163)
        ref[4,1,1,0]=o*(-0.973646144788)*x_ref[0]**(o-1)+(0.469976655077)
        ref[4,1,1,1]=o*(-0.458846284633)*x_ref[1]**(o-1)+(0.818397008807)
        ref[5,0,0,0]=o*(0.155460716472)*x_ref[0]**(o-1)+(-0.509095443426)
        ref[5,0,0,1]=o*(-0.0942934981524)*x_ref[1]**(o-1)+(0.754087972651)
        ref[5,0,1,0]=o*(-0.584172835053)*x_ref[0]**(o-1)+(-0.904579020559)
        ref[5,0,1,1]=o*(0.156301832106)*x_ref[1]**(o-1)+(0.169995805721)
        ref[5,1,0,0]=o*(-0.424977437614)*x_ref[0]**(o-1)+(0.244566550647)
        ref[5,1,0,1]=o*(-0.474400097866)*x_ref[1]**(o-1)+(-0.315912223349)
        ref[5,1,1,0]=o*(0.764961378837)*x_ref[0]**(o-1)+(-0.83633430067)
        ref[5,1,1,1]=o*(-0.446378328981)*x_ref[1]**(o-1)+(-0.278460992776)
      else:
        arg[0,0,0]=(0.723756735151)*x[0]**o+(0.738121670547)*x[0]+(0.384835073242)*x[1]**o+(-0.997944909714)*x[1]+(0.271533052097)*x[2]**o+(-0.667680363953)*x[2]
        arg[0,0,1]=(-0.279804914835)*x[0]**o+(0.789671404386)*x[0]+(-0.823187173328)*x[1]**o+(-0.0126377581755)*x[1]+(-0.942836469998)*x[2]**o+(0.416417451133)*x[2]
        arg[0,1,0]=(-0.69035514237)*x[0]**o+(0.388349204037)*x[0]+(0.807201832597)*x[1]**o+(-0.969007115064)*x[1]+(0.280506744137)*x[2]**o+(0.228554540968)*x[2]
        arg[0,1,1]=(0.893542584763)*x[0]**o+(-0.755974692532)*x[0]+(0.675321004205)*x[1]**o+(0.670278291297)*x[1]+(0.92092687885)*x[2]**o+(-0.516288388724)*x[2]
        arg[1,0,0]=(0.503764409596)*x[0]**o+(-0.39410032918)*x[0]+(0.532408717712)*x[1]**o+(0.0658355614843)*x[1]+(-0.533534889451)*x[2]**o+(0.846333211477)*x[2]
        arg[1,0,1]=(0.520531780859)*x[0]**o+(0.869039672332)*x[0]+(-0.706342144586)*x[1]**o+(0.0665768341658)*x[1]+(-0.909079295867)*x[2]**o+(-0.34153327843)*x[2]
        arg[1,1,0]=(-0.873546169914)*x[0]**o+(-0.554907283387)*x[0]+(-0.0560665557101)*x[1]**o+(0.436703623389)*x[1]+(0.730437022736)*x[2]**o+(0.100904171105)*x[2]
        arg[1,1,1]=(0.845958886681)*x[0]**o+(0.376444128389)*x[0]+(-0.393744308499)*x[1]**o+(-0.0940537397734)*x[1]+(0.178796464778)*x[2]**o+(-0.584448595407)*x[2]
        arg[2,0,0]=(0.000704382305339)*x[0]**o+(-0.836800712787)*x[0]+(0.671931504433)*x[1]**o+(0.263280364406)*x[1]+(-0.746980015754)*x[2]**o+(-0.985058787575)*x[2]
        arg[2,0,1]=(-0.540690929008)*x[0]**o+(0.656899216804)*x[0]+(-0.608036845804)*x[1]**o+(0.224943400248)*x[1]+(-0.312449005326)*x[2]**o+(-0.621458231634)*x[2]
        arg[2,1,0]=(0.440432866893)*x[0]**o+(0.972046189647)*x[0]+(0.060785199375)*x[1]**o+(0.304756432986)*x[1]+(-0.473510049103)*x[2]**o+(0.3707166351)*x[2]
        arg[2,1,1]=(0.96344472567)*x[0]**o+(-0.153932665404)*x[0]+(-0.43646142206)*x[1]**o+(0.4838463484)*x[1]+(0.613286612579)*x[2]**o+(0.84521461472)*x[2]
        arg[3,0,0]=(0.65147393118)*x[0]**o+(-0.886665762467)*x[0]+(-0.266000665973)*x[1]**o+(0.897979405808)*x[1]+(0.699333158084)*x[2]**o+(-0.460865970247)*x[2]
        arg[3,0,1]=(0.870253308112)*x[0]**o+(-0.248736315721)*x[0]+(-0.816719773596)*x[1]**o+(-0.89378202899)*x[1]+(0.524350309915)*x[2]**o+(0.107325872954)*x[2]
        arg[3,1,0]=(-0.152250686624)*x[0]**o+(-0.89784066484)*x[0]+(-0.327207891694)*x[1]**o+(0.346548492907)*x[1]+(0.409288642845)*x[2]**o+(-0.627349664194)*x[2]
        arg[3,1,1]=(0.398337202036)*x[0]**o+(0.307156588584)*x[0]+(0.0567088869624)*x[1]**o+(-0.132313990308)*x[1]+(-0.361821356587)*x[2]**o+(0.913518325149)*x[2]
        arg[4,0,0]=(0.184289428296)*x[0]**o+(0.138095407277)*x[0]+(0.514405012603)*x[1]**o+(0.590757249119)*x[1]+(-0.984116206238)*x[2]**o+(-0.176561783123)*x[2]
        arg[4,0,1]=(-0.804189448075)*x[0]**o+(0.662750787102)*x[0]+(0.361634380444)*x[1]**o+(-0.682665313256)*x[1]+(-0.0738989964344)*x[2]**o+(0.8870747144)*x[2]
        arg[4,1,0]=(-0.28290153618)*x[0]**o+(-0.942107675992)*x[0]+(0.308140792703)*x[1]**o+(0.686659531002)*x[1]+(0.789553636322)*x[2]**o+(0.587377542884)*x[2]
        arg[4,1,1]=(0.6137979501)*x[0]**o+(-0.564031937362)*x[0]+(-0.116407590432)*x[1]**o+(-0.119100735459)*x[1]+(0.673631378916)*x[2]**o+(0.766413057635)*x[2]
        arg[5,0,0]=(-0.871242033478)*x[0]**o+(0.823859447959)*x[0]+(-0.0506075089356)*x[1]**o+(-0.901254358367)*x[1]+(0.936710991394)*x[2]**o+(0.328438496312)*x[2]
        arg[5,0,1]=(-0.768255700059)*x[0]**o+(0.199623934067)*x[0]+(-0.0715921959057)*x[1]**o+(-0.542435290562)*x[1]+(-0.278962068381)*x[2]**o+(0.784363143932)*x[2]
        arg[5,1,0]=(0.897130313615)*x[0]**o+(-0.519033315355)*x[0]+(0.223176381444)*x[1]**o+(0.495645921147)*x[1]+(-0.0957696173909)*x[2]**o+(-0.768307051074)*x[2]
        arg[5,1,1]=(-0.944161779454)*x[0]**o+(0.430097328631)*x[0]+(0.86626977715)*x[1]**o+(-0.675612908116)*x[1]+(0.506821683353)*x[2]**o+(0.386559021354)*x[2]
        ref[0,0,0,0]=o*(0.723756735151)*x_ref[0]**(o-1)+(0.738121670547)
        ref[0,0,0,1]=o*(0.384835073242)*x_ref[1]**(o-1)+(-0.997944909714)
        ref[0,0,0,2]=o*(0.271533052097)*x_ref[2]**(o-1)+(-0.667680363953)
        ref[0,0,1,0]=o*(-0.279804914835)*x_ref[0]**(o-1)+(0.789671404386)
        ref[0,0,1,1]=o*(-0.823187173328)*x_ref[1]**(o-1)+(-0.0126377581755)
        ref[0,0,1,2]=o*(-0.942836469998)*x_ref[2]**(o-1)+(0.416417451133)
        ref[0,1,0,0]=o*(-0.69035514237)*x_ref[0]**(o-1)+(0.388349204037)
        ref[0,1,0,1]=o*(0.807201832597)*x_ref[1]**(o-1)+(-0.969007115064)
        ref[0,1,0,2]=o*(0.280506744137)*x_ref[2]**(o-1)+(0.228554540968)
        ref[0,1,1,0]=o*(0.893542584763)*x_ref[0]**(o-1)+(-0.755974692532)
        ref[0,1,1,1]=o*(0.675321004205)*x_ref[1]**(o-1)+(0.670278291297)
        ref[0,1,1,2]=o*(0.92092687885)*x_ref[2]**(o-1)+(-0.516288388724)
        ref[1,0,0,0]=o*(0.503764409596)*x_ref[0]**(o-1)+(-0.39410032918)
        ref[1,0,0,1]=o*(0.532408717712)*x_ref[1]**(o-1)+(0.0658355614843)
        ref[1,0,0,2]=o*(-0.533534889451)*x_ref[2]**(o-1)+(0.846333211477)
        ref[1,0,1,0]=o*(0.520531780859)*x_ref[0]**(o-1)+(0.869039672332)
        ref[1,0,1,1]=o*(-0.706342144586)*x_ref[1]**(o-1)+(0.0665768341658)
        ref[1,0,1,2]=o*(-0.909079295867)*x_ref[2]**(o-1)+(-0.34153327843)
        ref[1,1,0,0]=o*(-0.873546169914)*x_ref[0]**(o-1)+(-0.554907283387)
        ref[1,1,0,1]=o*(-0.0560665557101)*x_ref[1]**(o-1)+(0.436703623389)
        ref[1,1,0,2]=o*(0.730437022736)*x_ref[2]**(o-1)+(0.100904171105)
        ref[1,1,1,0]=o*(0.845958886681)*x_ref[0]**(o-1)+(0.376444128389)
        ref[1,1,1,1]=o*(-0.393744308499)*x_ref[1]**(o-1)+(-0.0940537397734)
        ref[1,1,1,2]=o*(0.178796464778)*x_ref[2]**(o-1)+(-0.584448595407)
        ref[2,0,0,0]=o*(0.000704382305339)*x_ref[0]**(o-1)+(-0.836800712787)
        ref[2,0,0,1]=o*(0.671931504433)*x_ref[1]**(o-1)+(0.263280364406)
        ref[2,0,0,2]=o*(-0.746980015754)*x_ref[2]**(o-1)+(-0.985058787575)
        ref[2,0,1,0]=o*(-0.540690929008)*x_ref[0]**(o-1)+(0.656899216804)
        ref[2,0,1,1]=o*(-0.608036845804)*x_ref[1]**(o-1)+(0.224943400248)
        ref[2,0,1,2]=o*(-0.312449005326)*x_ref[2]**(o-1)+(-0.621458231634)
        ref[2,1,0,0]=o*(0.440432866893)*x_ref[0]**(o-1)+(0.972046189647)
        ref[2,1,0,1]=o*(0.060785199375)*x_ref[1]**(o-1)+(0.304756432986)
        ref[2,1,0,2]=o*(-0.473510049103)*x_ref[2]**(o-1)+(0.3707166351)
        ref[2,1,1,0]=o*(0.96344472567)*x_ref[0]**(o-1)+(-0.153932665404)
        ref[2,1,1,1]=o*(-0.43646142206)*x_ref[1]**(o-1)+(0.4838463484)
        ref[2,1,1,2]=o*(0.613286612579)*x_ref[2]**(o-1)+(0.84521461472)
        ref[3,0,0,0]=o*(0.65147393118)*x_ref[0]**(o-1)+(-0.886665762467)
        ref[3,0,0,1]=o*(-0.266000665973)*x_ref[1]**(o-1)+(0.897979405808)
        ref[3,0,0,2]=o*(0.699333158084)*x_ref[2]**(o-1)+(-0.460865970247)
        ref[3,0,1,0]=o*(0.870253308112)*x_ref[0]**(o-1)+(-0.248736315721)
        ref[3,0,1,1]=o*(-0.816719773596)*x_ref[1]**(o-1)+(-0.89378202899)
        ref[3,0,1,2]=o*(0.524350309915)*x_ref[2]**(o-1)+(0.107325872954)
        ref[3,1,0,0]=o*(-0.152250686624)*x_ref[0]**(o-1)+(-0.89784066484)
        ref[3,1,0,1]=o*(-0.327207891694)*x_ref[1]**(o-1)+(0.346548492907)
        ref[3,1,0,2]=o*(0.409288642845)*x_ref[2]**(o-1)+(-0.627349664194)
        ref[3,1,1,0]=o*(0.398337202036)*x_ref[0]**(o-1)+(0.307156588584)
        ref[3,1,1,1]=o*(0.0567088869624)*x_ref[1]**(o-1)+(-0.132313990308)
        ref[3,1,1,2]=o*(-0.361821356587)*x_ref[2]**(o-1)+(0.913518325149)
        ref[4,0,0,0]=o*(0.184289428296)*x_ref[0]**(o-1)+(0.138095407277)
        ref[4,0,0,1]=o*(0.514405012603)*x_ref[1]**(o-1)+(0.590757249119)
        ref[4,0,0,2]=o*(-0.984116206238)*x_ref[2]**(o-1)+(-0.176561783123)
        ref[4,0,1,0]=o*(-0.804189448075)*x_ref[0]**(o-1)+(0.662750787102)
        ref[4,0,1,1]=o*(0.361634380444)*x_ref[1]**(o-1)+(-0.682665313256)
        ref[4,0,1,2]=o*(-0.0738989964344)*x_ref[2]**(o-1)+(0.8870747144)
        ref[4,1,0,0]=o*(-0.28290153618)*x_ref[0]**(o-1)+(-0.942107675992)
        ref[4,1,0,1]=o*(0.308140792703)*x_ref[1]**(o-1)+(0.686659531002)
        ref[4,1,0,2]=o*(0.789553636322)*x_ref[2]**(o-1)+(0.587377542884)
        ref[4,1,1,0]=o*(0.6137979501)*x_ref[0]**(o-1)+(-0.564031937362)
        ref[4,1,1,1]=o*(-0.116407590432)*x_ref[1]**(o-1)+(-0.119100735459)
        ref[4,1,1,2]=o*(0.673631378916)*x_ref[2]**(o-1)+(0.766413057635)
        ref[5,0,0,0]=o*(-0.871242033478)*x_ref[0]**(o-1)+(0.823859447959)
        ref[5,0,0,1]=o*(-0.0506075089356)*x_ref[1]**(o-1)+(-0.901254358367)
        ref[5,0,0,2]=o*(0.936710991394)*x_ref[2]**(o-1)+(0.328438496312)
        ref[5,0,1,0]=o*(-0.768255700059)*x_ref[0]**(o-1)+(0.199623934067)
        ref[5,0,1,1]=o*(-0.0715921959057)*x_ref[1]**(o-1)+(-0.542435290562)
        ref[5,0,1,2]=o*(-0.278962068381)*x_ref[2]**(o-1)+(0.784363143932)
        ref[5,1,0,0]=o*(0.897130313615)*x_ref[0]**(o-1)+(-0.519033315355)
        ref[5,1,0,1]=o*(0.223176381444)*x_ref[1]**(o-1)+(0.495645921147)
        ref[5,1,0,2]=o*(-0.0957696173909)*x_ref[2]**(o-1)+(-0.768307051074)
        ref[5,1,1,0]=o*(-0.944161779454)*x_ref[0]**(o-1)+(0.430097328631)
        ref[5,1,1,1]=o*(0.86626977715)*x_ref[1]**(o-1)+(-0.675612908116)
        ref[5,1,1,2]=o*(0.506821683353)*x_ref[2]**(o-1)+(0.386559021354)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.54415545025)*x[0]**o+(0.979947366774)*x[0]+(-0.405934730304)*x[1]**o+(0.424994268397)*x[1]
        ref[0]=o*(0.54415545025)*x_ref[0]**(o-1)+(0.979947366774)
        ref[1]=o*(-0.405934730304)*x_ref[1]**(o-1)+(0.424994268397)
      else:
        arg=(-0.0240792232301)*x[0]**o+(0.33156704088)*x[0]+(-0.051539264252)*x[1]**o+(-0.886370693155)*x[1]+(-0.954475798211)*x[2]**o+(0.238073928589)*x[2]
        ref[0]=o*(-0.0240792232301)*x_ref[0]**(o-1)+(0.33156704088)
        ref[1]=o*(-0.051539264252)*x_ref[1]**(o-1)+(-0.886370693155)
        ref[2]=o*(-0.954475798211)*x_ref[2]**(o-1)+(0.238073928589)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.526471606132)*x[0]**o+(-0.355534153967)*x[0]+(-0.792258102571)*x[1]**o+(0.386283553167)*x[1]
        arg[1]=(-0.892924864905)*x[0]**o+(0.19218757952)*x[0]+(0.00248964495987)*x[1]**o+(-0.687070938997)*x[1]
        ref[0,0]=o*(-0.526471606132)*x_ref[0]**(o-1)+(-0.355534153967)
        ref[0,1]=o*(-0.792258102571)*x_ref[1]**(o-1)+(0.386283553167)
        ref[1,0]=o*(-0.892924864905)*x_ref[0]**(o-1)+(0.19218757952)
        ref[1,1]=o*(0.00248964495987)*x_ref[1]**(o-1)+(-0.687070938997)
      else:
        arg[0]=(-0.254013984725)*x[0]**o+(0.126174108068)*x[0]+(-0.967653739084)*x[1]**o+(-0.524294999327)*x[1]+(0.374454913375)*x[2]**o+(0.783768839592)*x[2]
        arg[1]=(0.862328996944)*x[0]**o+(-0.0100272707295)*x[0]+(0.163524617753)*x[1]**o+(-0.19251370258)*x[1]+(0.889557068195)*x[2]**o+(0.573248942406)*x[2]
        ref[0,0]=o*(-0.254013984725)*x_ref[0]**(o-1)+(0.126174108068)
        ref[0,1]=o*(-0.967653739084)*x_ref[1]**(o-1)+(-0.524294999327)
        ref[0,2]=o*(0.374454913375)*x_ref[2]**(o-1)+(0.783768839592)
        ref[1,0]=o*(0.862328996944)*x_ref[0]**(o-1)+(-0.0100272707295)
        ref[1,1]=o*(0.163524617753)*x_ref[1]**(o-1)+(-0.19251370258)
        ref[1,2]=o*(0.889557068195)*x_ref[2]**(o-1)+(0.573248942406)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.712410724071)*x[0]**o+(0.642832165006)*x[0]+(0.409613677638)*x[1]**o+(-0.940707375829)*x[1]
        arg[0,1]=(-0.192677448978)*x[0]**o+(0.928105399392)*x[0]+(-0.0976390111113)*x[1]**o+(0.152541825282)*x[1]
        arg[0,2]=(-0.555865920622)*x[0]**o+(0.187618737934)*x[0]+(-0.400398722616)*x[1]**o+(0.132335647294)*x[1]
        arg[0,3]=(-0.0984345355899)*x[0]**o+(-0.136251593387)*x[0]+(0.820900167194)*x[1]**o+(-0.54323976155)*x[1]
        arg[0,4]=(-0.718763327727)*x[0]**o+(-0.993678837401)*x[0]+(0.0282625285716)*x[1]**o+(0.480464124636)*x[1]
        arg[1,0]=(-0.1635372419)*x[0]**o+(0.97173720053)*x[0]+(0.0423190406144)*x[1]**o+(-0.607779321237)*x[1]
        arg[1,1]=(-0.59220466746)*x[0]**o+(-0.329625880847)*x[0]+(0.176412343254)*x[1]**o+(0.955769019385)*x[1]
        arg[1,2]=(-0.516421195901)*x[0]**o+(0.147037193899)*x[0]+(0.247835053631)*x[1]**o+(-0.877986653535)*x[1]
        arg[1,3]=(-0.317266388999)*x[0]**o+(0.864393749347)*x[0]+(-0.957863705756)*x[1]**o+(-0.884975429798)*x[1]
        arg[1,4]=(0.540121554391)*x[0]**o+(0.808050078272)*x[0]+(0.703557587067)*x[1]**o+(-0.0976121077766)*x[1]
        arg[2,0]=(0.23658022222)*x[0]**o+(0.122038637852)*x[0]+(0.411570045798)*x[1]**o+(-0.434531687414)*x[1]
        arg[2,1]=(-0.378365306249)*x[0]**o+(0.511747532394)*x[0]+(0.586326813286)*x[1]**o+(0.699918966936)*x[1]
        arg[2,2]=(0.445092669694)*x[0]**o+(0.637517787944)*x[0]+(-0.668779388847)*x[1]**o+(0.487635262236)*x[1]
        arg[2,3]=(-0.232265851556)*x[0]**o+(0.391449220433)*x[0]+(0.647177607766)*x[1]**o+(0.207062996634)*x[1]
        arg[2,4]=(0.338412227828)*x[0]**o+(0.0205801366652)*x[0]+(-0.492628796478)*x[1]**o+(-0.683459852092)*x[1]
        arg[3,0]=(0.584296838468)*x[0]**o+(-0.598793253755)*x[0]+(0.707319426212)*x[1]**o+(0.678834769735)*x[1]
        arg[3,1]=(0.801295300134)*x[0]**o+(0.0494871119593)*x[0]+(0.378929822492)*x[1]**o+(-0.536309789421)*x[1]
        arg[3,2]=(-0.761535984131)*x[0]**o+(0.121584094596)*x[0]+(-0.00655849155175)*x[1]**o+(0.321872124523)*x[1]
        arg[3,3]=(0.384633290815)*x[0]**o+(-0.710356674518)*x[0]+(0.962059942069)*x[1]**o+(-0.593059919585)*x[1]
        arg[3,4]=(-0.608988807286)*x[0]**o+(-0.0776468065145)*x[0]+(-0.701400144867)*x[1]**o+(0.643928611178)*x[1]
        ref[0,0,0]=o*(0.712410724071)*x_ref[0]**(o-1)+(0.642832165006)
        ref[0,0,1]=o*(0.409613677638)*x_ref[1]**(o-1)+(-0.940707375829)
        ref[0,1,0]=o*(-0.192677448978)*x_ref[0]**(o-1)+(0.928105399392)
        ref[0,1,1]=o*(-0.0976390111113)*x_ref[1]**(o-1)+(0.152541825282)
        ref[0,2,0]=o*(-0.555865920622)*x_ref[0]**(o-1)+(0.187618737934)
        ref[0,2,1]=o*(-0.400398722616)*x_ref[1]**(o-1)+(0.132335647294)
        ref[0,3,0]=o*(-0.0984345355899)*x_ref[0]**(o-1)+(-0.136251593387)
        ref[0,3,1]=o*(0.820900167194)*x_ref[1]**(o-1)+(-0.54323976155)
        ref[0,4,0]=o*(-0.718763327727)*x_ref[0]**(o-1)+(-0.993678837401)
        ref[0,4,1]=o*(0.0282625285716)*x_ref[1]**(o-1)+(0.480464124636)
        ref[1,0,0]=o*(-0.1635372419)*x_ref[0]**(o-1)+(0.97173720053)
        ref[1,0,1]=o*(0.0423190406144)*x_ref[1]**(o-1)+(-0.607779321237)
        ref[1,1,0]=o*(-0.59220466746)*x_ref[0]**(o-1)+(-0.329625880847)
        ref[1,1,1]=o*(0.176412343254)*x_ref[1]**(o-1)+(0.955769019385)
        ref[1,2,0]=o*(-0.516421195901)*x_ref[0]**(o-1)+(0.147037193899)
        ref[1,2,1]=o*(0.247835053631)*x_ref[1]**(o-1)+(-0.877986653535)
        ref[1,3,0]=o*(-0.317266388999)*x_ref[0]**(o-1)+(0.864393749347)
        ref[1,3,1]=o*(-0.957863705756)*x_ref[1]**(o-1)+(-0.884975429798)
        ref[1,4,0]=o*(0.540121554391)*x_ref[0]**(o-1)+(0.808050078272)
        ref[1,4,1]=o*(0.703557587067)*x_ref[1]**(o-1)+(-0.0976121077766)
        ref[2,0,0]=o*(0.23658022222)*x_ref[0]**(o-1)+(0.122038637852)
        ref[2,0,1]=o*(0.411570045798)*x_ref[1]**(o-1)+(-0.434531687414)
        ref[2,1,0]=o*(-0.378365306249)*x_ref[0]**(o-1)+(0.511747532394)
        ref[2,1,1]=o*(0.586326813286)*x_ref[1]**(o-1)+(0.699918966936)
        ref[2,2,0]=o*(0.445092669694)*x_ref[0]**(o-1)+(0.637517787944)
        ref[2,2,1]=o*(-0.668779388847)*x_ref[1]**(o-1)+(0.487635262236)
        ref[2,3,0]=o*(-0.232265851556)*x_ref[0]**(o-1)+(0.391449220433)
        ref[2,3,1]=o*(0.647177607766)*x_ref[1]**(o-1)+(0.207062996634)
        ref[2,4,0]=o*(0.338412227828)*x_ref[0]**(o-1)+(0.0205801366652)
        ref[2,4,1]=o*(-0.492628796478)*x_ref[1]**(o-1)+(-0.683459852092)
        ref[3,0,0]=o*(0.584296838468)*x_ref[0]**(o-1)+(-0.598793253755)
        ref[3,0,1]=o*(0.707319426212)*x_ref[1]**(o-1)+(0.678834769735)
        ref[3,1,0]=o*(0.801295300134)*x_ref[0]**(o-1)+(0.0494871119593)
        ref[3,1,1]=o*(0.378929822492)*x_ref[1]**(o-1)+(-0.536309789421)
        ref[3,2,0]=o*(-0.761535984131)*x_ref[0]**(o-1)+(0.121584094596)
        ref[3,2,1]=o*(-0.00655849155175)*x_ref[1]**(o-1)+(0.321872124523)
        ref[3,3,0]=o*(0.384633290815)*x_ref[0]**(o-1)+(-0.710356674518)
        ref[3,3,1]=o*(0.962059942069)*x_ref[1]**(o-1)+(-0.593059919585)
        ref[3,4,0]=o*(-0.608988807286)*x_ref[0]**(o-1)+(-0.0776468065145)
        ref[3,4,1]=o*(-0.701400144867)*x_ref[1]**(o-1)+(0.643928611178)
      else:
        arg[0,0]=(-0.514546772198)*x[0]**o+(0.679859064768)*x[0]+(-0.494037106077)*x[1]**o+(-0.0598645161488)*x[1]+(-0.543474311879)*x[2]**o+(0.597237276677)*x[2]
        arg[0,1]=(0.0826513727579)*x[0]**o+(0.870535937411)*x[0]+(-0.194047768119)*x[1]**o+(-0.111616761775)*x[1]+(0.697073558803)*x[2]**o+(-0.170433075843)*x[2]
        arg[0,2]=(0.608560200938)*x[0]**o+(-0.28991661304)*x[0]+(0.929121366615)*x[1]**o+(0.445280468492)*x[1]+(-0.0671026162019)*x[2]**o+(-0.835086989851)*x[2]
        arg[0,3]=(-0.256476478337)*x[0]**o+(-0.857686365148)*x[0]+(-0.309306419219)*x[1]**o+(0.750579112592)*x[1]+(0.758254577714)*x[2]**o+(0.108972886739)*x[2]
        arg[0,4]=(0.189540467853)*x[0]**o+(-0.639906129587)*x[0]+(0.1086564431)*x[1]**o+(-0.254754163159)*x[1]+(-0.595415956315)*x[2]**o+(0.416643698623)*x[2]
        arg[1,0]=(0.649640741159)*x[0]**o+(-0.715604744977)*x[0]+(0.144698127223)*x[1]**o+(0.84058008507)*x[1]+(0.00226738664623)*x[2]**o+(0.143092105366)*x[2]
        arg[1,1]=(0.30428097785)*x[0]**o+(0.172630542192)*x[0]+(0.645118113075)*x[1]**o+(-0.394468022223)*x[1]+(0.731670923192)*x[2]**o+(-0.676021495379)*x[2]
        arg[1,2]=(0.56459927889)*x[0]**o+(-0.624980053566)*x[0]+(-0.524836958708)*x[1]**o+(0.282862320474)*x[1]+(0.99070455422)*x[2]**o+(0.000265746399286)*x[2]
        arg[1,3]=(-0.251929813217)*x[0]**o+(0.00685246758491)*x[0]+(-0.252595929082)*x[1]**o+(-0.997558334417)*x[1]+(-0.431329146194)*x[2]**o+(0.0685298495116)*x[2]
        arg[1,4]=(-0.850398225165)*x[0]**o+(-0.752363025861)*x[0]+(-0.669448553729)*x[1]**o+(0.809247179374)*x[1]+(-0.927467807251)*x[2]**o+(-0.498094160268)*x[2]
        arg[2,0]=(0.541736859663)*x[0]**o+(-0.651273876855)*x[0]+(-0.679670779769)*x[1]**o+(0.669564679419)*x[1]+(-0.825048237017)*x[2]**o+(-0.247744537755)*x[2]
        arg[2,1]=(0.909016713138)*x[0]**o+(-0.794124179572)*x[0]+(0.887294650854)*x[1]**o+(-0.198933351435)*x[1]+(-0.819671062456)*x[2]**o+(0.287455634612)*x[2]
        arg[2,2]=(-0.469337238665)*x[0]**o+(0.787828984218)*x[0]+(0.916042829702)*x[1]**o+(-0.475983675757)*x[1]+(-0.350660319046)*x[2]**o+(0.944222119413)*x[2]
        arg[2,3]=(-0.194629035155)*x[0]**o+(0.452058614199)*x[0]+(-0.565150768916)*x[1]**o+(-0.967642852138)*x[1]+(-0.345332973443)*x[2]**o+(-0.238359110833)*x[2]
        arg[2,4]=(-0.744197369952)*x[0]**o+(-0.0161450929106)*x[0]+(0.946555120156)*x[1]**o+(-0.336314723492)*x[1]+(0.212936909808)*x[2]**o+(-0.900582293569)*x[2]
        arg[3,0]=(0.90736518985)*x[0]**o+(-0.866096443075)*x[0]+(0.440227521532)*x[1]**o+(-0.545983384676)*x[1]+(0.433167375756)*x[2]**o+(-0.913060961228)*x[2]
        arg[3,1]=(0.33547032327)*x[0]**o+(0.766388218896)*x[0]+(-0.386141806114)*x[1]**o+(0.28471049668)*x[1]+(0.389347155974)*x[2]**o+(0.211663215285)*x[2]
        arg[3,2]=(0.193757801099)*x[0]**o+(0.0145439587225)*x[0]+(0.575506295918)*x[1]**o+(-0.678317336021)*x[1]+(0.64603157052)*x[2]**o+(-0.876667949279)*x[2]
        arg[3,3]=(-0.0518002857411)*x[0]**o+(-0.540021971724)*x[0]+(-0.490275636747)*x[1]**o+(-0.849147647677)*x[1]+(-0.0132783508877)*x[2]**o+(0.0859860768541)*x[2]
        arg[3,4]=(-0.332675003425)*x[0]**o+(-0.757986125506)*x[0]+(-0.487001539705)*x[1]**o+(0.898881577697)*x[1]+(0.56557767007)*x[2]**o+(0.181594113977)*x[2]
        ref[0,0,0]=o*(-0.514546772198)*x_ref[0]**(o-1)+(0.679859064768)
        ref[0,0,1]=o*(-0.494037106077)*x_ref[1]**(o-1)+(-0.0598645161488)
        ref[0,0,2]=o*(-0.543474311879)*x_ref[2]**(o-1)+(0.597237276677)
        ref[0,1,0]=o*(0.0826513727579)*x_ref[0]**(o-1)+(0.870535937411)
        ref[0,1,1]=o*(-0.194047768119)*x_ref[1]**(o-1)+(-0.111616761775)
        ref[0,1,2]=o*(0.697073558803)*x_ref[2]**(o-1)+(-0.170433075843)
        ref[0,2,0]=o*(0.608560200938)*x_ref[0]**(o-1)+(-0.28991661304)
        ref[0,2,1]=o*(0.929121366615)*x_ref[1]**(o-1)+(0.445280468492)
        ref[0,2,2]=o*(-0.0671026162019)*x_ref[2]**(o-1)+(-0.835086989851)
        ref[0,3,0]=o*(-0.256476478337)*x_ref[0]**(o-1)+(-0.857686365148)
        ref[0,3,1]=o*(-0.309306419219)*x_ref[1]**(o-1)+(0.750579112592)
        ref[0,3,2]=o*(0.758254577714)*x_ref[2]**(o-1)+(0.108972886739)
        ref[0,4,0]=o*(0.189540467853)*x_ref[0]**(o-1)+(-0.639906129587)
        ref[0,4,1]=o*(0.1086564431)*x_ref[1]**(o-1)+(-0.254754163159)
        ref[0,4,2]=o*(-0.595415956315)*x_ref[2]**(o-1)+(0.416643698623)
        ref[1,0,0]=o*(0.649640741159)*x_ref[0]**(o-1)+(-0.715604744977)
        ref[1,0,1]=o*(0.144698127223)*x_ref[1]**(o-1)+(0.84058008507)
        ref[1,0,2]=o*(0.00226738664623)*x_ref[2]**(o-1)+(0.143092105366)
        ref[1,1,0]=o*(0.30428097785)*x_ref[0]**(o-1)+(0.172630542192)
        ref[1,1,1]=o*(0.645118113075)*x_ref[1]**(o-1)+(-0.394468022223)
        ref[1,1,2]=o*(0.731670923192)*x_ref[2]**(o-1)+(-0.676021495379)
        ref[1,2,0]=o*(0.56459927889)*x_ref[0]**(o-1)+(-0.624980053566)
        ref[1,2,1]=o*(-0.524836958708)*x_ref[1]**(o-1)+(0.282862320474)
        ref[1,2,2]=o*(0.99070455422)*x_ref[2]**(o-1)+(0.000265746399286)
        ref[1,3,0]=o*(-0.251929813217)*x_ref[0]**(o-1)+(0.00685246758491)
        ref[1,3,1]=o*(-0.252595929082)*x_ref[1]**(o-1)+(-0.997558334417)
        ref[1,3,2]=o*(-0.431329146194)*x_ref[2]**(o-1)+(0.0685298495116)
        ref[1,4,0]=o*(-0.850398225165)*x_ref[0]**(o-1)+(-0.752363025861)
        ref[1,4,1]=o*(-0.669448553729)*x_ref[1]**(o-1)+(0.809247179374)
        ref[1,4,2]=o*(-0.927467807251)*x_ref[2]**(o-1)+(-0.498094160268)
        ref[2,0,0]=o*(0.541736859663)*x_ref[0]**(o-1)+(-0.651273876855)
        ref[2,0,1]=o*(-0.679670779769)*x_ref[1]**(o-1)+(0.669564679419)
        ref[2,0,2]=o*(-0.825048237017)*x_ref[2]**(o-1)+(-0.247744537755)
        ref[2,1,0]=o*(0.909016713138)*x_ref[0]**(o-1)+(-0.794124179572)
        ref[2,1,1]=o*(0.887294650854)*x_ref[1]**(o-1)+(-0.198933351435)
        ref[2,1,2]=o*(-0.819671062456)*x_ref[2]**(o-1)+(0.287455634612)
        ref[2,2,0]=o*(-0.469337238665)*x_ref[0]**(o-1)+(0.787828984218)
        ref[2,2,1]=o*(0.916042829702)*x_ref[1]**(o-1)+(-0.475983675757)
        ref[2,2,2]=o*(-0.350660319046)*x_ref[2]**(o-1)+(0.944222119413)
        ref[2,3,0]=o*(-0.194629035155)*x_ref[0]**(o-1)+(0.452058614199)
        ref[2,3,1]=o*(-0.565150768916)*x_ref[1]**(o-1)+(-0.967642852138)
        ref[2,3,2]=o*(-0.345332973443)*x_ref[2]**(o-1)+(-0.238359110833)
        ref[2,4,0]=o*(-0.744197369952)*x_ref[0]**(o-1)+(-0.0161450929106)
        ref[2,4,1]=o*(0.946555120156)*x_ref[1]**(o-1)+(-0.336314723492)
        ref[2,4,2]=o*(0.212936909808)*x_ref[2]**(o-1)+(-0.900582293569)
        ref[3,0,0]=o*(0.90736518985)*x_ref[0]**(o-1)+(-0.866096443075)
        ref[3,0,1]=o*(0.440227521532)*x_ref[1]**(o-1)+(-0.545983384676)
        ref[3,0,2]=o*(0.433167375756)*x_ref[2]**(o-1)+(-0.913060961228)
        ref[3,1,0]=o*(0.33547032327)*x_ref[0]**(o-1)+(0.766388218896)
        ref[3,1,1]=o*(-0.386141806114)*x_ref[1]**(o-1)+(0.28471049668)
        ref[3,1,2]=o*(0.389347155974)*x_ref[2]**(o-1)+(0.211663215285)
        ref[3,2,0]=o*(0.193757801099)*x_ref[0]**(o-1)+(0.0145439587225)
        ref[3,2,1]=o*(0.575506295918)*x_ref[1]**(o-1)+(-0.678317336021)
        ref[3,2,2]=o*(0.64603157052)*x_ref[2]**(o-1)+(-0.876667949279)
        ref[3,3,0]=o*(-0.0518002857411)*x_ref[0]**(o-1)+(-0.540021971724)
        ref[3,3,1]=o*(-0.490275636747)*x_ref[1]**(o-1)+(-0.849147647677)
        ref[3,3,2]=o*(-0.0132783508877)*x_ref[2]**(o-1)+(0.0859860768541)
        ref[3,4,0]=o*(-0.332675003425)*x_ref[0]**(o-1)+(-0.757986125506)
        ref[3,4,1]=o*(-0.487001539705)*x_ref[1]**(o-1)+(0.898881577697)
        ref[3,4,2]=o*(0.56557767007)*x_ref[2]**(o-1)+(0.181594113977)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.568278554396)*x[0]**o+(0.714635312506)*x[0]+(-0.121000061881)*x[1]**o+(0.650923087629)*x[1]
        arg[0,0,1]=(0.610070747198)*x[0]**o+(-0.17228586298)*x[0]+(-0.763207252556)*x[1]**o+(-0.417620736844)*x[1]
        arg[0,1,0]=(0.953437717333)*x[0]**o+(-0.877858891638)*x[0]+(-0.629855255168)*x[1]**o+(0.138168708132)*x[1]
        arg[0,1,1]=(-0.676346259275)*x[0]**o+(0.810400719592)*x[0]+(0.252517037374)*x[1]**o+(-0.0253025825974)*x[1]
        arg[1,0,0]=(0.0354009852553)*x[0]**o+(-0.930486981217)*x[0]+(0.515533244848)*x[1]**o+(0.39357313927)*x[1]
        arg[1,0,1]=(4.954730054e-05)*x[0]**o+(0.902841279458)*x[0]+(0.0974175050415)*x[1]**o+(-0.283224733511)*x[1]
        arg[1,1,0]=(-0.3695113121)*x[0]**o+(-0.425976895152)*x[0]+(-0.752863101988)*x[1]**o+(-0.135026977845)*x[1]
        arg[1,1,1]=(-0.627750038208)*x[0]**o+(-0.682825202306)*x[0]+(0.598327522753)*x[1]**o+(0.0402698525348)*x[1]
        arg[2,0,0]=(0.95982907811)*x[0]**o+(-0.786158711751)*x[0]+(0.6974717418)*x[1]**o+(-0.548248895713)*x[1]
        arg[2,0,1]=(-0.21063630362)*x[0]**o+(-0.979628903483)*x[0]+(0.514851452559)*x[1]**o+(-0.294218784425)*x[1]
        arg[2,1,0]=(-0.446140926476)*x[0]**o+(0.658918134389)*x[0]+(-0.166087456007)*x[1]**o+(0.328044805625)*x[1]
        arg[2,1,1]=(0.937161634589)*x[0]**o+(0.594322457074)*x[0]+(0.992336676032)*x[1]**o+(-0.499637100315)*x[1]
        arg[3,0,0]=(0.767677974163)*x[0]**o+(-0.820615663298)*x[0]+(0.558185245925)*x[1]**o+(-0.763525476816)*x[1]
        arg[3,0,1]=(-0.866581629552)*x[0]**o+(0.589993777037)*x[0]+(-0.0704817160146)*x[1]**o+(-0.897469280048)*x[1]
        arg[3,1,0]=(-0.290118027748)*x[0]**o+(0.211790364383)*x[0]+(-0.0688758465892)*x[1]**o+(0.777490292219)*x[1]
        arg[3,1,1]=(-0.969418722549)*x[0]**o+(-0.46884683812)*x[0]+(0.826696594862)*x[1]**o+(-0.926802811507)*x[1]
        arg[4,0,0]=(-0.230285931083)*x[0]**o+(0.905695021426)*x[0]+(-0.581906633624)*x[1]**o+(-0.69277147014)*x[1]
        arg[4,0,1]=(-0.251350536276)*x[0]**o+(0.352877501605)*x[0]+(0.290432460788)*x[1]**o+(0.614218150117)*x[1]
        arg[4,1,0]=(-0.475014635407)*x[0]**o+(0.86786497963)*x[0]+(-0.763349511499)*x[1]**o+(0.817847286727)*x[1]
        arg[4,1,1]=(0.0079711463132)*x[0]**o+(-0.20685339926)*x[0]+(0.377467915936)*x[1]**o+(-0.209268733968)*x[1]
        arg[5,0,0]=(0.795790630791)*x[0]**o+(0.0661201279192)*x[0]+(0.932665258262)*x[1]**o+(-0.4785884914)*x[1]
        arg[5,0,1]=(0.929422646068)*x[0]**o+(0.775378282324)*x[0]+(0.297482039582)*x[1]**o+(-0.700872789842)*x[1]
        arg[5,1,0]=(0.678688297204)*x[0]**o+(0.365975847321)*x[0]+(-0.565261107289)*x[1]**o+(0.0317556604619)*x[1]
        arg[5,1,1]=(-0.0231297032408)*x[0]**o+(-0.985865541949)*x[0]+(0.733821736669)*x[1]**o+(0.603255951212)*x[1]
        ref[0,0,0,0]=o*(0.568278554396)*x_ref[0]**(o-1)+(0.714635312506)
        ref[0,0,0,1]=o*(-0.121000061881)*x_ref[1]**(o-1)+(0.650923087629)
        ref[0,0,1,0]=o*(0.610070747198)*x_ref[0]**(o-1)+(-0.17228586298)
        ref[0,0,1,1]=o*(-0.763207252556)*x_ref[1]**(o-1)+(-0.417620736844)
        ref[0,1,0,0]=o*(0.953437717333)*x_ref[0]**(o-1)+(-0.877858891638)
        ref[0,1,0,1]=o*(-0.629855255168)*x_ref[1]**(o-1)+(0.138168708132)
        ref[0,1,1,0]=o*(-0.676346259275)*x_ref[0]**(o-1)+(0.810400719592)
        ref[0,1,1,1]=o*(0.252517037374)*x_ref[1]**(o-1)+(-0.0253025825974)
        ref[1,0,0,0]=o*(0.0354009852553)*x_ref[0]**(o-1)+(-0.930486981217)
        ref[1,0,0,1]=o*(0.515533244848)*x_ref[1]**(o-1)+(0.39357313927)
        ref[1,0,1,0]=o*(4.954730054e-05)*x_ref[0]**(o-1)+(0.902841279458)
        ref[1,0,1,1]=o*(0.0974175050415)*x_ref[1]**(o-1)+(-0.283224733511)
        ref[1,1,0,0]=o*(-0.3695113121)*x_ref[0]**(o-1)+(-0.425976895152)
        ref[1,1,0,1]=o*(-0.752863101988)*x_ref[1]**(o-1)+(-0.135026977845)
        ref[1,1,1,0]=o*(-0.627750038208)*x_ref[0]**(o-1)+(-0.682825202306)
        ref[1,1,1,1]=o*(0.598327522753)*x_ref[1]**(o-1)+(0.0402698525348)
        ref[2,0,0,0]=o*(0.95982907811)*x_ref[0]**(o-1)+(-0.786158711751)
        ref[2,0,0,1]=o*(0.6974717418)*x_ref[1]**(o-1)+(-0.548248895713)
        ref[2,0,1,0]=o*(-0.21063630362)*x_ref[0]**(o-1)+(-0.979628903483)
        ref[2,0,1,1]=o*(0.514851452559)*x_ref[1]**(o-1)+(-0.294218784425)
        ref[2,1,0,0]=o*(-0.446140926476)*x_ref[0]**(o-1)+(0.658918134389)
        ref[2,1,0,1]=o*(-0.166087456007)*x_ref[1]**(o-1)+(0.328044805625)
        ref[2,1,1,0]=o*(0.937161634589)*x_ref[0]**(o-1)+(0.594322457074)
        ref[2,1,1,1]=o*(0.992336676032)*x_ref[1]**(o-1)+(-0.499637100315)
        ref[3,0,0,0]=o*(0.767677974163)*x_ref[0]**(o-1)+(-0.820615663298)
        ref[3,0,0,1]=o*(0.558185245925)*x_ref[1]**(o-1)+(-0.763525476816)
        ref[3,0,1,0]=o*(-0.866581629552)*x_ref[0]**(o-1)+(0.589993777037)
        ref[3,0,1,1]=o*(-0.0704817160146)*x_ref[1]**(o-1)+(-0.897469280048)
        ref[3,1,0,0]=o*(-0.290118027748)*x_ref[0]**(o-1)+(0.211790364383)
        ref[3,1,0,1]=o*(-0.0688758465892)*x_ref[1]**(o-1)+(0.777490292219)
        ref[3,1,1,0]=o*(-0.969418722549)*x_ref[0]**(o-1)+(-0.46884683812)
        ref[3,1,1,1]=o*(0.826696594862)*x_ref[1]**(o-1)+(-0.926802811507)
        ref[4,0,0,0]=o*(-0.230285931083)*x_ref[0]**(o-1)+(0.905695021426)
        ref[4,0,0,1]=o*(-0.581906633624)*x_ref[1]**(o-1)+(-0.69277147014)
        ref[4,0,1,0]=o*(-0.251350536276)*x_ref[0]**(o-1)+(0.352877501605)
        ref[4,0,1,1]=o*(0.290432460788)*x_ref[1]**(o-1)+(0.614218150117)
        ref[4,1,0,0]=o*(-0.475014635407)*x_ref[0]**(o-1)+(0.86786497963)
        ref[4,1,0,1]=o*(-0.763349511499)*x_ref[1]**(o-1)+(0.817847286727)
        ref[4,1,1,0]=o*(0.0079711463132)*x_ref[0]**(o-1)+(-0.20685339926)
        ref[4,1,1,1]=o*(0.377467915936)*x_ref[1]**(o-1)+(-0.209268733968)
        ref[5,0,0,0]=o*(0.795790630791)*x_ref[0]**(o-1)+(0.0661201279192)
        ref[5,0,0,1]=o*(0.932665258262)*x_ref[1]**(o-1)+(-0.4785884914)
        ref[5,0,1,0]=o*(0.929422646068)*x_ref[0]**(o-1)+(0.775378282324)
        ref[5,0,1,1]=o*(0.297482039582)*x_ref[1]**(o-1)+(-0.700872789842)
        ref[5,1,0,0]=o*(0.678688297204)*x_ref[0]**(o-1)+(0.365975847321)
        ref[5,1,0,1]=o*(-0.565261107289)*x_ref[1]**(o-1)+(0.0317556604619)
        ref[5,1,1,0]=o*(-0.0231297032408)*x_ref[0]**(o-1)+(-0.985865541949)
        ref[5,1,1,1]=o*(0.733821736669)*x_ref[1]**(o-1)+(0.603255951212)
      else:
        arg[0,0,0]=(-0.211459759219)*x[0]**o+(-0.577684864446)*x[0]+(-0.391603126843)*x[1]**o+(0.260321825768)*x[1]+(-0.314038207978)*x[2]**o+(-0.981440041663)*x[2]
        arg[0,0,1]=(-0.876053365684)*x[0]**o+(-0.194899445893)*x[0]+(0.346950377505)*x[1]**o+(0.571865696578)*x[1]+(0.632901527485)*x[2]**o+(0.832258490904)*x[2]
        arg[0,1,0]=(-0.0779003885155)*x[0]**o+(0.583384314998)*x[0]+(-0.257262737021)*x[1]**o+(0.456730130081)*x[1]+(0.262812418811)*x[2]**o+(0.0815640391979)*x[2]
        arg[0,1,1]=(0.087484387098)*x[0]**o+(-0.415375968565)*x[0]+(0.31138114792)*x[1]**o+(0.832138685168)*x[1]+(0.552625436456)*x[2]**o+(-0.0969172211154)*x[2]
        arg[1,0,0]=(0.721326796741)*x[0]**o+(-0.556644260079)*x[0]+(0.14718294664)*x[1]**o+(0.688425309252)*x[1]+(-0.16480505764)*x[2]**o+(-0.914914176523)*x[2]
        arg[1,0,1]=(0.0932763269908)*x[0]**o+(0.359036469617)*x[0]+(0.568509073358)*x[1]**o+(-0.42133310351)*x[1]+(0.576833556667)*x[2]**o+(0.44466763296)*x[2]
        arg[1,1,0]=(0.564028751408)*x[0]**o+(-0.314840429631)*x[0]+(-0.910523044917)*x[1]**o+(-0.0377714078758)*x[1]+(0.0125836031881)*x[2]**o+(0.602273699833)*x[2]
        arg[1,1,1]=(0.135282471986)*x[0]**o+(0.163613895979)*x[0]+(0.115685034139)*x[1]**o+(-0.687078382924)*x[1]+(0.406385459521)*x[2]**o+(-0.401846794432)*x[2]
        arg[2,0,0]=(0.815695969791)*x[0]**o+(0.312205723135)*x[0]+(0.993404535838)*x[1]**o+(0.101946422158)*x[1]+(-0.331818829998)*x[2]**o+(-0.0034480605816)*x[2]
        arg[2,0,1]=(0.638646208966)*x[0]**o+(0.265838992948)*x[0]+(0.156594972273)*x[1]**o+(0.935292984535)*x[1]+(-0.930522319399)*x[2]**o+(0.540679813135)*x[2]
        arg[2,1,0]=(-0.778823826824)*x[0]**o+(0.435399707656)*x[0]+(0.123031900278)*x[1]**o+(-0.132556439303)*x[1]+(-0.29114590778)*x[2]**o+(-0.277636472779)*x[2]
        arg[2,1,1]=(-0.200541347963)*x[0]**o+(-0.526841282198)*x[0]+(0.670718267305)*x[1]**o+(0.0777161510281)*x[1]+(0.847335718393)*x[2]**o+(0.167185849891)*x[2]
        arg[3,0,0]=(0.481112023661)*x[0]**o+(-0.703558873662)*x[0]+(0.0661746324247)*x[1]**o+(0.0628113595423)*x[1]+(0.804110728888)*x[2]**o+(0.517297444342)*x[2]
        arg[3,0,1]=(-0.44648091284)*x[0]**o+(-0.976332452987)*x[0]+(-0.886297355627)*x[1]**o+(-0.910033293533)*x[1]+(0.946007442337)*x[2]**o+(-0.586006699603)*x[2]
        arg[3,1,0]=(0.686532047098)*x[0]**o+(0.647146535444)*x[0]+(-0.424413819624)*x[1]**o+(-0.266078497115)*x[1]+(-0.656118336723)*x[2]**o+(0.902145487914)*x[2]
        arg[3,1,1]=(-0.0562438759603)*x[0]**o+(0.600872472135)*x[0]+(-0.162016187788)*x[1]**o+(-0.685387926)*x[1]+(-0.899318041513)*x[2]**o+(0.459743174318)*x[2]
        arg[4,0,0]=(-0.923373905806)*x[0]**o+(-0.0340303642214)*x[0]+(-0.798971449517)*x[1]**o+(-0.45521271974)*x[1]+(-0.354501811953)*x[2]**o+(0.0668579007274)*x[2]
        arg[4,0,1]=(0.411397784181)*x[0]**o+(0.112609440102)*x[0]+(0.352899973694)*x[1]**o+(-0.0728024007714)*x[1]+(0.50804391825)*x[2]**o+(0.748548958571)*x[2]
        arg[4,1,0]=(0.875890044031)*x[0]**o+(0.674204747948)*x[0]+(-0.879286517401)*x[1]**o+(-0.00338161978492)*x[1]+(-0.372790489717)*x[2]**o+(0.480822881409)*x[2]
        arg[4,1,1]=(-0.224827091498)*x[0]**o+(0.854170572245)*x[0]+(-0.670467931784)*x[1]**o+(0.85225464592)*x[1]+(0.41885719338)*x[2]**o+(-0.623296073831)*x[2]
        arg[5,0,0]=(-0.662704253172)*x[0]**o+(0.690656552406)*x[0]+(-0.812436958497)*x[1]**o+(0.928013493065)*x[1]+(0.763792541097)*x[2]**o+(-0.430084528987)*x[2]
        arg[5,0,1]=(0.451167682598)*x[0]**o+(0.373015986015)*x[0]+(0.73334610852)*x[1]**o+(0.44892130407)*x[1]+(0.820233215547)*x[2]**o+(0.863864357936)*x[2]
        arg[5,1,0]=(-0.700840074185)*x[0]**o+(-0.637964737375)*x[0]+(0.0532062256138)*x[1]**o+(-0.902662868597)*x[1]+(-0.91668008633)*x[2]**o+(0.00253116545509)*x[2]
        arg[5,1,1]=(0.0461834200484)*x[0]**o+(0.102406795193)*x[0]+(0.599142154957)*x[1]**o+(0.374585801545)*x[1]+(-0.444968156109)*x[2]**o+(-0.932860816729)*x[2]
        ref[0,0,0,0]=o*(-0.211459759219)*x_ref[0]**(o-1)+(-0.577684864446)
        ref[0,0,0,1]=o*(-0.391603126843)*x_ref[1]**(o-1)+(0.260321825768)
        ref[0,0,0,2]=o*(-0.314038207978)*x_ref[2]**(o-1)+(-0.981440041663)
        ref[0,0,1,0]=o*(-0.876053365684)*x_ref[0]**(o-1)+(-0.194899445893)
        ref[0,0,1,1]=o*(0.346950377505)*x_ref[1]**(o-1)+(0.571865696578)
        ref[0,0,1,2]=o*(0.632901527485)*x_ref[2]**(o-1)+(0.832258490904)
        ref[0,1,0,0]=o*(-0.0779003885155)*x_ref[0]**(o-1)+(0.583384314998)
        ref[0,1,0,1]=o*(-0.257262737021)*x_ref[1]**(o-1)+(0.456730130081)
        ref[0,1,0,2]=o*(0.262812418811)*x_ref[2]**(o-1)+(0.0815640391979)
        ref[0,1,1,0]=o*(0.087484387098)*x_ref[0]**(o-1)+(-0.415375968565)
        ref[0,1,1,1]=o*(0.31138114792)*x_ref[1]**(o-1)+(0.832138685168)
        ref[0,1,1,2]=o*(0.552625436456)*x_ref[2]**(o-1)+(-0.0969172211154)
        ref[1,0,0,0]=o*(0.721326796741)*x_ref[0]**(o-1)+(-0.556644260079)
        ref[1,0,0,1]=o*(0.14718294664)*x_ref[1]**(o-1)+(0.688425309252)
        ref[1,0,0,2]=o*(-0.16480505764)*x_ref[2]**(o-1)+(-0.914914176523)
        ref[1,0,1,0]=o*(0.0932763269908)*x_ref[0]**(o-1)+(0.359036469617)
        ref[1,0,1,1]=o*(0.568509073358)*x_ref[1]**(o-1)+(-0.42133310351)
        ref[1,0,1,2]=o*(0.576833556667)*x_ref[2]**(o-1)+(0.44466763296)
        ref[1,1,0,0]=o*(0.564028751408)*x_ref[0]**(o-1)+(-0.314840429631)
        ref[1,1,0,1]=o*(-0.910523044917)*x_ref[1]**(o-1)+(-0.0377714078758)
        ref[1,1,0,2]=o*(0.0125836031881)*x_ref[2]**(o-1)+(0.602273699833)
        ref[1,1,1,0]=o*(0.135282471986)*x_ref[0]**(o-1)+(0.163613895979)
        ref[1,1,1,1]=o*(0.115685034139)*x_ref[1]**(o-1)+(-0.687078382924)
        ref[1,1,1,2]=o*(0.406385459521)*x_ref[2]**(o-1)+(-0.401846794432)
        ref[2,0,0,0]=o*(0.815695969791)*x_ref[0]**(o-1)+(0.312205723135)
        ref[2,0,0,1]=o*(0.993404535838)*x_ref[1]**(o-1)+(0.101946422158)
        ref[2,0,0,2]=o*(-0.331818829998)*x_ref[2]**(o-1)+(-0.0034480605816)
        ref[2,0,1,0]=o*(0.638646208966)*x_ref[0]**(o-1)+(0.265838992948)
        ref[2,0,1,1]=o*(0.156594972273)*x_ref[1]**(o-1)+(0.935292984535)
        ref[2,0,1,2]=o*(-0.930522319399)*x_ref[2]**(o-1)+(0.540679813135)
        ref[2,1,0,0]=o*(-0.778823826824)*x_ref[0]**(o-1)+(0.435399707656)
        ref[2,1,0,1]=o*(0.123031900278)*x_ref[1]**(o-1)+(-0.132556439303)
        ref[2,1,0,2]=o*(-0.29114590778)*x_ref[2]**(o-1)+(-0.277636472779)
        ref[2,1,1,0]=o*(-0.200541347963)*x_ref[0]**(o-1)+(-0.526841282198)
        ref[2,1,1,1]=o*(0.670718267305)*x_ref[1]**(o-1)+(0.0777161510281)
        ref[2,1,1,2]=o*(0.847335718393)*x_ref[2]**(o-1)+(0.167185849891)
        ref[3,0,0,0]=o*(0.481112023661)*x_ref[0]**(o-1)+(-0.703558873662)
        ref[3,0,0,1]=o*(0.0661746324247)*x_ref[1]**(o-1)+(0.0628113595423)
        ref[3,0,0,2]=o*(0.804110728888)*x_ref[2]**(o-1)+(0.517297444342)
        ref[3,0,1,0]=o*(-0.44648091284)*x_ref[0]**(o-1)+(-0.976332452987)
        ref[3,0,1,1]=o*(-0.886297355627)*x_ref[1]**(o-1)+(-0.910033293533)
        ref[3,0,1,2]=o*(0.946007442337)*x_ref[2]**(o-1)+(-0.586006699603)
        ref[3,1,0,0]=o*(0.686532047098)*x_ref[0]**(o-1)+(0.647146535444)
        ref[3,1,0,1]=o*(-0.424413819624)*x_ref[1]**(o-1)+(-0.266078497115)
        ref[3,1,0,2]=o*(-0.656118336723)*x_ref[2]**(o-1)+(0.902145487914)
        ref[3,1,1,0]=o*(-0.0562438759603)*x_ref[0]**(o-1)+(0.600872472135)
        ref[3,1,1,1]=o*(-0.162016187788)*x_ref[1]**(o-1)+(-0.685387926)
        ref[3,1,1,2]=o*(-0.899318041513)*x_ref[2]**(o-1)+(0.459743174318)
        ref[4,0,0,0]=o*(-0.923373905806)*x_ref[0]**(o-1)+(-0.0340303642214)
        ref[4,0,0,1]=o*(-0.798971449517)*x_ref[1]**(o-1)+(-0.45521271974)
        ref[4,0,0,2]=o*(-0.354501811953)*x_ref[2]**(o-1)+(0.0668579007274)
        ref[4,0,1,0]=o*(0.411397784181)*x_ref[0]**(o-1)+(0.112609440102)
        ref[4,0,1,1]=o*(0.352899973694)*x_ref[1]**(o-1)+(-0.0728024007714)
        ref[4,0,1,2]=o*(0.50804391825)*x_ref[2]**(o-1)+(0.748548958571)
        ref[4,1,0,0]=o*(0.875890044031)*x_ref[0]**(o-1)+(0.674204747948)
        ref[4,1,0,1]=o*(-0.879286517401)*x_ref[1]**(o-1)+(-0.00338161978492)
        ref[4,1,0,2]=o*(-0.372790489717)*x_ref[2]**(o-1)+(0.480822881409)
        ref[4,1,1,0]=o*(-0.224827091498)*x_ref[0]**(o-1)+(0.854170572245)
        ref[4,1,1,1]=o*(-0.670467931784)*x_ref[1]**(o-1)+(0.85225464592)
        ref[4,1,1,2]=o*(0.41885719338)*x_ref[2]**(o-1)+(-0.623296073831)
        ref[5,0,0,0]=o*(-0.662704253172)*x_ref[0]**(o-1)+(0.690656552406)
        ref[5,0,0,1]=o*(-0.812436958497)*x_ref[1]**(o-1)+(0.928013493065)
        ref[5,0,0,2]=o*(0.763792541097)*x_ref[2]**(o-1)+(-0.430084528987)
        ref[5,0,1,0]=o*(0.451167682598)*x_ref[0]**(o-1)+(0.373015986015)
        ref[5,0,1,1]=o*(0.73334610852)*x_ref[1]**(o-1)+(0.44892130407)
        ref[5,0,1,2]=o*(0.820233215547)*x_ref[2]**(o-1)+(0.863864357936)
        ref[5,1,0,0]=o*(-0.700840074185)*x_ref[0]**(o-1)+(-0.637964737375)
        ref[5,1,0,1]=o*(0.0532062256138)*x_ref[1]**(o-1)+(-0.902662868597)
        ref[5,1,0,2]=o*(-0.91668008633)*x_ref[2]**(o-1)+(0.00253116545509)
        ref[5,1,1,0]=o*(0.0461834200484)*x_ref[0]**(o-1)+(0.102406795193)
        ref[5,1,1,1]=o*(0.599142154957)*x_ref[1]**(o-1)+(0.374585801545)
        ref[5,1,1,2]=o*(-0.444968156109)*x_ref[2]**(o-1)+(-0.932860816729)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(1.37238770251)*x[0]+(-1.43917937019)*x[1]
        ref[0]=(1.37238770251)
        ref[1]=(-1.43917937019)
      else:
        arg=(0.951563810847)*x[0]+(-1.91408943437)*x[1]+(0.233861671105)*x[2]
        ref[0]=(0.951563810847)
        ref[1]=(-1.91408943437)
        ref[2]=(0.233861671105)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(1.42921344972)*x[0]+(-0.911436505176)*x[1]
        arg[1]=(1.25688300316)*x[0]+(0.0536640238292)*x[1]
        ref[0,0]=(1.42921344972)
        ref[0,1]=(-0.911436505176)
        ref[1,0]=(1.25688300316)
        ref[1,1]=(0.0536640238292)
      else:
        arg[0]=(-1.57773112011)*x[0]+(-0.697311790617)*x[1]+(-0.480770363787)*x[2]
        arg[1]=(-0.136063655594)*x[0]+(0.882253548646)*x[1]+(1.35773570995)*x[2]
        ref[0,0]=(-1.57773112011)
        ref[0,1]=(-0.697311790617)
        ref[0,2]=(-0.480770363787)
        ref[1,0]=(-0.136063655594)
        ref[1,1]=(0.882253548646)
        ref[1,2]=(1.35773570995)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.448247291087)*x[0]+(-0.00880442943882)*x[1]
        arg[0,1]=(-0.59784234455)*x[0]+(-0.94951606979)*x[1]
        arg[0,2]=(1.38316656933)*x[0]+(-0.124817047005)*x[1]
        arg[0,3]=(-0.0748469893414)*x[0]+(1.44145142047)*x[1]
        arg[0,4]=(1.81133385456)*x[0]+(-0.518406894403)*x[1]
        arg[1,0]=(-0.65460152624)*x[0]+(-0.350858526434)*x[1]
        arg[1,1]=(0.924058009558)*x[0]+(-0.40973995895)*x[1]
        arg[1,2]=(-0.299794705178)*x[0]+(-0.924557696915)*x[1]
        arg[1,3]=(-0.608513627169)*x[0]+(-0.141936714389)*x[1]
        arg[1,4]=(0.469099778997)*x[0]+(0.754132353456)*x[1]
        arg[2,0]=(-0.627098551951)*x[0]+(0.476865448119)*x[1]
        arg[2,1]=(-1.03564375134)*x[0]+(-0.280009187543)*x[1]
        arg[2,2]=(0.475083576156)*x[0]+(0.220248066616)*x[1]
        arg[2,3]=(0.0420499398173)*x[0]+(0.0808105284548)*x[1]
        arg[2,4]=(0.368173655662)*x[0]+(-0.58266909961)*x[1]
        arg[3,0]=(0.450851110298)*x[0]+(0.0844590226087)*x[1]
        arg[3,1]=(0.508010554654)*x[0]+(0.797109833792)*x[1]
        arg[3,2]=(0.711479388001)*x[0]+(1.12829906729)*x[1]
        arg[3,3]=(0.0588799697153)*x[0]+(-0.991874826409)*x[1]
        arg[3,4]=(0.743446045536)*x[0]+(0.65754450588)*x[1]
        ref[0,0,0]=(0.448247291087)
        ref[0,0,1]=(-0.00880442943882)
        ref[0,1,0]=(-0.59784234455)
        ref[0,1,1]=(-0.94951606979)
        ref[0,2,0]=(1.38316656933)
        ref[0,2,1]=(-0.124817047005)
        ref[0,3,0]=(-0.0748469893414)
        ref[0,3,1]=(1.44145142047)
        ref[0,4,0]=(1.81133385456)
        ref[0,4,1]=(-0.518406894403)
        ref[1,0,0]=(-0.65460152624)
        ref[1,0,1]=(-0.350858526434)
        ref[1,1,0]=(0.924058009558)
        ref[1,1,1]=(-0.40973995895)
        ref[1,2,0]=(-0.299794705178)
        ref[1,2,1]=(-0.924557696915)
        ref[1,3,0]=(-0.608513627169)
        ref[1,3,1]=(-0.141936714389)
        ref[1,4,0]=(0.469099778997)
        ref[1,4,1]=(0.754132353456)
        ref[2,0,0]=(-0.627098551951)
        ref[2,0,1]=(0.476865448119)
        ref[2,1,0]=(-1.03564375134)
        ref[2,1,1]=(-0.280009187543)
        ref[2,2,0]=(0.475083576156)
        ref[2,2,1]=(0.220248066616)
        ref[2,3,0]=(0.0420499398173)
        ref[2,3,1]=(0.0808105284548)
        ref[2,4,0]=(0.368173655662)
        ref[2,4,1]=(-0.58266909961)
        ref[3,0,0]=(0.450851110298)
        ref[3,0,1]=(0.0844590226087)
        ref[3,1,0]=(0.508010554654)
        ref[3,1,1]=(0.797109833792)
        ref[3,2,0]=(0.711479388001)
        ref[3,2,1]=(1.12829906729)
        ref[3,3,0]=(0.0588799697153)
        ref[3,3,1]=(-0.991874826409)
        ref[3,4,0]=(0.743446045536)
        ref[3,4,1]=(0.65754450588)
      else:
        arg[0,0]=(-0.43386387868)*x[0]+(0.321105491776)*x[1]+(0.384206679414)*x[2]
        arg[0,1]=(0.604698878149)*x[0]+(0.594375439325)*x[1]+(0.162832272176)*x[2]
        arg[0,2]=(-0.332194829416)*x[0]+(0.600821281937)*x[1]+(0.493622302813)*x[2]
        arg[0,3]=(0.846367268698)*x[0]+(-1.32744832381)*x[1]+(0.00466654421944)*x[2]
        arg[0,4]=(0.385111950936)*x[0]+(-0.176705693397)*x[1]+(0.461732370075)*x[2]
        arg[1,0]=(-0.14529643778)*x[0]+(1.15329219838)*x[1]+(0.157842247326)*x[2]
        arg[1,1]=(0.00817149012591)*x[0]+(-0.703898404732)*x[1]+(-1.14419500663)*x[2]
        arg[1,2]=(-0.124155034423)*x[0]+(0.906000496919)*x[1]+(-0.466071596735)*x[2]
        arg[1,3]=(0.413190740147)*x[0]+(0.968639404637)*x[1]+(-1.25413312698)*x[2]
        arg[1,4]=(0.521974966379)*x[0]+(0.936384991187)*x[1]+(-0.0630547476157)*x[2]
        arg[2,0]=(-0.334976510608)*x[0]+(-0.268364721371)*x[1]+(0.806974769679)*x[2]
        arg[2,1]=(-0.0672705162033)*x[0]+(0.459808873839)*x[1]+(0.0725634327358)*x[2]
        arg[2,2]=(0.142418012401)*x[0]+(-0.400301432498)*x[1]+(-1.37021128371)*x[2]
        arg[2,3]=(-1.60617980561)*x[0]+(0.0686943218654)*x[1]+(0.647509287658)*x[2]
        arg[2,4]=(0.0102366197695)*x[0]+(-1.24104511219)*x[1]+(-0.739463479584)*x[2]
        arg[3,0]=(-0.425526939568)*x[0]+(-0.946421307227)*x[1]+(0.133101457377)*x[2]
        arg[3,1]=(1.07183638389)*x[0]+(-1.21405997903)*x[1]+(-0.962984008606)*x[2]
        arg[3,2]=(-0.913456686897)*x[0]+(0.907233128077)*x[1]+(0.581609247137)*x[2]
        arg[3,3]=(-0.729544885457)*x[0]+(-0.433435638004)*x[1]+(0.357011166494)*x[2]
        arg[3,4]=(1.21906931308)*x[0]+(0.322487716786)*x[1]+(1.05602783674)*x[2]
        ref[0,0,0]=(-0.43386387868)
        ref[0,0,1]=(0.321105491776)
        ref[0,0,2]=(0.384206679414)
        ref[0,1,0]=(0.604698878149)
        ref[0,1,1]=(0.594375439325)
        ref[0,1,2]=(0.162832272176)
        ref[0,2,0]=(-0.332194829416)
        ref[0,2,1]=(0.600821281937)
        ref[0,2,2]=(0.493622302813)
        ref[0,3,0]=(0.846367268698)
        ref[0,3,1]=(-1.32744832381)
        ref[0,3,2]=(0.00466654421944)
        ref[0,4,0]=(0.385111950936)
        ref[0,4,1]=(-0.176705693397)
        ref[0,4,2]=(0.461732370075)
        ref[1,0,0]=(-0.14529643778)
        ref[1,0,1]=(1.15329219838)
        ref[1,0,2]=(0.157842247326)
        ref[1,1,0]=(0.00817149012591)
        ref[1,1,1]=(-0.703898404732)
        ref[1,1,2]=(-1.14419500663)
        ref[1,2,0]=(-0.124155034423)
        ref[1,2,1]=(0.906000496919)
        ref[1,2,2]=(-0.466071596735)
        ref[1,3,0]=(0.413190740147)
        ref[1,3,1]=(0.968639404637)
        ref[1,3,2]=(-1.25413312698)
        ref[1,4,0]=(0.521974966379)
        ref[1,4,1]=(0.936384991187)
        ref[1,4,2]=(-0.0630547476157)
        ref[2,0,0]=(-0.334976510608)
        ref[2,0,1]=(-0.268364721371)
        ref[2,0,2]=(0.806974769679)
        ref[2,1,0]=(-0.0672705162033)
        ref[2,1,1]=(0.459808873839)
        ref[2,1,2]=(0.0725634327358)
        ref[2,2,0]=(0.142418012401)
        ref[2,2,1]=(-0.400301432498)
        ref[2,2,2]=(-1.37021128371)
        ref[2,3,0]=(-1.60617980561)
        ref[2,3,1]=(0.0686943218654)
        ref[2,3,2]=(0.647509287658)
        ref[2,4,0]=(0.0102366197695)
        ref[2,4,1]=(-1.24104511219)
        ref[2,4,2]=(-0.739463479584)
        ref[3,0,0]=(-0.425526939568)
        ref[3,0,1]=(-0.946421307227)
        ref[3,0,2]=(0.133101457377)
        ref[3,1,0]=(1.07183638389)
        ref[3,1,1]=(-1.21405997903)
        ref[3,1,2]=(-0.962984008606)
        ref[3,2,0]=(-0.913456686897)
        ref[3,2,1]=(0.907233128077)
        ref[3,2,2]=(0.581609247137)
        ref[3,3,0]=(-0.729544885457)
        ref[3,3,1]=(-0.433435638004)
        ref[3,3,2]=(0.357011166494)
        ref[3,4,0]=(1.21906931308)
        ref[3,4,1]=(0.322487716786)
        ref[3,4,2]=(1.05602783674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.287546203823)*x[0]+(0.390321729967)*x[1]
        arg[0,0,1]=(-0.112653571553)*x[0]+(-0.359753186922)*x[1]
        arg[0,1,0]=(0.00656985674969)*x[0]+(-0.397210304393)*x[1]
        arg[0,1,1]=(-0.825698431533)*x[0]+(-1.20994477496)*x[1]
        arg[1,0,0]=(0.0456789352898)*x[0]+(0.201178709355)*x[1]
        arg[1,0,1]=(0.216619650199)*x[0]+(-0.684237394127)*x[1]
        arg[1,1,0]=(-0.452605086273)*x[0]+(0.383220784575)*x[1]
        arg[1,1,1]=(-0.385315869703)*x[0]+(-1.25937320051)*x[1]
        arg[2,0,0]=(-0.546290546777)*x[0]+(0.0659124729108)*x[1]
        arg[2,0,1]=(0.23413834434)*x[0]+(0.241510050424)*x[1]
        arg[2,1,0]=(-0.541097368124)*x[0]+(0.458682016611)*x[1]
        arg[2,1,1]=(0.314030173817)*x[0]+(0.083613884286)*x[1]
        arg[3,0,0]=(0.862005271666)*x[0]+(0.858693755091)*x[1]
        arg[3,0,1]=(-1.14220805291)*x[0]+(0.609875693913)*x[1]
        arg[3,1,0]=(-0.847069360014)*x[0]+(0.240069314456)*x[1]
        arg[3,1,1]=(0.480596211718)*x[0]+(-1.40901297384)*x[1]
        arg[4,0,0]=(-1.38036653523)*x[0]+(-0.0732955000278)*x[1]
        arg[4,0,1]=(0.0716061704803)*x[0]+(0.180127379412)*x[1]
        arg[4,1,0]=(1.26959959862)*x[0]+(-1.38131896436)*x[1]
        arg[4,1,1]=(0.204280361123)*x[0]+(0.532611514369)*x[1]
        arg[5,0,0]=(0.209536293545)*x[0]+(-0.303994906788)*x[1]
        arg[5,0,1]=(1.41150162289)*x[0]+(1.32515735407)*x[1]
        arg[5,1,0]=(1.59736027384)*x[0]+(-0.711055564868)*x[1]
        arg[5,1,1]=(-1.31799658024)*x[0]+(0.829209493108)*x[1]
        ref[0,0,0,0]=(-0.287546203823)
        ref[0,0,0,1]=(0.390321729967)
        ref[0,0,1,0]=(-0.112653571553)
        ref[0,0,1,1]=(-0.359753186922)
        ref[0,1,0,0]=(0.00656985674969)
        ref[0,1,0,1]=(-0.397210304393)
        ref[0,1,1,0]=(-0.825698431533)
        ref[0,1,1,1]=(-1.20994477496)
        ref[1,0,0,0]=(0.0456789352898)
        ref[1,0,0,1]=(0.201178709355)
        ref[1,0,1,0]=(0.216619650199)
        ref[1,0,1,1]=(-0.684237394127)
        ref[1,1,0,0]=(-0.452605086273)
        ref[1,1,0,1]=(0.383220784575)
        ref[1,1,1,0]=(-0.385315869703)
        ref[1,1,1,1]=(-1.25937320051)
        ref[2,0,0,0]=(-0.546290546777)
        ref[2,0,0,1]=(0.0659124729108)
        ref[2,0,1,0]=(0.23413834434)
        ref[2,0,1,1]=(0.241510050424)
        ref[2,1,0,0]=(-0.541097368124)
        ref[2,1,0,1]=(0.458682016611)
        ref[2,1,1,0]=(0.314030173817)
        ref[2,1,1,1]=(0.083613884286)
        ref[3,0,0,0]=(0.862005271666)
        ref[3,0,0,1]=(0.858693755091)
        ref[3,0,1,0]=(-1.14220805291)
        ref[3,0,1,1]=(0.609875693913)
        ref[3,1,0,0]=(-0.847069360014)
        ref[3,1,0,1]=(0.240069314456)
        ref[3,1,1,0]=(0.480596211718)
        ref[3,1,1,1]=(-1.40901297384)
        ref[4,0,0,0]=(-1.38036653523)
        ref[4,0,0,1]=(-0.0732955000278)
        ref[4,0,1,0]=(0.0716061704803)
        ref[4,0,1,1]=(0.180127379412)
        ref[4,1,0,0]=(1.26959959862)
        ref[4,1,0,1]=(-1.38131896436)
        ref[4,1,1,0]=(0.204280361123)
        ref[4,1,1,1]=(0.532611514369)
        ref[5,0,0,0]=(0.209536293545)
        ref[5,0,0,1]=(-0.303994906788)
        ref[5,0,1,0]=(1.41150162289)
        ref[5,0,1,1]=(1.32515735407)
        ref[5,1,0,0]=(1.59736027384)
        ref[5,1,0,1]=(-0.711055564868)
        ref[5,1,1,0]=(-1.31799658024)
        ref[5,1,1,1]=(0.829209493108)
      else:
        arg[0,0,0]=(0.832267408258)*x[0]+(-1.1995239623)*x[1]+(-1.81276973398)*x[2]
        arg[0,0,1]=(-0.0125232856194)*x[0]+(0.122543918666)*x[1]+(-1.12411286836)*x[2]
        arg[0,1,0]=(0.985611388383)*x[0]+(-1.17217207383)*x[1]+(-0.231241872108)*x[2]
        arg[0,1,1]=(0.625494139449)*x[0]+(0.784802683192)*x[1]+(0.612764525973)*x[2]
        arg[1,0,0]=(0.327644975022)*x[0]+(0.0552951801276)*x[1]+(-0.806887445103)*x[2]
        arg[1,0,1]=(0.194338798022)*x[0]+(0.799902026341)*x[1]+(1.70164249741)*x[2]
        arg[1,1,0]=(0.890832636865)*x[0]+(0.223899365292)*x[1]+(-0.283094526824)*x[2]
        arg[1,1,1]=(-0.122108330088)*x[0]+(-0.293413695547)*x[1]+(-0.371581852879)*x[2]
        arg[2,0,0]=(-0.469341678297)*x[0]+(0.152856292978)*x[1]+(0.765861314481)*x[2]
        arg[2,0,1]=(0.249219162734)*x[0]+(0.319622173697)*x[1]+(-0.182998471546)*x[2]
        arg[2,1,0]=(0.39687496584)*x[0]+(0.137084411978)*x[1]+(-0.34707250854)*x[2]
        arg[2,1,1]=(-0.382391798519)*x[0]+(0.755317411547)*x[1]+(0.424204162929)*x[2]
        arg[3,0,0]=(1.12637976255)*x[0]+(0.11632178776)*x[1]+(-0.615547388072)*x[2]
        arg[3,0,1]=(-0.819303259446)*x[0]+(0.916951216368)*x[1]+(-0.572792050441)*x[2]
        arg[3,1,0]=(0.194642055711)*x[0]+(0.949663114037)*x[1]+(-0.403001914682)*x[2]
        arg[3,1,1]=(0.902547242635)*x[0]+(-1.04027936291)*x[1]+(-1.78463515221)*x[2]
        arg[4,0,0]=(0.59060036737)*x[0]+(-0.580905638676)*x[1]+(-0.486098556884)*x[2]
        arg[4,0,1]=(-0.330634605524)*x[0]+(-0.758140973629)*x[1]+(-0.192282397672)*x[2]
        arg[4,1,0]=(1.40502801336)*x[0]+(0.199036161455)*x[1]+(-0.558660947885)*x[2]
        arg[4,1,1]=(-0.155588274273)*x[0]+(-0.57050736529)*x[1]+(1.1998664442)*x[2]
        arg[5,0,0]=(-0.754680865929)*x[0]+(1.29040984028)*x[1]+(-1.83778974463)*x[2]
        arg[5,0,1]=(0.274692501647)*x[0]+(0.368546080921)*x[1]+(-0.446376269232)*x[2]
        arg[5,1,0]=(-0.758704372861)*x[0]+(1.75555227426)*x[1]+(-0.13664688871)*x[2]
        arg[5,1,1]=(-0.568468068187)*x[0]+(-0.320218561544)*x[1]+(-0.827795466031)*x[2]
        ref[0,0,0,0]=(0.832267408258)
        ref[0,0,0,1]=(-1.1995239623)
        ref[0,0,0,2]=(-1.81276973398)
        ref[0,0,1,0]=(-0.0125232856194)
        ref[0,0,1,1]=(0.122543918666)
        ref[0,0,1,2]=(-1.12411286836)
        ref[0,1,0,0]=(0.985611388383)
        ref[0,1,0,1]=(-1.17217207383)
        ref[0,1,0,2]=(-0.231241872108)
        ref[0,1,1,0]=(0.625494139449)
        ref[0,1,1,1]=(0.784802683192)
        ref[0,1,1,2]=(0.612764525973)
        ref[1,0,0,0]=(0.327644975022)
        ref[1,0,0,1]=(0.0552951801276)
        ref[1,0,0,2]=(-0.806887445103)
        ref[1,0,1,0]=(0.194338798022)
        ref[1,0,1,1]=(0.799902026341)
        ref[1,0,1,2]=(1.70164249741)
        ref[1,1,0,0]=(0.890832636865)
        ref[1,1,0,1]=(0.223899365292)
        ref[1,1,0,2]=(-0.283094526824)
        ref[1,1,1,0]=(-0.122108330088)
        ref[1,1,1,1]=(-0.293413695547)
        ref[1,1,1,2]=(-0.371581852879)
        ref[2,0,0,0]=(-0.469341678297)
        ref[2,0,0,1]=(0.152856292978)
        ref[2,0,0,2]=(0.765861314481)
        ref[2,0,1,0]=(0.249219162734)
        ref[2,0,1,1]=(0.319622173697)
        ref[2,0,1,2]=(-0.182998471546)
        ref[2,1,0,0]=(0.39687496584)
        ref[2,1,0,1]=(0.137084411978)
        ref[2,1,0,2]=(-0.34707250854)
        ref[2,1,1,0]=(-0.382391798519)
        ref[2,1,1,1]=(0.755317411547)
        ref[2,1,1,2]=(0.424204162929)
        ref[3,0,0,0]=(1.12637976255)
        ref[3,0,0,1]=(0.11632178776)
        ref[3,0,0,2]=(-0.615547388072)
        ref[3,0,1,0]=(-0.819303259446)
        ref[3,0,1,1]=(0.916951216368)
        ref[3,0,1,2]=(-0.572792050441)
        ref[3,1,0,0]=(0.194642055711)
        ref[3,1,0,1]=(0.949663114037)
        ref[3,1,0,2]=(-0.403001914682)
        ref[3,1,1,0]=(0.902547242635)
        ref[3,1,1,1]=(-1.04027936291)
        ref[3,1,1,2]=(-1.78463515221)
        ref[4,0,0,0]=(0.59060036737)
        ref[4,0,0,1]=(-0.580905638676)
        ref[4,0,0,2]=(-0.486098556884)
        ref[4,0,1,0]=(-0.330634605524)
        ref[4,0,1,1]=(-0.758140973629)
        ref[4,0,1,2]=(-0.192282397672)
        ref[4,1,0,0]=(1.40502801336)
        ref[4,1,0,1]=(0.199036161455)
        ref[4,1,0,2]=(-0.558660947885)
        ref[4,1,1,0]=(-0.155588274273)
        ref[4,1,1,1]=(-0.57050736529)
        ref[4,1,1,2]=(1.1998664442)
        ref[5,0,0,0]=(-0.754680865929)
        ref[5,0,0,1]=(1.29040984028)
        ref[5,0,0,2]=(-1.83778974463)
        ref[5,0,1,0]=(0.274692501647)
        ref[5,0,1,1]=(0.368546080921)
        ref[5,0,1,2]=(-0.446376269232)
        ref[5,1,0,0]=(-0.758704372861)
        ref[5,1,0,1]=(1.75555227426)
        ref[5,1,0,2]=(-0.13664688871)
        ref[5,1,1,0]=(-0.568468068187)
        ref[5,1,1,1]=(-0.320218561544)
        ref[5,1,1,2]=(-0.827795466031)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(1.37238770251)*x[0]+(-1.43917937019)*x[1]
        ref[0]=(1.37238770251)
        ref[1]=(-1.43917937019)
      else:
        arg=(0.951563810847)*x[0]+(-1.91408943437)*x[1]+(0.233861671105)*x[2]
        ref[0]=(0.951563810847)
        ref[1]=(-1.91408943437)
        ref[2]=(0.233861671105)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(1.42921344972)*x[0]+(-0.911436505176)*x[1]
        arg[1]=(1.25688300316)*x[0]+(0.0536640238292)*x[1]
        ref[0,0]=(1.42921344972)
        ref[0,1]=(-0.911436505176)
        ref[1,0]=(1.25688300316)
        ref[1,1]=(0.0536640238292)
      else:
        arg[0]=(-1.57773112011)*x[0]+(-0.697311790617)*x[1]+(-0.480770363787)*x[2]
        arg[1]=(-0.136063655594)*x[0]+(0.882253548646)*x[1]+(1.35773570995)*x[2]
        ref[0,0]=(-1.57773112011)
        ref[0,1]=(-0.697311790617)
        ref[0,2]=(-0.480770363787)
        ref[1,0]=(-0.136063655594)
        ref[1,1]=(0.882253548646)
        ref[1,2]=(1.35773570995)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.448247291087)*x[0]+(-0.00880442943882)*x[1]
        arg[0,1]=(-0.59784234455)*x[0]+(-0.94951606979)*x[1]
        arg[0,2]=(1.38316656933)*x[0]+(-0.124817047005)*x[1]
        arg[0,3]=(-0.0748469893414)*x[0]+(1.44145142047)*x[1]
        arg[0,4]=(1.81133385456)*x[0]+(-0.518406894403)*x[1]
        arg[1,0]=(-0.65460152624)*x[0]+(-0.350858526434)*x[1]
        arg[1,1]=(0.924058009558)*x[0]+(-0.40973995895)*x[1]
        arg[1,2]=(-0.299794705178)*x[0]+(-0.924557696915)*x[1]
        arg[1,3]=(-0.608513627169)*x[0]+(-0.141936714389)*x[1]
        arg[1,4]=(0.469099778997)*x[0]+(0.754132353456)*x[1]
        arg[2,0]=(-0.627098551951)*x[0]+(0.476865448119)*x[1]
        arg[2,1]=(-1.03564375134)*x[0]+(-0.280009187543)*x[1]
        arg[2,2]=(0.475083576156)*x[0]+(0.220248066616)*x[1]
        arg[2,3]=(0.0420499398173)*x[0]+(0.0808105284548)*x[1]
        arg[2,4]=(0.368173655662)*x[0]+(-0.58266909961)*x[1]
        arg[3,0]=(0.450851110298)*x[0]+(0.0844590226087)*x[1]
        arg[3,1]=(0.508010554654)*x[0]+(0.797109833792)*x[1]
        arg[3,2]=(0.711479388001)*x[0]+(1.12829906729)*x[1]
        arg[3,3]=(0.0588799697153)*x[0]+(-0.991874826409)*x[1]
        arg[3,4]=(0.743446045536)*x[0]+(0.65754450588)*x[1]
        ref[0,0,0]=(0.448247291087)
        ref[0,0,1]=(-0.00880442943882)
        ref[0,1,0]=(-0.59784234455)
        ref[0,1,1]=(-0.94951606979)
        ref[0,2,0]=(1.38316656933)
        ref[0,2,1]=(-0.124817047005)
        ref[0,3,0]=(-0.0748469893414)
        ref[0,3,1]=(1.44145142047)
        ref[0,4,0]=(1.81133385456)
        ref[0,4,1]=(-0.518406894403)
        ref[1,0,0]=(-0.65460152624)
        ref[1,0,1]=(-0.350858526434)
        ref[1,1,0]=(0.924058009558)
        ref[1,1,1]=(-0.40973995895)
        ref[1,2,0]=(-0.299794705178)
        ref[1,2,1]=(-0.924557696915)
        ref[1,3,0]=(-0.608513627169)
        ref[1,3,1]=(-0.141936714389)
        ref[1,4,0]=(0.469099778997)
        ref[1,4,1]=(0.754132353456)
        ref[2,0,0]=(-0.627098551951)
        ref[2,0,1]=(0.476865448119)
        ref[2,1,0]=(-1.03564375134)
        ref[2,1,1]=(-0.280009187543)
        ref[2,2,0]=(0.475083576156)
        ref[2,2,1]=(0.220248066616)
        ref[2,3,0]=(0.0420499398173)
        ref[2,3,1]=(0.0808105284548)
        ref[2,4,0]=(0.368173655662)
        ref[2,4,1]=(-0.58266909961)
        ref[3,0,0]=(0.450851110298)
        ref[3,0,1]=(0.0844590226087)
        ref[3,1,0]=(0.508010554654)
        ref[3,1,1]=(0.797109833792)
        ref[3,2,0]=(0.711479388001)
        ref[3,2,1]=(1.12829906729)
        ref[3,3,0]=(0.0588799697153)
        ref[3,3,1]=(-0.991874826409)
        ref[3,4,0]=(0.743446045536)
        ref[3,4,1]=(0.65754450588)
      else:
        arg[0,0]=(-0.43386387868)*x[0]+(0.321105491776)*x[1]+(0.384206679414)*x[2]
        arg[0,1]=(0.604698878149)*x[0]+(0.594375439325)*x[1]+(0.162832272176)*x[2]
        arg[0,2]=(-0.332194829416)*x[0]+(0.600821281937)*x[1]+(0.493622302813)*x[2]
        arg[0,3]=(0.846367268698)*x[0]+(-1.32744832381)*x[1]+(0.00466654421944)*x[2]
        arg[0,4]=(0.385111950936)*x[0]+(-0.176705693397)*x[1]+(0.461732370075)*x[2]
        arg[1,0]=(-0.14529643778)*x[0]+(1.15329219838)*x[1]+(0.157842247326)*x[2]
        arg[1,1]=(0.00817149012591)*x[0]+(-0.703898404732)*x[1]+(-1.14419500663)*x[2]
        arg[1,2]=(-0.124155034423)*x[0]+(0.906000496919)*x[1]+(-0.466071596735)*x[2]
        arg[1,3]=(0.413190740147)*x[0]+(0.968639404637)*x[1]+(-1.25413312698)*x[2]
        arg[1,4]=(0.521974966379)*x[0]+(0.936384991187)*x[1]+(-0.0630547476157)*x[2]
        arg[2,0]=(-0.334976510608)*x[0]+(-0.268364721371)*x[1]+(0.806974769679)*x[2]
        arg[2,1]=(-0.0672705162033)*x[0]+(0.459808873839)*x[1]+(0.0725634327358)*x[2]
        arg[2,2]=(0.142418012401)*x[0]+(-0.400301432498)*x[1]+(-1.37021128371)*x[2]
        arg[2,3]=(-1.60617980561)*x[0]+(0.0686943218654)*x[1]+(0.647509287658)*x[2]
        arg[2,4]=(0.0102366197695)*x[0]+(-1.24104511219)*x[1]+(-0.739463479584)*x[2]
        arg[3,0]=(-0.425526939568)*x[0]+(-0.946421307227)*x[1]+(0.133101457377)*x[2]
        arg[3,1]=(1.07183638389)*x[0]+(-1.21405997903)*x[1]+(-0.962984008606)*x[2]
        arg[3,2]=(-0.913456686897)*x[0]+(0.907233128077)*x[1]+(0.581609247137)*x[2]
        arg[3,3]=(-0.729544885457)*x[0]+(-0.433435638004)*x[1]+(0.357011166494)*x[2]
        arg[3,4]=(1.21906931308)*x[0]+(0.322487716786)*x[1]+(1.05602783674)*x[2]
        ref[0,0,0]=(-0.43386387868)
        ref[0,0,1]=(0.321105491776)
        ref[0,0,2]=(0.384206679414)
        ref[0,1,0]=(0.604698878149)
        ref[0,1,1]=(0.594375439325)
        ref[0,1,2]=(0.162832272176)
        ref[0,2,0]=(-0.332194829416)
        ref[0,2,1]=(0.600821281937)
        ref[0,2,2]=(0.493622302813)
        ref[0,3,0]=(0.846367268698)
        ref[0,3,1]=(-1.32744832381)
        ref[0,3,2]=(0.00466654421944)
        ref[0,4,0]=(0.385111950936)
        ref[0,4,1]=(-0.176705693397)
        ref[0,4,2]=(0.461732370075)
        ref[1,0,0]=(-0.14529643778)
        ref[1,0,1]=(1.15329219838)
        ref[1,0,2]=(0.157842247326)
        ref[1,1,0]=(0.00817149012591)
        ref[1,1,1]=(-0.703898404732)
        ref[1,1,2]=(-1.14419500663)
        ref[1,2,0]=(-0.124155034423)
        ref[1,2,1]=(0.906000496919)
        ref[1,2,2]=(-0.466071596735)
        ref[1,3,0]=(0.413190740147)
        ref[1,3,1]=(0.968639404637)
        ref[1,3,2]=(-1.25413312698)
        ref[1,4,0]=(0.521974966379)
        ref[1,4,1]=(0.936384991187)
        ref[1,4,2]=(-0.0630547476157)
        ref[2,0,0]=(-0.334976510608)
        ref[2,0,1]=(-0.268364721371)
        ref[2,0,2]=(0.806974769679)
        ref[2,1,0]=(-0.0672705162033)
        ref[2,1,1]=(0.459808873839)
        ref[2,1,2]=(0.0725634327358)
        ref[2,2,0]=(0.142418012401)
        ref[2,2,1]=(-0.400301432498)
        ref[2,2,2]=(-1.37021128371)
        ref[2,3,0]=(-1.60617980561)
        ref[2,3,1]=(0.0686943218654)
        ref[2,3,2]=(0.647509287658)
        ref[2,4,0]=(0.0102366197695)
        ref[2,4,1]=(-1.24104511219)
        ref[2,4,2]=(-0.739463479584)
        ref[3,0,0]=(-0.425526939568)
        ref[3,0,1]=(-0.946421307227)
        ref[3,0,2]=(0.133101457377)
        ref[3,1,0]=(1.07183638389)
        ref[3,1,1]=(-1.21405997903)
        ref[3,1,2]=(-0.962984008606)
        ref[3,2,0]=(-0.913456686897)
        ref[3,2,1]=(0.907233128077)
        ref[3,2,2]=(0.581609247137)
        ref[3,3,0]=(-0.729544885457)
        ref[3,3,1]=(-0.433435638004)
        ref[3,3,2]=(0.357011166494)
        ref[3,4,0]=(1.21906931308)
        ref[3,4,1]=(0.322487716786)
        ref[3,4,2]=(1.05602783674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.287546203823)*x[0]+(0.390321729967)*x[1]
        arg[0,0,1]=(-0.112653571553)*x[0]+(-0.359753186922)*x[1]
        arg[0,1,0]=(0.00656985674969)*x[0]+(-0.397210304393)*x[1]
        arg[0,1,1]=(-0.825698431533)*x[0]+(-1.20994477496)*x[1]
        arg[1,0,0]=(0.0456789352898)*x[0]+(0.201178709355)*x[1]
        arg[1,0,1]=(0.216619650199)*x[0]+(-0.684237394127)*x[1]
        arg[1,1,0]=(-0.452605086273)*x[0]+(0.383220784575)*x[1]
        arg[1,1,1]=(-0.385315869703)*x[0]+(-1.25937320051)*x[1]
        arg[2,0,0]=(-0.546290546777)*x[0]+(0.0659124729108)*x[1]
        arg[2,0,1]=(0.23413834434)*x[0]+(0.241510050424)*x[1]
        arg[2,1,0]=(-0.541097368124)*x[0]+(0.458682016611)*x[1]
        arg[2,1,1]=(0.314030173817)*x[0]+(0.083613884286)*x[1]
        arg[3,0,0]=(0.862005271666)*x[0]+(0.858693755091)*x[1]
        arg[3,0,1]=(-1.14220805291)*x[0]+(0.609875693913)*x[1]
        arg[3,1,0]=(-0.847069360014)*x[0]+(0.240069314456)*x[1]
        arg[3,1,1]=(0.480596211718)*x[0]+(-1.40901297384)*x[1]
        arg[4,0,0]=(-1.38036653523)*x[0]+(-0.0732955000278)*x[1]
        arg[4,0,1]=(0.0716061704803)*x[0]+(0.180127379412)*x[1]
        arg[4,1,0]=(1.26959959862)*x[0]+(-1.38131896436)*x[1]
        arg[4,1,1]=(0.204280361123)*x[0]+(0.532611514369)*x[1]
        arg[5,0,0]=(0.209536293545)*x[0]+(-0.303994906788)*x[1]
        arg[5,0,1]=(1.41150162289)*x[0]+(1.32515735407)*x[1]
        arg[5,1,0]=(1.59736027384)*x[0]+(-0.711055564868)*x[1]
        arg[5,1,1]=(-1.31799658024)*x[0]+(0.829209493108)*x[1]
        ref[0,0,0,0]=(-0.287546203823)
        ref[0,0,0,1]=(0.390321729967)
        ref[0,0,1,0]=(-0.112653571553)
        ref[0,0,1,1]=(-0.359753186922)
        ref[0,1,0,0]=(0.00656985674969)
        ref[0,1,0,1]=(-0.397210304393)
        ref[0,1,1,0]=(-0.825698431533)
        ref[0,1,1,1]=(-1.20994477496)
        ref[1,0,0,0]=(0.0456789352898)
        ref[1,0,0,1]=(0.201178709355)
        ref[1,0,1,0]=(0.216619650199)
        ref[1,0,1,1]=(-0.684237394127)
        ref[1,1,0,0]=(-0.452605086273)
        ref[1,1,0,1]=(0.383220784575)
        ref[1,1,1,0]=(-0.385315869703)
        ref[1,1,1,1]=(-1.25937320051)
        ref[2,0,0,0]=(-0.546290546777)
        ref[2,0,0,1]=(0.0659124729108)
        ref[2,0,1,0]=(0.23413834434)
        ref[2,0,1,1]=(0.241510050424)
        ref[2,1,0,0]=(-0.541097368124)
        ref[2,1,0,1]=(0.458682016611)
        ref[2,1,1,0]=(0.314030173817)
        ref[2,1,1,1]=(0.083613884286)
        ref[3,0,0,0]=(0.862005271666)
        ref[3,0,0,1]=(0.858693755091)
        ref[3,0,1,0]=(-1.14220805291)
        ref[3,0,1,1]=(0.609875693913)
        ref[3,1,0,0]=(-0.847069360014)
        ref[3,1,0,1]=(0.240069314456)
        ref[3,1,1,0]=(0.480596211718)
        ref[3,1,1,1]=(-1.40901297384)
        ref[4,0,0,0]=(-1.38036653523)
        ref[4,0,0,1]=(-0.0732955000278)
        ref[4,0,1,0]=(0.0716061704803)
        ref[4,0,1,1]=(0.180127379412)
        ref[4,1,0,0]=(1.26959959862)
        ref[4,1,0,1]=(-1.38131896436)
        ref[4,1,1,0]=(0.204280361123)
        ref[4,1,1,1]=(0.532611514369)
        ref[5,0,0,0]=(0.209536293545)
        ref[5,0,0,1]=(-0.303994906788)
        ref[5,0,1,0]=(1.41150162289)
        ref[5,0,1,1]=(1.32515735407)
        ref[5,1,0,0]=(1.59736027384)
        ref[5,1,0,1]=(-0.711055564868)
        ref[5,1,1,0]=(-1.31799658024)
        ref[5,1,1,1]=(0.829209493108)
      else:
        arg[0,0,0]=(0.832267408258)*x[0]+(-1.1995239623)*x[1]+(-1.81276973398)*x[2]
        arg[0,0,1]=(-0.0125232856194)*x[0]+(0.122543918666)*x[1]+(-1.12411286836)*x[2]
        arg[0,1,0]=(0.985611388383)*x[0]+(-1.17217207383)*x[1]+(-0.231241872108)*x[2]
        arg[0,1,1]=(0.625494139449)*x[0]+(0.784802683192)*x[1]+(0.612764525973)*x[2]
        arg[1,0,0]=(0.327644975022)*x[0]+(0.0552951801276)*x[1]+(-0.806887445103)*x[2]
        arg[1,0,1]=(0.194338798022)*x[0]+(0.799902026341)*x[1]+(1.70164249741)*x[2]
        arg[1,1,0]=(0.890832636865)*x[0]+(0.223899365292)*x[1]+(-0.283094526824)*x[2]
        arg[1,1,1]=(-0.122108330088)*x[0]+(-0.293413695547)*x[1]+(-0.371581852879)*x[2]
        arg[2,0,0]=(-0.469341678297)*x[0]+(0.152856292978)*x[1]+(0.765861314481)*x[2]
        arg[2,0,1]=(0.249219162734)*x[0]+(0.319622173697)*x[1]+(-0.182998471546)*x[2]
        arg[2,1,0]=(0.39687496584)*x[0]+(0.137084411978)*x[1]+(-0.34707250854)*x[2]
        arg[2,1,1]=(-0.382391798519)*x[0]+(0.755317411547)*x[1]+(0.424204162929)*x[2]
        arg[3,0,0]=(1.12637976255)*x[0]+(0.11632178776)*x[1]+(-0.615547388072)*x[2]
        arg[3,0,1]=(-0.819303259446)*x[0]+(0.916951216368)*x[1]+(-0.572792050441)*x[2]
        arg[3,1,0]=(0.194642055711)*x[0]+(0.949663114037)*x[1]+(-0.403001914682)*x[2]
        arg[3,1,1]=(0.902547242635)*x[0]+(-1.04027936291)*x[1]+(-1.78463515221)*x[2]
        arg[4,0,0]=(0.59060036737)*x[0]+(-0.580905638676)*x[1]+(-0.486098556884)*x[2]
        arg[4,0,1]=(-0.330634605524)*x[0]+(-0.758140973629)*x[1]+(-0.192282397672)*x[2]
        arg[4,1,0]=(1.40502801336)*x[0]+(0.199036161455)*x[1]+(-0.558660947885)*x[2]
        arg[4,1,1]=(-0.155588274273)*x[0]+(-0.57050736529)*x[1]+(1.1998664442)*x[2]
        arg[5,0,0]=(-0.754680865929)*x[0]+(1.29040984028)*x[1]+(-1.83778974463)*x[2]
        arg[5,0,1]=(0.274692501647)*x[0]+(0.368546080921)*x[1]+(-0.446376269232)*x[2]
        arg[5,1,0]=(-0.758704372861)*x[0]+(1.75555227426)*x[1]+(-0.13664688871)*x[2]
        arg[5,1,1]=(-0.568468068187)*x[0]+(-0.320218561544)*x[1]+(-0.827795466031)*x[2]
        ref[0,0,0,0]=(0.832267408258)
        ref[0,0,0,1]=(-1.1995239623)
        ref[0,0,0,2]=(-1.81276973398)
        ref[0,0,1,0]=(-0.0125232856194)
        ref[0,0,1,1]=(0.122543918666)
        ref[0,0,1,2]=(-1.12411286836)
        ref[0,1,0,0]=(0.985611388383)
        ref[0,1,0,1]=(-1.17217207383)
        ref[0,1,0,2]=(-0.231241872108)
        ref[0,1,1,0]=(0.625494139449)
        ref[0,1,1,1]=(0.784802683192)
        ref[0,1,1,2]=(0.612764525973)
        ref[1,0,0,0]=(0.327644975022)
        ref[1,0,0,1]=(0.0552951801276)
        ref[1,0,0,2]=(-0.806887445103)
        ref[1,0,1,0]=(0.194338798022)
        ref[1,0,1,1]=(0.799902026341)
        ref[1,0,1,2]=(1.70164249741)
        ref[1,1,0,0]=(0.890832636865)
        ref[1,1,0,1]=(0.223899365292)
        ref[1,1,0,2]=(-0.283094526824)
        ref[1,1,1,0]=(-0.122108330088)
        ref[1,1,1,1]=(-0.293413695547)
        ref[1,1,1,2]=(-0.371581852879)
        ref[2,0,0,0]=(-0.469341678297)
        ref[2,0,0,1]=(0.152856292978)
        ref[2,0,0,2]=(0.765861314481)
        ref[2,0,1,0]=(0.249219162734)
        ref[2,0,1,1]=(0.319622173697)
        ref[2,0,1,2]=(-0.182998471546)
        ref[2,1,0,0]=(0.39687496584)
        ref[2,1,0,1]=(0.137084411978)
        ref[2,1,0,2]=(-0.34707250854)
        ref[2,1,1,0]=(-0.382391798519)
        ref[2,1,1,1]=(0.755317411547)
        ref[2,1,1,2]=(0.424204162929)
        ref[3,0,0,0]=(1.12637976255)
        ref[3,0,0,1]=(0.11632178776)
        ref[3,0,0,2]=(-0.615547388072)
        ref[3,0,1,0]=(-0.819303259446)
        ref[3,0,1,1]=(0.916951216368)
        ref[3,0,1,2]=(-0.572792050441)
        ref[3,1,0,0]=(0.194642055711)
        ref[3,1,0,1]=(0.949663114037)
        ref[3,1,0,2]=(-0.403001914682)
        ref[3,1,1,0]=(0.902547242635)
        ref[3,1,1,1]=(-1.04027936291)
        ref[3,1,1,2]=(-1.78463515221)
        ref[4,0,0,0]=(0.59060036737)
        ref[4,0,0,1]=(-0.580905638676)
        ref[4,0,0,2]=(-0.486098556884)
        ref[4,0,1,0]=(-0.330634605524)
        ref[4,0,1,1]=(-0.758140973629)
        ref[4,0,1,2]=(-0.192282397672)
        ref[4,1,0,0]=(1.40502801336)
        ref[4,1,0,1]=(0.199036161455)
        ref[4,1,0,2]=(-0.558660947885)
        ref[4,1,1,0]=(-0.155588274273)
        ref[4,1,1,1]=(-0.57050736529)
        ref[4,1,1,2]=(1.1998664442)
        ref[5,0,0,0]=(-0.754680865929)
        ref[5,0,0,1]=(1.29040984028)
        ref[5,0,0,2]=(-1.83778974463)
        ref[5,0,1,0]=(0.274692501647)
        ref[5,0,1,1]=(0.368546080921)
        ref[5,0,1,2]=(-0.446376269232)
        ref[5,1,0,0]=(-0.758704372861)
        ref[5,1,0,1]=(1.75555227426)
        ref[5,1,0,2]=(-0.13664688871)
        ref[5,1,1,0]=(-0.568468068187)
        ref[5,1,1,1]=(-0.320218561544)
        ref[5,1,1,2]=(-0.827795466031)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.132000741495)*x[0]**o+(0.122399178026)*x[0]+(-0.285047923022)*x[1]**o+(0.665814814322)*x[1]
        ref[0]=o*(-0.132000741495)*x_ref[0]**(o-1)+(0.122399178026)
        ref[1]=o*(-0.285047923022)*x_ref[1]**(o-1)+(0.665814814322)
      else:
        s=(0.596417375474)*x[0]**o+(0.780542320679)*x[0]+(-0.59348113736)*x[1]**o+(0.755472585752)*x[1]+(-0.423796298696)*x[2]**o+(0.917578429027)*x[2]
        ref[0]=o*(0.596417375474)*x_ref[0]**(o-1)+(0.780542320679)
        ref[1]=o*(-0.59348113736)*x_ref[1]**(o-1)+(0.755472585752)
        ref[2]=o*(-0.423796298696)*x_ref[2]**(o-1)+(0.917578429027)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.624689355591)*x[0]**o+(-0.747728245047)*x[0]+(0.692787021065)*x[1]**o+(-0.70629082384)*x[1]
        s[1]=(-0.225695886421)*x[0]**o+(-0.444535961493)*x[0]+(0.235690936232)*x[1]**o+(-0.970876426657)*x[1]
        ref[0,0]=o*(0.624689355591)*x_ref[0]**(o-1)+(-0.747728245047)
        ref[0,1]=o*(0.692787021065)*x_ref[1]**(o-1)+(-0.70629082384)
        ref[1,0]=o*(-0.225695886421)*x_ref[0]**(o-1)+(-0.444535961493)
        ref[1,1]=o*(0.235690936232)*x_ref[1]**(o-1)+(-0.970876426657)
      else:
        s[0]=(-0.459906225365)*x[0]**o+(-0.976920232334)*x[0]+(-0.359230869877)*x[1]**o+(0.99442848994)*x[1]+(-0.462181187731)*x[2]**o+(-0.272763004354)*x[2]
        s[1]=(-0.384545997957)*x[0]**o+(0.206838344134)*x[0]+(0.827958420952)*x[1]**o+(0.851604767316)*x[1]+(0.180277208362)*x[2]**o+(0.276165244173)*x[2]
        ref[0,0]=o*(-0.459906225365)*x_ref[0]**(o-1)+(-0.976920232334)
        ref[0,1]=o*(-0.359230869877)*x_ref[1]**(o-1)+(0.99442848994)
        ref[0,2]=o*(-0.462181187731)*x_ref[2]**(o-1)+(-0.272763004354)
        ref[1,0]=o*(-0.384545997957)*x_ref[0]**(o-1)+(0.206838344134)
        ref[1,1]=o*(0.827958420952)*x_ref[1]**(o-1)+(0.851604767316)
        ref[1,2]=o*(0.180277208362)*x_ref[2]**(o-1)+(0.276165244173)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.866700100162)*x[0]**o+(-0.443253975067)*x[0]+(0.930660789831)*x[1]**o+(0.348981739779)*x[1]
        s[0,1]=(0.595655533809)*x[0]**o+(0.425701794417)*x[0]+(-0.700204056577)*x[1]**o+(-0.606084182043)*x[1]
        s[0,2]=(-0.0975376318378)*x[0]**o+(0.0352659915524)*x[0]+(0.336096329457)*x[1]**o+(-0.604536128515)*x[1]
        s[0,3]=(0.932694885424)*x[0]**o+(0.848315864182)*x[0]+(0.294198667412)*x[1]**o+(-0.548590122662)*x[1]
        s[0,4]=(-0.464671727588)*x[0]**o+(-0.479615042615)*x[0]+(-0.205470141615)*x[1]**o+(-0.771280534115)*x[1]
        s[1,0]=(0.459832819849)*x[0]**o+(-0.703780783428)*x[0]+(0.101094024974)*x[1]**o+(-0.504757888238)*x[1]
        s[1,1]=(-0.74483211902)*x[0]**o+(0.467112390979)*x[0]+(-0.956618977211)*x[1]**o+(0.455481886754)*x[1]
        s[1,2]=(0.0976209987456)*x[0]**o+(0.44469736085)*x[0]+(-0.213220033441)*x[1]**o+(0.0448880653868)*x[1]
        s[1,3]=(0.18795879454)*x[0]**o+(0.603296276978)*x[0]+(-0.915713437955)*x[1]**o+(0.911772973549)*x[1]
        s[1,4]=(-0.960678163557)*x[0]**o+(0.000798715245087)*x[0]+(-0.195050046209)*x[1]**o+(0.55429412919)*x[1]
        s[2,0]=(-0.885615347975)*x[0]**o+(-0.0757793285493)*x[0]+(0.643863139156)*x[1]**o+(-0.617158285199)*x[1]
        s[2,1]=(0.232668453484)*x[0]**o+(0.586242977718)*x[0]+(-0.728810607608)*x[1]**o+(0.826248720042)*x[1]
        s[2,2]=(-0.893353803021)*x[0]**o+(-0.510393964603)*x[0]+(0.851185573516)*x[1]**o+(0.821075933157)*x[1]
        s[2,3]=(0.567570646968)*x[0]**o+(0.282673304306)*x[0]+(0.636861243255)*x[1]**o+(0.308681123745)*x[1]
        s[2,4]=(0.843072247533)*x[0]**o+(-0.302083248085)*x[0]+(-0.147818607759)*x[1]**o+(-0.796766854695)*x[1]
        s[3,0]=(-0.000711103721221)*x[0]**o+(0.730198459281)*x[0]+(0.395333992473)*x[1]**o+(0.743896557243)*x[1]
        s[3,1]=(0.615705307589)*x[0]**o+(0.74607803139)*x[0]+(0.837364825996)*x[1]**o+(0.362527888693)*x[1]
        s[3,2]=(-0.431332008116)*x[0]**o+(-0.5449015623)*x[0]+(0.0435062014388)*x[1]**o+(-0.408079384098)*x[1]
        s[3,3]=(-0.887047605579)*x[0]**o+(-0.966245172182)*x[0]+(-0.457094377191)*x[1]**o+(0.934841781784)*x[1]
        s[3,4]=(-0.6212484366)*x[0]**o+(0.99908421004)*x[0]+(0.75098732282)*x[1]**o+(-0.0868062878539)*x[1]
        ref[0,0,0]=o*(-0.866700100162)*x_ref[0]**(o-1)+(-0.443253975067)
        ref[0,0,1]=o*(0.930660789831)*x_ref[1]**(o-1)+(0.348981739779)
        ref[0,1,0]=o*(0.595655533809)*x_ref[0]**(o-1)+(0.425701794417)
        ref[0,1,1]=o*(-0.700204056577)*x_ref[1]**(o-1)+(-0.606084182043)
        ref[0,2,0]=o*(-0.0975376318378)*x_ref[0]**(o-1)+(0.0352659915524)
        ref[0,2,1]=o*(0.336096329457)*x_ref[1]**(o-1)+(-0.604536128515)
        ref[0,3,0]=o*(0.932694885424)*x_ref[0]**(o-1)+(0.848315864182)
        ref[0,3,1]=o*(0.294198667412)*x_ref[1]**(o-1)+(-0.548590122662)
        ref[0,4,0]=o*(-0.464671727588)*x_ref[0]**(o-1)+(-0.479615042615)
        ref[0,4,1]=o*(-0.205470141615)*x_ref[1]**(o-1)+(-0.771280534115)
        ref[1,0,0]=o*(0.459832819849)*x_ref[0]**(o-1)+(-0.703780783428)
        ref[1,0,1]=o*(0.101094024974)*x_ref[1]**(o-1)+(-0.504757888238)
        ref[1,1,0]=o*(-0.74483211902)*x_ref[0]**(o-1)+(0.467112390979)
        ref[1,1,1]=o*(-0.956618977211)*x_ref[1]**(o-1)+(0.455481886754)
        ref[1,2,0]=o*(0.0976209987456)*x_ref[0]**(o-1)+(0.44469736085)
        ref[1,2,1]=o*(-0.213220033441)*x_ref[1]**(o-1)+(0.0448880653868)
        ref[1,3,0]=o*(0.18795879454)*x_ref[0]**(o-1)+(0.603296276978)
        ref[1,3,1]=o*(-0.915713437955)*x_ref[1]**(o-1)+(0.911772973549)
        ref[1,4,0]=o*(-0.960678163557)*x_ref[0]**(o-1)+(0.000798715245087)
        ref[1,4,1]=o*(-0.195050046209)*x_ref[1]**(o-1)+(0.55429412919)
        ref[2,0,0]=o*(-0.885615347975)*x_ref[0]**(o-1)+(-0.0757793285493)
        ref[2,0,1]=o*(0.643863139156)*x_ref[1]**(o-1)+(-0.617158285199)
        ref[2,1,0]=o*(0.232668453484)*x_ref[0]**(o-1)+(0.586242977718)
        ref[2,1,1]=o*(-0.728810607608)*x_ref[1]**(o-1)+(0.826248720042)
        ref[2,2,0]=o*(-0.893353803021)*x_ref[0]**(o-1)+(-0.510393964603)
        ref[2,2,1]=o*(0.851185573516)*x_ref[1]**(o-1)+(0.821075933157)
        ref[2,3,0]=o*(0.567570646968)*x_ref[0]**(o-1)+(0.282673304306)
        ref[2,3,1]=o*(0.636861243255)*x_ref[1]**(o-1)+(0.308681123745)
        ref[2,4,0]=o*(0.843072247533)*x_ref[0]**(o-1)+(-0.302083248085)
        ref[2,4,1]=o*(-0.147818607759)*x_ref[1]**(o-1)+(-0.796766854695)
        ref[3,0,0]=o*(-0.000711103721221)*x_ref[0]**(o-1)+(0.730198459281)
        ref[3,0,1]=o*(0.395333992473)*x_ref[1]**(o-1)+(0.743896557243)
        ref[3,1,0]=o*(0.615705307589)*x_ref[0]**(o-1)+(0.74607803139)
        ref[3,1,1]=o*(0.837364825996)*x_ref[1]**(o-1)+(0.362527888693)
        ref[3,2,0]=o*(-0.431332008116)*x_ref[0]**(o-1)+(-0.5449015623)
        ref[3,2,1]=o*(0.0435062014388)*x_ref[1]**(o-1)+(-0.408079384098)
        ref[3,3,0]=o*(-0.887047605579)*x_ref[0]**(o-1)+(-0.966245172182)
        ref[3,3,1]=o*(-0.457094377191)*x_ref[1]**(o-1)+(0.934841781784)
        ref[3,4,0]=o*(-0.6212484366)*x_ref[0]**(o-1)+(0.99908421004)
        ref[3,4,1]=o*(0.75098732282)*x_ref[1]**(o-1)+(-0.0868062878539)
      else:
        s[0,0]=(0.729942219527)*x[0]**o+(-0.721160545053)*x[0]+(0.426397843912)*x[1]**o+(-0.213185647354)*x[1]+(0.728447138266)*x[2]**o+(-0.607791661013)*x[2]
        s[0,1]=(0.667616392653)*x[0]**o+(0.848051730251)*x[0]+(0.121550076489)*x[1]**o+(0.0843862117633)*x[1]+(-0.509323081526)*x[2]**o+(0.736521833786)*x[2]
        s[0,2]=(0.830127288575)*x[0]**o+(0.964772646621)*x[0]+(0.913688863179)*x[1]**o+(0.957018200531)*x[1]+(-0.694170056751)*x[2]**o+(0.105973113953)*x[2]
        s[0,3]=(-0.235173218805)*x[0]**o+(0.858875579758)*x[0]+(0.823162706883)*x[1]**o+(0.108717821259)*x[1]+(0.80411364975)*x[2]**o+(0.0348079894361)*x[2]
        s[0,4]=(0.78867654833)*x[0]**o+(-0.0191953378129)*x[0]+(-0.0840244773387)*x[1]**o+(0.614156921824)*x[1]+(-0.0190397891394)*x[2]**o+(0.750918932533)*x[2]
        s[1,0]=(-0.0931449972666)*x[0]**o+(0.887179037493)*x[0]+(0.163173866494)*x[1]**o+(-0.0429615650925)*x[1]+(0.246231141028)*x[2]**o+(0.835506100749)*x[2]
        s[1,1]=(-0.0644771805334)*x[0]**o+(-0.447016488171)*x[0]+(0.525187872112)*x[1]**o+(0.0698248427183)*x[1]+(0.206184560671)*x[2]**o+(0.0760145857359)*x[2]
        s[1,2]=(-0.468079677328)*x[0]**o+(0.209734412741)*x[0]+(-0.76390490599)*x[1]**o+(-0.890555632144)*x[1]+(0.0940465019231)*x[2]**o+(-0.4911374279)*x[2]
        s[1,3]=(0.0219043704096)*x[0]**o+(0.843668836143)*x[0]+(-0.496594214362)*x[1]**o+(-0.698887537236)*x[1]+(0.739801576727)*x[2]**o+(-0.283969520066)*x[2]
        s[1,4]=(0.626836103474)*x[0]**o+(-0.209145555082)*x[0]+(0.0527420557385)*x[1]**o+(0.447276924763)*x[1]+(0.753404625572)*x[2]**o+(-0.964517776982)*x[2]
        s[2,0]=(0.208209713508)*x[0]**o+(0.666027366366)*x[0]+(0.648558221549)*x[1]**o+(-0.867196856985)*x[1]+(-0.386920813405)*x[2]**o+(-0.859803466402)*x[2]
        s[2,1]=(0.192327689645)*x[0]**o+(0.706632402773)*x[0]+(-0.917135230541)*x[1]**o+(-0.518142383283)*x[1]+(0.63345047167)*x[2]**o+(0.452536991007)*x[2]
        s[2,2]=(-0.216559791626)*x[0]**o+(0.280416285835)*x[0]+(0.880532961859)*x[1]**o+(-0.685211600885)*x[1]+(0.435667186231)*x[2]**o+(0.632733331589)*x[2]
        s[2,3]=(0.52338353111)*x[0]**o+(-0.865976270688)*x[0]+(0.598776819058)*x[1]**o+(-0.404416433163)*x[1]+(-0.538779785754)*x[2]**o+(-0.708928887004)*x[2]
        s[2,4]=(0.523035959616)*x[0]**o+(0.732324929631)*x[0]+(0.213261893733)*x[1]**o+(0.620185443057)*x[1]+(0.984821746408)*x[2]**o+(0.358075460145)*x[2]
        s[3,0]=(-0.2094567637)*x[0]**o+(0.0635506980721)*x[0]+(0.540179140495)*x[1]**o+(-0.178193714382)*x[1]+(0.114963963742)*x[2]**o+(-0.232785400122)*x[2]
        s[3,1]=(0.581234387125)*x[0]**o+(-0.800223513435)*x[0]+(-0.580041308558)*x[1]**o+(-0.468466479829)*x[1]+(0.66171049066)*x[2]**o+(0.00411361715063)*x[2]
        s[3,2]=(0.972539025242)*x[0]**o+(-0.640993915582)*x[0]+(0.418443080441)*x[1]**o+(0.608840732702)*x[1]+(0.979209906273)*x[2]**o+(-0.984684975101)*x[2]
        s[3,3]=(0.80136860876)*x[0]**o+(0.526110649659)*x[0]+(-0.307964128679)*x[1]**o+(-0.162612676576)*x[1]+(-0.68486706357)*x[2]**o+(-0.234358375142)*x[2]
        s[3,4]=(0.146116143734)*x[0]**o+(0.581210816708)*x[0]+(-0.152105642806)*x[1]**o+(0.97096072826)*x[1]+(-0.0416344321982)*x[2]**o+(-0.933814919317)*x[2]
        ref[0,0,0]=o*(0.729942219527)*x_ref[0]**(o-1)+(-0.721160545053)
        ref[0,0,1]=o*(0.426397843912)*x_ref[1]**(o-1)+(-0.213185647354)
        ref[0,0,2]=o*(0.728447138266)*x_ref[2]**(o-1)+(-0.607791661013)
        ref[0,1,0]=o*(0.667616392653)*x_ref[0]**(o-1)+(0.848051730251)
        ref[0,1,1]=o*(0.121550076489)*x_ref[1]**(o-1)+(0.0843862117633)
        ref[0,1,2]=o*(-0.509323081526)*x_ref[2]**(o-1)+(0.736521833786)
        ref[0,2,0]=o*(0.830127288575)*x_ref[0]**(o-1)+(0.964772646621)
        ref[0,2,1]=o*(0.913688863179)*x_ref[1]**(o-1)+(0.957018200531)
        ref[0,2,2]=o*(-0.694170056751)*x_ref[2]**(o-1)+(0.105973113953)
        ref[0,3,0]=o*(-0.235173218805)*x_ref[0]**(o-1)+(0.858875579758)
        ref[0,3,1]=o*(0.823162706883)*x_ref[1]**(o-1)+(0.108717821259)
        ref[0,3,2]=o*(0.80411364975)*x_ref[2]**(o-1)+(0.0348079894361)
        ref[0,4,0]=o*(0.78867654833)*x_ref[0]**(o-1)+(-0.0191953378129)
        ref[0,4,1]=o*(-0.0840244773387)*x_ref[1]**(o-1)+(0.614156921824)
        ref[0,4,2]=o*(-0.0190397891394)*x_ref[2]**(o-1)+(0.750918932533)
        ref[1,0,0]=o*(-0.0931449972666)*x_ref[0]**(o-1)+(0.887179037493)
        ref[1,0,1]=o*(0.163173866494)*x_ref[1]**(o-1)+(-0.0429615650925)
        ref[1,0,2]=o*(0.246231141028)*x_ref[2]**(o-1)+(0.835506100749)
        ref[1,1,0]=o*(-0.0644771805334)*x_ref[0]**(o-1)+(-0.447016488171)
        ref[1,1,1]=o*(0.525187872112)*x_ref[1]**(o-1)+(0.0698248427183)
        ref[1,1,2]=o*(0.206184560671)*x_ref[2]**(o-1)+(0.0760145857359)
        ref[1,2,0]=o*(-0.468079677328)*x_ref[0]**(o-1)+(0.209734412741)
        ref[1,2,1]=o*(-0.76390490599)*x_ref[1]**(o-1)+(-0.890555632144)
        ref[1,2,2]=o*(0.0940465019231)*x_ref[2]**(o-1)+(-0.4911374279)
        ref[1,3,0]=o*(0.0219043704096)*x_ref[0]**(o-1)+(0.843668836143)
        ref[1,3,1]=o*(-0.496594214362)*x_ref[1]**(o-1)+(-0.698887537236)
        ref[1,3,2]=o*(0.739801576727)*x_ref[2]**(o-1)+(-0.283969520066)
        ref[1,4,0]=o*(0.626836103474)*x_ref[0]**(o-1)+(-0.209145555082)
        ref[1,4,1]=o*(0.0527420557385)*x_ref[1]**(o-1)+(0.447276924763)
        ref[1,4,2]=o*(0.753404625572)*x_ref[2]**(o-1)+(-0.964517776982)
        ref[2,0,0]=o*(0.208209713508)*x_ref[0]**(o-1)+(0.666027366366)
        ref[2,0,1]=o*(0.648558221549)*x_ref[1]**(o-1)+(-0.867196856985)
        ref[2,0,2]=o*(-0.386920813405)*x_ref[2]**(o-1)+(-0.859803466402)
        ref[2,1,0]=o*(0.192327689645)*x_ref[0]**(o-1)+(0.706632402773)
        ref[2,1,1]=o*(-0.917135230541)*x_ref[1]**(o-1)+(-0.518142383283)
        ref[2,1,2]=o*(0.63345047167)*x_ref[2]**(o-1)+(0.452536991007)
        ref[2,2,0]=o*(-0.216559791626)*x_ref[0]**(o-1)+(0.280416285835)
        ref[2,2,1]=o*(0.880532961859)*x_ref[1]**(o-1)+(-0.685211600885)
        ref[2,2,2]=o*(0.435667186231)*x_ref[2]**(o-1)+(0.632733331589)
        ref[2,3,0]=o*(0.52338353111)*x_ref[0]**(o-1)+(-0.865976270688)
        ref[2,3,1]=o*(0.598776819058)*x_ref[1]**(o-1)+(-0.404416433163)
        ref[2,3,2]=o*(-0.538779785754)*x_ref[2]**(o-1)+(-0.708928887004)
        ref[2,4,0]=o*(0.523035959616)*x_ref[0]**(o-1)+(0.732324929631)
        ref[2,4,1]=o*(0.213261893733)*x_ref[1]**(o-1)+(0.620185443057)
        ref[2,4,2]=o*(0.984821746408)*x_ref[2]**(o-1)+(0.358075460145)
        ref[3,0,0]=o*(-0.2094567637)*x_ref[0]**(o-1)+(0.0635506980721)
        ref[3,0,1]=o*(0.540179140495)*x_ref[1]**(o-1)+(-0.178193714382)
        ref[3,0,2]=o*(0.114963963742)*x_ref[2]**(o-1)+(-0.232785400122)
        ref[3,1,0]=o*(0.581234387125)*x_ref[0]**(o-1)+(-0.800223513435)
        ref[3,1,1]=o*(-0.580041308558)*x_ref[1]**(o-1)+(-0.468466479829)
        ref[3,1,2]=o*(0.66171049066)*x_ref[2]**(o-1)+(0.00411361715063)
        ref[3,2,0]=o*(0.972539025242)*x_ref[0]**(o-1)+(-0.640993915582)
        ref[3,2,1]=o*(0.418443080441)*x_ref[1]**(o-1)+(0.608840732702)
        ref[3,2,2]=o*(0.979209906273)*x_ref[2]**(o-1)+(-0.984684975101)
        ref[3,3,0]=o*(0.80136860876)*x_ref[0]**(o-1)+(0.526110649659)
        ref[3,3,1]=o*(-0.307964128679)*x_ref[1]**(o-1)+(-0.162612676576)
        ref[3,3,2]=o*(-0.68486706357)*x_ref[2]**(o-1)+(-0.234358375142)
        ref[3,4,0]=o*(0.146116143734)*x_ref[0]**(o-1)+(0.581210816708)
        ref[3,4,1]=o*(-0.152105642806)*x_ref[1]**(o-1)+(0.97096072826)
        ref[3,4,2]=o*(-0.0416344321982)*x_ref[2]**(o-1)+(-0.933814919317)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(0.210810831144)*x[0]**o+(0.993748550706)*x[0]+(0.225093899339)*x[1]**o+(-0.306898024507)*x[1]
        s[0,0,1]=(0.945391291392)*x[0]**o+(-0.34733930643)*x[0]+(0.523085690915)*x[1]**o+(-0.332680309461)*x[1]
        s[0,1,0]=(0.372088608669)*x[0]**o+(0.276443856788)*x[0]+(-0.439514323588)*x[1]**o+(0.985453767485)*x[1]
        s[0,1,1]=(-0.967561100768)*x[0]**o+(0.591908531375)*x[0]+(0.288385284239)*x[1]**o+(-0.422883693702)*x[1]
        s[1,0,0]=(-0.735303276087)*x[0]**o+(0.540176339329)*x[0]+(0.124141197609)*x[1]**o+(0.17505950278)*x[1]
        s[1,0,1]=(0.799321447619)*x[0]**o+(-0.521267395747)*x[0]+(-0.388235024602)*x[1]**o+(0.0783682529975)*x[1]
        s[1,1,0]=(-0.7923799998)*x[0]**o+(-0.779345662449)*x[0]+(0.507399627325)*x[1]**o+(-0.668039939194)*x[1]
        s[1,1,1]=(0.360260485497)*x[0]**o+(0.639639208398)*x[0]+(0.275791805772)*x[1]**o+(-0.923288698445)*x[1]
        s[2,0,0]=(0.338586096766)*x[0]**o+(-0.862613705959)*x[0]+(0.0321867760579)*x[1]**o+(0.418774919375)*x[1]
        s[2,0,1]=(0.0363215417361)*x[0]**o+(0.994712078504)*x[0]+(-0.260614790561)*x[1]**o+(-0.969097035084)*x[1]
        s[2,1,0]=(0.375366661281)*x[0]**o+(0.456874530834)*x[0]+(0.429399840953)*x[1]**o+(0.688205075709)*x[1]
        s[2,1,1]=(-0.918074889111)*x[0]**o+(-0.716561240269)*x[0]+(-0.477398945222)*x[1]**o+(-0.223142905734)*x[1]
        s[3,0,0]=(-0.420424618942)*x[0]**o+(-0.702691949655)*x[0]+(-0.389998593814)*x[1]**o+(0.778091583668)*x[1]
        s[3,0,1]=(-0.500317522801)*x[0]**o+(-0.906158794184)*x[0]+(-0.942667332637)*x[1]**o+(-0.38695660361)*x[1]
        s[3,1,0]=(0.652359792164)*x[0]**o+(0.663033146373)*x[0]+(-0.0863743948576)*x[1]**o+(-0.310130231197)*x[1]
        s[3,1,1]=(0.132520829002)*x[0]**o+(-0.692607399006)*x[0]+(0.484454939316)*x[1]**o+(0.848314808163)*x[1]
        s[4,0,0]=(-0.0372302792179)*x[0]**o+(-0.208111754532)*x[0]+(-0.84060197077)*x[1]**o+(-0.999842431749)*x[1]
        s[4,0,1]=(0.522677717768)*x[0]**o+(-0.796530255947)*x[0]+(-0.758152917507)*x[1]**o+(-0.748249123242)*x[1]
        s[4,1,0]=(-0.9338587977)*x[0]**o+(0.033736711901)*x[0]+(-0.521414748153)*x[1]**o+(-0.27924576252)*x[1]
        s[4,1,1]=(-0.728280335859)*x[0]**o+(0.0052784852763)*x[0]+(0.668426662412)*x[1]**o+(-0.539548867925)*x[1]
        s[5,0,0]=(0.183081849687)*x[0]**o+(0.867616703079)*x[0]+(-0.44917069636)*x[1]**o+(-0.422840947463)*x[1]
        s[5,0,1]=(0.598683594731)*x[0]**o+(-0.185366147497)*x[0]+(-0.133722684999)*x[1]**o+(-0.374233145702)*x[1]
        s[5,1,0]=(-0.4661027909)*x[0]**o+(-0.658486427256)*x[0]+(0.494385097276)*x[1]**o+(0.746131625723)*x[1]
        s[5,1,1]=(0.881329425205)*x[0]**o+(0.0663586900787)*x[0]+(0.638121309568)*x[1]**o+(-0.392008079426)*x[1]
        ref[0,0,0,0]=o*(0.210810831144)*x_ref[0]**(o-1)+(0.993748550706)
        ref[0,0,0,1]=o*(0.225093899339)*x_ref[1]**(o-1)+(-0.306898024507)
        ref[0,0,1,0]=o*(0.945391291392)*x_ref[0]**(o-1)+(-0.34733930643)
        ref[0,0,1,1]=o*(0.523085690915)*x_ref[1]**(o-1)+(-0.332680309461)
        ref[0,1,0,0]=o*(0.372088608669)*x_ref[0]**(o-1)+(0.276443856788)
        ref[0,1,0,1]=o*(-0.439514323588)*x_ref[1]**(o-1)+(0.985453767485)
        ref[0,1,1,0]=o*(-0.967561100768)*x_ref[0]**(o-1)+(0.591908531375)
        ref[0,1,1,1]=o*(0.288385284239)*x_ref[1]**(o-1)+(-0.422883693702)
        ref[1,0,0,0]=o*(-0.735303276087)*x_ref[0]**(o-1)+(0.540176339329)
        ref[1,0,0,1]=o*(0.124141197609)*x_ref[1]**(o-1)+(0.17505950278)
        ref[1,0,1,0]=o*(0.799321447619)*x_ref[0]**(o-1)+(-0.521267395747)
        ref[1,0,1,1]=o*(-0.388235024602)*x_ref[1]**(o-1)+(0.0783682529975)
        ref[1,1,0,0]=o*(-0.7923799998)*x_ref[0]**(o-1)+(-0.779345662449)
        ref[1,1,0,1]=o*(0.507399627325)*x_ref[1]**(o-1)+(-0.668039939194)
        ref[1,1,1,0]=o*(0.360260485497)*x_ref[0]**(o-1)+(0.639639208398)
        ref[1,1,1,1]=o*(0.275791805772)*x_ref[1]**(o-1)+(-0.923288698445)
        ref[2,0,0,0]=o*(0.338586096766)*x_ref[0]**(o-1)+(-0.862613705959)
        ref[2,0,0,1]=o*(0.0321867760579)*x_ref[1]**(o-1)+(0.418774919375)
        ref[2,0,1,0]=o*(0.0363215417361)*x_ref[0]**(o-1)+(0.994712078504)
        ref[2,0,1,1]=o*(-0.260614790561)*x_ref[1]**(o-1)+(-0.969097035084)
        ref[2,1,0,0]=o*(0.375366661281)*x_ref[0]**(o-1)+(0.456874530834)
        ref[2,1,0,1]=o*(0.429399840953)*x_ref[1]**(o-1)+(0.688205075709)
        ref[2,1,1,0]=o*(-0.918074889111)*x_ref[0]**(o-1)+(-0.716561240269)
        ref[2,1,1,1]=o*(-0.477398945222)*x_ref[1]**(o-1)+(-0.223142905734)
        ref[3,0,0,0]=o*(-0.420424618942)*x_ref[0]**(o-1)+(-0.702691949655)
        ref[3,0,0,1]=o*(-0.389998593814)*x_ref[1]**(o-1)+(0.778091583668)
        ref[3,0,1,0]=o*(-0.500317522801)*x_ref[0]**(o-1)+(-0.906158794184)
        ref[3,0,1,1]=o*(-0.942667332637)*x_ref[1]**(o-1)+(-0.38695660361)
        ref[3,1,0,0]=o*(0.652359792164)*x_ref[0]**(o-1)+(0.663033146373)
        ref[3,1,0,1]=o*(-0.0863743948576)*x_ref[1]**(o-1)+(-0.310130231197)
        ref[3,1,1,0]=o*(0.132520829002)*x_ref[0]**(o-1)+(-0.692607399006)
        ref[3,1,1,1]=o*(0.484454939316)*x_ref[1]**(o-1)+(0.848314808163)
        ref[4,0,0,0]=o*(-0.0372302792179)*x_ref[0]**(o-1)+(-0.208111754532)
        ref[4,0,0,1]=o*(-0.84060197077)*x_ref[1]**(o-1)+(-0.999842431749)
        ref[4,0,1,0]=o*(0.522677717768)*x_ref[0]**(o-1)+(-0.796530255947)
        ref[4,0,1,1]=o*(-0.758152917507)*x_ref[1]**(o-1)+(-0.748249123242)
        ref[4,1,0,0]=o*(-0.9338587977)*x_ref[0]**(o-1)+(0.033736711901)
        ref[4,1,0,1]=o*(-0.521414748153)*x_ref[1]**(o-1)+(-0.27924576252)
        ref[4,1,1,0]=o*(-0.728280335859)*x_ref[0]**(o-1)+(0.0052784852763)
        ref[4,1,1,1]=o*(0.668426662412)*x_ref[1]**(o-1)+(-0.539548867925)
        ref[5,0,0,0]=o*(0.183081849687)*x_ref[0]**(o-1)+(0.867616703079)
        ref[5,0,0,1]=o*(-0.44917069636)*x_ref[1]**(o-1)+(-0.422840947463)
        ref[5,0,1,0]=o*(0.598683594731)*x_ref[0]**(o-1)+(-0.185366147497)
        ref[5,0,1,1]=o*(-0.133722684999)*x_ref[1]**(o-1)+(-0.374233145702)
        ref[5,1,0,0]=o*(-0.4661027909)*x_ref[0]**(o-1)+(-0.658486427256)
        ref[5,1,0,1]=o*(0.494385097276)*x_ref[1]**(o-1)+(0.746131625723)
        ref[5,1,1,0]=o*(0.881329425205)*x_ref[0]**(o-1)+(0.0663586900787)
        ref[5,1,1,1]=o*(0.638121309568)*x_ref[1]**(o-1)+(-0.392008079426)
      else:
        s[0,0,0]=(0.495604153512)*x[0]**o+(-0.546051759361)*x[0]+(0.985443531442)*x[1]**o+(-0.394922383292)*x[1]+(-0.122301738852)*x[2]**o+(-0.221144495446)*x[2]
        s[0,0,1]=(-0.164179695077)*x[0]**o+(-0.488047064324)*x[0]+(0.556965694466)*x[1]**o+(-0.470423249373)*x[1]+(-0.625307475886)*x[2]**o+(-0.999651290798)*x[2]
        s[0,1,0]=(-0.858044039365)*x[0]**o+(-0.932289002735)*x[0]+(-0.343121624834)*x[1]**o+(-0.0210069830662)*x[1]+(0.951213562883)*x[2]**o+(-0.647040549286)*x[2]
        s[0,1,1]=(-0.680293757762)*x[0]**o+(0.820233518831)*x[0]+(0.0489418677095)*x[1]**o+(-0.852104985889)*x[1]+(0.960135497008)*x[2]**o+(-0.46075359959)*x[2]
        s[1,0,0]=(0.827213065477)*x[0]**o+(0.195435158962)*x[0]+(-0.0408345903614)*x[1]**o+(0.146957170071)*x[1]+(-0.423482853682)*x[2]**o+(0.113246501514)*x[2]
        s[1,0,1]=(-0.920613863373)*x[0]**o+(0.933412308922)*x[0]+(0.258934140982)*x[1]**o+(-0.689643171121)*x[1]+(0.893192051888)*x[2]**o+(-0.471852670906)*x[2]
        s[1,1,0]=(-0.981601898267)*x[0]**o+(-0.516390752771)*x[0]+(0.608397090998)*x[1]**o+(0.335529495102)*x[1]+(-0.0746490251171)*x[2]**o+(-0.50249402227)*x[2]
        s[1,1,1]=(0.367749783351)*x[0]**o+(-0.611995716855)*x[0]+(-0.980913863186)*x[1]**o+(0.299812200148)*x[1]+(0.562250470266)*x[2]**o+(0.0542265856797)*x[2]
        s[2,0,0]=(-0.317633103186)*x[0]**o+(-0.175141419305)*x[0]+(-0.114479468171)*x[1]**o+(-0.117966705382)*x[1]+(-0.983395580005)*x[2]**o+(-0.588457148786)*x[2]
        s[2,0,1]=(-0.45007739654)*x[0]**o+(-0.941613604155)*x[0]+(0.597714724696)*x[1]**o+(0.992095144994)*x[1]+(0.681322168708)*x[2]**o+(0.656828773187)*x[2]
        s[2,1,0]=(0.372959242179)*x[0]**o+(-0.266039538769)*x[0]+(0.420734431733)*x[1]**o+(0.530880738131)*x[1]+(0.593665348543)*x[2]**o+(-0.502919021867)*x[2]
        s[2,1,1]=(0.741984988727)*x[0]**o+(-0.662109553439)*x[0]+(-0.01222805343)*x[1]**o+(-0.261877080281)*x[1]+(0.584778610634)*x[2]**o+(0.248969331325)*x[2]
        s[3,0,0]=(-0.767798213786)*x[0]**o+(0.56387037017)*x[0]+(0.545042362526)*x[1]**o+(0.127002206725)*x[1]+(0.123826212114)*x[2]**o+(0.03202953512)*x[2]
        s[3,0,1]=(-0.54234471198)*x[0]**o+(-0.844655061807)*x[0]+(0.909493468686)*x[1]**o+(-0.601003138714)*x[1]+(-0.822776025243)*x[2]**o+(-0.569619092404)*x[2]
        s[3,1,0]=(-0.749598491666)*x[0]**o+(0.146640505619)*x[0]+(-0.782827623949)*x[1]**o+(0.0795239577275)*x[1]+(0.98951419773)*x[2]**o+(-0.878832103203)*x[2]
        s[3,1,1]=(0.917489698949)*x[0]**o+(0.791885047319)*x[0]+(0.415646018849)*x[1]**o+(0.194622530403)*x[1]+(0.123045865981)*x[2]**o+(-0.864537623753)*x[2]
        s[4,0,0]=(0.993338538326)*x[0]**o+(-0.130375655736)*x[0]+(0.088857735361)*x[1]**o+(-0.565025291359)*x[1]+(-0.543615061477)*x[2]**o+(0.509734903091)*x[2]
        s[4,0,1]=(-0.874483507391)*x[0]**o+(-0.993080113589)*x[0]+(0.269443230092)*x[1]**o+(-0.445853491314)*x[1]+(-0.765796954415)*x[2]**o+(-0.981106922583)*x[2]
        s[4,1,0]=(-0.650170302308)*x[0]**o+(0.222814951659)*x[0]+(0.553055011394)*x[1]**o+(-0.359146840173)*x[1]+(0.887910478865)*x[2]**o+(-0.446130800509)*x[2]
        s[4,1,1]=(0.512373070036)*x[0]**o+(0.427326569369)*x[0]+(0.311737845037)*x[1]**o+(0.998455706258)*x[1]+(-0.868170359085)*x[2]**o+(-0.924351117942)*x[2]
        s[5,0,0]=(0.129707961614)*x[0]**o+(0.668313579165)*x[0]+(-0.73474211614)*x[1]**o+(0.702827908609)*x[1]+(-0.0226884665962)*x[2]**o+(-0.00740206443655)*x[2]
        s[5,0,1]=(-0.113099473775)*x[0]**o+(0.655719358276)*x[0]+(0.0162829520583)*x[1]**o+(0.133582158275)*x[1]+(0.169685631206)*x[2]**o+(0.507126023907)*x[2]
        s[5,1,0]=(-0.0754735940537)*x[0]**o+(0.294940113895)*x[0]+(-0.98735456969)*x[1]**o+(0.401871193336)*x[1]+(-0.0770486468616)*x[2]**o+(-0.0168685832006)*x[2]
        s[5,1,1]=(0.971769998957)*x[0]**o+(-0.454726501195)*x[0]+(0.835505309792)*x[1]**o+(0.775091315278)*x[1]+(0.632907414827)*x[2]**o+(-0.10388782338)*x[2]
        ref[0,0,0,0]=o*(0.495604153512)*x_ref[0]**(o-1)+(-0.546051759361)
        ref[0,0,0,1]=o*(0.985443531442)*x_ref[1]**(o-1)+(-0.394922383292)
        ref[0,0,0,2]=o*(-0.122301738852)*x_ref[2]**(o-1)+(-0.221144495446)
        ref[0,0,1,0]=o*(-0.164179695077)*x_ref[0]**(o-1)+(-0.488047064324)
        ref[0,0,1,1]=o*(0.556965694466)*x_ref[1]**(o-1)+(-0.470423249373)
        ref[0,0,1,2]=o*(-0.625307475886)*x_ref[2]**(o-1)+(-0.999651290798)
        ref[0,1,0,0]=o*(-0.858044039365)*x_ref[0]**(o-1)+(-0.932289002735)
        ref[0,1,0,1]=o*(-0.343121624834)*x_ref[1]**(o-1)+(-0.0210069830662)
        ref[0,1,0,2]=o*(0.951213562883)*x_ref[2]**(o-1)+(-0.647040549286)
        ref[0,1,1,0]=o*(-0.680293757762)*x_ref[0]**(o-1)+(0.820233518831)
        ref[0,1,1,1]=o*(0.0489418677095)*x_ref[1]**(o-1)+(-0.852104985889)
        ref[0,1,1,2]=o*(0.960135497008)*x_ref[2]**(o-1)+(-0.46075359959)
        ref[1,0,0,0]=o*(0.827213065477)*x_ref[0]**(o-1)+(0.195435158962)
        ref[1,0,0,1]=o*(-0.0408345903614)*x_ref[1]**(o-1)+(0.146957170071)
        ref[1,0,0,2]=o*(-0.423482853682)*x_ref[2]**(o-1)+(0.113246501514)
        ref[1,0,1,0]=o*(-0.920613863373)*x_ref[0]**(o-1)+(0.933412308922)
        ref[1,0,1,1]=o*(0.258934140982)*x_ref[1]**(o-1)+(-0.689643171121)
        ref[1,0,1,2]=o*(0.893192051888)*x_ref[2]**(o-1)+(-0.471852670906)
        ref[1,1,0,0]=o*(-0.981601898267)*x_ref[0]**(o-1)+(-0.516390752771)
        ref[1,1,0,1]=o*(0.608397090998)*x_ref[1]**(o-1)+(0.335529495102)
        ref[1,1,0,2]=o*(-0.0746490251171)*x_ref[2]**(o-1)+(-0.50249402227)
        ref[1,1,1,0]=o*(0.367749783351)*x_ref[0]**(o-1)+(-0.611995716855)
        ref[1,1,1,1]=o*(-0.980913863186)*x_ref[1]**(o-1)+(0.299812200148)
        ref[1,1,1,2]=o*(0.562250470266)*x_ref[2]**(o-1)+(0.0542265856797)
        ref[2,0,0,0]=o*(-0.317633103186)*x_ref[0]**(o-1)+(-0.175141419305)
        ref[2,0,0,1]=o*(-0.114479468171)*x_ref[1]**(o-1)+(-0.117966705382)
        ref[2,0,0,2]=o*(-0.983395580005)*x_ref[2]**(o-1)+(-0.588457148786)
        ref[2,0,1,0]=o*(-0.45007739654)*x_ref[0]**(o-1)+(-0.941613604155)
        ref[2,0,1,1]=o*(0.597714724696)*x_ref[1]**(o-1)+(0.992095144994)
        ref[2,0,1,2]=o*(0.681322168708)*x_ref[2]**(o-1)+(0.656828773187)
        ref[2,1,0,0]=o*(0.372959242179)*x_ref[0]**(o-1)+(-0.266039538769)
        ref[2,1,0,1]=o*(0.420734431733)*x_ref[1]**(o-1)+(0.530880738131)
        ref[2,1,0,2]=o*(0.593665348543)*x_ref[2]**(o-1)+(-0.502919021867)
        ref[2,1,1,0]=o*(0.741984988727)*x_ref[0]**(o-1)+(-0.662109553439)
        ref[2,1,1,1]=o*(-0.01222805343)*x_ref[1]**(o-1)+(-0.261877080281)
        ref[2,1,1,2]=o*(0.584778610634)*x_ref[2]**(o-1)+(0.248969331325)
        ref[3,0,0,0]=o*(-0.767798213786)*x_ref[0]**(o-1)+(0.56387037017)
        ref[3,0,0,1]=o*(0.545042362526)*x_ref[1]**(o-1)+(0.127002206725)
        ref[3,0,0,2]=o*(0.123826212114)*x_ref[2]**(o-1)+(0.03202953512)
        ref[3,0,1,0]=o*(-0.54234471198)*x_ref[0]**(o-1)+(-0.844655061807)
        ref[3,0,1,1]=o*(0.909493468686)*x_ref[1]**(o-1)+(-0.601003138714)
        ref[3,0,1,2]=o*(-0.822776025243)*x_ref[2]**(o-1)+(-0.569619092404)
        ref[3,1,0,0]=o*(-0.749598491666)*x_ref[0]**(o-1)+(0.146640505619)
        ref[3,1,0,1]=o*(-0.782827623949)*x_ref[1]**(o-1)+(0.0795239577275)
        ref[3,1,0,2]=o*(0.98951419773)*x_ref[2]**(o-1)+(-0.878832103203)
        ref[3,1,1,0]=o*(0.917489698949)*x_ref[0]**(o-1)+(0.791885047319)
        ref[3,1,1,1]=o*(0.415646018849)*x_ref[1]**(o-1)+(0.194622530403)
        ref[3,1,1,2]=o*(0.123045865981)*x_ref[2]**(o-1)+(-0.864537623753)
        ref[4,0,0,0]=o*(0.993338538326)*x_ref[0]**(o-1)+(-0.130375655736)
        ref[4,0,0,1]=o*(0.088857735361)*x_ref[1]**(o-1)+(-0.565025291359)
        ref[4,0,0,2]=o*(-0.543615061477)*x_ref[2]**(o-1)+(0.509734903091)
        ref[4,0,1,0]=o*(-0.874483507391)*x_ref[0]**(o-1)+(-0.993080113589)
        ref[4,0,1,1]=o*(0.269443230092)*x_ref[1]**(o-1)+(-0.445853491314)
        ref[4,0,1,2]=o*(-0.765796954415)*x_ref[2]**(o-1)+(-0.981106922583)
        ref[4,1,0,0]=o*(-0.650170302308)*x_ref[0]**(o-1)+(0.222814951659)
        ref[4,1,0,1]=o*(0.553055011394)*x_ref[1]**(o-1)+(-0.359146840173)
        ref[4,1,0,2]=o*(0.887910478865)*x_ref[2]**(o-1)+(-0.446130800509)
        ref[4,1,1,0]=o*(0.512373070036)*x_ref[0]**(o-1)+(0.427326569369)
        ref[4,1,1,1]=o*(0.311737845037)*x_ref[1]**(o-1)+(0.998455706258)
        ref[4,1,1,2]=o*(-0.868170359085)*x_ref[2]**(o-1)+(-0.924351117942)
        ref[5,0,0,0]=o*(0.129707961614)*x_ref[0]**(o-1)+(0.668313579165)
        ref[5,0,0,1]=o*(-0.73474211614)*x_ref[1]**(o-1)+(0.702827908609)
        ref[5,0,0,2]=o*(-0.0226884665962)*x_ref[2]**(o-1)+(-0.00740206443655)
        ref[5,0,1,0]=o*(-0.113099473775)*x_ref[0]**(o-1)+(0.655719358276)
        ref[5,0,1,1]=o*(0.0162829520583)*x_ref[1]**(o-1)+(0.133582158275)
        ref[5,0,1,2]=o*(0.169685631206)*x_ref[2]**(o-1)+(0.507126023907)
        ref[5,1,0,0]=o*(-0.0754735940537)*x_ref[0]**(o-1)+(0.294940113895)
        ref[5,1,0,1]=o*(-0.98735456969)*x_ref[1]**(o-1)+(0.401871193336)
        ref[5,1,0,2]=o*(-0.0770486468616)*x_ref[2]**(o-1)+(-0.0168685832006)
        ref[5,1,1,0]=o*(0.971769998957)*x_ref[0]**(o-1)+(-0.454726501195)
        ref[5,1,1,1]=o*(0.835505309792)*x_ref[1]**(o-1)+(0.775091315278)
        ref[5,1,1,2]=o*(0.632907414827)*x_ref[2]**(o-1)+(-0.10388782338)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.143576812018)*x[0]**o+(0.772046830562)*x[0]+(0.799382081787)*x[1]**o+(-0.689145149202)*x[1]
        ref[0]=o*(0.143576812018)*x_ref[0]**(o-1)+(0.772046830562)
        ref[1]=o*(0.799382081787)*x_ref[1]**(o-1)+(-0.689145149202)
      else:
        s=(-0.921622979189)*x[0]**o+(-0.35132068829)*x[0]+(-0.668419889938)*x[1]**o+(-0.25240554797)*x[1]+(-0.266863972981)*x[2]**o+(0.59405466698)*x[2]
        ref[0]=o*(-0.921622979189)*x_ref[0]**(o-1)+(-0.35132068829)
        ref[1]=o*(-0.668419889938)*x_ref[1]**(o-1)+(-0.25240554797)
        ref[2]=o*(-0.266863972981)*x_ref[2]**(o-1)+(0.59405466698)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.572981720293)*x[0]**o+(-0.305280287068)*x[0]+(-0.552369316063)*x[1]**o+(-0.917368778197)*x[1]
        s[1]=(0.310389604089)*x[0]**o+(-0.316385112396)*x[0]+(0.713272118168)*x[1]**o+(0.856686290785)*x[1]
        ref[0,0]=o*(0.572981720293)*x_ref[0]**(o-1)+(-0.305280287068)
        ref[0,1]=o*(-0.552369316063)*x_ref[1]**(o-1)+(-0.917368778197)
        ref[1,0]=o*(0.310389604089)*x_ref[0]**(o-1)+(-0.316385112396)
        ref[1,1]=o*(0.713272118168)*x_ref[1]**(o-1)+(0.856686290785)
      else:
        s[0]=(-0.132099402207)*x[0]**o+(0.587263843257)*x[0]+(-0.326050434166)*x[1]**o+(0.873048232418)*x[1]+(-0.141344874565)*x[2]**o+(0.274084100427)*x[2]
        s[1]=(0.372494974451)*x[0]**o+(-0.482077621852)*x[0]+(-0.022004253515)*x[1]**o+(0.874572230513)*x[1]+(0.8369124866)*x[2]**o+(-0.979759147043)*x[2]
        ref[0,0]=o*(-0.132099402207)*x_ref[0]**(o-1)+(0.587263843257)
        ref[0,1]=o*(-0.326050434166)*x_ref[1]**(o-1)+(0.873048232418)
        ref[0,2]=o*(-0.141344874565)*x_ref[2]**(o-1)+(0.274084100427)
        ref[1,0]=o*(0.372494974451)*x_ref[0]**(o-1)+(-0.482077621852)
        ref[1,1]=o*(-0.022004253515)*x_ref[1]**(o-1)+(0.874572230513)
        ref[1,2]=o*(0.8369124866)*x_ref[2]**(o-1)+(-0.979759147043)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.993111549259)*x[0]**o+(0.958601813395)*x[0]+(0.985388059548)*x[1]**o+(-0.570742940525)*x[1]
        s[0,1]=(-0.223411180873)*x[0]**o+(0.480695178439)*x[0]+(0.0161612686521)*x[1]**o+(0.400987902598)*x[1]
        s[0,2]=(0.775656253974)*x[0]**o+(-0.191138960618)*x[0]+(-0.446251149518)*x[1]**o+(0.432178603961)*x[1]
        s[0,3]=(0.827622744839)*x[0]**o+(0.0366172895102)*x[0]+(0.347810808477)*x[1]**o+(-0.688818913998)*x[1]
        s[0,4]=(0.837082711466)*x[0]**o+(0.376527432997)*x[0]+(0.17741245651)*x[1]**o+(0.872147279603)*x[1]
        s[1,0]=(-0.590137160663)*x[0]**o+(-0.0703615079757)*x[0]+(0.229861545725)*x[1]**o+(0.0809472412982)*x[1]
        s[1,1]=(0.0223830240489)*x[0]**o+(0.228842693495)*x[0]+(-0.557074035399)*x[1]**o+(0.500538497902)*x[1]
        s[1,2]=(0.872181981766)*x[0]**o+(0.790601719428)*x[0]+(0.983055567067)*x[1]**o+(-0.353849465384)*x[1]
        s[1,3]=(0.037040061131)*x[0]**o+(0.293981509756)*x[0]+(-0.279061759005)*x[1]**o+(-0.362644845866)*x[1]
        s[1,4]=(0.695941674104)*x[0]**o+(-0.51481315942)*x[0]+(0.765262082274)*x[1]**o+(-0.813116917911)*x[1]
        s[2,0]=(-0.579924169032)*x[0]**o+(0.529857498501)*x[0]+(-0.485940906658)*x[1]**o+(0.559056145345)*x[1]
        s[2,1]=(0.604856738274)*x[0]**o+(-0.440825061381)*x[0]+(-0.0838758185405)*x[1]**o+(0.0188329882604)*x[1]
        s[2,2]=(0.118585827905)*x[0]**o+(0.618367201987)*x[0]+(0.352157918285)*x[1]**o+(0.253890253557)*x[1]
        s[2,3]=(-0.372665302477)*x[0]**o+(0.718734712755)*x[0]+(-0.487897528386)*x[1]**o+(0.808964263735)*x[1]
        s[2,4]=(0.776070811457)*x[0]**o+(-0.675210963521)*x[0]+(0.0331035702593)*x[1]**o+(-0.703100978257)*x[1]
        s[3,0]=(0.144238364778)*x[0]**o+(0.855900329896)*x[0]+(-0.291688407398)*x[1]**o+(-0.308539252205)*x[1]
        s[3,1]=(-0.57873830064)*x[0]**o+(-0.0982142438835)*x[0]+(0.249356200194)*x[1]**o+(-0.156027166502)*x[1]
        s[3,2]=(-0.70772301011)*x[0]**o+(0.615393644935)*x[0]+(-0.1313826883)*x[1]**o+(0.265287078264)*x[1]
        s[3,3]=(-0.671189805427)*x[0]**o+(-0.561302098778)*x[0]+(0.129817024131)*x[1]**o+(-0.738582319997)*x[1]
        s[3,4]=(-0.154389666509)*x[0]**o+(0.119040072318)*x[0]+(-0.113706823809)*x[1]**o+(0.0124037512924)*x[1]
        ref[0,0,0]=o*(-0.993111549259)*x_ref[0]**(o-1)+(0.958601813395)
        ref[0,0,1]=o*(0.985388059548)*x_ref[1]**(o-1)+(-0.570742940525)
        ref[0,1,0]=o*(-0.223411180873)*x_ref[0]**(o-1)+(0.480695178439)
        ref[0,1,1]=o*(0.0161612686521)*x_ref[1]**(o-1)+(0.400987902598)
        ref[0,2,0]=o*(0.775656253974)*x_ref[0]**(o-1)+(-0.191138960618)
        ref[0,2,1]=o*(-0.446251149518)*x_ref[1]**(o-1)+(0.432178603961)
        ref[0,3,0]=o*(0.827622744839)*x_ref[0]**(o-1)+(0.0366172895102)
        ref[0,3,1]=o*(0.347810808477)*x_ref[1]**(o-1)+(-0.688818913998)
        ref[0,4,0]=o*(0.837082711466)*x_ref[0]**(o-1)+(0.376527432997)
        ref[0,4,1]=o*(0.17741245651)*x_ref[1]**(o-1)+(0.872147279603)
        ref[1,0,0]=o*(-0.590137160663)*x_ref[0]**(o-1)+(-0.0703615079757)
        ref[1,0,1]=o*(0.229861545725)*x_ref[1]**(o-1)+(0.0809472412982)
        ref[1,1,0]=o*(0.0223830240489)*x_ref[0]**(o-1)+(0.228842693495)
        ref[1,1,1]=o*(-0.557074035399)*x_ref[1]**(o-1)+(0.500538497902)
        ref[1,2,0]=o*(0.872181981766)*x_ref[0]**(o-1)+(0.790601719428)
        ref[1,2,1]=o*(0.983055567067)*x_ref[1]**(o-1)+(-0.353849465384)
        ref[1,3,0]=o*(0.037040061131)*x_ref[0]**(o-1)+(0.293981509756)
        ref[1,3,1]=o*(-0.279061759005)*x_ref[1]**(o-1)+(-0.362644845866)
        ref[1,4,0]=o*(0.695941674104)*x_ref[0]**(o-1)+(-0.51481315942)
        ref[1,4,1]=o*(0.765262082274)*x_ref[1]**(o-1)+(-0.813116917911)
        ref[2,0,0]=o*(-0.579924169032)*x_ref[0]**(o-1)+(0.529857498501)
        ref[2,0,1]=o*(-0.485940906658)*x_ref[1]**(o-1)+(0.559056145345)
        ref[2,1,0]=o*(0.604856738274)*x_ref[0]**(o-1)+(-0.440825061381)
        ref[2,1,1]=o*(-0.0838758185405)*x_ref[1]**(o-1)+(0.0188329882604)
        ref[2,2,0]=o*(0.118585827905)*x_ref[0]**(o-1)+(0.618367201987)
        ref[2,2,1]=o*(0.352157918285)*x_ref[1]**(o-1)+(0.253890253557)
        ref[2,3,0]=o*(-0.372665302477)*x_ref[0]**(o-1)+(0.718734712755)
        ref[2,3,1]=o*(-0.487897528386)*x_ref[1]**(o-1)+(0.808964263735)
        ref[2,4,0]=o*(0.776070811457)*x_ref[0]**(o-1)+(-0.675210963521)
        ref[2,4,1]=o*(0.0331035702593)*x_ref[1]**(o-1)+(-0.703100978257)
        ref[3,0,0]=o*(0.144238364778)*x_ref[0]**(o-1)+(0.855900329896)
        ref[3,0,1]=o*(-0.291688407398)*x_ref[1]**(o-1)+(-0.308539252205)
        ref[3,1,0]=o*(-0.57873830064)*x_ref[0]**(o-1)+(-0.0982142438835)
        ref[3,1,1]=o*(0.249356200194)*x_ref[1]**(o-1)+(-0.156027166502)
        ref[3,2,0]=o*(-0.70772301011)*x_ref[0]**(o-1)+(0.615393644935)
        ref[3,2,1]=o*(-0.1313826883)*x_ref[1]**(o-1)+(0.265287078264)
        ref[3,3,0]=o*(-0.671189805427)*x_ref[0]**(o-1)+(-0.561302098778)
        ref[3,3,1]=o*(0.129817024131)*x_ref[1]**(o-1)+(-0.738582319997)
        ref[3,4,0]=o*(-0.154389666509)*x_ref[0]**(o-1)+(0.119040072318)
        ref[3,4,1]=o*(-0.113706823809)*x_ref[1]**(o-1)+(0.0124037512924)
      else:
        s[0,0]=(-0.829816450667)*x[0]**o+(-0.147235008007)*x[0]+(0.492783189082)*x[1]**o+(-0.512635666132)*x[1]+(0.890224791682)*x[2]**o+(-0.956119232758)*x[2]
        s[0,1]=(0.674209146363)*x[0]**o+(-0.922409931962)*x[0]+(-0.328441886277)*x[1]**o+(0.981094102408)*x[1]+(0.481174182835)*x[2]**o+(-0.44590959358)*x[2]
        s[0,2]=(0.695494078474)*x[0]**o+(-0.0618250391783)*x[0]+(0.731471533841)*x[1]**o+(-0.71550566694)*x[1]+(-0.239535873248)*x[2]**o+(-0.763157808991)*x[2]
        s[0,3]=(0.754158913667)*x[0]**o+(-0.603448930533)*x[0]+(-0.200959580291)*x[1]**o+(0.298079100922)*x[1]+(0.15701085915)*x[2]**o+(0.18247341435)*x[2]
        s[0,4]=(0.894484225202)*x[0]**o+(0.908683878491)*x[0]+(0.452273398852)*x[1]**o+(0.446075531002)*x[1]+(-0.527120277613)*x[2]**o+(0.0825220159349)*x[2]
        s[1,0]=(0.1117942579)*x[0]**o+(0.616938000848)*x[0]+(-0.490920496055)*x[1]**o+(-0.46576972952)*x[1]+(-0.0234475725539)*x[2]**o+(-0.762964610488)*x[2]
        s[1,1]=(0.10779128676)*x[0]**o+(0.323623835098)*x[0]+(-0.56638095467)*x[1]**o+(-0.485758771921)*x[1]+(0.149606176158)*x[2]**o+(0.0163417655889)*x[2]
        s[1,2]=(-0.220849737003)*x[0]**o+(-0.269143304567)*x[0]+(0.53654939437)*x[1]**o+(0.434383033636)*x[1]+(0.474884241083)*x[2]**o+(-0.0619940742667)*x[2]
        s[1,3]=(0.789745098309)*x[0]**o+(-0.882265656942)*x[0]+(-0.513103080365)*x[1]**o+(0.461829232085)*x[1]+(-0.762923723193)*x[2]**o+(0.300345919163)*x[2]
        s[1,4]=(0.225139496075)*x[0]**o+(0.254192406789)*x[0]+(-0.984611398843)*x[1]**o+(-0.519021894131)*x[1]+(0.916659347214)*x[2]**o+(0.466927379869)*x[2]
        s[2,0]=(-0.404728214636)*x[0]**o+(-0.0890438316072)*x[0]+(-0.97035417582)*x[1]**o+(0.202361572805)*x[1]+(-0.540497194191)*x[2]**o+(-0.80807606235)*x[2]
        s[2,1]=(0.140494227071)*x[0]**o+(0.757504635086)*x[0]+(0.831217423619)*x[1]**o+(-0.647503683145)*x[1]+(0.821831665921)*x[2]**o+(-0.121451056151)*x[2]
        s[2,2]=(-0.962347494034)*x[0]**o+(0.633512101597)*x[0]+(-0.828226238758)*x[1]**o+(0.968353578898)*x[1]+(-0.553755153946)*x[2]**o+(0.938911535056)*x[2]
        s[2,3]=(0.450492204938)*x[0]**o+(-0.475913651564)*x[0]+(-0.341560341273)*x[1]**o+(-0.923252138473)*x[1]+(0.17946107137)*x[2]**o+(0.337986869404)*x[2]
        s[2,4]=(0.217063428863)*x[0]**o+(0.702651458761)*x[0]+(-0.769522744816)*x[1]**o+(-0.231379966769)*x[1]+(-0.0579749690283)*x[2]**o+(-0.766607638409)*x[2]
        s[3,0]=(0.681151302876)*x[0]**o+(-0.29704479144)*x[0]+(0.537446794047)*x[1]**o+(0.872410938876)*x[1]+(-0.934271483068)*x[2]**o+(0.270440171512)*x[2]
        s[3,1]=(0.530874723279)*x[0]**o+(0.739974626875)*x[0]+(-0.240867894838)*x[1]**o+(0.625808519502)*x[1]+(0.747357445736)*x[2]**o+(-0.242935065692)*x[2]
        s[3,2]=(0.682623726875)*x[0]**o+(-0.34894979966)*x[0]+(-0.307826595564)*x[1]**o+(-0.869693179146)*x[1]+(-0.814910983419)*x[2]**o+(-0.98110829358)*x[2]
        s[3,3]=(0.232395211532)*x[0]**o+(0.504566395286)*x[0]+(-0.029022506301)*x[1]**o+(0.897710188132)*x[1]+(-0.107215395289)*x[2]**o+(-0.541445668914)*x[2]
        s[3,4]=(0.916061625636)*x[0]**o+(-0.39003337702)*x[0]+(0.926535863586)*x[1]**o+(0.585922306493)*x[1]+(-0.761902914251)*x[2]**o+(0.489922931455)*x[2]
        ref[0,0,0]=o*(-0.829816450667)*x_ref[0]**(o-1)+(-0.147235008007)
        ref[0,0,1]=o*(0.492783189082)*x_ref[1]**(o-1)+(-0.512635666132)
        ref[0,0,2]=o*(0.890224791682)*x_ref[2]**(o-1)+(-0.956119232758)
        ref[0,1,0]=o*(0.674209146363)*x_ref[0]**(o-1)+(-0.922409931962)
        ref[0,1,1]=o*(-0.328441886277)*x_ref[1]**(o-1)+(0.981094102408)
        ref[0,1,2]=o*(0.481174182835)*x_ref[2]**(o-1)+(-0.44590959358)
        ref[0,2,0]=o*(0.695494078474)*x_ref[0]**(o-1)+(-0.0618250391783)
        ref[0,2,1]=o*(0.731471533841)*x_ref[1]**(o-1)+(-0.71550566694)
        ref[0,2,2]=o*(-0.239535873248)*x_ref[2]**(o-1)+(-0.763157808991)
        ref[0,3,0]=o*(0.754158913667)*x_ref[0]**(o-1)+(-0.603448930533)
        ref[0,3,1]=o*(-0.200959580291)*x_ref[1]**(o-1)+(0.298079100922)
        ref[0,3,2]=o*(0.15701085915)*x_ref[2]**(o-1)+(0.18247341435)
        ref[0,4,0]=o*(0.894484225202)*x_ref[0]**(o-1)+(0.908683878491)
        ref[0,4,1]=o*(0.452273398852)*x_ref[1]**(o-1)+(0.446075531002)
        ref[0,4,2]=o*(-0.527120277613)*x_ref[2]**(o-1)+(0.0825220159349)
        ref[1,0,0]=o*(0.1117942579)*x_ref[0]**(o-1)+(0.616938000848)
        ref[1,0,1]=o*(-0.490920496055)*x_ref[1]**(o-1)+(-0.46576972952)
        ref[1,0,2]=o*(-0.0234475725539)*x_ref[2]**(o-1)+(-0.762964610488)
        ref[1,1,0]=o*(0.10779128676)*x_ref[0]**(o-1)+(0.323623835098)
        ref[1,1,1]=o*(-0.56638095467)*x_ref[1]**(o-1)+(-0.485758771921)
        ref[1,1,2]=o*(0.149606176158)*x_ref[2]**(o-1)+(0.0163417655889)
        ref[1,2,0]=o*(-0.220849737003)*x_ref[0]**(o-1)+(-0.269143304567)
        ref[1,2,1]=o*(0.53654939437)*x_ref[1]**(o-1)+(0.434383033636)
        ref[1,2,2]=o*(0.474884241083)*x_ref[2]**(o-1)+(-0.0619940742667)
        ref[1,3,0]=o*(0.789745098309)*x_ref[0]**(o-1)+(-0.882265656942)
        ref[1,3,1]=o*(-0.513103080365)*x_ref[1]**(o-1)+(0.461829232085)
        ref[1,3,2]=o*(-0.762923723193)*x_ref[2]**(o-1)+(0.300345919163)
        ref[1,4,0]=o*(0.225139496075)*x_ref[0]**(o-1)+(0.254192406789)
        ref[1,4,1]=o*(-0.984611398843)*x_ref[1]**(o-1)+(-0.519021894131)
        ref[1,4,2]=o*(0.916659347214)*x_ref[2]**(o-1)+(0.466927379869)
        ref[2,0,0]=o*(-0.404728214636)*x_ref[0]**(o-1)+(-0.0890438316072)
        ref[2,0,1]=o*(-0.97035417582)*x_ref[1]**(o-1)+(0.202361572805)
        ref[2,0,2]=o*(-0.540497194191)*x_ref[2]**(o-1)+(-0.80807606235)
        ref[2,1,0]=o*(0.140494227071)*x_ref[0]**(o-1)+(0.757504635086)
        ref[2,1,1]=o*(0.831217423619)*x_ref[1]**(o-1)+(-0.647503683145)
        ref[2,1,2]=o*(0.821831665921)*x_ref[2]**(o-1)+(-0.121451056151)
        ref[2,2,0]=o*(-0.962347494034)*x_ref[0]**(o-1)+(0.633512101597)
        ref[2,2,1]=o*(-0.828226238758)*x_ref[1]**(o-1)+(0.968353578898)
        ref[2,2,2]=o*(-0.553755153946)*x_ref[2]**(o-1)+(0.938911535056)
        ref[2,3,0]=o*(0.450492204938)*x_ref[0]**(o-1)+(-0.475913651564)
        ref[2,3,1]=o*(-0.341560341273)*x_ref[1]**(o-1)+(-0.923252138473)
        ref[2,3,2]=o*(0.17946107137)*x_ref[2]**(o-1)+(0.337986869404)
        ref[2,4,0]=o*(0.217063428863)*x_ref[0]**(o-1)+(0.702651458761)
        ref[2,4,1]=o*(-0.769522744816)*x_ref[1]**(o-1)+(-0.231379966769)
        ref[2,4,2]=o*(-0.0579749690283)*x_ref[2]**(o-1)+(-0.766607638409)
        ref[3,0,0]=o*(0.681151302876)*x_ref[0]**(o-1)+(-0.29704479144)
        ref[3,0,1]=o*(0.537446794047)*x_ref[1]**(o-1)+(0.872410938876)
        ref[3,0,2]=o*(-0.934271483068)*x_ref[2]**(o-1)+(0.270440171512)
        ref[3,1,0]=o*(0.530874723279)*x_ref[0]**(o-1)+(0.739974626875)
        ref[3,1,1]=o*(-0.240867894838)*x_ref[1]**(o-1)+(0.625808519502)
        ref[3,1,2]=o*(0.747357445736)*x_ref[2]**(o-1)+(-0.242935065692)
        ref[3,2,0]=o*(0.682623726875)*x_ref[0]**(o-1)+(-0.34894979966)
        ref[3,2,1]=o*(-0.307826595564)*x_ref[1]**(o-1)+(-0.869693179146)
        ref[3,2,2]=o*(-0.814910983419)*x_ref[2]**(o-1)+(-0.98110829358)
        ref[3,3,0]=o*(0.232395211532)*x_ref[0]**(o-1)+(0.504566395286)
        ref[3,3,1]=o*(-0.029022506301)*x_ref[1]**(o-1)+(0.897710188132)
        ref[3,3,2]=o*(-0.107215395289)*x_ref[2]**(o-1)+(-0.541445668914)
        ref[3,4,0]=o*(0.916061625636)*x_ref[0]**(o-1)+(-0.39003337702)
        ref[3,4,1]=o*(0.926535863586)*x_ref[1]**(o-1)+(0.585922306493)
        ref[3,4,2]=o*(-0.761902914251)*x_ref[2]**(o-1)+(0.489922931455)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.401271003072)*x[0]**o+(0.110662634074)*x[0]+(0.00658452640581)*x[1]**o+(0.0961174317738)*x[1]
        s[0,0,1]=(-0.697194977122)*x[0]**o+(-0.202095576104)*x[0]+(0.533118580807)*x[1]**o+(-0.806666747117)*x[1]
        s[0,1,0]=(0.330322436592)*x[0]**o+(-0.476215030667)*x[0]+(0.435516874614)*x[1]**o+(-0.305867365755)*x[1]
        s[0,1,1]=(0.663350700331)*x[0]**o+(0.719421457458)*x[0]+(0.56853576783)*x[1]**o+(-0.63138691405)*x[1]
        s[1,0,0]=(-0.330411596581)*x[0]**o+(0.562930649846)*x[0]+(0.372532798726)*x[1]**o+(-0.444620881121)*x[1]
        s[1,0,1]=(-0.128926814439)*x[0]**o+(-0.60950153677)*x[0]+(-0.958182145974)*x[1]**o+(0.355162970063)*x[1]
        s[1,1,0]=(0.197818586294)*x[0]**o+(0.821517517641)*x[0]+(-0.701366738238)*x[1]**o+(-0.506092465749)*x[1]
        s[1,1,1]=(0.0313418487908)*x[0]**o+(0.161293069084)*x[0]+(0.18801678809)*x[1]**o+(0.691344434767)*x[1]
        s[2,0,0]=(-0.684508001816)*x[0]**o+(-0.341824405415)*x[0]+(-0.51105537282)*x[1]**o+(-0.291070626075)*x[1]
        s[2,0,1]=(0.401725066632)*x[0]**o+(0.279867637006)*x[0]+(-0.899862170932)*x[1]**o+(-0.442022413224)*x[1]
        s[2,1,0]=(0.185395646678)*x[0]**o+(0.0999907785823)*x[0]+(-0.577244369816)*x[1]**o+(-0.0346150541428)*x[1]
        s[2,1,1]=(0.216201017349)*x[0]**o+(-0.295200102743)*x[0]+(-0.306163867778)*x[1]**o+(0.0910745810644)*x[1]
        s[3,0,0]=(0.804134508197)*x[0]**o+(-0.213041659269)*x[0]+(0.157574172841)*x[1]**o+(0.296960896944)*x[1]
        s[3,0,1]=(0.268965957593)*x[0]**o+(0.0238464096944)*x[0]+(-0.00156155375937)*x[1]**o+(0.958073421842)*x[1]
        s[3,1,0]=(0.517287095459)*x[0]**o+(-0.0687040369948)*x[0]+(0.801953753488)*x[1]**o+(0.521595036445)*x[1]
        s[3,1,1]=(0.175159770635)*x[0]**o+(-0.736749683511)*x[0]+(-0.630382453304)*x[1]**o+(0.188706240505)*x[1]
        s[4,0,0]=(-0.724712993645)*x[0]**o+(-0.387274725616)*x[0]+(0.694395156995)*x[1]**o+(-0.401125937857)*x[1]
        s[4,0,1]=(-0.86282575637)*x[0]**o+(-0.966750714134)*x[0]+(0.130551499076)*x[1]**o+(-0.50519607351)*x[1]
        s[4,1,0]=(0.721837471072)*x[0]**o+(0.780750959264)*x[0]+(0.642470119645)*x[1]**o+(-0.158180439909)*x[1]
        s[4,1,1]=(0.451682933961)*x[0]**o+(-0.802341009866)*x[0]+(-0.106997197799)*x[1]**o+(0.212654933949)*x[1]
        s[5,0,0]=(0.437602747572)*x[0]**o+(0.420962577059)*x[0]+(0.977375479522)*x[1]**o+(-0.57876475638)*x[1]
        s[5,0,1]=(-0.653922496012)*x[0]**o+(-0.197873016501)*x[0]+(-0.494086552357)*x[1]**o+(-0.879089855341)*x[1]
        s[5,1,0]=(-0.466463766139)*x[0]**o+(-0.0287797105685)*x[0]+(0.163811113598)*x[1]**o+(-0.752758839578)*x[1]
        s[5,1,1]=(-0.442295358061)*x[0]**o+(-0.928365763003)*x[0]+(-0.998523766886)*x[1]**o+(-0.210043550397)*x[1]
        ref[0,0,0,0]=o*(-0.401271003072)*x_ref[0]**(o-1)+(0.110662634074)
        ref[0,0,0,1]=o*(0.00658452640581)*x_ref[1]**(o-1)+(0.0961174317738)
        ref[0,0,1,0]=o*(-0.697194977122)*x_ref[0]**(o-1)+(-0.202095576104)
        ref[0,0,1,1]=o*(0.533118580807)*x_ref[1]**(o-1)+(-0.806666747117)
        ref[0,1,0,0]=o*(0.330322436592)*x_ref[0]**(o-1)+(-0.476215030667)
        ref[0,1,0,1]=o*(0.435516874614)*x_ref[1]**(o-1)+(-0.305867365755)
        ref[0,1,1,0]=o*(0.663350700331)*x_ref[0]**(o-1)+(0.719421457458)
        ref[0,1,1,1]=o*(0.56853576783)*x_ref[1]**(o-1)+(-0.63138691405)
        ref[1,0,0,0]=o*(-0.330411596581)*x_ref[0]**(o-1)+(0.562930649846)
        ref[1,0,0,1]=o*(0.372532798726)*x_ref[1]**(o-1)+(-0.444620881121)
        ref[1,0,1,0]=o*(-0.128926814439)*x_ref[0]**(o-1)+(-0.60950153677)
        ref[1,0,1,1]=o*(-0.958182145974)*x_ref[1]**(o-1)+(0.355162970063)
        ref[1,1,0,0]=o*(0.197818586294)*x_ref[0]**(o-1)+(0.821517517641)
        ref[1,1,0,1]=o*(-0.701366738238)*x_ref[1]**(o-1)+(-0.506092465749)
        ref[1,1,1,0]=o*(0.0313418487908)*x_ref[0]**(o-1)+(0.161293069084)
        ref[1,1,1,1]=o*(0.18801678809)*x_ref[1]**(o-1)+(0.691344434767)
        ref[2,0,0,0]=o*(-0.684508001816)*x_ref[0]**(o-1)+(-0.341824405415)
        ref[2,0,0,1]=o*(-0.51105537282)*x_ref[1]**(o-1)+(-0.291070626075)
        ref[2,0,1,0]=o*(0.401725066632)*x_ref[0]**(o-1)+(0.279867637006)
        ref[2,0,1,1]=o*(-0.899862170932)*x_ref[1]**(o-1)+(-0.442022413224)
        ref[2,1,0,0]=o*(0.185395646678)*x_ref[0]**(o-1)+(0.0999907785823)
        ref[2,1,0,1]=o*(-0.577244369816)*x_ref[1]**(o-1)+(-0.0346150541428)
        ref[2,1,1,0]=o*(0.216201017349)*x_ref[0]**(o-1)+(-0.295200102743)
        ref[2,1,1,1]=o*(-0.306163867778)*x_ref[1]**(o-1)+(0.0910745810644)
        ref[3,0,0,0]=o*(0.804134508197)*x_ref[0]**(o-1)+(-0.213041659269)
        ref[3,0,0,1]=o*(0.157574172841)*x_ref[1]**(o-1)+(0.296960896944)
        ref[3,0,1,0]=o*(0.268965957593)*x_ref[0]**(o-1)+(0.0238464096944)
        ref[3,0,1,1]=o*(-0.00156155375937)*x_ref[1]**(o-1)+(0.958073421842)
        ref[3,1,0,0]=o*(0.517287095459)*x_ref[0]**(o-1)+(-0.0687040369948)
        ref[3,1,0,1]=o*(0.801953753488)*x_ref[1]**(o-1)+(0.521595036445)
        ref[3,1,1,0]=o*(0.175159770635)*x_ref[0]**(o-1)+(-0.736749683511)
        ref[3,1,1,1]=o*(-0.630382453304)*x_ref[1]**(o-1)+(0.188706240505)
        ref[4,0,0,0]=o*(-0.724712993645)*x_ref[0]**(o-1)+(-0.387274725616)
        ref[4,0,0,1]=o*(0.694395156995)*x_ref[1]**(o-1)+(-0.401125937857)
        ref[4,0,1,0]=o*(-0.86282575637)*x_ref[0]**(o-1)+(-0.966750714134)
        ref[4,0,1,1]=o*(0.130551499076)*x_ref[1]**(o-1)+(-0.50519607351)
        ref[4,1,0,0]=o*(0.721837471072)*x_ref[0]**(o-1)+(0.780750959264)
        ref[4,1,0,1]=o*(0.642470119645)*x_ref[1]**(o-1)+(-0.158180439909)
        ref[4,1,1,0]=o*(0.451682933961)*x_ref[0]**(o-1)+(-0.802341009866)
        ref[4,1,1,1]=o*(-0.106997197799)*x_ref[1]**(o-1)+(0.212654933949)
        ref[5,0,0,0]=o*(0.437602747572)*x_ref[0]**(o-1)+(0.420962577059)
        ref[5,0,0,1]=o*(0.977375479522)*x_ref[1]**(o-1)+(-0.57876475638)
        ref[5,0,1,0]=o*(-0.653922496012)*x_ref[0]**(o-1)+(-0.197873016501)
        ref[5,0,1,1]=o*(-0.494086552357)*x_ref[1]**(o-1)+(-0.879089855341)
        ref[5,1,0,0]=o*(-0.466463766139)*x_ref[0]**(o-1)+(-0.0287797105685)
        ref[5,1,0,1]=o*(0.163811113598)*x_ref[1]**(o-1)+(-0.752758839578)
        ref[5,1,1,0]=o*(-0.442295358061)*x_ref[0]**(o-1)+(-0.928365763003)
        ref[5,1,1,1]=o*(-0.998523766886)*x_ref[1]**(o-1)+(-0.210043550397)
      else:
        s[0,0,0]=(-0.130526786389)*x[0]**o+(-0.798097445808)*x[0]+(-0.679305651671)*x[1]**o+(-0.805797697874)*x[1]+(0.344947047592)*x[2]**o+(-0.835073945341)*x[2]
        s[0,0,1]=(-0.725780665135)*x[0]**o+(-0.427130507741)*x[0]+(-0.172129968106)*x[1]**o+(-0.329971568167)*x[1]+(0.430421618441)*x[2]**o+(0.850034235311)*x[2]
        s[0,1,0]=(-0.0894725266496)*x[0]**o+(-0.599803400866)*x[0]+(-0.689863844979)*x[1]**o+(-0.433762379607)*x[1]+(0.0585705966933)*x[2]**o+(0.932652014579)*x[2]
        s[0,1,1]=(0.351726580485)*x[0]**o+(-0.46650217284)*x[0]+(-0.389405597178)*x[1]**o+(0.702473795612)*x[1]+(0.971136943329)*x[2]**o+(-0.149952798522)*x[2]
        s[1,0,0]=(-0.687968216558)*x[0]**o+(0.918982435037)*x[0]+(-0.425841096866)*x[1]**o+(0.738050157283)*x[1]+(-0.468164706689)*x[2]**o+(0.524190937704)*x[2]
        s[1,0,1]=(-0.0190626564569)*x[0]**o+(0.158516496015)*x[0]+(-0.591775293572)*x[1]**o+(0.0791152697184)*x[1]+(-0.661850377953)*x[2]**o+(-0.705879414238)*x[2]
        s[1,1,0]=(-0.538932477038)*x[0]**o+(0.781931047594)*x[0]+(-0.799541079517)*x[1]**o+(0.218777285901)*x[1]+(-0.475448704708)*x[2]**o+(-0.655013532097)*x[2]
        s[1,1,1]=(0.0320772665819)*x[0]**o+(-0.577577235436)*x[0]+(-0.736321547926)*x[1]**o+(-0.133681853862)*x[1]+(-0.0573985073549)*x[2]**o+(0.955212063412)*x[2]
        s[2,0,0]=(-0.257879955226)*x[0]**o+(-0.994024413493)*x[0]+(-0.679188188792)*x[1]**o+(-0.410885730889)*x[1]+(0.704555870926)*x[2]**o+(0.518649077753)*x[2]
        s[2,0,1]=(-0.901488792722)*x[0]**o+(-0.222487929076)*x[0]+(0.165342576493)*x[1]**o+(-0.850214182574)*x[1]+(0.294760812505)*x[2]**o+(-0.0831507413484)*x[2]
        s[2,1,0]=(0.316904380947)*x[0]**o+(0.627318211545)*x[0]+(-0.216902145108)*x[1]**o+(-0.673577316246)*x[1]+(0.122931487815)*x[2]**o+(-0.0949502423641)*x[2]
        s[2,1,1]=(-0.602869040889)*x[0]**o+(0.190451027649)*x[0]+(0.367827882092)*x[1]**o+(0.686234993984)*x[1]+(0.653140976608)*x[2]**o+(-0.168486517196)*x[2]
        s[3,0,0]=(0.271612350046)*x[0]**o+(-0.827115599685)*x[0]+(-0.586600506988)*x[1]**o+(0.128300770664)*x[1]+(-0.922582907477)*x[2]**o+(0.334699215375)*x[2]
        s[3,0,1]=(-0.595897268553)*x[0]**o+(0.425141939316)*x[0]+(0.713120162361)*x[1]**o+(-0.680621044232)*x[1]+(0.749485690034)*x[2]**o+(0.209547675753)*x[2]
        s[3,1,0]=(-0.530398143666)*x[0]**o+(0.137850234083)*x[0]+(0.953276619653)*x[1]**o+(-0.578161670579)*x[1]+(0.371898896542)*x[2]**o+(-0.381331753222)*x[2]
        s[3,1,1]=(0.822298482068)*x[0]**o+(-0.405836518978)*x[0]+(-0.647621688423)*x[1]**o+(-0.672131868147)*x[1]+(-0.438867520359)*x[2]**o+(-0.423328072541)*x[2]
        s[4,0,0]=(0.132863753285)*x[0]**o+(0.397422994959)*x[0]+(-0.804707213822)*x[1]**o+(0.536531749119)*x[1]+(0.411705453279)*x[2]**o+(-0.169385048425)*x[2]
        s[4,0,1]=(0.0701228685651)*x[0]**o+(0.639241932031)*x[0]+(-0.638730875998)*x[1]**o+(-0.922035920621)*x[1]+(-0.113168863437)*x[2]**o+(0.813287574357)*x[2]
        s[4,1,0]=(0.100082349838)*x[0]**o+(0.133955906808)*x[0]+(-0.928623633632)*x[1]**o+(-0.424742167549)*x[1]+(0.0239194885597)*x[2]**o+(-0.672806207969)*x[2]
        s[4,1,1]=(0.829725063543)*x[0]**o+(0.678552027954)*x[0]+(-0.812625405832)*x[1]**o+(-0.810345951883)*x[1]+(-0.211595232946)*x[2]**o+(-0.984554474963)*x[2]
        s[5,0,0]=(0.339493504495)*x[0]**o+(-0.903993357652)*x[0]+(-0.440233624866)*x[1]**o+(-0.430746451761)*x[1]+(0.111254680616)*x[2]**o+(0.709038943317)*x[2]
        s[5,0,1]=(0.931029457116)*x[0]**o+(-0.785398109537)*x[0]+(-0.326970114573)*x[1]**o+(0.0384958937297)*x[1]+(-0.896259053345)*x[2]**o+(-0.277237800601)*x[2]
        s[5,1,0]=(0.608509355178)*x[0]**o+(0.585955903578)*x[0]+(0.267506079769)*x[1]**o+(0.812314541749)*x[1]+(-0.283432880004)*x[2]**o+(0.909390413963)*x[2]
        s[5,1,1]=(0.23175779421)*x[0]**o+(-0.614072594905)*x[0]+(-0.979433217533)*x[1]**o+(-0.928308978862)*x[1]+(0.252162852495)*x[2]**o+(-0.395771235106)*x[2]
        ref[0,0,0,0]=o*(-0.130526786389)*x_ref[0]**(o-1)+(-0.798097445808)
        ref[0,0,0,1]=o*(-0.679305651671)*x_ref[1]**(o-1)+(-0.805797697874)
        ref[0,0,0,2]=o*(0.344947047592)*x_ref[2]**(o-1)+(-0.835073945341)
        ref[0,0,1,0]=o*(-0.725780665135)*x_ref[0]**(o-1)+(-0.427130507741)
        ref[0,0,1,1]=o*(-0.172129968106)*x_ref[1]**(o-1)+(-0.329971568167)
        ref[0,0,1,2]=o*(0.430421618441)*x_ref[2]**(o-1)+(0.850034235311)
        ref[0,1,0,0]=o*(-0.0894725266496)*x_ref[0]**(o-1)+(-0.599803400866)
        ref[0,1,0,1]=o*(-0.689863844979)*x_ref[1]**(o-1)+(-0.433762379607)
        ref[0,1,0,2]=o*(0.0585705966933)*x_ref[2]**(o-1)+(0.932652014579)
        ref[0,1,1,0]=o*(0.351726580485)*x_ref[0]**(o-1)+(-0.46650217284)
        ref[0,1,1,1]=o*(-0.389405597178)*x_ref[1]**(o-1)+(0.702473795612)
        ref[0,1,1,2]=o*(0.971136943329)*x_ref[2]**(o-1)+(-0.149952798522)
        ref[1,0,0,0]=o*(-0.687968216558)*x_ref[0]**(o-1)+(0.918982435037)
        ref[1,0,0,1]=o*(-0.425841096866)*x_ref[1]**(o-1)+(0.738050157283)
        ref[1,0,0,2]=o*(-0.468164706689)*x_ref[2]**(o-1)+(0.524190937704)
        ref[1,0,1,0]=o*(-0.0190626564569)*x_ref[0]**(o-1)+(0.158516496015)
        ref[1,0,1,1]=o*(-0.591775293572)*x_ref[1]**(o-1)+(0.0791152697184)
        ref[1,0,1,2]=o*(-0.661850377953)*x_ref[2]**(o-1)+(-0.705879414238)
        ref[1,1,0,0]=o*(-0.538932477038)*x_ref[0]**(o-1)+(0.781931047594)
        ref[1,1,0,1]=o*(-0.799541079517)*x_ref[1]**(o-1)+(0.218777285901)
        ref[1,1,0,2]=o*(-0.475448704708)*x_ref[2]**(o-1)+(-0.655013532097)
        ref[1,1,1,0]=o*(0.0320772665819)*x_ref[0]**(o-1)+(-0.577577235436)
        ref[1,1,1,1]=o*(-0.736321547926)*x_ref[1]**(o-1)+(-0.133681853862)
        ref[1,1,1,2]=o*(-0.0573985073549)*x_ref[2]**(o-1)+(0.955212063412)
        ref[2,0,0,0]=o*(-0.257879955226)*x_ref[0]**(o-1)+(-0.994024413493)
        ref[2,0,0,1]=o*(-0.679188188792)*x_ref[1]**(o-1)+(-0.410885730889)
        ref[2,0,0,2]=o*(0.704555870926)*x_ref[2]**(o-1)+(0.518649077753)
        ref[2,0,1,0]=o*(-0.901488792722)*x_ref[0]**(o-1)+(-0.222487929076)
        ref[2,0,1,1]=o*(0.165342576493)*x_ref[1]**(o-1)+(-0.850214182574)
        ref[2,0,1,2]=o*(0.294760812505)*x_ref[2]**(o-1)+(-0.0831507413484)
        ref[2,1,0,0]=o*(0.316904380947)*x_ref[0]**(o-1)+(0.627318211545)
        ref[2,1,0,1]=o*(-0.216902145108)*x_ref[1]**(o-1)+(-0.673577316246)
        ref[2,1,0,2]=o*(0.122931487815)*x_ref[2]**(o-1)+(-0.0949502423641)
        ref[2,1,1,0]=o*(-0.602869040889)*x_ref[0]**(o-1)+(0.190451027649)
        ref[2,1,1,1]=o*(0.367827882092)*x_ref[1]**(o-1)+(0.686234993984)
        ref[2,1,1,2]=o*(0.653140976608)*x_ref[2]**(o-1)+(-0.168486517196)
        ref[3,0,0,0]=o*(0.271612350046)*x_ref[0]**(o-1)+(-0.827115599685)
        ref[3,0,0,1]=o*(-0.586600506988)*x_ref[1]**(o-1)+(0.128300770664)
        ref[3,0,0,2]=o*(-0.922582907477)*x_ref[2]**(o-1)+(0.334699215375)
        ref[3,0,1,0]=o*(-0.595897268553)*x_ref[0]**(o-1)+(0.425141939316)
        ref[3,0,1,1]=o*(0.713120162361)*x_ref[1]**(o-1)+(-0.680621044232)
        ref[3,0,1,2]=o*(0.749485690034)*x_ref[2]**(o-1)+(0.209547675753)
        ref[3,1,0,0]=o*(-0.530398143666)*x_ref[0]**(o-1)+(0.137850234083)
        ref[3,1,0,1]=o*(0.953276619653)*x_ref[1]**(o-1)+(-0.578161670579)
        ref[3,1,0,2]=o*(0.371898896542)*x_ref[2]**(o-1)+(-0.381331753222)
        ref[3,1,1,0]=o*(0.822298482068)*x_ref[0]**(o-1)+(-0.405836518978)
        ref[3,1,1,1]=o*(-0.647621688423)*x_ref[1]**(o-1)+(-0.672131868147)
        ref[3,1,1,2]=o*(-0.438867520359)*x_ref[2]**(o-1)+(-0.423328072541)
        ref[4,0,0,0]=o*(0.132863753285)*x_ref[0]**(o-1)+(0.397422994959)
        ref[4,0,0,1]=o*(-0.804707213822)*x_ref[1]**(o-1)+(0.536531749119)
        ref[4,0,0,2]=o*(0.411705453279)*x_ref[2]**(o-1)+(-0.169385048425)
        ref[4,0,1,0]=o*(0.0701228685651)*x_ref[0]**(o-1)+(0.639241932031)
        ref[4,0,1,1]=o*(-0.638730875998)*x_ref[1]**(o-1)+(-0.922035920621)
        ref[4,0,1,2]=o*(-0.113168863437)*x_ref[2]**(o-1)+(0.813287574357)
        ref[4,1,0,0]=o*(0.100082349838)*x_ref[0]**(o-1)+(0.133955906808)
        ref[4,1,0,1]=o*(-0.928623633632)*x_ref[1]**(o-1)+(-0.424742167549)
        ref[4,1,0,2]=o*(0.0239194885597)*x_ref[2]**(o-1)+(-0.672806207969)
        ref[4,1,1,0]=o*(0.829725063543)*x_ref[0]**(o-1)+(0.678552027954)
        ref[4,1,1,1]=o*(-0.812625405832)*x_ref[1]**(o-1)+(-0.810345951883)
        ref[4,1,1,2]=o*(-0.211595232946)*x_ref[2]**(o-1)+(-0.984554474963)
        ref[5,0,0,0]=o*(0.339493504495)*x_ref[0]**(o-1)+(-0.903993357652)
        ref[5,0,0,1]=o*(-0.440233624866)*x_ref[1]**(o-1)+(-0.430746451761)
        ref[5,0,0,2]=o*(0.111254680616)*x_ref[2]**(o-1)+(0.709038943317)
        ref[5,0,1,0]=o*(0.931029457116)*x_ref[0]**(o-1)+(-0.785398109537)
        ref[5,0,1,1]=o*(-0.326970114573)*x_ref[1]**(o-1)+(0.0384958937297)
        ref[5,0,1,2]=o*(-0.896259053345)*x_ref[2]**(o-1)+(-0.277237800601)
        ref[5,1,0,0]=o*(0.608509355178)*x_ref[0]**(o-1)+(0.585955903578)
        ref[5,1,0,1]=o*(0.267506079769)*x_ref[1]**(o-1)+(0.812314541749)
        ref[5,1,0,2]=o*(-0.283432880004)*x_ref[2]**(o-1)+(0.909390413963)
        ref[5,1,1,0]=o*(0.23175779421)*x_ref[0]**(o-1)+(-0.614072594905)
        ref[5,1,1,1]=o*(-0.979433217533)*x_ref[1]**(o-1)+(-0.928308978862)
        ref[5,1,1,2]=o*(0.252162852495)*x_ref[2]**(o-1)+(-0.395771235106)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.435083239326)*x[0]+(-0.691333040625)*x[1]
        ref[0]=(0.435083239326)
        ref[1]=(-0.691333040625)
      else:
        s=(-0.845744322657)*x[0]+(1.16954436275)*x[1]+(-0.314198264294)*x[2]
        ref[0]=(-0.845744322657)
        ref[1]=(1.16954436275)
        ref[2]=(-0.314198264294)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-1.56901548345)*x[0]+(-1.12088095812)*x[1]
        s[1]=(0.376788177601)*x[0]+(-0.143805721474)*x[1]
        ref[0,0]=(-1.56901548345)
        ref[0,1]=(-1.12088095812)
        ref[1,0]=(0.376788177601)
        ref[1,1]=(-0.143805721474)
      else:
        s[0]=(-1.22983775782)*x[0]+(0.425438687122)*x[1]+(-0.209028867995)*x[2]
        s[1]=(-0.970298435902)*x[0]+(1.179109977)*x[1]+(-0.139193691146)*x[2]
        ref[0,0]=(-1.22983775782)
        ref[0,1]=(0.425438687122)
        ref[0,2]=(-0.209028867995)
        ref[1,0]=(-0.970298435902)
        ref[1,1]=(1.179109977)
        ref[1,2]=(-0.139193691146)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.227103956711)*x[0]+(-1.30307121808)*x[1]
        s[0,1]=(-0.706864797391)*x[0]+(-1.31928070332)*x[1]
        s[0,2]=(0.634642484639)*x[0]+(0.585731207147)*x[1]
        s[0,3]=(0.431900787729)*x[0]+(0.569282604392)*x[1]
        s[0,4]=(0.0340323913824)*x[0]+(-1.60115889253)*x[1]
        s[1,0]=(0.207234615543)*x[0]+(-0.026497432384)*x[1]
        s[1,1]=(0.401346730679)*x[0]+(0.45537938848)*x[1]
        s[1,2]=(-0.823742908589)*x[0]+(-0.644194821889)*x[1]
        s[1,3]=(-0.949631217459)*x[0]+(-0.32798097587)*x[1]
        s[1,4]=(0.080942942672)*x[0]+(0.155277152998)*x[1]
        s[2,0]=(0.554034527738)*x[0]+(-0.830836696374)*x[1]
        s[2,1]=(0.0740776979829)*x[0]+(-1.03972607973)*x[1]
        s[2,2]=(1.3414358663)*x[0]+(1.09045231286)*x[1]
        s[2,3]=(0.704268712291)*x[0]+(1.42775320769)*x[1]
        s[2,4]=(0.886813956436)*x[0]+(-0.516119087351)*x[1]
        s[3,0]=(0.128480826214)*x[0]+(-0.461671413746)*x[1]
        s[3,1]=(0.179829027295)*x[0]+(-0.832871937244)*x[1]
        s[3,2]=(0.378675852865)*x[0]+(-0.186442070411)*x[1]
        s[3,3]=(-0.445016424955)*x[0]+(-1.4956101833)*x[1]
        s[3,4]=(-0.753080545152)*x[0]+(-0.990468924099)*x[1]
        ref[0,0,0]=(-0.227103956711)
        ref[0,0,1]=(-1.30307121808)
        ref[0,1,0]=(-0.706864797391)
        ref[0,1,1]=(-1.31928070332)
        ref[0,2,0]=(0.634642484639)
        ref[0,2,1]=(0.585731207147)
        ref[0,3,0]=(0.431900787729)
        ref[0,3,1]=(0.569282604392)
        ref[0,4,0]=(0.0340323913824)
        ref[0,4,1]=(-1.60115889253)
        ref[1,0,0]=(0.207234615543)
        ref[1,0,1]=(-0.026497432384)
        ref[1,1,0]=(0.401346730679)
        ref[1,1,1]=(0.45537938848)
        ref[1,2,0]=(-0.823742908589)
        ref[1,2,1]=(-0.644194821889)
        ref[1,3,0]=(-0.949631217459)
        ref[1,3,1]=(-0.32798097587)
        ref[1,4,0]=(0.080942942672)
        ref[1,4,1]=(0.155277152998)
        ref[2,0,0]=(0.554034527738)
        ref[2,0,1]=(-0.830836696374)
        ref[2,1,0]=(0.0740776979829)
        ref[2,1,1]=(-1.03972607973)
        ref[2,2,0]=(1.3414358663)
        ref[2,2,1]=(1.09045231286)
        ref[2,3,0]=(0.704268712291)
        ref[2,3,1]=(1.42775320769)
        ref[2,4,0]=(0.886813956436)
        ref[2,4,1]=(-0.516119087351)
        ref[3,0,0]=(0.128480826214)
        ref[3,0,1]=(-0.461671413746)
        ref[3,1,0]=(0.179829027295)
        ref[3,1,1]=(-0.832871937244)
        ref[3,2,0]=(0.378675852865)
        ref[3,2,1]=(-0.186442070411)
        ref[3,3,0]=(-0.445016424955)
        ref[3,3,1]=(-1.4956101833)
        ref[3,4,0]=(-0.753080545152)
        ref[3,4,1]=(-0.990468924099)
      else:
        s[0,0]=(-1.70799712078)*x[0]+(-0.446362335826)*x[1]+(-0.263476523401)*x[2]
        s[0,1]=(-0.736145268076)*x[0]+(0.646755267112)*x[1]+(0.447221049824)*x[2]
        s[0,2]=(-0.642101757164)*x[0]+(-0.611769559201)*x[1]+(0.264550006471)*x[2]
        s[0,3]=(-0.676996071184)*x[0]+(-0.590821452023)*x[1]+(1.24075002019)*x[2]
        s[0,4]=(1.02823294921)*x[0]+(-0.888741194341)*x[1]+(-0.523239815445)*x[2]
        s[1,0]=(-0.59574377001)*x[0]+(-0.191278786974)*x[1]+(0.159150016923)*x[2]
        s[1,1]=(0.969247182713)*x[0]+(-1.28235270295)*x[1]+(0.849611232483)*x[2]
        s[1,2]=(-0.435529899282)*x[0]+(0.12900241589)*x[1]+(-0.0181780583503)*x[2]
        s[1,3]=(0.57112270455)*x[0]+(-0.991550186645)*x[1]+(0.00643683318083)*x[2]
        s[1,4]=(0.252122977044)*x[0]+(-1.26873100076)*x[1]+(-0.372881623479)*x[2]
        s[2,0]=(0.0976854139907)*x[0]+(-0.464484842151)*x[1]+(-0.695497231127)*x[2]
        s[2,1]=(-0.227165085737)*x[0]+(0.188692778493)*x[1]+(-1.51388182618)*x[2]
        s[2,2]=(0.962466954525)*x[0]+(-0.101670728061)*x[1]+(1.23116858758)*x[2]
        s[2,3]=(-0.609963828043)*x[0]+(-0.775332931826)*x[1]+(0.656264967614)*x[2]
        s[2,4]=(-1.54897910822)*x[0]+(-1.00555194316)*x[1]+(-0.812320917814)*x[2]
        s[3,0]=(-0.616002842225)*x[0]+(0.118716870548)*x[1]+(0.255205330827)*x[2]
        s[3,1]=(-0.26445657759)*x[0]+(0.447308749134)*x[1]+(0.220730408993)*x[2]
        s[3,2]=(-1.85693156269)*x[0]+(1.36352248787)*x[1]+(-0.0517488277821)*x[2]
        s[3,3]=(1.14523233511)*x[0]+(-1.34499926349)*x[1]+(0.228859653657)*x[2]
        s[3,4]=(-0.079222973052)*x[0]+(1.47684264227)*x[1]+(-0.530005798807)*x[2]
        ref[0,0,0]=(-1.70799712078)
        ref[0,0,1]=(-0.446362335826)
        ref[0,0,2]=(-0.263476523401)
        ref[0,1,0]=(-0.736145268076)
        ref[0,1,1]=(0.646755267112)
        ref[0,1,2]=(0.447221049824)
        ref[0,2,0]=(-0.642101757164)
        ref[0,2,1]=(-0.611769559201)
        ref[0,2,2]=(0.264550006471)
        ref[0,3,0]=(-0.676996071184)
        ref[0,3,1]=(-0.590821452023)
        ref[0,3,2]=(1.24075002019)
        ref[0,4,0]=(1.02823294921)
        ref[0,4,1]=(-0.888741194341)
        ref[0,4,2]=(-0.523239815445)
        ref[1,0,0]=(-0.59574377001)
        ref[1,0,1]=(-0.191278786974)
        ref[1,0,2]=(0.159150016923)
        ref[1,1,0]=(0.969247182713)
        ref[1,1,1]=(-1.28235270295)
        ref[1,1,2]=(0.849611232483)
        ref[1,2,0]=(-0.435529899282)
        ref[1,2,1]=(0.12900241589)
        ref[1,2,2]=(-0.0181780583503)
        ref[1,3,0]=(0.57112270455)
        ref[1,3,1]=(-0.991550186645)
        ref[1,3,2]=(0.00643683318083)
        ref[1,4,0]=(0.252122977044)
        ref[1,4,1]=(-1.26873100076)
        ref[1,4,2]=(-0.372881623479)
        ref[2,0,0]=(0.0976854139907)
        ref[2,0,1]=(-0.464484842151)
        ref[2,0,2]=(-0.695497231127)
        ref[2,1,0]=(-0.227165085737)
        ref[2,1,1]=(0.188692778493)
        ref[2,1,2]=(-1.51388182618)
        ref[2,2,0]=(0.962466954525)
        ref[2,2,1]=(-0.101670728061)
        ref[2,2,2]=(1.23116858758)
        ref[2,3,0]=(-0.609963828043)
        ref[2,3,1]=(-0.775332931826)
        ref[2,3,2]=(0.656264967614)
        ref[2,4,0]=(-1.54897910822)
        ref[2,4,1]=(-1.00555194316)
        ref[2,4,2]=(-0.812320917814)
        ref[3,0,0]=(-0.616002842225)
        ref[3,0,1]=(0.118716870548)
        ref[3,0,2]=(0.255205330827)
        ref[3,1,0]=(-0.26445657759)
        ref[3,1,1]=(0.447308749134)
        ref[3,1,2]=(0.220730408993)
        ref[3,2,0]=(-1.85693156269)
        ref[3,2,1]=(1.36352248787)
        ref[3,2,2]=(-0.0517488277821)
        ref[3,3,0]=(1.14523233511)
        ref[3,3,1]=(-1.34499926349)
        ref[3,3,2]=(0.228859653657)
        ref[3,4,0]=(-0.079222973052)
        ref[3,4,1]=(1.47684264227)
        ref[3,4,2]=(-0.530005798807)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(1.49153517734)*x[0]+(-0.579975803933)*x[1]
        s[0,0,1]=(-0.696469728625)*x[0]+(1.21538948965)*x[1]
        s[0,1,0]=(-0.0991175021321)*x[0]+(-1.24138963759)*x[1]
        s[0,1,1]=(0.720869772304)*x[0]+(0.693983413462)*x[1]
        s[1,0,0]=(0.539658773124)*x[0]+(0.558956410162)*x[1]
        s[1,0,1]=(0.219371791063)*x[0]+(-0.0645808851409)*x[1]
        s[1,1,0]=(-0.982429431088)*x[0]+(0.765912701256)*x[1]
        s[1,1,1]=(0.409666526322)*x[0]+(-0.667562489533)*x[1]
        s[2,0,0]=(-1.00270035018)*x[0]+(-0.159198118098)*x[1]
        s[2,0,1]=(1.17972133138)*x[0]+(0.921797063108)*x[1]
        s[2,1,0]=(1.4726821986)*x[0]+(0.2041326504)*x[1]
        s[2,1,1]=(0.365688842529)*x[0]+(0.293751041397)*x[1]
        s[3,0,0]=(-0.485937426896)*x[0]+(-0.0722100496386)*x[1]
        s[3,0,1]=(0.937772100531)*x[0]+(-0.57987183111)*x[1]
        s[3,1,0]=(1.16518206513)*x[0]+(-0.715128236725)*x[1]
        s[3,1,1]=(0.130107225088)*x[0]+(-0.0702897307647)*x[1]
        s[4,0,0]=(-0.396052259681)*x[0]+(0.618983002296)*x[1]
        s[4,0,1]=(0.10948716533)*x[0]+(-0.413031012862)*x[1]
        s[4,1,0]=(1.30735893111)*x[0]+(-1.10958529012)*x[1]
        s[4,1,1]=(0.250657387621)*x[0]+(1.45183909124)*x[1]
        s[5,0,0]=(0.0301234829293)*x[0]+(0.374132370619)*x[1]
        s[5,0,1]=(-0.0915198655839)*x[0]+(0.933624705533)*x[1]
        s[5,1,0]=(1.1572487942)*x[0]+(-0.00851123614571)*x[1]
        s[5,1,1]=(0.761078997709)*x[0]+(1.60352507361)*x[1]
        ref[0,0,0,0]=(1.49153517734)
        ref[0,0,0,1]=(-0.579975803933)
        ref[0,0,1,0]=(-0.696469728625)
        ref[0,0,1,1]=(1.21538948965)
        ref[0,1,0,0]=(-0.0991175021321)
        ref[0,1,0,1]=(-1.24138963759)
        ref[0,1,1,0]=(0.720869772304)
        ref[0,1,1,1]=(0.693983413462)
        ref[1,0,0,0]=(0.539658773124)
        ref[1,0,0,1]=(0.558956410162)
        ref[1,0,1,0]=(0.219371791063)
        ref[1,0,1,1]=(-0.0645808851409)
        ref[1,1,0,0]=(-0.982429431088)
        ref[1,1,0,1]=(0.765912701256)
        ref[1,1,1,0]=(0.409666526322)
        ref[1,1,1,1]=(-0.667562489533)
        ref[2,0,0,0]=(-1.00270035018)
        ref[2,0,0,1]=(-0.159198118098)
        ref[2,0,1,0]=(1.17972133138)
        ref[2,0,1,1]=(0.921797063108)
        ref[2,1,0,0]=(1.4726821986)
        ref[2,1,0,1]=(0.2041326504)
        ref[2,1,1,0]=(0.365688842529)
        ref[2,1,1,1]=(0.293751041397)
        ref[3,0,0,0]=(-0.485937426896)
        ref[3,0,0,1]=(-0.0722100496386)
        ref[3,0,1,0]=(0.937772100531)
        ref[3,0,1,1]=(-0.57987183111)
        ref[3,1,0,0]=(1.16518206513)
        ref[3,1,0,1]=(-0.715128236725)
        ref[3,1,1,0]=(0.130107225088)
        ref[3,1,1,1]=(-0.0702897307647)
        ref[4,0,0,0]=(-0.396052259681)
        ref[4,0,0,1]=(0.618983002296)
        ref[4,0,1,0]=(0.10948716533)
        ref[4,0,1,1]=(-0.413031012862)
        ref[4,1,0,0]=(1.30735893111)
        ref[4,1,0,1]=(-1.10958529012)
        ref[4,1,1,0]=(0.250657387621)
        ref[4,1,1,1]=(1.45183909124)
        ref[5,0,0,0]=(0.0301234829293)
        ref[5,0,0,1]=(0.374132370619)
        ref[5,0,1,0]=(-0.0915198655839)
        ref[5,0,1,1]=(0.933624705533)
        ref[5,1,0,0]=(1.1572487942)
        ref[5,1,0,1]=(-0.00851123614571)
        ref[5,1,1,0]=(0.761078997709)
        ref[5,1,1,1]=(1.60352507361)
      else:
        s[0,0,0]=(1.16204910106)*x[0]+(1.21138322906)*x[1]+(-0.0524201871512)*x[2]
        s[0,0,1]=(-1.62418470618)*x[0]+(0.862850207692)*x[1]+(-0.287954408686)*x[2]
        s[0,1,0]=(-0.560096422291)*x[0]+(-0.3064736255)*x[1]+(-0.104600769051)*x[2]
        s[0,1,1]=(-1.41163310088)*x[0]+(-0.234279745635)*x[1]+(-1.22939949417)*x[2]
        s[1,0,0]=(0.21959990013)*x[0]+(0.40893057141)*x[1]+(0.643102887922)*x[2]
        s[1,0,1]=(-0.191194957197)*x[0]+(1.47353348334)*x[1]+(0.732969151984)*x[2]
        s[1,1,0]=(-1.19345990561)*x[0]+(-0.336674594448)*x[1]+(-0.0578613018881)*x[2]
        s[1,1,1]=(1.6232908613)*x[0]+(-0.107926586217)*x[1]+(0.235855060031)*x[2]
        s[2,0,0]=(0.820509328296)*x[0]+(0.547288169297)*x[1]+(-1.55487561435)*x[2]
        s[2,0,1]=(0.0893134152711)*x[0]+(0.161241646611)*x[1]+(-0.214358478289)*x[2]
        s[2,1,0]=(-1.12660016765)*x[0]+(1.53125870766)*x[1]+(0.697333670681)*x[2]
        s[2,1,1]=(-0.822208944906)*x[0]+(0.361159545618)*x[1]+(-0.0753065880934)*x[2]
        s[3,0,0]=(0.371298368343)*x[0]+(-0.935563626025)*x[1]+(1.32294571078)*x[2]
        s[3,0,1]=(0.228437335618)*x[0]+(1.00946312467)*x[1]+(-0.353871040012)*x[2]
        s[3,1,0]=(0.315905225539)*x[0]+(-1.02555884678)*x[1]+(0.388333892139)*x[2]
        s[3,1,1]=(1.23848230316)*x[0]+(1.02037500431)*x[1]+(0.158105617744)*x[2]
        s[4,0,0]=(-1.45272742631)*x[0]+(0.543388885954)*x[1]+(0.182106047094)*x[2]
        s[4,0,1]=(-0.351765522363)*x[0]+(-0.281592529507)*x[1]+(-0.324592486431)*x[2]
        s[4,1,0]=(-0.0890034165869)*x[0]+(-0.000670251305565)*x[1]+(1.39235380789)*x[2]
        s[4,1,1]=(-0.127096612653)*x[0]+(0.454493346168)*x[1]+(-0.360338313101)*x[2]
        s[5,0,0]=(-0.96052998866)*x[0]+(-0.0674238906685)*x[1]+(-1.63834268246)*x[2]
        s[5,0,1]=(-0.462036588746)*x[0]+(1.1804169499)*x[1]+(1.13779511332)*x[2]
        s[5,1,0]=(0.0863661479908)*x[0]+(0.0691347259375)*x[1]+(0.625219063907)*x[2]
        s[5,1,1]=(0.50049088687)*x[0]+(-0.438895013688)*x[1]+(-0.419695143786)*x[2]
        ref[0,0,0,0]=(1.16204910106)
        ref[0,0,0,1]=(1.21138322906)
        ref[0,0,0,2]=(-0.0524201871512)
        ref[0,0,1,0]=(-1.62418470618)
        ref[0,0,1,1]=(0.862850207692)
        ref[0,0,1,2]=(-0.287954408686)
        ref[0,1,0,0]=(-0.560096422291)
        ref[0,1,0,1]=(-0.3064736255)
        ref[0,1,0,2]=(-0.104600769051)
        ref[0,1,1,0]=(-1.41163310088)
        ref[0,1,1,1]=(-0.234279745635)
        ref[0,1,1,2]=(-1.22939949417)
        ref[1,0,0,0]=(0.21959990013)
        ref[1,0,0,1]=(0.40893057141)
        ref[1,0,0,2]=(0.643102887922)
        ref[1,0,1,0]=(-0.191194957197)
        ref[1,0,1,1]=(1.47353348334)
        ref[1,0,1,2]=(0.732969151984)
        ref[1,1,0,0]=(-1.19345990561)
        ref[1,1,0,1]=(-0.336674594448)
        ref[1,1,0,2]=(-0.0578613018881)
        ref[1,1,1,0]=(1.6232908613)
        ref[1,1,1,1]=(-0.107926586217)
        ref[1,1,1,2]=(0.235855060031)
        ref[2,0,0,0]=(0.820509328296)
        ref[2,0,0,1]=(0.547288169297)
        ref[2,0,0,2]=(-1.55487561435)
        ref[2,0,1,0]=(0.0893134152711)
        ref[2,0,1,1]=(0.161241646611)
        ref[2,0,1,2]=(-0.214358478289)
        ref[2,1,0,0]=(-1.12660016765)
        ref[2,1,0,1]=(1.53125870766)
        ref[2,1,0,2]=(0.697333670681)
        ref[2,1,1,0]=(-0.822208944906)
        ref[2,1,1,1]=(0.361159545618)
        ref[2,1,1,2]=(-0.0753065880934)
        ref[3,0,0,0]=(0.371298368343)
        ref[3,0,0,1]=(-0.935563626025)
        ref[3,0,0,2]=(1.32294571078)
        ref[3,0,1,0]=(0.228437335618)
        ref[3,0,1,1]=(1.00946312467)
        ref[3,0,1,2]=(-0.353871040012)
        ref[3,1,0,0]=(0.315905225539)
        ref[3,1,0,1]=(-1.02555884678)
        ref[3,1,0,2]=(0.388333892139)
        ref[3,1,1,0]=(1.23848230316)
        ref[3,1,1,1]=(1.02037500431)
        ref[3,1,1,2]=(0.158105617744)
        ref[4,0,0,0]=(-1.45272742631)
        ref[4,0,0,1]=(0.543388885954)
        ref[4,0,0,2]=(0.182106047094)
        ref[4,0,1,0]=(-0.351765522363)
        ref[4,0,1,1]=(-0.281592529507)
        ref[4,0,1,2]=(-0.324592486431)
        ref[4,1,0,0]=(-0.0890034165869)
        ref[4,1,0,1]=(-0.000670251305565)
        ref[4,1,0,2]=(1.39235380789)
        ref[4,1,1,0]=(-0.127096612653)
        ref[4,1,1,1]=(0.454493346168)
        ref[4,1,1,2]=(-0.360338313101)
        ref[5,0,0,0]=(-0.96052998866)
        ref[5,0,0,1]=(-0.0674238906685)
        ref[5,0,0,2]=(-1.63834268246)
        ref[5,0,1,0]=(-0.462036588746)
        ref[5,0,1,1]=(1.1804169499)
        ref[5,0,1,2]=(1.13779511332)
        ref[5,1,0,0]=(0.0863661479908)
        ref[5,1,0,1]=(0.0691347259375)
        ref[5,1,0,2]=(0.625219063907)
        ref[5,1,1,0]=(0.50049088687)
        ref[5,1,1,1]=(-0.438895013688)
        ref[5,1,1,2]=(-0.419695143786)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.435083239326)*x[0]+(-0.691333040625)*x[1]
        ref[0]=(0.435083239326)
        ref[1]=(-0.691333040625)
      else:
        s=(-0.845744322657)*x[0]+(1.16954436275)*x[1]+(-0.314198264294)*x[2]
        ref[0]=(-0.845744322657)
        ref[1]=(1.16954436275)
        ref[2]=(-0.314198264294)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-1.56901548345)*x[0]+(-1.12088095812)*x[1]
        s[1]=(0.376788177601)*x[0]+(-0.143805721474)*x[1]
        ref[0,0]=(-1.56901548345)
        ref[0,1]=(-1.12088095812)
        ref[1,0]=(0.376788177601)
        ref[1,1]=(-0.143805721474)
      else:
        s[0]=(-1.22983775782)*x[0]+(0.425438687122)*x[1]+(-0.209028867995)*x[2]
        s[1]=(-0.970298435902)*x[0]+(1.179109977)*x[1]+(-0.139193691146)*x[2]
        ref[0,0]=(-1.22983775782)
        ref[0,1]=(0.425438687122)
        ref[0,2]=(-0.209028867995)
        ref[1,0]=(-0.970298435902)
        ref[1,1]=(1.179109977)
        ref[1,2]=(-0.139193691146)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.227103956711)*x[0]+(-1.30307121808)*x[1]
        s[0,1]=(-0.706864797391)*x[0]+(-1.31928070332)*x[1]
        s[0,2]=(0.634642484639)*x[0]+(0.585731207147)*x[1]
        s[0,3]=(0.431900787729)*x[0]+(0.569282604392)*x[1]
        s[0,4]=(0.0340323913824)*x[0]+(-1.60115889253)*x[1]
        s[1,0]=(0.207234615543)*x[0]+(-0.026497432384)*x[1]
        s[1,1]=(0.401346730679)*x[0]+(0.45537938848)*x[1]
        s[1,2]=(-0.823742908589)*x[0]+(-0.644194821889)*x[1]
        s[1,3]=(-0.949631217459)*x[0]+(-0.32798097587)*x[1]
        s[1,4]=(0.080942942672)*x[0]+(0.155277152998)*x[1]
        s[2,0]=(0.554034527738)*x[0]+(-0.830836696374)*x[1]
        s[2,1]=(0.0740776979829)*x[0]+(-1.03972607973)*x[1]
        s[2,2]=(1.3414358663)*x[0]+(1.09045231286)*x[1]
        s[2,3]=(0.704268712291)*x[0]+(1.42775320769)*x[1]
        s[2,4]=(0.886813956436)*x[0]+(-0.516119087351)*x[1]
        s[3,0]=(0.128480826214)*x[0]+(-0.461671413746)*x[1]
        s[3,1]=(0.179829027295)*x[0]+(-0.832871937244)*x[1]
        s[3,2]=(0.378675852865)*x[0]+(-0.186442070411)*x[1]
        s[3,3]=(-0.445016424955)*x[0]+(-1.4956101833)*x[1]
        s[3,4]=(-0.753080545152)*x[0]+(-0.990468924099)*x[1]
        ref[0,0,0]=(-0.227103956711)
        ref[0,0,1]=(-1.30307121808)
        ref[0,1,0]=(-0.706864797391)
        ref[0,1,1]=(-1.31928070332)
        ref[0,2,0]=(0.634642484639)
        ref[0,2,1]=(0.585731207147)
        ref[0,3,0]=(0.431900787729)
        ref[0,3,1]=(0.569282604392)
        ref[0,4,0]=(0.0340323913824)
        ref[0,4,1]=(-1.60115889253)
        ref[1,0,0]=(0.207234615543)
        ref[1,0,1]=(-0.026497432384)
        ref[1,1,0]=(0.401346730679)
        ref[1,1,1]=(0.45537938848)
        ref[1,2,0]=(-0.823742908589)
        ref[1,2,1]=(-0.644194821889)
        ref[1,3,0]=(-0.949631217459)
        ref[1,3,1]=(-0.32798097587)
        ref[1,4,0]=(0.080942942672)
        ref[1,4,1]=(0.155277152998)
        ref[2,0,0]=(0.554034527738)
        ref[2,0,1]=(-0.830836696374)
        ref[2,1,0]=(0.0740776979829)
        ref[2,1,1]=(-1.03972607973)
        ref[2,2,0]=(1.3414358663)
        ref[2,2,1]=(1.09045231286)
        ref[2,3,0]=(0.704268712291)
        ref[2,3,1]=(1.42775320769)
        ref[2,4,0]=(0.886813956436)
        ref[2,4,1]=(-0.516119087351)
        ref[3,0,0]=(0.128480826214)
        ref[3,0,1]=(-0.461671413746)
        ref[3,1,0]=(0.179829027295)
        ref[3,1,1]=(-0.832871937244)
        ref[3,2,0]=(0.378675852865)
        ref[3,2,1]=(-0.186442070411)
        ref[3,3,0]=(-0.445016424955)
        ref[3,3,1]=(-1.4956101833)
        ref[3,4,0]=(-0.753080545152)
        ref[3,4,1]=(-0.990468924099)
      else:
        s[0,0]=(-1.70799712078)*x[0]+(-0.446362335826)*x[1]+(-0.263476523401)*x[2]
        s[0,1]=(-0.736145268076)*x[0]+(0.646755267112)*x[1]+(0.447221049824)*x[2]
        s[0,2]=(-0.642101757164)*x[0]+(-0.611769559201)*x[1]+(0.264550006471)*x[2]
        s[0,3]=(-0.676996071184)*x[0]+(-0.590821452023)*x[1]+(1.24075002019)*x[2]
        s[0,4]=(1.02823294921)*x[0]+(-0.888741194341)*x[1]+(-0.523239815445)*x[2]
        s[1,0]=(-0.59574377001)*x[0]+(-0.191278786974)*x[1]+(0.159150016923)*x[2]
        s[1,1]=(0.969247182713)*x[0]+(-1.28235270295)*x[1]+(0.849611232483)*x[2]
        s[1,2]=(-0.435529899282)*x[0]+(0.12900241589)*x[1]+(-0.0181780583503)*x[2]
        s[1,3]=(0.57112270455)*x[0]+(-0.991550186645)*x[1]+(0.00643683318083)*x[2]
        s[1,4]=(0.252122977044)*x[0]+(-1.26873100076)*x[1]+(-0.372881623479)*x[2]
        s[2,0]=(0.0976854139907)*x[0]+(-0.464484842151)*x[1]+(-0.695497231127)*x[2]
        s[2,1]=(-0.227165085737)*x[0]+(0.188692778493)*x[1]+(-1.51388182618)*x[2]
        s[2,2]=(0.962466954525)*x[0]+(-0.101670728061)*x[1]+(1.23116858758)*x[2]
        s[2,3]=(-0.609963828043)*x[0]+(-0.775332931826)*x[1]+(0.656264967614)*x[2]
        s[2,4]=(-1.54897910822)*x[0]+(-1.00555194316)*x[1]+(-0.812320917814)*x[2]
        s[3,0]=(-0.616002842225)*x[0]+(0.118716870548)*x[1]+(0.255205330827)*x[2]
        s[3,1]=(-0.26445657759)*x[0]+(0.447308749134)*x[1]+(0.220730408993)*x[2]
        s[3,2]=(-1.85693156269)*x[0]+(1.36352248787)*x[1]+(-0.0517488277821)*x[2]
        s[3,3]=(1.14523233511)*x[0]+(-1.34499926349)*x[1]+(0.228859653657)*x[2]
        s[3,4]=(-0.079222973052)*x[0]+(1.47684264227)*x[1]+(-0.530005798807)*x[2]
        ref[0,0,0]=(-1.70799712078)
        ref[0,0,1]=(-0.446362335826)
        ref[0,0,2]=(-0.263476523401)
        ref[0,1,0]=(-0.736145268076)
        ref[0,1,1]=(0.646755267112)
        ref[0,1,2]=(0.447221049824)
        ref[0,2,0]=(-0.642101757164)
        ref[0,2,1]=(-0.611769559201)
        ref[0,2,2]=(0.264550006471)
        ref[0,3,0]=(-0.676996071184)
        ref[0,3,1]=(-0.590821452023)
        ref[0,3,2]=(1.24075002019)
        ref[0,4,0]=(1.02823294921)
        ref[0,4,1]=(-0.888741194341)
        ref[0,4,2]=(-0.523239815445)
        ref[1,0,0]=(-0.59574377001)
        ref[1,0,1]=(-0.191278786974)
        ref[1,0,2]=(0.159150016923)
        ref[1,1,0]=(0.969247182713)
        ref[1,1,1]=(-1.28235270295)
        ref[1,1,2]=(0.849611232483)
        ref[1,2,0]=(-0.435529899282)
        ref[1,2,1]=(0.12900241589)
        ref[1,2,2]=(-0.0181780583503)
        ref[1,3,0]=(0.57112270455)
        ref[1,3,1]=(-0.991550186645)
        ref[1,3,2]=(0.00643683318083)
        ref[1,4,0]=(0.252122977044)
        ref[1,4,1]=(-1.26873100076)
        ref[1,4,2]=(-0.372881623479)
        ref[2,0,0]=(0.0976854139907)
        ref[2,0,1]=(-0.464484842151)
        ref[2,0,2]=(-0.695497231127)
        ref[2,1,0]=(-0.227165085737)
        ref[2,1,1]=(0.188692778493)
        ref[2,1,2]=(-1.51388182618)
        ref[2,2,0]=(0.962466954525)
        ref[2,2,1]=(-0.101670728061)
        ref[2,2,2]=(1.23116858758)
        ref[2,3,0]=(-0.609963828043)
        ref[2,3,1]=(-0.775332931826)
        ref[2,3,2]=(0.656264967614)
        ref[2,4,0]=(-1.54897910822)
        ref[2,4,1]=(-1.00555194316)
        ref[2,4,2]=(-0.812320917814)
        ref[3,0,0]=(-0.616002842225)
        ref[3,0,1]=(0.118716870548)
        ref[3,0,2]=(0.255205330827)
        ref[3,1,0]=(-0.26445657759)
        ref[3,1,1]=(0.447308749134)
        ref[3,1,2]=(0.220730408993)
        ref[3,2,0]=(-1.85693156269)
        ref[3,2,1]=(1.36352248787)
        ref[3,2,2]=(-0.0517488277821)
        ref[3,3,0]=(1.14523233511)
        ref[3,3,1]=(-1.34499926349)
        ref[3,3,2]=(0.228859653657)
        ref[3,4,0]=(-0.079222973052)
        ref[3,4,1]=(1.47684264227)
        ref[3,4,2]=(-0.530005798807)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(1.49153517734)*x[0]+(-0.579975803933)*x[1]
        s[0,0,1]=(-0.696469728625)*x[0]+(1.21538948965)*x[1]
        s[0,1,0]=(-0.0991175021321)*x[0]+(-1.24138963759)*x[1]
        s[0,1,1]=(0.720869772304)*x[0]+(0.693983413462)*x[1]
        s[1,0,0]=(0.539658773124)*x[0]+(0.558956410162)*x[1]
        s[1,0,1]=(0.219371791063)*x[0]+(-0.0645808851409)*x[1]
        s[1,1,0]=(-0.982429431088)*x[0]+(0.765912701256)*x[1]
        s[1,1,1]=(0.409666526322)*x[0]+(-0.667562489533)*x[1]
        s[2,0,0]=(-1.00270035018)*x[0]+(-0.159198118098)*x[1]
        s[2,0,1]=(1.17972133138)*x[0]+(0.921797063108)*x[1]
        s[2,1,0]=(1.4726821986)*x[0]+(0.2041326504)*x[1]
        s[2,1,1]=(0.365688842529)*x[0]+(0.293751041397)*x[1]
        s[3,0,0]=(-0.485937426896)*x[0]+(-0.0722100496386)*x[1]
        s[3,0,1]=(0.937772100531)*x[0]+(-0.57987183111)*x[1]
        s[3,1,0]=(1.16518206513)*x[0]+(-0.715128236725)*x[1]
        s[3,1,1]=(0.130107225088)*x[0]+(-0.0702897307647)*x[1]
        s[4,0,0]=(-0.396052259681)*x[0]+(0.618983002296)*x[1]
        s[4,0,1]=(0.10948716533)*x[0]+(-0.413031012862)*x[1]
        s[4,1,0]=(1.30735893111)*x[0]+(-1.10958529012)*x[1]
        s[4,1,1]=(0.250657387621)*x[0]+(1.45183909124)*x[1]
        s[5,0,0]=(0.0301234829293)*x[0]+(0.374132370619)*x[1]
        s[5,0,1]=(-0.0915198655839)*x[0]+(0.933624705533)*x[1]
        s[5,1,0]=(1.1572487942)*x[0]+(-0.00851123614571)*x[1]
        s[5,1,1]=(0.761078997709)*x[0]+(1.60352507361)*x[1]
        ref[0,0,0,0]=(1.49153517734)
        ref[0,0,0,1]=(-0.579975803933)
        ref[0,0,1,0]=(-0.696469728625)
        ref[0,0,1,1]=(1.21538948965)
        ref[0,1,0,0]=(-0.0991175021321)
        ref[0,1,0,1]=(-1.24138963759)
        ref[0,1,1,0]=(0.720869772304)
        ref[0,1,1,1]=(0.693983413462)
        ref[1,0,0,0]=(0.539658773124)
        ref[1,0,0,1]=(0.558956410162)
        ref[1,0,1,0]=(0.219371791063)
        ref[1,0,1,1]=(-0.0645808851409)
        ref[1,1,0,0]=(-0.982429431088)
        ref[1,1,0,1]=(0.765912701256)
        ref[1,1,1,0]=(0.409666526322)
        ref[1,1,1,1]=(-0.667562489533)
        ref[2,0,0,0]=(-1.00270035018)
        ref[2,0,0,1]=(-0.159198118098)
        ref[2,0,1,0]=(1.17972133138)
        ref[2,0,1,1]=(0.921797063108)
        ref[2,1,0,0]=(1.4726821986)
        ref[2,1,0,1]=(0.2041326504)
        ref[2,1,1,0]=(0.365688842529)
        ref[2,1,1,1]=(0.293751041397)
        ref[3,0,0,0]=(-0.485937426896)
        ref[3,0,0,1]=(-0.0722100496386)
        ref[3,0,1,0]=(0.937772100531)
        ref[3,0,1,1]=(-0.57987183111)
        ref[3,1,0,0]=(1.16518206513)
        ref[3,1,0,1]=(-0.715128236725)
        ref[3,1,1,0]=(0.130107225088)
        ref[3,1,1,1]=(-0.0702897307647)
        ref[4,0,0,0]=(-0.396052259681)
        ref[4,0,0,1]=(0.618983002296)
        ref[4,0,1,0]=(0.10948716533)
        ref[4,0,1,1]=(-0.413031012862)
        ref[4,1,0,0]=(1.30735893111)
        ref[4,1,0,1]=(-1.10958529012)
        ref[4,1,1,0]=(0.250657387621)
        ref[4,1,1,1]=(1.45183909124)
        ref[5,0,0,0]=(0.0301234829293)
        ref[5,0,0,1]=(0.374132370619)
        ref[5,0,1,0]=(-0.0915198655839)
        ref[5,0,1,1]=(0.933624705533)
        ref[5,1,0,0]=(1.1572487942)
        ref[5,1,0,1]=(-0.00851123614571)
        ref[5,1,1,0]=(0.761078997709)
        ref[5,1,1,1]=(1.60352507361)
      else:
        s[0,0,0]=(1.16204910106)*x[0]+(1.21138322906)*x[1]+(-0.0524201871512)*x[2]
        s[0,0,1]=(-1.62418470618)*x[0]+(0.862850207692)*x[1]+(-0.287954408686)*x[2]
        s[0,1,0]=(-0.560096422291)*x[0]+(-0.3064736255)*x[1]+(-0.104600769051)*x[2]
        s[0,1,1]=(-1.41163310088)*x[0]+(-0.234279745635)*x[1]+(-1.22939949417)*x[2]
        s[1,0,0]=(0.21959990013)*x[0]+(0.40893057141)*x[1]+(0.643102887922)*x[2]
        s[1,0,1]=(-0.191194957197)*x[0]+(1.47353348334)*x[1]+(0.732969151984)*x[2]
        s[1,1,0]=(-1.19345990561)*x[0]+(-0.336674594448)*x[1]+(-0.0578613018881)*x[2]
        s[1,1,1]=(1.6232908613)*x[0]+(-0.107926586217)*x[1]+(0.235855060031)*x[2]
        s[2,0,0]=(0.820509328296)*x[0]+(0.547288169297)*x[1]+(-1.55487561435)*x[2]
        s[2,0,1]=(0.0893134152711)*x[0]+(0.161241646611)*x[1]+(-0.214358478289)*x[2]
        s[2,1,0]=(-1.12660016765)*x[0]+(1.53125870766)*x[1]+(0.697333670681)*x[2]
        s[2,1,1]=(-0.822208944906)*x[0]+(0.361159545618)*x[1]+(-0.0753065880934)*x[2]
        s[3,0,0]=(0.371298368343)*x[0]+(-0.935563626025)*x[1]+(1.32294571078)*x[2]
        s[3,0,1]=(0.228437335618)*x[0]+(1.00946312467)*x[1]+(-0.353871040012)*x[2]
        s[3,1,0]=(0.315905225539)*x[0]+(-1.02555884678)*x[1]+(0.388333892139)*x[2]
        s[3,1,1]=(1.23848230316)*x[0]+(1.02037500431)*x[1]+(0.158105617744)*x[2]
        s[4,0,0]=(-1.45272742631)*x[0]+(0.543388885954)*x[1]+(0.182106047094)*x[2]
        s[4,0,1]=(-0.351765522363)*x[0]+(-0.281592529507)*x[1]+(-0.324592486431)*x[2]
        s[4,1,0]=(-0.0890034165869)*x[0]+(-0.000670251305565)*x[1]+(1.39235380789)*x[2]
        s[4,1,1]=(-0.127096612653)*x[0]+(0.454493346168)*x[1]+(-0.360338313101)*x[2]
        s[5,0,0]=(-0.96052998866)*x[0]+(-0.0674238906685)*x[1]+(-1.63834268246)*x[2]
        s[5,0,1]=(-0.462036588746)*x[0]+(1.1804169499)*x[1]+(1.13779511332)*x[2]
        s[5,1,0]=(0.0863661479908)*x[0]+(0.0691347259375)*x[1]+(0.625219063907)*x[2]
        s[5,1,1]=(0.50049088687)*x[0]+(-0.438895013688)*x[1]+(-0.419695143786)*x[2]
        ref[0,0,0,0]=(1.16204910106)
        ref[0,0,0,1]=(1.21138322906)
        ref[0,0,0,2]=(-0.0524201871512)
        ref[0,0,1,0]=(-1.62418470618)
        ref[0,0,1,1]=(0.862850207692)
        ref[0,0,1,2]=(-0.287954408686)
        ref[0,1,0,0]=(-0.560096422291)
        ref[0,1,0,1]=(-0.3064736255)
        ref[0,1,0,2]=(-0.104600769051)
        ref[0,1,1,0]=(-1.41163310088)
        ref[0,1,1,1]=(-0.234279745635)
        ref[0,1,1,2]=(-1.22939949417)
        ref[1,0,0,0]=(0.21959990013)
        ref[1,0,0,1]=(0.40893057141)
        ref[1,0,0,2]=(0.643102887922)
        ref[1,0,1,0]=(-0.191194957197)
        ref[1,0,1,1]=(1.47353348334)
        ref[1,0,1,2]=(0.732969151984)
        ref[1,1,0,0]=(-1.19345990561)
        ref[1,1,0,1]=(-0.336674594448)
        ref[1,1,0,2]=(-0.0578613018881)
        ref[1,1,1,0]=(1.6232908613)
        ref[1,1,1,1]=(-0.107926586217)
        ref[1,1,1,2]=(0.235855060031)
        ref[2,0,0,0]=(0.820509328296)
        ref[2,0,0,1]=(0.547288169297)
        ref[2,0,0,2]=(-1.55487561435)
        ref[2,0,1,0]=(0.0893134152711)
        ref[2,0,1,1]=(0.161241646611)
        ref[2,0,1,2]=(-0.214358478289)
        ref[2,1,0,0]=(-1.12660016765)
        ref[2,1,0,1]=(1.53125870766)
        ref[2,1,0,2]=(0.697333670681)
        ref[2,1,1,0]=(-0.822208944906)
        ref[2,1,1,1]=(0.361159545618)
        ref[2,1,1,2]=(-0.0753065880934)
        ref[3,0,0,0]=(0.371298368343)
        ref[3,0,0,1]=(-0.935563626025)
        ref[3,0,0,2]=(1.32294571078)
        ref[3,0,1,0]=(0.228437335618)
        ref[3,0,1,1]=(1.00946312467)
        ref[3,0,1,2]=(-0.353871040012)
        ref[3,1,0,0]=(0.315905225539)
        ref[3,1,0,1]=(-1.02555884678)
        ref[3,1,0,2]=(0.388333892139)
        ref[3,1,1,0]=(1.23848230316)
        ref[3,1,1,1]=(1.02037500431)
        ref[3,1,1,2]=(0.158105617744)
        ref[4,0,0,0]=(-1.45272742631)
        ref[4,0,0,1]=(0.543388885954)
        ref[4,0,0,2]=(0.182106047094)
        ref[4,0,1,0]=(-0.351765522363)
        ref[4,0,1,1]=(-0.281592529507)
        ref[4,0,1,2]=(-0.324592486431)
        ref[4,1,0,0]=(-0.0890034165869)
        ref[4,1,0,1]=(-0.000670251305565)
        ref[4,1,0,2]=(1.39235380789)
        ref[4,1,1,0]=(-0.127096612653)
        ref[4,1,1,1]=(0.454493346168)
        ref[4,1,1,2]=(-0.360338313101)
        ref[5,0,0,0]=(-0.96052998866)
        ref[5,0,0,1]=(-0.0674238906685)
        ref[5,0,0,2]=(-1.63834268246)
        ref[5,0,1,0]=(-0.462036588746)
        ref[5,0,1,1]=(1.1804169499)
        ref[5,0,1,2]=(1.13779511332)
        ref[5,1,0,0]=(0.0863661479908)
        ref[5,1,0,1]=(0.0691347259375)
        ref[5,1,0,2]=(0.625219063907)
        ref[5,1,1,0]=(0.50049088687)
        ref[5,1,1,1]=(-0.438895013688)
        ref[5,1,1,2]=(-0.419695143786)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(-0.447987377779)*x[0]**o+(-0.3185298109)*x[0]+(0.748689324197)*x[1]**o+(-0.979369398128)*x[1]
        ref[0]=o*(-0.447987377779)*x_ref[0]**(o-1)+(-0.3185298109)
        ref[1]=o*(0.748689324197)*x_ref[1]**(o-1)+(-0.979369398128)
      else:
        arg=(-0.991175677257)*x[0]**o+(0.486310659385)*x[0]+(0.366862476014)*x[1]**o+(0.718311286349)*x[1]+(-0.593137377946)*x[2]**o+(-0.98259729892)*x[2]
        ref[0]=o*(-0.991175677257)*x_ref[0]**(o-1)+(0.486310659385)
        ref[1]=o*(0.366862476014)*x_ref[1]**(o-1)+(0.718311286349)
        ref[2]=o*(-0.593137377946)*x_ref[2]**(o-1)+(-0.98259729892)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(0.532427358862)*x[0]**o+(-0.699829406888)*x[0]+(-0.378181075858)*x[1]**o+(-0.163897932934)*x[1]
        arg[1]=(0.76393225396)*x[0]**o+(0.87331212216)*x[0]+(0.0261520274749)*x[1]**o+(-0.475078274702)*x[1]
        ref[0,0]=o*(0.532427358862)*x_ref[0]**(o-1)+(-0.699829406888)
        ref[0,1]=o*(-0.378181075858)*x_ref[1]**(o-1)+(-0.163897932934)
        ref[1,0]=o*(0.76393225396)*x_ref[0]**(o-1)+(0.87331212216)
        ref[1,1]=o*(0.0261520274749)*x_ref[1]**(o-1)+(-0.475078274702)
      else:
        arg[0]=(-0.559640148983)*x[0]**o+(0.868360358061)*x[0]+(0.148807680806)*x[1]**o+(-0.488426629736)*x[1]+(0.99381766343)*x[2]**o+(-0.0241703848022)*x[2]
        arg[1]=(-0.00270447073555)*x[0]**o+(0.599732416474)*x[0]+(-0.455449579349)*x[1]**o+(-0.791088816655)*x[1]+(0.850362671968)*x[2]**o+(-0.180726138889)*x[2]
        ref[0,0]=o*(-0.559640148983)*x_ref[0]**(o-1)+(0.868360358061)
        ref[0,1]=o*(0.148807680806)*x_ref[1]**(o-1)+(-0.488426629736)
        ref[0,2]=o*(0.99381766343)*x_ref[2]**(o-1)+(-0.0241703848022)
        ref[1,0]=o*(-0.00270447073555)*x_ref[0]**(o-1)+(0.599732416474)
        ref[1,1]=o*(-0.455449579349)*x_ref[1]**(o-1)+(-0.791088816655)
        ref[1,2]=o*(0.850362671968)*x_ref[2]**(o-1)+(-0.180726138889)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.157399152858)*x[0]**o+(0.00598352783814)*x[0]+(-0.904853822462)*x[1]**o+(0.767645769461)*x[1]
        arg[0,1]=(0.764556091331)*x[0]**o+(-0.167733604181)*x[0]+(0.723254547109)*x[1]**o+(-0.863633737324)*x[1]
        arg[0,2]=(0.480913138549)*x[0]**o+(0.505712705322)*x[0]+(-0.295398027463)*x[1]**o+(-0.0497846093953)*x[1]
        arg[0,3]=(-0.957208239652)*x[0]**o+(0.608346818502)*x[0]+(-0.194623401251)*x[1]**o+(0.670671588806)*x[1]
        arg[0,4]=(-0.179165159786)*x[0]**o+(-0.146090255471)*x[0]+(-0.413009197676)*x[1]**o+(-0.168508218849)*x[1]
        arg[1,0]=(0.659914628278)*x[0]**o+(0.123177996909)*x[0]+(0.713971230628)*x[1]**o+(-0.687569461154)*x[1]
        arg[1,1]=(0.857554477911)*x[0]**o+(0.119604916634)*x[0]+(-0.123185856515)*x[1]**o+(0.968800869965)*x[1]
        arg[1,2]=(0.566596883167)*x[0]**o+(-0.667227639224)*x[0]+(0.29838343622)*x[1]**o+(0.0172896834006)*x[1]
        arg[1,3]=(0.340418732306)*x[0]**o+(-0.115031635126)*x[0]+(0.212495048781)*x[1]**o+(-0.690939141829)*x[1]
        arg[1,4]=(-0.513225187356)*x[0]**o+(0.975159534619)*x[0]+(0.159694719764)*x[1]**o+(-0.441973185517)*x[1]
        arg[2,0]=(0.00598247134499)*x[0]**o+(-0.680866157243)*x[0]+(0.198740044836)*x[1]**o+(-0.776778848701)*x[1]
        arg[2,1]=(0.385572341196)*x[0]**o+(-0.911229388405)*x[0]+(-0.869563350649)*x[1]**o+(-0.453073292811)*x[1]
        arg[2,2]=(-0.678405510051)*x[0]**o+(0.570159849365)*x[0]+(-0.998909274112)*x[1]**o+(-0.56867397999)*x[1]
        arg[2,3]=(0.592650344218)*x[0]**o+(-0.990760992864)*x[0]+(0.54832364584)*x[1]**o+(-0.62848423403)*x[1]
        arg[2,4]=(-0.267222595774)*x[0]**o+(-0.398631721649)*x[0]+(-0.139351142788)*x[1]**o+(-0.973211734322)*x[1]
        arg[3,0]=(-0.748270220449)*x[0]**o+(-0.432648285296)*x[0]+(-0.572322019432)*x[1]**o+(-0.361258377472)*x[1]
        arg[3,1]=(-0.872970937458)*x[0]**o+(-0.412714429527)*x[0]+(0.28965384698)*x[1]**o+(0.0899128505144)*x[1]
        arg[3,2]=(-0.448552675711)*x[0]**o+(-0.0531593149976)*x[0]+(-0.445359129375)*x[1]**o+(-0.357551800127)*x[1]
        arg[3,3]=(-0.120676057603)*x[0]**o+(-0.736959379114)*x[0]+(-0.437089829553)*x[1]**o+(-0.272117257464)*x[1]
        arg[3,4]=(0.342279309395)*x[0]**o+(0.182206506048)*x[0]+(0.843948330758)*x[1]**o+(0.165354303251)*x[1]
        ref[0,0,0]=o*(0.157399152858)*x_ref[0]**(o-1)+(0.00598352783814)
        ref[0,0,1]=o*(-0.904853822462)*x_ref[1]**(o-1)+(0.767645769461)
        ref[0,1,0]=o*(0.764556091331)*x_ref[0]**(o-1)+(-0.167733604181)
        ref[0,1,1]=o*(0.723254547109)*x_ref[1]**(o-1)+(-0.863633737324)
        ref[0,2,0]=o*(0.480913138549)*x_ref[0]**(o-1)+(0.505712705322)
        ref[0,2,1]=o*(-0.295398027463)*x_ref[1]**(o-1)+(-0.0497846093953)
        ref[0,3,0]=o*(-0.957208239652)*x_ref[0]**(o-1)+(0.608346818502)
        ref[0,3,1]=o*(-0.194623401251)*x_ref[1]**(o-1)+(0.670671588806)
        ref[0,4,0]=o*(-0.179165159786)*x_ref[0]**(o-1)+(-0.146090255471)
        ref[0,4,1]=o*(-0.413009197676)*x_ref[1]**(o-1)+(-0.168508218849)
        ref[1,0,0]=o*(0.659914628278)*x_ref[0]**(o-1)+(0.123177996909)
        ref[1,0,1]=o*(0.713971230628)*x_ref[1]**(o-1)+(-0.687569461154)
        ref[1,1,0]=o*(0.857554477911)*x_ref[0]**(o-1)+(0.119604916634)
        ref[1,1,1]=o*(-0.123185856515)*x_ref[1]**(o-1)+(0.968800869965)
        ref[1,2,0]=o*(0.566596883167)*x_ref[0]**(o-1)+(-0.667227639224)
        ref[1,2,1]=o*(0.29838343622)*x_ref[1]**(o-1)+(0.0172896834006)
        ref[1,3,0]=o*(0.340418732306)*x_ref[0]**(o-1)+(-0.115031635126)
        ref[1,3,1]=o*(0.212495048781)*x_ref[1]**(o-1)+(-0.690939141829)
        ref[1,4,0]=o*(-0.513225187356)*x_ref[0]**(o-1)+(0.975159534619)
        ref[1,4,1]=o*(0.159694719764)*x_ref[1]**(o-1)+(-0.441973185517)
        ref[2,0,0]=o*(0.00598247134499)*x_ref[0]**(o-1)+(-0.680866157243)
        ref[2,0,1]=o*(0.198740044836)*x_ref[1]**(o-1)+(-0.776778848701)
        ref[2,1,0]=o*(0.385572341196)*x_ref[0]**(o-1)+(-0.911229388405)
        ref[2,1,1]=o*(-0.869563350649)*x_ref[1]**(o-1)+(-0.453073292811)
        ref[2,2,0]=o*(-0.678405510051)*x_ref[0]**(o-1)+(0.570159849365)
        ref[2,2,1]=o*(-0.998909274112)*x_ref[1]**(o-1)+(-0.56867397999)
        ref[2,3,0]=o*(0.592650344218)*x_ref[0]**(o-1)+(-0.990760992864)
        ref[2,3,1]=o*(0.54832364584)*x_ref[1]**(o-1)+(-0.62848423403)
        ref[2,4,0]=o*(-0.267222595774)*x_ref[0]**(o-1)+(-0.398631721649)
        ref[2,4,1]=o*(-0.139351142788)*x_ref[1]**(o-1)+(-0.973211734322)
        ref[3,0,0]=o*(-0.748270220449)*x_ref[0]**(o-1)+(-0.432648285296)
        ref[3,0,1]=o*(-0.572322019432)*x_ref[1]**(o-1)+(-0.361258377472)
        ref[3,1,0]=o*(-0.872970937458)*x_ref[0]**(o-1)+(-0.412714429527)
        ref[3,1,1]=o*(0.28965384698)*x_ref[1]**(o-1)+(0.0899128505144)
        ref[3,2,0]=o*(-0.448552675711)*x_ref[0]**(o-1)+(-0.0531593149976)
        ref[3,2,1]=o*(-0.445359129375)*x_ref[1]**(o-1)+(-0.357551800127)
        ref[3,3,0]=o*(-0.120676057603)*x_ref[0]**(o-1)+(-0.736959379114)
        ref[3,3,1]=o*(-0.437089829553)*x_ref[1]**(o-1)+(-0.272117257464)
        ref[3,4,0]=o*(0.342279309395)*x_ref[0]**(o-1)+(0.182206506048)
        ref[3,4,1]=o*(0.843948330758)*x_ref[1]**(o-1)+(0.165354303251)
      else:
        arg[0,0]=(-0.912611504385)*x[0]**o+(0.730660242466)*x[0]+(-0.807306513948)*x[1]**o+(-0.611619910819)*x[1]+(-0.567616988907)*x[2]**o+(0.554411283236)*x[2]
        arg[0,1]=(-0.900956431107)*x[0]**o+(-0.731441474835)*x[0]+(0.366905510731)*x[1]**o+(0.444763254457)*x[1]+(0.223337203463)*x[2]**o+(0.0823258304577)*x[2]
        arg[0,2]=(-0.300262400629)*x[0]**o+(0.781835992318)*x[0]+(-0.961025746467)*x[1]**o+(-0.0255420888548)*x[1]+(-0.195336914392)*x[2]**o+(0.631006440014)*x[2]
        arg[0,3]=(-0.0362225627667)*x[0]**o+(-0.258309391935)*x[0]+(0.915114590957)*x[1]**o+(-0.0261151730116)*x[1]+(-0.0359136191693)*x[2]**o+(0.34837541797)*x[2]
        arg[0,4]=(0.718483283111)*x[0]**o+(-0.981945434022)*x[0]+(0.715616160215)*x[1]**o+(-0.213673997408)*x[1]+(0.703864386687)*x[2]**o+(-0.258970624794)*x[2]
        arg[1,0]=(0.562339421788)*x[0]**o+(-0.469800316995)*x[0]+(-0.58860202695)*x[1]**o+(0.592493291759)*x[1]+(-0.0747446509827)*x[2]**o+(0.459846703357)*x[2]
        arg[1,1]=(-0.739792007188)*x[0]**o+(0.69804646711)*x[0]+(0.685574577582)*x[1]**o+(0.34906822136)*x[1]+(-0.346626103556)*x[2]**o+(-0.186290616908)*x[2]
        arg[1,2]=(0.605479936712)*x[0]**o+(0.23811484281)*x[0]+(-0.18270059933)*x[1]**o+(-0.883573478283)*x[1]+(0.558417469124)*x[2]**o+(0.0921284647122)*x[2]
        arg[1,3]=(0.829453952054)*x[0]**o+(0.0943225421949)*x[0]+(-0.377472410614)*x[1]**o+(0.422596289908)*x[1]+(0.992700203574)*x[2]**o+(0.954704595957)*x[2]
        arg[1,4]=(0.713437745608)*x[0]**o+(-0.90755469394)*x[0]+(-0.116969245983)*x[1]**o+(-0.615609779318)*x[1]+(-0.631014272933)*x[2]**o+(0.196556707186)*x[2]
        arg[2,0]=(-0.643142569205)*x[0]**o+(-0.522571508238)*x[0]+(-0.221094875426)*x[1]**o+(0.00263621704394)*x[1]+(-0.346088170378)*x[2]**o+(-0.323346603492)*x[2]
        arg[2,1]=(0.121192120404)*x[0]**o+(-0.910581465748)*x[0]+(0.661403514322)*x[1]**o+(0.314356472557)*x[1]+(0.740349932503)*x[2]**o+(0.397923840477)*x[2]
        arg[2,2]=(0.712887835251)*x[0]**o+(0.802827824977)*x[0]+(0.225275507294)*x[1]**o+(0.00655056099106)*x[1]+(-0.0211189465513)*x[2]**o+(-0.672450969446)*x[2]
        arg[2,3]=(0.220976204507)*x[0]**o+(0.384529933174)*x[0]+(-0.824967564367)*x[1]**o+(0.382079183434)*x[1]+(0.975192346085)*x[2]**o+(-0.845325222223)*x[2]
        arg[2,4]=(-0.316551069119)*x[0]**o+(0.194934627874)*x[0]+(0.192096230967)*x[1]**o+(-0.423775967466)*x[1]+(-0.669140845203)*x[2]**o+(0.367813824992)*x[2]
        arg[3,0]=(-0.361410077778)*x[0]**o+(0.377521588398)*x[0]+(-0.795093364568)*x[1]**o+(0.475257686757)*x[1]+(-0.996619080341)*x[2]**o+(-0.909105543506)*x[2]
        arg[3,1]=(-0.324100979259)*x[0]**o+(-0.950807456193)*x[0]+(-0.141687301441)*x[1]**o+(0.153568029515)*x[1]+(-0.698514966417)*x[2]**o+(0.312395968777)*x[2]
        arg[3,2]=(-0.997783460035)*x[0]**o+(0.328724967881)*x[0]+(-0.561572087716)*x[1]**o+(-0.0148890799876)*x[1]+(0.236142454998)*x[2]**o+(-0.36513107554)*x[2]
        arg[3,3]=(0.750818510719)*x[0]**o+(-0.484598204959)*x[0]+(-0.599436804638)*x[1]**o+(-0.236232976078)*x[1]+(-0.0789148183785)*x[2]**o+(-0.74437164566)*x[2]
        arg[3,4]=(-0.730664688591)*x[0]**o+(-0.94753100979)*x[0]+(-0.671463334034)*x[1]**o+(-0.139992464354)*x[1]+(0.286757519111)*x[2]**o+(0.512453551537)*x[2]
        ref[0,0,0]=o*(-0.912611504385)*x_ref[0]**(o-1)+(0.730660242466)
        ref[0,0,1]=o*(-0.807306513948)*x_ref[1]**(o-1)+(-0.611619910819)
        ref[0,0,2]=o*(-0.567616988907)*x_ref[2]**(o-1)+(0.554411283236)
        ref[0,1,0]=o*(-0.900956431107)*x_ref[0]**(o-1)+(-0.731441474835)
        ref[0,1,1]=o*(0.366905510731)*x_ref[1]**(o-1)+(0.444763254457)
        ref[0,1,2]=o*(0.223337203463)*x_ref[2]**(o-1)+(0.0823258304577)
        ref[0,2,0]=o*(-0.300262400629)*x_ref[0]**(o-1)+(0.781835992318)
        ref[0,2,1]=o*(-0.961025746467)*x_ref[1]**(o-1)+(-0.0255420888548)
        ref[0,2,2]=o*(-0.195336914392)*x_ref[2]**(o-1)+(0.631006440014)
        ref[0,3,0]=o*(-0.0362225627667)*x_ref[0]**(o-1)+(-0.258309391935)
        ref[0,3,1]=o*(0.915114590957)*x_ref[1]**(o-1)+(-0.0261151730116)
        ref[0,3,2]=o*(-0.0359136191693)*x_ref[2]**(o-1)+(0.34837541797)
        ref[0,4,0]=o*(0.718483283111)*x_ref[0]**(o-1)+(-0.981945434022)
        ref[0,4,1]=o*(0.715616160215)*x_ref[1]**(o-1)+(-0.213673997408)
        ref[0,4,2]=o*(0.703864386687)*x_ref[2]**(o-1)+(-0.258970624794)
        ref[1,0,0]=o*(0.562339421788)*x_ref[0]**(o-1)+(-0.469800316995)
        ref[1,0,1]=o*(-0.58860202695)*x_ref[1]**(o-1)+(0.592493291759)
        ref[1,0,2]=o*(-0.0747446509827)*x_ref[2]**(o-1)+(0.459846703357)
        ref[1,1,0]=o*(-0.739792007188)*x_ref[0]**(o-1)+(0.69804646711)
        ref[1,1,1]=o*(0.685574577582)*x_ref[1]**(o-1)+(0.34906822136)
        ref[1,1,2]=o*(-0.346626103556)*x_ref[2]**(o-1)+(-0.186290616908)
        ref[1,2,0]=o*(0.605479936712)*x_ref[0]**(o-1)+(0.23811484281)
        ref[1,2,1]=o*(-0.18270059933)*x_ref[1]**(o-1)+(-0.883573478283)
        ref[1,2,2]=o*(0.558417469124)*x_ref[2]**(o-1)+(0.0921284647122)
        ref[1,3,0]=o*(0.829453952054)*x_ref[0]**(o-1)+(0.0943225421949)
        ref[1,3,1]=o*(-0.377472410614)*x_ref[1]**(o-1)+(0.422596289908)
        ref[1,3,2]=o*(0.992700203574)*x_ref[2]**(o-1)+(0.954704595957)
        ref[1,4,0]=o*(0.713437745608)*x_ref[0]**(o-1)+(-0.90755469394)
        ref[1,4,1]=o*(-0.116969245983)*x_ref[1]**(o-1)+(-0.615609779318)
        ref[1,4,2]=o*(-0.631014272933)*x_ref[2]**(o-1)+(0.196556707186)
        ref[2,0,0]=o*(-0.643142569205)*x_ref[0]**(o-1)+(-0.522571508238)
        ref[2,0,1]=o*(-0.221094875426)*x_ref[1]**(o-1)+(0.00263621704394)
        ref[2,0,2]=o*(-0.346088170378)*x_ref[2]**(o-1)+(-0.323346603492)
        ref[2,1,0]=o*(0.121192120404)*x_ref[0]**(o-1)+(-0.910581465748)
        ref[2,1,1]=o*(0.661403514322)*x_ref[1]**(o-1)+(0.314356472557)
        ref[2,1,2]=o*(0.740349932503)*x_ref[2]**(o-1)+(0.397923840477)
        ref[2,2,0]=o*(0.712887835251)*x_ref[0]**(o-1)+(0.802827824977)
        ref[2,2,1]=o*(0.225275507294)*x_ref[1]**(o-1)+(0.00655056099106)
        ref[2,2,2]=o*(-0.0211189465513)*x_ref[2]**(o-1)+(-0.672450969446)
        ref[2,3,0]=o*(0.220976204507)*x_ref[0]**(o-1)+(0.384529933174)
        ref[2,3,1]=o*(-0.824967564367)*x_ref[1]**(o-1)+(0.382079183434)
        ref[2,3,2]=o*(0.975192346085)*x_ref[2]**(o-1)+(-0.845325222223)
        ref[2,4,0]=o*(-0.316551069119)*x_ref[0]**(o-1)+(0.194934627874)
        ref[2,4,1]=o*(0.192096230967)*x_ref[1]**(o-1)+(-0.423775967466)
        ref[2,4,2]=o*(-0.669140845203)*x_ref[2]**(o-1)+(0.367813824992)
        ref[3,0,0]=o*(-0.361410077778)*x_ref[0]**(o-1)+(0.377521588398)
        ref[3,0,1]=o*(-0.795093364568)*x_ref[1]**(o-1)+(0.475257686757)
        ref[3,0,2]=o*(-0.996619080341)*x_ref[2]**(o-1)+(-0.909105543506)
        ref[3,1,0]=o*(-0.324100979259)*x_ref[0]**(o-1)+(-0.950807456193)
        ref[3,1,1]=o*(-0.141687301441)*x_ref[1]**(o-1)+(0.153568029515)
        ref[3,1,2]=o*(-0.698514966417)*x_ref[2]**(o-1)+(0.312395968777)
        ref[3,2,0]=o*(-0.997783460035)*x_ref[0]**(o-1)+(0.328724967881)
        ref[3,2,1]=o*(-0.561572087716)*x_ref[1]**(o-1)+(-0.0148890799876)
        ref[3,2,2]=o*(0.236142454998)*x_ref[2]**(o-1)+(-0.36513107554)
        ref[3,3,0]=o*(0.750818510719)*x_ref[0]**(o-1)+(-0.484598204959)
        ref[3,3,1]=o*(-0.599436804638)*x_ref[1]**(o-1)+(-0.236232976078)
        ref[3,3,2]=o*(-0.0789148183785)*x_ref[2]**(o-1)+(-0.74437164566)
        ref[3,4,0]=o*(-0.730664688591)*x_ref[0]**(o-1)+(-0.94753100979)
        ref[3,4,1]=o*(-0.671463334034)*x_ref[1]**(o-1)+(-0.139992464354)
        ref[3,4,2]=o*(0.286757519111)*x_ref[2]**(o-1)+(0.512453551537)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.467597203998)*x[0]**o+(-0.878331243959)*x[0]+(-0.971530784104)*x[1]**o+(0.685854853102)*x[1]
        arg[0,0,1]=(0.291345615954)*x[0]**o+(-0.745469654319)*x[0]+(0.976621664002)*x[1]**o+(-0.881452217847)*x[1]
        arg[0,1,0]=(0.537843295475)*x[0]**o+(0.489619951145)*x[0]+(0.77330251637)*x[1]**o+(0.998031266089)*x[1]
        arg[0,1,1]=(0.355984068681)*x[0]**o+(-0.814720548776)*x[0]+(0.495564264626)*x[1]**o+(0.525926194911)*x[1]
        arg[1,0,0]=(0.0459062782507)*x[0]**o+(0.745363380399)*x[0]+(0.254962338793)*x[1]**o+(-0.818268107226)*x[1]
        arg[1,0,1]=(0.806972777032)*x[0]**o+(0.65759411882)*x[0]+(-0.714015350868)*x[1]**o+(0.785611067194)*x[1]
        arg[1,1,0]=(0.576206375389)*x[0]**o+(-0.502513595266)*x[0]+(0.258287392463)*x[1]**o+(0.0897418205816)*x[1]
        arg[1,1,1]=(-0.854645709831)*x[0]**o+(0.48910569315)*x[0]+(0.684503579045)*x[1]**o+(0.156890859871)*x[1]
        arg[2,0,0]=(-0.535650648427)*x[0]**o+(-0.189186476574)*x[0]+(-0.62891575537)*x[1]**o+(-0.37603690255)*x[1]
        arg[2,0,1]=(-0.153001902814)*x[0]**o+(0.028214773063)*x[0]+(-0.605102549169)*x[1]**o+(-0.566991398262)*x[1]
        arg[2,1,0]=(0.607976055886)*x[0]**o+(-0.247395663109)*x[0]+(0.424340230656)*x[1]**o+(0.210489427749)*x[1]
        arg[2,1,1]=(-0.513543937157)*x[0]**o+(0.345931529219)*x[0]+(0.893744279358)*x[1]**o+(-0.519602594857)*x[1]
        arg[3,0,0]=(-0.618812192169)*x[0]**o+(-0.897898654184)*x[0]+(-0.717700928924)*x[1]**o+(-0.21348885325)*x[1]
        arg[3,0,1]=(-0.582829504396)*x[0]**o+(-0.0726162868757)*x[0]+(0.366457797377)*x[1]**o+(0.479006714526)*x[1]
        arg[3,1,0]=(0.929143108169)*x[0]**o+(-0.302254589362)*x[0]+(-0.518817476772)*x[1]**o+(0.854860903894)*x[1]
        arg[3,1,1]=(-0.815422381361)*x[0]**o+(-0.183407261786)*x[0]+(0.841334455272)*x[1]**o+(-0.560841747647)*x[1]
        arg[4,0,0]=(0.746229089137)*x[0]**o+(-0.00173525360161)*x[0]+(0.729473923491)*x[1]**o+(-0.0152936558265)*x[1]
        arg[4,0,1]=(-0.0474071267025)*x[0]**o+(-0.380505263812)*x[0]+(0.166181839511)*x[1]**o+(-0.421069915995)*x[1]
        arg[4,1,0]=(-0.826801927705)*x[0]**o+(0.704799677183)*x[0]+(-0.0346961982898)*x[1]**o+(-0.401235214163)*x[1]
        arg[4,1,1]=(-0.973646144788)*x[0]**o+(0.469976655077)*x[0]+(-0.458846284633)*x[1]**o+(0.818397008807)*x[1]
        arg[5,0,0]=(0.155460716472)*x[0]**o+(-0.509095443426)*x[0]+(-0.0942934981524)*x[1]**o+(0.754087972651)*x[1]
        arg[5,0,1]=(-0.584172835053)*x[0]**o+(-0.904579020559)*x[0]+(0.156301832106)*x[1]**o+(0.169995805721)*x[1]
        arg[5,1,0]=(-0.424977437614)*x[0]**o+(0.244566550647)*x[0]+(-0.474400097866)*x[1]**o+(-0.315912223349)*x[1]
        arg[5,1,1]=(0.764961378837)*x[0]**o+(-0.83633430067)*x[0]+(-0.446378328981)*x[1]**o+(-0.278460992776)*x[1]
        ref[0,0,0,0]=o*(0.467597203998)*x_ref[0]**(o-1)+(-0.878331243959)
        ref[0,0,0,1]=o*(-0.971530784104)*x_ref[1]**(o-1)+(0.685854853102)
        ref[0,0,1,0]=o*(0.291345615954)*x_ref[0]**(o-1)+(-0.745469654319)
        ref[0,0,1,1]=o*(0.976621664002)*x_ref[1]**(o-1)+(-0.881452217847)
        ref[0,1,0,0]=o*(0.537843295475)*x_ref[0]**(o-1)+(0.489619951145)
        ref[0,1,0,1]=o*(0.77330251637)*x_ref[1]**(o-1)+(0.998031266089)
        ref[0,1,1,0]=o*(0.355984068681)*x_ref[0]**(o-1)+(-0.814720548776)
        ref[0,1,1,1]=o*(0.495564264626)*x_ref[1]**(o-1)+(0.525926194911)
        ref[1,0,0,0]=o*(0.0459062782507)*x_ref[0]**(o-1)+(0.745363380399)
        ref[1,0,0,1]=o*(0.254962338793)*x_ref[1]**(o-1)+(-0.818268107226)
        ref[1,0,1,0]=o*(0.806972777032)*x_ref[0]**(o-1)+(0.65759411882)
        ref[1,0,1,1]=o*(-0.714015350868)*x_ref[1]**(o-1)+(0.785611067194)
        ref[1,1,0,0]=o*(0.576206375389)*x_ref[0]**(o-1)+(-0.502513595266)
        ref[1,1,0,1]=o*(0.258287392463)*x_ref[1]**(o-1)+(0.0897418205816)
        ref[1,1,1,0]=o*(-0.854645709831)*x_ref[0]**(o-1)+(0.48910569315)
        ref[1,1,1,1]=o*(0.684503579045)*x_ref[1]**(o-1)+(0.156890859871)
        ref[2,0,0,0]=o*(-0.535650648427)*x_ref[0]**(o-1)+(-0.189186476574)
        ref[2,0,0,1]=o*(-0.62891575537)*x_ref[1]**(o-1)+(-0.37603690255)
        ref[2,0,1,0]=o*(-0.153001902814)*x_ref[0]**(o-1)+(0.028214773063)
        ref[2,0,1,1]=o*(-0.605102549169)*x_ref[1]**(o-1)+(-0.566991398262)
        ref[2,1,0,0]=o*(0.607976055886)*x_ref[0]**(o-1)+(-0.247395663109)
        ref[2,1,0,1]=o*(0.424340230656)*x_ref[1]**(o-1)+(0.210489427749)
        ref[2,1,1,0]=o*(-0.513543937157)*x_ref[0]**(o-1)+(0.345931529219)
        ref[2,1,1,1]=o*(0.893744279358)*x_ref[1]**(o-1)+(-0.519602594857)
        ref[3,0,0,0]=o*(-0.618812192169)*x_ref[0]**(o-1)+(-0.897898654184)
        ref[3,0,0,1]=o*(-0.717700928924)*x_ref[1]**(o-1)+(-0.21348885325)
        ref[3,0,1,0]=o*(-0.582829504396)*x_ref[0]**(o-1)+(-0.0726162868757)
        ref[3,0,1,1]=o*(0.366457797377)*x_ref[1]**(o-1)+(0.479006714526)
        ref[3,1,0,0]=o*(0.929143108169)*x_ref[0]**(o-1)+(-0.302254589362)
        ref[3,1,0,1]=o*(-0.518817476772)*x_ref[1]**(o-1)+(0.854860903894)
        ref[3,1,1,0]=o*(-0.815422381361)*x_ref[0]**(o-1)+(-0.183407261786)
        ref[3,1,1,1]=o*(0.841334455272)*x_ref[1]**(o-1)+(-0.560841747647)
        ref[4,0,0,0]=o*(0.746229089137)*x_ref[0]**(o-1)+(-0.00173525360161)
        ref[4,0,0,1]=o*(0.729473923491)*x_ref[1]**(o-1)+(-0.0152936558265)
        ref[4,0,1,0]=o*(-0.0474071267025)*x_ref[0]**(o-1)+(-0.380505263812)
        ref[4,0,1,1]=o*(0.166181839511)*x_ref[1]**(o-1)+(-0.421069915995)
        ref[4,1,0,0]=o*(-0.826801927705)*x_ref[0]**(o-1)+(0.704799677183)
        ref[4,1,0,1]=o*(-0.0346961982898)*x_ref[1]**(o-1)+(-0.401235214163)
        ref[4,1,1,0]=o*(-0.973646144788)*x_ref[0]**(o-1)+(0.469976655077)
        ref[4,1,1,1]=o*(-0.458846284633)*x_ref[1]**(o-1)+(0.818397008807)
        ref[5,0,0,0]=o*(0.155460716472)*x_ref[0]**(o-1)+(-0.509095443426)
        ref[5,0,0,1]=o*(-0.0942934981524)*x_ref[1]**(o-1)+(0.754087972651)
        ref[5,0,1,0]=o*(-0.584172835053)*x_ref[0]**(o-1)+(-0.904579020559)
        ref[5,0,1,1]=o*(0.156301832106)*x_ref[1]**(o-1)+(0.169995805721)
        ref[5,1,0,0]=o*(-0.424977437614)*x_ref[0]**(o-1)+(0.244566550647)
        ref[5,1,0,1]=o*(-0.474400097866)*x_ref[1]**(o-1)+(-0.315912223349)
        ref[5,1,1,0]=o*(0.764961378837)*x_ref[0]**(o-1)+(-0.83633430067)
        ref[5,1,1,1]=o*(-0.446378328981)*x_ref[1]**(o-1)+(-0.278460992776)
      else:
        arg[0,0,0]=(0.723756735151)*x[0]**o+(0.738121670547)*x[0]+(0.384835073242)*x[1]**o+(-0.997944909714)*x[1]+(0.271533052097)*x[2]**o+(-0.667680363953)*x[2]
        arg[0,0,1]=(-0.279804914835)*x[0]**o+(0.789671404386)*x[0]+(-0.823187173328)*x[1]**o+(-0.0126377581755)*x[1]+(-0.942836469998)*x[2]**o+(0.416417451133)*x[2]
        arg[0,1,0]=(-0.69035514237)*x[0]**o+(0.388349204037)*x[0]+(0.807201832597)*x[1]**o+(-0.969007115064)*x[1]+(0.280506744137)*x[2]**o+(0.228554540968)*x[2]
        arg[0,1,1]=(0.893542584763)*x[0]**o+(-0.755974692532)*x[0]+(0.675321004205)*x[1]**o+(0.670278291297)*x[1]+(0.92092687885)*x[2]**o+(-0.516288388724)*x[2]
        arg[1,0,0]=(0.503764409596)*x[0]**o+(-0.39410032918)*x[0]+(0.532408717712)*x[1]**o+(0.0658355614843)*x[1]+(-0.533534889451)*x[2]**o+(0.846333211477)*x[2]
        arg[1,0,1]=(0.520531780859)*x[0]**o+(0.869039672332)*x[0]+(-0.706342144586)*x[1]**o+(0.0665768341658)*x[1]+(-0.909079295867)*x[2]**o+(-0.34153327843)*x[2]
        arg[1,1,0]=(-0.873546169914)*x[0]**o+(-0.554907283387)*x[0]+(-0.0560665557101)*x[1]**o+(0.436703623389)*x[1]+(0.730437022736)*x[2]**o+(0.100904171105)*x[2]
        arg[1,1,1]=(0.845958886681)*x[0]**o+(0.376444128389)*x[0]+(-0.393744308499)*x[1]**o+(-0.0940537397734)*x[1]+(0.178796464778)*x[2]**o+(-0.584448595407)*x[2]
        arg[2,0,0]=(0.000704382305339)*x[0]**o+(-0.836800712787)*x[0]+(0.671931504433)*x[1]**o+(0.263280364406)*x[1]+(-0.746980015754)*x[2]**o+(-0.985058787575)*x[2]
        arg[2,0,1]=(-0.540690929008)*x[0]**o+(0.656899216804)*x[0]+(-0.608036845804)*x[1]**o+(0.224943400248)*x[1]+(-0.312449005326)*x[2]**o+(-0.621458231634)*x[2]
        arg[2,1,0]=(0.440432866893)*x[0]**o+(0.972046189647)*x[0]+(0.060785199375)*x[1]**o+(0.304756432986)*x[1]+(-0.473510049103)*x[2]**o+(0.3707166351)*x[2]
        arg[2,1,1]=(0.96344472567)*x[0]**o+(-0.153932665404)*x[0]+(-0.43646142206)*x[1]**o+(0.4838463484)*x[1]+(0.613286612579)*x[2]**o+(0.84521461472)*x[2]
        arg[3,0,0]=(0.65147393118)*x[0]**o+(-0.886665762467)*x[0]+(-0.266000665973)*x[1]**o+(0.897979405808)*x[1]+(0.699333158084)*x[2]**o+(-0.460865970247)*x[2]
        arg[3,0,1]=(0.870253308112)*x[0]**o+(-0.248736315721)*x[0]+(-0.816719773596)*x[1]**o+(-0.89378202899)*x[1]+(0.524350309915)*x[2]**o+(0.107325872954)*x[2]
        arg[3,1,0]=(-0.152250686624)*x[0]**o+(-0.89784066484)*x[0]+(-0.327207891694)*x[1]**o+(0.346548492907)*x[1]+(0.409288642845)*x[2]**o+(-0.627349664194)*x[2]
        arg[3,1,1]=(0.398337202036)*x[0]**o+(0.307156588584)*x[0]+(0.0567088869624)*x[1]**o+(-0.132313990308)*x[1]+(-0.361821356587)*x[2]**o+(0.913518325149)*x[2]
        arg[4,0,0]=(0.184289428296)*x[0]**o+(0.138095407277)*x[0]+(0.514405012603)*x[1]**o+(0.590757249119)*x[1]+(-0.984116206238)*x[2]**o+(-0.176561783123)*x[2]
        arg[4,0,1]=(-0.804189448075)*x[0]**o+(0.662750787102)*x[0]+(0.361634380444)*x[1]**o+(-0.682665313256)*x[1]+(-0.0738989964344)*x[2]**o+(0.8870747144)*x[2]
        arg[4,1,0]=(-0.28290153618)*x[0]**o+(-0.942107675992)*x[0]+(0.308140792703)*x[1]**o+(0.686659531002)*x[1]+(0.789553636322)*x[2]**o+(0.587377542884)*x[2]
        arg[4,1,1]=(0.6137979501)*x[0]**o+(-0.564031937362)*x[0]+(-0.116407590432)*x[1]**o+(-0.119100735459)*x[1]+(0.673631378916)*x[2]**o+(0.766413057635)*x[2]
        arg[5,0,0]=(-0.871242033478)*x[0]**o+(0.823859447959)*x[0]+(-0.0506075089356)*x[1]**o+(-0.901254358367)*x[1]+(0.936710991394)*x[2]**o+(0.328438496312)*x[2]
        arg[5,0,1]=(-0.768255700059)*x[0]**o+(0.199623934067)*x[0]+(-0.0715921959057)*x[1]**o+(-0.542435290562)*x[1]+(-0.278962068381)*x[2]**o+(0.784363143932)*x[2]
        arg[5,1,0]=(0.897130313615)*x[0]**o+(-0.519033315355)*x[0]+(0.223176381444)*x[1]**o+(0.495645921147)*x[1]+(-0.0957696173909)*x[2]**o+(-0.768307051074)*x[2]
        arg[5,1,1]=(-0.944161779454)*x[0]**o+(0.430097328631)*x[0]+(0.86626977715)*x[1]**o+(-0.675612908116)*x[1]+(0.506821683353)*x[2]**o+(0.386559021354)*x[2]
        ref[0,0,0,0]=o*(0.723756735151)*x_ref[0]**(o-1)+(0.738121670547)
        ref[0,0,0,1]=o*(0.384835073242)*x_ref[1]**(o-1)+(-0.997944909714)
        ref[0,0,0,2]=o*(0.271533052097)*x_ref[2]**(o-1)+(-0.667680363953)
        ref[0,0,1,0]=o*(-0.279804914835)*x_ref[0]**(o-1)+(0.789671404386)
        ref[0,0,1,1]=o*(-0.823187173328)*x_ref[1]**(o-1)+(-0.0126377581755)
        ref[0,0,1,2]=o*(-0.942836469998)*x_ref[2]**(o-1)+(0.416417451133)
        ref[0,1,0,0]=o*(-0.69035514237)*x_ref[0]**(o-1)+(0.388349204037)
        ref[0,1,0,1]=o*(0.807201832597)*x_ref[1]**(o-1)+(-0.969007115064)
        ref[0,1,0,2]=o*(0.280506744137)*x_ref[2]**(o-1)+(0.228554540968)
        ref[0,1,1,0]=o*(0.893542584763)*x_ref[0]**(o-1)+(-0.755974692532)
        ref[0,1,1,1]=o*(0.675321004205)*x_ref[1]**(o-1)+(0.670278291297)
        ref[0,1,1,2]=o*(0.92092687885)*x_ref[2]**(o-1)+(-0.516288388724)
        ref[1,0,0,0]=o*(0.503764409596)*x_ref[0]**(o-1)+(-0.39410032918)
        ref[1,0,0,1]=o*(0.532408717712)*x_ref[1]**(o-1)+(0.0658355614843)
        ref[1,0,0,2]=o*(-0.533534889451)*x_ref[2]**(o-1)+(0.846333211477)
        ref[1,0,1,0]=o*(0.520531780859)*x_ref[0]**(o-1)+(0.869039672332)
        ref[1,0,1,1]=o*(-0.706342144586)*x_ref[1]**(o-1)+(0.0665768341658)
        ref[1,0,1,2]=o*(-0.909079295867)*x_ref[2]**(o-1)+(-0.34153327843)
        ref[1,1,0,0]=o*(-0.873546169914)*x_ref[0]**(o-1)+(-0.554907283387)
        ref[1,1,0,1]=o*(-0.0560665557101)*x_ref[1]**(o-1)+(0.436703623389)
        ref[1,1,0,2]=o*(0.730437022736)*x_ref[2]**(o-1)+(0.100904171105)
        ref[1,1,1,0]=o*(0.845958886681)*x_ref[0]**(o-1)+(0.376444128389)
        ref[1,1,1,1]=o*(-0.393744308499)*x_ref[1]**(o-1)+(-0.0940537397734)
        ref[1,1,1,2]=o*(0.178796464778)*x_ref[2]**(o-1)+(-0.584448595407)
        ref[2,0,0,0]=o*(0.000704382305339)*x_ref[0]**(o-1)+(-0.836800712787)
        ref[2,0,0,1]=o*(0.671931504433)*x_ref[1]**(o-1)+(0.263280364406)
        ref[2,0,0,2]=o*(-0.746980015754)*x_ref[2]**(o-1)+(-0.985058787575)
        ref[2,0,1,0]=o*(-0.540690929008)*x_ref[0]**(o-1)+(0.656899216804)
        ref[2,0,1,1]=o*(-0.608036845804)*x_ref[1]**(o-1)+(0.224943400248)
        ref[2,0,1,2]=o*(-0.312449005326)*x_ref[2]**(o-1)+(-0.621458231634)
        ref[2,1,0,0]=o*(0.440432866893)*x_ref[0]**(o-1)+(0.972046189647)
        ref[2,1,0,1]=o*(0.060785199375)*x_ref[1]**(o-1)+(0.304756432986)
        ref[2,1,0,2]=o*(-0.473510049103)*x_ref[2]**(o-1)+(0.3707166351)
        ref[2,1,1,0]=o*(0.96344472567)*x_ref[0]**(o-1)+(-0.153932665404)
        ref[2,1,1,1]=o*(-0.43646142206)*x_ref[1]**(o-1)+(0.4838463484)
        ref[2,1,1,2]=o*(0.613286612579)*x_ref[2]**(o-1)+(0.84521461472)
        ref[3,0,0,0]=o*(0.65147393118)*x_ref[0]**(o-1)+(-0.886665762467)
        ref[3,0,0,1]=o*(-0.266000665973)*x_ref[1]**(o-1)+(0.897979405808)
        ref[3,0,0,2]=o*(0.699333158084)*x_ref[2]**(o-1)+(-0.460865970247)
        ref[3,0,1,0]=o*(0.870253308112)*x_ref[0]**(o-1)+(-0.248736315721)
        ref[3,0,1,1]=o*(-0.816719773596)*x_ref[1]**(o-1)+(-0.89378202899)
        ref[3,0,1,2]=o*(0.524350309915)*x_ref[2]**(o-1)+(0.107325872954)
        ref[3,1,0,0]=o*(-0.152250686624)*x_ref[0]**(o-1)+(-0.89784066484)
        ref[3,1,0,1]=o*(-0.327207891694)*x_ref[1]**(o-1)+(0.346548492907)
        ref[3,1,0,2]=o*(0.409288642845)*x_ref[2]**(o-1)+(-0.627349664194)
        ref[3,1,1,0]=o*(0.398337202036)*x_ref[0]**(o-1)+(0.307156588584)
        ref[3,1,1,1]=o*(0.0567088869624)*x_ref[1]**(o-1)+(-0.132313990308)
        ref[3,1,1,2]=o*(-0.361821356587)*x_ref[2]**(o-1)+(0.913518325149)
        ref[4,0,0,0]=o*(0.184289428296)*x_ref[0]**(o-1)+(0.138095407277)
        ref[4,0,0,1]=o*(0.514405012603)*x_ref[1]**(o-1)+(0.590757249119)
        ref[4,0,0,2]=o*(-0.984116206238)*x_ref[2]**(o-1)+(-0.176561783123)
        ref[4,0,1,0]=o*(-0.804189448075)*x_ref[0]**(o-1)+(0.662750787102)
        ref[4,0,1,1]=o*(0.361634380444)*x_ref[1]**(o-1)+(-0.682665313256)
        ref[4,0,1,2]=o*(-0.0738989964344)*x_ref[2]**(o-1)+(0.8870747144)
        ref[4,1,0,0]=o*(-0.28290153618)*x_ref[0]**(o-1)+(-0.942107675992)
        ref[4,1,0,1]=o*(0.308140792703)*x_ref[1]**(o-1)+(0.686659531002)
        ref[4,1,0,2]=o*(0.789553636322)*x_ref[2]**(o-1)+(0.587377542884)
        ref[4,1,1,0]=o*(0.6137979501)*x_ref[0]**(o-1)+(-0.564031937362)
        ref[4,1,1,1]=o*(-0.116407590432)*x_ref[1]**(o-1)+(-0.119100735459)
        ref[4,1,1,2]=o*(0.673631378916)*x_ref[2]**(o-1)+(0.766413057635)
        ref[5,0,0,0]=o*(-0.871242033478)*x_ref[0]**(o-1)+(0.823859447959)
        ref[5,0,0,1]=o*(-0.0506075089356)*x_ref[1]**(o-1)+(-0.901254358367)
        ref[5,0,0,2]=o*(0.936710991394)*x_ref[2]**(o-1)+(0.328438496312)
        ref[5,0,1,0]=o*(-0.768255700059)*x_ref[0]**(o-1)+(0.199623934067)
        ref[5,0,1,1]=o*(-0.0715921959057)*x_ref[1]**(o-1)+(-0.542435290562)
        ref[5,0,1,2]=o*(-0.278962068381)*x_ref[2]**(o-1)+(0.784363143932)
        ref[5,1,0,0]=o*(0.897130313615)*x_ref[0]**(o-1)+(-0.519033315355)
        ref[5,1,0,1]=o*(0.223176381444)*x_ref[1]**(o-1)+(0.495645921147)
        ref[5,1,0,2]=o*(-0.0957696173909)*x_ref[2]**(o-1)+(-0.768307051074)
        ref[5,1,1,0]=o*(-0.944161779454)*x_ref[0]**(o-1)+(0.430097328631)
        ref[5,1,1,1]=o*(0.86626977715)*x_ref[1]**(o-1)+(-0.675612908116)
        ref[5,1,1,2]=o*(0.506821683353)*x_ref[2]**(o-1)+(0.386559021354)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_Solution_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.54415545025)*x[0]**o+(0.979947366774)*x[0]+(-0.405934730304)*x[1]**o+(0.424994268397)*x[1]
        ref[0]=o*(0.54415545025)*x_ref[0]**(o-1)+(0.979947366774)
        ref[1]=o*(-0.405934730304)*x_ref[1]**(o-1)+(0.424994268397)
      else:
        arg=(-0.0240792232301)*x[0]**o+(0.33156704088)*x[0]+(-0.051539264252)*x[1]**o+(-0.886370693155)*x[1]+(-0.954475798211)*x[2]**o+(0.238073928589)*x[2]
        ref[0]=o*(-0.0240792232301)*x_ref[0]**(o-1)+(0.33156704088)
        ref[1]=o*(-0.051539264252)*x_ref[1]**(o-1)+(-0.886370693155)
        ref[2]=o*(-0.954475798211)*x_ref[2]**(o-1)+(0.238073928589)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_Solution_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.526471606132)*x[0]**o+(-0.355534153967)*x[0]+(-0.792258102571)*x[1]**o+(0.386283553167)*x[1]
        arg[1]=(-0.892924864905)*x[0]**o+(0.19218757952)*x[0]+(0.00248964495987)*x[1]**o+(-0.687070938997)*x[1]
        ref[0,0]=o*(-0.526471606132)*x_ref[0]**(o-1)+(-0.355534153967)
        ref[0,1]=o*(-0.792258102571)*x_ref[1]**(o-1)+(0.386283553167)
        ref[1,0]=o*(-0.892924864905)*x_ref[0]**(o-1)+(0.19218757952)
        ref[1,1]=o*(0.00248964495987)*x_ref[1]**(o-1)+(-0.687070938997)
      else:
        arg[0]=(-0.254013984725)*x[0]**o+(0.126174108068)*x[0]+(-0.967653739084)*x[1]**o+(-0.524294999327)*x[1]+(0.374454913375)*x[2]**o+(0.783768839592)*x[2]
        arg[1]=(0.862328996944)*x[0]**o+(-0.0100272707295)*x[0]+(0.163524617753)*x[1]**o+(-0.19251370258)*x[1]+(0.889557068195)*x[2]**o+(0.573248942406)*x[2]
        ref[0,0]=o*(-0.254013984725)*x_ref[0]**(o-1)+(0.126174108068)
        ref[0,1]=o*(-0.967653739084)*x_ref[1]**(o-1)+(-0.524294999327)
        ref[0,2]=o*(0.374454913375)*x_ref[2]**(o-1)+(0.783768839592)
        ref[1,0]=o*(0.862328996944)*x_ref[0]**(o-1)+(-0.0100272707295)
        ref[1,1]=o*(0.163524617753)*x_ref[1]**(o-1)+(-0.19251370258)
        ref[1,2]=o*(0.889557068195)*x_ref[2]**(o-1)+(0.573248942406)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_Solution_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.712410724071)*x[0]**o+(0.642832165006)*x[0]+(0.409613677638)*x[1]**o+(-0.940707375829)*x[1]
        arg[0,1]=(-0.192677448978)*x[0]**o+(0.928105399392)*x[0]+(-0.0976390111113)*x[1]**o+(0.152541825282)*x[1]
        arg[0,2]=(-0.555865920622)*x[0]**o+(0.187618737934)*x[0]+(-0.400398722616)*x[1]**o+(0.132335647294)*x[1]
        arg[0,3]=(-0.0984345355899)*x[0]**o+(-0.136251593387)*x[0]+(0.820900167194)*x[1]**o+(-0.54323976155)*x[1]
        arg[0,4]=(-0.718763327727)*x[0]**o+(-0.993678837401)*x[0]+(0.0282625285716)*x[1]**o+(0.480464124636)*x[1]
        arg[1,0]=(-0.1635372419)*x[0]**o+(0.97173720053)*x[0]+(0.0423190406144)*x[1]**o+(-0.607779321237)*x[1]
        arg[1,1]=(-0.59220466746)*x[0]**o+(-0.329625880847)*x[0]+(0.176412343254)*x[1]**o+(0.955769019385)*x[1]
        arg[1,2]=(-0.516421195901)*x[0]**o+(0.147037193899)*x[0]+(0.247835053631)*x[1]**o+(-0.877986653535)*x[1]
        arg[1,3]=(-0.317266388999)*x[0]**o+(0.864393749347)*x[0]+(-0.957863705756)*x[1]**o+(-0.884975429798)*x[1]
        arg[1,4]=(0.540121554391)*x[0]**o+(0.808050078272)*x[0]+(0.703557587067)*x[1]**o+(-0.0976121077766)*x[1]
        arg[2,0]=(0.23658022222)*x[0]**o+(0.122038637852)*x[0]+(0.411570045798)*x[1]**o+(-0.434531687414)*x[1]
        arg[2,1]=(-0.378365306249)*x[0]**o+(0.511747532394)*x[0]+(0.586326813286)*x[1]**o+(0.699918966936)*x[1]
        arg[2,2]=(0.445092669694)*x[0]**o+(0.637517787944)*x[0]+(-0.668779388847)*x[1]**o+(0.487635262236)*x[1]
        arg[2,3]=(-0.232265851556)*x[0]**o+(0.391449220433)*x[0]+(0.647177607766)*x[1]**o+(0.207062996634)*x[1]
        arg[2,4]=(0.338412227828)*x[0]**o+(0.0205801366652)*x[0]+(-0.492628796478)*x[1]**o+(-0.683459852092)*x[1]
        arg[3,0]=(0.584296838468)*x[0]**o+(-0.598793253755)*x[0]+(0.707319426212)*x[1]**o+(0.678834769735)*x[1]
        arg[3,1]=(0.801295300134)*x[0]**o+(0.0494871119593)*x[0]+(0.378929822492)*x[1]**o+(-0.536309789421)*x[1]
        arg[3,2]=(-0.761535984131)*x[0]**o+(0.121584094596)*x[0]+(-0.00655849155175)*x[1]**o+(0.321872124523)*x[1]
        arg[3,3]=(0.384633290815)*x[0]**o+(-0.710356674518)*x[0]+(0.962059942069)*x[1]**o+(-0.593059919585)*x[1]
        arg[3,4]=(-0.608988807286)*x[0]**o+(-0.0776468065145)*x[0]+(-0.701400144867)*x[1]**o+(0.643928611178)*x[1]
        ref[0,0,0]=o*(0.712410724071)*x_ref[0]**(o-1)+(0.642832165006)
        ref[0,0,1]=o*(0.409613677638)*x_ref[1]**(o-1)+(-0.940707375829)
        ref[0,1,0]=o*(-0.192677448978)*x_ref[0]**(o-1)+(0.928105399392)
        ref[0,1,1]=o*(-0.0976390111113)*x_ref[1]**(o-1)+(0.152541825282)
        ref[0,2,0]=o*(-0.555865920622)*x_ref[0]**(o-1)+(0.187618737934)
        ref[0,2,1]=o*(-0.400398722616)*x_ref[1]**(o-1)+(0.132335647294)
        ref[0,3,0]=o*(-0.0984345355899)*x_ref[0]**(o-1)+(-0.136251593387)
        ref[0,3,1]=o*(0.820900167194)*x_ref[1]**(o-1)+(-0.54323976155)
        ref[0,4,0]=o*(-0.718763327727)*x_ref[0]**(o-1)+(-0.993678837401)
        ref[0,4,1]=o*(0.0282625285716)*x_ref[1]**(o-1)+(0.480464124636)
        ref[1,0,0]=o*(-0.1635372419)*x_ref[0]**(o-1)+(0.97173720053)
        ref[1,0,1]=o*(0.0423190406144)*x_ref[1]**(o-1)+(-0.607779321237)
        ref[1,1,0]=o*(-0.59220466746)*x_ref[0]**(o-1)+(-0.329625880847)
        ref[1,1,1]=o*(0.176412343254)*x_ref[1]**(o-1)+(0.955769019385)
        ref[1,2,0]=o*(-0.516421195901)*x_ref[0]**(o-1)+(0.147037193899)
        ref[1,2,1]=o*(0.247835053631)*x_ref[1]**(o-1)+(-0.877986653535)
        ref[1,3,0]=o*(-0.317266388999)*x_ref[0]**(o-1)+(0.864393749347)
        ref[1,3,1]=o*(-0.957863705756)*x_ref[1]**(o-1)+(-0.884975429798)
        ref[1,4,0]=o*(0.540121554391)*x_ref[0]**(o-1)+(0.808050078272)
        ref[1,4,1]=o*(0.703557587067)*x_ref[1]**(o-1)+(-0.0976121077766)
        ref[2,0,0]=o*(0.23658022222)*x_ref[0]**(o-1)+(0.122038637852)
        ref[2,0,1]=o*(0.411570045798)*x_ref[1]**(o-1)+(-0.434531687414)
        ref[2,1,0]=o*(-0.378365306249)*x_ref[0]**(o-1)+(0.511747532394)
        ref[2,1,1]=o*(0.586326813286)*x_ref[1]**(o-1)+(0.699918966936)
        ref[2,2,0]=o*(0.445092669694)*x_ref[0]**(o-1)+(0.637517787944)
        ref[2,2,1]=o*(-0.668779388847)*x_ref[1]**(o-1)+(0.487635262236)
        ref[2,3,0]=o*(-0.232265851556)*x_ref[0]**(o-1)+(0.391449220433)
        ref[2,3,1]=o*(0.647177607766)*x_ref[1]**(o-1)+(0.207062996634)
        ref[2,4,0]=o*(0.338412227828)*x_ref[0]**(o-1)+(0.0205801366652)
        ref[2,4,1]=o*(-0.492628796478)*x_ref[1]**(o-1)+(-0.683459852092)
        ref[3,0,0]=o*(0.584296838468)*x_ref[0]**(o-1)+(-0.598793253755)
        ref[3,0,1]=o*(0.707319426212)*x_ref[1]**(o-1)+(0.678834769735)
        ref[3,1,0]=o*(0.801295300134)*x_ref[0]**(o-1)+(0.0494871119593)
        ref[3,1,1]=o*(0.378929822492)*x_ref[1]**(o-1)+(-0.536309789421)
        ref[3,2,0]=o*(-0.761535984131)*x_ref[0]**(o-1)+(0.121584094596)
        ref[3,2,1]=o*(-0.00655849155175)*x_ref[1]**(o-1)+(0.321872124523)
        ref[3,3,0]=o*(0.384633290815)*x_ref[0]**(o-1)+(-0.710356674518)
        ref[3,3,1]=o*(0.962059942069)*x_ref[1]**(o-1)+(-0.593059919585)
        ref[3,4,0]=o*(-0.608988807286)*x_ref[0]**(o-1)+(-0.0776468065145)
        ref[3,4,1]=o*(-0.701400144867)*x_ref[1]**(o-1)+(0.643928611178)
      else:
        arg[0,0]=(-0.514546772198)*x[0]**o+(0.679859064768)*x[0]+(-0.494037106077)*x[1]**o+(-0.0598645161488)*x[1]+(-0.543474311879)*x[2]**o+(0.597237276677)*x[2]
        arg[0,1]=(0.0826513727579)*x[0]**o+(0.870535937411)*x[0]+(-0.194047768119)*x[1]**o+(-0.111616761775)*x[1]+(0.697073558803)*x[2]**o+(-0.170433075843)*x[2]
        arg[0,2]=(0.608560200938)*x[0]**o+(-0.28991661304)*x[0]+(0.929121366615)*x[1]**o+(0.445280468492)*x[1]+(-0.0671026162019)*x[2]**o+(-0.835086989851)*x[2]
        arg[0,3]=(-0.256476478337)*x[0]**o+(-0.857686365148)*x[0]+(-0.309306419219)*x[1]**o+(0.750579112592)*x[1]+(0.758254577714)*x[2]**o+(0.108972886739)*x[2]
        arg[0,4]=(0.189540467853)*x[0]**o+(-0.639906129587)*x[0]+(0.1086564431)*x[1]**o+(-0.254754163159)*x[1]+(-0.595415956315)*x[2]**o+(0.416643698623)*x[2]
        arg[1,0]=(0.649640741159)*x[0]**o+(-0.715604744977)*x[0]+(0.144698127223)*x[1]**o+(0.84058008507)*x[1]+(0.00226738664623)*x[2]**o+(0.143092105366)*x[2]
        arg[1,1]=(0.30428097785)*x[0]**o+(0.172630542192)*x[0]+(0.645118113075)*x[1]**o+(-0.394468022223)*x[1]+(0.731670923192)*x[2]**o+(-0.676021495379)*x[2]
        arg[1,2]=(0.56459927889)*x[0]**o+(-0.624980053566)*x[0]+(-0.524836958708)*x[1]**o+(0.282862320474)*x[1]+(0.99070455422)*x[2]**o+(0.000265746399286)*x[2]
        arg[1,3]=(-0.251929813217)*x[0]**o+(0.00685246758491)*x[0]+(-0.252595929082)*x[1]**o+(-0.997558334417)*x[1]+(-0.431329146194)*x[2]**o+(0.0685298495116)*x[2]
        arg[1,4]=(-0.850398225165)*x[0]**o+(-0.752363025861)*x[0]+(-0.669448553729)*x[1]**o+(0.809247179374)*x[1]+(-0.927467807251)*x[2]**o+(-0.498094160268)*x[2]
        arg[2,0]=(0.541736859663)*x[0]**o+(-0.651273876855)*x[0]+(-0.679670779769)*x[1]**o+(0.669564679419)*x[1]+(-0.825048237017)*x[2]**o+(-0.247744537755)*x[2]
        arg[2,1]=(0.909016713138)*x[0]**o+(-0.794124179572)*x[0]+(0.887294650854)*x[1]**o+(-0.198933351435)*x[1]+(-0.819671062456)*x[2]**o+(0.287455634612)*x[2]
        arg[2,2]=(-0.469337238665)*x[0]**o+(0.787828984218)*x[0]+(0.916042829702)*x[1]**o+(-0.475983675757)*x[1]+(-0.350660319046)*x[2]**o+(0.944222119413)*x[2]
        arg[2,3]=(-0.194629035155)*x[0]**o+(0.452058614199)*x[0]+(-0.565150768916)*x[1]**o+(-0.967642852138)*x[1]+(-0.345332973443)*x[2]**o+(-0.238359110833)*x[2]
        arg[2,4]=(-0.744197369952)*x[0]**o+(-0.0161450929106)*x[0]+(0.946555120156)*x[1]**o+(-0.336314723492)*x[1]+(0.212936909808)*x[2]**o+(-0.900582293569)*x[2]
        arg[3,0]=(0.90736518985)*x[0]**o+(-0.866096443075)*x[0]+(0.440227521532)*x[1]**o+(-0.545983384676)*x[1]+(0.433167375756)*x[2]**o+(-0.913060961228)*x[2]
        arg[3,1]=(0.33547032327)*x[0]**o+(0.766388218896)*x[0]+(-0.386141806114)*x[1]**o+(0.28471049668)*x[1]+(0.389347155974)*x[2]**o+(0.211663215285)*x[2]
        arg[3,2]=(0.193757801099)*x[0]**o+(0.0145439587225)*x[0]+(0.575506295918)*x[1]**o+(-0.678317336021)*x[1]+(0.64603157052)*x[2]**o+(-0.876667949279)*x[2]
        arg[3,3]=(-0.0518002857411)*x[0]**o+(-0.540021971724)*x[0]+(-0.490275636747)*x[1]**o+(-0.849147647677)*x[1]+(-0.0132783508877)*x[2]**o+(0.0859860768541)*x[2]
        arg[3,4]=(-0.332675003425)*x[0]**o+(-0.757986125506)*x[0]+(-0.487001539705)*x[1]**o+(0.898881577697)*x[1]+(0.56557767007)*x[2]**o+(0.181594113977)*x[2]
        ref[0,0,0]=o*(-0.514546772198)*x_ref[0]**(o-1)+(0.679859064768)
        ref[0,0,1]=o*(-0.494037106077)*x_ref[1]**(o-1)+(-0.0598645161488)
        ref[0,0,2]=o*(-0.543474311879)*x_ref[2]**(o-1)+(0.597237276677)
        ref[0,1,0]=o*(0.0826513727579)*x_ref[0]**(o-1)+(0.870535937411)
        ref[0,1,1]=o*(-0.194047768119)*x_ref[1]**(o-1)+(-0.111616761775)
        ref[0,1,2]=o*(0.697073558803)*x_ref[2]**(o-1)+(-0.170433075843)
        ref[0,2,0]=o*(0.608560200938)*x_ref[0]**(o-1)+(-0.28991661304)
        ref[0,2,1]=o*(0.929121366615)*x_ref[1]**(o-1)+(0.445280468492)
        ref[0,2,2]=o*(-0.0671026162019)*x_ref[2]**(o-1)+(-0.835086989851)
        ref[0,3,0]=o*(-0.256476478337)*x_ref[0]**(o-1)+(-0.857686365148)
        ref[0,3,1]=o*(-0.309306419219)*x_ref[1]**(o-1)+(0.750579112592)
        ref[0,3,2]=o*(0.758254577714)*x_ref[2]**(o-1)+(0.108972886739)
        ref[0,4,0]=o*(0.189540467853)*x_ref[0]**(o-1)+(-0.639906129587)
        ref[0,4,1]=o*(0.1086564431)*x_ref[1]**(o-1)+(-0.254754163159)
        ref[0,4,2]=o*(-0.595415956315)*x_ref[2]**(o-1)+(0.416643698623)
        ref[1,0,0]=o*(0.649640741159)*x_ref[0]**(o-1)+(-0.715604744977)
        ref[1,0,1]=o*(0.144698127223)*x_ref[1]**(o-1)+(0.84058008507)
        ref[1,0,2]=o*(0.00226738664623)*x_ref[2]**(o-1)+(0.143092105366)
        ref[1,1,0]=o*(0.30428097785)*x_ref[0]**(o-1)+(0.172630542192)
        ref[1,1,1]=o*(0.645118113075)*x_ref[1]**(o-1)+(-0.394468022223)
        ref[1,1,2]=o*(0.731670923192)*x_ref[2]**(o-1)+(-0.676021495379)
        ref[1,2,0]=o*(0.56459927889)*x_ref[0]**(o-1)+(-0.624980053566)
        ref[1,2,1]=o*(-0.524836958708)*x_ref[1]**(o-1)+(0.282862320474)
        ref[1,2,2]=o*(0.99070455422)*x_ref[2]**(o-1)+(0.000265746399286)
        ref[1,3,0]=o*(-0.251929813217)*x_ref[0]**(o-1)+(0.00685246758491)
        ref[1,3,1]=o*(-0.252595929082)*x_ref[1]**(o-1)+(-0.997558334417)
        ref[1,3,2]=o*(-0.431329146194)*x_ref[2]**(o-1)+(0.0685298495116)
        ref[1,4,0]=o*(-0.850398225165)*x_ref[0]**(o-1)+(-0.752363025861)
        ref[1,4,1]=o*(-0.669448553729)*x_ref[1]**(o-1)+(0.809247179374)
        ref[1,4,2]=o*(-0.927467807251)*x_ref[2]**(o-1)+(-0.498094160268)
        ref[2,0,0]=o*(0.541736859663)*x_ref[0]**(o-1)+(-0.651273876855)
        ref[2,0,1]=o*(-0.679670779769)*x_ref[1]**(o-1)+(0.669564679419)
        ref[2,0,2]=o*(-0.825048237017)*x_ref[2]**(o-1)+(-0.247744537755)
        ref[2,1,0]=o*(0.909016713138)*x_ref[0]**(o-1)+(-0.794124179572)
        ref[2,1,1]=o*(0.887294650854)*x_ref[1]**(o-1)+(-0.198933351435)
        ref[2,1,2]=o*(-0.819671062456)*x_ref[2]**(o-1)+(0.287455634612)
        ref[2,2,0]=o*(-0.469337238665)*x_ref[0]**(o-1)+(0.787828984218)
        ref[2,2,1]=o*(0.916042829702)*x_ref[1]**(o-1)+(-0.475983675757)
        ref[2,2,2]=o*(-0.350660319046)*x_ref[2]**(o-1)+(0.944222119413)
        ref[2,3,0]=o*(-0.194629035155)*x_ref[0]**(o-1)+(0.452058614199)
        ref[2,3,1]=o*(-0.565150768916)*x_ref[1]**(o-1)+(-0.967642852138)
        ref[2,3,2]=o*(-0.345332973443)*x_ref[2]**(o-1)+(-0.238359110833)
        ref[2,4,0]=o*(-0.744197369952)*x_ref[0]**(o-1)+(-0.0161450929106)
        ref[2,4,1]=o*(0.946555120156)*x_ref[1]**(o-1)+(-0.336314723492)
        ref[2,4,2]=o*(0.212936909808)*x_ref[2]**(o-1)+(-0.900582293569)
        ref[3,0,0]=o*(0.90736518985)*x_ref[0]**(o-1)+(-0.866096443075)
        ref[3,0,1]=o*(0.440227521532)*x_ref[1]**(o-1)+(-0.545983384676)
        ref[3,0,2]=o*(0.433167375756)*x_ref[2]**(o-1)+(-0.913060961228)
        ref[3,1,0]=o*(0.33547032327)*x_ref[0]**(o-1)+(0.766388218896)
        ref[3,1,1]=o*(-0.386141806114)*x_ref[1]**(o-1)+(0.28471049668)
        ref[3,1,2]=o*(0.389347155974)*x_ref[2]**(o-1)+(0.211663215285)
        ref[3,2,0]=o*(0.193757801099)*x_ref[0]**(o-1)+(0.0145439587225)
        ref[3,2,1]=o*(0.575506295918)*x_ref[1]**(o-1)+(-0.678317336021)
        ref[3,2,2]=o*(0.64603157052)*x_ref[2]**(o-1)+(-0.876667949279)
        ref[3,3,0]=o*(-0.0518002857411)*x_ref[0]**(o-1)+(-0.540021971724)
        ref[3,3,1]=o*(-0.490275636747)*x_ref[1]**(o-1)+(-0.849147647677)
        ref[3,3,2]=o*(-0.0132783508877)*x_ref[2]**(o-1)+(0.0859860768541)
        ref[3,4,0]=o*(-0.332675003425)*x_ref[0]**(o-1)+(-0.757986125506)
        ref[3,4,1]=o*(-0.487001539705)*x_ref[1]**(o-1)+(0.898881577697)
        ref[3,4,2]=o*(0.56557767007)*x_ref[2]**(o-1)+(0.181594113977)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_Solution_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.568278554396)*x[0]**o+(0.714635312506)*x[0]+(-0.121000061881)*x[1]**o+(0.650923087629)*x[1]
        arg[0,0,1]=(0.610070747198)*x[0]**o+(-0.17228586298)*x[0]+(-0.763207252556)*x[1]**o+(-0.417620736844)*x[1]
        arg[0,1,0]=(0.953437717333)*x[0]**o+(-0.877858891638)*x[0]+(-0.629855255168)*x[1]**o+(0.138168708132)*x[1]
        arg[0,1,1]=(-0.676346259275)*x[0]**o+(0.810400719592)*x[0]+(0.252517037374)*x[1]**o+(-0.0253025825974)*x[1]
        arg[1,0,0]=(0.0354009852553)*x[0]**o+(-0.930486981217)*x[0]+(0.515533244848)*x[1]**o+(0.39357313927)*x[1]
        arg[1,0,1]=(4.954730054e-05)*x[0]**o+(0.902841279458)*x[0]+(0.0974175050415)*x[1]**o+(-0.283224733511)*x[1]
        arg[1,1,0]=(-0.3695113121)*x[0]**o+(-0.425976895152)*x[0]+(-0.752863101988)*x[1]**o+(-0.135026977845)*x[1]
        arg[1,1,1]=(-0.627750038208)*x[0]**o+(-0.682825202306)*x[0]+(0.598327522753)*x[1]**o+(0.0402698525348)*x[1]
        arg[2,0,0]=(0.95982907811)*x[0]**o+(-0.786158711751)*x[0]+(0.6974717418)*x[1]**o+(-0.548248895713)*x[1]
        arg[2,0,1]=(-0.21063630362)*x[0]**o+(-0.979628903483)*x[0]+(0.514851452559)*x[1]**o+(-0.294218784425)*x[1]
        arg[2,1,0]=(-0.446140926476)*x[0]**o+(0.658918134389)*x[0]+(-0.166087456007)*x[1]**o+(0.328044805625)*x[1]
        arg[2,1,1]=(0.937161634589)*x[0]**o+(0.594322457074)*x[0]+(0.992336676032)*x[1]**o+(-0.499637100315)*x[1]
        arg[3,0,0]=(0.767677974163)*x[0]**o+(-0.820615663298)*x[0]+(0.558185245925)*x[1]**o+(-0.763525476816)*x[1]
        arg[3,0,1]=(-0.866581629552)*x[0]**o+(0.589993777037)*x[0]+(-0.0704817160146)*x[1]**o+(-0.897469280048)*x[1]
        arg[3,1,0]=(-0.290118027748)*x[0]**o+(0.211790364383)*x[0]+(-0.0688758465892)*x[1]**o+(0.777490292219)*x[1]
        arg[3,1,1]=(-0.969418722549)*x[0]**o+(-0.46884683812)*x[0]+(0.826696594862)*x[1]**o+(-0.926802811507)*x[1]
        arg[4,0,0]=(-0.230285931083)*x[0]**o+(0.905695021426)*x[0]+(-0.581906633624)*x[1]**o+(-0.69277147014)*x[1]
        arg[4,0,1]=(-0.251350536276)*x[0]**o+(0.352877501605)*x[0]+(0.290432460788)*x[1]**o+(0.614218150117)*x[1]
        arg[4,1,0]=(-0.475014635407)*x[0]**o+(0.86786497963)*x[0]+(-0.763349511499)*x[1]**o+(0.817847286727)*x[1]
        arg[4,1,1]=(0.0079711463132)*x[0]**o+(-0.20685339926)*x[0]+(0.377467915936)*x[1]**o+(-0.209268733968)*x[1]
        arg[5,0,0]=(0.795790630791)*x[0]**o+(0.0661201279192)*x[0]+(0.932665258262)*x[1]**o+(-0.4785884914)*x[1]
        arg[5,0,1]=(0.929422646068)*x[0]**o+(0.775378282324)*x[0]+(0.297482039582)*x[1]**o+(-0.700872789842)*x[1]
        arg[5,1,0]=(0.678688297204)*x[0]**o+(0.365975847321)*x[0]+(-0.565261107289)*x[1]**o+(0.0317556604619)*x[1]
        arg[5,1,1]=(-0.0231297032408)*x[0]**o+(-0.985865541949)*x[0]+(0.733821736669)*x[1]**o+(0.603255951212)*x[1]
        ref[0,0,0,0]=o*(0.568278554396)*x_ref[0]**(o-1)+(0.714635312506)
        ref[0,0,0,1]=o*(-0.121000061881)*x_ref[1]**(o-1)+(0.650923087629)
        ref[0,0,1,0]=o*(0.610070747198)*x_ref[0]**(o-1)+(-0.17228586298)
        ref[0,0,1,1]=o*(-0.763207252556)*x_ref[1]**(o-1)+(-0.417620736844)
        ref[0,1,0,0]=o*(0.953437717333)*x_ref[0]**(o-1)+(-0.877858891638)
        ref[0,1,0,1]=o*(-0.629855255168)*x_ref[1]**(o-1)+(0.138168708132)
        ref[0,1,1,0]=o*(-0.676346259275)*x_ref[0]**(o-1)+(0.810400719592)
        ref[0,1,1,1]=o*(0.252517037374)*x_ref[1]**(o-1)+(-0.0253025825974)
        ref[1,0,0,0]=o*(0.0354009852553)*x_ref[0]**(o-1)+(-0.930486981217)
        ref[1,0,0,1]=o*(0.515533244848)*x_ref[1]**(o-1)+(0.39357313927)
        ref[1,0,1,0]=o*(4.954730054e-05)*x_ref[0]**(o-1)+(0.902841279458)
        ref[1,0,1,1]=o*(0.0974175050415)*x_ref[1]**(o-1)+(-0.283224733511)
        ref[1,1,0,0]=o*(-0.3695113121)*x_ref[0]**(o-1)+(-0.425976895152)
        ref[1,1,0,1]=o*(-0.752863101988)*x_ref[1]**(o-1)+(-0.135026977845)
        ref[1,1,1,0]=o*(-0.627750038208)*x_ref[0]**(o-1)+(-0.682825202306)
        ref[1,1,1,1]=o*(0.598327522753)*x_ref[1]**(o-1)+(0.0402698525348)
        ref[2,0,0,0]=o*(0.95982907811)*x_ref[0]**(o-1)+(-0.786158711751)
        ref[2,0,0,1]=o*(0.6974717418)*x_ref[1]**(o-1)+(-0.548248895713)
        ref[2,0,1,0]=o*(-0.21063630362)*x_ref[0]**(o-1)+(-0.979628903483)
        ref[2,0,1,1]=o*(0.514851452559)*x_ref[1]**(o-1)+(-0.294218784425)
        ref[2,1,0,0]=o*(-0.446140926476)*x_ref[0]**(o-1)+(0.658918134389)
        ref[2,1,0,1]=o*(-0.166087456007)*x_ref[1]**(o-1)+(0.328044805625)
        ref[2,1,1,0]=o*(0.937161634589)*x_ref[0]**(o-1)+(0.594322457074)
        ref[2,1,1,1]=o*(0.992336676032)*x_ref[1]**(o-1)+(-0.499637100315)
        ref[3,0,0,0]=o*(0.767677974163)*x_ref[0]**(o-1)+(-0.820615663298)
        ref[3,0,0,1]=o*(0.558185245925)*x_ref[1]**(o-1)+(-0.763525476816)
        ref[3,0,1,0]=o*(-0.866581629552)*x_ref[0]**(o-1)+(0.589993777037)
        ref[3,0,1,1]=o*(-0.0704817160146)*x_ref[1]**(o-1)+(-0.897469280048)
        ref[3,1,0,0]=o*(-0.290118027748)*x_ref[0]**(o-1)+(0.211790364383)
        ref[3,1,0,1]=o*(-0.0688758465892)*x_ref[1]**(o-1)+(0.777490292219)
        ref[3,1,1,0]=o*(-0.969418722549)*x_ref[0]**(o-1)+(-0.46884683812)
        ref[3,1,1,1]=o*(0.826696594862)*x_ref[1]**(o-1)+(-0.926802811507)
        ref[4,0,0,0]=o*(-0.230285931083)*x_ref[0]**(o-1)+(0.905695021426)
        ref[4,0,0,1]=o*(-0.581906633624)*x_ref[1]**(o-1)+(-0.69277147014)
        ref[4,0,1,0]=o*(-0.251350536276)*x_ref[0]**(o-1)+(0.352877501605)
        ref[4,0,1,1]=o*(0.290432460788)*x_ref[1]**(o-1)+(0.614218150117)
        ref[4,1,0,0]=o*(-0.475014635407)*x_ref[0]**(o-1)+(0.86786497963)
        ref[4,1,0,1]=o*(-0.763349511499)*x_ref[1]**(o-1)+(0.817847286727)
        ref[4,1,1,0]=o*(0.0079711463132)*x_ref[0]**(o-1)+(-0.20685339926)
        ref[4,1,1,1]=o*(0.377467915936)*x_ref[1]**(o-1)+(-0.209268733968)
        ref[5,0,0,0]=o*(0.795790630791)*x_ref[0]**(o-1)+(0.0661201279192)
        ref[5,0,0,1]=o*(0.932665258262)*x_ref[1]**(o-1)+(-0.4785884914)
        ref[5,0,1,0]=o*(0.929422646068)*x_ref[0]**(o-1)+(0.775378282324)
        ref[5,0,1,1]=o*(0.297482039582)*x_ref[1]**(o-1)+(-0.700872789842)
        ref[5,1,0,0]=o*(0.678688297204)*x_ref[0]**(o-1)+(0.365975847321)
        ref[5,1,0,1]=o*(-0.565261107289)*x_ref[1]**(o-1)+(0.0317556604619)
        ref[5,1,1,0]=o*(-0.0231297032408)*x_ref[0]**(o-1)+(-0.985865541949)
        ref[5,1,1,1]=o*(0.733821736669)*x_ref[1]**(o-1)+(0.603255951212)
      else:
        arg[0,0,0]=(-0.211459759219)*x[0]**o+(-0.577684864446)*x[0]+(-0.391603126843)*x[1]**o+(0.260321825768)*x[1]+(-0.314038207978)*x[2]**o+(-0.981440041663)*x[2]
        arg[0,0,1]=(-0.876053365684)*x[0]**o+(-0.194899445893)*x[0]+(0.346950377505)*x[1]**o+(0.571865696578)*x[1]+(0.632901527485)*x[2]**o+(0.832258490904)*x[2]
        arg[0,1,0]=(-0.0779003885155)*x[0]**o+(0.583384314998)*x[0]+(-0.257262737021)*x[1]**o+(0.456730130081)*x[1]+(0.262812418811)*x[2]**o+(0.0815640391979)*x[2]
        arg[0,1,1]=(0.087484387098)*x[0]**o+(-0.415375968565)*x[0]+(0.31138114792)*x[1]**o+(0.832138685168)*x[1]+(0.552625436456)*x[2]**o+(-0.0969172211154)*x[2]
        arg[1,0,0]=(0.721326796741)*x[0]**o+(-0.556644260079)*x[0]+(0.14718294664)*x[1]**o+(0.688425309252)*x[1]+(-0.16480505764)*x[2]**o+(-0.914914176523)*x[2]
        arg[1,0,1]=(0.0932763269908)*x[0]**o+(0.359036469617)*x[0]+(0.568509073358)*x[1]**o+(-0.42133310351)*x[1]+(0.576833556667)*x[2]**o+(0.44466763296)*x[2]
        arg[1,1,0]=(0.564028751408)*x[0]**o+(-0.314840429631)*x[0]+(-0.910523044917)*x[1]**o+(-0.0377714078758)*x[1]+(0.0125836031881)*x[2]**o+(0.602273699833)*x[2]
        arg[1,1,1]=(0.135282471986)*x[0]**o+(0.163613895979)*x[0]+(0.115685034139)*x[1]**o+(-0.687078382924)*x[1]+(0.406385459521)*x[2]**o+(-0.401846794432)*x[2]
        arg[2,0,0]=(0.815695969791)*x[0]**o+(0.312205723135)*x[0]+(0.993404535838)*x[1]**o+(0.101946422158)*x[1]+(-0.331818829998)*x[2]**o+(-0.0034480605816)*x[2]
        arg[2,0,1]=(0.638646208966)*x[0]**o+(0.265838992948)*x[0]+(0.156594972273)*x[1]**o+(0.935292984535)*x[1]+(-0.930522319399)*x[2]**o+(0.540679813135)*x[2]
        arg[2,1,0]=(-0.778823826824)*x[0]**o+(0.435399707656)*x[0]+(0.123031900278)*x[1]**o+(-0.132556439303)*x[1]+(-0.29114590778)*x[2]**o+(-0.277636472779)*x[2]
        arg[2,1,1]=(-0.200541347963)*x[0]**o+(-0.526841282198)*x[0]+(0.670718267305)*x[1]**o+(0.0777161510281)*x[1]+(0.847335718393)*x[2]**o+(0.167185849891)*x[2]
        arg[3,0,0]=(0.481112023661)*x[0]**o+(-0.703558873662)*x[0]+(0.0661746324247)*x[1]**o+(0.0628113595423)*x[1]+(0.804110728888)*x[2]**o+(0.517297444342)*x[2]
        arg[3,0,1]=(-0.44648091284)*x[0]**o+(-0.976332452987)*x[0]+(-0.886297355627)*x[1]**o+(-0.910033293533)*x[1]+(0.946007442337)*x[2]**o+(-0.586006699603)*x[2]
        arg[3,1,0]=(0.686532047098)*x[0]**o+(0.647146535444)*x[0]+(-0.424413819624)*x[1]**o+(-0.266078497115)*x[1]+(-0.656118336723)*x[2]**o+(0.902145487914)*x[2]
        arg[3,1,1]=(-0.0562438759603)*x[0]**o+(0.600872472135)*x[0]+(-0.162016187788)*x[1]**o+(-0.685387926)*x[1]+(-0.899318041513)*x[2]**o+(0.459743174318)*x[2]
        arg[4,0,0]=(-0.923373905806)*x[0]**o+(-0.0340303642214)*x[0]+(-0.798971449517)*x[1]**o+(-0.45521271974)*x[1]+(-0.354501811953)*x[2]**o+(0.0668579007274)*x[2]
        arg[4,0,1]=(0.411397784181)*x[0]**o+(0.112609440102)*x[0]+(0.352899973694)*x[1]**o+(-0.0728024007714)*x[1]+(0.50804391825)*x[2]**o+(0.748548958571)*x[2]
        arg[4,1,0]=(0.875890044031)*x[0]**o+(0.674204747948)*x[0]+(-0.879286517401)*x[1]**o+(-0.00338161978492)*x[1]+(-0.372790489717)*x[2]**o+(0.480822881409)*x[2]
        arg[4,1,1]=(-0.224827091498)*x[0]**o+(0.854170572245)*x[0]+(-0.670467931784)*x[1]**o+(0.85225464592)*x[1]+(0.41885719338)*x[2]**o+(-0.623296073831)*x[2]
        arg[5,0,0]=(-0.662704253172)*x[0]**o+(0.690656552406)*x[0]+(-0.812436958497)*x[1]**o+(0.928013493065)*x[1]+(0.763792541097)*x[2]**o+(-0.430084528987)*x[2]
        arg[5,0,1]=(0.451167682598)*x[0]**o+(0.373015986015)*x[0]+(0.73334610852)*x[1]**o+(0.44892130407)*x[1]+(0.820233215547)*x[2]**o+(0.863864357936)*x[2]
        arg[5,1,0]=(-0.700840074185)*x[0]**o+(-0.637964737375)*x[0]+(0.0532062256138)*x[1]**o+(-0.902662868597)*x[1]+(-0.91668008633)*x[2]**o+(0.00253116545509)*x[2]
        arg[5,1,1]=(0.0461834200484)*x[0]**o+(0.102406795193)*x[0]+(0.599142154957)*x[1]**o+(0.374585801545)*x[1]+(-0.444968156109)*x[2]**o+(-0.932860816729)*x[2]
        ref[0,0,0,0]=o*(-0.211459759219)*x_ref[0]**(o-1)+(-0.577684864446)
        ref[0,0,0,1]=o*(-0.391603126843)*x_ref[1]**(o-1)+(0.260321825768)
        ref[0,0,0,2]=o*(-0.314038207978)*x_ref[2]**(o-1)+(-0.981440041663)
        ref[0,0,1,0]=o*(-0.876053365684)*x_ref[0]**(o-1)+(-0.194899445893)
        ref[0,0,1,1]=o*(0.346950377505)*x_ref[1]**(o-1)+(0.571865696578)
        ref[0,0,1,2]=o*(0.632901527485)*x_ref[2]**(o-1)+(0.832258490904)
        ref[0,1,0,0]=o*(-0.0779003885155)*x_ref[0]**(o-1)+(0.583384314998)
        ref[0,1,0,1]=o*(-0.257262737021)*x_ref[1]**(o-1)+(0.456730130081)
        ref[0,1,0,2]=o*(0.262812418811)*x_ref[2]**(o-1)+(0.0815640391979)
        ref[0,1,1,0]=o*(0.087484387098)*x_ref[0]**(o-1)+(-0.415375968565)
        ref[0,1,1,1]=o*(0.31138114792)*x_ref[1]**(o-1)+(0.832138685168)
        ref[0,1,1,2]=o*(0.552625436456)*x_ref[2]**(o-1)+(-0.0969172211154)
        ref[1,0,0,0]=o*(0.721326796741)*x_ref[0]**(o-1)+(-0.556644260079)
        ref[1,0,0,1]=o*(0.14718294664)*x_ref[1]**(o-1)+(0.688425309252)
        ref[1,0,0,2]=o*(-0.16480505764)*x_ref[2]**(o-1)+(-0.914914176523)
        ref[1,0,1,0]=o*(0.0932763269908)*x_ref[0]**(o-1)+(0.359036469617)
        ref[1,0,1,1]=o*(0.568509073358)*x_ref[1]**(o-1)+(-0.42133310351)
        ref[1,0,1,2]=o*(0.576833556667)*x_ref[2]**(o-1)+(0.44466763296)
        ref[1,1,0,0]=o*(0.564028751408)*x_ref[0]**(o-1)+(-0.314840429631)
        ref[1,1,0,1]=o*(-0.910523044917)*x_ref[1]**(o-1)+(-0.0377714078758)
        ref[1,1,0,2]=o*(0.0125836031881)*x_ref[2]**(o-1)+(0.602273699833)
        ref[1,1,1,0]=o*(0.135282471986)*x_ref[0]**(o-1)+(0.163613895979)
        ref[1,1,1,1]=o*(0.115685034139)*x_ref[1]**(o-1)+(-0.687078382924)
        ref[1,1,1,2]=o*(0.406385459521)*x_ref[2]**(o-1)+(-0.401846794432)
        ref[2,0,0,0]=o*(0.815695969791)*x_ref[0]**(o-1)+(0.312205723135)
        ref[2,0,0,1]=o*(0.993404535838)*x_ref[1]**(o-1)+(0.101946422158)
        ref[2,0,0,2]=o*(-0.331818829998)*x_ref[2]**(o-1)+(-0.0034480605816)
        ref[2,0,1,0]=o*(0.638646208966)*x_ref[0]**(o-1)+(0.265838992948)
        ref[2,0,1,1]=o*(0.156594972273)*x_ref[1]**(o-1)+(0.935292984535)
        ref[2,0,1,2]=o*(-0.930522319399)*x_ref[2]**(o-1)+(0.540679813135)
        ref[2,1,0,0]=o*(-0.778823826824)*x_ref[0]**(o-1)+(0.435399707656)
        ref[2,1,0,1]=o*(0.123031900278)*x_ref[1]**(o-1)+(-0.132556439303)
        ref[2,1,0,2]=o*(-0.29114590778)*x_ref[2]**(o-1)+(-0.277636472779)
        ref[2,1,1,0]=o*(-0.200541347963)*x_ref[0]**(o-1)+(-0.526841282198)
        ref[2,1,1,1]=o*(0.670718267305)*x_ref[1]**(o-1)+(0.0777161510281)
        ref[2,1,1,2]=o*(0.847335718393)*x_ref[2]**(o-1)+(0.167185849891)
        ref[3,0,0,0]=o*(0.481112023661)*x_ref[0]**(o-1)+(-0.703558873662)
        ref[3,0,0,1]=o*(0.0661746324247)*x_ref[1]**(o-1)+(0.0628113595423)
        ref[3,0,0,2]=o*(0.804110728888)*x_ref[2]**(o-1)+(0.517297444342)
        ref[3,0,1,0]=o*(-0.44648091284)*x_ref[0]**(o-1)+(-0.976332452987)
        ref[3,0,1,1]=o*(-0.886297355627)*x_ref[1]**(o-1)+(-0.910033293533)
        ref[3,0,1,2]=o*(0.946007442337)*x_ref[2]**(o-1)+(-0.586006699603)
        ref[3,1,0,0]=o*(0.686532047098)*x_ref[0]**(o-1)+(0.647146535444)
        ref[3,1,0,1]=o*(-0.424413819624)*x_ref[1]**(o-1)+(-0.266078497115)
        ref[3,1,0,2]=o*(-0.656118336723)*x_ref[2]**(o-1)+(0.902145487914)
        ref[3,1,1,0]=o*(-0.0562438759603)*x_ref[0]**(o-1)+(0.600872472135)
        ref[3,1,1,1]=o*(-0.162016187788)*x_ref[1]**(o-1)+(-0.685387926)
        ref[3,1,1,2]=o*(-0.899318041513)*x_ref[2]**(o-1)+(0.459743174318)
        ref[4,0,0,0]=o*(-0.923373905806)*x_ref[0]**(o-1)+(-0.0340303642214)
        ref[4,0,0,1]=o*(-0.798971449517)*x_ref[1]**(o-1)+(-0.45521271974)
        ref[4,0,0,2]=o*(-0.354501811953)*x_ref[2]**(o-1)+(0.0668579007274)
        ref[4,0,1,0]=o*(0.411397784181)*x_ref[0]**(o-1)+(0.112609440102)
        ref[4,0,1,1]=o*(0.352899973694)*x_ref[1]**(o-1)+(-0.0728024007714)
        ref[4,0,1,2]=o*(0.50804391825)*x_ref[2]**(o-1)+(0.748548958571)
        ref[4,1,0,0]=o*(0.875890044031)*x_ref[0]**(o-1)+(0.674204747948)
        ref[4,1,0,1]=o*(-0.879286517401)*x_ref[1]**(o-1)+(-0.00338161978492)
        ref[4,1,0,2]=o*(-0.372790489717)*x_ref[2]**(o-1)+(0.480822881409)
        ref[4,1,1,0]=o*(-0.224827091498)*x_ref[0]**(o-1)+(0.854170572245)
        ref[4,1,1,1]=o*(-0.670467931784)*x_ref[1]**(o-1)+(0.85225464592)
        ref[4,1,1,2]=o*(0.41885719338)*x_ref[2]**(o-1)+(-0.623296073831)
        ref[5,0,0,0]=o*(-0.662704253172)*x_ref[0]**(o-1)+(0.690656552406)
        ref[5,0,0,1]=o*(-0.812436958497)*x_ref[1]**(o-1)+(0.928013493065)
        ref[5,0,0,2]=o*(0.763792541097)*x_ref[2]**(o-1)+(-0.430084528987)
        ref[5,0,1,0]=o*(0.451167682598)*x_ref[0]**(o-1)+(0.373015986015)
        ref[5,0,1,1]=o*(0.73334610852)*x_ref[1]**(o-1)+(0.44892130407)
        ref[5,0,1,2]=o*(0.820233215547)*x_ref[2]**(o-1)+(0.863864357936)
        ref[5,1,0,0]=o*(-0.700840074185)*x_ref[0]**(o-1)+(-0.637964737375)
        ref[5,1,0,1]=o*(0.0532062256138)*x_ref[1]**(o-1)+(-0.902662868597)
        ref[5,1,0,2]=o*(-0.91668008633)*x_ref[2]**(o-1)+(0.00253116545509)
        ref[5,1,1,0]=o*(0.0461834200484)*x_ref[0]**(o-1)+(0.102406795193)
        ref[5,1,1,1]=o*(0.599142154957)*x_ref[1]**(o-1)+(0.374585801545)
        ref[5,1,1,2]=o*(-0.444968156109)*x_ref[2]**(o-1)+(-0.932860816729)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(1.37238770251)*x[0]+(-1.43917937019)*x[1]
        ref[0]=(1.37238770251)
        ref[1]=(-1.43917937019)
      else:
        arg=(0.951563810847)*x[0]+(-1.91408943437)*x[1]+(0.233861671105)*x[2]
        ref[0]=(0.951563810847)
        ref[1]=(-1.91408943437)
        ref[2]=(0.233861671105)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(1.42921344972)*x[0]+(-0.911436505176)*x[1]
        arg[1]=(1.25688300316)*x[0]+(0.0536640238292)*x[1]
        ref[0,0]=(1.42921344972)
        ref[0,1]=(-0.911436505176)
        ref[1,0]=(1.25688300316)
        ref[1,1]=(0.0536640238292)
      else:
        arg[0]=(-1.57773112011)*x[0]+(-0.697311790617)*x[1]+(-0.480770363787)*x[2]
        arg[1]=(-0.136063655594)*x[0]+(0.882253548646)*x[1]+(1.35773570995)*x[2]
        ref[0,0]=(-1.57773112011)
        ref[0,1]=(-0.697311790617)
        ref[0,2]=(-0.480770363787)
        ref[1,0]=(-0.136063655594)
        ref[1,1]=(0.882253548646)
        ref[1,2]=(1.35773570995)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.448247291087)*x[0]+(-0.00880442943882)*x[1]
        arg[0,1]=(-0.59784234455)*x[0]+(-0.94951606979)*x[1]
        arg[0,2]=(1.38316656933)*x[0]+(-0.124817047005)*x[1]
        arg[0,3]=(-0.0748469893414)*x[0]+(1.44145142047)*x[1]
        arg[0,4]=(1.81133385456)*x[0]+(-0.518406894403)*x[1]
        arg[1,0]=(-0.65460152624)*x[0]+(-0.350858526434)*x[1]
        arg[1,1]=(0.924058009558)*x[0]+(-0.40973995895)*x[1]
        arg[1,2]=(-0.299794705178)*x[0]+(-0.924557696915)*x[1]
        arg[1,3]=(-0.608513627169)*x[0]+(-0.141936714389)*x[1]
        arg[1,4]=(0.469099778997)*x[0]+(0.754132353456)*x[1]
        arg[2,0]=(-0.627098551951)*x[0]+(0.476865448119)*x[1]
        arg[2,1]=(-1.03564375134)*x[0]+(-0.280009187543)*x[1]
        arg[2,2]=(0.475083576156)*x[0]+(0.220248066616)*x[1]
        arg[2,3]=(0.0420499398173)*x[0]+(0.0808105284548)*x[1]
        arg[2,4]=(0.368173655662)*x[0]+(-0.58266909961)*x[1]
        arg[3,0]=(0.450851110298)*x[0]+(0.0844590226087)*x[1]
        arg[3,1]=(0.508010554654)*x[0]+(0.797109833792)*x[1]
        arg[3,2]=(0.711479388001)*x[0]+(1.12829906729)*x[1]
        arg[3,3]=(0.0588799697153)*x[0]+(-0.991874826409)*x[1]
        arg[3,4]=(0.743446045536)*x[0]+(0.65754450588)*x[1]
        ref[0,0,0]=(0.448247291087)
        ref[0,0,1]=(-0.00880442943882)
        ref[0,1,0]=(-0.59784234455)
        ref[0,1,1]=(-0.94951606979)
        ref[0,2,0]=(1.38316656933)
        ref[0,2,1]=(-0.124817047005)
        ref[0,3,0]=(-0.0748469893414)
        ref[0,3,1]=(1.44145142047)
        ref[0,4,0]=(1.81133385456)
        ref[0,4,1]=(-0.518406894403)
        ref[1,0,0]=(-0.65460152624)
        ref[1,0,1]=(-0.350858526434)
        ref[1,1,0]=(0.924058009558)
        ref[1,1,1]=(-0.40973995895)
        ref[1,2,0]=(-0.299794705178)
        ref[1,2,1]=(-0.924557696915)
        ref[1,3,0]=(-0.608513627169)
        ref[1,3,1]=(-0.141936714389)
        ref[1,4,0]=(0.469099778997)
        ref[1,4,1]=(0.754132353456)
        ref[2,0,0]=(-0.627098551951)
        ref[2,0,1]=(0.476865448119)
        ref[2,1,0]=(-1.03564375134)
        ref[2,1,1]=(-0.280009187543)
        ref[2,2,0]=(0.475083576156)
        ref[2,2,1]=(0.220248066616)
        ref[2,3,0]=(0.0420499398173)
        ref[2,3,1]=(0.0808105284548)
        ref[2,4,0]=(0.368173655662)
        ref[2,4,1]=(-0.58266909961)
        ref[3,0,0]=(0.450851110298)
        ref[3,0,1]=(0.0844590226087)
        ref[3,1,0]=(0.508010554654)
        ref[3,1,1]=(0.797109833792)
        ref[3,2,0]=(0.711479388001)
        ref[3,2,1]=(1.12829906729)
        ref[3,3,0]=(0.0588799697153)
        ref[3,3,1]=(-0.991874826409)
        ref[3,4,0]=(0.743446045536)
        ref[3,4,1]=(0.65754450588)
      else:
        arg[0,0]=(-0.43386387868)*x[0]+(0.321105491776)*x[1]+(0.384206679414)*x[2]
        arg[0,1]=(0.604698878149)*x[0]+(0.594375439325)*x[1]+(0.162832272176)*x[2]
        arg[0,2]=(-0.332194829416)*x[0]+(0.600821281937)*x[1]+(0.493622302813)*x[2]
        arg[0,3]=(0.846367268698)*x[0]+(-1.32744832381)*x[1]+(0.00466654421944)*x[2]
        arg[0,4]=(0.385111950936)*x[0]+(-0.176705693397)*x[1]+(0.461732370075)*x[2]
        arg[1,0]=(-0.14529643778)*x[0]+(1.15329219838)*x[1]+(0.157842247326)*x[2]
        arg[1,1]=(0.00817149012591)*x[0]+(-0.703898404732)*x[1]+(-1.14419500663)*x[2]
        arg[1,2]=(-0.124155034423)*x[0]+(0.906000496919)*x[1]+(-0.466071596735)*x[2]
        arg[1,3]=(0.413190740147)*x[0]+(0.968639404637)*x[1]+(-1.25413312698)*x[2]
        arg[1,4]=(0.521974966379)*x[0]+(0.936384991187)*x[1]+(-0.0630547476157)*x[2]
        arg[2,0]=(-0.334976510608)*x[0]+(-0.268364721371)*x[1]+(0.806974769679)*x[2]
        arg[2,1]=(-0.0672705162033)*x[0]+(0.459808873839)*x[1]+(0.0725634327358)*x[2]
        arg[2,2]=(0.142418012401)*x[0]+(-0.400301432498)*x[1]+(-1.37021128371)*x[2]
        arg[2,3]=(-1.60617980561)*x[0]+(0.0686943218654)*x[1]+(0.647509287658)*x[2]
        arg[2,4]=(0.0102366197695)*x[0]+(-1.24104511219)*x[1]+(-0.739463479584)*x[2]
        arg[3,0]=(-0.425526939568)*x[0]+(-0.946421307227)*x[1]+(0.133101457377)*x[2]
        arg[3,1]=(1.07183638389)*x[0]+(-1.21405997903)*x[1]+(-0.962984008606)*x[2]
        arg[3,2]=(-0.913456686897)*x[0]+(0.907233128077)*x[1]+(0.581609247137)*x[2]
        arg[3,3]=(-0.729544885457)*x[0]+(-0.433435638004)*x[1]+(0.357011166494)*x[2]
        arg[3,4]=(1.21906931308)*x[0]+(0.322487716786)*x[1]+(1.05602783674)*x[2]
        ref[0,0,0]=(-0.43386387868)
        ref[0,0,1]=(0.321105491776)
        ref[0,0,2]=(0.384206679414)
        ref[0,1,0]=(0.604698878149)
        ref[0,1,1]=(0.594375439325)
        ref[0,1,2]=(0.162832272176)
        ref[0,2,0]=(-0.332194829416)
        ref[0,2,1]=(0.600821281937)
        ref[0,2,2]=(0.493622302813)
        ref[0,3,0]=(0.846367268698)
        ref[0,3,1]=(-1.32744832381)
        ref[0,3,2]=(0.00466654421944)
        ref[0,4,0]=(0.385111950936)
        ref[0,4,1]=(-0.176705693397)
        ref[0,4,2]=(0.461732370075)
        ref[1,0,0]=(-0.14529643778)
        ref[1,0,1]=(1.15329219838)
        ref[1,0,2]=(0.157842247326)
        ref[1,1,0]=(0.00817149012591)
        ref[1,1,1]=(-0.703898404732)
        ref[1,1,2]=(-1.14419500663)
        ref[1,2,0]=(-0.124155034423)
        ref[1,2,1]=(0.906000496919)
        ref[1,2,2]=(-0.466071596735)
        ref[1,3,0]=(0.413190740147)
        ref[1,3,1]=(0.968639404637)
        ref[1,3,2]=(-1.25413312698)
        ref[1,4,0]=(0.521974966379)
        ref[1,4,1]=(0.936384991187)
        ref[1,4,2]=(-0.0630547476157)
        ref[2,0,0]=(-0.334976510608)
        ref[2,0,1]=(-0.268364721371)
        ref[2,0,2]=(0.806974769679)
        ref[2,1,0]=(-0.0672705162033)
        ref[2,1,1]=(0.459808873839)
        ref[2,1,2]=(0.0725634327358)
        ref[2,2,0]=(0.142418012401)
        ref[2,2,1]=(-0.400301432498)
        ref[2,2,2]=(-1.37021128371)
        ref[2,3,0]=(-1.60617980561)
        ref[2,3,1]=(0.0686943218654)
        ref[2,3,2]=(0.647509287658)
        ref[2,4,0]=(0.0102366197695)
        ref[2,4,1]=(-1.24104511219)
        ref[2,4,2]=(-0.739463479584)
        ref[3,0,0]=(-0.425526939568)
        ref[3,0,1]=(-0.946421307227)
        ref[3,0,2]=(0.133101457377)
        ref[3,1,0]=(1.07183638389)
        ref[3,1,1]=(-1.21405997903)
        ref[3,1,2]=(-0.962984008606)
        ref[3,2,0]=(-0.913456686897)
        ref[3,2,1]=(0.907233128077)
        ref[3,2,2]=(0.581609247137)
        ref[3,3,0]=(-0.729544885457)
        ref[3,3,1]=(-0.433435638004)
        ref[3,3,2]=(0.357011166494)
        ref[3,4,0]=(1.21906931308)
        ref[3,4,1]=(0.322487716786)
        ref[3,4,2]=(1.05602783674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.287546203823)*x[0]+(0.390321729967)*x[1]
        arg[0,0,1]=(-0.112653571553)*x[0]+(-0.359753186922)*x[1]
        arg[0,1,0]=(0.00656985674969)*x[0]+(-0.397210304393)*x[1]
        arg[0,1,1]=(-0.825698431533)*x[0]+(-1.20994477496)*x[1]
        arg[1,0,0]=(0.0456789352898)*x[0]+(0.201178709355)*x[1]
        arg[1,0,1]=(0.216619650199)*x[0]+(-0.684237394127)*x[1]
        arg[1,1,0]=(-0.452605086273)*x[0]+(0.383220784575)*x[1]
        arg[1,1,1]=(-0.385315869703)*x[0]+(-1.25937320051)*x[1]
        arg[2,0,0]=(-0.546290546777)*x[0]+(0.0659124729108)*x[1]
        arg[2,0,1]=(0.23413834434)*x[0]+(0.241510050424)*x[1]
        arg[2,1,0]=(-0.541097368124)*x[0]+(0.458682016611)*x[1]
        arg[2,1,1]=(0.314030173817)*x[0]+(0.083613884286)*x[1]
        arg[3,0,0]=(0.862005271666)*x[0]+(0.858693755091)*x[1]
        arg[3,0,1]=(-1.14220805291)*x[0]+(0.609875693913)*x[1]
        arg[3,1,0]=(-0.847069360014)*x[0]+(0.240069314456)*x[1]
        arg[3,1,1]=(0.480596211718)*x[0]+(-1.40901297384)*x[1]
        arg[4,0,0]=(-1.38036653523)*x[0]+(-0.0732955000278)*x[1]
        arg[4,0,1]=(0.0716061704803)*x[0]+(0.180127379412)*x[1]
        arg[4,1,0]=(1.26959959862)*x[0]+(-1.38131896436)*x[1]
        arg[4,1,1]=(0.204280361123)*x[0]+(0.532611514369)*x[1]
        arg[5,0,0]=(0.209536293545)*x[0]+(-0.303994906788)*x[1]
        arg[5,0,1]=(1.41150162289)*x[0]+(1.32515735407)*x[1]
        arg[5,1,0]=(1.59736027384)*x[0]+(-0.711055564868)*x[1]
        arg[5,1,1]=(-1.31799658024)*x[0]+(0.829209493108)*x[1]
        ref[0,0,0,0]=(-0.287546203823)
        ref[0,0,0,1]=(0.390321729967)
        ref[0,0,1,0]=(-0.112653571553)
        ref[0,0,1,1]=(-0.359753186922)
        ref[0,1,0,0]=(0.00656985674969)
        ref[0,1,0,1]=(-0.397210304393)
        ref[0,1,1,0]=(-0.825698431533)
        ref[0,1,1,1]=(-1.20994477496)
        ref[1,0,0,0]=(0.0456789352898)
        ref[1,0,0,1]=(0.201178709355)
        ref[1,0,1,0]=(0.216619650199)
        ref[1,0,1,1]=(-0.684237394127)
        ref[1,1,0,0]=(-0.452605086273)
        ref[1,1,0,1]=(0.383220784575)
        ref[1,1,1,0]=(-0.385315869703)
        ref[1,1,1,1]=(-1.25937320051)
        ref[2,0,0,0]=(-0.546290546777)
        ref[2,0,0,1]=(0.0659124729108)
        ref[2,0,1,0]=(0.23413834434)
        ref[2,0,1,1]=(0.241510050424)
        ref[2,1,0,0]=(-0.541097368124)
        ref[2,1,0,1]=(0.458682016611)
        ref[2,1,1,0]=(0.314030173817)
        ref[2,1,1,1]=(0.083613884286)
        ref[3,0,0,0]=(0.862005271666)
        ref[3,0,0,1]=(0.858693755091)
        ref[3,0,1,0]=(-1.14220805291)
        ref[3,0,1,1]=(0.609875693913)
        ref[3,1,0,0]=(-0.847069360014)
        ref[3,1,0,1]=(0.240069314456)
        ref[3,1,1,0]=(0.480596211718)
        ref[3,1,1,1]=(-1.40901297384)
        ref[4,0,0,0]=(-1.38036653523)
        ref[4,0,0,1]=(-0.0732955000278)
        ref[4,0,1,0]=(0.0716061704803)
        ref[4,0,1,1]=(0.180127379412)
        ref[4,1,0,0]=(1.26959959862)
        ref[4,1,0,1]=(-1.38131896436)
        ref[4,1,1,0]=(0.204280361123)
        ref[4,1,1,1]=(0.532611514369)
        ref[5,0,0,0]=(0.209536293545)
        ref[5,0,0,1]=(-0.303994906788)
        ref[5,0,1,0]=(1.41150162289)
        ref[5,0,1,1]=(1.32515735407)
        ref[5,1,0,0]=(1.59736027384)
        ref[5,1,0,1]=(-0.711055564868)
        ref[5,1,1,0]=(-1.31799658024)
        ref[5,1,1,1]=(0.829209493108)
      else:
        arg[0,0,0]=(0.832267408258)*x[0]+(-1.1995239623)*x[1]+(-1.81276973398)*x[2]
        arg[0,0,1]=(-0.0125232856194)*x[0]+(0.122543918666)*x[1]+(-1.12411286836)*x[2]
        arg[0,1,0]=(0.985611388383)*x[0]+(-1.17217207383)*x[1]+(-0.231241872108)*x[2]
        arg[0,1,1]=(0.625494139449)*x[0]+(0.784802683192)*x[1]+(0.612764525973)*x[2]
        arg[1,0,0]=(0.327644975022)*x[0]+(0.0552951801276)*x[1]+(-0.806887445103)*x[2]
        arg[1,0,1]=(0.194338798022)*x[0]+(0.799902026341)*x[1]+(1.70164249741)*x[2]
        arg[1,1,0]=(0.890832636865)*x[0]+(0.223899365292)*x[1]+(-0.283094526824)*x[2]
        arg[1,1,1]=(-0.122108330088)*x[0]+(-0.293413695547)*x[1]+(-0.371581852879)*x[2]
        arg[2,0,0]=(-0.469341678297)*x[0]+(0.152856292978)*x[1]+(0.765861314481)*x[2]
        arg[2,0,1]=(0.249219162734)*x[0]+(0.319622173697)*x[1]+(-0.182998471546)*x[2]
        arg[2,1,0]=(0.39687496584)*x[0]+(0.137084411978)*x[1]+(-0.34707250854)*x[2]
        arg[2,1,1]=(-0.382391798519)*x[0]+(0.755317411547)*x[1]+(0.424204162929)*x[2]
        arg[3,0,0]=(1.12637976255)*x[0]+(0.11632178776)*x[1]+(-0.615547388072)*x[2]
        arg[3,0,1]=(-0.819303259446)*x[0]+(0.916951216368)*x[1]+(-0.572792050441)*x[2]
        arg[3,1,0]=(0.194642055711)*x[0]+(0.949663114037)*x[1]+(-0.403001914682)*x[2]
        arg[3,1,1]=(0.902547242635)*x[0]+(-1.04027936291)*x[1]+(-1.78463515221)*x[2]
        arg[4,0,0]=(0.59060036737)*x[0]+(-0.580905638676)*x[1]+(-0.486098556884)*x[2]
        arg[4,0,1]=(-0.330634605524)*x[0]+(-0.758140973629)*x[1]+(-0.192282397672)*x[2]
        arg[4,1,0]=(1.40502801336)*x[0]+(0.199036161455)*x[1]+(-0.558660947885)*x[2]
        arg[4,1,1]=(-0.155588274273)*x[0]+(-0.57050736529)*x[1]+(1.1998664442)*x[2]
        arg[5,0,0]=(-0.754680865929)*x[0]+(1.29040984028)*x[1]+(-1.83778974463)*x[2]
        arg[5,0,1]=(0.274692501647)*x[0]+(0.368546080921)*x[1]+(-0.446376269232)*x[2]
        arg[5,1,0]=(-0.758704372861)*x[0]+(1.75555227426)*x[1]+(-0.13664688871)*x[2]
        arg[5,1,1]=(-0.568468068187)*x[0]+(-0.320218561544)*x[1]+(-0.827795466031)*x[2]
        ref[0,0,0,0]=(0.832267408258)
        ref[0,0,0,1]=(-1.1995239623)
        ref[0,0,0,2]=(-1.81276973398)
        ref[0,0,1,0]=(-0.0125232856194)
        ref[0,0,1,1]=(0.122543918666)
        ref[0,0,1,2]=(-1.12411286836)
        ref[0,1,0,0]=(0.985611388383)
        ref[0,1,0,1]=(-1.17217207383)
        ref[0,1,0,2]=(-0.231241872108)
        ref[0,1,1,0]=(0.625494139449)
        ref[0,1,1,1]=(0.784802683192)
        ref[0,1,1,2]=(0.612764525973)
        ref[1,0,0,0]=(0.327644975022)
        ref[1,0,0,1]=(0.0552951801276)
        ref[1,0,0,2]=(-0.806887445103)
        ref[1,0,1,0]=(0.194338798022)
        ref[1,0,1,1]=(0.799902026341)
        ref[1,0,1,2]=(1.70164249741)
        ref[1,1,0,0]=(0.890832636865)
        ref[1,1,0,1]=(0.223899365292)
        ref[1,1,0,2]=(-0.283094526824)
        ref[1,1,1,0]=(-0.122108330088)
        ref[1,1,1,1]=(-0.293413695547)
        ref[1,1,1,2]=(-0.371581852879)
        ref[2,0,0,0]=(-0.469341678297)
        ref[2,0,0,1]=(0.152856292978)
        ref[2,0,0,2]=(0.765861314481)
        ref[2,0,1,0]=(0.249219162734)
        ref[2,0,1,1]=(0.319622173697)
        ref[2,0,1,2]=(-0.182998471546)
        ref[2,1,0,0]=(0.39687496584)
        ref[2,1,0,1]=(0.137084411978)
        ref[2,1,0,2]=(-0.34707250854)
        ref[2,1,1,0]=(-0.382391798519)
        ref[2,1,1,1]=(0.755317411547)
        ref[2,1,1,2]=(0.424204162929)
        ref[3,0,0,0]=(1.12637976255)
        ref[3,0,0,1]=(0.11632178776)
        ref[3,0,0,2]=(-0.615547388072)
        ref[3,0,1,0]=(-0.819303259446)
        ref[3,0,1,1]=(0.916951216368)
        ref[3,0,1,2]=(-0.572792050441)
        ref[3,1,0,0]=(0.194642055711)
        ref[3,1,0,1]=(0.949663114037)
        ref[3,1,0,2]=(-0.403001914682)
        ref[3,1,1,0]=(0.902547242635)
        ref[3,1,1,1]=(-1.04027936291)
        ref[3,1,1,2]=(-1.78463515221)
        ref[4,0,0,0]=(0.59060036737)
        ref[4,0,0,1]=(-0.580905638676)
        ref[4,0,0,2]=(-0.486098556884)
        ref[4,0,1,0]=(-0.330634605524)
        ref[4,0,1,1]=(-0.758140973629)
        ref[4,0,1,2]=(-0.192282397672)
        ref[4,1,0,0]=(1.40502801336)
        ref[4,1,0,1]=(0.199036161455)
        ref[4,1,0,2]=(-0.558660947885)
        ref[4,1,1,0]=(-0.155588274273)
        ref[4,1,1,1]=(-0.57050736529)
        ref[4,1,1,2]=(1.1998664442)
        ref[5,0,0,0]=(-0.754680865929)
        ref[5,0,0,1]=(1.29040984028)
        ref[5,0,0,2]=(-1.83778974463)
        ref[5,0,1,0]=(0.274692501647)
        ref[5,0,1,1]=(0.368546080921)
        ref[5,0,1,2]=(-0.446376269232)
        ref[5,1,0,0]=(-0.758704372861)
        ref[5,1,0,1]=(1.75555227426)
        ref[5,1,0,2]=(-0.13664688871)
        ref[5,1,1,0]=(-0.568468068187)
        ref[5,1,1,1]=(-0.320218561544)
        ref[5,1,1,2]=(-0.827795466031)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(1.37238770251)*x[0]+(-1.43917937019)*x[1]
        ref[0]=(1.37238770251)
        ref[1]=(-1.43917937019)
      else:
        arg=(0.951563810847)*x[0]+(-1.91408943437)*x[1]+(0.233861671105)*x[2]
        ref[0]=(0.951563810847)
        ref[1]=(-1.91408943437)
        ref[2]=(0.233861671105)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(1.42921344972)*x[0]+(-0.911436505176)*x[1]
        arg[1]=(1.25688300316)*x[0]+(0.0536640238292)*x[1]
        ref[0,0]=(1.42921344972)
        ref[0,1]=(-0.911436505176)
        ref[1,0]=(1.25688300316)
        ref[1,1]=(0.0536640238292)
      else:
        arg[0]=(-1.57773112011)*x[0]+(-0.697311790617)*x[1]+(-0.480770363787)*x[2]
        arg[1]=(-0.136063655594)*x[0]+(0.882253548646)*x[1]+(1.35773570995)*x[2]
        ref[0,0]=(-1.57773112011)
        ref[0,1]=(-0.697311790617)
        ref[0,2]=(-0.480770363787)
        ref[1,0]=(-0.136063655594)
        ref[1,1]=(0.882253548646)
        ref[1,2]=(1.35773570995)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.448247291087)*x[0]+(-0.00880442943882)*x[1]
        arg[0,1]=(-0.59784234455)*x[0]+(-0.94951606979)*x[1]
        arg[0,2]=(1.38316656933)*x[0]+(-0.124817047005)*x[1]
        arg[0,3]=(-0.0748469893414)*x[0]+(1.44145142047)*x[1]
        arg[0,4]=(1.81133385456)*x[0]+(-0.518406894403)*x[1]
        arg[1,0]=(-0.65460152624)*x[0]+(-0.350858526434)*x[1]
        arg[1,1]=(0.924058009558)*x[0]+(-0.40973995895)*x[1]
        arg[1,2]=(-0.299794705178)*x[0]+(-0.924557696915)*x[1]
        arg[1,3]=(-0.608513627169)*x[0]+(-0.141936714389)*x[1]
        arg[1,4]=(0.469099778997)*x[0]+(0.754132353456)*x[1]
        arg[2,0]=(-0.627098551951)*x[0]+(0.476865448119)*x[1]
        arg[2,1]=(-1.03564375134)*x[0]+(-0.280009187543)*x[1]
        arg[2,2]=(0.475083576156)*x[0]+(0.220248066616)*x[1]
        arg[2,3]=(0.0420499398173)*x[0]+(0.0808105284548)*x[1]
        arg[2,4]=(0.368173655662)*x[0]+(-0.58266909961)*x[1]
        arg[3,0]=(0.450851110298)*x[0]+(0.0844590226087)*x[1]
        arg[3,1]=(0.508010554654)*x[0]+(0.797109833792)*x[1]
        arg[3,2]=(0.711479388001)*x[0]+(1.12829906729)*x[1]
        arg[3,3]=(0.0588799697153)*x[0]+(-0.991874826409)*x[1]
        arg[3,4]=(0.743446045536)*x[0]+(0.65754450588)*x[1]
        ref[0,0,0]=(0.448247291087)
        ref[0,0,1]=(-0.00880442943882)
        ref[0,1,0]=(-0.59784234455)
        ref[0,1,1]=(-0.94951606979)
        ref[0,2,0]=(1.38316656933)
        ref[0,2,1]=(-0.124817047005)
        ref[0,3,0]=(-0.0748469893414)
        ref[0,3,1]=(1.44145142047)
        ref[0,4,0]=(1.81133385456)
        ref[0,4,1]=(-0.518406894403)
        ref[1,0,0]=(-0.65460152624)
        ref[1,0,1]=(-0.350858526434)
        ref[1,1,0]=(0.924058009558)
        ref[1,1,1]=(-0.40973995895)
        ref[1,2,0]=(-0.299794705178)
        ref[1,2,1]=(-0.924557696915)
        ref[1,3,0]=(-0.608513627169)
        ref[1,3,1]=(-0.141936714389)
        ref[1,4,0]=(0.469099778997)
        ref[1,4,1]=(0.754132353456)
        ref[2,0,0]=(-0.627098551951)
        ref[2,0,1]=(0.476865448119)
        ref[2,1,0]=(-1.03564375134)
        ref[2,1,1]=(-0.280009187543)
        ref[2,2,0]=(0.475083576156)
        ref[2,2,1]=(0.220248066616)
        ref[2,3,0]=(0.0420499398173)
        ref[2,3,1]=(0.0808105284548)
        ref[2,4,0]=(0.368173655662)
        ref[2,4,1]=(-0.58266909961)
        ref[3,0,0]=(0.450851110298)
        ref[3,0,1]=(0.0844590226087)
        ref[3,1,0]=(0.508010554654)
        ref[3,1,1]=(0.797109833792)
        ref[3,2,0]=(0.711479388001)
        ref[3,2,1]=(1.12829906729)
        ref[3,3,0]=(0.0588799697153)
        ref[3,3,1]=(-0.991874826409)
        ref[3,4,0]=(0.743446045536)
        ref[3,4,1]=(0.65754450588)
      else:
        arg[0,0]=(-0.43386387868)*x[0]+(0.321105491776)*x[1]+(0.384206679414)*x[2]
        arg[0,1]=(0.604698878149)*x[0]+(0.594375439325)*x[1]+(0.162832272176)*x[2]
        arg[0,2]=(-0.332194829416)*x[0]+(0.600821281937)*x[1]+(0.493622302813)*x[2]
        arg[0,3]=(0.846367268698)*x[0]+(-1.32744832381)*x[1]+(0.00466654421944)*x[2]
        arg[0,4]=(0.385111950936)*x[0]+(-0.176705693397)*x[1]+(0.461732370075)*x[2]
        arg[1,0]=(-0.14529643778)*x[0]+(1.15329219838)*x[1]+(0.157842247326)*x[2]
        arg[1,1]=(0.00817149012591)*x[0]+(-0.703898404732)*x[1]+(-1.14419500663)*x[2]
        arg[1,2]=(-0.124155034423)*x[0]+(0.906000496919)*x[1]+(-0.466071596735)*x[2]
        arg[1,3]=(0.413190740147)*x[0]+(0.968639404637)*x[1]+(-1.25413312698)*x[2]
        arg[1,4]=(0.521974966379)*x[0]+(0.936384991187)*x[1]+(-0.0630547476157)*x[2]
        arg[2,0]=(-0.334976510608)*x[0]+(-0.268364721371)*x[1]+(0.806974769679)*x[2]
        arg[2,1]=(-0.0672705162033)*x[0]+(0.459808873839)*x[1]+(0.0725634327358)*x[2]
        arg[2,2]=(0.142418012401)*x[0]+(-0.400301432498)*x[1]+(-1.37021128371)*x[2]
        arg[2,3]=(-1.60617980561)*x[0]+(0.0686943218654)*x[1]+(0.647509287658)*x[2]
        arg[2,4]=(0.0102366197695)*x[0]+(-1.24104511219)*x[1]+(-0.739463479584)*x[2]
        arg[3,0]=(-0.425526939568)*x[0]+(-0.946421307227)*x[1]+(0.133101457377)*x[2]
        arg[3,1]=(1.07183638389)*x[0]+(-1.21405997903)*x[1]+(-0.962984008606)*x[2]
        arg[3,2]=(-0.913456686897)*x[0]+(0.907233128077)*x[1]+(0.581609247137)*x[2]
        arg[3,3]=(-0.729544885457)*x[0]+(-0.433435638004)*x[1]+(0.357011166494)*x[2]
        arg[3,4]=(1.21906931308)*x[0]+(0.322487716786)*x[1]+(1.05602783674)*x[2]
        ref[0,0,0]=(-0.43386387868)
        ref[0,0,1]=(0.321105491776)
        ref[0,0,2]=(0.384206679414)
        ref[0,1,0]=(0.604698878149)
        ref[0,1,1]=(0.594375439325)
        ref[0,1,2]=(0.162832272176)
        ref[0,2,0]=(-0.332194829416)
        ref[0,2,1]=(0.600821281937)
        ref[0,2,2]=(0.493622302813)
        ref[0,3,0]=(0.846367268698)
        ref[0,3,1]=(-1.32744832381)
        ref[0,3,2]=(0.00466654421944)
        ref[0,4,0]=(0.385111950936)
        ref[0,4,1]=(-0.176705693397)
        ref[0,4,2]=(0.461732370075)
        ref[1,0,0]=(-0.14529643778)
        ref[1,0,1]=(1.15329219838)
        ref[1,0,2]=(0.157842247326)
        ref[1,1,0]=(0.00817149012591)
        ref[1,1,1]=(-0.703898404732)
        ref[1,1,2]=(-1.14419500663)
        ref[1,2,0]=(-0.124155034423)
        ref[1,2,1]=(0.906000496919)
        ref[1,2,2]=(-0.466071596735)
        ref[1,3,0]=(0.413190740147)
        ref[1,3,1]=(0.968639404637)
        ref[1,3,2]=(-1.25413312698)
        ref[1,4,0]=(0.521974966379)
        ref[1,4,1]=(0.936384991187)
        ref[1,4,2]=(-0.0630547476157)
        ref[2,0,0]=(-0.334976510608)
        ref[2,0,1]=(-0.268364721371)
        ref[2,0,2]=(0.806974769679)
        ref[2,1,0]=(-0.0672705162033)
        ref[2,1,1]=(0.459808873839)
        ref[2,1,2]=(0.0725634327358)
        ref[2,2,0]=(0.142418012401)
        ref[2,2,1]=(-0.400301432498)
        ref[2,2,2]=(-1.37021128371)
        ref[2,3,0]=(-1.60617980561)
        ref[2,3,1]=(0.0686943218654)
        ref[2,3,2]=(0.647509287658)
        ref[2,4,0]=(0.0102366197695)
        ref[2,4,1]=(-1.24104511219)
        ref[2,4,2]=(-0.739463479584)
        ref[3,0,0]=(-0.425526939568)
        ref[3,0,1]=(-0.946421307227)
        ref[3,0,2]=(0.133101457377)
        ref[3,1,0]=(1.07183638389)
        ref[3,1,1]=(-1.21405997903)
        ref[3,1,2]=(-0.962984008606)
        ref[3,2,0]=(-0.913456686897)
        ref[3,2,1]=(0.907233128077)
        ref[3,2,2]=(0.581609247137)
        ref[3,3,0]=(-0.729544885457)
        ref[3,3,1]=(-0.433435638004)
        ref[3,3,2]=(0.357011166494)
        ref[3,4,0]=(1.21906931308)
        ref[3,4,1]=(0.322487716786)
        ref[3,4,2]=(1.05602783674)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(-0.287546203823)*x[0]+(0.390321729967)*x[1]
        arg[0,0,1]=(-0.112653571553)*x[0]+(-0.359753186922)*x[1]
        arg[0,1,0]=(0.00656985674969)*x[0]+(-0.397210304393)*x[1]
        arg[0,1,1]=(-0.825698431533)*x[0]+(-1.20994477496)*x[1]
        arg[1,0,0]=(0.0456789352898)*x[0]+(0.201178709355)*x[1]
        arg[1,0,1]=(0.216619650199)*x[0]+(-0.684237394127)*x[1]
        arg[1,1,0]=(-0.452605086273)*x[0]+(0.383220784575)*x[1]
        arg[1,1,1]=(-0.385315869703)*x[0]+(-1.25937320051)*x[1]
        arg[2,0,0]=(-0.546290546777)*x[0]+(0.0659124729108)*x[1]
        arg[2,0,1]=(0.23413834434)*x[0]+(0.241510050424)*x[1]
        arg[2,1,0]=(-0.541097368124)*x[0]+(0.458682016611)*x[1]
        arg[2,1,1]=(0.314030173817)*x[0]+(0.083613884286)*x[1]
        arg[3,0,0]=(0.862005271666)*x[0]+(0.858693755091)*x[1]
        arg[3,0,1]=(-1.14220805291)*x[0]+(0.609875693913)*x[1]
        arg[3,1,0]=(-0.847069360014)*x[0]+(0.240069314456)*x[1]
        arg[3,1,1]=(0.480596211718)*x[0]+(-1.40901297384)*x[1]
        arg[4,0,0]=(-1.38036653523)*x[0]+(-0.0732955000278)*x[1]
        arg[4,0,1]=(0.0716061704803)*x[0]+(0.180127379412)*x[1]
        arg[4,1,0]=(1.26959959862)*x[0]+(-1.38131896436)*x[1]
        arg[4,1,1]=(0.204280361123)*x[0]+(0.532611514369)*x[1]
        arg[5,0,0]=(0.209536293545)*x[0]+(-0.303994906788)*x[1]
        arg[5,0,1]=(1.41150162289)*x[0]+(1.32515735407)*x[1]
        arg[5,1,0]=(1.59736027384)*x[0]+(-0.711055564868)*x[1]
        arg[5,1,1]=(-1.31799658024)*x[0]+(0.829209493108)*x[1]
        ref[0,0,0,0]=(-0.287546203823)
        ref[0,0,0,1]=(0.390321729967)
        ref[0,0,1,0]=(-0.112653571553)
        ref[0,0,1,1]=(-0.359753186922)
        ref[0,1,0,0]=(0.00656985674969)
        ref[0,1,0,1]=(-0.397210304393)
        ref[0,1,1,0]=(-0.825698431533)
        ref[0,1,1,1]=(-1.20994477496)
        ref[1,0,0,0]=(0.0456789352898)
        ref[1,0,0,1]=(0.201178709355)
        ref[1,0,1,0]=(0.216619650199)
        ref[1,0,1,1]=(-0.684237394127)
        ref[1,1,0,0]=(-0.452605086273)
        ref[1,1,0,1]=(0.383220784575)
        ref[1,1,1,0]=(-0.385315869703)
        ref[1,1,1,1]=(-1.25937320051)
        ref[2,0,0,0]=(-0.546290546777)
        ref[2,0,0,1]=(0.0659124729108)
        ref[2,0,1,0]=(0.23413834434)
        ref[2,0,1,1]=(0.241510050424)
        ref[2,1,0,0]=(-0.541097368124)
        ref[2,1,0,1]=(0.458682016611)
        ref[2,1,1,0]=(0.314030173817)
        ref[2,1,1,1]=(0.083613884286)
        ref[3,0,0,0]=(0.862005271666)
        ref[3,0,0,1]=(0.858693755091)
        ref[3,0,1,0]=(-1.14220805291)
        ref[3,0,1,1]=(0.609875693913)
        ref[3,1,0,0]=(-0.847069360014)
        ref[3,1,0,1]=(0.240069314456)
        ref[3,1,1,0]=(0.480596211718)
        ref[3,1,1,1]=(-1.40901297384)
        ref[4,0,0,0]=(-1.38036653523)
        ref[4,0,0,1]=(-0.0732955000278)
        ref[4,0,1,0]=(0.0716061704803)
        ref[4,0,1,1]=(0.180127379412)
        ref[4,1,0,0]=(1.26959959862)
        ref[4,1,0,1]=(-1.38131896436)
        ref[4,1,1,0]=(0.204280361123)
        ref[4,1,1,1]=(0.532611514369)
        ref[5,0,0,0]=(0.209536293545)
        ref[5,0,0,1]=(-0.303994906788)
        ref[5,0,1,0]=(1.41150162289)
        ref[5,0,1,1]=(1.32515735407)
        ref[5,1,0,0]=(1.59736027384)
        ref[5,1,0,1]=(-0.711055564868)
        ref[5,1,1,0]=(-1.31799658024)
        ref[5,1,1,1]=(0.829209493108)
      else:
        arg[0,0,0]=(0.832267408258)*x[0]+(-1.1995239623)*x[1]+(-1.81276973398)*x[2]
        arg[0,0,1]=(-0.0125232856194)*x[0]+(0.122543918666)*x[1]+(-1.12411286836)*x[2]
        arg[0,1,0]=(0.985611388383)*x[0]+(-1.17217207383)*x[1]+(-0.231241872108)*x[2]
        arg[0,1,1]=(0.625494139449)*x[0]+(0.784802683192)*x[1]+(0.612764525973)*x[2]
        arg[1,0,0]=(0.327644975022)*x[0]+(0.0552951801276)*x[1]+(-0.806887445103)*x[2]
        arg[1,0,1]=(0.194338798022)*x[0]+(0.799902026341)*x[1]+(1.70164249741)*x[2]
        arg[1,1,0]=(0.890832636865)*x[0]+(0.223899365292)*x[1]+(-0.283094526824)*x[2]
        arg[1,1,1]=(-0.122108330088)*x[0]+(-0.293413695547)*x[1]+(-0.371581852879)*x[2]
        arg[2,0,0]=(-0.469341678297)*x[0]+(0.152856292978)*x[1]+(0.765861314481)*x[2]
        arg[2,0,1]=(0.249219162734)*x[0]+(0.319622173697)*x[1]+(-0.182998471546)*x[2]
        arg[2,1,0]=(0.39687496584)*x[0]+(0.137084411978)*x[1]+(-0.34707250854)*x[2]
        arg[2,1,1]=(-0.382391798519)*x[0]+(0.755317411547)*x[1]+(0.424204162929)*x[2]
        arg[3,0,0]=(1.12637976255)*x[0]+(0.11632178776)*x[1]+(-0.615547388072)*x[2]
        arg[3,0,1]=(-0.819303259446)*x[0]+(0.916951216368)*x[1]+(-0.572792050441)*x[2]
        arg[3,1,0]=(0.194642055711)*x[0]+(0.949663114037)*x[1]+(-0.403001914682)*x[2]
        arg[3,1,1]=(0.902547242635)*x[0]+(-1.04027936291)*x[1]+(-1.78463515221)*x[2]
        arg[4,0,0]=(0.59060036737)*x[0]+(-0.580905638676)*x[1]+(-0.486098556884)*x[2]
        arg[4,0,1]=(-0.330634605524)*x[0]+(-0.758140973629)*x[1]+(-0.192282397672)*x[2]
        arg[4,1,0]=(1.40502801336)*x[0]+(0.199036161455)*x[1]+(-0.558660947885)*x[2]
        arg[4,1,1]=(-0.155588274273)*x[0]+(-0.57050736529)*x[1]+(1.1998664442)*x[2]
        arg[5,0,0]=(-0.754680865929)*x[0]+(1.29040984028)*x[1]+(-1.83778974463)*x[2]
        arg[5,0,1]=(0.274692501647)*x[0]+(0.368546080921)*x[1]+(-0.446376269232)*x[2]
        arg[5,1,0]=(-0.758704372861)*x[0]+(1.75555227426)*x[1]+(-0.13664688871)*x[2]
        arg[5,1,1]=(-0.568468068187)*x[0]+(-0.320218561544)*x[1]+(-0.827795466031)*x[2]
        ref[0,0,0,0]=(0.832267408258)
        ref[0,0,0,1]=(-1.1995239623)
        ref[0,0,0,2]=(-1.81276973398)
        ref[0,0,1,0]=(-0.0125232856194)
        ref[0,0,1,1]=(0.122543918666)
        ref[0,0,1,2]=(-1.12411286836)
        ref[0,1,0,0]=(0.985611388383)
        ref[0,1,0,1]=(-1.17217207383)
        ref[0,1,0,2]=(-0.231241872108)
        ref[0,1,1,0]=(0.625494139449)
        ref[0,1,1,1]=(0.784802683192)
        ref[0,1,1,2]=(0.612764525973)
        ref[1,0,0,0]=(0.327644975022)
        ref[1,0,0,1]=(0.0552951801276)
        ref[1,0,0,2]=(-0.806887445103)
        ref[1,0,1,0]=(0.194338798022)
        ref[1,0,1,1]=(0.799902026341)
        ref[1,0,1,2]=(1.70164249741)
        ref[1,1,0,0]=(0.890832636865)
        ref[1,1,0,1]=(0.223899365292)
        ref[1,1,0,2]=(-0.283094526824)
        ref[1,1,1,0]=(-0.122108330088)
        ref[1,1,1,1]=(-0.293413695547)
        ref[1,1,1,2]=(-0.371581852879)
        ref[2,0,0,0]=(-0.469341678297)
        ref[2,0,0,1]=(0.152856292978)
        ref[2,0,0,2]=(0.765861314481)
        ref[2,0,1,0]=(0.249219162734)
        ref[2,0,1,1]=(0.319622173697)
        ref[2,0,1,2]=(-0.182998471546)
        ref[2,1,0,0]=(0.39687496584)
        ref[2,1,0,1]=(0.137084411978)
        ref[2,1,0,2]=(-0.34707250854)
        ref[2,1,1,0]=(-0.382391798519)
        ref[2,1,1,1]=(0.755317411547)
        ref[2,1,1,2]=(0.424204162929)
        ref[3,0,0,0]=(1.12637976255)
        ref[3,0,0,1]=(0.11632178776)
        ref[3,0,0,2]=(-0.615547388072)
        ref[3,0,1,0]=(-0.819303259446)
        ref[3,0,1,1]=(0.916951216368)
        ref[3,0,1,2]=(-0.572792050441)
        ref[3,1,0,0]=(0.194642055711)
        ref[3,1,0,1]=(0.949663114037)
        ref[3,1,0,2]=(-0.403001914682)
        ref[3,1,1,0]=(0.902547242635)
        ref[3,1,1,1]=(-1.04027936291)
        ref[3,1,1,2]=(-1.78463515221)
        ref[4,0,0,0]=(0.59060036737)
        ref[4,0,0,1]=(-0.580905638676)
        ref[4,0,0,2]=(-0.486098556884)
        ref[4,0,1,0]=(-0.330634605524)
        ref[4,0,1,1]=(-0.758140973629)
        ref[4,0,1,2]=(-0.192282397672)
        ref[4,1,0,0]=(1.40502801336)
        ref[4,1,0,1]=(0.199036161455)
        ref[4,1,0,2]=(-0.558660947885)
        ref[4,1,1,0]=(-0.155588274273)
        ref[4,1,1,1]=(-0.57050736529)
        ref[4,1,1,2]=(1.1998664442)
        ref[5,0,0,0]=(-0.754680865929)
        ref[5,0,0,1]=(1.29040984028)
        ref[5,0,0,2]=(-1.83778974463)
        ref[5,0,1,0]=(0.274692501647)
        ref[5,0,1,1]=(0.368546080921)
        ref[5,0,1,2]=(-0.446376269232)
        ref[5,1,0,0]=(-0.758704372861)
        ref[5,1,0,1]=(1.75555227426)
        ref[5,1,0,2]=(-0.13664688871)
        ref[5,1,1,0]=(-0.568468068187)
        ref[5,1,1,1]=(-0.320218561544)
        ref[5,1,1,2]=(-0.827795466031)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.132000741495)*x[0]**o+(0.122399178026)*x[0]+(-0.285047923022)*x[1]**o+(0.665814814322)*x[1]
        ref[0]=o*(-0.132000741495)*x_ref[0]**(o-1)+(0.122399178026)
        ref[1]=o*(-0.285047923022)*x_ref[1]**(o-1)+(0.665814814322)
      else:
        s=(0.596417375474)*x[0]**o+(0.780542320679)*x[0]+(-0.59348113736)*x[1]**o+(0.755472585752)*x[1]+(-0.423796298696)*x[2]**o+(0.917578429027)*x[2]
        ref[0]=o*(0.596417375474)*x_ref[0]**(o-1)+(0.780542320679)
        ref[1]=o*(-0.59348113736)*x_ref[1]**(o-1)+(0.755472585752)
        ref[2]=o*(-0.423796298696)*x_ref[2]**(o-1)+(0.917578429027)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.624689355591)*x[0]**o+(-0.747728245047)*x[0]+(0.692787021065)*x[1]**o+(-0.70629082384)*x[1]
        s[1]=(-0.225695886421)*x[0]**o+(-0.444535961493)*x[0]+(0.235690936232)*x[1]**o+(-0.970876426657)*x[1]
        ref[0,0]=o*(0.624689355591)*x_ref[0]**(o-1)+(-0.747728245047)
        ref[0,1]=o*(0.692787021065)*x_ref[1]**(o-1)+(-0.70629082384)
        ref[1,0]=o*(-0.225695886421)*x_ref[0]**(o-1)+(-0.444535961493)
        ref[1,1]=o*(0.235690936232)*x_ref[1]**(o-1)+(-0.970876426657)
      else:
        s[0]=(-0.459906225365)*x[0]**o+(-0.976920232334)*x[0]+(-0.359230869877)*x[1]**o+(0.99442848994)*x[1]+(-0.462181187731)*x[2]**o+(-0.272763004354)*x[2]
        s[1]=(-0.384545997957)*x[0]**o+(0.206838344134)*x[0]+(0.827958420952)*x[1]**o+(0.851604767316)*x[1]+(0.180277208362)*x[2]**o+(0.276165244173)*x[2]
        ref[0,0]=o*(-0.459906225365)*x_ref[0]**(o-1)+(-0.976920232334)
        ref[0,1]=o*(-0.359230869877)*x_ref[1]**(o-1)+(0.99442848994)
        ref[0,2]=o*(-0.462181187731)*x_ref[2]**(o-1)+(-0.272763004354)
        ref[1,0]=o*(-0.384545997957)*x_ref[0]**(o-1)+(0.206838344134)
        ref[1,1]=o*(0.827958420952)*x_ref[1]**(o-1)+(0.851604767316)
        ref[1,2]=o*(0.180277208362)*x_ref[2]**(o-1)+(0.276165244173)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.866700100162)*x[0]**o+(-0.443253975067)*x[0]+(0.930660789831)*x[1]**o+(0.348981739779)*x[1]
        s[0,1]=(0.595655533809)*x[0]**o+(0.425701794417)*x[0]+(-0.700204056577)*x[1]**o+(-0.606084182043)*x[1]
        s[0,2]=(-0.0975376318378)*x[0]**o+(0.0352659915524)*x[0]+(0.336096329457)*x[1]**o+(-0.604536128515)*x[1]
        s[0,3]=(0.932694885424)*x[0]**o+(0.848315864182)*x[0]+(0.294198667412)*x[1]**o+(-0.548590122662)*x[1]
        s[0,4]=(-0.464671727588)*x[0]**o+(-0.479615042615)*x[0]+(-0.205470141615)*x[1]**o+(-0.771280534115)*x[1]
        s[1,0]=(0.459832819849)*x[0]**o+(-0.703780783428)*x[0]+(0.101094024974)*x[1]**o+(-0.504757888238)*x[1]
        s[1,1]=(-0.74483211902)*x[0]**o+(0.467112390979)*x[0]+(-0.956618977211)*x[1]**o+(0.455481886754)*x[1]
        s[1,2]=(0.0976209987456)*x[0]**o+(0.44469736085)*x[0]+(-0.213220033441)*x[1]**o+(0.0448880653868)*x[1]
        s[1,3]=(0.18795879454)*x[0]**o+(0.603296276978)*x[0]+(-0.915713437955)*x[1]**o+(0.911772973549)*x[1]
        s[1,4]=(-0.960678163557)*x[0]**o+(0.000798715245087)*x[0]+(-0.195050046209)*x[1]**o+(0.55429412919)*x[1]
        s[2,0]=(-0.885615347975)*x[0]**o+(-0.0757793285493)*x[0]+(0.643863139156)*x[1]**o+(-0.617158285199)*x[1]
        s[2,1]=(0.232668453484)*x[0]**o+(0.586242977718)*x[0]+(-0.728810607608)*x[1]**o+(0.826248720042)*x[1]
        s[2,2]=(-0.893353803021)*x[0]**o+(-0.510393964603)*x[0]+(0.851185573516)*x[1]**o+(0.821075933157)*x[1]
        s[2,3]=(0.567570646968)*x[0]**o+(0.282673304306)*x[0]+(0.636861243255)*x[1]**o+(0.308681123745)*x[1]
        s[2,4]=(0.843072247533)*x[0]**o+(-0.302083248085)*x[0]+(-0.147818607759)*x[1]**o+(-0.796766854695)*x[1]
        s[3,0]=(-0.000711103721221)*x[0]**o+(0.730198459281)*x[0]+(0.395333992473)*x[1]**o+(0.743896557243)*x[1]
        s[3,1]=(0.615705307589)*x[0]**o+(0.74607803139)*x[0]+(0.837364825996)*x[1]**o+(0.362527888693)*x[1]
        s[3,2]=(-0.431332008116)*x[0]**o+(-0.5449015623)*x[0]+(0.0435062014388)*x[1]**o+(-0.408079384098)*x[1]
        s[3,3]=(-0.887047605579)*x[0]**o+(-0.966245172182)*x[0]+(-0.457094377191)*x[1]**o+(0.934841781784)*x[1]
        s[3,4]=(-0.6212484366)*x[0]**o+(0.99908421004)*x[0]+(0.75098732282)*x[1]**o+(-0.0868062878539)*x[1]
        ref[0,0,0]=o*(-0.866700100162)*x_ref[0]**(o-1)+(-0.443253975067)
        ref[0,0,1]=o*(0.930660789831)*x_ref[1]**(o-1)+(0.348981739779)
        ref[0,1,0]=o*(0.595655533809)*x_ref[0]**(o-1)+(0.425701794417)
        ref[0,1,1]=o*(-0.700204056577)*x_ref[1]**(o-1)+(-0.606084182043)
        ref[0,2,0]=o*(-0.0975376318378)*x_ref[0]**(o-1)+(0.0352659915524)
        ref[0,2,1]=o*(0.336096329457)*x_ref[1]**(o-1)+(-0.604536128515)
        ref[0,3,0]=o*(0.932694885424)*x_ref[0]**(o-1)+(0.848315864182)
        ref[0,3,1]=o*(0.294198667412)*x_ref[1]**(o-1)+(-0.548590122662)
        ref[0,4,0]=o*(-0.464671727588)*x_ref[0]**(o-1)+(-0.479615042615)
        ref[0,4,1]=o*(-0.205470141615)*x_ref[1]**(o-1)+(-0.771280534115)
        ref[1,0,0]=o*(0.459832819849)*x_ref[0]**(o-1)+(-0.703780783428)
        ref[1,0,1]=o*(0.101094024974)*x_ref[1]**(o-1)+(-0.504757888238)
        ref[1,1,0]=o*(-0.74483211902)*x_ref[0]**(o-1)+(0.467112390979)
        ref[1,1,1]=o*(-0.956618977211)*x_ref[1]**(o-1)+(0.455481886754)
        ref[1,2,0]=o*(0.0976209987456)*x_ref[0]**(o-1)+(0.44469736085)
        ref[1,2,1]=o*(-0.213220033441)*x_ref[1]**(o-1)+(0.0448880653868)
        ref[1,3,0]=o*(0.18795879454)*x_ref[0]**(o-1)+(0.603296276978)
        ref[1,3,1]=o*(-0.915713437955)*x_ref[1]**(o-1)+(0.911772973549)
        ref[1,4,0]=o*(-0.960678163557)*x_ref[0]**(o-1)+(0.000798715245087)
        ref[1,4,1]=o*(-0.195050046209)*x_ref[1]**(o-1)+(0.55429412919)
        ref[2,0,0]=o*(-0.885615347975)*x_ref[0]**(o-1)+(-0.0757793285493)
        ref[2,0,1]=o*(0.643863139156)*x_ref[1]**(o-1)+(-0.617158285199)
        ref[2,1,0]=o*(0.232668453484)*x_ref[0]**(o-1)+(0.586242977718)
        ref[2,1,1]=o*(-0.728810607608)*x_ref[1]**(o-1)+(0.826248720042)
        ref[2,2,0]=o*(-0.893353803021)*x_ref[0]**(o-1)+(-0.510393964603)
        ref[2,2,1]=o*(0.851185573516)*x_ref[1]**(o-1)+(0.821075933157)
        ref[2,3,0]=o*(0.567570646968)*x_ref[0]**(o-1)+(0.282673304306)
        ref[2,3,1]=o*(0.636861243255)*x_ref[1]**(o-1)+(0.308681123745)
        ref[2,4,0]=o*(0.843072247533)*x_ref[0]**(o-1)+(-0.302083248085)
        ref[2,4,1]=o*(-0.147818607759)*x_ref[1]**(o-1)+(-0.796766854695)
        ref[3,0,0]=o*(-0.000711103721221)*x_ref[0]**(o-1)+(0.730198459281)
        ref[3,0,1]=o*(0.395333992473)*x_ref[1]**(o-1)+(0.743896557243)
        ref[3,1,0]=o*(0.615705307589)*x_ref[0]**(o-1)+(0.74607803139)
        ref[3,1,1]=o*(0.837364825996)*x_ref[1]**(o-1)+(0.362527888693)
        ref[3,2,0]=o*(-0.431332008116)*x_ref[0]**(o-1)+(-0.5449015623)
        ref[3,2,1]=o*(0.0435062014388)*x_ref[1]**(o-1)+(-0.408079384098)
        ref[3,3,0]=o*(-0.887047605579)*x_ref[0]**(o-1)+(-0.966245172182)
        ref[3,3,1]=o*(-0.457094377191)*x_ref[1]**(o-1)+(0.934841781784)
        ref[3,4,0]=o*(-0.6212484366)*x_ref[0]**(o-1)+(0.99908421004)
        ref[3,4,1]=o*(0.75098732282)*x_ref[1]**(o-1)+(-0.0868062878539)
      else:
        s[0,0]=(0.729942219527)*x[0]**o+(-0.721160545053)*x[0]+(0.426397843912)*x[1]**o+(-0.213185647354)*x[1]+(0.728447138266)*x[2]**o+(-0.607791661013)*x[2]
        s[0,1]=(0.667616392653)*x[0]**o+(0.848051730251)*x[0]+(0.121550076489)*x[1]**o+(0.0843862117633)*x[1]+(-0.509323081526)*x[2]**o+(0.736521833786)*x[2]
        s[0,2]=(0.830127288575)*x[0]**o+(0.964772646621)*x[0]+(0.913688863179)*x[1]**o+(0.957018200531)*x[1]+(-0.694170056751)*x[2]**o+(0.105973113953)*x[2]
        s[0,3]=(-0.235173218805)*x[0]**o+(0.858875579758)*x[0]+(0.823162706883)*x[1]**o+(0.108717821259)*x[1]+(0.80411364975)*x[2]**o+(0.0348079894361)*x[2]
        s[0,4]=(0.78867654833)*x[0]**o+(-0.0191953378129)*x[0]+(-0.0840244773387)*x[1]**o+(0.614156921824)*x[1]+(-0.0190397891394)*x[2]**o+(0.750918932533)*x[2]
        s[1,0]=(-0.0931449972666)*x[0]**o+(0.887179037493)*x[0]+(0.163173866494)*x[1]**o+(-0.0429615650925)*x[1]+(0.246231141028)*x[2]**o+(0.835506100749)*x[2]
        s[1,1]=(-0.0644771805334)*x[0]**o+(-0.447016488171)*x[0]+(0.525187872112)*x[1]**o+(0.0698248427183)*x[1]+(0.206184560671)*x[2]**o+(0.0760145857359)*x[2]
        s[1,2]=(-0.468079677328)*x[0]**o+(0.209734412741)*x[0]+(-0.76390490599)*x[1]**o+(-0.890555632144)*x[1]+(0.0940465019231)*x[2]**o+(-0.4911374279)*x[2]
        s[1,3]=(0.0219043704096)*x[0]**o+(0.843668836143)*x[0]+(-0.496594214362)*x[1]**o+(-0.698887537236)*x[1]+(0.739801576727)*x[2]**o+(-0.283969520066)*x[2]
        s[1,4]=(0.626836103474)*x[0]**o+(-0.209145555082)*x[0]+(0.0527420557385)*x[1]**o+(0.447276924763)*x[1]+(0.753404625572)*x[2]**o+(-0.964517776982)*x[2]
        s[2,0]=(0.208209713508)*x[0]**o+(0.666027366366)*x[0]+(0.648558221549)*x[1]**o+(-0.867196856985)*x[1]+(-0.386920813405)*x[2]**o+(-0.859803466402)*x[2]
        s[2,1]=(0.192327689645)*x[0]**o+(0.706632402773)*x[0]+(-0.917135230541)*x[1]**o+(-0.518142383283)*x[1]+(0.63345047167)*x[2]**o+(0.452536991007)*x[2]
        s[2,2]=(-0.216559791626)*x[0]**o+(0.280416285835)*x[0]+(0.880532961859)*x[1]**o+(-0.685211600885)*x[1]+(0.435667186231)*x[2]**o+(0.632733331589)*x[2]
        s[2,3]=(0.52338353111)*x[0]**o+(-0.865976270688)*x[0]+(0.598776819058)*x[1]**o+(-0.404416433163)*x[1]+(-0.538779785754)*x[2]**o+(-0.708928887004)*x[2]
        s[2,4]=(0.523035959616)*x[0]**o+(0.732324929631)*x[0]+(0.213261893733)*x[1]**o+(0.620185443057)*x[1]+(0.984821746408)*x[2]**o+(0.358075460145)*x[2]
        s[3,0]=(-0.2094567637)*x[0]**o+(0.0635506980721)*x[0]+(0.540179140495)*x[1]**o+(-0.178193714382)*x[1]+(0.114963963742)*x[2]**o+(-0.232785400122)*x[2]
        s[3,1]=(0.581234387125)*x[0]**o+(-0.800223513435)*x[0]+(-0.580041308558)*x[1]**o+(-0.468466479829)*x[1]+(0.66171049066)*x[2]**o+(0.00411361715063)*x[2]
        s[3,2]=(0.972539025242)*x[0]**o+(-0.640993915582)*x[0]+(0.418443080441)*x[1]**o+(0.608840732702)*x[1]+(0.979209906273)*x[2]**o+(-0.984684975101)*x[2]
        s[3,3]=(0.80136860876)*x[0]**o+(0.526110649659)*x[0]+(-0.307964128679)*x[1]**o+(-0.162612676576)*x[1]+(-0.68486706357)*x[2]**o+(-0.234358375142)*x[2]
        s[3,4]=(0.146116143734)*x[0]**o+(0.581210816708)*x[0]+(-0.152105642806)*x[1]**o+(0.97096072826)*x[1]+(-0.0416344321982)*x[2]**o+(-0.933814919317)*x[2]
        ref[0,0,0]=o*(0.729942219527)*x_ref[0]**(o-1)+(-0.721160545053)
        ref[0,0,1]=o*(0.426397843912)*x_ref[1]**(o-1)+(-0.213185647354)
        ref[0,0,2]=o*(0.728447138266)*x_ref[2]**(o-1)+(-0.607791661013)
        ref[0,1,0]=o*(0.667616392653)*x_ref[0]**(o-1)+(0.848051730251)
        ref[0,1,1]=o*(0.121550076489)*x_ref[1]**(o-1)+(0.0843862117633)
        ref[0,1,2]=o*(-0.509323081526)*x_ref[2]**(o-1)+(0.736521833786)
        ref[0,2,0]=o*(0.830127288575)*x_ref[0]**(o-1)+(0.964772646621)
        ref[0,2,1]=o*(0.913688863179)*x_ref[1]**(o-1)+(0.957018200531)
        ref[0,2,2]=o*(-0.694170056751)*x_ref[2]**(o-1)+(0.105973113953)
        ref[0,3,0]=o*(-0.235173218805)*x_ref[0]**(o-1)+(0.858875579758)
        ref[0,3,1]=o*(0.823162706883)*x_ref[1]**(o-1)+(0.108717821259)
        ref[0,3,2]=o*(0.80411364975)*x_ref[2]**(o-1)+(0.0348079894361)
        ref[0,4,0]=o*(0.78867654833)*x_ref[0]**(o-1)+(-0.0191953378129)
        ref[0,4,1]=o*(-0.0840244773387)*x_ref[1]**(o-1)+(0.614156921824)
        ref[0,4,2]=o*(-0.0190397891394)*x_ref[2]**(o-1)+(0.750918932533)
        ref[1,0,0]=o*(-0.0931449972666)*x_ref[0]**(o-1)+(0.887179037493)
        ref[1,0,1]=o*(0.163173866494)*x_ref[1]**(o-1)+(-0.0429615650925)
        ref[1,0,2]=o*(0.246231141028)*x_ref[2]**(o-1)+(0.835506100749)
        ref[1,1,0]=o*(-0.0644771805334)*x_ref[0]**(o-1)+(-0.447016488171)
        ref[1,1,1]=o*(0.525187872112)*x_ref[1]**(o-1)+(0.0698248427183)
        ref[1,1,2]=o*(0.206184560671)*x_ref[2]**(o-1)+(0.0760145857359)
        ref[1,2,0]=o*(-0.468079677328)*x_ref[0]**(o-1)+(0.209734412741)
        ref[1,2,1]=o*(-0.76390490599)*x_ref[1]**(o-1)+(-0.890555632144)
        ref[1,2,2]=o*(0.0940465019231)*x_ref[2]**(o-1)+(-0.4911374279)
        ref[1,3,0]=o*(0.0219043704096)*x_ref[0]**(o-1)+(0.843668836143)
        ref[1,3,1]=o*(-0.496594214362)*x_ref[1]**(o-1)+(-0.698887537236)
        ref[1,3,2]=o*(0.739801576727)*x_ref[2]**(o-1)+(-0.283969520066)
        ref[1,4,0]=o*(0.626836103474)*x_ref[0]**(o-1)+(-0.209145555082)
        ref[1,4,1]=o*(0.0527420557385)*x_ref[1]**(o-1)+(0.447276924763)
        ref[1,4,2]=o*(0.753404625572)*x_ref[2]**(o-1)+(-0.964517776982)
        ref[2,0,0]=o*(0.208209713508)*x_ref[0]**(o-1)+(0.666027366366)
        ref[2,0,1]=o*(0.648558221549)*x_ref[1]**(o-1)+(-0.867196856985)
        ref[2,0,2]=o*(-0.386920813405)*x_ref[2]**(o-1)+(-0.859803466402)
        ref[2,1,0]=o*(0.192327689645)*x_ref[0]**(o-1)+(0.706632402773)
        ref[2,1,1]=o*(-0.917135230541)*x_ref[1]**(o-1)+(-0.518142383283)
        ref[2,1,2]=o*(0.63345047167)*x_ref[2]**(o-1)+(0.452536991007)
        ref[2,2,0]=o*(-0.216559791626)*x_ref[0]**(o-1)+(0.280416285835)
        ref[2,2,1]=o*(0.880532961859)*x_ref[1]**(o-1)+(-0.685211600885)
        ref[2,2,2]=o*(0.435667186231)*x_ref[2]**(o-1)+(0.632733331589)
        ref[2,3,0]=o*(0.52338353111)*x_ref[0]**(o-1)+(-0.865976270688)
        ref[2,3,1]=o*(0.598776819058)*x_ref[1]**(o-1)+(-0.404416433163)
        ref[2,3,2]=o*(-0.538779785754)*x_ref[2]**(o-1)+(-0.708928887004)
        ref[2,4,0]=o*(0.523035959616)*x_ref[0]**(o-1)+(0.732324929631)
        ref[2,4,1]=o*(0.213261893733)*x_ref[1]**(o-1)+(0.620185443057)
        ref[2,4,2]=o*(0.984821746408)*x_ref[2]**(o-1)+(0.358075460145)
        ref[3,0,0]=o*(-0.2094567637)*x_ref[0]**(o-1)+(0.0635506980721)
        ref[3,0,1]=o*(0.540179140495)*x_ref[1]**(o-1)+(-0.178193714382)
        ref[3,0,2]=o*(0.114963963742)*x_ref[2]**(o-1)+(-0.232785400122)
        ref[3,1,0]=o*(0.581234387125)*x_ref[0]**(o-1)+(-0.800223513435)
        ref[3,1,1]=o*(-0.580041308558)*x_ref[1]**(o-1)+(-0.468466479829)
        ref[3,1,2]=o*(0.66171049066)*x_ref[2]**(o-1)+(0.00411361715063)
        ref[3,2,0]=o*(0.972539025242)*x_ref[0]**(o-1)+(-0.640993915582)
        ref[3,2,1]=o*(0.418443080441)*x_ref[1]**(o-1)+(0.608840732702)
        ref[3,2,2]=o*(0.979209906273)*x_ref[2]**(o-1)+(-0.984684975101)
        ref[3,3,0]=o*(0.80136860876)*x_ref[0]**(o-1)+(0.526110649659)
        ref[3,3,1]=o*(-0.307964128679)*x_ref[1]**(o-1)+(-0.162612676576)
        ref[3,3,2]=o*(-0.68486706357)*x_ref[2]**(o-1)+(-0.234358375142)
        ref[3,4,0]=o*(0.146116143734)*x_ref[0]**(o-1)+(0.581210816708)
        ref[3,4,1]=o*(-0.152105642806)*x_ref[1]**(o-1)+(0.97096072826)
        ref[3,4,2]=o*(-0.0416344321982)*x_ref[2]**(o-1)+(-0.933814919317)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(0.210810831144)*x[0]**o+(0.993748550706)*x[0]+(0.225093899339)*x[1]**o+(-0.306898024507)*x[1]
        s[0,0,1]=(0.945391291392)*x[0]**o+(-0.34733930643)*x[0]+(0.523085690915)*x[1]**o+(-0.332680309461)*x[1]
        s[0,1,0]=(0.372088608669)*x[0]**o+(0.276443856788)*x[0]+(-0.439514323588)*x[1]**o+(0.985453767485)*x[1]
        s[0,1,1]=(-0.967561100768)*x[0]**o+(0.591908531375)*x[0]+(0.288385284239)*x[1]**o+(-0.422883693702)*x[1]
        s[1,0,0]=(-0.735303276087)*x[0]**o+(0.540176339329)*x[0]+(0.124141197609)*x[1]**o+(0.17505950278)*x[1]
        s[1,0,1]=(0.799321447619)*x[0]**o+(-0.521267395747)*x[0]+(-0.388235024602)*x[1]**o+(0.0783682529975)*x[1]
        s[1,1,0]=(-0.7923799998)*x[0]**o+(-0.779345662449)*x[0]+(0.507399627325)*x[1]**o+(-0.668039939194)*x[1]
        s[1,1,1]=(0.360260485497)*x[0]**o+(0.639639208398)*x[0]+(0.275791805772)*x[1]**o+(-0.923288698445)*x[1]
        s[2,0,0]=(0.338586096766)*x[0]**o+(-0.862613705959)*x[0]+(0.0321867760579)*x[1]**o+(0.418774919375)*x[1]
        s[2,0,1]=(0.0363215417361)*x[0]**o+(0.994712078504)*x[0]+(-0.260614790561)*x[1]**o+(-0.969097035084)*x[1]
        s[2,1,0]=(0.375366661281)*x[0]**o+(0.456874530834)*x[0]+(0.429399840953)*x[1]**o+(0.688205075709)*x[1]
        s[2,1,1]=(-0.918074889111)*x[0]**o+(-0.716561240269)*x[0]+(-0.477398945222)*x[1]**o+(-0.223142905734)*x[1]
        s[3,0,0]=(-0.420424618942)*x[0]**o+(-0.702691949655)*x[0]+(-0.389998593814)*x[1]**o+(0.778091583668)*x[1]
        s[3,0,1]=(-0.500317522801)*x[0]**o+(-0.906158794184)*x[0]+(-0.942667332637)*x[1]**o+(-0.38695660361)*x[1]
        s[3,1,0]=(0.652359792164)*x[0]**o+(0.663033146373)*x[0]+(-0.0863743948576)*x[1]**o+(-0.310130231197)*x[1]
        s[3,1,1]=(0.132520829002)*x[0]**o+(-0.692607399006)*x[0]+(0.484454939316)*x[1]**o+(0.848314808163)*x[1]
        s[4,0,0]=(-0.0372302792179)*x[0]**o+(-0.208111754532)*x[0]+(-0.84060197077)*x[1]**o+(-0.999842431749)*x[1]
        s[4,0,1]=(0.522677717768)*x[0]**o+(-0.796530255947)*x[0]+(-0.758152917507)*x[1]**o+(-0.748249123242)*x[1]
        s[4,1,0]=(-0.9338587977)*x[0]**o+(0.033736711901)*x[0]+(-0.521414748153)*x[1]**o+(-0.27924576252)*x[1]
        s[4,1,1]=(-0.728280335859)*x[0]**o+(0.0052784852763)*x[0]+(0.668426662412)*x[1]**o+(-0.539548867925)*x[1]
        s[5,0,0]=(0.183081849687)*x[0]**o+(0.867616703079)*x[0]+(-0.44917069636)*x[1]**o+(-0.422840947463)*x[1]
        s[5,0,1]=(0.598683594731)*x[0]**o+(-0.185366147497)*x[0]+(-0.133722684999)*x[1]**o+(-0.374233145702)*x[1]
        s[5,1,0]=(-0.4661027909)*x[0]**o+(-0.658486427256)*x[0]+(0.494385097276)*x[1]**o+(0.746131625723)*x[1]
        s[5,1,1]=(0.881329425205)*x[0]**o+(0.0663586900787)*x[0]+(0.638121309568)*x[1]**o+(-0.392008079426)*x[1]
        ref[0,0,0,0]=o*(0.210810831144)*x_ref[0]**(o-1)+(0.993748550706)
        ref[0,0,0,1]=o*(0.225093899339)*x_ref[1]**(o-1)+(-0.306898024507)
        ref[0,0,1,0]=o*(0.945391291392)*x_ref[0]**(o-1)+(-0.34733930643)
        ref[0,0,1,1]=o*(0.523085690915)*x_ref[1]**(o-1)+(-0.332680309461)
        ref[0,1,0,0]=o*(0.372088608669)*x_ref[0]**(o-1)+(0.276443856788)
        ref[0,1,0,1]=o*(-0.439514323588)*x_ref[1]**(o-1)+(0.985453767485)
        ref[0,1,1,0]=o*(-0.967561100768)*x_ref[0]**(o-1)+(0.591908531375)
        ref[0,1,1,1]=o*(0.288385284239)*x_ref[1]**(o-1)+(-0.422883693702)
        ref[1,0,0,0]=o*(-0.735303276087)*x_ref[0]**(o-1)+(0.540176339329)
        ref[1,0,0,1]=o*(0.124141197609)*x_ref[1]**(o-1)+(0.17505950278)
        ref[1,0,1,0]=o*(0.799321447619)*x_ref[0]**(o-1)+(-0.521267395747)
        ref[1,0,1,1]=o*(-0.388235024602)*x_ref[1]**(o-1)+(0.0783682529975)
        ref[1,1,0,0]=o*(-0.7923799998)*x_ref[0]**(o-1)+(-0.779345662449)
        ref[1,1,0,1]=o*(0.507399627325)*x_ref[1]**(o-1)+(-0.668039939194)
        ref[1,1,1,0]=o*(0.360260485497)*x_ref[0]**(o-1)+(0.639639208398)
        ref[1,1,1,1]=o*(0.275791805772)*x_ref[1]**(o-1)+(-0.923288698445)
        ref[2,0,0,0]=o*(0.338586096766)*x_ref[0]**(o-1)+(-0.862613705959)
        ref[2,0,0,1]=o*(0.0321867760579)*x_ref[1]**(o-1)+(0.418774919375)
        ref[2,0,1,0]=o*(0.0363215417361)*x_ref[0]**(o-1)+(0.994712078504)
        ref[2,0,1,1]=o*(-0.260614790561)*x_ref[1]**(o-1)+(-0.969097035084)
        ref[2,1,0,0]=o*(0.375366661281)*x_ref[0]**(o-1)+(0.456874530834)
        ref[2,1,0,1]=o*(0.429399840953)*x_ref[1]**(o-1)+(0.688205075709)
        ref[2,1,1,0]=o*(-0.918074889111)*x_ref[0]**(o-1)+(-0.716561240269)
        ref[2,1,1,1]=o*(-0.477398945222)*x_ref[1]**(o-1)+(-0.223142905734)
        ref[3,0,0,0]=o*(-0.420424618942)*x_ref[0]**(o-1)+(-0.702691949655)
        ref[3,0,0,1]=o*(-0.389998593814)*x_ref[1]**(o-1)+(0.778091583668)
        ref[3,0,1,0]=o*(-0.500317522801)*x_ref[0]**(o-1)+(-0.906158794184)
        ref[3,0,1,1]=o*(-0.942667332637)*x_ref[1]**(o-1)+(-0.38695660361)
        ref[3,1,0,0]=o*(0.652359792164)*x_ref[0]**(o-1)+(0.663033146373)
        ref[3,1,0,1]=o*(-0.0863743948576)*x_ref[1]**(o-1)+(-0.310130231197)
        ref[3,1,1,0]=o*(0.132520829002)*x_ref[0]**(o-1)+(-0.692607399006)
        ref[3,1,1,1]=o*(0.484454939316)*x_ref[1]**(o-1)+(0.848314808163)
        ref[4,0,0,0]=o*(-0.0372302792179)*x_ref[0]**(o-1)+(-0.208111754532)
        ref[4,0,0,1]=o*(-0.84060197077)*x_ref[1]**(o-1)+(-0.999842431749)
        ref[4,0,1,0]=o*(0.522677717768)*x_ref[0]**(o-1)+(-0.796530255947)
        ref[4,0,1,1]=o*(-0.758152917507)*x_ref[1]**(o-1)+(-0.748249123242)
        ref[4,1,0,0]=o*(-0.9338587977)*x_ref[0]**(o-1)+(0.033736711901)
        ref[4,1,0,1]=o*(-0.521414748153)*x_ref[1]**(o-1)+(-0.27924576252)
        ref[4,1,1,0]=o*(-0.728280335859)*x_ref[0]**(o-1)+(0.0052784852763)
        ref[4,1,1,1]=o*(0.668426662412)*x_ref[1]**(o-1)+(-0.539548867925)
        ref[5,0,0,0]=o*(0.183081849687)*x_ref[0]**(o-1)+(0.867616703079)
        ref[5,0,0,1]=o*(-0.44917069636)*x_ref[1]**(o-1)+(-0.422840947463)
        ref[5,0,1,0]=o*(0.598683594731)*x_ref[0]**(o-1)+(-0.185366147497)
        ref[5,0,1,1]=o*(-0.133722684999)*x_ref[1]**(o-1)+(-0.374233145702)
        ref[5,1,0,0]=o*(-0.4661027909)*x_ref[0]**(o-1)+(-0.658486427256)
        ref[5,1,0,1]=o*(0.494385097276)*x_ref[1]**(o-1)+(0.746131625723)
        ref[5,1,1,0]=o*(0.881329425205)*x_ref[0]**(o-1)+(0.0663586900787)
        ref[5,1,1,1]=o*(0.638121309568)*x_ref[1]**(o-1)+(-0.392008079426)
      else:
        s[0,0,0]=(0.495604153512)*x[0]**o+(-0.546051759361)*x[0]+(0.985443531442)*x[1]**o+(-0.394922383292)*x[1]+(-0.122301738852)*x[2]**o+(-0.221144495446)*x[2]
        s[0,0,1]=(-0.164179695077)*x[0]**o+(-0.488047064324)*x[0]+(0.556965694466)*x[1]**o+(-0.470423249373)*x[1]+(-0.625307475886)*x[2]**o+(-0.999651290798)*x[2]
        s[0,1,0]=(-0.858044039365)*x[0]**o+(-0.932289002735)*x[0]+(-0.343121624834)*x[1]**o+(-0.0210069830662)*x[1]+(0.951213562883)*x[2]**o+(-0.647040549286)*x[2]
        s[0,1,1]=(-0.680293757762)*x[0]**o+(0.820233518831)*x[0]+(0.0489418677095)*x[1]**o+(-0.852104985889)*x[1]+(0.960135497008)*x[2]**o+(-0.46075359959)*x[2]
        s[1,0,0]=(0.827213065477)*x[0]**o+(0.195435158962)*x[0]+(-0.0408345903614)*x[1]**o+(0.146957170071)*x[1]+(-0.423482853682)*x[2]**o+(0.113246501514)*x[2]
        s[1,0,1]=(-0.920613863373)*x[0]**o+(0.933412308922)*x[0]+(0.258934140982)*x[1]**o+(-0.689643171121)*x[1]+(0.893192051888)*x[2]**o+(-0.471852670906)*x[2]
        s[1,1,0]=(-0.981601898267)*x[0]**o+(-0.516390752771)*x[0]+(0.608397090998)*x[1]**o+(0.335529495102)*x[1]+(-0.0746490251171)*x[2]**o+(-0.50249402227)*x[2]
        s[1,1,1]=(0.367749783351)*x[0]**o+(-0.611995716855)*x[0]+(-0.980913863186)*x[1]**o+(0.299812200148)*x[1]+(0.562250470266)*x[2]**o+(0.0542265856797)*x[2]
        s[2,0,0]=(-0.317633103186)*x[0]**o+(-0.175141419305)*x[0]+(-0.114479468171)*x[1]**o+(-0.117966705382)*x[1]+(-0.983395580005)*x[2]**o+(-0.588457148786)*x[2]
        s[2,0,1]=(-0.45007739654)*x[0]**o+(-0.941613604155)*x[0]+(0.597714724696)*x[1]**o+(0.992095144994)*x[1]+(0.681322168708)*x[2]**o+(0.656828773187)*x[2]
        s[2,1,0]=(0.372959242179)*x[0]**o+(-0.266039538769)*x[0]+(0.420734431733)*x[1]**o+(0.530880738131)*x[1]+(0.593665348543)*x[2]**o+(-0.502919021867)*x[2]
        s[2,1,1]=(0.741984988727)*x[0]**o+(-0.662109553439)*x[0]+(-0.01222805343)*x[1]**o+(-0.261877080281)*x[1]+(0.584778610634)*x[2]**o+(0.248969331325)*x[2]
        s[3,0,0]=(-0.767798213786)*x[0]**o+(0.56387037017)*x[0]+(0.545042362526)*x[1]**o+(0.127002206725)*x[1]+(0.123826212114)*x[2]**o+(0.03202953512)*x[2]
        s[3,0,1]=(-0.54234471198)*x[0]**o+(-0.844655061807)*x[0]+(0.909493468686)*x[1]**o+(-0.601003138714)*x[1]+(-0.822776025243)*x[2]**o+(-0.569619092404)*x[2]
        s[3,1,0]=(-0.749598491666)*x[0]**o+(0.146640505619)*x[0]+(-0.782827623949)*x[1]**o+(0.0795239577275)*x[1]+(0.98951419773)*x[2]**o+(-0.878832103203)*x[2]
        s[3,1,1]=(0.917489698949)*x[0]**o+(0.791885047319)*x[0]+(0.415646018849)*x[1]**o+(0.194622530403)*x[1]+(0.123045865981)*x[2]**o+(-0.864537623753)*x[2]
        s[4,0,0]=(0.993338538326)*x[0]**o+(-0.130375655736)*x[0]+(0.088857735361)*x[1]**o+(-0.565025291359)*x[1]+(-0.543615061477)*x[2]**o+(0.509734903091)*x[2]
        s[4,0,1]=(-0.874483507391)*x[0]**o+(-0.993080113589)*x[0]+(0.269443230092)*x[1]**o+(-0.445853491314)*x[1]+(-0.765796954415)*x[2]**o+(-0.981106922583)*x[2]
        s[4,1,0]=(-0.650170302308)*x[0]**o+(0.222814951659)*x[0]+(0.553055011394)*x[1]**o+(-0.359146840173)*x[1]+(0.887910478865)*x[2]**o+(-0.446130800509)*x[2]
        s[4,1,1]=(0.512373070036)*x[0]**o+(0.427326569369)*x[0]+(0.311737845037)*x[1]**o+(0.998455706258)*x[1]+(-0.868170359085)*x[2]**o+(-0.924351117942)*x[2]
        s[5,0,0]=(0.129707961614)*x[0]**o+(0.668313579165)*x[0]+(-0.73474211614)*x[1]**o+(0.702827908609)*x[1]+(-0.0226884665962)*x[2]**o+(-0.00740206443655)*x[2]
        s[5,0,1]=(-0.113099473775)*x[0]**o+(0.655719358276)*x[0]+(0.0162829520583)*x[1]**o+(0.133582158275)*x[1]+(0.169685631206)*x[2]**o+(0.507126023907)*x[2]
        s[5,1,0]=(-0.0754735940537)*x[0]**o+(0.294940113895)*x[0]+(-0.98735456969)*x[1]**o+(0.401871193336)*x[1]+(-0.0770486468616)*x[2]**o+(-0.0168685832006)*x[2]
        s[5,1,1]=(0.971769998957)*x[0]**o+(-0.454726501195)*x[0]+(0.835505309792)*x[1]**o+(0.775091315278)*x[1]+(0.632907414827)*x[2]**o+(-0.10388782338)*x[2]
        ref[0,0,0,0]=o*(0.495604153512)*x_ref[0]**(o-1)+(-0.546051759361)
        ref[0,0,0,1]=o*(0.985443531442)*x_ref[1]**(o-1)+(-0.394922383292)
        ref[0,0,0,2]=o*(-0.122301738852)*x_ref[2]**(o-1)+(-0.221144495446)
        ref[0,0,1,0]=o*(-0.164179695077)*x_ref[0]**(o-1)+(-0.488047064324)
        ref[0,0,1,1]=o*(0.556965694466)*x_ref[1]**(o-1)+(-0.470423249373)
        ref[0,0,1,2]=o*(-0.625307475886)*x_ref[2]**(o-1)+(-0.999651290798)
        ref[0,1,0,0]=o*(-0.858044039365)*x_ref[0]**(o-1)+(-0.932289002735)
        ref[0,1,0,1]=o*(-0.343121624834)*x_ref[1]**(o-1)+(-0.0210069830662)
        ref[0,1,0,2]=o*(0.951213562883)*x_ref[2]**(o-1)+(-0.647040549286)
        ref[0,1,1,0]=o*(-0.680293757762)*x_ref[0]**(o-1)+(0.820233518831)
        ref[0,1,1,1]=o*(0.0489418677095)*x_ref[1]**(o-1)+(-0.852104985889)
        ref[0,1,1,2]=o*(0.960135497008)*x_ref[2]**(o-1)+(-0.46075359959)
        ref[1,0,0,0]=o*(0.827213065477)*x_ref[0]**(o-1)+(0.195435158962)
        ref[1,0,0,1]=o*(-0.0408345903614)*x_ref[1]**(o-1)+(0.146957170071)
        ref[1,0,0,2]=o*(-0.423482853682)*x_ref[2]**(o-1)+(0.113246501514)
        ref[1,0,1,0]=o*(-0.920613863373)*x_ref[0]**(o-1)+(0.933412308922)
        ref[1,0,1,1]=o*(0.258934140982)*x_ref[1]**(o-1)+(-0.689643171121)
        ref[1,0,1,2]=o*(0.893192051888)*x_ref[2]**(o-1)+(-0.471852670906)
        ref[1,1,0,0]=o*(-0.981601898267)*x_ref[0]**(o-1)+(-0.516390752771)
        ref[1,1,0,1]=o*(0.608397090998)*x_ref[1]**(o-1)+(0.335529495102)
        ref[1,1,0,2]=o*(-0.0746490251171)*x_ref[2]**(o-1)+(-0.50249402227)
        ref[1,1,1,0]=o*(0.367749783351)*x_ref[0]**(o-1)+(-0.611995716855)
        ref[1,1,1,1]=o*(-0.980913863186)*x_ref[1]**(o-1)+(0.299812200148)
        ref[1,1,1,2]=o*(0.562250470266)*x_ref[2]**(o-1)+(0.0542265856797)
        ref[2,0,0,0]=o*(-0.317633103186)*x_ref[0]**(o-1)+(-0.175141419305)
        ref[2,0,0,1]=o*(-0.114479468171)*x_ref[1]**(o-1)+(-0.117966705382)
        ref[2,0,0,2]=o*(-0.983395580005)*x_ref[2]**(o-1)+(-0.588457148786)
        ref[2,0,1,0]=o*(-0.45007739654)*x_ref[0]**(o-1)+(-0.941613604155)
        ref[2,0,1,1]=o*(0.597714724696)*x_ref[1]**(o-1)+(0.992095144994)
        ref[2,0,1,2]=o*(0.681322168708)*x_ref[2]**(o-1)+(0.656828773187)
        ref[2,1,0,0]=o*(0.372959242179)*x_ref[0]**(o-1)+(-0.266039538769)
        ref[2,1,0,1]=o*(0.420734431733)*x_ref[1]**(o-1)+(0.530880738131)
        ref[2,1,0,2]=o*(0.593665348543)*x_ref[2]**(o-1)+(-0.502919021867)
        ref[2,1,1,0]=o*(0.741984988727)*x_ref[0]**(o-1)+(-0.662109553439)
        ref[2,1,1,1]=o*(-0.01222805343)*x_ref[1]**(o-1)+(-0.261877080281)
        ref[2,1,1,2]=o*(0.584778610634)*x_ref[2]**(o-1)+(0.248969331325)
        ref[3,0,0,0]=o*(-0.767798213786)*x_ref[0]**(o-1)+(0.56387037017)
        ref[3,0,0,1]=o*(0.545042362526)*x_ref[1]**(o-1)+(0.127002206725)
        ref[3,0,0,2]=o*(0.123826212114)*x_ref[2]**(o-1)+(0.03202953512)
        ref[3,0,1,0]=o*(-0.54234471198)*x_ref[0]**(o-1)+(-0.844655061807)
        ref[3,0,1,1]=o*(0.909493468686)*x_ref[1]**(o-1)+(-0.601003138714)
        ref[3,0,1,2]=o*(-0.822776025243)*x_ref[2]**(o-1)+(-0.569619092404)
        ref[3,1,0,0]=o*(-0.749598491666)*x_ref[0]**(o-1)+(0.146640505619)
        ref[3,1,0,1]=o*(-0.782827623949)*x_ref[1]**(o-1)+(0.0795239577275)
        ref[3,1,0,2]=o*(0.98951419773)*x_ref[2]**(o-1)+(-0.878832103203)
        ref[3,1,1,0]=o*(0.917489698949)*x_ref[0]**(o-1)+(0.791885047319)
        ref[3,1,1,1]=o*(0.415646018849)*x_ref[1]**(o-1)+(0.194622530403)
        ref[3,1,1,2]=o*(0.123045865981)*x_ref[2]**(o-1)+(-0.864537623753)
        ref[4,0,0,0]=o*(0.993338538326)*x_ref[0]**(o-1)+(-0.130375655736)
        ref[4,0,0,1]=o*(0.088857735361)*x_ref[1]**(o-1)+(-0.565025291359)
        ref[4,0,0,2]=o*(-0.543615061477)*x_ref[2]**(o-1)+(0.509734903091)
        ref[4,0,1,0]=o*(-0.874483507391)*x_ref[0]**(o-1)+(-0.993080113589)
        ref[4,0,1,1]=o*(0.269443230092)*x_ref[1]**(o-1)+(-0.445853491314)
        ref[4,0,1,2]=o*(-0.765796954415)*x_ref[2]**(o-1)+(-0.981106922583)
        ref[4,1,0,0]=o*(-0.650170302308)*x_ref[0]**(o-1)+(0.222814951659)
        ref[4,1,0,1]=o*(0.553055011394)*x_ref[1]**(o-1)+(-0.359146840173)
        ref[4,1,0,2]=o*(0.887910478865)*x_ref[2]**(o-1)+(-0.446130800509)
        ref[4,1,1,0]=o*(0.512373070036)*x_ref[0]**(o-1)+(0.427326569369)
        ref[4,1,1,1]=o*(0.311737845037)*x_ref[1]**(o-1)+(0.998455706258)
        ref[4,1,1,2]=o*(-0.868170359085)*x_ref[2]**(o-1)+(-0.924351117942)
        ref[5,0,0,0]=o*(0.129707961614)*x_ref[0]**(o-1)+(0.668313579165)
        ref[5,0,0,1]=o*(-0.73474211614)*x_ref[1]**(o-1)+(0.702827908609)
        ref[5,0,0,2]=o*(-0.0226884665962)*x_ref[2]**(o-1)+(-0.00740206443655)
        ref[5,0,1,0]=o*(-0.113099473775)*x_ref[0]**(o-1)+(0.655719358276)
        ref[5,0,1,1]=o*(0.0162829520583)*x_ref[1]**(o-1)+(0.133582158275)
        ref[5,0,1,2]=o*(0.169685631206)*x_ref[2]**(o-1)+(0.507126023907)
        ref[5,1,0,0]=o*(-0.0754735940537)*x_ref[0]**(o-1)+(0.294940113895)
        ref[5,1,0,1]=o*(-0.98735456969)*x_ref[1]**(o-1)+(0.401871193336)
        ref[5,1,0,2]=o*(-0.0770486468616)*x_ref[2]**(o-1)+(-0.0168685832006)
        ref[5,1,1,0]=o*(0.971769998957)*x_ref[0]**(o-1)+(-0.454726501195)
        ref[5,1,1,1]=o*(0.835505309792)*x_ref[1]**(o-1)+(0.775091315278)
        ref[5,1,1,2]=o*(0.632907414827)*x_ref[2]**(o-1)+(-0.10388782338)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_Solution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.143576812018)*x[0]**o+(0.772046830562)*x[0]+(0.799382081787)*x[1]**o+(-0.689145149202)*x[1]
        ref[0]=o*(0.143576812018)*x_ref[0]**(o-1)+(0.772046830562)
        ref[1]=o*(0.799382081787)*x_ref[1]**(o-1)+(-0.689145149202)
      else:
        s=(-0.921622979189)*x[0]**o+(-0.35132068829)*x[0]+(-0.668419889938)*x[1]**o+(-0.25240554797)*x[1]+(-0.266863972981)*x[2]**o+(0.59405466698)*x[2]
        ref[0]=o*(-0.921622979189)*x_ref[0]**(o-1)+(-0.35132068829)
        ref[1]=o*(-0.668419889938)*x_ref[1]**(o-1)+(-0.25240554797)
        ref[2]=o*(-0.266863972981)*x_ref[2]**(o-1)+(0.59405466698)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_Solution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.572981720293)*x[0]**o+(-0.305280287068)*x[0]+(-0.552369316063)*x[1]**o+(-0.917368778197)*x[1]
        s[1]=(0.310389604089)*x[0]**o+(-0.316385112396)*x[0]+(0.713272118168)*x[1]**o+(0.856686290785)*x[1]
        ref[0,0]=o*(0.572981720293)*x_ref[0]**(o-1)+(-0.305280287068)
        ref[0,1]=o*(-0.552369316063)*x_ref[1]**(o-1)+(-0.917368778197)
        ref[1,0]=o*(0.310389604089)*x_ref[0]**(o-1)+(-0.316385112396)
        ref[1,1]=o*(0.713272118168)*x_ref[1]**(o-1)+(0.856686290785)
      else:
        s[0]=(-0.132099402207)*x[0]**o+(0.587263843257)*x[0]+(-0.326050434166)*x[1]**o+(0.873048232418)*x[1]+(-0.141344874565)*x[2]**o+(0.274084100427)*x[2]
        s[1]=(0.372494974451)*x[0]**o+(-0.482077621852)*x[0]+(-0.022004253515)*x[1]**o+(0.874572230513)*x[1]+(0.8369124866)*x[2]**o+(-0.979759147043)*x[2]
        ref[0,0]=o*(-0.132099402207)*x_ref[0]**(o-1)+(0.587263843257)
        ref[0,1]=o*(-0.326050434166)*x_ref[1]**(o-1)+(0.873048232418)
        ref[0,2]=o*(-0.141344874565)*x_ref[2]**(o-1)+(0.274084100427)
        ref[1,0]=o*(0.372494974451)*x_ref[0]**(o-1)+(-0.482077621852)
        ref[1,1]=o*(-0.022004253515)*x_ref[1]**(o-1)+(0.874572230513)
        ref[1,2]=o*(0.8369124866)*x_ref[2]**(o-1)+(-0.979759147043)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_Solution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.993111549259)*x[0]**o+(0.958601813395)*x[0]+(0.985388059548)*x[1]**o+(-0.570742940525)*x[1]
        s[0,1]=(-0.223411180873)*x[0]**o+(0.480695178439)*x[0]+(0.0161612686521)*x[1]**o+(0.400987902598)*x[1]
        s[0,2]=(0.775656253974)*x[0]**o+(-0.191138960618)*x[0]+(-0.446251149518)*x[1]**o+(0.432178603961)*x[1]
        s[0,3]=(0.827622744839)*x[0]**o+(0.0366172895102)*x[0]+(0.347810808477)*x[1]**o+(-0.688818913998)*x[1]
        s[0,4]=(0.837082711466)*x[0]**o+(0.376527432997)*x[0]+(0.17741245651)*x[1]**o+(0.872147279603)*x[1]
        s[1,0]=(-0.590137160663)*x[0]**o+(-0.0703615079757)*x[0]+(0.229861545725)*x[1]**o+(0.0809472412982)*x[1]
        s[1,1]=(0.0223830240489)*x[0]**o+(0.228842693495)*x[0]+(-0.557074035399)*x[1]**o+(0.500538497902)*x[1]
        s[1,2]=(0.872181981766)*x[0]**o+(0.790601719428)*x[0]+(0.983055567067)*x[1]**o+(-0.353849465384)*x[1]
        s[1,3]=(0.037040061131)*x[0]**o+(0.293981509756)*x[0]+(-0.279061759005)*x[1]**o+(-0.362644845866)*x[1]
        s[1,4]=(0.695941674104)*x[0]**o+(-0.51481315942)*x[0]+(0.765262082274)*x[1]**o+(-0.813116917911)*x[1]
        s[2,0]=(-0.579924169032)*x[0]**o+(0.529857498501)*x[0]+(-0.485940906658)*x[1]**o+(0.559056145345)*x[1]
        s[2,1]=(0.604856738274)*x[0]**o+(-0.440825061381)*x[0]+(-0.0838758185405)*x[1]**o+(0.0188329882604)*x[1]
        s[2,2]=(0.118585827905)*x[0]**o+(0.618367201987)*x[0]+(0.352157918285)*x[1]**o+(0.253890253557)*x[1]
        s[2,3]=(-0.372665302477)*x[0]**o+(0.718734712755)*x[0]+(-0.487897528386)*x[1]**o+(0.808964263735)*x[1]
        s[2,4]=(0.776070811457)*x[0]**o+(-0.675210963521)*x[0]+(0.0331035702593)*x[1]**o+(-0.703100978257)*x[1]
        s[3,0]=(0.144238364778)*x[0]**o+(0.855900329896)*x[0]+(-0.291688407398)*x[1]**o+(-0.308539252205)*x[1]
        s[3,1]=(-0.57873830064)*x[0]**o+(-0.0982142438835)*x[0]+(0.249356200194)*x[1]**o+(-0.156027166502)*x[1]
        s[3,2]=(-0.70772301011)*x[0]**o+(0.615393644935)*x[0]+(-0.1313826883)*x[1]**o+(0.265287078264)*x[1]
        s[3,3]=(-0.671189805427)*x[0]**o+(-0.561302098778)*x[0]+(0.129817024131)*x[1]**o+(-0.738582319997)*x[1]
        s[3,4]=(-0.154389666509)*x[0]**o+(0.119040072318)*x[0]+(-0.113706823809)*x[1]**o+(0.0124037512924)*x[1]
        ref[0,0,0]=o*(-0.993111549259)*x_ref[0]**(o-1)+(0.958601813395)
        ref[0,0,1]=o*(0.985388059548)*x_ref[1]**(o-1)+(-0.570742940525)
        ref[0,1,0]=o*(-0.223411180873)*x_ref[0]**(o-1)+(0.480695178439)
        ref[0,1,1]=o*(0.0161612686521)*x_ref[1]**(o-1)+(0.400987902598)
        ref[0,2,0]=o*(0.775656253974)*x_ref[0]**(o-1)+(-0.191138960618)
        ref[0,2,1]=o*(-0.446251149518)*x_ref[1]**(o-1)+(0.432178603961)
        ref[0,3,0]=o*(0.827622744839)*x_ref[0]**(o-1)+(0.0366172895102)
        ref[0,3,1]=o*(0.347810808477)*x_ref[1]**(o-1)+(-0.688818913998)
        ref[0,4,0]=o*(0.837082711466)*x_ref[0]**(o-1)+(0.376527432997)
        ref[0,4,1]=o*(0.17741245651)*x_ref[1]**(o-1)+(0.872147279603)
        ref[1,0,0]=o*(-0.590137160663)*x_ref[0]**(o-1)+(-0.0703615079757)
        ref[1,0,1]=o*(0.229861545725)*x_ref[1]**(o-1)+(0.0809472412982)
        ref[1,1,0]=o*(0.0223830240489)*x_ref[0]**(o-1)+(0.228842693495)
        ref[1,1,1]=o*(-0.557074035399)*x_ref[1]**(o-1)+(0.500538497902)
        ref[1,2,0]=o*(0.872181981766)*x_ref[0]**(o-1)+(0.790601719428)
        ref[1,2,1]=o*(0.983055567067)*x_ref[1]**(o-1)+(-0.353849465384)
        ref[1,3,0]=o*(0.037040061131)*x_ref[0]**(o-1)+(0.293981509756)
        ref[1,3,1]=o*(-0.279061759005)*x_ref[1]**(o-1)+(-0.362644845866)
        ref[1,4,0]=o*(0.695941674104)*x_ref[0]**(o-1)+(-0.51481315942)
        ref[1,4,1]=o*(0.765262082274)*x_ref[1]**(o-1)+(-0.813116917911)
        ref[2,0,0]=o*(-0.579924169032)*x_ref[0]**(o-1)+(0.529857498501)
        ref[2,0,1]=o*(-0.485940906658)*x_ref[1]**(o-1)+(0.559056145345)
        ref[2,1,0]=o*(0.604856738274)*x_ref[0]**(o-1)+(-0.440825061381)
        ref[2,1,1]=o*(-0.0838758185405)*x_ref[1]**(o-1)+(0.0188329882604)
        ref[2,2,0]=o*(0.118585827905)*x_ref[0]**(o-1)+(0.618367201987)
        ref[2,2,1]=o*(0.352157918285)*x_ref[1]**(o-1)+(0.253890253557)
        ref[2,3,0]=o*(-0.372665302477)*x_ref[0]**(o-1)+(0.718734712755)
        ref[2,3,1]=o*(-0.487897528386)*x_ref[1]**(o-1)+(0.808964263735)
        ref[2,4,0]=o*(0.776070811457)*x_ref[0]**(o-1)+(-0.675210963521)
        ref[2,4,1]=o*(0.0331035702593)*x_ref[1]**(o-1)+(-0.703100978257)
        ref[3,0,0]=o*(0.144238364778)*x_ref[0]**(o-1)+(0.855900329896)
        ref[3,0,1]=o*(-0.291688407398)*x_ref[1]**(o-1)+(-0.308539252205)
        ref[3,1,0]=o*(-0.57873830064)*x_ref[0]**(o-1)+(-0.0982142438835)
        ref[3,1,1]=o*(0.249356200194)*x_ref[1]**(o-1)+(-0.156027166502)
        ref[3,2,0]=o*(-0.70772301011)*x_ref[0]**(o-1)+(0.615393644935)
        ref[3,2,1]=o*(-0.1313826883)*x_ref[1]**(o-1)+(0.265287078264)
        ref[3,3,0]=o*(-0.671189805427)*x_ref[0]**(o-1)+(-0.561302098778)
        ref[3,3,1]=o*(0.129817024131)*x_ref[1]**(o-1)+(-0.738582319997)
        ref[3,4,0]=o*(-0.154389666509)*x_ref[0]**(o-1)+(0.119040072318)
        ref[3,4,1]=o*(-0.113706823809)*x_ref[1]**(o-1)+(0.0124037512924)
      else:
        s[0,0]=(-0.829816450667)*x[0]**o+(-0.147235008007)*x[0]+(0.492783189082)*x[1]**o+(-0.512635666132)*x[1]+(0.890224791682)*x[2]**o+(-0.956119232758)*x[2]
        s[0,1]=(0.674209146363)*x[0]**o+(-0.922409931962)*x[0]+(-0.328441886277)*x[1]**o+(0.981094102408)*x[1]+(0.481174182835)*x[2]**o+(-0.44590959358)*x[2]
        s[0,2]=(0.695494078474)*x[0]**o+(-0.0618250391783)*x[0]+(0.731471533841)*x[1]**o+(-0.71550566694)*x[1]+(-0.239535873248)*x[2]**o+(-0.763157808991)*x[2]
        s[0,3]=(0.754158913667)*x[0]**o+(-0.603448930533)*x[0]+(-0.200959580291)*x[1]**o+(0.298079100922)*x[1]+(0.15701085915)*x[2]**o+(0.18247341435)*x[2]
        s[0,4]=(0.894484225202)*x[0]**o+(0.908683878491)*x[0]+(0.452273398852)*x[1]**o+(0.446075531002)*x[1]+(-0.527120277613)*x[2]**o+(0.0825220159349)*x[2]
        s[1,0]=(0.1117942579)*x[0]**o+(0.616938000848)*x[0]+(-0.490920496055)*x[1]**o+(-0.46576972952)*x[1]+(-0.0234475725539)*x[2]**o+(-0.762964610488)*x[2]
        s[1,1]=(0.10779128676)*x[0]**o+(0.323623835098)*x[0]+(-0.56638095467)*x[1]**o+(-0.485758771921)*x[1]+(0.149606176158)*x[2]**o+(0.0163417655889)*x[2]
        s[1,2]=(-0.220849737003)*x[0]**o+(-0.269143304567)*x[0]+(0.53654939437)*x[1]**o+(0.434383033636)*x[1]+(0.474884241083)*x[2]**o+(-0.0619940742667)*x[2]
        s[1,3]=(0.789745098309)*x[0]**o+(-0.882265656942)*x[0]+(-0.513103080365)*x[1]**o+(0.461829232085)*x[1]+(-0.762923723193)*x[2]**o+(0.300345919163)*x[2]
        s[1,4]=(0.225139496075)*x[0]**o+(0.254192406789)*x[0]+(-0.984611398843)*x[1]**o+(-0.519021894131)*x[1]+(0.916659347214)*x[2]**o+(0.466927379869)*x[2]
        s[2,0]=(-0.404728214636)*x[0]**o+(-0.0890438316072)*x[0]+(-0.97035417582)*x[1]**o+(0.202361572805)*x[1]+(-0.540497194191)*x[2]**o+(-0.80807606235)*x[2]
        s[2,1]=(0.140494227071)*x[0]**o+(0.757504635086)*x[0]+(0.831217423619)*x[1]**o+(-0.647503683145)*x[1]+(0.821831665921)*x[2]**o+(-0.121451056151)*x[2]
        s[2,2]=(-0.962347494034)*x[0]**o+(0.633512101597)*x[0]+(-0.828226238758)*x[1]**o+(0.968353578898)*x[1]+(-0.553755153946)*x[2]**o+(0.938911535056)*x[2]
        s[2,3]=(0.450492204938)*x[0]**o+(-0.475913651564)*x[0]+(-0.341560341273)*x[1]**o+(-0.923252138473)*x[1]+(0.17946107137)*x[2]**o+(0.337986869404)*x[2]
        s[2,4]=(0.217063428863)*x[0]**o+(0.702651458761)*x[0]+(-0.769522744816)*x[1]**o+(-0.231379966769)*x[1]+(-0.0579749690283)*x[2]**o+(-0.766607638409)*x[2]
        s[3,0]=(0.681151302876)*x[0]**o+(-0.29704479144)*x[0]+(0.537446794047)*x[1]**o+(0.872410938876)*x[1]+(-0.934271483068)*x[2]**o+(0.270440171512)*x[2]
        s[3,1]=(0.530874723279)*x[0]**o+(0.739974626875)*x[0]+(-0.240867894838)*x[1]**o+(0.625808519502)*x[1]+(0.747357445736)*x[2]**o+(-0.242935065692)*x[2]
        s[3,2]=(0.682623726875)*x[0]**o+(-0.34894979966)*x[0]+(-0.307826595564)*x[1]**o+(-0.869693179146)*x[1]+(-0.814910983419)*x[2]**o+(-0.98110829358)*x[2]
        s[3,3]=(0.232395211532)*x[0]**o+(0.504566395286)*x[0]+(-0.029022506301)*x[1]**o+(0.897710188132)*x[1]+(-0.107215395289)*x[2]**o+(-0.541445668914)*x[2]
        s[3,4]=(0.916061625636)*x[0]**o+(-0.39003337702)*x[0]+(0.926535863586)*x[1]**o+(0.585922306493)*x[1]+(-0.761902914251)*x[2]**o+(0.489922931455)*x[2]
        ref[0,0,0]=o*(-0.829816450667)*x_ref[0]**(o-1)+(-0.147235008007)
        ref[0,0,1]=o*(0.492783189082)*x_ref[1]**(o-1)+(-0.512635666132)
        ref[0,0,2]=o*(0.890224791682)*x_ref[2]**(o-1)+(-0.956119232758)
        ref[0,1,0]=o*(0.674209146363)*x_ref[0]**(o-1)+(-0.922409931962)
        ref[0,1,1]=o*(-0.328441886277)*x_ref[1]**(o-1)+(0.981094102408)
        ref[0,1,2]=o*(0.481174182835)*x_ref[2]**(o-1)+(-0.44590959358)
        ref[0,2,0]=o*(0.695494078474)*x_ref[0]**(o-1)+(-0.0618250391783)
        ref[0,2,1]=o*(0.731471533841)*x_ref[1]**(o-1)+(-0.71550566694)
        ref[0,2,2]=o*(-0.239535873248)*x_ref[2]**(o-1)+(-0.763157808991)
        ref[0,3,0]=o*(0.754158913667)*x_ref[0]**(o-1)+(-0.603448930533)
        ref[0,3,1]=o*(-0.200959580291)*x_ref[1]**(o-1)+(0.298079100922)
        ref[0,3,2]=o*(0.15701085915)*x_ref[2]**(o-1)+(0.18247341435)
        ref[0,4,0]=o*(0.894484225202)*x_ref[0]**(o-1)+(0.908683878491)
        ref[0,4,1]=o*(0.452273398852)*x_ref[1]**(o-1)+(0.446075531002)
        ref[0,4,2]=o*(-0.527120277613)*x_ref[2]**(o-1)+(0.0825220159349)
        ref[1,0,0]=o*(0.1117942579)*x_ref[0]**(o-1)+(0.616938000848)
        ref[1,0,1]=o*(-0.490920496055)*x_ref[1]**(o-1)+(-0.46576972952)
        ref[1,0,2]=o*(-0.0234475725539)*x_ref[2]**(o-1)+(-0.762964610488)
        ref[1,1,0]=o*(0.10779128676)*x_ref[0]**(o-1)+(0.323623835098)
        ref[1,1,1]=o*(-0.56638095467)*x_ref[1]**(o-1)+(-0.485758771921)
        ref[1,1,2]=o*(0.149606176158)*x_ref[2]**(o-1)+(0.0163417655889)
        ref[1,2,0]=o*(-0.220849737003)*x_ref[0]**(o-1)+(-0.269143304567)
        ref[1,2,1]=o*(0.53654939437)*x_ref[1]**(o-1)+(0.434383033636)
        ref[1,2,2]=o*(0.474884241083)*x_ref[2]**(o-1)+(-0.0619940742667)
        ref[1,3,0]=o*(0.789745098309)*x_ref[0]**(o-1)+(-0.882265656942)
        ref[1,3,1]=o*(-0.513103080365)*x_ref[1]**(o-1)+(0.461829232085)
        ref[1,3,2]=o*(-0.762923723193)*x_ref[2]**(o-1)+(0.300345919163)
        ref[1,4,0]=o*(0.225139496075)*x_ref[0]**(o-1)+(0.254192406789)
        ref[1,4,1]=o*(-0.984611398843)*x_ref[1]**(o-1)+(-0.519021894131)
        ref[1,4,2]=o*(0.916659347214)*x_ref[2]**(o-1)+(0.466927379869)
        ref[2,0,0]=o*(-0.404728214636)*x_ref[0]**(o-1)+(-0.0890438316072)
        ref[2,0,1]=o*(-0.97035417582)*x_ref[1]**(o-1)+(0.202361572805)
        ref[2,0,2]=o*(-0.540497194191)*x_ref[2]**(o-1)+(-0.80807606235)
        ref[2,1,0]=o*(0.140494227071)*x_ref[0]**(o-1)+(0.757504635086)
        ref[2,1,1]=o*(0.831217423619)*x_ref[1]**(o-1)+(-0.647503683145)
        ref[2,1,2]=o*(0.821831665921)*x_ref[2]**(o-1)+(-0.121451056151)
        ref[2,2,0]=o*(-0.962347494034)*x_ref[0]**(o-1)+(0.633512101597)
        ref[2,2,1]=o*(-0.828226238758)*x_ref[1]**(o-1)+(0.968353578898)
        ref[2,2,2]=o*(-0.553755153946)*x_ref[2]**(o-1)+(0.938911535056)
        ref[2,3,0]=o*(0.450492204938)*x_ref[0]**(o-1)+(-0.475913651564)
        ref[2,3,1]=o*(-0.341560341273)*x_ref[1]**(o-1)+(-0.923252138473)
        ref[2,3,2]=o*(0.17946107137)*x_ref[2]**(o-1)+(0.337986869404)
        ref[2,4,0]=o*(0.217063428863)*x_ref[0]**(o-1)+(0.702651458761)
        ref[2,4,1]=o*(-0.769522744816)*x_ref[1]**(o-1)+(-0.231379966769)
        ref[2,4,2]=o*(-0.0579749690283)*x_ref[2]**(o-1)+(-0.766607638409)
        ref[3,0,0]=o*(0.681151302876)*x_ref[0]**(o-1)+(-0.29704479144)
        ref[3,0,1]=o*(0.537446794047)*x_ref[1]**(o-1)+(0.872410938876)
        ref[3,0,2]=o*(-0.934271483068)*x_ref[2]**(o-1)+(0.270440171512)
        ref[3,1,0]=o*(0.530874723279)*x_ref[0]**(o-1)+(0.739974626875)
        ref[3,1,1]=o*(-0.240867894838)*x_ref[1]**(o-1)+(0.625808519502)
        ref[3,1,2]=o*(0.747357445736)*x_ref[2]**(o-1)+(-0.242935065692)
        ref[3,2,0]=o*(0.682623726875)*x_ref[0]**(o-1)+(-0.34894979966)
        ref[3,2,1]=o*(-0.307826595564)*x_ref[1]**(o-1)+(-0.869693179146)
        ref[3,2,2]=o*(-0.814910983419)*x_ref[2]**(o-1)+(-0.98110829358)
        ref[3,3,0]=o*(0.232395211532)*x_ref[0]**(o-1)+(0.504566395286)
        ref[3,3,1]=o*(-0.029022506301)*x_ref[1]**(o-1)+(0.897710188132)
        ref[3,3,2]=o*(-0.107215395289)*x_ref[2]**(o-1)+(-0.541445668914)
        ref[3,4,0]=o*(0.916061625636)*x_ref[0]**(o-1)+(-0.39003337702)
        ref[3,4,1]=o*(0.926535863586)*x_ref[1]**(o-1)+(0.585922306493)
        ref[3,4,2]=o*(-0.761902914251)*x_ref[2]**(o-1)+(0.489922931455)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_Solution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.401271003072)*x[0]**o+(0.110662634074)*x[0]+(0.00658452640581)*x[1]**o+(0.0961174317738)*x[1]
        s[0,0,1]=(-0.697194977122)*x[0]**o+(-0.202095576104)*x[0]+(0.533118580807)*x[1]**o+(-0.806666747117)*x[1]
        s[0,1,0]=(0.330322436592)*x[0]**o+(-0.476215030667)*x[0]+(0.435516874614)*x[1]**o+(-0.305867365755)*x[1]
        s[0,1,1]=(0.663350700331)*x[0]**o+(0.719421457458)*x[0]+(0.56853576783)*x[1]**o+(-0.63138691405)*x[1]
        s[1,0,0]=(-0.330411596581)*x[0]**o+(0.562930649846)*x[0]+(0.372532798726)*x[1]**o+(-0.444620881121)*x[1]
        s[1,0,1]=(-0.128926814439)*x[0]**o+(-0.60950153677)*x[0]+(-0.958182145974)*x[1]**o+(0.355162970063)*x[1]
        s[1,1,0]=(0.197818586294)*x[0]**o+(0.821517517641)*x[0]+(-0.701366738238)*x[1]**o+(-0.506092465749)*x[1]
        s[1,1,1]=(0.0313418487908)*x[0]**o+(0.161293069084)*x[0]+(0.18801678809)*x[1]**o+(0.691344434767)*x[1]
        s[2,0,0]=(-0.684508001816)*x[0]**o+(-0.341824405415)*x[0]+(-0.51105537282)*x[1]**o+(-0.291070626075)*x[1]
        s[2,0,1]=(0.401725066632)*x[0]**o+(0.279867637006)*x[0]+(-0.899862170932)*x[1]**o+(-0.442022413224)*x[1]
        s[2,1,0]=(0.185395646678)*x[0]**o+(0.0999907785823)*x[0]+(-0.577244369816)*x[1]**o+(-0.0346150541428)*x[1]
        s[2,1,1]=(0.216201017349)*x[0]**o+(-0.295200102743)*x[0]+(-0.306163867778)*x[1]**o+(0.0910745810644)*x[1]
        s[3,0,0]=(0.804134508197)*x[0]**o+(-0.213041659269)*x[0]+(0.157574172841)*x[1]**o+(0.296960896944)*x[1]
        s[3,0,1]=(0.268965957593)*x[0]**o+(0.0238464096944)*x[0]+(-0.00156155375937)*x[1]**o+(0.958073421842)*x[1]
        s[3,1,0]=(0.517287095459)*x[0]**o+(-0.0687040369948)*x[0]+(0.801953753488)*x[1]**o+(0.521595036445)*x[1]
        s[3,1,1]=(0.175159770635)*x[0]**o+(-0.736749683511)*x[0]+(-0.630382453304)*x[1]**o+(0.188706240505)*x[1]
        s[4,0,0]=(-0.724712993645)*x[0]**o+(-0.387274725616)*x[0]+(0.694395156995)*x[1]**o+(-0.401125937857)*x[1]
        s[4,0,1]=(-0.86282575637)*x[0]**o+(-0.966750714134)*x[0]+(0.130551499076)*x[1]**o+(-0.50519607351)*x[1]
        s[4,1,0]=(0.721837471072)*x[0]**o+(0.780750959264)*x[0]+(0.642470119645)*x[1]**o+(-0.158180439909)*x[1]
        s[4,1,1]=(0.451682933961)*x[0]**o+(-0.802341009866)*x[0]+(-0.106997197799)*x[1]**o+(0.212654933949)*x[1]
        s[5,0,0]=(0.437602747572)*x[0]**o+(0.420962577059)*x[0]+(0.977375479522)*x[1]**o+(-0.57876475638)*x[1]
        s[5,0,1]=(-0.653922496012)*x[0]**o+(-0.197873016501)*x[0]+(-0.494086552357)*x[1]**o+(-0.879089855341)*x[1]
        s[5,1,0]=(-0.466463766139)*x[0]**o+(-0.0287797105685)*x[0]+(0.163811113598)*x[1]**o+(-0.752758839578)*x[1]
        s[5,1,1]=(-0.442295358061)*x[0]**o+(-0.928365763003)*x[0]+(-0.998523766886)*x[1]**o+(-0.210043550397)*x[1]
        ref[0,0,0,0]=o*(-0.401271003072)*x_ref[0]**(o-1)+(0.110662634074)
        ref[0,0,0,1]=o*(0.00658452640581)*x_ref[1]**(o-1)+(0.0961174317738)
        ref[0,0,1,0]=o*(-0.697194977122)*x_ref[0]**(o-1)+(-0.202095576104)
        ref[0,0,1,1]=o*(0.533118580807)*x_ref[1]**(o-1)+(-0.806666747117)
        ref[0,1,0,0]=o*(0.330322436592)*x_ref[0]**(o-1)+(-0.476215030667)
        ref[0,1,0,1]=o*(0.435516874614)*x_ref[1]**(o-1)+(-0.305867365755)
        ref[0,1,1,0]=o*(0.663350700331)*x_ref[0]**(o-1)+(0.719421457458)
        ref[0,1,1,1]=o*(0.56853576783)*x_ref[1]**(o-1)+(-0.63138691405)
        ref[1,0,0,0]=o*(-0.330411596581)*x_ref[0]**(o-1)+(0.562930649846)
        ref[1,0,0,1]=o*(0.372532798726)*x_ref[1]**(o-1)+(-0.444620881121)
        ref[1,0,1,0]=o*(-0.128926814439)*x_ref[0]**(o-1)+(-0.60950153677)
        ref[1,0,1,1]=o*(-0.958182145974)*x_ref[1]**(o-1)+(0.355162970063)
        ref[1,1,0,0]=o*(0.197818586294)*x_ref[0]**(o-1)+(0.821517517641)
        ref[1,1,0,1]=o*(-0.701366738238)*x_ref[1]**(o-1)+(-0.506092465749)
        ref[1,1,1,0]=o*(0.0313418487908)*x_ref[0]**(o-1)+(0.161293069084)
        ref[1,1,1,1]=o*(0.18801678809)*x_ref[1]**(o-1)+(0.691344434767)
        ref[2,0,0,0]=o*(-0.684508001816)*x_ref[0]**(o-1)+(-0.341824405415)
        ref[2,0,0,1]=o*(-0.51105537282)*x_ref[1]**(o-1)+(-0.291070626075)
        ref[2,0,1,0]=o*(0.401725066632)*x_ref[0]**(o-1)+(0.279867637006)
        ref[2,0,1,1]=o*(-0.899862170932)*x_ref[1]**(o-1)+(-0.442022413224)
        ref[2,1,0,0]=o*(0.185395646678)*x_ref[0]**(o-1)+(0.0999907785823)
        ref[2,1,0,1]=o*(-0.577244369816)*x_ref[1]**(o-1)+(-0.0346150541428)
        ref[2,1,1,0]=o*(0.216201017349)*x_ref[0]**(o-1)+(-0.295200102743)
        ref[2,1,1,1]=o*(-0.306163867778)*x_ref[1]**(o-1)+(0.0910745810644)
        ref[3,0,0,0]=o*(0.804134508197)*x_ref[0]**(o-1)+(-0.213041659269)
        ref[3,0,0,1]=o*(0.157574172841)*x_ref[1]**(o-1)+(0.296960896944)
        ref[3,0,1,0]=o*(0.268965957593)*x_ref[0]**(o-1)+(0.0238464096944)
        ref[3,0,1,1]=o*(-0.00156155375937)*x_ref[1]**(o-1)+(0.958073421842)
        ref[3,1,0,0]=o*(0.517287095459)*x_ref[0]**(o-1)+(-0.0687040369948)
        ref[3,1,0,1]=o*(0.801953753488)*x_ref[1]**(o-1)+(0.521595036445)
        ref[3,1,1,0]=o*(0.175159770635)*x_ref[0]**(o-1)+(-0.736749683511)
        ref[3,1,1,1]=o*(-0.630382453304)*x_ref[1]**(o-1)+(0.188706240505)
        ref[4,0,0,0]=o*(-0.724712993645)*x_ref[0]**(o-1)+(-0.387274725616)
        ref[4,0,0,1]=o*(0.694395156995)*x_ref[1]**(o-1)+(-0.401125937857)
        ref[4,0,1,0]=o*(-0.86282575637)*x_ref[0]**(o-1)+(-0.966750714134)
        ref[4,0,1,1]=o*(0.130551499076)*x_ref[1]**(o-1)+(-0.50519607351)
        ref[4,1,0,0]=o*(0.721837471072)*x_ref[0]**(o-1)+(0.780750959264)
        ref[4,1,0,1]=o*(0.642470119645)*x_ref[1]**(o-1)+(-0.158180439909)
        ref[4,1,1,0]=o*(0.451682933961)*x_ref[0]**(o-1)+(-0.802341009866)
        ref[4,1,1,1]=o*(-0.106997197799)*x_ref[1]**(o-1)+(0.212654933949)
        ref[5,0,0,0]=o*(0.437602747572)*x_ref[0]**(o-1)+(0.420962577059)
        ref[5,0,0,1]=o*(0.977375479522)*x_ref[1]**(o-1)+(-0.57876475638)
        ref[5,0,1,0]=o*(-0.653922496012)*x_ref[0]**(o-1)+(-0.197873016501)
        ref[5,0,1,1]=o*(-0.494086552357)*x_ref[1]**(o-1)+(-0.879089855341)
        ref[5,1,0,0]=o*(-0.466463766139)*x_ref[0]**(o-1)+(-0.0287797105685)
        ref[5,1,0,1]=o*(0.163811113598)*x_ref[1]**(o-1)+(-0.752758839578)
        ref[5,1,1,0]=o*(-0.442295358061)*x_ref[0]**(o-1)+(-0.928365763003)
        ref[5,1,1,1]=o*(-0.998523766886)*x_ref[1]**(o-1)+(-0.210043550397)
      else:
        s[0,0,0]=(-0.130526786389)*x[0]**o+(-0.798097445808)*x[0]+(-0.679305651671)*x[1]**o+(-0.805797697874)*x[1]+(0.344947047592)*x[2]**o+(-0.835073945341)*x[2]
        s[0,0,1]=(-0.725780665135)*x[0]**o+(-0.427130507741)*x[0]+(-0.172129968106)*x[1]**o+(-0.329971568167)*x[1]+(0.430421618441)*x[2]**o+(0.850034235311)*x[2]
        s[0,1,0]=(-0.0894725266496)*x[0]**o+(-0.599803400866)*x[0]+(-0.689863844979)*x[1]**o+(-0.433762379607)*x[1]+(0.0585705966933)*x[2]**o+(0.932652014579)*x[2]
        s[0,1,1]=(0.351726580485)*x[0]**o+(-0.46650217284)*x[0]+(-0.389405597178)*x[1]**o+(0.702473795612)*x[1]+(0.971136943329)*x[2]**o+(-0.149952798522)*x[2]
        s[1,0,0]=(-0.687968216558)*x[0]**o+(0.918982435037)*x[0]+(-0.425841096866)*x[1]**o+(0.738050157283)*x[1]+(-0.468164706689)*x[2]**o+(0.524190937704)*x[2]
        s[1,0,1]=(-0.0190626564569)*x[0]**o+(0.158516496015)*x[0]+(-0.591775293572)*x[1]**o+(0.0791152697184)*x[1]+(-0.661850377953)*x[2]**o+(-0.705879414238)*x[2]
        s[1,1,0]=(-0.538932477038)*x[0]**o+(0.781931047594)*x[0]+(-0.799541079517)*x[1]**o+(0.218777285901)*x[1]+(-0.475448704708)*x[2]**o+(-0.655013532097)*x[2]
        s[1,1,1]=(0.0320772665819)*x[0]**o+(-0.577577235436)*x[0]+(-0.736321547926)*x[1]**o+(-0.133681853862)*x[1]+(-0.0573985073549)*x[2]**o+(0.955212063412)*x[2]
        s[2,0,0]=(-0.257879955226)*x[0]**o+(-0.994024413493)*x[0]+(-0.679188188792)*x[1]**o+(-0.410885730889)*x[1]+(0.704555870926)*x[2]**o+(0.518649077753)*x[2]
        s[2,0,1]=(-0.901488792722)*x[0]**o+(-0.222487929076)*x[0]+(0.165342576493)*x[1]**o+(-0.850214182574)*x[1]+(0.294760812505)*x[2]**o+(-0.0831507413484)*x[2]
        s[2,1,0]=(0.316904380947)*x[0]**o+(0.627318211545)*x[0]+(-0.216902145108)*x[1]**o+(-0.673577316246)*x[1]+(0.122931487815)*x[2]**o+(-0.0949502423641)*x[2]
        s[2,1,1]=(-0.602869040889)*x[0]**o+(0.190451027649)*x[0]+(0.367827882092)*x[1]**o+(0.686234993984)*x[1]+(0.653140976608)*x[2]**o+(-0.168486517196)*x[2]
        s[3,0,0]=(0.271612350046)*x[0]**o+(-0.827115599685)*x[0]+(-0.586600506988)*x[1]**o+(0.128300770664)*x[1]+(-0.922582907477)*x[2]**o+(0.334699215375)*x[2]
        s[3,0,1]=(-0.595897268553)*x[0]**o+(0.425141939316)*x[0]+(0.713120162361)*x[1]**o+(-0.680621044232)*x[1]+(0.749485690034)*x[2]**o+(0.209547675753)*x[2]
        s[3,1,0]=(-0.530398143666)*x[0]**o+(0.137850234083)*x[0]+(0.953276619653)*x[1]**o+(-0.578161670579)*x[1]+(0.371898896542)*x[2]**o+(-0.381331753222)*x[2]
        s[3,1,1]=(0.822298482068)*x[0]**o+(-0.405836518978)*x[0]+(-0.647621688423)*x[1]**o+(-0.672131868147)*x[1]+(-0.438867520359)*x[2]**o+(-0.423328072541)*x[2]
        s[4,0,0]=(0.132863753285)*x[0]**o+(0.397422994959)*x[0]+(-0.804707213822)*x[1]**o+(0.536531749119)*x[1]+(0.411705453279)*x[2]**o+(-0.169385048425)*x[2]
        s[4,0,1]=(0.0701228685651)*x[0]**o+(0.639241932031)*x[0]+(-0.638730875998)*x[1]**o+(-0.922035920621)*x[1]+(-0.113168863437)*x[2]**o+(0.813287574357)*x[2]
        s[4,1,0]=(0.100082349838)*x[0]**o+(0.133955906808)*x[0]+(-0.928623633632)*x[1]**o+(-0.424742167549)*x[1]+(0.0239194885597)*x[2]**o+(-0.672806207969)*x[2]
        s[4,1,1]=(0.829725063543)*x[0]**o+(0.678552027954)*x[0]+(-0.812625405832)*x[1]**o+(-0.810345951883)*x[1]+(-0.211595232946)*x[2]**o+(-0.984554474963)*x[2]
        s[5,0,0]=(0.339493504495)*x[0]**o+(-0.903993357652)*x[0]+(-0.440233624866)*x[1]**o+(-0.430746451761)*x[1]+(0.111254680616)*x[2]**o+(0.709038943317)*x[2]
        s[5,0,1]=(0.931029457116)*x[0]**o+(-0.785398109537)*x[0]+(-0.326970114573)*x[1]**o+(0.0384958937297)*x[1]+(-0.896259053345)*x[2]**o+(-0.277237800601)*x[2]
        s[5,1,0]=(0.608509355178)*x[0]**o+(0.585955903578)*x[0]+(0.267506079769)*x[1]**o+(0.812314541749)*x[1]+(-0.283432880004)*x[2]**o+(0.909390413963)*x[2]
        s[5,1,1]=(0.23175779421)*x[0]**o+(-0.614072594905)*x[0]+(-0.979433217533)*x[1]**o+(-0.928308978862)*x[1]+(0.252162852495)*x[2]**o+(-0.395771235106)*x[2]
        ref[0,0,0,0]=o*(-0.130526786389)*x_ref[0]**(o-1)+(-0.798097445808)
        ref[0,0,0,1]=o*(-0.679305651671)*x_ref[1]**(o-1)+(-0.805797697874)
        ref[0,0,0,2]=o*(0.344947047592)*x_ref[2]**(o-1)+(-0.835073945341)
        ref[0,0,1,0]=o*(-0.725780665135)*x_ref[0]**(o-1)+(-0.427130507741)
        ref[0,0,1,1]=o*(-0.172129968106)*x_ref[1]**(o-1)+(-0.329971568167)
        ref[0,0,1,2]=o*(0.430421618441)*x_ref[2]**(o-1)+(0.850034235311)
        ref[0,1,0,0]=o*(-0.0894725266496)*x_ref[0]**(o-1)+(-0.599803400866)
        ref[0,1,0,1]=o*(-0.689863844979)*x_ref[1]**(o-1)+(-0.433762379607)
        ref[0,1,0,2]=o*(0.0585705966933)*x_ref[2]**(o-1)+(0.932652014579)
        ref[0,1,1,0]=o*(0.351726580485)*x_ref[0]**(o-1)+(-0.46650217284)
        ref[0,1,1,1]=o*(-0.389405597178)*x_ref[1]**(o-1)+(0.702473795612)
        ref[0,1,1,2]=o*(0.971136943329)*x_ref[2]**(o-1)+(-0.149952798522)
        ref[1,0,0,0]=o*(-0.687968216558)*x_ref[0]**(o-1)+(0.918982435037)
        ref[1,0,0,1]=o*(-0.425841096866)*x_ref[1]**(o-1)+(0.738050157283)
        ref[1,0,0,2]=o*(-0.468164706689)*x_ref[2]**(o-1)+(0.524190937704)
        ref[1,0,1,0]=o*(-0.0190626564569)*x_ref[0]**(o-1)+(0.158516496015)
        ref[1,0,1,1]=o*(-0.591775293572)*x_ref[1]**(o-1)+(0.0791152697184)
        ref[1,0,1,2]=o*(-0.661850377953)*x_ref[2]**(o-1)+(-0.705879414238)
        ref[1,1,0,0]=o*(-0.538932477038)*x_ref[0]**(o-1)+(0.781931047594)
        ref[1,1,0,1]=o*(-0.799541079517)*x_ref[1]**(o-1)+(0.218777285901)
        ref[1,1,0,2]=o*(-0.475448704708)*x_ref[2]**(o-1)+(-0.655013532097)
        ref[1,1,1,0]=o*(0.0320772665819)*x_ref[0]**(o-1)+(-0.577577235436)
        ref[1,1,1,1]=o*(-0.736321547926)*x_ref[1]**(o-1)+(-0.133681853862)
        ref[1,1,1,2]=o*(-0.0573985073549)*x_ref[2]**(o-1)+(0.955212063412)
        ref[2,0,0,0]=o*(-0.257879955226)*x_ref[0]**(o-1)+(-0.994024413493)
        ref[2,0,0,1]=o*(-0.679188188792)*x_ref[1]**(o-1)+(-0.410885730889)
        ref[2,0,0,2]=o*(0.704555870926)*x_ref[2]**(o-1)+(0.518649077753)
        ref[2,0,1,0]=o*(-0.901488792722)*x_ref[0]**(o-1)+(-0.222487929076)
        ref[2,0,1,1]=o*(0.165342576493)*x_ref[1]**(o-1)+(-0.850214182574)
        ref[2,0,1,2]=o*(0.294760812505)*x_ref[2]**(o-1)+(-0.0831507413484)
        ref[2,1,0,0]=o*(0.316904380947)*x_ref[0]**(o-1)+(0.627318211545)
        ref[2,1,0,1]=o*(-0.216902145108)*x_ref[1]**(o-1)+(-0.673577316246)
        ref[2,1,0,2]=o*(0.122931487815)*x_ref[2]**(o-1)+(-0.0949502423641)
        ref[2,1,1,0]=o*(-0.602869040889)*x_ref[0]**(o-1)+(0.190451027649)
        ref[2,1,1,1]=o*(0.367827882092)*x_ref[1]**(o-1)+(0.686234993984)
        ref[2,1,1,2]=o*(0.653140976608)*x_ref[2]**(o-1)+(-0.168486517196)
        ref[3,0,0,0]=o*(0.271612350046)*x_ref[0]**(o-1)+(-0.827115599685)
        ref[3,0,0,1]=o*(-0.586600506988)*x_ref[1]**(o-1)+(0.128300770664)
        ref[3,0,0,2]=o*(-0.922582907477)*x_ref[2]**(o-1)+(0.334699215375)
        ref[3,0,1,0]=o*(-0.595897268553)*x_ref[0]**(o-1)+(0.425141939316)
        ref[3,0,1,1]=o*(0.713120162361)*x_ref[1]**(o-1)+(-0.680621044232)
        ref[3,0,1,2]=o*(0.749485690034)*x_ref[2]**(o-1)+(0.209547675753)
        ref[3,1,0,0]=o*(-0.530398143666)*x_ref[0]**(o-1)+(0.137850234083)
        ref[3,1,0,1]=o*(0.953276619653)*x_ref[1]**(o-1)+(-0.578161670579)
        ref[3,1,0,2]=o*(0.371898896542)*x_ref[2]**(o-1)+(-0.381331753222)
        ref[3,1,1,0]=o*(0.822298482068)*x_ref[0]**(o-1)+(-0.405836518978)
        ref[3,1,1,1]=o*(-0.647621688423)*x_ref[1]**(o-1)+(-0.672131868147)
        ref[3,1,1,2]=o*(-0.438867520359)*x_ref[2]**(o-1)+(-0.423328072541)
        ref[4,0,0,0]=o*(0.132863753285)*x_ref[0]**(o-1)+(0.397422994959)
        ref[4,0,0,1]=o*(-0.804707213822)*x_ref[1]**(o-1)+(0.536531749119)
        ref[4,0,0,2]=o*(0.411705453279)*x_ref[2]**(o-1)+(-0.169385048425)
        ref[4,0,1,0]=o*(0.0701228685651)*x_ref[0]**(o-1)+(0.639241932031)
        ref[4,0,1,1]=o*(-0.638730875998)*x_ref[1]**(o-1)+(-0.922035920621)
        ref[4,0,1,2]=o*(-0.113168863437)*x_ref[2]**(o-1)+(0.813287574357)
        ref[4,1,0,0]=o*(0.100082349838)*x_ref[0]**(o-1)+(0.133955906808)
        ref[4,1,0,1]=o*(-0.928623633632)*x_ref[1]**(o-1)+(-0.424742167549)
        ref[4,1,0,2]=o*(0.0239194885597)*x_ref[2]**(o-1)+(-0.672806207969)
        ref[4,1,1,0]=o*(0.829725063543)*x_ref[0]**(o-1)+(0.678552027954)
        ref[4,1,1,1]=o*(-0.812625405832)*x_ref[1]**(o-1)+(-0.810345951883)
        ref[4,1,1,2]=o*(-0.211595232946)*x_ref[2]**(o-1)+(-0.984554474963)
        ref[5,0,0,0]=o*(0.339493504495)*x_ref[0]**(o-1)+(-0.903993357652)
        ref[5,0,0,1]=o*(-0.440233624866)*x_ref[1]**(o-1)+(-0.430746451761)
        ref[5,0,0,2]=o*(0.111254680616)*x_ref[2]**(o-1)+(0.709038943317)
        ref[5,0,1,0]=o*(0.931029457116)*x_ref[0]**(o-1)+(-0.785398109537)
        ref[5,0,1,1]=o*(-0.326970114573)*x_ref[1]**(o-1)+(0.0384958937297)
        ref[5,0,1,2]=o*(-0.896259053345)*x_ref[2]**(o-1)+(-0.277237800601)
        ref[5,1,0,0]=o*(0.608509355178)*x_ref[0]**(o-1)+(0.585955903578)
        ref[5,1,0,1]=o*(0.267506079769)*x_ref[1]**(o-1)+(0.812314541749)
        ref[5,1,0,2]=o*(-0.283432880004)*x_ref[2]**(o-1)+(0.909390413963)
        ref[5,1,1,0]=o*(0.23175779421)*x_ref[0]**(o-1)+(-0.614072594905)
        ref[5,1,1,1]=o*(-0.979433217533)*x_ref[1]**(o-1)+(-0.928308978862)
        ref[5,1,1,2]=o*(0.252162852495)*x_ref[2]**(o-1)+(-0.395771235106)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.435083239326)*x[0]+(-0.691333040625)*x[1]
        ref[0]=(0.435083239326)
        ref[1]=(-0.691333040625)
      else:
        s=(-0.845744322657)*x[0]+(1.16954436275)*x[1]+(-0.314198264294)*x[2]
        ref[0]=(-0.845744322657)
        ref[1]=(1.16954436275)
        ref[2]=(-0.314198264294)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-1.56901548345)*x[0]+(-1.12088095812)*x[1]
        s[1]=(0.376788177601)*x[0]+(-0.143805721474)*x[1]
        ref[0,0]=(-1.56901548345)
        ref[0,1]=(-1.12088095812)
        ref[1,0]=(0.376788177601)
        ref[1,1]=(-0.143805721474)
      else:
        s[0]=(-1.22983775782)*x[0]+(0.425438687122)*x[1]+(-0.209028867995)*x[2]
        s[1]=(-0.970298435902)*x[0]+(1.179109977)*x[1]+(-0.139193691146)*x[2]
        ref[0,0]=(-1.22983775782)
        ref[0,1]=(0.425438687122)
        ref[0,2]=(-0.209028867995)
        ref[1,0]=(-0.970298435902)
        ref[1,1]=(1.179109977)
        ref[1,2]=(-0.139193691146)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.227103956711)*x[0]+(-1.30307121808)*x[1]
        s[0,1]=(-0.706864797391)*x[0]+(-1.31928070332)*x[1]
        s[0,2]=(0.634642484639)*x[0]+(0.585731207147)*x[1]
        s[0,3]=(0.431900787729)*x[0]+(0.569282604392)*x[1]
        s[0,4]=(0.0340323913824)*x[0]+(-1.60115889253)*x[1]
        s[1,0]=(0.207234615543)*x[0]+(-0.026497432384)*x[1]
        s[1,1]=(0.401346730679)*x[0]+(0.45537938848)*x[1]
        s[1,2]=(-0.823742908589)*x[0]+(-0.644194821889)*x[1]
        s[1,3]=(-0.949631217459)*x[0]+(-0.32798097587)*x[1]
        s[1,4]=(0.080942942672)*x[0]+(0.155277152998)*x[1]
        s[2,0]=(0.554034527738)*x[0]+(-0.830836696374)*x[1]
        s[2,1]=(0.0740776979829)*x[0]+(-1.03972607973)*x[1]
        s[2,2]=(1.3414358663)*x[0]+(1.09045231286)*x[1]
        s[2,3]=(0.704268712291)*x[0]+(1.42775320769)*x[1]
        s[2,4]=(0.886813956436)*x[0]+(-0.516119087351)*x[1]
        s[3,0]=(0.128480826214)*x[0]+(-0.461671413746)*x[1]
        s[3,1]=(0.179829027295)*x[0]+(-0.832871937244)*x[1]
        s[3,2]=(0.378675852865)*x[0]+(-0.186442070411)*x[1]
        s[3,3]=(-0.445016424955)*x[0]+(-1.4956101833)*x[1]
        s[3,4]=(-0.753080545152)*x[0]+(-0.990468924099)*x[1]
        ref[0,0,0]=(-0.227103956711)
        ref[0,0,1]=(-1.30307121808)
        ref[0,1,0]=(-0.706864797391)
        ref[0,1,1]=(-1.31928070332)
        ref[0,2,0]=(0.634642484639)
        ref[0,2,1]=(0.585731207147)
        ref[0,3,0]=(0.431900787729)
        ref[0,3,1]=(0.569282604392)
        ref[0,4,0]=(0.0340323913824)
        ref[0,4,1]=(-1.60115889253)
        ref[1,0,0]=(0.207234615543)
        ref[1,0,1]=(-0.026497432384)
        ref[1,1,0]=(0.401346730679)
        ref[1,1,1]=(0.45537938848)
        ref[1,2,0]=(-0.823742908589)
        ref[1,2,1]=(-0.644194821889)
        ref[1,3,0]=(-0.949631217459)
        ref[1,3,1]=(-0.32798097587)
        ref[1,4,0]=(0.080942942672)
        ref[1,4,1]=(0.155277152998)
        ref[2,0,0]=(0.554034527738)
        ref[2,0,1]=(-0.830836696374)
        ref[2,1,0]=(0.0740776979829)
        ref[2,1,1]=(-1.03972607973)
        ref[2,2,0]=(1.3414358663)
        ref[2,2,1]=(1.09045231286)
        ref[2,3,0]=(0.704268712291)
        ref[2,3,1]=(1.42775320769)
        ref[2,4,0]=(0.886813956436)
        ref[2,4,1]=(-0.516119087351)
        ref[3,0,0]=(0.128480826214)
        ref[3,0,1]=(-0.461671413746)
        ref[3,1,0]=(0.179829027295)
        ref[3,1,1]=(-0.832871937244)
        ref[3,2,0]=(0.378675852865)
        ref[3,2,1]=(-0.186442070411)
        ref[3,3,0]=(-0.445016424955)
        ref[3,3,1]=(-1.4956101833)
        ref[3,4,0]=(-0.753080545152)
        ref[3,4,1]=(-0.990468924099)
      else:
        s[0,0]=(-1.70799712078)*x[0]+(-0.446362335826)*x[1]+(-0.263476523401)*x[2]
        s[0,1]=(-0.736145268076)*x[0]+(0.646755267112)*x[1]+(0.447221049824)*x[2]
        s[0,2]=(-0.642101757164)*x[0]+(-0.611769559201)*x[1]+(0.264550006471)*x[2]
        s[0,3]=(-0.676996071184)*x[0]+(-0.590821452023)*x[1]+(1.24075002019)*x[2]
        s[0,4]=(1.02823294921)*x[0]+(-0.888741194341)*x[1]+(-0.523239815445)*x[2]
        s[1,0]=(-0.59574377001)*x[0]+(-0.191278786974)*x[1]+(0.159150016923)*x[2]
        s[1,1]=(0.969247182713)*x[0]+(-1.28235270295)*x[1]+(0.849611232483)*x[2]
        s[1,2]=(-0.435529899282)*x[0]+(0.12900241589)*x[1]+(-0.0181780583503)*x[2]
        s[1,3]=(0.57112270455)*x[0]+(-0.991550186645)*x[1]+(0.00643683318083)*x[2]
        s[1,4]=(0.252122977044)*x[0]+(-1.26873100076)*x[1]+(-0.372881623479)*x[2]
        s[2,0]=(0.0976854139907)*x[0]+(-0.464484842151)*x[1]+(-0.695497231127)*x[2]
        s[2,1]=(-0.227165085737)*x[0]+(0.188692778493)*x[1]+(-1.51388182618)*x[2]
        s[2,2]=(0.962466954525)*x[0]+(-0.101670728061)*x[1]+(1.23116858758)*x[2]
        s[2,3]=(-0.609963828043)*x[0]+(-0.775332931826)*x[1]+(0.656264967614)*x[2]
        s[2,4]=(-1.54897910822)*x[0]+(-1.00555194316)*x[1]+(-0.812320917814)*x[2]
        s[3,0]=(-0.616002842225)*x[0]+(0.118716870548)*x[1]+(0.255205330827)*x[2]
        s[3,1]=(-0.26445657759)*x[0]+(0.447308749134)*x[1]+(0.220730408993)*x[2]
        s[3,2]=(-1.85693156269)*x[0]+(1.36352248787)*x[1]+(-0.0517488277821)*x[2]
        s[3,3]=(1.14523233511)*x[0]+(-1.34499926349)*x[1]+(0.228859653657)*x[2]
        s[3,4]=(-0.079222973052)*x[0]+(1.47684264227)*x[1]+(-0.530005798807)*x[2]
        ref[0,0,0]=(-1.70799712078)
        ref[0,0,1]=(-0.446362335826)
        ref[0,0,2]=(-0.263476523401)
        ref[0,1,0]=(-0.736145268076)
        ref[0,1,1]=(0.646755267112)
        ref[0,1,2]=(0.447221049824)
        ref[0,2,0]=(-0.642101757164)
        ref[0,2,1]=(-0.611769559201)
        ref[0,2,2]=(0.264550006471)
        ref[0,3,0]=(-0.676996071184)
        ref[0,3,1]=(-0.590821452023)
        ref[0,3,2]=(1.24075002019)
        ref[0,4,0]=(1.02823294921)
        ref[0,4,1]=(-0.888741194341)
        ref[0,4,2]=(-0.523239815445)
        ref[1,0,0]=(-0.59574377001)
        ref[1,0,1]=(-0.191278786974)
        ref[1,0,2]=(0.159150016923)
        ref[1,1,0]=(0.969247182713)
        ref[1,1,1]=(-1.28235270295)
        ref[1,1,2]=(0.849611232483)
        ref[1,2,0]=(-0.435529899282)
        ref[1,2,1]=(0.12900241589)
        ref[1,2,2]=(-0.0181780583503)
        ref[1,3,0]=(0.57112270455)
        ref[1,3,1]=(-0.991550186645)
        ref[1,3,2]=(0.00643683318083)
        ref[1,4,0]=(0.252122977044)
        ref[1,4,1]=(-1.26873100076)
        ref[1,4,2]=(-0.372881623479)
        ref[2,0,0]=(0.0976854139907)
        ref[2,0,1]=(-0.464484842151)
        ref[2,0,2]=(-0.695497231127)
        ref[2,1,0]=(-0.227165085737)
        ref[2,1,1]=(0.188692778493)
        ref[2,1,2]=(-1.51388182618)
        ref[2,2,0]=(0.962466954525)
        ref[2,2,1]=(-0.101670728061)
        ref[2,2,2]=(1.23116858758)
        ref[2,3,0]=(-0.609963828043)
        ref[2,3,1]=(-0.775332931826)
        ref[2,3,2]=(0.656264967614)
        ref[2,4,0]=(-1.54897910822)
        ref[2,4,1]=(-1.00555194316)
        ref[2,4,2]=(-0.812320917814)
        ref[3,0,0]=(-0.616002842225)
        ref[3,0,1]=(0.118716870548)
        ref[3,0,2]=(0.255205330827)
        ref[3,1,0]=(-0.26445657759)
        ref[3,1,1]=(0.447308749134)
        ref[3,1,2]=(0.220730408993)
        ref[3,2,0]=(-1.85693156269)
        ref[3,2,1]=(1.36352248787)
        ref[3,2,2]=(-0.0517488277821)
        ref[3,3,0]=(1.14523233511)
        ref[3,3,1]=(-1.34499926349)
        ref[3,3,2]=(0.228859653657)
        ref[3,4,0]=(-0.079222973052)
        ref[3,4,1]=(1.47684264227)
        ref[3,4,2]=(-0.530005798807)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(1.49153517734)*x[0]+(-0.579975803933)*x[1]
        s[0,0,1]=(-0.696469728625)*x[0]+(1.21538948965)*x[1]
        s[0,1,0]=(-0.0991175021321)*x[0]+(-1.24138963759)*x[1]
        s[0,1,1]=(0.720869772304)*x[0]+(0.693983413462)*x[1]
        s[1,0,0]=(0.539658773124)*x[0]+(0.558956410162)*x[1]
        s[1,0,1]=(0.219371791063)*x[0]+(-0.0645808851409)*x[1]
        s[1,1,0]=(-0.982429431088)*x[0]+(0.765912701256)*x[1]
        s[1,1,1]=(0.409666526322)*x[0]+(-0.667562489533)*x[1]
        s[2,0,0]=(-1.00270035018)*x[0]+(-0.159198118098)*x[1]
        s[2,0,1]=(1.17972133138)*x[0]+(0.921797063108)*x[1]
        s[2,1,0]=(1.4726821986)*x[0]+(0.2041326504)*x[1]
        s[2,1,1]=(0.365688842529)*x[0]+(0.293751041397)*x[1]
        s[3,0,0]=(-0.485937426896)*x[0]+(-0.0722100496386)*x[1]
        s[3,0,1]=(0.937772100531)*x[0]+(-0.57987183111)*x[1]
        s[3,1,0]=(1.16518206513)*x[0]+(-0.715128236725)*x[1]
        s[3,1,1]=(0.130107225088)*x[0]+(-0.0702897307647)*x[1]
        s[4,0,0]=(-0.396052259681)*x[0]+(0.618983002296)*x[1]
        s[4,0,1]=(0.10948716533)*x[0]+(-0.413031012862)*x[1]
        s[4,1,0]=(1.30735893111)*x[0]+(-1.10958529012)*x[1]
        s[4,1,1]=(0.250657387621)*x[0]+(1.45183909124)*x[1]
        s[5,0,0]=(0.0301234829293)*x[0]+(0.374132370619)*x[1]
        s[5,0,1]=(-0.0915198655839)*x[0]+(0.933624705533)*x[1]
        s[5,1,0]=(1.1572487942)*x[0]+(-0.00851123614571)*x[1]
        s[5,1,1]=(0.761078997709)*x[0]+(1.60352507361)*x[1]
        ref[0,0,0,0]=(1.49153517734)
        ref[0,0,0,1]=(-0.579975803933)
        ref[0,0,1,0]=(-0.696469728625)
        ref[0,0,1,1]=(1.21538948965)
        ref[0,1,0,0]=(-0.0991175021321)
        ref[0,1,0,1]=(-1.24138963759)
        ref[0,1,1,0]=(0.720869772304)
        ref[0,1,1,1]=(0.693983413462)
        ref[1,0,0,0]=(0.539658773124)
        ref[1,0,0,1]=(0.558956410162)
        ref[1,0,1,0]=(0.219371791063)
        ref[1,0,1,1]=(-0.0645808851409)
        ref[1,1,0,0]=(-0.982429431088)
        ref[1,1,0,1]=(0.765912701256)
        ref[1,1,1,0]=(0.409666526322)
        ref[1,1,1,1]=(-0.667562489533)
        ref[2,0,0,0]=(-1.00270035018)
        ref[2,0,0,1]=(-0.159198118098)
        ref[2,0,1,0]=(1.17972133138)
        ref[2,0,1,1]=(0.921797063108)
        ref[2,1,0,0]=(1.4726821986)
        ref[2,1,0,1]=(0.2041326504)
        ref[2,1,1,0]=(0.365688842529)
        ref[2,1,1,1]=(0.293751041397)
        ref[3,0,0,0]=(-0.485937426896)
        ref[3,0,0,1]=(-0.0722100496386)
        ref[3,0,1,0]=(0.937772100531)
        ref[3,0,1,1]=(-0.57987183111)
        ref[3,1,0,0]=(1.16518206513)
        ref[3,1,0,1]=(-0.715128236725)
        ref[3,1,1,0]=(0.130107225088)
        ref[3,1,1,1]=(-0.0702897307647)
        ref[4,0,0,0]=(-0.396052259681)
        ref[4,0,0,1]=(0.618983002296)
        ref[4,0,1,0]=(0.10948716533)
        ref[4,0,1,1]=(-0.413031012862)
        ref[4,1,0,0]=(1.30735893111)
        ref[4,1,0,1]=(-1.10958529012)
        ref[4,1,1,0]=(0.250657387621)
        ref[4,1,1,1]=(1.45183909124)
        ref[5,0,0,0]=(0.0301234829293)
        ref[5,0,0,1]=(0.374132370619)
        ref[5,0,1,0]=(-0.0915198655839)
        ref[5,0,1,1]=(0.933624705533)
        ref[5,1,0,0]=(1.1572487942)
        ref[5,1,0,1]=(-0.00851123614571)
        ref[5,1,1,0]=(0.761078997709)
        ref[5,1,1,1]=(1.60352507361)
      else:
        s[0,0,0]=(1.16204910106)*x[0]+(1.21138322906)*x[1]+(-0.0524201871512)*x[2]
        s[0,0,1]=(-1.62418470618)*x[0]+(0.862850207692)*x[1]+(-0.287954408686)*x[2]
        s[0,1,0]=(-0.560096422291)*x[0]+(-0.3064736255)*x[1]+(-0.104600769051)*x[2]
        s[0,1,1]=(-1.41163310088)*x[0]+(-0.234279745635)*x[1]+(-1.22939949417)*x[2]
        s[1,0,0]=(0.21959990013)*x[0]+(0.40893057141)*x[1]+(0.643102887922)*x[2]
        s[1,0,1]=(-0.191194957197)*x[0]+(1.47353348334)*x[1]+(0.732969151984)*x[2]
        s[1,1,0]=(-1.19345990561)*x[0]+(-0.336674594448)*x[1]+(-0.0578613018881)*x[2]
        s[1,1,1]=(1.6232908613)*x[0]+(-0.107926586217)*x[1]+(0.235855060031)*x[2]
        s[2,0,0]=(0.820509328296)*x[0]+(0.547288169297)*x[1]+(-1.55487561435)*x[2]
        s[2,0,1]=(0.0893134152711)*x[0]+(0.161241646611)*x[1]+(-0.214358478289)*x[2]
        s[2,1,0]=(-1.12660016765)*x[0]+(1.53125870766)*x[1]+(0.697333670681)*x[2]
        s[2,1,1]=(-0.822208944906)*x[0]+(0.361159545618)*x[1]+(-0.0753065880934)*x[2]
        s[3,0,0]=(0.371298368343)*x[0]+(-0.935563626025)*x[1]+(1.32294571078)*x[2]
        s[3,0,1]=(0.228437335618)*x[0]+(1.00946312467)*x[1]+(-0.353871040012)*x[2]
        s[3,1,0]=(0.315905225539)*x[0]+(-1.02555884678)*x[1]+(0.388333892139)*x[2]
        s[3,1,1]=(1.23848230316)*x[0]+(1.02037500431)*x[1]+(0.158105617744)*x[2]
        s[4,0,0]=(-1.45272742631)*x[0]+(0.543388885954)*x[1]+(0.182106047094)*x[2]
        s[4,0,1]=(-0.351765522363)*x[0]+(-0.281592529507)*x[1]+(-0.324592486431)*x[2]
        s[4,1,0]=(-0.0890034165869)*x[0]+(-0.000670251305565)*x[1]+(1.39235380789)*x[2]
        s[4,1,1]=(-0.127096612653)*x[0]+(0.454493346168)*x[1]+(-0.360338313101)*x[2]
        s[5,0,0]=(-0.96052998866)*x[0]+(-0.0674238906685)*x[1]+(-1.63834268246)*x[2]
        s[5,0,1]=(-0.462036588746)*x[0]+(1.1804169499)*x[1]+(1.13779511332)*x[2]
        s[5,1,0]=(0.0863661479908)*x[0]+(0.0691347259375)*x[1]+(0.625219063907)*x[2]
        s[5,1,1]=(0.50049088687)*x[0]+(-0.438895013688)*x[1]+(-0.419695143786)*x[2]
        ref[0,0,0,0]=(1.16204910106)
        ref[0,0,0,1]=(1.21138322906)
        ref[0,0,0,2]=(-0.0524201871512)
        ref[0,0,1,0]=(-1.62418470618)
        ref[0,0,1,1]=(0.862850207692)
        ref[0,0,1,2]=(-0.287954408686)
        ref[0,1,0,0]=(-0.560096422291)
        ref[0,1,0,1]=(-0.3064736255)
        ref[0,1,0,2]=(-0.104600769051)
        ref[0,1,1,0]=(-1.41163310088)
        ref[0,1,1,1]=(-0.234279745635)
        ref[0,1,1,2]=(-1.22939949417)
        ref[1,0,0,0]=(0.21959990013)
        ref[1,0,0,1]=(0.40893057141)
        ref[1,0,0,2]=(0.643102887922)
        ref[1,0,1,0]=(-0.191194957197)
        ref[1,0,1,1]=(1.47353348334)
        ref[1,0,1,2]=(0.732969151984)
        ref[1,1,0,0]=(-1.19345990561)
        ref[1,1,0,1]=(-0.336674594448)
        ref[1,1,0,2]=(-0.0578613018881)
        ref[1,1,1,0]=(1.6232908613)
        ref[1,1,1,1]=(-0.107926586217)
        ref[1,1,1,2]=(0.235855060031)
        ref[2,0,0,0]=(0.820509328296)
        ref[2,0,0,1]=(0.547288169297)
        ref[2,0,0,2]=(-1.55487561435)
        ref[2,0,1,0]=(0.0893134152711)
        ref[2,0,1,1]=(0.161241646611)
        ref[2,0,1,2]=(-0.214358478289)
        ref[2,1,0,0]=(-1.12660016765)
        ref[2,1,0,1]=(1.53125870766)
        ref[2,1,0,2]=(0.697333670681)
        ref[2,1,1,0]=(-0.822208944906)
        ref[2,1,1,1]=(0.361159545618)
        ref[2,1,1,2]=(-0.0753065880934)
        ref[3,0,0,0]=(0.371298368343)
        ref[3,0,0,1]=(-0.935563626025)
        ref[3,0,0,2]=(1.32294571078)
        ref[3,0,1,0]=(0.228437335618)
        ref[3,0,1,1]=(1.00946312467)
        ref[3,0,1,2]=(-0.353871040012)
        ref[3,1,0,0]=(0.315905225539)
        ref[3,1,0,1]=(-1.02555884678)
        ref[3,1,0,2]=(0.388333892139)
        ref[3,1,1,0]=(1.23848230316)
        ref[3,1,1,1]=(1.02037500431)
        ref[3,1,1,2]=(0.158105617744)
        ref[4,0,0,0]=(-1.45272742631)
        ref[4,0,0,1]=(0.543388885954)
        ref[4,0,0,2]=(0.182106047094)
        ref[4,0,1,0]=(-0.351765522363)
        ref[4,0,1,1]=(-0.281592529507)
        ref[4,0,1,2]=(-0.324592486431)
        ref[4,1,0,0]=(-0.0890034165869)
        ref[4,1,0,1]=(-0.000670251305565)
        ref[4,1,0,2]=(1.39235380789)
        ref[4,1,1,0]=(-0.127096612653)
        ref[4,1,1,1]=(0.454493346168)
        ref[4,1,1,2]=(-0.360338313101)
        ref[5,0,0,0]=(-0.96052998866)
        ref[5,0,0,1]=(-0.0674238906685)
        ref[5,0,0,2]=(-1.63834268246)
        ref[5,0,1,0]=(-0.462036588746)
        ref[5,0,1,1]=(1.1804169499)
        ref[5,0,1,2]=(1.13779511332)
        ref[5,1,0,0]=(0.0863661479908)
        ref[5,1,0,1]=(0.0691347259375)
        ref[5,1,0,2]=(0.625219063907)
        ref[5,1,1,0]=(0.50049088687)
        ref[5,1,1,1]=(-0.438895013688)
        ref[5,1,1,2]=(-0.419695143786)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.435083239326)*x[0]+(-0.691333040625)*x[1]
        ref[0]=(0.435083239326)
        ref[1]=(-0.691333040625)
      else:
        s=(-0.845744322657)*x[0]+(1.16954436275)*x[1]+(-0.314198264294)*x[2]
        ref[0]=(-0.845744322657)
        ref[1]=(1.16954436275)
        ref[2]=(-0.314198264294)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-1.56901548345)*x[0]+(-1.12088095812)*x[1]
        s[1]=(0.376788177601)*x[0]+(-0.143805721474)*x[1]
        ref[0,0]=(-1.56901548345)
        ref[0,1]=(-1.12088095812)
        ref[1,0]=(0.376788177601)
        ref[1,1]=(-0.143805721474)
      else:
        s[0]=(-1.22983775782)*x[0]+(0.425438687122)*x[1]+(-0.209028867995)*x[2]
        s[1]=(-0.970298435902)*x[0]+(1.179109977)*x[1]+(-0.139193691146)*x[2]
        ref[0,0]=(-1.22983775782)
        ref[0,1]=(0.425438687122)
        ref[0,2]=(-0.209028867995)
        ref[1,0]=(-0.970298435902)
        ref[1,1]=(1.179109977)
        ref[1,2]=(-0.139193691146)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.227103956711)*x[0]+(-1.30307121808)*x[1]
        s[0,1]=(-0.706864797391)*x[0]+(-1.31928070332)*x[1]
        s[0,2]=(0.634642484639)*x[0]+(0.585731207147)*x[1]
        s[0,3]=(0.431900787729)*x[0]+(0.569282604392)*x[1]
        s[0,4]=(0.0340323913824)*x[0]+(-1.60115889253)*x[1]
        s[1,0]=(0.207234615543)*x[0]+(-0.026497432384)*x[1]
        s[1,1]=(0.401346730679)*x[0]+(0.45537938848)*x[1]
        s[1,2]=(-0.823742908589)*x[0]+(-0.644194821889)*x[1]
        s[1,3]=(-0.949631217459)*x[0]+(-0.32798097587)*x[1]
        s[1,4]=(0.080942942672)*x[0]+(0.155277152998)*x[1]
        s[2,0]=(0.554034527738)*x[0]+(-0.830836696374)*x[1]
        s[2,1]=(0.0740776979829)*x[0]+(-1.03972607973)*x[1]
        s[2,2]=(1.3414358663)*x[0]+(1.09045231286)*x[1]
        s[2,3]=(0.704268712291)*x[0]+(1.42775320769)*x[1]
        s[2,4]=(0.886813956436)*x[0]+(-0.516119087351)*x[1]
        s[3,0]=(0.128480826214)*x[0]+(-0.461671413746)*x[1]
        s[3,1]=(0.179829027295)*x[0]+(-0.832871937244)*x[1]
        s[3,2]=(0.378675852865)*x[0]+(-0.186442070411)*x[1]
        s[3,3]=(-0.445016424955)*x[0]+(-1.4956101833)*x[1]
        s[3,4]=(-0.753080545152)*x[0]+(-0.990468924099)*x[1]
        ref[0,0,0]=(-0.227103956711)
        ref[0,0,1]=(-1.30307121808)
        ref[0,1,0]=(-0.706864797391)
        ref[0,1,1]=(-1.31928070332)
        ref[0,2,0]=(0.634642484639)
        ref[0,2,1]=(0.585731207147)
        ref[0,3,0]=(0.431900787729)
        ref[0,3,1]=(0.569282604392)
        ref[0,4,0]=(0.0340323913824)
        ref[0,4,1]=(-1.60115889253)
        ref[1,0,0]=(0.207234615543)
        ref[1,0,1]=(-0.026497432384)
        ref[1,1,0]=(0.401346730679)
        ref[1,1,1]=(0.45537938848)
        ref[1,2,0]=(-0.823742908589)
        ref[1,2,1]=(-0.644194821889)
        ref[1,3,0]=(-0.949631217459)
        ref[1,3,1]=(-0.32798097587)
        ref[1,4,0]=(0.080942942672)
        ref[1,4,1]=(0.155277152998)
        ref[2,0,0]=(0.554034527738)
        ref[2,0,1]=(-0.830836696374)
        ref[2,1,0]=(0.0740776979829)
        ref[2,1,1]=(-1.03972607973)
        ref[2,2,0]=(1.3414358663)
        ref[2,2,1]=(1.09045231286)
        ref[2,3,0]=(0.704268712291)
        ref[2,3,1]=(1.42775320769)
        ref[2,4,0]=(0.886813956436)
        ref[2,4,1]=(-0.516119087351)
        ref[3,0,0]=(0.128480826214)
        ref[3,0,1]=(-0.461671413746)
        ref[3,1,0]=(0.179829027295)
        ref[3,1,1]=(-0.832871937244)
        ref[3,2,0]=(0.378675852865)
        ref[3,2,1]=(-0.186442070411)
        ref[3,3,0]=(-0.445016424955)
        ref[3,3,1]=(-1.4956101833)
        ref[3,4,0]=(-0.753080545152)
        ref[3,4,1]=(-0.990468924099)
      else:
        s[0,0]=(-1.70799712078)*x[0]+(-0.446362335826)*x[1]+(-0.263476523401)*x[2]
        s[0,1]=(-0.736145268076)*x[0]+(0.646755267112)*x[1]+(0.447221049824)*x[2]
        s[0,2]=(-0.642101757164)*x[0]+(-0.611769559201)*x[1]+(0.264550006471)*x[2]
        s[0,3]=(-0.676996071184)*x[0]+(-0.590821452023)*x[1]+(1.24075002019)*x[2]
        s[0,4]=(1.02823294921)*x[0]+(-0.888741194341)*x[1]+(-0.523239815445)*x[2]
        s[1,0]=(-0.59574377001)*x[0]+(-0.191278786974)*x[1]+(0.159150016923)*x[2]
        s[1,1]=(0.969247182713)*x[0]+(-1.28235270295)*x[1]+(0.849611232483)*x[2]
        s[1,2]=(-0.435529899282)*x[0]+(0.12900241589)*x[1]+(-0.0181780583503)*x[2]
        s[1,3]=(0.57112270455)*x[0]+(-0.991550186645)*x[1]+(0.00643683318083)*x[2]
        s[1,4]=(0.252122977044)*x[0]+(-1.26873100076)*x[1]+(-0.372881623479)*x[2]
        s[2,0]=(0.0976854139907)*x[0]+(-0.464484842151)*x[1]+(-0.695497231127)*x[2]
        s[2,1]=(-0.227165085737)*x[0]+(0.188692778493)*x[1]+(-1.51388182618)*x[2]
        s[2,2]=(0.962466954525)*x[0]+(-0.101670728061)*x[1]+(1.23116858758)*x[2]
        s[2,3]=(-0.609963828043)*x[0]+(-0.775332931826)*x[1]+(0.656264967614)*x[2]
        s[2,4]=(-1.54897910822)*x[0]+(-1.00555194316)*x[1]+(-0.812320917814)*x[2]
        s[3,0]=(-0.616002842225)*x[0]+(0.118716870548)*x[1]+(0.255205330827)*x[2]
        s[3,1]=(-0.26445657759)*x[0]+(0.447308749134)*x[1]+(0.220730408993)*x[2]
        s[3,2]=(-1.85693156269)*x[0]+(1.36352248787)*x[1]+(-0.0517488277821)*x[2]
        s[3,3]=(1.14523233511)*x[0]+(-1.34499926349)*x[1]+(0.228859653657)*x[2]
        s[3,4]=(-0.079222973052)*x[0]+(1.47684264227)*x[1]+(-0.530005798807)*x[2]
        ref[0,0,0]=(-1.70799712078)
        ref[0,0,1]=(-0.446362335826)
        ref[0,0,2]=(-0.263476523401)
        ref[0,1,0]=(-0.736145268076)
        ref[0,1,1]=(0.646755267112)
        ref[0,1,2]=(0.447221049824)
        ref[0,2,0]=(-0.642101757164)
        ref[0,2,1]=(-0.611769559201)
        ref[0,2,2]=(0.264550006471)
        ref[0,3,0]=(-0.676996071184)
        ref[0,3,1]=(-0.590821452023)
        ref[0,3,2]=(1.24075002019)
        ref[0,4,0]=(1.02823294921)
        ref[0,4,1]=(-0.888741194341)
        ref[0,4,2]=(-0.523239815445)
        ref[1,0,0]=(-0.59574377001)
        ref[1,0,1]=(-0.191278786974)
        ref[1,0,2]=(0.159150016923)
        ref[1,1,0]=(0.969247182713)
        ref[1,1,1]=(-1.28235270295)
        ref[1,1,2]=(0.849611232483)
        ref[1,2,0]=(-0.435529899282)
        ref[1,2,1]=(0.12900241589)
        ref[1,2,2]=(-0.0181780583503)
        ref[1,3,0]=(0.57112270455)
        ref[1,3,1]=(-0.991550186645)
        ref[1,3,2]=(0.00643683318083)
        ref[1,4,0]=(0.252122977044)
        ref[1,4,1]=(-1.26873100076)
        ref[1,4,2]=(-0.372881623479)
        ref[2,0,0]=(0.0976854139907)
        ref[2,0,1]=(-0.464484842151)
        ref[2,0,2]=(-0.695497231127)
        ref[2,1,0]=(-0.227165085737)
        ref[2,1,1]=(0.188692778493)
        ref[2,1,2]=(-1.51388182618)
        ref[2,2,0]=(0.962466954525)
        ref[2,2,1]=(-0.101670728061)
        ref[2,2,2]=(1.23116858758)
        ref[2,3,0]=(-0.609963828043)
        ref[2,3,1]=(-0.775332931826)
        ref[2,3,2]=(0.656264967614)
        ref[2,4,0]=(-1.54897910822)
        ref[2,4,1]=(-1.00555194316)
        ref[2,4,2]=(-0.812320917814)
        ref[3,0,0]=(-0.616002842225)
        ref[3,0,1]=(0.118716870548)
        ref[3,0,2]=(0.255205330827)
        ref[3,1,0]=(-0.26445657759)
        ref[3,1,1]=(0.447308749134)
        ref[3,1,2]=(0.220730408993)
        ref[3,2,0]=(-1.85693156269)
        ref[3,2,1]=(1.36352248787)
        ref[3,2,2]=(-0.0517488277821)
        ref[3,3,0]=(1.14523233511)
        ref[3,3,1]=(-1.34499926349)
        ref[3,3,2]=(0.228859653657)
        ref[3,4,0]=(-0.079222973052)
        ref[3,4,1]=(1.47684264227)
        ref[3,4,2]=(-0.530005798807)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(1.49153517734)*x[0]+(-0.579975803933)*x[1]
        s[0,0,1]=(-0.696469728625)*x[0]+(1.21538948965)*x[1]
        s[0,1,0]=(-0.0991175021321)*x[0]+(-1.24138963759)*x[1]
        s[0,1,1]=(0.720869772304)*x[0]+(0.693983413462)*x[1]
        s[1,0,0]=(0.539658773124)*x[0]+(0.558956410162)*x[1]
        s[1,0,1]=(0.219371791063)*x[0]+(-0.0645808851409)*x[1]
        s[1,1,0]=(-0.982429431088)*x[0]+(0.765912701256)*x[1]
        s[1,1,1]=(0.409666526322)*x[0]+(-0.667562489533)*x[1]
        s[2,0,0]=(-1.00270035018)*x[0]+(-0.159198118098)*x[1]
        s[2,0,1]=(1.17972133138)*x[0]+(0.921797063108)*x[1]
        s[2,1,0]=(1.4726821986)*x[0]+(0.2041326504)*x[1]
        s[2,1,1]=(0.365688842529)*x[0]+(0.293751041397)*x[1]
        s[3,0,0]=(-0.485937426896)*x[0]+(-0.0722100496386)*x[1]
        s[3,0,1]=(0.937772100531)*x[0]+(-0.57987183111)*x[1]
        s[3,1,0]=(1.16518206513)*x[0]+(-0.715128236725)*x[1]
        s[3,1,1]=(0.130107225088)*x[0]+(-0.0702897307647)*x[1]
        s[4,0,0]=(-0.396052259681)*x[0]+(0.618983002296)*x[1]
        s[4,0,1]=(0.10948716533)*x[0]+(-0.413031012862)*x[1]
        s[4,1,0]=(1.30735893111)*x[0]+(-1.10958529012)*x[1]
        s[4,1,1]=(0.250657387621)*x[0]+(1.45183909124)*x[1]
        s[5,0,0]=(0.0301234829293)*x[0]+(0.374132370619)*x[1]
        s[5,0,1]=(-0.0915198655839)*x[0]+(0.933624705533)*x[1]
        s[5,1,0]=(1.1572487942)*x[0]+(-0.00851123614571)*x[1]
        s[5,1,1]=(0.761078997709)*x[0]+(1.60352507361)*x[1]
        ref[0,0,0,0]=(1.49153517734)
        ref[0,0,0,1]=(-0.579975803933)
        ref[0,0,1,0]=(-0.696469728625)
        ref[0,0,1,1]=(1.21538948965)
        ref[0,1,0,0]=(-0.0991175021321)
        ref[0,1,0,1]=(-1.24138963759)
        ref[0,1,1,0]=(0.720869772304)
        ref[0,1,1,1]=(0.693983413462)
        ref[1,0,0,0]=(0.539658773124)
        ref[1,0,0,1]=(0.558956410162)
        ref[1,0,1,0]=(0.219371791063)
        ref[1,0,1,1]=(-0.0645808851409)
        ref[1,1,0,0]=(-0.982429431088)
        ref[1,1,0,1]=(0.765912701256)
        ref[1,1,1,0]=(0.409666526322)
        ref[1,1,1,1]=(-0.667562489533)
        ref[2,0,0,0]=(-1.00270035018)
        ref[2,0,0,1]=(-0.159198118098)
        ref[2,0,1,0]=(1.17972133138)
        ref[2,0,1,1]=(0.921797063108)
        ref[2,1,0,0]=(1.4726821986)
        ref[2,1,0,1]=(0.2041326504)
        ref[2,1,1,0]=(0.365688842529)
        ref[2,1,1,1]=(0.293751041397)
        ref[3,0,0,0]=(-0.485937426896)
        ref[3,0,0,1]=(-0.0722100496386)
        ref[3,0,1,0]=(0.937772100531)
        ref[3,0,1,1]=(-0.57987183111)
        ref[3,1,0,0]=(1.16518206513)
        ref[3,1,0,1]=(-0.715128236725)
        ref[3,1,1,0]=(0.130107225088)
        ref[3,1,1,1]=(-0.0702897307647)
        ref[4,0,0,0]=(-0.396052259681)
        ref[4,0,0,1]=(0.618983002296)
        ref[4,0,1,0]=(0.10948716533)
        ref[4,0,1,1]=(-0.413031012862)
        ref[4,1,0,0]=(1.30735893111)
        ref[4,1,0,1]=(-1.10958529012)
        ref[4,1,1,0]=(0.250657387621)
        ref[4,1,1,1]=(1.45183909124)
        ref[5,0,0,0]=(0.0301234829293)
        ref[5,0,0,1]=(0.374132370619)
        ref[5,0,1,0]=(-0.0915198655839)
        ref[5,0,1,1]=(0.933624705533)
        ref[5,1,0,0]=(1.1572487942)
        ref[5,1,0,1]=(-0.00851123614571)
        ref[5,1,1,0]=(0.761078997709)
        ref[5,1,1,1]=(1.60352507361)
      else:
        s[0,0,0]=(1.16204910106)*x[0]+(1.21138322906)*x[1]+(-0.0524201871512)*x[2]
        s[0,0,1]=(-1.62418470618)*x[0]+(0.862850207692)*x[1]+(-0.287954408686)*x[2]
        s[0,1,0]=(-0.560096422291)*x[0]+(-0.3064736255)*x[1]+(-0.104600769051)*x[2]
        s[0,1,1]=(-1.41163310088)*x[0]+(-0.234279745635)*x[1]+(-1.22939949417)*x[2]
        s[1,0,0]=(0.21959990013)*x[0]+(0.40893057141)*x[1]+(0.643102887922)*x[2]
        s[1,0,1]=(-0.191194957197)*x[0]+(1.47353348334)*x[1]+(0.732969151984)*x[2]
        s[1,1,0]=(-1.19345990561)*x[0]+(-0.336674594448)*x[1]+(-0.0578613018881)*x[2]
        s[1,1,1]=(1.6232908613)*x[0]+(-0.107926586217)*x[1]+(0.235855060031)*x[2]
        s[2,0,0]=(0.820509328296)*x[0]+(0.547288169297)*x[1]+(-1.55487561435)*x[2]
        s[2,0,1]=(0.0893134152711)*x[0]+(0.161241646611)*x[1]+(-0.214358478289)*x[2]
        s[2,1,0]=(-1.12660016765)*x[0]+(1.53125870766)*x[1]+(0.697333670681)*x[2]
        s[2,1,1]=(-0.822208944906)*x[0]+(0.361159545618)*x[1]+(-0.0753065880934)*x[2]
        s[3,0,0]=(0.371298368343)*x[0]+(-0.935563626025)*x[1]+(1.32294571078)*x[2]
        s[3,0,1]=(0.228437335618)*x[0]+(1.00946312467)*x[1]+(-0.353871040012)*x[2]
        s[3,1,0]=(0.315905225539)*x[0]+(-1.02555884678)*x[1]+(0.388333892139)*x[2]
        s[3,1,1]=(1.23848230316)*x[0]+(1.02037500431)*x[1]+(0.158105617744)*x[2]
        s[4,0,0]=(-1.45272742631)*x[0]+(0.543388885954)*x[1]+(0.182106047094)*x[2]
        s[4,0,1]=(-0.351765522363)*x[0]+(-0.281592529507)*x[1]+(-0.324592486431)*x[2]
        s[4,1,0]=(-0.0890034165869)*x[0]+(-0.000670251305565)*x[1]+(1.39235380789)*x[2]
        s[4,1,1]=(-0.127096612653)*x[0]+(0.454493346168)*x[1]+(-0.360338313101)*x[2]
        s[5,0,0]=(-0.96052998866)*x[0]+(-0.0674238906685)*x[1]+(-1.63834268246)*x[2]
        s[5,0,1]=(-0.462036588746)*x[0]+(1.1804169499)*x[1]+(1.13779511332)*x[2]
        s[5,1,0]=(0.0863661479908)*x[0]+(0.0691347259375)*x[1]+(0.625219063907)*x[2]
        s[5,1,1]=(0.50049088687)*x[0]+(-0.438895013688)*x[1]+(-0.419695143786)*x[2]
        ref[0,0,0,0]=(1.16204910106)
        ref[0,0,0,1]=(1.21138322906)
        ref[0,0,0,2]=(-0.0524201871512)
        ref[0,0,1,0]=(-1.62418470618)
        ref[0,0,1,1]=(0.862850207692)
        ref[0,0,1,2]=(-0.287954408686)
        ref[0,1,0,0]=(-0.560096422291)
        ref[0,1,0,1]=(-0.3064736255)
        ref[0,1,0,2]=(-0.104600769051)
        ref[0,1,1,0]=(-1.41163310088)
        ref[0,1,1,1]=(-0.234279745635)
        ref[0,1,1,2]=(-1.22939949417)
        ref[1,0,0,0]=(0.21959990013)
        ref[1,0,0,1]=(0.40893057141)
        ref[1,0,0,2]=(0.643102887922)
        ref[1,0,1,0]=(-0.191194957197)
        ref[1,0,1,1]=(1.47353348334)
        ref[1,0,1,2]=(0.732969151984)
        ref[1,1,0,0]=(-1.19345990561)
        ref[1,1,0,1]=(-0.336674594448)
        ref[1,1,0,2]=(-0.0578613018881)
        ref[1,1,1,0]=(1.6232908613)
        ref[1,1,1,1]=(-0.107926586217)
        ref[1,1,1,2]=(0.235855060031)
        ref[2,0,0,0]=(0.820509328296)
        ref[2,0,0,1]=(0.547288169297)
        ref[2,0,0,2]=(-1.55487561435)
        ref[2,0,1,0]=(0.0893134152711)
        ref[2,0,1,1]=(0.161241646611)
        ref[2,0,1,2]=(-0.214358478289)
        ref[2,1,0,0]=(-1.12660016765)
        ref[2,1,0,1]=(1.53125870766)
        ref[2,1,0,2]=(0.697333670681)
        ref[2,1,1,0]=(-0.822208944906)
        ref[2,1,1,1]=(0.361159545618)
        ref[2,1,1,2]=(-0.0753065880934)
        ref[3,0,0,0]=(0.371298368343)
        ref[3,0,0,1]=(-0.935563626025)
        ref[3,0,0,2]=(1.32294571078)
        ref[3,0,1,0]=(0.228437335618)
        ref[3,0,1,1]=(1.00946312467)
        ref[3,0,1,2]=(-0.353871040012)
        ref[3,1,0,0]=(0.315905225539)
        ref[3,1,0,1]=(-1.02555884678)
        ref[3,1,0,2]=(0.388333892139)
        ref[3,1,1,0]=(1.23848230316)
        ref[3,1,1,1]=(1.02037500431)
        ref[3,1,1,2]=(0.158105617744)
        ref[4,0,0,0]=(-1.45272742631)
        ref[4,0,0,1]=(0.543388885954)
        ref[4,0,0,2]=(0.182106047094)
        ref[4,0,1,0]=(-0.351765522363)
        ref[4,0,1,1]=(-0.281592529507)
        ref[4,0,1,2]=(-0.324592486431)
        ref[4,1,0,0]=(-0.0890034165869)
        ref[4,1,0,1]=(-0.000670251305565)
        ref[4,1,0,2]=(1.39235380789)
        ref[4,1,1,0]=(-0.127096612653)
        ref[4,1,1,1]=(0.454493346168)
        ref[4,1,1,2]=(-0.360338313101)
        ref[5,0,0,0]=(-0.96052998866)
        ref[5,0,0,1]=(-0.0674238906685)
        ref[5,0,0,2]=(-1.63834268246)
        ref[5,0,1,0]=(-0.462036588746)
        ref[5,0,1,1]=(1.1804169499)
        ref[5,0,1,2]=(1.13779511332)
        ref[5,1,0,0]=(0.0863661479908)
        ref[5,1,0,1]=(0.0691347259375)
        ref[5,1,0,2]=(0.625219063907)
        ref[5,1,1,0]=(0.50049088687)
        ref[5,1,1,1]=(-0.438895013688)
        ref[5,1,1,2]=(-0.419695143786)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(-0.169375355906)*x[0]**o+(0.270029734958)*x[0]+(-0.19196016538)*x[1]**o+(0.426264369437)*x[1]
        ref[0]=o*(-0.169375355906)*x_ref[0]**(o-1)+(0.270029734958)
        ref[1]=o*(-0.19196016538)*x_ref[1]**(o-1)+(0.426264369437)
      else:
        arg=(0.00913595455019)*x[0]**o+(-0.561918629059)*x[0]+(0.0164102679759)*x[1]**o+(0.20749162111)*x[1]+(0.588703418401)*x[2]**o+(0.841289333498)*x[2]
        ref[0]=o*(0.00913595455019)*x_ref[0]**(o-1)+(-0.561918629059)
        ref[1]=o*(0.0164102679759)*x_ref[1]**(o-1)+(0.20749162111)
        ref[2]=o*(0.588703418401)*x_ref[2]**(o-1)+(0.841289333498)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.290031175706)*x[0]**o+(0.690121137589)*x[0]+(0.916232953272)*x[1]**o+(0.427227803135)*x[1]
        arg[1]=(0.0325792679154)*x[0]**o+(0.773157737135)*x[0]+(-0.0443953478357)*x[1]**o+(0.772028975294)*x[1]
        ref[0,0]=o*(-0.290031175706)*x_ref[0]**(o-1)+(0.690121137589)
        ref[0,1]=o*(0.916232953272)*x_ref[1]**(o-1)+(0.427227803135)
        ref[1,0]=o*(0.0325792679154)*x_ref[0]**(o-1)+(0.773157737135)
        ref[1,1]=o*(-0.0443953478357)*x_ref[1]**(o-1)+(0.772028975294)
      else:
        arg[0]=(-0.119338345858)*x[0]**o+(-0.866134437839)*x[0]+(-0.727685742079)*x[1]**o+(0.465666609669)*x[1]+(-0.351266733199)*x[2]**o+(-0.960631632698)*x[2]
        arg[1]=(-0.731037524737)*x[0]**o+(0.415975622707)*x[0]+(-0.86639696362)*x[1]**o+(0.914277406921)*x[1]+(0.627502485514)*x[2]**o+(-0.855013681276)*x[2]
        ref[0,0]=o*(-0.119338345858)*x_ref[0]**(o-1)+(-0.866134437839)
        ref[0,1]=o*(-0.727685742079)*x_ref[1]**(o-1)+(0.465666609669)
        ref[0,2]=o*(-0.351266733199)*x_ref[2]**(o-1)+(-0.960631632698)
        ref[1,0]=o*(-0.731037524737)*x_ref[0]**(o-1)+(0.415975622707)
        ref[1,1]=o*(-0.86639696362)*x_ref[1]**(o-1)+(0.914277406921)
        ref[1,2]=o*(0.627502485514)*x_ref[2]**(o-1)+(-0.855013681276)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.148320408183)*x[0]**o+(-0.340902870434)*x[0]+(-0.0856445410272)*x[1]**o+(0.220499304831)*x[1]
        arg[0,1]=(0.915892845602)*x[0]**o+(-0.04770770327)*x[0]+(0.482403170451)*x[1]**o+(-0.639478756575)*x[1]
        arg[0,2]=(-0.109204159071)*x[0]**o+(0.641607024805)*x[0]+(-0.409310547283)*x[1]**o+(0.202984386731)*x[1]
        arg[0,3]=(0.436833770493)*x[0]**o+(-0.200611791892)*x[0]+(-0.74814554517)*x[1]**o+(0.571881526754)*x[1]
        arg[0,4]=(0.704812299415)*x[0]**o+(0.688151134045)*x[0]+(0.486135712907)*x[1]**o+(0.955314240096)*x[1]
        arg[1,0]=(-0.731790373321)*x[0]**o+(-0.711258122556)*x[0]+(-0.961514666684)*x[1]**o+(0.726573978492)*x[1]
        arg[1,1]=(0.975372389889)*x[0]**o+(0.5026238435)*x[0]+(0.948955767945)*x[1]**o+(-0.780842035663)*x[1]
        arg[1,2]=(0.742383399984)*x[0]**o+(-0.729678661652)*x[0]+(-0.815499612514)*x[1]**o+(-0.12280674993)*x[1]
        arg[1,3]=(0.323809901662)*x[0]**o+(-0.441906205977)*x[0]+(-0.308989501913)*x[1]**o+(-0.549353671447)*x[1]
        arg[1,4]=(0.711285710752)*x[0]**o+(0.297823778866)*x[0]+(0.645185023747)*x[1]**o+(0.825620074525)*x[1]
        arg[2,0]=(-0.373175217644)*x[0]**o+(-0.25651286112)*x[0]+(-0.46193047583)*x[1]**o+(-0.359820938943)*x[1]
        arg[2,1]=(-0.406101756283)*x[0]**o+(-0.38896428223)*x[0]+(-0.993257693721)*x[1]**o+(-0.910326366305)*x[1]
        arg[2,2]=(0.517566718504)*x[0]**o+(-0.582940900248)*x[0]+(-0.381600156082)*x[1]**o+(0.466575604699)*x[1]
        arg[2,3]=(0.323944380062)*x[0]**o+(-0.99359065412)*x[0]+(0.069389014076)*x[1]**o+(0.960192708612)*x[1]
        arg[2,4]=(-0.741362989132)*x[0]**o+(0.978137520347)*x[0]+(-0.398259900603)*x[1]**o+(-0.224949732925)*x[1]
        arg[3,0]=(-0.540235240193)*x[0]**o+(0.761488584116)*x[0]+(-0.100229643779)*x[1]**o+(-0.168724555964)*x[1]
        arg[3,1]=(0.565068220879)*x[0]**o+(-0.778936552173)*x[0]+(-0.933478028833)*x[1]**o+(-0.274446893155)*x[1]
        arg[3,2]=(-0.198878845493)*x[0]**o+(0.97185846381)*x[0]+(0.0345948289236)*x[1]**o+(0.359375885935)*x[1]
        arg[3,3]=(-0.815426880937)*x[0]**o+(-0.0734806259357)*x[0]+(-0.0452925239934)*x[1]**o+(0.976167480949)*x[1]
        arg[3,4]=(0.374670688698)*x[0]**o+(-0.427666745858)*x[0]+(-0.631640994091)*x[1]**o+(-0.312455066404)*x[1]
        ref[0,0,0]=o*(0.148320408183)*x_ref[0]**(o-1)+(-0.340902870434)
        ref[0,0,1]=o*(-0.0856445410272)*x_ref[1]**(o-1)+(0.220499304831)
        ref[0,1,0]=o*(0.915892845602)*x_ref[0]**(o-1)+(-0.04770770327)
        ref[0,1,1]=o*(0.482403170451)*x_ref[1]**(o-1)+(-0.639478756575)
        ref[0,2,0]=o*(-0.109204159071)*x_ref[0]**(o-1)+(0.641607024805)
        ref[0,2,1]=o*(-0.409310547283)*x_ref[1]**(o-1)+(0.202984386731)
        ref[0,3,0]=o*(0.436833770493)*x_ref[0]**(o-1)+(-0.200611791892)
        ref[0,3,1]=o*(-0.74814554517)*x_ref[1]**(o-1)+(0.571881526754)
        ref[0,4,0]=o*(0.704812299415)*x_ref[0]**(o-1)+(0.688151134045)
        ref[0,4,1]=o*(0.486135712907)*x_ref[1]**(o-1)+(0.955314240096)
        ref[1,0,0]=o*(-0.731790373321)*x_ref[0]**(o-1)+(-0.711258122556)
        ref[1,0,1]=o*(-0.961514666684)*x_ref[1]**(o-1)+(0.726573978492)
        ref[1,1,0]=o*(0.975372389889)*x_ref[0]**(o-1)+(0.5026238435)
        ref[1,1,1]=o*(0.948955767945)*x_ref[1]**(o-1)+(-0.780842035663)
        ref[1,2,0]=o*(0.742383399984)*x_ref[0]**(o-1)+(-0.729678661652)
        ref[1,2,1]=o*(-0.815499612514)*x_ref[1]**(o-1)+(-0.12280674993)
        ref[1,3,0]=o*(0.323809901662)*x_ref[0]**(o-1)+(-0.441906205977)
        ref[1,3,1]=o*(-0.308989501913)*x_ref[1]**(o-1)+(-0.549353671447)
        ref[1,4,0]=o*(0.711285710752)*x_ref[0]**(o-1)+(0.297823778866)
        ref[1,4,1]=o*(0.645185023747)*x_ref[1]**(o-1)+(0.825620074525)
        ref[2,0,0]=o*(-0.373175217644)*x_ref[0]**(o-1)+(-0.25651286112)
        ref[2,0,1]=o*(-0.46193047583)*x_ref[1]**(o-1)+(-0.359820938943)
        ref[2,1,0]=o*(-0.406101756283)*x_ref[0]**(o-1)+(-0.38896428223)
        ref[2,1,1]=o*(-0.993257693721)*x_ref[1]**(o-1)+(-0.910326366305)
        ref[2,2,0]=o*(0.517566718504)*x_ref[0]**(o-1)+(-0.582940900248)
        ref[2,2,1]=o*(-0.381600156082)*x_ref[1]**(o-1)+(0.466575604699)
        ref[2,3,0]=o*(0.323944380062)*x_ref[0]**(o-1)+(-0.99359065412)
        ref[2,3,1]=o*(0.069389014076)*x_ref[1]**(o-1)+(0.960192708612)
        ref[2,4,0]=o*(-0.741362989132)*x_ref[0]**(o-1)+(0.978137520347)
        ref[2,4,1]=o*(-0.398259900603)*x_ref[1]**(o-1)+(-0.224949732925)
        ref[3,0,0]=o*(-0.540235240193)*x_ref[0]**(o-1)+(0.761488584116)
        ref[3,0,1]=o*(-0.100229643779)*x_ref[1]**(o-1)+(-0.168724555964)
        ref[3,1,0]=o*(0.565068220879)*x_ref[0]**(o-1)+(-0.778936552173)
        ref[3,1,1]=o*(-0.933478028833)*x_ref[1]**(o-1)+(-0.274446893155)
        ref[3,2,0]=o*(-0.198878845493)*x_ref[0]**(o-1)+(0.97185846381)
        ref[3,2,1]=o*(0.0345948289236)*x_ref[1]**(o-1)+(0.359375885935)
        ref[3,3,0]=o*(-0.815426880937)*x_ref[0]**(o-1)+(-0.0734806259357)
        ref[3,3,1]=o*(-0.0452925239934)*x_ref[1]**(o-1)+(0.976167480949)
        ref[3,4,0]=o*(0.374670688698)*x_ref[0]**(o-1)+(-0.427666745858)
        ref[3,4,1]=o*(-0.631640994091)*x_ref[1]**(o-1)+(-0.312455066404)
      else:
        arg[0,0]=(0.284398400678)*x[0]**o+(0.219396164435)*x[0]+(0.220563761332)*x[1]**o+(0.167554964517)*x[1]+(0.234139558431)*x[2]**o+(-0.462262273341)*x[2]
        arg[0,1]=(0.161107248226)*x[0]**o+(0.23981806792)*x[0]+(-0.365265066245)*x[1]**o+(0.280522162689)*x[1]+(0.0813974911438)*x[2]**o+(-0.43305486384)*x[2]
        arg[0,2]=(0.813480874845)*x[0]**o+(0.600760069636)*x[0]+(-0.133815038485)*x[1]**o+(-0.954192097752)*x[1]+(-0.60756978514)*x[2]**o+(0.669710363926)*x[2]
        arg[0,3]=(-0.760542897936)*x[0]**o+(0.869702837484)*x[0]+(0.568626088907)*x[1]**o+(0.272054244422)*x[1]+(-0.0415999983147)*x[2]**o+(0.446199878551)*x[2]
        arg[0,4]=(0.0453200557409)*x[0]**o+(0.990444244175)*x[0]+(0.134265180195)*x[1]**o+(0.156166931353)*x[1]+(-0.787964274948)*x[2]**o+(0.0116920503615)*x[2]
        arg[1,0]=(-0.826245054647)*x[0]**o+(0.203727421041)*x[0]+(0.435254937068)*x[1]**o+(-0.271957510058)*x[1]+(0.24056400787)*x[2]**o+(-0.270322614397)*x[2]
        arg[1,1]=(-0.423339337474)*x[0]**o+(0.272439137387)*x[0]+(-0.0736213619539)*x[1]**o+(-0.776123259153)*x[1]+(-0.751736227059)*x[2]**o+(0.944253810697)*x[2]
        arg[1,2]=(0.189348519506)*x[0]**o+(-0.599085006375)*x[0]+(0.511802349511)*x[1]**o+(-0.309756999526)*x[1]+(-0.753093308971)*x[2]**o+(-0.109878524762)*x[2]
        arg[1,3]=(0.273889782311)*x[0]**o+(0.284416273071)*x[0]+(-0.793016836976)*x[1]**o+(0.584657111427)*x[1]+(0.202246448215)*x[2]**o+(0.669336280646)*x[2]
        arg[1,4]=(-0.310062687494)*x[0]**o+(-0.0558492819571)*x[0]+(0.661043054703)*x[1]**o+(0.708531199864)*x[1]+(0.0490202348501)*x[2]**o+(0.732983691308)*x[2]
        arg[2,0]=(0.967228638923)*x[0]**o+(-0.816725187456)*x[0]+(0.836005904943)*x[1]**o+(-0.618363868727)*x[1]+(-0.977101150283)*x[2]**o+(-0.960030792428)*x[2]
        arg[2,1]=(0.536503091036)*x[0]**o+(-0.786194077149)*x[0]+(-0.337609174718)*x[1]**o+(0.787326702105)*x[1]+(0.989900210061)*x[2]**o+(0.67077581859)*x[2]
        arg[2,2]=(-0.732242113661)*x[0]**o+(-0.565641876648)*x[0]+(0.175986347852)*x[1]**o+(0.862449074966)*x[1]+(0.0257656069443)*x[2]**o+(0.824927753427)*x[2]
        arg[2,3]=(-0.863272499797)*x[0]**o+(-0.817781186551)*x[0]+(0.168931269996)*x[1]**o+(-0.530060186715)*x[1]+(-0.516801368376)*x[2]**o+(-0.570137941882)*x[2]
        arg[2,4]=(-0.675287850319)*x[0]**o+(-0.672701614891)*x[0]+(-0.19851266374)*x[1]**o+(-0.0494587000627)*x[1]+(0.518006592112)*x[2]**o+(0.0337938303974)*x[2]
        arg[3,0]=(-0.175683006965)*x[0]**o+(-0.843980579113)*x[0]+(0.903358518604)*x[1]**o+(0.962156465407)*x[1]+(-0.0887392353523)*x[2]**o+(0.524587238729)*x[2]
        arg[3,1]=(0.0622433182846)*x[0]**o+(-0.756701969275)*x[0]+(0.721836266349)*x[1]**o+(0.457673032865)*x[1]+(0.704951904865)*x[2]**o+(0.781015828813)*x[2]
        arg[3,2]=(-0.457057269128)*x[0]**o+(-0.880403805028)*x[0]+(0.103553852578)*x[1]**o+(-0.807061998574)*x[1]+(-0.0352722677587)*x[2]**o+(0.0527948564177)*x[2]
        arg[3,3]=(0.161459807817)*x[0]**o+(0.698509459014)*x[0]+(-0.596487730094)*x[1]**o+(-0.906646770174)*x[1]+(0.567036380682)*x[2]**o+(0.490123167386)*x[2]
        arg[3,4]=(-0.668686195082)*x[0]**o+(0.911485638627)*x[0]+(-0.856948411336)*x[1]**o+(0.0348927535379)*x[1]+(0.17649505057)*x[2]**o+(-0.778920185271)*x[2]
        ref[0,0,0]=o*(0.284398400678)*x_ref[0]**(o-1)+(0.219396164435)
        ref[0,0,1]=o*(0.220563761332)*x_ref[1]**(o-1)+(0.167554964517)
        ref[0,0,2]=o*(0.234139558431)*x_ref[2]**(o-1)+(-0.462262273341)
        ref[0,1,0]=o*(0.161107248226)*x_ref[0]**(o-1)+(0.23981806792)
        ref[0,1,1]=o*(-0.365265066245)*x_ref[1]**(o-1)+(0.280522162689)
        ref[0,1,2]=o*(0.0813974911438)*x_ref[2]**(o-1)+(-0.43305486384)
        ref[0,2,0]=o*(0.813480874845)*x_ref[0]**(o-1)+(0.600760069636)
        ref[0,2,1]=o*(-0.133815038485)*x_ref[1]**(o-1)+(-0.954192097752)
        ref[0,2,2]=o*(-0.60756978514)*x_ref[2]**(o-1)+(0.669710363926)
        ref[0,3,0]=o*(-0.760542897936)*x_ref[0]**(o-1)+(0.869702837484)
        ref[0,3,1]=o*(0.568626088907)*x_ref[1]**(o-1)+(0.272054244422)
        ref[0,3,2]=o*(-0.0415999983147)*x_ref[2]**(o-1)+(0.446199878551)
        ref[0,4,0]=o*(0.0453200557409)*x_ref[0]**(o-1)+(0.990444244175)
        ref[0,4,1]=o*(0.134265180195)*x_ref[1]**(o-1)+(0.156166931353)
        ref[0,4,2]=o*(-0.787964274948)*x_ref[2]**(o-1)+(0.0116920503615)
        ref[1,0,0]=o*(-0.826245054647)*x_ref[0]**(o-1)+(0.203727421041)
        ref[1,0,1]=o*(0.435254937068)*x_ref[1]**(o-1)+(-0.271957510058)
        ref[1,0,2]=o*(0.24056400787)*x_ref[2]**(o-1)+(-0.270322614397)
        ref[1,1,0]=o*(-0.423339337474)*x_ref[0]**(o-1)+(0.272439137387)
        ref[1,1,1]=o*(-0.0736213619539)*x_ref[1]**(o-1)+(-0.776123259153)
        ref[1,1,2]=o*(-0.751736227059)*x_ref[2]**(o-1)+(0.944253810697)
        ref[1,2,0]=o*(0.189348519506)*x_ref[0]**(o-1)+(-0.599085006375)
        ref[1,2,1]=o*(0.511802349511)*x_ref[1]**(o-1)+(-0.309756999526)
        ref[1,2,2]=o*(-0.753093308971)*x_ref[2]**(o-1)+(-0.109878524762)
        ref[1,3,0]=o*(0.273889782311)*x_ref[0]**(o-1)+(0.284416273071)
        ref[1,3,1]=o*(-0.793016836976)*x_ref[1]**(o-1)+(0.584657111427)
        ref[1,3,2]=o*(0.202246448215)*x_ref[2]**(o-1)+(0.669336280646)
        ref[1,4,0]=o*(-0.310062687494)*x_ref[0]**(o-1)+(-0.0558492819571)
        ref[1,4,1]=o*(0.661043054703)*x_ref[1]**(o-1)+(0.708531199864)
        ref[1,4,2]=o*(0.0490202348501)*x_ref[2]**(o-1)+(0.732983691308)
        ref[2,0,0]=o*(0.967228638923)*x_ref[0]**(o-1)+(-0.816725187456)
        ref[2,0,1]=o*(0.836005904943)*x_ref[1]**(o-1)+(-0.618363868727)
        ref[2,0,2]=o*(-0.977101150283)*x_ref[2]**(o-1)+(-0.960030792428)
        ref[2,1,0]=o*(0.536503091036)*x_ref[0]**(o-1)+(-0.786194077149)
        ref[2,1,1]=o*(-0.337609174718)*x_ref[1]**(o-1)+(0.787326702105)
        ref[2,1,2]=o*(0.989900210061)*x_ref[2]**(o-1)+(0.67077581859)
        ref[2,2,0]=o*(-0.732242113661)*x_ref[0]**(o-1)+(-0.565641876648)
        ref[2,2,1]=o*(0.175986347852)*x_ref[1]**(o-1)+(0.862449074966)
        ref[2,2,2]=o*(0.0257656069443)*x_ref[2]**(o-1)+(0.824927753427)
        ref[2,3,0]=o*(-0.863272499797)*x_ref[0]**(o-1)+(-0.817781186551)
        ref[2,3,1]=o*(0.168931269996)*x_ref[1]**(o-1)+(-0.530060186715)
        ref[2,3,2]=o*(-0.516801368376)*x_ref[2]**(o-1)+(-0.570137941882)
        ref[2,4,0]=o*(-0.675287850319)*x_ref[0]**(o-1)+(-0.672701614891)
        ref[2,4,1]=o*(-0.19851266374)*x_ref[1]**(o-1)+(-0.0494587000627)
        ref[2,4,2]=o*(0.518006592112)*x_ref[2]**(o-1)+(0.0337938303974)
        ref[3,0,0]=o*(-0.175683006965)*x_ref[0]**(o-1)+(-0.843980579113)
        ref[3,0,1]=o*(0.903358518604)*x_ref[1]**(o-1)+(0.962156465407)
        ref[3,0,2]=o*(-0.0887392353523)*x_ref[2]**(o-1)+(0.524587238729)
        ref[3,1,0]=o*(0.0622433182846)*x_ref[0]**(o-1)+(-0.756701969275)
        ref[3,1,1]=o*(0.721836266349)*x_ref[1]**(o-1)+(0.457673032865)
        ref[3,1,2]=o*(0.704951904865)*x_ref[2]**(o-1)+(0.781015828813)
        ref[3,2,0]=o*(-0.457057269128)*x_ref[0]**(o-1)+(-0.880403805028)
        ref[3,2,1]=o*(0.103553852578)*x_ref[1]**(o-1)+(-0.807061998574)
        ref[3,2,2]=o*(-0.0352722677587)*x_ref[2]**(o-1)+(0.0527948564177)
        ref[3,3,0]=o*(0.161459807817)*x_ref[0]**(o-1)+(0.698509459014)
        ref[3,3,1]=o*(-0.596487730094)*x_ref[1]**(o-1)+(-0.906646770174)
        ref[3,3,2]=o*(0.567036380682)*x_ref[2]**(o-1)+(0.490123167386)
        ref[3,4,0]=o*(-0.668686195082)*x_ref[0]**(o-1)+(0.911485638627)
        ref[3,4,1]=o*(-0.856948411336)*x_ref[1]**(o-1)+(0.0348927535379)
        ref[3,4,2]=o*(0.17649505057)*x_ref[2]**(o-1)+(-0.778920185271)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.366719126165)*x[0]**o+(0.273305723392)*x[0]+(0.508134048679)*x[1]**o+(-0.0895788966541)*x[1]
        arg[0,0,1]=(-0.911986850385)*x[0]**o+(-0.98503317507)*x[0]+(-0.310897248279)*x[1]**o+(0.015206007134)*x[1]
        arg[0,1,0]=(-0.0827324676842)*x[0]**o+(0.566620026269)*x[0]+(-0.0581063189367)*x[1]**o+(0.379530936223)*x[1]
        arg[0,1,1]=(0.223247684076)*x[0]**o+(0.511247996036)*x[0]+(-0.904638808138)*x[1]**o+(-0.253982863496)*x[1]
        arg[1,0,0]=(-0.117040630362)*x[0]**o+(0.720200175398)*x[0]+(0.316014562047)*x[1]**o+(-0.88437304478)*x[1]
        arg[1,0,1]=(0.521261471788)*x[0]**o+(0.866618380963)*x[0]+(0.376866254345)*x[1]**o+(0.0905522236713)*x[1]
        arg[1,1,0]=(-0.0907011175214)*x[0]**o+(0.925809501719)*x[0]+(0.29066676347)*x[1]**o+(-0.77392365917)*x[1]
        arg[1,1,1]=(0.410922874646)*x[0]**o+(-0.801953959211)*x[0]+(-0.911104174539)*x[1]**o+(-0.640661859542)*x[1]
        arg[2,0,0]=(-0.449471274267)*x[0]**o+(0.8111872512)*x[0]+(0.0498814189394)*x[1]**o+(0.683703882124)*x[1]
        arg[2,0,1]=(-0.192396495147)*x[0]**o+(0.893457442524)*x[0]+(-0.946936565887)*x[1]**o+(-0.0290255758634)*x[1]
        arg[2,1,0]=(0.90847535637)*x[0]**o+(-0.517929281823)*x[0]+(0.389994692606)*x[1]**o+(-0.601708219886)*x[1]
        arg[2,1,1]=(0.401910281699)*x[0]**o+(0.485033224067)*x[0]+(-0.362139392478)*x[1]**o+(-0.740542640224)*x[1]
        arg[3,0,0]=(0.195815356554)*x[0]**o+(-0.584270182199)*x[0]+(-0.773247762709)*x[1]**o+(0.974109226972)*x[1]
        arg[3,0,1]=(0.516821334505)*x[0]**o+(-0.667535261235)*x[0]+(-0.687554420634)*x[1]**o+(-0.826211449593)*x[1]
        arg[3,1,0]=(-0.886684033283)*x[0]**o+(0.833394822235)*x[0]+(-0.509561014765)*x[1]**o+(-0.934990250803)*x[1]
        arg[3,1,1]=(-0.279505984235)*x[0]**o+(0.629210321924)*x[0]+(-0.245033518843)*x[1]**o+(0.148437993524)*x[1]
        arg[4,0,0]=(-0.120914130766)*x[0]**o+(-0.70386339291)*x[0]+(-0.37739143589)*x[1]**o+(-0.604979315802)*x[1]
        arg[4,0,1]=(-0.259492475213)*x[0]**o+(0.628922768093)*x[0]+(-0.973387886178)*x[1]**o+(0.285217747252)*x[1]
        arg[4,1,0]=(-0.986163107858)*x[0]**o+(-0.85082996164)*x[0]+(-0.560402256475)*x[1]**o+(-0.468278543242)*x[1]
        arg[4,1,1]=(0.370258762453)*x[0]**o+(0.870396552858)*x[0]+(0.57205142923)*x[1]**o+(0.332787259857)*x[1]
        arg[5,0,0]=(-0.570453853889)*x[0]**o+(0.253977538115)*x[0]+(0.628545140288)*x[1]**o+(-0.766550576107)*x[1]
        arg[5,0,1]=(-0.626776709555)*x[0]**o+(0.593526092433)*x[0]+(0.824052315861)*x[1]**o+(-0.409883522469)*x[1]
        arg[5,1,0]=(0.285256467088)*x[0]**o+(-0.808675657802)*x[0]+(-0.818251043807)*x[1]**o+(0.378756627769)*x[1]
        arg[5,1,1]=(-0.810805592247)*x[0]**o+(-0.879886206051)*x[0]+(0.378171206533)*x[1]**o+(0.478469089448)*x[1]
        ref[0,0,0,0]=o*(0.366719126165)*x_ref[0]**(o-1)+(0.273305723392)
        ref[0,0,0,1]=o*(0.508134048679)*x_ref[1]**(o-1)+(-0.0895788966541)
        ref[0,0,1,0]=o*(-0.911986850385)*x_ref[0]**(o-1)+(-0.98503317507)
        ref[0,0,1,1]=o*(-0.310897248279)*x_ref[1]**(o-1)+(0.015206007134)
        ref[0,1,0,0]=o*(-0.0827324676842)*x_ref[0]**(o-1)+(0.566620026269)
        ref[0,1,0,1]=o*(-0.0581063189367)*x_ref[1]**(o-1)+(0.379530936223)
        ref[0,1,1,0]=o*(0.223247684076)*x_ref[0]**(o-1)+(0.511247996036)
        ref[0,1,1,1]=o*(-0.904638808138)*x_ref[1]**(o-1)+(-0.253982863496)
        ref[1,0,0,0]=o*(-0.117040630362)*x_ref[0]**(o-1)+(0.720200175398)
        ref[1,0,0,1]=o*(0.316014562047)*x_ref[1]**(o-1)+(-0.88437304478)
        ref[1,0,1,0]=o*(0.521261471788)*x_ref[0]**(o-1)+(0.866618380963)
        ref[1,0,1,1]=o*(0.376866254345)*x_ref[1]**(o-1)+(0.0905522236713)
        ref[1,1,0,0]=o*(-0.0907011175214)*x_ref[0]**(o-1)+(0.925809501719)
        ref[1,1,0,1]=o*(0.29066676347)*x_ref[1]**(o-1)+(-0.77392365917)
        ref[1,1,1,0]=o*(0.410922874646)*x_ref[0]**(o-1)+(-0.801953959211)
        ref[1,1,1,1]=o*(-0.911104174539)*x_ref[1]**(o-1)+(-0.640661859542)
        ref[2,0,0,0]=o*(-0.449471274267)*x_ref[0]**(o-1)+(0.8111872512)
        ref[2,0,0,1]=o*(0.0498814189394)*x_ref[1]**(o-1)+(0.683703882124)
        ref[2,0,1,0]=o*(-0.192396495147)*x_ref[0]**(o-1)+(0.893457442524)
        ref[2,0,1,1]=o*(-0.946936565887)*x_ref[1]**(o-1)+(-0.0290255758634)
        ref[2,1,0,0]=o*(0.90847535637)*x_ref[0]**(o-1)+(-0.517929281823)
        ref[2,1,0,1]=o*(0.389994692606)*x_ref[1]**(o-1)+(-0.601708219886)
        ref[2,1,1,0]=o*(0.401910281699)*x_ref[0]**(o-1)+(0.485033224067)
        ref[2,1,1,1]=o*(-0.362139392478)*x_ref[1]**(o-1)+(-0.740542640224)
        ref[3,0,0,0]=o*(0.195815356554)*x_ref[0]**(o-1)+(-0.584270182199)
        ref[3,0,0,1]=o*(-0.773247762709)*x_ref[1]**(o-1)+(0.974109226972)
        ref[3,0,1,0]=o*(0.516821334505)*x_ref[0]**(o-1)+(-0.667535261235)
        ref[3,0,1,1]=o*(-0.687554420634)*x_ref[1]**(o-1)+(-0.826211449593)
        ref[3,1,0,0]=o*(-0.886684033283)*x_ref[0]**(o-1)+(0.833394822235)
        ref[3,1,0,1]=o*(-0.509561014765)*x_ref[1]**(o-1)+(-0.934990250803)
        ref[3,1,1,0]=o*(-0.279505984235)*x_ref[0]**(o-1)+(0.629210321924)
        ref[3,1,1,1]=o*(-0.245033518843)*x_ref[1]**(o-1)+(0.148437993524)
        ref[4,0,0,0]=o*(-0.120914130766)*x_ref[0]**(o-1)+(-0.70386339291)
        ref[4,0,0,1]=o*(-0.37739143589)*x_ref[1]**(o-1)+(-0.604979315802)
        ref[4,0,1,0]=o*(-0.259492475213)*x_ref[0]**(o-1)+(0.628922768093)
        ref[4,0,1,1]=o*(-0.973387886178)*x_ref[1]**(o-1)+(0.285217747252)
        ref[4,1,0,0]=o*(-0.986163107858)*x_ref[0]**(o-1)+(-0.85082996164)
        ref[4,1,0,1]=o*(-0.560402256475)*x_ref[1]**(o-1)+(-0.468278543242)
        ref[4,1,1,0]=o*(0.370258762453)*x_ref[0]**(o-1)+(0.870396552858)
        ref[4,1,1,1]=o*(0.57205142923)*x_ref[1]**(o-1)+(0.332787259857)
        ref[5,0,0,0]=o*(-0.570453853889)*x_ref[0]**(o-1)+(0.253977538115)
        ref[5,0,0,1]=o*(0.628545140288)*x_ref[1]**(o-1)+(-0.766550576107)
        ref[5,0,1,0]=o*(-0.626776709555)*x_ref[0]**(o-1)+(0.593526092433)
        ref[5,0,1,1]=o*(0.824052315861)*x_ref[1]**(o-1)+(-0.409883522469)
        ref[5,1,0,0]=o*(0.285256467088)*x_ref[0]**(o-1)+(-0.808675657802)
        ref[5,1,0,1]=o*(-0.818251043807)*x_ref[1]**(o-1)+(0.378756627769)
        ref[5,1,1,0]=o*(-0.810805592247)*x_ref[0]**(o-1)+(-0.879886206051)
        ref[5,1,1,1]=o*(0.378171206533)*x_ref[1]**(o-1)+(0.478469089448)
      else:
        arg[0,0,0]=(0.99032561737)*x[0]**o+(0.0664709329455)*x[0]+(0.118470702946)*x[1]**o+(0.339805181064)*x[1]+(-0.288819936068)*x[2]**o+(0.500626113335)*x[2]
        arg[0,0,1]=(-0.736833023385)*x[0]**o+(-0.160433791202)*x[0]+(-0.159295012314)*x[1]**o+(-0.0515197665154)*x[1]+(-0.199553046759)*x[2]**o+(0.912767397035)*x[2]
        arg[0,1,0]=(0.263643352646)*x[0]**o+(-0.408521561602)*x[0]+(-0.690441644678)*x[1]**o+(0.514925086967)*x[1]+(-0.321785011214)*x[2]**o+(-0.192767636106)*x[2]
        arg[0,1,1]=(-0.00679573309711)*x[0]**o+(-0.0919830030966)*x[0]+(0.0194635540782)*x[1]**o+(0.0364274479874)*x[1]+(0.324767149157)*x[2]**o+(-0.930770951272)*x[2]
        arg[1,0,0]=(0.129524159454)*x[0]**o+(-0.747948168258)*x[0]+(0.748091879992)*x[1]**o+(0.0701365933978)*x[1]+(-0.227595193537)*x[2]**o+(-0.515988222581)*x[2]
        arg[1,0,1]=(-0.369843455974)*x[0]**o+(0.902965907161)*x[0]+(0.841050267135)*x[1]**o+(0.985297243316)*x[1]+(-0.96057678523)*x[2]**o+(0.149329555515)*x[2]
        arg[1,1,0]=(-0.530918169849)*x[0]**o+(0.574734874263)*x[0]+(-0.56424658789)*x[1]**o+(-0.756154171146)*x[1]+(-0.181750429769)*x[2]**o+(-0.270972276027)*x[2]
        arg[1,1,1]=(0.18184543511)*x[0]**o+(0.810253164883)*x[0]+(-0.0289547932926)*x[1]**o+(0.690366471619)*x[1]+(0.862616271239)*x[2]**o+(0.157705360996)*x[2]
        arg[2,0,0]=(-0.732811475459)*x[0]**o+(-0.720788617286)*x[0]+(-0.72285785154)*x[1]**o+(-0.764360626538)*x[1]+(0.979115834974)*x[2]**o+(0.430570190112)*x[2]
        arg[2,0,1]=(-0.602364948602)*x[0]**o+(-0.313871272033)*x[0]+(0.931476652796)*x[1]**o+(-0.553237167378)*x[1]+(0.273773142735)*x[2]**o+(0.732796238419)*x[2]
        arg[2,1,0]=(0.723724167325)*x[0]**o+(-0.411330362765)*x[0]+(0.415411259564)*x[1]**o+(0.596006586583)*x[1]+(0.588177039382)*x[2]**o+(-0.365406082278)*x[2]
        arg[2,1,1]=(0.0622006916856)*x[0]**o+(0.583807018004)*x[0]+(0.292401319183)*x[1]**o+(-0.202393075873)*x[1]+(-0.544886623341)*x[2]**o+(-0.269087712755)*x[2]
        arg[3,0,0]=(-0.543339583883)*x[0]**o+(-0.594218090249)*x[0]+(0.463226896714)*x[1]**o+(0.613466907584)*x[1]+(0.147935164587)*x[2]**o+(0.696530808607)*x[2]
        arg[3,0,1]=(0.338109512519)*x[0]**o+(0.281428380284)*x[0]+(0.0445095482602)*x[1]**o+(0.898143737155)*x[1]+(0.0465131079784)*x[2]**o+(-0.311801195497)*x[2]
        arg[3,1,0]=(0.489699597199)*x[0]**o+(0.198461296558)*x[0]+(0.87785894323)*x[1]**o+(0.916226820395)*x[1]+(-0.473685913201)*x[2]**o+(-0.496145627682)*x[2]
        arg[3,1,1]=(0.249814020596)*x[0]**o+(-0.894315482252)*x[0]+(-0.778776373134)*x[1]**o+(0.760727700644)*x[1]+(0.743775735588)*x[2]**o+(-0.689040985439)*x[2]
        arg[4,0,0]=(-0.825052728563)*x[0]**o+(0.522046869865)*x[0]+(0.333964173928)*x[1]**o+(-0.76926539114)*x[1]+(0.412352181661)*x[2]**o+(-0.0067772174596)*x[2]
        arg[4,0,1]=(-0.478610156432)*x[0]**o+(0.750576911314)*x[0]+(-0.531813749953)*x[1]**o+(0.355951176778)*x[1]+(-0.852853671607)*x[2]**o+(0.980221079923)*x[2]
        arg[4,1,0]=(-0.641389506244)*x[0]**o+(0.881787042426)*x[0]+(-0.304080099649)*x[1]**o+(0.38448900132)*x[1]+(-0.047930997317)*x[2]**o+(-0.276038511441)*x[2]
        arg[4,1,1]=(0.606877338216)*x[0]**o+(0.693205674961)*x[0]+(-0.731785947563)*x[1]**o+(0.980888111857)*x[1]+(0.355758103413)*x[2]**o+(-0.493780352941)*x[2]
        arg[5,0,0]=(-0.542680086594)*x[0]**o+(-0.335712421489)*x[0]+(0.196257902892)*x[1]**o+(-0.923381865562)*x[1]+(0.396049043788)*x[2]**o+(0.231195219983)*x[2]
        arg[5,0,1]=(0.224328300764)*x[0]**o+(0.306254705923)*x[0]+(-0.298262471524)*x[1]**o+(0.318537412793)*x[1]+(-0.0971742408716)*x[2]**o+(-0.374440507273)*x[2]
        arg[5,1,0]=(-0.408985188836)*x[0]**o+(-0.932478279672)*x[0]+(0.700313488604)*x[1]**o+(0.259720889362)*x[1]+(0.862207003532)*x[2]**o+(0.646109275237)*x[2]
        arg[5,1,1]=(-0.0341740340851)*x[0]**o+(0.742119475866)*x[0]+(0.310640405448)*x[1]**o+(-0.295841203899)*x[1]+(-0.778064951853)*x[2]**o+(-0.847637829972)*x[2]
        ref[0,0,0,0]=o*(0.99032561737)*x_ref[0]**(o-1)+(0.0664709329455)
        ref[0,0,0,1]=o*(0.118470702946)*x_ref[1]**(o-1)+(0.339805181064)
        ref[0,0,0,2]=o*(-0.288819936068)*x_ref[2]**(o-1)+(0.500626113335)
        ref[0,0,1,0]=o*(-0.736833023385)*x_ref[0]**(o-1)+(-0.160433791202)
        ref[0,0,1,1]=o*(-0.159295012314)*x_ref[1]**(o-1)+(-0.0515197665154)
        ref[0,0,1,2]=o*(-0.199553046759)*x_ref[2]**(o-1)+(0.912767397035)
        ref[0,1,0,0]=o*(0.263643352646)*x_ref[0]**(o-1)+(-0.408521561602)
        ref[0,1,0,1]=o*(-0.690441644678)*x_ref[1]**(o-1)+(0.514925086967)
        ref[0,1,0,2]=o*(-0.321785011214)*x_ref[2]**(o-1)+(-0.192767636106)
        ref[0,1,1,0]=o*(-0.00679573309711)*x_ref[0]**(o-1)+(-0.0919830030966)
        ref[0,1,1,1]=o*(0.0194635540782)*x_ref[1]**(o-1)+(0.0364274479874)
        ref[0,1,1,2]=o*(0.324767149157)*x_ref[2]**(o-1)+(-0.930770951272)
        ref[1,0,0,0]=o*(0.129524159454)*x_ref[0]**(o-1)+(-0.747948168258)
        ref[1,0,0,1]=o*(0.748091879992)*x_ref[1]**(o-1)+(0.0701365933978)
        ref[1,0,0,2]=o*(-0.227595193537)*x_ref[2]**(o-1)+(-0.515988222581)
        ref[1,0,1,0]=o*(-0.369843455974)*x_ref[0]**(o-1)+(0.902965907161)
        ref[1,0,1,1]=o*(0.841050267135)*x_ref[1]**(o-1)+(0.985297243316)
        ref[1,0,1,2]=o*(-0.96057678523)*x_ref[2]**(o-1)+(0.149329555515)
        ref[1,1,0,0]=o*(-0.530918169849)*x_ref[0]**(o-1)+(0.574734874263)
        ref[1,1,0,1]=o*(-0.56424658789)*x_ref[1]**(o-1)+(-0.756154171146)
        ref[1,1,0,2]=o*(-0.181750429769)*x_ref[2]**(o-1)+(-0.270972276027)
        ref[1,1,1,0]=o*(0.18184543511)*x_ref[0]**(o-1)+(0.810253164883)
        ref[1,1,1,1]=o*(-0.0289547932926)*x_ref[1]**(o-1)+(0.690366471619)
        ref[1,1,1,2]=o*(0.862616271239)*x_ref[2]**(o-1)+(0.157705360996)
        ref[2,0,0,0]=o*(-0.732811475459)*x_ref[0]**(o-1)+(-0.720788617286)
        ref[2,0,0,1]=o*(-0.72285785154)*x_ref[1]**(o-1)+(-0.764360626538)
        ref[2,0,0,2]=o*(0.979115834974)*x_ref[2]**(o-1)+(0.430570190112)
        ref[2,0,1,0]=o*(-0.602364948602)*x_ref[0]**(o-1)+(-0.313871272033)
        ref[2,0,1,1]=o*(0.931476652796)*x_ref[1]**(o-1)+(-0.553237167378)
        ref[2,0,1,2]=o*(0.273773142735)*x_ref[2]**(o-1)+(0.732796238419)
        ref[2,1,0,0]=o*(0.723724167325)*x_ref[0]**(o-1)+(-0.411330362765)
        ref[2,1,0,1]=o*(0.415411259564)*x_ref[1]**(o-1)+(0.596006586583)
        ref[2,1,0,2]=o*(0.588177039382)*x_ref[2]**(o-1)+(-0.365406082278)
        ref[2,1,1,0]=o*(0.0622006916856)*x_ref[0]**(o-1)+(0.583807018004)
        ref[2,1,1,1]=o*(0.292401319183)*x_ref[1]**(o-1)+(-0.202393075873)
        ref[2,1,1,2]=o*(-0.544886623341)*x_ref[2]**(o-1)+(-0.269087712755)
        ref[3,0,0,0]=o*(-0.543339583883)*x_ref[0]**(o-1)+(-0.594218090249)
        ref[3,0,0,1]=o*(0.463226896714)*x_ref[1]**(o-1)+(0.613466907584)
        ref[3,0,0,2]=o*(0.147935164587)*x_ref[2]**(o-1)+(0.696530808607)
        ref[3,0,1,0]=o*(0.338109512519)*x_ref[0]**(o-1)+(0.281428380284)
        ref[3,0,1,1]=o*(0.0445095482602)*x_ref[1]**(o-1)+(0.898143737155)
        ref[3,0,1,2]=o*(0.0465131079784)*x_ref[2]**(o-1)+(-0.311801195497)
        ref[3,1,0,0]=o*(0.489699597199)*x_ref[0]**(o-1)+(0.198461296558)
        ref[3,1,0,1]=o*(0.87785894323)*x_ref[1]**(o-1)+(0.916226820395)
        ref[3,1,0,2]=o*(-0.473685913201)*x_ref[2]**(o-1)+(-0.496145627682)
        ref[3,1,1,0]=o*(0.249814020596)*x_ref[0]**(o-1)+(-0.894315482252)
        ref[3,1,1,1]=o*(-0.778776373134)*x_ref[1]**(o-1)+(0.760727700644)
        ref[3,1,1,2]=o*(0.743775735588)*x_ref[2]**(o-1)+(-0.689040985439)
        ref[4,0,0,0]=o*(-0.825052728563)*x_ref[0]**(o-1)+(0.522046869865)
        ref[4,0,0,1]=o*(0.333964173928)*x_ref[1]**(o-1)+(-0.76926539114)
        ref[4,0,0,2]=o*(0.412352181661)*x_ref[2]**(o-1)+(-0.0067772174596)
        ref[4,0,1,0]=o*(-0.478610156432)*x_ref[0]**(o-1)+(0.750576911314)
        ref[4,0,1,1]=o*(-0.531813749953)*x_ref[1]**(o-1)+(0.355951176778)
        ref[4,0,1,2]=o*(-0.852853671607)*x_ref[2]**(o-1)+(0.980221079923)
        ref[4,1,0,0]=o*(-0.641389506244)*x_ref[0]**(o-1)+(0.881787042426)
        ref[4,1,0,1]=o*(-0.304080099649)*x_ref[1]**(o-1)+(0.38448900132)
        ref[4,1,0,2]=o*(-0.047930997317)*x_ref[2]**(o-1)+(-0.276038511441)
        ref[4,1,1,0]=o*(0.606877338216)*x_ref[0]**(o-1)+(0.693205674961)
        ref[4,1,1,1]=o*(-0.731785947563)*x_ref[1]**(o-1)+(0.980888111857)
        ref[4,1,1,2]=o*(0.355758103413)*x_ref[2]**(o-1)+(-0.493780352941)
        ref[5,0,0,0]=o*(-0.542680086594)*x_ref[0]**(o-1)+(-0.335712421489)
        ref[5,0,0,1]=o*(0.196257902892)*x_ref[1]**(o-1)+(-0.923381865562)
        ref[5,0,0,2]=o*(0.396049043788)*x_ref[2]**(o-1)+(0.231195219983)
        ref[5,0,1,0]=o*(0.224328300764)*x_ref[0]**(o-1)+(0.306254705923)
        ref[5,0,1,1]=o*(-0.298262471524)*x_ref[1]**(o-1)+(0.318537412793)
        ref[5,0,1,2]=o*(-0.0971742408716)*x_ref[2]**(o-1)+(-0.374440507273)
        ref[5,1,0,0]=o*(-0.408985188836)*x_ref[0]**(o-1)+(-0.932478279672)
        ref[5,1,0,1]=o*(0.700313488604)*x_ref[1]**(o-1)+(0.259720889362)
        ref[5,1,0,2]=o*(0.862207003532)*x_ref[2]**(o-1)+(0.646109275237)
        ref[5,1,1,0]=o*(-0.0341740340851)*x_ref[0]**(o-1)+(0.742119475866)
        ref[5,1,1,1]=o*(0.310640405448)*x_ref[1]**(o-1)+(-0.295841203899)
        ref[5,1,1,2]=o*(-0.778064951853)*x_ref[2]**(o-1)+(-0.847637829972)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_Solution_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(-0.188926722839)*x[0]**o+(0.0528015322778)*x[0]+(0.394368405707)*x[1]**o+(0.573648619197)*x[1]
        ref[0]=o*(-0.188926722839)*x_ref[0]**(o-1)+(0.0528015322778)
        ref[1]=o*(0.394368405707)*x_ref[1]**(o-1)+(0.573648619197)
      else:
        arg=(0.208843709667)*x[0]**o+(-0.461780133723)*x[0]+(0.15028897973)*x[1]**o+(-0.594920423887)*x[1]+(0.517629763426)*x[2]**o+(0.403396285863)*x[2]
        ref[0]=o*(0.208843709667)*x_ref[0]**(o-1)+(-0.461780133723)
        ref[1]=o*(0.15028897973)*x_ref[1]**(o-1)+(-0.594920423887)
        ref[2]=o*(0.517629763426)*x_ref[2]**(o-1)+(0.403396285863)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_Solution_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(0.321445299816)*x[0]**o+(-0.133016832047)*x[0]+(0.663051729844)*x[1]**o+(-0.688058231943)*x[1]
        arg[1]=(-0.0269450330081)*x[0]**o+(0.0944332164606)*x[0]+(-0.285657276114)*x[1]**o+(-0.00151309619508)*x[1]
        ref[0,0]=o*(0.321445299816)*x_ref[0]**(o-1)+(-0.133016832047)
        ref[0,1]=o*(0.663051729844)*x_ref[1]**(o-1)+(-0.688058231943)
        ref[1,0]=o*(-0.0269450330081)*x_ref[0]**(o-1)+(0.0944332164606)
        ref[1,1]=o*(-0.285657276114)*x_ref[1]**(o-1)+(-0.00151309619508)
      else:
        arg[0]=(-0.779418963504)*x[0]**o+(-0.66017852882)*x[0]+(0.0796746454991)*x[1]**o+(-0.559782816375)*x[1]+(0.903645344893)*x[2]**o+(0.752370056114)*x[2]
        arg[1]=(-0.789198080015)*x[0]**o+(-0.61428352272)*x[0]+(-0.0678314090236)*x[1]**o+(-0.592189717525)*x[1]+(0.289759368386)*x[2]**o+(-0.612628874512)*x[2]
        ref[0,0]=o*(-0.779418963504)*x_ref[0]**(o-1)+(-0.66017852882)
        ref[0,1]=o*(0.0796746454991)*x_ref[1]**(o-1)+(-0.559782816375)
        ref[0,2]=o*(0.903645344893)*x_ref[2]**(o-1)+(0.752370056114)
        ref[1,0]=o*(-0.789198080015)*x_ref[0]**(o-1)+(-0.61428352272)
        ref[1,1]=o*(-0.0678314090236)*x_ref[1]**(o-1)+(-0.592189717525)
        ref[1,2]=o*(0.289759368386)*x_ref[2]**(o-1)+(-0.612628874512)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_Solution_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.23566716053)*x[0]**o+(-0.72341376636)*x[0]+(0.972511874619)*x[1]**o+(-0.915617397257)*x[1]
        arg[0,1]=(-0.767927641628)*x[0]**o+(0.381522882363)*x[0]+(0.147312331507)*x[1]**o+(0.593669546711)*x[1]
        arg[0,2]=(0.735354496752)*x[0]**o+(0.0328129903288)*x[0]+(-0.113236521952)*x[1]**o+(0.486761746739)*x[1]
        arg[0,3]=(-0.324209941994)*x[0]**o+(-0.235510827792)*x[0]+(0.167116943517)*x[1]**o+(0.0751459244008)*x[1]
        arg[0,4]=(-0.880038149806)*x[0]**o+(0.250698509807)*x[0]+(-0.249975000596)*x[1]**o+(-0.597287763096)*x[1]
        arg[1,0]=(0.0776176376816)*x[0]**o+(0.719231364575)*x[0]+(-0.573766708326)*x[1]**o+(-0.0657431038913)*x[1]
        arg[1,1]=(0.214586176047)*x[0]**o+(-0.542584637828)*x[0]+(-0.649916769602)*x[1]**o+(0.360674473897)*x[1]
        arg[1,2]=(-0.131191309002)*x[0]**o+(0.00956079049513)*x[0]+(-0.0399067337815)*x[1]**o+(-0.870803424616)*x[1]
        arg[1,3]=(-0.604243786872)*x[0]**o+(0.439677657963)*x[0]+(0.749342339715)*x[1]**o+(0.772192804507)*x[1]
        arg[1,4]=(0.461907488239)*x[0]**o+(-0.957858912937)*x[0]+(0.915893716197)*x[1]**o+(-0.858798101783)*x[1]
        arg[2,0]=(0.922598387398)*x[0]**o+(-0.194676923002)*x[0]+(-0.724972013231)*x[1]**o+(-0.83198174562)*x[1]
        arg[2,1]=(0.62648959348)*x[0]**o+(-0.968390708088)*x[0]+(0.955519507666)*x[1]**o+(0.0140489471727)*x[1]
        arg[2,2]=(-0.488249561275)*x[0]**o+(-0.354607683897)*x[0]+(-0.781278758499)*x[1]**o+(-0.206598205965)*x[1]
        arg[2,3]=(-0.503987693598)*x[0]**o+(0.581909066959)*x[0]+(0.133400791996)*x[1]**o+(-0.234595457546)*x[1]
        arg[2,4]=(-0.768789727431)*x[0]**o+(0.669013649323)*x[0]+(-0.618484878747)*x[1]**o+(-0.287461699457)*x[1]
        arg[3,0]=(0.125603852132)*x[0]**o+(0.83276119445)*x[0]+(-0.643188775199)*x[1]**o+(0.228300434999)*x[1]
        arg[3,1]=(0.645892506866)*x[0]**o+(0.701922167913)*x[0]+(-0.283958147744)*x[1]**o+(-0.781197564279)*x[1]
        arg[3,2]=(-0.0995754594857)*x[0]**o+(-0.106281614694)*x[0]+(0.00912723779796)*x[1]**o+(0.841448365384)*x[1]
        arg[3,3]=(-0.664740577186)*x[0]**o+(0.100401840489)*x[0]+(0.334300978415)*x[1]**o+(-0.383639183037)*x[1]
        arg[3,4]=(0.949394733501)*x[0]**o+(-0.0768203586407)*x[0]+(0.715238033192)*x[1]**o+(-0.175178433047)*x[1]
        ref[0,0,0]=o*(-0.23566716053)*x_ref[0]**(o-1)+(-0.72341376636)
        ref[0,0,1]=o*(0.972511874619)*x_ref[1]**(o-1)+(-0.915617397257)
        ref[0,1,0]=o*(-0.767927641628)*x_ref[0]**(o-1)+(0.381522882363)
        ref[0,1,1]=o*(0.147312331507)*x_ref[1]**(o-1)+(0.593669546711)
        ref[0,2,0]=o*(0.735354496752)*x_ref[0]**(o-1)+(0.0328129903288)
        ref[0,2,1]=o*(-0.113236521952)*x_ref[1]**(o-1)+(0.486761746739)
        ref[0,3,0]=o*(-0.324209941994)*x_ref[0]**(o-1)+(-0.235510827792)
        ref[0,3,1]=o*(0.167116943517)*x_ref[1]**(o-1)+(0.0751459244008)
        ref[0,4,0]=o*(-0.880038149806)*x_ref[0]**(o-1)+(0.250698509807)
        ref[0,4,1]=o*(-0.249975000596)*x_ref[1]**(o-1)+(-0.597287763096)
        ref[1,0,0]=o*(0.0776176376816)*x_ref[0]**(o-1)+(0.719231364575)
        ref[1,0,1]=o*(-0.573766708326)*x_ref[1]**(o-1)+(-0.0657431038913)
        ref[1,1,0]=o*(0.214586176047)*x_ref[0]**(o-1)+(-0.542584637828)
        ref[1,1,1]=o*(-0.649916769602)*x_ref[1]**(o-1)+(0.360674473897)
        ref[1,2,0]=o*(-0.131191309002)*x_ref[0]**(o-1)+(0.00956079049513)
        ref[1,2,1]=o*(-0.0399067337815)*x_ref[1]**(o-1)+(-0.870803424616)
        ref[1,3,0]=o*(-0.604243786872)*x_ref[0]**(o-1)+(0.439677657963)
        ref[1,3,1]=o*(0.749342339715)*x_ref[1]**(o-1)+(0.772192804507)
        ref[1,4,0]=o*(0.461907488239)*x_ref[0]**(o-1)+(-0.957858912937)
        ref[1,4,1]=o*(0.915893716197)*x_ref[1]**(o-1)+(-0.858798101783)
        ref[2,0,0]=o*(0.922598387398)*x_ref[0]**(o-1)+(-0.194676923002)
        ref[2,0,1]=o*(-0.724972013231)*x_ref[1]**(o-1)+(-0.83198174562)
        ref[2,1,0]=o*(0.62648959348)*x_ref[0]**(o-1)+(-0.968390708088)
        ref[2,1,1]=o*(0.955519507666)*x_ref[1]**(o-1)+(0.0140489471727)
        ref[2,2,0]=o*(-0.488249561275)*x_ref[0]**(o-1)+(-0.354607683897)
        ref[2,2,1]=o*(-0.781278758499)*x_ref[1]**(o-1)+(-0.206598205965)
        ref[2,3,0]=o*(-0.503987693598)*x_ref[0]**(o-1)+(0.581909066959)
        ref[2,3,1]=o*(0.133400791996)*x_ref[1]**(o-1)+(-0.234595457546)
        ref[2,4,0]=o*(-0.768789727431)*x_ref[0]**(o-1)+(0.669013649323)
        ref[2,4,1]=o*(-0.618484878747)*x_ref[1]**(o-1)+(-0.287461699457)
        ref[3,0,0]=o*(0.125603852132)*x_ref[0]**(o-1)+(0.83276119445)
        ref[3,0,1]=o*(-0.643188775199)*x_ref[1]**(o-1)+(0.228300434999)
        ref[3,1,0]=o*(0.645892506866)*x_ref[0]**(o-1)+(0.701922167913)
        ref[3,1,1]=o*(-0.283958147744)*x_ref[1]**(o-1)+(-0.781197564279)
        ref[3,2,0]=o*(-0.0995754594857)*x_ref[0]**(o-1)+(-0.106281614694)
        ref[3,2,1]=o*(0.00912723779796)*x_ref[1]**(o-1)+(0.841448365384)
        ref[3,3,0]=o*(-0.664740577186)*x_ref[0]**(o-1)+(0.100401840489)
        ref[3,3,1]=o*(0.334300978415)*x_ref[1]**(o-1)+(-0.383639183037)
        ref[3,4,0]=o*(0.949394733501)*x_ref[0]**(o-1)+(-0.0768203586407)
        ref[3,4,1]=o*(0.715238033192)*x_ref[1]**(o-1)+(-0.175178433047)
      else:
        arg[0,0]=(0.697509098917)*x[0]**o+(-0.571983680004)*x[0]+(0.456254556306)*x[1]**o+(0.407076527516)*x[1]+(-0.379511349609)*x[2]**o+(0.37799055755)*x[2]
        arg[0,1]=(-0.543310012054)*x[0]**o+(0.622613643381)*x[0]+(0.899028609397)*x[1]**o+(0.0358870746022)*x[1]+(0.923875380698)*x[2]**o+(-0.79605331054)*x[2]
        arg[0,2]=(0.613316685189)*x[0]**o+(0.308568047177)*x[0]+(0.0817110033202)*x[1]**o+(0.631167179752)*x[1]+(0.527259723046)*x[2]**o+(-0.44240882282)*x[2]
        arg[0,3]=(0.831261835833)*x[0]**o+(-0.0869775296331)*x[0]+(0.30622781536)*x[1]**o+(0.633056041347)*x[1]+(0.631243241273)*x[2]**o+(0.100112313541)*x[2]
        arg[0,4]=(-0.997030704248)*x[0]**o+(-0.374466395678)*x[0]+(0.0194535973217)*x[1]**o+(0.816505299829)*x[1]+(-0.412869831136)*x[2]**o+(-0.943976805293)*x[2]
        arg[1,0]=(0.555404162574)*x[0]**o+(-0.335389786458)*x[0]+(0.644585507787)*x[1]**o+(0.917280812233)*x[1]+(0.417410132456)*x[2]**o+(-0.26505427144)*x[2]
        arg[1,1]=(0.781185373116)*x[0]**o+(-0.852319927495)*x[0]+(-0.442528270016)*x[1]**o+(-0.538132919634)*x[1]+(-0.604047417588)*x[2]**o+(0.727395882535)*x[2]
        arg[1,2]=(-0.272462727471)*x[0]**o+(-0.834077558378)*x[0]+(0.763269297137)*x[1]**o+(-0.823180222291)*x[1]+(-0.162146758865)*x[2]**o+(0.595529366226)*x[2]
        arg[1,3]=(-0.0468429326976)*x[0]**o+(-0.179116342871)*x[0]+(-0.640563576973)*x[1]**o+(0.914070705704)*x[1]+(0.725297040984)*x[2]**o+(0.870302256055)*x[2]
        arg[1,4]=(-0.0632444730547)*x[0]**o+(-0.266315658984)*x[0]+(0.533681589116)*x[1]**o+(0.709939511175)*x[1]+(-0.60554532679)*x[2]**o+(0.915224044125)*x[2]
        arg[2,0]=(0.23330035938)*x[0]**o+(-0.385419740264)*x[0]+(0.120424890588)*x[1]**o+(0.591983224766)*x[1]+(0.925564960339)*x[2]**o+(-0.602528014193)*x[2]
        arg[2,1]=(0.786024569139)*x[0]**o+(0.619287163829)*x[0]+(-0.722187904611)*x[1]**o+(0.275352886478)*x[1]+(0.723591624164)*x[2]**o+(0.0909486746072)*x[2]
        arg[2,2]=(0.766293557291)*x[0]**o+(0.0610694643295)*x[0]+(0.375714644104)*x[1]**o+(-0.772092520637)*x[1]+(0.775058332825)*x[2]**o+(0.533525926942)*x[2]
        arg[2,3]=(-0.67254491898)*x[0]**o+(-0.0926113197516)*x[0]+(-0.615891770107)*x[1]**o+(-0.616600504392)*x[1]+(-0.808251600122)*x[2]**o+(-0.602309670223)*x[2]
        arg[2,4]=(0.80242562495)*x[0]**o+(0.359171734464)*x[0]+(-0.560256436586)*x[1]**o+(-0.420334736194)*x[1]+(0.871043977034)*x[2]**o+(0.156212055829)*x[2]
        arg[3,0]=(-0.0569856826934)*x[0]**o+(-0.381868544897)*x[0]+(-0.396013644801)*x[1]**o+(0.695329212849)*x[1]+(0.201188831017)*x[2]**o+(0.471078018432)*x[2]
        arg[3,1]=(0.28457474179)*x[0]**o+(-0.623305572154)*x[0]+(-0.442659858168)*x[1]**o+(0.725900635371)*x[1]+(-0.908206119008)*x[2]**o+(-0.190223696293)*x[2]
        arg[3,2]=(0.568043480074)*x[0]**o+(0.942235524274)*x[0]+(-0.720727900911)*x[1]**o+(0.771808109591)*x[1]+(-0.538996050264)*x[2]**o+(-0.641448383256)*x[2]
        arg[3,3]=(0.00707362989382)*x[0]**o+(0.139812417833)*x[0]+(-0.179986304531)*x[1]**o+(-0.546332565953)*x[1]+(-0.687783685855)*x[2]**o+(0.292513389166)*x[2]
        arg[3,4]=(0.657315930332)*x[0]**o+(0.0252304793386)*x[0]+(0.540737656651)*x[1]**o+(0.832327802479)*x[1]+(-0.822641904799)*x[2]**o+(-0.599972412377)*x[2]
        ref[0,0,0]=o*(0.697509098917)*x_ref[0]**(o-1)+(-0.571983680004)
        ref[0,0,1]=o*(0.456254556306)*x_ref[1]**(o-1)+(0.407076527516)
        ref[0,0,2]=o*(-0.379511349609)*x_ref[2]**(o-1)+(0.37799055755)
        ref[0,1,0]=o*(-0.543310012054)*x_ref[0]**(o-1)+(0.622613643381)
        ref[0,1,1]=o*(0.899028609397)*x_ref[1]**(o-1)+(0.0358870746022)
        ref[0,1,2]=o*(0.923875380698)*x_ref[2]**(o-1)+(-0.79605331054)
        ref[0,2,0]=o*(0.613316685189)*x_ref[0]**(o-1)+(0.308568047177)
        ref[0,2,1]=o*(0.0817110033202)*x_ref[1]**(o-1)+(0.631167179752)
        ref[0,2,2]=o*(0.527259723046)*x_ref[2]**(o-1)+(-0.44240882282)
        ref[0,3,0]=o*(0.831261835833)*x_ref[0]**(o-1)+(-0.0869775296331)
        ref[0,3,1]=o*(0.30622781536)*x_ref[1]**(o-1)+(0.633056041347)
        ref[0,3,2]=o*(0.631243241273)*x_ref[2]**(o-1)+(0.100112313541)
        ref[0,4,0]=o*(-0.997030704248)*x_ref[0]**(o-1)+(-0.374466395678)
        ref[0,4,1]=o*(0.0194535973217)*x_ref[1]**(o-1)+(0.816505299829)
        ref[0,4,2]=o*(-0.412869831136)*x_ref[2]**(o-1)+(-0.943976805293)
        ref[1,0,0]=o*(0.555404162574)*x_ref[0]**(o-1)+(-0.335389786458)
        ref[1,0,1]=o*(0.644585507787)*x_ref[1]**(o-1)+(0.917280812233)
        ref[1,0,2]=o*(0.417410132456)*x_ref[2]**(o-1)+(-0.26505427144)
        ref[1,1,0]=o*(0.781185373116)*x_ref[0]**(o-1)+(-0.852319927495)
        ref[1,1,1]=o*(-0.442528270016)*x_ref[1]**(o-1)+(-0.538132919634)
        ref[1,1,2]=o*(-0.604047417588)*x_ref[2]**(o-1)+(0.727395882535)
        ref[1,2,0]=o*(-0.272462727471)*x_ref[0]**(o-1)+(-0.834077558378)
        ref[1,2,1]=o*(0.763269297137)*x_ref[1]**(o-1)+(-0.823180222291)
        ref[1,2,2]=o*(-0.162146758865)*x_ref[2]**(o-1)+(0.595529366226)
        ref[1,3,0]=o*(-0.0468429326976)*x_ref[0]**(o-1)+(-0.179116342871)
        ref[1,3,1]=o*(-0.640563576973)*x_ref[1]**(o-1)+(0.914070705704)
        ref[1,3,2]=o*(0.725297040984)*x_ref[2]**(o-1)+(0.870302256055)
        ref[1,4,0]=o*(-0.0632444730547)*x_ref[0]**(o-1)+(-0.266315658984)
        ref[1,4,1]=o*(0.533681589116)*x_ref[1]**(o-1)+(0.709939511175)
        ref[1,4,2]=o*(-0.60554532679)*x_ref[2]**(o-1)+(0.915224044125)
        ref[2,0,0]=o*(0.23330035938)*x_ref[0]**(o-1)+(-0.385419740264)
        ref[2,0,1]=o*(0.120424890588)*x_ref[1]**(o-1)+(0.591983224766)
        ref[2,0,2]=o*(0.925564960339)*x_ref[2]**(o-1)+(-0.602528014193)
        ref[2,1,0]=o*(0.786024569139)*x_ref[0]**(o-1)+(0.619287163829)
        ref[2,1,1]=o*(-0.722187904611)*x_ref[1]**(o-1)+(0.275352886478)
        ref[2,1,2]=o*(0.723591624164)*x_ref[2]**(o-1)+(0.0909486746072)
        ref[2,2,0]=o*(0.766293557291)*x_ref[0]**(o-1)+(0.0610694643295)
        ref[2,2,1]=o*(0.375714644104)*x_ref[1]**(o-1)+(-0.772092520637)
        ref[2,2,2]=o*(0.775058332825)*x_ref[2]**(o-1)+(0.533525926942)
        ref[2,3,0]=o*(-0.67254491898)*x_ref[0]**(o-1)+(-0.0926113197516)
        ref[2,3,1]=o*(-0.615891770107)*x_ref[1]**(o-1)+(-0.616600504392)
        ref[2,3,2]=o*(-0.808251600122)*x_ref[2]**(o-1)+(-0.602309670223)
        ref[2,4,0]=o*(0.80242562495)*x_ref[0]**(o-1)+(0.359171734464)
        ref[2,4,1]=o*(-0.560256436586)*x_ref[1]**(o-1)+(-0.420334736194)
        ref[2,4,2]=o*(0.871043977034)*x_ref[2]**(o-1)+(0.156212055829)
        ref[3,0,0]=o*(-0.0569856826934)*x_ref[0]**(o-1)+(-0.381868544897)
        ref[3,0,1]=o*(-0.396013644801)*x_ref[1]**(o-1)+(0.695329212849)
        ref[3,0,2]=o*(0.201188831017)*x_ref[2]**(o-1)+(0.471078018432)
        ref[3,1,0]=o*(0.28457474179)*x_ref[0]**(o-1)+(-0.623305572154)
        ref[3,1,1]=o*(-0.442659858168)*x_ref[1]**(o-1)+(0.725900635371)
        ref[3,1,2]=o*(-0.908206119008)*x_ref[2]**(o-1)+(-0.190223696293)
        ref[3,2,0]=o*(0.568043480074)*x_ref[0]**(o-1)+(0.942235524274)
        ref[3,2,1]=o*(-0.720727900911)*x_ref[1]**(o-1)+(0.771808109591)
        ref[3,2,2]=o*(-0.538996050264)*x_ref[2]**(o-1)+(-0.641448383256)
        ref[3,3,0]=o*(0.00707362989382)*x_ref[0]**(o-1)+(0.139812417833)
        ref[3,3,1]=o*(-0.179986304531)*x_ref[1]**(o-1)+(-0.546332565953)
        ref[3,3,2]=o*(-0.687783685855)*x_ref[2]**(o-1)+(0.292513389166)
        ref[3,4,0]=o*(0.657315930332)*x_ref[0]**(o-1)+(0.0252304793386)
        ref[3,4,1]=o*(0.540737656651)*x_ref[1]**(o-1)+(0.832327802479)
        ref[3,4,2]=o*(-0.822641904799)*x_ref[2]**(o-1)+(-0.599972412377)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_Solution_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.711259674136)*x[0]**o+(0.597099394118)*x[0]+(0.656676641605)*x[1]**o+(0.800065074626)*x[1]
        arg[0,0,1]=(0.410575880869)*x[0]**o+(0.0290305819624)*x[0]+(-0.746442034948)*x[1]**o+(0.074826200729)*x[1]
        arg[0,1,0]=(0.478073638149)*x[0]**o+(0.856920711223)*x[0]+(0.517173340088)*x[1]**o+(-0.677846282063)*x[1]
        arg[0,1,1]=(0.285231489647)*x[0]**o+(0.0756593965609)*x[0]+(-0.355221676478)*x[1]**o+(-0.520564308221)*x[1]
        arg[1,0,0]=(-0.368777104698)*x[0]**o+(0.474996103371)*x[0]+(0.675654207136)*x[1]**o+(-0.0324979517659)*x[1]
        arg[1,0,1]=(0.498857835281)*x[0]**o+(-0.73836840989)*x[0]+(0.362789682422)*x[1]**o+(0.315425076703)*x[1]
        arg[1,1,0]=(-0.325384820094)*x[0]**o+(-0.895297521761)*x[0]+(0.563090918504)*x[1]**o+(0.589834648825)*x[1]
        arg[1,1,1]=(0.0378060061714)*x[0]**o+(-0.429064920806)*x[0]+(-0.467324629077)*x[1]**o+(0.938161415082)*x[1]
        arg[2,0,0]=(-0.527827738902)*x[0]**o+(-0.534425169092)*x[0]+(-0.185111533531)*x[1]**o+(-0.810076656622)*x[1]
        arg[2,0,1]=(-0.221712302033)*x[0]**o+(0.883619587624)*x[0]+(-0.972912188123)*x[1]**o+(0.0780565750992)*x[1]
        arg[2,1,0]=(-0.752942706408)*x[0]**o+(-0.153790659605)*x[0]+(-0.398384592556)*x[1]**o+(0.657319049074)*x[1]
        arg[2,1,1]=(0.428492876383)*x[0]**o+(0.427090189076)*x[0]+(-0.017400284896)*x[1]**o+(-0.730635641841)*x[1]
        arg[3,0,0]=(-0.644602330168)*x[0]**o+(0.706532342264)*x[0]+(-0.64532460698)*x[1]**o+(0.850059843173)*x[1]
        arg[3,0,1]=(-0.63066688687)*x[0]**o+(0.578132047851)*x[0]+(0.413335880906)*x[1]**o+(0.611233278281)*x[1]
        arg[3,1,0]=(-0.247378556309)*x[0]**o+(-0.631688468936)*x[0]+(-0.898130068425)*x[1]**o+(0.147210661251)*x[1]
        arg[3,1,1]=(0.979026126394)*x[0]**o+(-0.231003559285)*x[0]+(-0.884530953901)*x[1]**o+(-0.453907922804)*x[1]
        arg[4,0,0]=(-0.584910281652)*x[0]**o+(-0.808929690751)*x[0]+(0.661854511996)*x[1]**o+(0.262336033771)*x[1]
        arg[4,0,1]=(0.32006961919)*x[0]**o+(0.407637671852)*x[0]+(-0.36617527544)*x[1]**o+(-0.28786191714)*x[1]
        arg[4,1,0]=(0.718670448416)*x[0]**o+(-0.836956284868)*x[0]+(0.694744974874)*x[1]**o+(-0.176920245996)*x[1]
        arg[4,1,1]=(-0.892867770142)*x[0]**o+(-0.439240069099)*x[0]+(0.960956271989)*x[1]**o+(-0.559541152716)*x[1]
        arg[5,0,0]=(-0.159459484106)*x[0]**o+(0.497366075466)*x[0]+(0.776343693655)*x[1]**o+(0.326882397775)*x[1]
        arg[5,0,1]=(-0.769197598992)*x[0]**o+(-0.155931232816)*x[0]+(0.306889959757)*x[1]**o+(0.283798117232)*x[1]
        arg[5,1,0]=(0.329987736778)*x[0]**o+(-0.181917315586)*x[0]+(0.896508353752)*x[1]**o+(0.505276687831)*x[1]
        arg[5,1,1]=(-0.805264678584)*x[0]**o+(0.1541429191)*x[0]+(0.759370357364)*x[1]**o+(-0.858941373648)*x[1]
        ref[0,0,0,0]=o*(0.711259674136)*x_ref[0]**(o-1)+(0.597099394118)
        ref[0,0,0,1]=o*(0.656676641605)*x_ref[1]**(o-1)+(0.800065074626)
        ref[0,0,1,0]=o*(0.410575880869)*x_ref[0]**(o-1)+(0.0290305819624)
        ref[0,0,1,1]=o*(-0.746442034948)*x_ref[1]**(o-1)+(0.074826200729)
        ref[0,1,0,0]=o*(0.478073638149)*x_ref[0]**(o-1)+(0.856920711223)
        ref[0,1,0,1]=o*(0.517173340088)*x_ref[1]**(o-1)+(-0.677846282063)
        ref[0,1,1,0]=o*(0.285231489647)*x_ref[0]**(o-1)+(0.0756593965609)
        ref[0,1,1,1]=o*(-0.355221676478)*x_ref[1]**(o-1)+(-0.520564308221)
        ref[1,0,0,0]=o*(-0.368777104698)*x_ref[0]**(o-1)+(0.474996103371)
        ref[1,0,0,1]=o*(0.675654207136)*x_ref[1]**(o-1)+(-0.0324979517659)
        ref[1,0,1,0]=o*(0.498857835281)*x_ref[0]**(o-1)+(-0.73836840989)
        ref[1,0,1,1]=o*(0.362789682422)*x_ref[1]**(o-1)+(0.315425076703)
        ref[1,1,0,0]=o*(-0.325384820094)*x_ref[0]**(o-1)+(-0.895297521761)
        ref[1,1,0,1]=o*(0.563090918504)*x_ref[1]**(o-1)+(0.589834648825)
        ref[1,1,1,0]=o*(0.0378060061714)*x_ref[0]**(o-1)+(-0.429064920806)
        ref[1,1,1,1]=o*(-0.467324629077)*x_ref[1]**(o-1)+(0.938161415082)
        ref[2,0,0,0]=o*(-0.527827738902)*x_ref[0]**(o-1)+(-0.534425169092)
        ref[2,0,0,1]=o*(-0.185111533531)*x_ref[1]**(o-1)+(-0.810076656622)
        ref[2,0,1,0]=o*(-0.221712302033)*x_ref[0]**(o-1)+(0.883619587624)
        ref[2,0,1,1]=o*(-0.972912188123)*x_ref[1]**(o-1)+(0.0780565750992)
        ref[2,1,0,0]=o*(-0.752942706408)*x_ref[0]**(o-1)+(-0.153790659605)
        ref[2,1,0,1]=o*(-0.398384592556)*x_ref[1]**(o-1)+(0.657319049074)
        ref[2,1,1,0]=o*(0.428492876383)*x_ref[0]**(o-1)+(0.427090189076)
        ref[2,1,1,1]=o*(-0.017400284896)*x_ref[1]**(o-1)+(-0.730635641841)
        ref[3,0,0,0]=o*(-0.644602330168)*x_ref[0]**(o-1)+(0.706532342264)
        ref[3,0,0,1]=o*(-0.64532460698)*x_ref[1]**(o-1)+(0.850059843173)
        ref[3,0,1,0]=o*(-0.63066688687)*x_ref[0]**(o-1)+(0.578132047851)
        ref[3,0,1,1]=o*(0.413335880906)*x_ref[1]**(o-1)+(0.611233278281)
        ref[3,1,0,0]=o*(-0.247378556309)*x_ref[0]**(o-1)+(-0.631688468936)
        ref[3,1,0,1]=o*(-0.898130068425)*x_ref[1]**(o-1)+(0.147210661251)
        ref[3,1,1,0]=o*(0.979026126394)*x_ref[0]**(o-1)+(-0.231003559285)
        ref[3,1,1,1]=o*(-0.884530953901)*x_ref[1]**(o-1)+(-0.453907922804)
        ref[4,0,0,0]=o*(-0.584910281652)*x_ref[0]**(o-1)+(-0.808929690751)
        ref[4,0,0,1]=o*(0.661854511996)*x_ref[1]**(o-1)+(0.262336033771)
        ref[4,0,1,0]=o*(0.32006961919)*x_ref[0]**(o-1)+(0.407637671852)
        ref[4,0,1,1]=o*(-0.36617527544)*x_ref[1]**(o-1)+(-0.28786191714)
        ref[4,1,0,0]=o*(0.718670448416)*x_ref[0]**(o-1)+(-0.836956284868)
        ref[4,1,0,1]=o*(0.694744974874)*x_ref[1]**(o-1)+(-0.176920245996)
        ref[4,1,1,0]=o*(-0.892867770142)*x_ref[0]**(o-1)+(-0.439240069099)
        ref[4,1,1,1]=o*(0.960956271989)*x_ref[1]**(o-1)+(-0.559541152716)
        ref[5,0,0,0]=o*(-0.159459484106)*x_ref[0]**(o-1)+(0.497366075466)
        ref[5,0,0,1]=o*(0.776343693655)*x_ref[1]**(o-1)+(0.326882397775)
        ref[5,0,1,0]=o*(-0.769197598992)*x_ref[0]**(o-1)+(-0.155931232816)
        ref[5,0,1,1]=o*(0.306889959757)*x_ref[1]**(o-1)+(0.283798117232)
        ref[5,1,0,0]=o*(0.329987736778)*x_ref[0]**(o-1)+(-0.181917315586)
        ref[5,1,0,1]=o*(0.896508353752)*x_ref[1]**(o-1)+(0.505276687831)
        ref[5,1,1,0]=o*(-0.805264678584)*x_ref[0]**(o-1)+(0.1541429191)
        ref[5,1,1,1]=o*(0.759370357364)*x_ref[1]**(o-1)+(-0.858941373648)
      else:
        arg[0,0,0]=(0.168268053227)*x[0]**o+(-0.693455720059)*x[0]+(-0.929660815724)*x[1]**o+(0.986095168154)*x[1]+(0.736239785757)*x[2]**o+(-0.852231596742)*x[2]
        arg[0,0,1]=(-0.246755690586)*x[0]**o+(0.374568513964)*x[0]+(-0.964930789849)*x[1]**o+(0.911595670675)*x[1]+(-0.450825843592)*x[2]**o+(0.109918389584)*x[2]
        arg[0,1,0]=(-0.733872890764)*x[0]**o+(-0.673737586895)*x[0]+(0.948528017502)*x[1]**o+(0.00135698010622)*x[1]+(0.960702320728)*x[2]**o+(-0.0223793346126)*x[2]
        arg[0,1,1]=(0.117470384547)*x[0]**o+(0.390065805504)*x[0]+(0.116939897886)*x[1]**o+(0.201756470627)*x[1]+(0.649311190776)*x[2]**o+(-0.388330674246)*x[2]
        arg[1,0,0]=(0.848074231014)*x[0]**o+(-0.431699971504)*x[0]+(0.956477578677)*x[1]**o+(0.522633753414)*x[1]+(0.147733403055)*x[2]**o+(0.136817362225)*x[2]
        arg[1,0,1]=(0.482934543775)*x[0]**o+(-0.358181242578)*x[0]+(-0.013088278343)*x[1]**o+(0.002654195003)*x[1]+(0.960898421715)*x[2]**o+(0.23166323571)*x[2]
        arg[1,1,0]=(-0.60469206115)*x[0]**o+(-0.449073005118)*x[0]+(-0.0774626329204)*x[1]**o+(-0.135772353437)*x[1]+(-0.796309060217)*x[2]**o+(-0.0601212417342)*x[2]
        arg[1,1,1]=(0.41742915222)*x[0]**o+(-0.907531578792)*x[0]+(-0.998018453394)*x[1]**o+(0.38863222527)*x[1]+(-0.941909949653)*x[2]**o+(-0.962025400733)*x[2]
        arg[2,0,0]=(0.677803680186)*x[0]**o+(0.493757303449)*x[0]+(-0.700353467324)*x[1]**o+(0.713489396377)*x[1]+(-0.147276750564)*x[2]**o+(-0.666815984705)*x[2]
        arg[2,0,1]=(-0.196166170445)*x[0]**o+(-0.776357160477)*x[0]+(0.531757851468)*x[1]**o+(-0.0951663105473)*x[1]+(0.296695659682)*x[2]**o+(0.132047740242)*x[2]
        arg[2,1,0]=(0.89703218788)*x[0]**o+(0.446461958293)*x[0]+(0.849781349832)*x[1]**o+(0.495003438762)*x[1]+(0.200866318394)*x[2]**o+(-0.239158244355)*x[2]
        arg[2,1,1]=(0.699816693)*x[0]**o+(-0.498016262468)*x[0]+(0.467212425781)*x[1]**o+(0.507210685133)*x[1]+(0.872000089805)*x[2]**o+(-0.608415132584)*x[2]
        arg[3,0,0]=(-0.858779999521)*x[0]**o+(-0.934307938564)*x[0]+(0.998531969194)*x[1]**o+(-0.10694369616)*x[1]+(0.565515051636)*x[2]**o+(-0.340328286843)*x[2]
        arg[3,0,1]=(-0.734447160975)*x[0]**o+(0.592969337009)*x[0]+(0.769648460523)*x[1]**o+(0.939994123398)*x[1]+(-0.14168879251)*x[2]**o+(-0.151602759825)*x[2]
        arg[3,1,0]=(-0.204447368694)*x[0]**o+(-0.149512701586)*x[0]+(0.496643987821)*x[1]**o+(0.137154021271)*x[1]+(-0.207451973465)*x[2]**o+(0.976289933313)*x[2]
        arg[3,1,1]=(-0.498939396392)*x[0]**o+(-0.388761371521)*x[0]+(0.918876234523)*x[1]**o+(0.326334103437)*x[1]+(0.135801853065)*x[2]**o+(-0.513268577694)*x[2]
        arg[4,0,0]=(0.383369953978)*x[0]**o+(0.262190952188)*x[0]+(0.226913840907)*x[1]**o+(0.625767074123)*x[1]+(0.0736409418752)*x[2]**o+(-0.681643359069)*x[2]
        arg[4,0,1]=(-0.00395633017219)*x[0]**o+(0.103598757494)*x[0]+(0.245388867438)*x[1]**o+(0.0199139065324)*x[1]+(-0.85935209669)*x[2]**o+(0.780770429677)*x[2]
        arg[4,1,0]=(0.93284113054)*x[0]**o+(0.192867707515)*x[0]+(0.354258046046)*x[1]**o+(0.0244899514488)*x[1]+(-0.00723308587137)*x[2]**o+(0.724287748378)*x[2]
        arg[4,1,1]=(-0.701870704251)*x[0]**o+(0.29146856048)*x[0]+(-0.677358067691)*x[1]**o+(-0.055827889309)*x[1]+(0.96248585097)*x[2]**o+(0.100526700803)*x[2]
        arg[5,0,0]=(-0.582058132348)*x[0]**o+(0.5354038569)*x[0]+(-0.608415656952)*x[1]**o+(0.700352801694)*x[1]+(-0.705438310252)*x[2]**o+(0.969824711925)*x[2]
        arg[5,0,1]=(-0.573326444971)*x[0]**o+(0.00490482707944)*x[0]+(0.300312254171)*x[1]**o+(-0.620154697788)*x[1]+(-0.0703788860867)*x[2]**o+(-0.608567609941)*x[2]
        arg[5,1,0]=(0.346782282755)*x[0]**o+(-0.022146949553)*x[0]+(0.548359292626)*x[1]**o+(0.314315074761)*x[1]+(0.164808336191)*x[2]**o+(-0.983018613943)*x[2]
        arg[5,1,1]=(-0.357126808896)*x[0]**o+(0.547191129837)*x[0]+(-0.0128153418859)*x[1]**o+(-0.199507147279)*x[1]+(-0.637546467191)*x[2]**o+(0.65056997574)*x[2]
        ref[0,0,0,0]=o*(0.168268053227)*x_ref[0]**(o-1)+(-0.693455720059)
        ref[0,0,0,1]=o*(-0.929660815724)*x_ref[1]**(o-1)+(0.986095168154)
        ref[0,0,0,2]=o*(0.736239785757)*x_ref[2]**(o-1)+(-0.852231596742)
        ref[0,0,1,0]=o*(-0.246755690586)*x_ref[0]**(o-1)+(0.374568513964)
        ref[0,0,1,1]=o*(-0.964930789849)*x_ref[1]**(o-1)+(0.911595670675)
        ref[0,0,1,2]=o*(-0.450825843592)*x_ref[2]**(o-1)+(0.109918389584)
        ref[0,1,0,0]=o*(-0.733872890764)*x_ref[0]**(o-1)+(-0.673737586895)
        ref[0,1,0,1]=o*(0.948528017502)*x_ref[1]**(o-1)+(0.00135698010622)
        ref[0,1,0,2]=o*(0.960702320728)*x_ref[2]**(o-1)+(-0.0223793346126)
        ref[0,1,1,0]=o*(0.117470384547)*x_ref[0]**(o-1)+(0.390065805504)
        ref[0,1,1,1]=o*(0.116939897886)*x_ref[1]**(o-1)+(0.201756470627)
        ref[0,1,1,2]=o*(0.649311190776)*x_ref[2]**(o-1)+(-0.388330674246)
        ref[1,0,0,0]=o*(0.848074231014)*x_ref[0]**(o-1)+(-0.431699971504)
        ref[1,0,0,1]=o*(0.956477578677)*x_ref[1]**(o-1)+(0.522633753414)
        ref[1,0,0,2]=o*(0.147733403055)*x_ref[2]**(o-1)+(0.136817362225)
        ref[1,0,1,0]=o*(0.482934543775)*x_ref[0]**(o-1)+(-0.358181242578)
        ref[1,0,1,1]=o*(-0.013088278343)*x_ref[1]**(o-1)+(0.002654195003)
        ref[1,0,1,2]=o*(0.960898421715)*x_ref[2]**(o-1)+(0.23166323571)
        ref[1,1,0,0]=o*(-0.60469206115)*x_ref[0]**(o-1)+(-0.449073005118)
        ref[1,1,0,1]=o*(-0.0774626329204)*x_ref[1]**(o-1)+(-0.135772353437)
        ref[1,1,0,2]=o*(-0.796309060217)*x_ref[2]**(o-1)+(-0.0601212417342)
        ref[1,1,1,0]=o*(0.41742915222)*x_ref[0]**(o-1)+(-0.907531578792)
        ref[1,1,1,1]=o*(-0.998018453394)*x_ref[1]**(o-1)+(0.38863222527)
        ref[1,1,1,2]=o*(-0.941909949653)*x_ref[2]**(o-1)+(-0.962025400733)
        ref[2,0,0,0]=o*(0.677803680186)*x_ref[0]**(o-1)+(0.493757303449)
        ref[2,0,0,1]=o*(-0.700353467324)*x_ref[1]**(o-1)+(0.713489396377)
        ref[2,0,0,2]=o*(-0.147276750564)*x_ref[2]**(o-1)+(-0.666815984705)
        ref[2,0,1,0]=o*(-0.196166170445)*x_ref[0]**(o-1)+(-0.776357160477)
        ref[2,0,1,1]=o*(0.531757851468)*x_ref[1]**(o-1)+(-0.0951663105473)
        ref[2,0,1,2]=o*(0.296695659682)*x_ref[2]**(o-1)+(0.132047740242)
        ref[2,1,0,0]=o*(0.89703218788)*x_ref[0]**(o-1)+(0.446461958293)
        ref[2,1,0,1]=o*(0.849781349832)*x_ref[1]**(o-1)+(0.495003438762)
        ref[2,1,0,2]=o*(0.200866318394)*x_ref[2]**(o-1)+(-0.239158244355)
        ref[2,1,1,0]=o*(0.699816693)*x_ref[0]**(o-1)+(-0.498016262468)
        ref[2,1,1,1]=o*(0.467212425781)*x_ref[1]**(o-1)+(0.507210685133)
        ref[2,1,1,2]=o*(0.872000089805)*x_ref[2]**(o-1)+(-0.608415132584)
        ref[3,0,0,0]=o*(-0.858779999521)*x_ref[0]**(o-1)+(-0.934307938564)
        ref[3,0,0,1]=o*(0.998531969194)*x_ref[1]**(o-1)+(-0.10694369616)
        ref[3,0,0,2]=o*(0.565515051636)*x_ref[2]**(o-1)+(-0.340328286843)
        ref[3,0,1,0]=o*(-0.734447160975)*x_ref[0]**(o-1)+(0.592969337009)
        ref[3,0,1,1]=o*(0.769648460523)*x_ref[1]**(o-1)+(0.939994123398)
        ref[3,0,1,2]=o*(-0.14168879251)*x_ref[2]**(o-1)+(-0.151602759825)
        ref[3,1,0,0]=o*(-0.204447368694)*x_ref[0]**(o-1)+(-0.149512701586)
        ref[3,1,0,1]=o*(0.496643987821)*x_ref[1]**(o-1)+(0.137154021271)
        ref[3,1,0,2]=o*(-0.207451973465)*x_ref[2]**(o-1)+(0.976289933313)
        ref[3,1,1,0]=o*(-0.498939396392)*x_ref[0]**(o-1)+(-0.388761371521)
        ref[3,1,1,1]=o*(0.918876234523)*x_ref[1]**(o-1)+(0.326334103437)
        ref[3,1,1,2]=o*(0.135801853065)*x_ref[2]**(o-1)+(-0.513268577694)
        ref[4,0,0,0]=o*(0.383369953978)*x_ref[0]**(o-1)+(0.262190952188)
        ref[4,0,0,1]=o*(0.226913840907)*x_ref[1]**(o-1)+(0.625767074123)
        ref[4,0,0,2]=o*(0.0736409418752)*x_ref[2]**(o-1)+(-0.681643359069)
        ref[4,0,1,0]=o*(-0.00395633017219)*x_ref[0]**(o-1)+(0.103598757494)
        ref[4,0,1,1]=o*(0.245388867438)*x_ref[1]**(o-1)+(0.0199139065324)
        ref[4,0,1,2]=o*(-0.85935209669)*x_ref[2]**(o-1)+(0.780770429677)
        ref[4,1,0,0]=o*(0.93284113054)*x_ref[0]**(o-1)+(0.192867707515)
        ref[4,1,0,1]=o*(0.354258046046)*x_ref[1]**(o-1)+(0.0244899514488)
        ref[4,1,0,2]=o*(-0.00723308587137)*x_ref[2]**(o-1)+(0.724287748378)
        ref[4,1,1,0]=o*(-0.701870704251)*x_ref[0]**(o-1)+(0.29146856048)
        ref[4,1,1,1]=o*(-0.677358067691)*x_ref[1]**(o-1)+(-0.055827889309)
        ref[4,1,1,2]=o*(0.96248585097)*x_ref[2]**(o-1)+(0.100526700803)
        ref[5,0,0,0]=o*(-0.582058132348)*x_ref[0]**(o-1)+(0.5354038569)
        ref[5,0,0,1]=o*(-0.608415656952)*x_ref[1]**(o-1)+(0.700352801694)
        ref[5,0,0,2]=o*(-0.705438310252)*x_ref[2]**(o-1)+(0.969824711925)
        ref[5,0,1,0]=o*(-0.573326444971)*x_ref[0]**(o-1)+(0.00490482707944)
        ref[5,0,1,1]=o*(0.300312254171)*x_ref[1]**(o-1)+(-0.620154697788)
        ref[5,0,1,2]=o*(-0.0703788860867)*x_ref[2]**(o-1)+(-0.608567609941)
        ref[5,1,0,0]=o*(0.346782282755)*x_ref[0]**(o-1)+(-0.022146949553)
        ref[5,1,0,1]=o*(0.548359292626)*x_ref[1]**(o-1)+(0.314315074761)
        ref[5,1,0,2]=o*(0.164808336191)*x_ref[2]**(o-1)+(-0.983018613943)
        ref[5,1,1,0]=o*(-0.357126808896)*x_ref[0]**(o-1)+(0.547191129837)
        ref[5,1,1,1]=o*(-0.0128153418859)*x_ref[1]**(o-1)+(-0.199507147279)
        ref[5,1,1,2]=o*(-0.637546467191)*x_ref[2]**(o-1)+(0.65056997574)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.694260485776)*x[0]+(1.46246375886)*x[1]
        ref[0]=(0.694260485776)
        ref[1]=(1.46246375886)
      else:
        arg=(-1.10912081576)*x[0]+(1.19790363484)*x[1]+(0.555862453192)*x[2]
        ref[0]=(-1.10912081576)
        ref[1]=(1.19790363484)
        ref[2]=(0.555862453192)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.0963683715208)*x[0]+(-0.985421300426)*x[1]
        arg[1]=(0.50107481561)*x[0]+(0.296191075154)*x[1]
        ref[0,0]=(-0.0963683715208)
        ref[0,1]=(-0.985421300426)
        ref[1,0]=(0.50107481561)
        ref[1,1]=(0.296191075154)
      else:
        arg[0]=(0.790729361248)*x[0]+(0.3673431791)*x[1]+(0.031915702612)*x[2]
        arg[1]=(-0.218629391175)*x[0]+(-0.0619831001578)*x[1]+(-1.46460054046)*x[2]
        ref[0,0]=(0.790729361248)
        ref[0,1]=(0.3673431791)
        ref[0,2]=(0.031915702612)
        ref[1,0]=(-0.218629391175)
        ref[1,1]=(-0.0619831001578)
        ref[1,2]=(-1.46460054046)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.102266041928)*x[0]+(0.962985129019)*x[1]
        arg[0,1]=(1.22474578409)*x[0]+(-0.211640772922)*x[1]
        arg[0,2]=(-0.51306749275)*x[0]+(0.652502950912)*x[1]
        arg[0,3]=(1.89516926008)*x[0]+(0.317996805009)*x[1]
        arg[0,4]=(0.562531430083)*x[0]+(-0.194650166716)*x[1]
        arg[1,0]=(0.344353002434)*x[0]+(-0.830671914713)*x[1]
        arg[1,1]=(0.984363231775)*x[0]+(-0.0773767796506)*x[1]
        arg[1,2]=(-0.893978692143)*x[0]+(1.43362006873)*x[1]
        arg[1,3]=(0.603459290036)*x[0]+(-0.527471281229)*x[1]
        arg[1,4]=(-0.768879816113)*x[0]+(0.670739267079)*x[1]
        arg[2,0]=(0.466692398942)*x[0]+(-0.315362658981)*x[1]
        arg[2,1]=(1.30065290708)*x[0]+(-0.202369755569)*x[1]
        arg[2,2]=(-0.114571246331)*x[0]+(1.30250121342)*x[1]
        arg[2,3]=(-0.243608494456)*x[0]+(-0.103398411275)*x[1]
        arg[2,4]=(-0.262568543724)*x[0]+(-0.23979204654)*x[1]
        arg[3,0]=(0.0334727960327)*x[0]+(-0.24547366383)*x[1]
        arg[3,1]=(0.829837477348)*x[0]+(-0.416646135702)*x[1]
        arg[3,2]=(1.43081322579)*x[0]+(0.574724874607)*x[1]
        arg[3,3]=(1.16303985686)*x[0]+(-0.915348714324)*x[1]
        arg[3,4]=(0.773764690591)*x[0]+(0.28049189996)*x[1]
        ref[0,0,0]=(0.102266041928)
        ref[0,0,1]=(0.962985129019)
        ref[0,1,0]=(1.22474578409)
        ref[0,1,1]=(-0.211640772922)
        ref[0,2,0]=(-0.51306749275)
        ref[0,2,1]=(0.652502950912)
        ref[0,3,0]=(1.89516926008)
        ref[0,3,1]=(0.317996805009)
        ref[0,4,0]=(0.562531430083)
        ref[0,4,1]=(-0.194650166716)
        ref[1,0,0]=(0.344353002434)
        ref[1,0,1]=(-0.830671914713)
        ref[1,1,0]=(0.984363231775)
        ref[1,1,1]=(-0.0773767796506)
        ref[1,2,0]=(-0.893978692143)
        ref[1,2,1]=(1.43362006873)
        ref[1,3,0]=(0.603459290036)
        ref[1,3,1]=(-0.527471281229)
        ref[1,4,0]=(-0.768879816113)
        ref[1,4,1]=(0.670739267079)
        ref[2,0,0]=(0.466692398942)
        ref[2,0,1]=(-0.315362658981)
        ref[2,1,0]=(1.30065290708)
        ref[2,1,1]=(-0.202369755569)
        ref[2,2,0]=(-0.114571246331)
        ref[2,2,1]=(1.30250121342)
        ref[2,3,0]=(-0.243608494456)
        ref[2,3,1]=(-0.103398411275)
        ref[2,4,0]=(-0.262568543724)
        ref[2,4,1]=(-0.23979204654)
        ref[3,0,0]=(0.0334727960327)
        ref[3,0,1]=(-0.24547366383)
        ref[3,1,0]=(0.829837477348)
        ref[3,1,1]=(-0.416646135702)
        ref[3,2,0]=(1.43081322579)
        ref[3,2,1]=(0.574724874607)
        ref[3,3,0]=(1.16303985686)
        ref[3,3,1]=(-0.915348714324)
        ref[3,4,0]=(0.773764690591)
        ref[3,4,1]=(0.28049189996)
      else:
        arg[0,0]=(-0.998409457516)*x[0]+(0.374200474533)*x[1]+(-0.397603453862)*x[2]
        arg[0,1]=(-1.48600938216)*x[0]+(1.46358462866)*x[1]+(-0.393381462291)*x[2]
        arg[0,2]=(1.08201727329)*x[0]+(0.065185376372)*x[1]+(-1.32483264357)*x[2]
        arg[0,3]=(-0.518441162)*x[0]+(0.364373161676)*x[1]+(-1.25875574919)*x[2]
        arg[0,4]=(-0.836980279478)*x[0]+(1.21655914685)*x[1]+(0.1586084291)*x[2]
        arg[1,0]=(0.760774732939)*x[0]+(0.515414129191)*x[1]+(-1.50192024303)*x[2]
        arg[1,1]=(-0.653360330499)*x[0]+(1.71417096347)*x[1]+(-0.169129173291)*x[2]
        arg[1,2]=(1.07221333767)*x[0]+(-1.07638348405)*x[1]+(1.13393461135)*x[2]
        arg[1,3]=(0.847567825055)*x[0]+(-0.0298335468796)*x[1]+(0.45122717832)*x[2]
        arg[1,4]=(-0.108027217492)*x[0]+(-1.12654142913)*x[1]+(-0.781718943065)*x[2]
        arg[2,0]=(0.408863322164)*x[0]+(0.100914804776)*x[1]+(-0.471567638444)*x[2]
        arg[2,1]=(-1.42194256762)*x[0]+(-0.348217894465)*x[1]+(-0.830561377367)*x[2]
        arg[2,2]=(1.36876795155)*x[0]+(0.125252389466)*x[1]+(0.00641454191902)*x[2]
        arg[2,3]=(0.135243193372)*x[0]+(-1.04394510102)*x[1]+(0.0223029076973)*x[2]
        arg[2,4]=(-0.969785158394)*x[0]+(-0.48178611106)*x[1]+(-0.0556576599006)*x[2]
        arg[3,0]=(1.39484073703)*x[0]+(-1.0132109886)*x[1]+(0.0205156989927)*x[2]
        arg[3,1]=(-0.92783313898)*x[0]+(-1.27904129731)*x[1]+(-0.817768402689)*x[2]
        arg[3,2]=(1.21718392423)*x[0]+(-0.984091647405)*x[1]+(0.458009888947)*x[2]
        arg[3,3]=(-0.812990754761)*x[0]+(-0.104941245044)*x[1]+(-0.996531826869)*x[2]
        arg[3,4]=(-0.28581988131)*x[0]+(-1.24321114539)*x[1]+(0.408464222517)*x[2]
        ref[0,0,0]=(-0.998409457516)
        ref[0,0,1]=(0.374200474533)
        ref[0,0,2]=(-0.397603453862)
        ref[0,1,0]=(-1.48600938216)
        ref[0,1,1]=(1.46358462866)
        ref[0,1,2]=(-0.393381462291)
        ref[0,2,0]=(1.08201727329)
        ref[0,2,1]=(0.065185376372)
        ref[0,2,2]=(-1.32483264357)
        ref[0,3,0]=(-0.518441162)
        ref[0,3,1]=(0.364373161676)
        ref[0,3,2]=(-1.25875574919)
        ref[0,4,0]=(-0.836980279478)
        ref[0,4,1]=(1.21655914685)
        ref[0,4,2]=(0.1586084291)
        ref[1,0,0]=(0.760774732939)
        ref[1,0,1]=(0.515414129191)
        ref[1,0,2]=(-1.50192024303)
        ref[1,1,0]=(-0.653360330499)
        ref[1,1,1]=(1.71417096347)
        ref[1,1,2]=(-0.169129173291)
        ref[1,2,0]=(1.07221333767)
        ref[1,2,1]=(-1.07638348405)
        ref[1,2,2]=(1.13393461135)
        ref[1,3,0]=(0.847567825055)
        ref[1,3,1]=(-0.0298335468796)
        ref[1,3,2]=(0.45122717832)
        ref[1,4,0]=(-0.108027217492)
        ref[1,4,1]=(-1.12654142913)
        ref[1,4,2]=(-0.781718943065)
        ref[2,0,0]=(0.408863322164)
        ref[2,0,1]=(0.100914804776)
        ref[2,0,2]=(-0.471567638444)
        ref[2,1,0]=(-1.42194256762)
        ref[2,1,1]=(-0.348217894465)
        ref[2,1,2]=(-0.830561377367)
        ref[2,2,0]=(1.36876795155)
        ref[2,2,1]=(0.125252389466)
        ref[2,2,2]=(0.00641454191902)
        ref[2,3,0]=(0.135243193372)
        ref[2,3,1]=(-1.04394510102)
        ref[2,3,2]=(0.0223029076973)
        ref[2,4,0]=(-0.969785158394)
        ref[2,4,1]=(-0.48178611106)
        ref[2,4,2]=(-0.0556576599006)
        ref[3,0,0]=(1.39484073703)
        ref[3,0,1]=(-1.0132109886)
        ref[3,0,2]=(0.0205156989927)
        ref[3,1,0]=(-0.92783313898)
        ref[3,1,1]=(-1.27904129731)
        ref[3,1,2]=(-0.817768402689)
        ref[3,2,0]=(1.21718392423)
        ref[3,2,1]=(-0.984091647405)
        ref[3,2,2]=(0.458009888947)
        ref[3,3,0]=(-0.812990754761)
        ref[3,3,1]=(-0.104941245044)
        ref[3,3,2]=(-0.996531826869)
        ref[3,4,0]=(-0.28581988131)
        ref[3,4,1]=(-1.24321114539)
        ref[3,4,2]=(0.408464222517)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.39712204981)*x[0]+(-1.36315938849)*x[1]
        arg[0,0,1]=(0.654450268666)*x[0]+(-0.198258834351)*x[1]
        arg[0,1,0]=(0.21357757124)*x[0]+(0.193878132191)*x[1]
        arg[0,1,1]=(0.0762258436603)*x[0]+(-0.33284111684)*x[1]
        arg[1,0,0]=(-0.264232939297)*x[0]+(-0.726336076701)*x[1]
        arg[1,0,1]=(-0.179928817534)*x[0]+(-0.723247805844)*x[1]
        arg[1,1,0]=(0.923289681593)*x[0]+(0.986763135255)*x[1]
        arg[1,1,1]=(0.065387254507)*x[0]+(-0.710965008545)*x[1]
        arg[2,0,0]=(0.759853961917)*x[0]+(0.24486573839)*x[1]
        arg[2,0,1]=(-0.318336679378)*x[0]+(0.39328274922)*x[1]
        arg[2,1,0]=(-1.562818948)*x[0]+(-0.891340974738)*x[1]
        arg[2,1,1]=(0.0508006116584)*x[0]+(1.17654134987)*x[1]
        arg[3,0,0]=(-0.155883472458)*x[0]+(0.248809866836)*x[1]
        arg[3,0,1]=(0.287794710222)*x[0]+(-1.72525392296)*x[1]
        arg[3,1,0]=(0.00665202042795)*x[0]+(-0.0996395796968)*x[1]
        arg[3,1,1]=(-0.19809021298)*x[0]+(0.487549616652)*x[1]
        arg[4,0,0]=(0.764163773078)*x[0]+(-0.898694486894)*x[1]
        arg[4,0,1]=(-0.213042355119)*x[0]+(0.708925705544)*x[1]
        arg[4,1,0]=(1.55505564031)*x[0]+(-0.545746649289)*x[1]
        arg[4,1,1]=(-1.31623592165)*x[0]+(0.948772953354)*x[1]
        arg[5,0,0]=(0.221103671977)*x[0]+(0.866067107464)*x[1]
        arg[5,0,1]=(-0.0979530612779)*x[0]+(-0.371623102578)*x[1]
        arg[5,1,0]=(-0.348963169972)*x[0]+(-0.692055450872)*x[1]
        arg[5,1,1]=(-0.584053243695)*x[0]+(-0.487768032842)*x[1]
        ref[0,0,0,0]=(0.39712204981)
        ref[0,0,0,1]=(-1.36315938849)
        ref[0,0,1,0]=(0.654450268666)
        ref[0,0,1,1]=(-0.198258834351)
        ref[0,1,0,0]=(0.21357757124)
        ref[0,1,0,1]=(0.193878132191)
        ref[0,1,1,0]=(0.0762258436603)
        ref[0,1,1,1]=(-0.33284111684)
        ref[1,0,0,0]=(-0.264232939297)
        ref[1,0,0,1]=(-0.726336076701)
        ref[1,0,1,0]=(-0.179928817534)
        ref[1,0,1,1]=(-0.723247805844)
        ref[1,1,0,0]=(0.923289681593)
        ref[1,1,0,1]=(0.986763135255)
        ref[1,1,1,0]=(0.065387254507)
        ref[1,1,1,1]=(-0.710965008545)
        ref[2,0,0,0]=(0.759853961917)
        ref[2,0,0,1]=(0.24486573839)
        ref[2,0,1,0]=(-0.318336679378)
        ref[2,0,1,1]=(0.39328274922)
        ref[2,1,0,0]=(-1.562818948)
        ref[2,1,0,1]=(-0.891340974738)
        ref[2,1,1,0]=(0.0508006116584)
        ref[2,1,1,1]=(1.17654134987)
        ref[3,0,0,0]=(-0.155883472458)
        ref[3,0,0,1]=(0.248809866836)
        ref[3,0,1,0]=(0.287794710222)
        ref[3,0,1,1]=(-1.72525392296)
        ref[3,1,0,0]=(0.00665202042795)
        ref[3,1,0,1]=(-0.0996395796968)
        ref[3,1,1,0]=(-0.19809021298)
        ref[3,1,1,1]=(0.487549616652)
        ref[4,0,0,0]=(0.764163773078)
        ref[4,0,0,1]=(-0.898694486894)
        ref[4,0,1,0]=(-0.213042355119)
        ref[4,0,1,1]=(0.708925705544)
        ref[4,1,0,0]=(1.55505564031)
        ref[4,1,0,1]=(-0.545746649289)
        ref[4,1,1,0]=(-1.31623592165)
        ref[4,1,1,1]=(0.948772953354)
        ref[5,0,0,0]=(0.221103671977)
        ref[5,0,0,1]=(0.866067107464)
        ref[5,0,1,0]=(-0.0979530612779)
        ref[5,0,1,1]=(-0.371623102578)
        ref[5,1,0,0]=(-0.348963169972)
        ref[5,1,0,1]=(-0.692055450872)
        ref[5,1,1,0]=(-0.584053243695)
        ref[5,1,1,1]=(-0.487768032842)
      else:
        arg[0,0,0]=(-1.45023987007)*x[0]+(0.138479134523)*x[1]+(1.46404607112)*x[2]
        arg[0,0,1]=(-0.647908832568)*x[0]+(-0.503758830859)*x[1]+(1.23196203665)*x[2]
        arg[0,1,0]=(-0.0108286161243)*x[0]+(0.565615928686)*x[1]+(-0.357789821636)*x[2]
        arg[0,1,1]=(0.648224572901)*x[0]+(0.775857442933)*x[1]+(0.960991462312)*x[2]
        arg[1,0,0]=(0.340590670624)*x[0]+(0.0357759349963)*x[1]+(0.358139256621)*x[2]
        arg[1,0,1]=(0.569789735109)*x[0]+(1.40678949618)*x[1]+(-0.965577726422)*x[2]
        arg[1,1,0]=(-0.758717082212)*x[0]+(0.612968081377)*x[1]+(-0.417988159948)*x[2]
        arg[1,1,1]=(0.969867032567)*x[0]+(-0.0700234825891)*x[1]+(1.53879850558)*x[2]
        arg[2,0,0]=(-1.02952648125)*x[0]+(0.108642437375)*x[1]+(0.251073184196)*x[2]
        arg[2,0,1]=(0.794766888909)*x[0]+(1.50591346365)*x[1]+(-0.36870578891)*x[2]
        arg[2,1,0]=(-0.534487055503)*x[0]+(-1.09061679233)*x[1]+(0.799342633685)*x[2]
        arg[2,1,1]=(0.0435889213805)*x[0]+(-0.209888628166)*x[1]+(1.34633215915)*x[2]
        arg[3,0,0]=(-0.0344682106376)*x[0]+(-0.92386569455)*x[1]+(-0.551250384619)*x[2]
        arg[3,0,1]=(-0.168599962689)*x[0]+(0.204634495314)*x[1]+(-0.65426520271)*x[2]
        arg[3,1,0]=(-1.61759210628)*x[0]+(-0.5922211067)*x[1]+(-0.139092780207)*x[2]
        arg[3,1,1]=(-0.13862300062)*x[0]+(-1.45951050222)*x[1]+(0.300904423579)*x[2]
        arg[4,0,0]=(0.629590770742)*x[0]+(-0.217150209985)*x[1]+(-0.140384227583)*x[2]
        arg[4,0,1]=(-0.0461635416119)*x[0]+(0.525692990183)*x[1]+(-1.89728167146)*x[2]
        arg[4,1,0]=(-0.130297059668)*x[0]+(-0.138816250007)*x[1]+(1.26805054232)*x[2]
        arg[4,1,1]=(0.742488751566)*x[0]+(0.659272799907)*x[1]+(-0.370877917078)*x[2]
        arg[5,0,0]=(-0.521350561164)*x[0]+(-0.213669151076)*x[1]+(0.00949381597317)*x[2]
        arg[5,0,1]=(-0.951547526648)*x[0]+(-0.388340895795)*x[1]+(0.741997603334)*x[2]
        arg[5,1,0]=(-0.332922940005)*x[0]+(1.74852940259)*x[1]+(-0.484189519128)*x[2]
        arg[5,1,1]=(0.418660109098)*x[0]+(-0.690728767024)*x[1]+(-0.675947375912)*x[2]
        ref[0,0,0,0]=(-1.45023987007)
        ref[0,0,0,1]=(0.138479134523)
        ref[0,0,0,2]=(1.46404607112)
        ref[0,0,1,0]=(-0.647908832568)
        ref[0,0,1,1]=(-0.503758830859)
        ref[0,0,1,2]=(1.23196203665)
        ref[0,1,0,0]=(-0.0108286161243)
        ref[0,1,0,1]=(0.565615928686)
        ref[0,1,0,2]=(-0.357789821636)
        ref[0,1,1,0]=(0.648224572901)
        ref[0,1,1,1]=(0.775857442933)
        ref[0,1,1,2]=(0.960991462312)
        ref[1,0,0,0]=(0.340590670624)
        ref[1,0,0,1]=(0.0357759349963)
        ref[1,0,0,2]=(0.358139256621)
        ref[1,0,1,0]=(0.569789735109)
        ref[1,0,1,1]=(1.40678949618)
        ref[1,0,1,2]=(-0.965577726422)
        ref[1,1,0,0]=(-0.758717082212)
        ref[1,1,0,1]=(0.612968081377)
        ref[1,1,0,2]=(-0.417988159948)
        ref[1,1,1,0]=(0.969867032567)
        ref[1,1,1,1]=(-0.0700234825891)
        ref[1,1,1,2]=(1.53879850558)
        ref[2,0,0,0]=(-1.02952648125)
        ref[2,0,0,1]=(0.108642437375)
        ref[2,0,0,2]=(0.251073184196)
        ref[2,0,1,0]=(0.794766888909)
        ref[2,0,1,1]=(1.50591346365)
        ref[2,0,1,2]=(-0.36870578891)
        ref[2,1,0,0]=(-0.534487055503)
        ref[2,1,0,1]=(-1.09061679233)
        ref[2,1,0,2]=(0.799342633685)
        ref[2,1,1,0]=(0.0435889213805)
        ref[2,1,1,1]=(-0.209888628166)
        ref[2,1,1,2]=(1.34633215915)
        ref[3,0,0,0]=(-0.0344682106376)
        ref[3,0,0,1]=(-0.92386569455)
        ref[3,0,0,2]=(-0.551250384619)
        ref[3,0,1,0]=(-0.168599962689)
        ref[3,0,1,1]=(0.204634495314)
        ref[3,0,1,2]=(-0.65426520271)
        ref[3,1,0,0]=(-1.61759210628)
        ref[3,1,0,1]=(-0.5922211067)
        ref[3,1,0,2]=(-0.139092780207)
        ref[3,1,1,0]=(-0.13862300062)
        ref[3,1,1,1]=(-1.45951050222)
        ref[3,1,1,2]=(0.300904423579)
        ref[4,0,0,0]=(0.629590770742)
        ref[4,0,0,1]=(-0.217150209985)
        ref[4,0,0,2]=(-0.140384227583)
        ref[4,0,1,0]=(-0.0461635416119)
        ref[4,0,1,1]=(0.525692990183)
        ref[4,0,1,2]=(-1.89728167146)
        ref[4,1,0,0]=(-0.130297059668)
        ref[4,1,0,1]=(-0.138816250007)
        ref[4,1,0,2]=(1.26805054232)
        ref[4,1,1,0]=(0.742488751566)
        ref[4,1,1,1]=(0.659272799907)
        ref[4,1,1,2]=(-0.370877917078)
        ref[5,0,0,0]=(-0.521350561164)
        ref[5,0,0,1]=(-0.213669151076)
        ref[5,0,0,2]=(0.00949381597317)
        ref[5,0,1,0]=(-0.951547526648)
        ref[5,0,1,1]=(-0.388340895795)
        ref[5,0,1,2]=(0.741997603334)
        ref[5,1,0,0]=(-0.332922940005)
        ref[5,1,0,1]=(1.74852940259)
        ref[5,1,0,2]=(-0.484189519128)
        ref[5,1,1,0]=(0.418660109098)
        ref[5,1,1,1]=(-0.690728767024)
        ref[5,1,1,2]=(-0.675947375912)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.694260485776)*x[0]+(1.46246375886)*x[1]
        ref[0]=(0.694260485776)
        ref[1]=(1.46246375886)
      else:
        arg=(-1.10912081576)*x[0]+(1.19790363484)*x[1]+(0.555862453192)*x[2]
        ref[0]=(-1.10912081576)
        ref[1]=(1.19790363484)
        ref[2]=(0.555862453192)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.0963683715208)*x[0]+(-0.985421300426)*x[1]
        arg[1]=(0.50107481561)*x[0]+(0.296191075154)*x[1]
        ref[0,0]=(-0.0963683715208)
        ref[0,1]=(-0.985421300426)
        ref[1,0]=(0.50107481561)
        ref[1,1]=(0.296191075154)
      else:
        arg[0]=(0.790729361248)*x[0]+(0.3673431791)*x[1]+(0.031915702612)*x[2]
        arg[1]=(-0.218629391175)*x[0]+(-0.0619831001578)*x[1]+(-1.46460054046)*x[2]
        ref[0,0]=(0.790729361248)
        ref[0,1]=(0.3673431791)
        ref[0,2]=(0.031915702612)
        ref[1,0]=(-0.218629391175)
        ref[1,1]=(-0.0619831001578)
        ref[1,2]=(-1.46460054046)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.102266041928)*x[0]+(0.962985129019)*x[1]
        arg[0,1]=(1.22474578409)*x[0]+(-0.211640772922)*x[1]
        arg[0,2]=(-0.51306749275)*x[0]+(0.652502950912)*x[1]
        arg[0,3]=(1.89516926008)*x[0]+(0.317996805009)*x[1]
        arg[0,4]=(0.562531430083)*x[0]+(-0.194650166716)*x[1]
        arg[1,0]=(0.344353002434)*x[0]+(-0.830671914713)*x[1]
        arg[1,1]=(0.984363231775)*x[0]+(-0.0773767796506)*x[1]
        arg[1,2]=(-0.893978692143)*x[0]+(1.43362006873)*x[1]
        arg[1,3]=(0.603459290036)*x[0]+(-0.527471281229)*x[1]
        arg[1,4]=(-0.768879816113)*x[0]+(0.670739267079)*x[1]
        arg[2,0]=(0.466692398942)*x[0]+(-0.315362658981)*x[1]
        arg[2,1]=(1.30065290708)*x[0]+(-0.202369755569)*x[1]
        arg[2,2]=(-0.114571246331)*x[0]+(1.30250121342)*x[1]
        arg[2,3]=(-0.243608494456)*x[0]+(-0.103398411275)*x[1]
        arg[2,4]=(-0.262568543724)*x[0]+(-0.23979204654)*x[1]
        arg[3,0]=(0.0334727960327)*x[0]+(-0.24547366383)*x[1]
        arg[3,1]=(0.829837477348)*x[0]+(-0.416646135702)*x[1]
        arg[3,2]=(1.43081322579)*x[0]+(0.574724874607)*x[1]
        arg[3,3]=(1.16303985686)*x[0]+(-0.915348714324)*x[1]
        arg[3,4]=(0.773764690591)*x[0]+(0.28049189996)*x[1]
        ref[0,0,0]=(0.102266041928)
        ref[0,0,1]=(0.962985129019)
        ref[0,1,0]=(1.22474578409)
        ref[0,1,1]=(-0.211640772922)
        ref[0,2,0]=(-0.51306749275)
        ref[0,2,1]=(0.652502950912)
        ref[0,3,0]=(1.89516926008)
        ref[0,3,1]=(0.317996805009)
        ref[0,4,0]=(0.562531430083)
        ref[0,4,1]=(-0.194650166716)
        ref[1,0,0]=(0.344353002434)
        ref[1,0,1]=(-0.830671914713)
        ref[1,1,0]=(0.984363231775)
        ref[1,1,1]=(-0.0773767796506)
        ref[1,2,0]=(-0.893978692143)
        ref[1,2,1]=(1.43362006873)
        ref[1,3,0]=(0.603459290036)
        ref[1,3,1]=(-0.527471281229)
        ref[1,4,0]=(-0.768879816113)
        ref[1,4,1]=(0.670739267079)
        ref[2,0,0]=(0.466692398942)
        ref[2,0,1]=(-0.315362658981)
        ref[2,1,0]=(1.30065290708)
        ref[2,1,1]=(-0.202369755569)
        ref[2,2,0]=(-0.114571246331)
        ref[2,2,1]=(1.30250121342)
        ref[2,3,0]=(-0.243608494456)
        ref[2,3,1]=(-0.103398411275)
        ref[2,4,0]=(-0.262568543724)
        ref[2,4,1]=(-0.23979204654)
        ref[3,0,0]=(0.0334727960327)
        ref[3,0,1]=(-0.24547366383)
        ref[3,1,0]=(0.829837477348)
        ref[3,1,1]=(-0.416646135702)
        ref[3,2,0]=(1.43081322579)
        ref[3,2,1]=(0.574724874607)
        ref[3,3,0]=(1.16303985686)
        ref[3,3,1]=(-0.915348714324)
        ref[3,4,0]=(0.773764690591)
        ref[3,4,1]=(0.28049189996)
      else:
        arg[0,0]=(-0.998409457516)*x[0]+(0.374200474533)*x[1]+(-0.397603453862)*x[2]
        arg[0,1]=(-1.48600938216)*x[0]+(1.46358462866)*x[1]+(-0.393381462291)*x[2]
        arg[0,2]=(1.08201727329)*x[0]+(0.065185376372)*x[1]+(-1.32483264357)*x[2]
        arg[0,3]=(-0.518441162)*x[0]+(0.364373161676)*x[1]+(-1.25875574919)*x[2]
        arg[0,4]=(-0.836980279478)*x[0]+(1.21655914685)*x[1]+(0.1586084291)*x[2]
        arg[1,0]=(0.760774732939)*x[0]+(0.515414129191)*x[1]+(-1.50192024303)*x[2]
        arg[1,1]=(-0.653360330499)*x[0]+(1.71417096347)*x[1]+(-0.169129173291)*x[2]
        arg[1,2]=(1.07221333767)*x[0]+(-1.07638348405)*x[1]+(1.13393461135)*x[2]
        arg[1,3]=(0.847567825055)*x[0]+(-0.0298335468796)*x[1]+(0.45122717832)*x[2]
        arg[1,4]=(-0.108027217492)*x[0]+(-1.12654142913)*x[1]+(-0.781718943065)*x[2]
        arg[2,0]=(0.408863322164)*x[0]+(0.100914804776)*x[1]+(-0.471567638444)*x[2]
        arg[2,1]=(-1.42194256762)*x[0]+(-0.348217894465)*x[1]+(-0.830561377367)*x[2]
        arg[2,2]=(1.36876795155)*x[0]+(0.125252389466)*x[1]+(0.00641454191902)*x[2]
        arg[2,3]=(0.135243193372)*x[0]+(-1.04394510102)*x[1]+(0.0223029076973)*x[2]
        arg[2,4]=(-0.969785158394)*x[0]+(-0.48178611106)*x[1]+(-0.0556576599006)*x[2]
        arg[3,0]=(1.39484073703)*x[0]+(-1.0132109886)*x[1]+(0.0205156989927)*x[2]
        arg[3,1]=(-0.92783313898)*x[0]+(-1.27904129731)*x[1]+(-0.817768402689)*x[2]
        arg[3,2]=(1.21718392423)*x[0]+(-0.984091647405)*x[1]+(0.458009888947)*x[2]
        arg[3,3]=(-0.812990754761)*x[0]+(-0.104941245044)*x[1]+(-0.996531826869)*x[2]
        arg[3,4]=(-0.28581988131)*x[0]+(-1.24321114539)*x[1]+(0.408464222517)*x[2]
        ref[0,0,0]=(-0.998409457516)
        ref[0,0,1]=(0.374200474533)
        ref[0,0,2]=(-0.397603453862)
        ref[0,1,0]=(-1.48600938216)
        ref[0,1,1]=(1.46358462866)
        ref[0,1,2]=(-0.393381462291)
        ref[0,2,0]=(1.08201727329)
        ref[0,2,1]=(0.065185376372)
        ref[0,2,2]=(-1.32483264357)
        ref[0,3,0]=(-0.518441162)
        ref[0,3,1]=(0.364373161676)
        ref[0,3,2]=(-1.25875574919)
        ref[0,4,0]=(-0.836980279478)
        ref[0,4,1]=(1.21655914685)
        ref[0,4,2]=(0.1586084291)
        ref[1,0,0]=(0.760774732939)
        ref[1,0,1]=(0.515414129191)
        ref[1,0,2]=(-1.50192024303)
        ref[1,1,0]=(-0.653360330499)
        ref[1,1,1]=(1.71417096347)
        ref[1,1,2]=(-0.169129173291)
        ref[1,2,0]=(1.07221333767)
        ref[1,2,1]=(-1.07638348405)
        ref[1,2,2]=(1.13393461135)
        ref[1,3,0]=(0.847567825055)
        ref[1,3,1]=(-0.0298335468796)
        ref[1,3,2]=(0.45122717832)
        ref[1,4,0]=(-0.108027217492)
        ref[1,4,1]=(-1.12654142913)
        ref[1,4,2]=(-0.781718943065)
        ref[2,0,0]=(0.408863322164)
        ref[2,0,1]=(0.100914804776)
        ref[2,0,2]=(-0.471567638444)
        ref[2,1,0]=(-1.42194256762)
        ref[2,1,1]=(-0.348217894465)
        ref[2,1,2]=(-0.830561377367)
        ref[2,2,0]=(1.36876795155)
        ref[2,2,1]=(0.125252389466)
        ref[2,2,2]=(0.00641454191902)
        ref[2,3,0]=(0.135243193372)
        ref[2,3,1]=(-1.04394510102)
        ref[2,3,2]=(0.0223029076973)
        ref[2,4,0]=(-0.969785158394)
        ref[2,4,1]=(-0.48178611106)
        ref[2,4,2]=(-0.0556576599006)
        ref[3,0,0]=(1.39484073703)
        ref[3,0,1]=(-1.0132109886)
        ref[3,0,2]=(0.0205156989927)
        ref[3,1,0]=(-0.92783313898)
        ref[3,1,1]=(-1.27904129731)
        ref[3,1,2]=(-0.817768402689)
        ref[3,2,0]=(1.21718392423)
        ref[3,2,1]=(-0.984091647405)
        ref[3,2,2]=(0.458009888947)
        ref[3,3,0]=(-0.812990754761)
        ref[3,3,1]=(-0.104941245044)
        ref[3,3,2]=(-0.996531826869)
        ref[3,4,0]=(-0.28581988131)
        ref[3,4,1]=(-1.24321114539)
        ref[3,4,2]=(0.408464222517)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.39712204981)*x[0]+(-1.36315938849)*x[1]
        arg[0,0,1]=(0.654450268666)*x[0]+(-0.198258834351)*x[1]
        arg[0,1,0]=(0.21357757124)*x[0]+(0.193878132191)*x[1]
        arg[0,1,1]=(0.0762258436603)*x[0]+(-0.33284111684)*x[1]
        arg[1,0,0]=(-0.264232939297)*x[0]+(-0.726336076701)*x[1]
        arg[1,0,1]=(-0.179928817534)*x[0]+(-0.723247805844)*x[1]
        arg[1,1,0]=(0.923289681593)*x[0]+(0.986763135255)*x[1]
        arg[1,1,1]=(0.065387254507)*x[0]+(-0.710965008545)*x[1]
        arg[2,0,0]=(0.759853961917)*x[0]+(0.24486573839)*x[1]
        arg[2,0,1]=(-0.318336679378)*x[0]+(0.39328274922)*x[1]
        arg[2,1,0]=(-1.562818948)*x[0]+(-0.891340974738)*x[1]
        arg[2,1,1]=(0.0508006116584)*x[0]+(1.17654134987)*x[1]
        arg[3,0,0]=(-0.155883472458)*x[0]+(0.248809866836)*x[1]
        arg[3,0,1]=(0.287794710222)*x[0]+(-1.72525392296)*x[1]
        arg[3,1,0]=(0.00665202042795)*x[0]+(-0.0996395796968)*x[1]
        arg[3,1,1]=(-0.19809021298)*x[0]+(0.487549616652)*x[1]
        arg[4,0,0]=(0.764163773078)*x[0]+(-0.898694486894)*x[1]
        arg[4,0,1]=(-0.213042355119)*x[0]+(0.708925705544)*x[1]
        arg[4,1,0]=(1.55505564031)*x[0]+(-0.545746649289)*x[1]
        arg[4,1,1]=(-1.31623592165)*x[0]+(0.948772953354)*x[1]
        arg[5,0,0]=(0.221103671977)*x[0]+(0.866067107464)*x[1]
        arg[5,0,1]=(-0.0979530612779)*x[0]+(-0.371623102578)*x[1]
        arg[5,1,0]=(-0.348963169972)*x[0]+(-0.692055450872)*x[1]
        arg[5,1,1]=(-0.584053243695)*x[0]+(-0.487768032842)*x[1]
        ref[0,0,0,0]=(0.39712204981)
        ref[0,0,0,1]=(-1.36315938849)
        ref[0,0,1,0]=(0.654450268666)
        ref[0,0,1,1]=(-0.198258834351)
        ref[0,1,0,0]=(0.21357757124)
        ref[0,1,0,1]=(0.193878132191)
        ref[0,1,1,0]=(0.0762258436603)
        ref[0,1,1,1]=(-0.33284111684)
        ref[1,0,0,0]=(-0.264232939297)
        ref[1,0,0,1]=(-0.726336076701)
        ref[1,0,1,0]=(-0.179928817534)
        ref[1,0,1,1]=(-0.723247805844)
        ref[1,1,0,0]=(0.923289681593)
        ref[1,1,0,1]=(0.986763135255)
        ref[1,1,1,0]=(0.065387254507)
        ref[1,1,1,1]=(-0.710965008545)
        ref[2,0,0,0]=(0.759853961917)
        ref[2,0,0,1]=(0.24486573839)
        ref[2,0,1,0]=(-0.318336679378)
        ref[2,0,1,1]=(0.39328274922)
        ref[2,1,0,0]=(-1.562818948)
        ref[2,1,0,1]=(-0.891340974738)
        ref[2,1,1,0]=(0.0508006116584)
        ref[2,1,1,1]=(1.17654134987)
        ref[3,0,0,0]=(-0.155883472458)
        ref[3,0,0,1]=(0.248809866836)
        ref[3,0,1,0]=(0.287794710222)
        ref[3,0,1,1]=(-1.72525392296)
        ref[3,1,0,0]=(0.00665202042795)
        ref[3,1,0,1]=(-0.0996395796968)
        ref[3,1,1,0]=(-0.19809021298)
        ref[3,1,1,1]=(0.487549616652)
        ref[4,0,0,0]=(0.764163773078)
        ref[4,0,0,1]=(-0.898694486894)
        ref[4,0,1,0]=(-0.213042355119)
        ref[4,0,1,1]=(0.708925705544)
        ref[4,1,0,0]=(1.55505564031)
        ref[4,1,0,1]=(-0.545746649289)
        ref[4,1,1,0]=(-1.31623592165)
        ref[4,1,1,1]=(0.948772953354)
        ref[5,0,0,0]=(0.221103671977)
        ref[5,0,0,1]=(0.866067107464)
        ref[5,0,1,0]=(-0.0979530612779)
        ref[5,0,1,1]=(-0.371623102578)
        ref[5,1,0,0]=(-0.348963169972)
        ref[5,1,0,1]=(-0.692055450872)
        ref[5,1,1,0]=(-0.584053243695)
        ref[5,1,1,1]=(-0.487768032842)
      else:
        arg[0,0,0]=(-1.45023987007)*x[0]+(0.138479134523)*x[1]+(1.46404607112)*x[2]
        arg[0,0,1]=(-0.647908832568)*x[0]+(-0.503758830859)*x[1]+(1.23196203665)*x[2]
        arg[0,1,0]=(-0.0108286161243)*x[0]+(0.565615928686)*x[1]+(-0.357789821636)*x[2]
        arg[0,1,1]=(0.648224572901)*x[0]+(0.775857442933)*x[1]+(0.960991462312)*x[2]
        arg[1,0,0]=(0.340590670624)*x[0]+(0.0357759349963)*x[1]+(0.358139256621)*x[2]
        arg[1,0,1]=(0.569789735109)*x[0]+(1.40678949618)*x[1]+(-0.965577726422)*x[2]
        arg[1,1,0]=(-0.758717082212)*x[0]+(0.612968081377)*x[1]+(-0.417988159948)*x[2]
        arg[1,1,1]=(0.969867032567)*x[0]+(-0.0700234825891)*x[1]+(1.53879850558)*x[2]
        arg[2,0,0]=(-1.02952648125)*x[0]+(0.108642437375)*x[1]+(0.251073184196)*x[2]
        arg[2,0,1]=(0.794766888909)*x[0]+(1.50591346365)*x[1]+(-0.36870578891)*x[2]
        arg[2,1,0]=(-0.534487055503)*x[0]+(-1.09061679233)*x[1]+(0.799342633685)*x[2]
        arg[2,1,1]=(0.0435889213805)*x[0]+(-0.209888628166)*x[1]+(1.34633215915)*x[2]
        arg[3,0,0]=(-0.0344682106376)*x[0]+(-0.92386569455)*x[1]+(-0.551250384619)*x[2]
        arg[3,0,1]=(-0.168599962689)*x[0]+(0.204634495314)*x[1]+(-0.65426520271)*x[2]
        arg[3,1,0]=(-1.61759210628)*x[0]+(-0.5922211067)*x[1]+(-0.139092780207)*x[2]
        arg[3,1,1]=(-0.13862300062)*x[0]+(-1.45951050222)*x[1]+(0.300904423579)*x[2]
        arg[4,0,0]=(0.629590770742)*x[0]+(-0.217150209985)*x[1]+(-0.140384227583)*x[2]
        arg[4,0,1]=(-0.0461635416119)*x[0]+(0.525692990183)*x[1]+(-1.89728167146)*x[2]
        arg[4,1,0]=(-0.130297059668)*x[0]+(-0.138816250007)*x[1]+(1.26805054232)*x[2]
        arg[4,1,1]=(0.742488751566)*x[0]+(0.659272799907)*x[1]+(-0.370877917078)*x[2]
        arg[5,0,0]=(-0.521350561164)*x[0]+(-0.213669151076)*x[1]+(0.00949381597317)*x[2]
        arg[5,0,1]=(-0.951547526648)*x[0]+(-0.388340895795)*x[1]+(0.741997603334)*x[2]
        arg[5,1,0]=(-0.332922940005)*x[0]+(1.74852940259)*x[1]+(-0.484189519128)*x[2]
        arg[5,1,1]=(0.418660109098)*x[0]+(-0.690728767024)*x[1]+(-0.675947375912)*x[2]
        ref[0,0,0,0]=(-1.45023987007)
        ref[0,0,0,1]=(0.138479134523)
        ref[0,0,0,2]=(1.46404607112)
        ref[0,0,1,0]=(-0.647908832568)
        ref[0,0,1,1]=(-0.503758830859)
        ref[0,0,1,2]=(1.23196203665)
        ref[0,1,0,0]=(-0.0108286161243)
        ref[0,1,0,1]=(0.565615928686)
        ref[0,1,0,2]=(-0.357789821636)
        ref[0,1,1,0]=(0.648224572901)
        ref[0,1,1,1]=(0.775857442933)
        ref[0,1,1,2]=(0.960991462312)
        ref[1,0,0,0]=(0.340590670624)
        ref[1,0,0,1]=(0.0357759349963)
        ref[1,0,0,2]=(0.358139256621)
        ref[1,0,1,0]=(0.569789735109)
        ref[1,0,1,1]=(1.40678949618)
        ref[1,0,1,2]=(-0.965577726422)
        ref[1,1,0,0]=(-0.758717082212)
        ref[1,1,0,1]=(0.612968081377)
        ref[1,1,0,2]=(-0.417988159948)
        ref[1,1,1,0]=(0.969867032567)
        ref[1,1,1,1]=(-0.0700234825891)
        ref[1,1,1,2]=(1.53879850558)
        ref[2,0,0,0]=(-1.02952648125)
        ref[2,0,0,1]=(0.108642437375)
        ref[2,0,0,2]=(0.251073184196)
        ref[2,0,1,0]=(0.794766888909)
        ref[2,0,1,1]=(1.50591346365)
        ref[2,0,1,2]=(-0.36870578891)
        ref[2,1,0,0]=(-0.534487055503)
        ref[2,1,0,1]=(-1.09061679233)
        ref[2,1,0,2]=(0.799342633685)
        ref[2,1,1,0]=(0.0435889213805)
        ref[2,1,1,1]=(-0.209888628166)
        ref[2,1,1,2]=(1.34633215915)
        ref[3,0,0,0]=(-0.0344682106376)
        ref[3,0,0,1]=(-0.92386569455)
        ref[3,0,0,2]=(-0.551250384619)
        ref[3,0,1,0]=(-0.168599962689)
        ref[3,0,1,1]=(0.204634495314)
        ref[3,0,1,2]=(-0.65426520271)
        ref[3,1,0,0]=(-1.61759210628)
        ref[3,1,0,1]=(-0.5922211067)
        ref[3,1,0,2]=(-0.139092780207)
        ref[3,1,1,0]=(-0.13862300062)
        ref[3,1,1,1]=(-1.45951050222)
        ref[3,1,1,2]=(0.300904423579)
        ref[4,0,0,0]=(0.629590770742)
        ref[4,0,0,1]=(-0.217150209985)
        ref[4,0,0,2]=(-0.140384227583)
        ref[4,0,1,0]=(-0.0461635416119)
        ref[4,0,1,1]=(0.525692990183)
        ref[4,0,1,2]=(-1.89728167146)
        ref[4,1,0,0]=(-0.130297059668)
        ref[4,1,0,1]=(-0.138816250007)
        ref[4,1,0,2]=(1.26805054232)
        ref[4,1,1,0]=(0.742488751566)
        ref[4,1,1,1]=(0.659272799907)
        ref[4,1,1,2]=(-0.370877917078)
        ref[5,0,0,0]=(-0.521350561164)
        ref[5,0,0,1]=(-0.213669151076)
        ref[5,0,0,2]=(0.00949381597317)
        ref[5,0,1,0]=(-0.951547526648)
        ref[5,0,1,1]=(-0.388340895795)
        ref[5,0,1,2]=(0.741997603334)
        ref[5,1,0,0]=(-0.332922940005)
        ref[5,1,0,1]=(1.74852940259)
        ref[5,1,0,2]=(-0.484189519128)
        ref[5,1,1,0]=(0.418660109098)
        ref[5,1,1,1]=(-0.690728767024)
        ref[5,1,1,2]=(-0.675947375912)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.149491618448)*x[0]**o+(-0.419584280113)*x[0]+(-0.0503569168892)*x[1]**o+(-0.490263902194)*x[1]
        ref[0]=o*(-0.149491618448)*x_ref[0]**(o-1)+(-0.419584280113)
        ref[1]=o*(-0.0503569168892)*x_ref[1]**(o-1)+(-0.490263902194)
      else:
        s=(-0.757365066356)*x[0]**o+(-0.051930048541)*x[0]+(0.384778052543)*x[1]**o+(-0.305544758353)*x[1]+(-0.462618121798)*x[2]**o+(-0.424504978394)*x[2]
        ref[0]=o*(-0.757365066356)*x_ref[0]**(o-1)+(-0.051930048541)
        ref[1]=o*(0.384778052543)*x_ref[1]**(o-1)+(-0.305544758353)
        ref[2]=o*(-0.462618121798)*x_ref[2]**(o-1)+(-0.424504978394)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.923338270579)*x[0]**o+(0.803299589693)*x[0]+(-0.0850750563669)*x[1]**o+(0.965911232881)*x[1]
        s[1]=(-0.183491365698)*x[0]**o+(-0.842618551859)*x[0]+(0.712774298035)*x[1]**o+(-0.104669018361)*x[1]
        ref[0,0]=o*(-0.923338270579)*x_ref[0]**(o-1)+(0.803299589693)
        ref[0,1]=o*(-0.0850750563669)*x_ref[1]**(o-1)+(0.965911232881)
        ref[1,0]=o*(-0.183491365698)*x_ref[0]**(o-1)+(-0.842618551859)
        ref[1,1]=o*(0.712774298035)*x_ref[1]**(o-1)+(-0.104669018361)
      else:
        s[0]=(0.522068862618)*x[0]**o+(-0.928491651715)*x[0]+(-0.056593136934)*x[1]**o+(-0.00345895910273)*x[1]+(-0.313948793745)*x[2]**o+(0.555562844396)*x[2]
        s[1]=(0.0381288968852)*x[0]**o+(-0.226585510125)*x[0]+(-0.483387509698)*x[1]**o+(-0.0276908054388)*x[1]+(-0.535521100811)*x[2]**o+(0.0628178579102)*x[2]
        ref[0,0]=o*(0.522068862618)*x_ref[0]**(o-1)+(-0.928491651715)
        ref[0,1]=o*(-0.056593136934)*x_ref[1]**(o-1)+(-0.00345895910273)
        ref[0,2]=o*(-0.313948793745)*x_ref[2]**(o-1)+(0.555562844396)
        ref[1,0]=o*(0.0381288968852)*x_ref[0]**(o-1)+(-0.226585510125)
        ref[1,1]=o*(-0.483387509698)*x_ref[1]**(o-1)+(-0.0276908054388)
        ref[1,2]=o*(-0.535521100811)*x_ref[2]**(o-1)+(0.0628178579102)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.0438013069888)*x[0]**o+(-0.962692066069)*x[0]+(0.398086110381)*x[1]**o+(0.943207722394)*x[1]
        s[0,1]=(-0.834074575881)*x[0]**o+(0.984008650691)*x[0]+(-0.16037316175)*x[1]**o+(0.377982608876)*x[1]
        s[0,2]=(-0.245264047174)*x[0]**o+(-0.541684401688)*x[0]+(0.826167130517)*x[1]**o+(0.30004870121)*x[1]
        s[0,3]=(0.126658708414)*x[0]**o+(0.647586365584)*x[0]+(0.258184629361)*x[1]**o+(-0.808198823815)*x[1]
        s[0,4]=(-0.0303747201021)*x[0]**o+(-0.095123002442)*x[0]+(-0.033747901785)*x[1]**o+(-0.0503016700716)*x[1]
        s[1,0]=(0.680161896488)*x[0]**o+(0.106001120727)*x[0]+(-0.983896702986)*x[1]**o+(0.298572049969)*x[1]
        s[1,1]=(-0.215100953009)*x[0]**o+(0.778768869339)*x[0]+(0.887004074512)*x[1]**o+(0.441037801242)*x[1]
        s[1,2]=(-0.892563680898)*x[0]**o+(0.087087180763)*x[0]+(0.16490400964)*x[1]**o+(0.641654994831)*x[1]
        s[1,3]=(0.735567594275)*x[0]**o+(0.143078234647)*x[0]+(-0.741664410771)*x[1]**o+(-0.37591452739)*x[1]
        s[1,4]=(-0.0739884625591)*x[0]**o+(0.745076950828)*x[0]+(0.432379987995)*x[1]**o+(-0.0687199383234)*x[1]
        s[2,0]=(-0.885674457439)*x[0]**o+(-0.456184815781)*x[0]+(0.994806374374)*x[1]**o+(-0.31107028169)*x[1]
        s[2,1]=(0.987856338645)*x[0]**o+(-0.568662820963)*x[0]+(-0.742199948211)*x[1]**o+(0.658035428735)*x[1]
        s[2,2]=(-0.17580106245)*x[0]**o+(-0.589665223227)*x[0]+(0.580399510219)*x[1]**o+(-0.155064144488)*x[1]
        s[2,3]=(-0.913915914987)*x[0]**o+(0.3569536228)*x[0]+(0.23235960166)*x[1]**o+(-0.847113665947)*x[1]
        s[2,4]=(0.0909806665742)*x[0]**o+(-0.643246933435)*x[0]+(-0.480053101714)*x[1]**o+(0.336898095124)*x[1]
        s[3,0]=(-0.944997214199)*x[0]**o+(0.0242583099745)*x[0]+(-0.0537207940075)*x[1]**o+(-0.985424066301)*x[1]
        s[3,1]=(0.685233610656)*x[0]**o+(0.869977880645)*x[0]+(0.28306562014)*x[1]**o+(-0.144951426405)*x[1]
        s[3,2]=(-0.0432942460834)*x[0]**o+(0.706443143255)*x[0]+(-0.880454998397)*x[1]**o+(0.612846993665)*x[1]
        s[3,3]=(0.79137204642)*x[0]**o+(-0.534256675007)*x[0]+(-0.323548658904)*x[1]**o+(0.667614029523)*x[1]
        s[3,4]=(0.0642269065087)*x[0]**o+(0.299466531576)*x[0]+(-0.563060543682)*x[1]**o+(-0.820583826087)*x[1]
        ref[0,0,0]=o*(-0.0438013069888)*x_ref[0]**(o-1)+(-0.962692066069)
        ref[0,0,1]=o*(0.398086110381)*x_ref[1]**(o-1)+(0.943207722394)
        ref[0,1,0]=o*(-0.834074575881)*x_ref[0]**(o-1)+(0.984008650691)
        ref[0,1,1]=o*(-0.16037316175)*x_ref[1]**(o-1)+(0.377982608876)
        ref[0,2,0]=o*(-0.245264047174)*x_ref[0]**(o-1)+(-0.541684401688)
        ref[0,2,1]=o*(0.826167130517)*x_ref[1]**(o-1)+(0.30004870121)
        ref[0,3,0]=o*(0.126658708414)*x_ref[0]**(o-1)+(0.647586365584)
        ref[0,3,1]=o*(0.258184629361)*x_ref[1]**(o-1)+(-0.808198823815)
        ref[0,4,0]=o*(-0.0303747201021)*x_ref[0]**(o-1)+(-0.095123002442)
        ref[0,4,1]=o*(-0.033747901785)*x_ref[1]**(o-1)+(-0.0503016700716)
        ref[1,0,0]=o*(0.680161896488)*x_ref[0]**(o-1)+(0.106001120727)
        ref[1,0,1]=o*(-0.983896702986)*x_ref[1]**(o-1)+(0.298572049969)
        ref[1,1,0]=o*(-0.215100953009)*x_ref[0]**(o-1)+(0.778768869339)
        ref[1,1,1]=o*(0.887004074512)*x_ref[1]**(o-1)+(0.441037801242)
        ref[1,2,0]=o*(-0.892563680898)*x_ref[0]**(o-1)+(0.087087180763)
        ref[1,2,1]=o*(0.16490400964)*x_ref[1]**(o-1)+(0.641654994831)
        ref[1,3,0]=o*(0.735567594275)*x_ref[0]**(o-1)+(0.143078234647)
        ref[1,3,1]=o*(-0.741664410771)*x_ref[1]**(o-1)+(-0.37591452739)
        ref[1,4,0]=o*(-0.0739884625591)*x_ref[0]**(o-1)+(0.745076950828)
        ref[1,4,1]=o*(0.432379987995)*x_ref[1]**(o-1)+(-0.0687199383234)
        ref[2,0,0]=o*(-0.885674457439)*x_ref[0]**(o-1)+(-0.456184815781)
        ref[2,0,1]=o*(0.994806374374)*x_ref[1]**(o-1)+(-0.31107028169)
        ref[2,1,0]=o*(0.987856338645)*x_ref[0]**(o-1)+(-0.568662820963)
        ref[2,1,1]=o*(-0.742199948211)*x_ref[1]**(o-1)+(0.658035428735)
        ref[2,2,0]=o*(-0.17580106245)*x_ref[0]**(o-1)+(-0.589665223227)
        ref[2,2,1]=o*(0.580399510219)*x_ref[1]**(o-1)+(-0.155064144488)
        ref[2,3,0]=o*(-0.913915914987)*x_ref[0]**(o-1)+(0.3569536228)
        ref[2,3,1]=o*(0.23235960166)*x_ref[1]**(o-1)+(-0.847113665947)
        ref[2,4,0]=o*(0.0909806665742)*x_ref[0]**(o-1)+(-0.643246933435)
        ref[2,4,1]=o*(-0.480053101714)*x_ref[1]**(o-1)+(0.336898095124)
        ref[3,0,0]=o*(-0.944997214199)*x_ref[0]**(o-1)+(0.0242583099745)
        ref[3,0,1]=o*(-0.0537207940075)*x_ref[1]**(o-1)+(-0.985424066301)
        ref[3,1,0]=o*(0.685233610656)*x_ref[0]**(o-1)+(0.869977880645)
        ref[3,1,1]=o*(0.28306562014)*x_ref[1]**(o-1)+(-0.144951426405)
        ref[3,2,0]=o*(-0.0432942460834)*x_ref[0]**(o-1)+(0.706443143255)
        ref[3,2,1]=o*(-0.880454998397)*x_ref[1]**(o-1)+(0.612846993665)
        ref[3,3,0]=o*(0.79137204642)*x_ref[0]**(o-1)+(-0.534256675007)
        ref[3,3,1]=o*(-0.323548658904)*x_ref[1]**(o-1)+(0.667614029523)
        ref[3,4,0]=o*(0.0642269065087)*x_ref[0]**(o-1)+(0.299466531576)
        ref[3,4,1]=o*(-0.563060543682)*x_ref[1]**(o-1)+(-0.820583826087)
      else:
        s[0,0]=(0.554048013549)*x[0]**o+(0.628547931484)*x[0]+(-0.087120818445)*x[1]**o+(0.105429960285)*x[1]+(0.221041961114)*x[2]**o+(-0.617484121819)*x[2]
        s[0,1]=(-0.554948686776)*x[0]**o+(-0.271548517185)*x[0]+(0.167744163287)*x[1]**o+(-0.281562288124)*x[1]+(0.0995920439425)*x[2]**o+(0.952925293565)*x[2]
        s[0,2]=(-0.233604089512)*x[0]**o+(-0.395317021862)*x[0]+(-0.148631539093)*x[1]**o+(0.566615423332)*x[1]+(-0.851305702162)*x[2]**o+(0.522170109012)*x[2]
        s[0,3]=(0.33041096125)*x[0]**o+(0.531550689883)*x[0]+(0.43601483601)*x[1]**o+(0.508994516005)*x[1]+(0.848624945097)*x[2]**o+(0.367985533319)*x[2]
        s[0,4]=(0.452749219251)*x[0]**o+(-0.174905238359)*x[0]+(0.156533271339)*x[1]**o+(0.238697207856)*x[1]+(-0.287482388632)*x[2]**o+(-0.30704613609)*x[2]
        s[1,0]=(0.731135831652)*x[0]**o+(0.240028680462)*x[0]+(0.994673716713)*x[1]**o+(-0.14275472627)*x[1]+(0.0105619030307)*x[2]**o+(0.709488861006)*x[2]
        s[1,1]=(-0.494857482432)*x[0]**o+(-0.659820603466)*x[0]+(0.771064201278)*x[1]**o+(0.840663673162)*x[1]+(-0.141911130141)*x[2]**o+(-0.889454650127)*x[2]
        s[1,2]=(-0.492412677213)*x[0]**o+(0.432517067517)*x[0]+(0.239327954605)*x[1]**o+(-0.437437759354)*x[1]+(-0.188157683846)*x[2]**o+(-0.490906503638)*x[2]
        s[1,3]=(-0.313081049716)*x[0]**o+(-0.934448134549)*x[0]+(0.666459048562)*x[1]**o+(0.947256512054)*x[1]+(-0.710248833935)*x[2]**o+(-0.796385160956)*x[2]
        s[1,4]=(0.997217607294)*x[0]**o+(0.939841022657)*x[0]+(-0.857321524031)*x[1]**o+(-0.134948519166)*x[1]+(-0.383752839014)*x[2]**o+(-0.0694045288726)*x[2]
        s[2,0]=(0.884993378314)*x[0]**o+(-0.305179802237)*x[0]+(0.43264887709)*x[1]**o+(0.281387756518)*x[1]+(-0.887582464922)*x[2]**o+(-0.46966538775)*x[2]
        s[2,1]=(0.640836781354)*x[0]**o+(0.822302879707)*x[0]+(-0.78092397026)*x[1]**o+(-0.977840224136)*x[1]+(0.368581385054)*x[2]**o+(0.657227564549)*x[2]
        s[2,2]=(-0.148123980628)*x[0]**o+(-0.69167854219)*x[0]+(-0.93156581239)*x[1]**o+(-0.79482831934)*x[1]+(-0.816815232666)*x[2]**o+(0.0127598824004)*x[2]
        s[2,3]=(-0.610703986086)*x[0]**o+(0.274872054308)*x[0]+(0.0215106382843)*x[1]**o+(0.72724816172)*x[1]+(-0.915709139681)*x[2]**o+(0.321703565556)*x[2]
        s[2,4]=(0.381549090843)*x[0]**o+(-0.962121232421)*x[0]+(-0.897209804063)*x[1]**o+(-0.766441490629)*x[1]+(-0.741441127879)*x[2]**o+(-0.142615155341)*x[2]
        s[3,0]=(0.893922704275)*x[0]**o+(-0.683229796558)*x[0]+(0.900359565899)*x[1]**o+(0.785771212984)*x[1]+(0.216603036961)*x[2]**o+(0.144463289025)*x[2]
        s[3,1]=(0.360659657586)*x[0]**o+(-0.207597777231)*x[0]+(-0.350428345153)*x[1]**o+(-0.159358793155)*x[1]+(0.373616042827)*x[2]**o+(-0.0155120388232)*x[2]
        s[3,2]=(0.335734139009)*x[0]**o+(0.0880595596639)*x[0]+(-0.618682099923)*x[1]**o+(-0.409019944145)*x[1]+(0.56154653361)*x[2]**o+(-0.384031273173)*x[2]
        s[3,3]=(0.738466863893)*x[0]**o+(-0.305503217774)*x[0]+(-0.133472928746)*x[1]**o+(0.803290264921)*x[1]+(-0.919787049033)*x[2]**o+(0.138860079244)*x[2]
        s[3,4]=(0.0312193600407)*x[0]**o+(0.340894051514)*x[0]+(0.100198716313)*x[1]**o+(-0.196988878013)*x[1]+(0.0825369961302)*x[2]**o+(-0.652573275397)*x[2]
        ref[0,0,0]=o*(0.554048013549)*x_ref[0]**(o-1)+(0.628547931484)
        ref[0,0,1]=o*(-0.087120818445)*x_ref[1]**(o-1)+(0.105429960285)
        ref[0,0,2]=o*(0.221041961114)*x_ref[2]**(o-1)+(-0.617484121819)
        ref[0,1,0]=o*(-0.554948686776)*x_ref[0]**(o-1)+(-0.271548517185)
        ref[0,1,1]=o*(0.167744163287)*x_ref[1]**(o-1)+(-0.281562288124)
        ref[0,1,2]=o*(0.0995920439425)*x_ref[2]**(o-1)+(0.952925293565)
        ref[0,2,0]=o*(-0.233604089512)*x_ref[0]**(o-1)+(-0.395317021862)
        ref[0,2,1]=o*(-0.148631539093)*x_ref[1]**(o-1)+(0.566615423332)
        ref[0,2,2]=o*(-0.851305702162)*x_ref[2]**(o-1)+(0.522170109012)
        ref[0,3,0]=o*(0.33041096125)*x_ref[0]**(o-1)+(0.531550689883)
        ref[0,3,1]=o*(0.43601483601)*x_ref[1]**(o-1)+(0.508994516005)
        ref[0,3,2]=o*(0.848624945097)*x_ref[2]**(o-1)+(0.367985533319)
        ref[0,4,0]=o*(0.452749219251)*x_ref[0]**(o-1)+(-0.174905238359)
        ref[0,4,1]=o*(0.156533271339)*x_ref[1]**(o-1)+(0.238697207856)
        ref[0,4,2]=o*(-0.287482388632)*x_ref[2]**(o-1)+(-0.30704613609)
        ref[1,0,0]=o*(0.731135831652)*x_ref[0]**(o-1)+(0.240028680462)
        ref[1,0,1]=o*(0.994673716713)*x_ref[1]**(o-1)+(-0.14275472627)
        ref[1,0,2]=o*(0.0105619030307)*x_ref[2]**(o-1)+(0.709488861006)
        ref[1,1,0]=o*(-0.494857482432)*x_ref[0]**(o-1)+(-0.659820603466)
        ref[1,1,1]=o*(0.771064201278)*x_ref[1]**(o-1)+(0.840663673162)
        ref[1,1,2]=o*(-0.141911130141)*x_ref[2]**(o-1)+(-0.889454650127)
        ref[1,2,0]=o*(-0.492412677213)*x_ref[0]**(o-1)+(0.432517067517)
        ref[1,2,1]=o*(0.239327954605)*x_ref[1]**(o-1)+(-0.437437759354)
        ref[1,2,2]=o*(-0.188157683846)*x_ref[2]**(o-1)+(-0.490906503638)
        ref[1,3,0]=o*(-0.313081049716)*x_ref[0]**(o-1)+(-0.934448134549)
        ref[1,3,1]=o*(0.666459048562)*x_ref[1]**(o-1)+(0.947256512054)
        ref[1,3,2]=o*(-0.710248833935)*x_ref[2]**(o-1)+(-0.796385160956)
        ref[1,4,0]=o*(0.997217607294)*x_ref[0]**(o-1)+(0.939841022657)
        ref[1,4,1]=o*(-0.857321524031)*x_ref[1]**(o-1)+(-0.134948519166)
        ref[1,4,2]=o*(-0.383752839014)*x_ref[2]**(o-1)+(-0.0694045288726)
        ref[2,0,0]=o*(0.884993378314)*x_ref[0]**(o-1)+(-0.305179802237)
        ref[2,0,1]=o*(0.43264887709)*x_ref[1]**(o-1)+(0.281387756518)
        ref[2,0,2]=o*(-0.887582464922)*x_ref[2]**(o-1)+(-0.46966538775)
        ref[2,1,0]=o*(0.640836781354)*x_ref[0]**(o-1)+(0.822302879707)
        ref[2,1,1]=o*(-0.78092397026)*x_ref[1]**(o-1)+(-0.977840224136)
        ref[2,1,2]=o*(0.368581385054)*x_ref[2]**(o-1)+(0.657227564549)
        ref[2,2,0]=o*(-0.148123980628)*x_ref[0]**(o-1)+(-0.69167854219)
        ref[2,2,1]=o*(-0.93156581239)*x_ref[1]**(o-1)+(-0.79482831934)
        ref[2,2,2]=o*(-0.816815232666)*x_ref[2]**(o-1)+(0.0127598824004)
        ref[2,3,0]=o*(-0.610703986086)*x_ref[0]**(o-1)+(0.274872054308)
        ref[2,3,1]=o*(0.0215106382843)*x_ref[1]**(o-1)+(0.72724816172)
        ref[2,3,2]=o*(-0.915709139681)*x_ref[2]**(o-1)+(0.321703565556)
        ref[2,4,0]=o*(0.381549090843)*x_ref[0]**(o-1)+(-0.962121232421)
        ref[2,4,1]=o*(-0.897209804063)*x_ref[1]**(o-1)+(-0.766441490629)
        ref[2,4,2]=o*(-0.741441127879)*x_ref[2]**(o-1)+(-0.142615155341)
        ref[3,0,0]=o*(0.893922704275)*x_ref[0]**(o-1)+(-0.683229796558)
        ref[3,0,1]=o*(0.900359565899)*x_ref[1]**(o-1)+(0.785771212984)
        ref[3,0,2]=o*(0.216603036961)*x_ref[2]**(o-1)+(0.144463289025)
        ref[3,1,0]=o*(0.360659657586)*x_ref[0]**(o-1)+(-0.207597777231)
        ref[3,1,1]=o*(-0.350428345153)*x_ref[1]**(o-1)+(-0.159358793155)
        ref[3,1,2]=o*(0.373616042827)*x_ref[2]**(o-1)+(-0.0155120388232)
        ref[3,2,0]=o*(0.335734139009)*x_ref[0]**(o-1)+(0.0880595596639)
        ref[3,2,1]=o*(-0.618682099923)*x_ref[1]**(o-1)+(-0.409019944145)
        ref[3,2,2]=o*(0.56154653361)*x_ref[2]**(o-1)+(-0.384031273173)
        ref[3,3,0]=o*(0.738466863893)*x_ref[0]**(o-1)+(-0.305503217774)
        ref[3,3,1]=o*(-0.133472928746)*x_ref[1]**(o-1)+(0.803290264921)
        ref[3,3,2]=o*(-0.919787049033)*x_ref[2]**(o-1)+(0.138860079244)
        ref[3,4,0]=o*(0.0312193600407)*x_ref[0]**(o-1)+(0.340894051514)
        ref[3,4,1]=o*(0.100198716313)*x_ref[1]**(o-1)+(-0.196988878013)
        ref[3,4,2]=o*(0.0825369961302)*x_ref[2]**(o-1)+(-0.652573275397)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.178576436338)*x[0]**o+(0.548880215769)*x[0]+(-0.318815466914)*x[1]**o+(-0.798063605205)*x[1]
        s[0,0,1]=(-0.822727567562)*x[0]**o+(-0.947988410328)*x[0]+(0.756740500537)*x[1]**o+(0.997703924729)*x[1]
        s[0,1,0]=(0.145493259882)*x[0]**o+(0.512878926373)*x[0]+(0.188564032293)*x[1]**o+(0.900204838942)*x[1]
        s[0,1,1]=(-0.88128289676)*x[0]**o+(0.833100847964)*x[0]+(-0.419907100559)*x[1]**o+(-0.461899797784)*x[1]
        s[1,0,0]=(-0.185721953771)*x[0]**o+(0.0783345782049)*x[0]+(-0.939442057638)*x[1]**o+(-0.320245477333)*x[1]
        s[1,0,1]=(0.410922982405)*x[0]**o+(0.398440443266)*x[0]+(0.768185628746)*x[1]**o+(0.256153329651)*x[1]
        s[1,1,0]=(-0.526878655538)*x[0]**o+(-0.658168532232)*x[0]+(0.69953069671)*x[1]**o+(-0.916587227061)*x[1]
        s[1,1,1]=(0.872254289223)*x[0]**o+(-0.804299235664)*x[0]+(-0.245503808339)*x[1]**o+(-0.894388559613)*x[1]
        s[2,0,0]=(0.917787084987)*x[0]**o+(-0.826075775004)*x[0]+(0.755866825722)*x[1]**o+(0.212054055974)*x[1]
        s[2,0,1]=(0.722119407968)*x[0]**o+(-0.357088696868)*x[0]+(0.033293020384)*x[1]**o+(-0.383269284911)*x[1]
        s[2,1,0]=(0.856304318608)*x[0]**o+(-0.918845722628)*x[0]+(-0.17232500803)*x[1]**o+(0.859254907185)*x[1]
        s[2,1,1]=(0.440452546297)*x[0]**o+(0.0912337401501)*x[0]+(0.76436338959)*x[1]**o+(-0.628986501066)*x[1]
        s[3,0,0]=(-0.583286304487)*x[0]**o+(0.102129275171)*x[0]+(-0.811858544156)*x[1]**o+(-0.521643540274)*x[1]
        s[3,0,1]=(0.682705489393)*x[0]**o+(0.722776291806)*x[0]+(-0.0417392890997)*x[1]**o+(0.487556285425)*x[1]
        s[3,1,0]=(-0.958268636176)*x[0]**o+(-0.666213997791)*x[0]+(-0.702747133353)*x[1]**o+(0.465173353165)*x[1]
        s[3,1,1]=(-0.919518552212)*x[0]**o+(0.607551779709)*x[0]+(0.4478984898)*x[1]**o+(0.0217822207015)*x[1]
        s[4,0,0]=(-0.625448546192)*x[0]**o+(0.513680244128)*x[0]+(0.0190646155266)*x[1]**o+(0.0155037658268)*x[1]
        s[4,0,1]=(-0.573453340974)*x[0]**o+(0.452807235507)*x[0]+(0.902938708535)*x[1]**o+(0.123632055082)*x[1]
        s[4,1,0]=(0.760888494849)*x[0]**o+(-0.488658748567)*x[0]+(0.949883579341)*x[1]**o+(-0.205749942866)*x[1]
        s[4,1,1]=(-0.2860871769)*x[0]**o+(0.614822186727)*x[0]+(0.118313781134)*x[1]**o+(0.448320553312)*x[1]
        s[5,0,0]=(0.671714056383)*x[0]**o+(-0.441343906973)*x[0]+(0.235137729851)*x[1]**o+(0.0696929938911)*x[1]
        s[5,0,1]=(0.114159304365)*x[0]**o+(0.562991557608)*x[0]+(0.00444483447333)*x[1]**o+(0.434809595495)*x[1]
        s[5,1,0]=(0.0360353162225)*x[0]**o+(-0.366777208182)*x[0]+(-0.668602224156)*x[1]**o+(0.449496848779)*x[1]
        s[5,1,1]=(-0.982227533448)*x[0]**o+(-0.704480676355)*x[0]+(-0.187844946341)*x[1]**o+(0.0975753620283)*x[1]
        ref[0,0,0,0]=o*(-0.178576436338)*x_ref[0]**(o-1)+(0.548880215769)
        ref[0,0,0,1]=o*(-0.318815466914)*x_ref[1]**(o-1)+(-0.798063605205)
        ref[0,0,1,0]=o*(-0.822727567562)*x_ref[0]**(o-1)+(-0.947988410328)
        ref[0,0,1,1]=o*(0.756740500537)*x_ref[1]**(o-1)+(0.997703924729)
        ref[0,1,0,0]=o*(0.145493259882)*x_ref[0]**(o-1)+(0.512878926373)
        ref[0,1,0,1]=o*(0.188564032293)*x_ref[1]**(o-1)+(0.900204838942)
        ref[0,1,1,0]=o*(-0.88128289676)*x_ref[0]**(o-1)+(0.833100847964)
        ref[0,1,1,1]=o*(-0.419907100559)*x_ref[1]**(o-1)+(-0.461899797784)
        ref[1,0,0,0]=o*(-0.185721953771)*x_ref[0]**(o-1)+(0.0783345782049)
        ref[1,0,0,1]=o*(-0.939442057638)*x_ref[1]**(o-1)+(-0.320245477333)
        ref[1,0,1,0]=o*(0.410922982405)*x_ref[0]**(o-1)+(0.398440443266)
        ref[1,0,1,1]=o*(0.768185628746)*x_ref[1]**(o-1)+(0.256153329651)
        ref[1,1,0,0]=o*(-0.526878655538)*x_ref[0]**(o-1)+(-0.658168532232)
        ref[1,1,0,1]=o*(0.69953069671)*x_ref[1]**(o-1)+(-0.916587227061)
        ref[1,1,1,0]=o*(0.872254289223)*x_ref[0]**(o-1)+(-0.804299235664)
        ref[1,1,1,1]=o*(-0.245503808339)*x_ref[1]**(o-1)+(-0.894388559613)
        ref[2,0,0,0]=o*(0.917787084987)*x_ref[0]**(o-1)+(-0.826075775004)
        ref[2,0,0,1]=o*(0.755866825722)*x_ref[1]**(o-1)+(0.212054055974)
        ref[2,0,1,0]=o*(0.722119407968)*x_ref[0]**(o-1)+(-0.357088696868)
        ref[2,0,1,1]=o*(0.033293020384)*x_ref[1]**(o-1)+(-0.383269284911)
        ref[2,1,0,0]=o*(0.856304318608)*x_ref[0]**(o-1)+(-0.918845722628)
        ref[2,1,0,1]=o*(-0.17232500803)*x_ref[1]**(o-1)+(0.859254907185)
        ref[2,1,1,0]=o*(0.440452546297)*x_ref[0]**(o-1)+(0.0912337401501)
        ref[2,1,1,1]=o*(0.76436338959)*x_ref[1]**(o-1)+(-0.628986501066)
        ref[3,0,0,0]=o*(-0.583286304487)*x_ref[0]**(o-1)+(0.102129275171)
        ref[3,0,0,1]=o*(-0.811858544156)*x_ref[1]**(o-1)+(-0.521643540274)
        ref[3,0,1,0]=o*(0.682705489393)*x_ref[0]**(o-1)+(0.722776291806)
        ref[3,0,1,1]=o*(-0.0417392890997)*x_ref[1]**(o-1)+(0.487556285425)
        ref[3,1,0,0]=o*(-0.958268636176)*x_ref[0]**(o-1)+(-0.666213997791)
        ref[3,1,0,1]=o*(-0.702747133353)*x_ref[1]**(o-1)+(0.465173353165)
        ref[3,1,1,0]=o*(-0.919518552212)*x_ref[0]**(o-1)+(0.607551779709)
        ref[3,1,1,1]=o*(0.4478984898)*x_ref[1]**(o-1)+(0.0217822207015)
        ref[4,0,0,0]=o*(-0.625448546192)*x_ref[0]**(o-1)+(0.513680244128)
        ref[4,0,0,1]=o*(0.0190646155266)*x_ref[1]**(o-1)+(0.0155037658268)
        ref[4,0,1,0]=o*(-0.573453340974)*x_ref[0]**(o-1)+(0.452807235507)
        ref[4,0,1,1]=o*(0.902938708535)*x_ref[1]**(o-1)+(0.123632055082)
        ref[4,1,0,0]=o*(0.760888494849)*x_ref[0]**(o-1)+(-0.488658748567)
        ref[4,1,0,1]=o*(0.949883579341)*x_ref[1]**(o-1)+(-0.205749942866)
        ref[4,1,1,0]=o*(-0.2860871769)*x_ref[0]**(o-1)+(0.614822186727)
        ref[4,1,1,1]=o*(0.118313781134)*x_ref[1]**(o-1)+(0.448320553312)
        ref[5,0,0,0]=o*(0.671714056383)*x_ref[0]**(o-1)+(-0.441343906973)
        ref[5,0,0,1]=o*(0.235137729851)*x_ref[1]**(o-1)+(0.0696929938911)
        ref[5,0,1,0]=o*(0.114159304365)*x_ref[0]**(o-1)+(0.562991557608)
        ref[5,0,1,1]=o*(0.00444483447333)*x_ref[1]**(o-1)+(0.434809595495)
        ref[5,1,0,0]=o*(0.0360353162225)*x_ref[0]**(o-1)+(-0.366777208182)
        ref[5,1,0,1]=o*(-0.668602224156)*x_ref[1]**(o-1)+(0.449496848779)
        ref[5,1,1,0]=o*(-0.982227533448)*x_ref[0]**(o-1)+(-0.704480676355)
        ref[5,1,1,1]=o*(-0.187844946341)*x_ref[1]**(o-1)+(0.0975753620283)
      else:
        s[0,0,0]=(0.983153853699)*x[0]**o+(0.877999648347)*x[0]+(0.673605473153)*x[1]**o+(0.469503898589)*x[1]+(-0.069158689944)*x[2]**o+(-0.861916140555)*x[2]
        s[0,0,1]=(-0.553610878092)*x[0]**o+(0.0506456195871)*x[0]+(-0.474818319583)*x[1]**o+(0.578919616336)*x[1]+(0.264746159866)*x[2]**o+(-0.433622198203)*x[2]
        s[0,1,0]=(0.666034530743)*x[0]**o+(0.546828305775)*x[0]+(-0.512009992229)*x[1]**o+(-0.476398683846)*x[1]+(-0.570551384835)*x[2]**o+(-0.133446991616)*x[2]
        s[0,1,1]=(0.539382416391)*x[0]**o+(-0.278692810377)*x[0]+(0.10726626544)*x[1]**o+(-0.966085403104)*x[1]+(-0.658134958165)*x[2]**o+(0.924604577895)*x[2]
        s[1,0,0]=(-0.330421615628)*x[0]**o+(0.724947570665)*x[0]+(0.34637731641)*x[1]**o+(0.39702161282)*x[1]+(0.931725860392)*x[2]**o+(-0.673326580823)*x[2]
        s[1,0,1]=(0.353966196399)*x[0]**o+(0.472639435387)*x[0]+(-0.328174031243)*x[1]**o+(0.640610221795)*x[1]+(-0.900613354552)*x[2]**o+(0.796468730393)*x[2]
        s[1,1,0]=(0.754328288128)*x[0]**o+(-0.846972319862)*x[0]+(0.845142974251)*x[1]**o+(0.144707523269)*x[1]+(-0.135529271924)*x[2]**o+(0.232438947532)*x[2]
        s[1,1,1]=(0.0826879113034)*x[0]**o+(-0.532753089482)*x[0]+(-0.617867675502)*x[1]**o+(0.327150779323)*x[1]+(-0.773918767657)*x[2]**o+(0.0695419192896)*x[2]
        s[2,0,0]=(0.861666752281)*x[0]**o+(-0.369904662594)*x[0]+(-0.809447635327)*x[1]**o+(0.314495362669)*x[1]+(0.672158503953)*x[2]**o+(0.779729363207)*x[2]
        s[2,0,1]=(0.784251939885)*x[0]**o+(-0.657779846806)*x[0]+(-0.694796642388)*x[1]**o+(0.549290583812)*x[1]+(0.452712301324)*x[2]**o+(0.0965361955733)*x[2]
        s[2,1,0]=(0.526035829224)*x[0]**o+(-0.5285468198)*x[0]+(0.426231604453)*x[1]**o+(-0.89107298075)*x[1]+(0.365149699988)*x[2]**o+(0.87436690224)*x[2]
        s[2,1,1]=(-0.516537342331)*x[0]**o+(-0.793927917391)*x[0]+(0.795130846095)*x[1]**o+(0.782928826071)*x[1]+(-0.874002864146)*x[2]**o+(0.465526006676)*x[2]
        s[3,0,0]=(-0.861917370502)*x[0]**o+(-0.0636266572453)*x[0]+(-0.0140222042452)*x[1]**o+(0.143759402339)*x[1]+(-0.247983260546)*x[2]**o+(-0.596358912223)*x[2]
        s[3,0,1]=(0.120302367516)*x[0]**o+(-0.701350969734)*x[0]+(-0.775100614072)*x[1]**o+(-0.248023481318)*x[1]+(0.849345512002)*x[2]**o+(0.542529986112)*x[2]
        s[3,1,0]=(-0.347014578244)*x[0]**o+(0.611296458158)*x[0]+(-0.11214501466)*x[1]**o+(0.516100884048)*x[1]+(-0.746366893941)*x[2]**o+(-0.429950357634)*x[2]
        s[3,1,1]=(-0.154199420675)*x[0]**o+(-0.533525033722)*x[0]+(-0.305802182284)*x[1]**o+(0.0249510778534)*x[1]+(0.511832901845)*x[2]**o+(0.865423961695)*x[2]
        s[4,0,0]=(0.270951033321)*x[0]**o+(0.293153287275)*x[0]+(0.499008893086)*x[1]**o+(0.401454188898)*x[1]+(-0.0659433492694)*x[2]**o+(0.0822124121616)*x[2]
        s[4,0,1]=(-0.770956381696)*x[0]**o+(0.962572964564)*x[0]+(-0.173527764809)*x[1]**o+(-0.764724502533)*x[1]+(0.686939133626)*x[2]**o+(0.589061927699)*x[2]
        s[4,1,0]=(0.856144070276)*x[0]**o+(-0.970851254884)*x[0]+(-0.200238695073)*x[1]**o+(0.961653891428)*x[1]+(-0.921215646185)*x[2]**o+(-0.927582743302)*x[2]
        s[4,1,1]=(0.518201773728)*x[0]**o+(0.403940302766)*x[0]+(-0.0333026733014)*x[1]**o+(0.602796446678)*x[1]+(0.956622086807)*x[2]**o+(0.445065392727)*x[2]
        s[5,0,0]=(0.847162899069)*x[0]**o+(-0.11079738914)*x[0]+(0.815117104026)*x[1]**o+(-0.0126064617312)*x[1]+(0.434649696776)*x[2]**o+(-0.925434721648)*x[2]
        s[5,0,1]=(0.827068835777)*x[0]**o+(0.267176529776)*x[0]+(-0.954551141359)*x[1]**o+(-0.847951054931)*x[1]+(0.939843025231)*x[2]**o+(-0.405674486263)*x[2]
        s[5,1,0]=(0.303863917444)*x[0]**o+(0.33377358223)*x[0]+(0.160136648003)*x[1]**o+(0.557340036308)*x[1]+(0.348362672822)*x[2]**o+(-0.719624087847)*x[2]
        s[5,1,1]=(-0.61009211312)*x[0]**o+(0.948776841387)*x[0]+(0.0348157372912)*x[1]**o+(0.415543380004)*x[1]+(-0.104796276752)*x[2]**o+(0.109481954783)*x[2]
        ref[0,0,0,0]=o*(0.983153853699)*x_ref[0]**(o-1)+(0.877999648347)
        ref[0,0,0,1]=o*(0.673605473153)*x_ref[1]**(o-1)+(0.469503898589)
        ref[0,0,0,2]=o*(-0.069158689944)*x_ref[2]**(o-1)+(-0.861916140555)
        ref[0,0,1,0]=o*(-0.553610878092)*x_ref[0]**(o-1)+(0.0506456195871)
        ref[0,0,1,1]=o*(-0.474818319583)*x_ref[1]**(o-1)+(0.578919616336)
        ref[0,0,1,2]=o*(0.264746159866)*x_ref[2]**(o-1)+(-0.433622198203)
        ref[0,1,0,0]=o*(0.666034530743)*x_ref[0]**(o-1)+(0.546828305775)
        ref[0,1,0,1]=o*(-0.512009992229)*x_ref[1]**(o-1)+(-0.476398683846)
        ref[0,1,0,2]=o*(-0.570551384835)*x_ref[2]**(o-1)+(-0.133446991616)
        ref[0,1,1,0]=o*(0.539382416391)*x_ref[0]**(o-1)+(-0.278692810377)
        ref[0,1,1,1]=o*(0.10726626544)*x_ref[1]**(o-1)+(-0.966085403104)
        ref[0,1,1,2]=o*(-0.658134958165)*x_ref[2]**(o-1)+(0.924604577895)
        ref[1,0,0,0]=o*(-0.330421615628)*x_ref[0]**(o-1)+(0.724947570665)
        ref[1,0,0,1]=o*(0.34637731641)*x_ref[1]**(o-1)+(0.39702161282)
        ref[1,0,0,2]=o*(0.931725860392)*x_ref[2]**(o-1)+(-0.673326580823)
        ref[1,0,1,0]=o*(0.353966196399)*x_ref[0]**(o-1)+(0.472639435387)
        ref[1,0,1,1]=o*(-0.328174031243)*x_ref[1]**(o-1)+(0.640610221795)
        ref[1,0,1,2]=o*(-0.900613354552)*x_ref[2]**(o-1)+(0.796468730393)
        ref[1,1,0,0]=o*(0.754328288128)*x_ref[0]**(o-1)+(-0.846972319862)
        ref[1,1,0,1]=o*(0.845142974251)*x_ref[1]**(o-1)+(0.144707523269)
        ref[1,1,0,2]=o*(-0.135529271924)*x_ref[2]**(o-1)+(0.232438947532)
        ref[1,1,1,0]=o*(0.0826879113034)*x_ref[0]**(o-1)+(-0.532753089482)
        ref[1,1,1,1]=o*(-0.617867675502)*x_ref[1]**(o-1)+(0.327150779323)
        ref[1,1,1,2]=o*(-0.773918767657)*x_ref[2]**(o-1)+(0.0695419192896)
        ref[2,0,0,0]=o*(0.861666752281)*x_ref[0]**(o-1)+(-0.369904662594)
        ref[2,0,0,1]=o*(-0.809447635327)*x_ref[1]**(o-1)+(0.314495362669)
        ref[2,0,0,2]=o*(0.672158503953)*x_ref[2]**(o-1)+(0.779729363207)
        ref[2,0,1,0]=o*(0.784251939885)*x_ref[0]**(o-1)+(-0.657779846806)
        ref[2,0,1,1]=o*(-0.694796642388)*x_ref[1]**(o-1)+(0.549290583812)
        ref[2,0,1,2]=o*(0.452712301324)*x_ref[2]**(o-1)+(0.0965361955733)
        ref[2,1,0,0]=o*(0.526035829224)*x_ref[0]**(o-1)+(-0.5285468198)
        ref[2,1,0,1]=o*(0.426231604453)*x_ref[1]**(o-1)+(-0.89107298075)
        ref[2,1,0,2]=o*(0.365149699988)*x_ref[2]**(o-1)+(0.87436690224)
        ref[2,1,1,0]=o*(-0.516537342331)*x_ref[0]**(o-1)+(-0.793927917391)
        ref[2,1,1,1]=o*(0.795130846095)*x_ref[1]**(o-1)+(0.782928826071)
        ref[2,1,1,2]=o*(-0.874002864146)*x_ref[2]**(o-1)+(0.465526006676)
        ref[3,0,0,0]=o*(-0.861917370502)*x_ref[0]**(o-1)+(-0.0636266572453)
        ref[3,0,0,1]=o*(-0.0140222042452)*x_ref[1]**(o-1)+(0.143759402339)
        ref[3,0,0,2]=o*(-0.247983260546)*x_ref[2]**(o-1)+(-0.596358912223)
        ref[3,0,1,0]=o*(0.120302367516)*x_ref[0]**(o-1)+(-0.701350969734)
        ref[3,0,1,1]=o*(-0.775100614072)*x_ref[1]**(o-1)+(-0.248023481318)
        ref[3,0,1,2]=o*(0.849345512002)*x_ref[2]**(o-1)+(0.542529986112)
        ref[3,1,0,0]=o*(-0.347014578244)*x_ref[0]**(o-1)+(0.611296458158)
        ref[3,1,0,1]=o*(-0.11214501466)*x_ref[1]**(o-1)+(0.516100884048)
        ref[3,1,0,2]=o*(-0.746366893941)*x_ref[2]**(o-1)+(-0.429950357634)
        ref[3,1,1,0]=o*(-0.154199420675)*x_ref[0]**(o-1)+(-0.533525033722)
        ref[3,1,1,1]=o*(-0.305802182284)*x_ref[1]**(o-1)+(0.0249510778534)
        ref[3,1,1,2]=o*(0.511832901845)*x_ref[2]**(o-1)+(0.865423961695)
        ref[4,0,0,0]=o*(0.270951033321)*x_ref[0]**(o-1)+(0.293153287275)
        ref[4,0,0,1]=o*(0.499008893086)*x_ref[1]**(o-1)+(0.401454188898)
        ref[4,0,0,2]=o*(-0.0659433492694)*x_ref[2]**(o-1)+(0.0822124121616)
        ref[4,0,1,0]=o*(-0.770956381696)*x_ref[0]**(o-1)+(0.962572964564)
        ref[4,0,1,1]=o*(-0.173527764809)*x_ref[1]**(o-1)+(-0.764724502533)
        ref[4,0,1,2]=o*(0.686939133626)*x_ref[2]**(o-1)+(0.589061927699)
        ref[4,1,0,0]=o*(0.856144070276)*x_ref[0]**(o-1)+(-0.970851254884)
        ref[4,1,0,1]=o*(-0.200238695073)*x_ref[1]**(o-1)+(0.961653891428)
        ref[4,1,0,2]=o*(-0.921215646185)*x_ref[2]**(o-1)+(-0.927582743302)
        ref[4,1,1,0]=o*(0.518201773728)*x_ref[0]**(o-1)+(0.403940302766)
        ref[4,1,1,1]=o*(-0.0333026733014)*x_ref[1]**(o-1)+(0.602796446678)
        ref[4,1,1,2]=o*(0.956622086807)*x_ref[2]**(o-1)+(0.445065392727)
        ref[5,0,0,0]=o*(0.847162899069)*x_ref[0]**(o-1)+(-0.11079738914)
        ref[5,0,0,1]=o*(0.815117104026)*x_ref[1]**(o-1)+(-0.0126064617312)
        ref[5,0,0,2]=o*(0.434649696776)*x_ref[2]**(o-1)+(-0.925434721648)
        ref[5,0,1,0]=o*(0.827068835777)*x_ref[0]**(o-1)+(0.267176529776)
        ref[5,0,1,1]=o*(-0.954551141359)*x_ref[1]**(o-1)+(-0.847951054931)
        ref[5,0,1,2]=o*(0.939843025231)*x_ref[2]**(o-1)+(-0.405674486263)
        ref[5,1,0,0]=o*(0.303863917444)*x_ref[0]**(o-1)+(0.33377358223)
        ref[5,1,0,1]=o*(0.160136648003)*x_ref[1]**(o-1)+(0.557340036308)
        ref[5,1,0,2]=o*(0.348362672822)*x_ref[2]**(o-1)+(-0.719624087847)
        ref[5,1,1,0]=o*(-0.61009211312)*x_ref[0]**(o-1)+(0.948776841387)
        ref[5,1,1,1]=o*(0.0348157372912)*x_ref[1]**(o-1)+(0.415543380004)
        ref[5,1,1,2]=o*(-0.104796276752)*x_ref[2]**(o-1)+(0.109481954783)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_Solution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.875674079407)*x[0]**o+(-0.108360259885)*x[0]+(0.875125430202)*x[1]**o+(0.552749512786)*x[1]
        ref[0]=o*(0.875674079407)*x_ref[0]**(o-1)+(-0.108360259885)
        ref[1]=o*(0.875125430202)*x_ref[1]**(o-1)+(0.552749512786)
      else:
        s=(0.0715819443551)*x[0]**o+(-0.619431747175)*x[0]+(-0.369266459921)*x[1]**o+(-0.105717390052)*x[1]+(-0.0432478189165)*x[2]**o+(0.725545839252)*x[2]
        ref[0]=o*(0.0715819443551)*x_ref[0]**(o-1)+(-0.619431747175)
        ref[1]=o*(-0.369266459921)*x_ref[1]**(o-1)+(-0.105717390052)
        ref[2]=o*(-0.0432478189165)*x_ref[2]**(o-1)+(0.725545839252)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_Solution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.626220139204)*x[0]**o+(0.729385849716)*x[0]+(-0.182106069322)*x[1]**o+(0.00285725500104)*x[1]
        s[1]=(-0.315419266082)*x[0]**o+(0.152806559812)*x[0]+(0.193255632909)*x[1]**o+(0.0415291248123)*x[1]
        ref[0,0]=o*(-0.626220139204)*x_ref[0]**(o-1)+(0.729385849716)
        ref[0,1]=o*(-0.182106069322)*x_ref[1]**(o-1)+(0.00285725500104)
        ref[1,0]=o*(-0.315419266082)*x_ref[0]**(o-1)+(0.152806559812)
        ref[1,1]=o*(0.193255632909)*x_ref[1]**(o-1)+(0.0415291248123)
      else:
        s[0]=(0.378833186935)*x[0]**o+(0.334764252787)*x[0]+(0.335440021844)*x[1]**o+(0.0418556980547)*x[1]+(-0.355548870692)*x[2]**o+(0.188469092541)*x[2]
        s[1]=(0.554815928082)*x[0]**o+(-0.245327666469)*x[0]+(-0.807150174696)*x[1]**o+(-0.0186528579571)*x[1]+(-0.644067730056)*x[2]**o+(-0.56385913942)*x[2]
        ref[0,0]=o*(0.378833186935)*x_ref[0]**(o-1)+(0.334764252787)
        ref[0,1]=o*(0.335440021844)*x_ref[1]**(o-1)+(0.0418556980547)
        ref[0,2]=o*(-0.355548870692)*x_ref[2]**(o-1)+(0.188469092541)
        ref[1,0]=o*(0.554815928082)*x_ref[0]**(o-1)+(-0.245327666469)
        ref[1,1]=o*(-0.807150174696)*x_ref[1]**(o-1)+(-0.0186528579571)
        ref[1,2]=o*(-0.644067730056)*x_ref[2]**(o-1)+(-0.56385913942)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_Solution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.169259870335)*x[0]**o+(-0.219393962977)*x[0]+(-0.377170386042)*x[1]**o+(-0.298077349995)*x[1]
        s[0,1]=(0.260540461556)*x[0]**o+(-0.00323982683281)*x[0]+(-0.238801075348)*x[1]**o+(-0.512630151171)*x[1]
        s[0,2]=(0.417777690389)*x[0]**o+(0.76705549948)*x[0]+(0.411966415931)*x[1]**o+(0.634271298164)*x[1]
        s[0,3]=(-0.514577227559)*x[0]**o+(-0.449963426549)*x[0]+(-0.400962023711)*x[1]**o+(-0.980313307287)*x[1]
        s[0,4]=(0.210149387291)*x[0]**o+(0.339419881455)*x[0]+(-0.122433722312)*x[1]**o+(-0.52322566908)*x[1]
        s[1,0]=(-0.456855849349)*x[0]**o+(-0.759902063915)*x[0]+(-0.743965861438)*x[1]**o+(-0.331770868496)*x[1]
        s[1,1]=(0.0972196005334)*x[0]**o+(-0.69511305009)*x[0]+(0.402041128045)*x[1]**o+(-0.924200483761)*x[1]
        s[1,2]=(-0.368298482402)*x[0]**o+(-0.351904909828)*x[0]+(-0.29784981961)*x[1]**o+(-0.0850625471026)*x[1]
        s[1,3]=(-0.79959026688)*x[0]**o+(-0.143869612796)*x[0]+(0.545276895212)*x[1]**o+(0.0277273177848)*x[1]
        s[1,4]=(-0.786210582915)*x[0]**o+(0.17248521577)*x[0]+(0.682847252483)*x[1]**o+(0.975158490563)*x[1]
        s[2,0]=(-0.00180696139571)*x[0]**o+(0.0644626145328)*x[0]+(-0.332784099857)*x[1]**o+(-0.62718290936)*x[1]
        s[2,1]=(0.529388118928)*x[0]**o+(0.679837615948)*x[0]+(-0.800039234078)*x[1]**o+(0.762585600198)*x[1]
        s[2,2]=(0.202108406753)*x[0]**o+(0.602440423874)*x[0]+(-0.544751041557)*x[1]**o+(-0.479387390671)*x[1]
        s[2,3]=(0.339973882877)*x[0]**o+(0.465854524429)*x[0]+(0.318692368851)*x[1]**o+(0.869234081961)*x[1]
        s[2,4]=(0.337044733833)*x[0]**o+(-0.401620568253)*x[0]+(-0.0302231005328)*x[1]**o+(0.218919539213)*x[1]
        s[3,0]=(-0.961845777661)*x[0]**o+(-0.509480342678)*x[0]+(0.970831236524)*x[1]**o+(-0.234720318489)*x[1]
        s[3,1]=(0.266147738115)*x[0]**o+(-0.0146288256163)*x[0]+(0.38625117249)*x[1]**o+(-0.449526886565)*x[1]
        s[3,2]=(-0.0829643094057)*x[0]**o+(-0.31837766369)*x[0]+(0.868932473361)*x[1]**o+(0.0753845710061)*x[1]
        s[3,3]=(-0.311549181378)*x[0]**o+(-0.0704226477679)*x[0]+(-0.80469940492)*x[1]**o+(0.17810271607)*x[1]
        s[3,4]=(0.496421938706)*x[0]**o+(-0.94316161479)*x[0]+(-0.759730393246)*x[1]**o+(0.857927191642)*x[1]
        ref[0,0,0]=o*(-0.169259870335)*x_ref[0]**(o-1)+(-0.219393962977)
        ref[0,0,1]=o*(-0.377170386042)*x_ref[1]**(o-1)+(-0.298077349995)
        ref[0,1,0]=o*(0.260540461556)*x_ref[0]**(o-1)+(-0.00323982683281)
        ref[0,1,1]=o*(-0.238801075348)*x_ref[1]**(o-1)+(-0.512630151171)
        ref[0,2,0]=o*(0.417777690389)*x_ref[0]**(o-1)+(0.76705549948)
        ref[0,2,1]=o*(0.411966415931)*x_ref[1]**(o-1)+(0.634271298164)
        ref[0,3,0]=o*(-0.514577227559)*x_ref[0]**(o-1)+(-0.449963426549)
        ref[0,3,1]=o*(-0.400962023711)*x_ref[1]**(o-1)+(-0.980313307287)
        ref[0,4,0]=o*(0.210149387291)*x_ref[0]**(o-1)+(0.339419881455)
        ref[0,4,1]=o*(-0.122433722312)*x_ref[1]**(o-1)+(-0.52322566908)
        ref[1,0,0]=o*(-0.456855849349)*x_ref[0]**(o-1)+(-0.759902063915)
        ref[1,0,1]=o*(-0.743965861438)*x_ref[1]**(o-1)+(-0.331770868496)
        ref[1,1,0]=o*(0.0972196005334)*x_ref[0]**(o-1)+(-0.69511305009)
        ref[1,1,1]=o*(0.402041128045)*x_ref[1]**(o-1)+(-0.924200483761)
        ref[1,2,0]=o*(-0.368298482402)*x_ref[0]**(o-1)+(-0.351904909828)
        ref[1,2,1]=o*(-0.29784981961)*x_ref[1]**(o-1)+(-0.0850625471026)
        ref[1,3,0]=o*(-0.79959026688)*x_ref[0]**(o-1)+(-0.143869612796)
        ref[1,3,1]=o*(0.545276895212)*x_ref[1]**(o-1)+(0.0277273177848)
        ref[1,4,0]=o*(-0.786210582915)*x_ref[0]**(o-1)+(0.17248521577)
        ref[1,4,1]=o*(0.682847252483)*x_ref[1]**(o-1)+(0.975158490563)
        ref[2,0,0]=o*(-0.00180696139571)*x_ref[0]**(o-1)+(0.0644626145328)
        ref[2,0,1]=o*(-0.332784099857)*x_ref[1]**(o-1)+(-0.62718290936)
        ref[2,1,0]=o*(0.529388118928)*x_ref[0]**(o-1)+(0.679837615948)
        ref[2,1,1]=o*(-0.800039234078)*x_ref[1]**(o-1)+(0.762585600198)
        ref[2,2,0]=o*(0.202108406753)*x_ref[0]**(o-1)+(0.602440423874)
        ref[2,2,1]=o*(-0.544751041557)*x_ref[1]**(o-1)+(-0.479387390671)
        ref[2,3,0]=o*(0.339973882877)*x_ref[0]**(o-1)+(0.465854524429)
        ref[2,3,1]=o*(0.318692368851)*x_ref[1]**(o-1)+(0.869234081961)
        ref[2,4,0]=o*(0.337044733833)*x_ref[0]**(o-1)+(-0.401620568253)
        ref[2,4,1]=o*(-0.0302231005328)*x_ref[1]**(o-1)+(0.218919539213)
        ref[3,0,0]=o*(-0.961845777661)*x_ref[0]**(o-1)+(-0.509480342678)
        ref[3,0,1]=o*(0.970831236524)*x_ref[1]**(o-1)+(-0.234720318489)
        ref[3,1,0]=o*(0.266147738115)*x_ref[0]**(o-1)+(-0.0146288256163)
        ref[3,1,1]=o*(0.38625117249)*x_ref[1]**(o-1)+(-0.449526886565)
        ref[3,2,0]=o*(-0.0829643094057)*x_ref[0]**(o-1)+(-0.31837766369)
        ref[3,2,1]=o*(0.868932473361)*x_ref[1]**(o-1)+(0.0753845710061)
        ref[3,3,0]=o*(-0.311549181378)*x_ref[0]**(o-1)+(-0.0704226477679)
        ref[3,3,1]=o*(-0.80469940492)*x_ref[1]**(o-1)+(0.17810271607)
        ref[3,4,0]=o*(0.496421938706)*x_ref[0]**(o-1)+(-0.94316161479)
        ref[3,4,1]=o*(-0.759730393246)*x_ref[1]**(o-1)+(0.857927191642)
      else:
        s[0,0]=(0.323269202132)*x[0]**o+(-0.0941675978056)*x[0]+(0.573540368658)*x[1]**o+(-0.476661689561)*x[1]+(-0.332358000871)*x[2]**o+(-0.817017744339)*x[2]
        s[0,1]=(0.116142610738)*x[0]**o+(0.0212217636606)*x[0]+(0.494731157541)*x[1]**o+(0.295191373453)*x[1]+(0.0528301662295)*x[2]**o+(-0.444651733146)*x[2]
        s[0,2]=(0.456064446338)*x[0]**o+(-0.69428330869)*x[0]+(0.677929037713)*x[1]**o+(-0.320468300843)*x[1]+(-0.87760608927)*x[2]**o+(-0.659457899497)*x[2]
        s[0,3]=(-0.189428313133)*x[0]**o+(-0.799817301777)*x[0]+(0.100831997136)*x[1]**o+(-0.113782495668)*x[1]+(0.455358915627)*x[2]**o+(-0.708374606976)*x[2]
        s[0,4]=(-0.118907620954)*x[0]**o+(-0.353855451309)*x[0]+(-0.773766114927)*x[1]**o+(-0.845133073805)*x[1]+(0.803289878128)*x[2]**o+(-0.852839704894)*x[2]
        s[1,0]=(-0.887023265539)*x[0]**o+(-0.792304610398)*x[0]+(0.0362402407711)*x[1]**o+(-0.260378118021)*x[1]+(0.142912579168)*x[2]**o+(0.544143870337)*x[2]
        s[1,1]=(0.805678954301)*x[0]**o+(0.701112598554)*x[0]+(0.922879760039)*x[1]**o+(-0.219024852502)*x[1]+(0.365922209323)*x[2]**o+(-0.420665475533)*x[2]
        s[1,2]=(0.832254579623)*x[0]**o+(-0.413462011593)*x[0]+(0.0852114474998)*x[1]**o+(-0.0932078507511)*x[1]+(-0.248076148787)*x[2]**o+(-0.428017096895)*x[2]
        s[1,3]=(-0.4967053049)*x[0]**o+(-0.50471234025)*x[0]+(-0.117548830517)*x[1]**o+(0.81561414915)*x[1]+(0.61128237476)*x[2]**o+(0.695201350291)*x[2]
        s[1,4]=(0.0281073079277)*x[0]**o+(0.62841659852)*x[0]+(0.685439649725)*x[1]**o+(-0.093589324186)*x[1]+(-0.848012915274)*x[2]**o+(-0.800065648129)*x[2]
        s[2,0]=(-0.35245094908)*x[0]**o+(0.40394009353)*x[0]+(-0.420215110026)*x[1]**o+(0.488912100088)*x[1]+(0.679732915429)*x[2]**o+(-0.362263037628)*x[2]
        s[2,1]=(-0.515442192308)*x[0]**o+(0.634797288073)*x[0]+(0.461169879242)*x[1]**o+(-0.51743910056)*x[1]+(-0.227585094451)*x[2]**o+(0.00407235369734)*x[2]
        s[2,2]=(-0.442867477277)*x[0]**o+(-0.212399902385)*x[0]+(-0.768565150664)*x[1]**o+(-0.398107108767)*x[1]+(-0.23802726279)*x[2]**o+(-0.838176628616)*x[2]
        s[2,3]=(0.491473254552)*x[0]**o+(0.00763471198539)*x[0]+(0.255444615913)*x[1]**o+(-0.655565675326)*x[1]+(0.324558324765)*x[2]**o+(-0.224946422831)*x[2]
        s[2,4]=(0.880395740489)*x[0]**o+(0.940871214043)*x[0]+(0.708978280722)*x[1]**o+(0.448470774528)*x[1]+(0.37065402316)*x[2]**o+(0.761394343858)*x[2]
        s[3,0]=(-0.455887951889)*x[0]**o+(0.00570480201782)*x[0]+(0.413555793704)*x[1]**o+(-0.313434157216)*x[1]+(0.339744130043)*x[2]**o+(-0.105392254622)*x[2]
        s[3,1]=(0.0528573842143)*x[0]**o+(-0.11565565)*x[0]+(-0.0443876403016)*x[1]**o+(-0.716723173535)*x[1]+(0.100057602497)*x[2]**o+(-0.891795165388)*x[2]
        s[3,2]=(-0.317795319526)*x[0]**o+(0.210753288047)*x[0]+(-0.666306164291)*x[1]**o+(-0.561602279575)*x[1]+(0.55557323269)*x[2]**o+(-0.479408955582)*x[2]
        s[3,3]=(0.49345832209)*x[0]**o+(-0.0684540951119)*x[0]+(0.69792572139)*x[1]**o+(-0.941053746122)*x[1]+(0.127906327934)*x[2]**o+(0.97752169437)*x[2]
        s[3,4]=(-0.553212311288)*x[0]**o+(0.718429728418)*x[0]+(0.179731047542)*x[1]**o+(0.169591114318)*x[1]+(0.402679892192)*x[2]**o+(0.0690728029503)*x[2]
        ref[0,0,0]=o*(0.323269202132)*x_ref[0]**(o-1)+(-0.0941675978056)
        ref[0,0,1]=o*(0.573540368658)*x_ref[1]**(o-1)+(-0.476661689561)
        ref[0,0,2]=o*(-0.332358000871)*x_ref[2]**(o-1)+(-0.817017744339)
        ref[0,1,0]=o*(0.116142610738)*x_ref[0]**(o-1)+(0.0212217636606)
        ref[0,1,1]=o*(0.494731157541)*x_ref[1]**(o-1)+(0.295191373453)
        ref[0,1,2]=o*(0.0528301662295)*x_ref[2]**(o-1)+(-0.444651733146)
        ref[0,2,0]=o*(0.456064446338)*x_ref[0]**(o-1)+(-0.69428330869)
        ref[0,2,1]=o*(0.677929037713)*x_ref[1]**(o-1)+(-0.320468300843)
        ref[0,2,2]=o*(-0.87760608927)*x_ref[2]**(o-1)+(-0.659457899497)
        ref[0,3,0]=o*(-0.189428313133)*x_ref[0]**(o-1)+(-0.799817301777)
        ref[0,3,1]=o*(0.100831997136)*x_ref[1]**(o-1)+(-0.113782495668)
        ref[0,3,2]=o*(0.455358915627)*x_ref[2]**(o-1)+(-0.708374606976)
        ref[0,4,0]=o*(-0.118907620954)*x_ref[0]**(o-1)+(-0.353855451309)
        ref[0,4,1]=o*(-0.773766114927)*x_ref[1]**(o-1)+(-0.845133073805)
        ref[0,4,2]=o*(0.803289878128)*x_ref[2]**(o-1)+(-0.852839704894)
        ref[1,0,0]=o*(-0.887023265539)*x_ref[0]**(o-1)+(-0.792304610398)
        ref[1,0,1]=o*(0.0362402407711)*x_ref[1]**(o-1)+(-0.260378118021)
        ref[1,0,2]=o*(0.142912579168)*x_ref[2]**(o-1)+(0.544143870337)
        ref[1,1,0]=o*(0.805678954301)*x_ref[0]**(o-1)+(0.701112598554)
        ref[1,1,1]=o*(0.922879760039)*x_ref[1]**(o-1)+(-0.219024852502)
        ref[1,1,2]=o*(0.365922209323)*x_ref[2]**(o-1)+(-0.420665475533)
        ref[1,2,0]=o*(0.832254579623)*x_ref[0]**(o-1)+(-0.413462011593)
        ref[1,2,1]=o*(0.0852114474998)*x_ref[1]**(o-1)+(-0.0932078507511)
        ref[1,2,2]=o*(-0.248076148787)*x_ref[2]**(o-1)+(-0.428017096895)
        ref[1,3,0]=o*(-0.4967053049)*x_ref[0]**(o-1)+(-0.50471234025)
        ref[1,3,1]=o*(-0.117548830517)*x_ref[1]**(o-1)+(0.81561414915)
        ref[1,3,2]=o*(0.61128237476)*x_ref[2]**(o-1)+(0.695201350291)
        ref[1,4,0]=o*(0.0281073079277)*x_ref[0]**(o-1)+(0.62841659852)
        ref[1,4,1]=o*(0.685439649725)*x_ref[1]**(o-1)+(-0.093589324186)
        ref[1,4,2]=o*(-0.848012915274)*x_ref[2]**(o-1)+(-0.800065648129)
        ref[2,0,0]=o*(-0.35245094908)*x_ref[0]**(o-1)+(0.40394009353)
        ref[2,0,1]=o*(-0.420215110026)*x_ref[1]**(o-1)+(0.488912100088)
        ref[2,0,2]=o*(0.679732915429)*x_ref[2]**(o-1)+(-0.362263037628)
        ref[2,1,0]=o*(-0.515442192308)*x_ref[0]**(o-1)+(0.634797288073)
        ref[2,1,1]=o*(0.461169879242)*x_ref[1]**(o-1)+(-0.51743910056)
        ref[2,1,2]=o*(-0.227585094451)*x_ref[2]**(o-1)+(0.00407235369734)
        ref[2,2,0]=o*(-0.442867477277)*x_ref[0]**(o-1)+(-0.212399902385)
        ref[2,2,1]=o*(-0.768565150664)*x_ref[1]**(o-1)+(-0.398107108767)
        ref[2,2,2]=o*(-0.23802726279)*x_ref[2]**(o-1)+(-0.838176628616)
        ref[2,3,0]=o*(0.491473254552)*x_ref[0]**(o-1)+(0.00763471198539)
        ref[2,3,1]=o*(0.255444615913)*x_ref[1]**(o-1)+(-0.655565675326)
        ref[2,3,2]=o*(0.324558324765)*x_ref[2]**(o-1)+(-0.224946422831)
        ref[2,4,0]=o*(0.880395740489)*x_ref[0]**(o-1)+(0.940871214043)
        ref[2,4,1]=o*(0.708978280722)*x_ref[1]**(o-1)+(0.448470774528)
        ref[2,4,2]=o*(0.37065402316)*x_ref[2]**(o-1)+(0.761394343858)
        ref[3,0,0]=o*(-0.455887951889)*x_ref[0]**(o-1)+(0.00570480201782)
        ref[3,0,1]=o*(0.413555793704)*x_ref[1]**(o-1)+(-0.313434157216)
        ref[3,0,2]=o*(0.339744130043)*x_ref[2]**(o-1)+(-0.105392254622)
        ref[3,1,0]=o*(0.0528573842143)*x_ref[0]**(o-1)+(-0.11565565)
        ref[3,1,1]=o*(-0.0443876403016)*x_ref[1]**(o-1)+(-0.716723173535)
        ref[3,1,2]=o*(0.100057602497)*x_ref[2]**(o-1)+(-0.891795165388)
        ref[3,2,0]=o*(-0.317795319526)*x_ref[0]**(o-1)+(0.210753288047)
        ref[3,2,1]=o*(-0.666306164291)*x_ref[1]**(o-1)+(-0.561602279575)
        ref[3,2,2]=o*(0.55557323269)*x_ref[2]**(o-1)+(-0.479408955582)
        ref[3,3,0]=o*(0.49345832209)*x_ref[0]**(o-1)+(-0.0684540951119)
        ref[3,3,1]=o*(0.69792572139)*x_ref[1]**(o-1)+(-0.941053746122)
        ref[3,3,2]=o*(0.127906327934)*x_ref[2]**(o-1)+(0.97752169437)
        ref[3,4,0]=o*(-0.553212311288)*x_ref[0]**(o-1)+(0.718429728418)
        ref[3,4,1]=o*(0.179731047542)*x_ref[1]**(o-1)+(0.169591114318)
        ref[3,4,2]=o*(0.402679892192)*x_ref[2]**(o-1)+(0.0690728029503)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_Solution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(0.64178693834)*x[0]**o+(0.436828658021)*x[0]+(0.525622010733)*x[1]**o+(-0.0947845294558)*x[1]
        s[0,0,1]=(0.960929069825)*x[0]**o+(-0.0925500837249)*x[0]+(0.516628691912)*x[1]**o+(-0.713433036463)*x[1]
        s[0,1,0]=(-0.130010775216)*x[0]**o+(-0.569868353102)*x[0]+(-0.834642447971)*x[1]**o+(-0.262937552972)*x[1]
        s[0,1,1]=(-0.720926391535)*x[0]**o+(0.803576593732)*x[0]+(-0.613803785285)*x[1]**o+(0.713012726555)*x[1]
        s[1,0,0]=(-0.960850249593)*x[0]**o+(-0.891033054973)*x[0]+(0.750989994686)*x[1]**o+(0.983245329088)*x[1]
        s[1,0,1]=(-0.383882505108)*x[0]**o+(0.980533529745)*x[0]+(0.891761213135)*x[1]**o+(0.758837665244)*x[1]
        s[1,1,0]=(0.933866632845)*x[0]**o+(0.618830180361)*x[0]+(0.083925621148)*x[1]**o+(-0.681925439524)*x[1]
        s[1,1,1]=(0.344074931746)*x[0]**o+(-0.0411811037823)*x[0]+(0.582962923539)*x[1]**o+(0.242279633347)*x[1]
        s[2,0,0]=(0.0949732595833)*x[0]**o+(0.947484666339)*x[0]+(-0.856377492747)*x[1]**o+(0.0911692318532)*x[1]
        s[2,0,1]=(-0.294511041595)*x[0]**o+(0.252889321608)*x[0]+(-0.204790616562)*x[1]**o+(0.318333561489)*x[1]
        s[2,1,0]=(0.252492478572)*x[0]**o+(-0.594109481282)*x[0]+(-0.0260393414988)*x[1]**o+(0.563948256582)*x[1]
        s[2,1,1]=(-0.714450749502)*x[0]**o+(0.302407929025)*x[0]+(0.147817927373)*x[1]**o+(-0.830954899815)*x[1]
        s[3,0,0]=(0.56545292458)*x[0]**o+(0.486566855088)*x[0]+(0.990278375714)*x[1]**o+(0.229104000835)*x[1]
        s[3,0,1]=(-0.734062795067)*x[0]**o+(-0.191169476406)*x[0]+(-0.251242465039)*x[1]**o+(0.390333958806)*x[1]
        s[3,1,0]=(-0.337774363178)*x[0]**o+(0.700342293089)*x[0]+(-0.836836760567)*x[1]**o+(-0.0429509469196)*x[1]
        s[3,1,1]=(0.8888799528)*x[0]**o+(0.225714398328)*x[0]+(0.622017922629)*x[1]**o+(-0.614413552831)*x[1]
        s[4,0,0]=(0.882570398452)*x[0]**o+(-0.0550944535693)*x[0]+(-0.674688995642)*x[1]**o+(0.489860414323)*x[1]
        s[4,0,1]=(0.192190815605)*x[0]**o+(0.615603974895)*x[0]+(-0.713940642233)*x[1]**o+(0.692854992944)*x[1]
        s[4,1,0]=(0.383266832431)*x[0]**o+(0.294686738948)*x[0]+(0.119658392209)*x[1]**o+(-0.550041082523)*x[1]
        s[4,1,1]=(-0.0639894922603)*x[0]**o+(0.0322835841591)*x[0]+(-0.148146549294)*x[1]**o+(-0.472048255747)*x[1]
        s[5,0,0]=(-0.596603356022)*x[0]**o+(0.0764697562663)*x[0]+(0.23065037103)*x[1]**o+(-0.993102200263)*x[1]
        s[5,0,1]=(-0.268998655952)*x[0]**o+(0.911781569559)*x[0]+(-0.424230615868)*x[1]**o+(-0.196872693015)*x[1]
        s[5,1,0]=(-0.145090787501)*x[0]**o+(0.694829120617)*x[0]+(-0.972729923357)*x[1]**o+(-0.737541664159)*x[1]
        s[5,1,1]=(-0.196228029528)*x[0]**o+(-0.385307212434)*x[0]+(-0.793635672028)*x[1]**o+(-0.547670068083)*x[1]
        ref[0,0,0,0]=o*(0.64178693834)*x_ref[0]**(o-1)+(0.436828658021)
        ref[0,0,0,1]=o*(0.525622010733)*x_ref[1]**(o-1)+(-0.0947845294558)
        ref[0,0,1,0]=o*(0.960929069825)*x_ref[0]**(o-1)+(-0.0925500837249)
        ref[0,0,1,1]=o*(0.516628691912)*x_ref[1]**(o-1)+(-0.713433036463)
        ref[0,1,0,0]=o*(-0.130010775216)*x_ref[0]**(o-1)+(-0.569868353102)
        ref[0,1,0,1]=o*(-0.834642447971)*x_ref[1]**(o-1)+(-0.262937552972)
        ref[0,1,1,0]=o*(-0.720926391535)*x_ref[0]**(o-1)+(0.803576593732)
        ref[0,1,1,1]=o*(-0.613803785285)*x_ref[1]**(o-1)+(0.713012726555)
        ref[1,0,0,0]=o*(-0.960850249593)*x_ref[0]**(o-1)+(-0.891033054973)
        ref[1,0,0,1]=o*(0.750989994686)*x_ref[1]**(o-1)+(0.983245329088)
        ref[1,0,1,0]=o*(-0.383882505108)*x_ref[0]**(o-1)+(0.980533529745)
        ref[1,0,1,1]=o*(0.891761213135)*x_ref[1]**(o-1)+(0.758837665244)
        ref[1,1,0,0]=o*(0.933866632845)*x_ref[0]**(o-1)+(0.618830180361)
        ref[1,1,0,1]=o*(0.083925621148)*x_ref[1]**(o-1)+(-0.681925439524)
        ref[1,1,1,0]=o*(0.344074931746)*x_ref[0]**(o-1)+(-0.0411811037823)
        ref[1,1,1,1]=o*(0.582962923539)*x_ref[1]**(o-1)+(0.242279633347)
        ref[2,0,0,0]=o*(0.0949732595833)*x_ref[0]**(o-1)+(0.947484666339)
        ref[2,0,0,1]=o*(-0.856377492747)*x_ref[1]**(o-1)+(0.0911692318532)
        ref[2,0,1,0]=o*(-0.294511041595)*x_ref[0]**(o-1)+(0.252889321608)
        ref[2,0,1,1]=o*(-0.204790616562)*x_ref[1]**(o-1)+(0.318333561489)
        ref[2,1,0,0]=o*(0.252492478572)*x_ref[0]**(o-1)+(-0.594109481282)
        ref[2,1,0,1]=o*(-0.0260393414988)*x_ref[1]**(o-1)+(0.563948256582)
        ref[2,1,1,0]=o*(-0.714450749502)*x_ref[0]**(o-1)+(0.302407929025)
        ref[2,1,1,1]=o*(0.147817927373)*x_ref[1]**(o-1)+(-0.830954899815)
        ref[3,0,0,0]=o*(0.56545292458)*x_ref[0]**(o-1)+(0.486566855088)
        ref[3,0,0,1]=o*(0.990278375714)*x_ref[1]**(o-1)+(0.229104000835)
        ref[3,0,1,0]=o*(-0.734062795067)*x_ref[0]**(o-1)+(-0.191169476406)
        ref[3,0,1,1]=o*(-0.251242465039)*x_ref[1]**(o-1)+(0.390333958806)
        ref[3,1,0,0]=o*(-0.337774363178)*x_ref[0]**(o-1)+(0.700342293089)
        ref[3,1,0,1]=o*(-0.836836760567)*x_ref[1]**(o-1)+(-0.0429509469196)
        ref[3,1,1,0]=o*(0.8888799528)*x_ref[0]**(o-1)+(0.225714398328)
        ref[3,1,1,1]=o*(0.622017922629)*x_ref[1]**(o-1)+(-0.614413552831)
        ref[4,0,0,0]=o*(0.882570398452)*x_ref[0]**(o-1)+(-0.0550944535693)
        ref[4,0,0,1]=o*(-0.674688995642)*x_ref[1]**(o-1)+(0.489860414323)
        ref[4,0,1,0]=o*(0.192190815605)*x_ref[0]**(o-1)+(0.615603974895)
        ref[4,0,1,1]=o*(-0.713940642233)*x_ref[1]**(o-1)+(0.692854992944)
        ref[4,1,0,0]=o*(0.383266832431)*x_ref[0]**(o-1)+(0.294686738948)
        ref[4,1,0,1]=o*(0.119658392209)*x_ref[1]**(o-1)+(-0.550041082523)
        ref[4,1,1,0]=o*(-0.0639894922603)*x_ref[0]**(o-1)+(0.0322835841591)
        ref[4,1,1,1]=o*(-0.148146549294)*x_ref[1]**(o-1)+(-0.472048255747)
        ref[5,0,0,0]=o*(-0.596603356022)*x_ref[0]**(o-1)+(0.0764697562663)
        ref[5,0,0,1]=o*(0.23065037103)*x_ref[1]**(o-1)+(-0.993102200263)
        ref[5,0,1,0]=o*(-0.268998655952)*x_ref[0]**(o-1)+(0.911781569559)
        ref[5,0,1,1]=o*(-0.424230615868)*x_ref[1]**(o-1)+(-0.196872693015)
        ref[5,1,0,0]=o*(-0.145090787501)*x_ref[0]**(o-1)+(0.694829120617)
        ref[5,1,0,1]=o*(-0.972729923357)*x_ref[1]**(o-1)+(-0.737541664159)
        ref[5,1,1,0]=o*(-0.196228029528)*x_ref[0]**(o-1)+(-0.385307212434)
        ref[5,1,1,1]=o*(-0.793635672028)*x_ref[1]**(o-1)+(-0.547670068083)
      else:
        s[0,0,0]=(0.130537344403)*x[0]**o+(-0.716585435036)*x[0]+(0.885628707489)*x[1]**o+(0.359657295083)*x[1]+(0.849414417001)*x[2]**o+(0.417004569061)*x[2]
        s[0,0,1]=(0.582291844761)*x[0]**o+(0.515724457456)*x[0]+(0.768535647866)*x[1]**o+(0.34108474273)*x[1]+(-0.620508267743)*x[2]**o+(-0.342013446761)*x[2]
        s[0,1,0]=(-0.531235123931)*x[0]**o+(-0.765634705075)*x[0]+(0.228864105339)*x[1]**o+(-0.491530876451)*x[1]+(0.521065315283)*x[2]**o+(-0.951623087256)*x[2]
        s[0,1,1]=(0.0716827639786)*x[0]**o+(-0.16499705145)*x[0]+(-0.0190243436099)*x[1]**o+(-0.926607070761)*x[1]+(-0.55142315912)*x[2]**o+(-0.0794842630428)*x[2]
        s[1,0,0]=(0.182139977684)*x[0]**o+(-0.0362484111368)*x[0]+(-0.752143193872)*x[1]**o+(-0.316455062654)*x[1]+(0.269718723768)*x[2]**o+(-0.313902463376)*x[2]
        s[1,0,1]=(0.743368204514)*x[0]**o+(-0.822345231625)*x[0]+(0.814851367707)*x[1]**o+(0.702205919957)*x[1]+(-0.0503255941307)*x[2]**o+(-0.0318030472244)*x[2]
        s[1,1,0]=(0.022342448883)*x[0]**o+(-0.352009543909)*x[0]+(0.5204277493)*x[1]**o+(-0.741699265227)*x[1]+(0.00608894949938)*x[2]**o+(0.965930515392)*x[2]
        s[1,1,1]=(-0.0920262487111)*x[0]**o+(0.249857266149)*x[0]+(0.916353890515)*x[1]**o+(-0.0594616510094)*x[1]+(-0.402620394698)*x[2]**o+(-0.00864935399915)*x[2]
        s[2,0,0]=(-0.944378332174)*x[0]**o+(0.501614469906)*x[0]+(0.829832603587)*x[1]**o+(0.4202031628)*x[1]+(0.886338738184)*x[2]**o+(0.811714471885)*x[2]
        s[2,0,1]=(0.0413505829692)*x[0]**o+(-0.59249386093)*x[0]+(0.478645406371)*x[1]**o+(0.221389337839)*x[1]+(0.18016770891)*x[2]**o+(-0.783010309878)*x[2]
        s[2,1,0]=(0.641325174447)*x[0]**o+(0.859809801095)*x[0]+(0.238917443521)*x[1]**o+(0.169387212912)*x[1]+(0.118386087343)*x[2]**o+(0.7091562331)*x[2]
        s[2,1,1]=(0.683767548268)*x[0]**o+(-0.488504930782)*x[0]+(0.421423574907)*x[1]**o+(0.677076626099)*x[1]+(-0.0955208633306)*x[2]**o+(0.545750792826)*x[2]
        s[3,0,0]=(-0.925067175669)*x[0]**o+(0.550204660551)*x[0]+(0.435686889479)*x[1]**o+(-0.171725370784)*x[1]+(0.182097753276)*x[2]**o+(0.493662375098)*x[2]
        s[3,0,1]=(-0.719237805296)*x[0]**o+(0.669817079155)*x[0]+(0.660680613106)*x[1]**o+(-0.729927483007)*x[1]+(-0.162704346009)*x[2]**o+(-0.559206291015)*x[2]
        s[3,1,0]=(0.129735268583)*x[0]**o+(-0.411853842222)*x[0]+(0.045438376206)*x[1]**o+(0.327804746872)*x[1]+(-0.639433216582)*x[2]**o+(0.451106136717)*x[2]
        s[3,1,1]=(-0.811557092293)*x[0]**o+(-0.818758274715)*x[0]+(-0.712815291675)*x[1]**o+(0.729038274951)*x[1]+(-0.826837009476)*x[2]**o+(0.340025404977)*x[2]
        s[4,0,0]=(-0.507284842259)*x[0]**o+(0.457554461786)*x[0]+(0.419345933038)*x[1]**o+(-0.7773529718)*x[1]+(-0.626475804952)*x[2]**o+(-0.784196320861)*x[2]
        s[4,0,1]=(-0.0340250322306)*x[0]**o+(-0.0936499769437)*x[0]+(-0.777584776075)*x[1]**o+(-0.435876632622)*x[1]+(0.999336374444)*x[2]**o+(-0.247149214414)*x[2]
        s[4,1,0]=(0.21620967013)*x[0]**o+(0.213039866961)*x[0]+(0.83049354428)*x[1]**o+(0.651430926943)*x[1]+(0.201648695549)*x[2]**o+(-0.609802970877)*x[2]
        s[4,1,1]=(0.842001670092)*x[0]**o+(-0.868487783326)*x[0]+(-0.288512200338)*x[1]**o+(-0.654831485218)*x[1]+(-0.111859725689)*x[2]**o+(0.728195130453)*x[2]
        s[5,0,0]=(0.473856201092)*x[0]**o+(0.377649949188)*x[0]+(0.511638340284)*x[1]**o+(-0.150021940198)*x[1]+(-0.801655915671)*x[2]**o+(-0.603617347175)*x[2]
        s[5,0,1]=(0.945961549464)*x[0]**o+(0.40151064288)*x[0]+(0.151039821252)*x[1]**o+(0.977148741848)*x[1]+(0.0729651283448)*x[2]**o+(0.365625847237)*x[2]
        s[5,1,0]=(0.613037024504)*x[0]**o+(-0.743080801009)*x[0]+(-0.518736037968)*x[1]**o+(0.240273029617)*x[1]+(0.437464780338)*x[2]**o+(0.897940669321)*x[2]
        s[5,1,1]=(-0.149245273038)*x[0]**o+(-0.387037171901)*x[0]+(0.0606843165198)*x[1]**o+(-0.713188961184)*x[1]+(0.622672840108)*x[2]**o+(-0.579757019852)*x[2]
        ref[0,0,0,0]=o*(0.130537344403)*x_ref[0]**(o-1)+(-0.716585435036)
        ref[0,0,0,1]=o*(0.885628707489)*x_ref[1]**(o-1)+(0.359657295083)
        ref[0,0,0,2]=o*(0.849414417001)*x_ref[2]**(o-1)+(0.417004569061)
        ref[0,0,1,0]=o*(0.582291844761)*x_ref[0]**(o-1)+(0.515724457456)
        ref[0,0,1,1]=o*(0.768535647866)*x_ref[1]**(o-1)+(0.34108474273)
        ref[0,0,1,2]=o*(-0.620508267743)*x_ref[2]**(o-1)+(-0.342013446761)
        ref[0,1,0,0]=o*(-0.531235123931)*x_ref[0]**(o-1)+(-0.765634705075)
        ref[0,1,0,1]=o*(0.228864105339)*x_ref[1]**(o-1)+(-0.491530876451)
        ref[0,1,0,2]=o*(0.521065315283)*x_ref[2]**(o-1)+(-0.951623087256)
        ref[0,1,1,0]=o*(0.0716827639786)*x_ref[0]**(o-1)+(-0.16499705145)
        ref[0,1,1,1]=o*(-0.0190243436099)*x_ref[1]**(o-1)+(-0.926607070761)
        ref[0,1,1,2]=o*(-0.55142315912)*x_ref[2]**(o-1)+(-0.0794842630428)
        ref[1,0,0,0]=o*(0.182139977684)*x_ref[0]**(o-1)+(-0.0362484111368)
        ref[1,0,0,1]=o*(-0.752143193872)*x_ref[1]**(o-1)+(-0.316455062654)
        ref[1,0,0,2]=o*(0.269718723768)*x_ref[2]**(o-1)+(-0.313902463376)
        ref[1,0,1,0]=o*(0.743368204514)*x_ref[0]**(o-1)+(-0.822345231625)
        ref[1,0,1,1]=o*(0.814851367707)*x_ref[1]**(o-1)+(0.702205919957)
        ref[1,0,1,2]=o*(-0.0503255941307)*x_ref[2]**(o-1)+(-0.0318030472244)
        ref[1,1,0,0]=o*(0.022342448883)*x_ref[0]**(o-1)+(-0.352009543909)
        ref[1,1,0,1]=o*(0.5204277493)*x_ref[1]**(o-1)+(-0.741699265227)
        ref[1,1,0,2]=o*(0.00608894949938)*x_ref[2]**(o-1)+(0.965930515392)
        ref[1,1,1,0]=o*(-0.0920262487111)*x_ref[0]**(o-1)+(0.249857266149)
        ref[1,1,1,1]=o*(0.916353890515)*x_ref[1]**(o-1)+(-0.0594616510094)
        ref[1,1,1,2]=o*(-0.402620394698)*x_ref[2]**(o-1)+(-0.00864935399915)
        ref[2,0,0,0]=o*(-0.944378332174)*x_ref[0]**(o-1)+(0.501614469906)
        ref[2,0,0,1]=o*(0.829832603587)*x_ref[1]**(o-1)+(0.4202031628)
        ref[2,0,0,2]=o*(0.886338738184)*x_ref[2]**(o-1)+(0.811714471885)
        ref[2,0,1,0]=o*(0.0413505829692)*x_ref[0]**(o-1)+(-0.59249386093)
        ref[2,0,1,1]=o*(0.478645406371)*x_ref[1]**(o-1)+(0.221389337839)
        ref[2,0,1,2]=o*(0.18016770891)*x_ref[2]**(o-1)+(-0.783010309878)
        ref[2,1,0,0]=o*(0.641325174447)*x_ref[0]**(o-1)+(0.859809801095)
        ref[2,1,0,1]=o*(0.238917443521)*x_ref[1]**(o-1)+(0.169387212912)
        ref[2,1,0,2]=o*(0.118386087343)*x_ref[2]**(o-1)+(0.7091562331)
        ref[2,1,1,0]=o*(0.683767548268)*x_ref[0]**(o-1)+(-0.488504930782)
        ref[2,1,1,1]=o*(0.421423574907)*x_ref[1]**(o-1)+(0.677076626099)
        ref[2,1,1,2]=o*(-0.0955208633306)*x_ref[2]**(o-1)+(0.545750792826)
        ref[3,0,0,0]=o*(-0.925067175669)*x_ref[0]**(o-1)+(0.550204660551)
        ref[3,0,0,1]=o*(0.435686889479)*x_ref[1]**(o-1)+(-0.171725370784)
        ref[3,0,0,2]=o*(0.182097753276)*x_ref[2]**(o-1)+(0.493662375098)
        ref[3,0,1,0]=o*(-0.719237805296)*x_ref[0]**(o-1)+(0.669817079155)
        ref[3,0,1,1]=o*(0.660680613106)*x_ref[1]**(o-1)+(-0.729927483007)
        ref[3,0,1,2]=o*(-0.162704346009)*x_ref[2]**(o-1)+(-0.559206291015)
        ref[3,1,0,0]=o*(0.129735268583)*x_ref[0]**(o-1)+(-0.411853842222)
        ref[3,1,0,1]=o*(0.045438376206)*x_ref[1]**(o-1)+(0.327804746872)
        ref[3,1,0,2]=o*(-0.639433216582)*x_ref[2]**(o-1)+(0.451106136717)
        ref[3,1,1,0]=o*(-0.811557092293)*x_ref[0]**(o-1)+(-0.818758274715)
        ref[3,1,1,1]=o*(-0.712815291675)*x_ref[1]**(o-1)+(0.729038274951)
        ref[3,1,1,2]=o*(-0.826837009476)*x_ref[2]**(o-1)+(0.340025404977)
        ref[4,0,0,0]=o*(-0.507284842259)*x_ref[0]**(o-1)+(0.457554461786)
        ref[4,0,0,1]=o*(0.419345933038)*x_ref[1]**(o-1)+(-0.7773529718)
        ref[4,0,0,2]=o*(-0.626475804952)*x_ref[2]**(o-1)+(-0.784196320861)
        ref[4,0,1,0]=o*(-0.0340250322306)*x_ref[0]**(o-1)+(-0.0936499769437)
        ref[4,0,1,1]=o*(-0.777584776075)*x_ref[1]**(o-1)+(-0.435876632622)
        ref[4,0,1,2]=o*(0.999336374444)*x_ref[2]**(o-1)+(-0.247149214414)
        ref[4,1,0,0]=o*(0.21620967013)*x_ref[0]**(o-1)+(0.213039866961)
        ref[4,1,0,1]=o*(0.83049354428)*x_ref[1]**(o-1)+(0.651430926943)
        ref[4,1,0,2]=o*(0.201648695549)*x_ref[2]**(o-1)+(-0.609802970877)
        ref[4,1,1,0]=o*(0.842001670092)*x_ref[0]**(o-1)+(-0.868487783326)
        ref[4,1,1,1]=o*(-0.288512200338)*x_ref[1]**(o-1)+(-0.654831485218)
        ref[4,1,1,2]=o*(-0.111859725689)*x_ref[2]**(o-1)+(0.728195130453)
        ref[5,0,0,0]=o*(0.473856201092)*x_ref[0]**(o-1)+(0.377649949188)
        ref[5,0,0,1]=o*(0.511638340284)*x_ref[1]**(o-1)+(-0.150021940198)
        ref[5,0,0,2]=o*(-0.801655915671)*x_ref[2]**(o-1)+(-0.603617347175)
        ref[5,0,1,0]=o*(0.945961549464)*x_ref[0]**(o-1)+(0.40151064288)
        ref[5,0,1,1]=o*(0.151039821252)*x_ref[1]**(o-1)+(0.977148741848)
        ref[5,0,1,2]=o*(0.0729651283448)*x_ref[2]**(o-1)+(0.365625847237)
        ref[5,1,0,0]=o*(0.613037024504)*x_ref[0]**(o-1)+(-0.743080801009)
        ref[5,1,0,1]=o*(-0.518736037968)*x_ref[1]**(o-1)+(0.240273029617)
        ref[5,1,0,2]=o*(0.437464780338)*x_ref[2]**(o-1)+(0.897940669321)
        ref[5,1,1,0]=o*(-0.149245273038)*x_ref[0]**(o-1)+(-0.387037171901)
        ref[5,1,1,1]=o*(0.0606843165198)*x_ref[1]**(o-1)+(-0.713188961184)
        ref[5,1,1,2]=o*(0.622672840108)*x_ref[2]**(o-1)+(-0.579757019852)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(1.05116031237)*x[0]+(-0.156242880188)*x[1]
        ref[0]=(1.05116031237)
        ref[1]=(-0.156242880188)
      else:
        s=(-0.978032091964)*x[0]+(0.994143533164)*x[1]+(1.36141912157)*x[2]
        ref[0]=(-0.978032091964)
        ref[1]=(0.994143533164)
        ref[2]=(1.36141912157)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.908181468063)*x[0]+(0.383654148719)*x[1]
        s[1]=(0.996727924056)*x[0]+(1.13801031707)*x[1]
        ref[0,0]=(0.908181468063)
        ref[0,1]=(0.383654148719)
        ref[1,0]=(0.996727924056)
        ref[1,1]=(1.13801031707)
      else:
        s[0]=(-0.431683467168)*x[0]+(-0.610893064868)*x[1]+(-0.39989804071)*x[2]
        s[1]=(0.820400856313)*x[0]+(0.790210621031)*x[1]+(-0.0581477423106)*x[2]
        ref[0,0]=(-0.431683467168)
        ref[0,1]=(-0.610893064868)
        ref[0,2]=(-0.39989804071)
        ref[1,0]=(0.820400856313)
        ref[1,1]=(0.790210621031)
        ref[1,2]=(-0.0581477423106)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.202521929549)*x[0]+(1.24388265835)*x[1]
        s[0,1]=(0.466921175909)*x[0]+(0.0938124013357)*x[1]
        s[0,2]=(-0.337565182967)*x[0]+(-0.66213972177)*x[1]
        s[0,3]=(-1.1265817109)*x[0]+(1.45099893506)*x[1]
        s[0,4]=(1.85945279233)*x[0]+(0.614232583853)*x[1]
        s[1,0]=(0.773998701601)*x[0]+(1.3572752273)*x[1]
        s[1,1]=(1.503889482)*x[0]+(-1.22986193243)*x[1]
        s[1,2]=(1.17229177208)*x[0]+(-1.005468476)*x[1]
        s[1,3]=(-1.27864883602)*x[0]+(0.27430357882)*x[1]
        s[1,4]=(-0.0170497525828)*x[0]+(-1.07936325294)*x[1]
        s[2,0]=(-0.172836979766)*x[0]+(0.481503531609)*x[1]
        s[2,1]=(0.240407365629)*x[0]+(-1.2996101353)*x[1]
        s[2,2]=(-1.35821876592)*x[0]+(1.06176151561)*x[1]
        s[2,3]=(-1.93289948536)*x[0]+(1.7647044408)*x[1]
        s[2,4]=(-1.16491431281)*x[0]+(0.483870146591)*x[1]
        s[3,0]=(0.646925654514)*x[0]+(-0.079632957461)*x[1]
        s[3,1]=(0.93955736882)*x[0]+(-0.314382763761)*x[1]
        s[3,2]=(0.903199444521)*x[0]+(-0.675377265588)*x[1]
        s[3,3]=(-0.373383381553)*x[0]+(0.140227745605)*x[1]
        s[3,4]=(0.199285967708)*x[0]+(0.440148792869)*x[1]
        ref[0,0,0]=(0.202521929549)
        ref[0,0,1]=(1.24388265835)
        ref[0,1,0]=(0.466921175909)
        ref[0,1,1]=(0.0938124013357)
        ref[0,2,0]=(-0.337565182967)
        ref[0,2,1]=(-0.66213972177)
        ref[0,3,0]=(-1.1265817109)
        ref[0,3,1]=(1.45099893506)
        ref[0,4,0]=(1.85945279233)
        ref[0,4,1]=(0.614232583853)
        ref[1,0,0]=(0.773998701601)
        ref[1,0,1]=(1.3572752273)
        ref[1,1,0]=(1.503889482)
        ref[1,1,1]=(-1.22986193243)
        ref[1,2,0]=(1.17229177208)
        ref[1,2,1]=(-1.005468476)
        ref[1,3,0]=(-1.27864883602)
        ref[1,3,1]=(0.27430357882)
        ref[1,4,0]=(-0.0170497525828)
        ref[1,4,1]=(-1.07936325294)
        ref[2,0,0]=(-0.172836979766)
        ref[2,0,1]=(0.481503531609)
        ref[2,1,0]=(0.240407365629)
        ref[2,1,1]=(-1.2996101353)
        ref[2,2,0]=(-1.35821876592)
        ref[2,2,1]=(1.06176151561)
        ref[2,3,0]=(-1.93289948536)
        ref[2,3,1]=(1.7647044408)
        ref[2,4,0]=(-1.16491431281)
        ref[2,4,1]=(0.483870146591)
        ref[3,0,0]=(0.646925654514)
        ref[3,0,1]=(-0.079632957461)
        ref[3,1,0]=(0.93955736882)
        ref[3,1,1]=(-0.314382763761)
        ref[3,2,0]=(0.903199444521)
        ref[3,2,1]=(-0.675377265588)
        ref[3,3,0]=(-0.373383381553)
        ref[3,3,1]=(0.140227745605)
        ref[3,4,0]=(0.199285967708)
        ref[3,4,1]=(0.440148792869)
      else:
        s[0,0]=(-0.454164125322)*x[0]+(-0.522669711039)*x[1]+(-1.69156961037)*x[2]
        s[0,1]=(1.02678975575)*x[0]+(-0.627069283778)*x[1]+(0.755190616163)*x[2]
        s[0,2]=(0.689068203494)*x[0]+(-0.677393706098)*x[1]+(-0.822365020628)*x[2]
        s[0,3]=(0.488491352905)*x[0]+(1.19039296387)*x[1]+(0.293029595575)*x[2]
        s[0,4]=(1.3254184285)*x[0]+(-0.67206171589)*x[1]+(1.95627951713)*x[2]
        s[1,0]=(-1.01024336678)*x[0]+(0.553509575201)*x[1]+(0.206480917751)*x[2]
        s[1,1]=(-0.679488104614)*x[0]+(0.561501215346)*x[1]+(-0.157738070792)*x[2]
        s[1,2]=(1.09058998896)*x[0]+(0.358786992535)*x[1]+(-0.866813461522)*x[2]
        s[1,3]=(0.945307210944)*x[0]+(-0.592121399828)*x[1]+(-1.23798843507)*x[2]
        s[1,4]=(-0.334377129623)*x[0]+(0.282409196274)*x[1]+(1.11154300742)*x[2]
        s[2,0]=(0.31913061339)*x[0]+(1.47252246089)*x[1]+(-0.0516509049482)*x[2]
        s[2,1]=(-0.889919147968)*x[0]+(0.750571626982)*x[1]+(1.44049343766)*x[2]
        s[2,2]=(-0.648407056167)*x[0]+(-0.961481146656)*x[1]+(0.747319150361)*x[2]
        s[2,3]=(1.49042928912)*x[0]+(1.31212422706)*x[1]+(0.510609313139)*x[2]
        s[2,4]=(-0.812306629075)*x[0]+(1.27041274415)*x[1]+(0.740119854826)*x[2]
        s[3,0]=(-1.11671812457)*x[0]+(0.382559318776)*x[1]+(-0.12879810898)*x[2]
        s[3,1]=(0.253402604664)*x[0]+(0.433299751265)*x[1]+(-0.282468707216)*x[2]
        s[3,2]=(1.32928877441)*x[0]+(-1.62629303652)*x[1]+(1.09372049629)*x[2]
        s[3,3]=(-1.20558999975)*x[0]+(-0.25798278657)*x[1]+(-0.0869196687657)*x[2]
        s[3,4]=(-1.31791924188)*x[0]+(0.0386230359027)*x[1]+(0.00644144564451)*x[2]
        ref[0,0,0]=(-0.454164125322)
        ref[0,0,1]=(-0.522669711039)
        ref[0,0,2]=(-1.69156961037)
        ref[0,1,0]=(1.02678975575)
        ref[0,1,1]=(-0.627069283778)
        ref[0,1,2]=(0.755190616163)
        ref[0,2,0]=(0.689068203494)
        ref[0,2,1]=(-0.677393706098)
        ref[0,2,2]=(-0.822365020628)
        ref[0,3,0]=(0.488491352905)
        ref[0,3,1]=(1.19039296387)
        ref[0,3,2]=(0.293029595575)
        ref[0,4,0]=(1.3254184285)
        ref[0,4,1]=(-0.67206171589)
        ref[0,4,2]=(1.95627951713)
        ref[1,0,0]=(-1.01024336678)
        ref[1,0,1]=(0.553509575201)
        ref[1,0,2]=(0.206480917751)
        ref[1,1,0]=(-0.679488104614)
        ref[1,1,1]=(0.561501215346)
        ref[1,1,2]=(-0.157738070792)
        ref[1,2,0]=(1.09058998896)
        ref[1,2,1]=(0.358786992535)
        ref[1,2,2]=(-0.866813461522)
        ref[1,3,0]=(0.945307210944)
        ref[1,3,1]=(-0.592121399828)
        ref[1,3,2]=(-1.23798843507)
        ref[1,4,0]=(-0.334377129623)
        ref[1,4,1]=(0.282409196274)
        ref[1,4,2]=(1.11154300742)
        ref[2,0,0]=(0.31913061339)
        ref[2,0,1]=(1.47252246089)
        ref[2,0,2]=(-0.0516509049482)
        ref[2,1,0]=(-0.889919147968)
        ref[2,1,1]=(0.750571626982)
        ref[2,1,2]=(1.44049343766)
        ref[2,2,0]=(-0.648407056167)
        ref[2,2,1]=(-0.961481146656)
        ref[2,2,2]=(0.747319150361)
        ref[2,3,0]=(1.49042928912)
        ref[2,3,1]=(1.31212422706)
        ref[2,3,2]=(0.510609313139)
        ref[2,4,0]=(-0.812306629075)
        ref[2,4,1]=(1.27041274415)
        ref[2,4,2]=(0.740119854826)
        ref[3,0,0]=(-1.11671812457)
        ref[3,0,1]=(0.382559318776)
        ref[3,0,2]=(-0.12879810898)
        ref[3,1,0]=(0.253402604664)
        ref[3,1,1]=(0.433299751265)
        ref[3,1,2]=(-0.282468707216)
        ref[3,2,0]=(1.32928877441)
        ref[3,2,1]=(-1.62629303652)
        ref[3,2,2]=(1.09372049629)
        ref[3,3,0]=(-1.20558999975)
        ref[3,3,1]=(-0.25798278657)
        ref[3,3,2]=(-0.0869196687657)
        ref[3,4,0]=(-1.31791924188)
        ref[3,4,1]=(0.0386230359027)
        ref[3,4,2]=(0.00644144564451)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.500249599152)*x[0]+(1.67754131181)*x[1]
        s[0,0,1]=(1.1446224976)*x[0]+(0.293592946461)*x[1]
        s[0,1,0]=(0.662825578178)*x[0]+(0.778202393248)*x[1]
        s[0,1,1]=(1.120980772)*x[0]+(0.95813878487)*x[1]
        s[1,0,0]=(-0.462148842008)*x[0]+(-1.63817011088)*x[1]
        s[1,0,1]=(1.49522487924)*x[0]+(1.17105843575)*x[1]
        s[1,1,0]=(0.0273404170297)*x[0]+(-0.60627667625)*x[1]
        s[1,1,1]=(0.328333508501)*x[0]+(-0.487743429505)*x[1]
        s[2,0,0]=(0.690757853756)*x[0]+(0.415349096316)*x[1]
        s[2,0,1]=(-1.76899408416)*x[0]+(0.338928282096)*x[1]
        s[2,1,0]=(-1.65374086962)*x[0]+(0.666255100879)*x[1]
        s[2,1,1]=(0.845913196898)*x[0]+(0.524414663858)*x[1]
        s[3,0,0]=(-0.0489138005056)*x[0]+(-0.306296688032)*x[1]
        s[3,0,1]=(-0.619051141066)*x[0]+(0.399377559544)*x[1]
        s[3,1,0]=(0.407991784518)*x[0]+(-1.14997392594)*x[1]
        s[3,1,1]=(-0.115512243924)*x[0]+(0.296839190724)*x[1]
        s[4,0,0]=(1.45399807861)*x[0]+(-0.807761883642)*x[1]
        s[4,0,1]=(0.639797195198)*x[0]+(1.35664991322)*x[1]
        s[4,1,0]=(-0.884543551657)*x[0]+(0.702070741415)*x[1]
        s[4,1,1]=(-0.280174966716)*x[0]+(0.0172627474218)*x[1]
        s[5,0,0]=(0.235198909788)*x[0]+(0.0707623144647)*x[1]
        s[5,0,1]=(0.670946179418)*x[0]+(0.746509554365)*x[1]
        s[5,1,0]=(1.20081036792)*x[0]+(0.158928139216)*x[1]
        s[5,1,1]=(1.06402047834)*x[0]+(-0.567749347509)*x[1]
        ref[0,0,0,0]=(-0.500249599152)
        ref[0,0,0,1]=(1.67754131181)
        ref[0,0,1,0]=(1.1446224976)
        ref[0,0,1,1]=(0.293592946461)
        ref[0,1,0,0]=(0.662825578178)
        ref[0,1,0,1]=(0.778202393248)
        ref[0,1,1,0]=(1.120980772)
        ref[0,1,1,1]=(0.95813878487)
        ref[1,0,0,0]=(-0.462148842008)
        ref[1,0,0,1]=(-1.63817011088)
        ref[1,0,1,0]=(1.49522487924)
        ref[1,0,1,1]=(1.17105843575)
        ref[1,1,0,0]=(0.0273404170297)
        ref[1,1,0,1]=(-0.60627667625)
        ref[1,1,1,0]=(0.328333508501)
        ref[1,1,1,1]=(-0.487743429505)
        ref[2,0,0,0]=(0.690757853756)
        ref[2,0,0,1]=(0.415349096316)
        ref[2,0,1,0]=(-1.76899408416)
        ref[2,0,1,1]=(0.338928282096)
        ref[2,1,0,0]=(-1.65374086962)
        ref[2,1,0,1]=(0.666255100879)
        ref[2,1,1,0]=(0.845913196898)
        ref[2,1,1,1]=(0.524414663858)
        ref[3,0,0,0]=(-0.0489138005056)
        ref[3,0,0,1]=(-0.306296688032)
        ref[3,0,1,0]=(-0.619051141066)
        ref[3,0,1,1]=(0.399377559544)
        ref[3,1,0,0]=(0.407991784518)
        ref[3,1,0,1]=(-1.14997392594)
        ref[3,1,1,0]=(-0.115512243924)
        ref[3,1,1,1]=(0.296839190724)
        ref[4,0,0,0]=(1.45399807861)
        ref[4,0,0,1]=(-0.807761883642)
        ref[4,0,1,0]=(0.639797195198)
        ref[4,0,1,1]=(1.35664991322)
        ref[4,1,0,0]=(-0.884543551657)
        ref[4,1,0,1]=(0.702070741415)
        ref[4,1,1,0]=(-0.280174966716)
        ref[4,1,1,1]=(0.0172627474218)
        ref[5,0,0,0]=(0.235198909788)
        ref[5,0,0,1]=(0.0707623144647)
        ref[5,0,1,0]=(0.670946179418)
        ref[5,0,1,1]=(0.746509554365)
        ref[5,1,0,0]=(1.20081036792)
        ref[5,1,0,1]=(0.158928139216)
        ref[5,1,1,0]=(1.06402047834)
        ref[5,1,1,1]=(-0.567749347509)
      else:
        s[0,0,0]=(-0.709307825835)*x[0]+(-0.212632569057)*x[1]+(-1.36262175379)*x[2]
        s[0,0,1]=(1.26136861771)*x[0]+(-0.247247821335)*x[1]+(0.952472163507)*x[2]
        s[0,1,0]=(-0.313562941587)*x[0]+(0.335230615547)*x[1]+(0.633669942964)*x[2]
        s[0,1,1]=(0.543435495949)*x[0]+(-0.278928766765)*x[1]+(-0.560857410629)*x[2]
        s[1,0,0]=(-0.228999374753)*x[0]+(0.627231691819)*x[1]+(-0.110282807377)*x[2]
        s[1,0,1]=(-0.183700669251)*x[0]+(0.361641358769)*x[1]+(-0.565719344787)*x[2]
        s[1,1,0]=(0.0392225368869)*x[0]+(0.530481764326)*x[1]+(-1.16647413286)*x[2]
        s[1,1,1]=(1.10307486886)*x[0]+(1.62013473483)*x[1]+(1.08663406043)*x[2]
        s[2,0,0]=(0.418269374961)*x[0]+(-0.881366670214)*x[1]+(-0.19081724529)*x[2]
        s[2,0,1]=(-0.888542773287)*x[0]+(0.973322751168)*x[1]+(-0.678806455499)*x[2]
        s[2,1,0]=(0.203680986809)*x[0]+(-0.64202022526)*x[1]+(-0.342367020762)*x[2]
        s[2,1,1]=(-0.984982113825)*x[0]+(1.52327025412)*x[1]+(-1.16664161507)*x[2]
        s[3,0,0]=(-0.430857332595)*x[0]+(1.37801740474)*x[1]+(1.47536591267)*x[2]
        s[3,0,1]=(-1.49872232697)*x[0]+(0.679321175644)*x[1]+(0.233255377135)*x[2]
        s[3,1,0]=(0.00673347423284)*x[0]+(-0.659736891512)*x[1]+(-1.24483031223)*x[2]
        s[3,1,1]=(-1.35939956697)*x[0]+(-1.05370704987)*x[1]+(0.974871664002)*x[2]
        s[4,0,0]=(-1.56401623595)*x[0]+(0.0396404235057)*x[1]+(0.529626464189)*x[2]
        s[4,0,1]=(-0.903524096133)*x[0]+(-0.0504449605469)*x[1]+(-0.38093760213)*x[2]
        s[4,1,0]=(-0.570517363798)*x[0]+(1.50697108327)*x[1]+(0.45722182765)*x[2]
        s[4,1,1]=(0.855238285486)*x[0]+(-1.19608277327)*x[1]+(-0.454258653189)*x[2]
        s[5,0,0]=(0.264880357574)*x[0]+(-0.137240568006)*x[1]+(-1.43157116771)*x[2]
        s[5,0,1]=(1.42099564204)*x[0]+(-0.584391783811)*x[1]+(-0.194445105527)*x[2]
        s[5,1,0]=(-1.41719313241)*x[0]+(1.18436247042)*x[1]+(0.774373387123)*x[2]
        s[5,1,1]=(0.894745586845)*x[0]+(0.272141343375)*x[1]+(-0.563792436747)*x[2]
        ref[0,0,0,0]=(-0.709307825835)
        ref[0,0,0,1]=(-0.212632569057)
        ref[0,0,0,2]=(-1.36262175379)
        ref[0,0,1,0]=(1.26136861771)
        ref[0,0,1,1]=(-0.247247821335)
        ref[0,0,1,2]=(0.952472163507)
        ref[0,1,0,0]=(-0.313562941587)
        ref[0,1,0,1]=(0.335230615547)
        ref[0,1,0,2]=(0.633669942964)
        ref[0,1,1,0]=(0.543435495949)
        ref[0,1,1,1]=(-0.278928766765)
        ref[0,1,1,2]=(-0.560857410629)
        ref[1,0,0,0]=(-0.228999374753)
        ref[1,0,0,1]=(0.627231691819)
        ref[1,0,0,2]=(-0.110282807377)
        ref[1,0,1,0]=(-0.183700669251)
        ref[1,0,1,1]=(0.361641358769)
        ref[1,0,1,2]=(-0.565719344787)
        ref[1,1,0,0]=(0.0392225368869)
        ref[1,1,0,1]=(0.530481764326)
        ref[1,1,0,2]=(-1.16647413286)
        ref[1,1,1,0]=(1.10307486886)
        ref[1,1,1,1]=(1.62013473483)
        ref[1,1,1,2]=(1.08663406043)
        ref[2,0,0,0]=(0.418269374961)
        ref[2,0,0,1]=(-0.881366670214)
        ref[2,0,0,2]=(-0.19081724529)
        ref[2,0,1,0]=(-0.888542773287)
        ref[2,0,1,1]=(0.973322751168)
        ref[2,0,1,2]=(-0.678806455499)
        ref[2,1,0,0]=(0.203680986809)
        ref[2,1,0,1]=(-0.64202022526)
        ref[2,1,0,2]=(-0.342367020762)
        ref[2,1,1,0]=(-0.984982113825)
        ref[2,1,1,1]=(1.52327025412)
        ref[2,1,1,2]=(-1.16664161507)
        ref[3,0,0,0]=(-0.430857332595)
        ref[3,0,0,1]=(1.37801740474)
        ref[3,0,0,2]=(1.47536591267)
        ref[3,0,1,0]=(-1.49872232697)
        ref[3,0,1,1]=(0.679321175644)
        ref[3,0,1,2]=(0.233255377135)
        ref[3,1,0,0]=(0.00673347423284)
        ref[3,1,0,1]=(-0.659736891512)
        ref[3,1,0,2]=(-1.24483031223)
        ref[3,1,1,0]=(-1.35939956697)
        ref[3,1,1,1]=(-1.05370704987)
        ref[3,1,1,2]=(0.974871664002)
        ref[4,0,0,0]=(-1.56401623595)
        ref[4,0,0,1]=(0.0396404235057)
        ref[4,0,0,2]=(0.529626464189)
        ref[4,0,1,0]=(-0.903524096133)
        ref[4,0,1,1]=(-0.0504449605469)
        ref[4,0,1,2]=(-0.38093760213)
        ref[4,1,0,0]=(-0.570517363798)
        ref[4,1,0,1]=(1.50697108327)
        ref[4,1,0,2]=(0.45722182765)
        ref[4,1,1,0]=(0.855238285486)
        ref[4,1,1,1]=(-1.19608277327)
        ref[4,1,1,2]=(-0.454258653189)
        ref[5,0,0,0]=(0.264880357574)
        ref[5,0,0,1]=(-0.137240568006)
        ref[5,0,0,2]=(-1.43157116771)
        ref[5,0,1,0]=(1.42099564204)
        ref[5,0,1,1]=(-0.584391783811)
        ref[5,0,1,2]=(-0.194445105527)
        ref[5,1,0,0]=(-1.41719313241)
        ref[5,1,0,1]=(1.18436247042)
        ref[5,1,0,2]=(0.774373387123)
        ref[5,1,1,0]=(0.894745586845)
        ref[5,1,1,1]=(0.272141343375)
        ref[5,1,1,2]=(-0.563792436747)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(1.05116031237)*x[0]+(-0.156242880188)*x[1]
        ref[0]=(1.05116031237)
        ref[1]=(-0.156242880188)
      else:
        s=(-0.978032091964)*x[0]+(0.994143533164)*x[1]+(1.36141912157)*x[2]
        ref[0]=(-0.978032091964)
        ref[1]=(0.994143533164)
        ref[2]=(1.36141912157)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.908181468063)*x[0]+(0.383654148719)*x[1]
        s[1]=(0.996727924056)*x[0]+(1.13801031707)*x[1]
        ref[0,0]=(0.908181468063)
        ref[0,1]=(0.383654148719)
        ref[1,0]=(0.996727924056)
        ref[1,1]=(1.13801031707)
      else:
        s[0]=(-0.431683467168)*x[0]+(-0.610893064868)*x[1]+(-0.39989804071)*x[2]
        s[1]=(0.820400856313)*x[0]+(0.790210621031)*x[1]+(-0.0581477423106)*x[2]
        ref[0,0]=(-0.431683467168)
        ref[0,1]=(-0.610893064868)
        ref[0,2]=(-0.39989804071)
        ref[1,0]=(0.820400856313)
        ref[1,1]=(0.790210621031)
        ref[1,2]=(-0.0581477423106)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.202521929549)*x[0]+(1.24388265835)*x[1]
        s[0,1]=(0.466921175909)*x[0]+(0.0938124013357)*x[1]
        s[0,2]=(-0.337565182967)*x[0]+(-0.66213972177)*x[1]
        s[0,3]=(-1.1265817109)*x[0]+(1.45099893506)*x[1]
        s[0,4]=(1.85945279233)*x[0]+(0.614232583853)*x[1]
        s[1,0]=(0.773998701601)*x[0]+(1.3572752273)*x[1]
        s[1,1]=(1.503889482)*x[0]+(-1.22986193243)*x[1]
        s[1,2]=(1.17229177208)*x[0]+(-1.005468476)*x[1]
        s[1,3]=(-1.27864883602)*x[0]+(0.27430357882)*x[1]
        s[1,4]=(-0.0170497525828)*x[0]+(-1.07936325294)*x[1]
        s[2,0]=(-0.172836979766)*x[0]+(0.481503531609)*x[1]
        s[2,1]=(0.240407365629)*x[0]+(-1.2996101353)*x[1]
        s[2,2]=(-1.35821876592)*x[0]+(1.06176151561)*x[1]
        s[2,3]=(-1.93289948536)*x[0]+(1.7647044408)*x[1]
        s[2,4]=(-1.16491431281)*x[0]+(0.483870146591)*x[1]
        s[3,0]=(0.646925654514)*x[0]+(-0.079632957461)*x[1]
        s[3,1]=(0.93955736882)*x[0]+(-0.314382763761)*x[1]
        s[3,2]=(0.903199444521)*x[0]+(-0.675377265588)*x[1]
        s[3,3]=(-0.373383381553)*x[0]+(0.140227745605)*x[1]
        s[3,4]=(0.199285967708)*x[0]+(0.440148792869)*x[1]
        ref[0,0,0]=(0.202521929549)
        ref[0,0,1]=(1.24388265835)
        ref[0,1,0]=(0.466921175909)
        ref[0,1,1]=(0.0938124013357)
        ref[0,2,0]=(-0.337565182967)
        ref[0,2,1]=(-0.66213972177)
        ref[0,3,0]=(-1.1265817109)
        ref[0,3,1]=(1.45099893506)
        ref[0,4,0]=(1.85945279233)
        ref[0,4,1]=(0.614232583853)
        ref[1,0,0]=(0.773998701601)
        ref[1,0,1]=(1.3572752273)
        ref[1,1,0]=(1.503889482)
        ref[1,1,1]=(-1.22986193243)
        ref[1,2,0]=(1.17229177208)
        ref[1,2,1]=(-1.005468476)
        ref[1,3,0]=(-1.27864883602)
        ref[1,3,1]=(0.27430357882)
        ref[1,4,0]=(-0.0170497525828)
        ref[1,4,1]=(-1.07936325294)
        ref[2,0,0]=(-0.172836979766)
        ref[2,0,1]=(0.481503531609)
        ref[2,1,0]=(0.240407365629)
        ref[2,1,1]=(-1.2996101353)
        ref[2,2,0]=(-1.35821876592)
        ref[2,2,1]=(1.06176151561)
        ref[2,3,0]=(-1.93289948536)
        ref[2,3,1]=(1.7647044408)
        ref[2,4,0]=(-1.16491431281)
        ref[2,4,1]=(0.483870146591)
        ref[3,0,0]=(0.646925654514)
        ref[3,0,1]=(-0.079632957461)
        ref[3,1,0]=(0.93955736882)
        ref[3,1,1]=(-0.314382763761)
        ref[3,2,0]=(0.903199444521)
        ref[3,2,1]=(-0.675377265588)
        ref[3,3,0]=(-0.373383381553)
        ref[3,3,1]=(0.140227745605)
        ref[3,4,0]=(0.199285967708)
        ref[3,4,1]=(0.440148792869)
      else:
        s[0,0]=(-0.454164125322)*x[0]+(-0.522669711039)*x[1]+(-1.69156961037)*x[2]
        s[0,1]=(1.02678975575)*x[0]+(-0.627069283778)*x[1]+(0.755190616163)*x[2]
        s[0,2]=(0.689068203494)*x[0]+(-0.677393706098)*x[1]+(-0.822365020628)*x[2]
        s[0,3]=(0.488491352905)*x[0]+(1.19039296387)*x[1]+(0.293029595575)*x[2]
        s[0,4]=(1.3254184285)*x[0]+(-0.67206171589)*x[1]+(1.95627951713)*x[2]
        s[1,0]=(-1.01024336678)*x[0]+(0.553509575201)*x[1]+(0.206480917751)*x[2]
        s[1,1]=(-0.679488104614)*x[0]+(0.561501215346)*x[1]+(-0.157738070792)*x[2]
        s[1,2]=(1.09058998896)*x[0]+(0.358786992535)*x[1]+(-0.866813461522)*x[2]
        s[1,3]=(0.945307210944)*x[0]+(-0.592121399828)*x[1]+(-1.23798843507)*x[2]
        s[1,4]=(-0.334377129623)*x[0]+(0.282409196274)*x[1]+(1.11154300742)*x[2]
        s[2,0]=(0.31913061339)*x[0]+(1.47252246089)*x[1]+(-0.0516509049482)*x[2]
        s[2,1]=(-0.889919147968)*x[0]+(0.750571626982)*x[1]+(1.44049343766)*x[2]
        s[2,2]=(-0.648407056167)*x[0]+(-0.961481146656)*x[1]+(0.747319150361)*x[2]
        s[2,3]=(1.49042928912)*x[0]+(1.31212422706)*x[1]+(0.510609313139)*x[2]
        s[2,4]=(-0.812306629075)*x[0]+(1.27041274415)*x[1]+(0.740119854826)*x[2]
        s[3,0]=(-1.11671812457)*x[0]+(0.382559318776)*x[1]+(-0.12879810898)*x[2]
        s[3,1]=(0.253402604664)*x[0]+(0.433299751265)*x[1]+(-0.282468707216)*x[2]
        s[3,2]=(1.32928877441)*x[0]+(-1.62629303652)*x[1]+(1.09372049629)*x[2]
        s[3,3]=(-1.20558999975)*x[0]+(-0.25798278657)*x[1]+(-0.0869196687657)*x[2]
        s[3,4]=(-1.31791924188)*x[0]+(0.0386230359027)*x[1]+(0.00644144564451)*x[2]
        ref[0,0,0]=(-0.454164125322)
        ref[0,0,1]=(-0.522669711039)
        ref[0,0,2]=(-1.69156961037)
        ref[0,1,0]=(1.02678975575)
        ref[0,1,1]=(-0.627069283778)
        ref[0,1,2]=(0.755190616163)
        ref[0,2,0]=(0.689068203494)
        ref[0,2,1]=(-0.677393706098)
        ref[0,2,2]=(-0.822365020628)
        ref[0,3,0]=(0.488491352905)
        ref[0,3,1]=(1.19039296387)
        ref[0,3,2]=(0.293029595575)
        ref[0,4,0]=(1.3254184285)
        ref[0,4,1]=(-0.67206171589)
        ref[0,4,2]=(1.95627951713)
        ref[1,0,0]=(-1.01024336678)
        ref[1,0,1]=(0.553509575201)
        ref[1,0,2]=(0.206480917751)
        ref[1,1,0]=(-0.679488104614)
        ref[1,1,1]=(0.561501215346)
        ref[1,1,2]=(-0.157738070792)
        ref[1,2,0]=(1.09058998896)
        ref[1,2,1]=(0.358786992535)
        ref[1,2,2]=(-0.866813461522)
        ref[1,3,0]=(0.945307210944)
        ref[1,3,1]=(-0.592121399828)
        ref[1,3,2]=(-1.23798843507)
        ref[1,4,0]=(-0.334377129623)
        ref[1,4,1]=(0.282409196274)
        ref[1,4,2]=(1.11154300742)
        ref[2,0,0]=(0.31913061339)
        ref[2,0,1]=(1.47252246089)
        ref[2,0,2]=(-0.0516509049482)
        ref[2,1,0]=(-0.889919147968)
        ref[2,1,1]=(0.750571626982)
        ref[2,1,2]=(1.44049343766)
        ref[2,2,0]=(-0.648407056167)
        ref[2,2,1]=(-0.961481146656)
        ref[2,2,2]=(0.747319150361)
        ref[2,3,0]=(1.49042928912)
        ref[2,3,1]=(1.31212422706)
        ref[2,3,2]=(0.510609313139)
        ref[2,4,0]=(-0.812306629075)
        ref[2,4,1]=(1.27041274415)
        ref[2,4,2]=(0.740119854826)
        ref[3,0,0]=(-1.11671812457)
        ref[3,0,1]=(0.382559318776)
        ref[3,0,2]=(-0.12879810898)
        ref[3,1,0]=(0.253402604664)
        ref[3,1,1]=(0.433299751265)
        ref[3,1,2]=(-0.282468707216)
        ref[3,2,0]=(1.32928877441)
        ref[3,2,1]=(-1.62629303652)
        ref[3,2,2]=(1.09372049629)
        ref[3,3,0]=(-1.20558999975)
        ref[3,3,1]=(-0.25798278657)
        ref[3,3,2]=(-0.0869196687657)
        ref[3,4,0]=(-1.31791924188)
        ref[3,4,1]=(0.0386230359027)
        ref[3,4,2]=(0.00644144564451)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.500249599152)*x[0]+(1.67754131181)*x[1]
        s[0,0,1]=(1.1446224976)*x[0]+(0.293592946461)*x[1]
        s[0,1,0]=(0.662825578178)*x[0]+(0.778202393248)*x[1]
        s[0,1,1]=(1.120980772)*x[0]+(0.95813878487)*x[1]
        s[1,0,0]=(-0.462148842008)*x[0]+(-1.63817011088)*x[1]
        s[1,0,1]=(1.49522487924)*x[0]+(1.17105843575)*x[1]
        s[1,1,0]=(0.0273404170297)*x[0]+(-0.60627667625)*x[1]
        s[1,1,1]=(0.328333508501)*x[0]+(-0.487743429505)*x[1]
        s[2,0,0]=(0.690757853756)*x[0]+(0.415349096316)*x[1]
        s[2,0,1]=(-1.76899408416)*x[0]+(0.338928282096)*x[1]
        s[2,1,0]=(-1.65374086962)*x[0]+(0.666255100879)*x[1]
        s[2,1,1]=(0.845913196898)*x[0]+(0.524414663858)*x[1]
        s[3,0,0]=(-0.0489138005056)*x[0]+(-0.306296688032)*x[1]
        s[3,0,1]=(-0.619051141066)*x[0]+(0.399377559544)*x[1]
        s[3,1,0]=(0.407991784518)*x[0]+(-1.14997392594)*x[1]
        s[3,1,1]=(-0.115512243924)*x[0]+(0.296839190724)*x[1]
        s[4,0,0]=(1.45399807861)*x[0]+(-0.807761883642)*x[1]
        s[4,0,1]=(0.639797195198)*x[0]+(1.35664991322)*x[1]
        s[4,1,0]=(-0.884543551657)*x[0]+(0.702070741415)*x[1]
        s[4,1,1]=(-0.280174966716)*x[0]+(0.0172627474218)*x[1]
        s[5,0,0]=(0.235198909788)*x[0]+(0.0707623144647)*x[1]
        s[5,0,1]=(0.670946179418)*x[0]+(0.746509554365)*x[1]
        s[5,1,0]=(1.20081036792)*x[0]+(0.158928139216)*x[1]
        s[5,1,1]=(1.06402047834)*x[0]+(-0.567749347509)*x[1]
        ref[0,0,0,0]=(-0.500249599152)
        ref[0,0,0,1]=(1.67754131181)
        ref[0,0,1,0]=(1.1446224976)
        ref[0,0,1,1]=(0.293592946461)
        ref[0,1,0,0]=(0.662825578178)
        ref[0,1,0,1]=(0.778202393248)
        ref[0,1,1,0]=(1.120980772)
        ref[0,1,1,1]=(0.95813878487)
        ref[1,0,0,0]=(-0.462148842008)
        ref[1,0,0,1]=(-1.63817011088)
        ref[1,0,1,0]=(1.49522487924)
        ref[1,0,1,1]=(1.17105843575)
        ref[1,1,0,0]=(0.0273404170297)
        ref[1,1,0,1]=(-0.60627667625)
        ref[1,1,1,0]=(0.328333508501)
        ref[1,1,1,1]=(-0.487743429505)
        ref[2,0,0,0]=(0.690757853756)
        ref[2,0,0,1]=(0.415349096316)
        ref[2,0,1,0]=(-1.76899408416)
        ref[2,0,1,1]=(0.338928282096)
        ref[2,1,0,0]=(-1.65374086962)
        ref[2,1,0,1]=(0.666255100879)
        ref[2,1,1,0]=(0.845913196898)
        ref[2,1,1,1]=(0.524414663858)
        ref[3,0,0,0]=(-0.0489138005056)
        ref[3,0,0,1]=(-0.306296688032)
        ref[3,0,1,0]=(-0.619051141066)
        ref[3,0,1,1]=(0.399377559544)
        ref[3,1,0,0]=(0.407991784518)
        ref[3,1,0,1]=(-1.14997392594)
        ref[3,1,1,0]=(-0.115512243924)
        ref[3,1,1,1]=(0.296839190724)
        ref[4,0,0,0]=(1.45399807861)
        ref[4,0,0,1]=(-0.807761883642)
        ref[4,0,1,0]=(0.639797195198)
        ref[4,0,1,1]=(1.35664991322)
        ref[4,1,0,0]=(-0.884543551657)
        ref[4,1,0,1]=(0.702070741415)
        ref[4,1,1,0]=(-0.280174966716)
        ref[4,1,1,1]=(0.0172627474218)
        ref[5,0,0,0]=(0.235198909788)
        ref[5,0,0,1]=(0.0707623144647)
        ref[5,0,1,0]=(0.670946179418)
        ref[5,0,1,1]=(0.746509554365)
        ref[5,1,0,0]=(1.20081036792)
        ref[5,1,0,1]=(0.158928139216)
        ref[5,1,1,0]=(1.06402047834)
        ref[5,1,1,1]=(-0.567749347509)
      else:
        s[0,0,0]=(-0.709307825835)*x[0]+(-0.212632569057)*x[1]+(-1.36262175379)*x[2]
        s[0,0,1]=(1.26136861771)*x[0]+(-0.247247821335)*x[1]+(0.952472163507)*x[2]
        s[0,1,0]=(-0.313562941587)*x[0]+(0.335230615547)*x[1]+(0.633669942964)*x[2]
        s[0,1,1]=(0.543435495949)*x[0]+(-0.278928766765)*x[1]+(-0.560857410629)*x[2]
        s[1,0,0]=(-0.228999374753)*x[0]+(0.627231691819)*x[1]+(-0.110282807377)*x[2]
        s[1,0,1]=(-0.183700669251)*x[0]+(0.361641358769)*x[1]+(-0.565719344787)*x[2]
        s[1,1,0]=(0.0392225368869)*x[0]+(0.530481764326)*x[1]+(-1.16647413286)*x[2]
        s[1,1,1]=(1.10307486886)*x[0]+(1.62013473483)*x[1]+(1.08663406043)*x[2]
        s[2,0,0]=(0.418269374961)*x[0]+(-0.881366670214)*x[1]+(-0.19081724529)*x[2]
        s[2,0,1]=(-0.888542773287)*x[0]+(0.973322751168)*x[1]+(-0.678806455499)*x[2]
        s[2,1,0]=(0.203680986809)*x[0]+(-0.64202022526)*x[1]+(-0.342367020762)*x[2]
        s[2,1,1]=(-0.984982113825)*x[0]+(1.52327025412)*x[1]+(-1.16664161507)*x[2]
        s[3,0,0]=(-0.430857332595)*x[0]+(1.37801740474)*x[1]+(1.47536591267)*x[2]
        s[3,0,1]=(-1.49872232697)*x[0]+(0.679321175644)*x[1]+(0.233255377135)*x[2]
        s[3,1,0]=(0.00673347423284)*x[0]+(-0.659736891512)*x[1]+(-1.24483031223)*x[2]
        s[3,1,1]=(-1.35939956697)*x[0]+(-1.05370704987)*x[1]+(0.974871664002)*x[2]
        s[4,0,0]=(-1.56401623595)*x[0]+(0.0396404235057)*x[1]+(0.529626464189)*x[2]
        s[4,0,1]=(-0.903524096133)*x[0]+(-0.0504449605469)*x[1]+(-0.38093760213)*x[2]
        s[4,1,0]=(-0.570517363798)*x[0]+(1.50697108327)*x[1]+(0.45722182765)*x[2]
        s[4,1,1]=(0.855238285486)*x[0]+(-1.19608277327)*x[1]+(-0.454258653189)*x[2]
        s[5,0,0]=(0.264880357574)*x[0]+(-0.137240568006)*x[1]+(-1.43157116771)*x[2]
        s[5,0,1]=(1.42099564204)*x[0]+(-0.584391783811)*x[1]+(-0.194445105527)*x[2]
        s[5,1,0]=(-1.41719313241)*x[0]+(1.18436247042)*x[1]+(0.774373387123)*x[2]
        s[5,1,1]=(0.894745586845)*x[0]+(0.272141343375)*x[1]+(-0.563792436747)*x[2]
        ref[0,0,0,0]=(-0.709307825835)
        ref[0,0,0,1]=(-0.212632569057)
        ref[0,0,0,2]=(-1.36262175379)
        ref[0,0,1,0]=(1.26136861771)
        ref[0,0,1,1]=(-0.247247821335)
        ref[0,0,1,2]=(0.952472163507)
        ref[0,1,0,0]=(-0.313562941587)
        ref[0,1,0,1]=(0.335230615547)
        ref[0,1,0,2]=(0.633669942964)
        ref[0,1,1,0]=(0.543435495949)
        ref[0,1,1,1]=(-0.278928766765)
        ref[0,1,1,2]=(-0.560857410629)
        ref[1,0,0,0]=(-0.228999374753)
        ref[1,0,0,1]=(0.627231691819)
        ref[1,0,0,2]=(-0.110282807377)
        ref[1,0,1,0]=(-0.183700669251)
        ref[1,0,1,1]=(0.361641358769)
        ref[1,0,1,2]=(-0.565719344787)
        ref[1,1,0,0]=(0.0392225368869)
        ref[1,1,0,1]=(0.530481764326)
        ref[1,1,0,2]=(-1.16647413286)
        ref[1,1,1,0]=(1.10307486886)
        ref[1,1,1,1]=(1.62013473483)
        ref[1,1,1,2]=(1.08663406043)
        ref[2,0,0,0]=(0.418269374961)
        ref[2,0,0,1]=(-0.881366670214)
        ref[2,0,0,2]=(-0.19081724529)
        ref[2,0,1,0]=(-0.888542773287)
        ref[2,0,1,1]=(0.973322751168)
        ref[2,0,1,2]=(-0.678806455499)
        ref[2,1,0,0]=(0.203680986809)
        ref[2,1,0,1]=(-0.64202022526)
        ref[2,1,0,2]=(-0.342367020762)
        ref[2,1,1,0]=(-0.984982113825)
        ref[2,1,1,1]=(1.52327025412)
        ref[2,1,1,2]=(-1.16664161507)
        ref[3,0,0,0]=(-0.430857332595)
        ref[3,0,0,1]=(1.37801740474)
        ref[3,0,0,2]=(1.47536591267)
        ref[3,0,1,0]=(-1.49872232697)
        ref[3,0,1,1]=(0.679321175644)
        ref[3,0,1,2]=(0.233255377135)
        ref[3,1,0,0]=(0.00673347423284)
        ref[3,1,0,1]=(-0.659736891512)
        ref[3,1,0,2]=(-1.24483031223)
        ref[3,1,1,0]=(-1.35939956697)
        ref[3,1,1,1]=(-1.05370704987)
        ref[3,1,1,2]=(0.974871664002)
        ref[4,0,0,0]=(-1.56401623595)
        ref[4,0,0,1]=(0.0396404235057)
        ref[4,0,0,2]=(0.529626464189)
        ref[4,0,1,0]=(-0.903524096133)
        ref[4,0,1,1]=(-0.0504449605469)
        ref[4,0,1,2]=(-0.38093760213)
        ref[4,1,0,0]=(-0.570517363798)
        ref[4,1,0,1]=(1.50697108327)
        ref[4,1,0,2]=(0.45722182765)
        ref[4,1,1,0]=(0.855238285486)
        ref[4,1,1,1]=(-1.19608277327)
        ref[4,1,1,2]=(-0.454258653189)
        ref[5,0,0,0]=(0.264880357574)
        ref[5,0,0,1]=(-0.137240568006)
        ref[5,0,0,2]=(-1.43157116771)
        ref[5,0,1,0]=(1.42099564204)
        ref[5,0,1,1]=(-0.584391783811)
        ref[5,0,1,2]=(-0.194445105527)
        ref[5,1,0,0]=(-1.41719313241)
        ref[5,1,0,1]=(1.18436247042)
        ref[5,1,0,2]=(0.774373387123)
        ref[5,1,1,0]=(0.894745586845)
        ref[5,1,1,1]=(0.272141343375)
        ref[5,1,1,2]=(-0.563792436747)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(-0.169375355906)*x[0]**o+(0.270029734958)*x[0]+(-0.19196016538)*x[1]**o+(0.426264369437)*x[1]
        ref[0]=o*(-0.169375355906)*x_ref[0]**(o-1)+(0.270029734958)
        ref[1]=o*(-0.19196016538)*x_ref[1]**(o-1)+(0.426264369437)
      else:
        arg=(0.00913595455019)*x[0]**o+(-0.561918629059)*x[0]+(0.0164102679759)*x[1]**o+(0.20749162111)*x[1]+(0.588703418401)*x[2]**o+(0.841289333498)*x[2]
        ref[0]=o*(0.00913595455019)*x_ref[0]**(o-1)+(-0.561918629059)
        ref[1]=o*(0.0164102679759)*x_ref[1]**(o-1)+(0.20749162111)
        ref[2]=o*(0.588703418401)*x_ref[2]**(o-1)+(0.841289333498)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.290031175706)*x[0]**o+(0.690121137589)*x[0]+(0.916232953272)*x[1]**o+(0.427227803135)*x[1]
        arg[1]=(0.0325792679154)*x[0]**o+(0.773157737135)*x[0]+(-0.0443953478357)*x[1]**o+(0.772028975294)*x[1]
        ref[0,0]=o*(-0.290031175706)*x_ref[0]**(o-1)+(0.690121137589)
        ref[0,1]=o*(0.916232953272)*x_ref[1]**(o-1)+(0.427227803135)
        ref[1,0]=o*(0.0325792679154)*x_ref[0]**(o-1)+(0.773157737135)
        ref[1,1]=o*(-0.0443953478357)*x_ref[1]**(o-1)+(0.772028975294)
      else:
        arg[0]=(-0.119338345858)*x[0]**o+(-0.866134437839)*x[0]+(-0.727685742079)*x[1]**o+(0.465666609669)*x[1]+(-0.351266733199)*x[2]**o+(-0.960631632698)*x[2]
        arg[1]=(-0.731037524737)*x[0]**o+(0.415975622707)*x[0]+(-0.86639696362)*x[1]**o+(0.914277406921)*x[1]+(0.627502485514)*x[2]**o+(-0.855013681276)*x[2]
        ref[0,0]=o*(-0.119338345858)*x_ref[0]**(o-1)+(-0.866134437839)
        ref[0,1]=o*(-0.727685742079)*x_ref[1]**(o-1)+(0.465666609669)
        ref[0,2]=o*(-0.351266733199)*x_ref[2]**(o-1)+(-0.960631632698)
        ref[1,0]=o*(-0.731037524737)*x_ref[0]**(o-1)+(0.415975622707)
        ref[1,1]=o*(-0.86639696362)*x_ref[1]**(o-1)+(0.914277406921)
        ref[1,2]=o*(0.627502485514)*x_ref[2]**(o-1)+(-0.855013681276)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.148320408183)*x[0]**o+(-0.340902870434)*x[0]+(-0.0856445410272)*x[1]**o+(0.220499304831)*x[1]
        arg[0,1]=(0.915892845602)*x[0]**o+(-0.04770770327)*x[0]+(0.482403170451)*x[1]**o+(-0.639478756575)*x[1]
        arg[0,2]=(-0.109204159071)*x[0]**o+(0.641607024805)*x[0]+(-0.409310547283)*x[1]**o+(0.202984386731)*x[1]
        arg[0,3]=(0.436833770493)*x[0]**o+(-0.200611791892)*x[0]+(-0.74814554517)*x[1]**o+(0.571881526754)*x[1]
        arg[0,4]=(0.704812299415)*x[0]**o+(0.688151134045)*x[0]+(0.486135712907)*x[1]**o+(0.955314240096)*x[1]
        arg[1,0]=(-0.731790373321)*x[0]**o+(-0.711258122556)*x[0]+(-0.961514666684)*x[1]**o+(0.726573978492)*x[1]
        arg[1,1]=(0.975372389889)*x[0]**o+(0.5026238435)*x[0]+(0.948955767945)*x[1]**o+(-0.780842035663)*x[1]
        arg[1,2]=(0.742383399984)*x[0]**o+(-0.729678661652)*x[0]+(-0.815499612514)*x[1]**o+(-0.12280674993)*x[1]
        arg[1,3]=(0.323809901662)*x[0]**o+(-0.441906205977)*x[0]+(-0.308989501913)*x[1]**o+(-0.549353671447)*x[1]
        arg[1,4]=(0.711285710752)*x[0]**o+(0.297823778866)*x[0]+(0.645185023747)*x[1]**o+(0.825620074525)*x[1]
        arg[2,0]=(-0.373175217644)*x[0]**o+(-0.25651286112)*x[0]+(-0.46193047583)*x[1]**o+(-0.359820938943)*x[1]
        arg[2,1]=(-0.406101756283)*x[0]**o+(-0.38896428223)*x[0]+(-0.993257693721)*x[1]**o+(-0.910326366305)*x[1]
        arg[2,2]=(0.517566718504)*x[0]**o+(-0.582940900248)*x[0]+(-0.381600156082)*x[1]**o+(0.466575604699)*x[1]
        arg[2,3]=(0.323944380062)*x[0]**o+(-0.99359065412)*x[0]+(0.069389014076)*x[1]**o+(0.960192708612)*x[1]
        arg[2,4]=(-0.741362989132)*x[0]**o+(0.978137520347)*x[0]+(-0.398259900603)*x[1]**o+(-0.224949732925)*x[1]
        arg[3,0]=(-0.540235240193)*x[0]**o+(0.761488584116)*x[0]+(-0.100229643779)*x[1]**o+(-0.168724555964)*x[1]
        arg[3,1]=(0.565068220879)*x[0]**o+(-0.778936552173)*x[0]+(-0.933478028833)*x[1]**o+(-0.274446893155)*x[1]
        arg[3,2]=(-0.198878845493)*x[0]**o+(0.97185846381)*x[0]+(0.0345948289236)*x[1]**o+(0.359375885935)*x[1]
        arg[3,3]=(-0.815426880937)*x[0]**o+(-0.0734806259357)*x[0]+(-0.0452925239934)*x[1]**o+(0.976167480949)*x[1]
        arg[3,4]=(0.374670688698)*x[0]**o+(-0.427666745858)*x[0]+(-0.631640994091)*x[1]**o+(-0.312455066404)*x[1]
        ref[0,0,0]=o*(0.148320408183)*x_ref[0]**(o-1)+(-0.340902870434)
        ref[0,0,1]=o*(-0.0856445410272)*x_ref[1]**(o-1)+(0.220499304831)
        ref[0,1,0]=o*(0.915892845602)*x_ref[0]**(o-1)+(-0.04770770327)
        ref[0,1,1]=o*(0.482403170451)*x_ref[1]**(o-1)+(-0.639478756575)
        ref[0,2,0]=o*(-0.109204159071)*x_ref[0]**(o-1)+(0.641607024805)
        ref[0,2,1]=o*(-0.409310547283)*x_ref[1]**(o-1)+(0.202984386731)
        ref[0,3,0]=o*(0.436833770493)*x_ref[0]**(o-1)+(-0.200611791892)
        ref[0,3,1]=o*(-0.74814554517)*x_ref[1]**(o-1)+(0.571881526754)
        ref[0,4,0]=o*(0.704812299415)*x_ref[0]**(o-1)+(0.688151134045)
        ref[0,4,1]=o*(0.486135712907)*x_ref[1]**(o-1)+(0.955314240096)
        ref[1,0,0]=o*(-0.731790373321)*x_ref[0]**(o-1)+(-0.711258122556)
        ref[1,0,1]=o*(-0.961514666684)*x_ref[1]**(o-1)+(0.726573978492)
        ref[1,1,0]=o*(0.975372389889)*x_ref[0]**(o-1)+(0.5026238435)
        ref[1,1,1]=o*(0.948955767945)*x_ref[1]**(o-1)+(-0.780842035663)
        ref[1,2,0]=o*(0.742383399984)*x_ref[0]**(o-1)+(-0.729678661652)
        ref[1,2,1]=o*(-0.815499612514)*x_ref[1]**(o-1)+(-0.12280674993)
        ref[1,3,0]=o*(0.323809901662)*x_ref[0]**(o-1)+(-0.441906205977)
        ref[1,3,1]=o*(-0.308989501913)*x_ref[1]**(o-1)+(-0.549353671447)
        ref[1,4,0]=o*(0.711285710752)*x_ref[0]**(o-1)+(0.297823778866)
        ref[1,4,1]=o*(0.645185023747)*x_ref[1]**(o-1)+(0.825620074525)
        ref[2,0,0]=o*(-0.373175217644)*x_ref[0]**(o-1)+(-0.25651286112)
        ref[2,0,1]=o*(-0.46193047583)*x_ref[1]**(o-1)+(-0.359820938943)
        ref[2,1,0]=o*(-0.406101756283)*x_ref[0]**(o-1)+(-0.38896428223)
        ref[2,1,1]=o*(-0.993257693721)*x_ref[1]**(o-1)+(-0.910326366305)
        ref[2,2,0]=o*(0.517566718504)*x_ref[0]**(o-1)+(-0.582940900248)
        ref[2,2,1]=o*(-0.381600156082)*x_ref[1]**(o-1)+(0.466575604699)
        ref[2,3,0]=o*(0.323944380062)*x_ref[0]**(o-1)+(-0.99359065412)
        ref[2,3,1]=o*(0.069389014076)*x_ref[1]**(o-1)+(0.960192708612)
        ref[2,4,0]=o*(-0.741362989132)*x_ref[0]**(o-1)+(0.978137520347)
        ref[2,4,1]=o*(-0.398259900603)*x_ref[1]**(o-1)+(-0.224949732925)
        ref[3,0,0]=o*(-0.540235240193)*x_ref[0]**(o-1)+(0.761488584116)
        ref[3,0,1]=o*(-0.100229643779)*x_ref[1]**(o-1)+(-0.168724555964)
        ref[3,1,0]=o*(0.565068220879)*x_ref[0]**(o-1)+(-0.778936552173)
        ref[3,1,1]=o*(-0.933478028833)*x_ref[1]**(o-1)+(-0.274446893155)
        ref[3,2,0]=o*(-0.198878845493)*x_ref[0]**(o-1)+(0.97185846381)
        ref[3,2,1]=o*(0.0345948289236)*x_ref[1]**(o-1)+(0.359375885935)
        ref[3,3,0]=o*(-0.815426880937)*x_ref[0]**(o-1)+(-0.0734806259357)
        ref[3,3,1]=o*(-0.0452925239934)*x_ref[1]**(o-1)+(0.976167480949)
        ref[3,4,0]=o*(0.374670688698)*x_ref[0]**(o-1)+(-0.427666745858)
        ref[3,4,1]=o*(-0.631640994091)*x_ref[1]**(o-1)+(-0.312455066404)
      else:
        arg[0,0]=(0.284398400678)*x[0]**o+(0.219396164435)*x[0]+(0.220563761332)*x[1]**o+(0.167554964517)*x[1]+(0.234139558431)*x[2]**o+(-0.462262273341)*x[2]
        arg[0,1]=(0.161107248226)*x[0]**o+(0.23981806792)*x[0]+(-0.365265066245)*x[1]**o+(0.280522162689)*x[1]+(0.0813974911438)*x[2]**o+(-0.43305486384)*x[2]
        arg[0,2]=(0.813480874845)*x[0]**o+(0.600760069636)*x[0]+(-0.133815038485)*x[1]**o+(-0.954192097752)*x[1]+(-0.60756978514)*x[2]**o+(0.669710363926)*x[2]
        arg[0,3]=(-0.760542897936)*x[0]**o+(0.869702837484)*x[0]+(0.568626088907)*x[1]**o+(0.272054244422)*x[1]+(-0.0415999983147)*x[2]**o+(0.446199878551)*x[2]
        arg[0,4]=(0.0453200557409)*x[0]**o+(0.990444244175)*x[0]+(0.134265180195)*x[1]**o+(0.156166931353)*x[1]+(-0.787964274948)*x[2]**o+(0.0116920503615)*x[2]
        arg[1,0]=(-0.826245054647)*x[0]**o+(0.203727421041)*x[0]+(0.435254937068)*x[1]**o+(-0.271957510058)*x[1]+(0.24056400787)*x[2]**o+(-0.270322614397)*x[2]
        arg[1,1]=(-0.423339337474)*x[0]**o+(0.272439137387)*x[0]+(-0.0736213619539)*x[1]**o+(-0.776123259153)*x[1]+(-0.751736227059)*x[2]**o+(0.944253810697)*x[2]
        arg[1,2]=(0.189348519506)*x[0]**o+(-0.599085006375)*x[0]+(0.511802349511)*x[1]**o+(-0.309756999526)*x[1]+(-0.753093308971)*x[2]**o+(-0.109878524762)*x[2]
        arg[1,3]=(0.273889782311)*x[0]**o+(0.284416273071)*x[0]+(-0.793016836976)*x[1]**o+(0.584657111427)*x[1]+(0.202246448215)*x[2]**o+(0.669336280646)*x[2]
        arg[1,4]=(-0.310062687494)*x[0]**o+(-0.0558492819571)*x[0]+(0.661043054703)*x[1]**o+(0.708531199864)*x[1]+(0.0490202348501)*x[2]**o+(0.732983691308)*x[2]
        arg[2,0]=(0.967228638923)*x[0]**o+(-0.816725187456)*x[0]+(0.836005904943)*x[1]**o+(-0.618363868727)*x[1]+(-0.977101150283)*x[2]**o+(-0.960030792428)*x[2]
        arg[2,1]=(0.536503091036)*x[0]**o+(-0.786194077149)*x[0]+(-0.337609174718)*x[1]**o+(0.787326702105)*x[1]+(0.989900210061)*x[2]**o+(0.67077581859)*x[2]
        arg[2,2]=(-0.732242113661)*x[0]**o+(-0.565641876648)*x[0]+(0.175986347852)*x[1]**o+(0.862449074966)*x[1]+(0.0257656069443)*x[2]**o+(0.824927753427)*x[2]
        arg[2,3]=(-0.863272499797)*x[0]**o+(-0.817781186551)*x[0]+(0.168931269996)*x[1]**o+(-0.530060186715)*x[1]+(-0.516801368376)*x[2]**o+(-0.570137941882)*x[2]
        arg[2,4]=(-0.675287850319)*x[0]**o+(-0.672701614891)*x[0]+(-0.19851266374)*x[1]**o+(-0.0494587000627)*x[1]+(0.518006592112)*x[2]**o+(0.0337938303974)*x[2]
        arg[3,0]=(-0.175683006965)*x[0]**o+(-0.843980579113)*x[0]+(0.903358518604)*x[1]**o+(0.962156465407)*x[1]+(-0.0887392353523)*x[2]**o+(0.524587238729)*x[2]
        arg[3,1]=(0.0622433182846)*x[0]**o+(-0.756701969275)*x[0]+(0.721836266349)*x[1]**o+(0.457673032865)*x[1]+(0.704951904865)*x[2]**o+(0.781015828813)*x[2]
        arg[3,2]=(-0.457057269128)*x[0]**o+(-0.880403805028)*x[0]+(0.103553852578)*x[1]**o+(-0.807061998574)*x[1]+(-0.0352722677587)*x[2]**o+(0.0527948564177)*x[2]
        arg[3,3]=(0.161459807817)*x[0]**o+(0.698509459014)*x[0]+(-0.596487730094)*x[1]**o+(-0.906646770174)*x[1]+(0.567036380682)*x[2]**o+(0.490123167386)*x[2]
        arg[3,4]=(-0.668686195082)*x[0]**o+(0.911485638627)*x[0]+(-0.856948411336)*x[1]**o+(0.0348927535379)*x[1]+(0.17649505057)*x[2]**o+(-0.778920185271)*x[2]
        ref[0,0,0]=o*(0.284398400678)*x_ref[0]**(o-1)+(0.219396164435)
        ref[0,0,1]=o*(0.220563761332)*x_ref[1]**(o-1)+(0.167554964517)
        ref[0,0,2]=o*(0.234139558431)*x_ref[2]**(o-1)+(-0.462262273341)
        ref[0,1,0]=o*(0.161107248226)*x_ref[0]**(o-1)+(0.23981806792)
        ref[0,1,1]=o*(-0.365265066245)*x_ref[1]**(o-1)+(0.280522162689)
        ref[0,1,2]=o*(0.0813974911438)*x_ref[2]**(o-1)+(-0.43305486384)
        ref[0,2,0]=o*(0.813480874845)*x_ref[0]**(o-1)+(0.600760069636)
        ref[0,2,1]=o*(-0.133815038485)*x_ref[1]**(o-1)+(-0.954192097752)
        ref[0,2,2]=o*(-0.60756978514)*x_ref[2]**(o-1)+(0.669710363926)
        ref[0,3,0]=o*(-0.760542897936)*x_ref[0]**(o-1)+(0.869702837484)
        ref[0,3,1]=o*(0.568626088907)*x_ref[1]**(o-1)+(0.272054244422)
        ref[0,3,2]=o*(-0.0415999983147)*x_ref[2]**(o-1)+(0.446199878551)
        ref[0,4,0]=o*(0.0453200557409)*x_ref[0]**(o-1)+(0.990444244175)
        ref[0,4,1]=o*(0.134265180195)*x_ref[1]**(o-1)+(0.156166931353)
        ref[0,4,2]=o*(-0.787964274948)*x_ref[2]**(o-1)+(0.0116920503615)
        ref[1,0,0]=o*(-0.826245054647)*x_ref[0]**(o-1)+(0.203727421041)
        ref[1,0,1]=o*(0.435254937068)*x_ref[1]**(o-1)+(-0.271957510058)
        ref[1,0,2]=o*(0.24056400787)*x_ref[2]**(o-1)+(-0.270322614397)
        ref[1,1,0]=o*(-0.423339337474)*x_ref[0]**(o-1)+(0.272439137387)
        ref[1,1,1]=o*(-0.0736213619539)*x_ref[1]**(o-1)+(-0.776123259153)
        ref[1,1,2]=o*(-0.751736227059)*x_ref[2]**(o-1)+(0.944253810697)
        ref[1,2,0]=o*(0.189348519506)*x_ref[0]**(o-1)+(-0.599085006375)
        ref[1,2,1]=o*(0.511802349511)*x_ref[1]**(o-1)+(-0.309756999526)
        ref[1,2,2]=o*(-0.753093308971)*x_ref[2]**(o-1)+(-0.109878524762)
        ref[1,3,0]=o*(0.273889782311)*x_ref[0]**(o-1)+(0.284416273071)
        ref[1,3,1]=o*(-0.793016836976)*x_ref[1]**(o-1)+(0.584657111427)
        ref[1,3,2]=o*(0.202246448215)*x_ref[2]**(o-1)+(0.669336280646)
        ref[1,4,0]=o*(-0.310062687494)*x_ref[0]**(o-1)+(-0.0558492819571)
        ref[1,4,1]=o*(0.661043054703)*x_ref[1]**(o-1)+(0.708531199864)
        ref[1,4,2]=o*(0.0490202348501)*x_ref[2]**(o-1)+(0.732983691308)
        ref[2,0,0]=o*(0.967228638923)*x_ref[0]**(o-1)+(-0.816725187456)
        ref[2,0,1]=o*(0.836005904943)*x_ref[1]**(o-1)+(-0.618363868727)
        ref[2,0,2]=o*(-0.977101150283)*x_ref[2]**(o-1)+(-0.960030792428)
        ref[2,1,0]=o*(0.536503091036)*x_ref[0]**(o-1)+(-0.786194077149)
        ref[2,1,1]=o*(-0.337609174718)*x_ref[1]**(o-1)+(0.787326702105)
        ref[2,1,2]=o*(0.989900210061)*x_ref[2]**(o-1)+(0.67077581859)
        ref[2,2,0]=o*(-0.732242113661)*x_ref[0]**(o-1)+(-0.565641876648)
        ref[2,2,1]=o*(0.175986347852)*x_ref[1]**(o-1)+(0.862449074966)
        ref[2,2,2]=o*(0.0257656069443)*x_ref[2]**(o-1)+(0.824927753427)
        ref[2,3,0]=o*(-0.863272499797)*x_ref[0]**(o-1)+(-0.817781186551)
        ref[2,3,1]=o*(0.168931269996)*x_ref[1]**(o-1)+(-0.530060186715)
        ref[2,3,2]=o*(-0.516801368376)*x_ref[2]**(o-1)+(-0.570137941882)
        ref[2,4,0]=o*(-0.675287850319)*x_ref[0]**(o-1)+(-0.672701614891)
        ref[2,4,1]=o*(-0.19851266374)*x_ref[1]**(o-1)+(-0.0494587000627)
        ref[2,4,2]=o*(0.518006592112)*x_ref[2]**(o-1)+(0.0337938303974)
        ref[3,0,0]=o*(-0.175683006965)*x_ref[0]**(o-1)+(-0.843980579113)
        ref[3,0,1]=o*(0.903358518604)*x_ref[1]**(o-1)+(0.962156465407)
        ref[3,0,2]=o*(-0.0887392353523)*x_ref[2]**(o-1)+(0.524587238729)
        ref[3,1,0]=o*(0.0622433182846)*x_ref[0]**(o-1)+(-0.756701969275)
        ref[3,1,1]=o*(0.721836266349)*x_ref[1]**(o-1)+(0.457673032865)
        ref[3,1,2]=o*(0.704951904865)*x_ref[2]**(o-1)+(0.781015828813)
        ref[3,2,0]=o*(-0.457057269128)*x_ref[0]**(o-1)+(-0.880403805028)
        ref[3,2,1]=o*(0.103553852578)*x_ref[1]**(o-1)+(-0.807061998574)
        ref[3,2,2]=o*(-0.0352722677587)*x_ref[2]**(o-1)+(0.0527948564177)
        ref[3,3,0]=o*(0.161459807817)*x_ref[0]**(o-1)+(0.698509459014)
        ref[3,3,1]=o*(-0.596487730094)*x_ref[1]**(o-1)+(-0.906646770174)
        ref[3,3,2]=o*(0.567036380682)*x_ref[2]**(o-1)+(0.490123167386)
        ref[3,4,0]=o*(-0.668686195082)*x_ref[0]**(o-1)+(0.911485638627)
        ref[3,4,1]=o*(-0.856948411336)*x_ref[1]**(o-1)+(0.0348927535379)
        ref[3,4,2]=o*(0.17649505057)*x_ref[2]**(o-1)+(-0.778920185271)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.366719126165)*x[0]**o+(0.273305723392)*x[0]+(0.508134048679)*x[1]**o+(-0.0895788966541)*x[1]
        arg[0,0,1]=(-0.911986850385)*x[0]**o+(-0.98503317507)*x[0]+(-0.310897248279)*x[1]**o+(0.015206007134)*x[1]
        arg[0,1,0]=(-0.0827324676842)*x[0]**o+(0.566620026269)*x[0]+(-0.0581063189367)*x[1]**o+(0.379530936223)*x[1]
        arg[0,1,1]=(0.223247684076)*x[0]**o+(0.511247996036)*x[0]+(-0.904638808138)*x[1]**o+(-0.253982863496)*x[1]
        arg[1,0,0]=(-0.117040630362)*x[0]**o+(0.720200175398)*x[0]+(0.316014562047)*x[1]**o+(-0.88437304478)*x[1]
        arg[1,0,1]=(0.521261471788)*x[0]**o+(0.866618380963)*x[0]+(0.376866254345)*x[1]**o+(0.0905522236713)*x[1]
        arg[1,1,0]=(-0.0907011175214)*x[0]**o+(0.925809501719)*x[0]+(0.29066676347)*x[1]**o+(-0.77392365917)*x[1]
        arg[1,1,1]=(0.410922874646)*x[0]**o+(-0.801953959211)*x[0]+(-0.911104174539)*x[1]**o+(-0.640661859542)*x[1]
        arg[2,0,0]=(-0.449471274267)*x[0]**o+(0.8111872512)*x[0]+(0.0498814189394)*x[1]**o+(0.683703882124)*x[1]
        arg[2,0,1]=(-0.192396495147)*x[0]**o+(0.893457442524)*x[0]+(-0.946936565887)*x[1]**o+(-0.0290255758634)*x[1]
        arg[2,1,0]=(0.90847535637)*x[0]**o+(-0.517929281823)*x[0]+(0.389994692606)*x[1]**o+(-0.601708219886)*x[1]
        arg[2,1,1]=(0.401910281699)*x[0]**o+(0.485033224067)*x[0]+(-0.362139392478)*x[1]**o+(-0.740542640224)*x[1]
        arg[3,0,0]=(0.195815356554)*x[0]**o+(-0.584270182199)*x[0]+(-0.773247762709)*x[1]**o+(0.974109226972)*x[1]
        arg[3,0,1]=(0.516821334505)*x[0]**o+(-0.667535261235)*x[0]+(-0.687554420634)*x[1]**o+(-0.826211449593)*x[1]
        arg[3,1,0]=(-0.886684033283)*x[0]**o+(0.833394822235)*x[0]+(-0.509561014765)*x[1]**o+(-0.934990250803)*x[1]
        arg[3,1,1]=(-0.279505984235)*x[0]**o+(0.629210321924)*x[0]+(-0.245033518843)*x[1]**o+(0.148437993524)*x[1]
        arg[4,0,0]=(-0.120914130766)*x[0]**o+(-0.70386339291)*x[0]+(-0.37739143589)*x[1]**o+(-0.604979315802)*x[1]
        arg[4,0,1]=(-0.259492475213)*x[0]**o+(0.628922768093)*x[0]+(-0.973387886178)*x[1]**o+(0.285217747252)*x[1]
        arg[4,1,0]=(-0.986163107858)*x[0]**o+(-0.85082996164)*x[0]+(-0.560402256475)*x[1]**o+(-0.468278543242)*x[1]
        arg[4,1,1]=(0.370258762453)*x[0]**o+(0.870396552858)*x[0]+(0.57205142923)*x[1]**o+(0.332787259857)*x[1]
        arg[5,0,0]=(-0.570453853889)*x[0]**o+(0.253977538115)*x[0]+(0.628545140288)*x[1]**o+(-0.766550576107)*x[1]
        arg[5,0,1]=(-0.626776709555)*x[0]**o+(0.593526092433)*x[0]+(0.824052315861)*x[1]**o+(-0.409883522469)*x[1]
        arg[5,1,0]=(0.285256467088)*x[0]**o+(-0.808675657802)*x[0]+(-0.818251043807)*x[1]**o+(0.378756627769)*x[1]
        arg[5,1,1]=(-0.810805592247)*x[0]**o+(-0.879886206051)*x[0]+(0.378171206533)*x[1]**o+(0.478469089448)*x[1]
        ref[0,0,0,0]=o*(0.366719126165)*x_ref[0]**(o-1)+(0.273305723392)
        ref[0,0,0,1]=o*(0.508134048679)*x_ref[1]**(o-1)+(-0.0895788966541)
        ref[0,0,1,0]=o*(-0.911986850385)*x_ref[0]**(o-1)+(-0.98503317507)
        ref[0,0,1,1]=o*(-0.310897248279)*x_ref[1]**(o-1)+(0.015206007134)
        ref[0,1,0,0]=o*(-0.0827324676842)*x_ref[0]**(o-1)+(0.566620026269)
        ref[0,1,0,1]=o*(-0.0581063189367)*x_ref[1]**(o-1)+(0.379530936223)
        ref[0,1,1,0]=o*(0.223247684076)*x_ref[0]**(o-1)+(0.511247996036)
        ref[0,1,1,1]=o*(-0.904638808138)*x_ref[1]**(o-1)+(-0.253982863496)
        ref[1,0,0,0]=o*(-0.117040630362)*x_ref[0]**(o-1)+(0.720200175398)
        ref[1,0,0,1]=o*(0.316014562047)*x_ref[1]**(o-1)+(-0.88437304478)
        ref[1,0,1,0]=o*(0.521261471788)*x_ref[0]**(o-1)+(0.866618380963)
        ref[1,0,1,1]=o*(0.376866254345)*x_ref[1]**(o-1)+(0.0905522236713)
        ref[1,1,0,0]=o*(-0.0907011175214)*x_ref[0]**(o-1)+(0.925809501719)
        ref[1,1,0,1]=o*(0.29066676347)*x_ref[1]**(o-1)+(-0.77392365917)
        ref[1,1,1,0]=o*(0.410922874646)*x_ref[0]**(o-1)+(-0.801953959211)
        ref[1,1,1,1]=o*(-0.911104174539)*x_ref[1]**(o-1)+(-0.640661859542)
        ref[2,0,0,0]=o*(-0.449471274267)*x_ref[0]**(o-1)+(0.8111872512)
        ref[2,0,0,1]=o*(0.0498814189394)*x_ref[1]**(o-1)+(0.683703882124)
        ref[2,0,1,0]=o*(-0.192396495147)*x_ref[0]**(o-1)+(0.893457442524)
        ref[2,0,1,1]=o*(-0.946936565887)*x_ref[1]**(o-1)+(-0.0290255758634)
        ref[2,1,0,0]=o*(0.90847535637)*x_ref[0]**(o-1)+(-0.517929281823)
        ref[2,1,0,1]=o*(0.389994692606)*x_ref[1]**(o-1)+(-0.601708219886)
        ref[2,1,1,0]=o*(0.401910281699)*x_ref[0]**(o-1)+(0.485033224067)
        ref[2,1,1,1]=o*(-0.362139392478)*x_ref[1]**(o-1)+(-0.740542640224)
        ref[3,0,0,0]=o*(0.195815356554)*x_ref[0]**(o-1)+(-0.584270182199)
        ref[3,0,0,1]=o*(-0.773247762709)*x_ref[1]**(o-1)+(0.974109226972)
        ref[3,0,1,0]=o*(0.516821334505)*x_ref[0]**(o-1)+(-0.667535261235)
        ref[3,0,1,1]=o*(-0.687554420634)*x_ref[1]**(o-1)+(-0.826211449593)
        ref[3,1,0,0]=o*(-0.886684033283)*x_ref[0]**(o-1)+(0.833394822235)
        ref[3,1,0,1]=o*(-0.509561014765)*x_ref[1]**(o-1)+(-0.934990250803)
        ref[3,1,1,0]=o*(-0.279505984235)*x_ref[0]**(o-1)+(0.629210321924)
        ref[3,1,1,1]=o*(-0.245033518843)*x_ref[1]**(o-1)+(0.148437993524)
        ref[4,0,0,0]=o*(-0.120914130766)*x_ref[0]**(o-1)+(-0.70386339291)
        ref[4,0,0,1]=o*(-0.37739143589)*x_ref[1]**(o-1)+(-0.604979315802)
        ref[4,0,1,0]=o*(-0.259492475213)*x_ref[0]**(o-1)+(0.628922768093)
        ref[4,0,1,1]=o*(-0.973387886178)*x_ref[1]**(o-1)+(0.285217747252)
        ref[4,1,0,0]=o*(-0.986163107858)*x_ref[0]**(o-1)+(-0.85082996164)
        ref[4,1,0,1]=o*(-0.560402256475)*x_ref[1]**(o-1)+(-0.468278543242)
        ref[4,1,1,0]=o*(0.370258762453)*x_ref[0]**(o-1)+(0.870396552858)
        ref[4,1,1,1]=o*(0.57205142923)*x_ref[1]**(o-1)+(0.332787259857)
        ref[5,0,0,0]=o*(-0.570453853889)*x_ref[0]**(o-1)+(0.253977538115)
        ref[5,0,0,1]=o*(0.628545140288)*x_ref[1]**(o-1)+(-0.766550576107)
        ref[5,0,1,0]=o*(-0.626776709555)*x_ref[0]**(o-1)+(0.593526092433)
        ref[5,0,1,1]=o*(0.824052315861)*x_ref[1]**(o-1)+(-0.409883522469)
        ref[5,1,0,0]=o*(0.285256467088)*x_ref[0]**(o-1)+(-0.808675657802)
        ref[5,1,0,1]=o*(-0.818251043807)*x_ref[1]**(o-1)+(0.378756627769)
        ref[5,1,1,0]=o*(-0.810805592247)*x_ref[0]**(o-1)+(-0.879886206051)
        ref[5,1,1,1]=o*(0.378171206533)*x_ref[1]**(o-1)+(0.478469089448)
      else:
        arg[0,0,0]=(0.99032561737)*x[0]**o+(0.0664709329455)*x[0]+(0.118470702946)*x[1]**o+(0.339805181064)*x[1]+(-0.288819936068)*x[2]**o+(0.500626113335)*x[2]
        arg[0,0,1]=(-0.736833023385)*x[0]**o+(-0.160433791202)*x[0]+(-0.159295012314)*x[1]**o+(-0.0515197665154)*x[1]+(-0.199553046759)*x[2]**o+(0.912767397035)*x[2]
        arg[0,1,0]=(0.263643352646)*x[0]**o+(-0.408521561602)*x[0]+(-0.690441644678)*x[1]**o+(0.514925086967)*x[1]+(-0.321785011214)*x[2]**o+(-0.192767636106)*x[2]
        arg[0,1,1]=(-0.00679573309711)*x[0]**o+(-0.0919830030966)*x[0]+(0.0194635540782)*x[1]**o+(0.0364274479874)*x[1]+(0.324767149157)*x[2]**o+(-0.930770951272)*x[2]
        arg[1,0,0]=(0.129524159454)*x[0]**o+(-0.747948168258)*x[0]+(0.748091879992)*x[1]**o+(0.0701365933978)*x[1]+(-0.227595193537)*x[2]**o+(-0.515988222581)*x[2]
        arg[1,0,1]=(-0.369843455974)*x[0]**o+(0.902965907161)*x[0]+(0.841050267135)*x[1]**o+(0.985297243316)*x[1]+(-0.96057678523)*x[2]**o+(0.149329555515)*x[2]
        arg[1,1,0]=(-0.530918169849)*x[0]**o+(0.574734874263)*x[0]+(-0.56424658789)*x[1]**o+(-0.756154171146)*x[1]+(-0.181750429769)*x[2]**o+(-0.270972276027)*x[2]
        arg[1,1,1]=(0.18184543511)*x[0]**o+(0.810253164883)*x[0]+(-0.0289547932926)*x[1]**o+(0.690366471619)*x[1]+(0.862616271239)*x[2]**o+(0.157705360996)*x[2]
        arg[2,0,0]=(-0.732811475459)*x[0]**o+(-0.720788617286)*x[0]+(-0.72285785154)*x[1]**o+(-0.764360626538)*x[1]+(0.979115834974)*x[2]**o+(0.430570190112)*x[2]
        arg[2,0,1]=(-0.602364948602)*x[0]**o+(-0.313871272033)*x[0]+(0.931476652796)*x[1]**o+(-0.553237167378)*x[1]+(0.273773142735)*x[2]**o+(0.732796238419)*x[2]
        arg[2,1,0]=(0.723724167325)*x[0]**o+(-0.411330362765)*x[0]+(0.415411259564)*x[1]**o+(0.596006586583)*x[1]+(0.588177039382)*x[2]**o+(-0.365406082278)*x[2]
        arg[2,1,1]=(0.0622006916856)*x[0]**o+(0.583807018004)*x[0]+(0.292401319183)*x[1]**o+(-0.202393075873)*x[1]+(-0.544886623341)*x[2]**o+(-0.269087712755)*x[2]
        arg[3,0,0]=(-0.543339583883)*x[0]**o+(-0.594218090249)*x[0]+(0.463226896714)*x[1]**o+(0.613466907584)*x[1]+(0.147935164587)*x[2]**o+(0.696530808607)*x[2]
        arg[3,0,1]=(0.338109512519)*x[0]**o+(0.281428380284)*x[0]+(0.0445095482602)*x[1]**o+(0.898143737155)*x[1]+(0.0465131079784)*x[2]**o+(-0.311801195497)*x[2]
        arg[3,1,0]=(0.489699597199)*x[0]**o+(0.198461296558)*x[0]+(0.87785894323)*x[1]**o+(0.916226820395)*x[1]+(-0.473685913201)*x[2]**o+(-0.496145627682)*x[2]
        arg[3,1,1]=(0.249814020596)*x[0]**o+(-0.894315482252)*x[0]+(-0.778776373134)*x[1]**o+(0.760727700644)*x[1]+(0.743775735588)*x[2]**o+(-0.689040985439)*x[2]
        arg[4,0,0]=(-0.825052728563)*x[0]**o+(0.522046869865)*x[0]+(0.333964173928)*x[1]**o+(-0.76926539114)*x[1]+(0.412352181661)*x[2]**o+(-0.0067772174596)*x[2]
        arg[4,0,1]=(-0.478610156432)*x[0]**o+(0.750576911314)*x[0]+(-0.531813749953)*x[1]**o+(0.355951176778)*x[1]+(-0.852853671607)*x[2]**o+(0.980221079923)*x[2]
        arg[4,1,0]=(-0.641389506244)*x[0]**o+(0.881787042426)*x[0]+(-0.304080099649)*x[1]**o+(0.38448900132)*x[1]+(-0.047930997317)*x[2]**o+(-0.276038511441)*x[2]
        arg[4,1,1]=(0.606877338216)*x[0]**o+(0.693205674961)*x[0]+(-0.731785947563)*x[1]**o+(0.980888111857)*x[1]+(0.355758103413)*x[2]**o+(-0.493780352941)*x[2]
        arg[5,0,0]=(-0.542680086594)*x[0]**o+(-0.335712421489)*x[0]+(0.196257902892)*x[1]**o+(-0.923381865562)*x[1]+(0.396049043788)*x[2]**o+(0.231195219983)*x[2]
        arg[5,0,1]=(0.224328300764)*x[0]**o+(0.306254705923)*x[0]+(-0.298262471524)*x[1]**o+(0.318537412793)*x[1]+(-0.0971742408716)*x[2]**o+(-0.374440507273)*x[2]
        arg[5,1,0]=(-0.408985188836)*x[0]**o+(-0.932478279672)*x[0]+(0.700313488604)*x[1]**o+(0.259720889362)*x[1]+(0.862207003532)*x[2]**o+(0.646109275237)*x[2]
        arg[5,1,1]=(-0.0341740340851)*x[0]**o+(0.742119475866)*x[0]+(0.310640405448)*x[1]**o+(-0.295841203899)*x[1]+(-0.778064951853)*x[2]**o+(-0.847637829972)*x[2]
        ref[0,0,0,0]=o*(0.99032561737)*x_ref[0]**(o-1)+(0.0664709329455)
        ref[0,0,0,1]=o*(0.118470702946)*x_ref[1]**(o-1)+(0.339805181064)
        ref[0,0,0,2]=o*(-0.288819936068)*x_ref[2]**(o-1)+(0.500626113335)
        ref[0,0,1,0]=o*(-0.736833023385)*x_ref[0]**(o-1)+(-0.160433791202)
        ref[0,0,1,1]=o*(-0.159295012314)*x_ref[1]**(o-1)+(-0.0515197665154)
        ref[0,0,1,2]=o*(-0.199553046759)*x_ref[2]**(o-1)+(0.912767397035)
        ref[0,1,0,0]=o*(0.263643352646)*x_ref[0]**(o-1)+(-0.408521561602)
        ref[0,1,0,1]=o*(-0.690441644678)*x_ref[1]**(o-1)+(0.514925086967)
        ref[0,1,0,2]=o*(-0.321785011214)*x_ref[2]**(o-1)+(-0.192767636106)
        ref[0,1,1,0]=o*(-0.00679573309711)*x_ref[0]**(o-1)+(-0.0919830030966)
        ref[0,1,1,1]=o*(0.0194635540782)*x_ref[1]**(o-1)+(0.0364274479874)
        ref[0,1,1,2]=o*(0.324767149157)*x_ref[2]**(o-1)+(-0.930770951272)
        ref[1,0,0,0]=o*(0.129524159454)*x_ref[0]**(o-1)+(-0.747948168258)
        ref[1,0,0,1]=o*(0.748091879992)*x_ref[1]**(o-1)+(0.0701365933978)
        ref[1,0,0,2]=o*(-0.227595193537)*x_ref[2]**(o-1)+(-0.515988222581)
        ref[1,0,1,0]=o*(-0.369843455974)*x_ref[0]**(o-1)+(0.902965907161)
        ref[1,0,1,1]=o*(0.841050267135)*x_ref[1]**(o-1)+(0.985297243316)
        ref[1,0,1,2]=o*(-0.96057678523)*x_ref[2]**(o-1)+(0.149329555515)
        ref[1,1,0,0]=o*(-0.530918169849)*x_ref[0]**(o-1)+(0.574734874263)
        ref[1,1,0,1]=o*(-0.56424658789)*x_ref[1]**(o-1)+(-0.756154171146)
        ref[1,1,0,2]=o*(-0.181750429769)*x_ref[2]**(o-1)+(-0.270972276027)
        ref[1,1,1,0]=o*(0.18184543511)*x_ref[0]**(o-1)+(0.810253164883)
        ref[1,1,1,1]=o*(-0.0289547932926)*x_ref[1]**(o-1)+(0.690366471619)
        ref[1,1,1,2]=o*(0.862616271239)*x_ref[2]**(o-1)+(0.157705360996)
        ref[2,0,0,0]=o*(-0.732811475459)*x_ref[0]**(o-1)+(-0.720788617286)
        ref[2,0,0,1]=o*(-0.72285785154)*x_ref[1]**(o-1)+(-0.764360626538)
        ref[2,0,0,2]=o*(0.979115834974)*x_ref[2]**(o-1)+(0.430570190112)
        ref[2,0,1,0]=o*(-0.602364948602)*x_ref[0]**(o-1)+(-0.313871272033)
        ref[2,0,1,1]=o*(0.931476652796)*x_ref[1]**(o-1)+(-0.553237167378)
        ref[2,0,1,2]=o*(0.273773142735)*x_ref[2]**(o-1)+(0.732796238419)
        ref[2,1,0,0]=o*(0.723724167325)*x_ref[0]**(o-1)+(-0.411330362765)
        ref[2,1,0,1]=o*(0.415411259564)*x_ref[1]**(o-1)+(0.596006586583)
        ref[2,1,0,2]=o*(0.588177039382)*x_ref[2]**(o-1)+(-0.365406082278)
        ref[2,1,1,0]=o*(0.0622006916856)*x_ref[0]**(o-1)+(0.583807018004)
        ref[2,1,1,1]=o*(0.292401319183)*x_ref[1]**(o-1)+(-0.202393075873)
        ref[2,1,1,2]=o*(-0.544886623341)*x_ref[2]**(o-1)+(-0.269087712755)
        ref[3,0,0,0]=o*(-0.543339583883)*x_ref[0]**(o-1)+(-0.594218090249)
        ref[3,0,0,1]=o*(0.463226896714)*x_ref[1]**(o-1)+(0.613466907584)
        ref[3,0,0,2]=o*(0.147935164587)*x_ref[2]**(o-1)+(0.696530808607)
        ref[3,0,1,0]=o*(0.338109512519)*x_ref[0]**(o-1)+(0.281428380284)
        ref[3,0,1,1]=o*(0.0445095482602)*x_ref[1]**(o-1)+(0.898143737155)
        ref[3,0,1,2]=o*(0.0465131079784)*x_ref[2]**(o-1)+(-0.311801195497)
        ref[3,1,0,0]=o*(0.489699597199)*x_ref[0]**(o-1)+(0.198461296558)
        ref[3,1,0,1]=o*(0.87785894323)*x_ref[1]**(o-1)+(0.916226820395)
        ref[3,1,0,2]=o*(-0.473685913201)*x_ref[2]**(o-1)+(-0.496145627682)
        ref[3,1,1,0]=o*(0.249814020596)*x_ref[0]**(o-1)+(-0.894315482252)
        ref[3,1,1,1]=o*(-0.778776373134)*x_ref[1]**(o-1)+(0.760727700644)
        ref[3,1,1,2]=o*(0.743775735588)*x_ref[2]**(o-1)+(-0.689040985439)
        ref[4,0,0,0]=o*(-0.825052728563)*x_ref[0]**(o-1)+(0.522046869865)
        ref[4,0,0,1]=o*(0.333964173928)*x_ref[1]**(o-1)+(-0.76926539114)
        ref[4,0,0,2]=o*(0.412352181661)*x_ref[2]**(o-1)+(-0.0067772174596)
        ref[4,0,1,0]=o*(-0.478610156432)*x_ref[0]**(o-1)+(0.750576911314)
        ref[4,0,1,1]=o*(-0.531813749953)*x_ref[1]**(o-1)+(0.355951176778)
        ref[4,0,1,2]=o*(-0.852853671607)*x_ref[2]**(o-1)+(0.980221079923)
        ref[4,1,0,0]=o*(-0.641389506244)*x_ref[0]**(o-1)+(0.881787042426)
        ref[4,1,0,1]=o*(-0.304080099649)*x_ref[1]**(o-1)+(0.38448900132)
        ref[4,1,0,2]=o*(-0.047930997317)*x_ref[2]**(o-1)+(-0.276038511441)
        ref[4,1,1,0]=o*(0.606877338216)*x_ref[0]**(o-1)+(0.693205674961)
        ref[4,1,1,1]=o*(-0.731785947563)*x_ref[1]**(o-1)+(0.980888111857)
        ref[4,1,1,2]=o*(0.355758103413)*x_ref[2]**(o-1)+(-0.493780352941)
        ref[5,0,0,0]=o*(-0.542680086594)*x_ref[0]**(o-1)+(-0.335712421489)
        ref[5,0,0,1]=o*(0.196257902892)*x_ref[1]**(o-1)+(-0.923381865562)
        ref[5,0,0,2]=o*(0.396049043788)*x_ref[2]**(o-1)+(0.231195219983)
        ref[5,0,1,0]=o*(0.224328300764)*x_ref[0]**(o-1)+(0.306254705923)
        ref[5,0,1,1]=o*(-0.298262471524)*x_ref[1]**(o-1)+(0.318537412793)
        ref[5,0,1,2]=o*(-0.0971742408716)*x_ref[2]**(o-1)+(-0.374440507273)
        ref[5,1,0,0]=o*(-0.408985188836)*x_ref[0]**(o-1)+(-0.932478279672)
        ref[5,1,0,1]=o*(0.700313488604)*x_ref[1]**(o-1)+(0.259720889362)
        ref[5,1,0,2]=o*(0.862207003532)*x_ref[2]**(o-1)+(0.646109275237)
        ref[5,1,1,0]=o*(-0.0341740340851)*x_ref[0]**(o-1)+(0.742119475866)
        ref[5,1,1,1]=o*(0.310640405448)*x_ref[1]**(o-1)+(-0.295841203899)
        ref[5,1,1,2]=o*(-0.778064951853)*x_ref[2]**(o-1)+(-0.847637829972)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_Solution_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(-0.188926722839)*x[0]**o+(0.0528015322778)*x[0]+(0.394368405707)*x[1]**o+(0.573648619197)*x[1]
        ref[0]=o*(-0.188926722839)*x_ref[0]**(o-1)+(0.0528015322778)
        ref[1]=o*(0.394368405707)*x_ref[1]**(o-1)+(0.573648619197)
      else:
        arg=(0.208843709667)*x[0]**o+(-0.461780133723)*x[0]+(0.15028897973)*x[1]**o+(-0.594920423887)*x[1]+(0.517629763426)*x[2]**o+(0.403396285863)*x[2]
        ref[0]=o*(0.208843709667)*x_ref[0]**(o-1)+(-0.461780133723)
        ref[1]=o*(0.15028897973)*x_ref[1]**(o-1)+(-0.594920423887)
        ref[2]=o*(0.517629763426)*x_ref[2]**(o-1)+(0.403396285863)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_Solution_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(0.321445299816)*x[0]**o+(-0.133016832047)*x[0]+(0.663051729844)*x[1]**o+(-0.688058231943)*x[1]
        arg[1]=(-0.0269450330081)*x[0]**o+(0.0944332164606)*x[0]+(-0.285657276114)*x[1]**o+(-0.00151309619508)*x[1]
        ref[0,0]=o*(0.321445299816)*x_ref[0]**(o-1)+(-0.133016832047)
        ref[0,1]=o*(0.663051729844)*x_ref[1]**(o-1)+(-0.688058231943)
        ref[1,0]=o*(-0.0269450330081)*x_ref[0]**(o-1)+(0.0944332164606)
        ref[1,1]=o*(-0.285657276114)*x_ref[1]**(o-1)+(-0.00151309619508)
      else:
        arg[0]=(-0.779418963504)*x[0]**o+(-0.66017852882)*x[0]+(0.0796746454991)*x[1]**o+(-0.559782816375)*x[1]+(0.903645344893)*x[2]**o+(0.752370056114)*x[2]
        arg[1]=(-0.789198080015)*x[0]**o+(-0.61428352272)*x[0]+(-0.0678314090236)*x[1]**o+(-0.592189717525)*x[1]+(0.289759368386)*x[2]**o+(-0.612628874512)*x[2]
        ref[0,0]=o*(-0.779418963504)*x_ref[0]**(o-1)+(-0.66017852882)
        ref[0,1]=o*(0.0796746454991)*x_ref[1]**(o-1)+(-0.559782816375)
        ref[0,2]=o*(0.903645344893)*x_ref[2]**(o-1)+(0.752370056114)
        ref[1,0]=o*(-0.789198080015)*x_ref[0]**(o-1)+(-0.61428352272)
        ref[1,1]=o*(-0.0678314090236)*x_ref[1]**(o-1)+(-0.592189717525)
        ref[1,2]=o*(0.289759368386)*x_ref[2]**(o-1)+(-0.612628874512)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_Solution_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(-0.23566716053)*x[0]**o+(-0.72341376636)*x[0]+(0.972511874619)*x[1]**o+(-0.915617397257)*x[1]
        arg[0,1]=(-0.767927641628)*x[0]**o+(0.381522882363)*x[0]+(0.147312331507)*x[1]**o+(0.593669546711)*x[1]
        arg[0,2]=(0.735354496752)*x[0]**o+(0.0328129903288)*x[0]+(-0.113236521952)*x[1]**o+(0.486761746739)*x[1]
        arg[0,3]=(-0.324209941994)*x[0]**o+(-0.235510827792)*x[0]+(0.167116943517)*x[1]**o+(0.0751459244008)*x[1]
        arg[0,4]=(-0.880038149806)*x[0]**o+(0.250698509807)*x[0]+(-0.249975000596)*x[1]**o+(-0.597287763096)*x[1]
        arg[1,0]=(0.0776176376816)*x[0]**o+(0.719231364575)*x[0]+(-0.573766708326)*x[1]**o+(-0.0657431038913)*x[1]
        arg[1,1]=(0.214586176047)*x[0]**o+(-0.542584637828)*x[0]+(-0.649916769602)*x[1]**o+(0.360674473897)*x[1]
        arg[1,2]=(-0.131191309002)*x[0]**o+(0.00956079049513)*x[0]+(-0.0399067337815)*x[1]**o+(-0.870803424616)*x[1]
        arg[1,3]=(-0.604243786872)*x[0]**o+(0.439677657963)*x[0]+(0.749342339715)*x[1]**o+(0.772192804507)*x[1]
        arg[1,4]=(0.461907488239)*x[0]**o+(-0.957858912937)*x[0]+(0.915893716197)*x[1]**o+(-0.858798101783)*x[1]
        arg[2,0]=(0.922598387398)*x[0]**o+(-0.194676923002)*x[0]+(-0.724972013231)*x[1]**o+(-0.83198174562)*x[1]
        arg[2,1]=(0.62648959348)*x[0]**o+(-0.968390708088)*x[0]+(0.955519507666)*x[1]**o+(0.0140489471727)*x[1]
        arg[2,2]=(-0.488249561275)*x[0]**o+(-0.354607683897)*x[0]+(-0.781278758499)*x[1]**o+(-0.206598205965)*x[1]
        arg[2,3]=(-0.503987693598)*x[0]**o+(0.581909066959)*x[0]+(0.133400791996)*x[1]**o+(-0.234595457546)*x[1]
        arg[2,4]=(-0.768789727431)*x[0]**o+(0.669013649323)*x[0]+(-0.618484878747)*x[1]**o+(-0.287461699457)*x[1]
        arg[3,0]=(0.125603852132)*x[0]**o+(0.83276119445)*x[0]+(-0.643188775199)*x[1]**o+(0.228300434999)*x[1]
        arg[3,1]=(0.645892506866)*x[0]**o+(0.701922167913)*x[0]+(-0.283958147744)*x[1]**o+(-0.781197564279)*x[1]
        arg[3,2]=(-0.0995754594857)*x[0]**o+(-0.106281614694)*x[0]+(0.00912723779796)*x[1]**o+(0.841448365384)*x[1]
        arg[3,3]=(-0.664740577186)*x[0]**o+(0.100401840489)*x[0]+(0.334300978415)*x[1]**o+(-0.383639183037)*x[1]
        arg[3,4]=(0.949394733501)*x[0]**o+(-0.0768203586407)*x[0]+(0.715238033192)*x[1]**o+(-0.175178433047)*x[1]
        ref[0,0,0]=o*(-0.23566716053)*x_ref[0]**(o-1)+(-0.72341376636)
        ref[0,0,1]=o*(0.972511874619)*x_ref[1]**(o-1)+(-0.915617397257)
        ref[0,1,0]=o*(-0.767927641628)*x_ref[0]**(o-1)+(0.381522882363)
        ref[0,1,1]=o*(0.147312331507)*x_ref[1]**(o-1)+(0.593669546711)
        ref[0,2,0]=o*(0.735354496752)*x_ref[0]**(o-1)+(0.0328129903288)
        ref[0,2,1]=o*(-0.113236521952)*x_ref[1]**(o-1)+(0.486761746739)
        ref[0,3,0]=o*(-0.324209941994)*x_ref[0]**(o-1)+(-0.235510827792)
        ref[0,3,1]=o*(0.167116943517)*x_ref[1]**(o-1)+(0.0751459244008)
        ref[0,4,0]=o*(-0.880038149806)*x_ref[0]**(o-1)+(0.250698509807)
        ref[0,4,1]=o*(-0.249975000596)*x_ref[1]**(o-1)+(-0.597287763096)
        ref[1,0,0]=o*(0.0776176376816)*x_ref[0]**(o-1)+(0.719231364575)
        ref[1,0,1]=o*(-0.573766708326)*x_ref[1]**(o-1)+(-0.0657431038913)
        ref[1,1,0]=o*(0.214586176047)*x_ref[0]**(o-1)+(-0.542584637828)
        ref[1,1,1]=o*(-0.649916769602)*x_ref[1]**(o-1)+(0.360674473897)
        ref[1,2,0]=o*(-0.131191309002)*x_ref[0]**(o-1)+(0.00956079049513)
        ref[1,2,1]=o*(-0.0399067337815)*x_ref[1]**(o-1)+(-0.870803424616)
        ref[1,3,0]=o*(-0.604243786872)*x_ref[0]**(o-1)+(0.439677657963)
        ref[1,3,1]=o*(0.749342339715)*x_ref[1]**(o-1)+(0.772192804507)
        ref[1,4,0]=o*(0.461907488239)*x_ref[0]**(o-1)+(-0.957858912937)
        ref[1,4,1]=o*(0.915893716197)*x_ref[1]**(o-1)+(-0.858798101783)
        ref[2,0,0]=o*(0.922598387398)*x_ref[0]**(o-1)+(-0.194676923002)
        ref[2,0,1]=o*(-0.724972013231)*x_ref[1]**(o-1)+(-0.83198174562)
        ref[2,1,0]=o*(0.62648959348)*x_ref[0]**(o-1)+(-0.968390708088)
        ref[2,1,1]=o*(0.955519507666)*x_ref[1]**(o-1)+(0.0140489471727)
        ref[2,2,0]=o*(-0.488249561275)*x_ref[0]**(o-1)+(-0.354607683897)
        ref[2,2,1]=o*(-0.781278758499)*x_ref[1]**(o-1)+(-0.206598205965)
        ref[2,3,0]=o*(-0.503987693598)*x_ref[0]**(o-1)+(0.581909066959)
        ref[2,3,1]=o*(0.133400791996)*x_ref[1]**(o-1)+(-0.234595457546)
        ref[2,4,0]=o*(-0.768789727431)*x_ref[0]**(o-1)+(0.669013649323)
        ref[2,4,1]=o*(-0.618484878747)*x_ref[1]**(o-1)+(-0.287461699457)
        ref[3,0,0]=o*(0.125603852132)*x_ref[0]**(o-1)+(0.83276119445)
        ref[3,0,1]=o*(-0.643188775199)*x_ref[1]**(o-1)+(0.228300434999)
        ref[3,1,0]=o*(0.645892506866)*x_ref[0]**(o-1)+(0.701922167913)
        ref[3,1,1]=o*(-0.283958147744)*x_ref[1]**(o-1)+(-0.781197564279)
        ref[3,2,0]=o*(-0.0995754594857)*x_ref[0]**(o-1)+(-0.106281614694)
        ref[3,2,1]=o*(0.00912723779796)*x_ref[1]**(o-1)+(0.841448365384)
        ref[3,3,0]=o*(-0.664740577186)*x_ref[0]**(o-1)+(0.100401840489)
        ref[3,3,1]=o*(0.334300978415)*x_ref[1]**(o-1)+(-0.383639183037)
        ref[3,4,0]=o*(0.949394733501)*x_ref[0]**(o-1)+(-0.0768203586407)
        ref[3,4,1]=o*(0.715238033192)*x_ref[1]**(o-1)+(-0.175178433047)
      else:
        arg[0,0]=(0.697509098917)*x[0]**o+(-0.571983680004)*x[0]+(0.456254556306)*x[1]**o+(0.407076527516)*x[1]+(-0.379511349609)*x[2]**o+(0.37799055755)*x[2]
        arg[0,1]=(-0.543310012054)*x[0]**o+(0.622613643381)*x[0]+(0.899028609397)*x[1]**o+(0.0358870746022)*x[1]+(0.923875380698)*x[2]**o+(-0.79605331054)*x[2]
        arg[0,2]=(0.613316685189)*x[0]**o+(0.308568047177)*x[0]+(0.0817110033202)*x[1]**o+(0.631167179752)*x[1]+(0.527259723046)*x[2]**o+(-0.44240882282)*x[2]
        arg[0,3]=(0.831261835833)*x[0]**o+(-0.0869775296331)*x[0]+(0.30622781536)*x[1]**o+(0.633056041347)*x[1]+(0.631243241273)*x[2]**o+(0.100112313541)*x[2]
        arg[0,4]=(-0.997030704248)*x[0]**o+(-0.374466395678)*x[0]+(0.0194535973217)*x[1]**o+(0.816505299829)*x[1]+(-0.412869831136)*x[2]**o+(-0.943976805293)*x[2]
        arg[1,0]=(0.555404162574)*x[0]**o+(-0.335389786458)*x[0]+(0.644585507787)*x[1]**o+(0.917280812233)*x[1]+(0.417410132456)*x[2]**o+(-0.26505427144)*x[2]
        arg[1,1]=(0.781185373116)*x[0]**o+(-0.852319927495)*x[0]+(-0.442528270016)*x[1]**o+(-0.538132919634)*x[1]+(-0.604047417588)*x[2]**o+(0.727395882535)*x[2]
        arg[1,2]=(-0.272462727471)*x[0]**o+(-0.834077558378)*x[0]+(0.763269297137)*x[1]**o+(-0.823180222291)*x[1]+(-0.162146758865)*x[2]**o+(0.595529366226)*x[2]
        arg[1,3]=(-0.0468429326976)*x[0]**o+(-0.179116342871)*x[0]+(-0.640563576973)*x[1]**o+(0.914070705704)*x[1]+(0.725297040984)*x[2]**o+(0.870302256055)*x[2]
        arg[1,4]=(-0.0632444730547)*x[0]**o+(-0.266315658984)*x[0]+(0.533681589116)*x[1]**o+(0.709939511175)*x[1]+(-0.60554532679)*x[2]**o+(0.915224044125)*x[2]
        arg[2,0]=(0.23330035938)*x[0]**o+(-0.385419740264)*x[0]+(0.120424890588)*x[1]**o+(0.591983224766)*x[1]+(0.925564960339)*x[2]**o+(-0.602528014193)*x[2]
        arg[2,1]=(0.786024569139)*x[0]**o+(0.619287163829)*x[0]+(-0.722187904611)*x[1]**o+(0.275352886478)*x[1]+(0.723591624164)*x[2]**o+(0.0909486746072)*x[2]
        arg[2,2]=(0.766293557291)*x[0]**o+(0.0610694643295)*x[0]+(0.375714644104)*x[1]**o+(-0.772092520637)*x[1]+(0.775058332825)*x[2]**o+(0.533525926942)*x[2]
        arg[2,3]=(-0.67254491898)*x[0]**o+(-0.0926113197516)*x[0]+(-0.615891770107)*x[1]**o+(-0.616600504392)*x[1]+(-0.808251600122)*x[2]**o+(-0.602309670223)*x[2]
        arg[2,4]=(0.80242562495)*x[0]**o+(0.359171734464)*x[0]+(-0.560256436586)*x[1]**o+(-0.420334736194)*x[1]+(0.871043977034)*x[2]**o+(0.156212055829)*x[2]
        arg[3,0]=(-0.0569856826934)*x[0]**o+(-0.381868544897)*x[0]+(-0.396013644801)*x[1]**o+(0.695329212849)*x[1]+(0.201188831017)*x[2]**o+(0.471078018432)*x[2]
        arg[3,1]=(0.28457474179)*x[0]**o+(-0.623305572154)*x[0]+(-0.442659858168)*x[1]**o+(0.725900635371)*x[1]+(-0.908206119008)*x[2]**o+(-0.190223696293)*x[2]
        arg[3,2]=(0.568043480074)*x[0]**o+(0.942235524274)*x[0]+(-0.720727900911)*x[1]**o+(0.771808109591)*x[1]+(-0.538996050264)*x[2]**o+(-0.641448383256)*x[2]
        arg[3,3]=(0.00707362989382)*x[0]**o+(0.139812417833)*x[0]+(-0.179986304531)*x[1]**o+(-0.546332565953)*x[1]+(-0.687783685855)*x[2]**o+(0.292513389166)*x[2]
        arg[3,4]=(0.657315930332)*x[0]**o+(0.0252304793386)*x[0]+(0.540737656651)*x[1]**o+(0.832327802479)*x[1]+(-0.822641904799)*x[2]**o+(-0.599972412377)*x[2]
        ref[0,0,0]=o*(0.697509098917)*x_ref[0]**(o-1)+(-0.571983680004)
        ref[0,0,1]=o*(0.456254556306)*x_ref[1]**(o-1)+(0.407076527516)
        ref[0,0,2]=o*(-0.379511349609)*x_ref[2]**(o-1)+(0.37799055755)
        ref[0,1,0]=o*(-0.543310012054)*x_ref[0]**(o-1)+(0.622613643381)
        ref[0,1,1]=o*(0.899028609397)*x_ref[1]**(o-1)+(0.0358870746022)
        ref[0,1,2]=o*(0.923875380698)*x_ref[2]**(o-1)+(-0.79605331054)
        ref[0,2,0]=o*(0.613316685189)*x_ref[0]**(o-1)+(0.308568047177)
        ref[0,2,1]=o*(0.0817110033202)*x_ref[1]**(o-1)+(0.631167179752)
        ref[0,2,2]=o*(0.527259723046)*x_ref[2]**(o-1)+(-0.44240882282)
        ref[0,3,0]=o*(0.831261835833)*x_ref[0]**(o-1)+(-0.0869775296331)
        ref[0,3,1]=o*(0.30622781536)*x_ref[1]**(o-1)+(0.633056041347)
        ref[0,3,2]=o*(0.631243241273)*x_ref[2]**(o-1)+(0.100112313541)
        ref[0,4,0]=o*(-0.997030704248)*x_ref[0]**(o-1)+(-0.374466395678)
        ref[0,4,1]=o*(0.0194535973217)*x_ref[1]**(o-1)+(0.816505299829)
        ref[0,4,2]=o*(-0.412869831136)*x_ref[2]**(o-1)+(-0.943976805293)
        ref[1,0,0]=o*(0.555404162574)*x_ref[0]**(o-1)+(-0.335389786458)
        ref[1,0,1]=o*(0.644585507787)*x_ref[1]**(o-1)+(0.917280812233)
        ref[1,0,2]=o*(0.417410132456)*x_ref[2]**(o-1)+(-0.26505427144)
        ref[1,1,0]=o*(0.781185373116)*x_ref[0]**(o-1)+(-0.852319927495)
        ref[1,1,1]=o*(-0.442528270016)*x_ref[1]**(o-1)+(-0.538132919634)
        ref[1,1,2]=o*(-0.604047417588)*x_ref[2]**(o-1)+(0.727395882535)
        ref[1,2,0]=o*(-0.272462727471)*x_ref[0]**(o-1)+(-0.834077558378)
        ref[1,2,1]=o*(0.763269297137)*x_ref[1]**(o-1)+(-0.823180222291)
        ref[1,2,2]=o*(-0.162146758865)*x_ref[2]**(o-1)+(0.595529366226)
        ref[1,3,0]=o*(-0.0468429326976)*x_ref[0]**(o-1)+(-0.179116342871)
        ref[1,3,1]=o*(-0.640563576973)*x_ref[1]**(o-1)+(0.914070705704)
        ref[1,3,2]=o*(0.725297040984)*x_ref[2]**(o-1)+(0.870302256055)
        ref[1,4,0]=o*(-0.0632444730547)*x_ref[0]**(o-1)+(-0.266315658984)
        ref[1,4,1]=o*(0.533681589116)*x_ref[1]**(o-1)+(0.709939511175)
        ref[1,4,2]=o*(-0.60554532679)*x_ref[2]**(o-1)+(0.915224044125)
        ref[2,0,0]=o*(0.23330035938)*x_ref[0]**(o-1)+(-0.385419740264)
        ref[2,0,1]=o*(0.120424890588)*x_ref[1]**(o-1)+(0.591983224766)
        ref[2,0,2]=o*(0.925564960339)*x_ref[2]**(o-1)+(-0.602528014193)
        ref[2,1,0]=o*(0.786024569139)*x_ref[0]**(o-1)+(0.619287163829)
        ref[2,1,1]=o*(-0.722187904611)*x_ref[1]**(o-1)+(0.275352886478)
        ref[2,1,2]=o*(0.723591624164)*x_ref[2]**(o-1)+(0.0909486746072)
        ref[2,2,0]=o*(0.766293557291)*x_ref[0]**(o-1)+(0.0610694643295)
        ref[2,2,1]=o*(0.375714644104)*x_ref[1]**(o-1)+(-0.772092520637)
        ref[2,2,2]=o*(0.775058332825)*x_ref[2]**(o-1)+(0.533525926942)
        ref[2,3,0]=o*(-0.67254491898)*x_ref[0]**(o-1)+(-0.0926113197516)
        ref[2,3,1]=o*(-0.615891770107)*x_ref[1]**(o-1)+(-0.616600504392)
        ref[2,3,2]=o*(-0.808251600122)*x_ref[2]**(o-1)+(-0.602309670223)
        ref[2,4,0]=o*(0.80242562495)*x_ref[0]**(o-1)+(0.359171734464)
        ref[2,4,1]=o*(-0.560256436586)*x_ref[1]**(o-1)+(-0.420334736194)
        ref[2,4,2]=o*(0.871043977034)*x_ref[2]**(o-1)+(0.156212055829)
        ref[3,0,0]=o*(-0.0569856826934)*x_ref[0]**(o-1)+(-0.381868544897)
        ref[3,0,1]=o*(-0.396013644801)*x_ref[1]**(o-1)+(0.695329212849)
        ref[3,0,2]=o*(0.201188831017)*x_ref[2]**(o-1)+(0.471078018432)
        ref[3,1,0]=o*(0.28457474179)*x_ref[0]**(o-1)+(-0.623305572154)
        ref[3,1,1]=o*(-0.442659858168)*x_ref[1]**(o-1)+(0.725900635371)
        ref[3,1,2]=o*(-0.908206119008)*x_ref[2]**(o-1)+(-0.190223696293)
        ref[3,2,0]=o*(0.568043480074)*x_ref[0]**(o-1)+(0.942235524274)
        ref[3,2,1]=o*(-0.720727900911)*x_ref[1]**(o-1)+(0.771808109591)
        ref[3,2,2]=o*(-0.538996050264)*x_ref[2]**(o-1)+(-0.641448383256)
        ref[3,3,0]=o*(0.00707362989382)*x_ref[0]**(o-1)+(0.139812417833)
        ref[3,3,1]=o*(-0.179986304531)*x_ref[1]**(o-1)+(-0.546332565953)
        ref[3,3,2]=o*(-0.687783685855)*x_ref[2]**(o-1)+(0.292513389166)
        ref[3,4,0]=o*(0.657315930332)*x_ref[0]**(o-1)+(0.0252304793386)
        ref[3,4,1]=o*(0.540737656651)*x_ref[1]**(o-1)+(0.832327802479)
        ref[3,4,2]=o*(-0.822641904799)*x_ref[2]**(o-1)+(-0.599972412377)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_Solution_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.711259674136)*x[0]**o+(0.597099394118)*x[0]+(0.656676641605)*x[1]**o+(0.800065074626)*x[1]
        arg[0,0,1]=(0.410575880869)*x[0]**o+(0.0290305819624)*x[0]+(-0.746442034948)*x[1]**o+(0.074826200729)*x[1]
        arg[0,1,0]=(0.478073638149)*x[0]**o+(0.856920711223)*x[0]+(0.517173340088)*x[1]**o+(-0.677846282063)*x[1]
        arg[0,1,1]=(0.285231489647)*x[0]**o+(0.0756593965609)*x[0]+(-0.355221676478)*x[1]**o+(-0.520564308221)*x[1]
        arg[1,0,0]=(-0.368777104698)*x[0]**o+(0.474996103371)*x[0]+(0.675654207136)*x[1]**o+(-0.0324979517659)*x[1]
        arg[1,0,1]=(0.498857835281)*x[0]**o+(-0.73836840989)*x[0]+(0.362789682422)*x[1]**o+(0.315425076703)*x[1]
        arg[1,1,0]=(-0.325384820094)*x[0]**o+(-0.895297521761)*x[0]+(0.563090918504)*x[1]**o+(0.589834648825)*x[1]
        arg[1,1,1]=(0.0378060061714)*x[0]**o+(-0.429064920806)*x[0]+(-0.467324629077)*x[1]**o+(0.938161415082)*x[1]
        arg[2,0,0]=(-0.527827738902)*x[0]**o+(-0.534425169092)*x[0]+(-0.185111533531)*x[1]**o+(-0.810076656622)*x[1]
        arg[2,0,1]=(-0.221712302033)*x[0]**o+(0.883619587624)*x[0]+(-0.972912188123)*x[1]**o+(0.0780565750992)*x[1]
        arg[2,1,0]=(-0.752942706408)*x[0]**o+(-0.153790659605)*x[0]+(-0.398384592556)*x[1]**o+(0.657319049074)*x[1]
        arg[2,1,1]=(0.428492876383)*x[0]**o+(0.427090189076)*x[0]+(-0.017400284896)*x[1]**o+(-0.730635641841)*x[1]
        arg[3,0,0]=(-0.644602330168)*x[0]**o+(0.706532342264)*x[0]+(-0.64532460698)*x[1]**o+(0.850059843173)*x[1]
        arg[3,0,1]=(-0.63066688687)*x[0]**o+(0.578132047851)*x[0]+(0.413335880906)*x[1]**o+(0.611233278281)*x[1]
        arg[3,1,0]=(-0.247378556309)*x[0]**o+(-0.631688468936)*x[0]+(-0.898130068425)*x[1]**o+(0.147210661251)*x[1]
        arg[3,1,1]=(0.979026126394)*x[0]**o+(-0.231003559285)*x[0]+(-0.884530953901)*x[1]**o+(-0.453907922804)*x[1]
        arg[4,0,0]=(-0.584910281652)*x[0]**o+(-0.808929690751)*x[0]+(0.661854511996)*x[1]**o+(0.262336033771)*x[1]
        arg[4,0,1]=(0.32006961919)*x[0]**o+(0.407637671852)*x[0]+(-0.36617527544)*x[1]**o+(-0.28786191714)*x[1]
        arg[4,1,0]=(0.718670448416)*x[0]**o+(-0.836956284868)*x[0]+(0.694744974874)*x[1]**o+(-0.176920245996)*x[1]
        arg[4,1,1]=(-0.892867770142)*x[0]**o+(-0.439240069099)*x[0]+(0.960956271989)*x[1]**o+(-0.559541152716)*x[1]
        arg[5,0,0]=(-0.159459484106)*x[0]**o+(0.497366075466)*x[0]+(0.776343693655)*x[1]**o+(0.326882397775)*x[1]
        arg[5,0,1]=(-0.769197598992)*x[0]**o+(-0.155931232816)*x[0]+(0.306889959757)*x[1]**o+(0.283798117232)*x[1]
        arg[5,1,0]=(0.329987736778)*x[0]**o+(-0.181917315586)*x[0]+(0.896508353752)*x[1]**o+(0.505276687831)*x[1]
        arg[5,1,1]=(-0.805264678584)*x[0]**o+(0.1541429191)*x[0]+(0.759370357364)*x[1]**o+(-0.858941373648)*x[1]
        ref[0,0,0,0]=o*(0.711259674136)*x_ref[0]**(o-1)+(0.597099394118)
        ref[0,0,0,1]=o*(0.656676641605)*x_ref[1]**(o-1)+(0.800065074626)
        ref[0,0,1,0]=o*(0.410575880869)*x_ref[0]**(o-1)+(0.0290305819624)
        ref[0,0,1,1]=o*(-0.746442034948)*x_ref[1]**(o-1)+(0.074826200729)
        ref[0,1,0,0]=o*(0.478073638149)*x_ref[0]**(o-1)+(0.856920711223)
        ref[0,1,0,1]=o*(0.517173340088)*x_ref[1]**(o-1)+(-0.677846282063)
        ref[0,1,1,0]=o*(0.285231489647)*x_ref[0]**(o-1)+(0.0756593965609)
        ref[0,1,1,1]=o*(-0.355221676478)*x_ref[1]**(o-1)+(-0.520564308221)
        ref[1,0,0,0]=o*(-0.368777104698)*x_ref[0]**(o-1)+(0.474996103371)
        ref[1,0,0,1]=o*(0.675654207136)*x_ref[1]**(o-1)+(-0.0324979517659)
        ref[1,0,1,0]=o*(0.498857835281)*x_ref[0]**(o-1)+(-0.73836840989)
        ref[1,0,1,1]=o*(0.362789682422)*x_ref[1]**(o-1)+(0.315425076703)
        ref[1,1,0,0]=o*(-0.325384820094)*x_ref[0]**(o-1)+(-0.895297521761)
        ref[1,1,0,1]=o*(0.563090918504)*x_ref[1]**(o-1)+(0.589834648825)
        ref[1,1,1,0]=o*(0.0378060061714)*x_ref[0]**(o-1)+(-0.429064920806)
        ref[1,1,1,1]=o*(-0.467324629077)*x_ref[1]**(o-1)+(0.938161415082)
        ref[2,0,0,0]=o*(-0.527827738902)*x_ref[0]**(o-1)+(-0.534425169092)
        ref[2,0,0,1]=o*(-0.185111533531)*x_ref[1]**(o-1)+(-0.810076656622)
        ref[2,0,1,0]=o*(-0.221712302033)*x_ref[0]**(o-1)+(0.883619587624)
        ref[2,0,1,1]=o*(-0.972912188123)*x_ref[1]**(o-1)+(0.0780565750992)
        ref[2,1,0,0]=o*(-0.752942706408)*x_ref[0]**(o-1)+(-0.153790659605)
        ref[2,1,0,1]=o*(-0.398384592556)*x_ref[1]**(o-1)+(0.657319049074)
        ref[2,1,1,0]=o*(0.428492876383)*x_ref[0]**(o-1)+(0.427090189076)
        ref[2,1,1,1]=o*(-0.017400284896)*x_ref[1]**(o-1)+(-0.730635641841)
        ref[3,0,0,0]=o*(-0.644602330168)*x_ref[0]**(o-1)+(0.706532342264)
        ref[3,0,0,1]=o*(-0.64532460698)*x_ref[1]**(o-1)+(0.850059843173)
        ref[3,0,1,0]=o*(-0.63066688687)*x_ref[0]**(o-1)+(0.578132047851)
        ref[3,0,1,1]=o*(0.413335880906)*x_ref[1]**(o-1)+(0.611233278281)
        ref[3,1,0,0]=o*(-0.247378556309)*x_ref[0]**(o-1)+(-0.631688468936)
        ref[3,1,0,1]=o*(-0.898130068425)*x_ref[1]**(o-1)+(0.147210661251)
        ref[3,1,1,0]=o*(0.979026126394)*x_ref[0]**(o-1)+(-0.231003559285)
        ref[3,1,1,1]=o*(-0.884530953901)*x_ref[1]**(o-1)+(-0.453907922804)
        ref[4,0,0,0]=o*(-0.584910281652)*x_ref[0]**(o-1)+(-0.808929690751)
        ref[4,0,0,1]=o*(0.661854511996)*x_ref[1]**(o-1)+(0.262336033771)
        ref[4,0,1,0]=o*(0.32006961919)*x_ref[0]**(o-1)+(0.407637671852)
        ref[4,0,1,1]=o*(-0.36617527544)*x_ref[1]**(o-1)+(-0.28786191714)
        ref[4,1,0,0]=o*(0.718670448416)*x_ref[0]**(o-1)+(-0.836956284868)
        ref[4,1,0,1]=o*(0.694744974874)*x_ref[1]**(o-1)+(-0.176920245996)
        ref[4,1,1,0]=o*(-0.892867770142)*x_ref[0]**(o-1)+(-0.439240069099)
        ref[4,1,1,1]=o*(0.960956271989)*x_ref[1]**(o-1)+(-0.559541152716)
        ref[5,0,0,0]=o*(-0.159459484106)*x_ref[0]**(o-1)+(0.497366075466)
        ref[5,0,0,1]=o*(0.776343693655)*x_ref[1]**(o-1)+(0.326882397775)
        ref[5,0,1,0]=o*(-0.769197598992)*x_ref[0]**(o-1)+(-0.155931232816)
        ref[5,0,1,1]=o*(0.306889959757)*x_ref[1]**(o-1)+(0.283798117232)
        ref[5,1,0,0]=o*(0.329987736778)*x_ref[0]**(o-1)+(-0.181917315586)
        ref[5,1,0,1]=o*(0.896508353752)*x_ref[1]**(o-1)+(0.505276687831)
        ref[5,1,1,0]=o*(-0.805264678584)*x_ref[0]**(o-1)+(0.1541429191)
        ref[5,1,1,1]=o*(0.759370357364)*x_ref[1]**(o-1)+(-0.858941373648)
      else:
        arg[0,0,0]=(0.168268053227)*x[0]**o+(-0.693455720059)*x[0]+(-0.929660815724)*x[1]**o+(0.986095168154)*x[1]+(0.736239785757)*x[2]**o+(-0.852231596742)*x[2]
        arg[0,0,1]=(-0.246755690586)*x[0]**o+(0.374568513964)*x[0]+(-0.964930789849)*x[1]**o+(0.911595670675)*x[1]+(-0.450825843592)*x[2]**o+(0.109918389584)*x[2]
        arg[0,1,0]=(-0.733872890764)*x[0]**o+(-0.673737586895)*x[0]+(0.948528017502)*x[1]**o+(0.00135698010622)*x[1]+(0.960702320728)*x[2]**o+(-0.0223793346126)*x[2]
        arg[0,1,1]=(0.117470384547)*x[0]**o+(0.390065805504)*x[0]+(0.116939897886)*x[1]**o+(0.201756470627)*x[1]+(0.649311190776)*x[2]**o+(-0.388330674246)*x[2]
        arg[1,0,0]=(0.848074231014)*x[0]**o+(-0.431699971504)*x[0]+(0.956477578677)*x[1]**o+(0.522633753414)*x[1]+(0.147733403055)*x[2]**o+(0.136817362225)*x[2]
        arg[1,0,1]=(0.482934543775)*x[0]**o+(-0.358181242578)*x[0]+(-0.013088278343)*x[1]**o+(0.002654195003)*x[1]+(0.960898421715)*x[2]**o+(0.23166323571)*x[2]
        arg[1,1,0]=(-0.60469206115)*x[0]**o+(-0.449073005118)*x[0]+(-0.0774626329204)*x[1]**o+(-0.135772353437)*x[1]+(-0.796309060217)*x[2]**o+(-0.0601212417342)*x[2]
        arg[1,1,1]=(0.41742915222)*x[0]**o+(-0.907531578792)*x[0]+(-0.998018453394)*x[1]**o+(0.38863222527)*x[1]+(-0.941909949653)*x[2]**o+(-0.962025400733)*x[2]
        arg[2,0,0]=(0.677803680186)*x[0]**o+(0.493757303449)*x[0]+(-0.700353467324)*x[1]**o+(0.713489396377)*x[1]+(-0.147276750564)*x[2]**o+(-0.666815984705)*x[2]
        arg[2,0,1]=(-0.196166170445)*x[0]**o+(-0.776357160477)*x[0]+(0.531757851468)*x[1]**o+(-0.0951663105473)*x[1]+(0.296695659682)*x[2]**o+(0.132047740242)*x[2]
        arg[2,1,0]=(0.89703218788)*x[0]**o+(0.446461958293)*x[0]+(0.849781349832)*x[1]**o+(0.495003438762)*x[1]+(0.200866318394)*x[2]**o+(-0.239158244355)*x[2]
        arg[2,1,1]=(0.699816693)*x[0]**o+(-0.498016262468)*x[0]+(0.467212425781)*x[1]**o+(0.507210685133)*x[1]+(0.872000089805)*x[2]**o+(-0.608415132584)*x[2]
        arg[3,0,0]=(-0.858779999521)*x[0]**o+(-0.934307938564)*x[0]+(0.998531969194)*x[1]**o+(-0.10694369616)*x[1]+(0.565515051636)*x[2]**o+(-0.340328286843)*x[2]
        arg[3,0,1]=(-0.734447160975)*x[0]**o+(0.592969337009)*x[0]+(0.769648460523)*x[1]**o+(0.939994123398)*x[1]+(-0.14168879251)*x[2]**o+(-0.151602759825)*x[2]
        arg[3,1,0]=(-0.204447368694)*x[0]**o+(-0.149512701586)*x[0]+(0.496643987821)*x[1]**o+(0.137154021271)*x[1]+(-0.207451973465)*x[2]**o+(0.976289933313)*x[2]
        arg[3,1,1]=(-0.498939396392)*x[0]**o+(-0.388761371521)*x[0]+(0.918876234523)*x[1]**o+(0.326334103437)*x[1]+(0.135801853065)*x[2]**o+(-0.513268577694)*x[2]
        arg[4,0,0]=(0.383369953978)*x[0]**o+(0.262190952188)*x[0]+(0.226913840907)*x[1]**o+(0.625767074123)*x[1]+(0.0736409418752)*x[2]**o+(-0.681643359069)*x[2]
        arg[4,0,1]=(-0.00395633017219)*x[0]**o+(0.103598757494)*x[0]+(0.245388867438)*x[1]**o+(0.0199139065324)*x[1]+(-0.85935209669)*x[2]**o+(0.780770429677)*x[2]
        arg[4,1,0]=(0.93284113054)*x[0]**o+(0.192867707515)*x[0]+(0.354258046046)*x[1]**o+(0.0244899514488)*x[1]+(-0.00723308587137)*x[2]**o+(0.724287748378)*x[2]
        arg[4,1,1]=(-0.701870704251)*x[0]**o+(0.29146856048)*x[0]+(-0.677358067691)*x[1]**o+(-0.055827889309)*x[1]+(0.96248585097)*x[2]**o+(0.100526700803)*x[2]
        arg[5,0,0]=(-0.582058132348)*x[0]**o+(0.5354038569)*x[0]+(-0.608415656952)*x[1]**o+(0.700352801694)*x[1]+(-0.705438310252)*x[2]**o+(0.969824711925)*x[2]
        arg[5,0,1]=(-0.573326444971)*x[0]**o+(0.00490482707944)*x[0]+(0.300312254171)*x[1]**o+(-0.620154697788)*x[1]+(-0.0703788860867)*x[2]**o+(-0.608567609941)*x[2]
        arg[5,1,0]=(0.346782282755)*x[0]**o+(-0.022146949553)*x[0]+(0.548359292626)*x[1]**o+(0.314315074761)*x[1]+(0.164808336191)*x[2]**o+(-0.983018613943)*x[2]
        arg[5,1,1]=(-0.357126808896)*x[0]**o+(0.547191129837)*x[0]+(-0.0128153418859)*x[1]**o+(-0.199507147279)*x[1]+(-0.637546467191)*x[2]**o+(0.65056997574)*x[2]
        ref[0,0,0,0]=o*(0.168268053227)*x_ref[0]**(o-1)+(-0.693455720059)
        ref[0,0,0,1]=o*(-0.929660815724)*x_ref[1]**(o-1)+(0.986095168154)
        ref[0,0,0,2]=o*(0.736239785757)*x_ref[2]**(o-1)+(-0.852231596742)
        ref[0,0,1,0]=o*(-0.246755690586)*x_ref[0]**(o-1)+(0.374568513964)
        ref[0,0,1,1]=o*(-0.964930789849)*x_ref[1]**(o-1)+(0.911595670675)
        ref[0,0,1,2]=o*(-0.450825843592)*x_ref[2]**(o-1)+(0.109918389584)
        ref[0,1,0,0]=o*(-0.733872890764)*x_ref[0]**(o-1)+(-0.673737586895)
        ref[0,1,0,1]=o*(0.948528017502)*x_ref[1]**(o-1)+(0.00135698010622)
        ref[0,1,0,2]=o*(0.960702320728)*x_ref[2]**(o-1)+(-0.0223793346126)
        ref[0,1,1,0]=o*(0.117470384547)*x_ref[0]**(o-1)+(0.390065805504)
        ref[0,1,1,1]=o*(0.116939897886)*x_ref[1]**(o-1)+(0.201756470627)
        ref[0,1,1,2]=o*(0.649311190776)*x_ref[2]**(o-1)+(-0.388330674246)
        ref[1,0,0,0]=o*(0.848074231014)*x_ref[0]**(o-1)+(-0.431699971504)
        ref[1,0,0,1]=o*(0.956477578677)*x_ref[1]**(o-1)+(0.522633753414)
        ref[1,0,0,2]=o*(0.147733403055)*x_ref[2]**(o-1)+(0.136817362225)
        ref[1,0,1,0]=o*(0.482934543775)*x_ref[0]**(o-1)+(-0.358181242578)
        ref[1,0,1,1]=o*(-0.013088278343)*x_ref[1]**(o-1)+(0.002654195003)
        ref[1,0,1,2]=o*(0.960898421715)*x_ref[2]**(o-1)+(0.23166323571)
        ref[1,1,0,0]=o*(-0.60469206115)*x_ref[0]**(o-1)+(-0.449073005118)
        ref[1,1,0,1]=o*(-0.0774626329204)*x_ref[1]**(o-1)+(-0.135772353437)
        ref[1,1,0,2]=o*(-0.796309060217)*x_ref[2]**(o-1)+(-0.0601212417342)
        ref[1,1,1,0]=o*(0.41742915222)*x_ref[0]**(o-1)+(-0.907531578792)
        ref[1,1,1,1]=o*(-0.998018453394)*x_ref[1]**(o-1)+(0.38863222527)
        ref[1,1,1,2]=o*(-0.941909949653)*x_ref[2]**(o-1)+(-0.962025400733)
        ref[2,0,0,0]=o*(0.677803680186)*x_ref[0]**(o-1)+(0.493757303449)
        ref[2,0,0,1]=o*(-0.700353467324)*x_ref[1]**(o-1)+(0.713489396377)
        ref[2,0,0,2]=o*(-0.147276750564)*x_ref[2]**(o-1)+(-0.666815984705)
        ref[2,0,1,0]=o*(-0.196166170445)*x_ref[0]**(o-1)+(-0.776357160477)
        ref[2,0,1,1]=o*(0.531757851468)*x_ref[1]**(o-1)+(-0.0951663105473)
        ref[2,0,1,2]=o*(0.296695659682)*x_ref[2]**(o-1)+(0.132047740242)
        ref[2,1,0,0]=o*(0.89703218788)*x_ref[0]**(o-1)+(0.446461958293)
        ref[2,1,0,1]=o*(0.849781349832)*x_ref[1]**(o-1)+(0.495003438762)
        ref[2,1,0,2]=o*(0.200866318394)*x_ref[2]**(o-1)+(-0.239158244355)
        ref[2,1,1,0]=o*(0.699816693)*x_ref[0]**(o-1)+(-0.498016262468)
        ref[2,1,1,1]=o*(0.467212425781)*x_ref[1]**(o-1)+(0.507210685133)
        ref[2,1,1,2]=o*(0.872000089805)*x_ref[2]**(o-1)+(-0.608415132584)
        ref[3,0,0,0]=o*(-0.858779999521)*x_ref[0]**(o-1)+(-0.934307938564)
        ref[3,0,0,1]=o*(0.998531969194)*x_ref[1]**(o-1)+(-0.10694369616)
        ref[3,0,0,2]=o*(0.565515051636)*x_ref[2]**(o-1)+(-0.340328286843)
        ref[3,0,1,0]=o*(-0.734447160975)*x_ref[0]**(o-1)+(0.592969337009)
        ref[3,0,1,1]=o*(0.769648460523)*x_ref[1]**(o-1)+(0.939994123398)
        ref[3,0,1,2]=o*(-0.14168879251)*x_ref[2]**(o-1)+(-0.151602759825)
        ref[3,1,0,0]=o*(-0.204447368694)*x_ref[0]**(o-1)+(-0.149512701586)
        ref[3,1,0,1]=o*(0.496643987821)*x_ref[1]**(o-1)+(0.137154021271)
        ref[3,1,0,2]=o*(-0.207451973465)*x_ref[2]**(o-1)+(0.976289933313)
        ref[3,1,1,0]=o*(-0.498939396392)*x_ref[0]**(o-1)+(-0.388761371521)
        ref[3,1,1,1]=o*(0.918876234523)*x_ref[1]**(o-1)+(0.326334103437)
        ref[3,1,1,2]=o*(0.135801853065)*x_ref[2]**(o-1)+(-0.513268577694)
        ref[4,0,0,0]=o*(0.383369953978)*x_ref[0]**(o-1)+(0.262190952188)
        ref[4,0,0,1]=o*(0.226913840907)*x_ref[1]**(o-1)+(0.625767074123)
        ref[4,0,0,2]=o*(0.0736409418752)*x_ref[2]**(o-1)+(-0.681643359069)
        ref[4,0,1,0]=o*(-0.00395633017219)*x_ref[0]**(o-1)+(0.103598757494)
        ref[4,0,1,1]=o*(0.245388867438)*x_ref[1]**(o-1)+(0.0199139065324)
        ref[4,0,1,2]=o*(-0.85935209669)*x_ref[2]**(o-1)+(0.780770429677)
        ref[4,1,0,0]=o*(0.93284113054)*x_ref[0]**(o-1)+(0.192867707515)
        ref[4,1,0,1]=o*(0.354258046046)*x_ref[1]**(o-1)+(0.0244899514488)
        ref[4,1,0,2]=o*(-0.00723308587137)*x_ref[2]**(o-1)+(0.724287748378)
        ref[4,1,1,0]=o*(-0.701870704251)*x_ref[0]**(o-1)+(0.29146856048)
        ref[4,1,1,1]=o*(-0.677358067691)*x_ref[1]**(o-1)+(-0.055827889309)
        ref[4,1,1,2]=o*(0.96248585097)*x_ref[2]**(o-1)+(0.100526700803)
        ref[5,0,0,0]=o*(-0.582058132348)*x_ref[0]**(o-1)+(0.5354038569)
        ref[5,0,0,1]=o*(-0.608415656952)*x_ref[1]**(o-1)+(0.700352801694)
        ref[5,0,0,2]=o*(-0.705438310252)*x_ref[2]**(o-1)+(0.969824711925)
        ref[5,0,1,0]=o*(-0.573326444971)*x_ref[0]**(o-1)+(0.00490482707944)
        ref[5,0,1,1]=o*(0.300312254171)*x_ref[1]**(o-1)+(-0.620154697788)
        ref[5,0,1,2]=o*(-0.0703788860867)*x_ref[2]**(o-1)+(-0.608567609941)
        ref[5,1,0,0]=o*(0.346782282755)*x_ref[0]**(o-1)+(-0.022146949553)
        ref[5,1,0,1]=o*(0.548359292626)*x_ref[1]**(o-1)+(0.314315074761)
        ref[5,1,0,2]=o*(0.164808336191)*x_ref[2]**(o-1)+(-0.983018613943)
        ref[5,1,1,0]=o*(-0.357126808896)*x_ref[0]**(o-1)+(0.547191129837)
        ref[5,1,1,1]=o*(-0.0128153418859)*x_ref[1]**(o-1)+(-0.199507147279)
        ref[5,1,1,2]=o*(-0.637546467191)*x_ref[2]**(o-1)+(0.65056997574)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.694260485776)*x[0]+(1.46246375886)*x[1]
        ref[0]=(0.694260485776)
        ref[1]=(1.46246375886)
      else:
        arg=(-1.10912081576)*x[0]+(1.19790363484)*x[1]+(0.555862453192)*x[2]
        ref[0]=(-1.10912081576)
        ref[1]=(1.19790363484)
        ref[2]=(0.555862453192)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.0963683715208)*x[0]+(-0.985421300426)*x[1]
        arg[1]=(0.50107481561)*x[0]+(0.296191075154)*x[1]
        ref[0,0]=(-0.0963683715208)
        ref[0,1]=(-0.985421300426)
        ref[1,0]=(0.50107481561)
        ref[1,1]=(0.296191075154)
      else:
        arg[0]=(0.790729361248)*x[0]+(0.3673431791)*x[1]+(0.031915702612)*x[2]
        arg[1]=(-0.218629391175)*x[0]+(-0.0619831001578)*x[1]+(-1.46460054046)*x[2]
        ref[0,0]=(0.790729361248)
        ref[0,1]=(0.3673431791)
        ref[0,2]=(0.031915702612)
        ref[1,0]=(-0.218629391175)
        ref[1,1]=(-0.0619831001578)
        ref[1,2]=(-1.46460054046)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.102266041928)*x[0]+(0.962985129019)*x[1]
        arg[0,1]=(1.22474578409)*x[0]+(-0.211640772922)*x[1]
        arg[0,2]=(-0.51306749275)*x[0]+(0.652502950912)*x[1]
        arg[0,3]=(1.89516926008)*x[0]+(0.317996805009)*x[1]
        arg[0,4]=(0.562531430083)*x[0]+(-0.194650166716)*x[1]
        arg[1,0]=(0.344353002434)*x[0]+(-0.830671914713)*x[1]
        arg[1,1]=(0.984363231775)*x[0]+(-0.0773767796506)*x[1]
        arg[1,2]=(-0.893978692143)*x[0]+(1.43362006873)*x[1]
        arg[1,3]=(0.603459290036)*x[0]+(-0.527471281229)*x[1]
        arg[1,4]=(-0.768879816113)*x[0]+(0.670739267079)*x[1]
        arg[2,0]=(0.466692398942)*x[0]+(-0.315362658981)*x[1]
        arg[2,1]=(1.30065290708)*x[0]+(-0.202369755569)*x[1]
        arg[2,2]=(-0.114571246331)*x[0]+(1.30250121342)*x[1]
        arg[2,3]=(-0.243608494456)*x[0]+(-0.103398411275)*x[1]
        arg[2,4]=(-0.262568543724)*x[0]+(-0.23979204654)*x[1]
        arg[3,0]=(0.0334727960327)*x[0]+(-0.24547366383)*x[1]
        arg[3,1]=(0.829837477348)*x[0]+(-0.416646135702)*x[1]
        arg[3,2]=(1.43081322579)*x[0]+(0.574724874607)*x[1]
        arg[3,3]=(1.16303985686)*x[0]+(-0.915348714324)*x[1]
        arg[3,4]=(0.773764690591)*x[0]+(0.28049189996)*x[1]
        ref[0,0,0]=(0.102266041928)
        ref[0,0,1]=(0.962985129019)
        ref[0,1,0]=(1.22474578409)
        ref[0,1,1]=(-0.211640772922)
        ref[0,2,0]=(-0.51306749275)
        ref[0,2,1]=(0.652502950912)
        ref[0,3,0]=(1.89516926008)
        ref[0,3,1]=(0.317996805009)
        ref[0,4,0]=(0.562531430083)
        ref[0,4,1]=(-0.194650166716)
        ref[1,0,0]=(0.344353002434)
        ref[1,0,1]=(-0.830671914713)
        ref[1,1,0]=(0.984363231775)
        ref[1,1,1]=(-0.0773767796506)
        ref[1,2,0]=(-0.893978692143)
        ref[1,2,1]=(1.43362006873)
        ref[1,3,0]=(0.603459290036)
        ref[1,3,1]=(-0.527471281229)
        ref[1,4,0]=(-0.768879816113)
        ref[1,4,1]=(0.670739267079)
        ref[2,0,0]=(0.466692398942)
        ref[2,0,1]=(-0.315362658981)
        ref[2,1,0]=(1.30065290708)
        ref[2,1,1]=(-0.202369755569)
        ref[2,2,0]=(-0.114571246331)
        ref[2,2,1]=(1.30250121342)
        ref[2,3,0]=(-0.243608494456)
        ref[2,3,1]=(-0.103398411275)
        ref[2,4,0]=(-0.262568543724)
        ref[2,4,1]=(-0.23979204654)
        ref[3,0,0]=(0.0334727960327)
        ref[3,0,1]=(-0.24547366383)
        ref[3,1,0]=(0.829837477348)
        ref[3,1,1]=(-0.416646135702)
        ref[3,2,0]=(1.43081322579)
        ref[3,2,1]=(0.574724874607)
        ref[3,3,0]=(1.16303985686)
        ref[3,3,1]=(-0.915348714324)
        ref[3,4,0]=(0.773764690591)
        ref[3,4,1]=(0.28049189996)
      else:
        arg[0,0]=(-0.998409457516)*x[0]+(0.374200474533)*x[1]+(-0.397603453862)*x[2]
        arg[0,1]=(-1.48600938216)*x[0]+(1.46358462866)*x[1]+(-0.393381462291)*x[2]
        arg[0,2]=(1.08201727329)*x[0]+(0.065185376372)*x[1]+(-1.32483264357)*x[2]
        arg[0,3]=(-0.518441162)*x[0]+(0.364373161676)*x[1]+(-1.25875574919)*x[2]
        arg[0,4]=(-0.836980279478)*x[0]+(1.21655914685)*x[1]+(0.1586084291)*x[2]
        arg[1,0]=(0.760774732939)*x[0]+(0.515414129191)*x[1]+(-1.50192024303)*x[2]
        arg[1,1]=(-0.653360330499)*x[0]+(1.71417096347)*x[1]+(-0.169129173291)*x[2]
        arg[1,2]=(1.07221333767)*x[0]+(-1.07638348405)*x[1]+(1.13393461135)*x[2]
        arg[1,3]=(0.847567825055)*x[0]+(-0.0298335468796)*x[1]+(0.45122717832)*x[2]
        arg[1,4]=(-0.108027217492)*x[0]+(-1.12654142913)*x[1]+(-0.781718943065)*x[2]
        arg[2,0]=(0.408863322164)*x[0]+(0.100914804776)*x[1]+(-0.471567638444)*x[2]
        arg[2,1]=(-1.42194256762)*x[0]+(-0.348217894465)*x[1]+(-0.830561377367)*x[2]
        arg[2,2]=(1.36876795155)*x[0]+(0.125252389466)*x[1]+(0.00641454191902)*x[2]
        arg[2,3]=(0.135243193372)*x[0]+(-1.04394510102)*x[1]+(0.0223029076973)*x[2]
        arg[2,4]=(-0.969785158394)*x[0]+(-0.48178611106)*x[1]+(-0.0556576599006)*x[2]
        arg[3,0]=(1.39484073703)*x[0]+(-1.0132109886)*x[1]+(0.0205156989927)*x[2]
        arg[3,1]=(-0.92783313898)*x[0]+(-1.27904129731)*x[1]+(-0.817768402689)*x[2]
        arg[3,2]=(1.21718392423)*x[0]+(-0.984091647405)*x[1]+(0.458009888947)*x[2]
        arg[3,3]=(-0.812990754761)*x[0]+(-0.104941245044)*x[1]+(-0.996531826869)*x[2]
        arg[3,4]=(-0.28581988131)*x[0]+(-1.24321114539)*x[1]+(0.408464222517)*x[2]
        ref[0,0,0]=(-0.998409457516)
        ref[0,0,1]=(0.374200474533)
        ref[0,0,2]=(-0.397603453862)
        ref[0,1,0]=(-1.48600938216)
        ref[0,1,1]=(1.46358462866)
        ref[0,1,2]=(-0.393381462291)
        ref[0,2,0]=(1.08201727329)
        ref[0,2,1]=(0.065185376372)
        ref[0,2,2]=(-1.32483264357)
        ref[0,3,0]=(-0.518441162)
        ref[0,3,1]=(0.364373161676)
        ref[0,3,2]=(-1.25875574919)
        ref[0,4,0]=(-0.836980279478)
        ref[0,4,1]=(1.21655914685)
        ref[0,4,2]=(0.1586084291)
        ref[1,0,0]=(0.760774732939)
        ref[1,0,1]=(0.515414129191)
        ref[1,0,2]=(-1.50192024303)
        ref[1,1,0]=(-0.653360330499)
        ref[1,1,1]=(1.71417096347)
        ref[1,1,2]=(-0.169129173291)
        ref[1,2,0]=(1.07221333767)
        ref[1,2,1]=(-1.07638348405)
        ref[1,2,2]=(1.13393461135)
        ref[1,3,0]=(0.847567825055)
        ref[1,3,1]=(-0.0298335468796)
        ref[1,3,2]=(0.45122717832)
        ref[1,4,0]=(-0.108027217492)
        ref[1,4,1]=(-1.12654142913)
        ref[1,4,2]=(-0.781718943065)
        ref[2,0,0]=(0.408863322164)
        ref[2,0,1]=(0.100914804776)
        ref[2,0,2]=(-0.471567638444)
        ref[2,1,0]=(-1.42194256762)
        ref[2,1,1]=(-0.348217894465)
        ref[2,1,2]=(-0.830561377367)
        ref[2,2,0]=(1.36876795155)
        ref[2,2,1]=(0.125252389466)
        ref[2,2,2]=(0.00641454191902)
        ref[2,3,0]=(0.135243193372)
        ref[2,3,1]=(-1.04394510102)
        ref[2,3,2]=(0.0223029076973)
        ref[2,4,0]=(-0.969785158394)
        ref[2,4,1]=(-0.48178611106)
        ref[2,4,2]=(-0.0556576599006)
        ref[3,0,0]=(1.39484073703)
        ref[3,0,1]=(-1.0132109886)
        ref[3,0,2]=(0.0205156989927)
        ref[3,1,0]=(-0.92783313898)
        ref[3,1,1]=(-1.27904129731)
        ref[3,1,2]=(-0.817768402689)
        ref[3,2,0]=(1.21718392423)
        ref[3,2,1]=(-0.984091647405)
        ref[3,2,2]=(0.458009888947)
        ref[3,3,0]=(-0.812990754761)
        ref[3,3,1]=(-0.104941245044)
        ref[3,3,2]=(-0.996531826869)
        ref[3,4,0]=(-0.28581988131)
        ref[3,4,1]=(-1.24321114539)
        ref[3,4,2]=(0.408464222517)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.39712204981)*x[0]+(-1.36315938849)*x[1]
        arg[0,0,1]=(0.654450268666)*x[0]+(-0.198258834351)*x[1]
        arg[0,1,0]=(0.21357757124)*x[0]+(0.193878132191)*x[1]
        arg[0,1,1]=(0.0762258436603)*x[0]+(-0.33284111684)*x[1]
        arg[1,0,0]=(-0.264232939297)*x[0]+(-0.726336076701)*x[1]
        arg[1,0,1]=(-0.179928817534)*x[0]+(-0.723247805844)*x[1]
        arg[1,1,0]=(0.923289681593)*x[0]+(0.986763135255)*x[1]
        arg[1,1,1]=(0.065387254507)*x[0]+(-0.710965008545)*x[1]
        arg[2,0,0]=(0.759853961917)*x[0]+(0.24486573839)*x[1]
        arg[2,0,1]=(-0.318336679378)*x[0]+(0.39328274922)*x[1]
        arg[2,1,0]=(-1.562818948)*x[0]+(-0.891340974738)*x[1]
        arg[2,1,1]=(0.0508006116584)*x[0]+(1.17654134987)*x[1]
        arg[3,0,0]=(-0.155883472458)*x[0]+(0.248809866836)*x[1]
        arg[3,0,1]=(0.287794710222)*x[0]+(-1.72525392296)*x[1]
        arg[3,1,0]=(0.00665202042795)*x[0]+(-0.0996395796968)*x[1]
        arg[3,1,1]=(-0.19809021298)*x[0]+(0.487549616652)*x[1]
        arg[4,0,0]=(0.764163773078)*x[0]+(-0.898694486894)*x[1]
        arg[4,0,1]=(-0.213042355119)*x[0]+(0.708925705544)*x[1]
        arg[4,1,0]=(1.55505564031)*x[0]+(-0.545746649289)*x[1]
        arg[4,1,1]=(-1.31623592165)*x[0]+(0.948772953354)*x[1]
        arg[5,0,0]=(0.221103671977)*x[0]+(0.866067107464)*x[1]
        arg[5,0,1]=(-0.0979530612779)*x[0]+(-0.371623102578)*x[1]
        arg[5,1,0]=(-0.348963169972)*x[0]+(-0.692055450872)*x[1]
        arg[5,1,1]=(-0.584053243695)*x[0]+(-0.487768032842)*x[1]
        ref[0,0,0,0]=(0.39712204981)
        ref[0,0,0,1]=(-1.36315938849)
        ref[0,0,1,0]=(0.654450268666)
        ref[0,0,1,1]=(-0.198258834351)
        ref[0,1,0,0]=(0.21357757124)
        ref[0,1,0,1]=(0.193878132191)
        ref[0,1,1,0]=(0.0762258436603)
        ref[0,1,1,1]=(-0.33284111684)
        ref[1,0,0,0]=(-0.264232939297)
        ref[1,0,0,1]=(-0.726336076701)
        ref[1,0,1,0]=(-0.179928817534)
        ref[1,0,1,1]=(-0.723247805844)
        ref[1,1,0,0]=(0.923289681593)
        ref[1,1,0,1]=(0.986763135255)
        ref[1,1,1,0]=(0.065387254507)
        ref[1,1,1,1]=(-0.710965008545)
        ref[2,0,0,0]=(0.759853961917)
        ref[2,0,0,1]=(0.24486573839)
        ref[2,0,1,0]=(-0.318336679378)
        ref[2,0,1,1]=(0.39328274922)
        ref[2,1,0,0]=(-1.562818948)
        ref[2,1,0,1]=(-0.891340974738)
        ref[2,1,1,0]=(0.0508006116584)
        ref[2,1,1,1]=(1.17654134987)
        ref[3,0,0,0]=(-0.155883472458)
        ref[3,0,0,1]=(0.248809866836)
        ref[3,0,1,0]=(0.287794710222)
        ref[3,0,1,1]=(-1.72525392296)
        ref[3,1,0,0]=(0.00665202042795)
        ref[3,1,0,1]=(-0.0996395796968)
        ref[3,1,1,0]=(-0.19809021298)
        ref[3,1,1,1]=(0.487549616652)
        ref[4,0,0,0]=(0.764163773078)
        ref[4,0,0,1]=(-0.898694486894)
        ref[4,0,1,0]=(-0.213042355119)
        ref[4,0,1,1]=(0.708925705544)
        ref[4,1,0,0]=(1.55505564031)
        ref[4,1,0,1]=(-0.545746649289)
        ref[4,1,1,0]=(-1.31623592165)
        ref[4,1,1,1]=(0.948772953354)
        ref[5,0,0,0]=(0.221103671977)
        ref[5,0,0,1]=(0.866067107464)
        ref[5,0,1,0]=(-0.0979530612779)
        ref[5,0,1,1]=(-0.371623102578)
        ref[5,1,0,0]=(-0.348963169972)
        ref[5,1,0,1]=(-0.692055450872)
        ref[5,1,1,0]=(-0.584053243695)
        ref[5,1,1,1]=(-0.487768032842)
      else:
        arg[0,0,0]=(-1.45023987007)*x[0]+(0.138479134523)*x[1]+(1.46404607112)*x[2]
        arg[0,0,1]=(-0.647908832568)*x[0]+(-0.503758830859)*x[1]+(1.23196203665)*x[2]
        arg[0,1,0]=(-0.0108286161243)*x[0]+(0.565615928686)*x[1]+(-0.357789821636)*x[2]
        arg[0,1,1]=(0.648224572901)*x[0]+(0.775857442933)*x[1]+(0.960991462312)*x[2]
        arg[1,0,0]=(0.340590670624)*x[0]+(0.0357759349963)*x[1]+(0.358139256621)*x[2]
        arg[1,0,1]=(0.569789735109)*x[0]+(1.40678949618)*x[1]+(-0.965577726422)*x[2]
        arg[1,1,0]=(-0.758717082212)*x[0]+(0.612968081377)*x[1]+(-0.417988159948)*x[2]
        arg[1,1,1]=(0.969867032567)*x[0]+(-0.0700234825891)*x[1]+(1.53879850558)*x[2]
        arg[2,0,0]=(-1.02952648125)*x[0]+(0.108642437375)*x[1]+(0.251073184196)*x[2]
        arg[2,0,1]=(0.794766888909)*x[0]+(1.50591346365)*x[1]+(-0.36870578891)*x[2]
        arg[2,1,0]=(-0.534487055503)*x[0]+(-1.09061679233)*x[1]+(0.799342633685)*x[2]
        arg[2,1,1]=(0.0435889213805)*x[0]+(-0.209888628166)*x[1]+(1.34633215915)*x[2]
        arg[3,0,0]=(-0.0344682106376)*x[0]+(-0.92386569455)*x[1]+(-0.551250384619)*x[2]
        arg[3,0,1]=(-0.168599962689)*x[0]+(0.204634495314)*x[1]+(-0.65426520271)*x[2]
        arg[3,1,0]=(-1.61759210628)*x[0]+(-0.5922211067)*x[1]+(-0.139092780207)*x[2]
        arg[3,1,1]=(-0.13862300062)*x[0]+(-1.45951050222)*x[1]+(0.300904423579)*x[2]
        arg[4,0,0]=(0.629590770742)*x[0]+(-0.217150209985)*x[1]+(-0.140384227583)*x[2]
        arg[4,0,1]=(-0.0461635416119)*x[0]+(0.525692990183)*x[1]+(-1.89728167146)*x[2]
        arg[4,1,0]=(-0.130297059668)*x[0]+(-0.138816250007)*x[1]+(1.26805054232)*x[2]
        arg[4,1,1]=(0.742488751566)*x[0]+(0.659272799907)*x[1]+(-0.370877917078)*x[2]
        arg[5,0,0]=(-0.521350561164)*x[0]+(-0.213669151076)*x[1]+(0.00949381597317)*x[2]
        arg[5,0,1]=(-0.951547526648)*x[0]+(-0.388340895795)*x[1]+(0.741997603334)*x[2]
        arg[5,1,0]=(-0.332922940005)*x[0]+(1.74852940259)*x[1]+(-0.484189519128)*x[2]
        arg[5,1,1]=(0.418660109098)*x[0]+(-0.690728767024)*x[1]+(-0.675947375912)*x[2]
        ref[0,0,0,0]=(-1.45023987007)
        ref[0,0,0,1]=(0.138479134523)
        ref[0,0,0,2]=(1.46404607112)
        ref[0,0,1,0]=(-0.647908832568)
        ref[0,0,1,1]=(-0.503758830859)
        ref[0,0,1,2]=(1.23196203665)
        ref[0,1,0,0]=(-0.0108286161243)
        ref[0,1,0,1]=(0.565615928686)
        ref[0,1,0,2]=(-0.357789821636)
        ref[0,1,1,0]=(0.648224572901)
        ref[0,1,1,1]=(0.775857442933)
        ref[0,1,1,2]=(0.960991462312)
        ref[1,0,0,0]=(0.340590670624)
        ref[1,0,0,1]=(0.0357759349963)
        ref[1,0,0,2]=(0.358139256621)
        ref[1,0,1,0]=(0.569789735109)
        ref[1,0,1,1]=(1.40678949618)
        ref[1,0,1,2]=(-0.965577726422)
        ref[1,1,0,0]=(-0.758717082212)
        ref[1,1,0,1]=(0.612968081377)
        ref[1,1,0,2]=(-0.417988159948)
        ref[1,1,1,0]=(0.969867032567)
        ref[1,1,1,1]=(-0.0700234825891)
        ref[1,1,1,2]=(1.53879850558)
        ref[2,0,0,0]=(-1.02952648125)
        ref[2,0,0,1]=(0.108642437375)
        ref[2,0,0,2]=(0.251073184196)
        ref[2,0,1,0]=(0.794766888909)
        ref[2,0,1,1]=(1.50591346365)
        ref[2,0,1,2]=(-0.36870578891)
        ref[2,1,0,0]=(-0.534487055503)
        ref[2,1,0,1]=(-1.09061679233)
        ref[2,1,0,2]=(0.799342633685)
        ref[2,1,1,0]=(0.0435889213805)
        ref[2,1,1,1]=(-0.209888628166)
        ref[2,1,1,2]=(1.34633215915)
        ref[3,0,0,0]=(-0.0344682106376)
        ref[3,0,0,1]=(-0.92386569455)
        ref[3,0,0,2]=(-0.551250384619)
        ref[3,0,1,0]=(-0.168599962689)
        ref[3,0,1,1]=(0.204634495314)
        ref[3,0,1,2]=(-0.65426520271)
        ref[3,1,0,0]=(-1.61759210628)
        ref[3,1,0,1]=(-0.5922211067)
        ref[3,1,0,2]=(-0.139092780207)
        ref[3,1,1,0]=(-0.13862300062)
        ref[3,1,1,1]=(-1.45951050222)
        ref[3,1,1,2]=(0.300904423579)
        ref[4,0,0,0]=(0.629590770742)
        ref[4,0,0,1]=(-0.217150209985)
        ref[4,0,0,2]=(-0.140384227583)
        ref[4,0,1,0]=(-0.0461635416119)
        ref[4,0,1,1]=(0.525692990183)
        ref[4,0,1,2]=(-1.89728167146)
        ref[4,1,0,0]=(-0.130297059668)
        ref[4,1,0,1]=(-0.138816250007)
        ref[4,1,0,2]=(1.26805054232)
        ref[4,1,1,0]=(0.742488751566)
        ref[4,1,1,1]=(0.659272799907)
        ref[4,1,1,2]=(-0.370877917078)
        ref[5,0,0,0]=(-0.521350561164)
        ref[5,0,0,1]=(-0.213669151076)
        ref[5,0,0,2]=(0.00949381597317)
        ref[5,0,1,0]=(-0.951547526648)
        ref[5,0,1,1]=(-0.388340895795)
        ref[5,0,1,2]=(0.741997603334)
        ref[5,1,0,0]=(-0.332922940005)
        ref[5,1,0,1]=(1.74852940259)
        ref[5,1,0,2]=(-0.484189519128)
        ref[5,1,1,0]=(0.418660109098)
        ref[5,1,1,1]=(-0.690728767024)
        ref[5,1,1,2]=(-0.675947375912)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        arg=(0.694260485776)*x[0]+(1.46246375886)*x[1]
        ref[0]=(0.694260485776)
        ref[1]=(1.46246375886)
      else:
        arg=(-1.10912081576)*x[0]+(1.19790363484)*x[1]+(0.555862453192)*x[2]
        ref[0]=(-1.10912081576)
        ref[1]=(1.19790363484)
        ref[2]=(0.555862453192)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        arg[0]=(-0.0963683715208)*x[0]+(-0.985421300426)*x[1]
        arg[1]=(0.50107481561)*x[0]+(0.296191075154)*x[1]
        ref[0,0]=(-0.0963683715208)
        ref[0,1]=(-0.985421300426)
        ref[1,0]=(0.50107481561)
        ref[1,1]=(0.296191075154)
      else:
        arg[0]=(0.790729361248)*x[0]+(0.3673431791)*x[1]+(0.031915702612)*x[2]
        arg[1]=(-0.218629391175)*x[0]+(-0.0619831001578)*x[1]+(-1.46460054046)*x[2]
        ref[0,0]=(0.790729361248)
        ref[0,1]=(0.3673431791)
        ref[0,2]=(0.031915702612)
        ref[1,0]=(-0.218629391175)
        ref[1,1]=(-0.0619831001578)
        ref[1,2]=(-1.46460054046)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        arg[0,0]=(0.102266041928)*x[0]+(0.962985129019)*x[1]
        arg[0,1]=(1.22474578409)*x[0]+(-0.211640772922)*x[1]
        arg[0,2]=(-0.51306749275)*x[0]+(0.652502950912)*x[1]
        arg[0,3]=(1.89516926008)*x[0]+(0.317996805009)*x[1]
        arg[0,4]=(0.562531430083)*x[0]+(-0.194650166716)*x[1]
        arg[1,0]=(0.344353002434)*x[0]+(-0.830671914713)*x[1]
        arg[1,1]=(0.984363231775)*x[0]+(-0.0773767796506)*x[1]
        arg[1,2]=(-0.893978692143)*x[0]+(1.43362006873)*x[1]
        arg[1,3]=(0.603459290036)*x[0]+(-0.527471281229)*x[1]
        arg[1,4]=(-0.768879816113)*x[0]+(0.670739267079)*x[1]
        arg[2,0]=(0.466692398942)*x[0]+(-0.315362658981)*x[1]
        arg[2,1]=(1.30065290708)*x[0]+(-0.202369755569)*x[1]
        arg[2,2]=(-0.114571246331)*x[0]+(1.30250121342)*x[1]
        arg[2,3]=(-0.243608494456)*x[0]+(-0.103398411275)*x[1]
        arg[2,4]=(-0.262568543724)*x[0]+(-0.23979204654)*x[1]
        arg[3,0]=(0.0334727960327)*x[0]+(-0.24547366383)*x[1]
        arg[3,1]=(0.829837477348)*x[0]+(-0.416646135702)*x[1]
        arg[3,2]=(1.43081322579)*x[0]+(0.574724874607)*x[1]
        arg[3,3]=(1.16303985686)*x[0]+(-0.915348714324)*x[1]
        arg[3,4]=(0.773764690591)*x[0]+(0.28049189996)*x[1]
        ref[0,0,0]=(0.102266041928)
        ref[0,0,1]=(0.962985129019)
        ref[0,1,0]=(1.22474578409)
        ref[0,1,1]=(-0.211640772922)
        ref[0,2,0]=(-0.51306749275)
        ref[0,2,1]=(0.652502950912)
        ref[0,3,0]=(1.89516926008)
        ref[0,3,1]=(0.317996805009)
        ref[0,4,0]=(0.562531430083)
        ref[0,4,1]=(-0.194650166716)
        ref[1,0,0]=(0.344353002434)
        ref[1,0,1]=(-0.830671914713)
        ref[1,1,0]=(0.984363231775)
        ref[1,1,1]=(-0.0773767796506)
        ref[1,2,0]=(-0.893978692143)
        ref[1,2,1]=(1.43362006873)
        ref[1,3,0]=(0.603459290036)
        ref[1,3,1]=(-0.527471281229)
        ref[1,4,0]=(-0.768879816113)
        ref[1,4,1]=(0.670739267079)
        ref[2,0,0]=(0.466692398942)
        ref[2,0,1]=(-0.315362658981)
        ref[2,1,0]=(1.30065290708)
        ref[2,1,1]=(-0.202369755569)
        ref[2,2,0]=(-0.114571246331)
        ref[2,2,1]=(1.30250121342)
        ref[2,3,0]=(-0.243608494456)
        ref[2,3,1]=(-0.103398411275)
        ref[2,4,0]=(-0.262568543724)
        ref[2,4,1]=(-0.23979204654)
        ref[3,0,0]=(0.0334727960327)
        ref[3,0,1]=(-0.24547366383)
        ref[3,1,0]=(0.829837477348)
        ref[3,1,1]=(-0.416646135702)
        ref[3,2,0]=(1.43081322579)
        ref[3,2,1]=(0.574724874607)
        ref[3,3,0]=(1.16303985686)
        ref[3,3,1]=(-0.915348714324)
        ref[3,4,0]=(0.773764690591)
        ref[3,4,1]=(0.28049189996)
      else:
        arg[0,0]=(-0.998409457516)*x[0]+(0.374200474533)*x[1]+(-0.397603453862)*x[2]
        arg[0,1]=(-1.48600938216)*x[0]+(1.46358462866)*x[1]+(-0.393381462291)*x[2]
        arg[0,2]=(1.08201727329)*x[0]+(0.065185376372)*x[1]+(-1.32483264357)*x[2]
        arg[0,3]=(-0.518441162)*x[0]+(0.364373161676)*x[1]+(-1.25875574919)*x[2]
        arg[0,4]=(-0.836980279478)*x[0]+(1.21655914685)*x[1]+(0.1586084291)*x[2]
        arg[1,0]=(0.760774732939)*x[0]+(0.515414129191)*x[1]+(-1.50192024303)*x[2]
        arg[1,1]=(-0.653360330499)*x[0]+(1.71417096347)*x[1]+(-0.169129173291)*x[2]
        arg[1,2]=(1.07221333767)*x[0]+(-1.07638348405)*x[1]+(1.13393461135)*x[2]
        arg[1,3]=(0.847567825055)*x[0]+(-0.0298335468796)*x[1]+(0.45122717832)*x[2]
        arg[1,4]=(-0.108027217492)*x[0]+(-1.12654142913)*x[1]+(-0.781718943065)*x[2]
        arg[2,0]=(0.408863322164)*x[0]+(0.100914804776)*x[1]+(-0.471567638444)*x[2]
        arg[2,1]=(-1.42194256762)*x[0]+(-0.348217894465)*x[1]+(-0.830561377367)*x[2]
        arg[2,2]=(1.36876795155)*x[0]+(0.125252389466)*x[1]+(0.00641454191902)*x[2]
        arg[2,3]=(0.135243193372)*x[0]+(-1.04394510102)*x[1]+(0.0223029076973)*x[2]
        arg[2,4]=(-0.969785158394)*x[0]+(-0.48178611106)*x[1]+(-0.0556576599006)*x[2]
        arg[3,0]=(1.39484073703)*x[0]+(-1.0132109886)*x[1]+(0.0205156989927)*x[2]
        arg[3,1]=(-0.92783313898)*x[0]+(-1.27904129731)*x[1]+(-0.817768402689)*x[2]
        arg[3,2]=(1.21718392423)*x[0]+(-0.984091647405)*x[1]+(0.458009888947)*x[2]
        arg[3,3]=(-0.812990754761)*x[0]+(-0.104941245044)*x[1]+(-0.996531826869)*x[2]
        arg[3,4]=(-0.28581988131)*x[0]+(-1.24321114539)*x[1]+(0.408464222517)*x[2]
        ref[0,0,0]=(-0.998409457516)
        ref[0,0,1]=(0.374200474533)
        ref[0,0,2]=(-0.397603453862)
        ref[0,1,0]=(-1.48600938216)
        ref[0,1,1]=(1.46358462866)
        ref[0,1,2]=(-0.393381462291)
        ref[0,2,0]=(1.08201727329)
        ref[0,2,1]=(0.065185376372)
        ref[0,2,2]=(-1.32483264357)
        ref[0,3,0]=(-0.518441162)
        ref[0,3,1]=(0.364373161676)
        ref[0,3,2]=(-1.25875574919)
        ref[0,4,0]=(-0.836980279478)
        ref[0,4,1]=(1.21655914685)
        ref[0,4,2]=(0.1586084291)
        ref[1,0,0]=(0.760774732939)
        ref[1,0,1]=(0.515414129191)
        ref[1,0,2]=(-1.50192024303)
        ref[1,1,0]=(-0.653360330499)
        ref[1,1,1]=(1.71417096347)
        ref[1,1,2]=(-0.169129173291)
        ref[1,2,0]=(1.07221333767)
        ref[1,2,1]=(-1.07638348405)
        ref[1,2,2]=(1.13393461135)
        ref[1,3,0]=(0.847567825055)
        ref[1,3,1]=(-0.0298335468796)
        ref[1,3,2]=(0.45122717832)
        ref[1,4,0]=(-0.108027217492)
        ref[1,4,1]=(-1.12654142913)
        ref[1,4,2]=(-0.781718943065)
        ref[2,0,0]=(0.408863322164)
        ref[2,0,1]=(0.100914804776)
        ref[2,0,2]=(-0.471567638444)
        ref[2,1,0]=(-1.42194256762)
        ref[2,1,1]=(-0.348217894465)
        ref[2,1,2]=(-0.830561377367)
        ref[2,2,0]=(1.36876795155)
        ref[2,2,1]=(0.125252389466)
        ref[2,2,2]=(0.00641454191902)
        ref[2,3,0]=(0.135243193372)
        ref[2,3,1]=(-1.04394510102)
        ref[2,3,2]=(0.0223029076973)
        ref[2,4,0]=(-0.969785158394)
        ref[2,4,1]=(-0.48178611106)
        ref[2,4,2]=(-0.0556576599006)
        ref[3,0,0]=(1.39484073703)
        ref[3,0,1]=(-1.0132109886)
        ref[3,0,2]=(0.0205156989927)
        ref[3,1,0]=(-0.92783313898)
        ref[3,1,1]=(-1.27904129731)
        ref[3,1,2]=(-0.817768402689)
        ref[3,2,0]=(1.21718392423)
        ref[3,2,1]=(-0.984091647405)
        ref[3,2,2]=(0.458009888947)
        ref[3,3,0]=(-0.812990754761)
        ref[3,3,1]=(-0.104941245044)
        ref[3,3,2]=(-0.996531826869)
        ref[3,4,0]=(-0.28581988131)
        ref[3,4,1]=(-1.24321114539)
        ref[3,4,2]=(0.408464222517)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        arg[0,0,0]=(0.39712204981)*x[0]+(-1.36315938849)*x[1]
        arg[0,0,1]=(0.654450268666)*x[0]+(-0.198258834351)*x[1]
        arg[0,1,0]=(0.21357757124)*x[0]+(0.193878132191)*x[1]
        arg[0,1,1]=(0.0762258436603)*x[0]+(-0.33284111684)*x[1]
        arg[1,0,0]=(-0.264232939297)*x[0]+(-0.726336076701)*x[1]
        arg[1,0,1]=(-0.179928817534)*x[0]+(-0.723247805844)*x[1]
        arg[1,1,0]=(0.923289681593)*x[0]+(0.986763135255)*x[1]
        arg[1,1,1]=(0.065387254507)*x[0]+(-0.710965008545)*x[1]
        arg[2,0,0]=(0.759853961917)*x[0]+(0.24486573839)*x[1]
        arg[2,0,1]=(-0.318336679378)*x[0]+(0.39328274922)*x[1]
        arg[2,1,0]=(-1.562818948)*x[0]+(-0.891340974738)*x[1]
        arg[2,1,1]=(0.0508006116584)*x[0]+(1.17654134987)*x[1]
        arg[3,0,0]=(-0.155883472458)*x[0]+(0.248809866836)*x[1]
        arg[3,0,1]=(0.287794710222)*x[0]+(-1.72525392296)*x[1]
        arg[3,1,0]=(0.00665202042795)*x[0]+(-0.0996395796968)*x[1]
        arg[3,1,1]=(-0.19809021298)*x[0]+(0.487549616652)*x[1]
        arg[4,0,0]=(0.764163773078)*x[0]+(-0.898694486894)*x[1]
        arg[4,0,1]=(-0.213042355119)*x[0]+(0.708925705544)*x[1]
        arg[4,1,0]=(1.55505564031)*x[0]+(-0.545746649289)*x[1]
        arg[4,1,1]=(-1.31623592165)*x[0]+(0.948772953354)*x[1]
        arg[5,0,0]=(0.221103671977)*x[0]+(0.866067107464)*x[1]
        arg[5,0,1]=(-0.0979530612779)*x[0]+(-0.371623102578)*x[1]
        arg[5,1,0]=(-0.348963169972)*x[0]+(-0.692055450872)*x[1]
        arg[5,1,1]=(-0.584053243695)*x[0]+(-0.487768032842)*x[1]
        ref[0,0,0,0]=(0.39712204981)
        ref[0,0,0,1]=(-1.36315938849)
        ref[0,0,1,0]=(0.654450268666)
        ref[0,0,1,1]=(-0.198258834351)
        ref[0,1,0,0]=(0.21357757124)
        ref[0,1,0,1]=(0.193878132191)
        ref[0,1,1,0]=(0.0762258436603)
        ref[0,1,1,1]=(-0.33284111684)
        ref[1,0,0,0]=(-0.264232939297)
        ref[1,0,0,1]=(-0.726336076701)
        ref[1,0,1,0]=(-0.179928817534)
        ref[1,0,1,1]=(-0.723247805844)
        ref[1,1,0,0]=(0.923289681593)
        ref[1,1,0,1]=(0.986763135255)
        ref[1,1,1,0]=(0.065387254507)
        ref[1,1,1,1]=(-0.710965008545)
        ref[2,0,0,0]=(0.759853961917)
        ref[2,0,0,1]=(0.24486573839)
        ref[2,0,1,0]=(-0.318336679378)
        ref[2,0,1,1]=(0.39328274922)
        ref[2,1,0,0]=(-1.562818948)
        ref[2,1,0,1]=(-0.891340974738)
        ref[2,1,1,0]=(0.0508006116584)
        ref[2,1,1,1]=(1.17654134987)
        ref[3,0,0,0]=(-0.155883472458)
        ref[3,0,0,1]=(0.248809866836)
        ref[3,0,1,0]=(0.287794710222)
        ref[3,0,1,1]=(-1.72525392296)
        ref[3,1,0,0]=(0.00665202042795)
        ref[3,1,0,1]=(-0.0996395796968)
        ref[3,1,1,0]=(-0.19809021298)
        ref[3,1,1,1]=(0.487549616652)
        ref[4,0,0,0]=(0.764163773078)
        ref[4,0,0,1]=(-0.898694486894)
        ref[4,0,1,0]=(-0.213042355119)
        ref[4,0,1,1]=(0.708925705544)
        ref[4,1,0,0]=(1.55505564031)
        ref[4,1,0,1]=(-0.545746649289)
        ref[4,1,1,0]=(-1.31623592165)
        ref[4,1,1,1]=(0.948772953354)
        ref[5,0,0,0]=(0.221103671977)
        ref[5,0,0,1]=(0.866067107464)
        ref[5,0,1,0]=(-0.0979530612779)
        ref[5,0,1,1]=(-0.371623102578)
        ref[5,1,0,0]=(-0.348963169972)
        ref[5,1,0,1]=(-0.692055450872)
        ref[5,1,1,0]=(-0.584053243695)
        ref[5,1,1,1]=(-0.487768032842)
      else:
        arg[0,0,0]=(-1.45023987007)*x[0]+(0.138479134523)*x[1]+(1.46404607112)*x[2]
        arg[0,0,1]=(-0.647908832568)*x[0]+(-0.503758830859)*x[1]+(1.23196203665)*x[2]
        arg[0,1,0]=(-0.0108286161243)*x[0]+(0.565615928686)*x[1]+(-0.357789821636)*x[2]
        arg[0,1,1]=(0.648224572901)*x[0]+(0.775857442933)*x[1]+(0.960991462312)*x[2]
        arg[1,0,0]=(0.340590670624)*x[0]+(0.0357759349963)*x[1]+(0.358139256621)*x[2]
        arg[1,0,1]=(0.569789735109)*x[0]+(1.40678949618)*x[1]+(-0.965577726422)*x[2]
        arg[1,1,0]=(-0.758717082212)*x[0]+(0.612968081377)*x[1]+(-0.417988159948)*x[2]
        arg[1,1,1]=(0.969867032567)*x[0]+(-0.0700234825891)*x[1]+(1.53879850558)*x[2]
        arg[2,0,0]=(-1.02952648125)*x[0]+(0.108642437375)*x[1]+(0.251073184196)*x[2]
        arg[2,0,1]=(0.794766888909)*x[0]+(1.50591346365)*x[1]+(-0.36870578891)*x[2]
        arg[2,1,0]=(-0.534487055503)*x[0]+(-1.09061679233)*x[1]+(0.799342633685)*x[2]
        arg[2,1,1]=(0.0435889213805)*x[0]+(-0.209888628166)*x[1]+(1.34633215915)*x[2]
        arg[3,0,0]=(-0.0344682106376)*x[0]+(-0.92386569455)*x[1]+(-0.551250384619)*x[2]
        arg[3,0,1]=(-0.168599962689)*x[0]+(0.204634495314)*x[1]+(-0.65426520271)*x[2]
        arg[3,1,0]=(-1.61759210628)*x[0]+(-0.5922211067)*x[1]+(-0.139092780207)*x[2]
        arg[3,1,1]=(-0.13862300062)*x[0]+(-1.45951050222)*x[1]+(0.300904423579)*x[2]
        arg[4,0,0]=(0.629590770742)*x[0]+(-0.217150209985)*x[1]+(-0.140384227583)*x[2]
        arg[4,0,1]=(-0.0461635416119)*x[0]+(0.525692990183)*x[1]+(-1.89728167146)*x[2]
        arg[4,1,0]=(-0.130297059668)*x[0]+(-0.138816250007)*x[1]+(1.26805054232)*x[2]
        arg[4,1,1]=(0.742488751566)*x[0]+(0.659272799907)*x[1]+(-0.370877917078)*x[2]
        arg[5,0,0]=(-0.521350561164)*x[0]+(-0.213669151076)*x[1]+(0.00949381597317)*x[2]
        arg[5,0,1]=(-0.951547526648)*x[0]+(-0.388340895795)*x[1]+(0.741997603334)*x[2]
        arg[5,1,0]=(-0.332922940005)*x[0]+(1.74852940259)*x[1]+(-0.484189519128)*x[2]
        arg[5,1,1]=(0.418660109098)*x[0]+(-0.690728767024)*x[1]+(-0.675947375912)*x[2]
        ref[0,0,0,0]=(-1.45023987007)
        ref[0,0,0,1]=(0.138479134523)
        ref[0,0,0,2]=(1.46404607112)
        ref[0,0,1,0]=(-0.647908832568)
        ref[0,0,1,1]=(-0.503758830859)
        ref[0,0,1,2]=(1.23196203665)
        ref[0,1,0,0]=(-0.0108286161243)
        ref[0,1,0,1]=(0.565615928686)
        ref[0,1,0,2]=(-0.357789821636)
        ref[0,1,1,0]=(0.648224572901)
        ref[0,1,1,1]=(0.775857442933)
        ref[0,1,1,2]=(0.960991462312)
        ref[1,0,0,0]=(0.340590670624)
        ref[1,0,0,1]=(0.0357759349963)
        ref[1,0,0,2]=(0.358139256621)
        ref[1,0,1,0]=(0.569789735109)
        ref[1,0,1,1]=(1.40678949618)
        ref[1,0,1,2]=(-0.965577726422)
        ref[1,1,0,0]=(-0.758717082212)
        ref[1,1,0,1]=(0.612968081377)
        ref[1,1,0,2]=(-0.417988159948)
        ref[1,1,1,0]=(0.969867032567)
        ref[1,1,1,1]=(-0.0700234825891)
        ref[1,1,1,2]=(1.53879850558)
        ref[2,0,0,0]=(-1.02952648125)
        ref[2,0,0,1]=(0.108642437375)
        ref[2,0,0,2]=(0.251073184196)
        ref[2,0,1,0]=(0.794766888909)
        ref[2,0,1,1]=(1.50591346365)
        ref[2,0,1,2]=(-0.36870578891)
        ref[2,1,0,0]=(-0.534487055503)
        ref[2,1,0,1]=(-1.09061679233)
        ref[2,1,0,2]=(0.799342633685)
        ref[2,1,1,0]=(0.0435889213805)
        ref[2,1,1,1]=(-0.209888628166)
        ref[2,1,1,2]=(1.34633215915)
        ref[3,0,0,0]=(-0.0344682106376)
        ref[3,0,0,1]=(-0.92386569455)
        ref[3,0,0,2]=(-0.551250384619)
        ref[3,0,1,0]=(-0.168599962689)
        ref[3,0,1,1]=(0.204634495314)
        ref[3,0,1,2]=(-0.65426520271)
        ref[3,1,0,0]=(-1.61759210628)
        ref[3,1,0,1]=(-0.5922211067)
        ref[3,1,0,2]=(-0.139092780207)
        ref[3,1,1,0]=(-0.13862300062)
        ref[3,1,1,1]=(-1.45951050222)
        ref[3,1,1,2]=(0.300904423579)
        ref[4,0,0,0]=(0.629590770742)
        ref[4,0,0,1]=(-0.217150209985)
        ref[4,0,0,2]=(-0.140384227583)
        ref[4,0,1,0]=(-0.0461635416119)
        ref[4,0,1,1]=(0.525692990183)
        ref[4,0,1,2]=(-1.89728167146)
        ref[4,1,0,0]=(-0.130297059668)
        ref[4,1,0,1]=(-0.138816250007)
        ref[4,1,0,2]=(1.26805054232)
        ref[4,1,1,0]=(0.742488751566)
        ref[4,1,1,1]=(0.659272799907)
        ref[4,1,1,2]=(-0.370877917078)
        ref[5,0,0,0]=(-0.521350561164)
        ref[5,0,0,1]=(-0.213669151076)
        ref[5,0,0,2]=(0.00949381597317)
        ref[5,0,1,0]=(-0.951547526648)
        ref[5,0,1,1]=(-0.388340895795)
        ref[5,0,1,2]=(0.741997603334)
        ref[5,1,0,0]=(-0.332922940005)
        ref[5,1,0,1]=(1.74852940259)
        ref[5,1,0,2]=(-0.484189519128)
        ref[5,1,1,0]=(0.418660109098)
        ref[5,1,1,1]=(-0.690728767024)
        ref[5,1,1,2]=(-0.675947375912)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(-0.149491618448)*x[0]**o+(-0.419584280113)*x[0]+(-0.0503569168892)*x[1]**o+(-0.490263902194)*x[1]
        ref[0]=o*(-0.149491618448)*x_ref[0]**(o-1)+(-0.419584280113)
        ref[1]=o*(-0.0503569168892)*x_ref[1]**(o-1)+(-0.490263902194)
      else:
        s=(-0.757365066356)*x[0]**o+(-0.051930048541)*x[0]+(0.384778052543)*x[1]**o+(-0.305544758353)*x[1]+(-0.462618121798)*x[2]**o+(-0.424504978394)*x[2]
        ref[0]=o*(-0.757365066356)*x_ref[0]**(o-1)+(-0.051930048541)
        ref[1]=o*(0.384778052543)*x_ref[1]**(o-1)+(-0.305544758353)
        ref[2]=o*(-0.462618121798)*x_ref[2]**(o-1)+(-0.424504978394)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.923338270579)*x[0]**o+(0.803299589693)*x[0]+(-0.0850750563669)*x[1]**o+(0.965911232881)*x[1]
        s[1]=(-0.183491365698)*x[0]**o+(-0.842618551859)*x[0]+(0.712774298035)*x[1]**o+(-0.104669018361)*x[1]
        ref[0,0]=o*(-0.923338270579)*x_ref[0]**(o-1)+(0.803299589693)
        ref[0,1]=o*(-0.0850750563669)*x_ref[1]**(o-1)+(0.965911232881)
        ref[1,0]=o*(-0.183491365698)*x_ref[0]**(o-1)+(-0.842618551859)
        ref[1,1]=o*(0.712774298035)*x_ref[1]**(o-1)+(-0.104669018361)
      else:
        s[0]=(0.522068862618)*x[0]**o+(-0.928491651715)*x[0]+(-0.056593136934)*x[1]**o+(-0.00345895910273)*x[1]+(-0.313948793745)*x[2]**o+(0.555562844396)*x[2]
        s[1]=(0.0381288968852)*x[0]**o+(-0.226585510125)*x[0]+(-0.483387509698)*x[1]**o+(-0.0276908054388)*x[1]+(-0.535521100811)*x[2]**o+(0.0628178579102)*x[2]
        ref[0,0]=o*(0.522068862618)*x_ref[0]**(o-1)+(-0.928491651715)
        ref[0,1]=o*(-0.056593136934)*x_ref[1]**(o-1)+(-0.00345895910273)
        ref[0,2]=o*(-0.313948793745)*x_ref[2]**(o-1)+(0.555562844396)
        ref[1,0]=o*(0.0381288968852)*x_ref[0]**(o-1)+(-0.226585510125)
        ref[1,1]=o*(-0.483387509698)*x_ref[1]**(o-1)+(-0.0276908054388)
        ref[1,2]=o*(-0.535521100811)*x_ref[2]**(o-1)+(0.0628178579102)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.0438013069888)*x[0]**o+(-0.962692066069)*x[0]+(0.398086110381)*x[1]**o+(0.943207722394)*x[1]
        s[0,1]=(-0.834074575881)*x[0]**o+(0.984008650691)*x[0]+(-0.16037316175)*x[1]**o+(0.377982608876)*x[1]
        s[0,2]=(-0.245264047174)*x[0]**o+(-0.541684401688)*x[0]+(0.826167130517)*x[1]**o+(0.30004870121)*x[1]
        s[0,3]=(0.126658708414)*x[0]**o+(0.647586365584)*x[0]+(0.258184629361)*x[1]**o+(-0.808198823815)*x[1]
        s[0,4]=(-0.0303747201021)*x[0]**o+(-0.095123002442)*x[0]+(-0.033747901785)*x[1]**o+(-0.0503016700716)*x[1]
        s[1,0]=(0.680161896488)*x[0]**o+(0.106001120727)*x[0]+(-0.983896702986)*x[1]**o+(0.298572049969)*x[1]
        s[1,1]=(-0.215100953009)*x[0]**o+(0.778768869339)*x[0]+(0.887004074512)*x[1]**o+(0.441037801242)*x[1]
        s[1,2]=(-0.892563680898)*x[0]**o+(0.087087180763)*x[0]+(0.16490400964)*x[1]**o+(0.641654994831)*x[1]
        s[1,3]=(0.735567594275)*x[0]**o+(0.143078234647)*x[0]+(-0.741664410771)*x[1]**o+(-0.37591452739)*x[1]
        s[1,4]=(-0.0739884625591)*x[0]**o+(0.745076950828)*x[0]+(0.432379987995)*x[1]**o+(-0.0687199383234)*x[1]
        s[2,0]=(-0.885674457439)*x[0]**o+(-0.456184815781)*x[0]+(0.994806374374)*x[1]**o+(-0.31107028169)*x[1]
        s[2,1]=(0.987856338645)*x[0]**o+(-0.568662820963)*x[0]+(-0.742199948211)*x[1]**o+(0.658035428735)*x[1]
        s[2,2]=(-0.17580106245)*x[0]**o+(-0.589665223227)*x[0]+(0.580399510219)*x[1]**o+(-0.155064144488)*x[1]
        s[2,3]=(-0.913915914987)*x[0]**o+(0.3569536228)*x[0]+(0.23235960166)*x[1]**o+(-0.847113665947)*x[1]
        s[2,4]=(0.0909806665742)*x[0]**o+(-0.643246933435)*x[0]+(-0.480053101714)*x[1]**o+(0.336898095124)*x[1]
        s[3,0]=(-0.944997214199)*x[0]**o+(0.0242583099745)*x[0]+(-0.0537207940075)*x[1]**o+(-0.985424066301)*x[1]
        s[3,1]=(0.685233610656)*x[0]**o+(0.869977880645)*x[0]+(0.28306562014)*x[1]**o+(-0.144951426405)*x[1]
        s[3,2]=(-0.0432942460834)*x[0]**o+(0.706443143255)*x[0]+(-0.880454998397)*x[1]**o+(0.612846993665)*x[1]
        s[3,3]=(0.79137204642)*x[0]**o+(-0.534256675007)*x[0]+(-0.323548658904)*x[1]**o+(0.667614029523)*x[1]
        s[3,4]=(0.0642269065087)*x[0]**o+(0.299466531576)*x[0]+(-0.563060543682)*x[1]**o+(-0.820583826087)*x[1]
        ref[0,0,0]=o*(-0.0438013069888)*x_ref[0]**(o-1)+(-0.962692066069)
        ref[0,0,1]=o*(0.398086110381)*x_ref[1]**(o-1)+(0.943207722394)
        ref[0,1,0]=o*(-0.834074575881)*x_ref[0]**(o-1)+(0.984008650691)
        ref[0,1,1]=o*(-0.16037316175)*x_ref[1]**(o-1)+(0.377982608876)
        ref[0,2,0]=o*(-0.245264047174)*x_ref[0]**(o-1)+(-0.541684401688)
        ref[0,2,1]=o*(0.826167130517)*x_ref[1]**(o-1)+(0.30004870121)
        ref[0,3,0]=o*(0.126658708414)*x_ref[0]**(o-1)+(0.647586365584)
        ref[0,3,1]=o*(0.258184629361)*x_ref[1]**(o-1)+(-0.808198823815)
        ref[0,4,0]=o*(-0.0303747201021)*x_ref[0]**(o-1)+(-0.095123002442)
        ref[0,4,1]=o*(-0.033747901785)*x_ref[1]**(o-1)+(-0.0503016700716)
        ref[1,0,0]=o*(0.680161896488)*x_ref[0]**(o-1)+(0.106001120727)
        ref[1,0,1]=o*(-0.983896702986)*x_ref[1]**(o-1)+(0.298572049969)
        ref[1,1,0]=o*(-0.215100953009)*x_ref[0]**(o-1)+(0.778768869339)
        ref[1,1,1]=o*(0.887004074512)*x_ref[1]**(o-1)+(0.441037801242)
        ref[1,2,0]=o*(-0.892563680898)*x_ref[0]**(o-1)+(0.087087180763)
        ref[1,2,1]=o*(0.16490400964)*x_ref[1]**(o-1)+(0.641654994831)
        ref[1,3,0]=o*(0.735567594275)*x_ref[0]**(o-1)+(0.143078234647)
        ref[1,3,1]=o*(-0.741664410771)*x_ref[1]**(o-1)+(-0.37591452739)
        ref[1,4,0]=o*(-0.0739884625591)*x_ref[0]**(o-1)+(0.745076950828)
        ref[1,4,1]=o*(0.432379987995)*x_ref[1]**(o-1)+(-0.0687199383234)
        ref[2,0,0]=o*(-0.885674457439)*x_ref[0]**(o-1)+(-0.456184815781)
        ref[2,0,1]=o*(0.994806374374)*x_ref[1]**(o-1)+(-0.31107028169)
        ref[2,1,0]=o*(0.987856338645)*x_ref[0]**(o-1)+(-0.568662820963)
        ref[2,1,1]=o*(-0.742199948211)*x_ref[1]**(o-1)+(0.658035428735)
        ref[2,2,0]=o*(-0.17580106245)*x_ref[0]**(o-1)+(-0.589665223227)
        ref[2,2,1]=o*(0.580399510219)*x_ref[1]**(o-1)+(-0.155064144488)
        ref[2,3,0]=o*(-0.913915914987)*x_ref[0]**(o-1)+(0.3569536228)
        ref[2,3,1]=o*(0.23235960166)*x_ref[1]**(o-1)+(-0.847113665947)
        ref[2,4,0]=o*(0.0909806665742)*x_ref[0]**(o-1)+(-0.643246933435)
        ref[2,4,1]=o*(-0.480053101714)*x_ref[1]**(o-1)+(0.336898095124)
        ref[3,0,0]=o*(-0.944997214199)*x_ref[0]**(o-1)+(0.0242583099745)
        ref[3,0,1]=o*(-0.0537207940075)*x_ref[1]**(o-1)+(-0.985424066301)
        ref[3,1,0]=o*(0.685233610656)*x_ref[0]**(o-1)+(0.869977880645)
        ref[3,1,1]=o*(0.28306562014)*x_ref[1]**(o-1)+(-0.144951426405)
        ref[3,2,0]=o*(-0.0432942460834)*x_ref[0]**(o-1)+(0.706443143255)
        ref[3,2,1]=o*(-0.880454998397)*x_ref[1]**(o-1)+(0.612846993665)
        ref[3,3,0]=o*(0.79137204642)*x_ref[0]**(o-1)+(-0.534256675007)
        ref[3,3,1]=o*(-0.323548658904)*x_ref[1]**(o-1)+(0.667614029523)
        ref[3,4,0]=o*(0.0642269065087)*x_ref[0]**(o-1)+(0.299466531576)
        ref[3,4,1]=o*(-0.563060543682)*x_ref[1]**(o-1)+(-0.820583826087)
      else:
        s[0,0]=(0.554048013549)*x[0]**o+(0.628547931484)*x[0]+(-0.087120818445)*x[1]**o+(0.105429960285)*x[1]+(0.221041961114)*x[2]**o+(-0.617484121819)*x[2]
        s[0,1]=(-0.554948686776)*x[0]**o+(-0.271548517185)*x[0]+(0.167744163287)*x[1]**o+(-0.281562288124)*x[1]+(0.0995920439425)*x[2]**o+(0.952925293565)*x[2]
        s[0,2]=(-0.233604089512)*x[0]**o+(-0.395317021862)*x[0]+(-0.148631539093)*x[1]**o+(0.566615423332)*x[1]+(-0.851305702162)*x[2]**o+(0.522170109012)*x[2]
        s[0,3]=(0.33041096125)*x[0]**o+(0.531550689883)*x[0]+(0.43601483601)*x[1]**o+(0.508994516005)*x[1]+(0.848624945097)*x[2]**o+(0.367985533319)*x[2]
        s[0,4]=(0.452749219251)*x[0]**o+(-0.174905238359)*x[0]+(0.156533271339)*x[1]**o+(0.238697207856)*x[1]+(-0.287482388632)*x[2]**o+(-0.30704613609)*x[2]
        s[1,0]=(0.731135831652)*x[0]**o+(0.240028680462)*x[0]+(0.994673716713)*x[1]**o+(-0.14275472627)*x[1]+(0.0105619030307)*x[2]**o+(0.709488861006)*x[2]
        s[1,1]=(-0.494857482432)*x[0]**o+(-0.659820603466)*x[0]+(0.771064201278)*x[1]**o+(0.840663673162)*x[1]+(-0.141911130141)*x[2]**o+(-0.889454650127)*x[2]
        s[1,2]=(-0.492412677213)*x[0]**o+(0.432517067517)*x[0]+(0.239327954605)*x[1]**o+(-0.437437759354)*x[1]+(-0.188157683846)*x[2]**o+(-0.490906503638)*x[2]
        s[1,3]=(-0.313081049716)*x[0]**o+(-0.934448134549)*x[0]+(0.666459048562)*x[1]**o+(0.947256512054)*x[1]+(-0.710248833935)*x[2]**o+(-0.796385160956)*x[2]
        s[1,4]=(0.997217607294)*x[0]**o+(0.939841022657)*x[0]+(-0.857321524031)*x[1]**o+(-0.134948519166)*x[1]+(-0.383752839014)*x[2]**o+(-0.0694045288726)*x[2]
        s[2,0]=(0.884993378314)*x[0]**o+(-0.305179802237)*x[0]+(0.43264887709)*x[1]**o+(0.281387756518)*x[1]+(-0.887582464922)*x[2]**o+(-0.46966538775)*x[2]
        s[2,1]=(0.640836781354)*x[0]**o+(0.822302879707)*x[0]+(-0.78092397026)*x[1]**o+(-0.977840224136)*x[1]+(0.368581385054)*x[2]**o+(0.657227564549)*x[2]
        s[2,2]=(-0.148123980628)*x[0]**o+(-0.69167854219)*x[0]+(-0.93156581239)*x[1]**o+(-0.79482831934)*x[1]+(-0.816815232666)*x[2]**o+(0.0127598824004)*x[2]
        s[2,3]=(-0.610703986086)*x[0]**o+(0.274872054308)*x[0]+(0.0215106382843)*x[1]**o+(0.72724816172)*x[1]+(-0.915709139681)*x[2]**o+(0.321703565556)*x[2]
        s[2,4]=(0.381549090843)*x[0]**o+(-0.962121232421)*x[0]+(-0.897209804063)*x[1]**o+(-0.766441490629)*x[1]+(-0.741441127879)*x[2]**o+(-0.142615155341)*x[2]
        s[3,0]=(0.893922704275)*x[0]**o+(-0.683229796558)*x[0]+(0.900359565899)*x[1]**o+(0.785771212984)*x[1]+(0.216603036961)*x[2]**o+(0.144463289025)*x[2]
        s[3,1]=(0.360659657586)*x[0]**o+(-0.207597777231)*x[0]+(-0.350428345153)*x[1]**o+(-0.159358793155)*x[1]+(0.373616042827)*x[2]**o+(-0.0155120388232)*x[2]
        s[3,2]=(0.335734139009)*x[0]**o+(0.0880595596639)*x[0]+(-0.618682099923)*x[1]**o+(-0.409019944145)*x[1]+(0.56154653361)*x[2]**o+(-0.384031273173)*x[2]
        s[3,3]=(0.738466863893)*x[0]**o+(-0.305503217774)*x[0]+(-0.133472928746)*x[1]**o+(0.803290264921)*x[1]+(-0.919787049033)*x[2]**o+(0.138860079244)*x[2]
        s[3,4]=(0.0312193600407)*x[0]**o+(0.340894051514)*x[0]+(0.100198716313)*x[1]**o+(-0.196988878013)*x[1]+(0.0825369961302)*x[2]**o+(-0.652573275397)*x[2]
        ref[0,0,0]=o*(0.554048013549)*x_ref[0]**(o-1)+(0.628547931484)
        ref[0,0,1]=o*(-0.087120818445)*x_ref[1]**(o-1)+(0.105429960285)
        ref[0,0,2]=o*(0.221041961114)*x_ref[2]**(o-1)+(-0.617484121819)
        ref[0,1,0]=o*(-0.554948686776)*x_ref[0]**(o-1)+(-0.271548517185)
        ref[0,1,1]=o*(0.167744163287)*x_ref[1]**(o-1)+(-0.281562288124)
        ref[0,1,2]=o*(0.0995920439425)*x_ref[2]**(o-1)+(0.952925293565)
        ref[0,2,0]=o*(-0.233604089512)*x_ref[0]**(o-1)+(-0.395317021862)
        ref[0,2,1]=o*(-0.148631539093)*x_ref[1]**(o-1)+(0.566615423332)
        ref[0,2,2]=o*(-0.851305702162)*x_ref[2]**(o-1)+(0.522170109012)
        ref[0,3,0]=o*(0.33041096125)*x_ref[0]**(o-1)+(0.531550689883)
        ref[0,3,1]=o*(0.43601483601)*x_ref[1]**(o-1)+(0.508994516005)
        ref[0,3,2]=o*(0.848624945097)*x_ref[2]**(o-1)+(0.367985533319)
        ref[0,4,0]=o*(0.452749219251)*x_ref[0]**(o-1)+(-0.174905238359)
        ref[0,4,1]=o*(0.156533271339)*x_ref[1]**(o-1)+(0.238697207856)
        ref[0,4,2]=o*(-0.287482388632)*x_ref[2]**(o-1)+(-0.30704613609)
        ref[1,0,0]=o*(0.731135831652)*x_ref[0]**(o-1)+(0.240028680462)
        ref[1,0,1]=o*(0.994673716713)*x_ref[1]**(o-1)+(-0.14275472627)
        ref[1,0,2]=o*(0.0105619030307)*x_ref[2]**(o-1)+(0.709488861006)
        ref[1,1,0]=o*(-0.494857482432)*x_ref[0]**(o-1)+(-0.659820603466)
        ref[1,1,1]=o*(0.771064201278)*x_ref[1]**(o-1)+(0.840663673162)
        ref[1,1,2]=o*(-0.141911130141)*x_ref[2]**(o-1)+(-0.889454650127)
        ref[1,2,0]=o*(-0.492412677213)*x_ref[0]**(o-1)+(0.432517067517)
        ref[1,2,1]=o*(0.239327954605)*x_ref[1]**(o-1)+(-0.437437759354)
        ref[1,2,2]=o*(-0.188157683846)*x_ref[2]**(o-1)+(-0.490906503638)
        ref[1,3,0]=o*(-0.313081049716)*x_ref[0]**(o-1)+(-0.934448134549)
        ref[1,3,1]=o*(0.666459048562)*x_ref[1]**(o-1)+(0.947256512054)
        ref[1,3,2]=o*(-0.710248833935)*x_ref[2]**(o-1)+(-0.796385160956)
        ref[1,4,0]=o*(0.997217607294)*x_ref[0]**(o-1)+(0.939841022657)
        ref[1,4,1]=o*(-0.857321524031)*x_ref[1]**(o-1)+(-0.134948519166)
        ref[1,4,2]=o*(-0.383752839014)*x_ref[2]**(o-1)+(-0.0694045288726)
        ref[2,0,0]=o*(0.884993378314)*x_ref[0]**(o-1)+(-0.305179802237)
        ref[2,0,1]=o*(0.43264887709)*x_ref[1]**(o-1)+(0.281387756518)
        ref[2,0,2]=o*(-0.887582464922)*x_ref[2]**(o-1)+(-0.46966538775)
        ref[2,1,0]=o*(0.640836781354)*x_ref[0]**(o-1)+(0.822302879707)
        ref[2,1,1]=o*(-0.78092397026)*x_ref[1]**(o-1)+(-0.977840224136)
        ref[2,1,2]=o*(0.368581385054)*x_ref[2]**(o-1)+(0.657227564549)
        ref[2,2,0]=o*(-0.148123980628)*x_ref[0]**(o-1)+(-0.69167854219)
        ref[2,2,1]=o*(-0.93156581239)*x_ref[1]**(o-1)+(-0.79482831934)
        ref[2,2,2]=o*(-0.816815232666)*x_ref[2]**(o-1)+(0.0127598824004)
        ref[2,3,0]=o*(-0.610703986086)*x_ref[0]**(o-1)+(0.274872054308)
        ref[2,3,1]=o*(0.0215106382843)*x_ref[1]**(o-1)+(0.72724816172)
        ref[2,3,2]=o*(-0.915709139681)*x_ref[2]**(o-1)+(0.321703565556)
        ref[2,4,0]=o*(0.381549090843)*x_ref[0]**(o-1)+(-0.962121232421)
        ref[2,4,1]=o*(-0.897209804063)*x_ref[1]**(o-1)+(-0.766441490629)
        ref[2,4,2]=o*(-0.741441127879)*x_ref[2]**(o-1)+(-0.142615155341)
        ref[3,0,0]=o*(0.893922704275)*x_ref[0]**(o-1)+(-0.683229796558)
        ref[3,0,1]=o*(0.900359565899)*x_ref[1]**(o-1)+(0.785771212984)
        ref[3,0,2]=o*(0.216603036961)*x_ref[2]**(o-1)+(0.144463289025)
        ref[3,1,0]=o*(0.360659657586)*x_ref[0]**(o-1)+(-0.207597777231)
        ref[3,1,1]=o*(-0.350428345153)*x_ref[1]**(o-1)+(-0.159358793155)
        ref[3,1,2]=o*(0.373616042827)*x_ref[2]**(o-1)+(-0.0155120388232)
        ref[3,2,0]=o*(0.335734139009)*x_ref[0]**(o-1)+(0.0880595596639)
        ref[3,2,1]=o*(-0.618682099923)*x_ref[1]**(o-1)+(-0.409019944145)
        ref[3,2,2]=o*(0.56154653361)*x_ref[2]**(o-1)+(-0.384031273173)
        ref[3,3,0]=o*(0.738466863893)*x_ref[0]**(o-1)+(-0.305503217774)
        ref[3,3,1]=o*(-0.133472928746)*x_ref[1]**(o-1)+(0.803290264921)
        ref[3,3,2]=o*(-0.919787049033)*x_ref[2]**(o-1)+(0.138860079244)
        ref[3,4,0]=o*(0.0312193600407)*x_ref[0]**(o-1)+(0.340894051514)
        ref[3,4,1]=o*(0.100198716313)*x_ref[1]**(o-1)+(-0.196988878013)
        ref[3,4,2]=o*(0.0825369961302)*x_ref[2]**(o-1)+(-0.652573275397)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.178576436338)*x[0]**o+(0.548880215769)*x[0]+(-0.318815466914)*x[1]**o+(-0.798063605205)*x[1]
        s[0,0,1]=(-0.822727567562)*x[0]**o+(-0.947988410328)*x[0]+(0.756740500537)*x[1]**o+(0.997703924729)*x[1]
        s[0,1,0]=(0.145493259882)*x[0]**o+(0.512878926373)*x[0]+(0.188564032293)*x[1]**o+(0.900204838942)*x[1]
        s[0,1,1]=(-0.88128289676)*x[0]**o+(0.833100847964)*x[0]+(-0.419907100559)*x[1]**o+(-0.461899797784)*x[1]
        s[1,0,0]=(-0.185721953771)*x[0]**o+(0.0783345782049)*x[0]+(-0.939442057638)*x[1]**o+(-0.320245477333)*x[1]
        s[1,0,1]=(0.410922982405)*x[0]**o+(0.398440443266)*x[0]+(0.768185628746)*x[1]**o+(0.256153329651)*x[1]
        s[1,1,0]=(-0.526878655538)*x[0]**o+(-0.658168532232)*x[0]+(0.69953069671)*x[1]**o+(-0.916587227061)*x[1]
        s[1,1,1]=(0.872254289223)*x[0]**o+(-0.804299235664)*x[0]+(-0.245503808339)*x[1]**o+(-0.894388559613)*x[1]
        s[2,0,0]=(0.917787084987)*x[0]**o+(-0.826075775004)*x[0]+(0.755866825722)*x[1]**o+(0.212054055974)*x[1]
        s[2,0,1]=(0.722119407968)*x[0]**o+(-0.357088696868)*x[0]+(0.033293020384)*x[1]**o+(-0.383269284911)*x[1]
        s[2,1,0]=(0.856304318608)*x[0]**o+(-0.918845722628)*x[0]+(-0.17232500803)*x[1]**o+(0.859254907185)*x[1]
        s[2,1,1]=(0.440452546297)*x[0]**o+(0.0912337401501)*x[0]+(0.76436338959)*x[1]**o+(-0.628986501066)*x[1]
        s[3,0,0]=(-0.583286304487)*x[0]**o+(0.102129275171)*x[0]+(-0.811858544156)*x[1]**o+(-0.521643540274)*x[1]
        s[3,0,1]=(0.682705489393)*x[0]**o+(0.722776291806)*x[0]+(-0.0417392890997)*x[1]**o+(0.487556285425)*x[1]
        s[3,1,0]=(-0.958268636176)*x[0]**o+(-0.666213997791)*x[0]+(-0.702747133353)*x[1]**o+(0.465173353165)*x[1]
        s[3,1,1]=(-0.919518552212)*x[0]**o+(0.607551779709)*x[0]+(0.4478984898)*x[1]**o+(0.0217822207015)*x[1]
        s[4,0,0]=(-0.625448546192)*x[0]**o+(0.513680244128)*x[0]+(0.0190646155266)*x[1]**o+(0.0155037658268)*x[1]
        s[4,0,1]=(-0.573453340974)*x[0]**o+(0.452807235507)*x[0]+(0.902938708535)*x[1]**o+(0.123632055082)*x[1]
        s[4,1,0]=(0.760888494849)*x[0]**o+(-0.488658748567)*x[0]+(0.949883579341)*x[1]**o+(-0.205749942866)*x[1]
        s[4,1,1]=(-0.2860871769)*x[0]**o+(0.614822186727)*x[0]+(0.118313781134)*x[1]**o+(0.448320553312)*x[1]
        s[5,0,0]=(0.671714056383)*x[0]**o+(-0.441343906973)*x[0]+(0.235137729851)*x[1]**o+(0.0696929938911)*x[1]
        s[5,0,1]=(0.114159304365)*x[0]**o+(0.562991557608)*x[0]+(0.00444483447333)*x[1]**o+(0.434809595495)*x[1]
        s[5,1,0]=(0.0360353162225)*x[0]**o+(-0.366777208182)*x[0]+(-0.668602224156)*x[1]**o+(0.449496848779)*x[1]
        s[5,1,1]=(-0.982227533448)*x[0]**o+(-0.704480676355)*x[0]+(-0.187844946341)*x[1]**o+(0.0975753620283)*x[1]
        ref[0,0,0,0]=o*(-0.178576436338)*x_ref[0]**(o-1)+(0.548880215769)
        ref[0,0,0,1]=o*(-0.318815466914)*x_ref[1]**(o-1)+(-0.798063605205)
        ref[0,0,1,0]=o*(-0.822727567562)*x_ref[0]**(o-1)+(-0.947988410328)
        ref[0,0,1,1]=o*(0.756740500537)*x_ref[1]**(o-1)+(0.997703924729)
        ref[0,1,0,0]=o*(0.145493259882)*x_ref[0]**(o-1)+(0.512878926373)
        ref[0,1,0,1]=o*(0.188564032293)*x_ref[1]**(o-1)+(0.900204838942)
        ref[0,1,1,0]=o*(-0.88128289676)*x_ref[0]**(o-1)+(0.833100847964)
        ref[0,1,1,1]=o*(-0.419907100559)*x_ref[1]**(o-1)+(-0.461899797784)
        ref[1,0,0,0]=o*(-0.185721953771)*x_ref[0]**(o-1)+(0.0783345782049)
        ref[1,0,0,1]=o*(-0.939442057638)*x_ref[1]**(o-1)+(-0.320245477333)
        ref[1,0,1,0]=o*(0.410922982405)*x_ref[0]**(o-1)+(0.398440443266)
        ref[1,0,1,1]=o*(0.768185628746)*x_ref[1]**(o-1)+(0.256153329651)
        ref[1,1,0,0]=o*(-0.526878655538)*x_ref[0]**(o-1)+(-0.658168532232)
        ref[1,1,0,1]=o*(0.69953069671)*x_ref[1]**(o-1)+(-0.916587227061)
        ref[1,1,1,0]=o*(0.872254289223)*x_ref[0]**(o-1)+(-0.804299235664)
        ref[1,1,1,1]=o*(-0.245503808339)*x_ref[1]**(o-1)+(-0.894388559613)
        ref[2,0,0,0]=o*(0.917787084987)*x_ref[0]**(o-1)+(-0.826075775004)
        ref[2,0,0,1]=o*(0.755866825722)*x_ref[1]**(o-1)+(0.212054055974)
        ref[2,0,1,0]=o*(0.722119407968)*x_ref[0]**(o-1)+(-0.357088696868)
        ref[2,0,1,1]=o*(0.033293020384)*x_ref[1]**(o-1)+(-0.383269284911)
        ref[2,1,0,0]=o*(0.856304318608)*x_ref[0]**(o-1)+(-0.918845722628)
        ref[2,1,0,1]=o*(-0.17232500803)*x_ref[1]**(o-1)+(0.859254907185)
        ref[2,1,1,0]=o*(0.440452546297)*x_ref[0]**(o-1)+(0.0912337401501)
        ref[2,1,1,1]=o*(0.76436338959)*x_ref[1]**(o-1)+(-0.628986501066)
        ref[3,0,0,0]=o*(-0.583286304487)*x_ref[0]**(o-1)+(0.102129275171)
        ref[3,0,0,1]=o*(-0.811858544156)*x_ref[1]**(o-1)+(-0.521643540274)
        ref[3,0,1,0]=o*(0.682705489393)*x_ref[0]**(o-1)+(0.722776291806)
        ref[3,0,1,1]=o*(-0.0417392890997)*x_ref[1]**(o-1)+(0.487556285425)
        ref[3,1,0,0]=o*(-0.958268636176)*x_ref[0]**(o-1)+(-0.666213997791)
        ref[3,1,0,1]=o*(-0.702747133353)*x_ref[1]**(o-1)+(0.465173353165)
        ref[3,1,1,0]=o*(-0.919518552212)*x_ref[0]**(o-1)+(0.607551779709)
        ref[3,1,1,1]=o*(0.4478984898)*x_ref[1]**(o-1)+(0.0217822207015)
        ref[4,0,0,0]=o*(-0.625448546192)*x_ref[0]**(o-1)+(0.513680244128)
        ref[4,0,0,1]=o*(0.0190646155266)*x_ref[1]**(o-1)+(0.0155037658268)
        ref[4,0,1,0]=o*(-0.573453340974)*x_ref[0]**(o-1)+(0.452807235507)
        ref[4,0,1,1]=o*(0.902938708535)*x_ref[1]**(o-1)+(0.123632055082)
        ref[4,1,0,0]=o*(0.760888494849)*x_ref[0]**(o-1)+(-0.488658748567)
        ref[4,1,0,1]=o*(0.949883579341)*x_ref[1]**(o-1)+(-0.205749942866)
        ref[4,1,1,0]=o*(-0.2860871769)*x_ref[0]**(o-1)+(0.614822186727)
        ref[4,1,1,1]=o*(0.118313781134)*x_ref[1]**(o-1)+(0.448320553312)
        ref[5,0,0,0]=o*(0.671714056383)*x_ref[0]**(o-1)+(-0.441343906973)
        ref[5,0,0,1]=o*(0.235137729851)*x_ref[1]**(o-1)+(0.0696929938911)
        ref[5,0,1,0]=o*(0.114159304365)*x_ref[0]**(o-1)+(0.562991557608)
        ref[5,0,1,1]=o*(0.00444483447333)*x_ref[1]**(o-1)+(0.434809595495)
        ref[5,1,0,0]=o*(0.0360353162225)*x_ref[0]**(o-1)+(-0.366777208182)
        ref[5,1,0,1]=o*(-0.668602224156)*x_ref[1]**(o-1)+(0.449496848779)
        ref[5,1,1,0]=o*(-0.982227533448)*x_ref[0]**(o-1)+(-0.704480676355)
        ref[5,1,1,1]=o*(-0.187844946341)*x_ref[1]**(o-1)+(0.0975753620283)
      else:
        s[0,0,0]=(0.983153853699)*x[0]**o+(0.877999648347)*x[0]+(0.673605473153)*x[1]**o+(0.469503898589)*x[1]+(-0.069158689944)*x[2]**o+(-0.861916140555)*x[2]
        s[0,0,1]=(-0.553610878092)*x[0]**o+(0.0506456195871)*x[0]+(-0.474818319583)*x[1]**o+(0.578919616336)*x[1]+(0.264746159866)*x[2]**o+(-0.433622198203)*x[2]
        s[0,1,0]=(0.666034530743)*x[0]**o+(0.546828305775)*x[0]+(-0.512009992229)*x[1]**o+(-0.476398683846)*x[1]+(-0.570551384835)*x[2]**o+(-0.133446991616)*x[2]
        s[0,1,1]=(0.539382416391)*x[0]**o+(-0.278692810377)*x[0]+(0.10726626544)*x[1]**o+(-0.966085403104)*x[1]+(-0.658134958165)*x[2]**o+(0.924604577895)*x[2]
        s[1,0,0]=(-0.330421615628)*x[0]**o+(0.724947570665)*x[0]+(0.34637731641)*x[1]**o+(0.39702161282)*x[1]+(0.931725860392)*x[2]**o+(-0.673326580823)*x[2]
        s[1,0,1]=(0.353966196399)*x[0]**o+(0.472639435387)*x[0]+(-0.328174031243)*x[1]**o+(0.640610221795)*x[1]+(-0.900613354552)*x[2]**o+(0.796468730393)*x[2]
        s[1,1,0]=(0.754328288128)*x[0]**o+(-0.846972319862)*x[0]+(0.845142974251)*x[1]**o+(0.144707523269)*x[1]+(-0.135529271924)*x[2]**o+(0.232438947532)*x[2]
        s[1,1,1]=(0.0826879113034)*x[0]**o+(-0.532753089482)*x[0]+(-0.617867675502)*x[1]**o+(0.327150779323)*x[1]+(-0.773918767657)*x[2]**o+(0.0695419192896)*x[2]
        s[2,0,0]=(0.861666752281)*x[0]**o+(-0.369904662594)*x[0]+(-0.809447635327)*x[1]**o+(0.314495362669)*x[1]+(0.672158503953)*x[2]**o+(0.779729363207)*x[2]
        s[2,0,1]=(0.784251939885)*x[0]**o+(-0.657779846806)*x[0]+(-0.694796642388)*x[1]**o+(0.549290583812)*x[1]+(0.452712301324)*x[2]**o+(0.0965361955733)*x[2]
        s[2,1,0]=(0.526035829224)*x[0]**o+(-0.5285468198)*x[0]+(0.426231604453)*x[1]**o+(-0.89107298075)*x[1]+(0.365149699988)*x[2]**o+(0.87436690224)*x[2]
        s[2,1,1]=(-0.516537342331)*x[0]**o+(-0.793927917391)*x[0]+(0.795130846095)*x[1]**o+(0.782928826071)*x[1]+(-0.874002864146)*x[2]**o+(0.465526006676)*x[2]
        s[3,0,0]=(-0.861917370502)*x[0]**o+(-0.0636266572453)*x[0]+(-0.0140222042452)*x[1]**o+(0.143759402339)*x[1]+(-0.247983260546)*x[2]**o+(-0.596358912223)*x[2]
        s[3,0,1]=(0.120302367516)*x[0]**o+(-0.701350969734)*x[0]+(-0.775100614072)*x[1]**o+(-0.248023481318)*x[1]+(0.849345512002)*x[2]**o+(0.542529986112)*x[2]
        s[3,1,0]=(-0.347014578244)*x[0]**o+(0.611296458158)*x[0]+(-0.11214501466)*x[1]**o+(0.516100884048)*x[1]+(-0.746366893941)*x[2]**o+(-0.429950357634)*x[2]
        s[3,1,1]=(-0.154199420675)*x[0]**o+(-0.533525033722)*x[0]+(-0.305802182284)*x[1]**o+(0.0249510778534)*x[1]+(0.511832901845)*x[2]**o+(0.865423961695)*x[2]
        s[4,0,0]=(0.270951033321)*x[0]**o+(0.293153287275)*x[0]+(0.499008893086)*x[1]**o+(0.401454188898)*x[1]+(-0.0659433492694)*x[2]**o+(0.0822124121616)*x[2]
        s[4,0,1]=(-0.770956381696)*x[0]**o+(0.962572964564)*x[0]+(-0.173527764809)*x[1]**o+(-0.764724502533)*x[1]+(0.686939133626)*x[2]**o+(0.589061927699)*x[2]
        s[4,1,0]=(0.856144070276)*x[0]**o+(-0.970851254884)*x[0]+(-0.200238695073)*x[1]**o+(0.961653891428)*x[1]+(-0.921215646185)*x[2]**o+(-0.927582743302)*x[2]
        s[4,1,1]=(0.518201773728)*x[0]**o+(0.403940302766)*x[0]+(-0.0333026733014)*x[1]**o+(0.602796446678)*x[1]+(0.956622086807)*x[2]**o+(0.445065392727)*x[2]
        s[5,0,0]=(0.847162899069)*x[0]**o+(-0.11079738914)*x[0]+(0.815117104026)*x[1]**o+(-0.0126064617312)*x[1]+(0.434649696776)*x[2]**o+(-0.925434721648)*x[2]
        s[5,0,1]=(0.827068835777)*x[0]**o+(0.267176529776)*x[0]+(-0.954551141359)*x[1]**o+(-0.847951054931)*x[1]+(0.939843025231)*x[2]**o+(-0.405674486263)*x[2]
        s[5,1,0]=(0.303863917444)*x[0]**o+(0.33377358223)*x[0]+(0.160136648003)*x[1]**o+(0.557340036308)*x[1]+(0.348362672822)*x[2]**o+(-0.719624087847)*x[2]
        s[5,1,1]=(-0.61009211312)*x[0]**o+(0.948776841387)*x[0]+(0.0348157372912)*x[1]**o+(0.415543380004)*x[1]+(-0.104796276752)*x[2]**o+(0.109481954783)*x[2]
        ref[0,0,0,0]=o*(0.983153853699)*x_ref[0]**(o-1)+(0.877999648347)
        ref[0,0,0,1]=o*(0.673605473153)*x_ref[1]**(o-1)+(0.469503898589)
        ref[0,0,0,2]=o*(-0.069158689944)*x_ref[2]**(o-1)+(-0.861916140555)
        ref[0,0,1,0]=o*(-0.553610878092)*x_ref[0]**(o-1)+(0.0506456195871)
        ref[0,0,1,1]=o*(-0.474818319583)*x_ref[1]**(o-1)+(0.578919616336)
        ref[0,0,1,2]=o*(0.264746159866)*x_ref[2]**(o-1)+(-0.433622198203)
        ref[0,1,0,0]=o*(0.666034530743)*x_ref[0]**(o-1)+(0.546828305775)
        ref[0,1,0,1]=o*(-0.512009992229)*x_ref[1]**(o-1)+(-0.476398683846)
        ref[0,1,0,2]=o*(-0.570551384835)*x_ref[2]**(o-1)+(-0.133446991616)
        ref[0,1,1,0]=o*(0.539382416391)*x_ref[0]**(o-1)+(-0.278692810377)
        ref[0,1,1,1]=o*(0.10726626544)*x_ref[1]**(o-1)+(-0.966085403104)
        ref[0,1,1,2]=o*(-0.658134958165)*x_ref[2]**(o-1)+(0.924604577895)
        ref[1,0,0,0]=o*(-0.330421615628)*x_ref[0]**(o-1)+(0.724947570665)
        ref[1,0,0,1]=o*(0.34637731641)*x_ref[1]**(o-1)+(0.39702161282)
        ref[1,0,0,2]=o*(0.931725860392)*x_ref[2]**(o-1)+(-0.673326580823)
        ref[1,0,1,0]=o*(0.353966196399)*x_ref[0]**(o-1)+(0.472639435387)
        ref[1,0,1,1]=o*(-0.328174031243)*x_ref[1]**(o-1)+(0.640610221795)
        ref[1,0,1,2]=o*(-0.900613354552)*x_ref[2]**(o-1)+(0.796468730393)
        ref[1,1,0,0]=o*(0.754328288128)*x_ref[0]**(o-1)+(-0.846972319862)
        ref[1,1,0,1]=o*(0.845142974251)*x_ref[1]**(o-1)+(0.144707523269)
        ref[1,1,0,2]=o*(-0.135529271924)*x_ref[2]**(o-1)+(0.232438947532)
        ref[1,1,1,0]=o*(0.0826879113034)*x_ref[0]**(o-1)+(-0.532753089482)
        ref[1,1,1,1]=o*(-0.617867675502)*x_ref[1]**(o-1)+(0.327150779323)
        ref[1,1,1,2]=o*(-0.773918767657)*x_ref[2]**(o-1)+(0.0695419192896)
        ref[2,0,0,0]=o*(0.861666752281)*x_ref[0]**(o-1)+(-0.369904662594)
        ref[2,0,0,1]=o*(-0.809447635327)*x_ref[1]**(o-1)+(0.314495362669)
        ref[2,0,0,2]=o*(0.672158503953)*x_ref[2]**(o-1)+(0.779729363207)
        ref[2,0,1,0]=o*(0.784251939885)*x_ref[0]**(o-1)+(-0.657779846806)
        ref[2,0,1,1]=o*(-0.694796642388)*x_ref[1]**(o-1)+(0.549290583812)
        ref[2,0,1,2]=o*(0.452712301324)*x_ref[2]**(o-1)+(0.0965361955733)
        ref[2,1,0,0]=o*(0.526035829224)*x_ref[0]**(o-1)+(-0.5285468198)
        ref[2,1,0,1]=o*(0.426231604453)*x_ref[1]**(o-1)+(-0.89107298075)
        ref[2,1,0,2]=o*(0.365149699988)*x_ref[2]**(o-1)+(0.87436690224)
        ref[2,1,1,0]=o*(-0.516537342331)*x_ref[0]**(o-1)+(-0.793927917391)
        ref[2,1,1,1]=o*(0.795130846095)*x_ref[1]**(o-1)+(0.782928826071)
        ref[2,1,1,2]=o*(-0.874002864146)*x_ref[2]**(o-1)+(0.465526006676)
        ref[3,0,0,0]=o*(-0.861917370502)*x_ref[0]**(o-1)+(-0.0636266572453)
        ref[3,0,0,1]=o*(-0.0140222042452)*x_ref[1]**(o-1)+(0.143759402339)
        ref[3,0,0,2]=o*(-0.247983260546)*x_ref[2]**(o-1)+(-0.596358912223)
        ref[3,0,1,0]=o*(0.120302367516)*x_ref[0]**(o-1)+(-0.701350969734)
        ref[3,0,1,1]=o*(-0.775100614072)*x_ref[1]**(o-1)+(-0.248023481318)
        ref[3,0,1,2]=o*(0.849345512002)*x_ref[2]**(o-1)+(0.542529986112)
        ref[3,1,0,0]=o*(-0.347014578244)*x_ref[0]**(o-1)+(0.611296458158)
        ref[3,1,0,1]=o*(-0.11214501466)*x_ref[1]**(o-1)+(0.516100884048)
        ref[3,1,0,2]=o*(-0.746366893941)*x_ref[2]**(o-1)+(-0.429950357634)
        ref[3,1,1,0]=o*(-0.154199420675)*x_ref[0]**(o-1)+(-0.533525033722)
        ref[3,1,1,1]=o*(-0.305802182284)*x_ref[1]**(o-1)+(0.0249510778534)
        ref[3,1,1,2]=o*(0.511832901845)*x_ref[2]**(o-1)+(0.865423961695)
        ref[4,0,0,0]=o*(0.270951033321)*x_ref[0]**(o-1)+(0.293153287275)
        ref[4,0,0,1]=o*(0.499008893086)*x_ref[1]**(o-1)+(0.401454188898)
        ref[4,0,0,2]=o*(-0.0659433492694)*x_ref[2]**(o-1)+(0.0822124121616)
        ref[4,0,1,0]=o*(-0.770956381696)*x_ref[0]**(o-1)+(0.962572964564)
        ref[4,0,1,1]=o*(-0.173527764809)*x_ref[1]**(o-1)+(-0.764724502533)
        ref[4,0,1,2]=o*(0.686939133626)*x_ref[2]**(o-1)+(0.589061927699)
        ref[4,1,0,0]=o*(0.856144070276)*x_ref[0]**(o-1)+(-0.970851254884)
        ref[4,1,0,1]=o*(-0.200238695073)*x_ref[1]**(o-1)+(0.961653891428)
        ref[4,1,0,2]=o*(-0.921215646185)*x_ref[2]**(o-1)+(-0.927582743302)
        ref[4,1,1,0]=o*(0.518201773728)*x_ref[0]**(o-1)+(0.403940302766)
        ref[4,1,1,1]=o*(-0.0333026733014)*x_ref[1]**(o-1)+(0.602796446678)
        ref[4,1,1,2]=o*(0.956622086807)*x_ref[2]**(o-1)+(0.445065392727)
        ref[5,0,0,0]=o*(0.847162899069)*x_ref[0]**(o-1)+(-0.11079738914)
        ref[5,0,0,1]=o*(0.815117104026)*x_ref[1]**(o-1)+(-0.0126064617312)
        ref[5,0,0,2]=o*(0.434649696776)*x_ref[2]**(o-1)+(-0.925434721648)
        ref[5,0,1,0]=o*(0.827068835777)*x_ref[0]**(o-1)+(0.267176529776)
        ref[5,0,1,1]=o*(-0.954551141359)*x_ref[1]**(o-1)+(-0.847951054931)
        ref[5,0,1,2]=o*(0.939843025231)*x_ref[2]**(o-1)+(-0.405674486263)
        ref[5,1,0,0]=o*(0.303863917444)*x_ref[0]**(o-1)+(0.33377358223)
        ref[5,1,0,1]=o*(0.160136648003)*x_ref[1]**(o-1)+(0.557340036308)
        ref[5,1,0,2]=o*(0.348362672822)*x_ref[2]**(o-1)+(-0.719624087847)
        ref[5,1,1,0]=o*(-0.61009211312)*x_ref[0]**(o-1)+(0.948776841387)
        ref[5,1,1,1]=o*(0.0348157372912)*x_ref[1]**(o-1)+(0.415543380004)
        ref[5,1,1,2]=o*(-0.104796276752)*x_ref[2]**(o-1)+(0.109481954783)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_Solution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(0.875674079407)*x[0]**o+(-0.108360259885)*x[0]+(0.875125430202)*x[1]**o+(0.552749512786)*x[1]
        ref[0]=o*(0.875674079407)*x_ref[0]**(o-1)+(-0.108360259885)
        ref[1]=o*(0.875125430202)*x_ref[1]**(o-1)+(0.552749512786)
      else:
        s=(0.0715819443551)*x[0]**o+(-0.619431747175)*x[0]+(-0.369266459921)*x[1]**o+(-0.105717390052)*x[1]+(-0.0432478189165)*x[2]**o+(0.725545839252)*x[2]
        ref[0]=o*(0.0715819443551)*x_ref[0]**(o-1)+(-0.619431747175)
        ref[1]=o*(-0.369266459921)*x_ref[1]**(o-1)+(-0.105717390052)
        ref[2]=o*(-0.0432478189165)*x_ref[2]**(o-1)+(0.725545839252)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_Solution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(-0.626220139204)*x[0]**o+(0.729385849716)*x[0]+(-0.182106069322)*x[1]**o+(0.00285725500104)*x[1]
        s[1]=(-0.315419266082)*x[0]**o+(0.152806559812)*x[0]+(0.193255632909)*x[1]**o+(0.0415291248123)*x[1]
        ref[0,0]=o*(-0.626220139204)*x_ref[0]**(o-1)+(0.729385849716)
        ref[0,1]=o*(-0.182106069322)*x_ref[1]**(o-1)+(0.00285725500104)
        ref[1,0]=o*(-0.315419266082)*x_ref[0]**(o-1)+(0.152806559812)
        ref[1,1]=o*(0.193255632909)*x_ref[1]**(o-1)+(0.0415291248123)
      else:
        s[0]=(0.378833186935)*x[0]**o+(0.334764252787)*x[0]+(0.335440021844)*x[1]**o+(0.0418556980547)*x[1]+(-0.355548870692)*x[2]**o+(0.188469092541)*x[2]
        s[1]=(0.554815928082)*x[0]**o+(-0.245327666469)*x[0]+(-0.807150174696)*x[1]**o+(-0.0186528579571)*x[1]+(-0.644067730056)*x[2]**o+(-0.56385913942)*x[2]
        ref[0,0]=o*(0.378833186935)*x_ref[0]**(o-1)+(0.334764252787)
        ref[0,1]=o*(0.335440021844)*x_ref[1]**(o-1)+(0.0418556980547)
        ref[0,2]=o*(-0.355548870692)*x_ref[2]**(o-1)+(0.188469092541)
        ref[1,0]=o*(0.554815928082)*x_ref[0]**(o-1)+(-0.245327666469)
        ref[1,1]=o*(-0.807150174696)*x_ref[1]**(o-1)+(-0.0186528579571)
        ref[1,2]=o*(-0.644067730056)*x_ref[2]**(o-1)+(-0.56385913942)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_Solution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(-0.169259870335)*x[0]**o+(-0.219393962977)*x[0]+(-0.377170386042)*x[1]**o+(-0.298077349995)*x[1]
        s[0,1]=(0.260540461556)*x[0]**o+(-0.00323982683281)*x[0]+(-0.238801075348)*x[1]**o+(-0.512630151171)*x[1]
        s[0,2]=(0.417777690389)*x[0]**o+(0.76705549948)*x[0]+(0.411966415931)*x[1]**o+(0.634271298164)*x[1]
        s[0,3]=(-0.514577227559)*x[0]**o+(-0.449963426549)*x[0]+(-0.400962023711)*x[1]**o+(-0.980313307287)*x[1]
        s[0,4]=(0.210149387291)*x[0]**o+(0.339419881455)*x[0]+(-0.122433722312)*x[1]**o+(-0.52322566908)*x[1]
        s[1,0]=(-0.456855849349)*x[0]**o+(-0.759902063915)*x[0]+(-0.743965861438)*x[1]**o+(-0.331770868496)*x[1]
        s[1,1]=(0.0972196005334)*x[0]**o+(-0.69511305009)*x[0]+(0.402041128045)*x[1]**o+(-0.924200483761)*x[1]
        s[1,2]=(-0.368298482402)*x[0]**o+(-0.351904909828)*x[0]+(-0.29784981961)*x[1]**o+(-0.0850625471026)*x[1]
        s[1,3]=(-0.79959026688)*x[0]**o+(-0.143869612796)*x[0]+(0.545276895212)*x[1]**o+(0.0277273177848)*x[1]
        s[1,4]=(-0.786210582915)*x[0]**o+(0.17248521577)*x[0]+(0.682847252483)*x[1]**o+(0.975158490563)*x[1]
        s[2,0]=(-0.00180696139571)*x[0]**o+(0.0644626145328)*x[0]+(-0.332784099857)*x[1]**o+(-0.62718290936)*x[1]
        s[2,1]=(0.529388118928)*x[0]**o+(0.679837615948)*x[0]+(-0.800039234078)*x[1]**o+(0.762585600198)*x[1]
        s[2,2]=(0.202108406753)*x[0]**o+(0.602440423874)*x[0]+(-0.544751041557)*x[1]**o+(-0.479387390671)*x[1]
        s[2,3]=(0.339973882877)*x[0]**o+(0.465854524429)*x[0]+(0.318692368851)*x[1]**o+(0.869234081961)*x[1]
        s[2,4]=(0.337044733833)*x[0]**o+(-0.401620568253)*x[0]+(-0.0302231005328)*x[1]**o+(0.218919539213)*x[1]
        s[3,0]=(-0.961845777661)*x[0]**o+(-0.509480342678)*x[0]+(0.970831236524)*x[1]**o+(-0.234720318489)*x[1]
        s[3,1]=(0.266147738115)*x[0]**o+(-0.0146288256163)*x[0]+(0.38625117249)*x[1]**o+(-0.449526886565)*x[1]
        s[3,2]=(-0.0829643094057)*x[0]**o+(-0.31837766369)*x[0]+(0.868932473361)*x[1]**o+(0.0753845710061)*x[1]
        s[3,3]=(-0.311549181378)*x[0]**o+(-0.0704226477679)*x[0]+(-0.80469940492)*x[1]**o+(0.17810271607)*x[1]
        s[3,4]=(0.496421938706)*x[0]**o+(-0.94316161479)*x[0]+(-0.759730393246)*x[1]**o+(0.857927191642)*x[1]
        ref[0,0,0]=o*(-0.169259870335)*x_ref[0]**(o-1)+(-0.219393962977)
        ref[0,0,1]=o*(-0.377170386042)*x_ref[1]**(o-1)+(-0.298077349995)
        ref[0,1,0]=o*(0.260540461556)*x_ref[0]**(o-1)+(-0.00323982683281)
        ref[0,1,1]=o*(-0.238801075348)*x_ref[1]**(o-1)+(-0.512630151171)
        ref[0,2,0]=o*(0.417777690389)*x_ref[0]**(o-1)+(0.76705549948)
        ref[0,2,1]=o*(0.411966415931)*x_ref[1]**(o-1)+(0.634271298164)
        ref[0,3,0]=o*(-0.514577227559)*x_ref[0]**(o-1)+(-0.449963426549)
        ref[0,3,1]=o*(-0.400962023711)*x_ref[1]**(o-1)+(-0.980313307287)
        ref[0,4,0]=o*(0.210149387291)*x_ref[0]**(o-1)+(0.339419881455)
        ref[0,4,1]=o*(-0.122433722312)*x_ref[1]**(o-1)+(-0.52322566908)
        ref[1,0,0]=o*(-0.456855849349)*x_ref[0]**(o-1)+(-0.759902063915)
        ref[1,0,1]=o*(-0.743965861438)*x_ref[1]**(o-1)+(-0.331770868496)
        ref[1,1,0]=o*(0.0972196005334)*x_ref[0]**(o-1)+(-0.69511305009)
        ref[1,1,1]=o*(0.402041128045)*x_ref[1]**(o-1)+(-0.924200483761)
        ref[1,2,0]=o*(-0.368298482402)*x_ref[0]**(o-1)+(-0.351904909828)
        ref[1,2,1]=o*(-0.29784981961)*x_ref[1]**(o-1)+(-0.0850625471026)
        ref[1,3,0]=o*(-0.79959026688)*x_ref[0]**(o-1)+(-0.143869612796)
        ref[1,3,1]=o*(0.545276895212)*x_ref[1]**(o-1)+(0.0277273177848)
        ref[1,4,0]=o*(-0.786210582915)*x_ref[0]**(o-1)+(0.17248521577)
        ref[1,4,1]=o*(0.682847252483)*x_ref[1]**(o-1)+(0.975158490563)
        ref[2,0,0]=o*(-0.00180696139571)*x_ref[0]**(o-1)+(0.0644626145328)
        ref[2,0,1]=o*(-0.332784099857)*x_ref[1]**(o-1)+(-0.62718290936)
        ref[2,1,0]=o*(0.529388118928)*x_ref[0]**(o-1)+(0.679837615948)
        ref[2,1,1]=o*(-0.800039234078)*x_ref[1]**(o-1)+(0.762585600198)
        ref[2,2,0]=o*(0.202108406753)*x_ref[0]**(o-1)+(0.602440423874)
        ref[2,2,1]=o*(-0.544751041557)*x_ref[1]**(o-1)+(-0.479387390671)
        ref[2,3,0]=o*(0.339973882877)*x_ref[0]**(o-1)+(0.465854524429)
        ref[2,3,1]=o*(0.318692368851)*x_ref[1]**(o-1)+(0.869234081961)
        ref[2,4,0]=o*(0.337044733833)*x_ref[0]**(o-1)+(-0.401620568253)
        ref[2,4,1]=o*(-0.0302231005328)*x_ref[1]**(o-1)+(0.218919539213)
        ref[3,0,0]=o*(-0.961845777661)*x_ref[0]**(o-1)+(-0.509480342678)
        ref[3,0,1]=o*(0.970831236524)*x_ref[1]**(o-1)+(-0.234720318489)
        ref[3,1,0]=o*(0.266147738115)*x_ref[0]**(o-1)+(-0.0146288256163)
        ref[3,1,1]=o*(0.38625117249)*x_ref[1]**(o-1)+(-0.449526886565)
        ref[3,2,0]=o*(-0.0829643094057)*x_ref[0]**(o-1)+(-0.31837766369)
        ref[3,2,1]=o*(0.868932473361)*x_ref[1]**(o-1)+(0.0753845710061)
        ref[3,3,0]=o*(-0.311549181378)*x_ref[0]**(o-1)+(-0.0704226477679)
        ref[3,3,1]=o*(-0.80469940492)*x_ref[1]**(o-1)+(0.17810271607)
        ref[3,4,0]=o*(0.496421938706)*x_ref[0]**(o-1)+(-0.94316161479)
        ref[3,4,1]=o*(-0.759730393246)*x_ref[1]**(o-1)+(0.857927191642)
      else:
        s[0,0]=(0.323269202132)*x[0]**o+(-0.0941675978056)*x[0]+(0.573540368658)*x[1]**o+(-0.476661689561)*x[1]+(-0.332358000871)*x[2]**o+(-0.817017744339)*x[2]
        s[0,1]=(0.116142610738)*x[0]**o+(0.0212217636606)*x[0]+(0.494731157541)*x[1]**o+(0.295191373453)*x[1]+(0.0528301662295)*x[2]**o+(-0.444651733146)*x[2]
        s[0,2]=(0.456064446338)*x[0]**o+(-0.69428330869)*x[0]+(0.677929037713)*x[1]**o+(-0.320468300843)*x[1]+(-0.87760608927)*x[2]**o+(-0.659457899497)*x[2]
        s[0,3]=(-0.189428313133)*x[0]**o+(-0.799817301777)*x[0]+(0.100831997136)*x[1]**o+(-0.113782495668)*x[1]+(0.455358915627)*x[2]**o+(-0.708374606976)*x[2]
        s[0,4]=(-0.118907620954)*x[0]**o+(-0.353855451309)*x[0]+(-0.773766114927)*x[1]**o+(-0.845133073805)*x[1]+(0.803289878128)*x[2]**o+(-0.852839704894)*x[2]
        s[1,0]=(-0.887023265539)*x[0]**o+(-0.792304610398)*x[0]+(0.0362402407711)*x[1]**o+(-0.260378118021)*x[1]+(0.142912579168)*x[2]**o+(0.544143870337)*x[2]
        s[1,1]=(0.805678954301)*x[0]**o+(0.701112598554)*x[0]+(0.922879760039)*x[1]**o+(-0.219024852502)*x[1]+(0.365922209323)*x[2]**o+(-0.420665475533)*x[2]
        s[1,2]=(0.832254579623)*x[0]**o+(-0.413462011593)*x[0]+(0.0852114474998)*x[1]**o+(-0.0932078507511)*x[1]+(-0.248076148787)*x[2]**o+(-0.428017096895)*x[2]
        s[1,3]=(-0.4967053049)*x[0]**o+(-0.50471234025)*x[0]+(-0.117548830517)*x[1]**o+(0.81561414915)*x[1]+(0.61128237476)*x[2]**o+(0.695201350291)*x[2]
        s[1,4]=(0.0281073079277)*x[0]**o+(0.62841659852)*x[0]+(0.685439649725)*x[1]**o+(-0.093589324186)*x[1]+(-0.848012915274)*x[2]**o+(-0.800065648129)*x[2]
        s[2,0]=(-0.35245094908)*x[0]**o+(0.40394009353)*x[0]+(-0.420215110026)*x[1]**o+(0.488912100088)*x[1]+(0.679732915429)*x[2]**o+(-0.362263037628)*x[2]
        s[2,1]=(-0.515442192308)*x[0]**o+(0.634797288073)*x[0]+(0.461169879242)*x[1]**o+(-0.51743910056)*x[1]+(-0.227585094451)*x[2]**o+(0.00407235369734)*x[2]
        s[2,2]=(-0.442867477277)*x[0]**o+(-0.212399902385)*x[0]+(-0.768565150664)*x[1]**o+(-0.398107108767)*x[1]+(-0.23802726279)*x[2]**o+(-0.838176628616)*x[2]
        s[2,3]=(0.491473254552)*x[0]**o+(0.00763471198539)*x[0]+(0.255444615913)*x[1]**o+(-0.655565675326)*x[1]+(0.324558324765)*x[2]**o+(-0.224946422831)*x[2]
        s[2,4]=(0.880395740489)*x[0]**o+(0.940871214043)*x[0]+(0.708978280722)*x[1]**o+(0.448470774528)*x[1]+(0.37065402316)*x[2]**o+(0.761394343858)*x[2]
        s[3,0]=(-0.455887951889)*x[0]**o+(0.00570480201782)*x[0]+(0.413555793704)*x[1]**o+(-0.313434157216)*x[1]+(0.339744130043)*x[2]**o+(-0.105392254622)*x[2]
        s[3,1]=(0.0528573842143)*x[0]**o+(-0.11565565)*x[0]+(-0.0443876403016)*x[1]**o+(-0.716723173535)*x[1]+(0.100057602497)*x[2]**o+(-0.891795165388)*x[2]
        s[3,2]=(-0.317795319526)*x[0]**o+(0.210753288047)*x[0]+(-0.666306164291)*x[1]**o+(-0.561602279575)*x[1]+(0.55557323269)*x[2]**o+(-0.479408955582)*x[2]
        s[3,3]=(0.49345832209)*x[0]**o+(-0.0684540951119)*x[0]+(0.69792572139)*x[1]**o+(-0.941053746122)*x[1]+(0.127906327934)*x[2]**o+(0.97752169437)*x[2]
        s[3,4]=(-0.553212311288)*x[0]**o+(0.718429728418)*x[0]+(0.179731047542)*x[1]**o+(0.169591114318)*x[1]+(0.402679892192)*x[2]**o+(0.0690728029503)*x[2]
        ref[0,0,0]=o*(0.323269202132)*x_ref[0]**(o-1)+(-0.0941675978056)
        ref[0,0,1]=o*(0.573540368658)*x_ref[1]**(o-1)+(-0.476661689561)
        ref[0,0,2]=o*(-0.332358000871)*x_ref[2]**(o-1)+(-0.817017744339)
        ref[0,1,0]=o*(0.116142610738)*x_ref[0]**(o-1)+(0.0212217636606)
        ref[0,1,1]=o*(0.494731157541)*x_ref[1]**(o-1)+(0.295191373453)
        ref[0,1,2]=o*(0.0528301662295)*x_ref[2]**(o-1)+(-0.444651733146)
        ref[0,2,0]=o*(0.456064446338)*x_ref[0]**(o-1)+(-0.69428330869)
        ref[0,2,1]=o*(0.677929037713)*x_ref[1]**(o-1)+(-0.320468300843)
        ref[0,2,2]=o*(-0.87760608927)*x_ref[2]**(o-1)+(-0.659457899497)
        ref[0,3,0]=o*(-0.189428313133)*x_ref[0]**(o-1)+(-0.799817301777)
        ref[0,3,1]=o*(0.100831997136)*x_ref[1]**(o-1)+(-0.113782495668)
        ref[0,3,2]=o*(0.455358915627)*x_ref[2]**(o-1)+(-0.708374606976)
        ref[0,4,0]=o*(-0.118907620954)*x_ref[0]**(o-1)+(-0.353855451309)
        ref[0,4,1]=o*(-0.773766114927)*x_ref[1]**(o-1)+(-0.845133073805)
        ref[0,4,2]=o*(0.803289878128)*x_ref[2]**(o-1)+(-0.852839704894)
        ref[1,0,0]=o*(-0.887023265539)*x_ref[0]**(o-1)+(-0.792304610398)
        ref[1,0,1]=o*(0.0362402407711)*x_ref[1]**(o-1)+(-0.260378118021)
        ref[1,0,2]=o*(0.142912579168)*x_ref[2]**(o-1)+(0.544143870337)
        ref[1,1,0]=o*(0.805678954301)*x_ref[0]**(o-1)+(0.701112598554)
        ref[1,1,1]=o*(0.922879760039)*x_ref[1]**(o-1)+(-0.219024852502)
        ref[1,1,2]=o*(0.365922209323)*x_ref[2]**(o-1)+(-0.420665475533)
        ref[1,2,0]=o*(0.832254579623)*x_ref[0]**(o-1)+(-0.413462011593)
        ref[1,2,1]=o*(0.0852114474998)*x_ref[1]**(o-1)+(-0.0932078507511)
        ref[1,2,2]=o*(-0.248076148787)*x_ref[2]**(o-1)+(-0.428017096895)
        ref[1,3,0]=o*(-0.4967053049)*x_ref[0]**(o-1)+(-0.50471234025)
        ref[1,3,1]=o*(-0.117548830517)*x_ref[1]**(o-1)+(0.81561414915)
        ref[1,3,2]=o*(0.61128237476)*x_ref[2]**(o-1)+(0.695201350291)
        ref[1,4,0]=o*(0.0281073079277)*x_ref[0]**(o-1)+(0.62841659852)
        ref[1,4,1]=o*(0.685439649725)*x_ref[1]**(o-1)+(-0.093589324186)
        ref[1,4,2]=o*(-0.848012915274)*x_ref[2]**(o-1)+(-0.800065648129)
        ref[2,0,0]=o*(-0.35245094908)*x_ref[0]**(o-1)+(0.40394009353)
        ref[2,0,1]=o*(-0.420215110026)*x_ref[1]**(o-1)+(0.488912100088)
        ref[2,0,2]=o*(0.679732915429)*x_ref[2]**(o-1)+(-0.362263037628)
        ref[2,1,0]=o*(-0.515442192308)*x_ref[0]**(o-1)+(0.634797288073)
        ref[2,1,1]=o*(0.461169879242)*x_ref[1]**(o-1)+(-0.51743910056)
        ref[2,1,2]=o*(-0.227585094451)*x_ref[2]**(o-1)+(0.00407235369734)
        ref[2,2,0]=o*(-0.442867477277)*x_ref[0]**(o-1)+(-0.212399902385)
        ref[2,2,1]=o*(-0.768565150664)*x_ref[1]**(o-1)+(-0.398107108767)
        ref[2,2,2]=o*(-0.23802726279)*x_ref[2]**(o-1)+(-0.838176628616)
        ref[2,3,0]=o*(0.491473254552)*x_ref[0]**(o-1)+(0.00763471198539)
        ref[2,3,1]=o*(0.255444615913)*x_ref[1]**(o-1)+(-0.655565675326)
        ref[2,3,2]=o*(0.324558324765)*x_ref[2]**(o-1)+(-0.224946422831)
        ref[2,4,0]=o*(0.880395740489)*x_ref[0]**(o-1)+(0.940871214043)
        ref[2,4,1]=o*(0.708978280722)*x_ref[1]**(o-1)+(0.448470774528)
        ref[2,4,2]=o*(0.37065402316)*x_ref[2]**(o-1)+(0.761394343858)
        ref[3,0,0]=o*(-0.455887951889)*x_ref[0]**(o-1)+(0.00570480201782)
        ref[3,0,1]=o*(0.413555793704)*x_ref[1]**(o-1)+(-0.313434157216)
        ref[3,0,2]=o*(0.339744130043)*x_ref[2]**(o-1)+(-0.105392254622)
        ref[3,1,0]=o*(0.0528573842143)*x_ref[0]**(o-1)+(-0.11565565)
        ref[3,1,1]=o*(-0.0443876403016)*x_ref[1]**(o-1)+(-0.716723173535)
        ref[3,1,2]=o*(0.100057602497)*x_ref[2]**(o-1)+(-0.891795165388)
        ref[3,2,0]=o*(-0.317795319526)*x_ref[0]**(o-1)+(0.210753288047)
        ref[3,2,1]=o*(-0.666306164291)*x_ref[1]**(o-1)+(-0.561602279575)
        ref[3,2,2]=o*(0.55557323269)*x_ref[2]**(o-1)+(-0.479408955582)
        ref[3,3,0]=o*(0.49345832209)*x_ref[0]**(o-1)+(-0.0684540951119)
        ref[3,3,1]=o*(0.69792572139)*x_ref[1]**(o-1)+(-0.941053746122)
        ref[3,3,2]=o*(0.127906327934)*x_ref[2]**(o-1)+(0.97752169437)
        ref[3,4,0]=o*(-0.553212311288)*x_ref[0]**(o-1)+(0.718429728418)
        ref[3,4,1]=o*(0.179731047542)*x_ref[1]**(o-1)+(0.169591114318)
        ref[3,4,2]=o*(0.402679892192)*x_ref[2]**(o-1)+(0.0690728029503)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_Solution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(0.64178693834)*x[0]**o+(0.436828658021)*x[0]+(0.525622010733)*x[1]**o+(-0.0947845294558)*x[1]
        s[0,0,1]=(0.960929069825)*x[0]**o+(-0.0925500837249)*x[0]+(0.516628691912)*x[1]**o+(-0.713433036463)*x[1]
        s[0,1,0]=(-0.130010775216)*x[0]**o+(-0.569868353102)*x[0]+(-0.834642447971)*x[1]**o+(-0.262937552972)*x[1]
        s[0,1,1]=(-0.720926391535)*x[0]**o+(0.803576593732)*x[0]+(-0.613803785285)*x[1]**o+(0.713012726555)*x[1]
        s[1,0,0]=(-0.960850249593)*x[0]**o+(-0.891033054973)*x[0]+(0.750989994686)*x[1]**o+(0.983245329088)*x[1]
        s[1,0,1]=(-0.383882505108)*x[0]**o+(0.980533529745)*x[0]+(0.891761213135)*x[1]**o+(0.758837665244)*x[1]
        s[1,1,0]=(0.933866632845)*x[0]**o+(0.618830180361)*x[0]+(0.083925621148)*x[1]**o+(-0.681925439524)*x[1]
        s[1,1,1]=(0.344074931746)*x[0]**o+(-0.0411811037823)*x[0]+(0.582962923539)*x[1]**o+(0.242279633347)*x[1]
        s[2,0,0]=(0.0949732595833)*x[0]**o+(0.947484666339)*x[0]+(-0.856377492747)*x[1]**o+(0.0911692318532)*x[1]
        s[2,0,1]=(-0.294511041595)*x[0]**o+(0.252889321608)*x[0]+(-0.204790616562)*x[1]**o+(0.318333561489)*x[1]
        s[2,1,0]=(0.252492478572)*x[0]**o+(-0.594109481282)*x[0]+(-0.0260393414988)*x[1]**o+(0.563948256582)*x[1]
        s[2,1,1]=(-0.714450749502)*x[0]**o+(0.302407929025)*x[0]+(0.147817927373)*x[1]**o+(-0.830954899815)*x[1]
        s[3,0,0]=(0.56545292458)*x[0]**o+(0.486566855088)*x[0]+(0.990278375714)*x[1]**o+(0.229104000835)*x[1]
        s[3,0,1]=(-0.734062795067)*x[0]**o+(-0.191169476406)*x[0]+(-0.251242465039)*x[1]**o+(0.390333958806)*x[1]
        s[3,1,0]=(-0.337774363178)*x[0]**o+(0.700342293089)*x[0]+(-0.836836760567)*x[1]**o+(-0.0429509469196)*x[1]
        s[3,1,1]=(0.8888799528)*x[0]**o+(0.225714398328)*x[0]+(0.622017922629)*x[1]**o+(-0.614413552831)*x[1]
        s[4,0,0]=(0.882570398452)*x[0]**o+(-0.0550944535693)*x[0]+(-0.674688995642)*x[1]**o+(0.489860414323)*x[1]
        s[4,0,1]=(0.192190815605)*x[0]**o+(0.615603974895)*x[0]+(-0.713940642233)*x[1]**o+(0.692854992944)*x[1]
        s[4,1,0]=(0.383266832431)*x[0]**o+(0.294686738948)*x[0]+(0.119658392209)*x[1]**o+(-0.550041082523)*x[1]
        s[4,1,1]=(-0.0639894922603)*x[0]**o+(0.0322835841591)*x[0]+(-0.148146549294)*x[1]**o+(-0.472048255747)*x[1]
        s[5,0,0]=(-0.596603356022)*x[0]**o+(0.0764697562663)*x[0]+(0.23065037103)*x[1]**o+(-0.993102200263)*x[1]
        s[5,0,1]=(-0.268998655952)*x[0]**o+(0.911781569559)*x[0]+(-0.424230615868)*x[1]**o+(-0.196872693015)*x[1]
        s[5,1,0]=(-0.145090787501)*x[0]**o+(0.694829120617)*x[0]+(-0.972729923357)*x[1]**o+(-0.737541664159)*x[1]
        s[5,1,1]=(-0.196228029528)*x[0]**o+(-0.385307212434)*x[0]+(-0.793635672028)*x[1]**o+(-0.547670068083)*x[1]
        ref[0,0,0,0]=o*(0.64178693834)*x_ref[0]**(o-1)+(0.436828658021)
        ref[0,0,0,1]=o*(0.525622010733)*x_ref[1]**(o-1)+(-0.0947845294558)
        ref[0,0,1,0]=o*(0.960929069825)*x_ref[0]**(o-1)+(-0.0925500837249)
        ref[0,0,1,1]=o*(0.516628691912)*x_ref[1]**(o-1)+(-0.713433036463)
        ref[0,1,0,0]=o*(-0.130010775216)*x_ref[0]**(o-1)+(-0.569868353102)
        ref[0,1,0,1]=o*(-0.834642447971)*x_ref[1]**(o-1)+(-0.262937552972)
        ref[0,1,1,0]=o*(-0.720926391535)*x_ref[0]**(o-1)+(0.803576593732)
        ref[0,1,1,1]=o*(-0.613803785285)*x_ref[1]**(o-1)+(0.713012726555)
        ref[1,0,0,0]=o*(-0.960850249593)*x_ref[0]**(o-1)+(-0.891033054973)
        ref[1,0,0,1]=o*(0.750989994686)*x_ref[1]**(o-1)+(0.983245329088)
        ref[1,0,1,0]=o*(-0.383882505108)*x_ref[0]**(o-1)+(0.980533529745)
        ref[1,0,1,1]=o*(0.891761213135)*x_ref[1]**(o-1)+(0.758837665244)
        ref[1,1,0,0]=o*(0.933866632845)*x_ref[0]**(o-1)+(0.618830180361)
        ref[1,1,0,1]=o*(0.083925621148)*x_ref[1]**(o-1)+(-0.681925439524)
        ref[1,1,1,0]=o*(0.344074931746)*x_ref[0]**(o-1)+(-0.0411811037823)
        ref[1,1,1,1]=o*(0.582962923539)*x_ref[1]**(o-1)+(0.242279633347)
        ref[2,0,0,0]=o*(0.0949732595833)*x_ref[0]**(o-1)+(0.947484666339)
        ref[2,0,0,1]=o*(-0.856377492747)*x_ref[1]**(o-1)+(0.0911692318532)
        ref[2,0,1,0]=o*(-0.294511041595)*x_ref[0]**(o-1)+(0.252889321608)
        ref[2,0,1,1]=o*(-0.204790616562)*x_ref[1]**(o-1)+(0.318333561489)
        ref[2,1,0,0]=o*(0.252492478572)*x_ref[0]**(o-1)+(-0.594109481282)
        ref[2,1,0,1]=o*(-0.0260393414988)*x_ref[1]**(o-1)+(0.563948256582)
        ref[2,1,1,0]=o*(-0.714450749502)*x_ref[0]**(o-1)+(0.302407929025)
        ref[2,1,1,1]=o*(0.147817927373)*x_ref[1]**(o-1)+(-0.830954899815)
        ref[3,0,0,0]=o*(0.56545292458)*x_ref[0]**(o-1)+(0.486566855088)
        ref[3,0,0,1]=o*(0.990278375714)*x_ref[1]**(o-1)+(0.229104000835)
        ref[3,0,1,0]=o*(-0.734062795067)*x_ref[0]**(o-1)+(-0.191169476406)
        ref[3,0,1,1]=o*(-0.251242465039)*x_ref[1]**(o-1)+(0.390333958806)
        ref[3,1,0,0]=o*(-0.337774363178)*x_ref[0]**(o-1)+(0.700342293089)
        ref[3,1,0,1]=o*(-0.836836760567)*x_ref[1]**(o-1)+(-0.0429509469196)
        ref[3,1,1,0]=o*(0.8888799528)*x_ref[0]**(o-1)+(0.225714398328)
        ref[3,1,1,1]=o*(0.622017922629)*x_ref[1]**(o-1)+(-0.614413552831)
        ref[4,0,0,0]=o*(0.882570398452)*x_ref[0]**(o-1)+(-0.0550944535693)
        ref[4,0,0,1]=o*(-0.674688995642)*x_ref[1]**(o-1)+(0.489860414323)
        ref[4,0,1,0]=o*(0.192190815605)*x_ref[0]**(o-1)+(0.615603974895)
        ref[4,0,1,1]=o*(-0.713940642233)*x_ref[1]**(o-1)+(0.692854992944)
        ref[4,1,0,0]=o*(0.383266832431)*x_ref[0]**(o-1)+(0.294686738948)
        ref[4,1,0,1]=o*(0.119658392209)*x_ref[1]**(o-1)+(-0.550041082523)
        ref[4,1,1,0]=o*(-0.0639894922603)*x_ref[0]**(o-1)+(0.0322835841591)
        ref[4,1,1,1]=o*(-0.148146549294)*x_ref[1]**(o-1)+(-0.472048255747)
        ref[5,0,0,0]=o*(-0.596603356022)*x_ref[0]**(o-1)+(0.0764697562663)
        ref[5,0,0,1]=o*(0.23065037103)*x_ref[1]**(o-1)+(-0.993102200263)
        ref[5,0,1,0]=o*(-0.268998655952)*x_ref[0]**(o-1)+(0.911781569559)
        ref[5,0,1,1]=o*(-0.424230615868)*x_ref[1]**(o-1)+(-0.196872693015)
        ref[5,1,0,0]=o*(-0.145090787501)*x_ref[0]**(o-1)+(0.694829120617)
        ref[5,1,0,1]=o*(-0.972729923357)*x_ref[1]**(o-1)+(-0.737541664159)
        ref[5,1,1,0]=o*(-0.196228029528)*x_ref[0]**(o-1)+(-0.385307212434)
        ref[5,1,1,1]=o*(-0.793635672028)*x_ref[1]**(o-1)+(-0.547670068083)
      else:
        s[0,0,0]=(0.130537344403)*x[0]**o+(-0.716585435036)*x[0]+(0.885628707489)*x[1]**o+(0.359657295083)*x[1]+(0.849414417001)*x[2]**o+(0.417004569061)*x[2]
        s[0,0,1]=(0.582291844761)*x[0]**o+(0.515724457456)*x[0]+(0.768535647866)*x[1]**o+(0.34108474273)*x[1]+(-0.620508267743)*x[2]**o+(-0.342013446761)*x[2]
        s[0,1,0]=(-0.531235123931)*x[0]**o+(-0.765634705075)*x[0]+(0.228864105339)*x[1]**o+(-0.491530876451)*x[1]+(0.521065315283)*x[2]**o+(-0.951623087256)*x[2]
        s[0,1,1]=(0.0716827639786)*x[0]**o+(-0.16499705145)*x[0]+(-0.0190243436099)*x[1]**o+(-0.926607070761)*x[1]+(-0.55142315912)*x[2]**o+(-0.0794842630428)*x[2]
        s[1,0,0]=(0.182139977684)*x[0]**o+(-0.0362484111368)*x[0]+(-0.752143193872)*x[1]**o+(-0.316455062654)*x[1]+(0.269718723768)*x[2]**o+(-0.313902463376)*x[2]
        s[1,0,1]=(0.743368204514)*x[0]**o+(-0.822345231625)*x[0]+(0.814851367707)*x[1]**o+(0.702205919957)*x[1]+(-0.0503255941307)*x[2]**o+(-0.0318030472244)*x[2]
        s[1,1,0]=(0.022342448883)*x[0]**o+(-0.352009543909)*x[0]+(0.5204277493)*x[1]**o+(-0.741699265227)*x[1]+(0.00608894949938)*x[2]**o+(0.965930515392)*x[2]
        s[1,1,1]=(-0.0920262487111)*x[0]**o+(0.249857266149)*x[0]+(0.916353890515)*x[1]**o+(-0.0594616510094)*x[1]+(-0.402620394698)*x[2]**o+(-0.00864935399915)*x[2]
        s[2,0,0]=(-0.944378332174)*x[0]**o+(0.501614469906)*x[0]+(0.829832603587)*x[1]**o+(0.4202031628)*x[1]+(0.886338738184)*x[2]**o+(0.811714471885)*x[2]
        s[2,0,1]=(0.0413505829692)*x[0]**o+(-0.59249386093)*x[0]+(0.478645406371)*x[1]**o+(0.221389337839)*x[1]+(0.18016770891)*x[2]**o+(-0.783010309878)*x[2]
        s[2,1,0]=(0.641325174447)*x[0]**o+(0.859809801095)*x[0]+(0.238917443521)*x[1]**o+(0.169387212912)*x[1]+(0.118386087343)*x[2]**o+(0.7091562331)*x[2]
        s[2,1,1]=(0.683767548268)*x[0]**o+(-0.488504930782)*x[0]+(0.421423574907)*x[1]**o+(0.677076626099)*x[1]+(-0.0955208633306)*x[2]**o+(0.545750792826)*x[2]
        s[3,0,0]=(-0.925067175669)*x[0]**o+(0.550204660551)*x[0]+(0.435686889479)*x[1]**o+(-0.171725370784)*x[1]+(0.182097753276)*x[2]**o+(0.493662375098)*x[2]
        s[3,0,1]=(-0.719237805296)*x[0]**o+(0.669817079155)*x[0]+(0.660680613106)*x[1]**o+(-0.729927483007)*x[1]+(-0.162704346009)*x[2]**o+(-0.559206291015)*x[2]
        s[3,1,0]=(0.129735268583)*x[0]**o+(-0.411853842222)*x[0]+(0.045438376206)*x[1]**o+(0.327804746872)*x[1]+(-0.639433216582)*x[2]**o+(0.451106136717)*x[2]
        s[3,1,1]=(-0.811557092293)*x[0]**o+(-0.818758274715)*x[0]+(-0.712815291675)*x[1]**o+(0.729038274951)*x[1]+(-0.826837009476)*x[2]**o+(0.340025404977)*x[2]
        s[4,0,0]=(-0.507284842259)*x[0]**o+(0.457554461786)*x[0]+(0.419345933038)*x[1]**o+(-0.7773529718)*x[1]+(-0.626475804952)*x[2]**o+(-0.784196320861)*x[2]
        s[4,0,1]=(-0.0340250322306)*x[0]**o+(-0.0936499769437)*x[0]+(-0.777584776075)*x[1]**o+(-0.435876632622)*x[1]+(0.999336374444)*x[2]**o+(-0.247149214414)*x[2]
        s[4,1,0]=(0.21620967013)*x[0]**o+(0.213039866961)*x[0]+(0.83049354428)*x[1]**o+(0.651430926943)*x[1]+(0.201648695549)*x[2]**o+(-0.609802970877)*x[2]
        s[4,1,1]=(0.842001670092)*x[0]**o+(-0.868487783326)*x[0]+(-0.288512200338)*x[1]**o+(-0.654831485218)*x[1]+(-0.111859725689)*x[2]**o+(0.728195130453)*x[2]
        s[5,0,0]=(0.473856201092)*x[0]**o+(0.377649949188)*x[0]+(0.511638340284)*x[1]**o+(-0.150021940198)*x[1]+(-0.801655915671)*x[2]**o+(-0.603617347175)*x[2]
        s[5,0,1]=(0.945961549464)*x[0]**o+(0.40151064288)*x[0]+(0.151039821252)*x[1]**o+(0.977148741848)*x[1]+(0.0729651283448)*x[2]**o+(0.365625847237)*x[2]
        s[5,1,0]=(0.613037024504)*x[0]**o+(-0.743080801009)*x[0]+(-0.518736037968)*x[1]**o+(0.240273029617)*x[1]+(0.437464780338)*x[2]**o+(0.897940669321)*x[2]
        s[5,1,1]=(-0.149245273038)*x[0]**o+(-0.387037171901)*x[0]+(0.0606843165198)*x[1]**o+(-0.713188961184)*x[1]+(0.622672840108)*x[2]**o+(-0.579757019852)*x[2]
        ref[0,0,0,0]=o*(0.130537344403)*x_ref[0]**(o-1)+(-0.716585435036)
        ref[0,0,0,1]=o*(0.885628707489)*x_ref[1]**(o-1)+(0.359657295083)
        ref[0,0,0,2]=o*(0.849414417001)*x_ref[2]**(o-1)+(0.417004569061)
        ref[0,0,1,0]=o*(0.582291844761)*x_ref[0]**(o-1)+(0.515724457456)
        ref[0,0,1,1]=o*(0.768535647866)*x_ref[1]**(o-1)+(0.34108474273)
        ref[0,0,1,2]=o*(-0.620508267743)*x_ref[2]**(o-1)+(-0.342013446761)
        ref[0,1,0,0]=o*(-0.531235123931)*x_ref[0]**(o-1)+(-0.765634705075)
        ref[0,1,0,1]=o*(0.228864105339)*x_ref[1]**(o-1)+(-0.491530876451)
        ref[0,1,0,2]=o*(0.521065315283)*x_ref[2]**(o-1)+(-0.951623087256)
        ref[0,1,1,0]=o*(0.0716827639786)*x_ref[0]**(o-1)+(-0.16499705145)
        ref[0,1,1,1]=o*(-0.0190243436099)*x_ref[1]**(o-1)+(-0.926607070761)
        ref[0,1,1,2]=o*(-0.55142315912)*x_ref[2]**(o-1)+(-0.0794842630428)
        ref[1,0,0,0]=o*(0.182139977684)*x_ref[0]**(o-1)+(-0.0362484111368)
        ref[1,0,0,1]=o*(-0.752143193872)*x_ref[1]**(o-1)+(-0.316455062654)
        ref[1,0,0,2]=o*(0.269718723768)*x_ref[2]**(o-1)+(-0.313902463376)
        ref[1,0,1,0]=o*(0.743368204514)*x_ref[0]**(o-1)+(-0.822345231625)
        ref[1,0,1,1]=o*(0.814851367707)*x_ref[1]**(o-1)+(0.702205919957)
        ref[1,0,1,2]=o*(-0.0503255941307)*x_ref[2]**(o-1)+(-0.0318030472244)
        ref[1,1,0,0]=o*(0.022342448883)*x_ref[0]**(o-1)+(-0.352009543909)
        ref[1,1,0,1]=o*(0.5204277493)*x_ref[1]**(o-1)+(-0.741699265227)
        ref[1,1,0,2]=o*(0.00608894949938)*x_ref[2]**(o-1)+(0.965930515392)
        ref[1,1,1,0]=o*(-0.0920262487111)*x_ref[0]**(o-1)+(0.249857266149)
        ref[1,1,1,1]=o*(0.916353890515)*x_ref[1]**(o-1)+(-0.0594616510094)
        ref[1,1,1,2]=o*(-0.402620394698)*x_ref[2]**(o-1)+(-0.00864935399915)
        ref[2,0,0,0]=o*(-0.944378332174)*x_ref[0]**(o-1)+(0.501614469906)
        ref[2,0,0,1]=o*(0.829832603587)*x_ref[1]**(o-1)+(0.4202031628)
        ref[2,0,0,2]=o*(0.886338738184)*x_ref[2]**(o-1)+(0.811714471885)
        ref[2,0,1,0]=o*(0.0413505829692)*x_ref[0]**(o-1)+(-0.59249386093)
        ref[2,0,1,1]=o*(0.478645406371)*x_ref[1]**(o-1)+(0.221389337839)
        ref[2,0,1,2]=o*(0.18016770891)*x_ref[2]**(o-1)+(-0.783010309878)
        ref[2,1,0,0]=o*(0.641325174447)*x_ref[0]**(o-1)+(0.859809801095)
        ref[2,1,0,1]=o*(0.238917443521)*x_ref[1]**(o-1)+(0.169387212912)
        ref[2,1,0,2]=o*(0.118386087343)*x_ref[2]**(o-1)+(0.7091562331)
        ref[2,1,1,0]=o*(0.683767548268)*x_ref[0]**(o-1)+(-0.488504930782)
        ref[2,1,1,1]=o*(0.421423574907)*x_ref[1]**(o-1)+(0.677076626099)
        ref[2,1,1,2]=o*(-0.0955208633306)*x_ref[2]**(o-1)+(0.545750792826)
        ref[3,0,0,0]=o*(-0.925067175669)*x_ref[0]**(o-1)+(0.550204660551)
        ref[3,0,0,1]=o*(0.435686889479)*x_ref[1]**(o-1)+(-0.171725370784)
        ref[3,0,0,2]=o*(0.182097753276)*x_ref[2]**(o-1)+(0.493662375098)
        ref[3,0,1,0]=o*(-0.719237805296)*x_ref[0]**(o-1)+(0.669817079155)
        ref[3,0,1,1]=o*(0.660680613106)*x_ref[1]**(o-1)+(-0.729927483007)
        ref[3,0,1,2]=o*(-0.162704346009)*x_ref[2]**(o-1)+(-0.559206291015)
        ref[3,1,0,0]=o*(0.129735268583)*x_ref[0]**(o-1)+(-0.411853842222)
        ref[3,1,0,1]=o*(0.045438376206)*x_ref[1]**(o-1)+(0.327804746872)
        ref[3,1,0,2]=o*(-0.639433216582)*x_ref[2]**(o-1)+(0.451106136717)
        ref[3,1,1,0]=o*(-0.811557092293)*x_ref[0]**(o-1)+(-0.818758274715)
        ref[3,1,1,1]=o*(-0.712815291675)*x_ref[1]**(o-1)+(0.729038274951)
        ref[3,1,1,2]=o*(-0.826837009476)*x_ref[2]**(o-1)+(0.340025404977)
        ref[4,0,0,0]=o*(-0.507284842259)*x_ref[0]**(o-1)+(0.457554461786)
        ref[4,0,0,1]=o*(0.419345933038)*x_ref[1]**(o-1)+(-0.7773529718)
        ref[4,0,0,2]=o*(-0.626475804952)*x_ref[2]**(o-1)+(-0.784196320861)
        ref[4,0,1,0]=o*(-0.0340250322306)*x_ref[0]**(o-1)+(-0.0936499769437)
        ref[4,0,1,1]=o*(-0.777584776075)*x_ref[1]**(o-1)+(-0.435876632622)
        ref[4,0,1,2]=o*(0.999336374444)*x_ref[2]**(o-1)+(-0.247149214414)
        ref[4,1,0,0]=o*(0.21620967013)*x_ref[0]**(o-1)+(0.213039866961)
        ref[4,1,0,1]=o*(0.83049354428)*x_ref[1]**(o-1)+(0.651430926943)
        ref[4,1,0,2]=o*(0.201648695549)*x_ref[2]**(o-1)+(-0.609802970877)
        ref[4,1,1,0]=o*(0.842001670092)*x_ref[0]**(o-1)+(-0.868487783326)
        ref[4,1,1,1]=o*(-0.288512200338)*x_ref[1]**(o-1)+(-0.654831485218)
        ref[4,1,1,2]=o*(-0.111859725689)*x_ref[2]**(o-1)+(0.728195130453)
        ref[5,0,0,0]=o*(0.473856201092)*x_ref[0]**(o-1)+(0.377649949188)
        ref[5,0,0,1]=o*(0.511638340284)*x_ref[1]**(o-1)+(-0.150021940198)
        ref[5,0,0,2]=o*(-0.801655915671)*x_ref[2]**(o-1)+(-0.603617347175)
        ref[5,0,1,0]=o*(0.945961549464)*x_ref[0]**(o-1)+(0.40151064288)
        ref[5,0,1,1]=o*(0.151039821252)*x_ref[1]**(o-1)+(0.977148741848)
        ref[5,0,1,2]=o*(0.0729651283448)*x_ref[2]**(o-1)+(0.365625847237)
        ref[5,1,0,0]=o*(0.613037024504)*x_ref[0]**(o-1)+(-0.743080801009)
        ref[5,1,0,1]=o*(-0.518736037968)*x_ref[1]**(o-1)+(0.240273029617)
        ref[5,1,0,2]=o*(0.437464780338)*x_ref[2]**(o-1)+(0.897940669321)
        ref[5,1,1,0]=o*(-0.149245273038)*x_ref[0]**(o-1)+(-0.387037171901)
        ref[5,1,1,1]=o*(0.0606843165198)*x_ref[1]**(o-1)+(-0.713188961184)
        ref[5,1,1,2]=o*(0.622672840108)*x_ref[2]**(o-1)+(-0.579757019852)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(1.05116031237)*x[0]+(-0.156242880188)*x[1]
        ref[0]=(1.05116031237)
        ref[1]=(-0.156242880188)
      else:
        s=(-0.978032091964)*x[0]+(0.994143533164)*x[1]+(1.36141912157)*x[2]
        ref[0]=(-0.978032091964)
        ref[1]=(0.994143533164)
        ref[2]=(1.36141912157)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.908181468063)*x[0]+(0.383654148719)*x[1]
        s[1]=(0.996727924056)*x[0]+(1.13801031707)*x[1]
        ref[0,0]=(0.908181468063)
        ref[0,1]=(0.383654148719)
        ref[1,0]=(0.996727924056)
        ref[1,1]=(1.13801031707)
      else:
        s[0]=(-0.431683467168)*x[0]+(-0.610893064868)*x[1]+(-0.39989804071)*x[2]
        s[1]=(0.820400856313)*x[0]+(0.790210621031)*x[1]+(-0.0581477423106)*x[2]
        ref[0,0]=(-0.431683467168)
        ref[0,1]=(-0.610893064868)
        ref[0,2]=(-0.39989804071)
        ref[1,0]=(0.820400856313)
        ref[1,1]=(0.790210621031)
        ref[1,2]=(-0.0581477423106)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.202521929549)*x[0]+(1.24388265835)*x[1]
        s[0,1]=(0.466921175909)*x[0]+(0.0938124013357)*x[1]
        s[0,2]=(-0.337565182967)*x[0]+(-0.66213972177)*x[1]
        s[0,3]=(-1.1265817109)*x[0]+(1.45099893506)*x[1]
        s[0,4]=(1.85945279233)*x[0]+(0.614232583853)*x[1]
        s[1,0]=(0.773998701601)*x[0]+(1.3572752273)*x[1]
        s[1,1]=(1.503889482)*x[0]+(-1.22986193243)*x[1]
        s[1,2]=(1.17229177208)*x[0]+(-1.005468476)*x[1]
        s[1,3]=(-1.27864883602)*x[0]+(0.27430357882)*x[1]
        s[1,4]=(-0.0170497525828)*x[0]+(-1.07936325294)*x[1]
        s[2,0]=(-0.172836979766)*x[0]+(0.481503531609)*x[1]
        s[2,1]=(0.240407365629)*x[0]+(-1.2996101353)*x[1]
        s[2,2]=(-1.35821876592)*x[0]+(1.06176151561)*x[1]
        s[2,3]=(-1.93289948536)*x[0]+(1.7647044408)*x[1]
        s[2,4]=(-1.16491431281)*x[0]+(0.483870146591)*x[1]
        s[3,0]=(0.646925654514)*x[0]+(-0.079632957461)*x[1]
        s[3,1]=(0.93955736882)*x[0]+(-0.314382763761)*x[1]
        s[3,2]=(0.903199444521)*x[0]+(-0.675377265588)*x[1]
        s[3,3]=(-0.373383381553)*x[0]+(0.140227745605)*x[1]
        s[3,4]=(0.199285967708)*x[0]+(0.440148792869)*x[1]
        ref[0,0,0]=(0.202521929549)
        ref[0,0,1]=(1.24388265835)
        ref[0,1,0]=(0.466921175909)
        ref[0,1,1]=(0.0938124013357)
        ref[0,2,0]=(-0.337565182967)
        ref[0,2,1]=(-0.66213972177)
        ref[0,3,0]=(-1.1265817109)
        ref[0,3,1]=(1.45099893506)
        ref[0,4,0]=(1.85945279233)
        ref[0,4,1]=(0.614232583853)
        ref[1,0,0]=(0.773998701601)
        ref[1,0,1]=(1.3572752273)
        ref[1,1,0]=(1.503889482)
        ref[1,1,1]=(-1.22986193243)
        ref[1,2,0]=(1.17229177208)
        ref[1,2,1]=(-1.005468476)
        ref[1,3,0]=(-1.27864883602)
        ref[1,3,1]=(0.27430357882)
        ref[1,4,0]=(-0.0170497525828)
        ref[1,4,1]=(-1.07936325294)
        ref[2,0,0]=(-0.172836979766)
        ref[2,0,1]=(0.481503531609)
        ref[2,1,0]=(0.240407365629)
        ref[2,1,1]=(-1.2996101353)
        ref[2,2,0]=(-1.35821876592)
        ref[2,2,1]=(1.06176151561)
        ref[2,3,0]=(-1.93289948536)
        ref[2,3,1]=(1.7647044408)
        ref[2,4,0]=(-1.16491431281)
        ref[2,4,1]=(0.483870146591)
        ref[3,0,0]=(0.646925654514)
        ref[3,0,1]=(-0.079632957461)
        ref[3,1,0]=(0.93955736882)
        ref[3,1,1]=(-0.314382763761)
        ref[3,2,0]=(0.903199444521)
        ref[3,2,1]=(-0.675377265588)
        ref[3,3,0]=(-0.373383381553)
        ref[3,3,1]=(0.140227745605)
        ref[3,4,0]=(0.199285967708)
        ref[3,4,1]=(0.440148792869)
      else:
        s[0,0]=(-0.454164125322)*x[0]+(-0.522669711039)*x[1]+(-1.69156961037)*x[2]
        s[0,1]=(1.02678975575)*x[0]+(-0.627069283778)*x[1]+(0.755190616163)*x[2]
        s[0,2]=(0.689068203494)*x[0]+(-0.677393706098)*x[1]+(-0.822365020628)*x[2]
        s[0,3]=(0.488491352905)*x[0]+(1.19039296387)*x[1]+(0.293029595575)*x[2]
        s[0,4]=(1.3254184285)*x[0]+(-0.67206171589)*x[1]+(1.95627951713)*x[2]
        s[1,0]=(-1.01024336678)*x[0]+(0.553509575201)*x[1]+(0.206480917751)*x[2]
        s[1,1]=(-0.679488104614)*x[0]+(0.561501215346)*x[1]+(-0.157738070792)*x[2]
        s[1,2]=(1.09058998896)*x[0]+(0.358786992535)*x[1]+(-0.866813461522)*x[2]
        s[1,3]=(0.945307210944)*x[0]+(-0.592121399828)*x[1]+(-1.23798843507)*x[2]
        s[1,4]=(-0.334377129623)*x[0]+(0.282409196274)*x[1]+(1.11154300742)*x[2]
        s[2,0]=(0.31913061339)*x[0]+(1.47252246089)*x[1]+(-0.0516509049482)*x[2]
        s[2,1]=(-0.889919147968)*x[0]+(0.750571626982)*x[1]+(1.44049343766)*x[2]
        s[2,2]=(-0.648407056167)*x[0]+(-0.961481146656)*x[1]+(0.747319150361)*x[2]
        s[2,3]=(1.49042928912)*x[0]+(1.31212422706)*x[1]+(0.510609313139)*x[2]
        s[2,4]=(-0.812306629075)*x[0]+(1.27041274415)*x[1]+(0.740119854826)*x[2]
        s[3,0]=(-1.11671812457)*x[0]+(0.382559318776)*x[1]+(-0.12879810898)*x[2]
        s[3,1]=(0.253402604664)*x[0]+(0.433299751265)*x[1]+(-0.282468707216)*x[2]
        s[3,2]=(1.32928877441)*x[0]+(-1.62629303652)*x[1]+(1.09372049629)*x[2]
        s[3,3]=(-1.20558999975)*x[0]+(-0.25798278657)*x[1]+(-0.0869196687657)*x[2]
        s[3,4]=(-1.31791924188)*x[0]+(0.0386230359027)*x[1]+(0.00644144564451)*x[2]
        ref[0,0,0]=(-0.454164125322)
        ref[0,0,1]=(-0.522669711039)
        ref[0,0,2]=(-1.69156961037)
        ref[0,1,0]=(1.02678975575)
        ref[0,1,1]=(-0.627069283778)
        ref[0,1,2]=(0.755190616163)
        ref[0,2,0]=(0.689068203494)
        ref[0,2,1]=(-0.677393706098)
        ref[0,2,2]=(-0.822365020628)
        ref[0,3,0]=(0.488491352905)
        ref[0,3,1]=(1.19039296387)
        ref[0,3,2]=(0.293029595575)
        ref[0,4,0]=(1.3254184285)
        ref[0,4,1]=(-0.67206171589)
        ref[0,4,2]=(1.95627951713)
        ref[1,0,0]=(-1.01024336678)
        ref[1,0,1]=(0.553509575201)
        ref[1,0,2]=(0.206480917751)
        ref[1,1,0]=(-0.679488104614)
        ref[1,1,1]=(0.561501215346)
        ref[1,1,2]=(-0.157738070792)
        ref[1,2,0]=(1.09058998896)
        ref[1,2,1]=(0.358786992535)
        ref[1,2,2]=(-0.866813461522)
        ref[1,3,0]=(0.945307210944)
        ref[1,3,1]=(-0.592121399828)
        ref[1,3,2]=(-1.23798843507)
        ref[1,4,0]=(-0.334377129623)
        ref[1,4,1]=(0.282409196274)
        ref[1,4,2]=(1.11154300742)
        ref[2,0,0]=(0.31913061339)
        ref[2,0,1]=(1.47252246089)
        ref[2,0,2]=(-0.0516509049482)
        ref[2,1,0]=(-0.889919147968)
        ref[2,1,1]=(0.750571626982)
        ref[2,1,2]=(1.44049343766)
        ref[2,2,0]=(-0.648407056167)
        ref[2,2,1]=(-0.961481146656)
        ref[2,2,2]=(0.747319150361)
        ref[2,3,0]=(1.49042928912)
        ref[2,3,1]=(1.31212422706)
        ref[2,3,2]=(0.510609313139)
        ref[2,4,0]=(-0.812306629075)
        ref[2,4,1]=(1.27041274415)
        ref[2,4,2]=(0.740119854826)
        ref[3,0,0]=(-1.11671812457)
        ref[3,0,1]=(0.382559318776)
        ref[3,0,2]=(-0.12879810898)
        ref[3,1,0]=(0.253402604664)
        ref[3,1,1]=(0.433299751265)
        ref[3,1,2]=(-0.282468707216)
        ref[3,2,0]=(1.32928877441)
        ref[3,2,1]=(-1.62629303652)
        ref[3,2,2]=(1.09372049629)
        ref[3,3,0]=(-1.20558999975)
        ref[3,3,1]=(-0.25798278657)
        ref[3,3,2]=(-0.0869196687657)
        ref[3,4,0]=(-1.31791924188)
        ref[3,4,1]=(0.0386230359027)
        ref[3,4,2]=(0.00644144564451)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.500249599152)*x[0]+(1.67754131181)*x[1]
        s[0,0,1]=(1.1446224976)*x[0]+(0.293592946461)*x[1]
        s[0,1,0]=(0.662825578178)*x[0]+(0.778202393248)*x[1]
        s[0,1,1]=(1.120980772)*x[0]+(0.95813878487)*x[1]
        s[1,0,0]=(-0.462148842008)*x[0]+(-1.63817011088)*x[1]
        s[1,0,1]=(1.49522487924)*x[0]+(1.17105843575)*x[1]
        s[1,1,0]=(0.0273404170297)*x[0]+(-0.60627667625)*x[1]
        s[1,1,1]=(0.328333508501)*x[0]+(-0.487743429505)*x[1]
        s[2,0,0]=(0.690757853756)*x[0]+(0.415349096316)*x[1]
        s[2,0,1]=(-1.76899408416)*x[0]+(0.338928282096)*x[1]
        s[2,1,0]=(-1.65374086962)*x[0]+(0.666255100879)*x[1]
        s[2,1,1]=(0.845913196898)*x[0]+(0.524414663858)*x[1]
        s[3,0,0]=(-0.0489138005056)*x[0]+(-0.306296688032)*x[1]
        s[3,0,1]=(-0.619051141066)*x[0]+(0.399377559544)*x[1]
        s[3,1,0]=(0.407991784518)*x[0]+(-1.14997392594)*x[1]
        s[3,1,1]=(-0.115512243924)*x[0]+(0.296839190724)*x[1]
        s[4,0,0]=(1.45399807861)*x[0]+(-0.807761883642)*x[1]
        s[4,0,1]=(0.639797195198)*x[0]+(1.35664991322)*x[1]
        s[4,1,0]=(-0.884543551657)*x[0]+(0.702070741415)*x[1]
        s[4,1,1]=(-0.280174966716)*x[0]+(0.0172627474218)*x[1]
        s[5,0,0]=(0.235198909788)*x[0]+(0.0707623144647)*x[1]
        s[5,0,1]=(0.670946179418)*x[0]+(0.746509554365)*x[1]
        s[5,1,0]=(1.20081036792)*x[0]+(0.158928139216)*x[1]
        s[5,1,1]=(1.06402047834)*x[0]+(-0.567749347509)*x[1]
        ref[0,0,0,0]=(-0.500249599152)
        ref[0,0,0,1]=(1.67754131181)
        ref[0,0,1,0]=(1.1446224976)
        ref[0,0,1,1]=(0.293592946461)
        ref[0,1,0,0]=(0.662825578178)
        ref[0,1,0,1]=(0.778202393248)
        ref[0,1,1,0]=(1.120980772)
        ref[0,1,1,1]=(0.95813878487)
        ref[1,0,0,0]=(-0.462148842008)
        ref[1,0,0,1]=(-1.63817011088)
        ref[1,0,1,0]=(1.49522487924)
        ref[1,0,1,1]=(1.17105843575)
        ref[1,1,0,0]=(0.0273404170297)
        ref[1,1,0,1]=(-0.60627667625)
        ref[1,1,1,0]=(0.328333508501)
        ref[1,1,1,1]=(-0.487743429505)
        ref[2,0,0,0]=(0.690757853756)
        ref[2,0,0,1]=(0.415349096316)
        ref[2,0,1,0]=(-1.76899408416)
        ref[2,0,1,1]=(0.338928282096)
        ref[2,1,0,0]=(-1.65374086962)
        ref[2,1,0,1]=(0.666255100879)
        ref[2,1,1,0]=(0.845913196898)
        ref[2,1,1,1]=(0.524414663858)
        ref[3,0,0,0]=(-0.0489138005056)
        ref[3,0,0,1]=(-0.306296688032)
        ref[3,0,1,0]=(-0.619051141066)
        ref[3,0,1,1]=(0.399377559544)
        ref[3,1,0,0]=(0.407991784518)
        ref[3,1,0,1]=(-1.14997392594)
        ref[3,1,1,0]=(-0.115512243924)
        ref[3,1,1,1]=(0.296839190724)
        ref[4,0,0,0]=(1.45399807861)
        ref[4,0,0,1]=(-0.807761883642)
        ref[4,0,1,0]=(0.639797195198)
        ref[4,0,1,1]=(1.35664991322)
        ref[4,1,0,0]=(-0.884543551657)
        ref[4,1,0,1]=(0.702070741415)
        ref[4,1,1,0]=(-0.280174966716)
        ref[4,1,1,1]=(0.0172627474218)
        ref[5,0,0,0]=(0.235198909788)
        ref[5,0,0,1]=(0.0707623144647)
        ref[5,0,1,0]=(0.670946179418)
        ref[5,0,1,1]=(0.746509554365)
        ref[5,1,0,0]=(1.20081036792)
        ref[5,1,0,1]=(0.158928139216)
        ref[5,1,1,0]=(1.06402047834)
        ref[5,1,1,1]=(-0.567749347509)
      else:
        s[0,0,0]=(-0.709307825835)*x[0]+(-0.212632569057)*x[1]+(-1.36262175379)*x[2]
        s[0,0,1]=(1.26136861771)*x[0]+(-0.247247821335)*x[1]+(0.952472163507)*x[2]
        s[0,1,0]=(-0.313562941587)*x[0]+(0.335230615547)*x[1]+(0.633669942964)*x[2]
        s[0,1,1]=(0.543435495949)*x[0]+(-0.278928766765)*x[1]+(-0.560857410629)*x[2]
        s[1,0,0]=(-0.228999374753)*x[0]+(0.627231691819)*x[1]+(-0.110282807377)*x[2]
        s[1,0,1]=(-0.183700669251)*x[0]+(0.361641358769)*x[1]+(-0.565719344787)*x[2]
        s[1,1,0]=(0.0392225368869)*x[0]+(0.530481764326)*x[1]+(-1.16647413286)*x[2]
        s[1,1,1]=(1.10307486886)*x[0]+(1.62013473483)*x[1]+(1.08663406043)*x[2]
        s[2,0,0]=(0.418269374961)*x[0]+(-0.881366670214)*x[1]+(-0.19081724529)*x[2]
        s[2,0,1]=(-0.888542773287)*x[0]+(0.973322751168)*x[1]+(-0.678806455499)*x[2]
        s[2,1,0]=(0.203680986809)*x[0]+(-0.64202022526)*x[1]+(-0.342367020762)*x[2]
        s[2,1,1]=(-0.984982113825)*x[0]+(1.52327025412)*x[1]+(-1.16664161507)*x[2]
        s[3,0,0]=(-0.430857332595)*x[0]+(1.37801740474)*x[1]+(1.47536591267)*x[2]
        s[3,0,1]=(-1.49872232697)*x[0]+(0.679321175644)*x[1]+(0.233255377135)*x[2]
        s[3,1,0]=(0.00673347423284)*x[0]+(-0.659736891512)*x[1]+(-1.24483031223)*x[2]
        s[3,1,1]=(-1.35939956697)*x[0]+(-1.05370704987)*x[1]+(0.974871664002)*x[2]
        s[4,0,0]=(-1.56401623595)*x[0]+(0.0396404235057)*x[1]+(0.529626464189)*x[2]
        s[4,0,1]=(-0.903524096133)*x[0]+(-0.0504449605469)*x[1]+(-0.38093760213)*x[2]
        s[4,1,0]=(-0.570517363798)*x[0]+(1.50697108327)*x[1]+(0.45722182765)*x[2]
        s[4,1,1]=(0.855238285486)*x[0]+(-1.19608277327)*x[1]+(-0.454258653189)*x[2]
        s[5,0,0]=(0.264880357574)*x[0]+(-0.137240568006)*x[1]+(-1.43157116771)*x[2]
        s[5,0,1]=(1.42099564204)*x[0]+(-0.584391783811)*x[1]+(-0.194445105527)*x[2]
        s[5,1,0]=(-1.41719313241)*x[0]+(1.18436247042)*x[1]+(0.774373387123)*x[2]
        s[5,1,1]=(0.894745586845)*x[0]+(0.272141343375)*x[1]+(-0.563792436747)*x[2]
        ref[0,0,0,0]=(-0.709307825835)
        ref[0,0,0,1]=(-0.212632569057)
        ref[0,0,0,2]=(-1.36262175379)
        ref[0,0,1,0]=(1.26136861771)
        ref[0,0,1,1]=(-0.247247821335)
        ref[0,0,1,2]=(0.952472163507)
        ref[0,1,0,0]=(-0.313562941587)
        ref[0,1,0,1]=(0.335230615547)
        ref[0,1,0,2]=(0.633669942964)
        ref[0,1,1,0]=(0.543435495949)
        ref[0,1,1,1]=(-0.278928766765)
        ref[0,1,1,2]=(-0.560857410629)
        ref[1,0,0,0]=(-0.228999374753)
        ref[1,0,0,1]=(0.627231691819)
        ref[1,0,0,2]=(-0.110282807377)
        ref[1,0,1,0]=(-0.183700669251)
        ref[1,0,1,1]=(0.361641358769)
        ref[1,0,1,2]=(-0.565719344787)
        ref[1,1,0,0]=(0.0392225368869)
        ref[1,1,0,1]=(0.530481764326)
        ref[1,1,0,2]=(-1.16647413286)
        ref[1,1,1,0]=(1.10307486886)
        ref[1,1,1,1]=(1.62013473483)
        ref[1,1,1,2]=(1.08663406043)
        ref[2,0,0,0]=(0.418269374961)
        ref[2,0,0,1]=(-0.881366670214)
        ref[2,0,0,2]=(-0.19081724529)
        ref[2,0,1,0]=(-0.888542773287)
        ref[2,0,1,1]=(0.973322751168)
        ref[2,0,1,2]=(-0.678806455499)
        ref[2,1,0,0]=(0.203680986809)
        ref[2,1,0,1]=(-0.64202022526)
        ref[2,1,0,2]=(-0.342367020762)
        ref[2,1,1,0]=(-0.984982113825)
        ref[2,1,1,1]=(1.52327025412)
        ref[2,1,1,2]=(-1.16664161507)
        ref[3,0,0,0]=(-0.430857332595)
        ref[3,0,0,1]=(1.37801740474)
        ref[3,0,0,2]=(1.47536591267)
        ref[3,0,1,0]=(-1.49872232697)
        ref[3,0,1,1]=(0.679321175644)
        ref[3,0,1,2]=(0.233255377135)
        ref[3,1,0,0]=(0.00673347423284)
        ref[3,1,0,1]=(-0.659736891512)
        ref[3,1,0,2]=(-1.24483031223)
        ref[3,1,1,0]=(-1.35939956697)
        ref[3,1,1,1]=(-1.05370704987)
        ref[3,1,1,2]=(0.974871664002)
        ref[4,0,0,0]=(-1.56401623595)
        ref[4,0,0,1]=(0.0396404235057)
        ref[4,0,0,2]=(0.529626464189)
        ref[4,0,1,0]=(-0.903524096133)
        ref[4,0,1,1]=(-0.0504449605469)
        ref[4,0,1,2]=(-0.38093760213)
        ref[4,1,0,0]=(-0.570517363798)
        ref[4,1,0,1]=(1.50697108327)
        ref[4,1,0,2]=(0.45722182765)
        ref[4,1,1,0]=(0.855238285486)
        ref[4,1,1,1]=(-1.19608277327)
        ref[4,1,1,2]=(-0.454258653189)
        ref[5,0,0,0]=(0.264880357574)
        ref[5,0,0,1]=(-0.137240568006)
        ref[5,0,0,2]=(-1.43157116771)
        ref[5,0,1,0]=(1.42099564204)
        ref[5,0,1,1]=(-0.584391783811)
        ref[5,0,1,2]=(-0.194445105527)
        ref[5,1,0,0]=(-1.41719313241)
        ref[5,1,0,1]=(1.18436247042)
        ref[5,1,0,2]=(0.774373387123)
        ref[5,1,1,0]=(0.894745586845)
        ref[5,1,1,1]=(0.272141343375)
        ref[5,1,1,2]=(-0.563792436747)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank0(self):
      """
      tests gradient for rank 0 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(),dim=dim)
      s=Data(0,(),w)
      ref=Data(0,()+(dim,),w_ref)
      if dim==2:
        s=(1.05116031237)*x[0]+(-0.156242880188)*x[1]
        ref[0]=(1.05116031237)
        ref[1]=(-0.156242880188)
      else:
        s=(-0.978032091964)*x[0]+(0.994143533164)*x[1]+(1.36141912157)*x[2]
        ref[0]=(-0.978032091964)
        ref[1]=(0.994143533164)
        ref[2]=(1.36141912157)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),()+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank1(self):
      """
      tests gradient for rank 1 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(2,),dim=dim)
      s=Data(0,(2,),w)
      ref=Data(0,(2,)+(dim,),w_ref)
      if dim==2:
        s[0]=(0.908181468063)*x[0]+(0.383654148719)*x[1]
        s[1]=(0.996727924056)*x[0]+(1.13801031707)*x[1]
        ref[0,0]=(0.908181468063)
        ref[0,1]=(0.383654148719)
        ref[1,0]=(0.996727924056)
        ref[1,1]=(1.13801031707)
      else:
        s[0]=(-0.431683467168)*x[0]+(-0.610893064868)*x[1]+(-0.39989804071)*x[2]
        s[1]=(0.820400856313)*x[0]+(0.790210621031)*x[1]+(-0.0581477423106)*x[2]
        ref[0,0]=(-0.431683467168)
        ref[0,1]=(-0.610893064868)
        ref[0,2]=(-0.39989804071)
        ref[1,0]=(0.820400856313)
        ref[1,1]=(0.790210621031)
        ref[1,2]=(-0.0581477423106)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank2(self):
      """
      tests gradient for rank 2 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(4, 5),dim=dim)
      s=Data(0,(4, 5),w)
      ref=Data(0,(4, 5)+(dim,),w_ref)
      if dim==2:
        s[0,0]=(0.202521929549)*x[0]+(1.24388265835)*x[1]
        s[0,1]=(0.466921175909)*x[0]+(0.0938124013357)*x[1]
        s[0,2]=(-0.337565182967)*x[0]+(-0.66213972177)*x[1]
        s[0,3]=(-1.1265817109)*x[0]+(1.45099893506)*x[1]
        s[0,4]=(1.85945279233)*x[0]+(0.614232583853)*x[1]
        s[1,0]=(0.773998701601)*x[0]+(1.3572752273)*x[1]
        s[1,1]=(1.503889482)*x[0]+(-1.22986193243)*x[1]
        s[1,2]=(1.17229177208)*x[0]+(-1.005468476)*x[1]
        s[1,3]=(-1.27864883602)*x[0]+(0.27430357882)*x[1]
        s[1,4]=(-0.0170497525828)*x[0]+(-1.07936325294)*x[1]
        s[2,0]=(-0.172836979766)*x[0]+(0.481503531609)*x[1]
        s[2,1]=(0.240407365629)*x[0]+(-1.2996101353)*x[1]
        s[2,2]=(-1.35821876592)*x[0]+(1.06176151561)*x[1]
        s[2,3]=(-1.93289948536)*x[0]+(1.7647044408)*x[1]
        s[2,4]=(-1.16491431281)*x[0]+(0.483870146591)*x[1]
        s[3,0]=(0.646925654514)*x[0]+(-0.079632957461)*x[1]
        s[3,1]=(0.93955736882)*x[0]+(-0.314382763761)*x[1]
        s[3,2]=(0.903199444521)*x[0]+(-0.675377265588)*x[1]
        s[3,3]=(-0.373383381553)*x[0]+(0.140227745605)*x[1]
        s[3,4]=(0.199285967708)*x[0]+(0.440148792869)*x[1]
        ref[0,0,0]=(0.202521929549)
        ref[0,0,1]=(1.24388265835)
        ref[0,1,0]=(0.466921175909)
        ref[0,1,1]=(0.0938124013357)
        ref[0,2,0]=(-0.337565182967)
        ref[0,2,1]=(-0.66213972177)
        ref[0,3,0]=(-1.1265817109)
        ref[0,3,1]=(1.45099893506)
        ref[0,4,0]=(1.85945279233)
        ref[0,4,1]=(0.614232583853)
        ref[1,0,0]=(0.773998701601)
        ref[1,0,1]=(1.3572752273)
        ref[1,1,0]=(1.503889482)
        ref[1,1,1]=(-1.22986193243)
        ref[1,2,0]=(1.17229177208)
        ref[1,2,1]=(-1.005468476)
        ref[1,3,0]=(-1.27864883602)
        ref[1,3,1]=(0.27430357882)
        ref[1,4,0]=(-0.0170497525828)
        ref[1,4,1]=(-1.07936325294)
        ref[2,0,0]=(-0.172836979766)
        ref[2,0,1]=(0.481503531609)
        ref[2,1,0]=(0.240407365629)
        ref[2,1,1]=(-1.2996101353)
        ref[2,2,0]=(-1.35821876592)
        ref[2,2,1]=(1.06176151561)
        ref[2,3,0]=(-1.93289948536)
        ref[2,3,1]=(1.7647044408)
        ref[2,4,0]=(-1.16491431281)
        ref[2,4,1]=(0.483870146591)
        ref[3,0,0]=(0.646925654514)
        ref[3,0,1]=(-0.079632957461)
        ref[3,1,0]=(0.93955736882)
        ref[3,1,1]=(-0.314382763761)
        ref[3,2,0]=(0.903199444521)
        ref[3,2,1]=(-0.675377265588)
        ref[3,3,0]=(-0.373383381553)
        ref[3,3,1]=(0.140227745605)
        ref[3,4,0]=(0.199285967708)
        ref[3,4,1]=(0.440148792869)
      else:
        s[0,0]=(-0.454164125322)*x[0]+(-0.522669711039)*x[1]+(-1.69156961037)*x[2]
        s[0,1]=(1.02678975575)*x[0]+(-0.627069283778)*x[1]+(0.755190616163)*x[2]
        s[0,2]=(0.689068203494)*x[0]+(-0.677393706098)*x[1]+(-0.822365020628)*x[2]
        s[0,3]=(0.488491352905)*x[0]+(1.19039296387)*x[1]+(0.293029595575)*x[2]
        s[0,4]=(1.3254184285)*x[0]+(-0.67206171589)*x[1]+(1.95627951713)*x[2]
        s[1,0]=(-1.01024336678)*x[0]+(0.553509575201)*x[1]+(0.206480917751)*x[2]
        s[1,1]=(-0.679488104614)*x[0]+(0.561501215346)*x[1]+(-0.157738070792)*x[2]
        s[1,2]=(1.09058998896)*x[0]+(0.358786992535)*x[1]+(-0.866813461522)*x[2]
        s[1,3]=(0.945307210944)*x[0]+(-0.592121399828)*x[1]+(-1.23798843507)*x[2]
        s[1,4]=(-0.334377129623)*x[0]+(0.282409196274)*x[1]+(1.11154300742)*x[2]
        s[2,0]=(0.31913061339)*x[0]+(1.47252246089)*x[1]+(-0.0516509049482)*x[2]
        s[2,1]=(-0.889919147968)*x[0]+(0.750571626982)*x[1]+(1.44049343766)*x[2]
        s[2,2]=(-0.648407056167)*x[0]+(-0.961481146656)*x[1]+(0.747319150361)*x[2]
        s[2,3]=(1.49042928912)*x[0]+(1.31212422706)*x[1]+(0.510609313139)*x[2]
        s[2,4]=(-0.812306629075)*x[0]+(1.27041274415)*x[1]+(0.740119854826)*x[2]
        s[3,0]=(-1.11671812457)*x[0]+(0.382559318776)*x[1]+(-0.12879810898)*x[2]
        s[3,1]=(0.253402604664)*x[0]+(0.433299751265)*x[1]+(-0.282468707216)*x[2]
        s[3,2]=(1.32928877441)*x[0]+(-1.62629303652)*x[1]+(1.09372049629)*x[2]
        s[3,3]=(-1.20558999975)*x[0]+(-0.25798278657)*x[1]+(-0.0869196687657)*x[2]
        s[3,4]=(-1.31791924188)*x[0]+(0.0386230359027)*x[1]+(0.00644144564451)*x[2]
        ref[0,0,0]=(-0.454164125322)
        ref[0,0,1]=(-0.522669711039)
        ref[0,0,2]=(-1.69156961037)
        ref[0,1,0]=(1.02678975575)
        ref[0,1,1]=(-0.627069283778)
        ref[0,1,2]=(0.755190616163)
        ref[0,2,0]=(0.689068203494)
        ref[0,2,1]=(-0.677393706098)
        ref[0,2,2]=(-0.822365020628)
        ref[0,3,0]=(0.488491352905)
        ref[0,3,1]=(1.19039296387)
        ref[0,3,2]=(0.293029595575)
        ref[0,4,0]=(1.3254184285)
        ref[0,4,1]=(-0.67206171589)
        ref[0,4,2]=(1.95627951713)
        ref[1,0,0]=(-1.01024336678)
        ref[1,0,1]=(0.553509575201)
        ref[1,0,2]=(0.206480917751)
        ref[1,1,0]=(-0.679488104614)
        ref[1,1,1]=(0.561501215346)
        ref[1,1,2]=(-0.157738070792)
        ref[1,2,0]=(1.09058998896)
        ref[1,2,1]=(0.358786992535)
        ref[1,2,2]=(-0.866813461522)
        ref[1,3,0]=(0.945307210944)
        ref[1,3,1]=(-0.592121399828)
        ref[1,3,2]=(-1.23798843507)
        ref[1,4,0]=(-0.334377129623)
        ref[1,4,1]=(0.282409196274)
        ref[1,4,2]=(1.11154300742)
        ref[2,0,0]=(0.31913061339)
        ref[2,0,1]=(1.47252246089)
        ref[2,0,2]=(-0.0516509049482)
        ref[2,1,0]=(-0.889919147968)
        ref[2,1,1]=(0.750571626982)
        ref[2,1,2]=(1.44049343766)
        ref[2,2,0]=(-0.648407056167)
        ref[2,2,1]=(-0.961481146656)
        ref[2,2,2]=(0.747319150361)
        ref[2,3,0]=(1.49042928912)
        ref[2,3,1]=(1.31212422706)
        ref[2,3,2]=(0.510609313139)
        ref[2,4,0]=(-0.812306629075)
        ref[2,4,1]=(1.27041274415)
        ref[2,4,2]=(0.740119854826)
        ref[3,0,0]=(-1.11671812457)
        ref[3,0,1]=(0.382559318776)
        ref[3,0,2]=(-0.12879810898)
        ref[3,1,0]=(0.253402604664)
        ref[3,1,1]=(0.433299751265)
        ref[3,1,2]=(-0.282468707216)
        ref[3,2,0]=(1.32928877441)
        ref[3,2,1]=(-1.62629303652)
        ref[3,2,2]=(1.09372049629)
        ref[3,3,0]=(-1.20558999975)
        ref[3,3,1]=(-0.25798278657)
        ref[3,3,2]=(-0.0869196687657)
        ref[3,4,0]=(-1.31791924188)
        ref[3,4,1]=(0.0386230359027)
        ref[3,4,2]=(0.00644144564451)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_grad_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction_rank3(self):
      """
      tests gradient for rank 3 Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports gardient on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(6, 2, 2),dim=dim)
      s=Data(0,(6, 2, 2),w)
      ref=Data(0,(6, 2, 2)+(dim,),w_ref)
      if dim==2:
        s[0,0,0]=(-0.500249599152)*x[0]+(1.67754131181)*x[1]
        s[0,0,1]=(1.1446224976)*x[0]+(0.293592946461)*x[1]
        s[0,1,0]=(0.662825578178)*x[0]+(0.778202393248)*x[1]
        s[0,1,1]=(1.120980772)*x[0]+(0.95813878487)*x[1]
        s[1,0,0]=(-0.462148842008)*x[0]+(-1.63817011088)*x[1]
        s[1,0,1]=(1.49522487924)*x[0]+(1.17105843575)*x[1]
        s[1,1,0]=(0.0273404170297)*x[0]+(-0.60627667625)*x[1]
        s[1,1,1]=(0.328333508501)*x[0]+(-0.487743429505)*x[1]
        s[2,0,0]=(0.690757853756)*x[0]+(0.415349096316)*x[1]
        s[2,0,1]=(-1.76899408416)*x[0]+(0.338928282096)*x[1]
        s[2,1,0]=(-1.65374086962)*x[0]+(0.666255100879)*x[1]
        s[2,1,1]=(0.845913196898)*x[0]+(0.524414663858)*x[1]
        s[3,0,0]=(-0.0489138005056)*x[0]+(-0.306296688032)*x[1]
        s[3,0,1]=(-0.619051141066)*x[0]+(0.399377559544)*x[1]
        s[3,1,0]=(0.407991784518)*x[0]+(-1.14997392594)*x[1]
        s[3,1,1]=(-0.115512243924)*x[0]+(0.296839190724)*x[1]
        s[4,0,0]=(1.45399807861)*x[0]+(-0.807761883642)*x[1]
        s[4,0,1]=(0.639797195198)*x[0]+(1.35664991322)*x[1]
        s[4,1,0]=(-0.884543551657)*x[0]+(0.702070741415)*x[1]
        s[4,1,1]=(-0.280174966716)*x[0]+(0.0172627474218)*x[1]
        s[5,0,0]=(0.235198909788)*x[0]+(0.0707623144647)*x[1]
        s[5,0,1]=(0.670946179418)*x[0]+(0.746509554365)*x[1]
        s[5,1,0]=(1.20081036792)*x[0]+(0.158928139216)*x[1]
        s[5,1,1]=(1.06402047834)*x[0]+(-0.567749347509)*x[1]
        ref[0,0,0,0]=(-0.500249599152)
        ref[0,0,0,1]=(1.67754131181)
        ref[0,0,1,0]=(1.1446224976)
        ref[0,0,1,1]=(0.293592946461)
        ref[0,1,0,0]=(0.662825578178)
        ref[0,1,0,1]=(0.778202393248)
        ref[0,1,1,0]=(1.120980772)
        ref[0,1,1,1]=(0.95813878487)
        ref[1,0,0,0]=(-0.462148842008)
        ref[1,0,0,1]=(-1.63817011088)
        ref[1,0,1,0]=(1.49522487924)
        ref[1,0,1,1]=(1.17105843575)
        ref[1,1,0,0]=(0.0273404170297)
        ref[1,1,0,1]=(-0.60627667625)
        ref[1,1,1,0]=(0.328333508501)
        ref[1,1,1,1]=(-0.487743429505)
        ref[2,0,0,0]=(0.690757853756)
        ref[2,0,0,1]=(0.415349096316)
        ref[2,0,1,0]=(-1.76899408416)
        ref[2,0,1,1]=(0.338928282096)
        ref[2,1,0,0]=(-1.65374086962)
        ref[2,1,0,1]=(0.666255100879)
        ref[2,1,1,0]=(0.845913196898)
        ref[2,1,1,1]=(0.524414663858)
        ref[3,0,0,0]=(-0.0489138005056)
        ref[3,0,0,1]=(-0.306296688032)
        ref[3,0,1,0]=(-0.619051141066)
        ref[3,0,1,1]=(0.399377559544)
        ref[3,1,0,0]=(0.407991784518)
        ref[3,1,0,1]=(-1.14997392594)
        ref[3,1,1,0]=(-0.115512243924)
        ref[3,1,1,1]=(0.296839190724)
        ref[4,0,0,0]=(1.45399807861)
        ref[4,0,0,1]=(-0.807761883642)
        ref[4,0,1,0]=(0.639797195198)
        ref[4,0,1,1]=(1.35664991322)
        ref[4,1,0,0]=(-0.884543551657)
        ref[4,1,0,1]=(0.702070741415)
        ref[4,1,1,0]=(-0.280174966716)
        ref[4,1,1,1]=(0.0172627474218)
        ref[5,0,0,0]=(0.235198909788)
        ref[5,0,0,1]=(0.0707623144647)
        ref[5,0,1,0]=(0.670946179418)
        ref[5,0,1,1]=(0.746509554365)
        ref[5,1,0,0]=(1.20081036792)
        ref[5,1,0,1]=(0.158928139216)
        ref[5,1,1,0]=(1.06402047834)
        ref[5,1,1,1]=(-0.567749347509)
      else:
        s[0,0,0]=(-0.709307825835)*x[0]+(-0.212632569057)*x[1]+(-1.36262175379)*x[2]
        s[0,0,1]=(1.26136861771)*x[0]+(-0.247247821335)*x[1]+(0.952472163507)*x[2]
        s[0,1,0]=(-0.313562941587)*x[0]+(0.335230615547)*x[1]+(0.633669942964)*x[2]
        s[0,1,1]=(0.543435495949)*x[0]+(-0.278928766765)*x[1]+(-0.560857410629)*x[2]
        s[1,0,0]=(-0.228999374753)*x[0]+(0.627231691819)*x[1]+(-0.110282807377)*x[2]
        s[1,0,1]=(-0.183700669251)*x[0]+(0.361641358769)*x[1]+(-0.565719344787)*x[2]
        s[1,1,0]=(0.0392225368869)*x[0]+(0.530481764326)*x[1]+(-1.16647413286)*x[2]
        s[1,1,1]=(1.10307486886)*x[0]+(1.62013473483)*x[1]+(1.08663406043)*x[2]
        s[2,0,0]=(0.418269374961)*x[0]+(-0.881366670214)*x[1]+(-0.19081724529)*x[2]
        s[2,0,1]=(-0.888542773287)*x[0]+(0.973322751168)*x[1]+(-0.678806455499)*x[2]
        s[2,1,0]=(0.203680986809)*x[0]+(-0.64202022526)*x[1]+(-0.342367020762)*x[2]
        s[2,1,1]=(-0.984982113825)*x[0]+(1.52327025412)*x[1]+(-1.16664161507)*x[2]
        s[3,0,0]=(-0.430857332595)*x[0]+(1.37801740474)*x[1]+(1.47536591267)*x[2]
        s[3,0,1]=(-1.49872232697)*x[0]+(0.679321175644)*x[1]+(0.233255377135)*x[2]
        s[3,1,0]=(0.00673347423284)*x[0]+(-0.659736891512)*x[1]+(-1.24483031223)*x[2]
        s[3,1,1]=(-1.35939956697)*x[0]+(-1.05370704987)*x[1]+(0.974871664002)*x[2]
        s[4,0,0]=(-1.56401623595)*x[0]+(0.0396404235057)*x[1]+(0.529626464189)*x[2]
        s[4,0,1]=(-0.903524096133)*x[0]+(-0.0504449605469)*x[1]+(-0.38093760213)*x[2]
        s[4,1,0]=(-0.570517363798)*x[0]+(1.50697108327)*x[1]+(0.45722182765)*x[2]
        s[4,1,1]=(0.855238285486)*x[0]+(-1.19608277327)*x[1]+(-0.454258653189)*x[2]
        s[5,0,0]=(0.264880357574)*x[0]+(-0.137240568006)*x[1]+(-1.43157116771)*x[2]
        s[5,0,1]=(1.42099564204)*x[0]+(-0.584391783811)*x[1]+(-0.194445105527)*x[2]
        s[5,1,0]=(-1.41719313241)*x[0]+(1.18436247042)*x[1]+(0.774373387123)*x[2]
        s[5,1,1]=(0.894745586845)*x[0]+(0.272141343375)*x[1]+(-0.563792436747)*x[2]
        ref[0,0,0,0]=(-0.709307825835)
        ref[0,0,0,1]=(-0.212632569057)
        ref[0,0,0,2]=(-1.36262175379)
        ref[0,0,1,0]=(1.26136861771)
        ref[0,0,1,1]=(-0.247247821335)
        ref[0,0,1,2]=(0.952472163507)
        ref[0,1,0,0]=(-0.313562941587)
        ref[0,1,0,1]=(0.335230615547)
        ref[0,1,0,2]=(0.633669942964)
        ref[0,1,1,0]=(0.543435495949)
        ref[0,1,1,1]=(-0.278928766765)
        ref[0,1,1,2]=(-0.560857410629)
        ref[1,0,0,0]=(-0.228999374753)
        ref[1,0,0,1]=(0.627231691819)
        ref[1,0,0,2]=(-0.110282807377)
        ref[1,0,1,0]=(-0.183700669251)
        ref[1,0,1,1]=(0.361641358769)
        ref[1,0,1,2]=(-0.565719344787)
        ref[1,1,0,0]=(0.0392225368869)
        ref[1,1,0,1]=(0.530481764326)
        ref[1,1,0,2]=(-1.16647413286)
        ref[1,1,1,0]=(1.10307486886)
        ref[1,1,1,1]=(1.62013473483)
        ref[1,1,1,2]=(1.08663406043)
        ref[2,0,0,0]=(0.418269374961)
        ref[2,0,0,1]=(-0.881366670214)
        ref[2,0,0,2]=(-0.19081724529)
        ref[2,0,1,0]=(-0.888542773287)
        ref[2,0,1,1]=(0.973322751168)
        ref[2,0,1,2]=(-0.678806455499)
        ref[2,1,0,0]=(0.203680986809)
        ref[2,1,0,1]=(-0.64202022526)
        ref[2,1,0,2]=(-0.342367020762)
        ref[2,1,1,0]=(-0.984982113825)
        ref[2,1,1,1]=(1.52327025412)
        ref[2,1,1,2]=(-1.16664161507)
        ref[3,0,0,0]=(-0.430857332595)
        ref[3,0,0,1]=(1.37801740474)
        ref[3,0,0,2]=(1.47536591267)
        ref[3,0,1,0]=(-1.49872232697)
        ref[3,0,1,1]=(0.679321175644)
        ref[3,0,1,2]=(0.233255377135)
        ref[3,1,0,0]=(0.00673347423284)
        ref[3,1,0,1]=(-0.659736891512)
        ref[3,1,0,2]=(-1.24483031223)
        ref[3,1,1,0]=(-1.35939956697)
        ref[3,1,1,1]=(-1.05370704987)
        ref[3,1,1,2]=(0.974871664002)
        ref[4,0,0,0]=(-1.56401623595)
        ref[4,0,0,1]=(0.0396404235057)
        ref[4,0,0,2]=(0.529626464189)
        ref[4,0,1,0]=(-0.903524096133)
        ref[4,0,1,1]=(-0.0504449605469)
        ref[4,0,1,2]=(-0.38093760213)
        ref[4,1,0,0]=(-0.570517363798)
        ref[4,1,0,1]=(1.50697108327)
        ref[4,1,0,2]=(0.45722182765)
        ref[4,1,1,0]=(0.855238285486)
        ref[4,1,1,1]=(-1.19608277327)
        ref[4,1,1,2]=(-0.454258653189)
        ref[5,0,0,0]=(0.264880357574)
        ref[5,0,0,1]=(-0.137240568006)
        ref[5,0,0,2]=(-1.43157116771)
        ref[5,0,1,0]=(1.42099564204)
        ref[5,0,1,1]=(-0.584391783811)
        ref[5,0,1,2]=(-0.194445105527)
        ref[5,1,0,0]=(-1.41719313241)
        ref[5,1,0,1]=(1.18436247042)
        ref[5,1,0,2]=(0.774373387123)
        ref[5,1,1,0]=(0.894745586845)
        ref[5,1,1,1]=(0.272141343375)
        ref[5,1,1,2]=(-0.563792436747)
      res=grad(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2)+(dim,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromData_ContinuousFunction(self):
      """
      tests divergence of Data on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.39999188185)*x[0]**o+(-0.0985302665504)*x[0]+(0.103912148113)*x[1]**o+(-0.440893151166)*x[1]
        arg[1]=(-0.550867128413)*x[0]**o+(-0.141593933069)*x[0]+(0.70592012062)*x[1]**o+(-0.836308099686)*x[1]
        ref=o*(-0.39999188185)*x_ref[0]**(o-1)+o*(0.70592012062)*x_ref[1]**(o-1)+(-0.934838366236)
      else:
        arg[0]=(-0.00405079499937)*x[0]**o+(-0.282117848793)*x[0]+(0.979898720034)*x[1]**o+(-0.418106638625)*x[1]+(-0.443550810851)*x[2]**o+(-0.114976349698)*x[2]
        arg[1]=(0.194746221496)*x[0]**o+(0.324772666848)*x[0]+(0.887813387362)*x[1]**o+(-0.9362867149)*x[1]+(-0.837328978457)*x[2]**o+(0.666079514358)*x[2]
        arg[2]=(0.974797445328)*x[0]**o+(0.00365347225195)*x[0]+(-0.285413216102)*x[1]**o+(-0.253930177142)*x[1]+(0.306713249275)*x[2]**o+(-0.651133960743)*x[2]
        ref=o*(-0.00405079499937)*x_ref[0]**(o-1)+o*(0.887813387362)*x_ref[1]**(o-1)+o*(0.306713249275)*x_ref[2]**(o-1)+(-1.86953852444)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromData_Solution(self):
      """
      tests divergence of Data on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.756094856241)*x[0]**o+(-0.185186427186)*x[0]+(0.15709960228)*x[1]**o+(-0.408372399703)*x[1]
        arg[1]=(-0.680793761252)*x[0]**o+(0.815275471992)*x[0]+(-0.0564602888317)*x[1]**o+(0.542507598499)*x[1]
        ref=o*(-0.756094856241)*x_ref[0]**(o-1)+o*(-0.0564602888317)*x_ref[1]**(o-1)+(0.357321171313)
      else:
        arg[0]=(-0.934829971573)*x[0]**o+(-0.202157515712)*x[0]+(0.972095997183)*x[1]**o+(0.430890291958)*x[1]+(0.973333736054)*x[2]**o+(0.563067835398)*x[2]
        arg[1]=(-0.676905980848)*x[0]**o+(0.721377714968)*x[0]+(-0.594993636371)*x[1]**o+(0.678979489168)*x[1]+(-0.491382641255)*x[2]**o+(-0.237396572727)*x[2]
        arg[2]=(-0.429697616397)*x[0]**o+(-0.8280062193)*x[0]+(0.549112225165)*x[1]**o+(0.73490904828)*x[1]+(0.543450722632)*x[2]**o+(-0.910092410543)*x[2]
        ref=o*(-0.934829971573)*x_ref[0]**(o-1)+o*(-0.594993636371)*x_ref[1]**(o-1)+o*(0.543450722632)*x_ref[2]**(o-1)+(-0.433270437087)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromData_ReducedSolution(self):
      """
      tests divergence of Data on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.701055390328)*x[0]+(-0.873109677704)*x[1]
        arg[1]=(-0.459521943793)*x[0]+(-1.63064815751)*x[1]
        ref=(-2.33170354784)
      else:
        arg[0]=(-0.408202035271)*x[0]+(-0.114769362644)*x[1]+(1.5510990077)*x[2]
        arg[1]=(0.17481433607)*x[0]+(-0.466969145604)*x[1]+(1.25289823435)*x[2]
        arg[2]=(-0.215477376297)*x[0]+(0.437173510517)*x[1]+(1.00334359062)*x[2]
        ref=(0.128172409746)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromData_ReducedContinuousFunction(self):
      """
      tests divergence of Data on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.701055390328)*x[0]+(-0.873109677704)*x[1]
        arg[1]=(-0.459521943793)*x[0]+(-1.63064815751)*x[1]
        ref=(-2.33170354784)
      else:
        arg[0]=(-0.408202035271)*x[0]+(-0.114769362644)*x[1]+(1.5510990077)*x[2]
        arg[1]=(0.17481433607)*x[0]+(-0.466969145604)*x[1]+(1.25289823435)*x[2]
        arg[2]=(-0.215477376297)*x[0]+(0.437173510517)*x[1]+(1.00334359062)*x[2]
        ref=(0.128172409746)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromSymbol_ContinuousFunction(self):
      """
      tests divergence of Symbol on the FunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.600081772347)*x[0]**o+(-0.45657364476)*x[0]+(0.100558790895)*x[1]**o+(-0.055643420941)*x[1]
        s[1]=(-0.568472071712)*x[0]**o+(-0.683739238637)*x[0]+(-0.14866031141)*x[1]**o+(-0.17686218803)*x[1]
        ref=o*(0.600081772347)*x_ref[0]**(o-1)+o*(-0.14866031141)*x_ref[1]**(o-1)+(-0.63343583279)
      else:
        s[0]=(-0.822374031685)*x[0]**o+(-0.587511521237)*x[0]+(0.963639060657)*x[1]**o+(-0.190070692051)*x[1]+(0.558162506917)*x[2]**o+(-0.460808186343)*x[2]
        s[1]=(-0.798572575819)*x[0]**o+(-0.101266909891)*x[0]+(-0.516195739148)*x[1]**o+(0.617726153299)*x[1]+(-0.34240818464)*x[2]**o+(0.335550364849)*x[2]
        s[2]=(-0.213593757489)*x[0]**o+(-0.366203371298)*x[0]+(0.531651196423)*x[1]**o+(0.199631639651)*x[1]+(0.775999930783)*x[2]**o+(-0.3541802777)*x[2]
        ref=o*(-0.822374031685)*x_ref[0]**(o-1)+o*(-0.516195739148)*x_ref[1]**(o-1)+o*(0.775999930783)*x_ref[2]**(o-1)+(-0.323965645638)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromSymbol_Solution(self):
      """
      tests divergence of Symbol on the FunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.737547486132)*x[0]**o+(-0.8688345451)*x[0]+(0.750434018309)*x[1]**o+(0.140443192943)*x[1]
        s[1]=(-0.581930862436)*x[0]**o+(-0.0165116457364)*x[0]+(-0.575385479046)*x[1]**o+(0.840871277956)*x[1]
        ref=o*(0.737547486132)*x_ref[0]**(o-1)+o*(-0.575385479046)*x_ref[1]**(o-1)+(-0.0279632671436)
      else:
        s[0]=(0.270325198436)*x[0]**o+(-0.47191239181)*x[0]+(-0.999528728847)*x[1]**o+(-0.103752839996)*x[1]+(-0.76928188931)*x[2]**o+(0.595426116602)*x[2]
        s[1]=(0.269434059222)*x[0]**o+(0.387014929786)*x[0]+(-0.0817482722299)*x[1]**o+(-0.36264415709)*x[1]+(-0.693828731298)*x[2]**o+(-0.82629322569)*x[2]
        s[2]=(0.171309519816)*x[0]**o+(-0.696536709305)*x[0]+(-0.835027772202)*x[1]**o+(-0.916627860891)*x[1]+(0.35239086527)*x[2]**o+(-0.921910360255)*x[2]
        ref=o*(0.270325198436)*x_ref[0]**(o-1)+o*(-0.0817482722299)*x_ref[1]**(o-1)+o*(0.35239086527)*x_ref[2]**(o-1)+(-1.75646690915)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromSymbol_ReducedSolution(self):
      """
      tests divergence of Symbol on the FunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.759081801873)*x[0]+(1.19486066363)*x[1]
        s[1]=(0.124293887621)*x[0]+(0.239495671437)*x[1]
        ref=(-0.519586130436)
      else:
        s[0]=(0.114143993825)*x[0]+(1.00937429349)*x[1]+(-0.402505788806)*x[2]
        s[1]=(1.3389839478)*x[0]+(-1.18006174569)*x[1]+(-0.7113433796)*x[2]
        s[2]=(-0.082639774749)*x[0]+(-0.349220712526)*x[1]+(0.477197377967)*x[2]
        ref=(-0.588720373901)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnBoundary_fromSymbol_ReducedContinuousFunction(self):
      """
      tests divergence of Symbol on the FunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.759081801873)*x[0]+(1.19486066363)*x[1]
        s[1]=(0.124293887621)*x[0]+(0.239495671437)*x[1]
        ref=(-0.519586130436)
      else:
        s[0]=(0.114143993825)*x[0]+(1.00937429349)*x[1]+(-0.402505788806)*x[2]
        s[1]=(1.3389839478)*x[0]+(-1.18006174569)*x[1]+(-0.7113433796)*x[2]
        s[2]=(-0.082639774749)*x[0]+(-0.349220712526)*x[1]+(0.477197377967)*x[2]
        ref=(-0.588720373901)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromData_ContinuousFunction(self):
      """
      tests divergence of Data on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.39999188185)*x[0]**o+(-0.0985302665504)*x[0]+(0.103912148113)*x[1]**o+(-0.440893151166)*x[1]
        arg[1]=(-0.550867128413)*x[0]**o+(-0.141593933069)*x[0]+(0.70592012062)*x[1]**o+(-0.836308099686)*x[1]
        ref=o*(-0.39999188185)*x_ref[0]**(o-1)+o*(0.70592012062)*x_ref[1]**(o-1)+(-0.934838366236)
      else:
        arg[0]=(-0.00405079499937)*x[0]**o+(-0.282117848793)*x[0]+(0.979898720034)*x[1]**o+(-0.418106638625)*x[1]+(-0.443550810851)*x[2]**o+(-0.114976349698)*x[2]
        arg[1]=(0.194746221496)*x[0]**o+(0.324772666848)*x[0]+(0.887813387362)*x[1]**o+(-0.9362867149)*x[1]+(-0.837328978457)*x[2]**o+(0.666079514358)*x[2]
        arg[2]=(0.974797445328)*x[0]**o+(0.00365347225195)*x[0]+(-0.285413216102)*x[1]**o+(-0.253930177142)*x[1]+(0.306713249275)*x[2]**o+(-0.651133960743)*x[2]
        ref=o*(-0.00405079499937)*x_ref[0]**(o-1)+o*(0.887813387362)*x_ref[1]**(o-1)+o*(0.306713249275)*x_ref[2]**(o-1)+(-1.86953852444)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromData_Solution(self):
      """
      tests divergence of Data on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.756094856241)*x[0]**o+(-0.185186427186)*x[0]+(0.15709960228)*x[1]**o+(-0.408372399703)*x[1]
        arg[1]=(-0.680793761252)*x[0]**o+(0.815275471992)*x[0]+(-0.0564602888317)*x[1]**o+(0.542507598499)*x[1]
        ref=o*(-0.756094856241)*x_ref[0]**(o-1)+o*(-0.0564602888317)*x_ref[1]**(o-1)+(0.357321171313)
      else:
        arg[0]=(-0.934829971573)*x[0]**o+(-0.202157515712)*x[0]+(0.972095997183)*x[1]**o+(0.430890291958)*x[1]+(0.973333736054)*x[2]**o+(0.563067835398)*x[2]
        arg[1]=(-0.676905980848)*x[0]**o+(0.721377714968)*x[0]+(-0.594993636371)*x[1]**o+(0.678979489168)*x[1]+(-0.491382641255)*x[2]**o+(-0.237396572727)*x[2]
        arg[2]=(-0.429697616397)*x[0]**o+(-0.8280062193)*x[0]+(0.549112225165)*x[1]**o+(0.73490904828)*x[1]+(0.543450722632)*x[2]**o+(-0.910092410543)*x[2]
        ref=o*(-0.934829971573)*x_ref[0]**(o-1)+o*(-0.594993636371)*x_ref[1]**(o-1)+o*(0.543450722632)*x_ref[2]**(o-1)+(-0.433270437087)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromData_ReducedSolution(self):
      """
      tests divergence of Data on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.701055390328)*x[0]+(-0.873109677704)*x[1]
        arg[1]=(-0.459521943793)*x[0]+(-1.63064815751)*x[1]
        ref=(-2.33170354784)
      else:
        arg[0]=(-0.408202035271)*x[0]+(-0.114769362644)*x[1]+(1.5510990077)*x[2]
        arg[1]=(0.17481433607)*x[0]+(-0.466969145604)*x[1]+(1.25289823435)*x[2]
        arg[2]=(-0.215477376297)*x[0]+(0.437173510517)*x[1]+(1.00334359062)*x[2]
        ref=(0.128172409746)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromData_ReducedContinuousFunction(self):
      """
      tests divergence of Data on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.701055390328)*x[0]+(-0.873109677704)*x[1]
        arg[1]=(-0.459521943793)*x[0]+(-1.63064815751)*x[1]
        ref=(-2.33170354784)
      else:
        arg[0]=(-0.408202035271)*x[0]+(-0.114769362644)*x[1]+(1.5510990077)*x[2]
        arg[1]=(0.17481433607)*x[0]+(-0.466969145604)*x[1]+(1.25289823435)*x[2]
        arg[2]=(-0.215477376297)*x[0]+(0.437173510517)*x[1]+(1.00334359062)*x[2]
        ref=(0.128172409746)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromSymbol_ContinuousFunction(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnBoundary

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.600081772347)*x[0]**o+(-0.45657364476)*x[0]+(0.100558790895)*x[1]**o+(-0.055643420941)*x[1]
        s[1]=(-0.568472071712)*x[0]**o+(-0.683739238637)*x[0]+(-0.14866031141)*x[1]**o+(-0.17686218803)*x[1]
        ref=o*(0.600081772347)*x_ref[0]**(o-1)+o*(-0.14866031141)*x_ref[1]**(o-1)+(-0.63343583279)
      else:
        s[0]=(-0.822374031685)*x[0]**o+(-0.587511521237)*x[0]+(0.963639060657)*x[1]**o+(-0.190070692051)*x[1]+(0.558162506917)*x[2]**o+(-0.460808186343)*x[2]
        s[1]=(-0.798572575819)*x[0]**o+(-0.101266909891)*x[0]+(-0.516195739148)*x[1]**o+(0.617726153299)*x[1]+(-0.34240818464)*x[2]**o+(0.335550364849)*x[2]
        s[2]=(-0.213593757489)*x[0]**o+(-0.366203371298)*x[0]+(0.531651196423)*x[1]**o+(0.199631639651)*x[1]+(0.775999930783)*x[2]**o+(-0.3541802777)*x[2]
        ref=o*(-0.822374031685)*x_ref[0]**(o-1)+o*(-0.516195739148)*x_ref[1]**(o-1)+o*(0.775999930783)*x_ref[2]**(o-1)+(-0.323965645638)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromSymbol_Solution(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnBoundary

      assumptions: Solution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.737547486132)*x[0]**o+(-0.8688345451)*x[0]+(0.750434018309)*x[1]**o+(0.140443192943)*x[1]
        s[1]=(-0.581930862436)*x[0]**o+(-0.0165116457364)*x[0]+(-0.575385479046)*x[1]**o+(0.840871277956)*x[1]
        ref=o*(0.737547486132)*x_ref[0]**(o-1)+o*(-0.575385479046)*x_ref[1]**(o-1)+(-0.0279632671436)
      else:
        s[0]=(0.270325198436)*x[0]**o+(-0.47191239181)*x[0]+(-0.999528728847)*x[1]**o+(-0.103752839996)*x[1]+(-0.76928188931)*x[2]**o+(0.595426116602)*x[2]
        s[1]=(0.269434059222)*x[0]**o+(0.387014929786)*x[0]+(-0.0817482722299)*x[1]**o+(-0.36264415709)*x[1]+(-0.693828731298)*x[2]**o+(-0.82629322569)*x[2]
        s[2]=(0.171309519816)*x[0]**o+(-0.696536709305)*x[0]+(-0.835027772202)*x[1]**o+(-0.916627860891)*x[1]+(0.35239086527)*x[2]**o+(-0.921910360255)*x[2]
        ref=o*(0.270325198436)*x_ref[0]**(o-1)+o*(-0.0817482722299)*x_ref[1]**(o-1)+o*(0.35239086527)*x_ref[2]**(o-1)+(-1.75646690915)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromSymbol_ReducedSolution(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.759081801873)*x[0]+(1.19486066363)*x[1]
        s[1]=(0.124293887621)*x[0]+(0.239495671437)*x[1]
        ref=(-0.519586130436)
      else:
        s[0]=(0.114143993825)*x[0]+(1.00937429349)*x[1]+(-0.402505788806)*x[2]
        s[1]=(1.3389839478)*x[0]+(-1.18006174569)*x[1]+(-0.7113433796)*x[2]
        s[2]=(-0.082639774749)*x[0]+(-0.349220712526)*x[1]+(0.477197377967)*x[2]
        ref=(-0.588720373901)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnBoundary_fromSymbol_ReducedContinuousFunction(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnBoundary

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnBoundary
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnBoundary(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.759081801873)*x[0]+(1.19486066363)*x[1]
        s[1]=(0.124293887621)*x[0]+(0.239495671437)*x[1]
        ref=(-0.519586130436)
      else:
        s[0]=(0.114143993825)*x[0]+(1.00937429349)*x[1]+(-0.402505788806)*x[2]
        s[1]=(1.3389839478)*x[0]+(-1.18006174569)*x[1]+(-0.7113433796)*x[2]
        s[2]=(-0.082639774749)*x[0]+(-0.349220712526)*x[1]+(0.477197377967)*x[2]
        ref=(-0.588720373901)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromData_ContinuousFunction(self):
      """
      tests divergence of Data on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.168949755871)*x[0]**o+(-0.850028843808)*x[0]+(-0.120402543124)*x[1]**o+(-0.962848557803)*x[1]
        arg[1]=(-0.654809749448)*x[0]**o+(-0.86782287971)*x[0]+(0.063297622975)*x[1]**o+(0.400298481087)*x[1]
        ref=o*(-0.168949755871)*x_ref[0]**(o-1)+o*(0.063297622975)*x_ref[1]**(o-1)+(-0.449730362722)
      else:
        arg[0]=(-0.171176882625)*x[0]**o+(-0.983025148621)*x[0]+(-0.287555867477)*x[1]**o+(-0.689164237089)*x[1]+(0.700322207054)*x[2]**o+(0.224056812199)*x[2]
        arg[1]=(-0.112978574269)*x[0]**o+(-0.858210458939)*x[0]+(-0.9204250284)*x[1]**o+(0.123297295575)*x[1]+(-0.824406652345)*x[2]**o+(-0.514156554116)*x[2]
        arg[2]=(0.51215310895)*x[0]**o+(-0.484568237326)*x[0]+(0.324174170946)*x[1]**o+(0.00636990564332)*x[1]+(0.640205070676)*x[2]**o+(0.642111592847)*x[2]
        ref=o*(-0.171176882625)*x_ref[0]**(o-1)+o*(-0.9204250284)*x_ref[1]**(o-1)+o*(0.640205070676)*x_ref[2]**(o-1)+(-0.217616260198)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromData_Solution(self):
      """
      tests divergence of Data on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.878817592363)*x[0]**o+(-0.670917442592)*x[0]+(0.497874113437)*x[1]**o+(-0.923717382234)*x[1]
        arg[1]=(-0.26042852826)*x[0]**o+(0.0668495878627)*x[0]+(-0.24188250168)*x[1]**o+(-0.189346966858)*x[1]
        ref=o*(-0.878817592363)*x_ref[0]**(o-1)+o*(-0.24188250168)*x_ref[1]**(o-1)+(-0.86026440945)
      else:
        arg[0]=(-0.853829042745)*x[0]**o+(0.708253823804)*x[0]+(-0.420888906754)*x[1]**o+(0.94422817266)*x[1]+(-0.0454733411462)*x[2]**o+(-0.588636328569)*x[2]
        arg[1]=(0.502105411994)*x[0]**o+(0.618635348918)*x[0]+(-0.459898919811)*x[1]**o+(0.0206615239957)*x[1]+(-0.479378221929)*x[2]**o+(-0.0383028811998)*x[2]
        arg[2]=(-0.624273042239)*x[0]**o+(0.11218158851)*x[0]+(0.300065081794)*x[1]**o+(-0.823140829548)*x[1]+(-0.537442909756)*x[2]**o+(-0.537901446348)*x[2]
        ref=o*(-0.853829042745)*x_ref[0]**(o-1)+o*(-0.459898919811)*x_ref[1]**(o-1)+o*(-0.537442909756)*x_ref[2]**(o-1)+(0.191013901452)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromData_ReducedSolution(self):
      """
      tests divergence of Data on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(1.40908912517)*x[0]+(0.980544777047)*x[1]
        arg[1]=(1.54735707911)*x[0]+(0.136617363775)*x[1]
        ref=(1.54570648895)
      else:
        arg[0]=(0.542899657951)*x[0]+(0.445471953165)*x[1]+(0.832668361385)*x[2]
        arg[1]=(-0.658976315278)*x[0]+(0.467903824571)*x[1]+(-1.00818943414)*x[2]
        arg[2]=(-1.10271841197)*x[0]+(1.48792855886)*x[1]+(-0.560275626738)*x[2]
        ref=(0.450527855784)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromData_ReducedContinuousFunction(self):
      """
      tests divergence of Data on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(1.40908912517)*x[0]+(0.980544777047)*x[1]
        arg[1]=(1.54735707911)*x[0]+(0.136617363775)*x[1]
        ref=(1.54570648895)
      else:
        arg[0]=(0.542899657951)*x[0]+(0.445471953165)*x[1]+(0.832668361385)*x[2]
        arg[1]=(-0.658976315278)*x[0]+(0.467903824571)*x[1]+(-1.00818943414)*x[2]
        arg[2]=(-1.10271841197)*x[0]+(1.48792855886)*x[1]+(-0.560275626738)*x[2]
        ref=(0.450527855784)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromSymbol_ContinuousFunction(self):
      """
      tests divergence of Symbol on the FunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.161146122811)*x[0]**o+(-0.167192157833)*x[0]+(-0.375618887584)*x[1]**o+(0.0878245139474)*x[1]
        s[1]=(-0.537393073871)*x[0]**o+(0.120284625555)*x[0]+(-0.732878618183)*x[1]**o+(0.950964015707)*x[1]
        ref=o*(0.161146122811)*x_ref[0]**(o-1)+o*(-0.732878618183)*x_ref[1]**(o-1)+(0.783771857874)
      else:
        s[0]=(0.663467704633)*x[0]**o+(0.192717069721)*x[0]+(-0.348590092108)*x[1]**o+(-0.00702534646765)*x[1]+(0.359300718583)*x[2]**o+(0.325682691269)*x[2]
        s[1]=(0.127384376464)*x[0]**o+(-0.974260164601)*x[0]+(0.623191833228)*x[1]**o+(0.927154841547)*x[1]+(0.66921644471)*x[2]**o+(0.730046317457)*x[2]
        s[2]=(0.477753719763)*x[0]**o+(0.303349068035)*x[0]+(0.297684454456)*x[1]**o+(-0.23387142141)*x[1]+(-0.160162179877)*x[2]**o+(0.321427279218)*x[2]
        ref=o*(0.663467704633)*x_ref[0]**(o-1)+o*(0.623191833228)*x_ref[1]**(o-1)+o*(-0.160162179877)*x_ref[2]**(o-1)+(1.44129919049)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromSymbol_Solution(self):
      """
      tests divergence of Symbol on the FunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.714164520534)*x[0]**o+(0.82104115064)*x[0]+(-0.101329353755)*x[1]**o+(0.211581405872)*x[1]
        s[1]=(0.154441871043)*x[0]**o+(-0.545965134865)*x[0]+(0.232672960523)*x[1]**o+(0.613108805891)*x[1]
        ref=o*(-0.714164520534)*x_ref[0]**(o-1)+o*(0.232672960523)*x_ref[1]**(o-1)+(1.43414995653)
      else:
        s[0]=(0.668585271141)*x[0]**o+(0.367611537166)*x[0]+(0.241825965301)*x[1]**o+(-0.111380450261)*x[1]+(-0.928428911379)*x[2]**o+(-0.282714026941)*x[2]
        s[1]=(-0.733274659976)*x[0]**o+(0.827799978056)*x[0]+(0.957107141065)*x[1]**o+(0.654786636056)*x[1]+(-0.327647026546)*x[2]**o+(-0.263782117053)*x[2]
        s[2]=(-0.770358571743)*x[0]**o+(-0.793594113842)*x[0]+(-0.870794666744)*x[1]**o+(-0.760800400466)*x[1]+(0.987607632645)*x[2]**o+(0.960902635484)*x[2]
        ref=o*(0.668585271141)*x_ref[0]**(o-1)+o*(0.957107141065)*x_ref[1]**(o-1)+o*(0.987607632645)*x_ref[2]**(o-1)+(1.98330080871)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromSymbol_ReducedSolution(self):
      """
      tests divergence of Symbol on the FunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.445276807175)*x[0]+(-0.878942915065)*x[1]
        s[1]=(-0.0238953550121)*x[0]+(0.361680637262)*x[1]
        ref=(0.806957444436)
      else:
        s[0]=(0.0467419526691)*x[0]+(-0.00176696769735)*x[1]+(1.42280058205)*x[2]
        s[1]=(0.184131344951)*x[0]+(0.64372416222)*x[1]+(-1.33372814614)*x[2]
        s[2]=(0.284337855687)*x[0]+(-0.185747325055)*x[1]+(0.328830085951)*x[2]
        ref=(1.01929620084)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactZero_fromSymbol_ReducedContinuousFunction(self):
      """
      tests divergence of Symbol on the FunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.445276807175)*x[0]+(-0.878942915065)*x[1]
        s[1]=(-0.0238953550121)*x[0]+(0.361680637262)*x[1]
        ref=(0.806957444436)
      else:
        s[0]=(0.0467419526691)*x[0]+(-0.00176696769735)*x[1]+(1.42280058205)*x[2]
        s[1]=(0.184131344951)*x[0]+(0.64372416222)*x[1]+(-1.33372814614)*x[2]
        s[2]=(0.284337855687)*x[0]+(-0.185747325055)*x[1]+(0.328830085951)*x[2]
        ref=(1.01929620084)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromData_ContinuousFunction(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.168949755871)*x[0]**o+(-0.850028843808)*x[0]+(-0.120402543124)*x[1]**o+(-0.962848557803)*x[1]
        arg[1]=(-0.654809749448)*x[0]**o+(-0.86782287971)*x[0]+(0.063297622975)*x[1]**o+(0.400298481087)*x[1]
        ref=o*(-0.168949755871)*x_ref[0]**(o-1)+o*(0.063297622975)*x_ref[1]**(o-1)+(-0.449730362722)
      else:
        arg[0]=(-0.171176882625)*x[0]**o+(-0.983025148621)*x[0]+(-0.287555867477)*x[1]**o+(-0.689164237089)*x[1]+(0.700322207054)*x[2]**o+(0.224056812199)*x[2]
        arg[1]=(-0.112978574269)*x[0]**o+(-0.858210458939)*x[0]+(-0.9204250284)*x[1]**o+(0.123297295575)*x[1]+(-0.824406652345)*x[2]**o+(-0.514156554116)*x[2]
        arg[2]=(0.51215310895)*x[0]**o+(-0.484568237326)*x[0]+(0.324174170946)*x[1]**o+(0.00636990564332)*x[1]+(0.640205070676)*x[2]**o+(0.642111592847)*x[2]
        ref=o*(-0.171176882625)*x_ref[0]**(o-1)+o*(-0.9204250284)*x_ref[1]**(o-1)+o*(0.640205070676)*x_ref[2]**(o-1)+(-0.217616260198)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromData_Solution(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.878817592363)*x[0]**o+(-0.670917442592)*x[0]+(0.497874113437)*x[1]**o+(-0.923717382234)*x[1]
        arg[1]=(-0.26042852826)*x[0]**o+(0.0668495878627)*x[0]+(-0.24188250168)*x[1]**o+(-0.189346966858)*x[1]
        ref=o*(-0.878817592363)*x_ref[0]**(o-1)+o*(-0.24188250168)*x_ref[1]**(o-1)+(-0.86026440945)
      else:
        arg[0]=(-0.853829042745)*x[0]**o+(0.708253823804)*x[0]+(-0.420888906754)*x[1]**o+(0.94422817266)*x[1]+(-0.0454733411462)*x[2]**o+(-0.588636328569)*x[2]
        arg[1]=(0.502105411994)*x[0]**o+(0.618635348918)*x[0]+(-0.459898919811)*x[1]**o+(0.0206615239957)*x[1]+(-0.479378221929)*x[2]**o+(-0.0383028811998)*x[2]
        arg[2]=(-0.624273042239)*x[0]**o+(0.11218158851)*x[0]+(0.300065081794)*x[1]**o+(-0.823140829548)*x[1]+(-0.537442909756)*x[2]**o+(-0.537901446348)*x[2]
        ref=o*(-0.853829042745)*x_ref[0]**(o-1)+o*(-0.459898919811)*x_ref[1]**(o-1)+o*(-0.537442909756)*x_ref[2]**(o-1)+(0.191013901452)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromData_ReducedSolution(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(1.40908912517)*x[0]+(0.980544777047)*x[1]
        arg[1]=(1.54735707911)*x[0]+(0.136617363775)*x[1]
        ref=(1.54570648895)
      else:
        arg[0]=(0.542899657951)*x[0]+(0.445471953165)*x[1]+(0.832668361385)*x[2]
        arg[1]=(-0.658976315278)*x[0]+(0.467903824571)*x[1]+(-1.00818943414)*x[2]
        arg[2]=(-1.10271841197)*x[0]+(1.48792855886)*x[1]+(-0.560275626738)*x[2]
        ref=(0.450527855784)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromData_ReducedContinuousFunction(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(1.40908912517)*x[0]+(0.980544777047)*x[1]
        arg[1]=(1.54735707911)*x[0]+(0.136617363775)*x[1]
        ref=(1.54570648895)
      else:
        arg[0]=(0.542899657951)*x[0]+(0.445471953165)*x[1]+(0.832668361385)*x[2]
        arg[1]=(-0.658976315278)*x[0]+(0.467903824571)*x[1]+(-1.00818943414)*x[2]
        arg[2]=(-1.10271841197)*x[0]+(1.48792855886)*x[1]+(-0.560275626738)*x[2]
        ref=(0.450527855784)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromSymbol_ContinuousFunction(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactZero

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.161146122811)*x[0]**o+(-0.167192157833)*x[0]+(-0.375618887584)*x[1]**o+(0.0878245139474)*x[1]
        s[1]=(-0.537393073871)*x[0]**o+(0.120284625555)*x[0]+(-0.732878618183)*x[1]**o+(0.950964015707)*x[1]
        ref=o*(0.161146122811)*x_ref[0]**(o-1)+o*(-0.732878618183)*x_ref[1]**(o-1)+(0.783771857874)
      else:
        s[0]=(0.663467704633)*x[0]**o+(0.192717069721)*x[0]+(-0.348590092108)*x[1]**o+(-0.00702534646765)*x[1]+(0.359300718583)*x[2]**o+(0.325682691269)*x[2]
        s[1]=(0.127384376464)*x[0]**o+(-0.974260164601)*x[0]+(0.623191833228)*x[1]**o+(0.927154841547)*x[1]+(0.66921644471)*x[2]**o+(0.730046317457)*x[2]
        s[2]=(0.477753719763)*x[0]**o+(0.303349068035)*x[0]+(0.297684454456)*x[1]**o+(-0.23387142141)*x[1]+(-0.160162179877)*x[2]**o+(0.321427279218)*x[2]
        ref=o*(0.663467704633)*x_ref[0]**(o-1)+o*(0.623191833228)*x_ref[1]**(o-1)+o*(-0.160162179877)*x_ref[2]**(o-1)+(1.44129919049)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromSymbol_Solution(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactZero

      assumptions: Solution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.714164520534)*x[0]**o+(0.82104115064)*x[0]+(-0.101329353755)*x[1]**o+(0.211581405872)*x[1]
        s[1]=(0.154441871043)*x[0]**o+(-0.545965134865)*x[0]+(0.232672960523)*x[1]**o+(0.613108805891)*x[1]
        ref=o*(-0.714164520534)*x_ref[0]**(o-1)+o*(0.232672960523)*x_ref[1]**(o-1)+(1.43414995653)
      else:
        s[0]=(0.668585271141)*x[0]**o+(0.367611537166)*x[0]+(0.241825965301)*x[1]**o+(-0.111380450261)*x[1]+(-0.928428911379)*x[2]**o+(-0.282714026941)*x[2]
        s[1]=(-0.733274659976)*x[0]**o+(0.827799978056)*x[0]+(0.957107141065)*x[1]**o+(0.654786636056)*x[1]+(-0.327647026546)*x[2]**o+(-0.263782117053)*x[2]
        s[2]=(-0.770358571743)*x[0]**o+(-0.793594113842)*x[0]+(-0.870794666744)*x[1]**o+(-0.760800400466)*x[1]+(0.987607632645)*x[2]**o+(0.960902635484)*x[2]
        ref=o*(0.668585271141)*x_ref[0]**(o-1)+o*(0.957107141065)*x_ref[1]**(o-1)+o*(0.987607632645)*x_ref[2]**(o-1)+(1.98330080871)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromSymbol_ReducedSolution(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.445276807175)*x[0]+(-0.878942915065)*x[1]
        s[1]=(-0.0238953550121)*x[0]+(0.361680637262)*x[1]
        ref=(0.806957444436)
      else:
        s[0]=(0.0467419526691)*x[0]+(-0.00176696769735)*x[1]+(1.42280058205)*x[2]
        s[1]=(0.184131344951)*x[0]+(0.64372416222)*x[1]+(-1.33372814614)*x[2]
        s[2]=(0.284337855687)*x[0]+(-0.185747325055)*x[1]+(0.328830085951)*x[2]
        ref=(1.01929620084)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactZero_fromSymbol_ReducedContinuousFunction(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactZero

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactZero
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactZero(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.445276807175)*x[0]+(-0.878942915065)*x[1]
        s[1]=(-0.0238953550121)*x[0]+(0.361680637262)*x[1]
        ref=(0.806957444436)
      else:
        s[0]=(0.0467419526691)*x[0]+(-0.00176696769735)*x[1]+(1.42280058205)*x[2]
        s[1]=(0.184131344951)*x[0]+(0.64372416222)*x[1]+(-1.33372814614)*x[2]
        s[2]=(0.284337855687)*x[0]+(-0.185747325055)*x[1]+(0.328830085951)*x[2]
        ref=(1.01929620084)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromData_ContinuousFunction(self):
      """
      tests divergence of Data on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.506403770066)*x[0]**o+(0.32584268281)*x[0]+(-0.83529844868)*x[1]**o+(0.83697332673)*x[1]
        arg[1]=(-0.85254932286)*x[0]**o+(0.751060747065)*x[0]+(-0.672601723159)*x[1]**o+(0.33751172176)*x[1]
        ref=o*(-0.506403770066)*x_ref[0]**(o-1)+o*(-0.672601723159)*x_ref[1]**(o-1)+(0.66335440457)
      else:
        arg[0]=(-0.437811901761)*x[0]**o+(-0.135276240019)*x[0]+(0.683969739116)*x[1]**o+(-0.207547375093)*x[1]+(0.028594861031)*x[2]**o+(0.721966907461)*x[2]
        arg[1]=(0.315283027578)*x[0]**o+(0.724158692434)*x[0]+(-0.746372201791)*x[1]**o+(-0.370798791131)*x[1]+(-0.677893413238)*x[2]**o+(0.645317237187)*x[2]
        arg[2]=(0.0795681320657)*x[0]**o+(-0.165210743397)*x[0]+(0.866175461075)*x[1]**o+(0.10183311172)*x[1]+(-0.642389179022)*x[2]**o+(-0.917391184653)*x[2]
        ref=o*(-0.437811901761)*x_ref[0]**(o-1)+o*(-0.746372201791)*x_ref[1]**(o-1)+o*(-0.642389179022)*x_ref[2]**(o-1)+(-1.4234662158)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromData_Solution(self):
      """
      tests divergence of Data on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(0.46385553428)*x[0]**o+(-0.91930502543)*x[0]+(0.316745110613)*x[1]**o+(-0.0996899204895)*x[1]
        arg[1]=(-0.473415736992)*x[0]**o+(0.514718374433)*x[0]+(-0.473429315265)*x[1]**o+(-0.194316615233)*x[1]
        ref=o*(0.46385553428)*x_ref[0]**(o-1)+o*(-0.473429315265)*x_ref[1]**(o-1)+(-1.11362164066)
      else:
        arg[0]=(0.740236725813)*x[0]**o+(0.887898498849)*x[0]+(-0.44803045798)*x[1]**o+(0.160930903094)*x[1]+(-0.530527194219)*x[2]**o+(0.0622347651074)*x[2]
        arg[1]=(0.36748785661)*x[0]**o+(-0.618052241987)*x[0]+(-0.57186325537)*x[1]**o+(-0.180576934525)*x[1]+(-0.381568503551)*x[2]**o+(0.0356882174068)*x[2]
        arg[2]=(0.070144257808)*x[0]**o+(0.823052625211)*x[0]+(-0.426522615959)*x[1]**o+(-0.294700653199)*x[1]+(0.209922433427)*x[2]**o+(-0.532056800218)*x[2]
        ref=o*(0.740236725813)*x_ref[0]**(o-1)+o*(-0.57186325537)*x_ref[1]**(o-1)+o*(0.209922433427)*x_ref[2]**(o-1)+(0.175264764106)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromData_ReducedSolution(self):
      """
      tests divergence of Data on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(0.0337229572599)*x[0]+(0.820391566535)*x[1]
        arg[1]=(0.640567267185)*x[0]+(1.03537999206)*x[1]
        ref=(1.06910294932)
      else:
        arg[0]=(-0.0797501374913)*x[0]+(-0.798826730006)*x[1]+(-0.590979881432)*x[2]
        arg[1]=(-0.414547879032)*x[0]+(0.845538994985)*x[1]+(-1.21203164133)*x[2]
        arg[2]=(-0.438232052406)*x[0]+(-0.35977163998)*x[1]+(-1.01489636413)*x[2]
        ref=(-0.249107506635)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromData_ReducedContinuousFunction(self):
      """
      tests divergence of Data on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(0.0337229572599)*x[0]+(0.820391566535)*x[1]
        arg[1]=(0.640567267185)*x[0]+(1.03537999206)*x[1]
        ref=(1.06910294932)
      else:
        arg[0]=(-0.0797501374913)*x[0]+(-0.798826730006)*x[1]+(-0.590979881432)*x[2]
        arg[1]=(-0.414547879032)*x[0]+(0.845538994985)*x[1]+(-1.21203164133)*x[2]
        arg[2]=(-0.438232052406)*x[0]+(-0.35977163998)*x[1]+(-1.01489636413)*x[2]
        ref=(-0.249107506635)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromSymbol_ContinuousFunction(self):
      """
      tests divergence of Symbol on the FunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.0898941553957)*x[0]**o+(-0.856300995796)*x[0]+(0.348298533454)*x[1]**o+(0.0351975574428)*x[1]
        s[1]=(-0.930316249846)*x[0]**o+(-0.0295682442002)*x[0]+(-0.648381042093)*x[1]**o+(0.0737127687654)*x[1]
        ref=o*(-0.0898941553957)*x_ref[0]**(o-1)+o*(-0.648381042093)*x_ref[1]**(o-1)+(-0.782588227031)
      else:
        s[0]=(-0.163987987624)*x[0]**o+(-0.511857517274)*x[0]+(0.741666372996)*x[1]**o+(0.496640177195)*x[1]+(0.460061066559)*x[2]**o+(-0.558420509462)*x[2]
        s[1]=(0.71525289068)*x[0]**o+(0.886545582)*x[0]+(-0.904766629428)*x[1]**o+(0.146361771148)*x[1]+(0.570424059304)*x[2]**o+(0.305997230748)*x[2]
        s[2]=(0.615370647779)*x[0]**o+(0.471712532771)*x[0]+(0.570468738833)*x[1]**o+(-0.728014206718)*x[1]+(-0.38659264723)*x[2]**o+(0.717051171147)*x[2]
        ref=o*(-0.163987987624)*x_ref[0]**(o-1)+o*(-0.904766629428)*x_ref[1]**(o-1)+o*(-0.38659264723)*x_ref[2]**(o-1)+(0.351555425021)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromSymbol_Solution(self):
      """
      tests divergence of Symbol on the FunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.194773312667)*x[0]**o+(-0.265468977683)*x[0]+(-0.0352177899992)*x[1]**o+(0.463717858721)*x[1]
        s[1]=(0.139203691371)*x[0]**o+(-0.799672510649)*x[0]+(-0.367277227148)*x[1]**o+(0.111274990466)*x[1]
        ref=o*(0.194773312667)*x_ref[0]**(o-1)+o*(-0.367277227148)*x_ref[1]**(o-1)+(-0.154193987217)
      else:
        s[0]=(0.0505125806914)*x[0]**o+(0.149858893872)*x[0]+(-0.095881973707)*x[1]**o+(-0.850488788785)*x[1]+(-0.0110026442959)*x[2]**o+(-0.407209597266)*x[2]
        s[1]=(0.72998229305)*x[0]**o+(0.745287781387)*x[0]+(-0.940255747769)*x[1]**o+(0.67408292569)*x[1]+(-0.149905927735)*x[2]**o+(0.0681385797299)*x[2]
        s[2]=(-0.314939233587)*x[0]**o+(0.716212656447)*x[0]+(-0.78946710376)*x[1]**o+(-0.867972385662)*x[1]+(0.299407109746)*x[2]**o+(-0.601525689779)*x[2]
        ref=o*(0.0505125806914)*x_ref[0]**(o-1)+o*(-0.940255747769)*x_ref[1]**(o-1)+o*(0.299407109746)*x_ref[2]**(o-1)+(0.222416129783)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromSymbol_ReducedSolution(self):
      """
      tests divergence of Symbol on the FunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.98773096396)*x[0]+(0.381969528266)*x[1]
        s[1]=(0.508967927604)*x[0]+(0.742652762961)*x[1]
        ref=(-0.245078200999)
      else:
        s[0]=(0.0672222632785)*x[0]+(0.303873945969)*x[1]+(-0.471524980136)*x[2]
        s[1]=(0.63378224019)*x[0]+(-0.865690740572)*x[1]+(-0.0997380430882)*x[2]
        s[2]=(0.711173440054)*x[0]+(0.845130507211)*x[1]+(-1.10616823701)*x[2]
        ref=(-1.9046367143)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onFunctionOnContactOne_fromSymbol_ReducedContinuousFunction(self):
      """
      tests divergence of Symbol on the FunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on FunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=FunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.98773096396)*x[0]+(0.381969528266)*x[1]
        s[1]=(0.508967927604)*x[0]+(0.742652762961)*x[1]
        ref=(-0.245078200999)
      else:
        s[0]=(0.0672222632785)*x[0]+(0.303873945969)*x[1]+(-0.471524980136)*x[2]
        s[1]=(0.63378224019)*x[0]+(-0.865690740572)*x[1]+(-0.0997380430882)*x[2]
        s[2]=(0.711173440054)*x[0]+(0.845130507211)*x[1]+(-1.10616823701)*x[2]
        ref=(-1.9046367143)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromData_ContinuousFunction(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(-0.506403770066)*x[0]**o+(0.32584268281)*x[0]+(-0.83529844868)*x[1]**o+(0.83697332673)*x[1]
        arg[1]=(-0.85254932286)*x[0]**o+(0.751060747065)*x[0]+(-0.672601723159)*x[1]**o+(0.33751172176)*x[1]
        ref=o*(-0.506403770066)*x_ref[0]**(o-1)+o*(-0.672601723159)*x_ref[1]**(o-1)+(0.66335440457)
      else:
        arg[0]=(-0.437811901761)*x[0]**o+(-0.135276240019)*x[0]+(0.683969739116)*x[1]**o+(-0.207547375093)*x[1]+(0.028594861031)*x[2]**o+(0.721966907461)*x[2]
        arg[1]=(0.315283027578)*x[0]**o+(0.724158692434)*x[0]+(-0.746372201791)*x[1]**o+(-0.370798791131)*x[1]+(-0.677893413238)*x[2]**o+(0.645317237187)*x[2]
        arg[2]=(0.0795681320657)*x[0]**o+(-0.165210743397)*x[0]+(0.866175461075)*x[1]**o+(0.10183311172)*x[1]+(-0.642389179022)*x[2]**o+(-0.917391184653)*x[2]
        ref=o*(-0.437811901761)*x_ref[0]**(o-1)+o*(-0.746372201791)*x_ref[1]**(o-1)+o*(-0.642389179022)*x_ref[2]**(o-1)+(-1.4234662158)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromData_Solution(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(0.46385553428)*x[0]**o+(-0.91930502543)*x[0]+(0.316745110613)*x[1]**o+(-0.0996899204895)*x[1]
        arg[1]=(-0.473415736992)*x[0]**o+(0.514718374433)*x[0]+(-0.473429315265)*x[1]**o+(-0.194316615233)*x[1]
        ref=o*(0.46385553428)*x_ref[0]**(o-1)+o*(-0.473429315265)*x_ref[1]**(o-1)+(-1.11362164066)
      else:
        arg[0]=(0.740236725813)*x[0]**o+(0.887898498849)*x[0]+(-0.44803045798)*x[1]**o+(0.160930903094)*x[1]+(-0.530527194219)*x[2]**o+(0.0622347651074)*x[2]
        arg[1]=(0.36748785661)*x[0]**o+(-0.618052241987)*x[0]+(-0.57186325537)*x[1]**o+(-0.180576934525)*x[1]+(-0.381568503551)*x[2]**o+(0.0356882174068)*x[2]
        arg[2]=(0.070144257808)*x[0]**o+(0.823052625211)*x[0]+(-0.426522615959)*x[1]**o+(-0.294700653199)*x[1]+(0.209922433427)*x[2]**o+(-0.532056800218)*x[2]
        ref=o*(0.740236725813)*x_ref[0]**(o-1)+o*(-0.57186325537)*x_ref[1]**(o-1)+o*(0.209922433427)*x_ref[2]**(o-1)+(0.175264764106)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromData_ReducedSolution(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(0.0337229572599)*x[0]+(0.820391566535)*x[1]
        arg[1]=(0.640567267185)*x[0]+(1.03537999206)*x[1]
        ref=(1.06910294932)
      else:
        arg[0]=(-0.0797501374913)*x[0]+(-0.798826730006)*x[1]+(-0.590979881432)*x[2]
        arg[1]=(-0.414547879032)*x[0]+(0.845538994985)*x[1]+(-1.21203164133)*x[2]
        arg[2]=(-0.438232052406)*x[0]+(-0.35977163998)*x[1]+(-1.01489636413)*x[2]
        ref=(-0.249107506635)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromData_ReducedContinuousFunction(self):
      """
      tests divergence of Data on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Vector(0,w)
      if dim==2:
        arg[0]=(0.0337229572599)*x[0]+(0.820391566535)*x[1]
        arg[1]=(0.640567267185)*x[0]+(1.03537999206)*x[1]
        ref=(1.06910294932)
      else:
        arg[0]=(-0.0797501374913)*x[0]+(-0.798826730006)*x[1]+(-0.590979881432)*x[2]
        arg[1]=(-0.414547879032)*x[0]+(0.845538994985)*x[1]+(-1.21203164133)*x[2]
        arg[2]=(-0.438232052406)*x[0]+(-0.35977163998)*x[1]+(-1.01489636413)*x[2]
        ref=(-0.249107506635)
      res=div(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromSymbol_ContinuousFunction(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactOne

      assumptions: ContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.0898941553957)*x[0]**o+(-0.856300995796)*x[0]+(0.348298533454)*x[1]**o+(0.0351975574428)*x[1]
        s[1]=(-0.930316249846)*x[0]**o+(-0.0295682442002)*x[0]+(-0.648381042093)*x[1]**o+(0.0737127687654)*x[1]
        ref=o*(-0.0898941553957)*x_ref[0]**(o-1)+o*(-0.648381042093)*x_ref[1]**(o-1)+(-0.782588227031)
      else:
        s[0]=(-0.163987987624)*x[0]**o+(-0.511857517274)*x[0]+(0.741666372996)*x[1]**o+(0.496640177195)*x[1]+(0.460061066559)*x[2]**o+(-0.558420509462)*x[2]
        s[1]=(0.71525289068)*x[0]**o+(0.886545582)*x[0]+(-0.904766629428)*x[1]**o+(0.146361771148)*x[1]+(0.570424059304)*x[2]**o+(0.305997230748)*x[2]
        s[2]=(0.615370647779)*x[0]**o+(0.471712532771)*x[0]+(0.570468738833)*x[1]**o+(-0.728014206718)*x[1]+(-0.38659264723)*x[2]**o+(0.717051171147)*x[2]
        ref=o*(-0.163987987624)*x_ref[0]**(o-1)+o*(-0.904766629428)*x_ref[1]**(o-1)+o*(-0.38659264723)*x_ref[2]**(o-1)+(0.351555425021)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromSymbol_Solution(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactOne

      assumptions: Solution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=self.order
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=Solution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(0.194773312667)*x[0]**o+(-0.265468977683)*x[0]+(-0.0352177899992)*x[1]**o+(0.463717858721)*x[1]
        s[1]=(0.139203691371)*x[0]**o+(-0.799672510649)*x[0]+(-0.367277227148)*x[1]**o+(0.111274990466)*x[1]
        ref=o*(0.194773312667)*x_ref[0]**(o-1)+o*(-0.367277227148)*x_ref[1]**(o-1)+(-0.154193987217)
      else:
        s[0]=(0.0505125806914)*x[0]**o+(0.149858893872)*x[0]+(-0.095881973707)*x[1]**o+(-0.850488788785)*x[1]+(-0.0110026442959)*x[2]**o+(-0.407209597266)*x[2]
        s[1]=(0.72998229305)*x[0]**o+(0.745287781387)*x[0]+(-0.940255747769)*x[1]**o+(0.67408292569)*x[1]+(-0.149905927735)*x[2]**o+(0.0681385797299)*x[2]
        s[2]=(-0.314939233587)*x[0]**o+(0.716212656447)*x[0]+(-0.78946710376)*x[1]**o+(-0.867972385662)*x[1]+(0.299407109746)*x[2]**o+(-0.601525689779)*x[2]
        ref=o*(0.0505125806914)*x_ref[0]**(o-1)+o*(-0.940255747769)*x_ref[1]**(o-1)+o*(0.299407109746)*x_ref[2]**(o-1)+(0.222416129783)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromSymbol_ReducedSolution(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedSolution(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedSolution(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.98773096396)*x[0]+(0.381969528266)*x[1]
        s[1]=(0.508967927604)*x[0]+(0.742652762961)*x[1]
        ref=(-0.245078200999)
      else:
        s[0]=(0.0672222632785)*x[0]+(0.303873945969)*x[1]+(-0.471524980136)*x[2]
        s[1]=(0.63378224019)*x[0]+(-0.865690740572)*x[1]+(-0.0997380430882)*x[2]
        s[2]=(0.711173440054)*x[0]+(0.845130507211)*x[1]+(-1.10616823701)*x[2]
        ref=(-1.9046367143)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_div_onReducedFunctionOnContactOne_fromSymbol_ReducedContinuousFunction(self):
      """
      tests divergence of Symbol on the ReducedFunctionOnContactOne

      assumptions: ReducedContinuousFunction(self.domain) exists
                   self.domain supports div on ReducedFunctionOnContactOne
      """
      o=1
      dim=self.domain.getDim()
      w_ref=ReducedFunctionOnContactOne(self.domain)
      x_ref=w_ref.getX()
      w=ReducedContinuousFunction(self.domain)
      x=w.getX()
      arg=Symbol(shape=(dim,),dim=dim)
      s=Vector(0,w)
      if dim==2:
        s[0]=(-0.98773096396)*x[0]+(0.381969528266)*x[1]
        s[1]=(0.508967927604)*x[0]+(0.742652762961)*x[1]
        ref=(-0.245078200999)
      else:
        s[0]=(0.0672222632785)*x[0]+(0.303873945969)*x[1]+(-0.471524980136)*x[2]
        s[1]=(0.63378224019)*x[0]+(-0.865690740572)*x[1]+(-0.0997380430882)*x[2]
        s[2]=(0.711173440054)*x[0]+(0.845130507211)*x[1]+(-1.10616823701)*x[2]
        ref=(-1.9046367143)
      res=div(arg,where=w_ref)
      sub=res.substitute({arg:s})
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertEqual(sub.getFunctionSpace(),w_ref,"wrong function space of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")


   def test_grad_onFunctionOnBoundary_fromConstant_onSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = 5.325699e-03
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.13328403643902254, 0.5180789608364019, 0.8473725228751088] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.17471334053708842, 0.7234268336897315], [0.2919940087255526, 0.5287096643695982], [0.3600144177470853, 0.35466127794259317], [0.06483217063319047, 0.5949901915322034]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.9213100479981251, 0.12253171725837597, 0.41994324535779426, 0.6230497578536783], [0.1001149950416077, 0.8502477745680308, 0.5471058713009459, 0.8520717833220935]], [[0.5605303053115659, 0.49756584705484774, 0.7280422312943363, 0.08916113667035397], [0.649327630049585, 0.8114318117377761, 0.8911899718596958, 0.056764605912336674]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 3.915707e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.06377171484537192, 0.966040756184193, 0.5464099064244966] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.37242745827480905, 0.16608261172807715], [0.43907681880296734, 0.7977345148667832], [0.2937376350868136, 0.03224768113034626], [0.3876880508278723, 0.5411522936006017]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.9088607246376975, 0.6860485775980993, 0.1370063406124229, 0.21288135010167197], [0.3469162081847519, 0.8262827813281785, 0.45611765063606224, 0.979888975901961]], [[0.29193594226471176, 0.5566897857838056, 0.16435159604887106, 0.7680255608180384], [0.1620674264045665, 0.059896127276754196, 0.9077249650000538, 0.697086176083336]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 1.928511e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.6533280686124567, 0.9316016729816435, 0.2860543625783294] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.290868619614702, 0.7162558637433156], [0.028740680920193884, 0.6687416379809296], [0.04643092735506238, 0.8662211685746547], [0.029984509764989564, 0.6473352108879699]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.023591427336316473, 0.27411924872867377, 0.18959184507685145, 0.7516856839689471], [0.3833584126891021, 0.2600919607462666, 0.7915351284354711, 0.6099430108957569]], [[0.5557805213238012, 0.2672054314856013, 0.26092389049502895, 0.8815476687888248], [0.4159112668969126, 0.6305965624321637, 0.0378813260655978, 0.010936714360735955]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 3.225850e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.11113933659770825, 0.2652307611386975, 0.9768302434415] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.43895483447843786, 0.3705453998519529], [0.7856811089849307, 0.2893400116026289], [0.6494291474582917, 0.3216748663311576], [0.9734234162840023, 0.12129760399119671]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.5423618140986598, 0.8878156583097585, 0.25231512194709027, 0.06379277224136626], [0.2244003241340048, 0.5523724730635384, 0.6635290044481768, 0.9598137336965046]], [[0.06612978226427024, 0.8879137312571469, 0.1151104415793851, 0.00742273174452468], [0.8812027196335972, 0.870396411327305, 0.6752530851574361, 0.05171713668191147]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = 2.570651e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.31557680732533466, 0.7066169398998586, 0.9211177636658148] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.20002823296573136, 0.6620894695331346], [0.8522485364166127, 0.2213598049260288], [0.6773798712937313, 0.15804162040124936], [0.9035793861521892, 0.2772446707143794]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.626859285544357, 0.978181779065287, 0.411852849330247, 0.6371983938795727], [0.7064981455837864, 0.439494462027905, 0.7176349285510094, 0.6344938189469543]], [[0.6489954852227914, 0.9053722153820701, 0.7270914774110729, 0.06455024993536962], [0.03551831997286925, 0.6924121288324315, 0.13608030614353783, 0.14219036290318632]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 1.663927e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.22280627106323836, 0.17674510475567107, 0.020739142341216654] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.9281463106171305, 0.0861785507659526], [0.20481360627679546, 0.61474028972928], [0.4346142500904996, 0.6826125749128275], [0.9676483959735505, 0.004030408958624165]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.17409515413758514, 0.06443606167564908, 0.12142659712375536, 0.9325006274502955], [0.27623132535403216, 0.6696906599028458, 0.06494885848683274, 0.8161638075291826]], [[0.18605405395934893, 0.8273852528797611, 0.8196768796468152, 0.2817640409991363], [0.06993111167302168, 0.612771747737222, 0.8418321336640362, 0.7132065633379493]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 2.544264e-02
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.1905449698257855, 0.44353572143192566, 0.31837399209945927] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.7069981287341179, 0.22391519172519325], [0.5935596941135006, 0.773732962337458], [0.3579287667186175, 0.30704382627669646], [0.6145669245227324, 0.31153548410609844]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.355188219921715, 0.6014282473114086, 0.5941643626488761, 0.5878689700538938], [0.29074522133426794, 0.21622897569223032, 0.5366123306387235, 0.6446336306577326]], [[0.7117204152650095, 0.31111017017222453, 0.36711556833588066, 0.3604234227121649], [0.7270995644363838, 0.3588969002172844, 0.9813760466336967, 0.5834196954785033]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 1.988179e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.6478656215035142, 0.6600302427812709, 0.09360460856735675] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.20052243915182166, 0.2862854360122983], [0.7537797857942006, 0.7701986775776483], [0.5762752929646884, 0.09943478024026742], [0.41014920545731726, 0.2432727398882053]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnBoundary_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnBoundary
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnBoundary(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.9389141351043594, 0.8147125185453394, 0.4863265324626874, 0.2914068570775302], [0.42700589160123725, 0.183768854368919, 0.9484492907302342, 0.37701332047751923]], [[0.1569594611550882, 0.1341537338578087, 0.35362215752966164, 0.04872017112907634], [0.6421837556116794, 0.20442915782723126, 0.5736624923251025, 0.44637140938161535]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = 9.029388e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.907548586087531, 0.707317124485586, 0.8995803949864449] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.2999902729340933, 0.044588138347334105], [0.5849117114829943, 0.9992874981172579], [0.8341634631871544, 0.10545135301119335], [0.358648729878584, 0.5582992029507422]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.836403736156003, 0.29786838780382785, 0.30738923417000974, 0.6835833744789601], [0.44732249531502544, 0.8010845703821015, 0.6735902741269875, 0.6950264940140927]], [[0.11444353237638027, 0.04906250268219181, 0.5777221032528653, 0.458967712710838], [0.5498101526171087, 0.7310917893467106, 0.8461044440889665, 0.1517882061417145]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 7.957408e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.0013522567015791243, 0.8341989782079291, 0.7788990041622172] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.3433433211730025, 0.8483801077181371], [0.01879976321157739, 0.9886595759120674], [0.4793613768457512, 0.034789632355608946], [0.6013518368182185, 0.9099251278789463]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.004268096134998456, 0.6228740101326737, 0.8792699746660345, 0.024175300967381075], [0.17360804600587165, 0.4438147477535489, 0.8303413665858226, 0.3373827734760221]], [[0.8212269016903279, 0.7790662460292276, 0.5246688109670851, 0.4296496240473877], [0.4318635056975825, 0.6591761894263933, 0.868305431080897, 0.8745372396233271]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 6.930144e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.43039014192498826, 0.3282259009434936, 0.5300279421680291] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.0467302291835493, 0.21756782592678447], [0.5649636423784632, 0.634196025489312], [0.12626039189865346, 0.2816191388234631], [0.5633558912300907, 0.41492854425122283]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.5322400314181076, 0.22866870266286987, 0.413896078912124, 0.2646726496735461], [0.5526230061530046, 0.7466862381236534, 0.8074259698321433, 0.7014038277054564]], [[0.9414081637252353, 0.05636422862206092, 0.6871399482281239, 0.2887350017776229], [0.39404123118079826, 0.8104641817835231, 0.7373779531988631, 0.49946399301720656]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 7.707884e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.5232292363917099, 0.8931293188553918, 0.5812188088398242] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.26687908233641555, 0.4251828593279853], [0.2826514251456448, 0.5901807274550085], [0.8789901409693618, 0.2942788629122738], [0.8027763572249443, 0.5296242351380728]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactZero_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactZero(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.47045049288803176, 0.6207450777194974, 0.5624386764704142, 0.14973184755069158], [0.6211128943488801, 0.5237252846377405, 0.12649240479070234, 0.9336566953931132]], [[0.1500355466712322, 0.4462868541421865, 0.2807053569848985, 0.8096831766130468], [0.8279613670328214, 0.10776444652559936, 0.3768428303007917, 0.5525635903908854]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = 9.449773e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.19521101121951812, 0.28714141197255205, 0.34625007788269224] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.11401362766025736, 0.7018197235840259], [0.006532810128159983, 0.737541882731158], [0.6843257628080699, 0.9886981954348636], [0.33802726145856266, 0.7781846374447284]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.8833833762040265, 0.9728842360655732, 0.3542657059733225, 0.505950349413704], [0.040119678091724564, 0.08040257685829655, 0.27203091307749216, 0.2732437060954299]], [[0.17170951925014444, 0.7555888718823888, 0.9181699371777938, 0.0533750242261255], [0.5666509111370258, 0.40097984437465484, 0.46445997770413183, 0.7794312685709942]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 4.408085e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.034933512516942655, 0.9800178002215272, 0.36426619967479745] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.181262789680669, 0.3486511342822154], [0.7906209928751575, 0.8310253226954649], [0.9755804626005933, 0.004856247591845997], [0.8350685565096081, 0.6521163395567604]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.7624877352377937, 0.29455697397539016, 0.35597530806671485, 0.1769771752868431], [0.8278501171334879, 0.6612879867586324, 0.2108226358029297, 0.6565168256101444]], [[0.073903841131432, 0.40319958073796325, 0.44982934816931064, 0.16213847221780098], [0.53112512717715, 0.9889543030220338, 0.827289845278779, 0.8958058319625968]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 3.547293e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.5336376939404084, 0.12070081134673638, 0.7301505047048527] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.4384318821977905, 0.19742948498016177], [0.000868854537104502, 0.30641409989780166], [0.8978945954452449, 0.6961902987809375], [0.8738974106674391, 0.6593713103392849]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.4647091925810939, 0.5886421334207723, 0.9828869971579512, 0.5246499201155416], [0.356258805505017, 0.4642405842691707, 0.30224518620208296, 0.18095574745237797]], [[0.31537604994487456, 0.4151507737829214, 0.9172446637657757, 0.02564131369125655], [0.4589562348907429, 0.156731449959858, 0.7075700764658797, 0.011739475607361993]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 4.633141e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.3465183165757971, 0.28586755536866093, 0.05015182702075227] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.6526532625537063, 0.7701098960911085], [0.4499339798186688, 0.7206838063007455], [0.02473512734449168, 0.7026595718891236], [0.8737679013583777, 0.8323673434504008]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onReducedFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on ReducedFunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = ReducedFunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.34548655206650203, 0.18866358590977605, 0.7881296922547287, 0.5333995307603652], [0.5656650311923244, 0.447609070673588, 0.7098356484325665, 0.8530883432967371]], [[0.7388333050567967, 0.23926364943752398, 0.13273395500670215, 0.7405506857776485], [0.9048447936731181, 0.5466702038766015, 0.5166781959953796, 0.7707732173924752]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = 8.331514e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.960953822224517, 0.45782323441793726, 0.6234723050577922] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.9242847282549886, 0.4353970256009895], [0.3596598029557798, 0.18072819289469744], [0.12103014692584535, 0.30223820949059066], [0.3746576203324995, 0.9440547503471269]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.1133672646781918, 0.47673129863326724, 0.13009035306580008, 0.7434898533554899], [0.3153636496490576, 0.09592374842943141, 0.18277231428374585, 0.9220575412366814]], [[0.29337276622857067, 0.7386496151277152, 0.5749881917541155, 0.473446487738974], [0.5369842798857408, 0.27927761877573376, 0.37933856466614724, 0.47627921748832935]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onReducedSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 9.981430e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onReducedSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.4910439815888492, 0.6484854794550088, 0.47493020075964065] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onReducedSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.8279210875223708, 0.2338380667906539], [0.4315567111758656, 0.3852814141988875], [0.4012781631925967, 0.8905175994912358], [0.9496152369641803, 0.08909287805428256]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onReducedSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.481210129538855, 0.0723440492180496, 0.4567948984438164, 0.826495921948382], [0.11006036373991257, 0.7373575076348994, 0.9416880096622194, 0.32686072281352907]], [[0.7182587257539528, 0.26224493030090035, 0.9591798675504234, 0.9763653717687493], [0.18820099519437516, 0.6182720985697959, 0.9391823958161781, 0.09042308985011616]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 5.312874e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.6479174153745072, 0.7561314707056226, 0.9442415088857188] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.9950426856851823, 0.42876376065618327], [0.32879582533962826, 0.6010940221721649], [0.7062683300945141, 0.7636144582012837], [0.011317421171781361, 0.49653823818308407]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactZero_fromConstant_onContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactZero
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactZero(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.9104360760245774, 0.7086942730558727, 0.4382452050895169, 0.09268609254772164], [0.9466986099511558, 0.4025000415915436, 0.24214152764634334, 0.48541323585758]], [[0.6324255289447923, 0.10894062583819653, 0.6272657066951097, 0.9486334024540664], [0.9544294908748299, 0.16115781291479903, 0.5422070091150973, 0.5628477736603812]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = 8.380454e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([0.31445073912851196, 0.5668367672347497, 0.8000695114763042] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[0.10555226400339657, 0.8383175203256023], [0.9031323610028994, 0.3306202527274066], [0.5424693719793899, 0.08148727380386034], [0.9037394782691902, 0.8887054812423583]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = Solution(self.domain)
      val = numpy.array([[[0.17143955494105834, 0.9594782779369886, 0.9553663667076671, 0.9020290010115394], [0.4487613712309403, 0.3139150890431325, 0.8896073355353095, 0.12567668642329166]], [[0.5783462159222819, 0.5959007963742765, 0.06716227591886281, 0.10038749802185687], [0.9067139704521134, 0.7705652417204064, 0.4172450480292257, 0.6869999982188596]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedSolution_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = 1.465850e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedSolution_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([0.9781472496184622, 0.34877213123446604, 0.2874082424916701] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedSolution_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[0.07230934823267299, 0.6610812687098071], [0.8033284017291904, 0.1472855502096575], [0.6917385472190036, 0.5036435396175376], [0.3964648016752985, 0.16711710933459878]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedSolution_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedSolution(self.domain)
      val = numpy.array([[[0.26363909916652295, 0.3049876598173691, 0.7448532875524292, 0.07641933863496264], [0.3932484137127511, 0.19445926695599747, 0.34136542682105786, 0.18951397004968829]], [[0.04719926005945296, 0.4432679085087615, 0.45059952841276385, 0.4983004656292932], [0.4173414468938297, 0.861814775959532, 0.4178270464042314, 0.3608703337621374]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = 3.797274e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([0.804333583754331, 0.07117284292376358, 0.2659813115866344] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[0.015901753253646733, 0.9749998355010222], [0.7750637065487529, 0.9665168468802372], [0.6956132796891801, 0.9098699253363709], [0.2842107882721583, 0.878769256462077]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ContinuousFunction(self.domain)
      val = numpy.array([[[0.6813045153981402, 0.8695411370765636, 0.1601397385237573, 0.944713648774908], [0.04480763634580254, 0.44951462026678646, 0.5242822705763874, 0.6687236299387497]], [[0.37056536329284284, 0.0515672412245316, 0.2021199953443663, 0.40011474496973276], [0.9074436025303534, 0.012254573846022576, 0.46017514695539585, 0.5521387566363697]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank0(self):
      """
      test gradient of constant rank 0 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = 3.727099e-01
      arg = Data(val,(),w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),()+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank1(self):
      """
      test gradient of constant rank 1 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([0.031068422686192365, 0.13548924746922697, 0.8882464304650891] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(3,)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank2(self):
      """
      test gradient of constant rank 2 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[0.6714863064397392, 0.42463336467014945], [0.20028775482321604, 0.6291533593059164], [0.7055358672164853, 0.6513587696416855], [0.028483862355425438, 0.024048093640512302]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(4, 2)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")
   def test_grad_onFunctionOnContactOne_fromConstant_onReducedContinuousFunction_rank3(self):
      """
      test gradient of constant rank 3 Data on FunctionOnContactOne
      """
      DIM = self.domain.getDim()
      w_ref = FunctionOnContactOne(self.domain)
      w_arg = ReducedContinuousFunction(self.domain)
      val = numpy.array([[[0.814666306523813, 0.5849826990998531, 0.3338406933569058, 0.5276257782517414], [0.11292857096801256, 0.699944560000377, 0.36021518360446525, 0.9987353809416907]], [[0.5726211101583571, 0.6381734253875894, 0.0004888301733868561, 0.20683242396858892], [0.3134077439738643, 0.6974727370337456, 0.8993008211890828, 0.796389910384546]]] ,numpy.float_)
      arg = Data(val,w_arg)
      res=grad(arg,where=w_ref)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getFunctionSpace(),w_ref,"wrong functionspace of result.")
      self.assertEqual(res.getShape(),(2, 2, 4)+(DIM,),"wrong shape of result.")
      self.assertTrue(Lsup(res)<=self.RES_TOL,"wrong result")

